/*$File***********************************************************************

        File               assign_str.cpp

        Contents           string assignment manipulation routines.

        Prototype File     assign_str.h

        Created            Vladimir Schneider, vladimir@winpte.com
                           Sep 22, 2004

        Copyright (c) 2004 Systemcorp ALG, All Rights Reserved

******X******************X************************X************************X$*/

/*$Func************************************************************************

      Synopsis           BOOL_RETURN_CODE FC_INI_ASSIGN_STR(SC_LONG_PARAM bClearString)

         bClearString    if !0 will also clear the current assignment
                         string. Default = TRUE.

      Description        Clears the global variables in preparation for
                         calls to FC_ASSIGN_STRING function which will
                         accumulate the assignment profile string.

      Return Value       0

      Remarks

         Warning

         Preconditions

         Locks

         Side Effects    Clears global variables used for the
                         accumulation of the assignment string

         Limitations

         See Also        FC_ASSIGN_STRING, FC_GET_ASSIGN_STRING

******X******************X************************X************************X$*/

Prompt == FUNCTION FC_INI_ASSIGN_STR ==
CREATE OR REPLACE FUNCTION FC_INI_ASSIGN_STR
(
     sh_bClearString PLS_INTEGER := 1
) RETURN PLS_INTEGER
AS
   bClearString PLS_INTEGER := Coalesce(sh_bClearString, 0);
BEGIN
   If bClearString <> 0 Then
      ASSIGN_STR.g_assign_str_contour := '';
      ASSIGN_STR.g_assign_str_had_calls := 0;
   End If;
   
   ASSIGN_STR.g_assign_str_first_day := 0;
   ASSIGN_STR.g_assign_str_last_day := 0;
   ASSIGN_STR.g_assign_str_last_minutes := 0;
   
   RETURN 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INI_ASSIGN_STR', 18+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_INI_ASSIGN_STR(19) : assign_str.sql(65) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_INI_ASSIGN_STR;
ALTER FUNCTION FC_INI_ASSIGN_STR COMPILE DEBUG;

/*$Func************************************************************************

      Synopsis           BOOL_RETURN_CODE FC_ASSIGN_STRING(SC_LONG_PARAM h_julian_day, short h_julian_day_Ind, SC_LONG_PARAM h_minutes, short h_minutes_Ind)

         h_julian_day    day of assignment, must be >= value passed in last
                         call. For the first call can be any value.

         h_julian_day_Ind  ignored

         h_minutes       number of minutes assigned

         h_minutes_Ind   ignored


      Description        Add a day to the existing contour string. Will
                         accumulate days which have the same assignment
                         minutes into X*# where X is number of days and # is
                         the number of assigned minutes.

      Return Value       0 if success, else error

      Remarks            If days are skipped between calls their assigned
                         minutes are assumed to be 0 and this will be
                         reflected in the assignment string.

                         A call for the same day is allowed as long as the
                         h_minutes == 0 in this call or the last call. If
                         the last call was made with h_minutes == 0 then the
                         h_minutes from this call will be used.

                         This is designed to facilitate creating a contour
                         string for a know date range. For example:

                         FC_ASSIGN_STRING(SC_PARAM(h_start_date), 0, 0);

                         while (h_date <= h_finish_date)
                         {
                             FC_ASSIGN_STRING(SC_PARAM(h_date), SC_PARAM(h_minutes));
                         }

                         FC_ASSIGN_STRING(SC_PARAM(h_finish_date), 0, 0);

                         The first and last call are made regardless whether
                         these calls will be/or have been made in the loop.
                         This is by design and eliminates the need for the
                         callers to do a lot of extra logic.

         Warning         The string is only updated when the assigned
                         minutes changes from the last call. All days must
                         be added in increasing order. Always use
                         FC_GET_ASSIGN_STR to get the results from all the
                         builds.

         Preconditions   FC_INI_ASSIGN_STR was called

         Locks

         Side Effects    Modifies global variables used for assignment
                         string

         Limitations     If the current assignment string
                         > SC_ASSIGN_STRING_LENGTH, function will fail.

         See Also        FC_GET_ASSIGN_STR, FC_INI_ASSIGN_STR

******X******************X************************X************************X$*/

Prompt == FUNCTION FC_ASSIGN_STRING ==
CREATE OR REPLACE FUNCTION FC_ASSIGN_STRING
(
     sh_h_julian_day PLS_INTEGER
   , sh_h_minutes PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_julian_day PLS_INTEGER := sh_h_julian_day;
   h_minutes PLS_INTEGER := sh_h_minutes;
   null_return LONG;
BEGIN
   If Coalesce(h_julian_day, 0) < ASSIGN_STR.g_assign_str_last_day Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_ASSIGN_STRING', 12+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_ASSIGN_STRING(13) : assign_str.sql(153) : ' || SYSTIMESTAMP || ' ' || 'ID: ' || '''' || 'day < last_day' || '''');
      END IF; 
      RETURN PMODEFINES.PMO_RETURN_ERROR(-1, 'day < last_day', False);
   End If;
   
   If Coalesce(h_julian_day, 0) = ASSIGN_STR.g_assign_str_last_day Then
      If Coalesce(h_minutes, 0) <> 0 AND ASSIGN_STR.g_assign_str_last_minutes <> 0 AND Coalesce(
            h_minutes, 0) <> ASSIGN_STR.g_assign_str_last_minutes Then
         RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
      End If;
      
      -- just in case this could be the second, and the first call could have been just for
      -- the date, with 0 minutes. In this case we use the non-zero value as the right one.
      If Coalesce(h_minutes, 0) <> 0 Then
         ASSIGN_STR.g_assign_str_last_minutes := Coalesce(h_minutes, 0);
      End If;
      RETURN 0;
   End If;
   
   If Length(ASSIGN_STR.g_assign_str_contour) > PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   If NOT ASSIGN_STR.g_assign_str_first_day <> 0 Then
      -- first call
      ASSIGN_STR.g_assign_str_contour := '';
      ASSIGN_STR.g_assign_str_first_day := Coalesce(h_julian_day, 0);
      ASSIGN_STR.g_assign_str_last_day := Coalesce(h_julian_day, 0);
      ASSIGN_STR.g_assign_str_last_minutes := Coalesce(h_minutes, 0);
      ASSIGN_STR.g_assign_str_had_calls := 0;
   Else
      If Coalesce(h_julian_day, 0) > ASSIGN_STR.g_assign_str_last_day + 1 
            AND (ASSIGN_STR.g_assign_str_last_minutes <> 0 OR Coalesce(h_minutes, 0) <> 0) Then
         -- skipped days have to be added as 0 assigned
         null_return := FC_ASSIGN_STRING(ASSIGN_STR.g_assign_str_last_day + 1, 0);
         
         -- remember that the above call changed g_assign_str_last_day to g_assign_str_last_day+1
         If Coalesce(h_julian_day, 0) - 1 > ASSIGN_STR.g_assign_str_last_day Then
            null_return := FC_ASSIGN_STRING(Coalesce(h_julian_day, 0) - 1, 0);
         End If;
      End If;
      
      If ASSIGN_STR.g_assign_str_last_minutes <> Coalesce(h_minutes, 0) Then
         If (ASSIGN_STR.g_assign_str_contour IS NULL) Then
            ASSIGN_STR.g_assign_str_contour := ASSIGN_STR.g_assign_str_contour || 
                                                                  TO_CHAR(ASSIGN_STR.g_assign_str_last_day 
                                                                  - ASSIGN_STR.g_assign_str_first_day 
                                                                  + 1) || '*' 
                                                                  || TO_CHAR(ASSIGN_STR.g_assign_str_last_minutes);
         Else
            ASSIGN_STR.g_assign_str_contour := ASSIGN_STR.g_assign_str_contour || ',' 
                                                                  || TO_CHAR(ASSIGN_STR.g_assign_str_last_day 
                                                                  - ASSIGN_STR.g_assign_str_first_day 
                                                                  + 1) || '*' 
                                                                  || TO_CHAR(ASSIGN_STR.g_assign_str_last_minutes);
         End If;
         
         ASSIGN_STR.g_assign_str_first_day := Coalesce(h_julian_day, 0);
         ASSIGN_STR.g_assign_str_last_minutes := Coalesce(h_minutes, 0);
      End If;
      
      ASSIGN_STR.g_assign_str_last_day := Coalesce(h_julian_day, 0);
   End If;
   
   ASSIGN_STR.g_assign_str_had_calls := ASSIGN_STR.g_assign_str_had_calls + 1;
   
   RETURN 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSIGN_STRING', 80+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ASSIGN_STRING(81) : assign_str.sql(221) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ASSIGN_STRING;
ALTER FUNCTION FC_ASSIGN_STRING COMPILE DEBUG;

/*$Func************************************************************************

      Synopsis           BOOL_RETURN_CODE FC_GET_ASSIGN_STR(SC_CHAR_PARAM h_contour[SC_H_8K_STR], short h_contour_Ind, SC_LONG_PARAM bIncludeTrailingZero)

         h_contour       buffer for returning the contour string

         h_contour_Ind   ignored, cleared to 0

         bIncludeTrailingZero  0 - if the lass assignment is 0 minutes
                         don't include it in the assignment string, else
                         include traling 0 assigments. Default is 0.

      Description        Returns the accumulated assignment string and
                         prepares the global variables for the next string.

      Return Value       0 if success, else failed

      Remarks            Multiple calls to  this function are allowed and
                         will return the same value. However, each call is
                         does a string copy so is not free.

         Warning         Function will fail if the no calls to
                         FC_ASSIGN_STRING are made after a call to
                         FC_INI_ASSIGN_STIRING

         Preconditions   at least one call to FC_ASSIGN_STRING

         Locks

         Side Effects    Clears global variables used for accumulating the
                         assignment string (except the assignment string
                         itself) in preparation for calls to
                         FC_ASSIGN_STRING.

         Limitations

         See Also

******X******************X************************X************************X$*/

Prompt == FUNCTION FC_GET_ASSIGN_STR ==
CREATE OR REPLACE FUNCTION FC_GET_ASSIGN_STR
(
     h_contour IN OUT NOCOPY VARCHAR2
   , sh_bIncludeTrailingZero PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   bIncludeTrailingZero PLS_INTEGER := Coalesce(sh_bIncludeTrailingZero, 0);
   null_return LONG;
BEGIN
   If NOT ASSIGN_STR.g_assign_str_first_day <> 0 AND NOT ASSIGN_STR.g_assign_str_had_calls <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_GET_ASSIGN_STR', 11+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_GET_ASSIGN_STR(12) : assign_str.sql(281) : ' || SYSTIMESTAMP || ' ' || 'ID: ' || '''' || 'FC_ASSIGN_STRING was not called' || '''');
      END IF; 
      RETURN PMODEFINES.PMO_RETURN_ERROR(-1, 'FC_ASSIGN_STRING was not called', False);
   End If;
   
   If ASSIGN_STR.g_assign_str_first_day <> 0 AND (ASSIGN_STR.g_assign_str_last_minutes <> 0 
         OR bIncludeTrailingZero <> 0) Then
      null_return := FC_ASSIGN_STRING(ASSIGN_STR.g_assign_str_last_day + 1
                                , ASSIGN_STR.g_assign_str_last_minutes + 1);
   End If;
   
   h_contour := ASSIGN_STR.g_assign_str_contour;
   null_return := FC_INI_ASSIGN_STR(0);
   
   RETURN 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ASSIGN_STR', 27+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_ASSIGN_STR(28) : assign_str.sql(297) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_ASSIGN_STR;
ALTER FUNCTION FC_GET_ASSIGN_STR COMPILE DEBUG;

Prompt == FUNCTION SP__PROFILE_ASSIGN_STR ==
CREATE OR REPLACE FUNCTION SP__PROFILE_ASSIGN_STR RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP__PROFILE_ASSIGN_STR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP__PROFILE_ASSIGN_STR'), '-- ' || SYSTIMESTAMP, ':r := SP__PROFILE_ASSIGN_STR();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP__PROFILE_ASSIGN_STR', 10+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP__PROFILE_ASSIGN_STR(11) : assign_str.sql(316) : ' || SYSTIMESTAMP);
   END IF; 

   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP__PROFILE_ASSIGN_STR', 17+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP__PROFILE_ASSIGN_STR(18) : assign_str.sql(323) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP__PROFILE_ASSIGN_STR', 29+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP__PROFILE_ASSIGN_STR(30) : assign_str.sql(335) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP__PROFILE_ASSIGN_STR;
ALTER FUNCTION SP__PROFILE_ASSIGN_STR COMPILE DEBUG;

Prompt == FUNCTION SP__TEST_ASSIGN_STR ==
CREATE OR REPLACE FUNCTION SP__TEST_ASSIGN_STR RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP__TEST_ASSIGN_STR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP__TEST_ASSIGN_STR'), '-- ' || SYSTIMESTAMP, ':r := SP__TEST_ASSIGN_STR();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP__TEST_ASSIGN_STR', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP__TEST_ASSIGN_STR(9) : assign_str.sql(358) : ' || SYSTIMESTAMP);
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP__TEST_ASSIGN_STR', 12+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP__TEST_ASSIGN_STR(13) : assign_str.sql(362) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION SP__TEST_ASSIGN_STR;
ALTER FUNCTION SP__TEST_ASSIGN_STR COMPILE DEBUG;

