/****************************************************************************************/

Prompt == FUNCTION SP_I_COMPET ==
CREATE OR REPLACE FUNCTION SP_I_COMPET
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_uniqueID VARCHAR2(32);
   h_level PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_COMPET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_COMPET'), '-- ' || SYSTIMESTAMP, ':r := SP_I_COMPET(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_COMPET', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_COMPET(30) : competencies.sql(33) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_uniqueID := SYS_GUID();
   
   -- Validate parent_id
   If (p_parent_id IS NULL) Then
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_COMPETENCIES', h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1
         INTO h_rank, h_level_id
         FROM TMT_COMPETENCIES WHERE ELEMENT_ID IS NULL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 51+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(52) : competencies.sql(55) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_COMPETENCIES(
         RANK, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID,
         CREATED_BY, EXPERIENCE_ID, PROFICIENCY_LEVEL_ID,
         REC_USER)
         VALUES (
         h_rank, h_uniqueID, p_name,
         p_type_id, h_level_id,
         p_rec_user, 2771, 0,
         p_rec_user);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 77+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(78) : competencies.sql(81) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_I_COMPET', 84+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_I_COMPET(85) : competencies.sql(88) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      PMODEFINES.PMO_GET_MAX_RANK('TMT_COMPETENCIES', h_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1
         INTO h_rank, h_level_id
         FROM TMT_COMPETENCIES WHERE ELEMENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 107+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(108) : competencies.sql(111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (Coalesce(p_type_id, 0) = PMO_CNST.SC_ROLE_CATEGORY) OR (Coalesce(
             p_type_id, 0) = PMO_CNST.SC_ROLE_CLASSIFICATION) OR (Coalesce(
             p_type_id, 0) = PMO_CNST.SC_ROLE) OR (Coalesce(
             p_type_id, 0) = PMO_CNST.SC_SKILL_CATEGORY) OR (Coalesce(
             p_type_id, 0) = PMO_CNST.SC_SKILL_CLASSIFICATION) OR (Coalesce(
             p_type_id, 0) = PMO_CNST.SC_SKILL) Then
         h_type_id := 0;
      Else
         h_type_id := 725;
      End If;
      
      BEGIN
         INSERT INTO TMT_COMPETENCIES(
         RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID,
         CREATED_BY, EXPERIENCE_ID, PROFICIENCY_LEVEL_ID,
         REC_USER)
         VALUES(
         h_rank, p_parent_id, h_uniqueID, p_name,
         p_type_id, h_level_id,
         p_rec_user, 2771, h_type_id,
         p_rec_user);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 144+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(145) : competencies.sql(148) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_COMPET', 151+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_COMPET(152) : competencies.sql(155) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 158+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(159) : competencies.sql(162) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_COMPET', 170+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_COMPET(171) : competencies.sql(174) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_COMPET;
ALTER FUNCTION SP_I_COMPET COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_COMPET ==
CREATE OR REPLACE FUNCTION SP_U_COMPET
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_experience PLS_INTEGER
   , p_preference_level_id PLS_INTEGER
   , p_last_used_datetime VARCHAR2
   , p_last_used_location VARCHAR2
   , p_target_attainment_date VARCHAR2
   , p_acquired_on VARCHAR2
   , p_mandatory VARCHAR2
   , p_mandatory_details VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_COMPET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_COMPET'), '-- ' || SYSTIMESTAMP, ':r := SP_U_COMPET(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || COALESCE(TO_CHAR(p_experience), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_preference_level_id), 'NULL')
          || ', ' || '''' || p_last_used_datetime || ''''
          || ', ' || '''' || p_last_used_location || ''''
          || ', ' || '''' || p_target_attainment_date || ''''
          || ', ' || '''' || p_acquired_on || ''''
          || ', ' || '''' || p_mandatory || ''''
          || ', ' || '''' || p_mandatory_details || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET', 40+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_COMPET(41) : competencies.sql(231) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_COMPET', 45+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_COMPET(46) : competencies.sql(236) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET', 48+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_COMPET(49) : competencies.sql(239) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_COMPET', 57+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_COMPET(58) : competencies.sql(248) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   BEGIN
      UPDATE TMT_COMPETENCIES SET
      ELEMENT_NAME = p_name,
      DESCRIPTION = p_description,
      EXPERIENCE_ID = p_experience,
      PREFERENCE_LEVEL_ID = p_preference_level_id,
      LAST_USED_DATETIME = p_last_used_datetime,
      LAST_USED_LOCATION = p_last_used_location,
      REC_STATUS = 'U',
      REC_DATETIME = SYSTIMESTAMP,
      TARGET_ATTAINMENT_DATE = p_target_attainment_date,
      ACQUIRED_ON = p_acquired_on,
      MANDATORY = p_mandatory,
      MANDATORY_DETAILS = p_mandatory_details,
      REC_USER = p_rec_user
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET', 94+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET(95) : competencies.sql(285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_COMPET', 100+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_COMPET(101) : competencies.sql(291) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET', 107+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET(108) : competencies.sql(298) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET', 119+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET(120) : competencies.sql(310) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_COMPET;
ALTER FUNCTION SP_U_COMPET COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_COMP_PROF ==
CREATE OR REPLACE FUNCTION SP_U_COMP_PROF
(
     p_id VARCHAR2
   , p_proficiency_level_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_name VARCHAR2(100);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_COMP_PROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_COMP_PROF'), '-- ' || SYSTIMESTAMP, ':r := SP_U_COMP_PROF(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_proficiency_level_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMP_PROF', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_COMP_PROF(22) : competencies.sql(348) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_COMP_PROF', 26+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_COMP_PROF(27) : competencies.sql(353) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMP_PROF', 29+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_COMP_PROF(30) : competencies.sql(356) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_COMP_PROF', 38+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_COMP_PROF(39) : competencies.sql(365) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   If ((p_proficiency_level_id IS NOT NULL)) AND (Coalesce(p_proficiency_level_id, 0) <> 0) Then
      BEGIN
         SELECT COALESCE(DESCRIPTION, '')
         INTO h_name
         FROM TMT_DATAFIELDS
         WHERE ELEMENT_ID = p_proficiency_level_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMP_PROF', 61+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMP_PROF(62) : competencies.sql(388) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --strcpy(h_name, "");
      h_name := '';
   End If;
   
   BEGIN
      UPDATE TMT_COMPETENCIES SET
      ELEMENT_NAME = h_name,
      PROFICIENCY_LEVEL_ID = p_proficiency_level_id,
      REC_STATUS = 'U',
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER = p_rec_user
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMP_PROF', 88+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMP_PROF(89) : competencies.sql(415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_COMP_PROF', 94+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_COMP_PROF(95) : competencies.sql(421) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMP_PROF', 101+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMP_PROF(102) : competencies.sql(428) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMP_PROF', 113+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMP_PROF(114) : competencies.sql(440) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_COMP_PROF;
ALTER FUNCTION SP_U_COMP_PROF COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION SP_U_COMPET_RATE ==
CREATE OR REPLACE FUNCTION SP_U_COMPET_RATE
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_regular_cost NUMBER
   , p_regular_sale NUMBER
   , p_special_cost NUMBER
   , p_special_sale NUMBER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_profile_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_prj_exchange_rate NUMBER := 0.00;
   h_prj_currency_id PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_COMPET_RATE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_COMPET_RATE'), '-- ' || SYSTIMESTAMP, ':r := SP_U_COMPET_RATE(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_regular_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_regular_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_special_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_special_sale), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_RATE', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_COMPET_RATE(33) : competencies.sql(489) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_COMPET_RATE', 37+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_COMPET_RATE(38) : competencies.sql(494) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_RATE', 40+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_COMPET_RATE(41) : competencies.sql(497) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 57+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(58) : competencies.sql(514) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_uniqueID := SYS_GUID();
   
   BEGIN
      INSERT INTO RATES(
      RATE_ID,
      REGULAR_COST,
      REGULAR_SALE,
      SPECIAL_COST,
      SPECIAL_SALE,
      REC_USER
      ) VALUES (
      h_uniqueID,
      p_regular_cost,
      p_regular_sale,
      p_special_cost,
      p_special_sale,
      p_rec_user
      );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 90+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(91) : competencies.sql(547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (p_parent_id IS NULL) Then
      BEGIN
         UPDATE TMT_COMPETENCIES SET
         RATE_ID = h_uniqueID,
         REC_STATUS = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 113+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(114) : competencies.sql(570) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_COMPETENCIES SET
         RATE_ID = h_uniqueID,
         REC_STATUS = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER = p_rec_user
         WHERE ELEMENT_ID = p_id
         AND PARENT_ID = p_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 136+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(137) : competencies.sql(593) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   /*  if (h_type_id != SC_RESOURCE)
       {
           BEGIN_SQL
               DECLARE C_PROF CURSOR WITH HOLD FOR
               SELECT PROJECT_ID, PROFILE_ID
               FROM PROFILES WHERE ROLE_ID = HOST_DB(p_id, ind_u_compet_rates);
           END_SQL_NOCHECK;
   
           BEGIN_SQL
               OPEN C_PROF;
           END_SQL;
   
           while(1==1)
           {
               BEGIN_SQL
                   FETCH C_PROF INTO HOST(h_project_id), HOST(h_profile_id);
               END_SQL;
               if (SC_STATE != 0) break;
   
               //currency conversion from system to project
               BEGIN_SQL
                   SELECT CURRENCY_ID INTO HOST(h_prj_currency_id)
                   FROM TMT_WBS WHERE ELEMENT_ID = HOST(h_project_id);
               END_SQL;
   
               BEGIN_SQL
                   SELECT DECIMAL(ELEMENT_NAME, 18, 3) INTO HOST(h_prj_exchange_rate)
                   FROM TMT_DATAFIELDS WHERE ELEMENT_ID = HOST(h_prj_currency_id);
               END_SQL;
   
               BEGIN_SQL
                   UPDATE TASK_ASSIGNMENTS SET
                       REGULAR_COST = HOST_DB(p_regular_cost, ind_u_compet_rates) * HOST(h_prj_exchange_rate),
                       SPECIAL_COST = HOST_DB(p_special_cost, ind_u_compet_rates) * HOST(h_prj_exchange_rate)
                   WHERE RESOURCE_ID = HOST(h_profile_id)
                   AND PROJECT_ID = HOST(h_project_id);
               END_SQL;
   
           }
   
           BEGIN_SQL
               CLOSE C_PROF WITH RELEASE;
           END_SQL;
       }
   */
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_COMPET_RATE', 188+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_COMPET_RATE(189) : competencies.sql(645) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 195+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(196) : competencies.sql(652) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_RATE', 207+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_RATE(208) : competencies.sql(664) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_COMPET_RATE;
ALTER FUNCTION SP_U_COMPET_RATE COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION SP_U_COMPET_DF ==
CREATE OR REPLACE FUNCTION SP_U_COMPET_DF
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_default_flag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_resource_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_COMPET_DF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_COMPET_DF'), '-- ' || SYSTIMESTAMP, ':r := SP_U_COMPET_DF(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_default_flag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_DF', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_COMPET_DF(22) : competencies.sql(702) : ' || SYSTIMESTAMP);
   END IF; 

   If ((p_id IS NULL)) OR ((p_default_flag IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_COMPET_DF', 26+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_COMPET_DF(27) : competencies.sql(707) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_DF', 29+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_COMPET_DF(30) : competencies.sql(710) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   BEGIN
      SELECT RESOURCE_ID INTO h_resource_id
      FROM TMT_COMPETENCIES WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(46) : competencies.sql(726) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(p_default_flag, 0) = 1 Then
      -- default
      BEGIN
         UPDATE TMT_COMPETENCIES
         SET DEFAULT_FLAG = '0'
         WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(67) : competencies.sql(747) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_COMPETENCIES
         SET DEFAULT_FLAG = '1'
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(86) : competencies.sql(766) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE RESOURCES
         SET PRIMARY_ROLE = p_parent_id
         WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 104+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(105) : competencies.sql(785) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_COMPETENCIES
         SET DEFAULT_FLAG = '0'
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 123+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(124) : competencies.sql(804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_COMPET_DF', 130+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_COMPET_DF(131) : competencies.sql(811) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 137+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(138) : competencies.sql(818) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_DF', 149+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_DF(150) : competencies.sql(830) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_COMPET_DF;
ALTER FUNCTION SP_U_COMPET_DF COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION SP_U_COMPET_STRATEGIC ==
CREATE OR REPLACE FUNCTION SP_U_COMPET_STRATEGIC
(
     p_id VARCHAR2
   , p_strategic_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_host_table PLS_INTEGER := 0;
   h_id VARCHAR2(32);
   h_strategic_flag VARCHAR2(1);
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_COMPET_STRATEGIC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_COMPET_STRATEGIC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_COMPET_STRATEGIC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_strategic_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_STRATEGIC', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_COMPET_STRATEGIC(24) : competencies.sql(870) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_id IS NULL)) OR ((p_strategic_flag IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_COMPET_STRATEGIC', 30+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_COMPET_STRATEGIC(31) : competencies.sql(877) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_COMPET_STRATEGIC', 33+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_COMPET_STRATEGIC(34) : competencies.sql(880) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_host_table := 0;
   h_id := p_id;
   h_strategic_flag := p_strategic_flag;
   h_rec_user := p_rec_user;
   h_error_id := h_id;
   
   PMO_DB2M.SC_STATE := FC_U_ITEM_STRATEGIC(h_host_table, h_id, h_strategic_flag, h_rec_user
                                         , h_error_id);
   
   --trace_printf("\n 713 SC_STATE %d  ---  h_id %s",SC_STATE,h_id);//trace_flush();
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_COMPET_STRATEGIC', 49+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_COMPET_STRATEGIC(50) : competencies.sql(896) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_STRATEGIC', 56+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_STRATEGIC(57) : competencies.sql(903) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_COMPET_STRATEGIC', 68+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_COMPET_STRATEGIC(69) : competencies.sql(915) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_COMPET_STRATEGIC;
ALTER FUNCTION SP_U_COMPET_STRATEGIC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_COMPET ==
CREATE OR REPLACE FUNCTION SP_D_COMPET
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_exists VARCHAR2(32);
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_count PLS_INTEGER := 0;
   h_dependency_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC20 IS
         SELECT TC.ELEMENT_ID FROM TMT_COMPETENCIES TC, PROFILES P
         WHERE  ROWNUM <= 1 AND  P.ROLE_ID = TC.ELEMENT_ID
         AND TC.RANK LIKE h_rank || '%';

      CURSOR CC2A IS
         SELECT TC.ELEMENT_ID FROM TMT_COMPETENCIES TC, RESOURCE_ASSIGN_ROLES RAR
         WHERE  ROWNUM <= 1 AND  RAR.ROLE_ID = TC.ELEMENT_ID
         AND TC.RANK LIKE h_rank || '%';

      CURSOR CC2B IS
         SELECT TC.ELEMENT_ID FROM TMT_COMPETENCIES TC, RESOURCE_ASSIGN_SKILLS RAS
         WHERE  ROWNUM <= 1 AND  RAS.SKILL_ID = TC.ELEMENT_ID
         AND TC.RANK LIKE h_rank || '%';

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_COMPET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_COMPET'), '-- ' || SYSTIMESTAMP, ':r := SP_D_COMPET(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 42+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_COMPET(43) : competencies.sql(974) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_COMPET', 50+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_COMPET(51) : competencies.sql(982) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      IF h_rec_status = 'X' THEN 
         EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || 'TMT_COMPETENCIES' || ' a 
         WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS 
         (SELECT b.REC_STATUS 
                  FROM TMT_ORGANIZATIONS b 
                 WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)' 
            INTO h_rec_status 
            Using h_id; 
      ELSE 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || 'TMT_COMPETENCIES' || ' 
              WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') ' 
            INTO h_rec_status 
            Using h_id; 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, h_id, True); 
      WHEN OTHERS THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id , True); 
   END;
   
   BEGIN
      SELECT COUNT(*) INTO h_count
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = p_id
      AND ELEMENT_ID NOT IN ('DEFAULTCATEGORY', 'DEFAULTCLASS', 'DEFAULTCOMPETENCY', 'DEFAULTLEVEL');
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 85+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(86) : competencies.sql(1017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_COMPET', 92+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_COMPET(93) : competencies.sql(1024) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 95+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(96) : competencies.sql(1027) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE, p_id, True);
   End If;
   
   If (p_parent_id IS NULL) Then
      BEGIN
         SELECT RANK INTO h_rank
         FROM TMT_COMPETENCIES
         WHERE ELEMENT_ID = p_id AND PARENT_ID IS NULL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 111+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(112) : competencies.sql(1043) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --strcpy(h_rec_status, "");
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_COMPET', 121+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_COMPET(122) : competencies.sql(1053) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_COMPET', 133+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_COMPET(134) : competencies.sql(1065) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 136+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(137) : competencies.sql(1068) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_COMPETENCIES
         WHERE ELEMENT_ID = p_id
         AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(152) : competencies.sql(1083) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      SELECT COUNT(*) INTO h_count
      FROM TMT_COMPETENCIES
      WHERE RANK LIKE h_rank || '%'
      AND TYPE_ID = PMO_CNST.SC_RESOURCE;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 168+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(169) : competencies.sql(1100) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_COMPET', 175+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_COMPET(176) : competencies.sql(1107) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 178+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(179) : competencies.sql(1110) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE, p_id, True);
   End If;
   
   --???
   
   OPEN CC20;
   
   FETCH CC20 INTO h_exists;
   IF CC20%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_dependency_count := 1;
   Else
      h_dependency_count := 0;
   End If;
   
   BEGIN
      CLOSE CC20 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 208+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(209) : competencies.sql(1140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_dependency_count, 0) = 0 Then
      OPEN CC2A;
      
      FETCH CC2A INTO h_exists;
      IF CC2A%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_dependency_count := 1;
      Else
         h_dependency_count := 0;
      End If;
      
      BEGIN
         CLOSE CC2A ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 237+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(238) : competencies.sql(1169) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_dependency_count, 0) = 0 Then
         OPEN CC2B;
         
         FETCH CC2B INTO h_exists;
         IF CC2B%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_dependency_count := 1;
         Else
            h_dependency_count := 0;
         End If;
         
         BEGIN
            CLOSE CC2B ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 266+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(267) : competencies.sql(1198) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --hack
   If Coalesce(h_dependency_count, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_COMPET', 276+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_COMPET(277) : competencies.sql(1208) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE_RECORD), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 279+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(280) : competencies.sql(1211) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE_RECORD, p_id, True);
   End If;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      If Coalesce(h_dependency_count, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_COMPET', 287+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_COMPET(288) : competencies.sql(1219) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE_RECORD), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 290+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(291) : competencies.sql(1222) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE_RECORD, p_id, True);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_COMPET', 295+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_COMPET(296) : competencies.sql(1227) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 298+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(299) : competencies.sql(1230) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(0, p_id, True);
      End If;
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      --Delete Tree
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      BEGIN
         DELETE FROM TMT_COMPETENCIES WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 332+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(333) : competencies.sql(1264) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Update Parent's Hierarchy Info
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_COMPETENCIES
               SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID IS NULL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 354+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(355) : competencies.sql(1286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_COMPETENCIES
               SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 373+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(374) : competencies.sql(1305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_COMPETENCIES
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IS NULL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 394+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(395) : competencies.sql(1326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_COMPETENCIES
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 413+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(414) : competencies.sql(1345) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_COMPET', 422+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_COMPET(423) : competencies.sql(1354) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_COMPET', 425+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_COMPET(426) : competencies.sql(1357) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      -- Update Parent's Hierarchy Info
      If (p_parent_id IS NULL) Then
         BEGIN
            UPDATE TMT_COMPETENCIES
            SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID IS NULL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 446+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(447) : competencies.sql(1378) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_COMPETENCIES
            SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 465+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(466) : competencies.sql(1397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- Lock Tree
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_COMPETENCIES SET
         REC_STATUS   = p_del_flag,
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 501+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(502) : competencies.sql(1433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_COMPET', 508+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_COMPET(509) : competencies.sql(1440) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 515+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(516) : competencies.sql(1447) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_COMPET', 527+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_COMPET(528) : competencies.sql(1459) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_COMPET;
ALTER FUNCTION SP_D_COMPET COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ROLE ==
CREATE OR REPLACE FUNCTION SP_S_ROLE
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_sql VARCHAR2(2048);
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ROLE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ROLE'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ROLE(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ROLE', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ROLE(20) : competencies.sql(1495) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_id IS NULL)) Then
      If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
         -- Expand 3 levels (0, 1, 2)  by default
         h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
         h_sql := h_sql || 'T.ELEMENT_NAME, ';
         h_sql := h_sql || 'CASE WHEN T.LEVEL_ID = 3 THEN 0 ELSE T.CHILD_COUNT END AS CHILD_COUNT, ';
         h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.CREATED_BY, ';
         h_sql := h_sql || 
                  '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.CREATED_BY) AS CREATED_BY_NAME, ';
         h_sql := h_sql || 
                  'T.CREATED_DATETIME, T.PROFICIENCY_LEVEL_ID, T.PREFERENCE_LEVEL_ID, R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE, ';
         h_sql := h_sql || 'T.EXPERIENCE_ID, T.LAST_USED_DATETIME, T.LAST_USED_LOCATION, T.DEFAULT_FLAG ';
         h_sql := h_sql || 'FROM TMT_COMPETENCIES T LEFT OUTER JOIN RATES R ON R.RATE_ID = T.RATE_ID ';
         h_sql := h_sql || 'WHERE T.TYPE_ID IN (501, 71, 72, 30, 73) ';
         h_sql := h_sql || 'AND T.LEVEL_ID < 4 ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 ';
         h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 13 ORDER BY RANK ';
      Else
         h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
         h_sql := h_sql || 'T.ELEMENT_NAME, ';
         h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_COMPETENCIES WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
         h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.CREATED_BY, ';
         h_sql := h_sql || 
                  '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.CREATED_BY) AS CREATED_BY_NAME, ';
         h_sql := h_sql || 
                  'T.CREATED_DATETIME, T.PROFICIENCY_LEVEL_ID, T.PREFERENCE_LEVEL_ID, R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE, ';
         h_sql := h_sql || 'T.EXPERIENCE_ID, T.LAST_USED_DATETIME, T.LAST_USED_LOCATION, T.DEFAULT_FLAG ';
         h_sql := h_sql || 'FROM TMT_COMPETENCIES T LEFT OUTER JOIN RATES R ON R.RATE_ID = T.RATE_ID ';
         h_sql := h_sql || 'WHERE T.TYPE_ID IN (501, 71, 72, 30, 73) ';
         h_sql := h_sql || 'AND T.PARENT_ID IS NULL ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 ';
         
         If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 1) Then
            h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         End If;
         h_sql := h_sql || 'AND T.TYPE_ID <> 13 ORDER BY RANK ';
      End If;
   Else
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_ROLE', 66+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_ROLE(67) : competencies.sql(1542) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
      h_sql := h_sql || 'T.ELEMENT_NAME, ';
      h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(CHILD_COUNT) FROM TMT_COMPETENCIES WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.CREATED_BY, ';
      h_sql := h_sql || 
                  '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.CREATED_BY) AS CREATED_BY_NAME, ';
      h_sql := h_sql || 
                  'T.CREATED_DATETIME, T.PROFICIENCY_LEVEL_ID, T.PREFERENCE_LEVEL_ID, R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE, ';
      h_sql := h_sql || 'T.EXPERIENCE_ID, T.LAST_USED_DATETIME, T.LAST_USED_LOCATION, T.DEFAULT_FLAG ';
      h_sql := h_sql || 'FROM TMT_COMPETENCIES T LEFT OUTER JOIN RATES R ON R.RATE_ID = T.RATE_ID ';
      h_sql := h_sql || 'WHERE T.TYPE_ID IN (501, 71, 72, 30, 73) ';
      
      If Coalesce(p_bitflag, 0) = PMO_CNST.SC_REFRESH Then
         h_sql := h_sql || 'AND T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' ';
      Else
         h_sql := h_sql || 'AND (T.PARENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''') ';
         
         If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 1) Then
            h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         End If;
      End If;
      
      h_sql := h_sql || 'AND T.TYPE_ID <> 13 ORDER BY RANK ';
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ROLE', 110+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ROLE(111) : competencies.sql(1586) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ROLE', 113+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ROLE(114) : competencies.sql(1589) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ROLE', 120+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ROLE(121) : competencies.sql(1596) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ROLE', 132+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ROLE(133) : competencies.sql(1608) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ROLE;
ALTER FUNCTION SP_S_ROLE COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_SKILL ==
CREATE OR REPLACE FUNCTION SP_S_SKILL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_SKILL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_SKILL'), '-- ' || SYSTIMESTAMP, ':r := SP_S_SKILL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_SKILL', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_SKILL(20) : competencies.sql(1644) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Execute Select Statement
   h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
   h_sql := h_sql || 'T.ELEMENT_NAME, ';
   h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_COMPETENCIES WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
   h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.CREATED_BY, ';
   h_sql := h_sql || 
                  '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.CREATED_BY) AS CREATED_BY_NAME, ';
   h_sql := h_sql || 
                  'T.CREATED_DATETIME, T.PROFICIENCY_LEVEL_ID, T.PREFERENCE_LEVEL_ID, R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE, ';
   h_sql := h_sql || 'T.EXPERIENCE_ID, T.LAST_USED_DATETIME, T.LAST_USED_LOCATION ';
   h_sql := h_sql || 'FROM TMT_COMPETENCIES T LEFT OUTER JOIN RATES R ON R.RATE_ID = T.RATE_ID ';
   h_sql := h_sql || 'WHERE T.TYPE_ID IN (502, 74, 75, 29, 76) ';
   
   -- Validate id
   If ((p_id IS NULL)) Then
      h_sql := h_sql || 'AND T.PARENT_ID IS NULL ';
      h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
      h_sql := h_sql || 'AND T.TYPE_ID <> 0 ';
      h_sql := h_sql || 'AND T.TYPE_ID <> 13 ORDER BY RANK ';
   Else
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_SKILL', 47+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_SKILL(48) : competencies.sql(1672) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      If Coalesce(p_bitflag, 0) = PMO_CNST.SC_REFRESH Then
         h_sql := h_sql || 'AND T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' ';
      Else
         h_sql := h_sql || 'AND (T.PARENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''') ';
         
         If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 1) Then
            h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         End If;
      End If;
      
      h_sql := h_sql || 'AND T.TYPE_ID <> 13 ORDER BY RANK ';
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_SKILL', 78+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_SKILL(79) : competencies.sql(1703) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_SKILL', 81+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_SKILL(82) : competencies.sql(1706) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_SKILL', 88+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_SKILL(89) : competencies.sql(1713) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_SKILL', 100+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_SKILL(101) : competencies.sql(1725) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_SKILL;
ALTER FUNCTION SP_S_SKILL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_A_RES_COMPET ==
CREATE OR REPLACE FUNCTION SP_A_RES_COMPET
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_exists VARCHAR2(32);
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_resource_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   h_level PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_proficiency_level_id PLS_INTEGER := 0;
   h_resource PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RES_COMPET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RES_COMPET'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RES_COMPET(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_COMPET', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RES_COMPET(33) : competencies.sql(1774) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_id IS NULL)) OR ((p_resource_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_COMPET', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_COMPET(40) : competencies.sql(1781) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_COMPET', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_COMPET(43) : competencies.sql(1784) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_A_RES_COMPET', 49+1) THEN 
      LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_A_RES_COMPET(50) : competencies.sql(1791) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_COMPETENCIES' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_COMPETENCIES' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   h_resource_id := p_resource_id;
   BEGIN 
      SELECT REC_STATUS INTO h_rec_status FROM RESOURCES WHERE RESOURCE_ID = h_resource_id; 
      IF SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') THEN 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True); 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_resource_id, True); 
   END;
   
   BEGIN
      SELECT PARENT_ID INTO h_parent_id
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 81+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(82) : competencies.sql(1823) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_exists
      FROM TMT_COMPETENCIES
      WHERE RESOURCE_ID = p_resource_id
      AND PARENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(98) : competencies.sql(1839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RES_COMPET', 104+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RES_COMPET(105) : competencies.sql(1846) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_COMPET', 107+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_COMPET(108) : competencies.sql(1849) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS, True);
   End If;
   
   -- check for same competencies but different level
   BEGIN
      SELECT COUNT(TC.ELEMENT_ID) INTO h_count
      FROM TMT_COMPETENCIES TC
      WHERE TC.RESOURCE_ID = h_resource_id
      AND EXISTS(
      SELECT ELEMENT_ID
      FROM TMT_COMPETENCIES
      WHERE PARENT_ID = h_parent_id
      AND ELEMENT_ID = TC.PARENT_ID);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 128+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(129) : competencies.sql(1870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_COMPET', 135+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_COMPET(136) : competencies.sql(1877) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_COMPETENCY), 'NULL') || ' ID: ' || '''' || h_resource_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_COMPET', 138+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_COMPET(139) : competencies.sql(1880) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_COMPETENCY, h_resource_id
            , True);
   End If;
   
   PMODEFINES.PMO_GET_MAX_RANK('TMT_COMPETENCIES', h_id, h_max_rank);
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   h_uniqueID := SYS_GUID();
   
   BEGIN
      SELECT RTRIM(Rank) || h_rankBase64, LEVEL_ID + 1, PROFICIENCY_LEVEL_ID
      INTO h_rank, h_level_id, h_proficiency_level_id
      FROM TMT_COMPETENCIES WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 158+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(159) : competencies.sql(1900) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO TMT_COMPETENCIES(
      RANK,
      PARENT_ID,
      ELEMENT_ID,
      TYPE_ID,
      LEVEL_ID,
      RESOURCE_ID,
      CREATED_BY,
      PROFICIENCY_LEVEL_ID,
      EXPERIENCE_ID,
      REC_USER)
      VALUES(
      h_rank,
      h_id,
      h_uniqueID,
      PMO_CNST.SC_RESOURCE,
      h_level_id,
      h_resource_id,
      p_rec_user,
      h_proficiency_level_id,
      2771,
      p_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 196+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(197) : competencies.sql(1938) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RES_COMPET', 202+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RES_COMPET(203) : competencies.sql(1944) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 209+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(210) : competencies.sql(1951) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_COMPET', 221+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_COMPET(222) : competencies.sql(1963) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RES_COMPET;
ALTER FUNCTION SP_A_RES_COMPET COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UA_RES_COMPET ==
CREATE OR REPLACE FUNCTION SP_UA_RES_COMPET
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UA_RES_COMPET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UA_RES_COMPET'), '-- ' || SYSTIMESTAMP, ':r := SP_UA_RES_COMPET(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UA_RES_COMPET', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UA_RES_COMPET(20) : competencies.sql(1999) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT PARENT_ID INTO h_id
      FROM TMT_COMPETENCIES WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_COMPET', 31+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_COMPET(32) : competencies.sql(2011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         UPDATE TMT_COMPETENCIES
         SET CHILD_COUNT = CHILD_COUNT - 1
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_COMPET', 51+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_COMPET(52) : competencies.sql(2031) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         -- Execute Delete Statement
         BEGIN
            DELETE FROM TMT_COMPETENCIES
            WHERE ELEMENT_ID = p_id
            AND RESOURCE_ID = p_resource_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_COMPET', 72+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_COMPET(73) : competencies.sql(2052) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UA_RES_COMPET', 80+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UA_RES_COMPET(81) : competencies.sql(2060) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_COMPET', 87+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_COMPET(88) : competencies.sql(2067) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_COMPET', 99+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_COMPET(100) : competencies.sql(2079) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UA_RES_COMPET;
ALTER FUNCTION SP_UA_RES_COMPET COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_ROLE ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_ROLE
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ2 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_ROLE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_ROLE'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_ROLE(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ROLE', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_ROLE(27) : competencies.sql(2122) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_ROLE', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_ROLE(32) : competencies.sql(2127) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ROLE', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ROLE(35) : competencies.sql(2130) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_id := p_id;
   
   BEGIN
      SELECT RANK
      INTO h_rank
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID =h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ROLE', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ROLE(53) : competencies.sql(2148) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --if (!(IsNullSpParam(ind_s_res_compet, p_bitflag))  (*p_bitflag == 2))
   If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 2) Then
      OPEN SELECT_READ1 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      T1.LEVEL_ID AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION, T1.CREATED_BY,
      (SELECT FULL_NAME FROM RESOURCES
      WHERE RESOURCE_ID = T1.CREATED_BY) AS CREATED_BY_NAME,
      T1.CREATED_DATETIME, T1.PROFICIENCY_LEVEL_ID, T1.PREFERENCE_LEVEL_ID,
      R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE,
      T1.EXPERIENCE_ID, T1.LAST_USED_DATETIME, T1.LAST_USED_LOCATION,
      T1.DEFAULT_FLAG, T1.PARENT_ID AS ORIGINAL_ID, T1.STRATEGIC_FLAG,
      COALESCE(T1.TARGET_ATTAINMENT_DATE, TO_DATE('1899-12-30')) AS TARGET_ATTAINMENT_DATE,
      COALESCE(T1.ACQUIRED_ON, TO_DATE('1899-12-30')) AS ACQUIRED_ON, T1.MANDATORY, T1.MANDATORY_DETAILS
      FROM TMT_COMPETENCIES T1 LEFT OUTER JOIN RATES R ON R.RATE_ID = T1.RATE_ID
      WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
      AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
      AND T1.TYPE_ID IN (501, 71, 72, 30, 73)
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ROLE', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ROLE(78) : competencies.sql(2173) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ1;
   Else
      OPEN SELECT_READ2 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID,
      (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = (SELECT PARENT_ID FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID)), ', '), (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID))) AS ELEMENT_NAME,
      1 AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION, T1.CREATED_BY,
      (SELECT FULL_NAME FROM RESOURCES
      WHERE RESOURCE_ID = T1.CREATED_BY) AS CREATED_BY_NAME,
      T1.CREATED_DATETIME, T1.PROFICIENCY_LEVEL_ID, T1.PREFERENCE_LEVEL_ID,
      R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE,
      T1.EXPERIENCE_ID, T1.LAST_USED_DATETIME, T1.LAST_USED_LOCATION,
      T1.DEFAULT_FLAG, T1.PARENT_ID AS ORIGINAL_ID, T1.STRATEGIC_FLAG,
      COALESCE(T1.TARGET_ATTAINMENT_DATE, TO_DATE('1899-12-30')) AS TARGET_ATTAINMENT_DATE,
      COALESCE(T1.ACQUIRED_ON, TO_DATE('1899-12-30')) AS ACQUIRED_ON, T1.MANDATORY, T1.MANDATORY_DETAILS
      FROM TMT_COMPETENCIES T1 LEFT OUTER JOIN RATES R ON R.RATE_ID = T1.RATE_ID
      WHERE T1.RESOURCE_ID = p_id
      AND (SELECT TYPE_ID FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID) = 73
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ROLE', 103+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ROLE(104) : competencies.sql(2199) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ2;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ROLE', 112+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ROLE(113) : competencies.sql(2208) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ROLE', 124+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ROLE(125) : competencies.sql(2220) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_ROLE;
ALTER FUNCTION SP_S_RES_A_ROLE COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_SKILL ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_SKILL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ3 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ4 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_SKILL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_SKILL'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_SKILL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_SKILL', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_SKILL(27) : competencies.sql(2263) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_SKILL', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_SKILL(32) : competencies.sql(2268) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_SKILL', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_SKILL(35) : competencies.sql(2271) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_id := p_id;
   
   BEGIN
      SELECT RANK
      INTO h_rank
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID =h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_SKILL', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_SKILL(53) : competencies.sql(2289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --if (!(IsNullSpParam(ind_s_res_compet, p_bitflag))  (*p_bitflag == 2))
   If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 2) Then
      OPEN SELECT_READ3 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      T1.LEVEL_ID AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION, T1.CREATED_BY,
      (SELECT FULL_NAME FROM RESOURCES
      WHERE RESOURCE_ID = T1.CREATED_BY) AS CREATED_BY_NAME,
      T1.CREATED_DATETIME, T1.PROFICIENCY_LEVEL_ID, T1.PREFERENCE_LEVEL_ID,
      R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE,
      T1.EXPERIENCE_ID, T1.LAST_USED_DATETIME, T1.LAST_USED_LOCATION, T1.DEFAULT_FLAG,
      T1.PARENT_ID AS ORIGINAL_ID, T1.STRATEGIC_FLAG,
      COALESCE(T1.TARGET_ATTAINMENT_DATE, TO_DATE('1899-12-30')) AS TARGET_ATTAINMENT_DATE,
      COALESCE(T1.ACQUIRED_ON, TO_DATE('1899-12-30')) AS ACQUIRED_ON, T1.MANDATORY, T1.MANDATORY_DETAILS
      FROM TMT_COMPETENCIES T1 LEFT OUTER JOIN RATES R ON R.RATE_ID = T1.RATE_ID
      WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
      AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
      AND T1.TYPE_ID IN (502, 74, 75, 29, 76)
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_SKILL', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_SKILL(78) : competencies.sql(2314) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ3;
   Else
      OPEN SELECT_READ4 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID,
      (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = (SELECT PARENT_ID FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID)), ', '), (SELECT RTRIM(ELEMENT_NAME)
      FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID))) AS ELEMENT_NAME,
      1 AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION, T1.CREATED_BY,
      (SELECT FULL_NAME FROM RESOURCES
      WHERE RESOURCE_ID = T1.CREATED_BY) AS CREATED_BY_NAME,
      T1.CREATED_DATETIME, T1.PROFICIENCY_LEVEL_ID, T1.PREFERENCE_LEVEL_ID,
      R.REGULAR_COST, R.REGULAR_SALE, R.SPECIAL_COST, R.SPECIAL_SALE,
      T1.EXPERIENCE_ID, T1.LAST_USED_DATETIME, T1.LAST_USED_LOCATION,
      T1.DEFAULT_FLAG, T1.PARENT_ID AS ORIGINAL_ID, T1.STRATEGIC_FLAG,
      COALESCE(T1.TARGET_ATTAINMENT_DATE, TO_DATE('1899-12-30')) AS TARGET_ATTAINMENT_DATE,
      COALESCE(T1.ACQUIRED_ON, TO_DATE('1899-12-30')) AS ACQUIRED_ON, T1.MANDATORY, T1.MANDATORY_DETAILS
      FROM TMT_COMPETENCIES T1 LEFT OUTER JOIN RATES R ON R.RATE_ID = T1.RATE_ID
      WHERE T1.RESOURCE_ID = p_id
      AND (SELECT TYPE_ID FROM TMT_COMPETENCIES
      WHERE ELEMENT_ID = T1.PARENT_ID) = 76
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_SKILL', 104+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_SKILL(105) : competencies.sql(2341) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ4;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_SKILL', 113+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_SKILL(114) : competencies.sql(2350) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_SKILL', 125+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_SKILL(126) : competencies.sql(2362) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_SKILL;
ALTER FUNCTION SP_S_RES_A_SKILL COMPILE DEBUG;

