/****************************************************************************************/

Prompt == FUNCTION FC_I_CONTACT ==
CREATE OR REPLACE FUNCTION FC_I_CONTACT
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , sh_h_type_id PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := FC_I_CONTACT(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_CONTACT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_CONTACT(26) : contacts.sql(29) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (h_uniqueID IS NULL) Then
      h_uniqueID := SYS_GUID();
   End If;
   
   -- Validate parent_id
   If (h_parent_id IS NULL) Then
      -- Get the max sibling level from the root
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CONTACTS', h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- insert-select
      BEGIN
         INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, ASSIGN_NAMES)
         SELECT
         RTRIM(COALESCE(RANK, '')) || h_rankBase64,
         h_parent_id,
         h_uniqueID,
         h_name,
         h_type_id,
         LEVEL_ID + 1,
         h_rec_user,
         h_rec_user || ';'
         FROM TMT_CONTACTS WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_CONTACT', 65+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_CONTACT(66) : contacts.sql(69) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Validate parent_id
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'FC_I_CONTACT', 72+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: FC_I_CONTACT(73) : contacts.sql(76) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_CONTACTS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, False); 
      END;
      
      -- Get next sibling count
      PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTACTS', h_parent_id, h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- insert-select
      BEGIN
         INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, ASSIGN_NAMES)
         SELECT
         RTRIM(Rank) || h_rankBase64,
         h_parent_id,
         h_uniqueID,
         h_name,
         h_type_id,
         LEVEL_ID + 1,
         h_rec_user,
         h_rec_user || ';'
         FROM TMT_CONTACTS WHERE ELEMENT_ID = h_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_CONTACT', 112+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_CONTACT(113) : contacts.sql(116) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_CONTACT', 119+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_CONTACT(120) : contacts.sql(123) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_CONTACT;
ALTER FUNCTION FC_I_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_I_CONTACT ==
CREATE OR REPLACE FUNCTION SP_I_CONTACT
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_I_CONTACT(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_CONTACT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_CONTACT(26) : contacts.sql(159) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_name := p_name;
   h_parent_id := p_parent_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   h_uniqueID := NULL;
   
   PMO_DB2M.SC_STATE := FC_I_CONTACT(h_name, h_uniqueID, h_parent_id, h_type_id, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_CONTACT', 39+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_CONTACT(40) : contacts.sql(173) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTACT', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTACT(47) : contacts.sql(180) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTACT', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTACT(59) : contacts.sql(192) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_CONTACT;
ALTER FUNCTION SP_I_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTACT ==
CREATE OR REPLACE FUNCTION SP_U_CONTACT
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTACT', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTACT(23) : contacts.sql(231) : ' || SYSTIMESTAMP);
   END IF; 

   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_CONTACT', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_CONTACT(29) : contacts.sql(237) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTACT', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTACT(32) : contacts.sql(240) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Execute Update Statement
   If (p_parent_id IS NULL) Then
      BEGIN
         UPDATE TMT_CONTACTS
         SET ELEMENT_NAME = p_name,
         DESCRIPTION  = p_description,
         REC_STATUS   = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 56+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(57) : contacts.sql(265) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_CONTACTS
         SET ELEMENT_NAME = p_name,
         DESCRIPTION  = p_description,
         REC_STATUS   = 'U',
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 79+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(80) : contacts.sql(288) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTACT', 86+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTACT(87) : contacts.sql(295) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 93+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(94) : contacts.sql(302) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTACT', 105+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTACT(106) : contacts.sql(314) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTACT;
ALTER FUNCTION SP_U_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_CONTACT ==
CREATE OR REPLACE FUNCTION SP_D_CONTACT
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_exists VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_private_count PLS_INTEGER := 0;
   h_dependency_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC1 IS
         SELECT ELEMENT_ID FROM TMT_CONTACTS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND PRIVATE_FLAG = 'Y';

      CURSOR CC2 IS
         SELECT ELEMENT_ID FROM TMT_CONTACTS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND TYPE_ID = PMO_CNST.SC_RESOURCE;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_D_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_CONTACT(36) : contacts.sql(366) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   h_id := p_id;
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_CONTACT', 44+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_CONTACT(45) : contacts.sql(375) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      IF h_rec_status = 'X' THEN 
         EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || 'TMT_CONTACTS' || ' a 
         WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS 
         (SELECT b.REC_STATUS 
                  FROM TMT_ORGANIZATIONS b 
                 WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)' 
            INTO h_rec_status 
            Using h_id; 
      ELSE 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || 'TMT_CONTACTS' || ' 
              WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') ' 
            INTO h_rec_status 
            Using h_id; 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, h_id, True); 
      WHEN OTHERS THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id , True); 
   END;
   
   -- Validate parent id
   If ((p_parent_id IS NULL)) Then
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 79+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(80) : contacts.sql(410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --strcpy(h_rec_status, "");
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_CONTACT', 88+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_CONTACT(89) : contacts.sql(419) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_CONTACTS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_CONTACT', 101+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_CONTACT(102) : contacts.sql(432) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 104+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(105) : contacts.sql(435) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 118+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(119) : contacts.sql(449) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN CC1;
   
   FETCH CC1 INTO h_exists;
   IF CC1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_private_count := 1;
   Else
      h_private_count := 0;
   End If;
   
   BEGIN
      CLOSE CC1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 147+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(148) : contacts.sql(478) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CC2;
   
   FETCH CC2 INTO h_exists;
   IF CC2%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_dependency_count := 1;
   Else
      h_dependency_count := 0;
   End If;
   
   BEGIN
      CLOSE CC2 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 175+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(176) : contacts.sql(506) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      If (Coalesce(h_private_count, 0) <> 0) OR (Coalesce(h_dependency_count, 0) <> 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 183+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(184) : contacts.sql(514) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE_RECORD), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 186+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(187) : contacts.sql(517) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE_RECORD, p_id, True);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 191+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(192) : contacts.sql(522) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 194+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(195) : contacts.sql(525) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(0, p_id, True);
      End If;
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      If Coalesce(h_private_count, 0) = 0 Then
         -- Delete Tree
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         BEGIN
            DELETE FROM TMT_CONTACTS WHERE RANK LIKE h_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 229+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(230) : contacts.sql(560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- Update Parent's Hierarchy Info
         If SubStr(h_rec_status, 1, 1) = 'D' Then
            If (p_parent_id IS NULL) Then
               BEGIN
                  UPDATE TMT_CONTACTS SET DELETED_COUNT = DELETED_COUNT - 1
                  WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 250+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(251) : contacts.sql(581) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_CONTACTS SET DELETED_COUNT = DELETED_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 268+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(269) : contacts.sql(599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If (p_parent_id IS NULL) Then
               BEGIN
                  UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 288+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(289) : contacts.sql(619) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 306+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(307) : contacts.sql(637) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 314+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(315) : contacts.sql(645) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PRIVATE_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 317+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(318) : contacts.sql(648) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PRIVATE_CHILD_EXISTS, p_id, True);
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_CONTACT', 324+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_CONTACT(325) : contacts.sql(655) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 327+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(328) : contacts.sql(658) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      If Coalesce(h_private_count, 0) = 0 Then
         -- Update Parent's Hierarchy Info
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 348+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(349) : contacts.sql(679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_CONTACTS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 366+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(367) : contacts.sql(697) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         -- Lock Tree
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_CONTACTS SET
            REC_STATUS   = p_del_flag,
            REC_DATETIME = SYSTIMESTAMP,
            REC_USER     = p_rec_user
            WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 402+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(403) : contacts.sql(733) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTACT', 408+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTACT(409) : contacts.sql(739) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PRIVATE_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTACT', 411+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTACT(412) : contacts.sql(742) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PRIVATE_CHILD_EXISTS, p_id, True);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_CONTACT', 418+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_CONTACT(419) : contacts.sql(749) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 425+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(426) : contacts.sql(756) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTACT', 437+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTACT(438) : contacts.sql(768) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_CONTACT;
ALTER FUNCTION SP_D_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_CONTACT ==
CREATE OR REPLACE FUNCTION SP_S_CONTACT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_CONTACT(21) : contacts.sql(805) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
      h_sql := h_sql || 
                  'CASE WHEN TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, ';
      h_sql := h_sql || 'T.CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION, ';
      h_sql := h_sql || 
                  'CASE WHEN TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE '''' END AS TITLE, ';
      h_sql := h_sql || 'CASE WHEN T.TYPE_ID = 13 THEN ';
      h_sql := h_sql || '((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_sql := h_sql || 
                  'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) || '', '' || ';
      h_sql := h_sql || '(SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_sql := h_sql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_sql := h_sql || 
                  'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) ';
      h_sql := h_sql || 'ELSE '''' END AS COMPETENCY ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      -- Validate id
      If ((p_id IS NULL)) Then
         h_sql := h_sql || 
                  'WHERE (PMO_ORASTD.LOCATE(T.RANK, (SELECT RANK FROM TMT_CONTACTS WHERE PRIVATE_FLAG = ''Y'')) = 1 OR T.PARENT_ID IS NULL) ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 ';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_CONTACT', 54+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_CONTACT(55) : contacts.sql(839) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_CONTACTS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
         END;
         
         h_sql := h_sql || 'WHERE (T.PARENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ''') AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
      End If;
   ElsIf Coalesce(p_bitflag, 0) = 1 Then
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_CONTACTS
         WHERE ELEMENT_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 87+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(88) : contacts.sql(872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_sql := h_sql || 'WHERE ((T.RANK LIKE ''0000%%'') OR (T.RANK LIKE ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || '%%'')) AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_CONTACT_LIST);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_FOLDER);
      h_sql := h_sql || ') ';
   Else
      h_sql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_sql := h_sql || 
                  'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.PRIVATE_FLAG, T.DESCRIPTION ';
      h_sql := h_sql || 'FROM TMT_CONTACTS T ';
      
      -- Validate id
      If ((p_id IS NULL)) Then
         h_sql := h_sql || 
                  'WHERE (PMO_ORASTD.LOCATE(T.RANK, (SELECT RANK FROM TMT_CONTACTS WHERE PRIVATE_FLAG = ''Y'')) = 1 OR T.PARENT_ID IS NULL) ';
         h_sql := h_sql || 'AND T.TYPE_ID <> 0 AND T.RANK NOT LIKE ''qqqq%''';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_CONTACT', 114+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_CONTACT(115) : contacts.sql(899) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_CONTACTS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
         END;
         
         If Coalesce(p_bitflag, 0) = PMO_CNST.SC_REFRESH Then
            h_sql := h_sql || 'WHERE T.ELEMENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''' AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID = 12 ';
         Else
            h_sql := h_sql || 'WHERE (T.PARENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''' OR T.ELEMENT_ID = ''';
            h_sql := h_sql || h_id;
            h_sql := h_sql || ''') AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') AND T.TYPE_ID = 12 ';
         End If;
      End If;
      h_sql := h_sql || 'AND T.TYPE_ID <> 13 ';
   End If;
   
   h_sql := h_sql || ' ORDER BY RANK ';
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 143+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTACT(144) : contacts.sql(928) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTACT', 146+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTACT(147) : contacts.sql(931) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 154+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(155) : contacts.sql(939) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTACT', 166+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTACT(167) : contacts.sql(951) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_CONTACT;
ALTER FUNCTION SP_S_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_A_RES_CONTACT ==
CREATE OR REPLACE FUNCTION SP_A_RES_CONTACT
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_exists VARCHAR2(32);
   h_name VARCHAR2(255);
   h_rec_status VARCHAR2(1);
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RES_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RES_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RES_CONTACT(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_CONTACT', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RES_CONTACT(27) : contacts.sql(994) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_id IS NULL)) OR ((p_resource_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_CONTACT', 33+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_CONTACT(34) : contacts.sql(1001) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_CONTACT', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_CONTACT(37) : contacts.sql(1004) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   -- Validate id
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_A_RES_CONTACT', 44+1) THEN 
      LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_A_RES_CONTACT(45) : contacts.sql(1012) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_CONTACTS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_CONTACTS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   BEGIN
      SELECT NICK_NAME INTO h_name
      FROM RESOURCES WHERE RESOURCE_ID  = p_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 64+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(65) : contacts.sql(1032) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Validate resource id
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_CONTACT', 72+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_CONTACT(73) : contacts.sql(1040) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_resource_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_CONTACT', 75+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_CONTACT(76) : contacts.sql(1043) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_resource_id, True);
   End If;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_exists FROM TMT_CONTACTS
      WHERE ELEMENT_ID = p_resource_id AND PARENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 89+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(90) : contacts.sql(1057) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RES_CONTACT', 96+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RES_CONTACT(97) : contacts.sql(1064) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_CONTACT', 99+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_CONTACT(100) : contacts.sql(1067) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS, True);
   End If;
   
   -- Get next sibling count
   PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTACTS', h_id, h_max_rank);
   
   --Get Base 64
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   
   -- insert-select
   BEGIN
      INSERT INTO TMT_CONTACTS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER)
      SELECT RTRIM(Rank) || h_rankBase64, p_id, p_resource_id, h_name, PMO_CNST.SC_RESOURCE, LEVEL_ID + 1,
      p_rec_user FROM TMT_CONTACTS WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 125+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(126) : contacts.sql(1093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CONTACTS
      SET ASSIGN_NAMES = RTRIM(ASSIGN_NAMES) || h_name || ';'
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 144+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(145) : contacts.sql(1112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RES_CONTACT', 150+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RES_CONTACT(151) : contacts.sql(1118) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 157+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(158) : contacts.sql(1125) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_CONTACT', 169+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_CONTACT(170) : contacts.sql(1137) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RES_CONTACT;
ALTER FUNCTION SP_A_RES_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UA_RES_CONTACT ==
CREATE OR REPLACE FUNCTION SP_UA_RES_CONTACT
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_full_name VARCHAR2(99);
   h_length PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UA_RES_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UA_RES_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_UA_RES_CONTACT(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UA_RES_CONTACT', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UA_RES_CONTACT(21) : contacts.sql(1174) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Execute Delete Statement
   BEGIN
      DELETE FROM TMT_CONTACTS WHERE PARENT_ID = p_id AND ELEMENT_ID = p_resource_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 38+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(39) : contacts.sql(1192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT NICK_NAME || ';', LENGTH(NICK_NAME) + 1
         INTO h_full_name, h_length
         FROM RESOURCES
         WHERE RESOURCE_ID  = p_resource_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(56) : contacts.sql(1209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update the parent CHILD_COUNT field
      BEGIN
         UPDATE TMT_CONTACTS
         SET CHILD_COUNT  = CHILD_COUNT - 1,
         ASSIGN_NAMES = SUBSTR(ASSIGN_NAMES , 1,InStr(ASSIGN_NAMES,h_full_name)-1) || SUBSTR(ASSIGN_NAMES , InStr(ASSIGN_NAMES,h_full_name) + h_length)
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 76+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(77) : contacts.sql(1230) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UA_RES_CONTACT', 83+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UA_RES_CONTACT(84) : contacts.sql(1237) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 90+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(91) : contacts.sql(1244) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_CONTACT', 102+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_CONTACT(103) : contacts.sql(1256) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UA_RES_CONTACT;
ALTER FUNCTION SP_UA_RES_CONTACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_CONTACT ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_CONTACT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SELECT_READ1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SELECT_READ2 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_CONTACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_CONTACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_CONTACT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_CONTACT(27) : contacts.sql(1299) : ' || SYSTIMESTAMP);
   END IF; 

   -- Execute Select Statement
   If ((p_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_CONTACT', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_CONTACT(33) : contacts.sql(1305) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_CONTACT(36) : contacts.sql(1308) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_id := p_id;
   
   If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
      BEGIN
         SELECT RANK
         INTO h_rank
         FROM TMT_CONTACTS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 54+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(55) : contacts.sql(1327) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN SELECT_READ1 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_CONTACTS T1
      WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
      AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
      AND T1.TYPE_ID NOT in (0, 13)
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 69+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_CONTACT(70) : contacts.sql(1342) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ1;
   Else
      OPEN SELECT_READ2 FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_CONTACTS T1
      WHERE T1.ELEMENT_ID IN (SELECT PARENT_ID FROM TMT_CONTACTS
      WHERE ELEMENT_ID = p_id)
      AND T1.TYPE_ID < 5000
      AND T1.TYPE_ID <> 0
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_CONTACT', 84+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_CONTACT(85) : contacts.sql(1357) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN SELECT_READ2;
   End If;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 92+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(93) : contacts.sql(1365) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_CONTACT', 104+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_CONTACT(105) : contacts.sql(1377) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_CONTACT;
ALTER FUNCTION SP_S_RES_A_CONTACT COMPILE DEBUG;

