/****************************************************************************************/

Prompt == FUNCTION SP_I_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_I_CONTRACT
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_name VARCHAR2(100);
   h_parent_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_client_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_uniqueID VARCHAR2(32);
   h_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_I_CONTRACT(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_CONTRACT', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_CONTRACT(31) : contracts.sql(34) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_name := p_name;
   h_parent_id := p_parent_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   
   h_uniqueID := SYS_GUID();
   h_element_id := h_uniqueID;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_client_id
      FROM TMT_CLIENTS
      WHERE DEFAULT_FLAG = 'Y';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT(54) : contracts.sql(57) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_parent_id IS NULL) Then
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CONTRACTS', h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      h_rankBase64 := h_rankBase64;
      
      BEGIN
         INSERT INTO TMT_CONTRACTS (RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, 
         CLIENT_ID, TYPE_ID, LEVEL_ID, REC_USER)
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64,
         h_parent_id, h_element_id, h_element_name, 
         h_client_id, h_type_id, LEVEL_ID + 1, h_rec_user
         FROM TMT_CONTRACTS 
         WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT(82) : contracts.sql(85) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTRACTS', h_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      h_rankBase64 := h_rankBase64;
      
      BEGIN
         INSERT INTO TMT_CONTRACTS (RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, 
         CLIENT_ID, TYPE_ID, LEVEL_ID, REC_USER)
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64,
         h_parent_id, h_element_id, h_element_name, 
         h_client_id, h_type_id, LEVEL_ID + 1, h_rec_user
         FROM TMT_CONTRACTS 
         WHERE ELEMENT_ID = h_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT', 108+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT(109) : contracts.sql(112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_CONTRACT', 115+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_CONTRACT(116) : contracts.sql(119) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT', 122+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT(123) : contracts.sql(126) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT', 134+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT(135) : contracts.sql(138) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_CONTRACT;
ALTER FUNCTION SP_I_CONTRACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT_TYPE ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT_TYPE
(
     p_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_last_charged_date DATE;
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT_TYPE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT_TYPE'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT_TYPE(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_TYPE', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT_TYPE(23) : contracts.sql(177) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_id := p_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   
   BEGIN
      SELECT LAST_CHARGED_DATE INTO h_last_charged_date
      FROM TMT_CONTRACTS
      WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_TYPE', 41+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_TYPE(42) : contracts.sql(196) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_TYPE', 48+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_TYPE(49) : contracts.sql(203) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_TYPE', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_TYPE(52) : contracts.sql(206) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_element_id, True);
   End If;
   
   If (h_last_charged_date IS NOT NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_TYPE', 58+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_TYPE(59) : contracts.sql(213) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_CHANGE_CHARGED_CONTRACT), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_TYPE', 61+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_TYPE(62) : contracts.sql(216) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_CHANGE_CHARGED_CONTRACT, h_element_id
            , True);
   End If;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET CONTRACT_TYPE = h_type_id
      WHERE ELEMENT_ID = h_element_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_TYPE', 81+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_TYPE(82) : contracts.sql(236) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT_TYPE', 87+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT_TYPE(88) : contracts.sql(242) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_TYPE', 94+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_TYPE(95) : contracts.sql(249) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_TYPE', 106+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_TYPE(107) : contracts.sql(261) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT_TYPE;
ALTER FUNCTION SP_U_CONTRACT_TYPE COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT_MAX ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT_MAX
(
     p_id VARCHAR2
   , p_max_amount NUMBER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_last_charged_date DATE;
   h_rec_user VARCHAR2(32);
   h_max_amount NUMBER := 0;
   h_charged_amount NUMBER := 0;
   h_old_max_amount NUMBER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT_MAX') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT_MAX'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT_MAX(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_max_amount), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_MAX', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT_MAX(25) : contracts.sql(302) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_id := p_id;
   h_max_amount := p_max_amount;
   h_rec_user := p_rec_user;
   
   BEGIN
      SELECT LAST_CHARGED_DATE, CHARGED_AMOUNT, MAX_AMOUNT 
      INTO h_last_charged_date, h_charged_amount, h_old_max_amount
      FROM TMT_CONTRACTS
      WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_MAX', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_MAX(45) : contracts.sql(322) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_MAX', 51+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_MAX(52) : contracts.sql(329) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_MAX', 54+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_MAX(55) : contracts.sql(332) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_element_id, True);
   End If;
   
   If ((h_last_charged_date IS NOT NULL)) AND (Coalesce(h_old_max_amount, 0) <= Coalesce(
         h_charged_amount, 0)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_MAX', 62+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_MAX(63) : contracts.sql(340) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_CHANGE_CHARGED_CONTRACT), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_MAX', 65+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_MAX(66) : contracts.sql(343) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_CHANGE_CHARGED_CONTRACT, h_element_id
            , True);
   End If;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET MAX_AMOUNT = h_max_amount
      WHERE ELEMENT_ID = h_element_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_MAX', 85+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_MAX(86) : contracts.sql(363) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT_MAX', 91+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT_MAX(92) : contracts.sql(369) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_MAX', 98+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_MAX(99) : contracts.sql(376) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_MAX', 110+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_MAX(111) : contracts.sql(388) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT_MAX;
ALTER FUNCTION SP_U_CONTRACT_MAX COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT_CURRENCY ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT_CURRENCY
(
     p_id VARCHAR2
   , p_currency_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_currency_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT_CURRENCY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT_CURRENCY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT_CURRENCY(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_currency_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CURRENCY', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT_CURRENCY(22) : contracts.sql(426) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_id := p_id;
   h_currency_id := p_currency_id;
   h_rec_user := p_rec_user;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET CURRENCY_ID = h_currency_id
      WHERE ELEMENT_ID = h_element_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CURRENCY', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CURRENCY(45) : contracts.sql(449) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT_CURRENCY', 50+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT_CURRENCY(51) : contracts.sql(455) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CURRENCY', 57+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CURRENCY(58) : contracts.sql(462) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CURRENCY', 69+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CURRENCY(70) : contracts.sql(474) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT_CURRENCY;
ALTER FUNCTION SP_U_CONTRACT_CURRENCY COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT_CLIENT ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT_CLIENT
(
     p_id VARCHAR2
   , p_client_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_client_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT_CLIENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT_CLIENT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT_CLIENT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_client_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CLIENT', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT_CLIENT(22) : contracts.sql(512) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_id := p_id;
   h_client_id := p_client_id;
   h_rec_user := p_rec_user;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET CLIENT_ID = h_client_id
      WHERE ELEMENT_ID = h_element_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CLIENT', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CLIENT(45) : contracts.sql(535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT_CLIENT', 50+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT_CLIENT(51) : contracts.sql(541) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CLIENT', 57+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CLIENT(58) : contracts.sql(548) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CLIENT', 69+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CLIENT(70) : contracts.sql(560) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT_CLIENT;
ALTER FUNCTION SP_U_CONTRACT_CLIENT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_external_reference VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_element_name VARCHAR2(100);
   h_external_reference VARCHAR2(32);
   h_description VARCHAR2(255);
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_external_reference || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT(28) : contracts.sql(604) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_element_id := p_id;
   h_element_name := p_name;
   h_external_reference := p_external_reference;
   h_description := p_description;
   h_rec_user := p_rec_user;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET EXTERNAL_REFERENCE = h_external_reference,
      ELEMENT_NAME = h_element_name,
      DESCRIPTION = h_description
      WHERE ELEMENT_ID = h_element_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT(55) : contracts.sql(631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT', 60+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT(61) : contracts.sql(637) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT', 67+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT(68) : contracts.sql(644) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT', 79+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT(80) : contracts.sql(656) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT;
ALTER FUNCTION SP_U_CONTRACT COMPILE DEBUG;

/****************************************************************************************/

-- contracts:

-- 1- assigned contracts can not be deleted

-- 2- if a folder contains an assigned contracts can be deleted

Prompt == FUNCTION SP_D_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_D_CONTRACT
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_assigned_contract_exists PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_D_CONTRACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTRACT', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_CONTRACT(26) : contracts.sql(704) : ' || SYSTIMESTAMP);
   END IF; 

   h_element_id := p_id;
   h_parent_id := p_parent_id;
   
   BEGIN
      SELECT RANK 
      INTO h_rank
      FROM TMT_CONTRACTS
      WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 42+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(43) : contracts.sql(721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_assigned_contract_exists := 0;
      -- check for assigned contracts
      BEGIN
         SELECT 1 INTO h_assigned_contract_exists
         FROM DUAL WHERE EXISTS(
         SELECT TC.ELEMENT_ID
         FROM TMT_CONTRACTS TC, TASK_ASSIGNMENTS TA
         WHERE TC.ELEMENT_ID = TA.CONTRACT_ID
         AND TC.RANK LIKE h_rank || '%');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 63+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(64) : contracts.sql(742) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\n h_assigned_contract_exists:%d", h_assigned_contract_exists);
      
      If Coalesce(h_assigned_contract_exists, 0) = 0 Then
         BEGIN
            DELETE FROM TMT_CONTRACTS
            WHERE RANK LIKE h_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 84+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(85) : contracts.sql(763) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("\n h_rank:%s", h_rank);
         
         If (h_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_CONTRACTS 
               SET CHILD_COUNT = CHILD_COUNT - 1, 
               DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 107+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(108) : contracts.sql(786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_CONTRACTS 
               SET CHILD_COUNT = CHILD_COUNT - 1, 
               DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 127+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(128) : contracts.sql(806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTRACT', 134+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTRACT(135) : contracts.sql(813) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_IN_USE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTRACT', 137+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTRACT(138) : contracts.sql(816) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_IN_USE, h_element_id, True);
      End If;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_CONTRACT', 143+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_CONTRACT(144) : contracts.sql(822) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTRACT', 146+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTRACT(147) : contracts.sql(825) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_element_id, True);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_CONTRACT', 152+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_CONTRACT(153) : contracts.sql(831) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 159+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(160) : contracts.sql(838) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT', 171+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT(172) : contracts.sql(850) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_CONTRACT;
ALTER FUNCTION SP_D_CONTRACT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_S_CONTRACT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      CSNULL0 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CSNULL2 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CSNULL1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CSNULL5 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_CONTRACT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_CONTRACT(34) : contracts.sql(900) : ' || SYSTIMESTAMP);
   END IF; 

   h_element_id := p_id;
   h_bitflag := p_bitflag;
   
   If ((h_element_id IS NULL)) AND (Coalesce(h_bitflag, 0) = 0) Then
      OPEN CSNULL0 FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.REC_USER,
      T.CONTRACT_TYPE, T.CLIENT_ID, C.ELEMENT_NAME AS CLIENT_NAME,
      T.CHARGED_AMOUNT, LAST_CHARGED_DATE, T.MAX_AMOUNT, T.CURRENCY_ID, T.EXTERNAL_REFERENCE
      FROM TMT_CONTRACTS T
      JOIN TMT_CLIENTS C ON C.ELEMENT_ID = T.CLIENT_ID
      WHERE T.PARENT_ID IS NULL
      AND T.RANK <> '0000'
      AND T.REC_STATUS NOT in ('D', 'L', 'A')
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT(52) : contracts.sql(918) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CSNULL0;
   ElsIf Coalesce(h_bitflag, 0) = 0 Then
      OPEN CSNULL2 FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.REC_USER,
      T.CONTRACT_TYPE, T.CLIENT_ID, C.ELEMENT_NAME AS CLIENT_NAME,
      T.MAX_AMOUNT, LAST_CHARGED_DATE, T.MAX_AMOUNT, T.CURRENCY_ID, T.EXTERNAL_REFERENCE
      FROM TMT_CONTRACTS T
      JOIN TMT_CLIENTS C ON C.ELEMENT_ID = T.CLIENT_ID
      WHERE T.ELEMENT_ID = h_element_id
      UNION ALL
      SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.REC_USER,
      T.CONTRACT_TYPE, T.CLIENT_ID, C.ELEMENT_NAME AS CLIENT_NAME,
      T.MAX_AMOUNT, LAST_CHARGED_DATE, T.MAX_AMOUNT, T.CURRENCY_ID, T.EXTERNAL_REFERENCE
      FROM TMT_CONTRACTS T
      JOIN TMT_CLIENTS C ON C.ELEMENT_ID = T.CLIENT_ID
      WHERE T.PARENT_ID = h_element_id
      AND T.REC_STATUS NOT in ('D', 'L', 'A')
      ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT', 73+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT(74) : contracts.sql(940) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CSNULL2;
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      OPEN CSNULL1 FOR SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.REC_USER,
      T.CONTRACT_TYPE, T.CLIENT_ID, '' AS CLIENT_NAME,
      T.MAX_AMOUNT, T.LAST_CHARGED_DATE, T.MAX_AMOUNT, T.CURRENCY_ID, T.EXTERNAL_REFERENCE
      FROM TMT_CONTRACTS T
      WHERE T.RANK LIKE '0000%'
      AND T.TYPE_ID = PMO_CNST.SC_CONTRACT
      AND T.REC_STATUS NOT IN ('D', 'L', 'A')
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT', 87+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT(88) : contracts.sql(954) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CSNULL1;
   ElsIf Coalesce(h_bitflag, 0) = 5 Then
      OPEN CSNULL5 FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION, T.REC_USER,
      T.CONTRACT_TYPE, T.CLIENT_ID, C.ELEMENT_NAME AS CLIENT_NAME,
      T.MAX_AMOUNT, LAST_CHARGED_DATE, T.MAX_AMOUNT, T.CURRENCY_ID, T.EXTERNAL_REFERENCE
      FROM TMT_CONTRACTS T
      JOIN TMT_CLIENTS C ON C.ELEMENT_ID = T.CLIENT_ID
      WHERE T.ELEMENT_ID = h_element_id
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT', 100+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT(101) : contracts.sql(967) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CSNULL5;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTRACT', 109+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTRACT(110) : contracts.sql(976) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTRACT', 121+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTRACT(122) : contracts.sql(988) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_CONTRACT;
ALTER FUNCTION SP_S_CONTRACT COMPILE DEBUG;

/*****************************************************************************************/

-- Assign/unassign a contract to an assignment or change contract for an assignment

-- p_id: tsk_assignment_id in task_assignments table.

-- p_contract_id: contract_id in TMT_CONTRACTS table.

--  null: unassign;

--  not null: assign/change

/*****************************************************************************************/

Prompt == FUNCTION SP_A_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_A_CONTRACT
(
     p_id VARCHAR2
   , p_contract_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_contract_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_count PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_A_CONTRACT(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_contract_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CONTRACT', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_CONTRACT(24) : contracts.sql(1040) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_contract_id := p_contract_id;
   h_rec_user := p_rec_user;
   
   If ((h_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_CONTRACT', 34+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_CONTRACT(35) : contracts.sql(1051) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CONTRACT', 37+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_CONTRACT(38) : contracts.sql(1054) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      SELECT PROJECT_ID INTO h_project_id
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CONTRACT', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CONTRACT(53) : contracts.sql(1069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_CONTRACT', 59+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_CONTRACT(60) : contracts.sql(1076) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CONTRACT', 62+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_CONTRACT(63) : contracts.sql(1079) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   h_count := 0;
   BEGIN
      SELECT COUNT(PROJECT_ID) INTO h_count
      FROM TASK_ASSIGNMENTS
      WHERE CONTRACT_ID = h_contract_id 
      AND PROJECT_ID <> h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CONTRACT', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CONTRACT(80) : contracts.sql(1096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_CONTRACT', 86+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_CONTRACT(87) : contracts.sql(1103) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CONTRACT_USED_BY_ANOTHER_PROJE), 'NULL') || ' ID: ' || '''' || h_contract_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CONTRACT', 89+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_CONTRACT(90) : contracts.sql(1106) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CONTRACT_USED_BY_ANOTHER_PROJE, h_contract_id
            , True);
   End If;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS
      SET CONTRACT_ID = h_contract_id
      WHERE TSK_ASSIGNMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CONTRACT', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CONTRACT(110) : contracts.sql(1126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_CONTRACT', 115+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_CONTRACT(116) : contracts.sql(1132) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CONTRACT', 122+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CONTRACT(123) : contracts.sql(1139) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CONTRACT', 134+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CONTRACT(135) : contracts.sql(1151) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_CONTRACT;
ALTER FUNCTION SP_A_CONTRACT COMPILE DEBUG;

/*****************************************************************************************/

-- Insert a new row in contract_breakdown table

-- p_id: contract_id.

/*****************************************************************************************/

Prompt == FUNCTION SP_I_CONTRACT_CCB ==
CREATE OR REPLACE FUNCTION SP_I_CONTRACT_CCB
(
     p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_contract_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_charge_date DATE;
   h_start_date DATE;
   h_charge_amount NUMBER := 0;
   h_count PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_CONTRACT_CCB') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_CONTRACT_CCB'), '-- ' || SYSTIMESTAMP, ':r := SP_I_CONTRACT_CCB(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_CONTRACT_CCB', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_CONTRACT_CCB(24) : contracts.sql(1197) : ' || SYSTIMESTAMP);
   END IF; 

   h_contract_id := p_id;
   h_rec_user := p_rec_user;
   
   If (h_contract_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_CONTRACT_CCB', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_CONTRACT_CCB(32) : contracts.sql(1205) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_CONTRACT_CCB', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_CONTRACT_CCB(35) : contracts.sql(1208) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_count := 0;
   BEGIN
      SELECT MAX(CHARGE_DATE), COUNT(CONTRACT_ID) 
      INTO h_start_date, h_count
      FROM CONTRACT_BREAKDOWN
      WHERE CONTRACT_ID = h_contract_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(52) : contracts.sql(1225) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 0 Then
      h_charge_amount := 0.00;
      BEGIN
         SELECT START_DATE INTO h_charge_date
         FROM FISCAL_CALENDAR
         WHERE TO_DATE(SYSDATE) BETWEEN START_DATE AND FINISH_DATE;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 68+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(69) : contracts.sql(1242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT CHARGE_AMOUNT INTO h_charge_amount
         FROM CONTRACT_BREAKDOWN
         WHERE CONTRACT_ID = h_contract_id
         AND CHARGE_DATE = h_start_date;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 84+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(85) : contracts.sql(1258) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MIN(START_DATE) INTO h_charge_date
         FROM FISCAL_CALENDAR
         WHERE START_DATE > TO_DATE(SubStr(h_start_date, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 99+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(100) : contracts.sql(1273) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_id := SYS_GUID();
   h_id := h_id;
   BEGIN
      INSERT INTO CONTRACT_BREAKDOWN (ELEMENT_ID, CONTRACT_ID, CHARGE_DATE, CHARGE_AMOUNT, REC_USER)
      VALUES (h_id, h_contract_id, h_charge_date, h_charge_amount, h_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 120+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(121) : contracts.sql(1294) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_CONTRACT_CCB', 126+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_CONTRACT_CCB(127) : contracts.sql(1300) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 133+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(134) : contracts.sql(1307) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_CONTRACT_CCB', 145+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_CONTRACT_CCB(146) : contracts.sql(1319) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_CONTRACT_CCB;
ALTER FUNCTION SP_I_CONTRACT_CCB COMPILE DEBUG;

/*****************************************************************************************/

-- Select contract cross charge breakdown

-- p_id: contract_id.

/*****************************************************************************************/

Prompt == FUNCTION SP_S_CONTRACT_CCB ==
CREATE OR REPLACE FUNCTION SP_S_CONTRACT_CCB
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_contract_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      CCTRCCB PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_CONTRACT_CCB') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_CONTRACT_CCB'), '-- ' || SYSTIMESTAMP, ':r := SP_S_CONTRACT_CCB(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT_CCB', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_CONTRACT_CCB(25) : contracts.sql(1366) : ' || SYSTIMESTAMP);
   END IF; 

   h_contract_id := p_id;
   
   If (h_contract_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_CONTRACT_CCB', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_CONTRACT_CCB(32) : contracts.sql(1373) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT_CCB', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT_CCB(35) : contracts.sql(1376) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   OPEN CCTRCCB FOR SELECT 0 AS LEVEL_ID, PMO_CNST.SC_CONTRACT AS TYPE_ID, h_contract_id AS PARENT_ID,
   ELEMENT_ID, 0 AS CHILD_COUNT, '' AS ELEMENT_NAME, CHARGE_DATE, CHARGE_AMOUNT
   FROM CONTRACT_BREAKDOWN
   WHERE CONTRACT_ID = h_contract_id
   ORDER BY CHARGE_DATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_CONTRACT_CCB', 45+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_CONTRACT_CCB(46) : contracts.sql(1387) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN CCTRCCB;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTRACT_CCB', 53+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTRACT_CCB(54) : contracts.sql(1395) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_CONTRACT_CCB', 65+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_CONTRACT_CCB(66) : contracts.sql(1407) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_CONTRACT_CCB;
ALTER FUNCTION SP_S_CONTRACT_CCB COMPILE DEBUG;

/*****************************************************************************************/

-- Update charge date of contract cross charge breakdown

-- p_id: element_id.

/*****************************************************************************************/

Prompt == FUNCTION SP_U_CONTRACT_CCB ==
CREATE OR REPLACE FUNCTION SP_U_CONTRACT_CCB
(
     p_id VARCHAR2
   , p_charge_date VARCHAR2
   , p_charge_amount NUMBER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_contract_id VARCHAR2(32);
   h_charge_date DATE;
   h_rec_user VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   h_charge_amount NUMBER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CONTRACT_CCB') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CONTRACT_CCB'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CONTRACT_CCB(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_charge_date || ''''
          || ', ' || COALESCE(TO_CHAR(p_charge_amount), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CCB', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CONTRACT_CCB(27) : contracts.sql(1456) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_charge_date := p_charge_date;
   h_charge_amount := p_charge_amount;
   h_rec_user := p_rec_user;
   
   If (h_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_CONTRACT_CCB', 38+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_CONTRACT_CCB(39) : contracts.sql(1468) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CCB', 41+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_CCB(42) : contracts.sql(1471) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      SELECT CONTRACT_ID INTO h_contract_id
      FROM CONTRACT_BREAKDOWN 
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 56+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(57) : contracts.sql(1486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_CCB', 63+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_CCB(64) : contracts.sql(1493) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CCB', 66+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_CCB(67) : contracts.sql(1496) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   BEGIN
      SELECT START_DATE INTO h_charge_date
      FROM FISCAL_CALENDAR
      WHERE TO_DATE(SubStr(h_charge_date, 1, 10)) BETWEEN START_DATE AND FINISH_DATE;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 81+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(82) : contracts.sql(1511) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_count := 0;
   BEGIN
      SELECT COUNT(*) INTO h_count
      FROM CONTRACT_BREAKDOWN 
      WHERE CONTRACT_ID = h_contract_id
      AND CHARGE_DATE = h_charge_date
      AND ELEMENT_ID <> h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 99+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(100) : contracts.sql(1529) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CONTRACT_CCB', 106+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CONTRACT_CCB(107) : contracts.sql(1536) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CONTRACT_BREAKDOWN_ALREADY_EXI), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CONTRACT_CCB', 109+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CONTRACT_CCB(110) : contracts.sql(1539) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CONTRACT_BREAKDOWN_ALREADY_EXI, h_id
            , True);
   End If;
   
   BEGIN
      UPDATE CONTRACT_BREAKDOWN
      SET CHARGE_DATE = h_charge_date,
      CHARGE_AMOUNT = h_charge_amount
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 130+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(131) : contracts.sql(1560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CONTRACT_CCB', 136+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CONTRACT_CCB(137) : contracts.sql(1566) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 143+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(144) : contracts.sql(1573) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CONTRACT_CCB', 155+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CONTRACT_CCB(156) : contracts.sql(1585) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CONTRACT_CCB;
ALTER FUNCTION SP_U_CONTRACT_CCB COMPILE DEBUG;

/*****************************************************************************************/

-- Delete contract cross charge breakdown

-- p_id: element_id.

/*****************************************************************************************/

Prompt == FUNCTION SP_D_CONTRACT_CCB ==
CREATE OR REPLACE FUNCTION SP_D_CONTRACT_CCB
(
     p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_charge_date DATE;
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_CONTRACT_CCB') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_CONTRACT_CCB'), '-- ' || SYSTIMESTAMP, ':r := SP_D_CONTRACT_CCB(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTRACT_CCB', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_CONTRACT_CCB(20) : contracts.sql(1627) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_rec_user := p_rec_user;
   
   If (h_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_CONTRACT_CCB', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_CONTRACT_CCB(30) : contracts.sql(1637) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_CONTRACT_CCB', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_CONTRACT_CCB(33) : contracts.sql(1640) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      DELETE FROM CONTRACT_BREAKDOWN WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT_CCB', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT_CCB(50) : contracts.sql(1657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_CONTRACT_CCB', 55+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_CONTRACT_CCB(56) : contracts.sql(1663) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT_CCB', 62+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT_CCB(63) : contracts.sql(1670) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_CONTRACT_CCB', 74+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_CONTRACT_CCB(75) : contracts.sql(1682) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_CONTRACT_CCB;
ALTER FUNCTION SP_D_CONTRACT_CCB COMPILE DEBUG;

/**********************************************************************************/

-- This function moves a Contract to higher level in the same category

Prompt == FUNCTION FC_MOVE_UP_CONTRACT ==
CREATE OR REPLACE FUNCTION FC_MOVE_UP_CONTRACT
(
     sh_h_target_level PLS_INTEGER
   , h_target_rank IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_target_level PLS_INTEGER := sh_h_target_level;
   h_cur_rank VARCHAR2(200);
   h_next_rank VARCHAR2(200);
   h_extend_rank VARCHAR2(200);
   h_len PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR MVUP_CNT IS
         SELECT RANK
         FROM TMT_CONTRACTS
         WHERE LEVEL_ID = h_target_level
         AND RANK BETWEEN h_target_rank AND h_source_rank
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_MOVE_UP_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_MOVE_UP_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := FC_MOVE_UP_CONTRACT(' || 
       COALESCE(TO_CHAR(sh_h_target_level), 'NULL')
          || ', ' || '''' || h_target_rank || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_UP_CONTRACT', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_MOVE_UP_CONTRACT(32) : contracts.sql(1732) : ' || SYSTIMESTAMP);
   END IF; 

   h_len := Coalesce(Length(h_source_rank), 0);
   BEGIN
      SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) INTO h_extend_rank
      FROM DUAL
      WHERE 1 = 1;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_UP_CONTRACT', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_UP_CONTRACT(46) : contracts.sql(1746) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN MVUP_CNT;
   
   h_next_rank := h_source_rank;
   LOOP
      FETCH MVUP_CNT INTO h_cur_rank;
      IF MVUP_CNT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      BEGIN
         UPDATE TMT_CONTRACTS
         SET RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_cur_rank)
          || h_extend_rank
         ELSE RTRIM(h_cur_rank)
          || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) || h_extend_rank END
         WHERE (SUBSTR(RANK, LENGTH(RANK)- 3, 4 ) <> h_extend_rank)
         AND RANK LIKE h_next_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_UP_CONTRACT', 82+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_UP_CONTRACT(83) : contracts.sql(1783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_next_rank := h_cur_rank;
   END LOOP;
   BEGIN
      CLOSE MVUP_CNT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_UP_CONTRACT', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_UP_CONTRACT(98) : contracts.sql(1798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET RANK = SUBSTR(RANK, 1, LENGTH(RANK) - 4)
      WHERE RANK LIKE SUBSTR(h_target_rank, 1, LENGTH(h_target_rank) -4) || '%'
      AND SUBSTR(RANK, LENGTH(RANK) - 3, 4) = h_extend_rank;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_UP_CONTRACT', 117+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_UP_CONTRACT(118) : contracts.sql(1818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_UP_CONTRACT', 123+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MOVE_UP_CONTRACT(124) : contracts.sql(1824) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_UP_CONTRACT', 127+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MOVE_UP_CONTRACT(128) : contracts.sql(1828) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_MOVE_UP_CONTRACT;
ALTER FUNCTION FC_MOVE_UP_CONTRACT COMPILE DEBUG;

/**********************************************************************************/

-- This function moves a Contract to lower level in the same category

Prompt == FUNCTION FC_MOVE_DOWN_CONTRACT ==
CREATE OR REPLACE FUNCTION FC_MOVE_DOWN_CONTRACT
(
     sh_h_target_level PLS_INTEGER
   , h_target_rank IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_target_level PLS_INTEGER := sh_h_target_level;
   h_cur_rank VARCHAR2(200);
   h_next_rank VARCHAR2(200);
   h_extend_rank VARCHAR2(200);
   h_len PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR MVDOWN_CNT IS
         SELECT RANK
         FROM TMT_CONTRACTS
         WHERE LEVEL_ID = h_target_level
         AND RANK BETWEEN h_source_rank AND h_target_rank
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_MOVE_DOWN_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_MOVE_DOWN_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := FC_MOVE_DOWN_CONTRACT(' || 
       COALESCE(TO_CHAR(sh_h_target_level), 'NULL')
          || ', ' || '''' || h_target_rank || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_DOWN_CONTRACT', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_MOVE_DOWN_CONTRACT(32) : contracts.sql(1872) : ' || SYSTIMESTAMP);
   END IF; 

   h_len := Coalesce(Length(h_source_rank), 0);
   BEGIN
      SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) INTO h_extend_rank
      FROM DUAL
      WHERE 1 = 1;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_DOWN_CONTRACT', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_DOWN_CONTRACT(46) : contracts.sql(1886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN MVDOWN_CNT;
   
   h_next_rank := h_target_rank;
   LOOP
      FETCH MVDOWN_CNT INTO h_cur_rank;
      IF MVDOWN_CNT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      BEGIN
         UPDATE TMT_CONTRACTS
         SET RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_next_rank)
          || h_extend_rank
         ELSE RTRIM(h_next_rank)
          || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) || h_extend_rank END
         WHERE (SUBSTR(RANK, LENGTH(RANK)- 3, 4 ) <> h_extend_rank)  AND RANK LIKE h_cur_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_DOWN_CONTRACT', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_DOWN_CONTRACT(82) : contracts.sql(1922) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_next_rank := h_cur_rank;
   END LOOP;
   BEGIN
      CLOSE MVDOWN_CNT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_DOWN_CONTRACT', 96+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_DOWN_CONTRACT(97) : contracts.sql(1937) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET RANK = SUBSTR(RANK, 1, LENGTH(RANK) - 4)
      WHERE RANK LIKE SUBSTR(h_target_rank,1, LENGTH(h_target_rank) -4) || '%'
      AND SUBSTR(RANK, LENGTH(RANK) - 3, 4) = h_extend_rank;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MOVE_DOWN_CONTRACT', 116+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MOVE_DOWN_CONTRACT(117) : contracts.sql(1957) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_DOWN_CONTRACT', 122+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MOVE_DOWN_CONTRACT(123) : contracts.sql(1963) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MOVE_DOWN_CONTRACT', 126+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MOVE_DOWN_CONTRACT(127) : contracts.sql(1967) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_MOVE_DOWN_CONTRACT;
ALTER FUNCTION FC_MOVE_DOWN_CONTRACT COMPILE DEBUG;

/****************************************************************************************/

-- move a contract to another category in a special position

Prompt == FUNCTION FC_M_CONTRACT_LOCATON ==
CREATE OR REPLACE FUNCTION FC_M_CONTRACT_LOCATON
(
     sh_h_target_level PLS_INTEGER
   , sh_h_source_level PLS_INTEGER
   , h_target_rank IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_target_parent_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_target_level PLS_INTEGER := sh_h_target_level;
   h_source_level PLS_INTEGER := sh_h_source_level;
   h_cur_rank VARCHAR2(200);
   h_next_rank VARCHAR2(200);
   h_extend_rank VARCHAR2(200);
   h_len PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(200);
   h_new_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR MVDIF_CNT IS
         SELECT RANK
         FROM TMT_CONTRACTS
         WHERE RANK LIKE SUBSTR(h_target_rank, 1, LENGTH(h_target_rank) - 4) || '%'
         AND LEVEL_ID = h_target_level 
         AND RANK >= h_target_rank
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_CONTRACT_LOCATON') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_CONTRACT_LOCATON'), '-- ' || SYSTIMESTAMP, ':r := FC_M_CONTRACT_LOCATON(' || 
       COALESCE(TO_CHAR(sh_h_target_level), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_source_level), 'NULL')
          || ', ' || '''' || h_target_rank || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_target_parent_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_CONTRACT_LOCATON', 40+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_CONTRACT_LOCATON(41) : contracts.sql(2020) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) INTO h_extend_rank
      FROM DUAL
      WHERE 1 = 1;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(54) : contracts.sql(2033) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN MVDIF_CNT;
   
   h_next_rank := h_source_rank;
   LOOP
      FETCH MVDIF_CNT INTO h_cur_rank;
      IF MVDIF_CNT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_len := Coalesce(Length(h_next_rank), 0);
      
      BEGIN
         UPDATE TMT_CONTRACTS
         SET RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_cur_rank)
          || h_extend_rank
         ELSE RTRIM(h_cur_rank)
          || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) || h_extend_rank END,
         PARENT_ID = CASE WHEN RANK = h_source_rank THEN h_target_parent_id
         ELSE PARENT_ID END,
         LEVEL_ID = CASE WHEN h_next_rank = h_source_rank
         THEN LEVEL_ID - h_source_level + h_target_level ELSE LEVEL_ID END
         WHERE (SUBSTR(RANK, LENGTH(RANK)- 3, 4 ) <> h_extend_rank)
         AND RANK LIKE h_next_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 96+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(97) : contracts.sql(2076) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_next_rank := h_cur_rank;
   END LOOP;
   BEGIN
      CLOSE MVDIF_CNT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 111+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(112) : contracts.sql(2091) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_target_parent_id IS NULL) Then
      BEGIN
         UPDATE TMT_CONTRACTS
         SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 131+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(132) : contracts.sql(2111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MAX_RANK INTO h_max_rank
         FROM TMT_CONTRACTS
         WHERE ELEMENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 146+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(147) : contracts.sql(2126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_CONTRACTS
         SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE ELEMENT_ID = h_target_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 165+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(166) : contracts.sql(2145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MAX_RANK INTO h_max_rank
         FROM TMT_CONTRACTS
         WHERE ELEMENT_ID = h_target_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 180+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(181) : contracts.sql(2160) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      SELECT SUBSTR(h_target_rank, 1, LENGTH(h_target_rank) - 4) || h_rankBase64
      INTO h_new_rank
      FROM DUAL
      WHERE 1 = 1;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 199+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(200) : contracts.sql(2179) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_len := Coalesce(Length(h_next_rank), 0);
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_new_rank)
       || h_extend_rank
      ELSE RTRIM(h_new_rank)
       || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) || h_extend_rank END,
      PARENT_ID = CASE WHEN RANK = h_source_rank THEN h_target_parent_id
      ELSE PARENT_ID END
      WHERE (SUBSTR(RANK, LENGTH(RANK)- 3, 4 ) <> h_extend_rank)
      AND RANK LIKE h_next_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 226+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(227) : contracts.sql(2206) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CONTRACTS
      SET RANK = SUBSTR(RANK, 1, LENGTH(RANK) - 4)
      WHERE RANK LIKE SUBSTR(h_target_rank, 1, LENGTH(h_target_rank) -4) || '%'
      AND SUBSTR(RANK, LENGTH(RANK) - 3, 4) = h_extend_rank;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT_LOCATON', 246+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT_LOCATON(247) : contracts.sql(2226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_CONTRACT_LOCATON', 252+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_CONTRACT_LOCATON(253) : contracts.sql(2232) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_CONTRACT_LOCATON', 256+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_CONTRACT_LOCATON(257) : contracts.sql(2236) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_CONTRACT_LOCATON;
ALTER FUNCTION FC_M_CONTRACT_LOCATON COMPILE DEBUG;

/****************************************************************************************/

-- contracts:

-- h_bitflag determine the move type

-- normal move: 4

-- move to sibling item: 5

Prompt == FUNCTION FC_M_CONTRACT ==
CREATE OR REPLACE FUNCTION FC_M_CONTRACT
(
     h_target_id IN OUT NOCOPY VARCHAR2
   , h_source_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_source_level PLS_INTEGER := 0;
   h_source_rank VARCHAR2(200);
   h_source_parent_id VARCHAR2(32);
   h_target_parent_id VARCHAR2(32);
   h_target_rank VARCHAR2(200);
   h_target_level PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := FC_M_CONTRACT(' || 
       '''' || h_target_id || ''''
          || ', ' || '''' || h_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_CONTRACT', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_CONTRACT(31) : contracts.sql(2285) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT PARENT_ID, RANK, LEVEL_ID
      INTO h_source_parent_id, h_source_rank, h_source_level
      FROM TMT_CONTRACTS
      WHERE ELEMENT_ID = h_source_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(45) : contracts.sql(2299) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_target_id IS NULL) Then
      BEGIN
         SELECT PARENT_ID, RANK, LEVEL_ID
         INTO h_target_parent_id, h_target_rank, h_target_level
         FROM TMT_CONTRACTS
         WHERE ELEMENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 61+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(62) : contracts.sql(2316) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT PARENT_ID, RANK,LEVEL_ID
         INTO h_target_parent_id, h_target_rank, h_target_level
         FROM TMT_CONTRACTS
         WHERE ELEMENT_ID = h_target_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 77+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(78) : contracts.sql(2332) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (BITAND(Coalesce(h_bitflag, 0), 1)) = 1 Then
      If (((h_target_parent_id IS NULL)) AND ((h_source_parent_id IS NULL))) OR (PMO_ORASTD.strcmp(
            h_target_parent_id, h_source_parent_id) = PMO_CNST.SC_MATCH) Then
         If PMO_ORASTD.strcmp(h_source_rank, h_target_rank) > PMO_CNST.SC_MATCH Then
            PMO_DB2M.SC_STATE := FC_MOVE_UP_CONTRACT(h_target_level, h_target_rank, h_source_rank, h_rec_user);
         Else
            PMO_DB2M.SC_STATE := FC_MOVE_DOWN_CONTRACT(h_target_level, h_target_rank, h_source_rank, h_rec_user);
         End If;
      Else
         -- move sibling to another category
         PMO_DB2M.SC_STATE := FC_M_CONTRACT_LOCATON(h_target_level, h_source_level, h_target_rank
                                           , h_source_rank, h_target_parent_id, h_rec_user);
         
         BEGIN
            UPDATE TMT_CONTRACTS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID = h_source_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 110+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(111) : contracts.sql(2365) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      --update source hierarchy
      If (h_source_parent_id IS NOT NULL) Then
         BEGIN
            UPDATE TMT_CONTRACTS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID = h_source_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 132+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(133) : contracts.sql(2387) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (h_target_id IS NULL) Then
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CONTRACTS', h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         h_level_id := 0;
         h_rank := h_target_rank || h_rankBase64;
         h_rank := h_rank;
      Else
         PMODEFINES.PMO_GET_MAX_RANK('TMT_CONTRACTS', h_target_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         h_level_id := Coalesce(h_target_level, 0) + 1;
         h_level_id := Coalesce(h_level_id, 0);
         h_rank := h_target_rank || h_rankBase64;
         h_rank := h_rank;
      End If;
      
      h_len := Coalesce(Length(h_source_rank), 0);
      h_len := Coalesce(h_len, 0);
      
      --trace_printf("\n h_source_rank:(%s)", h_source_rank);
      
      BEGIN
         UPDATE TMT_CONTRACTS
         SET LEVEL_ID = LEVEL_ID + h_level_id - h_source_level,
         RANK = h_rank || CASE WHEN LENGTH(RANK) = h_len THEN '' ELSE SUBSTR(RANK, h_len + 1, LENGTH(RANK) - h_len) END,
         PARENT_ID = CASE WHEN ELEMENT_ID = h_source_id THEN h_target_id ELSE PARENT_ID END,
         REC_USER     = h_rec_user,
         REC_STATUS   = 'U',
         REC_DATETIME = SYSTIMESTAMP
         WHERE RANK LIKE h_source_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_CONTRACT', 179+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_CONTRACT(180) : contracts.sql(2434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_CONTRACT', 189+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_CONTRACT(190) : contracts.sql(2444) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_CONTRACT;
ALTER FUNCTION FC_M_CONTRACT COMPILE DEBUG;

/************************************************************************************************/

Prompt == FUNCTION SP_M_CONTRACT ==
CREATE OR REPLACE FUNCTION SP_M_CONTRACT
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_user VARCHAR2(32);
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_CONTRACT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_CONTRACT'), '-- ' || SYSTIMESTAMP, ':r := SP_M_CONTRACT(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_CONTRACT', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_CONTRACT(25) : contracts.sql(2479) : ' || SYSTIMESTAMP);
   END IF; 

   h_target_id := p_target_id;
   h_source_id := p_source_id;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   PMO_DB2M.SC_STATE := FC_M_CONTRACT(h_target_id, h_source_id, h_bitflag, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_CONTRACT', 35+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_CONTRACT(36) : contracts.sql(2490) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_source_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_source_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_CONTRACT', 43+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_CONTRACT(44) : contracts.sql(2498) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_CONTRACT', 55+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_CONTRACT(56) : contracts.sql(2510) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_CONTRACT;
ALTER FUNCTION SP_M_CONTRACT COMPILE DEBUG;

