/****************************************************************************************/

Prompt == FUNCTION SP_I_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_I_DATAFIELD
(
     p_name VARCHAR2
   , p_description VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_parent_id_int PLS_INTEGER := 0;
   h_id_count PLS_INTEGER := 0;
   h_level PLS_INTEGER := 0;
   h_broker_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_I_DATAFIELD(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_DATAFIELD', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_DATAFIELD(33) : datafields.sql(36) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_broker_type := 90;
   
   BEGIN
      SELECT COUNT(ELEMENT_ID) INTO h_id_count FROM TMT_DATAFIELDS;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(48) : datafields.sql(51) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_id_count, 0) = 0 Then
      h_uniqueID := 1;
   Else
      BEGIN
         SELECT MAX(ELEMENT_ID)+1 INTO h_uniqueID FROM TMT_DATAFIELDS;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 63+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(64) : datafields.sql(67) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   -- Validate parent_id
   If (p_parent_id IS NULL) Then
      -- Get the max sibling level from the root
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_DATAFIELDS', h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      --Execute Insert Statement
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1 INTO h_rank, h_level
         FROM TMT_DATAFIELDS WHERE ELEMENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 87+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(88) : datafields.sql(91) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_DATAFIELDS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, DESCRIPTION)
         VALUES(h_rank, -1, h_uniqueID, p_name, p_type_id, h_level, p_rec_user, p_description);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(106) : datafields.sql(109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Validate parent_id
      h_parent_id := p_parent_id;
      h_parent_id_int := TO_NUMBER(h_parent_id);
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_I_DATAFIELD', 115+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_I_DATAFIELD(116) : datafields.sql(119) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_parent_id_int), 'NULL'));
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_DATAFIELDS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id_int; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id_int, True); 
      END;
      
      -- Get next sibling count
      PMODEFINES.PMO_GET_MAX_RANK('TMT_DATAFIELDS', h_parent_id_int, h_max_rank);
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      --Execute Insert Statement
      BEGIN
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64, LEVEL_ID + 1 INTO h_rank, h_level
         FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_parent_id_int;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 142+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(143) : datafields.sql(146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_DATAFIELDS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, DESCRIPTION)
         VALUES(h_rank, h_parent_id_int, h_uniqueID, p_name, p_type_id, h_level, p_rec_user, p_description);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 160+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(161) : datafields.sql(164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 180+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(181) : datafields.sql(184) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_DATAFIELD', 192+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_DATAFIELD(193) : datafields.sql(196) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_uniqueID), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 199+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(200) : datafields.sql(203) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_DATAFIELD', 211+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_DATAFIELD(212) : datafields.sql(215) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_DATAFIELD;
ALTER FUNCTION SP_I_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_U_DATAFIELD
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_id_int PLS_INTEGER := 0;
   h_rec_status VARCHAR2(1);
   h_broker_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_U_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_DATAFIELD', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_DATAFIELD(26) : datafields.sql(257) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_DATAFIELD', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_DATAFIELD(33) : datafields.sql(264) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_DATAFIELD', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_DATAFIELD(36) : datafields.sql(267) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_broker_type := 90;
   
   h_id := p_id;
   h_id_int := TO_NUMBER(h_id);
   
   BEGIN
      SELECT TYPE_ID, REC_STATUS 
      INTO h_type_id, h_rec_status
      FROM TMT_DATAFIELDS 
      WHERE ELEMENT_ID = h_id_int;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 56+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(57) : datafields.sql(288) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_DATAFIELD', 63+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_DATAFIELD(64) : datafields.sql(295) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_DATAFIELD', 66+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_DATAFIELD(67) : datafields.sql(298) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
   End If;
   
   BEGIN
      UPDATE TMT_DATAFIELDS SET
      ELEMENT_NAME = p_name,
      DESCRIPTION  = p_description,
      REC_STATUS   = 'U',
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER     = p_rec_user
      WHERE ELEMENT_ID = h_id_int;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 89+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(90) : datafields.sql(321) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If Coalesce(h_type_id, 0) = 7 Then
         -- Add rank like '0000' || '%' to force the query using index
         BEGIN
            UPDATE TMT_COMPETENCIES SET
            ELEMENT_NAME = p_description
            WHERE PROFICIENCY_LEVEL_ID = h_id_int
            AND RANK LIKE '0000' || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(113) : datafields.sql(344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(132) : datafields.sql(363) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_DATAFIELD', 143+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_DATAFIELD(144) : datafields.sql(375) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id_int, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 150+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(151) : datafields.sql(382) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_DATAFIELD', 162+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_DATAFIELD(163) : datafields.sql(394) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_DATAFIELD;
ALTER FUNCTION SP_U_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_S_DATAFIELD
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_id_int PLS_INTEGER := 0;
   h_fs_flag PLS_INTEGER := 0;
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_S_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_DATAFIELD(22) : datafields.sql(432) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   BEGIN
      SELECT SETTING_VALUE INTO h_fs_flag
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'REPOSITORY_TYPE';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 36+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(37) : datafields.sql(447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_sql := 
         'SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, DELETED_COUNT, REC_STATUS, POSITION, DESCRIPTION ';
   h_sql := h_sql || 'FROM TMT_DATAFIELDS ';
   
   If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
      h_sql := h_sql || 'WHERE ELEMENT_ID IS NOT NULL ';
   Else
      -- Validate id
      If ((p_id IS NULL)) Then
         h_sql := h_sql || 'WHERE PARENT_ID IS NULL ';
         h_sql := h_sql || 'AND TYPE_ID <> 0 ';
      Else
         h_id := p_id;
         h_id_int := TO_NUMBER(h_id);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_DATAFIELD', 56+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_DATAFIELD(57) : datafields.sql(467) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_DATAFIELDS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id_int; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id_int, True); 
         END;
         
         h_sql := h_sql || 'WHERE ( PARENT_ID = ';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ' OR ELEMENT_ID = ';
         h_sql := h_sql || h_id;
         h_sql := h_sql || ' ) ';
      End If;
   End If;
   If Coalesce(h_fs_flag, 0) <> 0 Then
      h_sql := h_sql || 'AND REC_STATUS <> ''D'' ';
   End If;
   h_sql := h_sql || ' ORDER BY RANK ';
   
   --Return a resultset
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 81+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_DATAFIELD(82) : datafields.sql(492) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_DATAFIELD', 84+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_DATAFIELD(85) : datafields.sql(495) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 91+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(92) : datafields.sql(502) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_DATAFIELD', 103+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_DATAFIELD(104) : datafields.sql(514) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_DATAFIELD;
ALTER FUNCTION SP_S_DATAFIELD COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_DATAFIELD ==
CREATE OR REPLACE FUNCTION SP_D_DATAFIELD
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_parent_id_int PLS_INTEGER := 0;
   h_id_int PLS_INTEGER := 0;
   h_broker_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_DATAFIELD') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_DATAFIELD'), '-- ' || SYSTIMESTAMP, ':r := SP_D_DATAFIELD(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_DATAFIELD', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_DATAFIELD(28) : datafields.sql(558) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_broker_type := 90;
   
   -- Validate id
   h_id := p_id;
   h_id_int := TO_NUMBER(h_id);
   
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_DATAFIELD', 40+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_DATAFIELD(41) : datafields.sql(571) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   BEGIN 
      IF h_rec_status = 'X' THEN 
         EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || 'TMT_DATAFIELDS' || ' a 
         WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS 
         (SELECT b.REC_STATUS 
                  FROM TMT_ORGANIZATIONS b 
                 WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)' 
            INTO h_rec_status 
            Using h_id_int; 
      ELSE 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || 'TMT_DATAFIELDS' || ' 
              WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') ' 
            INTO h_rec_status 
            Using h_id_int; 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, h_id_int, True); 
      WHEN OTHERS THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id_int , True); 
   END;
   
   -- Validate parent id
   If ((p_parent_id IS NULL)) Then
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_id_int AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 75+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(76) : datafields.sql(606) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      h_parent_id_int := TO_NUMBER(h_parent_id);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_DATAFIELD', 85+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_DATAFIELD(86) : datafields.sql(616) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_DATAFIELDS' || '''' || ' ID: ' || COALESCE(TO_CHAR(h_parent_id_int), 'NULL'));
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_DATAFIELDS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id_int; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id_int, True); 
      END;
      
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_DATAFIELD', 98+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_DATAFIELD(99) : datafields.sql(629) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_DATAFIELD', 101+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_DATAFIELD(102) : datafields.sql(632) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_id_int AND PARENT_ID = h_parent_id_int;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 115+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(116) : datafields.sql(646) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_DATAFIELD', 123+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_DATAFIELD(124) : datafields.sql(654) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_DATAFIELD', 126+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_DATAFIELD(127) : datafields.sql(657) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(0, h_id_int, True);
   End If;
   
   -- Execute Delete Statement
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      -- Delete Tree
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      BEGIN
         DELETE FROM TMT_DATAFIELDS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 159+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(160) : datafields.sql(690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update Parent's Hierarchy Info
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_DATAFIELDS SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(181) : datafields.sql(711) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_DATAFIELDS SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id_int;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 198+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(199) : datafields.sql(729) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 218+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(219) : datafields.sql(749) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID = h_parent_id_int;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 236+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(237) : datafields.sql(767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_DATAFIELD', 245+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_DATAFIELD(246) : datafields.sql(776) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_DATAFIELD', 248+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_DATAFIELD(249) : datafields.sql(779) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      -- Update Parent's Hierarchy Info
      If (p_parent_id IS NULL) Then
         BEGIN
            UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 268+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(269) : datafields.sql(799) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_DATAFIELDS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = h_parent_id_int;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 286+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(287) : datafields.sql(817) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_DATAFIELDS SET
         REC_STATUS   = p_del_flag,
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 321+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(322) : datafields.sql(852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         BEGIN
            UPDATE CHANGE_LOG SET MODIF_DATE_TIME = SYSTIMESTAMP WHERE BROKER_TYPE = h_broker_type;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 341+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(342) : datafields.sql(872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         PMO_DB2M.SC_STATE := 0;
      END;
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_DATAFIELD', 353+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_DATAFIELD(354) : datafields.sql(884) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || COALESCE(TO_CHAR(h_id_int), 'NULL'));
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id_int, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 360+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(361) : datafields.sql(891) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_DATAFIELD', 372+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_DATAFIELD(373) : datafields.sql(903) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_DATAFIELD;
ALTER FUNCTION SP_D_DATAFIELD COMPILE DEBUG;

