/****************************************************************************************/

-- Converting contingency columns to contingency row

/****************************************************************************************/

Prompt == FUNCTION SP_MIG_CONTINGENCY ==
CREATE OR REPLACE FUNCTION SP_MIG_CONTINGENCY
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
      CURSOR C_CONTINGENCY IS
         SELECT CONTINGENCY_ID
         FROM PMO_CONTINGENCY;

      CURSOR C_CON_ROLLUP IS
         SELECT DISTINCT B.PROJECT_ID
         FROM BUDGETS B, TMT_CHARGECODES CC
         WHERE (B.CONTINGENCY_BUDGET + B.CONTINGENCY_ETC) > 0.00
         AND CC.ELEMENT_ID = B.CHARGECODE_ID
         AND CC.RANK LIKE '0000%';

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_MIG_CONTINGENCY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_MIG_CONTINGENCY'), '-- ' || SYSTIMESTAMP, ':r := SP_MIG_CONTINGENCY(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_MIG_CONTINGENCY', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_MIG_CONTINGENCY(34) : migration_60_61.sql(41) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_CONTINGENCY';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_CONTINGENCY 
      (CHARGECODE_ID, CHARGECODE_NAME, REC_STATUS)
      SELECT B.CHARGECODE_ID, CC.ELEMENT_NAME, CC.REC_STATUS
      FROM BUDGETS B, TMT_CHARGECODES CC
      WHERE (B.CONTINGENCY_BUDGET + B.CONTINGENCY_ETC) > 0.00
      AND CC.ELEMENT_ID = B.CHARGECODE_ID
      AND CC.RANK LIKE '0000%'
      GROUP BY B.CHARGECODE_ID, CC.ELEMENT_NAME, CC.REC_STATUS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 59+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(60) : migration_60_61.sql(67) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_CONTINGENCY
      SET CONTINGENCY_ID = SYS_GUID()
      WHERE CONTINGENCY_ID IS NuLL;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 78+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(79) : migration_60_61.sql(86) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CHARGECODES', h_table_type);
   
   --Get Base 64
   PMO_STPU.toBase64(Coalesce(h_table_type, 0), h_rank);
   h_rank := h_rank;
   h_uniqueID := SYS_GUID();
   h_uniqueID := h_uniqueID;
   
   -- insert-select
   BEGIN
      INSERT INTO TMT_CHARGECODES
      (RANK, PARENT_ID, ELEMENT_ID,
      ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER)
      SELECT RTRIM(COALESCE(RANK, '')) || h_rank, ELEMENT_ID, h_uniqueID,
      'System Contingency', PMO_CNST.SC_FOLDER, 0, REC_USER
      FROM TMT_CHARGECODES
      WHERE ELEMENT_ID IS NuLL;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 110+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(111) : migration_60_61.sql(118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_CONTINGENCY;
   
   h_table_type := 0;
   LOOP
      FETCH C_CONTINGENCY INTO h_element_id;
      IF C_CONTINGENCY%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         EXIT;
      End If;
      
      h_table_type := Coalesce(h_table_type, 0) + 1;
      PMO_STPU.toBase64(Coalesce(h_table_type, 0), h_rank);
      
      BEGIN
         INSERT INTO TMT_CHARGECODES
         (RANK, PARENT_ID, ELEMENT_ID,
         ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, REC_STATUS)
         SELECT RTRIM(COALESCE(CC.RANK, '')) || h_rank, h_uniqueID, h_element_id,
         TMP.CHARGECODE_NAME, PMO_CNST.SC_CONTINGENCY, 1, CC.REC_USER, TMP.REC_STATUS
         FROM TMT_CHARGECODES CC, PMO_CONTINGENCY TMP
         WHERE CC.ELEMENT_ID = h_uniqueID
         AND TMP.CONTINGENCY_ID = h_element_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 152+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(153) : migration_60_61.sql(160) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   BEGIN
      CLOSE C_CONTINGENCY;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 165+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(166) : migration_60_61.sql(173) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CHARGECODES P
      SET P.MAX_RANK = h_table_type+ 1,
      P.CHILD_COUNT = (SELECT COUNT(*) FROM TMT_CHARGECODES C 
      WHERE C.PARENT_ID = P.ELEMENT_ID AND C.REC_STATUS <> 'D')
      WHERE P.ELEMENT_ID = h_uniqueID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 186+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(187) : migration_60_61.sql(194) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO BUDGETS 
      (BUDGET_ID, PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, CAPITAL_EXPENSE, 
      BILLABLE, CREATED_BY, REC_USER, REC_STATUS, REC_DATETIME, ROLLUP_BITS, 
      TYPE_ID, ESTIMATED_BUDGET, ESTIMATED_ETC, ESTIMATED_TOTAL, 
      ESTIMATED_ETC_TOTAL, ESTIMATED_BASELINE1, ESTIMATED_BASELINE2, 
      ESTIMATED_BASELINE3)
      SELECT SYS_GUID(), B.PROJECT_ID, B.ELEMENT_ID, TMP.CONTINGENCY_ID, PMO_CNST.SC_CONTINGENCY, 
      PMO_CNST.SC_BILLABLE_NO, B.CREATED_BY, B.REC_USER, B.REC_STATUS, B.REC_DATETIME, 1048576, 
      B.TYPE_ID, B.CONTINGENCY_BUDGET, B.CONTINGENCY_ETC, B.CONTINGENCY_TOTAL, 
      B.CONTINGENCY_ETC_TOTAL, B.CONTINGENCY_BASELINE1, B.CONTINGENCY_BASELINE2, 
      B.CONTINGENCY_BASELINE3
      FROM BUDGETS B, TMT_CHARGECODES CC, PMO_CONTINGENCY TMP
      WHERE (B.CONTINGENCY_BUDGET + B.CONTINGENCY_ETC) > 0.00
      AND CC.ELEMENT_ID = B.CHARGECODE_ID
      AND CC.RANK LIKE '0000%'
      AND TMP.CHARGECODE_ID = B.CHARGECODE_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 218+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(219) : migration_60_61.sql(226) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_WBS
      SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
      WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
      FROM BUDGETS B, TMT_CHARGECODES CC
      WHERE (B.CONTINGENCY_BUDGET + B.CONTINGENCY_ETC) > 0.00
      AND CC.ELEMENT_ID = B.CHARGECODE_ID
      AND CC.RANK LIKE '0000%');
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 241+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(242) : migration_60_61.sql(249) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_CRI
      SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
      WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
      FROM BUDGETS B, TMT_CHARGECODES CC
      WHERE (B.CONTINGENCY_BUDGET + B.CONTINGENCY_ETC) > 0.00
      AND CC.ELEMENT_ID = B.CHARGECODE_ID
      AND CC.RANK LIKE '0000%');
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 264+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(265) : migration_60_61.sql(272) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_CON_ROLLUP;
   
   LOOP
      FETCH C_CON_ROLLUP INTO h_element_id;
      IF C_CON_ROLLUP%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         EXIT;
      End If;
      
      PMO_DB2M.SC_STATE := FC_Rollup(h_element_id);
   END LOOP;
   BEGIN
      CLOSE C_CON_ROLLUP;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 293+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(294) : migration_60_61.sql(301) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_MIG_CONTINGENCY', 299+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_MIG_CONTINGENCY(300) : migration_60_61.sql(307) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 306+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(307) : migration_60_61.sql(314) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_CONTINGENCY', 318+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_CONTINGENCY(319) : migration_60_61.sql(326) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_MIG_CONTINGENCY;
ALTER FUNCTION SP_MIG_CONTINGENCY COMPILE DEBUG;

/*******************************************************************************/

-- Move all cost centers from tmt_cost_centers to tmt_clients

-- tmt_cost_centers table will be deleted from 52.

/*******************************************************************************/

Prompt == FUNCTION SP_T_COST_CENTER_2_CLIENT ==
CREATE OR REPLACE FUNCTION SP_T_COST_CENTER_2_CLIENT
(
     p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_level_id PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_element_id VARCHAR2(32);
   h_element_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rec_datetime TIMESTAMP;
   h_description VARCHAR2(2047);
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR ALL_CCS IS
         SELECT LEVEL_ID, TYPE_ID, ELEMENT_ID, ELEMENT_NAME, PARENT_ID, REC_USER, REC_STATUS, REC_DATETIME, DESCRIPTION
         FROM TMT_COST_CENTERS
         WHERE LEVEL_ID > -1
         ORDER BY RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_T_COST_CENTER_2_CLIENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_T_COST_CENTER_2_CLIENT'), '-- ' || SYSTIMESTAMP, ':r := SP_T_COST_CENTER_2_CLIENT(' || 
       COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_T_COST_CENTER_2_CLIENT', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_T_COST_CENTER_2_CLIENT(33) : migration_60_61.sql(381) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN ALL_CCS;
   
   LOOP
      FETCH ALL_CCS
      INTO h_level_id, h_type_id, h_element_id, h_element_name, h_parent_id, h_rec_user, h_rec_status, h_rec_datetime, h_description;
      IF ALL_CCS
      %FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If Coalesce(h_type_id, 0) <> PMO_CNST.SC_FOLDER Then
         h_type_id := PMO_CNST.SC_CLIENTS;
      End If;
      
      If (h_parent_id IS NULL) Then
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_CLIENTS', h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         BEGIN
            INSERT INTO TMT_CLIENTS (RANK, LEVEL_ID, TYPE_ID, ELEMENT_ID, ELEMENT_NAME, PARENT_ID, REC_USER, REC_STATUS, REC_DATETIME, DESCRIPTION )
            SELECT RTRIM(RANK) || h_rankBase64, h_level_id, h_type_id, h_element_id, h_element_name, h_parent_id, h_rec_user, h_rec_status, h_rec_datetime, h_description
            FROM TMT_CLIENTS WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 73+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(74) : migration_60_61.sql(422) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT RTRIM(RANK)
            INTO h_rank
            FROM TMT_CLIENTS WHERE ELEMENT_ID = h_parent_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 88+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(89) : migration_60_61.sql(437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_CLIENTS', h_parent_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         BEGIN
            INSERT INTO TMT_CLIENTS(RANK, LEVEL_ID, TYPE_ID, ELEMENT_ID, ELEMENT_NAME, PARENT_ID, REC_USER, REC_STATUS, REC_DATETIME, DESCRIPTION )
            VALUES(RTRIM(h_rank) || h_rankBase64, h_level_id, h_type_id, h_element_id, h_element_name, h_parent_id, h_rec_user, h_rec_status, h_rec_datetime, h_description);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 109+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(110) : migration_60_61.sql(458) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE ALL_CCS;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 124+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(125) : migration_60_61.sql(473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_T_COST_CENTER_2_CLIENT', 130+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_T_COST_CENTER_2_CLIENT(131) : migration_60_61.sql(479) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_element_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_element_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 137+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(138) : migration_60_61.sql(486) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_T_COST_CENTER_2_CLIENT', 149+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_T_COST_CENTER_2_CLIENT(150) : migration_60_61.sql(498) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_T_COST_CENTER_2_CLIENT;
ALTER FUNCTION SP_T_COST_CENTER_2_CLIENT COMPILE DEBUG;


