/****************************************************************************************/

Prompt == FUNCTION SP_I_ORG ==
CREATE OR REPLACE FUNCTION SP_I_ORG
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_uniqueID VARCHAR2(32);
   h_max_rank PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_I_ORG(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_ORG', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_ORG(27) : organizations.sql(30) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_LONG(h_level);
   
   h_uniqueID := SYS_GUID();
   
   If (p_parent_id IS NULL) Then
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_ORGANIZATIONS', h_max_rank);
      
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_ORGANIZATIONS
         (RANK, PARENT_ID,ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER)
         SELECT RTRIM(COALESCE(RANK, '')) || h_rankBase64,
         p_parent_id,
         h_uniqueID,
         p_name,
         p_type_id,
         LEVEL_ID + 1,
         p_rec_user
         FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ORG', 60+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ORG(61) : organizations.sql(64) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_I_ORG', 67+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_I_ORG(68) : organizations.sql(71) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      
      PMODEFINES.PMO_GET_MAX_RANK('TMT_ORGANIZATIONS', h_parent_id, h_max_rank);
      
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_ORGANIZATIONS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER)
         SELECT RTRIM(Rank) || h_rankBase64,p_parent_id
         , h_uniqueID,p_name
         , p_type_id
         , LEVEL_ID + 1, p_rec_user
         FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ORG', 99+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ORG(100) : organizations.sql(103) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_ORG', 106+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_ORG(107) : organizations.sql(110) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ORG', 113+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ORG(114) : organizations.sql(117) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ORG', 125+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ORG(126) : organizations.sql(129) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_ORG;
ALTER FUNCTION SP_I_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_ORG ==
CREATE OR REPLACE FUNCTION SP_U_ORG
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_update PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ORG(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ORG', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ORG(24) : organizations.sql(169) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_ORG', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_ORG(29) : organizations.sql(174) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ORG', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_ORG(32) : organizations.sql(177) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_ORG', 40+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_ORG(41) : organizations.sql(186) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   BEGIN
      UPDATE TMT_ORGANIZATIONS SET
      ELEMENT_NAME=p_name,
      DESCRIPTION=p_description,
      REC_STATUS='U',
      REC_DATETIME=SYSTIMESTAMP,
      REC_USER=p_rec_user
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ORG', 69+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ORG(70) : organizations.sql(215) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT
         case WHEN EXISTS( SELECT ELEMENT_ID FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id AND PARENT_ID IS NuLL) THEN 1
         else 0 END
         into h_update
         FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ORG', 87+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ORG(88) : organizations.sql(233) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ORG', 94+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ORG(95) : organizations.sql(240) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ORG', 101+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ORG(102) : organizations.sql(247) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ORG', 113+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ORG(114) : organizations.sql(259) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ORG;
ALTER FUNCTION SP_U_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_D_ORG ==
CREATE OR REPLACE FUNCTION SP_D_ORG
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_current_datetime TIMESTAMP;
   h_exists VARCHAR2(32);
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_default_count PLS_INTEGER := 0;
   h_dependency_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC1 IS
         SELECT ELEMENT_ID FROM TMT_ORGANIZATIONS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND DEFAULT_FLAG = 'Y';

      CURSOR CC2 IS
         SELECT ELEMENT_ID FROM TMT_ORGANIZATIONS
         WHERE  ROWNUM <= 1 AND  RANK LIKE h_rank || '%' AND TYPE_ID = PMO_CNST.SC_RESOURCE;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_D_ORG(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ORG', 36+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_ORG(37) : organizations.sql(312) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --  SC_CHAR(h_status, SC_H_FLAG);
   
   --added by paria
   h_parent_id := NULL;
   
   -----------------
   
   h_id := p_id;
   h_rec_status := p_del_flag;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_ORG', 50+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_ORG(51) : organizations.sql(326) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      IF h_rec_status = 'X' THEN 
         EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || 'TMT_ORGANIZATIONS' || ' a 
         WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS 
         (SELECT b.REC_STATUS 
                  FROM TMT_ORGANIZATIONS b 
                 WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)' 
            INTO h_rec_status 
            Using h_id; 
      ELSE 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || 'TMT_ORGANIZATIONS' || ' 
              WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') ' 
            INTO h_rec_status 
            Using h_id; 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, h_id, True); 
      WHEN OTHERS THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id , True); 
   END;
   
   If ((p_parent_id IS NULL)) Then
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 83+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(84) : organizations.sql(359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_rec_status := '';
   Else
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_D_ORG', 92+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_D_ORG(93) : organizations.sql(368) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_ORG', 104+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_ORG(105) : organizations.sql(380) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ORG', 107+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_ORG(108) : organizations.sql(383) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 121+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(122) : organizations.sql(397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   OPEN CC1;
   
   FETCH CC1 INTO h_exists;
   IF CC1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_default_count := 1;
   Else
      h_default_count := 0;
   End If;
   
   BEGIN
      CLOSE CC1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 150+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(151) : organizations.sql(426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CC2;
   
   FETCH CC2 INTO h_exists;
   IF CC2%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_dependency_count := 1;
   Else
      h_dependency_count := 0;
   End If;
   
   BEGIN
      CLOSE CC2 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 178+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(179) : organizations.sql(454) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      If Coalesce(h_default_count, 0) = 0 Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         BEGIN
            DELETE FROM TMT_ORGANIZATIONS WHERE RANK LIKE h_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 210+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(211) : organizations.sql(486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If SubStr(h_rec_status, 1, 1) = 'D' Then
            --comment by paria
            --MCR_UPDATE_TMT_ORGANIZATIONS(DELETED_COUNT);
            If (h_parent_id IS NULL) Then
               BEGIN
                  UPDATE TMT_ORGANIZATIONS SET DELETED_COUNT = DELETED_COUNT - 1 
                              WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 232+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(233) : organizations.sql(508) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_ORGANIZATIONS SET DELETED_COUNT = DELETED_COUNT - 1 
                              WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 250+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(251) : organizations.sql(526) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            --comment by paria
            --MCR_UPDATE_TMT_ORGANIZATIONS(CHILD_COUNT);
            
            If (h_parent_id IS NULL) Then
               BEGIN
                  UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1 
                              WHERE ELEMENT_ID IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 273+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(274) : organizations.sql(549) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1 
                              WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 291+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(292) : organizations.sql(567) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_ORG', 299+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_ORG(300) : organizations.sql(575) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DEFAULT_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ORG', 302+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_ORG(303) : organizations.sql(578) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.DEFAULT_CHILD_EXISTS, p_id, True);
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_ORG', 309+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_ORG(310) : organizations.sql(585) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ORG', 312+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_ORG(313) : organizations.sql(588) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      If Coalesce(h_default_count, 0) = 0 Then
         If (p_parent_id IS NULL) Then
            BEGIN
               UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 332+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(333) : organizations.sql(608) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 350+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(351) : organizations.sql(626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            SELECT (SYSTIMESTAMP)  INTO  h_current_datetime FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 364+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(365) : organizations.sql(640) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_ORGANIZATIONS
            SET REC_STATUS=p_del_flag,
            REC_DATETIME=h_current_datetime,
            REC_USER=p_rec_user
            WHERE RANK LIKE h_rank || '%' AND REC_STATUS IN ('I','U');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 398+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(399) : organizations.sql(674) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_ORG', 404+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_ORG(405) : organizations.sql(680) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DEFAULT_CHILD_EXISTS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ORG', 407+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_ORG(408) : organizations.sql(683) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.DEFAULT_CHILD_EXISTS, p_id, True);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_ORG', 414+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_ORG(415) : organizations.sql(690) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 421+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(422) : organizations.sql(697) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ORG', 433+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ORG(434) : organizations.sql(709) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_ORG;
ALTER FUNCTION SP_D_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ORG ==
CREATE OR REPLACE FUNCTION SP_S_ORG
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66672A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      org_ref_set PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66672B PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ORG(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ORG', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ORG(29) : organizations.sql(754) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_bitflag := p_bitflag;
   
   If ((p_id IS NULL)) Then
      OPEN C66672A FOR SELECT
      T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      (SELECT COUNT(C.ELEMENT_ID) FROM TMT_ORGANIZATIONS C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)) AS CHILD_COUNT,
      T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION,
      '' AS CREATED_BY, '' AS CREATED_BY_NAME,
      '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
      0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
      0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
      T.DEFAULT_FLAG, T.FS_CODE
      FROM TMT_ORGANIZATIONS T
      WHERE T.PARENT_ID IS NuLL
      AND T.REC_STATUS NOT IN ('D', 'L', 'A')
      AND T.TYPE_ID NOT IN (0, PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ORG', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ORG(52) : organizations.sql(777) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66672A;
   Else
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_ORG', 57+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_ORG(58) : organizations.sql(783) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      If Coalesce(h_bitflag, 0) = PMO_CNST.SC_REFRESH Then
         OPEN org_ref_set FOR SELECT
         T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
         (SELECT COUNT(C.ELEMENT_ID) FROM TMT_ORGANIZATIONS C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)) AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION,
         '' AS CREATED_BY, '' AS CREATED_BY_NAME,
         '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
         0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
         0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
         T.DEFAULT_FLAG, T.FS_CODE
         FROM TMT_ORGANIZATIONS T
         WHERE T.ELEMENT_ID = h_id
         AND T.REC_STATUS NOT IN ('D', 'L', 'A')
         AND T.TYPE_ID NOT IN (PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)
         ORDER BY T.RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ORG', 84+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_ORG(85) : organizations.sql(810) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN org_ref_set;
      Else
         OPEN C66672B FOR SELECT
         T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
         (SELECT COUNT(C.ELEMENT_ID) FROM TMT_ORGANIZATIONS C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)) AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DESCRIPTION,
         '' AS CREATED_BY, '' AS CREATED_BY_NAME,
         '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
         0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
         0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
         T.DEFAULT_FLAG, T.FS_CODE
         FROM TMT_ORGANIZATIONS T
         WHERE ( T.PARENT_ID = h_id
         OR T.ELEMENT_ID = h_id )
         AND T.REC_STATUS NOT IN ('D', 'L', 'A')
         AND T.TYPE_ID NOT IN (PMO_CNST.SC_RESOURCE, PMO_CNST.SC_ASSETS)
         ORDER BY T.RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ORG', 104+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_ORG(105) : organizations.sql(830) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C66672B;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ORG', 114+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ORG(115) : organizations.sql(840) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ORG', 126+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ORG(127) : organizations.sql(852) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ORG;
ALTER FUNCTION SP_S_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_FS_ORG ==
CREATE OR REPLACE FUNCTION SP_S_FS_ORG
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C6667A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667B PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_FS_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_FS_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_S_FS_ORG(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FS_ORG', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_FS_ORG(25) : organizations.sql(893) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_CHAR(h_default_id, SC_H_ID);
   --SC_CHAR(h_rank, SC_H_FULL_RANK);
   --PmoString h_sql(SC_H_LONG_DESC);
   
   If ((p_id IS NULL)) Then
      OPEN C6667A FOR SELECT
      T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION AS REPOSITORY_LIST,
      T.FS_CODE
      FROM TMT_ORGANIZATIONS T
      WHERE T.PARENT_ID IS NuLL
      AND T.REC_STATUS NOT IN ('D', 'L', 'A')
      AND T.TYPE_ID <> 0
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FS_ORG', 43+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_FS_ORG(44) : organizations.sql(912) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C6667A;
   Else
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_FS_ORG', 49+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_FS_ORG(50) : organizations.sql(918) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      OPEN C6667B FOR SELECT
      T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID,
      CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.ELEMENT_NAME END AS ELEMENT_NAME,
      T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION AS REPOSITORY_LIST,
      T.FS_CODE
      FROM TMT_ORGANIZATIONS T
      WHERE ( T.PARENT_ID = h_id
      OR T.ELEMENT_ID = h_id )
      AND T.REC_STATUS NOT IN ('D', 'L', 'A')
      ORDER BY T.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_FS_ORG', 71+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_FS_ORG(72) : organizations.sql(940) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C6667B;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_FS_ORG', 80+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_FS_ORG(81) : organizations.sql(949) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_FS_ORG', 92+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_FS_ORG(93) : organizations.sql(961) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_FS_ORG;
ALTER FUNCTION SP_S_FS_ORG COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_A_RES_ORG ==
CREATE OR REPLACE FUNCTION FC_A_RES_ORG
(
     h_resource_id_param IN OUT NOCOPY VARCHAR2
   , h_id_param IN OUT NOCOPY VARCHAR2
   , h_rec_user_param IN OUT NOCOPY VARCHAR2
   , h_error_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_exists VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_fs_code VARCHAR2(50);
   h_repository_list VARCHAR2(255);
   h_max_rank PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CRLST IS
         SELECT * FROM (SELECT DESCRIPTION
         FROM TMT_ORGANIZATIONS
         WHERE (DESCRIPTION IS NOT NuLL AND DESCRIPTION <> '')
         AND (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         ORDER BY RANK DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_A_RES_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_A_RES_ORG'), '-- ' || SYSTIMESTAMP, ':r := FC_A_RES_ORG(' || 
       '''' || h_resource_id_param || ''''
          || ', ' || '''' || h_id_param || ''''
          || ', ' || '''' || h_rec_user_param || ''''
          || ', ' || '''' || h_error_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 36+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_A_RES_ORG(37) : organizations.sql(1014) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := h_id_param;
   
   BEGIN
      SELECT RANK INTO h_rank FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(52) : organizations.sql(1029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      --RETURN_ERROR(RECORD_NOT_FOUND, h_id);
      h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
      h_error_id := h_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 61+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_A_RES_ORG(62) : organizations.sql(1039) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   End If;
   
   h_repository_list := NULL;
   -- get repozitory list
   
   OPEN CRLST;
   
   FETCH CRLST INTO h_repository_list;
   IF CRLST%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   BEGIN
      CLOSE CRLST ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 86+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(87) : organizations.sql(1064) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- get resource id and validate fs code
   h_resource_id := h_resource_id_param;
   
   BEGIN
      SELECT FS_CODE INTO h_fs_code FROM RESOURCES WHERE RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 102+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(103) : organizations.sql(1080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      --RETURN_ERROR(RECORD_NOT_FOUND, h_resource_id);
      h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
      h_error_id := h_resource_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 112+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_A_RES_ORG(113) : organizations.sql(1090) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   End If;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_exists FROM TMT_ORGANIZATIONS
      WHERE ELEMENT_ID = h_resource_id_param AND PARENT_ID =h_id_param;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 126+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(127) : organizations.sql(1104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      --RETURN_INVALID(ASSIGNMENT_ALREADY_EXISTS);
      h_temp_sc_state := PMO_CNST.ASSIGNMENT_ALREADY_EXISTS;
      h_error_id := h_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 136+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_A_RES_ORG(137) : organizations.sql(1114) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   End If;
   
   BEGIN
      SELECT PARENT_ID INTO h_exists FROM TMT_ORGANIZATIONS
      WHERE ELEMENT_ID = h_resource_id_param;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 150+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(151) : organizations.sql(1128) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         DELETE FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_resource_id_param;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 168+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(169) : organizations.sql(1146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         BEGIN
            UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID = h_exists;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 187+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(188) : organizations.sql(1165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   PMODEFINES.PMO_GET_MAX_RANK('TMT_ORGANIZATIONS', h_id, h_max_rank);
   
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      INSERT INTO TMT_ORGANIZATIONS(RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, FS_CODE, DESCRIPTION, REC_USER)
      SELECT RTRIM(RANK) || h_rankBase64, h_id, h_resource_id, PMO_CNST.SC_RESOURCE,
      LEVEL_ID + 1, h_fs_code, h_repository_list,h_rec_user_param
      FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_RES_ORG', 213+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_RES_ORG(214) : organizations.sql(1191) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_fs_code IS NOT NULL) Then
      h_id := h_resource_id;
   End If;
   
   h_temp_sc_state := PMO_DB2M.SC_STATE;
   h_error_id := h_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 225+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_A_RES_ORG(226) : organizations.sql(1203) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_temp_sc_state, 0);
   
   --RETURN_STATUS(SC_STATE, h_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_RES_ORG', 232+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_A_RES_ORG(233) : organizations.sql(1210) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_A_RES_ORG;
ALTER FUNCTION FC_A_RES_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_A_RES_ORG ==
CREATE OR REPLACE FUNCTION SP_A_RES_ORG
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_resource_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RES_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RES_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RES_ORG(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_ORG', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RES_ORG(24) : organizations.sql(1244) : ' || SYSTIMESTAMP);
   END IF; 

   h_resource_id := p_resource_id;
   h_id := p_id;
   h_rec_user := p_rec_user;
   
   If (h_id IS NULL) OR (h_resource_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_ORG', 32+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_ORG(33) : organizations.sql(1253) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_ORG', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_ORG(36) : organizations.sql(1256) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
   End If;
   
   h_temp_sc_state := FC_A_RES_ORG(h_resource_id, h_id, h_rec_user, h_error_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RES_ORG', 43+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RES_ORG(44) : organizations.sql(1264) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(Coalesce(h_temp_sc_state, 0), h_error_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_ORG', 50+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_ORG(51) : organizations.sql(1271) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_ORG', 62+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_ORG(63) : organizations.sql(1283) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RES_ORG;
ALTER FUNCTION SP_A_RES_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UA_RES_ORG ==
CREATE OR REPLACE FUNCTION SP_UA_RES_ORG
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UA_RES_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UA_RES_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_UA_RES_ORG(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UA_RES_ORG', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UA_RES_ORG(19) : organizations.sql(1318) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_CHAR(h_id, SC_H_ID);
   
   --InitializeHost(SC_PARAM(h_id), id, 0);
   
   -- Execute Delete Statement
   BEGIN
      DELETE FROM TMT_ORGANIZATIONS WHERE PARENT_ID = p_id
      AND ELEMENT_ID = p_resource_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_ORG', 40+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_ORG(41) : organizations.sql(1340) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      -- Update the parent CHILD_COUNT field
      BEGIN
         UPDATE TMT_ORGANIZATIONS SET    CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_ORG', 59+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_ORG(60) : organizations.sql(1359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UA_RES_ORG', 66+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UA_RES_ORG(67) : organizations.sql(1366) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_ORG', 73+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_ORG(74) : organizations.sql(1373) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_ORG', 85+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_ORG(86) : organizations.sql(1385) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UA_RES_ORG;
ALTER FUNCTION SP_UA_RES_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_ORG ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_ORG
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66673A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66673B PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_ORG(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ORG', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_ORG(26) : organizations.sql(1427) : ' || SYSTIMESTAMP);
   END IF; 

   If ((p_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_ORG', 30+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_ORG(31) : organizations.sql(1432) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ORG', 33+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ORG(34) : organizations.sql(1435) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --if( !(IsNullSpParam(ind_s_res_org, p_bitflag))  (*p_bitflag == 2))
   If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 2) Then
      BEGIN
         SELECT RANK INTO h_temp_rank FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ORG', 48+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ORG(49) : organizations.sql(1450) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C66673A FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_ORGANIZATIONS T1 WHERE (SUBSTR(h_temp_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_temp_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
      AND (RANK LIKE SUBSTR(h_temp_rank, 1, 8) || '%'))
      AND T1.TYPE_ID NOT in (0, 13)
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ORG', 62+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ORG(63) : organizations.sql(1464) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66673A;
   Else
      OPEN C66673B FOR SELECT T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      CASE WHEN T1.PARENT_ID IS NuLL THEN 0 ELSE 1 END AS CHILD_COUNT,
      T1.DELETED_COUNT, T1.REC_STATUS, T1.DESCRIPTION
      FROM TMT_ORGANIZATIONS T1 WHERE T1.ELEMENT_ID IN (SELECT PARENT_ID FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_id/*HOST(h_id)*/)
      AND T1.TYPE_ID < 5000
      AND T1.TYPE_ID <> 0
      AND T1.REC_STATUS in ('I', 'U', 'O')
      ORDER BY T1.RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_ORG', 75+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_ORG(76) : organizations.sql(1477) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66673B;
   End If;
   
   --return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ORG', 85+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ORG(86) : organizations.sql(1487) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_ORG', 97+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_ORG(98) : organizations.sql(1499) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_ORG;
ALTER FUNCTION SP_S_RES_A_ORG COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_U_FS_REPOSITORY ==
CREATE OR REPLACE FUNCTION FC_U_FS_REPOSITORY
(
     h_id IN OUT NOCOPY VARCHAR2
   , h_repository_list IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_c_id VARCHAR2(32);
   h_c_parent_rank VARCHAR2(200);
   h_rank VARCHAR2(200);
   h_old_repository_list VARCHAR2(255);
   h_temp_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_temp_str VARCHAR2(100);
   h_temp_str_len PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_ORG IS
         SELECT ELEMENT_ID, SUBSTR(RANK, 1, (LENGTH(RANK) - 4))
         FROM TMT_ORGANIZATIONS
         WHERE RANK LIKE h_rank || '%'
         AND TYPE_ID = 13;

      CURSOR C_ORG_CHK IS
         SELECT ELEMENT_ID FROM TMT_ORGANIZATIONS
         WHERE  ROWNUM <= 1 AND  RANK > h_rank AND RANK <= h_c_parent_rank
         AND (DESCRIPTION <> '')
         AND TYPE_ID <> 13;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_FS_REPOSITORY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_FS_REPOSITORY'), '-- ' || SYSTIMESTAMP, ':r := FC_U_FS_REPOSITORY(' || 
       '''' || h_id || ''''
          || ', ' || '''' || h_repository_list || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_FS_REPOSITORY', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_FS_REPOSITORY(39) : organizations.sql(1554) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_CHAR(h_id, SC_H_ID);
   --SC_CHAR(h_repository_list, SC_H_DESCRIPTION);
   
   BEGIN
      SELECT  ( LENGTH(h_repository_list) )  INTO  h_temp_str_len FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(53) : organizations.sql(1568) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_str_len, 0) = 0 Then
      h_repository_list := '';
   End If;
   
   BEGIN
      SELECT TYPE_ID, RANK INTO h_type_id, h_rank
      FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 70+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(71) : organizations.sql(1586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --Comment By paria
   --if(( SC_SUCCESS )  ( h_type_id_Ind != -1))
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_type_id IS NOT NULL)) Then
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
         --update resource's repository list
         BEGIN
            UPDATE TMT_ORGANIZATIONS 
            SET DESCRIPTION = h_repository_list
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 93+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(94) : organizations.sql(1609) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PENDING_MONITOR 
            SET SOURCE = 'O'
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(113) : organizations.sql(1628) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PENDING_MONITOR( ELEMENT_ID, SOURCE )
               VALUES( h_id, 'O' );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 131+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(132) : organizations.sql(1647) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT DESCRIPTION INTO h_old_repository_list
            FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 146+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(147) : organizations.sql(1662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --update organization
         BEGIN
            UPDATE TMT_ORGANIZATIONS 
            SET DESCRIPTION = h_repository_list
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 166+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(167) : organizations.sql(1682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_flag := 0;
         
         BEGIN
            SELECT  ( SUBSTR(h_repository_list, 1, InStr(h_repository_list, ';')) )
             INTO  h_temp_str FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 182+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(183) : organizations.sql(1698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT  ( LENGTH(h_temp_str) )  INTO  h_temp_str_len FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(196) : organizations.sql(1711) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT  ( SUBSTR(h_repository_list, h_temp_str_len+1, LENGTH(h_repository_list)) )
             INTO  h_repository_list FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 209+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(210) : organizations.sql(1725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         WHILE Coalesce(h_temp_str_len, 0) > 1 LOOP
            --update organization's resource repository
            
            OPEN C_ORG;
            
            LOOP
               FETCH C_ORG INTO h_c_id, h_c_parent_rank;
               IF C_ORG%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> 0 Then
                  EXIT;
               End If;
               
               --validate start
               
               OPEN C_ORG_CHK;
               
               FETCH C_ORG_CHK INTO h_temp_id;
               IF C_ORG_CHK%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               h_update := PMO_DB2M.SC_STATE;
               
               BEGIN
                  CLOSE C_ORG_CHK ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 251+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(252) : organizations.sql(1767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --validate end
               
               If Coalesce(h_update, 0) = 100 Then
                  --update resource's repository list
                  BEGIN
                     UPDATE TMT_ORGANIZATIONS SET
                     DESCRIPTION = CASE WHEN InStr( DESCRIPTION, h_temp_str) > 0 THEN DESCRIPTION ELSE COALESCE(DESCRIPTION, '') || h_temp_str END
                     WHERE ELEMENT_ID = h_c_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 273+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(274) : organizations.sql(1789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C_ORG ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 288+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(289) : organizations.sql(1804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_flag, 0) = 0 Then
               BEGIN
                  SELECT  ( CASE WHEN InStr(h_old_repository_list, h_temp_str) > 0 THEN 0 ELSE 1 END )
                   INTO  h_flag FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 303+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(304) : organizations.sql(1819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               SELECT  ( SUBSTR(h_repository_list, 1, InStr(h_repository_list, ';')) )  INTO  h_temp_str FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 317+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(318) : organizations.sql(1833) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT  ( LENGTH(h_temp_str) )  INTO  h_temp_str_len FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 330+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(331) : organizations.sql(1846) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT  ( SUBSTR(h_repository_list, h_temp_str_len+1, LENGTH(h_repository_list)) )  INTO  h_repository_list FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 343+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(344) : organizations.sql(1859) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         If Coalesce(h_flag, 0) = 1 Then
            BEGIN
               UPDATE PENDING_MONITOR 
               SET SOURCE = 'O'
               WHERE ELEMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 364+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(365) : organizations.sql(1880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PENDING_MONITOR( ELEMENT_ID, SOURCE )
                  VALUES( h_id, 'O' );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_FS_REPOSITORY', 383+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_FS_REPOSITORY(384) : organizations.sql(1899) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_FS_REPOSITORY', 393+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_FS_REPOSITORY(394) : organizations.sql(1909) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_FS_REPOSITORY', 397+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_FS_REPOSITORY(398) : organizations.sql(1913) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_FS_REPOSITORY;
ALTER FUNCTION FC_U_FS_REPOSITORY COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_FS_REPOSITORY ==
CREATE OR REPLACE FUNCTION SP_U_FS_REPOSITORY
(
     p_id VARCHAR2
   , p_repository_list VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_repository_list VARCHAR2(255);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_FS_REPOSITORY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_FS_REPOSITORY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_FS_REPOSITORY(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_repository_list || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_FS_REPOSITORY', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_FS_REPOSITORY(21) : organizations.sql(1944) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_repository_list := p_repository_list;
   
   PMO_DB2M.SC_STATE := FC_U_FS_REPOSITORY(h_id, h_repository_list);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_FS_REPOSITORY', 29+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_FS_REPOSITORY(30) : organizations.sql(1953) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_REPOSITORY', 36+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_REPOSITORY(37) : organizations.sql(1960) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_REPOSITORY', 48+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_REPOSITORY(49) : organizations.sql(1972) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_FS_REPOSITORY;
ALTER FUNCTION SP_U_FS_REPOSITORY COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_FS_ORG ==
CREATE OR REPLACE FUNCTION SP_U_FS_ORG
(
     p_id VARCHAR2
   , p_fs_code VARCHAR2
   , p_element_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_status VARCHAR2
   , p_flag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_uniqueID VARCHAR2(32);
   h_element_name VARCHAR2(100);
   h_max_rank PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
   h_status VARCHAR2(1);
   h_level_id PLS_INTEGER := 0;
   h_source_rank VARCHAR2(200);
   h_target_rank VARCHAR2(200);
   h_source_id VARCHAR2(32);
   h_source_parent_id VARCHAR2(32);
-- Declare Cursors
      CURSOR CfsI001 IS
         SELECT MAX_RANK + 1
         FROM TMT_ORGANIZATIONS
         WHERE ELEMENT_ID IS NuLL FOR UPDATE OF MAX_RANK, CHILD_COUNT;

      CURSOR CfsI005a IS
         SELECT MAX_RANK + 1, RANK
         FROM TMT_ORGANIZATIONS
         WHERE ELEMENT_ID = h_parent_id FOR UPDATE OF MAX_RANK, CHILD_COUNT;

      CURSOR CfsI005 IS
         SELECT MAX_RANK + 1 FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id FOR UPDATE OF MAX_RANK, CHILD_COUNT;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_FS_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_FS_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_U_FS_ORG(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_fs_code || ''''
          || ', ' || '''' || p_element_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_status || ''''
          || ', ' || COALESCE(TO_CHAR(p_flag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_FS_ORG', 55+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_FS_ORG(56) : organizations.sql(2044) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_LONG(h_level);
   
   h_status := p_status;
   
   --Execute Update
   If (p_parent_id IS NULL) Then
      BEGIN
         SELECT ELEMENT_ID, ELEMENT_NAME, REC_STATUS
         INTO h_uniqueID, h_element_name, h_rec_status
         FROM TMT_ORGANIZATIONS
         WHERE FS_CODE = p_fs_code AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 75+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(76) : organizations.sql(2064) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT ELEMENT_ID, ELEMENT_NAME, REC_STATUS
         INTO h_uniqueID, h_element_name, h_rec_status
         FROM TMT_ORGANIZATIONS
         WHERE FS_CODE = p_fs_code
         AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(93) : organizations.sql(2081) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      If (p_id IS NULL) Then
         --if ((ind_u_fs_org->p_status == 0)    (checkFlagStatus(h_status, 'D')) )
         If ((p_status IS NOT NULL)) AND (SubStr(h_status, 1, 1) = 'D') Then
            -- don't insert closed organization
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_FS_ORG', 104+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_FS_ORG(105) : organizations.sql(2093) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.FS_NOTHING_CHANGED), 'NULL') || ' ID: ' || '''' || p_fs_code || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_FS_ORG', 107+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_FS_ORG(108) : organizations.sql(2096) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.FS_NOTHING_CHANGED, p_fs_code, True);
         End If;
         h_uniqueID := SYS_GUID();
      Else
         h_uniqueID := p_id;
      End If;
      
      If (p_parent_id IS NULL) Then
         --          GET_ROOT_MAX_RANK(CfsI001, TMT_ORGANIZATIONS, h_max_rank);
         
         OPEN CfsI001;
         
         FETCH CfsI001 INTO h_max_rank;
         IF CfsI001%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            UPDATE TMT_ORGANIZATIONS
            SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
            WHERE CURRENT OF CfsI001;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 142+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(143) : organizations.sql(2131) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            CLOSE CfsI001;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 155+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(156) : organizations.sql(2144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         BEGIN
            INSERT INTO TMT_ORGANIZATIONS(FS_CODE, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, REC_STATUS)
            SELECT p_fs_code, RTRIM(COALESCE(RANK, '')) || h_rankBase64,p_parent_id, h_uniqueID, p_element_name, p_type_id, LEVEL_ID + 1, p_rec_user, p_status FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 175+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(176) : organizations.sql(2164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_parent_id := p_parent_id;
         
         BEGIN
            SELECT RANK, LEVEL_ID + 1
            INTO h_rank, h_level_id
            FROM TMT_ORGANIZATIONS
            WHERE ELEMENT_ID = h_parent_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 193+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(194) : organizations.sql(2182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- verify the existence of the current organization under the same country and company
         BEGIN
            SELECT RANK, ELEMENT_ID, PARENT_ID
            INTO h_source_rank, h_source_id, h_source_parent_id
            FROM TMT_ORGANIZATIONS
            WHERE RANK LIKE SUBSTR(h_rank, 1, 8) || '%'
            AND FS_CODE = p_fs_code
            AND LEVEL_ID = h_level_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 212+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(213) : organizations.sql(2201) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            OPEN CfsI005a;
            FETCH CfsI005a INTO h_max_rank, h_target_rank;
            IF CfsI005a%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            BEGIN
               UPDATE TMT_ORGANIZATIONS
               SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
               WHERE CURRENT OF CfsI005a;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 239+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(240) : organizations.sql(2228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            BEGIN
               CLOSE CfsI005a;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 251+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(252) : organizations.sql(2240) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_source_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_source_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_ORGANIZATIONS
               SET RANK = CASE WHEN LEVEL_ID = h_level_id THEN h_target_rank || h_rankBase64
               ELSE h_target_rank || h_rankBase64 || SUBSTR(RANK, (h_level_id * 4 + 9), ((LEVEL_ID - h_level_id) * 4)) END
               WHERE RANK LIKE h_source_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 286+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(287) : organizations.sql(2275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_ORGANIZATIONS
               SET PARENT_ID = h_parent_id,
               ELEMENT_NAME = p_element_name,
               REC_STATUS = p_status,
               REC_USER = p_rec_user,
               REC_DATETIME = SYSTIMESTAMP
               WHERE ELEMENT_ID = h_source_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 309+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(310) : organizations.sql(2298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_ORGANIZATIONS
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID = h_source_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 328+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(329) : organizations.sql(2317) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_FS_ORG', 334+1) THEN 
               LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_FS_ORG(335) : organizations.sql(2323) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_ORGANIZATIONS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
            END IF; 
            BEGIN 
               EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_ORGANIZATIONS' || ' WHERE ELEMENT_ID = :i' 
                  INTO h_rec_status 
                  Using h_parent_id; 
            EXCEPTION 
               WHEN NO_DATA_FOUND THEN 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
            END;
            
            --              GET_MAX_RANK(CfsI005, TMT_ORGANIZATIONS, h_parent_id, h_max_rank);
            
            OPEN CfsI005;
            FETCH CfsI005 INTO h_max_rank;
            IF CfsI005%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            BEGIN
               UPDATE TMT_ORGANIZATIONS SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1 WHERE CURRENT OF CfsI005;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 366+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(367) : organizations.sql(2355) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            BEGIN
               CLOSE CfsI005;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 378+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(379) : organizations.sql(2367) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            BEGIN
               INSERT INTO TMT_ORGANIZATIONS(FS_CODE, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, REC_STATUS)
               SELECT p_fs_code, RTRIM(Rank) || h_rankBase64, p_parent_id, h_uniqueID, p_element_name, p_type_id, LEVEL_ID + 1, p_rec_user, p_status FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = p_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 398+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(399) : organizations.sql(2387) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      BEGIN
         SELECT (CASE WHEN (RTRIM(h_element_name) = RTRIM(p_element_name)
         AND RTRIM(h_rec_status) = RTRIM(p_status)) THEN 0 ELSE 1 END)
          INTO  h_flag FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 415+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(416) : organizations.sql(2404) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_flag, 0) = 1 Then
         --Execute Update
         BEGIN
            UPDATE TMT_ORGANIZATIONS SET
            ELEMENT_NAME = p_element_name,
            REC_STATUS = p_status,
            REC_USER = p_rec_user,
            REC_DATETIME = SYSTIMESTAMP
            WHERE ELEMENT_ID = h_uniqueID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 439+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(440) : organizations.sql(2428) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      -- comment by paria
      --if ((ind_u_fs_org->status == 0)  (h_status[0] == 'D'))
      --if ((ind_u_fs_org->p_status == 0)  (checkFlagStatus(h_status,'D')))
      If ((p_status IS NOT NULL)) AND (SubStr(h_status, 1, 1) = 'D') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_FS_ORG', 449+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_FS_ORG(450) : organizations.sql(2438) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.FS_NOTHING_CHANGED), 'NULL') || ' ID: ' || '''' || p_fs_code || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_FS_ORG', 452+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_FS_ORG(453) : organizations.sql(2441) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.FS_NOTHING_CHANGED, p_fs_code, True);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_FS_ORG', 459+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_FS_ORG(460) : organizations.sql(2448) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 466+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(467) : organizations.sql(2455) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_FS_ORG', 478+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_FS_ORG(479) : organizations.sql(2467) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_FS_ORG;
ALTER FUNCTION SP_U_FS_ORG COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_FS_ORG_SRCH ==
CREATE OR REPLACE FUNCTION SP_FS_ORG_SRCH
(
     p_type_id PLS_INTEGER
   , p_parent_id VARCHAR2
   , p_short_filter_string VARCHAR2
   , p_filter_type PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rank_like VARCHAR2(200);
   h_short_filter_string VARCHAR2(100);
   h_str_level_id VARCHAR2(1);
   h_table_name VARCHAR2(100);
   h_filter_type PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_sql VARCHAR2(2048);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_FS_ORG_SRCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_FS_ORG_SRCH'), '-- ' || SYSTIMESTAMP, ':r := SP_FS_ORG_SRCH(' || 
       COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_short_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(p_filter_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FS_ORG_SRCH', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_FS_ORG_SRCH(30) : organizations.sql(2513) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_LONG(h_type_id);
   
   --InitializeHost(SC_PARAM(h_type_id), *type_id, ind_fs_org_srch->type_id);
   h_filter_type := p_filter_type;
   h_parent_id := p_parent_id;
   h_short_filter_string := p_short_filter_string;
   
   h_table_name := 'TMT_ORGANIZATIONS ';
   If (p_parent_id IS NULL) Then
      BEGIN
         SELECT RANK, LEVEL_ID + 2
         INTO h_rank, h_level_id
         FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID IS NULL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FS_ORG_SRCH', 52+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FS_ORG_SRCH(53) : organizations.sql(2536) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT RANK, LEVEL_ID + 1
         INTO h_rank, h_level_id
         FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FS_ORG_SRCH', 67+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FS_ORG_SRCH(68) : organizations.sql(2551) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_FS_ORG_SRCH', 74+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_FS_ORG_SRCH(75) : organizations.sql(2558) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_parent_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FS_ORG_SRCH', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_FS_ORG_SRCH(78) : organizations.sql(2561) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_parent_id, True);
   End If;
   
   h_rank_like := h_rank;
   h_rank_like := SubStr(h_rank_like || '%%', 1, 200);
   
   If (p_parent_id IS NOT NULL) Then
      h_sql := 
            'SELECT T.LEVEL_ID AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_sql := h_sql || 'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.FS_CODE FROM ';
      h_sql := h_sql || h_table_name;
      --Comment By paria
      --if(ind_fs_org_srch->parent_id == 0)
      If (p_parent_id IS NOT NULL) Then
         h_sql := h_sql || ' T WHERE T.ELEMENT_ID = ''';
         h_sql := h_sql || h_parent_id;
         h_sql := h_sql || ''' UNION ALL ';
      Else
         h_sql := h_sql || ' T WHERE T.ELEMENT_ID IS NULL UNION ALL ';
      End If;
      h_sql := h_sql || 'SELECT Trunc(RTRIM(''';
   Else
      h_sql := 'SELECT Trunc(RTRIM(''';
   End If;
   
   h_str_level_id := TO_CHAR(Coalesce(h_level_id, 0));
   h_sql := h_sql || h_str_level_id;
   h_sql := h_sql || ''')) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
   If Coalesce(h_filter_type, 0) = 1 Then
      --      h_sql.strcat("R.FULL_NAME AS ELEMENT_NAME, ");
      --      h_sql.strcat("0 AS CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.FS_CODE FROM ");
      h_sql := h_sql || 
                  'CASE WHEN T.TYPE_ID = 13 THEN R.FULL_NAME ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, ';
      h_sql := h_sql || 'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.FS_CODE FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' T LEFT JOIN RESOURCES R ON R.RESOURCE_ID = T.ELEMENT_ID ';
      h_sql := h_sql || 'WHERE T.RANK LIKE ''';
      h_sql := h_sql || h_rank_like;
      h_sql := h_sql || ''' AND T.RANK <> ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || ''' ';
      h_sql := h_sql || 'AND UPPER(T.FS_CODE) LIKE UPPER(''';
      h_sql := h_sql || '%%';
      h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
      h_sql := h_sql || h_short_filter_string;
      h_sql := h_sql || ''') ';
   Else
      h_sql := h_sql || 
                  'CASE WHEN T.TYPE_ID = 13 THEN R.FULL_NAME ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, ';
      h_sql := h_sql || 'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ';
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.FS_CODE FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' T LEFT JOIN RESOURCES R ON R.RESOURCE_ID = T.ELEMENT_ID ';
      h_sql := h_sql || 'WHERE T.RANK LIKE ''';
      h_sql := h_sql || h_rank_like;
      h_sql := h_sql || ''' AND T.RANK <> ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || ''' ';
      h_sql := h_sql || 'AND (UPPER(T.ELEMENT_NAME) LIKE UPPER(''';
      h_sql := h_sql || '%%';
      h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
      h_sql := h_sql || h_short_filter_string;
      h_sql := h_sql || ''') ';
      h_sql := h_sql || 'OR UPPER(R.FULL_NAME) LIKE UPPER(''';
      h_sql := h_sql || '%%';
      h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
      h_sql := h_sql || h_short_filter_string;
      h_sql := h_sql || ''')) ';
   End If;
   h_sql := h_sql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
   h_sql := h_sql || 'ORDER BY RANK ';
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FS_ORG_SRCH', 160+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_FS_ORG_SRCH(161) : organizations.sql(2644) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_FS_ORG_SRCH', 163+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_FS_ORG_SRCH(164) : organizations.sql(2647) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FS_ORG_SRCH', 171+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FS_ORG_SRCH(172) : organizations.sql(2655) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_FS_ORG_SRCH', 183+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_FS_ORG_SRCH(184) : organizations.sql(2667) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_FS_ORG_SRCH;
ALTER FUNCTION SP_FS_ORG_SRCH COMPILE DEBUG;

/**************************************************************/

Prompt == FUNCTION FC_M_ORG ==
CREATE OR REPLACE FUNCTION FC_M_ORG
(
     h_target IN OUT NOCOPY VARCHAR2
   , h_source IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , h_error_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_cid VARCHAR2(32);
   h_cr VARCHAR2(200);
   h_r VARCHAR2(200);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_source_repository_list VARCHAR2(255);
   h_repository_list VARCHAR2(255);
   h_source_parent_id VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_target_id VARCHAR2(32);
   h_target_parent_id VARCHAR2(32);
   h_target_rank VARCHAR2(200);
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_source_level_id PLS_INTEGER := 0;
   h_target_level_id PLS_INTEGER := 0;
   h_target_type_id PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_t_repository_list VARCHAR2(255);
   h_t_source_list VARCHAR2(255);
   h_t_source_list1 VARCHAR2(255);
   token1 LONG;
   pos1 LONG;
   lsep VARCHAR2(1) NOT NULL := ';';
   token2 LONG;
   pos2 LONG;
   isFind PLS_INTEGER NOT NULL := 0;
   isEmptyToken PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR C_ORG2 IS
         SELECT ELEMENT_ID, RANK FROM TMT_ORGANIZATIONS
         WHERE RANK >= h_r ORDER BY RANK DESC;

      CURSOR C_REP_LIST IS
         SELECT * FROM (SELECT DESCRIPTION
         FROM TMT_ORGANIZATIONS
         WHERE (DESCRIPTION IS NOT NuLL AND DESCRIPTION <> '')
         AND (SUBSTR(h_target_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_target_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_target_rank, 1, 8) || '%'))
         ORDER BY RANK DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_ORG'), '-- ' || SYSTIMESTAMP, ':r := FC_M_ORG(' || 
       '''' || h_target || ''''
          || ', ' || '''' || h_source || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || '''' || h_error_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 62+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_ORG(63) : organizations.sql(2746) : ' || SYSTIMESTAMP);
   END IF; 

   --Local host variables
   
   --char repository_list[SC_H_DESCRIPTION];
   --char temp_source_list[SC_H_DESCRIPTION];
   --char h_t_source_list1[SC_H_DESCRIPTION];
   
   BEGIN
      SELECT PARENT_ID, RANK, LEVEL_ID
      INTO h_source_parent_id, h_source_rank, h_source_level_id
      FROM TMT_ORGANIZATIONS
      WHERE ELEMENT_ID = h_source;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 82+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(83) : organizations.sql(2766) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      --RETURN_ERROR(RECORD_NOT_FOUND, h_source);
      h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
      h_error_id := h_source;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 92+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(93) : organizations.sql(2776) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   End If;
   
   /*
           (*bitflag == 1): move the source as the previous sibling of the target
           (*bitflag == 0): move the source as the last child of the target
       */
   
   If Coalesce(h_bitflag, 0) = 1 Then
      If (h_target IS NOT NULL) Then
         BEGIN
            SELECT PARENT_ID, RANK INTO h_target_parent_id, h_r
            FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_target;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 113+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(114) : organizations.sql(2797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --RETURN_ERROR(RECORD_NOT_FOUND, h_target);
            h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
            h_error_id := h_target;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 123+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(124) : organizations.sql(2807) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         h_target_id := h_target_parent_id;
         
         BEGIN
            SELECT TYPE_ID, LEVEL_ID, RTRIM(RANK), RTRIM(DESCRIPTION)
            INTO h_target_type_id, h_target_level_id, h_target_rank, h_repository_list
            FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_target_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 140+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(141) : organizations.sql(2824) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --RETURN_ERROR(RECORD_NOT_FOUND, h_target_id);
            h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
            h_error_id := h_target_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 150+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(151) : organizations.sql(2834) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_ORGANIZATIONS', h_target_id, h_max_rank);
         
         --push all siblings
         
         OPEN C_ORG2;
         
         LOOP
            FETCH C_ORG2 INTO h_cid, h_cr;
            IF C_ORG2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            PMO_STPU.incrBase64(h_cr, Coalesce(h_target_level_id, 0) + 2);
            
            BEGIN
               UPDATE TMT_ORGANIZATIONS SET
               RANK = h_cr
               WHERE ELEMENT_ID = h_cid;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 189+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(190) : organizations.sql(2873) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         BEGIN
            CLOSE C_ORG2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 203+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(204) : organizations.sql(2887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_level_id := Coalesce(h_target_level_id, 0) + 1;
         
         h_rank := h_r;
      End If;
      --else
      --{
      --  RETURN_INVALID(INVALID_PARAMS);
      --}
   Else
      If (h_target IS NOT NULL) Then
         h_target_id := h_target;
         
         BEGIN
            SELECT TYPE_ID, LEVEL_ID, RTRIM(RANK), RTRIM(DESCRIPTION)
            INTO h_target_type_id, h_target_level_id, h_target_rank, h_repository_list
            FROM TMT_ORGANIZATIONS
            WHERE ELEMENT_ID = h_target_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 231+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(232) : organizations.sql(2915) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --RETURN_ERROR(RECORD_NOT_FOUND, h_target_id);
            h_temp_sc_state := PMO_CNST.RECORD_NOT_FOUND;
            h_error_id := h_target_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 241+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(242) : organizations.sql(2925) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_ORGANIZATIONS', h_target_id, h_max_rank);
      Else
         h_target_id := '';
         h_target_type_id := 0;
         h_target_level_id := -1;
         h_target_rank := '0000';
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_ORGANIZATIONS', h_max_rank);
      End If;
      
      h_level_id := Coalesce(h_target_level_id, 0) + 1;
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      h_rank := h_target_rank;
      h_rank := SubStr(h_rank || h_rankBase64, 1, 200);
   End If;
   
   --update source hierarchy
   If (h_source_parent_id IS NULL) Then
      BEGIN
         UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 275+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(276) : organizations.sql(2959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_ORGANIZATIONS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID = h_source_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 292+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(293) : organizations.sql(2976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   -- rebuild repository lists for all resources under the under
   -- the new lists have to contain the old list and the new repositories of the new parents
   BEGIN
      SELECT DESCRIPTION INTO h_source_repository_list
      FROM TMT_ORGANIZATIONS
      WHERE ELEMENT_ID = h_source;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 310+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(311) : organizations.sql(2994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- select not empty repository list of target or its closest parent
   
   OPEN C_REP_LIST;
   
   FETCH C_REP_LIST INTO h_repository_list;
   IF C_REP_LIST%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   BEGIN
      CLOSE C_REP_LIST ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 334+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(335) : organizations.sql(3018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- merge repository lists (h_source_repository_list, h_repository_list)
   If ((h_source_repository_list IS NOT NULL)) 
         AND (Coalesce(Length(h_source_repository_list), 0) > 0) Then
      If ((h_repository_list IS NULL)) OR (Coalesce(Length(h_repository_list), 0) = 0) Then
         h_repository_list := h_source_repository_list;
      Else
         -- merge h_source_repository_list and h_repository_list
         --strcpy(h_t_repository_list, h_source_repository_list);
         h_t_repository_list := h_source_repository_list;
         --strcpy(h_t_source_list, h_source_repository_list);
         h_t_source_list := h_source_repository_list;
         --strcpy(h_t_source_list1, h_source_repository_list);
         h_t_source_list1 := h_source_repository_list;
         
         --          char *token1 = h_repository_list;
         --          char *pos1 = h_repository_list;
         --          char lsep[] = ";";
         
         token1 := h_repository_list;
         pos1 := h_repository_list;
         
         token1 := PMO_STPU.REstrtok(token1, lsep, pos1);
         
         --          char *token2 = h_source_repository_list;
         --          char *pos2 = h_source_repository_list;
         --          int isFind = 0;
         
         token2 := h_source_repository_list;
         pos2 := h_source_repository_list;
         isFind := 0;
         
         token2 := PMO_STPU.REstrtok(token2, lsep, pos2);
         
         If PMO_ORASTD.strcmp(token1, token2) = 0 Then
            isFind := 1;
         End If;
         
         isEmptyToken := 0;
         WHILE (isFind = 0) AND (isEmptyToken = 0) LOOP
            token2 := PMO_STPU.REstrtok(NULL, lsep, pos2);
            If (token2 IS NULL) Then
               isEmptyToken := 1;
            Else
               If PMO_ORASTD.strcmp(token1, token2) = 0 Then
                  isFind := 1;
                  EXIT;
               End If;
            End If;
         END LOOP;
         If isFind = 0 Then
            h_t_repository_list := SubStr(h_t_repository_list || token1, 1, 255);
            h_t_repository_list := SubStr(h_t_repository_list || ';', 1, 255);
         End If;
         isFind := 0;
         
         isEmptyToken := 0;
         WHILE isEmptyToken = 0 LOOP
            token1 := PMO_STPU.REstrtok(NULL, lsep, pos1);
            If (token1 IS NULL) Then
               isEmptyToken := 1;
            Else
               isFind := 0;
               token2 := h_t_source_list;
               pos2 := h_t_source_list;
               
               token2 := PMO_STPU.REstrtok(token2, lsep, pos2);
               If PMO_ORASTD.strcmp(token1, token2) = 0 Then
                  isFind := 1;
               End If;
               
               token2 := PMO_STPU.REstrtok(NULL, lsep, pos2);
               WHILE (isFind = 0) AND (NOT (token2 IS NULL)) LOOP
                  If PMO_ORASTD.strcmp(token1, token2) = 0 Then
                     isFind := 1;
                     EXIT;
                  End If;
                  token2 := PMO_STPU.REstrtok(NULL, lsep, pos2);
               END LOOP;
               
               If isFind = 0 Then
                  h_t_repository_list := SubStr(h_t_repository_list || token1, 1, 255);
                  h_t_repository_list := SubStr(h_t_repository_list || ';', 1, 255);
               End If;
            End If;
         END LOOP;
         --strcpy(h_repository_list, repository_list);
         h_repository_list := h_t_repository_list;
      End If;
   End If;
   
   --Update source
   BEGIN
      UPDATE TMT_ORGANIZATIONS
      SET PARENT_ID = h_target_id,
      LEVEL_ID = h_level_id,
      RANK = h_rank,
      REC_USER = h_rec_user,
      REC_STATUS = 'U',
      DESCRIPTION = h_repository_list,
      REC_DATETIME = SYSTIMESTAMP
      WHERE ELEMENT_ID = h_source;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ORG', 450+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ORG(451) : organizations.sql(3134) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := 0;
   h_error_id := h_source;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 458+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(459) : organizations.sql(3142) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_temp_sc_state, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ORG', 463+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_ORG(464) : organizations.sql(3147) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_ORG;
ALTER FUNCTION FC_M_ORG COMPILE DEBUG;

/********************************************************************************************************/

Prompt == FUNCTION SP_M_ORG ==
CREATE OR REPLACE FUNCTION SP_M_ORG
(
     p_target VARCHAR2
   , p_source VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target VARCHAR2(32);
   h_source VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
   h_error_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_ORG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_ORG'), '-- ' || SYSTIMESTAMP, ':r := SP_M_ORG(' || 
       '''' || p_target || ''''
          || ', ' || '''' || p_source || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_ORG', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_ORG(27) : organizations.sql(3184) : ' || SYSTIMESTAMP);
   END IF; 

   h_target := p_target;
   h_source := p_source;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   If ((h_source IS NULL)) OR ((h_bitflag IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_ORG', 36+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_ORG(37) : organizations.sql(3194) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_ORG', 39+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_ORG(40) : organizations.sql(3197) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   ElsIf (Coalesce(h_bitflag, 0) < 0) OR (Coalesce(h_bitflag, 0) > 1) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_ORG', 44+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_ORG(45) : organizations.sql(3202) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_ORG', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_ORG(48) : organizations.sql(3205) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_temp_sc_state := FC_M_ORG(h_target, h_source, h_bitflag, h_rec_user, h_error_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_ORG', 55+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_ORG(56) : organizations.sql(3213) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(Coalesce(h_temp_sc_state, 0), h_error_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_ORG', 63+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_ORG(64) : organizations.sql(3221) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_ORG', 75+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_ORG(76) : organizations.sql(3233) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_ORG;
ALTER FUNCTION SP_M_ORG COMPILE DEBUG;

