/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * Author: Tudor Toma
 * Revision: 42
 * Date: 7/31/00 6:18p
 *
 **/

/*
#Stored Procedure  :SP_SEARCH_2
#Short Description :Search and Assign a resource or profile to a element.
#Description       :Search or Search and Assign a resource or profile with effort to a WBS or CRI element.
				   :If search/assign a participant (without effort) all effort related parameters are not required.
				   :Search criteria is a sum of input parameters, witch tells how to filter out the result set.
				   :This function gets called from RPM client, Import and Level/Calculate.
RULE1: cannot assign more then once a resource to a element.
RULE2: cannot assign a resource to a project if the resource is already assigned to a task whitin the project.
RULE3: cannot assign to a task from project reserve more effort then remaining effort at the project.
RULE4: if bitflag = 9 then assignment without effort, otherwise assignment with effort.
RULE5: if bitflag between 1 and 8 the assignment is contoured: contour > 0 and work > 0
RULE6: if search by name then partially ignore the search criteria: org_id, geo_id, pool_id, work_loc, and_roles, and_skills, shw_roles, shw_skills
RULE7: if effort = 0 and bitflag < 9 then both start  finish dates are required
RULE8: if assign units <> 100%, then bitflag < 9 and effort > 0 and both start  finish dates are required
RULE9: cannot assign a resource if is not active, effective date is in the future or discontinue date is in the past
#Table Name        :
#Parameters        :
 $$bitflag  = 0 - ASP ---------------------------------------------------------------------------------------------
            = 1 - Daily         -- contour3
            = 2 - Weekly        -- contour3{, contur1}, {amount falg}
            = 3 - Monthly(C)    -- contour3{, contur2}
            = 4 - Monthly(B)    -- contour3{, contur2}
            = 5 - Monthly(A)    -- contour3{, contur2 | contour2  countour1}
            = 6 - Yearly(C)     -- contour2, contur4
            = 7 - Yearly(B)     -- contour2, contur4, contur1(only one day)
            = 8 - Yearly(A)     -- contour2, contur4, contur1(many days)
            = 9 - Participant

 $$contour (contour structure) ------------------------------------------------------------------------------------
  contur1:  DAY:    bits 0-6    0 = Su, 1 = Mo, 2 = Tu, 3 = We, 4 = Th, 5 = Fr, 6 = Sa,
  contur2:  WEEK:   bits 7-11   A) 7 = week1, 8 = week2, 8 = week3, 9 = week4
                                B) 1-First/2-Second/3-Third/4-Fourth/5-Last
                                C) The Day 1 to 31
  contour3:     EVRY:   bits 12-16  EVRY 1 to 31 day / week / month
  contour4:     MONTH:  bits 17-28  17 = Jan, 18 = Feb, ... 28 = Dec

  2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
  8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
                                                -------------
                                                S F T W T M S
                                      ---------
                                      w1 to w4  A(7 days from the 1st)
                                      5 4 3 2 1 B(Last, Fourth, Third, Second, First)
                                      31 .... 1 C(The Day 6)
                            ----------
                            31 ........ 1 (Day)
                            31 ........ 1 (week)
                            12 ........ 1 (month)
  -------------------------
  D N O S A J J M A M F J (Month)

  Exemple:
  2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 
  9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 
    ----------------------- --------- --4-3-2-1 ------------- A) 7=week1, 8=week2, 9=week3, 10=week4
                                      5-4-3-2-1               B) 1-First/2-Second/3-Third/4-Fourth/5-Last
  X D N O S A J J M A M F J --EVERY-- -THE-DAY- S F T W T M S C) The Day 1 to 31
  -------------------------contour-------------------------------------bitflag-----------------------------------------
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 = 4096      1 (every day) 	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 = 8192      1 (every 2nd day) 
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 = 12288     1 (every 3rd day) 
  1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 = 536875014 2 (Monday, Tuesday, every week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 = 4102      2 (Monday, Tuesday, every week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 1 0 = 8198      2 (Monday, Monday, every 2nd week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 = 4864      3 (6th day of every month)
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 1 0 = 8322      4 (1st Monday of every 2nd month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 = 4224      5 (week1 of every month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 = 4480      5 (week1, week2 of every month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 1 0 0 0 0 0 0 0 = 8576      5 (week1, week2 of every 2nd month)
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 1 1 0 = 8454      5 (Monday, Tuesday, week2 every 2nd month)	
  1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 1 1 0 = 536879366 5 (Monday, Tuesday, week2 every 2nd month)	
  0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 = 301990656 6 (6th day of month 9, 12)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 = 301990146 7 (1st Monday of month 1, 2)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 1 0 0 = 4356      7 (2nd Tuesday, every month)	
  0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 1 1 0 = 301990534 8 (Monday, Tuesday; week1, week3 of month 9, 12)	
  1 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 1 1 0 = 838861446 8 (Monday, Tuesday; week1, week3 of month 9, 12)	


 $$consider_planned_sched ------------------------------------------------------------------------------------------
  CONSIDER                                              inner current project   outer current project   all projects
  ------------------------------------------------------------------------------------------------------------------
  scheduled                                                         1                   2                   3
  ------------------------------------------------------------------------------------------------------------------
  scheduled  planned                                               4                   5                   6
  ------------------------------------------------------------------------------------------------------------------
  NONE(consider all available time)                                 7
  ------------------------------------------------------------------------------------------------------------------
  scheduled outer current project				                                        13                  14
  ------------------------------------------------------------------------------------------------------------------
  scheduled  planned outer current project				                                15                  16
  ------------------------------------------------------------------------------------------------------------------
  scheduled outer current project  planned all projects            20
  ------------------------------------------------------------------------------------------------------------------
  scheduled outer current project  planned inner current project   21
  ------------------------------------------------------------------------------------------------------------------
  scheduled all projects    planned outer current project          22
  ------------------------------------------------------------------------------------------------------------------
  planned inner current project                                     24
  ------------------------------------------------------------------------------------------------------------------
  search from project assignments		= 10  pool_id   = resource_id
  search create new profile				= 12  and_roles = the first role_id is used to create a new profile
  duplicate profile						= 12  pool_id   = profile_id
  ------------------------------------------------------------------------------------------------------------------
  Note: bits 30 - 31 (in addition to above values): Consider All Scop Element Exceptions = 1 consider, 0 don't consider

  $$eunit, applies to effort
		= 0 MINUTE (default), 
		= 1 h, 
		= 2 d, 
		= 3 w

  $$unit, applies to work
		= 0 h/d,
		= 3 h/w, 
		= 4 h/m, 
		= 1 d/w, 
		= 2 d/m, 
		= 5 w/m
		= 100 then work = percent units

#Returns           :Result Set of resource(s), Best Start date, Best Finish date and daly contour (5*480,2*0,3*400,...) 
				   :For search the number of rows depends on search criteria and is limited by max_rows parameter.
				   :For search/assign is always only one row, the resource whith has been assigned
#Errors            :The errors are returned only in search/assign
				   : TASK_ASSIGNMENT_ERROR (200062) - many reasons the assignment failed.
				   : CHILD_TASK_ASSIGNMENT_ALREADY_EXISTS (20083) - violation of RULE2
				   : CANNOT_ASSIGN_FROM_PROJECT_AFFECTED (20085) - violation of RULE3
				   : RESOURCE_NOT_ACTIVE (200019) - violation of RULE9
				   : F_DATE_EARLIER_THAN_S_DATE - finish date is earlier then start date
				   : 200220 - violation of RULE7
*/

Prompt == FUNCTION SP_SEARCH_2 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_2
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , fname VARCHAR2
   , cost001 VARCHAR2
   , org_id001 VARCHAR2
   , geo_id001 VARCHAR2
   , pool_id001 VARCHAR2
   , work_location001 PLS_INTEGER
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , shw_roles VARCHAR2
   , shw_skills VARCHAR2
   , max_rows PLS_INTEGER
   , p_update PLS_INTEGER
   , work_type PLS_INTEGER
   , p_type PLS_INTEGER
   , last_id VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_wbs_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rs_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_fname VARCHAR2(255);
   h_cost VARCHAR2(10);
   h_and_roles VARCHAR2(8000);
   h_and_skills VARCHAR2(8000);
   h_shw_roles VARCHAR2(8000);
   h_shw_skills VARCHAR2(8000);
   h_and_role VARCHAR2(8000);
   h_and_skill VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_source_ids VARCHAR2(8000);
   h_target_ids VARCHAR2(8000);
   h_res_id VARCHAR2(32);
   h_last_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_dayofyear DATE;
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_res_type VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_skill_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_rec_status VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_work_location PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_max_rows PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_res_cost NUMBER := 0.00;
   h_rate_echange NUMBER := 0.00;
   h_total PLS_INTEGER := 0;
   h_pool_type PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_active VARCHAR2(1);
   h_task_type PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
   h_temp_stmt VARCHAR2(7999);
   h_table_type PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_profile_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_char VARCHAR2(1);
   h_temp_var PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_row PLS_INTEGER NOT NULL := 0;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   token VARCHAR2(32);
   i_noskill PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
   bCreateProfile PLS_INTEGER;
   len PLS_INTEGER;
   k PLS_INTEGER;
   nLen PLS_INTEGER;
   nLen001 PLS_INTEGER;
   nLen002 PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C12 PMO_DB2M.REF_CURSOR;

      CURSOR C42 IS
         SELECT PROJECT_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_project_id
         AND RESOURCE_ID = h_res_id;

      CURSOR CU42 IS
         SELECT
         RS_ID
         , RES_ID
         , RTRIM(ROLE_ID)
         , COALESCE(ROLE_NAME , '')
         , RTRIM(SKILL_ID)
         , COALESCE(SKILL_NAME , '')
         , COALESCE(SHOW_ROLE , '')
         , COALESCE(SHOW_SKILL , '')
         FROM PMO_TMP_SEARCH
         ORDER BY RES_ID;

      -- Resultset Cursor Declaration
      C775C0 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C775E1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C775E0 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_2'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_2(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || '''' || fname || ''''
          || ', ' || '''' || cost001 || ''''
          || ', ' || '''' || org_id001 || ''''
          || ', ' || '''' || geo_id001 || ''''
          || ', ' || '''' || pool_id001 || ''''
          || ', ' || COALESCE(TO_CHAR(work_location001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || shw_roles || ''''
          || ', ' || '''' || shw_skills || ''''
          || ', ' || COALESCE(TO_CHAR(max_rows), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || last_id || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 196+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_2(197) : cplan.sql(343) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --to contain work location passed to function when null then 265
   
   
   -- Parse parameters to local variables
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_fname := fname;
   h_cost := cost001;
   h_and_roles := and_roles;
   h_and_skills := and_skills;
   h_shw_roles := shw_roles;
   h_shw_skills := shw_skills;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_max_rows := max_rows;
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := p_type;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_update := p_update;
   h_work_type := work_type;
   
   h_task_type := 0;
   h_sched_work := 0;
   --local variables
   /*int iii = 0;*/
   --iSF == 0 when finish_date is null
   
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((prj_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 239+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(240) : cplan.sql(386) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 242+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(243) : cplan.sql(389) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((fname IS NULL)) AND ((org_id001 IS NULL)) AND ((geo_id001 IS NULL)) AND ((pool_id001 IS NULL)) 
         AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
      If ((and_roles IS NULL)) AND (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 251+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(252) : cplan.sql(398) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 254+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(255) : cplan.sql(401) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf ((and_roles IS NULL)) AND ((and_skills IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 259+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(260) : cplan.sql(406) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 262+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(263) : cplan.sql(409) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 270+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(271) : cplan.sql(417) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 273+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(274) : cplan.sql(420) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 280+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(281) : cplan.sql(427) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 283+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(284) : cplan.sql(430) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If Coalesce(h_bitflag, 0) = 5 AND Coalesce(h_unit, 0) = 5 AND Coalesce(h_work, 0) = 5 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 290+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(291) : cplan.sql(437) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 293+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(294) : cplan.sql(440) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --check if work_location is null then 265 otherwise put value passed and use to update resource_assignmnets table
   If ((work_location001 IS NULL)) Then
      h_work_location := 265;
   Else
      h_work_location := work_location001;
   End If;
   
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((start_date001 IS NOT NULL)) 
             AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 317+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(318) : cplan.sql(464) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 324+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(325) : cplan.sql(471) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 327+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(328) : cplan.sql(474) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      iSF := 1;
   End If;
   
   If ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 343+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(344) : cplan.sql(490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   
   -- CONFIRM: what is the pool type
   --h_pool_type = (h_type >> 30)  0x0003; // pool type
   h_pool_type := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_type, 0), PMO_CNST.SC_C0000000)), 30);
   -- pool type
   h_type := BITAND(Coalesce(h_type, 0), PMO_CNST.SC_1FFFFFFF);
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   
   h_res_type := CASE WHEN Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE THEN 1 ELSE 0 END;
   
   If Coalesce(h_type, 0) > 0 Then
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 375+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(376) : cplan.sql(522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 391+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(392) : cplan.sql(538) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 398+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(399) : cplan.sql(545) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 401+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(402) : cplan.sql(548) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
      
      h_table_type := GetTable(Coalesce(h_type, 0));
      If Coalesce(h_table_type, 0) = 0 Then
         h_type := GetAssnType(Coalesce(h_type, 0));
         If Coalesce(h_type, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 412+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(413) : cplan.sql(559) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 415+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(416) : cplan.sql(562) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         h_table_type := PMO_CNST.SC_TABLE_WBS;
      End If;
      h_task_id := h_project_id;
      
      If Coalesce(h_pool_type, 0) <> 0 Then
         h_res_id := pool_id001;
      Else
         -- TOVALIDATE: no one tries to search/assign by name
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 428+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(429) : cplan.sql(575) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 431+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(432) : cplan.sql(578) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      --*****************************************************************************
      --
      --        Search Assign if h_type != 0
      --
      --****X******************X************************X************************X***
      h_temp_sc_state := AssignResource(h_table_type, h_type, h_task_id, h_res_id, h_rec_user
                                  , h_assign_id);
      
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 446+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(447) : cplan.sql(593) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 449+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(450) : cplan.sql(596) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_sc_state, 0), h_assign_id, True);
      End If;
      If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 455+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(456) : cplan.sql(602) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 458+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(459) : cplan.sql(605) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_assign_id, True);
      End If;
      
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
         If isProjectType(Coalesce(h_type, 0)) <> 0 Then
            BEGIN
               SELECT
               PROJECT_ID
               , COALESCE(MINUTE_PER_DAY, 480)
               , COALESCE(MINUTE_PER_WEEK, 2400)
               , PRIORITY
               INTO
               h_project_id
               , h_per_day
               , h_per_week
               , h_priority
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_task_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 484+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(485) : cplan.sql(631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT
               B.TASK_TYPE
               , B.UNRESOLVED_START
               , B.UNRESOLVED_FINISH
               , B.PROJECT_ID
               , COALESCE(A.MINUTE_PER_DAY, 480)
               , COALESCE(A.MINUTE_PER_WEEK, 2400)
               , A.PRIORITY
               , B.PLANNED_EFFORT + B.SCHED_EFFORT
               INTO
               h_task_type
               , h_task_start_DT
               , h_task_finish_DT
               , h_project_id
               , h_per_day
               , h_per_week
               , h_priority
               , h_sched_work
               FROM TMT_WBS A, TMT_WBS B
               WHERE B.ELEMENT_ID = h_task_id
               AND A.ELEMENT_ID = B.PROJECT_ID;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 517+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(518) : cplan.sql(664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 525+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(526) : cplan.sql(672) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 528+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(529) : cplan.sql(675) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
      ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
         BEGIN
            SELECT
            PROJECT_ID
            , TYPE_ID
            , WBS_ID
            INTO
            h_project_id
            , h_type
            , h_wbs_id
            FROM TMT_CRI
            WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 550+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(551) : cplan.sql(697) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 557+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(558) : cplan.sql(704) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 560+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(561) : cplan.sql(707) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
         
         -- RULE: CRI Start date cannot be before the rolled up start date
         -- BUG: if search start was between project start and start_DT then cri start would not be adjusted
         BEGIN
            SELECT
            CASE
            WHEN START_DT > h_start_DT
            THEN START_DT
            ELSE TO_TIMESTAMP(h_start_DT)
            END
            , COALESCE(MINUTE_PER_DAY, 480)
            , COALESCE(MINUTE_PER_WEEK, 2400)
            , PRIORITY
            INTO
            h_start_DT
            , h_per_day
            , h_per_week
            , h_priority
            FROM TMT_WBS
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 591+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(592) : cplan.sql(738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 598+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(599) : cplan.sql(745) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 601+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(602) : cplan.sql(748) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_project_id, True);
         End If;
         
         If (h_wbs_id IS NOT NULL) Then
            -- RULE: all effort assignments done on Original CRI that has been transferred
            -- will be done on the transferred counterpart instead.
            -- RULE: WBS CRI has the same ID as the original CRI element
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            h_type := FC_MIRROR_TYPE(h_type);
         End If;
      Else
         -- Table type for effort must be for elements in WBS or CRI tables
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 616+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(617) : cplan.sql(763) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 619+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(620) : cplan.sql(766) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
      End If;
      
      -- search/assign project and other types
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         -- h_consider != SC_SRCH_PROFILE
         BEGIN
            SELECT
            NICK_NAME
            , PMO_ORASTD.JULIAN_DAY(h_start_DT)
            , ACTIVE
            , COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE) ,0)
            , COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE) ,0)
            INTO
            h_full_name
            , h_julian_day
            , h_active
            , h_j_efective_date
            , h_j_disconti_date
            FROM RESOURCES
            WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 648+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(649) : cplan.sql(795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 655+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(656) : cplan.sql(802) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 658+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(659) : cplan.sql(805) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If SubStr(h_active, 1, 1) = '0' OR (Coalesce(h_j_disconti_date, 0) > 0 AND Coalesce(
                  h_julian_day, 0) > Coalesce(h_j_disconti_date, 0)) OR Coalesce(
                  h_julian_day, 0) < Coalesce(h_j_efective_date, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 666+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(667) : cplan.sql(813) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 669+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(670) : cplan.sql(816) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
            End If;
         End If;
      End If;
   End If;
   
   -- BUG: if two people do search for profile on the same project the later search will
   -- delete the profiles from the first search.
   If Coalesce(h_type, 0) = 0 Then
      --*****************************************************************************
      --
      --    Search Only
      --
      --****X******************X************************X************************X***
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE AND (and_roles IS NULL) AND Coalesce(
            h_pool_type, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 688+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(689) : cplan.sql(835) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_COMPETENCY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 691+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(692) : cplan.sql(838) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_COMPETENCY, h_project_id
               , True);
      End If;
      
      -- Resource or Profile
      If Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT Then
         BEGIN
            SELECT
            PRIORITY
            , COALESCE(MINUTE_PER_DAY, 480)
            , COALESCE(MINUTE_PER_WEEK, 2400)
            INTO
            h_priority
            , h_per_day
            , h_per_week
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 716+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(717) : cplan.sql(863) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- BUGCHECK: can pool_id be passed in the project_id parameter and what about h_project_id
         -- which is a pool_id instead
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT
               480
               , 2400
               INTO
               h_per_day
               , h_per_week
               FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 738+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(739) : cplan.sql(885) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 745+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(746) : cplan.sql(892) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 748+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(749) : cplan.sql(895) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
            End If;
         End If;
      End If;
   End If;
   
   -- parameter clean-up, just in case
   If Coalesce(contour001, 0) = 0 AND Coalesce(h_unit, 0) <> 100 Then
      h_work := 0;
   End If;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      -- Dates are not used for participants so we initialize the date if one is not provided
      If (start_date001 IS NULL) Then
         h_start_DT := '1899-12-31 00:00:00';
         h_finish_DT := '1899-12-31 00:00:00';
      Else
         h_start_DT := start_date001;
         h_finish_DT := start_date001;
      End If;
      
      h_effort := 0;
   Else
      -- not participant
      If iSF = 0 Then
         -- We only have the start date
         h_finish_DT := '';
         --InitializeHost(SC_PARAM(h_duration), 120);
         h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
         
         If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 785+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(786) : cplan.sql(932) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 788+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(789) : cplan.sql(935) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         ElsIf Coalesce(h_temp_var, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 793+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(794) : cplan.sql(940) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 796+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(797) : cplan.sql(943) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
         End If;
      Else
         h_finish_DT := finish_date001;
      End If;
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
         If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
            h_eunit := 2;
         ElsIf Coalesce(h_unit, 0) = 0 Then
            h_eunit := 1;
         ElsIf (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
            h_eunit := 1;
         ElsIf Coalesce(h_unit, 0) = 5 Then
            h_eunit := 3;
         End If;
      End If;
   End If;
   
   --*****************************************************************************
   --
   --    Consolidated Profile Handling for both search and search/assign
   --
   --****X******************X************************X************************X***
   If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
      bCreateProfile := 0;
      
      h_and_roles := and_roles;
      h_and_skills := and_skills;
      
      -- Get the first role_id
      h_role_id := SubStr(h_and_roles, 1, 32);
      h_rec_status := NULL;
      
      -- RULE: assigning the same profile to the same element (task) create a duplicate
      -- profile with the same role
      
      -- RULE: assigning a non existent profile to an element (task) creates a
      -- new profile with the first role from the role filter.
      If Coalesce(h_pool_type, 0) = 0 Then
         h_res_id := SYS_GUID();
         bCreateProfile := 1;
      Else
         -- create profile
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM PROFILES
            WHERE PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 851+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(852) : cplan.sql(998) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If Coalesce(h_type, 0) > 0 Then
               BEGIN
                  SELECT
                  RES_ASSIGNMENT_ID
                  INTO
                  h_uniqueID
                  FROM RESOURCE_ASSIGNMENTS
                  WHERE ELEMENT_ID = h_task_id
                  AND RESOURCE_ID = h_res_id
                  AND RESOURCE_TYPE = '1'
                  AND PARTICIPANT = PMO_CNST.SC_TEAMMEMBER;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 874+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(875) : cplan.sql(1021) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  bCreateProfile := 1;
               End If;
            End If;
         Else
            bCreateProfile := 1;
         End If;
      End If;
      
      If bCreateProfile <> 0 Then
         BEGIN
            SELECT
            COALESCE(MAX(PROF_NUMBER),0)+1
            INTO
            h_number
            FROM PROFILES
            WHERE PROJECT_ID=h_project_id
            AND ROLE_ID=h_role_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 903+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(904) : cplan.sql(1050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT
            PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME)
            , ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')')
            INTO h_full_name
            FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
            WHERE TC.ELEMENT_ID = h_role_id
            AND TCP.ELEMENT_ID = TC.PARENT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 922+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(923) : cplan.sql(1069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 929+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(930) : cplan.sql(1076) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 932+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(933) : cplan.sql(1079) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_role_id, True);
         End If;
         
         h_char := CASE WHEN Coalesce(h_type, 0) > 0 THEN 'U' ELSE 'D' END;
         
         BEGIN
            INSERT INTO PROFILES
            (
            PROFILE_ID
            , PROJECT_ID
            , ROLE_ID
            , PROF_NUMBER
            , PROFILE_NAME
            , REC_STATUS
            , POOL_ID
            , GEO_ID
            , ORG_ID
            )
            VALUES
            (
            h_res_id
            , h_project_id
            , h_role_id
            , h_number
            , h_full_name
            , h_char
            , pool_id001
            , geo_id001
            , org_id001
            );
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 974+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(975) : cplan.sql(1121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf SubStr(h_rec_status, 1, 1) = 'D' Then
         -- this must be search/assign because otherwise the 'D' profiles
         -- have been deleted.
         BEGIN
            UPDATE PROFILES SET REC_STATUS = 'U',
            POOL_ID = pool_id001,
            GEO_ID  = geo_id001,
            ORG_ID  = org_id001
            WHERE PROFILE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 997+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(998) : cplan.sql(1144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- Clean-up deleted profiles
      -- TOREMOVE: this mechanism is not really needed anymore, profiles are created on the fly
      BEGIN
         DELETE FROM RESOURCE_ASSIGN_SKILLS
         WHERE RES_ASSIGNMENT_ID IN (
         SELECT PROFILE_ID
         FROM PROFILES
         WHERE PROJECT_ID = h_project_id
         AND REC_STATUS='D'
         AND PROFILE_ID <> h_res_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1023+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1024) : cplan.sql(1170) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_ASSIGN_ROLES
         WHERE RES_ASSIGNMENT_ID IN (
         SELECT PROFILE_ID
         FROM PROFILES
         WHERE PROJECT_ID = h_project_id
         AND REC_STATUS='D'
         AND PROFILE_ID <> h_res_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1046+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1047) : cplan.sql(1193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PROFILES
         WHERE PROJECT_ID = h_project_id
         AND REC_STATUS='D'
         AND PROFILE_ID <> h_res_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1066+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1067) : cplan.sql(1213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --else
   --{
   --    // Clean-up deleted profiles
   --    // TOREMOVE: this mechanism is not really needed anymore, profiles are created on the fly
   --    BEGIN_SQL
   --        DELETE FROM RESOURCE_ASSIGN_SKILLS
   --        WHERE RES_ASSIGNMENT_ID IN (
   --                SELECT PROFILE_ID
   --                FROM PROFILES
   --                WHERE PROJECT_ID = HOST(h_project_id)
   --                        AND REC_STATUS='D');
   --    END_SQL;
   
   --    BEGIN_SQL
   --        DELETE FROM RESOURCE_ASSIGN_ROLES
   --        WHERE RES_ASSIGNMENT_ID IN (
   --                SELECT PROFILE_ID
   --                FROM PROFILES
   --                WHERE PROJECT_ID = HOST(h_project_id)
   --                        AND REC_STATUS='D');
   --    END_SQL;
   
   --    BEGIN_SQL
   --        DELETE FROM PROFILES
   --        WHERE PROJECT_ID = HOST(h_project_id)
   --                AND REC_STATUS='D';
   --    END_SQL;
   --}
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   If Coalesce(h_type, 0) = 0 AND Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE AND Coalesce(
            h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
      If (last_id IS NOT NULL) Then
         h_last_id := last_id;
      End If;
   End If;
   
   If Coalesce(h_max_rows, 0) = 0 Then
      h_max_rows := PMO_CNST.SC_MAX_ROWS;
   End If;
   
   If (and_roles IS NULL) AND (and_skills IS NULL) AND (shw_roles IS NULL) 
         AND ((shw_skills IS NULL)) Then
      i_noskill := 1;
   End If;
   
   --trace_printf("h_consider: %d, h_type: %d, h_res_id: %s\n", h_consider,h_type,h_res_id);
   stmt := '';
   stwhere := '';
   If Coalesce(h_type, 0) > 0 Then
      -- TODO: this can be removed if search assign is handled at the top.
      --*****************************************************************************
      --
      --    Build Dynamic String for Search/Assign
      --
      --****X******************X************************X************************X***
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         stmt := 'SELECT ''' || h_res_id || ''' AS RESOURCE_ID '
                || 'FROM TMT_COMPETENCIES r '
                || 'WHERE r.TYPE_ID=73 '
                || '      AND r.ELEMENT_ID = ''' || h_role_id || '''';
      ElsIf Coalesce(h_consider, 0) = 10 Then
         stmt := 'SELECT ra.RESOURCE_ID '
                || 'FROM RESOURCE_ASSIGNMENTS ra '
                || 'JOIN TASK_ASSIGNMENTS ta ON ra.ELEMENT_ID=''' || h_project_id || ''' '
                || 'AND ra.RESOURCE_ID=''' || h_res_id || ''' '
                || 'AND ra.RES_ASSIGNMENT_ID=ta.RES_ASSIGNMENT_ID ';
      Else
         stmt := 'SELECT r.RESOURCE_ID '
                || 'FROM RESOURCES r '
                || 'WHERE r.CALENDAR_ID IS NOT NuLL '
                || '        AND r.RESOURCE_ID = ''' || h_res_id || ''' ';
      End If;
   Else
      --*****************************************************************************
      --
      --    Build Dynamic String for Search
      --
      --****X******************X************************X************************X***
      --removed the initailaization of h_total to work_location outsid condiitons
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE AND Coalesce(
            h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT AND Coalesce(
            bitflag001, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT Then
         -- Searching for Effort of Resource and not from Project
         -- Filter by: Pool ID, Geo ID, Name, Location ID, etc., not effort based
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0007';
         PMO_DB2M.SC_STATE := 0;
         
         If (pool_id001 IS NULL) Then
            -- Pool ID is NULL, use a list of all pools accessible by rec_user
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_007a';
            PMO_DB2M.SC_STATE := 0;
            
            BEGIN
               INSERT INTO PMO_TMP_CPLN_007a
               SELECT ELEMENT_ID, RANK
               FROM TMT_POOLS
               WHERE PUBLIC_FLAG = '1'
               UNION ALL
               ---- All pools to which the resource is assigned
               SELECT P.ELEMENT_ID, P.RANK
               FROM TMT_POOLS P
               JOIN RESOURCE_ASSIGNMENTS RA ON
               P.ELEMENT_ID = RA.ELEMENT_ID
               AND RA.ELEMENT_TYPE = 11
               AND RA.RESOURCE_ID = h_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1191+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1192) : cplan.sql(1338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- REDUNDANT: table B was same as M
            /*				BEGIN_SQL
                                INSERT INTO session.PMO_TMP_CPLN_0007
                                SELECT M.RANK
                                FROM session.PMO_TMP_CPLN_007a M
                                WHERE NOT EXISTS (
                                    SELECT B.ELEMENT_ID
                                    FROM session.PMO_TMP_CPLN_007a A, session.PMO_TMP_CPLN_007a B
                                    WHERE A.RANK < B.RANK
                                        AND SUBSTR(B.RANK, 1, LENGTH(A.RANK)) = A.RANK
                                        AND M.ELEMENT_ID = B.ELEMENT_ID);
            				END_SQL;*/
            -- NOTE: at this point it is possible to have a pool twice, once for public, and
            -- once for assigned to pool
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0007
               SELECT M.RANK
               FROM PMO_TMP_CPLN_007a M
               WHERE NOT EXISTS (
               SELECT A.ELEMENT_ID
               FROM PMO_TMP_CPLN_007a A
               WHERE A.RANK < M.RANK
               AND SUBSTR(M.RANK, 1, LENGTH(A.RANK)) = A.RANK);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1229+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1230) : cplan.sql(1376) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            -- Pool is given use the rank.
            h_res_id := pool_id001;
            
            -- Get Rank for the resource from pools
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0007
               SELECT RANK FROM TMT_POOLS
               WHERE ELEMENT_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1252+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1253) : cplan.sql(1399) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (fname IS NOT NULL) Then
         -- Name filter is given
         len := CASE WHEN 
                 Coalesce(Length(fname), 0) > 32767 THEN BITAND(Coalesce(Length(fname), 0), 65535) - 65536 ELSE BITAND(Coalesce(Length(fname), 0), 65535) END;
         k := 0;
         
         -- Make sure the name filter is not all wildcards
         WHILE k < len LOOP
            If Coalesce(Ascii(SubStr(h_fname, k + 1, 1)), 0) <> 37 
                  AND Coalesce(Ascii(SubStr(h_fname, k + 1, 1)), 0) <> 95 Then
               EXIT;
            End If;
            k := k + 1;
         END LOOP;
         
         If k = len Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 1276+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(1277) : cplan.sql(1423) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1279+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1280) : cplan.sql(1426) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         BEGIN
            SELECT REPLACE(h_fname,'''','''''')
            INTO h_fname
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1294+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1295) : cplan.sql(1441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         -- RULE: name filtering is ignored on SC_SRCH_PROFILE
         stmt := 
               'SELECT p.PROFILE_ID AS RESOURCE_ID FROM PROFILES p JOIN TMT_COMPETENCIES r ON p.ROLE_ID = r.ELEMENT_ID';
         stwhere := ' WHERE p.PROFILE_ID=''';
         stwhere := stwhere || h_res_id;
         stwhere := stwhere || '''';
      ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
         -- RULE: name filtering is ignored on SC_SRCH_FROM_PROJECT
         stmt := 
               'SELECT r.RESOURCE_ID FROM RESOURCES r JOIN RESOURCE_ASSIGNMENTS ra ON BITAND(ra.PARTICIPANT,4096)>0 AND ra.ELEMENT_ID=''';
         stmt := stmt || h_project_id;
         stmt := stmt || ''' AND ra.RESOURCE_ID=r.RESOURCE_ID';
         stwhere := ' WHERE 0=0 ';
      Else
         If Coalesce(bitflag001, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
            -- RULE: Search Participants ignores pool filtering.
            -- Search Participant with Name filter
            stmt := 'SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r';
            stwhere := ' WHERE 0=0 ';
            
            If (last_id IS NOT NULL) Then
               stwhere := stwhere || ' AND r.RESOURCE_ID > ''';
               stwhere := stwhere || h_last_id;
               stwhere := stwhere || ''' ';
            End If;
            
            stwhere := stwhere || ' AND r.CALENDAR_ID IS NOT NuLL ';
            
            If (fname IS NOT NULL) Then
               stwhere := stwhere || 'AND UPPER(r.FULL_NAME) like UPPER(''';
               stwhere := stwhere || h_fname;
               stwhere := stwhere || '%'')';
            End If;
         Else
            -- search for resource effort with name filter
            stmt := stmt || 'SELECT DISTINCT r.RESOURCE_ID '
                   || 'FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r';
            
            stwhere := ' WHERE 0=0 ';
            
            If (last_id IS NOT NULL) Then
               stwhere := stwhere || ' AND r.RESOURCE_ID > ''';
               stwhere := stwhere || h_last_id;
               stwhere := stwhere || ''' ';
            End If;
            
            stwhere := stwhere || 'AND TP.ELEMENT_ID=R.RESOURCE_ID '
                   || '  AND (MP.RANK >= SUBSTR(TP.RANK, 1, 8) '
                   || '        AND MP.RANK <= TP.RANK '
                   || '        AND length(MP.RANK) <= length(TP.RANK) '
                   || '        AND substr(TP.RANK, 1, length(MP.RANK)) = MP.RANK) '
                   || '  AND r.CALENDAR_ID IS NOT NuLL ';
            
            If (fname IS NOT NULL) Then
               stwhere := stwhere || '  AND UPPER(r.FULL_NAME) LIKE UPPER(''' || h_fname || '%'')';
            End If;
         End If;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         If (pool_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank
               FROM TMT_POOLS
               WHERE ELEMENT_ID = pool_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1372+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1373) : cplan.sql(1519) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               nLen := Coalesce(Length(h_rank), 0);
               
               -- current pool and all sub-pools
               stmt := stmt || ',TMT_POOLS p';
               stwhere := stwhere || ' AND r.RESOURCE_ID = p.ELEMENT_ID '
                      || 'AND p.RANK LIKE ''' || h_rank || '%'' ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1386+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1387) : cplan.sql(1533) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || pool_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1389+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1390) : cplan.sql(1536) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, pool_id001, True);
            End If;
         End If;
         
         If (org_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank
               FROM TMT_ORGANIZATIONS
               WHERE ELEMENT_ID = org_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1406+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1407) : cplan.sql(1553) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               stmt := stmt || ',TMT_ORGANIZATIONS o';
               stwhere := stwhere || ' AND r.RESOURCE_ID=o.ELEMENT_ID '
                      || 'AND o.RANK LIKE ''' || h_rank || '%'' ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1417+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1418) : cplan.sql(1564) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || org_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1420+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1421) : cplan.sql(1567) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, org_id001, True);
            End If;
         End If;
         
         If (geo_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank
               FROM TMT_GEOGRAPHIES
               WHERE ELEMENT_ID = geo_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1437+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1438) : cplan.sql(1584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               stmt := stmt || ',TMT_GEOGRAPHIES g';
               stwhere := stwhere || ' AND r.RESOURCE_ID=g.ELEMENT_ID '
                      || 'AND g.RANK LIKE ''' || h_rank || '%'' ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1448+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1449) : cplan.sql(1595) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || geo_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1451+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1452) : cplan.sql(1598) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, geo_id001, True);
            End If;
         End If;
         
         If Coalesce(h_work_location, 0) > 0 AND Coalesce(h_work_location, 0) <> 265 Then
            stwhere := stwhere || ' AND r.WORK_LOCATION=' || TO_CHAR(Coalesce(work_location001, 0)) || ' ';
         End If;
      End If;
      
      --Parse Roles list ( string of IDs, lenght 32, no comma separator: 'ID32ID32ID32')
      If (and_roles IS NOT NULL) Then
         nLen001 := Coalesce(Length(h_and_roles), 0);
         
         i := 0;
         WHILE i < nLen001 LOOP
            -- Get next role_id
            h_role_id := PMO_STPU.strnpcpy(32, 32, i+1, h_and_roles);
            
            BEGIN
               SELECT RANK INTO h_rank
               FROM TMT_COMPETENCIES
               WHERE ELEMENT_ID = h_role_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1481+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1482) : cplan.sql(1628) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1488+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1489) : cplan.sql(1635) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1491+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1492) : cplan.sql(1638) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_role_id, True);
            End If;
            
            stwhere := stwhere || ' AND EXISTS '
                   || '(SELECT t1.ELEMENT_ID '
                   || '  FROM TMT_COMPETENCIES t1'
                   || '  WHERE t1.RANK LIKE ''' || h_rank || '%''';
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               stwhere := stwhere || 'AND r.ELEMENT_ID=t1.ELEMENT_ID) ';
            Else
               stwhere := stwhere || 'AND r.RESOURCE_ID=t1.RESOURCE_ID) ';
            End If;
            
            -- profile search only for the first id in role list
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               EXIT;
            End If;
         i := i + 32;
         END LOOP;
      End If;
      
      -- RULE: skills list applies only to resources
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         nLen002 := Coalesce(Length(h_and_skills), 0);
         
         i := 0;
         WHILE i < nLen002 LOOP
            -- Get next skill id
            h_role_id := PMO_STPU.strnpcpy(32, 32, i+1, h_and_skills);
            
            BEGIN
               SELECT RANK INTO h_rank
               FROM TMT_COMPETENCIES
               WHERE ELEMENT_ID = h_role_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1534+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1535) : cplan.sql(1681) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1541+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1542) : cplan.sql(1688) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1544+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1545) : cplan.sql(1691) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_role_id, True);
            End If;
            
            stwhere := stwhere || ' AND EXISTS '
                   || '(SELECT t1.ELEMENT_ID '
                   || '  FROM TMT_COMPETENCIES t1'
                   || '  WHERE t1.RANK LIKE ''' || h_rank || '%'''
                   || '     AND r.RESOURCE_ID=t1.RESOURCE_ID) ';
         i := i + 32;
         END LOOP;
      End If;
      
      stmt := stmt || stwhere;
   End If;
   
   stmt := stmt || ' ORDER BY RESOURCE_ID  ';
   
   --trace_printf("%s\n\n", stmt); trace_flush();
   
   OPEN C12 FOR stmt;
   
   -- At this point we have resource and profile id's that meet the filtering requirements
   LOOP
      FETCH C12 INTO h_res_id;
      IF C12%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("FETCH C12: SC_STATE = %d, h_res_id: %s\n", SC_STATE,h_res_id);//trace_flush();
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      -- RULE: if resource is assigned at the project level then this resource is not returned
      -- in the result set unless the search is done on the project
      If Coalesce(h_type, 0) = 0 AND Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT AND Coalesce(
            h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT AND (Coalesce(
            h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE OR (pool_id001 IS NULL)) Then
         -- Search for EFFORT not from Project for Resource or new profile
         
         OPEN C42;
         
         FETCH C42 INTO h_uniqueID;
         IF C42%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               CLOSE C42 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1606+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1607) : cplan.sql(1753) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- skip this resource
            -- TODO: may need to let client know why resource is not returned as part of the
            -- result set.
            GOTO Continue0000;
         Else
            BEGIN
               CLOSE C42 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1624+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1625) : cplan.sql(1771) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_and_roles := and_roles;
      h_and_skills := and_skills;
      If Coalesce(h_type, 0) > 0 Then
         If SubStr(h_res_type, 1, 1) = '1' Then
            --Get a unique ID
            h_assign_id := SYS_GUID();
            h_exists := 0;
         Else
            If isProjectType(Coalesce(h_type, 0)) <> 0 Then
               -- RULE: If assigned to task cannot assign to project
               BEGIN
                  SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var 
                  FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id
                  AND RESOURCE_ID = h_res_id
                  AND RESOURCE_TYPE = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1653+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1654) : cplan.sql(1800) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_temp_var, 0) > 0 Then
                  BEGIN
                     CLOSE C12 ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1667+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1668) : cplan.sql(1814) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1673+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1674) : cplan.sql(1820) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1676+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1677) : cplan.sql(1823) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                        , True);
               End If;
            End If;
         End If;
         
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_and_roles, h_and_skills, h_shw_roles, h_shw_skills, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, Coalesce(h_per_day, 0), 0
                        , 0.00, 0.00);
         
         If i_return <> 0 Then
            BEGIN
               CLOSE C12 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1700+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1701) : cplan.sql(1847) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If i_return < PMO_CNST.DB_OFFSET Then
               i_return := PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1710+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1711) : cplan.sql(1857) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(i_return), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1713+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1714) : cplan.sql(1860) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(i_return, h_assign_id, True);
         ElsIf (SubStr(h_res_type, 1, 1) = '1') AND (SubStr(h_rec_status, 1, 1) = 'D') Then
            BEGIN
               UPDATE PROFILES SET REC_STATUS='U' WHERE PROFILE_ID = h_res_id AND REC_STATUS='D';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1729+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1730) : cplan.sql(1876) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         EXIT;
      Else
         If Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT Then
            h_start_DT := start_date001;
            h_finish_DT := finish_date001;
         End If;
         
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_and_roles, h_and_skills, h_shw_roles, h_shw_skills, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
         
         If i_return = 0 Then
            i_row := i_row + 1;
         End If;
         
         If i_row = Coalesce(h_max_rows, 0) Then
            EXIT;
         End If;
      End If;
      <<Continue0000>>
      NULL;
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE C12 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1769+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1770) : cplan.sql(1916) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- TOREVIEW: August 24, 2004
   If (i_return = PMO_CNST.ASSIGN_STRING_TOLONG) OR ((i_return <> 0) AND (Coalesce(
         h_type, 0) > 0)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1778+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1779) : cplan.sql(1925) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(i_return), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1781+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1782) : cplan.sql(1928) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(i_return, h_assign_id, True);
   End If;
   
   If i_noskill <> 1 Then
      h_rs_id := 'NOTNULL';
      h_uniqueID := '';
      h_last_id := '';
      
      OPEN CU42;
      
      i := 0;
      
      LOOP
         FETCH CU42 INTO
         h_task_id                 -- RS_ID
         , h_res_id                  -- RES_ID
         , h_role_id                 -- RTRIM(ROLE_ID)
         , h_and_role                -- COALESCE(ROLE_NAME , '')
         , h_skill_id                -- RTRIM(SKILL_ID)
         , h_and_skill               -- COALESCE(SKILL_NAME , '')
         , h_shw_role                -- COALESCE(SHOW_ROLE , '')
         , h_shw_skill ;             -- COALESCE(SHOW_SKILL , '');
         IF CU42%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         -- NOTE: table row rules seems that first row for resource will have
         -- role_id and res_id equal
         If (h_role_id IS NOT NULL) AND PMO_ORASTD.strcmp(h_role_id, h_res_id) = 0 
               AND Coalesce(Length(h_rs_id), 0) = 0 Then
            -- Needed further down for profiles
            h_rs_id := h_task_id;
         End If;
         
         -- not first row for the resource
         If PMO_ORASTD.strcmp(h_uniqueID, h_res_id) <> 0 Then
            -- first row for the resource
            If i <> 0 Then
               -- save previous resource aggregates
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET
                  ROLE_ID = h_source_ids
                  , ROLE_NAME = h_and_roles
                  , SKILL_ID = h_target_ids
                  , SKILL_NAME = h_and_skills
                  , SHOW_ROLE = h_shw_roles
                  , SHOW_SKILL = h_shw_skills
                  WHERE RS_ID = h_last_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1847+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1848) : cplan.sql(1994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_and_roles := '';
            h_and_skills := '';
            h_shw_roles := '';
            h_shw_skills := '';
            h_source_ids := '';
            -- role ids
            h_target_ids := '';
            -- skill ids
            
            -- save id of current resource
            h_uniqueID := h_res_id;
            i := 0;
         End If;
         
         If i <> 0 AND (h_last_id IS NOT NULL) Then
            -- clear data for last rs_id we used
            BEGIN
               UPDATE PMO_TMP_SEARCH SET
               ROLE_ID = NuLL
               , ROLE_NAME = NuLL
               , SKILL_ID = NuLL
               , SKILL_NAME = NuLL
               , SHOW_ROLE = NuLL
               , SHOW_SKILL = NuLL
               WHERE RS_ID = h_last_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1888+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1889) : cplan.sql(2035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (h_and_role IS NOT NULL) Then
            If (h_and_roles IS NOT NULL) Then
               h_and_roles := SubStr(h_and_roles || ',', 1, 8000);
            End If;
            
            h_and_roles := SubStr(h_and_roles || h_and_role, 1, 8000);
         End If;
         
         If (h_role_id IS NOT NULL) Then
            h_source_ids := SubStr(h_source_ids || h_role_id, 1, 8000);
         End If;
         
         If (h_and_skill IS NOT NULL) Then
            If (h_and_skills IS NOT NULL) Then
               h_and_skills := SubStr(h_and_skills || ',', 1, 8000);
            End If;
            
            h_and_skills := SubStr(h_and_skills || h_and_skill, 1, 8000);
         End If;
         
         If (h_skill_id IS NOT NULL) Then
            h_target_ids := SubStr(h_target_ids || h_skill_id, 1, 8000);
         End If;
         
         If (h_shw_role IS NOT NULL) Then
            If (h_shw_roles IS NOT NULL) Then
               h_shw_roles := SubStr(h_shw_roles || ',', 1, 8000);
            End If;
            
            h_shw_roles := SubStr(h_shw_roles || h_shw_role, 1, 8000);
         End If;
         
         If (h_shw_skill IS NOT NULL) Then
            If (h_shw_skills IS NOT NULL) Then
               h_shw_skills := SubStr(h_shw_skills || ',', 1, 8000);
            End If;
            
            h_shw_skills := SubStr(h_shw_skills || h_shw_skill, 1, 8000);
         End If;
         --trace_printf("i = %d\nROLES = %s\nSKILLS = %s\n\n", i,h_source_ids,h_target_ids);//trace_flush();
         
         i := i + 1;
         
         h_last_id := h_task_id;
      END LOOP;
      BEGIN
         CLOSE CU42 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1947+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1948) : cplan.sql(2094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If i <> 0 Then
         -- save previous resource aggregates
         BEGIN
            UPDATE PMO_TMP_SEARCH SET
            ROLE_ID = h_source_ids
            , ROLE_NAME = h_and_roles
            , SKILL_ID = h_target_ids
            , SKILL_NAME = h_and_skills
            , SHOW_ROLE = h_shw_roles
            , SHOW_SKILL = h_shw_skills
            WHERE RS_ID = h_last_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1973+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1974) : cplan.sql(2120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If (finish_date001 IS NOT NULL) Then
      If Coalesce(h_work, 0) = 0 Then
         BEGIN
            SELECT COALESCE(SUM(AVAILABLE_MIN),0) INTO h_effort
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID=h_project_id
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1993+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1994) : cplan.sql(2140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_duration := 0;
         NULL;
      End If;
   End If;
   
   BEGIN
      SELECT CURRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) INTO h_currency_id, h_rate_echange
      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
      WHERE tw.ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2013+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2014) : cplan.sql(2160) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE: %d, h_currency_id: %d:%d, h_rate_echange: %d:%.2f\n",SC_STATE,h_currency_id_Ind,h_currency_id,h_rate_echange_Ind,h_rate_echange);//trace_flush();
   --trace_printf("h_consider: %d, h_res_id: %d:%s, h_rs_id: %s\n", h_consider,h_res_id_Ind,h_res_id,h_rs_id);
   
   If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
      --trace_printf("Null: h_res_id = %s\n",h_res_id);//trace_flush();
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0005';
      PMO_DB2M.SC_STATE := 0;
      
      -- CURIOUS: h_rs_id is the rs_id of the first record found up in the loop that
      -- aggregates skills, roles etc.
      BEGIN
         INSERT INTO PMO_TMP_CPLN_0005
         SELECT RES_ID
         , REGULAR_COST
         , REGULAR_SALE
         , SPECIAL_COST
         , SPECIAL_SALE
         FROM PMO_TMP_SEARCH
         WHERE RS_ID = h_rs_id
         AND (REGULAR_COST + REGULAR_SALE) > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2049+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2050) : cplan.sql(2196) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_role_id = %s\n",SC_STATE,h_role_id);//trace_flush();
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0005
            SELECT RES_ID
            , MAX(REGULAR_COST) AS REGULAR_COST
            , MAX(REGULAR_SALE) AS REGULAR_SALE
            , MAX(SPECIAL_COST) AS SPECIAL_COST
            , MAX(SPECIAL_SALE) AS SPECIAL_SALE
            FROM PMO_TMP_SEARCH
            WHERE RS_ID <> h_rs_id
            GROUP BY RES_ID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2076+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2077) : cplan.sql(2223) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0005
               SELECT RES_ID, 0.00, 0.00, 0.00, 0.00
               FROM PMO_TMP_SEARCH
               WHERE RS_ID = h_rs_id
               GROUP BY RES_ID;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2098+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2099) : cplan.sql(2245) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      OPEN C775C0 FOR SELECT h_res_id AS ELEMENT_ID, 41 AS TYPE_ID, p.PROFILE_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, h_work_location AS WORK_LOCATION,
      o1.ELEMENT_NAME AS ORGANIZATION,g1.ELEMENT_NAME AS GEOGRAPHY,
      ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_COST * h_rate_echange ELSE a.REGULAR_COST * h_rate_echange END,2) AS REGULAR_COST,
      ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.REGULAR_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_SALE * h_rate_echange ELSE a.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,
      ROUND(CASE WHEN pa.SPECIAL_COST>0 THEN pa.SPECIAL_COST WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_COST * h_rate_echange ELSE a.SPECIAL_COST * h_rate_echange END,2) AS SPECIAL_COST,
      ROUND(CASE WHEN pa.SPECIAL_RATE>0 THEN pa.SPECIAL_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_SALE * h_rate_echange ELSE a.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,
      rs.START_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
      '' AS VERIFIED, 0 AS STATUS2_ID, '' AS FULL_NAME, pp.ELEMENT_NAME AS POOL_NAME, '' AS E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, h_and_roles AS ROLE_ID, h_and_skills AS SKILL_ID, 0 AS ASSIGNED_FLAG,
      'PMOFFICE' AS FS_LABOR_CODE, TO_TIMESTAMP('1899-12-31 00:00:00') AS RES_MODIFIED_DATE
      FROM PMO_TMP_CPLN_0005 temp JOIN PMO_TMP_SEARCH rs ON rs.RES_ID = temp.RES_ID
      LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
      JOIN PROFILES p ON p.PROFILE_ID = h_res_id
      JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEMENT_ID
      LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID
      LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON p.ORG_ID = o1.ELEMENT_ID
      LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON p.GEO_ID = g1.ELEMENT_ID
      LEFT OUTER JOIN TMT_POOLS pp ON pp.ELEMENT_ID = p.POOL_ID
      WHERE NOT (rs.ROLE_NAME IS NuLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuLL);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2124+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2125) : cplan.sql(2271) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C775C0;
   Else
      If i_noskill = 1 Then
         OPEN C775E1 FOR SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,
         o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.REGULAR_COST ELSE rs.REGULAR_COST * h_rate_echange END,2) AS REGULAR_COST,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.REGULAR_SALE ELSE rs.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_COST ELSE rs.SPECIAL_COST * h_rate_echange END,2) AS SPECIAL_COST,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_SALE ELSE rs.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,
         rs.START_DATE, rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         rv.FULL_NAME AS VERIFIED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME, RM.E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCHED_WORK,0) AS ASSIGNED_FLAG,
         r.FS_LABOR_CODE, r.REC_DATETIME AS RES_MODIFIED_DATE
         FROM PMO_TMP_SEARCH rs JOIN TMT_POOLS p ON rs.RES_ID=p.ELEMENT_ID
         JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.PARENT_ID
         LEFT OUTER JOIN RESOURCES RM ON pp.MANAGER_ID=RM.RESOURCE_ID
         JOIN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEFT OUTER JOIN RESOURCES rv ON rv.RESOURCE_ID=r.VERIFIED_1
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMENT_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID
         ORDER BY ELEMENT_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2149+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2150) : cplan.sql(2296) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775E1;
      Else
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0006';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0006
            SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, 1
            FROM PMO_TMP_SEARCH WHERE DEFAULT_FLAG = '1';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2171+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2172) : cplan.sql(2318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0006
            SELECT  RES_ID,                             MAX(REGULAR_COST) AS REGULAR_COST,
            MAX(REGULAR_SALE) AS REGULAR_SALE,  MAX(SPECIAL_COST) AS SPECIAL_COST,
            MAX(SPECIAL_SALE) AS SPECIAL_SALE , 0
            FROM PMO_TMP_SEARCH
            WHERE RES_ID NOT IN (SELECT RES_ID
            FROM PMO_TMP_SEARCH WHERE DEFAULT_FLAG = '1')
            GROUP BY RES_ID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2196) : cplan.sql(2342) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN C775E0 FOR SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,
         o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,
         
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST
         WHEN a.REGULAR_COST>0 THEN a.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.REGULAR_COST * h_rate_echange
         END,2) AS REGULAR_COST,
         
         ROUND(CASE WHEN pa.REGULAR_COST> 0 THEN pa.REGULAR_RATE
         WHEN a.REGULAR_COST > 0 THEN a.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.REGULAR_SALE * h_rate_echange
         END,2) AS REGULAR_SALE,
         
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.SPECIAL_COST
         WHEN a.REGULAR_COST>0 THEN a.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.SPECIAL_COST * h_rate_echange
         END,2) AS SPECIAL_COST,
         
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.SPECIAL_RATE
         WHEN a.REGULAR_SALE>0 THEN a.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.SPECIAL_SALE * h_rate_echange
         END,2) AS SPECIAL_SALE,
         
         rs.START_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         rv.FULL_NAME AS VERIFIED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME, RM.E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,
         rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCHED_WORK,0) AS ASSIGNED_FLAG,
         r.FS_LABOR_CODE, r.REC_DATETIME AS RES_MODIFIED_DATE
         FROM PMO_TMP_SEARCH rs
         LEFT JOIN PMO_TMP_CPLN_0006 def ON rs.RES_ID=def.RES_ID AND def.DEFAULT_FLAG = 1
         LEFT JOIN PMO_TMP_CPLN_0006 prf ON rs.RES_ID=prf.RES_ID AND prf.DEFAULT_FLAG = 0
         JOIN TMT_POOLS p ON rs.RES_ID=p.ELEMENT_ID
         JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.PARENT_ID
         LEFT OUTER JOIN RESOURCES RM ON pp.MANAGER_ID=RM.RESOURCE_ID
         JOIN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEFT OUTER JOIN RESOURCES rv ON rv.RESOURCE_ID=r.VERIFIED_1
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         LEFT OUTER JOIN RESOURCES rr ON r.VERIFIED_1=rr.RESOURCE_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMENT_ID LEFT OUTER JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID LEFT OUTER JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID
         WHERE NOT (rs.ROLE_NAME IS NuLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuLL)
         ORDER BY ELEMENT_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2246+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2247) : cplan.sql(2393) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775E0;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2256+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2257) : cplan.sql(2403) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2268+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2269) : cplan.sql(2415) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_2;
ALTER FUNCTION SP_SEARCH_2 COMPILE DEBUG;

/*#################################################################################################*/

-- Search - Assign in Assigned tab (Cplan view)

/*#################################################################################################*/

/*
#Stored Procedure  :SP_SEARCH_1
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_SEARCH_1 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_1
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , type001 PLS_INTEGER
   , and_roles VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_finish_D DATE;
   h_res_type VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_active VARCHAR2(1);
   h_rec_status VARCHAR2(1);
   h_res_cost NUMBER := 0.00;
   stmt VARCHAR2(7999);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_cri_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_task_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   id VARCHAR2(32);
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_1') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_1'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_1(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(type001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 109+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_1(110) : cplan.sql(2555) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   
   -- Parse parameters to local variables
   
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := type001;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_work_type := work_type;
   
   h_task_id := h_project_id;
   h_dayofyear := '';
   h_prj_assign_id := NULL;
   h_table_type := GetTable(Coalesce(h_type, 0));
   h_task_type := 0;
   
   --local variables
   /*int iii = 0;*/
   -- 0 = SD, 1 = SD + FD
   h_res_cost := 0.00;
   h_update := 0;
   
   If (prj_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 150+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(151) : cplan.sql(2596) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 153+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(154) : cplan.sql(2599) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If (and_roles IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 160+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(161) : cplan.sql(2606) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 163+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(164) : cplan.sql(2609) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 170+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(171) : cplan.sql(2616) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 173+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(174) : cplan.sql(2619) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 180+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(181) : cplan.sql(2626) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 183+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(184) : cplan.sql(2629) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If Coalesce(h_bitflag, 0) = 5 AND Coalesce(h_unit, 0) = 5 AND Coalesce(h_work, 0) = 5 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 190+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(191) : cplan.sql(2636) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 193+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(194) : cplan.sql(2639) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL)) AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 209+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(210) : cplan.sql(2655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 216+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(217) : cplan.sql(2662) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 219+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(220) : cplan.sql(2665) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      
      iSF := 1;
   ElsIf ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 235+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(236) : cplan.sql(2681) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   If (finish_date001 IS NULL) Then
      h_finish_DT := '';
   End If;
   
   h_res_id := SubStr(h_res_id, 1, 32);
   h_res_type := SubStr(h_res_type, 1, 1);
   h_and_roles := and_roles;
   
   If Coalesce(h_type, 0) > 0 Then
      -- Get resource_id and resource type
      h_res_id := SubStr(SubStr(h_and_roles, 1, 32), 1, 32);
      h_res_type := PMO_STPU.strnpcpy(1, 1, 33, h_and_roles);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
            INTO h_project_id, h_per_day, h_per_week, h_priority
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 266+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(267) : cplan.sql(2712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT B.TYPE_ID, B.TASK_TYPE, B.UNRESOLVED_START, B.UNRESOLVED_FINISH, B.PROJECT_ID,
            COALESCE(A.MINUTE_PER_DAY, 480), COALESCE(A.MINUTE_PER_WEEK, 2400), A.PRIORITY
            INTO h_type, h_task_type, h_task_start_DT, h_task_finish_DT, h_project_id,
            h_per_day, h_per_week, h_priority FROM TMT_WBS A, TMT_WBS B
            WHERE B.ELEMENT_ID = h_task_id AND B.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND A.ELEMENT_ID = B.PROJECT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 283+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(284) : cplan.sql(2729) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT PROJECT_ID, WBS_ID INTO h_project_id, id
            FROM TMT_CRI WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 299+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(300) : cplan.sql(2745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If (id IS NOT NULL) Then
               h_table_type := 1;
            End If;
            BEGIN
               SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
               INTO h_per_day, h_per_week, h_priority
               FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 318+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(319) : cplan.sql(2764) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 324+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(325) : cplan.sql(2770) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 327+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(328) : cplan.sql(2773) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
      Else
         h_table_type := 1;
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 346+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(347) : cplan.sql(2792) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 362+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(363) : cplan.sql(2808) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 369+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(370) : cplan.sql(2815) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 372+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(373) : cplan.sql(2818) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
   Else
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
         INTO h_per_day, h_per_week, h_priority
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 388+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(389) : cplan.sql(2834) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT 480, 2400 INTO h_per_day, h_per_week FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 401+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(402) : cplan.sql(2847) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 407+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(408) : cplan.sql(2853) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 410+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(411) : cplan.sql(2856) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
         End If;
      End If;
   End If;
   
   /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_type > 0) )
       {
   //      InitializeHost(SC_PARAM(h_start_DT), SC_PARAM(h_task_start_DT));
   //      InitializeHost(SC_PARAM(h_finish_DT), SC_PARAM(h_task_finish_DT));
           if (h_consider != SC_SRCH_PROFILE)
           {
               h_consider = 7;
           }
           iSF = 1;
   //      h_effort   = 0;
           h_work     = 0;
       }
       else
       {*/
   If iSF = 0 Then
      h_start_DT := start_date001;
      h_finish_DT := finish_date001;
      h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
      
      If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 441+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(442) : cplan.sql(2887) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 444+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(445) : cplan.sql(2890) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf Coalesce(h_temp_var, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 449+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(450) : cplan.sql(2895) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 452+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(453) : cplan.sql(2898) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
      End If;
      
      h_finish_DT := h_finish_DT;
      If (h_finish_DT IS NULL) Then
         h_finish_DT := '';
      End If;
   ElsIf (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
      If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
         h_eunit := 2;
      ElsIf Coalesce(h_unit, 0) = 0 Then
         h_eunit := 1;
      ElsIf (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
         h_eunit := 1;
      ElsIf Coalesce(h_unit, 0) = 5 Then
         h_work := Coalesce(h_per_week, 0) * Coalesce(h_work, 0);
         h_eunit := 3;
      End If;
   End If;
   --}
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   h_shw_role := '';
   h_shw_skill := '';
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   
   If Coalesce(h_type, 0) > 0 Then
      h_temp_sc_state := AssignResource(h_table_type, h_type, h_task_id, h_res_id, h_rec_user
                                  , h_assign_id);
      
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 490+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(491) : cplan.sql(2936) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 493+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(494) : cplan.sql(2939) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_sc_state, 0), h_assign_id, True);
      End If;
      
      h_start_DT := NULL;
      
      BEGIN
         SELECT MIN(BAK_START_DT), PMO_ORASTD.DAYS(h_start_date) - PMO_ORASTD.DAYS(MIN(BAK_START_DT))
         INTO h_start_DT, h_total 
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id 
         AND TASK_ID = h_project_id 
         AND RESOURCE_ID = h_res_id 
         AND RESOURCE_TYPE = h_res_type;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 514+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(515) : cplan.sql(2960) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_start_DT IS NOT NULL)) Then
         h_from_prj := 1;
         If Coalesce(h_total, 0) < 0 Then
            h_start_date := h_start_DT;
         End If;
      End If;
      
      h_work_type := 0;
      h_total := 0;
      If iSF = 1 Then
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND
               TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND SCHED_WORK >= h_effort THEN 0 ELSE 1 END,SCHED_WORK
               INTO h_prj_assign_id, h_start_DT, h_total, h_sched_work
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 542+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(543) : cplan.sql(2988) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               h_from_prj := 1;
               If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                  BEGIN
                     SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                     TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                     INTO h_prj_assign_id, h_start_DT, h_total 
                     FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 562+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(563) : cplan.sql(3008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_work_type := 2;
               End If;
            ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) OR (Coalesce(h_total, 0) = 1) Then
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                  TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                  INTO h_prj_assign_id, h_start_DT, h_total 
                  FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 582+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(583) : cplan.sql(3028) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
               TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
               INTO h_prj_assign_id, h_start_DT, h_total 
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 600+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(601) : cplan.sql(3046) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK
               INTO h_prj_assign_id, h_start_DT,h_total, h_sched_work 
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 618+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(619) : cplan.sql(3064) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               h_from_prj := 1;
               If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                  BEGIN
                     SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                     INTO h_prj_assign_id, h_start_DT,h_total 
                     FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 637+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(638) : cplan.sql(3083) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_work_type := 2;
                  NULL;
               End If;
            ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                  INTO h_prj_assign_id, h_start_DT,h_total 
                  FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 657+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(658) : cplan.sql(3103) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
               INTO h_prj_assign_id, h_start_DT,h_total 
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 674+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(675) : cplan.sql(3120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If Coalesce(h_total, 0) = 1 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 684+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(685) : cplan.sql(3130) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 687+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(688) : cplan.sql(3133) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
         ElsIf (h_prj_assign_id IS NOT NULL) Then
            If iSF = 1 Then
               h_sched_work := 0;
               BEGIN
                  SELECT SUM(SCHED_MIN) INTO h_sched_work 
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id 
                  AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 704+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(705) : cplan.sql(3150) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_sched_work := 0;
               BEGIN
                  SELECT SUM(SCHED_MIN) INTO h_sched_work 
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id 
                  AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 721+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(722) : cplan.sql(3167) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_sched_work, 0) < Coalesce(h_effort, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 729+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(730) : cplan.sql(3175) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 732+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(733) : cplan.sql(3178) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
            End If;
         End If;
         h_from_prj := 1;
      ElsIf (Coalesce(h_from_prj, 0) = 1) AND (isNotProjectType(Coalesce(h_type, 0))) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 740+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(741) : cplan.sql(3186) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 743+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(744) : cplan.sql(3189) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
      End If;
      
      If SubStr(h_res_type, 1, 1) = '0' Then
         h_shw_role := SubStr(h_shw_role || h_res_id, 1, 8000);
         
         BEGIN
            SELECT NICK_NAME, PMO_ORASTD.JULIAN_DAY(h_start_DT), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
            INTO h_full_name, h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
            FROM RESOURCES 
            WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 762+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(763) : cplan.sql(3208) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 769+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(770) : cplan.sql(3215) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 772+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(773) : cplan.sql(3218) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         If (SubStr(h_active, 1, 1) = '0') OR ((Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(
               h_julian_day, 0) > Coalesce(h_j_disconti_date, 0))) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 780+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(781) : cplan.sql(3226) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 783+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(784) : cplan.sql(3229) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_participant, h_security_id
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '0' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 798+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(799) : cplan.sql(3244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_shw_skill := SubStr(h_shw_skill || h_res_id, 1, 8000);
         
         BEGIN
            SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 813+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(814) : cplan.sql(3259) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 820+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(821) : cplan.sql(3266) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 823+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(824) : cplan.sql(3269) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         BEGIN
            SELECT BAK_START_DT, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)) AND BAK_WORK >= h_effort THEN 0 ELSE 1 END
            INTO h_start_DT, h_total 
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 839+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(840) : cplan.sql(3285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_total, 0) = 1) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 846+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(847) : cplan.sql(3292) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 849+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(850) : cplan.sql(3295) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_participant, h_security_id
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 864+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(865) : cplan.sql(3310) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_start_DT := h_start_DT;
      
      If Coalesce(h_from_prj, 0) = 1 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_date)) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 882+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(883) : cplan.sql(3328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_date
            ELSE h_start_DT END
            INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 898+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(899) : cplan.sql(3344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_totpr := 10;
         h_finish_DT := finish_date001;
         If Coalesce(h_work_type, 0) = 0 Then
            h_update := 3;
         End If;
         
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
         
         If i_return = 0 Then
            If Coalesce(h_work_type, 0) = 0 Then
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 929+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(930) : cplan.sql(3375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='1' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 946+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(947) : cplan.sql(3392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         --h_bitflag = (h_bitflag | SC_BIT_30);  // assign from constrained by project (type > 0)
         h_start_DT := h_start_date;
         h_finish_DT := finish_date001;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_totpr := PMO_CNST.SC_SRCH_PROFILE;
         Else
            h_totpr := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
         End If;
         
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
         
         If i_return = 0 Then
            BEGIN
               UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 982+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(983) : cplan.sql(3428) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      --Parse resource_ids list
      
      If Coalesce(Length(h_and_roles), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 33);
      End If;
      If numItems > 0 Then
         h_count := 0;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            h_from_prj := 0;
            -- Get next resource_id and resource type
            --pmo_substr(h_res_id, h_and_roles, h_count*33, 32);       //strncpy( h_res_id, h_and_roles[h_count*33], 32);
            h_res_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 33+1, h_and_roles);
            --pmo_substr(h_res_type, h_and_roles, h_count*33+32, 1);   //strncpy( h_res_type, h_and_roles[h_count*33+32], 1);
            h_res_type := PMO_STPU.strnpcpy(1, 1, Coalesce(h_count, 0) * 33 + 32+1, h_and_roles);
            
            h_start_date := start_date001;
            h_from_prj := 0;
            h_total := 0;
            h_start_DT := h_start_date;
            
            BEGIN
               SELECT COUNT(*), MIN(PROPOSED_START) INTO h_total, h_start_DT 
               FROM TASK_ASSIGNMENTS
               WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id
               AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = h_res_type;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1022+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1023) : cplan.sql(3468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_total, 0) > 0) AND ((h_start_DT IS NOT NULL)) Then
               h_from_prj := 1;
               If PMO_ORASTD.strcmp(h_start_date, h_start_DT) < 0 Then
                  h_start_date := h_start_DT;
               End If;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_consider := PMO_CNST.SC_SRCH_PROFILE;
               If Coalesce(Length(h_shw_skill), 0) = 0 Then
                  h_shw_skill := h_res_id;
               Else
                  h_shw_skill := SubStr(h_shw_skill || ''',''', 1, 8000);
                  h_shw_skill := SubStr(h_shw_skill || h_res_id, 1, 8000);
               End If;
            Else
               If Coalesce(Length(h_shw_role), 0) = 0 Then
                  h_shw_role := h_res_id;
               Else
                  h_shw_role := SubStr(h_shw_role || ''',''', 1, 8000);
                  h_shw_role := SubStr(h_shw_role || h_res_id, 1, 8000);
               End If;
            End If;
            
            h_work_type := 0;
            h_total := 0;
            
            If Coalesce(h_from_prj, 0) = 1 Then
               If iSF = 1 Then
                  If SubStr(h_res_type, 1, 1) = '0' Then
                     BEGIN
                        SELECT PROJECT_ASSIGNMENT_ID, SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND
                        TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND SCHED_WORK >= h_effort THEN 0 ELSE 1 END,SCHED_WORK
                        INTO h_prj_assign_id, h_start_DT, h_total, h_sched_work
                        FROM TASK_ASSIGNMENTS
                        WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '2';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1069+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1070) : cplan.sql(3515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                        h_from_prj := 1;
                        If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                           BEGIN
                              SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                              TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                              INTO h_prj_assign_id, h_start_DT, h_total 
                              FROM TASK_ASSIGNMENTS
                              WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                              PMO_DB2M.SC_STATE := SQLCODE;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1089+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1090) : cplan.sql(3535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                        Else
                           h_work_type := 2;
                        End If;
                     ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) OR (Coalesce(h_total, 0) = 1) Then
                        BEGIN
                           SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                           TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                           INTO h_prj_assign_id, h_start_DT, h_total 
                           FROM TASK_ASSIGNMENTS
                           WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1109+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1110) : cplan.sql(3555) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     BEGIN
                        SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                        TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                        INTO h_prj_assign_id, h_start_DT, h_total FROM TASK_ASSIGNMENTS
                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                        AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1127+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1128) : cplan.sql(3573) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  If SubStr(h_res_type, 1, 1) = '0' Then
                     BEGIN
                        SELECT PROJECT_ASSIGNMENT_ID, SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK
                        INTO h_prj_assign_id, h_start_DT,h_total, h_sched_work FROM TASK_ASSIGNMENTS
                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                        AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1145+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1146) : cplan.sql(3591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                        If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                           BEGIN
                              SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                              INTO h_prj_assign_id, h_start_DT,h_total FROM TASK_ASSIGNMENTS
                              WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                              AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                              PMO_DB2M.SC_STATE := SQLCODE;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1163+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1164) : cplan.sql(3609) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                        Else
                           h_work_type := 2;
                           NULL;
                        End If;
                     ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                        BEGIN
                           SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                           INTO h_prj_assign_id, h_start_DT,h_total FROM TASK_ASSIGNMENTS
                           WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                           AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1183+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1184) : cplan.sql(3629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     BEGIN
                        SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                        INTO h_prj_assign_id, h_start_DT,h_total FROM TASK_ASSIGNMENTS
                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                        AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1200+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1201) : cplan.sql(3646) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  If Coalesce(h_total, 0) = 1 Then
                     h_from_prj := 2;
                  Else
                     If (h_prj_assign_id IS NOT NULL) Then
                        If iSF = 1 Then
                           h_sched_work := 0;
                           BEGIN
                              SELECT SUM(SCHED_MIN) INTO h_sched_work 
                              FROM TASK_DAY_OF_YEAR
                              WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id 
                              AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
                              PMO_DB2M.SC_STATE := SQLCODE;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1225+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1226) : cplan.sql(3671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                        Else
                           h_sched_work := 0;
                           BEGIN
                              SELECT SUM(SCHED_MIN) INTO h_sched_work 
                              FROM TASK_DAY_OF_YEAR
                              WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id 
                              AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
                              PMO_DB2M.SC_STATE := SQLCODE;
                           EXCEPTION
                              WHEN OTHERS THEN
                              BEGIN
                                 PMO_DB2M.SC_STATE := SQLCODE;
                                 IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1242+1) THEN 
                                    LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1243) : cplan.sql(3688) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                                 END IF; 
                              END;
                           END;
                        End If;
                        If Coalesce(h_sched_work, 0) < Coalesce(h_effort, 0) Then
                           h_count := Coalesce(h_count, 0) + 1;
                           GOTO Continue0000;
                        End If;
                     End If;
                     h_from_prj := 1;
                  End If;
               Else
                  h_count := Coalesce(h_count, 0) + 1;
                  GOTO Continue0000;
               End If;
               
               h_start_DT := h_start_DT;
               h_update := 0;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1272+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1273) : cplan.sql(3718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_date ELSE h_start_DT END
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1287+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1288) : cplan.sql(3733) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_totpr := 10;
               h_finish_DT := finish_date001;
               If Coalesce(h_work_type, 0) = 0 Then
                  h_update := 3;
               End If;
               
               i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               
               If i_return = 0 Then
                  If Coalesce(h_work_type, 0) = 0 Then
                     BEGIN
                        UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1318+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1319) : cplan.sql(3764) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='1' WHERE RES_ID=h_res_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1335+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1336) : cplan.sql(3781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf Coalesce(h_from_prj, 0) = 0 Then
               h_start_DT := h_start_date;
               h_finish_DT := finish_date001;
               
               If SubStr(h_res_type, 1, 1) = '1' Then
                  h_totpr := PMO_CNST.SC_SRCH_PROFILE;
               Else
                  h_totpr := Coalesce(h_consider, 0);
               End If;
               
               i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               If i_return = 0 Then
                  BEGIN
                     UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1369+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1370) : cplan.sql(3815) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            h_count := Coalesce(h_count, 0) + 1;
            <<Continue0000>>
            NULL;
         END LOOP;
         -- end of while
      End If;
   End If;
   
   If (i_return <> 0) AND (Coalesce(h_type, 0) > 0) Then
      If Coalesce(h_participant, 0) = 0 Then
         BEGIN
            DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1397+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1398) : cplan.sql(3843) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET SECURITY_ID = h_security_id, PARTICIPANT = h_participant
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1415+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1416) : cplan.sql(3861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If i_return = 100 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1423+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1424) : cplan.sql(3869) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1426+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1427) : cplan.sql(3872) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
      ElsIf i_return = PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1431+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1432) : cplan.sql(3877) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1434+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1435) : cplan.sql(3880) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI, h_res_id
               , True);
      ElsIf i_return = PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1440+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1441) : cplan.sql(3886) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1443+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1444) : cplan.sql(3889) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE, h_res_id
               , True);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1449+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1450) : cplan.sql(3895) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1452+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1453) : cplan.sql(3898) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_assign_id, True);
      End If;
   End If;
   
   If (Coalesce(Length(h_shw_role), 0) > 0) AND (Coalesce(Length(h_shw_skill), 0) > 0) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0003';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0003 SELECT RESOURCE_ID, RESOURCE_TYPE, '
             || ' MIN(PLANNED_START) AS PLANNED_START,  MAX(PLANNED_FINISH) AS PLANNED_FINISH, '
             || ' SUM(PLANNED_WORK) AS PLANNED_EFFORT,  MIN(SCHED_START) AS START_DATE,  '
             || ' MAX(SCHED_FINISH) AS FINISH_DATE,     SUM(SCHED_WORK) AS WORK,  '
             || ' SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=''' || h_project_id || ''' AND '
             || '      (RESOURCE_ID IN (''' || h_shw_role || ''') OR RESOURCE_ID IN (''' || h_shw_skill 
                  || ''')) GROUP BY RESOURCE_ID, RESOURCE_TYPE ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1484+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1485) : cplan.sql(3930) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 
            'INSERT INTO PMO_TMP_CPLN_0002 SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,  SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || ' MIN(SCHED_START) AS PRJ_SCHED_START,        MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || ' SUM(SCHED_WORK) AS SCHED_REMAINING  '
             || ' FROM TASK_ASSIGNMENTS '
             || ' WHERE TASK_ID=''' || h_project_id || ''' AND '
             || ' (RESOURCE_ID IN (''' || h_shw_role || ''') OR RESOURCE_ID IN (''' || h_shw_skill 
                  || ''')) GROUP BY RESOURCE_ID ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1516+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1517) : cplan.sql(3962) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      stmt := 
            'INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,'
             || 'PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,BEST_START,BEST_FINISH,'
             || 'ASSIGNABLE,PLANNED,SCHED,WORK_EFFORT,PERIOD_AVAILABILITY,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,'
             || 'PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,'
             || 'REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE) '
             || 'SELECT DISTINCT r.RESOURCE_ID,r.FULL_NAME AS ELEMENT_NAME,''0'','
             || 'CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END AS TYPE_ID,'
             || 'r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,'
             || 'rs.START_DATE, rs.FINISH_DATE, rs.ASSIGNABLE, rs.PLANNED, rs.SCHED, rs.PROPOSED, (rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,'
             || 'tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,'
             || 'tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE '
             || 'FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE=''0'' AND a.RESOURCE_ID=r.RESOURCE_ID '
             || 'LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=''' || h_project_id 
                  || ''' AND r.RESOURCE_ID = pa.RESOURCE_ID '
             || 'LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=''' || h_project_id 
                  || ''' AND r.RESOURCE_ID = ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON r.RESOURCE_ID=rs.RES_ID WHERE r.RESOURCE_ID IN (''' 
                  || h_shw_role || ''') '
             || 'UNION ALL '
             || 'SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME AS ELEMENT_NAME, ''1'', 41 AS TYPE_ID,'
             || ''''', a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,'
             || 'rs.START_DATE , rs.FINISH_DATE, rs.ASSIGNABLE ,rs.PLANNED,rs.SCHED, rs.PROPOSED,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,'
             || 'tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,'
             || 'tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE '
             || 'FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE=''1'' AND p.PROFILE_ID=a.RESOURCE_ID '
             || 'LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID= ''' || h_project_id 
                  || ''' AND p.PROFILE_ID = pa.RESOURCE_ID '
             || 'JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID '
             || 'JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID LEFT OUTER JOIN PMO_TMP_SEARCH rs ON a.RESOURCE_ID=rs.RES_ID '
             || 'LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID '
             || 'WHERE p.PROFILE_ID IN (''' || h_shw_skill || ''') '
             || 'ORDER BY TYPE_ID,ELEMENT_NAME';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1571+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1572) : cplan.sql(4017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      stmt := 
            'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME, T.TYPE_ID AS TYPE_ID, '
             || ' T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || ' T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR,T.PLANNED AS PLANNED_WORK,'
             || ' T.SCHED AS SCHED_WORK, T.WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,T.PERIOD_AVAILABILITY, '
             || ' ' || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || ' FROM PMO_TMP_CPLN01_TEMP T'
             || ' LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID WHERE T.RESOURCE_TYPE IN (''0'') '
             || ' UNION ALL '
             || ' SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, '
             || ' '''' AS E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || ' T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR, '
             || ' T.PLANNED AS PLANNED_WORK,T.SCHED AS SCHED_WORK, T.WORK_EFFORT, ' || TO_CHAR(Coalesce(
                  h_eunit, 0)) || ' AS UNIT_EFFORT, '
             || ' T.PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,'''' AS OBJECTIVES,'''' AS RANK, '
             || ' T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || '  T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || ' FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || ' WHERE T.RESOURCE_TYPE = ''1'' ORDER BY TYPE_ID,ELEMENT_NAME';
   ElsIf Coalesce(Length(h_shw_skill), 0) = 0 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0001';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0001 '
             || 'SELECT  RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PLANNED_FINISH,   SUM(PLANNED_WORK) AS PLANNED_EFFORT, '
             || ' MIN(SCHED_START) AS START_DATE,          MAX(SCHED_FINISH) AS FINISH_DATE, '
             || ' SUM(SCHED_WORK) AS WORK,                 SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=''' || h_project_id || ''' AND '
             || ' RESOURCE_ID IN (''' || h_shw_role || ''') AND PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1625+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1626) : cplan.sql(4071) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0002 '
             || 'SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,      MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || ' MIN(SCHED_START) AS PRJ_SCHED_START,       MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || ' SUM(SCHED_WORK) AS SCHED_REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE TASK_ID=''' || h_project_id || ''' AND RESOURCE_ID IN (''' 
                  || h_shw_role || ''') GROUP BY RESOURCE_ID';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1655+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1656) : cplan.sql(4101) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      stmt := 
            'INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,'
             || 'PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,'
             || 'PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,'
             || 'REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE)'
             || 'SELECT DISTINCT r.RESOURCE_ID, r.FULL_NAME,''0'',CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END ,'
             || 'r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,'
             || 'tb.PROJECT_ASSIGNMENT_ID, tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,'
             || 'tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE '
             || 'FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON a.RESOURCE_ID=r.RESOURCE_ID '
             || 'LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=''' || h_project_id 
                  || ''' AND r.RESOURCE_ID = pa.RESOURCE_ID '
             || 'LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=''' || h_project_id 
                  || ''' AND r.RESOURCE_ID = ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID '
             || 'WHERE r.RESOURCE_ID IN (''' || h_shw_role || ''')';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1691+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1692) : cplan.sql(4137) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      stmt := 
            'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME, T.TYPE_ID, '
             || 'T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK, '
             || 'rs.SCHED AS SCHED_WORK, rs.PROPOSED AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, '
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,T.PROJECT_ASSIGNMENT_ID, T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || 'FROM PMO_TMP_CPLN01_TEMP T '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || 'ORDER BY ELEMENT_NAME';
   Else
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0001';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0001 '
             || 'SELECT RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, '
             || '       MAX(PLANNED_FINISH) AS PLANNED_FINISH,    SUM(PLANNED_WORK) AS PLANNED_EFFORT, '
             || '       MIN(SCHED_START) AS START_DATE,           MAX(SCHED_FINISH) AS FINISH_DATE, '
             || '       SUM(SCHED_WORK) AS WORK,                  SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS '
             || ' WHERE PROJECT_ID=''' || h_project_id || '''        AND '
             || '       RESOURCE_ID IN (''' || h_shw_skill || ''')   AND '
             || '       PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1736+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1737) : cplan.sql(4182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0002 '
             || 'SELECT    RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,          MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || '          MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,     SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || '          MIN(SCHED_START) AS PRJ_SCHED_START,           MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || '          SUM(SCHED_WORK) AS SCHED_REMAINING  '
             || ' FROM TASK_ASSIGNMENTS WHERE TASK_ID=''' || h_project_id || ''' AND RESOURCE_ID IN (''' 
                  || h_shw_skill || ''') GROUP BY RESOURCE_ID ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1766+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1767) : cplan.sql(4212) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      stmt := 
            'INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,'
             || 'PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,'
             || 'PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,'
             || 'REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE) '
             || 'SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME ,''1'', 41 ,'
             || ''''' , a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,'
             || 'tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,'
             || 'tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE '
             || 'FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON p.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID '
             || 'LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=''' || h_project_id 
                  || ''' AND p.PROFILE_ID = pa.RESOURCE_ID '
             || 'JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID '
             || 'LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID '
             || 'WHERE p.PROFILE_ID IN (''' || h_shw_skill || ''')';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1802+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1803) : cplan.sql(4248) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      stmt := 'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, '
             || 'T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, rs.PROPOSED AS WORK_EFFORT, ' 
                  || TO_CHAR(Coalesce(h_eunit, 0)) || ' AS UNIT_EFFORT, '
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,  '
             || 'T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || 'FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || 'ORDER BY ELEMENT_NAME';
   End If;
   --trace_printf("%s\n", (char *)stmt);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1824+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1825) : cplan.sql(4270) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1827+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1828) : cplan.sql(4273) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(stmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1835+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1836) : cplan.sql(4281) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1847+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1848) : cplan.sql(4293) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_1;
ALTER FUNCTION SP_SEARCH_1 COMPILE DEBUG;

/*######################################################################################*/

-- Search - Assign in Affected tab (Cplan view)

/*######################################################################################*/

/*
#Stored Procedure  :SP_SEARCH_3
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_SEARCH_3 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_3
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , type001 PLS_INTEGER
   , and_roles VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_rec_status VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_pa_timephase VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_SD DATE;
   h_FD DATE;
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_res_cost NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_3') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_3'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_3(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(type001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 103+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_3(104) : cplan.sql(4427) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   
   -- Parse parameters to local variables
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := type001;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_work_type := work_type;
   
   h_task_id := h_project_id;
   h_table_type := GetTable(Coalesce(h_type, 0));
   
   --local variables
   /*int iii = 0;*/
   -- 0 = SD, 1 = SD + FD
   h_res_cost := 0.00;
   h_update := 0;
   
   If (prj_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 140+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(141) : cplan.sql(4464) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 143+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(144) : cplan.sql(4467) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 150+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(151) : cplan.sql(4474) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 153+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(154) : cplan.sql(4477) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 160+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(161) : cplan.sql(4484) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 163+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(164) : cplan.sql(4487) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If Coalesce(h_bitflag, 0) = 5 AND Coalesce(h_unit, 0) = 5 AND Coalesce(h_work, 0) = 5 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 170+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(171) : cplan.sql(4494) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 173+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(174) : cplan.sql(4497) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((start_date001 IS NOT NULL)) 
             AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(191) : cplan.sql(4514) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 197+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(198) : cplan.sql(4521) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 200+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(201) : cplan.sql(4524) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      
      iSF := 1;
   ElsIf ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 216+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(217) : cplan.sql(4540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF));
   --trace_printf("h_type: %d, h_consider: %d, h_cri: %d\n",h_type,h_consider,h_cri);
   
   If Coalesce(h_type, 0) = 0 Then
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
         INTO h_per_day, h_per_week, h_priority
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 240+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(241) : cplan.sql(4564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT 480, 2400, 0 INTO h_per_day, h_per_week, h_priority FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 254+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(255) : cplan.sql(4578) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 260+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(261) : cplan.sql(4584) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 263+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(264) : cplan.sql(4587) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
         End If;
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 280+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(281) : cplan.sql(4604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 296+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(297) : cplan.sql(4620) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 303+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(304) : cplan.sql(4627) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 306+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(307) : cplan.sql(4630) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
   Else
      If (and_roles IS NULL) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 314+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(315) : cplan.sql(4638) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 317+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(318) : cplan.sql(4641) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
      End If;
      
      h_and_roles := and_roles;
      
      --pmo_substr(h_res_id, h_and_roles, 0, 32);   //    strnpcpy(h_res_id, sizeof(h_res_id), h_and_roles[0], 32);
      --NullifyStrPos(h_res_id, 32);                //    h_res_id[32] = '\0';
      h_res_id := SubStr(SubStr(h_and_roles, 1, 32), 1, 32);
      
      --pmo_substr(h_res_type, h_and_roles, 32, 1);    //  strnpcpy(h_res_type, sizeof(h_res_type), h_and_roles[32], 1);
      --NullifyStrPos(h_res_type, 1);                  //  h_res_type[1] = '\0';
      h_res_type := PMO_STPU.strnpcpy(1, 1, 33, h_and_roles);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
            INTO h_project_id, h_per_day, h_per_week, h_priority
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 343+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(344) : cplan.sql(4667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 350+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(351) : cplan.sql(4674) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 353+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(354) : cplan.sql(4677) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 359+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(360) : cplan.sql(4683) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 362+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(363) : cplan.sql(4686) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
      End If;
   End If;
   --trace_printf("iSF: %d, h_unit: %d, h_effort: %d\n",iSF,h_unit,h_effort);
   
   If (Coalesce(contour001, 0) = 0) AND (Coalesce(h_unit, 0) <> 100) Then
      h_work := 0;
   End If;
   
   If iSF = 0 Then
      -- We only have the start date
      h_start_DT := start_date001;
      h_finish_DT := '';
      --InitializeHost(SC_PARAM(h_duration), 120);
      h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
      
      If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 385+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(386) : cplan.sql(4709) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 388+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(389) : cplan.sql(4712) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf Coalesce(h_temp_var, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 393+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(394) : cplan.sql(4717) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 396+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(397) : cplan.sql(4720) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
      End If;
   End If;
   If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
      If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
         h_eunit := 2;
      ElsIf Coalesce(h_unit, 0) = 0 Then
         h_eunit := 1;
      ElsIf (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
         h_eunit := 1;
      ElsIf Coalesce(h_unit, 0) = 5 Then
         h_work := Coalesce(h_per_week, 0) * Coalesce(h_work, 0);
         h_eunit := 3;
      End If;
   End If;
   
   --trace_printf("h_work: %d, h_duration = %d, h_effort = %d, h_start_DT = %s\n", h_work,h_duration,h_effort,h_start_DT);
   If Coalesce(h_effort, 0) > 0 Then
      If Coalesce(h_duration, 0) > 5000 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 418+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(419) : cplan.sql(4742) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_SEARCH_ASSIGN), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 421+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(422) : cplan.sql(4745) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_SEARCH_ASSIGN, h_project_id, True);
      End If;
      If Coalesce(h_duration, 0) < 30 Then
         h_duration := 50;
      Else
         h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
      End If;
   End If;
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   --Parse resource_ids list
   h_shw_role := '';
   h_shw_skill := '';
   h_dayofyear := '';
   h_assign_id := '';
   
   h_res_id := SubStr(h_res_id, 1, 32);
   --h_res_id[32] = '\0';
   h_res_type := SubStr(h_res_type, 1, 1);
   --h_res_type[1] = '\0';
   
   --trace_printf("h_type = %d, h_start_date = %s, h_res_id = %s, h_res_type: %s, h_work_type: %d\n", h_type,h_start_date,h_res_id,h_res_type,h_work_type);
   --trace_printf("h_start_DT = %s, h_finish_DT = %s\n", h_start_DT,h_finish_DT);
   If Coalesce(h_type, 0) > 0 Then
      h_SD := NULL;
      h_start_DT := h_start_date;
      If SubStr(h_res_type, 1, 1) = '1' Then
         /*BEGIN_SQL
                         SELECT CASE WHEN DATE(HOST(h_start_date)) BETWEEN DATE(BAK_START_DT) AND DATE(BAK_FINISH_DT)
                         THEN BAK_START_DT + 1 DAYS ELSE PMO_TOTIMESTAMP(HOST(h_start_date)) END
                         INTO HOST(h_start_DT) FROM TASK_ASSIGNMENTS
                         WHERE TASK_ID=HOST(h_project_id) AND RESOURCE_ID=HOST(h_res_id) AND RESOURCE_TYPE='1';
                     END_SQL;
         //trace_printf("A_PROF: h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_total,h_start_DT,h_finish_DT);
         */
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
            h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 472+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(473) : cplan.sql(4796) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 485+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(486) : cplan.sql(4809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 492+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(493) : cplan.sql(4816) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 495+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(496) : cplan.sql(4819) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         h_shw_skill := h_res_id;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 512+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(513) : cplan.sql(4836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf SubStr(h_res_type, 1, 1) = '0' Then
         BEGIN
            SELECT NICK_NAME INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 525+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(526) : cplan.sql(4849) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 532+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(533) : cplan.sql(4856) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 535+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(536) : cplan.sql(4859) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         h_shw_role := h_res_id;
         
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
            h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 555+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(556) : cplan.sql(4879) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("A_RES_0: SC_STATE = %d, h_start_DT: %d:%s, h_SD: %d:%s\n", SC_STATE,h_start_DT_Ind,h_start_DT,h_SD_Ind,h_SD);
         
         /*          BEGIN_SQL
                         SELECT CASE WHEN DATE(HOST(h_start_date)) BETWEEN DATE(BAK_START_DT) AND DATE(BAK_FINISH_DT)
                         THEN BAK_START_DT + 1 DAYS ELSE PMO_TOTIMESTAMP(HOST(h_start_date)) END, TSK_ASSIGNMENT_ID,
                         DATE(BAK_START_DT), DATE(BAK_FINISH_DT), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                         INTO HOST(h_start_DT), HOST(h_prj_assignment_id),
                         HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                         FROM TASK_ASSIGNMENTS
                         WHERE TASK_ID=HOST(h_project_id) AND RESOURCE_ID=HOST(h_res_id) AND RESOURCE_TYPE='0' AND BIT_FLAG='0';
                     END_SQL;*/
         --trace_printf("A_RES_0: SC_STATE = %d, h_start_DT: %d:%s\n", SC_STATE,h_start_DT_Ind,h_start_DT);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               REGULAR_COST, REGULAR_RATE
               INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
               h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 586+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(587) : cplan.sql(4910) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("A_RES_2: SC_STATE = %d, h_work_type = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", SC_STATE,h_work_type,h_total,h_start_DT,h_finish_DT);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '0' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 603+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(604) : cplan.sql(4927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 609+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(610) : cplan.sql(4933) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 612+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(613) : cplan.sql(4936) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      --trace_printf("***: h_shw_role = %s, h_shw_skill: %s\n", h_shw_role,h_shw_skill);
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_PARTICIPANT_GROUP
                                 , h_temp2_security_id);
         
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT =
            CASE
            WHEN SECURITY_ID = h_temp2_security_id AND ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) THEN PMO_CNST.SC_AFFECTED
            WHEN SECURITY_ID = h_temp2_security_id THEN PMO_CNST.SC_TEAMMEMBER
            WHEN SECURITY_ID = h_temp_security_id THEN PMO_CNST.SC_AFFECTED
            ELSE PMO_STPU.BIT_OR( PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
            END, SECURITY_ID =
            CASE
            WHEN SECURITY_ID = h_temp2_security_id THEN  h_temp_security_id
            ELSE SECURITY_ID
            END
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 647+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(648) : cplan.sql(4971) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
      Else
         --Get a unique ID
         h_assign_id := SYS_GUID();
         
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         
         h_temp_participant := PMO_CNST.SC_AFFECTED;
         h_temp_bitflag := 1;
         
         h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
         
         BEGIN
            INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, TABLE_TYPE,
            RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME)
            VALUES (h_assign_id, h_project_id, h_task_id, h_type, h_table_type,
            h_res_type, h_res_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 681+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(682) : cplan.sql(5005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("h_prj_assignment_id: %d:%s, h_SD: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id,h_SD_Ind,h_SD);//trace_flush();
      If (Coalesce(h_work_type, 0) = 0) AND ((h_SD IS NOT NULL)) Then
         BEGIN
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 697+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(698) : cplan.sql(5021) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);
         
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id, h_SD
                                            , h_FD, h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
      End If;
      
      h_start_DT := h_start_DT;
      
      --h_bitflag = (h_bitflag | SC_BIT_30);  // assign from constrained by project (type > 0)
      --strcpy(h_start_DT, h_start_date);
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         h_totpr := PMO_CNST.SC_SRCH_PROFILE;
      Else
         --h_totpr = 4;
         h_totpr := Coalesce(h_consider, 0);
      End If;
      --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_effort = %d\n", h_start_DT,h_finish_DT,h_effort);
      
      i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
      
      --trace_printf("h_prj_assignment_id: %s, i_return: %d\n", h_prj_assignment_id,i_return);
      If i_return = 0 Then
         BEGIN
            UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 744+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(745) : cplan.sql(5068) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      h_shw_role := '';
      h_shw_skill := '';
      
      h_and_roles := and_roles;
      --Parse resource_ids list
      If Coalesce(Length(h_and_roles), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 33);
      End If;
      
      If numItems > 0 Then
         h_count := 0;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            -- Get next skill_id
            --pmo_substr(h_res_id,   h_and_roles, h_count*33, 32);      //strncpy( h_res_id, h_and_roles[h_count*33], 32);
            h_res_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 33+1, h_and_roles);
            --pmo_substr(h_res_type, h_and_roles, h_count*33+32, 1);      //strncpy( h_res_type, h_and_roles[h_count*33+32], 1);
            h_res_type := PMO_STPU.strnpcpy(1, 1, Coalesce(h_count, 0) * 33 + 32+1, h_and_roles);
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_totpr := PMO_CNST.SC_SRCH_PROFILE;
               
               If Coalesce(Length(h_shw_skill), 0) = 0 Then
                  h_shw_skill := h_res_id;
               Else
                  h_shw_skill := SubStr(h_shw_skill || ''',''', 1, 8000);
                  h_shw_skill := SubStr(h_shw_skill || h_res_id, 1, 8000);
               End If;
            Else
               h_totpr := Coalesce(h_consider, 0);
               
               If Coalesce(Length(h_shw_role), 0) = 0 Then
                  h_shw_role := h_res_id;
               Else
                  h_shw_role := SubStr(h_shw_role || ''',''', 1, 8000);
                  h_shw_role := SubStr(h_shw_role || h_res_id, 1, 8000);
               End If;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               BEGIN
                  SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                  THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                  INTO h_start_DT FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 802+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(803) : cplan.sql(5126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_total := 0;
               h_work_type := 0;
               --trace_printf("S_PROF: h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_total,h_start_DT,h_finish_DT);
            Else
               BEGIN
                  SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                  THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                  INTO h_start_DT FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 822+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(823) : cplan.sql(5146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  BEGIN
                     SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                     THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                     INTO h_start_DT FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 839+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(840) : cplan.sql(5163) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("S_RES: h_work_type = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_work_type,h_total,h_start_DT,h_finish_DT);
            End If;
            
            h_start_DT := h_start_DT;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_totpr := PMO_CNST.SC_SRCH_PROFILE;
            Else
               h_totpr := Coalesce(h_consider, 0);
            End If;
            
            h_start_DT := h_start_date;
            
            i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
            --trace_printf("SP_SEARCH_3: i_return = %d\n",i_return);
            
            If i_return = 0 Then
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 877+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(878) : cplan.sql(5201) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_count := Coalesce(h_count, 0) + 1;
         END LOOP;
      End If;
   End If;
   --trace_printf("i_return = %d\n", i_return);
   
   If (i_return <> 0) AND (Coalesce(h_type, 0) > 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 891+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(892) : cplan.sql(5215) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_FROM_PROJECT_AFF), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 894+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(895) : cplan.sql(5218) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_FROM_PROJECT_AFF, h_assign_id
            , True);
   End If;
   
   If Coalesce(h_type, 0) > 0 Then
      h_project_id := h_task_id;
   End If;
   
   --trace_printf("SEARCH_3: h_shw_role = %s, h_shw_skill = %s\n",h_shw_role,h_shw_skill); //trace_flush();
   
   If (Coalesce(Length(h_shw_role), 0) > 0) AND (Coalesce(Length(h_shw_skill), 0) > 0) Then
      stmt := 
            'SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID, '
             || 'p.E_MAIL_ADDRESS,b.PLANNED_START,b.PLANNED_FINISH,b.BEST AS PLANNED_EFFORT, '
             || 'a.SCHED_START AS START_DATE, a.SCHED_FINISH AS FINISH_DATE, a.BEST AS WORK, a.SCHED_WORK AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,'
             || 'rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) || ' AS WORK_EFFORT, ' 
                  || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY,'
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM RESOURCE_ASSIGNMENTS t LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG=''2'' '
             || 'LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG=''0'' '
             || 'JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON t.RESOURCE_ID=rs.RES_ID '
             || 'WHERE t.ELEMENT_ID=''' || h_project_id 
                  || ''' AND t.RESOURCE_TYPE=''0'' AND t.RESOURCE_ID IN (''' || h_shw_role 
                  || ''') AND BITAND(t.PARTICIPANT, 4112) > 0 '
             || 'UNION ALL '
             || 'SELECT r.PROFILE_ID AS ELEMENT_ID, ta.RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, '
             || ''''' AS E_MAIL_ADDRESS,ta.PLANNED_START,ta.PLANNED_FINISH,ta.BEST AS PLANNED_EFFORT,'
             || PMO_CNST.STR_1899_12_31_00_00_00 || ' AS START_DATE, ' || PMO_CNST.STR_1899_12_31_00_00_00 
                  || ' AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) 
                  || ' AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) || ' AS UNIT_EFFORT,'
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY , rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM TASK_ASSIGNMENTS ta JOIN PROFILES r ON ta.TASK_ID=''' || h_project_id 
                  || ''' AND ta.RESOURCE_TYPE=''1'' AND ta.RESOURCE_ID IN (''' || h_shw_skill 
                  || ''') AND r.PROFILE_ID=ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON ta.RESOURCE_ID=rs.RES_ID '
             || 'ORDER BY TYPE_ID';
   ElsIf Coalesce(Length(h_shw_role), 0) > 0 Then
      stmt := 
            'SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID, '
             || 'p.E_MAIL_ADDRESS,b.PLANNED_START,b.PLANNED_FINISH,b.BEST AS PLANNED_EFFORT, '
             || 'a.SCHED_START AS START_DATE, a.SCHED_FINISH AS FINISH_DATE, a.BEST AS WORK, a.SCHED_WORK AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,'
             || 'rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) || ' AS WORK_EFFORT, ' 
                  || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY,'
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM RESOURCE_ASSIGNMENTS t LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG=''2'' '
             || 'LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG=''0'' '
             || 'JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON t.RESOURCE_ID=rs.RES_ID '
             || 'WHERE t.ELEMENT_ID=''' || h_project_id 
                  || ''' AND t.RESOURCE_TYPE=''0'' AND t.RESOURCE_ID IN (''' || h_shw_role 
                  || ''') AND BITAND(t.PARTICIPANT, 4112) > 0 '
             || 'ORDER BY TYPE_ID';
   ElsIf Coalesce(Length(h_shw_skill), 0) > 0 Then
      stmt := 
            'SELECT r.PROFILE_ID AS ELEMENT_ID, ta.RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, '
             || ''''' AS E_MAIL_ADDRESS,ta.PLANNED_START,ta.PLANNED_FINISH,ta.BEST AS PLANNED_EFFORT,'
             || PMO_CNST.STR_1899_12_31_00_00_00 || ' AS START_DATE, ' || PMO_CNST.STR_1899_12_31_00_00_00 
                  || ' AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) 
                  || ' AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) || ' AS UNIT_EFFORT,'
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY , rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM TASK_ASSIGNMENTS ta JOIN PROFILES r ON ta.TASK_ID=''' || h_project_id 
                  || ''' AND ta.RESOURCE_TYPE=''1'' AND ta.RESOURCE_ID IN (''' || h_shw_skill 
                  || ''') AND r.PROFILE_ID=ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON ta.RESOURCE_ID=rs.RES_ID '
             || 'ORDER BY TYPE_ID';
   End If;
   --trace_printf("%s\n", (char *)stmt);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 980+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(981) : cplan.sql(5304) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 983+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(984) : cplan.sql(5307) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(stmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 991+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(992) : cplan.sql(5315) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 1003+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(1004) : cplan.sql(5327) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_3;
ALTER FUNCTION SP_SEARCH_3 COMPILE DEBUG;

/*
#Stored Procedure  :SP_I_PROFILE
#Short Description :Assign a profile to a task or project
#Description       :used tables: RESOURCE_ASSIGNMENTS, PROFILES
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_I_PROFILE ==
CREATE OR REPLACE FUNCTION SP_I_PROFILE
(
     prj_id VARCHAR2
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , org_id001 VARCHAR2
   , geo_id001 VARCHAR2
   , pool_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_role_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   numItems PLS_INTEGER;
   numItems001 PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_PROFILE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_PROFILE'), '-- ' || SYSTIMESTAMP, ':r := SP_I_PROFILE(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || org_id001 || ''''
          || ', ' || '''' || geo_id001 || ''''
          || ', ' || '''' || pool_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_PROFILE(35) : cplan.sql(5386) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((prj_id IS NULL)) OR ((and_roles IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_PROFILE', 41+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_PROFILE(42) : cplan.sql(5393) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_PROFILE(45) : cplan.sql(5396) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --  InitializeHost(SC_PARAM(h_and_roles), and_roles, ind_i_prof->and_roles);
   h_and_roles := and_roles;
   --pmo_substr(h_role_id, h_and_roles, 0, 32);    //strncpy( h_role_id, h_and_roles[0], 32);
   --NullifyStrPos(h_role_id, 32);   //h_role_id[32] = '\0';
   h_role_id := SubStr(SubStr(h_and_roles, 1, 32), 1, 32);
   
   BEGIN
      SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
      FROM PROFILES
      WHERE PROJECT_ID=prj_id/**prj_id*/ AND ROLE_ID=h_role_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 65+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(66) : cplan.sql(5417) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')')
      INTO h_full_name FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
      WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(81) : cplan.sql(5432) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("a_SC_STATE: %d, h_full_name: %s\n", SC_STATE,h_full_name);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_I_PROFILE', 88+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_I_PROFILE(89) : cplan.sql(5440) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 91+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_PROFILE(92) : cplan.sql(5443) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_role_id, True);
   End If;
   
   --Get a unique id
   h_uniqueID := SYS_GUID();
   
   BEGIN
      INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME, POOL_ID, GEO_ID, ORG_ID)
      VALUES(h_uniqueID, prj_id, h_role_id, h_number, h_full_name, pool_id001, geo_id001, org_id001);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 112+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(113) : cplan.sql(5464) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      --Parse Roles list
      numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
      If numItems > 1 Then
         h_count := 1;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            -- Get next skill_id
            --strncpy( h_role_id, h_and_roles[h_count*32], 32);
            --pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
            h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_roles);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGN_ROLES
               SELECT h_uniqueID, ELEMENT_ID, PROFICIENCY_LEVEL_ID, PREFERENCE_LEVEL_ID,
               EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
               FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_role_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 143+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(144) : cplan.sql(5495) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            h_count := Coalesce(h_count, 0) + 1;
         END LOOP;
      End If;
      
      --Parse Skills list
      If (and_skills IS NOT NULL) Then
         --          InitializeHost(SC_PARAM(h_and_roles), and_skills, ind_i_prof->and_skills);
         h_and_roles := and_skills;
         numItems001 := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
         
         If numItems001 > 0 Then
            h_count := 0;
            WHILE Coalesce(h_count, 0) < numItems001 LOOP
               -- Get next skill_id
               --strncpy( h_role_id, h_and_roles[h_count*32], 32);
               --pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
               h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_roles);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_uniqueID, h_role_id, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID,
                  PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                  FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_role_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 180+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(181) : cplan.sql(5532) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_count := Coalesce(h_count, 0) + 1;
            END LOOP;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_PROFILE', 192+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_PROFILE(193) : cplan.sql(5544) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 199+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(200) : cplan.sql(5551) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 211+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(212) : cplan.sql(5563) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_PROFILE;
ALTER FUNCTION SP_I_PROFILE COMPILE DEBUG;

/*######################################################################################*/

/* assign a profile to a task or project, used tables: RESOURCE_ASSIGNMENTS, PROFILES   */

/****************************************************************************************/

/*  RES_ID is profile_id, role_id is role_id in TMT_COMPETENCIES    */

/****************************************************************************************/

/*
#Stored Procedure  :SP_A_TSKPROF
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_TSKPROF ==
CREATE OR REPLACE FUNCTION SP_A_TSKPROF
(
     task_id001 VARCHAR2
   , res_id001 VARCHAR2
   , role_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_role_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_full_name VARCHAR2(255);
   h_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_sc_state PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_TSKPROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_TSKPROF'), '-- ' || SYSTIMESTAMP, ':r := SP_A_TSKPROF(' || 
       '''' || task_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || role_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 37+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_TSKPROF(38) : cplan.sql(5635) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((task_id001 IS NULL)) OR ((res_id001 IS NULL)) OR ((role_id001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_TSKPROF', 44+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_TSKPROF(45) : cplan.sql(5642) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(48) : cplan.sql(5645) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   BEGIN
      SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_WBS WHERE ELEMENT_ID = task_id001;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 60+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(61) : cplan.sql(5658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_CRI WHERE ELEMENT_ID = task_id001;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 74+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(75) : cplan.sql(5672) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 81+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(82) : cplan.sql(5679) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || task_id001 || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 84+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(85) : cplan.sql(5682) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, task_id001, True);
      End If;
      h_cri := 1;
      h_table_type := PMO_CNST.SC_TABLE_CRI;
   Else
      h_cri := 0;
      h_table_type := PMO_CNST.SC_TABLE_WBS;
   End If;
   
   h_res_id := res_id001;
   
   -- if it is the profile (res_id) does not exists it will be created on the fly using res_id as role_id
   BEGIN
      SELECT PROFILE_ID, PROFILE_NAME, REC_STATUS INTO h_res_id, h_full_name, h_rec_status
      FROM PROFILES WHERE PROFILE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(108) : cplan.sql(5705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      h_and_roles := role_id001;
      
      h_role_id := SubStr(h_and_roles, 1, 32);
      
      BEGIN
         SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
         FROM PROFILES
         WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 127+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(128) : cplan.sql(5725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')') INTO h_full_name
         FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
         WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 142+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(143) : cplan.sql(5740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 149+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(150) : cplan.sql(5747) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 152+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(153) : cplan.sql(5750) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
      End If;
      
      BEGIN
         INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME)
         VALUES(h_res_id, h_project_id, h_role_id, h_number, h_full_name);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 170+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(171) : cplan.sql(5768) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 177+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(178) : cplan.sql(5775) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 180+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(181) : cplan.sql(5778) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, res_id001, True);
      End If;
   ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If isNotProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT a.RES_ASSIGNMENT_ID INTO h_uniqueID 
            FROM RESOURCE_ASSIGNMENTS a
            WHERE a.ELEMENT_ID = task_id001  
            AND a.RESOURCE_ID = res_id001
            AND a.RESOURCE_TYPE = '1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 198+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(199) : cplan.sql(5796) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 205+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(206) : cplan.sql(5803) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 208+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(209) : cplan.sql(5806) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, res_id001, True);
         End If;
      End If;
      
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         BEGIN
            UPDATE PROFILES SET REC_STATUS = 'U' WHERE PROFILE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 227+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(228) : cplan.sql(5825) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 234+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(235) : cplan.sql(5832) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 237+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(238) : cplan.sql(5835) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, res_id001, True);
   End If;
   
   h_bitflag := 0;
   BEGIN
      SELECT RES_ASSIGNMENT_ID INTO h_uniqueID 
      FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = task_id001 
      AND RESOURCE_ID = h_res_id 
      AND RESOURCE_TYPE = '1';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 255+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(256) : cplan.sql(5853) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1__* SC_STATE: %d, h_type: %d, h_cri: %d\n", SC_STATE, h_type, h_cri);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      If Coalesce(h_cri, 0) = 0 Then
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = task_id001;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 284+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(285) : cplan.sql(5882) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_cri, 0) = 1 Then
         BEGIN
            UPDATE TMT_CRI SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = task_id001;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 310+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(311) : cplan.sql(5908) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --Get a unique ID
      h_uniqueID := SYS_GUID();
      h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         h_temp_participant := PMO_CNST.SC_AFFECTED;
      Else
         h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
      End If;
      
      h_temp_res_type := '1';
      h_temp_bitflag := 1;
      
      h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_temp_res_type, h_temp_bitflag);
      
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
         RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME,TABLE_TYPE) 
         VALUES (h_uniqueID, h_project_id,task_id001, h_type, 
         '1', h_res_id, h_temp_participant, h_temp_security_id, rec_user001, SYSTIMESTAMP,h_table_type);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 348+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(349) : cplan.sql(5946) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* h_temp_participant: %d, h_temp_security_id: %s\n", h_temp_participant,h_temp_security_id);
   ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
         WHERE RES_ASSIGNMENT_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 367+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(368) : cplan.sql(5965) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_uniqueID);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_TSKPROF', 376+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_TSKPROF(377) : cplan.sql(5974) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 383+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(384) : cplan.sql(5981) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 395+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(396) : cplan.sql(5993) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_TSKPROF;
ALTER FUNCTION SP_A_TSKPROF COMPILE DEBUG;

/***************************************************************************************************/

/*
#Stored Procedure  :SP_A_TSKRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_TSKRES ==
CREATE OR REPLACE FUNCTION SP_A_TSKRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_TSKRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_TSKRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_TSKRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKRES', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_TSKRES(26) : cplan.sql(6045) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_TSKRES', 30+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_TSKRES(31) : cplan.sql(6050) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKRES', 33+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKRES(34) : cplan.sql(6053) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   
   BEGIN
      SELECT TYPE_ID, PROJECT_ID INTO h_type_id, h_project_id
      FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(55) : cplan.sql(6074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_TSKRES', 65+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_TSKRES(66) : cplan.sql(6085) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 72+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(73) : cplan.sql(6092) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 84+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(85) : cplan.sql(6104) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_TSKRES;
ALTER FUNCTION SP_A_TSKRES COMPILE DEBUG;

/*
#Stored Procedure  :SP_A_CRIRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_CRIRES ==
CREATE OR REPLACE FUNCTION SP_A_CRIRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_CRIRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_CRIRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_CRIRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CRIRES', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_CRIRES(25) : cplan.sql(6153) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_CRIRES', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_CRIRES(30) : cplan.sql(6158) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CRIRES', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_CRIRES(33) : cplan.sql(6161) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(52) : cplan.sql(6180) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("WBS_SC_STATE: %d, h_type_id: %d, h_element_id: %s\n", SC_STATE,h_type_id,h_element_id);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_CRI WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(67) : cplan.sql(6195) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("CRI_SC_STATE: %d, h_type_id: %d, h_element_id: %s\n", SC_STATE,h_type_id,h_element_id);
      h_table_type := PMO_CNST.SC_TABLE_CRI;
   Else
      h_table_type := PMO_CNST.SC_TABLE_WBS;
   End If;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_CRIRES', 82+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_CRIRES(83) : cplan.sql(6211) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 89+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(90) : cplan.sql(6218) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 101+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(102) : cplan.sql(6230) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_CRIRES;
ALTER FUNCTION SP_A_CRIRES COMPILE DEBUG;

/***************************************************************************************************/

/*
#Stored Procedure  :SP_A_REQRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_REQRES ==
CREATE OR REPLACE FUNCTION SP_A_REQRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_REQRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_REQRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_REQRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_REQRES', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_REQRES(24) : cplan.sql(6280) : ' || SYSTIMESTAMP);
   END IF; 

   If (id IS NULL) OR (res_id001 IS NULL) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_REQRES', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_REQRES(29) : cplan.sql(6285) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_REQRES', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_REQRES(32) : cplan.sql(6288) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 50+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(51) : cplan.sql(6307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- commented by paria because we don't have tmt_requirement table anymore 11/18/2002
   
   h_ra_id := SYS_GUID();
   
   --  SC_STATE = AssignResource( SC_PARAM(h_type_id), SC_PARAM(h_element_id), SC_PARAM(h_resource_id), SC_PARAM(h_rec_user), SC_PARAM(h_ra_id));
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_REQRES', 62+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_REQRES(63) : cplan.sql(6319) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 69+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(70) : cplan.sql(6326) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 81+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(82) : cplan.sql(6338) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_REQRES;
ALTER FUNCTION SP_A_REQRES COMPILE DEBUG;

/*
#Stored Procedure  :SP_A_POOLRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_POOLRES ==
CREATE OR REPLACE FUNCTION SP_A_POOLRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_POOLRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_POOLRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_POOLRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_POOLRES', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_POOLRES(25) : cplan.sql(6387) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_POOLRES', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_POOLRES(30) : cplan.sql(6392) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_POOLRES', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_POOLRES(33) : cplan.sql(6395) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := PMO_CNST.SC_TABLE_POL;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_POOLS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(53) : cplan.sql(6415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_POOLRES', 63+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_POOLRES(64) : cplan.sql(6426) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 70+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(71) : cplan.sql(6433) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 82+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(83) : cplan.sql(6445) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_POOLRES;
ALTER FUNCTION SP_A_POOLRES COMPILE DEBUG;

/***************************************************************************************************/

/*
#Stored Procedure  :SP_A_DOCRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_DOCRES ==
CREATE OR REPLACE FUNCTION SP_A_DOCRES
(
     table_type001 PLS_INTEGER
   , id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_DOCRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_DOCRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_DOCRES(' || 
       COALESCE(TO_CHAR(table_type001), 'NULL')
          || ', ' || '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_DOCRES', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_DOCRES(27) : cplan.sql(6498) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_DOCRES', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_DOCRES(32) : cplan.sql(6503) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_DOCRES', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_DOCRES(35) : cplan.sql(6506) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := table_type001;
   
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(56) : cplan.sql(6527) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SCD Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 68+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(69) : cplan.sql(6540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(82) : cplan.sql(6553) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(95) : cplan.sql(6566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_CLIENT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 107+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(108) : cplan.sql(6579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_ASSET_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 120+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(121) : cplan.sql(6592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_VENDOR_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 133+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(134) : cplan.sql(6605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_DOCRES', 145+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_DOCRES(146) : cplan.sql(6617) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 152+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(153) : cplan.sql(6624) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 164+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(165) : cplan.sql(6636) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_DOCRES;
ALTER FUNCTION SP_A_DOCRES COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_U_CREATETMPTABLE ==
CREATE OR REPLACE FUNCTION FC_U_CREATETMPTABLE RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_CREATETMPTABLE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_CREATETMPTABLE'), '-- ' || SYSTIMESTAMP, ':r := FC_U_CREATETMPTABLE();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_CREATETMPTABLE', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_CREATETMPTABLE(9) : cplan.sql(6661) : ' || SYSTIMESTAMP);
   END IF; 

   --define temporary table for distinct selection of records in case the select list contains CLOB type of data
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN01_TEMP';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_CREATETMPTABLE', 17+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_CREATETMPTABLE(18) : cplan.sql(6670) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_CREATETMPTABLE;
ALTER FUNCTION FC_U_CREATETMPTABLE COMPILE DEBUG;

/************************************************************************************************/

/*****************************************************************************************/

Prompt == FUNCTION FC_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION FC_U_TSKRES2
(
     h_assign_id IN OUT NOCOPY VARCHAR2
   , h_and_roles IN OUT NOCOPY VARCHAR2
   , h_and_skills IN OUT NOCOPY VARCHAR2
   , sh_h_work_location PLS_INTEGER
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_contour_type PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , h_assign_string IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_work_location PLS_INTEGER := sh_h_work_location;
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_contour_type PLS_INTEGER := sh_h_contour_type;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_last_submitted DATE;
   h_tsk_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_calendar_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_full_name VARCHAR2(255);
   h_null_str VARCHAR2(1);
   h_min_per_day PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp2_security_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_zero PLS_INTEGER := 0;
   h_res_cost NUMBER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TSKRES2(' || 
       '''' || h_assign_id || ''''
          || ', ' || '''' || h_and_roles || ''''
          || ', ' || '''' || h_and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_location), 'NULL')
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || '''' || h_assign_string || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 94+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TSKRES2(95) : cplan.sql(6777) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
      h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_C0000000)), 30);
      -- consider CRI  WBS
      h_consider := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_1FFFFFFF);
      h_bitflag := 0;
      h_rollup := 0;
   ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
      h_rollup := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
      -- set do not rollup (import from MSP)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
      -- remove set do not rollup
      h_consider := 0;
   Else
      h_consider := Coalesce(h_bitflag, 0);
      h_bitflag := 0;
      h_rollup := 0;
   End If;
   
   h_duration := 0;
   h_res_type := '0';
   h_work_todate := 0;
   h_last_submitted := NULL;
   h_SD := NULL;
   h_FD := NULL;
   h_null_str := NULL;
   h_res_cost := 0.00;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      h_tsk_assignment_id := h_assign_id;
      BEGIN
         SELECT
         PROJECT_ID,
         TASK_ID,
         RESOURCE_ID,
         RESOURCE_TYPE,
         LAST_SUBMITTED,
         WORK_TODATE,
         TO_DATE(SubStr(SCHED_START, 1, 10)),
         TO_DATE(SubStr(SCHED_FINISH, 1, 10)),
         BILLABLE,
         CAPITAL_EXPENSE,
         REGULAR_COST,
         REGULAR_RATE
         INTO
         h_project_id,
         h_task_id,
         h_res_id,
         h_res_type,
         h_last_submitted,
         h_work_todate,
         h_SD,
         h_FD,
         h_billable,
         h_capital_expense,
         h_reg_cost,
         h_reg_sale
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 163+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(164) : cplan.sql(6846) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 170+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(171) : cplan.sql(6853) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
   Else
      BEGIN
         SELECT PROJECT_ID, ELEMENT_ID, RESOURCE_ID, ELEMENT_TYPE, 
         RESOURCE_TYPE, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
         INTO h_project_id, h_task_id, h_res_id, h_type,
         h_res_type, h_security_id, h_transfed_cri
         FROM RESOURCE_ASSIGNMENTS 
         WHERE RES_ASSIGNMENT_ID = h_assign_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(189) : cplan.sql(6871) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 195+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(196) : cplan.sql(6878) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      ElsIf isWBSTransferedDeliv(Coalesce(h_type, 0)) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 200+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(201) : cplan.sql(6883) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
      
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS SET WORK_LOCATION = h_work_location
         WHERE RES_ASSIGNMENT_ID = h_assign_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 218+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(219) : cplan.sql(6901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If NOT (isProjectType(Coalesce(h_type, 0))) <> 0 Then
         BEGIN
            SELECT TSK_ASSIGNMENT_ID INTO h_tsk_assignment_id
            FROM TASK_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 233+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(234) : cplan.sql(6916) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 240+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(241) : cplan.sql(6923) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_ALREADY_ASSIGNED;
         End If;
      End If;
      
      h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
   End If;
   
   /********************************************************************************************************************************/
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), CALENDAR_ID, PRIORITY
      INTO h_min_per_day, h_per_week, h_calendar_id, h_priority
      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 260+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(261) : cplan.sql(6943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_rollup, 0) = 0 Then
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 5) Then
         h_work := Coalesce(h_min_per_day, 0) * Coalesce(h_work, 0);
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         --if ( IsNullSpParam(ind_u_assign, finish_date))
         If (h_finish_date IS NULL) Then
            If Coalesce(h_work, 0) > 0 Then
               h_effort := Coalesce(h_occurrence, 0) * Coalesce(h_work, 0);
            End If;
         Else
            BEGIN
               SELECT SUM(AVAILABLE_MIN) INTO h_effort FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID=h_project_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 286+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(287) : cplan.sql(6969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_eunit, 0) = 1 Then
            h_effort := Coalesce(h_effort, 0) * 60;
         ElsIf Coalesce(h_eunit, 0) = 2 Then
            h_effort := Coalesce(h_min_per_day, 0) * Coalesce(h_effort, 0);
         ElsIf Coalesce(h_eunit, 0) = 3 Then
            h_effort := Coalesce(h_per_week, 0) * Coalesce(h_effort, 0);
         End If;
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 303+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(304) : cplan.sql(6986) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
   End If;
   
   If Coalesce(h_bitflag, 0) = 2 Then
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove assign from constrained by project (type > 0)
   ElsIf (SubStr(h_res_type, 1, 1) = '0') AND (Coalesce(h_work_type, 0) = 0) Then
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, UNRESOLVED_START, UNRESOLVED_FINISH, PLANNED_WORK+SCHED_WORK, BIT_FLAG
         INTO h_tsk_assignment_id, h_start_DT, h_finish_DT, h_sched_work, h_bit_flag
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 324+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(325) : cplan.sql(7007) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (h_tsk_assignment_id IS NOT NULL) Then
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
         -- remove assign from constrained by project (type > 0)
         If (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT
               , h_finish_date, 10) >= 0) Then
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               If Coalesce(h_rollup, 0) = 0 Then
                  h_bitflag := 2;
               End If;
               h_work_type := 2;
            End If;
         Else
            If Coalesce(h_bitflag, 0) = 1 Then
               h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
            End If;
            
            If Coalesce(h_sched_work, 0) < Coalesce(h_effort, 0) Then
               If Coalesce(h_sched_work, 0) > 0 AND PMO_ORASTD.strncmp(h_start_date, h_finish_DT, 10) >= 0 Then
                  If SubStr(h_bit_flag, 1, 1) = '0' Then
                     h_update := 3;
                  End If;
                  h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
                  h_consider := PMO_CNST.SC_SRCH_FROM_PROJECT;
                  h_temp_sc_state := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                               , h_and_roles, h_and_skills, h_null_str, h_null_str, h_start_date
                               , h_finish_DT, h_zero, h_sched_work, h_work, h_eunit, h_unit, h_zero
                               , h_consider, h_res_type, h_work_type, h_bitflag, h_contour
                               , h_occurrence, h_update, h_min_per_day, h_priority, h_rec_user
                               , Coalesce(h_min_per_day, 0), 0, 0.00, 0.00);
               End If;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET ETC = h_effort
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 373+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(374) : cplan.sql(7056) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 379+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(380) : cplan.sql(7062) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            End If;
         End If;
      End If;
   End If;
   
   If (Coalesce(h_bitflag, 0) = 0) OR (Coalesce(h_occurrence, 0) = Coalesce(h_work_todate, 0)) Then
      If Coalesce(h_rollup, 0) <> 0 Then
         h_bitflag := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), Coalesce(h_rollup, 0)));
         -- set do not rollup
      End If;
      
      h_temp_sc_state := U_TSKRES2(h_assign_id, h_project_id, h_task_id, h_res_id, h_and_roles
                             , h_and_skills, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                             , h_unit, h_res_type, h_work_type, h_bitflag, h_contour, h_contour_type
                             , h_occurrence, h_consider, h_assign_string, h_rec_user, h_min_per_day
                             , 0.00, 0.00);
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      If Coalesce(h_rollup, 0) <> 0 Then
         h_bitflag := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), Coalesce(h_rollup, 0)));
         -- set do not rollup
      End If;
      /*
              if ( (IsNotNullParam(h_SD))  (IsNotNullParam(h_FD)))
              {
                  InitializeHost(SC_PARAM(h_tph_delete), -1);
                  h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                              SC_PARAM(h_project_id),
                              SC_PARAM(h_task_id),
                              SC_PARAM(h_tsk_assignment_id),
                              SC_PARAM(h_SD),
                              SC_PARAM(h_FD),
                              SC_PARAM(h_reg_cost),
                              SC_PARAM(h_reg_sale),
                              SC_PARAM(h_tph_delete),
                              SC_PARAM(h_capital_expense),
                              SC_PARAM(h_billable));
              }
      
              h_temp_sc_state = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
      
              // Tudor * fix for -803
              Nullify(SC_PARAM(h_dayofyear));
              BEGIN_SQL
                  SELECT MAX(DAY_YEAR) INTO HOST(h_dayofyear) FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND ACTUAL_FLAG = 1;
              END_SQL_NOCHECK;
      //trace_printf("***: h_last_submitted = %s, h_dayofyear: %d:%s\n",h_last_submitted,h_dayofyear_Ind,h_dayofyear);
      
              if ( (IsNotNullParam(h_dayofyear))  (strncmp(h_last_submitted, h_dayofyear, 10) != 0) )
              {
                  BEGIN_SQL
                      UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = HOST(h_dayofyear)
                      WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
                  END_SQL;
      //trace_printf("***: SC_STATE = %d, h_dayofyear: %d:%s\n",SC_STATE,h_dayofyear_Ind,h_dayofyear);
      
                  BEGIN_SQL
                      SELECT TIMESTAMP(DATE(HOST(h_dayofyear)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
                  END_SQL;
              }
              else
              {
                  BEGIN_SQL
                      SELECT TIMESTAMP(DATE(HOST(h_last_submitted)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
                  END_SQL;
              }
      //trace_printf("FC_RESCHEDULE: h_start_date = %s, h_effort: %d\n",h_start_date,h_effort);
      
              InitializeHost(SC_PARAM(h_effort), h_effort - h_work_todate);
              InitializeHost(SC_PARAM(h_temp_var), 2);
              h_temp_sc_state = FC_RESCHEDULE(SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_start_date), SC_PARAM(h_effort), SC_PARAM(h_temp_var), SC_PARAM(h_contour), SC_PARAM(h_res_id));
      */
      h_temp_sc_state := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_and_roles
                             , h_and_skills, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                             , h_unit, h_res_type, h_work_type, h_bitflag, h_contour, h_contour_type
                             , h_occurrence, h_consider, h_assign_string, h_rec_user, h_min_per_day
                             , 0.00, 0.00);
   End If;
   --trace_printf("U_TSKRES2: SC_STATE = %d, h_assign_id = %s\n",SC_STATE,h_assign_id);
   
   If Coalesce(h_temp_sc_state, 0) = 110 Then
      PMO_DB2M.SC_STATE := PMO_CNST.RESOURCE_UNAVAILABLE;
   ElsIf Coalesce(h_temp_sc_state, 0) <> 0 AND Coalesce(h_temp_sc_state, 0) <> 100 Then
      PMO_DB2M.SC_STATE := PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 473+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(474) : cplan.sql(7156) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TSKRES2;
ALTER FUNCTION FC_U_TSKRES2 COMPILE DEBUG;

/*
#Stored Procedure  :SP_U_TSKRES2
#Short Description :Update resource/profile assignment
#Description       :Gets call from FC_SEARCH_2
#Table Name        :
#Parameters        :
 $$if bitflag  = 0 then insert new assignment (assign_id = RES_ASSIGNMENT_ID)
               = 1 then update an existing assignment (assign_id = TSK_ASSIGNMENT_ID)
               = 2 then insert new assignment using project assignment (assign_id = RES_ASSIGNMENT_ID)
               = 3 then reject a profile replacement, uncommit everything
               = 9 MSP
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION SP_U_TSKRES2
(
     assign_id VARCHAR2
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , work_location001 PLS_INTEGER
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , work_type PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , contour001 PLS_INTEGER
   , contour_type001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , assign_string001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_assign_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_and_skills VARCHAR2(8000);
   h_work_location PLS_INTEGER := 0;
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_effort PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_rec_user VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := SP_U_TSKRES2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(work_location001), 'NULL')
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour_type001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || '''' || assign_string001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TSKRES2', 64+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_TSKRES2(65) : cplan.sql(7244) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((assign_id IS NULL)) OR ((start_date001 IS NULL)) OR ((finish_date001 IS NULL)) 
         OR ((assign_string001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_TSKRES2', 72+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_TSKRES2(73) : cplan.sql(7252) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TSKRES2', 75+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_TSKRES2(76) : cplan.sql(7255) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_assign_id := assign_id;
   h_and_roles := and_roles;
   h_and_skills := and_skills;
   h_start_date := start_date001;
   h_finish_date := finish_date001;
   h_assign_string := assign_string001;
   h_rec_user := rec_user001;
   
   h_work_location := work_location001;
   h_effort := effort001;
   h_work := work001;
   h_eunit := eunit;
   h_unit := unit;
   h_work_type := work_type;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_contour_type := contour_type001;
   h_occurrence := occurrence;
   
   PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_assign_id, h_and_roles, h_and_skills, h_work_location
                                  , h_start_date, h_finish_date, h_effort, h_work, h_eunit, h_unit
                                  , h_work_type, h_bitflag, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_TSKRES2', 105+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_TSKRES2(106) : cplan.sql(7285) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_assign_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TSKRES2', 113+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TSKRES2(114) : cplan.sql(7293) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TSKRES2', 125+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TSKRES2(126) : cplan.sql(7305) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_TSKRES2;
ALTER FUNCTION SP_U_TSKRES2 COMPILE DEBUG;

/*
#Stored Procedure  :SP_ASSIGN_MSP
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_ASSIGN_MSP ==
CREATE OR REPLACE FUNCTION SP_ASSIGN_MSP
(
     id VARCHAR2
   , action_type PLS_INTEGER
   , task_id001 VARCHAR2
   , start_dt001 VARCHAR2
   , finish_dt001 VARCHAR2
   , effort001 PLS_INTEGER
   , assign_string001 VARCHAR2
   , work_todate001 PLS_INTEGER
   , percent_complete001 PLS_INTEGER
   , eetc PLS_INTEGER
   , data_flag PLS_INTEGER
   , units PLS_INTEGER
   , contour_type001 PLS_INTEGER
   , baseline_start001 VARCHAR2
   , baseline_finish001 VARCHAR2
   , baseline_duration001 PLS_INTEGER
   , baseline_work001 PLS_INTEGER
   , actual_start001 VARCHAR2
   , actual_finish001 VARCHAR2
   , actual_effort001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_name VARCHAR2(255);
   h_full_name VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_subject VARCHAR2(2048);
   h_revision VARCHAR2(16);
   h_temp_id VARCHAR2(32);
   h_temp_id2 VARCHAR2(32);
   h_wkf_def_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_work_location PLS_INTEGER := 0;
   h_zero PLS_INTEGER := 0;
   h_two PLS_INTEGER := 2;
   h_work PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_units PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 480;
   h_impex_rollup PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_data_flag PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_finish_D DATE;
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_null_str VARCHAR2(1);
   h_start_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_actual_start TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_rec_DT TIMESTAMP;
   h_lock_DT TIMESTAMP;
   h_assign_string VARCHAR2(8192);
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_action_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_H_WBS IS
         SELECT * FROM (SELECT ELEMENT_NAME,RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR CAF IS
         SELECT SCHED_MIN, DAY_YEAR, ACTUAL_FLAG
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         FOR UPDATE OF ACTUAL_MIN,ACTUAL_FLAG;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ASSIGN_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ASSIGN_MSP'), '-- ' || SYSTIMESTAMP, ':r := SP_ASSIGN_MSP(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(action_type), 'NULL')
          || ', ' || '''' || task_id001 || ''''
          || ', ' || '''' || start_dt001 || ''''
          || ', ' || '''' || finish_dt001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || '''' || assign_string001 || ''''
          || ', ' || COALESCE(TO_CHAR(work_todate001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(percent_complete001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eetc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(data_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour_type001), 'NULL')
          || ', ' || '''' || baseline_start001 || ''''
          || ', ' || '''' || baseline_finish001 || ''''
          || ', ' || COALESCE(TO_CHAR(baseline_duration001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(baseline_work001), 'NULL')
          || ', ' || '''' || actual_start001 || ''''
          || ', ' || '''' || actual_finish001 || ''''
          || ', ' || COALESCE(TO_CHAR(actual_effort001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 123+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ASSIGN_MSP(124) : cplan.sql(7453) : ' || SYSTIMESTAMP);
   END IF; 

   If ((action_type IS NULL)) OR ((Coalesce(action_type, 0) < 1) OR (Coalesce(
         action_type, 0) > 3)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 129+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(130) : cplan.sql(7459) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 132+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(133) : cplan.sql(7462) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((id IS NULL)) OR ((data_flag IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 139+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(140) : cplan.sql(7469) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 142+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(143) : cplan.sql(7472) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_actual_start := NULL;
   h_rec_DT := NULL;
   h_work_location := 265;
   h_start_DT := start_dt001;
   h_finish_DT := finish_dt001;
   h_assign_string := assign_string001;
   h_rec_user := rec_user001;
   h_effort := effort001;
   h_data_flag := data_flag;
   h_action_type := action_type;
   h_units := units;
   h_actual_start := actual_start001;
   h_work_todate := work_todate001;
   h_occurrence := 0;
   --trace_printf("h_effort: %d, h_work_todate = %d, h_action_type: %d, data_flag = %d, h_actual_start: %d:%s\n",h_effort,h_work_todate,h_action_type,h_data_flag,h_actual_start_Ind,h_actual_start);
   --trace_printf("h_start_DT: %d:%s, h_finish_DT: %d:%s, h_assign_string: %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_assign_string);
   
   If Coalesce(Length(h_assign_string), 0) < PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
      If (Coalesce(h_action_type, 0) = 1) OR (Coalesce(h_action_type, 0) = 3) Then
         -- insert assignment
         If (task_id001 IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 171+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(172) : cplan.sql(7501) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 174+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(175) : cplan.sql(7504) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         h_resource_id := id;
         h_task_id := task_id001;
         h_rec_user := rec_user001;
         
         BEGIN
            SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type_id
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 192+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(193) : cplan.sql(7522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_ra_id := SYS_GUID();
         h_table_type := GetTable(Coalesce(h_type_id, 0));
         
         PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_task_id, h_resource_id, h_rec_user
                                    , h_ra_id);
         --trace_printf("1_3***** SC_STATE = %d,h_ra_id = %d;%s,h_resource_id: %s\n",SC_STATE, h_ra_id_Ind, h_ra_id,h_resource_id);
         
         If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(h_action_type, 0) = 3) AND (Coalesce(h_effort, 0) = 0) Then
            h_temp_var := 0;
            BEGIN
               SELECT COUNT('X') INTO h_temp_var FROM DUAL
               WHERE EXISTS (SELECT 'X' FROM PROFILES
               WHERE PROJECT_ID = h_project_id AND PROFILE_ID = h_resource_id);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 216+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(217) : cplan.sql(7546) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) <> 0 Then
               h_action_type := 1;
               h_effort := 5;
               h_assign_string := '1*5';
               
               If (h_start_DT IS NOT NULL) Then
                  BEGIN
                     SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT)) + (5 / 1440.0) INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 235+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(236) : cplan.sql(7565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1_* SC_STATE = %d, h_finish_DT = %d;%s\n",SC_STATE, h_finish_DT_Ind, h_finish_DT);
               End If;
            End If;
         End If;
         
         If (Coalesce(h_action_type, 0) = 1) AND (isWBSDeliv(Coalesce(h_type_id, 0))) <> 0 Then
            h_action_type := 3;
         End If;
      Else
         h_ra_id := id;
         
         BEGIN
            SELECT PROJECT_ID, TASK_ID, RESOURCE_ID, WORK_TODATE, PROJECT_ASSIGNMENT_ID,
            UNITS_MSP, C_EFFORT, C_CONTOUR, CONTOUR_TYPE, REC_DATETIME
            INTO h_project_id, h_task_id, h_resource_id, h_work_todate, h_prj_assignment_id,
            h_percent_units, h_work, h_contour, h_contour_type, h_rec_DT
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_ra_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 262+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(263) : cplan.sql(7592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("***** SC_STATE = %d, h_work = %d, h_contour_type: %d, h_percent_units: %d, h_prj_assignment_id: %d %s\n",SC_STATE, h_work,h_contour_type,h_percent_units,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(work_todate001, 0) <> Coalesce(
               h_work_todate, 0)) Then
            h_actual_start := NULL;
            h_count := -1;
         End If;
      End If;
      
      --trace_printf("h_action_type: %d, h_start_DT: %d:%s, h_finish_DT: %d:%s, h_assign_string: %s\n",h_action_type,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_assign_string);
      If Coalesce(h_action_type, 0) <> 3 Then
         If PMO_DB2M.SC_STATE <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 279+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(280) : cplan.sql(7609) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 282+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(283) : cplan.sql(7612) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_ra_id, True);
         End If;
         --trace_printf("***** h_ra_id = %s, h_effort: %d, h_assign_string: %s\n",h_ra_id,h_effort,h_assign_string);
         
         If (h_start_DT IS NULL) OR (h_finish_DT IS NULL) OR (h_effort IS NULL) 
               OR (h_assign_string IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 291+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(292) : cplan.sql(7621) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 294+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(295) : cplan.sql(7624) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         h_impex_rollup := PMO_CNST.SC_BIT_28;
         --28TH BIT ON -- DON'T ROLLUP
         
         If Coalesce(h_units, 0) > 103079040 Then
            h_units := 103079040;
         End If;
         
         --if ( (h_percent_units == 0) || (h_percent_units == 10000) )
         --      {
         
         BEGIN
            SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 317+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(318) : cplan.sql(7647) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_work, 0) > 0 Then
            --              h_temp_var = int((h_units * h_per_day)/500.0 + 0.5) * 5;    // round up to 5 minutes
            h_temp_var := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.4);
            --trace_printf("MSP___* h_temp_var: %d, h_units: %d\n",h_temp_var,h_units);
            If Coalesce(h_units, 0) <> Coalesce(h_temp_var, 0) Then
               h_work := 0;
               h_contour := 0;
            Else
               h_units := 0;
            End If;
         End If;
         --      }
         /*
                 else if ( h_percent_units != h_units )
                 {
                     h_work = 0;
                     h_contour = 0;
                     h_contour_type = 0;
                 }*/
         --trace_printf("MSP___* h_temp_var: %d, h_units: %d, h_work: %d, h_contour_type: %d, h_contour: %d, h_effort: %d\n",h_temp_var,h_units,h_work,h_contour_type,h_contour,h_effort);
         
         /*********************************************************************************
             data_flag ___OLD
             *********************************************************************************
             bit 0:  0 = plane,      1 = schedule
             bit 1:  0 = nothing,    1 = update baseline
             bit 2:  0 = nothing,    1 = update actual
             bit 3:  0 = task,       1 = non task (ST, Deliverable, Project, ...)
             *********************************************************************************
             data_flag ___NEW
             *********************************************************************************
             bit 1(1):   0 - plan
                         1 - sched
             bit 4(16):  0 = reimport,   ( no baseline, no actual: if there is actual ignore it )
                         1 = cold import ( check for baseline and actual, update actual)
             **********************************************************************************/
         
         h_null_str := NULL;
         
         If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
            If Coalesce(h_action_type, 0) = 1 Then
               If Coalesce(work_todate001, 0) > 0 Then
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_two
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("1___________SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                        WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND RESOURCE_ID = h_resource_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 385+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(386) : cplan.sql(7715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_zero
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("2___________SC_STATE = %d\n",SC_STATE);
               End If;
            Else
               h_impex_rollup := Coalesce(h_impex_rollup, 0) + 1;
               If Coalesce(work_todate001, 0) > 0 Then
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_two
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("3___________SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 421+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(422) : cplan.sql(7751) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_zero
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("4___________SC_STATE = %d\n",SC_STATE);
               End If;
            End If;
            
            If (PMO_DB2M.SC_STATE = 0) AND ((BITAND(Coalesce(data_flag, 0), 2)) = 2) Then
               If (Coalesce(work_todate001, 0) = Coalesce(effort001, 0)) AND (Coalesce(eetc, 0) = 0) AND (Coalesce(
                                           percent_complete001, 0) = 10000) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                     UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                     ACTUAL_START = h_start_DT, ACTUAL_FINISH = h_finish_DT,
                     WORK_TODATE = work_todate001,
                     ETC = eetc
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 456+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(457) : cplan.sql(7786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 474+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(475) : cplan.sql(7804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_DT
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 492+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(493) : cplan.sql(7822) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 511+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(512) : cplan.sql(7841) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         Else
            h_data_flag := BITAND(Coalesce(h_data_flag, 0), PMO_CNST.SC_NOT_32);
            --      h_impex_rollup = 536870912; //29TH BIT  -- ROLLUP
            If Coalesce(h_action_type, 0) = 1 Then
               PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_data_flag
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
               --trace_printf("5___________SC_STATE = %d, h_impex_rollup: %d\n",SC_STATE,h_impex_rollup);
               
               If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(data_flag, 0) = 2) AND (Coalesce(h_count, 0) = 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND RESOURCE_ID = h_resource_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 542+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(543) : cplan.sql(7872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If (baseline_start001 IS NOT NULL) Then
                  h_lock_DT := baseline_start001;
                  /*BEGIN_SQL
                                          SELECT PMO_TOTIMESTAMP(HOST(h_lock_DT)) INTO HOST(h_lock_DT) FROM DUAL;
                                      END_SQL;*/
                  
                  BEGIN
                     SELECT SUBSTR(h_lock_DT,1,10) || '-' || SUBSTR(h_lock_DT,12,2) || '.' || SUBSTR(h_lock_DT,15,2) || '.' || SUBSTR(h_lock_DT,18,9) INTO h_lock_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 562+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(563) : cplan.sql(7892) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_ORASTD.strncmp(h_lock_DT, h_rec_DT, 19) < 0 Then
                     h_type_id := PMO_CNST.SC_TASK;
                     h_table_type := 1;
                     
                     BEGIN
                        SELECT ELEMENT_ID INTO h_wkf_def_id
                        FROM TMT_WORKFLOW_CYCLES
                        WHERE PARENT_ID = 'NOTIFICATION';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 581+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(582) : cplan.sql(7911) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_task_id, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_uniqueID);
                     
                     If Coalesce(h_temp_sc_state, 0) = 0 Then
                        h_temp_id := NULL;
                        h_temp_id2 := NULL;
                        h_element_name := NULL;
                        h_revision := NULL;
                        h_full_name := NULL;
                        
                        OPEN C_H_WBS;
                        
                        FETCH C_H_WBS INTO h_element_name, h_revision;
                        IF C_H_WBS%FOUND THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                        
                        BEGIN
                           CLOSE C_H_WBS ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 613+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(614) : cplan.sql(7943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        BEGIN
                           SELECT FULL_NAME INTO h_full_name FROM RESOURCES
                           WHERE RESOURCE_ID = h_resource_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 627+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(628) : cplan.sql(7957) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        --strcpy(h_subject,"Import of assignment failed.");
                        h_subject := 'Import of assignment failed.';
                        --strcpy(h_message,"Import cannot modify assignment for resource: ");
                        h_message := 'Import cannot modify assignment for resource: ';
                        h_message := SubStr(h_message || h_full_name, 1, 16384);
                        h_message := SubStr(h_message || ' on the task:', 1, 16384);
                        h_message := SubStr(h_message || h_element_name, 1, 16384);
                        h_message := SubStr(h_message || 
                                 '. Assignment was modified in Timesheets during the time of export of the project.'
                                 , 1, 16384);
                        
                        h_temp_sc_state := FC_U_WORKFLOW(h_uniqueID, h_revision, h_zero, h_zero, h_null_str, h_zero
                                 , h_subject, h_message, h_rec_user, h_null_str, h_null_str, h_zero
                                 , h_temp_id, h_temp_id2, h_rec_user);
                        
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 648+1) THEN 
                           LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(649) : cplan.sql(7978) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
                        END IF; 
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 651+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(652) : cplan.sql(7981) : ' || SYSTIMESTAMP);
                        END IF; 
                        PMODEFINES.PMO_RETURN_ERROR_SP(0, h_ra_id, True);
                     End If;
                  End If;
               End If;
               
               h_impex_rollup := Coalesce(h_impex_rollup, 0) + 1;
               PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_data_flag
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
               --trace_printf("6___________SC_STATE = %d, h_ra_id: %s\n",SC_STATE,h_ra_id);
               
               --              if( (SC_STATE == 0)  (*data_flag == 2)  (h_count == 0)  IsNullParam(h_actual_start)  (*actual_effort == 0)  (*work_todate == 0) )
               If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(data_flag, 0) = 2) AND (Coalesce(h_count, 0) = 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 681+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(682) : cplan.sql(8011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
         --trace_printf("SC_STATE = %d, h_count: %d, action_type = %d, data_flag = %d, h_actual_start: %d:%s\n",SC_STATE,h_count,h_action_type,*data_flag,h_actual_start_Ind,h_actual_start);
         
         If PMO_DB2M.SC_STATE <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 692+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(693) : cplan.sql(8022) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 695+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(696) : cplan.sql(8025) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_ra_id, True);
         End If;
         
         h_contour_type := contour_type001;
         If (Coalesce(h_units, 0) <> 10000) AND (Coalesce(h_contour_type, 0) = 0) Then
            h_contour_type := 1;
         End If;
         
         If Coalesce(h_action_type, 0) = 1 Then
            If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
               If (baseline_start001 IS NULL) Then
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 728+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(729) : cplan.sql(8058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units,
                        CONTOUR_TYPE = h_contour_type
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 748+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(749) : cplan.sql(8078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 791+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(792) : cplan.sql(8121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     --trace_printf("ACTUAL: SC_STATE: %d, h_ra_id: %d:%s\n", SC_STATE, h_ra_id_Ind,h_ra_id);
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 828+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(829) : cplan.sql(8158) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                  UNITS_MSP = h_units,
                  CONTOUR_TYPE = h_contour_type
                  WHERE RES_ASSIGNMENT_ID = h_ra_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 850+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(851) : cplan.sql(8180) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
               If (baseline_start001 IS NULL) Then
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 878+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(879) : cplan.sql(8208) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 898+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(899) : cplan.sql(8228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))  // update actual
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 941+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(942) : cplan.sql(8271) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 978+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(979) : cplan.sql(8308) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf Coalesce(h_count, 0) = 0 Then
               If (h_prj_assignment_id IS NULL) Then
                  If Coalesce(eetc, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001,
                        ETC = h_effort - work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1003+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1004) : cplan.sql(8333) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1023+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1024) : cplan.sql(8353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                     UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                     WORK_TODATE = work_todate001
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1044+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1045) : cplan.sql(8374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("A_* SC_STATE = %d, h_ra_id: %s,h_prj_assignment_id: %d %s\n",SC_STATE,h_ra_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
            End If;
         End If;
         
         --  if( ((*data_flag  17) == 17)  (IsNotNullSpParam(ind_assign_msp, actual_start)  (*actual_effort > 0)) || (*work_todate > 0) )
         If ((h_actual_start IS NOT NULL) AND (Coalesce(actual_effort001, 0) > 0)) OR (Coalesce(
               work_todate001, 0) > 0) Then
            h_finish_D := '';
            h_finish_D := NULL;
            h_total := 0;
            h_count := 0;
            
            If Coalesce(h_action_type, 0) = 1 Then
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_tsk_assignment_id, h_effort, h_task_id, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE RES_ASSIGNMENT_ID = h_ra_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1073+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1074) : cplan.sql(8403) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_tsk_assignment_id := h_ra_id;
               BEGIN
                  SELECT SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_effort, h_task_id, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1090+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1091) : cplan.sql(8420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("SC_STATE = %d, h_work_todate: %d, h_effort = %d, h_tsk_assignment_id = %s, h_ra_id = %s\n",SC_STATE,h_work_todate,h_effort,h_tsk_assignment_id,h_ra_id);
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               If (Coalesce(work_todate001, 0) = Coalesce(effort001, 0)) AND (Coalesce(eetc, 0) = 0) AND (Coalesce(
                                           percent_complete001, 0) = 10000) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1113+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1114) : cplan.sql(8443) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET WORK_TODATE = SCHED_WORK,
                     LAST_SUBMITTED = (SELECT MAX(DAY_YEAR) FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id)
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1133+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1134) : cplan.sql(8463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  /*InitializeHost(SC_PARAM(h_tph_delete), 0);
                                  h_billable = (h_billable | SC_BIT_30);
                  
                                  h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                                              SC_PARAM(h_project_id),
                                              SC_PARAM(h_task_id),
                                              SC_PARAM(h_tsk_assignment_id),
                                              SC_PARAM(h_SD),
                                              SC_PARAM(h_FD),
                                              SC_PARAM(h_reg_cost),
                                              SC_PARAM(h_reg_sale),
                                              SC_PARAM(h_tph_delete),
                                              SC_PARAM(h_capital_expense),
                                              SC_PARAM(h_billable));*/
               Else
                  OPEN CAF;
                  
                  LOOP
                     FETCH CAF INTO h_minutes, h_dayofyear, h_count;
                     IF CAF%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                     If PMO_DB2M.SC_STATE <> 0 Then
                        EXIT;
                     End If;
                     
                     --trace_printf("h_dayofyear = %s, h_total = %d, h_effort: %d, h_minutes = %d, h_count: %d\n", h_dayofyear,h_total,h_effort,h_minutes,h_count);
                     h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                     
                     If Coalesce(h_count, 0) = 0 Then
                        If Coalesce(h_total, 0) > Coalesce(h_work_todate, 0) Then
                           If Coalesce(h_minutes, 0) >= Coalesce(h_work_todate, 0) Then
                              h_minutes := Coalesce(h_work_todate, 0);
                           Else
                              h_minutes := Coalesce(h_work_todate, 0) - Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                           End If;
                        End If;
                        
                        BEGIN
                           UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1 WHERE CURRENT OF CAF;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1190+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1191) : cplan.sql(8520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        h_finish_D := h_dayofyear;
                     End If;
                     
                     If Coalesce(h_total, 0) >= Coalesce(h_work_todate, 0) Then
                        EXIT;
                     End If;
                  END LOOP;
                  
                  BEGIN
                     CLOSE CAF ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1211+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1212) : cplan.sql(8541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("h_total = %d, h_work_todate: %d, h_finish_D = %d:%s, h_tsk_assignment_id = %s\n",h_total,h_work_todate,h_finish_D_Ind,h_finish_D,h_tsk_assignment_id);
                  
                  If (Coalesce(h_total, 0) > 0) AND ((h_finish_D IS NOT NULL)) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_D WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1230+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1231) : cplan.sql(8560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     /*if (h_total > h_work_todate)
                                         {
                                             InitializeHost(SC_PARAM(h_tph_delete), -1);
                                             h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                                                         SC_PARAM(h_project_id),
                                                         SC_PARAM(h_task_id),
                                                         SC_PARAM(h_tsk_assignment_id),
                                                         SC_PARAM(h_SD),
                                                         SC_PARAM(h_FD),
                                                         SC_PARAM(h_reg_cost),
                                                         SC_PARAM(h_reg_sale),
                                                         SC_PARAM(h_tph_delete),
                                                         SC_PARAM(h_capital_expense),
                                                         SC_PARAM(h_billable));
                     
                                             InitializeHost(SC_PARAM(h_bit_minutes), h_minutes - h_total + h_work_todate);
                                             h_temp_sc_state = FC_ACTUAL_DAY( SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_dayofyear), SC_PARAM(h_bit_minutes));
                     
                                             h_temp_sc_state = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
                     
                                             InitializeHost(SC_PARAM(h_etc), *eetc, ind_assign_msp->eetc);
                                             BEGIN_SQL
                                                 SELECT TIMESTAMP(DATE(HOST(h_dayofyear)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
                                             END_SQL;
                     //trace_printf("FC_RESCHEDULE: h_start_date = %s, h_etc: %d\n",h_start_date,h_etc);
                     
                                             InitializeHost(SC_PARAM(h_temp_var), 2);
                                             h_temp_sc_state = FC_RESCHEDULE(SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_start_date), SC_PARAM(h_etc), SC_PARAM(h_temp_var), SC_PARAM(h_contour), SC_PARAM(h_rec_user));
                     
                                             BEGIN_SQL
                                                 SELECT TASK_ID, DATE(SCHED_START), DATE(SCHED_FINISH), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                                                 INTO HOST(h_task_id), HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                                                 FROM TASK_ASSIGNMENTS
                                                 WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
                                             END_SQL;
                     
                                             InitializeHost(SC_PARAM(h_tph_delete), 0);
                     
                                             h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                                                         SC_PARAM(h_project_id),
                                                         SC_PARAM(h_task_id),
                                                         SC_PARAM(h_tsk_assignment_id),
                                                         SC_PARAM(h_SD),
                                                         SC_PARAM(h_FD),
                                                         SC_PARAM(h_reg_cost),
                                                         SC_PARAM(h_reg_sale),
                                                         SC_PARAM(h_tph_delete),
                                                         SC_PARAM(h_capital_expense),
                                                         SC_PARAM(h_billable));
                                         }
                     
                                         if ( h_action_type == 1 )
                                         {
                                             InitializeHost(SC_PARAM(h_tph_delete), 0);
                                             h_billable = (h_billable | SC_BIT_30);
                     
                                             h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                                                         SC_PARAM(h_project_id),
                                                         SC_PARAM(h_task_id),
                                                         SC_PARAM(h_tsk_assignment_id),
                                                         SC_PARAM(h_SD),
                                                         SC_PARAM(h_FD),
                                                         SC_PARAM(h_reg_cost),
                                                         SC_PARAM(h_reg_sale),
                                                         SC_PARAM(h_tph_delete),
                                                         SC_PARAM(h_capital_expense),
                                                         SC_PARAM(h_billable));
                                         }*/
                  End If;
               End If;
               
               -- populate timesheets
               If Coalesce(h_action_type, 0) = 1 Then
                  If (BITAND(Coalesce(data_flag, 0), 32)) = 32 Then
                     h_temp_sc_state := FC_GENERATE_TS_ALL(h_tsk_assignment_id, h_rec_user);
                  Else
                     h_temp_sc_state := FC_GENERATE_TS_MSP(h_tsk_assignment_id, h_rec_user);
                  End If;
               End If;
            End If;
         End If;
      End If;
      
      If PMO_DB2M.SC_STATE = 100 Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   --trace_printf("END: SC_STATE = %d\n",SC_STATE);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ASSIGN_MSP', 1326+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ASSIGN_MSP(1327) : cplan.sql(8656) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1334+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1335) : cplan.sql(8664) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1346+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1347) : cplan.sql(8676) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ASSIGN_MSP;
ALTER FUNCTION SP_ASSIGN_MSP COMPILE DEBUG;

/*
#Stored Procedure  :SP_A_RESTOPROF
#Short Description :Replace a profile with a resource
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_A_RESTOPROF ==
CREATE OR REPLACE FUNCTION SP_A_RESTOPROF
(
     prj_id VARCHAR2
   , profile_id001 VARCHAR2
   , res_id001 VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_cur_prj_assign_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_profile_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_task_start_DT TIMESTAMP;
   h_task_finish_DT TIMESTAMP;
   h_null_str VARCHAR2(1);
   h_start_D DATE;
   h_finish_D DATE;
   h_res_type VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_role_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_in VARCHAR2(255);
   h_profile_name VARCHAR2(255);
   h_labor_code VARCHAR2(50);
   h_participant PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_assigned PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_owner PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_contour_changed PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_spec_cost NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_rate_echange NUMBER := 0.00;
   h_currency_id PLS_INTEGER := 0;
   h_donot_replace_bits PLS_INTEGER := 0;
   h_donot_replace_rate PLS_INTEGER := 0;
   h_donot_replace_cost PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_same_labor PLS_INTEGER := 0;
   h_fs_project PLS_INTEGER := 0;
   i_return PLS_INTEGER := 0;
   h_and_roles VARCHAR2(4000);
   h_and_skills VARCHAR2(4000);
   h_temp_res_cost NUMBER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_pa_timephase VARCHAR2(1);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_old_cost NUMBER := 0;
   h_old_sale NUMBER := 0;
   h_pa_count PLS_INTEGER := 0;
   h_ta_count PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_planned_effort PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_timecode_1_id VARCHAR2(32);
   h_timecode_2_id VARCHAR2(32);
   h_old_prj_assign_id VARCHAR2(32);
   i_isTask PLS_INTEGER NOT NULL := (isWBSTask(Coalesce(h_type_id, 0)));
   l_data_flag PLS_INTEGER NOT NULL := 0;
   i_many PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CRCAL1 IS
         SELECT AVAILABLE_MIN FROM CALENDAR_DAY_OF_YEAR
         WHERE  ROWNUM <= 1 AND  ELEMENT_ID = h_res_id;

      CURSOR CSA1 IS
         SELECT BAK_WORK FROM TASK_ASSIGNMENTS
         WHERE  ROWNUM <= 1 AND  PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND
         PROJECT_ASSIGNMENT_ID IS NuLL;

      CURSOR C6a IS
         SELECT r.RES_ASSIGNMENT_ID, t.TSK_ASSIGNMENT_ID, t.TASK_ID, r.ELEMENT_TYPE, t.PLANNED_WORK, t.UNITS_MSP,
         TO_DATE(SubStr(t.PLANNED_START, 1, 10)), TO_DATE(SubStr(t.PLANNED_FINISH, 1, 10)),t.PLANNED_START, t.PLANNED_FINISH, CASE WHEN t.PLANNED_DURATION = 0 THEN w.PLANNED_DURATION ELSE t.PLANNED_DURATION END,
         t.PROJECT_ASSIGNMENT_ID, t.BILLABLE, t.CAPITAL_EXPENSE, t.REGULAR_COST, t.REGULAR_RATE,
         t.C_EFFORT, t.T_UNIT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE
         FROM TMT_WBS w, RESOURCE_ASSIGNMENTS r, TASK_ASSIGNMENTS t
         WHERE r.PROJECT_ID = h_project_id AND r.RESOURCE_ID = h_profile_id AND r.RESOURCE_TYPE = '1' 
         AND t.RES_ASSIGNMENT_ID = r.RES_ASSIGNMENT_ID AND r.ELEMENT_ID = w.ELEMENT_ID 
         AND r.ELEMENT_TYPE IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank
         ORDER BY t.PLANNED_START;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RESTOPROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RESTOPROF'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RESTOPROF(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || profile_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 161+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RESTOPROF(162) : cplan.sql(8862) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   -- 0 - res is not assigned in
   
   -- Parse parameters to local variables
   h_task_id := prj_id;
   h_profile_id := profile_id001;
   h_res_id := res_id001;
   h_rec_user := rec_user001;
   h_start_DT := start_date001;
   h_consider := consider_planned_sched;
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) OR (Coalesce(
             h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) Then
      h_consider := 7;
   End If;
   
   h_finish_DT := h_start_DT;
   h_labor_code := '';
   -- Labor Code
   h_null_str := '';
   h_occurrence := 0;
   -- occurrence
   h_finish_date := '';
   
   h_sched_start := NULL;
   h_sched_finish := NULL;
   
   OPEN CRCAL1;
   
   FETCH CRCAL1 INTO h_count;
   IF CRCAL1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   h_count := PMO_DB2M.SC_STATE;
   
   BEGIN
      CLOSE CRCAL1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 210+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(211) : cplan.sql(8911) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 100 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 217+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(218) : cplan.sql(8918) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 220+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(221) : cplan.sql(8921) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
   End If;
   
   BEGIN
      SELECT TYPE_ID, RANK, PROJECT_ID, TIMECODE_1_ID, TIMECODE_2_ID, TASK_TYPE, UNRESOLVED_DURATION
      INTO h_type_id, h_rank, h_project_id, h_timecode_1_id, h_timecode_2_id, h_task_type, h_duration
      FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 235+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(236) : cplan.sql(8936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_duration: %d\n", SC_STATE,h_duration);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 243+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(244) : cplan.sql(8944) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 246+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(247) : cplan.sql(8947) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
   End If;
   
   h_temp_var := 0;
   BEGIN
      SELECT LEVEL_LOCK INTO h_temp_var
      FROM PROJECT_DETAILS
      WHERE PROJECT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 262+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(263) : cplan.sql(8963) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_var, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 269+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(270) : cplan.sql(8970) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_LEVEL), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 272+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(273) : cplan.sql(8973) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_LEVEL, h_project_id, True);
   End If;
   
   BEGIN
      SELECT tw.TYPE_ID, CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), COALESCE(tw.MINUTE_PER_WEEK, 2400), tw.PRIORITY, tw.CURRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4)
      INTO h_type, h_fs_project, h_per_day, h_per_week, h_priority, h_currency_id, h_rate_echange
      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID WHERE tw.ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 287+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(288) : cplan.sql(8988) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 294+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(295) : cplan.sql(8995) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 297+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(298) : cplan.sql(8998) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
   End If;
   
   BEGIN
      SELECT NICK_NAME,FS_LABOR_CODE INTO h_full_name,h_labor_code FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 310+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(311) : cplan.sql(9011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 317+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(318) : cplan.sql(9018) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 320+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(321) : cplan.sql(9021) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
   End If;
   
   BEGIN
      SELECT ROLE_ID INTO h_role_id FROM PROFILES WHERE PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 333+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(334) : cplan.sql(9034) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_ORASTD.strcmp(h_role_id, 'DEFAULTLEVEL') = 0 Then
      h_role_id := '';
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 344+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(345) : cplan.sql(9045) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_profile_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 347+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(348) : cplan.sql(9048) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_profile_id, True);
   End If;
   
   h_donot_replace_bits := work_type;
   
   h_donot_replace_cost := BITAND(Coalesce(h_donot_replace_bits, 0), 4);
   h_from_prj := BITAND(Coalesce(h_donot_replace_bits, 0), 2);
   h_donot_replace_rate := BITAND(Coalesce(h_donot_replace_bits, 0), 1);
   
   /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
       {
           h_from_prj = 2;
           h_assigned = 0;
       }*/
   
   --trace_printf("***** h_assigned = %d, h_from_prj: %d, i_isTask: %d, h_type_id: %d\n",h_assigned,h_from_prj,i_isTask,h_type_id);
   If i_isTask <> 0 Then
      If Coalesce(h_from_prj, 0) = 2 Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 377+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(378) : cplan.sql(9078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_from_prj, 0) = 0 Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID <> PROJECT_ID AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 392+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(393) : cplan.sql(9093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 400+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(401) : cplan.sql(9101) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 403+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(404) : cplan.sql(9104) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
               , True);
      End If;
   Else
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_ta_count FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND PROJECT_ID <> TASK_ID AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 418+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(419) : cplan.sql(9119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_pa_count FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND PROJECT_ID = TASK_ID AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 432+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(433) : cplan.sql(9133) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (isProjectType(Coalesce(h_type_id, 0))) <> 0 AND ((Coalesce(h_from_prj, 0) = 0) OR (Coalesce(
            h_ta_count, 0) = 0)) Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0'
            AND PROJECT_ASSIGNMENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 449+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(450) : cplan.sql(9150) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_temp_var, 0) > 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 456+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(457) : cplan.sql(9157) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 459+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(460) : cplan.sql(9160) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                  , True);
         End If;
      End If;
   End If;
   
   /********************************************************************************
       Salling Rates           Cost Ratess             Client
       =================================================================================
       h_donot_replace_rate    h_donot_replace_cost    h_donot_replace_bits
       =================================================================================
       Affect this resource to the project             NON = 2, YES = 0
       =================================================================================
       1                       1                       2 (2 + 0) 2 = NO affected
       0                       0                       7 (2 + 5)
       1                       0                       6 (2 + 4)
       0                       1                       3 (2 + 1)
       *********************************************************************************/
   
   h_res_type := '0';
   h_assign_id := '';
   h_prj_assign_id := '';
   h_prj_assignment_id := NULL;
   h_start_date := NULL;
   --trace_printf("*** h_start_DT: %d %s, h_ta_count: %d, h_pa_count: %d, h_from_prj: %d, h_donot_replace_rate: %d, h_donot_replace_cost: %d\n", h_start_DT_Ind,h_start_DT,h_ta_count,h_pa_count,h_from_prj, h_donot_replace_rate, h_donot_replace_cost);
   
   /*
           if h_assigned = 0, then: res is assigned in the project     -> don't create PA
           if h_assigned = 1, then: res is affected to the project     -> create/merge PA
           if h_assigned = 2, then: res is not assigned in the project -> create PA
       */
   
   OPEN CSA1;
   
   FETCH CSA1 INTO h_count;
   IF CSA1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_assigned := 0;
   Else
      h_assigned := 2;
   End If;
   
   BEGIN
      CLOSE CSA1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 516+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(517) : cplan.sql(9217) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("***** h_assigned = %d, h_from_prj: %d, i_isTask: %d\n",h_assigned,h_from_prj,i_isTask);
   If Coalesce(h_assigned, 0) = 2 Then
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND
         RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND TASK_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 532+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(533) : cplan.sql(9233) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) > 0 Then
         h_assigned := 1;
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH INTO h_sched_start, h_sched_finish
            FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND BIT_FLAG = '0' AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND TASK_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 549+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(550) : cplan.sql(9250) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4_* SC_STATE: %d, h_sched_start: %d:%s, h_sched_finish: %d:%s\n",SC_STATE,h_sched_start_Ind,h_sched_start,h_sched_finish_Ind,h_sched_finish);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 557+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(558) : cplan.sql(9258) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 560+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(561) : cplan.sql(9261) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                  , True);
         ElsIf ((h_sched_start IS NOT NULL)) AND ((h_sched_finish IS NOT NULL)) Then
            If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) > 0) OR (PMO_ORASTD.strncmp(h_sched_finish
                  , h_start_date, 10) < 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 568+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(569) : cplan.sql(9269) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 571+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(572) : cplan.sql(9272) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                     , True);
            End If;
         End If;
      End If;
      
      --trace_printf("h_assigned = %d, h_count: %d, h_from_prj: %d\n",h_assigned,h_count,h_from_prj);
      If Coalesce(h_from_prj, 0) = 2 Then
         h_assigned := 0;
         h_prj_assignment_id := h_prj_assignment_id;
         --h_prj_assignment_id_Ind = 0;
      Else
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_prj_assign_id, h_owner
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_project_id 
            AND RESOURCE_TYPE = h_res_type 
            AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 597+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(598) : cplan.sql(9298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("0__ SC_STATE: %d, h_prj_assign_id: %d:%s, h_owner: %d\n",SC_STATE,h_prj_assign_id_Ind,h_prj_assign_id,h_owner);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            h_prj_assign_id := '';
            h_prj_assign_id := SYS_GUID();
            
            h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
            h_temp_participant := PMO_CNST.SC_AFFECTED;
            h_temp_res_type := '0';
            h_temp_bitflag := 1;
            
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_temp_res_type, h_temp_bitflag);
            
            h_table_type := GetTable(Coalesce(h_type, 0));
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PROFILE_ID, PARTICIPANT, SECURITY_ID, TABLE_TYPE,
               REC_USER, REC_DATETIME)
               VALUES (h_prj_assign_id, h_project_id, h_project_id, h_type,
               h_res_type, h_res_id, h_profile_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, h_table_type,
               rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 634+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(635) : cplan.sql(9335) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RA: SC_STATE = %d, h_type: %d, h_prj_assign_id = %s, h_temp_security_id: %d:%s\n",SC_STATE,h_type,h_prj_assign_id,h_temp_security_id_Ind,h_temp_security_id);
            
            BEGIN
               SELECT PROFILE_NAME INTO h_profile_name FROM PROFILES WHERE PROFILE_ID = h_profile_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 648+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(649) : cplan.sql(9349) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
               h_total := 0;
            Else
               BEGIN
                  SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_total FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id AND TASK_ID <> h_task_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 665+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(666) : cplan.sql(9366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("0.p__ h_total = %d\n",h_total);
            
            If Coalesce(h_total, 0) = 0 Then
               -- first try to replace profile_name with resource_name (if profile_name already exists at the project level)
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name)
                  WHERE ELEMENT_ID = h_project_id AND PROJECT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 687+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(688) : cplan.sql(9388) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- if profile has not been assigned to the project level, then add resource name to the list
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
                  ELSE
                  CASE
                  WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
                  ELSE ASSIGN_NAMES || h_full_name || ';'
                  END
                  END
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 714+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(715) : cplan.sql(9415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               -- if profile is still assigned within the project, then add resource name to the list
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
                  ELSE
                  CASE
                  WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
                  ELSE ASSIGN_NAMES || h_full_name || ';'
                  END
                  END
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 741+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(742) : cplan.sql(9442) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID INTO h_prj_assignment_id FROM TASK_ASSIGNMENTS
               WHERE RES_ASSIGNMENT_ID = h_prj_assign_id AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 756+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(757) : cplan.sql(9457) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("0.2__ SC_STATE = %d, h_owner = %d, h_prj_assignment_id = %d:%s, h_prj_assign_id: %d:%s\n",SC_STATE,h_owner,h_prj_assignment_id_Ind,h_prj_assignment_id,h_prj_assign_id_Ind,h_prj_assign_id);
            
            If Coalesce(h_owner, 0) = PMO_CNST.SC_PARTICIPANT Then
               h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT = PMO_CNST.SC_AFFECTED, SECURITY_ID = h_temp_security_id
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 779+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(780) : cplan.sql(9480) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            ElsIf (BITAND(Coalesce(h_owner, 0), PMO_CNST.SC_AFFECTED)) <> PMO_CNST.SC_AFFECTED Then
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_AFFECTED)
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 800+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(801) : cplan.sql(9501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            Else
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 821+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(822) : cplan.sql(9522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            End If;
            --trace_printf("0.3__ SC_STATE = %d, h_temp_security_id = %s\n",SC_STATE,h_temp_security_id);
         End If;
      End If;
   Else
      h_prj_assignment_id := h_prj_assignment_id;
      --h_prj_assignment_id_Ind = 0;
   End If;
   
   -- replace RATES =================================================================
   --trace_printf("SC_STATE = %d, h_res_id = %s, h_currency_id = %d, h_rate_echange = %.2f\n",SC_STATE,h_res_id,h_currency_id,h_rate_echange);
   h_and_roles := '';
   h_and_skills := '';
   h_finish_date := '';
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   If (Coalesce(h_donot_replace_cost, 0) = 0) OR (Coalesce(h_donot_replace_rate, 0) = 0) Then
      h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, h_and_roles, h_and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
   End If;
   
   BEGIN
      SELECT CASE WHEN h_currency_id = rs.CURRENCY_ID THEN 1
      ELSE h_rate_echange / PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) END
      INTO h_rate_echange
      FROM RESOURCES rs JOIN TMT_DATAFIELDS td ON rs.CURRENCY_ID = td.ELEMENT_ID AND rs.CURRENCY_ID > 0
      WHERE RESOURCE_ID=h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 861+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(862) : cplan.sql(9562) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      h_rate_echange := 1.00;
   End If;
   
   --trace_printf("1_* h_reg_cost = %.2f, h_spec_cost = %.2f, h_reg_sale = %.2f, h_spec_sale = %.2f\n",h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale);
   --trace_printf("prj_assignment: h_assigned = %d, h_count = %d, h_prj_assignment_id: %d:%s\nh_project_id = %s,h_task_id = %s\n",h_assigned,h_count,h_prj_assignment_id_Ind,h_prj_assignment_id,h_project_id,h_task_id);
   h_passed := 0;
   h_total := 0;
   h_old_prj_assign_id := NULL;
   --trace_printf("A_ prj_assignment: h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   
   BEGIN
      SELECT ASSIGNMENT_BILLABLE_FLAG, TIMEPHASE_PROJECT_ASSIGNMENT INTO h_billable, h_pa_timephase
      FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 886+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(887) : cplan.sql(9587) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("77:_SC_STATE: %d, h_pa_timephase: %d:%s, h_profile_id: %d:%s\n", SC_STATE,h_pa_timephase_Ind,h_pa_timephase,h_profile_id_Ind,h_profile_id);
   
   If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
      BEGIN
         SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_profile_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 902+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(903) : cplan.sql(9603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("p.a_SC_STATE: %d, h_total: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_total,h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      If Coalesce(h_total, 0) = 0 Then
         --strcpy(h_task_id,h_project_id);
         h_task_id := h_project_id;
      Else
         h_total := 0;
      End If;
   End If;
   
   If Coalesce(h_assigned, 0) > 0 Then
      If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
         BEGIN
            SELECT TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,PLANNED_WORK, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE, TASK_ID, UNITS_MSP, C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR
            INTO h_prj_assignment_id, h_id, h_planned_effort, h_start_D, h_finish_D, h_billable, h_capital_expense,
            h_old_cost, h_old_sale, h_task_id, h_percent_units, h_work, h_eunit, h_unit, h_contour
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID = h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 931+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(932) : cplan.sql(9632) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("p.0_SC_STATE: %d, h_planned_effort: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_planned_effort,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         h_old_prj_assign_id := h_prj_assignment_id;
         --trace_printf("1__* h_old_prj_assign_id: %d:%s\n", h_old_prj_assign_id_Ind,h_old_prj_assign_id);
         
         h_contour_changed := 0;
         If Coalesce(h_pa_count, 0) = 1 Then
            If (Coalesce(h_contour, 0) <> Coalesce(contour001, 0)) OR (Coalesce(h_work, 0) <> Coalesce(
                                       work001, 0)) Then
               If (Coalesce(h_assigned, 0) = 1) AND ((BITAND(Coalesce(
                     bitflag001, 0), PMO_CNST.SC_0000FFFF)) <> 0) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 947+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(948) : cplan.sql(9648) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 950+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(951) : cplan.sql(9651) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
               Else
                  h_contour_changed := 1;
                  
                  If Coalesce(contour001, 0) = 0 Then
                     h_percent_units := 0;
                  End If;
               End If;
            End If;
            
            h_effort := effort001;
            h_start_date := start_date001;
            h_finish_date := finish_date001;
            h_work := work001;
            -- amount for countour
            h_occurrence := occurrence;
            -- occurrence
            h_eunit := eunit;
            h_unit := unit;
            h_bitflag := bitflag001;
            h_contour := contour001;
         End If;
         --trace_printf("** CONTOUR ** h_bitflag: %d, h_assigned: %d, h_work: %d, h_contour: %d, h_contour_changed: %d\n", h_bitflag,h_assigned,h_work,h_contour,h_contour_changed);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_prj_assignment_id IS NOT NULL)) Then
            If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
               h_tph_delete := -1;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
            End If;
            
            If Coalesce(h_assigned, 0) = 1 Then
               BEGIN
                  DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 997+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(998) : cplan.sql(9698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DEL_OLD_TA: SC_STATE: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);
               
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1015+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1016) : cplan.sql(9716) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DEL_OLD_TDY: SC_STATE: %d\n", SC_STATE);
               
               h_prj_assignment_id := h_old_prj_assign_id;
               BEGIN
                  SELECT MAX(DAY_YEAR) INTO h_start_D FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1031+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1032) : cplan.sql(9732) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_ORASTD.strncmp(h_start_date, h_start_D, 10) <= 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_D, 1, 10)) + 1     , PMO_ORASTD.TIME(h_start_date))
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1047+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1048) : cplan.sql(9748) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
                  h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
                  
                  If Coalesce(h_billable, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_COST = h_reg_cost,
                        SPECIAL_COST = h_spec_cost,
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1081+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1082) : cplan.sql(9782) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_COST = h_reg_cost,
                        SPECIAL_COST = h_spec_cost,
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                        ESTIMATION_LABOR_REVENUE = ((PLANNED_WORK+PLANNED_WORK)/60.00) * h_reg_sale,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1108+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1109) : cplan.sql(9809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf Coalesce(h_donot_replace_cost, 0) = 0 Then
                  h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                     REGULAR_COST = h_reg_cost,
                     SPECIAL_COST = h_spec_cost,
                     UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                     ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1136+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1137) : cplan.sql(9837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
                  h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
                  
                  If Coalesce(h_billable, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1163+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1164) : cplan.sql(9864) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_REVENUE = ((PLANNED_WORK+PLANNED_WORK)/60.00) * h_reg_sale,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1187+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1188) : cplan.sql(9888) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                     UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0'
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1208+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1209) : cplan.sql(9909) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1226+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1227) : cplan.sql(9927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_prj_assignment_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_SKILL
                  FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1245+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1246) : cplan.sql(9946) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("U_PA_SC_STATE: %d, h_rate_echange: %.2f, h_old_cost: %.2f, h_old_sale: %.2f, h_reg_cost: %.2f, h_reg_sale: %.2f\n", SC_STATE,h_rate_echange,h_old_cost,h_old_sale,h_reg_cost,h_reg_sale);
            
            h_count := 1;
            --trace_printf("0__ prj_assignment: h_pa_count: %d, h_prj_assignment_id: %d %s, h_id = %s\n",h_pa_count,h_prj_assignment_id_Ind,h_prj_assignment_id,h_id);
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1267+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1268) : cplan.sql(9968) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("DEL_OLD_RA: SC_STATE: %d, h_id: %d:%s\n", SC_STATE,h_id_Ind,h_id);
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1285+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1286) : cplan.sql(9986) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_pa_count, 0) = 1) AND (Coalesce(h_ta_count, 0) = 0) Then
               h_work_type := 0;
               h_cri := 0;
               If PMO_FUNC.AssignLong(h_contour_changed, 1) <> 0 Then
                  h_update := 2;
               Else
                  h_update := 1;
               End If;
               
               If Coalesce(contour001, 0) = 0 Then
                  h_work := 0;
               End If;
               
               h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
               
               If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 1310+1) THEN 
                     LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(1311) : cplan.sql(10011) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1313+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1314) : cplan.sql(10014) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
               ElsIf Coalesce(h_temp_var, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1318+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1319) : cplan.sql(10019) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1321+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1322) : cplan.sql(10022) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
               End If;
               
               --trace_printf("*** h_start_DT = %s, h_finish_date = %s, h_duration = %d, h_effort: %d, h_work: %d\n",h_start_DT,h_finish_date,h_duration,h_effort,h_work);
               
               If Coalesce(h_work, 0) = 0 Then
                  h_bitflag := PMO_CNST.SC_BIT_27;
               Else
                  h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27);
               End If;
               
               --trace_printf("****** Project Assignment ONLY - all effort to PA\n");
               --trace_printf("AA_1: h_bitflag = %d, h_start_date = %s, h_finish_date = %s, h_duration = %d, h_work_type = %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_work_type);
               
               -- assign to the project
               h_temp_res_cost := 0.00;
               i_return := FC_SEARCH_2(h_prj_assignment_id, h_project_id, h_project_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               
               --trace_printf("ONLY to the project: i_return = %d, h_count = %d\n", i_return,h_count);
               
               If Coalesce(i_return, 0) = 110 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1349+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1350) : cplan.sql(10050) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1352+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1353) : cplan.sql(10053) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
               ElsIf Coalesce(i_return, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1357+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1358) : cplan.sql(10058) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1360+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1361) : cplan.sql(10061) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               Else
                  h_temp_var := 4;
                  h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(h_per_day, 0)
                              , Coalesce(h_effort, 0));
               End If;
            End If;
         End If;
      ElsIf SubStr(h_pa_timephase, 1, 1) = 'Y' Then
         -- Get the old project assignment id
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id 
            AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1385+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1386) : cplan.sql(10086) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("old.1_SC_STATE: %d, h_planned_effort: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_planned_effort,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_old_prj_assign_id IS NOT NULL)) Then
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
         End If;
      Else
         -- Get the old project assignment id
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id 
            AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1410+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1411) : cplan.sql(10111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("old.2_SC_STATE: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
      End If;
   ElsIf SubStr(h_pa_timephase, 1, 1) = 'Y' Then
      -- Get the old project assignment id
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id 
         AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1430+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1431) : cplan.sql(10131) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("old.3_SC_STATE: %d, h_planned_effort: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_planned_effort,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_old_prj_assign_id IS NOT NULL)) Then
         h_tph_delete := -1;
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
      End If;
   Else
      -- Get the old project assignment id
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id 
         AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1455+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1456) : cplan.sql(10156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("old.4_SC_STATE: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
   End If;
   
   --trace_printf("1__ prj_assignment: h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   --trace_printf("3_* h_start_D: %d:%s, h_finish_D: %d:%s\n",h_start_D_Ind,h_start_D,h_finish_D_Ind,h_finish_D);
   --trace_printf("5_* h_old_prj_assign_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_old_prj_assign_id_Ind,h_old_prj_assign_id,h_assigned,h_from_prj);
   -- resource is not assigned in the project or affected in schedule mode  create project assignment
   If ((h_prj_assignment_id IS NULL)) AND (Coalesce(h_assigned, 0) > 0) AND (Coalesce(
         h_from_prj, 0) = 0) Then
      h_prj_assignment_id := '';
      h_prj_assignment_id := SYS_GUID();
      --h_prj_assignment_id_Ind = 0;
      --InitializeHost(SC_PARAM(h_total), h_total);  //h_total_Ind = 0;
      
      If Coalesce(unit, 0) = 100 Then
         h_percent_units := Coalesce(h_work, 0);
      End If;
      
      If (Coalesce(h_donot_replace_cost, 0) = 0) OR (Coalesce(h_donot_replace_rate, 0) = 0) Then
         If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
            h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
            h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
            h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
            h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale,h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1501+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1502) : cplan.sql(10202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
            h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
            h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1526+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1527) : cplan.sql(10227) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
            h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1551+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1552) : cplan.sql(10252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
            PLANNED_START,PLANNED_FINISH,BIT_FLAG,BAK_START_DT,BAK_FINISH_DT,RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
            BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
            VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
            h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
            h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1574+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1575) : cplan.sql(10275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("SC_STATE: %d, h_capital_expense: %d, h_billable = %d,h_table_type: %d\n",SC_STATE,h_capital_expense,h_billable,h_table_type);
      --trace_printf("2_* h_start_DT = %d:%s, h_finish_DT = %d:%s\n",h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT);
      
      h_count := 1;
   End If;
   h_rank := SubStr(h_rank || '%', 1, 200);
   h_tsk_assignment_id := '';
   h_effort := effort001;
   
   --trace_printf("==========: h_prj_assignment_id: %d:%s, h_count = %d, h_assigned: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_count,h_assigned);
   --trace_printf("1_* h_start_DT = %d:%s, h_finish_DT = %d:%s, h_type_id: %d, i_isTask: %d\n",h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_type_id,i_isTask);
   
   -- Replace the profile with Resource
   BEGIN
      SELECT PROFILE_NAME INTO h_profile_name FROM PROFILES WHERE PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1600+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1601) : cplan.sql(10301) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_old_cost := 0.00;
   h_old_sale := 0.00;
   h_total := 0;
   --  InitializeHost(SC_PARAM(h_days), -1);
   
   If (i_isTask = 0) AND (Coalesce(h_ta_count, 0) = 1) Then
      BEGIN
         SELECT MIN(t.PLANNED_START) INTO h_start_date
         FROM TMT_WBS w, TASK_ASSIGNMENTS t
         WHERE t.PROJECT_ID = h_project_id AND t.RESOURCE_ID = h_profile_id AND
         t.TASK_ID = w.ELEMENT_ID AND w.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1622+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1623) : cplan.sql(10323) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*BEGIN_SQL
                  SELECT DAYS(HOST(h_start_DT)) - DAYS(HOST(h_start_date)) INTO HOST(h_days) FROM DUAL;
              END_SQL;*/
   End If;
   --trace_printf("h_start_date: %s, h_start_DT: %s, h_days: %d\n", h_start_date,h_start_DT,h_days);
   
   h_start_date := h_start_DT;
   
   -- replace all instances inside the project_id
   
   OPEN C6a;
   
   LOOP
      FETCH C6a INTO h_assign_id, h_tsk_assignment_id, h_task_id, h_type, h_effort, h_percent_units,
      h_SD,h_FD,h_task_start_DT,h_task_finish_DT,h_duration,
      h_cur_prj_assign_id, h_billable,h_capital_expense,h_old_cost,h_old_sale,
      h_work, h_eunit, h_unit, h_contour, h_bitflag;
      IF C6a%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      --trace_printf("FETCH: h_assign_id: %s, h_tsk_assignment_id: %s\n",h_assign_id,h_tsk_assignment_id);//trace_flush();
      --trace_printf("h_task_id: %s, h_type: %d, h_effort: %d, h_duration: %d\n",h_task_id,h_type,h_effort,h_duration);//trace_flush();
      --trace_printf("h_bitflag: %d, h_contour: %d, h_work: %d, h_percent_units: %d\n",h_contour,h_work,h_percent_units);//trace_flush();
      
      If Coalesce(h_effort, 0) = 0 Then
         h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_assign_id, 0);
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1672+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1673) : cplan.sql(10373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1689+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1690) : cplan.sql(10390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1706+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1707) : cplan.sql(10407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1723+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1724) : cplan.sql(10424) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1740+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1741) : cplan.sql(10441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1757+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1758) : cplan.sql(10458) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT TSK_ASSIGNMENT_ID INTO h_id FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND RESOURCE_ID = h_res_id
            AND RESOURCE_TYPE ='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1772+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1773) : cplan.sql(10473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1.0__ SC_STATE: %d, h_id: %s, h_assigned: %d, h_tsk_assignment_id: %d:%s\n",SC_STATE,h_id,h_assigned,h_tsk_assignment_id_Ind,h_tsk_assignment_id); trace_flush();
         --trace_printf("A1_** h_prj_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id); trace_flush();
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            If Coalesce(h_assigned, 0) = 0 Then
               h_prj_assignment_id := h_tsk_assignment_id;
            End If;
            
            -- Replace the profile with Resource
            BEGIN
               SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_id, h_participant, h_security_id
               FROM RESOURCE_ASSIGNMENTS 
               WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE='0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1795+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1796) : cplan.sql(10496) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1__ SC_STATE = %d, h_id = %s, h_participant = %d, h_security_id = %s\n",SC_STATE,h_id,h_participant,h_security_id);//trace_flush();
            --trace_printf("A2_** h_prj_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id); trace_flush();
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER)) = PMO_CNST.SC_TEAMMEMBER Then
                  h_effort := 0;
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET 
                     PROFILE_ID = h_profile_id 
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1819+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1820) : cplan.sql(10520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET 
                     PARTICIPANT = PMO_CNST.SC_TEAMMEMBER, 
                     SECURITY_ID = h_temp_security_id, 
                     PROFILE_ID = h_profile_id 
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1843+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1844) : cplan.sql(10544) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET 
                     PARTICIPANT = PMO_STPU.BIT_OR(h_participant,PMO_CNST.SC_TEAMMEMBER), 
                     SECURITY_ID = h_security_id, 
                     PROFILE_ID = h_profile_id 
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1864+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1865) : cplan.sql(10565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_effort, 0) > 0) Then
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1883+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1884) : cplan.sql(10584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';') - 1)
                  INTO h_id,h_in FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra
                  WHERE tw.ELEMENT_ID = h_task_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND ra.PARTICIPANT = 32 
                  AND ra.RESOURCE_ID = tw.CREATED_BY AND NOT EXISTS(
                  SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1901+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1902) : cplan.sql(10602) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("2__ SC_STATE: %d, h_id: %d:%s, h_in: %s\nh_full_name: %s, h_profile_name: %s\n",SC_STATE,h_id_Ind,h_id,h_in,h_full_name,h_profile_name);
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_id IS NOT NULL)) Then
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_RESPONSIBLE_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1923+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1924) : cplan.sql(10624) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE='0',
                     PROFILE_ID = h_profile_id, PARTICIPANT = 48, SECURITY_ID=h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1942+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1943) : cplan.sql(10643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_in, h_full_name) 
                     WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1960+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1961) : cplan.sql(10661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name) WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1980+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1981) : cplan.sql(10681) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                     PROFILE_ID = h_profile_id, PARTICIPANT = 16, SECURITY_ID=h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1999+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2000) : cplan.sql(10700) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            --trace_printf("2.1__ SC_STATE: %d, h_temp_security_id: %s\n",SC_STATE,h_temp_security_id);
            
            --trace_printf("h_effort = %d, h_assigned: %d, h_tsk_assignment_id: %s\nh_prj_assignment_id: %d:%s\n",h_effort,h_assigned,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
            If Coalesce(h_effort, 0) > 0 Then
               If (Coalesce(h_fs_project, 0) = 1) AND ((h_labor_code IS NOT NULL)) Then
                  BEGIN
                     SELECT CASE WHEN FS_LABOR_CODE IS NuLL THEN 0 WHEN FS_LABOR_CODE = h_labor_code THEN 1 ELSE 2 END
                     INTO h_same_labor FROM TMT_WBS WHERE ELEMENT_ID=h_task_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2019+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2020) : cplan.sql(10720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("SC_STATE = %d, h_same_labor = %d,h_labor_code = %s\n", SC_STATE,h_same_labor,h_labor_code);
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_same_labor, 0) = 0) Then
                     BEGIN
                        UPDATE TMT_WBS SET FS_LABOR_CODE = h_labor_code WHERE ELEMENT_ID=h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2038+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2039) : cplan.sql(10739) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_same_labor, 0) = 2) Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2051+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2052) : cplan.sql(10752) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2057+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2058) : cplan.sql(10758) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_LABOR_CODE), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2060+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2061) : cplan.sql(10761) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_LABOR_CODE, h_task_id
                           , True);
                  End If;
               End If;
               
               h_tph_delete := -1;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_old_cost, h_old_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
               --trace_printf("SC_STATE: %d, h_old_cost: %.2f, h_old_sale: %.2f\n", SC_STATE,h_old_cost,h_old_sale);
               
               /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
                                       {
                                           BEGIN_SQL
                                               DELETE FROM TASK_DAY_OF_YEAR tdy WHERE tdy.DAY_YEAR IN (
                                               SELECT ta.DAY_YEAR FROM TASK_DAY_OF_YEAR ta
                                               WHERE ta.PROJECT_ID = HOST(h_project_id) AND ta.TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND NOT EXISTS (
                                               SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR r WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR = ta.DAY_YEAR));
                                           END_SQL_NOCHECK;
               
                                           BEGIN_SQL
                                               UPDATE TASK_DAY_OF_YEAR SET RESOURCE_ID = HOST(h_res_id), PARENT_ID = HOST(h_prj_assignment_id)
                                               WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
                                           END_SQL;
               
                                           h_remaining = h_effort - h_remaining;
                                           h_temp_var = int ((double) h_per_day * ((double) h_effort / (double) h_remaining) + 0.4);
                                           if ( (h_percent_units > 0)  (h_percent_units > 0) )
                                           {
                                               h_temp_var = h_temp_var * (h_percent_units / 10000.00);
                                           }
               
               //trace_printf("h_effort: %d, h_remaining: %d, h_per_day: %d, h_percent_units: %d, h_temp_var: %d\n", h_effort,h_remaining,h_per_day,h_percent_units,h_temp_var);
                                           h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                                                         SC_PARAM(h_tsk_assignment_id),
                                                                         SC_PARAM(h_start_DT),
                                                                         SC_PARAM(h_per_day),
                                                                         SC_PARAM(h_temp_var));
               
                                           BEGIN_SQL
                                               SELECT TIMESTAMP(MIN(DAY_YEAR),TIME(HOST(h_task_start_DT))), TIMESTAMP(MAX(DAY_YEAR),TIME(HOST(h_task_finish_DT)))
                                               INTO HOST(h_task_start_DT), HOST(h_task_finish_DT)
                                               FROM TASK_DAY_OF_YEAR
                                               WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
                                           END_SQL;
               
                                           BEGIN_SQL
                                               UPDATE TASK_ASSIGNMENTS SET PLANNED_START = HOST(h_task_start_DT), PLANNED_FINISH = HOST(h_task_finish_DT),
                                               RESOURCE_ID = HOST(h_res_id), RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                                               CASE WHEN HOST(h_assigned) = 0 THEN NuLL ELSE HOST(h_prj_assignment_id) END,
                                               REGULAR_COST = CASE WHEN HOST(h_donot_replace_cost) = 0 THEN HOST(h_reg_cost)  ELSE REGULAR_COST END,
                                               SPECIAL_COST = CASE WHEN HOST(h_donot_replace_cost) = 0 THEN HOST(h_spec_cost) ELSE SPECIAL_COST END,
                                               REGULAR_RATE = CASE WHEN HOST(h_donot_replace_rate) = 0 THEN HOST(h_reg_sale)  ELSE REGULAR_RATE END,
                                               SPECIAL_RATE = CASE WHEN HOST(h_donot_replace_rate) = 0 THEN HOST(h_spec_sale) ELSE SPECIAL_RATE END,
                                               ROLLUP_BITS  = BIT_OR(ROLLUP_BITS, SC_LABOR_BIT)
                                               WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
                                           END_SQL;
               
                                           h_temp_var = int((((double) h_effort / (double) h_remaining) * 10000.00) + 0.4) + SC_BIT_30;
                                           h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day, h_effort);
                                       }
                                       else
                                       {*/
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2137+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2138) : cplan.sql(10838) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_finish_date := '';
               --trace_printf("*** h_start_DT = %s, h_finish_date = %s, h_duration = %d, i_isTask: %d\n",h_start_DT,h_finish_date,h_duration,i_isTask);
               
               If i_isTask <> 0 OR (Coalesce(h_pa_count, 0) + Coalesce(h_ta_count, 0) = 1) Then
                  --trace_printf("****** Task Assignment ONLY - all effort to TA bitflag: %d, contour: %d %d, work: %d %d\n", *bitflag,h_contour,*contour,h_work,*work);
                  h_effort := effort001;
                  h_start_date := start_date001;
                  h_finish_date := finish_date001;
                  h_work := work001;
                  -- amount for countour
                  h_occurrence := occurrence;
                  -- occurrence
                  h_eunit := eunit;
                  h_unit := unit;
                  h_bitflag := bitflag001;
                  h_contour := contour001;
                  
                  If (((BITAND(Coalesce(bitflag001, 0), PMO_CNST.SC_0000FFFF)) <> 0) AND ((Coalesce(
                        h_contour, 0) <> Coalesce(contour001, 0)) OR (Coalesce(
                        h_work, 0) <> Coalesce(work001, 0)))) Then
                     If Coalesce(unit, 0) = 100 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                           CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                           REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                           SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                           REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                           SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                           UNITS_MSP    = work001, ETC = h_effort,
                           C_CONTOUR    = contour001,
                           ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2185+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2186) : cplan.sql(10886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                           CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                           REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                           SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                           REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                           SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                           C_EFFORT     = work001, ETC = h_effort,
                           C_UNIT       = unit,
                           C_CONTOUR    = contour001,
                           ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2213+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2214) : cplan.sql(10914) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     --trace_printf("SC_STATE: %d, h_contour: %d, h_unit: %d, h_work: %d\n", SC_STATE,h_contour,h_unit,h_work);
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                        CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                        REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                        SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                        REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                        SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                        ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT), ETC = h_effort
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2241+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2242) : cplan.sql(10942) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  If (Coalesce(h_contour, 0) = 0) AND (Coalesce(h_unit, 0) <> 100) Then
                     h_work := 0;
                  ElsIf Coalesce(h_unit, 0) = 100 Then
                     h_percent_units := Coalesce(h_work, 0);
                  End If;
                  
                  h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
                  
                  If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2267+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2268) : cplan.sql(10968) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 2273+1) THEN 
                        LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(2274) : cplan.sql(10974) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2276+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2277) : cplan.sql(10977) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
                  ElsIf Coalesce(h_temp_var, 0) <> 0 Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2288+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2289) : cplan.sql(10989) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2294+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2295) : cplan.sql(10995) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2297+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2298) : cplan.sql(10998) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
                  End If;
               Else
                  --trace_printf("****** Many Task Assignments ****** h_bitflag: %d, h_duration: %d\n", h_bitflag,h_duration);
                  --InitializeHost(SC_PARAM(h_bitflag),0);
                  If (Coalesce(h_duration, 0) = 0) OR (Coalesce(h_duration, 0) < Coalesce(h_per_day, 0)) Then
                     h_duration := Coalesce(h_per_day, 0);
                  Else
                     h_duration := Trunc(Coalesce(h_duration, 0) / Coalesce(h_per_day, 0));
                  End If;
                  
                  If Coalesce(h_duration, 0) < 30 Then
                     h_duration := Coalesce(h_duration, 0) * 50;
                  Else
                     h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
                  End If;
                  
                  /*if ( h_days != -1 )
                                                  {
                                                      BEGIN_SQL
                                                          SELECT TIMESTAMP(HOST(h_task_start_DT)) + HOST(h_days) DAYS
                                                          INTO HOST(h_start_date)
                                                          FROM DUAL;
                                                      END_SQL;
                                                  }*/
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                     CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                     REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                     SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                     REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                     SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                     ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2345+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2346) : cplan.sql(11046) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_work_type := 0;
               h_update := 1;
               h_cri := 0;
               
               If Coalesce(h_work, 0) = 0 Then
                  h_bitflag := PMO_CNST.SC_BIT_27;
               ElsIf isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
                  h_bitflag := PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_27);
               Else
                  h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27);
               End If;
               
               h_occurrence := 0;
               -- occurrence
               --InitializeHost(SC_PARAM(h_consider), 6);
               
               --trace_printf("AA_2: h_bitflag: %d, h_start_date: %s, h_finish_date: %s, h_duration: %d, h_per_day: %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_per_day);
               --trace_printf("h_consider: %d, h_contour: %d, h_work: %d, h_unit: %d, h_effort: %d\n", h_consider, h_contour, h_work, h_unit, h_effort);
               -- assign to the project
               h_temp_res_cost := 0.00;
               i_return := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               
               --trace_printf("assign to the task: i_return = %d, h_count = %d\n", i_return,h_count);
               
               If Coalesce(i_return, 0) = 110 Then
                  BEGIN
                     CLOSE C6a ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2388+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2389) : cplan.sql(11089) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2394+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2395) : cplan.sql(11095) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2397+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2398) : cplan.sql(11098) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
               ElsIf Coalesce(i_return, 0) <> 0 Then
                  BEGIN
                     CLOSE C6a ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2409+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2410) : cplan.sql(11110) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2415+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2416) : cplan.sql(11116) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2418+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2419) : cplan.sql(11119) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               End If;
               /*else
                                           {
                                               InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
                                               h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day, h_effort);
                                           }*/
               --}
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';','') WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2442+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2443) : cplan.sql(11143) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2.a__ SC_STATE: %d, h_task_id: %s, h_profile_name: %s\n",SC_STATE,h_task_id,h_profile_name);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES
                  SELECT h_tsk_assignment_id, c.ELEMENT_ID, c.PROFICIENCY_LEVEL_ID, c.EXPERIENCE_ID, c.PREFERENCE_LEVEL_ID, c.LAST_USED_DATETIME, '0'
                  FROM TMT_COMPETENCIES c JOIN PROFILES p ON c.ELEMENT_ID = p.ROLE_ID WHERE p.PROFILE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2462+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2463) : cplan.sql(11163) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("h_tsk_assignment_id = %s, h_profile_id = %s\n",h_tsk_assignment_id,h_profile_id);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES
                  SELECT h_tsk_assignment_id, ROLE_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_ROLE
                  FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2482+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2483) : cplan.sql(11183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_tsk_assignment_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_SKILL
                  FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2501+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2502) : cplan.sql(11202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_passed := Coalesce(h_passed, 0) + 1;
               h_total := Coalesce(h_total, 0) + Coalesce(h_effort, 0);
               
               h_temp_res_type := '1';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_profile_id, h_temp_res_type);
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_res_id, h_temp_res_type);
               
               BEGIN
                  UPDATE EFFORT_WEEK SET RESOURCE_ID = h_res_id
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2528+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2529) : cplan.sql(11229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --Update SUMMARY_TIMESHEETS
               If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_COST = h_reg_cost  ,
                     ACTUAL_SPECIAL_COST = h_spec_cost ,
                     ACTUAL_REGULAR_RATE = h_reg_sale  ,
                     ACTUAL_SPECIAL_RATE = h_spec_sale ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2553+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2554) : cplan.sql(11254) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_donot_replace_cost, 0) = 0 Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_COST = h_reg_cost  ,
                     ACTUAL_SPECIAL_COST = h_spec_cost ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2574+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2575) : cplan.sql(11275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_sale := Coalesce(h_old_sale, 0);
               ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_RATE = h_reg_sale  ,
                     ACTUAL_SPECIAL_RATE = h_spec_sale ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2597+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2598) : cplan.sql(11298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_cost := Coalesce(h_old_cost, 0);
               Else
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2618+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2619) : cplan.sql(11319) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_cost := Coalesce(h_old_cost, 0);
                  h_reg_sale := Coalesce(h_old_sale, 0);
               End If;
               
               /*InitializeHost(SC_PARAM(h_tph_delete), 0);
                                       SC_STATE = FC_TPHASE_SCHED_ASSIGN(
                                               SC_PARAM(h_project_id),
                                               SC_PARAM(h_task_id),
                                               SC_PARAM(h_tsk_assignment_id),
                                               SC_PARAM(h_SD),
                                               SC_PARAM(h_FD),
                                               SC_PARAM(h_reg_cost),
                                               SC_PARAM(h_reg_sale),
                                               SC_PARAM(h_tph_delete),
                                               SC_PARAM(h_capital_expense),
                                               SC_PARAM(h_billable));*/
               --trace_printf("SC_STATE: %d, h_reg_cost: %.2f, h_reg_sale: %.2f\n", SC_STATE,h_reg_cost,h_reg_sale);
            End If;
         End If;
      End If;
      
      PMO_DB2M.SC_STATE := FC_D_RA_AFTER_REPLACE(h_project_id, h_profile_id, h_rec_user);
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE C6a ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2656+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2657) : cplan.sql(11357) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("*** h_passed = %d\n",h_passed);
   h_count := 0;
   If Coalesce(h_passed, 0) > 0 Then
      h_effort := effort001;
      -- total effort for period
      
      --trace_printf("*** h_effort = %d, h_total = %d\n", h_effort, h_total);
      If Coalesce(h_effort, 0) >= Coalesce(h_total, 0) Then
         h_effort := Coalesce(h_effort, 0) - Coalesce(h_total, 0);
         If (Coalesce(h_effort, 0) < Coalesce(h_planned_effort, 0)) AND (Coalesce(h_assigned, 0) > 0) 
                                     AND ((h_prj_assignment_id IS NOT NULL)) Then
            h_effort := Coalesce(h_planned_effort, 0);
         End If;
         --trace_printf("*** h_effort = %d, h_assigned = %d, h_prj_assignment_id = %d %s\n",h_effort,h_assigned,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_assigned, 0) > 0) 
               AND ((h_prj_assignment_id IS NOT NULL)) Then
            h_start_date := start_date001;
            h_finish_date := finish_date001;
            h_work := work001;
            -- amount for countour
            h_occurrence := occurrence;
            -- occurrence
            h_eunit := eunit;
            h_unit := unit;
            h_bitflag := bitflag001;
            h_contour := contour001;
            
            If Coalesce(contour001, 0) = 0 Then
               h_work := 0;
            End If;
            
            h_temp_var := FC_CALC_DURATION(h_start_DT, h_finish_DT, Coalesce(h_eunit, 0), Coalesce(h_unit, 0)
                               , Coalesce(h_consider, 0), Coalesce(h_bitflag, 0), Coalesce(
                               h_contour, 0), Coalesce(h_occurrence, 0), Coalesce(h_per_day, 0)
                               , Coalesce(h_per_week, 0), h_work, h_effort, h_duration);
            
            If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 2700+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(2701) : cplan.sql(11401) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2703+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2704) : cplan.sql(11404) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            ElsIf Coalesce(h_temp_var, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2708+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2709) : cplan.sql(11409) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2711+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2712) : cplan.sql(11412) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
            End If;
            
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PA_SUM';
            PMO_DB2M.SC_STATE := 0;
            
            BEGIN
               INSERT INTO PMO_TMP_PA_SUM SELECT DAY_YEAR, SUM(SCHED_MIN)
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id
               GROUP BY DAY_YEAR;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2735+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2736) : cplan.sql(11436) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --              if ( IsNullSpParam(ind_a_restoprof, finish_date))
            --strcpy(h_finish_date, "");
            h_finish_date := '';
            --trace_printf("*** h_start_date = %s, h_finish_date = %s, h_duration = %d\n",h_start_date,h_finish_date,h_duration);
            
            h_work_type := 0;
            h_update := 0;
            h_cri := 0;
            h_type := 1;
            
            If Coalesce(h_work, 0) > 0 Then
               h_work := PMO_FUNC.BIT_OR(Coalesce(h_work, 0), PMO_CNST.SC_BIT_30);
               h_bitflag := 1;
            Else
               h_bitflag := 0;
            End If;
            --trace_printf("Remaining to PA: h_bitflag = %d, h_start_date = %s, h_finish_date = %s, h_duration = %d, h_work_type = %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_work_type);
            -- assign to the project
            h_temp_res_cost := 0.00;
            i_return := FC_SEARCH_2(h_prj_assign_id, h_project_id, h_project_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
            
            --trace_printf("assign to the project: i_return = %d, h_count = %d\n", i_return,h_count);
            
            If Coalesce(i_return, 0) = 110 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2768+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2769) : cplan.sql(11469) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2771+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2772) : cplan.sql(11472) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
            ElsIf Coalesce(i_return, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2776+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2777) : cplan.sql(11477) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2779+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2780) : cplan.sql(11480) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                     , True);
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                  RESOURCE_ID  = h_res_id, RESOURCE_TYPE = '0',
                  REGULAR_COST = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_cost  * h_rate_echange ELSE REGULAR_COST END,
                  SPECIAL_COST = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_cost * h_rate_echange ELSE SPECIAL_COST END,
                  REGULAR_RATE = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_sale  * h_rate_echange ELSE REGULAR_RATE END,
                  SPECIAL_RATE = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_sale * h_rate_echange ELSE SPECIAL_RATE END
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2802+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2803) : cplan.sql(11503) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      BEGIN
         SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGNMENTS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2819+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2820) : cplan.sql(11520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_count: %d, h_project_id: %s, h_profile_name: %s\n", SC_STATE,h_count,h_project_id,h_profile_name);
      
      If Coalesce(h_count, 0) = 0 Then
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';','')
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2839+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2840) : cplan.sql(11540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   h_from_prj := 0;
   
   --trace_printf("7_* h_old_prj_assign_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_old_prj_assign_id_Ind,h_old_prj_assign_id,h_assigned,h_from_prj);
   If (h_old_prj_assign_id IS NOT NULL) Then
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASSIGNMENT_BIT) WHERE ELEMENT_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2862+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2863) : cplan.sql(11563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("OLD_WBS_SC_LABOR_BIT SC_STATE: %d\n", SC_STATE); //trace_flush();
      
      If Coalesce(h_assigned, 0) = 0 Then
         BEGIN
            SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_old_prj_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2878+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2879) : cplan.sql(11579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("OLD_TA_COUNT h_count: %d\n", h_count);
         
         If Coalesce(h_count, 0) = 0 Then
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND
               ELEMENT_ID = h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2898+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2899) : cplan.sql(11599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2915+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2916) : cplan.sql(11616) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If Coalesce(h_count, 0) > 0 Then
         -- CALL UPDATE assign string function
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_old_prj_assign_id, h_temp_var, 0, 0);
         
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_start_DT, h_finish_DT, h_effort, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_old_prj_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2937+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2938) : cplan.sql(11638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("OLD_* SC_STATE: %d, h_start_D = %d:%s, h_finish_D = %d:%s\n", SC_STATE, h_start_D_Ind,h_start_D, h_finish_D_Ind,h_finish_D);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If Coalesce(h_billable, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_START = h_start_DT,
                  ESTIMATED_PLANNED_FINISH = h_finish_DT,
                  ESTIMATED_PLANNED_EFFORT = h_effort,
                  ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_old_cost,
                  ESTIMATED_PLANNED_REVENUE = 0.00
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2963+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2964) : cplan.sql(11664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_START = h_start_DT,
                  ESTIMATED_PLANNED_FINISH = h_finish_DT,
                  ESTIMATED_PLANNED_EFFORT = h_effort,
                  ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_old_cost,
                  ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_old_sale
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2986+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2987) : cplan.sql(11687) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("OLD_PA SC_STATE: %d, h_profile_id: %s, h_start_DT = %d:%s, h_finish_DT = %d:%s, h_effort: %d\n", SC_STATE, h_profile_id,h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_effort);
            
            If (SubStr(h_pa_timephase, 1, 1) = 'Y') Then
               h_tph_delete := 0;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
               --trace_printf("OLD_TPHASE_SCHED_ASSIGN SC_STATE = %d, h_old_prj_assign_id = %s, h_total = %d\n",SC_STATE, h_old_prj_assign_id, h_total);
            End If;
         End If;
         
         If ((h_prj_assignment_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_old_prj_assign_id
               , h_prj_assignment_id) = 0) Then
            h_from_prj := 1;
         End If;
      End If;
   End If;
   
   --trace_printf("9_* h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   If (Coalesce(h_assigned, 0) > 0) AND ((h_prj_assignment_id IS NOT NULL)) Then
      --Check if affect to project is chosen and there are still assignments from projects
      If Coalesce(h_donot_replace_bits, 0) = 1 Then
         BEGIN
            SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_prj_assignment_id
            AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3023+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3024) : cplan.sql(11724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) > 0 Then
            --Nullify PROJECT_ASSIGNMENT_ID in TA and Parent_id in TDY
            BEGIN
               UPDATE TASK_ASSIGNMENTS
               SET PROJECT_ASSIGNMENT_ID = Null
               WHERE PROJECT_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_prj_assignment_id
               AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3045+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3046) : cplan.sql(11746) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_DAY_OF_YEAR
               SET PARENT_ID = TSK_ASSIGNMENT_ID
               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id
               AND RESOURCE_ID = h_profile_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3065+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3066) : cplan.sql(11766) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      -- CALL UPDATE assign string function
      h_temp_var := 3;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
      
      BEGIN
         SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         INTO h_start_DT, h_finish_DT, h_effort, h_start_D, h_finish_D, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3086+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3087) : cplan.sql(11787) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("NEW_* SC_STATE: %d, h_start_D = %d:%s, h_finish_D = %d:%s, h_billable: %d\n", SC_STATE, h_start_D_Ind,h_start_D, h_finish_D_Ind,h_finish_D,h_billable);
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If Coalesce(h_billable, 0) = 0 Then
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START  = h_start_DT,
               ESTIMATED_PLANNED_FINISH = h_finish_DT,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST   = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = 0.00
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3112+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3113) : cplan.sql(11813) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START  = h_start_DT,
               ESTIMATED_PLANNED_FINISH = h_finish_DT,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST   = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_reg_sale
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3135+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3136) : cplan.sql(11836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("NEW_PA SC_STATE: %d, h_res_id: %s, h_start_DT = %d:%s, h_finish_DT = %d:%s, h_effort: %d\n", SC_STATE, h_res_id,h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_effort);
         
         If (SubStr(h_pa_timephase, 1, 1) = 'Y') AND (Coalesce(h_from_prj, 0) = 0) Then
            h_tph_delete := 0;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id
                                            , h_start_D, h_finish_D, h_reg_cost, h_reg_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
            --trace_printf("NEW_TPHASE_SCHED_ASSIGN SC_STATE = %d, h_prj_assignment_id = %s\n",SC_STATE, h_prj_assignment_id);
         End If;
      End If;
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   --trace_printf("RETURN_STATUS: SC_STATE = %d,h_prj_assignment_id = %s\n",SC_STATE,h_prj_assignment_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RESTOPROF', 3161+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RESTOPROF(3162) : cplan.sql(11862) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_prj_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_prj_assignment_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3168+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3169) : cplan.sql(11869) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3180+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3181) : cplan.sql(11881) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RESTOPROF;
ALTER FUNCTION SP_A_RESTOPROF COMPILE DEBUG;

/*
#Stored Procedure  :SP_U_RATES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_U_RATES ==
CREATE OR REPLACE FUNCTION SP_U_RATES
(
     id VARCHAR2
   , reg_cost NUMBER
   , spec_cost NUMBER
   , reg_rate NUMBER
   , spec_rate NUMBER
   , rate_adjustment001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_reg_cost NUMBER := 0;
   h_reg_rate NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_rate NUMBER := 0;
   h_rate_adjustment PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RATES'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RATES(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(rate_adjustment001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RATES(35) : cplan.sql(11940) : ' || SYSTIMESTAMP);
   END IF; 

   If (id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_RATES', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_RATES(40) : cplan.sql(11945) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_RATES(43) : cplan.sql(11948) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   h_flag := 1;
   --update rates_edited
   
   h_id := id;
   h_res_id := res_id001;
   h_reg_cost := reg_cost;
   h_spec_cost := spec_cost;
   h_reg_rate := reg_rate;
   h_spec_rate := spec_rate;
   h_rate_adjustment := rate_adjustment001;
   
   PMO_DB2M.SC_STATE := FC_U_RATES(h_id, h_reg_cost, h_spec_cost, h_reg_rate, h_spec_rate
                                , h_rate_adjustment, h_res_id, h_flag);
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RATES', 66+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RATES(67) : cplan.sql(11972) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES', 73+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES(74) : cplan.sql(11979) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES', 85+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES(86) : cplan.sql(11991) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RATES;
ALTER FUNCTION SP_U_RATES COMPILE DEBUG;

/*
#Stored Procedure  :SP_U_RATES_EDITED
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_U_RATES_EDITED ==
CREATE OR REPLACE FUNCTION SP_U_RATES_EDITED
(
     id VARCHAR2
   , rates_edited001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_reg_cost NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_reg_rate NUMBER := 0;
   h_spec_rate NUMBER := 0;
   h_flag PLS_INTEGER := 0;
   h_old_reg_cost NUMBER := 0;
   h_old_reg_sale NUMBER := 0;
   h_old_spec_cost NUMBER := 0;
   h_old_spec_rate NUMBER := 0;
   h_rate_id VARCHAR2(32);
   h_rate_adjustment PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RATES_EDITED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RATES_EDITED'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RATES_EDITED(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(rates_edited001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES_EDITED', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RATES_EDITED(32) : cplan.sql(12047) : ' || SYSTIMESTAMP);
   END IF; 

   h_resource_id := res_id001;
   
   h_flag := 0;
   
   BEGIN
      SELECT REGULAR_COST, REGULAR_RATE,
      SPECIAL_COST, SPECIAL_RATE, RATE_ADJUSTMENT
      INTO   h_old_reg_cost, h_old_reg_sale ,
      h_old_spec_cost,h_old_spec_rate, h_rate_adjustment
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(52) : cplan.sql(12067) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT RATE_ID INTO h_rate_id
      FROM RESOURCES
      WHERE RESOURCE_ID = res_id001;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(67) : cplan.sql(12082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT REGULAR_COST, REGULAR_SALE,
      SPECIAL_COST, SPECIAL_SALE
      INTO   h_reg_cost, h_reg_rate ,
      h_spec_cost,h_spec_rate
      FROM RATES
      WHERE RATE_ID = h_rate_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(85) : cplan.sql(12100) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS
      SET RATES_EDITED = BITAND(RATES_EDITED,rates_edited001)
      WHERE TSK_ASSIGNMENT_ID = id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 103+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(104) : cplan.sql(12119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (BITAND(Coalesce(rates_edited001, 0), 1)) > 0 Then
      h_old_reg_cost := h_reg_cost;
      h_old_spec_cost := h_spec_cost;
   End If;
   If (BITAND(Coalesce(rates_edited001, 0), 2)) > 0 Then
      h_old_reg_sale := h_reg_rate;
      h_old_spec_rate := h_spec_rate;
   End If;
   
   PMO_DB2M.SC_STATE := FC_U_RATES(h_id, h_old_reg_cost, h_old_spec_cost, h_old_reg_sale
                                , h_old_spec_rate, h_rate_adjustment, h_resource_id, h_flag);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RATES_EDITED', 121+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RATES_EDITED(122) : cplan.sql(12137) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 128+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(129) : cplan.sql(12144) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 140+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(141) : cplan.sql(12156) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RATES_EDITED;
ALTER FUNCTION SP_U_RATES_EDITED COMPILE DEBUG;

/*
#Stored Procedure  :SP_U_TIMECODE2
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_U_TIMECODE2 ==
CREATE OR REPLACE FUNCTION SP_U_TIMECODE2
(
     assign_id VARCHAR2
   , id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_TIMECODE2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_TIMECODE2'), '-- ' || SYSTIMESTAMP, ':r := SP_U_TIMECODE2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TIMECODE2', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_TIMECODE2(18) : cplan.sql(12198) : ' || SYSTIMESTAMP);
   END IF; 

   If ((assign_id IS NULL)) OR ((id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_TIMECODE2', 22+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_TIMECODE2(23) : cplan.sql(12203) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TIMECODE2', 25+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_TIMECODE2(26) : cplan.sql(12206) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Update user record
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET TIMECODE2_ID = id WHERE TSK_ASSIGNMENT_ID = assign_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 43+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(44) : cplan.sql(12224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_TIMECODE2', 49+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_TIMECODE2(50) : cplan.sql(12230) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || assign_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, assign_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 56+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(57) : cplan.sql(12237) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 68+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(69) : cplan.sql(12249) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_TIMECODE2;
ALTER FUNCTION SP_U_TIMECODE2 COMPILE DEBUG;

/*
#Stored Procedure  :SP_S_PROFILE
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
 $$bitflag       Description
 ----------------------------------------------------------------------------
   0	-  WBS Project global view: Resources  Profile with Competencies  Skills
   1	-  WBS Project global view: Resources  Profiles
   6	-  CRI Project global view: Resources  Profiles
   2,3	-  WBS  CRI - Assignments View
   4	-  WBS  CRI - All assignments with effort (no participants, no project assignments) in the current project
   5	-  WBS  CRI - All participants (no effort) in the current project
   8	-  WBS  CRI - All assignments (effort, participants, project assignments) in the current project
   9	-  WBS  CRI - All assignments for a Resource / Profile under a element (project, deliverable, task)
   7	-  WBS Project Assignments
   11	-  DOC Assignments View (SC_TABLE_DOC, SC_TABLE_POL_DOC, SC_TABLE_RES_DOC, SC_TABLE_AST_DOC,SC_TABLE_CLN_DOC,SC_TABLE_VDR_DOC)
   12	-  WBS  CRI - display nickname for All assignments (effort, participants, project assignments) in the current project
   13	-  Returns nickname for All assignments (effort, participants, project assignments) in the current pool

#Returns           :
#Errors            :
*/

Prompt == FUNCTION SP_S_PROFILE ==
CREATE OR REPLACE FUNCTION SP_S_PROFILE
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_in VARCHAR2(255);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_level_state PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C7751A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751B PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751C PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751T PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751D PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751E PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751F PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751G PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751H PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751J PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751L PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751M PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751N PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C77513N PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      cln_ast_res PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_PROFILE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_PROFILE'), '-- ' || SYSTIMESTAMP, ':r := SP_S_PROFILE(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 75+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_PROFILE(76) : cplan.sql(12364) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 82+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(83) : cplan.sql(12371) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 85+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(86) : cplan.sql(12374) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   -- Parse parameters to local variables
   
   h_id := id;
   h_bitflag := bitflag001;
   h_rec_user := res_id001;
   --trace_printf("h_id: %d %s\n",h_id_Ind,h_id);//trace_flush();
   
   If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 3) Then
      BEGIN
         SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480), B.LEVEL_STATE
         INTO h_project_id, h_type_id, h_per_day, h_level_state
         FROM TMT_WBS A, TMT_WBS B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 108+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(109) : cplan.sql(12397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("WBS_SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480)
            INTO h_project_id, h_type_id, h_per_day
            FROM TMT_WBS A, TMT_CRI B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJECT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 125+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(126) : cplan.sql(12414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CRI_SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT 480 INTO h_per_day FROM TMT_POOLS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 140+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(141) : cplan.sql(12429) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(148) : cplan.sql(12436) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 150+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(151) : cplan.sql(12439) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
            End If;
         End If;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 10 Then
      BEGIN
         SELECT PROJECT_ID INTO h_project_id
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 166+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(167) : cplan.sql(12455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 173+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(174) : cplan.sql(12462) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 176+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(177) : cplan.sql(12465) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
      
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_state
         FROM TMT_WBS WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(191) : cplan.sql(12479) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 13 Then
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_POOLS WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 203+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(204) : cplan.sql(12492) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 210+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(211) : cplan.sql(12499) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 213+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(214) : cplan.sql(12502) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
      
      h_per_day := 480;
   ElsIf (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND (Coalesce(h_bitflag, 0) <> 11) 
                AND (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_ASSET_ASSIGNED_RESOURCES) AND (Coalesce(
                h_bitflag, 0) <> PMO_CNST.SC_CLIENT_ASSIGNED_RESOURCES) Then
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_state
         FROM TMT_WBS WHERE ELEMENT_ID = id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(232) : cplan.sql(12520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 238+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(239) : cplan.sql(12527) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 241+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(242) : cplan.sql(12530) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
   End If;
   --trace_printf("SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
   --trace_printf("\n 7390 h_bitflag %d, h_rank: %s",h_bitflag,h_rank); trace_flush();
   
   If Coalesce(h_bitflag, 0) = 0 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_ASGNDRES_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_ASGNDRES_INFO
         (
         RESOURCE_ID,
         PARENT_ID,
         RATE_ID,
         PROFICIENCY_LEVEL_ID,
         PREFERENCE_LEVEL_ID,
         EXPERIENCE_ID,
         LAST_USED_DATETIME
         )
         SELECT DISTINCT
         a.RESOURCE_ID,
         r.PARENT_ID,
         r.RATE_ID,
         r.PROFICIENCY_LEVEL_ID,
         r.PREFERENCE_LEVEL_ID,
         r.EXPERIENCE_ID,
         r.LAST_USED_DATETIME
         FROM RESOURCE_ASSIGNMENTS a JOIN TMT_COMPETENCIES r ON a.RESOURCE_ID=r.RESOURCE_ID
         WHERE PROJECT_ID = h_id AND RESOURCE_TYPE = '0';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 285+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(286) : cplan.sql(12574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROF_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_PROF_INFO (PROFILE_ID)
         SELECT DISTINCT RESOURCE_ID
         FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND RESOURCE_TYPE = '1';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 309+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(310) : cplan.sql(12598) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751A FOR SELECT CASE WHEN EXISTS(SELECT * FROM TASK_ASSIGNMENTS ta WHERE ta.PROJECT_ID=h_id AND ta.RESOURCE_ID = RES.RESOURCE_ID) THEN 13 ELSE 185 END AS RES_TYPE,
      RES.RESOURCE_ID, p.FULL_NAME AS ELEMENT_NAME, p.STATUS1_ID, p.E_MAIL_ADDRESS, d.TYPE_ID, d.ELEMENT_NAME AS ROLE_NAME,
      RES.PROFICIENCY_LEVEL_ID, RES.PREFERENCE_LEVEL_ID, RES.EXPERIENCE_ID, RES.LAST_USED_DATETIME AS LAST_USED,rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_ASGNDRES_INFO RES JOIN RESOURCES p ON RES.RESOURCE_ID=p.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=RES.PARENT_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=RES.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID, p.PROFILE_NAME AS ELEMENT_NAME, 0 AS STATUS1_ID, '' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID,r.EXPERIENCE_ID,r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE,'' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF  JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PROFILE_NAME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      JOIN RESOURCE_ASSIGN_SKILLS s ON p.PROFILE_ID=s.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.SKILL_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PROFILE_NAME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      JOIN RESOURCE_ASSIGN_ROLES s ON p.PROFILE_ID=s.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.ROLE_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      ORDER BY RES_TYPE, RESOURCE_ID, TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 358+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(359) : cplan.sql(12647) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751A;
   ElsIf (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_DATA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_DATA (RESOURCE_ID, TYPE_ID)
         SELECT DISTINCT
         ra.RESOURCE_ID,
         CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END AS TYPE_ID
         FROM RESOURCE_ASSIGNMENTS ra LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=h_id AND ra.RESOURCE_ID = ta.RESOURCE_ID
         WHERE ra.PROJECT_ID=h_id AND ra.RESOURCE_TYPE='0' AND ta.PROJECT_ASSIGNMENT_ID IS NuLL AND
         NOT EXISTS(SELECT * FROM TASK_ASSIGNMENTS t WHERE ra.RESOURCE_ID = t.RESOURCE_ID AND t.TASK_ID=h_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 384+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(385) : cplan.sql(12673) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME)
         SELECT DISTINCT r.PROFILE_ID ,r.PROFILE_NAME
         FROM RESOURCE_ASSIGNMENTS a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID 
         JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID 
         JOIN TASK_ASSIGNMENTS t ON t.RES_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID AND t.PROJECT_ASSIGNMENT_ID IS NuLL
         WHERE a.PROJECT_ID=h_id AND a.ELEMENT_TYPE>1;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 409+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(410) : cplan.sql(12698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751B FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,a.TYPE_ID,p.E_MAIL_ADDRESS,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_DATA a JOIN RESOURCES p ON a.RESOURCE_ID=p.RESOURCE_ID
      
      UNION ALL
      
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP
      ORDER BY TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 429+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(430) : cplan.sql(12718) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751B;
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      OPEN C7751C FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,
      a.PROFILE_ID, p.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID,a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.PLANNED_START,t.PLANNED_FINISH,
      t.PLANNED_WORK,t.START_DT,t.FINISH_DT,h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,'' AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,
      t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID,TC.ELEMENT_NAME AS TIMECODE2, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, BILLABLE,
      t.ASSIGNMENT_DELAY
      FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON t.TASK_ID = h_id AND a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN PROFILES p ON a.RESOURCE_ID = p.PROFILE_ID LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
      WHERE a.ELEMENT_ID = h_id AND a.RESOURCE_TYPE='1' ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 445+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(446) : cplan.sql(12734) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751C;
   ElsIf Coalesce(h_bitflag, 0) = 3 Then
      If isTask(Coalesce(h_type_id, 0)) <> 0 Then
         h_count := 0;
         BEGIN
            SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 459+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(460) : cplan.sql(12748) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) = 0 Then
            h_bit_flag := '1';
         Else
            BEGIN
               SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (
               SELECT 'X' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_id AND BIT_FLAG = '0');
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 476+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(477) : cplan.sql(12765) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (
                  SELECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv
                  ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURCE_ID
                  WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = h_id AND
                  (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     ) <= apv.WEEK_OF);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 494+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(495) : cplan.sql(12783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_count, 0) = 0 Then
                  h_bit_flag := '2';
               Else
                  h_bit_flag := '1';
               End If;
            Else
               h_bit_flag := '1';
            End If;
         End If;
         --trace_printf("h_project_id: %s,h_id: %s, h_bit_flag: %s, h_level_state: %d\n",h_project_id,h_id,h_bit_flag,h_level_state);
         
         --at the task level in staffing view return empty string as security is and name
         --trace_printf("1___SC_WBS_TASK__SC_CRI_TASK__________(h_bitflag == 3)\n");
         
         OPEN C7751T FOR SELECT t.ELEMENT_ID, a.RESOURCE_ID, t.ELEMENT_NAME, t.TYPE_ID, '' AS RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
         t.SD, t.FD, t.START_DT AS PLANNED_START, t.FINISH_DT AS PLANNED_FINISH,
         t.SCHED_EFFORT + t.PLANNED_EFFORT AS PLANNED_WORK, t.SCHED_EFFORT + t.PLANNED_EFFORT - t.WORK_TODATE AS REMAINING,
         0 UNITS_PERCENT, t.MILESTONE_PERCENT,
         CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(t.SCHED_EFFORT + t.PLANNED_EFFORT))*10000) END AS WORK_PERCENT,
         TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_FINISH_DT,0 AS BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,t.PROPOSED_START, t.PROPOSED_FINISH, t.PROPOSED_EFFORT AS PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, 0 AS WORK_LOCATION, '' AS SECURITY_ID, '' AS SECURITY_NAME,
         '' AS CRI_SECURITY_ID, '' AS CRI_SECURITY_NAME, 0.00 AS REGULAR_COST,0.00 AS REGULAR_RATE,0.00 AS SPECIAL_COST,0.00 AS SPECIAL_RATE,
         '' AS FS_LABOR_CODE,t.REC_STATUS,'' AS ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_EFFORT AS BASELINE1_WORK,
         '' AS PROJECT_ASSIGNMENT_ID,'' AS TIMECODE2_ID, '' AS TIMECODE2_NAME, 0 AS T_UNIT, 0 AS C_EFFORT, 0 AS C_UNIT, 0 AS C_CONTOUR, 0 AS CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE, 0 AS CAPITAL_EXPENSE,
         0 AS ASSIGNMENT_DELAY, h_bit_flag AS BIT_FLAG, 0 AS RATE_ADJUSTMENT, t.ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_START ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_FINISH ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_DURATION ELSE 0 END AS LEVEL_EFFORT, t.WORK_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG,
         '' AS LABOR_COST_CC_ID, '' AS LABOR_COST_CC_NAME, '' AS LABOR_REVENUE_CC_ID, '' AS LABOR_REVENUE_CC_NAME, 'N' AS DEFAULT_LABOR_FLAG,
         '' AS CONTRACT_ID, '' AS CONTRACT_NAME
         FROM TMT_WBS t LEFT JOIN RESOURCE_ASSIGNMENTS a ON a.ELEMENT_ID = t.ELEMENT_ID AND a.RESOURCE_ID = h_rec_user
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         WHERE t.PROJECT_ID = h_project_id AND t.ELEMENT_ID = h_id
         UNION ALL
         SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID, a.RESOURCE_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID)
         ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
         CASE WHEN t.TSK_ASSIGNMENT_ID IS NuLL THEN 185 WHEN a.RESOURCE_TYPE='1' THEN 41 ELSE 13 END AS TYPE_ID,
         a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT, t.UNRESOLVED_FINISH AS FINISH_DT,
         CASE WHEN t.SD = PMO_CNST.SC_UFS OR t.SD = PMO_CNST.SC_FS THEN PMO_CNST.SC_SS ELSE t.SD END AS SD, CASE WHEN t.FD = PMO_CNST.SC_FF THEN PMO_CNST.SC_SF ELSE t.FD END AS FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
         CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS PLANNED_WORK,
         CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK - WORK_TODATE ELSE PLANNED_WORK END AS REMAINING,
         CASE WHEN t.ASSIGN_UNITS > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCENT,
         0 AS MILESTONE_PERCENT,
         CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))*10000) END AS WORK_PERCENT,
         t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
         COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
         CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
         ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
         t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
         CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_START,t.UNRESOLVED_START) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
         CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINISH,t.UNRESOLVED_FINISH) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
         CASE WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = 0 THEN t.ETC ELSE t.LEVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 1 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG,
         T.LABOR_COST_CC_ID,
         EC.ELEMENT_NAME AS LABOR_COST_CC_NAME,
         T.LABOR_REVENUE_CC_ID,
         CC.ELEMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR_FLAG,
         T.CONTRACT_ID, CTR.ELEMENT_NAME AS CONTRACT_NAME
         FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
         LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
         LEFT OUTER JOIN TMT_CHARGECODES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID
         LEFT OUTER JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID
         LEFT OUTER JOIN TMT_CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID
         WHERE a.PROJECT_ID = h_project_id AND a.ELEMENT_ID = h_id AND a.REC_STATUS NOT IN ('L','D')
         ORDER BY TOP_ID, ELEMENT_NAME;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 573+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(574) : cplan.sql(12862) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751T;
      Else
         --trace_printf("2_______________(h_bitflag == 3)\n");
         
         OPEN C7751D FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,a.RESOURCE_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID)
         ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
         CASE WHEN t.TSK_ASSIGNMENT_ID IS NuLL THEN 185 WHEN a.RESOURCE_TYPE='1' THEN 41 ELSE 13 END AS TYPE_ID,
         a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
         t.SD, t.FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
         BEST AS PLANNED_WORK, CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS REMAINING,
         t.UNITS_MSP AS UNITS_PERCENT, t.MILESTONE_PERCENT, 0 AS WORK_PERCENT,
         t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
         COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
         CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
         ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
         t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, 0 AS DURATION,
         t.LEVEL_START, t.LEVEL_FINISH, t.LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,
         CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND (t.LEVEL_START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FINISH) THEN '1' ELSE '0' END AS LEVEL_AFFECTED_FLAG,
         T.LABOR_COST_CC_ID,
         EC.ELEMENT_NAME AS LABOR_COST_CC_NAME,
         T.LABOR_REVENUE_CC_ID,
         CC.ELEMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR_FLAG,
         T.CONTRACT_ID, CTR.ELEMENT_NAME AS CONTRACT_NAME
         FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
         LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
         LEFT OUTER JOIN TMT_CHARGECODES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID
         LEFT OUTER JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID
         LEFT OUTER JOIN TMT_CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID
         WHERE a.REC_STATUS NOT IN ('L','D') AND a.ELEMENT_ID = h_id
         ORDER BY TYPE_ID, ELEMENT_NAME;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 613+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(614) : cplan.sql(12902) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751D;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 4 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_SET';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_SET
         (
         RESOURCE_ID,
         RESOURCE_TYPE ,
         PLANNED_START ,
         PLANNED_FINISH ,
         PLANNED_EFFORT ,
         START_DATE ,
         FINISH_DATE ,
         WORK ,
         REMAINING
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START) AS PLANNED_START,
         MAX(PLANNED_FINISH) AS PLANNED_FINISH,
         SUM(PLANNED_WORK) AS PLANNED_EFFORT,
         MIN(SCHED_START) AS START_DATE,
         MAX(SCHED_FINISH) AS FINISH_DATE,
         SUM(SCHED_WORK) AS WORK,
         SUM(ETC) AS REMAINING
         FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_id AND PROJECT_ASSIGNMENT_ID IS NuLL GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 656+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(657) : cplan.sql(12945) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, PLANNED_START, PLANNED_FINISH, PLANNED_EFFORT,START_DATE, FINISH_DATE,
         WORK, REMAINING)
         SELECT DISTINCT r.PROFILE_ID,r.PROFILE_NAME,
         a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING
         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 680+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(681) : cplan.sql(12969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751E FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      
      UNION ALL
      
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP ORDER BY TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 697+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(698) : cplan.sql(12986) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751E;
   ElsIf Coalesce(h_bitflag, 0) = 5 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_SET';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_SET
         (
         RESOURCE_ID,
         RESOURCE_TYPE ,
         PLANNED_START ,
         PLANNED_FINISH ,
         PLANNED_EFFORT ,
         START_DATE ,
         FINISH_DATE ,
         WORK ,
         REMAINING
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START) AS PLANNED_START,
         MAX(PLANNED_FINISH) AS PLANNED_FINISH,
         SUM(PLANNED_WORK) AS PLANNED_EFFORT,
         MIN(SCHED_START) AS START_DATE,
         MAX(SCHED_FINISH) AS FINISH_DATE,
         SUM(SCHED_WORK) AS WORK,
         SUM(ETC) AS REMAINING
         FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_id AND PROJECT_ASSIGNMENT_ID IS NuLL GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 739+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(740) : cplan.sql(13028) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, RESOURCE_TYPE, TYPE_ID, E_MAIL_ADDRESS, PLANNED_START, PLANNED_FINISH, PLANNED_EFFORT,START_DATE, FINISH_DATE,
         WORK, REMAINING, FS_LABOR_CODE)
         SELECT DISTINCT r.PROFILE_ID ,r.PROFILE_NAME ,'1',41 ,'',
         a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         ''
         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
         UNION ALL
         SELECT DISTINCT ra.RESOURCE_ID ,p.FULL_NAME ,'0',185 ,p.E_MAIL_ADDRESS,
         TO_TIMESTAMP('1899-12-31 00:00:00') , TO_TIMESTAMP('1899-12-31 00:00:00') ,0 ,
         TO_TIMESTAMP('1899-12-31 00:00:00') , TO_TIMESTAMP('1899-12-31 00:00:00') , 0 , 0 ,p.FS_LABOR_CODE
         FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
         WHERE ra.PROJECT_ID=h_id AND ra.RESOURCE_ID = p.RESOURCE_ID
         AND NOT EXISTS(SELECT ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta
         WHERE ta.RES_ASSIGNMENT_ID IN (SELECT a.RES_ASSIGNMENT_ID
         FROM RESOURCE_ASSIGNMENTS a WHERE a.PROJECT_ID = ra.PROJECT_ID AND a.RESOURCE_ID = ra.RESOURCE_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 773+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(774) : cplan.sql(13062) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751F FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      UNION ALL
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,TYPE_ID,E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP WHERE RESOURCE_TYPE='1' AND TYPE_ID = 41
      UNION ALL
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,TYPE_ID,E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,
      START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP
      WHERE RESOURCE_TYPE = '0' AND TYPE_ID = 185
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 796+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(797) : cplan.sql(13085) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751F;
      
      /*BEGIN_SQL
                  DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_RES_SET(
                      RESOURCE_ID             CHAR(32),
                      PROJECT_ASSIGNMENT_ID   CHAR(32),
                      PLANNED_START           TIMESTAMP,
                      PLANNED_FINISH          TIMESTAMP,
                      PLANNED_EFFORT          BIGINT,
                      START_DATE              TIMESTAMP,
                      FINISH_DATE             TIMESTAMP,
                      WORK                    BIGINT,
                      REMAINING               BIGINT
                  ) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
              END_SQL_NOCHECK;
      
              BEGIN_SQL
                  INSERT INTO session.PMO_TMP_RES_SET
                  (
                    RESOURCE_ID,
                    PROJECT_ASSIGNMENT_ID,
                    PLANNED_START,
                    PLANNED_FINISH,
                    PLANNED_EFFORT,
                    START_DATE,
                    FINISH_DATE,
                    WORK,
                    REMAINING
                  )
                      SELECT
                      RESOURCE_ID,
                      PROJECT_ASSIGNMENT_ID,
                      MIN(PLANNED_START) AS PLANNED_START,
                      MAX(PLANNED_FINISH) AS PLANNED_FINISH,
                      SUM(PLANNED_WORK) AS PLANNED_EFFORT,
                      MIN(SCHED_START) AS START_DATE,
                      MAX(SCHED_FINISH) AS FINISH_DATE,
                      SUM(SCHED_WORK) AS WORK,
                      SUM(ETC) AS REMAINING
                      FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=HOST(h_id) AND RESOURCE_TYPE = '0' GROUP BY RESOURCE_ID, PROJECT_ASSIGNMENT_ID;
              END_SQL;
      
              BEGIN_SQL
                  DECLARE C7751F CURSOR WITH HOLD FOR
      
                  SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
                  a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,HOST(h_per_day) AS MINUTE_PER_DAY,
                  '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,
                  '' AS RANK, a.PROJECT_ASSIGNMENT_ID
                  FROM session.PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_ID=p.RESOURCE_ID
                  UNION ALL
                  SELECT DISTINCT ra.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,185 AS TYPE_ID,p.E_MAIL_ADDRESS,
                  SC_1899_12_31_00_00_00 AS PLANNED_START, SC_1899_12_31_00_00_00 AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
                  SC_1899_12_31_00_00_00 AS START_DATE, SC_1899_12_31_00_00_00 AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,HOST(h_per_day) AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
                  0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,
                  '' AS RANK, '' AS PROJECT_ASSIGNMENT_ID
                  FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
                  WHERE ra.PROJECT_ID=HOST(h_id) AND ra.RESOURCE_ID = p.RESOURCE_ID
                      AND NOT EXISTS(SELECT ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta
                          WHERE ta.RES_ASSIGNMENT_ID IN (SELECT a.RES_ASSIGNMENT_ID
                              FROM RESOURCE_ASSIGNMENTS a WHERE a.PROJECT_ID = ra.PROJECT_ID AND a.RESOURCE_ID = ra.RESOURCE_ID))
                  ORDER BY TYPE_ID DESC, ELEMENT_NAME FOR READ ONLY;
              END_SQL_NOCHECK;
      
              BEGIN_SQL
                  RETURN_CURSOR(C7751F);
              END_SQL;*/
   ElsIf Coalesce(h_bitflag, 0) = 7 Then
      OPEN C7751G FOR SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      b.PLANNED_START, b.PLANNED_FINISH, b.BEST AS PLANNED_EFFORT, b.PLANNED_WORK AS PLANNED_REMAINING, a.BAK_START_DT AS START_DATE, a.BAK_FINISH_DT AS FINISH_DATE,
      a.BEST AS WORK, a.SCHED_WORK AS REMAINING, h_per_day AS MINUTE_PER_DAY,'' AS BEST_START, '' AS BEST_FINISH,
      0 AS CALENDAR,0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE, pa.DESCRIPTION,
      pa.LEVEL_PLANNED_START, pa.LEVEL_PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,
      pa.ESTIMATED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMATED_BASELINE1_EFFORT,
      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELINE3_FINISH, pa.ESTIMATED_BASELINE3_EFFORT
      FROM RESOURCE_ASSIGNMENTS t JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = t.RESOURCE_ID
      LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG='2'
      LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG='0'
      JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID
      WHERE t.ELEMENT_ID=h_id AND t.RESOURCE_TYPE='0' AND (a.TSK_ASSIGNMENT_ID IS NOT NuLL OR b.TSK_ASSIGNMENT_ID IS NOT NuLL)
      UNION ALL
      SELECT t.RESOURCE_ID AS ELEMENT_ID, a.RES_ASSIGNMENT_ID, p.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      t.PLANNED_START, t.PLANNED_FINISH,t.BEST AS PLANNED_EFFORT,t.PLANNED_WORK AS PLANNED_REMAINING,TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE,
      0 AS WORK,0 AS REMAINING,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START, '' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, t.ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE, pa.DESCRIPTION,
      pa.LEVEL_PLANNED_START, pa.LEVEL_PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,
      pa.ESTIMATED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMATED_BASELINE1_EFFORT,
      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELINE3_FINISH, pa.ESTIMATED_BASELINE3_EFFORT
      FROM RESOURCE_ASSIGNMENTS a JOIN PROFILES p ON a.RESOURCE_ID=p.PROFILE_ID
      JOIN TASK_ASSIGNMENTS t ON t.RES_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID
      JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = a.RESOURCE_ID
      WHERE a.ELEMENT_ID=h_id AND a.RESOURCE_TYPE='1'
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 894+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(895) : cplan.sql(13183) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751G;
   ElsIf Coalesce(h_bitflag, 0) = 8 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PRJ_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_PRJ_INFO
         (RESOURCE_ID,
         RESOURCE_TYPE,
         PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,
         PLANNED_REMAINING,
         PLANNED_EFFORT,
         PRJ_SCHED_START ,
         PRJ_SCHED_FINISH,
         SCHED_REMAINING,
         WORK,
         TYPE_ID
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START)  AS PRJ_PLANNED_START,
         MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,
         SUM(PLANNED_WORK)   AS PLANNED_REMAINING,
         SUM(CASE WHEN BIT_FLAG='0' THEN BEST ELSE 0 END) AS PLANNED_EFFORT,
         MIN(SCHED_START)    AS PRJ_SCHED_START,
         MAX(SCHED_FINISH)   AS PRJ_SCHED_FINISH,
         SUM(SCHED_WORK)     AS SCHED_REMAINING,
         SUM(CASE WHEN BIT_FLAG='2' THEN BEST ELSE 0 END) AS WORK, 13
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID=h_id AND TASK_ID=h_id
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 940+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(941) : cplan.sql(13229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("I1_* SC_STATE: %d, h_project_id: %s\n", SC_STATE,h_project_id);
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_INFO
         (
         RESOURCE_ID,
         RESOURCE_TYPE,
         PLANNED_START,
         PLANNED_FINISH,
         PLANNED_EFFORT,
         START_DATE,
         FINISH_DATE,
         WORK,
         REMAINING,
         PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,
         PLANNED_REMAINING,
         PRJ_SCHED_START,
         PRJ_SCHED_FINISH,
         SCHED_REMAINING,
         LEVEL_PLANNED_START,
         LEVEL_PLANNED_FINISH,
         LEVEL_SCHED_START,
         LEVEL_SCHED_FINISH,
         ESTIMATED_BASELINE1_START,
         ESTIMATED_BASELINE1_FINISH,
         ESTIMATED_BASELINE1_EFFORT,
         ESTIMATED_BASELINE3_START,
         ESTIMATED_BASELINE3_FINISH,
         ESTIMATED_BASELINE3_EFFORT,
         REGULAR_COST,
         REGULAR_RATE,
         SPECIAL_COST,
         SPECIAL_RATE,
         TYPE_ID,
         DESCRIPTION
         )
         SELECT
         a.RESOURCE_ID,      a.RESOURCE_TYPE,        a.PLANNED_START,        a.PLANNED_FINISH,
         a.PLANNED_WORK,     a.SCHED_START,          a.SCHED_FINISH,         a.SCHED_WORK,
         a.ETC,              b.PRJ_PLANNED_START,    b.PRJ_PLANNED_FINISH,   b.PLANNED_REMAINING,
         b.PRJ_SCHED_START,  b.PRJ_SCHED_FINISH,     b.SCHED_REMAINING,
         a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
         a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
         a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
         a.REGULAR_COST,     a.REGULAR_RATE,         a.SPECIAL_COST,         a.SPECIAL_RATE,
         COALESCE(b.TYPE_ID,CASE WHEN (a.PLANNED_WORK+a.SCHED_WORK) = 0 THEN 185 ELSE 13 END), a.DESCRIPTION
         FROM PROJECT_ASSIGNMENTS a LEFT OUTER JOIN PMO_TMP_PRJ_INFO b ON a.RESOURCE_ID = b.RESOURCE_ID
         WHERE a.PROJECT_ID=h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1007+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1008) : cplan.sql(13296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("I2_* SC_STATE: %d, h_id: %s\n", SC_STATE,h_id);
      
      BEGIN
         UPDATE PMO_TMP_RES_INFO RI SET TYPE_ID = 13 WHERE TYPE_ID = 185 AND EXISTS(
         SELECT TASK_ID FROM TASK_ASSIGNMENTS T WHERE T.PROJECT_ID=h_id AND T.RESOURCE_ID = RI.RESOURCE_ID AND T.RESOURCE_TYPE = '0');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1026+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1027) : cplan.sql(13315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751H FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, a.TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      CASE WHEN a.PRJ_PLANNED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURCE_ID ELSE '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED_FINISH, a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_REMAINING,
      h_id AS PROJECT_ID,
      a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
      a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
      a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.SPECIAL_RATE, a.DESCRIPTION
      FROM PMO_TMP_RES_INFO a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      UNION ALL
      SELECT  r.PROFILE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      CASE WHEN a.PRJ_PLANNED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURCE_ID ELSE '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED_FINISH, a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_REMAINING,
      h_id AS PROJECT_ID,
      a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
      a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
      a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.SPECIAL_RATE, a.DESCRIPTION
      FROM PMO_TMP_RES_INFO a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1055+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1056) : cplan.sql(13344) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751H;
   ElsIf Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      BEGIN
         SELECT PROFILE_NAME INTO h_in FROM PROFILES WHERE PROFILE_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1067+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1068) : cplan.sql(13356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         OPEN C7751J FOR SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,30 AS ROLE_TYPE, p.ROLE_ID, h_in AS ROLE_NAME,
         ORG_ID, org.ELEMENT_NAME AS ORG_NAME, GEO_ID, geo.ELEMENT_NAME AS GEO_NAME, POOL_ID, pool.ELEMENT_NAME AS POOL_NAME
         FROM PROFILES p
         LEFT JOIN TMT_ORGANIZATIONS org ON org.ELEMENT_ID  = ORG_ID
         LEFT JOIN TMT_GEOGRAPHIES geo   ON geo.ELEMENT_ID  = GEO_ID
         LEFT JOIN TMT_POOLS pool        ON pool.ELEMENT_ID = POOL_ID
         WHERE p.PROFILE_ID = h_id
         UNION ALL
         SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,ds.TYPE_ID AS ROLE_TYPE,
         cs.ELEMENT_ID AS ROLE_ID, PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', '), RTRIM(cs.ELEMENT_NAME)) AS ROLE_NAME,
         '' ORG_ID, '' AS ORG_NAME, '' GEO_ID, '' AS GEO_NAME, '' POOL_ID, '' AS POOL_NAME
         FROM RESOURCE_ASSIGN_ROLES r LEFT OUTER JOIN TMT_COMPETENCIES cs ON r.ROLE_ID=cs.ELEMENT_ID
         LEFT OUTER JOIN TMT_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT_ID WHERE r.RES_ASSIGNMENT_ID=h_id
         UNION ALL
         SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,ds.TYPE_ID AS ROLE_TYPE,
         cs.ELEMENT_ID AS ROLE_ID, PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', '), RTRIM(cs.ELEMENT_NAME)) AS ROLE_NAME,
         '' ORG_ID, '' AS ORG_NAME, '' GEO_ID, '' AS GEO_NAME, '' POOL_ID, '' AS POOL_NAME
         FROM RESOURCE_ASSIGN_SKILLS s LEFT OUTER JOIN TMT_COMPETENCIES cs ON s.SKILL_ID=cs.ELEMENT_ID
         LEFT OUTER JOIN TMT_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT_ID WHERE s.RES_ASSIGNMENT_ID=h_id
         ORDER BY ELEMENT_ID,TYPE_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1094+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1095) : cplan.sql(13383) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751J;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 10 Then
      OPEN C7751L FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID, a.RESOURCE_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID)
      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
      CASE WHEN t.TSK_ASSIGNMENT_ID IS NuLL THEN 185 WHEN a.RESOURCE_TYPE='1' THEN 41 ELSE 13 END AS TYPE_ID,
      a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
      t.SD, t.FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS PLANNED_WORK,
      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK - WORK_TODATE ELSE PLANNED_WORK END AS REMAINING,
      CASE WHEN t.ASSIGN_UNITS > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCENT,
      0 AS MILESTONE_PERCENT,
      CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))*10000) END AS WORK_PERCENT,
      t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
      t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
      t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
      h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
      CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
      ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
      t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
      t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_START,t.UNRESOLVED_START) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINISH,t.UNRESOLVED_FINISH) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
      CASE WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = 0 THEN t.ETC ELSE t.LEVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,
      CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND (t.LEVEL_START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FINISH) THEN '1' ELSE '0' END AS LEVEL_AFFECTED_FLAG,
      T.LABOR_COST_CC_ID,
      EC.ELEMENT_NAME AS LABOR_COST_CC_NAME,
      T.LABOR_REVENUE_CC_ID,
      CC.ELEMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR_FLAG,
      T.CONTRACT_ID, CTR.ELEMENT_NAME AS CONTRACT_NAME
      FROM RESOURCE_ASSIGNMENTS a JOIN TASK_ASSIGNMENTS t ON t.TSK_ASSIGNMENT_ID = h_id AND a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
      LEFT OUTER JOIN TMT_CHARGECODES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID
      LEFT OUTER JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID
      LEFT OUTER JOIN TMT_CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1137+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1138) : cplan.sql(13426) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751L;
   ElsIf Coalesce(h_bitflag, 0) = 11 Then
      OPEN C7751M FOR SELECT a.RES_ASSIGNMENT_ID AS ELEMENT_ID, a.RESOURCE_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID)
      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
      CASE WHEN a.RESOURCE_TYPE='1' THEN PMO_CNST.SC_PROFILE ELSE 185 END AS TYPE_ID,  a.RES_ASSIGNMENT_ID, a.PARTICIPANT, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,
      0 AS PLANNED_WORK,TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS SCHED_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS SCHED_FINISH, 0 AS SCHED_WORK,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_FINISH_DT, 0 AS BAK_WORK,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS FORECAST_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS FORECAST_FINISH,0 AS ETC,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS ACTUAL_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS ACTUAL_FINISH,
      0 AS ACTUAL,h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, TO_TIMESTAMP('1899-12-31 00:00:00') AS ACCEPTED_DATE,
      a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME, COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME,
      0 AS REGULAR_COST,0 AS REGULAR_RATE,0 AS SPECIAL_COST, 0 AS SPECIAL_RATE,
      CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
      ELSE '' END AS FS_LABOR_CODE,
      a.REC_STATUS,'' AS ASSIGN_STRING,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS BASELINE1_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS BASELINE1_FINISH,0 AS BASELINE1_WORK,
      '' AS PROJECT_ASSIGNMENT_ID, '' AS TIMECODE2_ID, '' AS TIMECODE2_NAME, 0 AS T_UNIT,
      0 AS C_EFFORT, 0 AS C_UNIT, 0 AS C_CONTOUR, 0 AS CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE,
      0 AS DELAY
      FROM RESOURCE_ASSIGNMENTS a
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      WHERE a.REC_STATUS NOT IN ('L','D') AND a.ELEMENT_ID = h_id
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1166+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1167) : cplan.sql(13455) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751M;
   ElsIf Coalesce(h_bitflag, 0) = 12 Then
      -- returns all assignments DISPLAY NICKNAME(effort, participants, project assignments)
      -- in the current project (id = PROJECT_ID)
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RTA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RTA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT RESOURCE_ID, RESOURCE_TYPE, 'N' AS PA_FLAG
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND
         PROJECT_ASSIGNMENT_ID IS NuLL AND
         TABLE_TYPE = PMO_CNST.SC_TABLE_WBS
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1194+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1195) : cplan.sql(13483) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CP_PRJ';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CP_PRJ (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT  RESOURCE_ID, RESOURCE_TYPE, 'Y' AS PA_FLAG
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND
         TASK_ID = h_id
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1220+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1221) : cplan.sql(13509) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RESOURCE_DATA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RESOURCE_DATA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT a.RESOURCE_ID, a.RESOURCE_TYPE, a.PA_FLAG
         FROM PMO_TMP_RTA a LEFT OUTER JOIN PMO_TMP_CP_PRJ b ON a.RESOURCE_ID=b.RESOURCE_ID
         UNION
         SELECT b.RESOURCE_ID, b.RESOURCE_TYPE, b.PA_FLAG
         FROM PMO_TMP_CP_PRJ b LEFT OUTER JOIN PMO_TMP_RTA a ON a.RESOURCE_ID=b.RESOURCE_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1246+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1247) : cplan.sql(13535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751N FOR SELECT a.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID, p.NICK_NAME, a.PA_FLAG
      FROM PMO_TMP_RESOURCE_DATA a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      LEFT OUTER JOIN PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID
      
      UNION ALL
      
      SELECT DISTINCT r.PROFILE_ID AS ELEMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID, ' ' AS NICK_NAME, a.PA_FLAG
      FROM PMO_TMP_RESOURCE_DATA a JOIN PROFILES r ON a.RESOURCE_TYPE = '1' AND r.PROFILE_ID = a.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
      LEFT OUTER JOIN PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID
      
      UNION ALL
      
      SELECT DISTINCT ra.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 185 AS TYPE_ID, p.NICK_NAME, 'N' AS PA_FLAG
      FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
      WHERE ra.PROJECT_ID=h_id AND
      ra.RESOURCE_ID = p.RESOURCE_ID AND
      NOT EXISTS(SELECT rd.RESOURCE_ID FROM PMO_TMP_RESOURCE_DATA rd WHERE ra.RESOURCE_ID = rd.RESOURCE_ID)
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1272+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1273) : cplan.sql(13561) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751N;
   ElsIf Coalesce(h_bitflag, 0) = 13 Then
      -- returns all assignments DISPLAY NICKNAME(effort, participants, project assignments)
      -- in the current pool (id = ELEMENT_ID)
      
      OPEN C77513N FOR SELECT t.ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID, p.NICK_NAME, 'N' AS PA_FLAG
      FROM TMT_POOLS t JOIN RESOURCES p
      ON t.ELEMENT_ID = p.RESOURCE_ID
      WHERE t.TYPE_ID = 13
      AND t.RANK LIKE h_rank || '%'
      ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1286+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1287) : cplan.sql(13575) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C77513N;
   ElsIf (Coalesce(h_bitflag, 0) = PMO_CNST.SC_ASSET_ASSIGNED_RESOURCES) OR (Coalesce(
                h_bitflag, 0) = PMO_CNST.SC_CLIENT_ASSIGNED_RESOURCES) Then
      -- returns all resources assigned to an asset(resource can be asssigned indirectly it means
      -- resource who assigned to a document/cri attached to the asset element will be selected in this query)
      
      OPEN cln_ast_res FOR SELECT R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME,PMO_CNST.SC_RESOURCE AS TYPE_ID, R.NICK_NAME, 'N' AS PA_FLAG
      FROM PROJECT_ASSIGNMENTS RA, RESOURCES R
      WHERE RA.PROJECT_ID = h_id
      AND RA.RESOURCE_ID = R.RESOURCE_ID
      ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1300+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1301) : cplan.sql(13589) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN cln_ast_res;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_S_PROFILE', 1305+1) THEN 
         LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_S_PROFILE(1306) : cplan.sql(13594) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_STATUS(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1314+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1315) : cplan.sql(13603) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1326+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1327) : cplan.sql(13615) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_PROFILE;
ALTER FUNCTION SP_S_PROFILE COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_NAMES ==
CREATE OR REPLACE FUNCTION FC_U_NAMES
(
     a_element_id IN OUT NOCOPY VARCHAR2
   , h_full_name IN OUT NOCOPY VARCHAR2
   , a_participant_id IN OUT NOCOPY VARCHAR2
   , sh_h_table_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_NAMES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_NAMES'), '-- ' || SYSTIMESTAMP, ':r := FC_U_NAMES(' || 
       '''' || a_element_id || ''''
          || ', ' || '''' || h_full_name || ''''
          || ', ' || '''' || a_participant_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_NAMES', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_NAMES(20) : cplan.sql(13651) : ' || SYSTIMESTAMP);
   END IF; 

   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 46+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(47) : cplan.sql(13678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_POL THEN
         BEGIN
            UPDATE TMT_POOLS SET MANAGER_ID = a_participant_id, ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 76+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(77) : cplan.sql(13708) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         BEGIN
            UPDATE TMT_CRI SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 105+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(106) : cplan.sql(13737) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            UPDATE TMT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 135+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(136) : cplan.sql(13767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            UPDATE TMT_POOL_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(166) : cplan.sql(13797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            UPDATE TMT_RESOURCE_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(196) : cplan.sql(13827) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            UPDATE TMT_ASSET_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 225+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(226) : cplan.sql(13857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            UPDATE TMT_CLIENT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 255+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(256) : cplan.sql(13887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            UPDATE TMT_VENDOR_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 285+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(286) : cplan.sql(13917) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      ELSE
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 315+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(316) : cplan.sql(13947) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
   END CASE;
   --trace_printf("***_ASSIGN_NAMES: SC_STATE: %d, h_full_name: %s, h_table_type: %d\n", SC_STATE,h_full_name,h_table_type);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_NAMES', 323+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_NAMES(324) : cplan.sql(13955) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_NAMES;
ALTER FUNCTION FC_U_NAMES COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_WP_DATES ==
CREATE OR REPLACE FUNCTION SP_U_WP_DATES
(
     assign_id VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_actual_start TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_SD DATE;
   h_FD DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_last_submitted DATE;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_DATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_DATES'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_DATES(' || 
       '''' || assign_id || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 74+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_DATES(75) : cplan.sql(14045) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --Validate res_assignment_id
   h_tsk_assignment_id := assign_id;
   h_rec_user := res_id001;
   
   BEGIN
      SELECT PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,
      RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT,
      UNITS_MSP, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNRESOLVED_FINISH,
      CASE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCHED_WORK END, ASSIGNMENT_DELAY, LAST_SCHED_MIN,
      TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
      REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
      B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
      MILESTONE_PERCENT, TASK_ID
      INTO h_prj_assign_id, h_project_id, h_res_id,
      h_res_type, h_bit_flag, h_last_submitted, h_contour, h_work, h_eunit, h_unit,
      h_percent_units, h_actual_start, h_work_todate, h_start_date, h_finish_date,
      h_effort, h_delay, h_last_day,
      h_SD, h_FD, h_billable, h_capital_expense,
      h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
      h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
      h_work_percent, h_task_id
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 108+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(109) : cplan.sql(14079) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_SD: %d %s, h_FD: %d %s\n", SC_STATE,h_SD_Ind,h_SD,h_FD_Ind,h_FD);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 116+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(117) : cplan.sql(14087) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(120) : cplan.sql(14090) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_tsk_assignment_id
            , True);
   End If;
   
   If (p_planned_start IS NOT NULL) AND (p_planned_finish IS NOT NULL) Then
      h_sched_start := p_planned_start;
      h_sched_finish := p_planned_finish;
      
      If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
            h_sched_finish, h_finish_date, 10) <> 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 132+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(133) : cplan.sql(14103) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 135+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(136) : cplan.sql(14106) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, h_tsk_assignment_id
               , True);
      End If;
      
      BEGIN
         SELECT PROJECT_ID, MINUTE_PER_DAY
         INTO h_project_id, h_per_day
         FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(152) : cplan.sql(14122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_per_day: %d\n", SC_STATE,h_per_day);//trace_flush();
      
      --trace_printf("h_start_date: %s, h_finish_date: %s\n", h_start_date,h_finish_date);
      If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
         h_start_date := h_sched_start;
         h_delay := 0;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_DATES', 163+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_DATES(164) : cplan.sql(14134) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 166+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(167) : cplan.sql(14137) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   End If;
   
   --trace_printf("**===> h_last_day: %d, h_prj_assign_id: %d %s\n", h_last_day,h_prj_assign_id_Ind,h_prj_assign_id);
   h_tph_delete := -1;
   h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_last_submitted, h_last_day, 0);
   End If;
   
   --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
   If (h_prj_assign_id IS NOT NULL) Then
      h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
   Else
      If SubStr(h_bit_flag, 1, 1) = '2' Then
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
      Else
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
      End If;
   End If;
   
   If (h_last_submitted IS NOT NULL) Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
         INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 205+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(206) : cplan.sql(14176) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
   End If;
   
   --trace_printf("1_FC_DEL_NOACTUAL: h_skip: %d,h_start_date: %s, h_effort: %d, h_percent_units: %d\n",h_skip,h_start_date,h_effort,h_percent_units);
   
   h_temp_var := 2;
   h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_rec_user, Coalesce(h_delay, 0), Coalesce(h_percent_units, 0));
   --trace_printf("2_FC_RESCHEDULE: h_temp_sc_state: %d\n",h_temp_sc_state); //trace_flush();
   
   If Coalesce(h_temp_sc_state, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 222+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(223) : cplan.sql(14193) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 225+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(226) : cplan.sql(14196) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_tsk_assignment_id
            , True);
   Else
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),
         BAK_START_DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(SCHED_FINISH,PLANNED_FINISH),
         BAK_ASSIGN_STRING = ASSIGN_STRING
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 245+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(246) : cplan.sql(14216) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_temp_var := FC_Rollup(h_project_id);
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 253+1) THEN 
      LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(254) : cplan.sql(14224) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 256+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(257) : cplan.sql(14227) : ' || SYSTIMESTAMP);
   END IF; 
   PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_tsk_assignment_id
         , True);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_DATES', 262+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_DATES(263) : cplan.sql(14233) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_tsk_assignment_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 269+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(270) : cplan.sql(14240) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 281+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(282) : cplan.sql(14252) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_DATES;
ALTER FUNCTION SP_U_WP_DATES COMPILE DEBUG;

/**************************************************************************/

Prompt == FUNCTION FC_U_WP_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_U_WP_ACTUAL
(
     h_assign_id IN OUT NOCOPY VARCHAR2
   , sh_h_new_work_percent PLS_INTEGER
   , sh_h_actual PLS_INTEGER
   , sh_h_units PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , h_error_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_new_work_percent PLS_INTEGER := sh_h_new_work_percent;
   h_actual PLS_INTEGER := sh_h_actual;
   h_units PLS_INTEGER := sh_h_units;
   h_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_last_submitted DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_rec_status VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_close_task PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   Debug_PerfTiming PLS_INTEGER := 0;
   t_total NUMBER := 0;
   s_time_1 PLS_INTEGER := 0;
   s_time_2 PLS_INTEGER := 0;
   s_time_3 PLS_INTEGER := 0;
   s_time_4 PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_WP_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_WP_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_U_WP_ACTUAL(' || 
       '''' || h_assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_new_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_units), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || '''' || h_error_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 82+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_WP_ACTUAL(83) : cplan.sql(14351) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := h_assign_id;
   
   BEGIN
      SELECT PROJECT_ID, RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK-WORK_TODATE, ETC,
      SCHED_WORK, LAST_SCHED_MIN, PROJECT_ASSIGNMENT_ID, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT, CASE WHEN ASSIGN_UNITS > UNITS_MSP THEN ASSIGN_UNITS ELSE UNITS_MSP END,
      UNRESOLVED_START, UNRESOLVED_FINISH, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
      REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
      B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
      MILESTONE_PERCENT, TASK_ID, ROLLUP_BITS
      INTO h_project_id, h_res_id, h_res_type, h_bit_flag, h_last_submitted, h_work_todate, h_remaining, h_etc,
      h_effort, h_last_day, h_prj_assign_id, h_contour, h_work, h_eunit, h_unit, h_percent_units,
      h_start_date, h_finish_date, h_SD, h_FD, h_billable, h_capital_expense,
      h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
      h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
      h_work_percent, h_task_id, h_rollup_bits
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 108+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(109) : cplan.sql(14377) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT PROJECT_ID, TASK_TYPE, SCHED_EFFORT, WORK_TODATE, MILESTONE_PERCENT, SCHED_DURATION
         INTO h_project_id, h_task_type, h_effort, h_work_todate, h_work_percent, h_duration
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 125+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(126) : cplan.sql(14394) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         h_error_id := h_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 133+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(134) : cplan.sql(14402) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK
         INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 150+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(151) : cplan.sql(14419) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS
            INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 167+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(168) : cplan.sql(14436) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            h_error_id := h_project_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 175+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(176) : cplan.sql(14444) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.LEV_LOCKED_BY_COPY;
         End If;
      End If;
      
      -- varify if there are steps in timesheets
      BEGIN
         SELECT COUNT('X')
         INTO h_temp_var
         FROM DUAL
         WHERE EXISTS(
         SELECT A.ELEMENT_ID
         FROM TASK_ASSIGNMENTS TA, ACTIVITIES A
         WHERE TA.PROJECT_ID = h_project_id
         AND TA.TASK_ID = h_id
         AND TA.TSK_ASSIGNMENT_ID = A.TSK_ASSIGNMENT_ID
         AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT_ID
         AND A.TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(201) : cplan.sql(14469) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         h_error_id := h_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 208+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(209) : cplan.sql(14477) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.SC_TIMESHEET_STEP_EXISTS;
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'FC_U_WP_ACTUAL', 215+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: FC_U_WP_ACTUAL(216) : cplan.sql(14484) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 218+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(219) : cplan.sql(14487) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS, False);
      ElsIf Coalesce(h_actual, 0) <> Coalesce(h_work_todate, 0) Then
         h_close_task := 0;
         If Coalesce(h_actual, 0) > Coalesce(h_effort, 0) Then
            h_error_id := h_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 226+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(227) : cplan.sql(14495) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH;
         ElsIf Coalesce(h_actual, 0) = Coalesce(h_effort, 0) Then
            h_close_task := 1;
         End If;
         
         h_new_actual := h_actual;
         h_temp_var := FC_U_TASK_PERCENT(h_project_id, h_id, h_new_actual, h_rec_user, Coalesce(
                                h_close_task, 0));
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'FC_U_WP_ACTUAL', 238+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: FC_U_WP_ACTUAL(239) : cplan.sql(14507) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 241+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(242) : cplan.sql(14510) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.INVALID_PARAMS, False);
      End If;
   Else
      If Coalesce(h_effort, 0) = 0 Then
         h_error_id := h_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 249+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(250) : cplan.sql(14518) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.INVALID_PARAMS;
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK
         INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 266+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(267) : cplan.sql(14535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS
            INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 283+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(284) : cplan.sql(14552) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            h_error_id := h_project_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 291+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(292) : cplan.sql(14560) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.LEV_LOCKED_BY_COPY;
         End If;
      End If;
      
      -- varify if there are steps in timesheets
      BEGIN
         SELECT COUNT('X')
         INTO h_temp_var
         FROM DUAL
         WHERE EXISTS(
         SELECT ELEMENT_ID
         FROM ACTIVITIES
         WHERE TSK_ASSIGNMENT_ID = h_id
         AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
         AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 314+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(315) : cplan.sql(14583) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         h_error_id := h_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 322+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(323) : cplan.sql(14591) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.SC_TIMESHEET_STEP_EXISTS;
      End If;
      
      BEGIN
         SELECT PROJECT_ID, MINUTE_PER_DAY
         INTO h_project_id, h_per_day
         FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 339+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(340) : cplan.sql(14608) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_sched_effort := Coalesce(h_effort, 0);
      If Coalesce(h_actual, 0) <> Coalesce(h_work_todate, 0) Then
         h_new_actual := h_actual;
         -- new actual work
         If Coalesce(h_new_actual, 0) > Coalesce(h_effort, 0) Then
            h_tph_delete := -1;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
            
            If Coalesce(h_temp_var, 0) = 0 Then
               If Coalesce(h_remaining, 0) <> Coalesce(h_etc, 0) Then
                  If PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0 Then
                     h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_effort, 0);
                     h_work_percent := 10000;
                     
                     If Coalesce(h_new_actual, 0) < (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) Then
                        h_etc := (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) - Coalesce(h_new_actual, 0);
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS
                           SET CONSIDER = 7,
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual,
                           WORK_TODATE = h_new_actual,
                           ETC = h_etc,
                           MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 380+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(381) : cplan.sql(14649) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        h_work_percent := PMO_FUNC.BIT_OR(Coalesce(h_work_percent, 0), PMO_CNST.SC_BIT_30);
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS
                           SET CONSIDER = 7,
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual,
                           WORK_TODATE = h_new_actual,
                           ETC = 0,
                           MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 406+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(407) : cplan.sql(14675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     If Coalesce(h_new_actual, 0) < (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) Then
                        h_effort := Coalesce(h_etc, 0) - Coalesce(h_effort, 0);
                        h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS
                           SET CONSIDER = 7,
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_etc,
                           WORK_TODATE = h_new_actual
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 433+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(434) : cplan.sql(14702) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_effort, 0);
                        h_work_percent := 10000;
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS
                           SET CONSIDER = 7,
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual,
                           WORK_TODATE = h_new_actual,
                           ETC = 0,
                           MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 460+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(461) : cplan.sql(14729) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               Else
                  h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_work_todate, 0);
                  h_work_percent := 10000;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS
                     SET CONSIDER = 7,
                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                     SCHED_WORK = h_new_actual,
                     WORK_TODATE = h_new_actual,
                     ETC = 0,
                     MILESTONE_PERCENT = 10000
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 489+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(490) : cplan.sql(14758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If (h_last_submitted IS NOT NULL) Then
                  h_week_of := NULL;
                  BEGIN
                     SELECT MAX(WEEK_OF)
                     INTO h_week_of
                     FROM TIMESHEET_APPROVAL_STATUS
                     WHERE ELEMENT_ID = h_project_id
                     AND RESOURCE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 509+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(510) : cplan.sql(14778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("TIMESHEET_APPROVAL_STATUS: SC_STATE: %d, h_start_date: %s, h_week_of: %s\n", SC_STATE,h_start_date,h_week_of);//trace_flush();
                  
                  If (h_week_of IS NOT NULL) Then
                     BEGIN
                        SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 524+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(525) : cplan.sql(14793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If PMO_ORASTD.strncmp(h_last_submitted, h_week_of, 10) < 0 Then
                        h_start_date := h_week_of;
                     End If;
                  ElsIf Coalesce(h_contour, 0) > 0 Then
                     h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                     
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 545+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(546) : cplan.sql(14814) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 560+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(561) : cplan.sql(14829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("*** h_start_date: %s\n", h_start_date);//trace_flush();
               End If;
               
               --trace_printf("*** h_work_percent: %d, h_start_date: %s, h_effort: %d\n", h_work_percent,h_start_date,h_effort);//trace_flush();
               If (h_prj_assign_id IS NOT NULL) Then
                  h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 2);
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 0);
                  Else
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 1);
                  End If;
               End If;
               
               h_temp_var := 2;
               h_temp_var := FC_RESCHEDULE(h_id, h_start_date, h_effort, h_temp_var, h_contour, h_rec_user, 0
                            , Coalesce(h_percent_units, 0));
               
               If Coalesce(h_temp_var, 0) <> 0 Then
                  h_error_id := h_id;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 586+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(587) : cplan.sql(14855) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN Coalesce(h_temp_var, 0);
               End If;
               
               h_effort := h_new_actual;
            End If;
         Else
            --timeLog (file,"2_* START Actual <= Schedule:");
            h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
         End If;
         
         --trace_printf("h_work_percent: %d, h_last_day: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_last_day,h_new_actual,h_work_todate,h_effort); //trace_flush();
         --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD); //trace_flush();
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_id, h_last_submitted
                                  , h_bit_flag, h_effort, h_new_actual, h_sched_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, Coalesce(h_per_day, 0));
         
         --trace_printf("*_FC_U_ASSIGN_PERCENT: h_temp_var: %d\n", h_temp_var); //trace_flush();
         h_error_id := h_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 611+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(612) : cplan.sql(14880) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_var, 0);
      ElsIf Coalesce(h_new_work_percent, 0) <> Coalesce(h_work_percent, 0) Then
         /*if (*p_work_percent == 10000)
                     {
                         BEGIN_SQL
                             UPDATE TASK_ASSIGNMENTS SET
                                 SCHED_WORK = WORK_TODATE,
                                 ETC = 0, FD = SC_AF,
                                 MILESTONE_PERCENT = 10000
                             WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
                         END_SQL;
         
                         if( SC_SUCCESS )
                         {
                             InitializeHost(SC_PARAM(h_temp_var), 1);
                             h_temp_var = FC_ETC_SCHEDULE(SC_PARAM(h_id), SC_PARAM(h_temp_var), SC_PARAM(h_rec_user));
                         }
                     }
                     else if ( h_work_percent == 10000 )
                     {
                         BEGIN_SQL
                             UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_wp),
                             ETC = SCHED_WORK - WORK_TODATE, FD = SC_SF, ACTUAL_FINISH = SC_NULL, ACTUAL_DURATION = 0
                             WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
                         END_SQL;
                     }
                     else
                     {
                         BEGIN_SQL
                             UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_wp),
                             ETC = SCHED_WORK - WORK_TODATE
                             WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
                         END_SQL;
                     }*/
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET MILESTONE_PERCENT = h_new_work_percent,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 663+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(664) : cplan.sql(14932) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         /*
                     BEGIN_SQL
                         UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = HOST(h_rollup_bits) WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
                     END_SQL;
         */
         
         BEGIN
            UPDATE TMT_WBS
            SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 688+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(689) : cplan.sql(14957) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --RETURN_ERROR(SC_STATE, h_id);
      ElsIf Coalesce(h_units, 0) <> Coalesce(h_percent_units, 0) Then
         If Coalesce(h_units, 0) = 0 Then
            h_percent_units := 100;
         Else
            If (Coalesce(h_units, 0) < 1) OR (Coalesce(h_units, 0) > 103079040) Then
               h_error_id := h_id;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 701+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(702) : cplan.sql(14970) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.INVALID_PARAMS;
            End If;
            
            h_percent_units := Coalesce(h_units, 0);
         End If;
         
         If (h_prj_assign_id IS NOT NULL) Then
            BEGIN
               SELECT UNITS_MSP
               INTO h_temp_var
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 721+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(722) : cplan.sql(14990) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);//trace_flush();
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_percent_units, 0) > Coalesce(
                  h_temp_var, 0)) Then
               h_error_id := h_prj_assign_id;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 731+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(732) : cplan.sql(15000) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
         End If;
         
         /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
                     {
                         InitializeHost(SC_PARAM(h_tph_delete), -1);
                         h_temp_var = FC_TPHASE_SCHED_ASSIGN(
                                     SC_PARAM(h_project_id),
                                     SC_PARAM(h_task_id),
                                     SC_PARAM(h_id),
                                     SC_PARAM(h_SD),
                                     SC_PARAM(h_FD),
                                     SC_PARAM(h_reg_cost),
                                     SC_PARAM(h_reg_sale),
                                     SC_PARAM(h_tph_delete),
                                     SC_PARAM(h_capital_expense),
                                     SC_PARAM(h_billable));
         
                         if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
                         {
                             h_temp_var = FC_U_TDY_LAST_ACTUAL(SC_PARAM(h_project_id),
                                             SC_PARAM(h_task_id),
                                             SC_PARAM(h_res_id),
                                             SC_PARAM(h_prj_assign_id),
                                             SC_PARAM(h_id),
                                             SC_PARAM(h_last_submitted),
                                             SC_PARAM(h_last_day) );
         
                         }
         
                         h_minutes = int(((h_percent_units * h_per_day) / 10000.00) + 0.4);
                         h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                                   SC_PARAM(h_id),
                                                   SC_PARAM(h_start_date),
                                                   SC_PARAM(h_per_day),
                                                   SC_PARAM(h_minutes));
         
                         InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
                         h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_id), SC_PARAM(h_temp_var));
         
                         BEGIN_SQL
                             SELECT DATE(UNRESOLVED_START), DATE(UNRESOLVED_FINISH) INTO HOST(h_SD), HOST(h_FD)
                             FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_id) ;
                         END_SQL;
         
                         InitializeHost(SC_PARAM(h_tph_delete), 0);
                         h_temp_var = FC_TPHASE_SCHED_ASSIGN(
                                     SC_PARAM(h_project_id),
                                     SC_PARAM(h_task_id),
                                     SC_PARAM(h_id),
                                     SC_PARAM(h_SD),
                                     SC_PARAM(h_FD),
                                     SC_PARAM(h_reg_cost),
                                     SC_PARAM(h_reg_sale),
                                     SC_PARAM(h_tph_delete),
                                     SC_PARAM(h_capital_expense),
                                     SC_PARAM(h_billable));
                     }
                     else
                     {*/
         h_tph_delete := -1;
         h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
         
         If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
            h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id, h_id
                                   , h_last_submitted, h_last_day, 0);
         End If;
         
         --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
         If (h_prj_assign_id IS NOT NULL) Then
            h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 2);
         Else
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 0);
            Else
               h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 1);
            End If;
         End If;
         
         If (h_last_submitted IS NOT NULL) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
               INTO h_start_date
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 825+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(826) : cplan.sql(15094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
         End If;
         
         --trace_printf("1_WP_FC_DEL_NOACTUAL: h_start_date: %s, h_effort: %d, h_percent_units: %d\n", h_start_date,h_effort,h_percent_units);
         
         h_contour := 0;
         h_temp_var := 2;
         h_temp_var := FC_RESCHEDULE(h_id, h_start_date, h_effort, h_temp_var, h_contour, h_rec_user, 0
                            , Coalesce(h_percent_units, 0));
         --trace_printf("2_WP_FC_RESCHEDULE: h_temp_var: %d\n",h_temp_var); //trace_flush();
         
         If Coalesce(h_temp_var, 0) <> 0 Then
            h_error_id := h_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 844+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(845) : cplan.sql(15113) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS
               SET CONSIDER = 7,
               UNITS_MSP = h_percent_units,
               C_EFFORT = 0,
               C_UNIT = 0,
               C_CONTOUR = 0,
               CONTOUR_TYPE = 0,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WP_ACTUAL', 868+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WP_ACTUAL(869) : cplan.sql(15137) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --}
      End If;
      --timeLog (file,"1.1_* FC_Rollup:");
   End If;
   
   PMO_DB2M.SC_STATE := FC_Rollup(h_project_id);
   --trace_printf("**_FC_Rollup_** SC_STATE: %d\n",SC_STATE);
   
   --meLog (file,"1.2_* FC_Rollup:");
   
   --trace_printf("**_SP_U_WP_ACTUAL_** SC_STATE: %d\n",SC_STATE);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WP_ACTUAL', 886+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_WP_ACTUAL(887) : cplan.sql(15155) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_WP_ACTUAL;
ALTER FUNCTION FC_U_WP_ACTUAL COMPILE DEBUG;

/******************************************************************************************************************
update Actual Work, %W, %C in Work Packages
assign_id:      TSK_ASSIGNMENT_ID or TASK_ID
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_WP_ACTUAL ==
CREATE OR REPLACE FUNCTION SP_U_WP_ACTUAL
(
     assign_id VARCHAR2
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , p_units PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_assign_id VARCHAR2(32);
   h_new_work_percent PLS_INTEGER := 0;
   h_actual PLS_INTEGER := 0;
   h_units PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_ACTUAL(' || 
       '''' || assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_ACTUAL(29) : cplan.sql(15197) : ' || SYSTIMESTAMP);
   END IF; 

   h_error_id := NULL;
   
   h_assign_id := assign_id;
   h_new_work_percent := p_work_percent;
   h_actual := p_actual;
   h_units := p_units;
   h_rec_user := res_id001;
   
   PMO_DB2M.SC_STATE := FC_U_WP_ACTUAL(h_assign_id, h_new_work_percent, h_actual, h_units, h_rec_user
                                    , h_error_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_ACTUAL', 43+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_ACTUAL(44) : cplan.sql(15212) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_error_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 50+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(51) : cplan.sql(15219) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 62+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(63) : cplan.sql(15231) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_ACTUAL;
ALTER FUNCTION SP_U_WP_ACTUAL COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_PARTICIPANT ==
CREATE OR REPLACE FUNCTION SP_U_PARTICIPANT
(
     assign_id VARCHAR2
   , security_id001 VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , p_delay PLS_INTEGER
   , p_units PLS_INTEGER
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_security_id VARCHAR2(32);
   a_element_id VARCHAR2(32);
   a_participant_id VARCHAR2(255);
   a_participant_name VARCHAR2(255);
   a_security_name VARCHAR2(32);
   h_full_name VARCHAR2(255);
   a_res_assignment_id VARCHAR2(32);
   a_security_id VARCHAR2(32);
   a_temp_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_created_by VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_actual_start TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_SD DATE;
   h_FD DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_last_submitted DATE;
   a_assignment_count PLS_INTEGER := 0;
   a_element_type PLS_INTEGER := 0;
   h_responsible PLS_INTEGER := 0;
   a_participant PLS_INTEGER := 0;
   a_resource_type PLS_INTEGER := 0;
   a_security_value PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_close_task PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   stmt VARCHAR2(7999);
   i_resched PLS_INTEGER;
   i_bitflag PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_P_DELAY PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_R_DELAY PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_PARTICIPANT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_PARTICIPANT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_PARTICIPANT(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || security_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 120+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_PARTICIPANT(121) : cplan.sql(15373) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_task_type := 0;
   h_created_by := '';
   h_rec_user := res_id001;
   a_security_id := security_id001;
   
   --Validate res_assignment_id
   a_temp_id := assign_id;
   
   BEGIN
      SELECT ra.RES_ASSIGNMENT_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,
      CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
      CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
      ra.RESOURCE_ID,
      ra.PROJECT_ID, ra.TABLE_TYPE
      INTO a_res_assignment_id, a_participant, a_element_id, a_element_type,
      a_resource_type,
      h_transfed_cri,
      a_participant_id,
      h_project_id, h_table_type
      FROM RESOURCE_ASSIGNMENTS ra
      WHERE ra.RES_ASSIGNMENT_ID = a_temp_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 152+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(153) : cplan.sql(15405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,
         RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT, CONTOUR_TYPE, 
         CASE WHEN ASSIGN_UNITS > UNITS_MSP THEN ASSIGN_UNITS ELSE UNITS_MSP END, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNRESOLVED_FINISH,
         CASE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCHED_WORK END, ASSIGNMENT_DELAY,LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
         B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
         MILESTONE_PERCENT, TASK_ID, ROLLUP_BITS
         INTO a_res_assignment_id, h_prj_assign_id, h_project_id, h_res_id,
         h_res_type, h_bit_flag, h_last_submitted, h_contour, h_work, h_eunit, h_unit, h_bitflag,
         h_percent_units, h_actual_start, h_work_todate, h_start_date, h_finish_date,
         h_effort, h_delay, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
         h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
         h_work_percent, h_task_id, h_rollup_bits
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = a_temp_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 184+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(185) : cplan.sql(15437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, a_temp_id: %s\n", SC_STATE,a_temp_id);//trace_flush();
      --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY,/*UNRESOLVED_START, UNRESOLVED_FINISH*/ START_DT, FINISH_DT, UNRESOLVED_DURATION,
            ASSIGN_COUNT, SD, WORK_TODATE, PLANNED_EFFORT + SCHED_EFFORT, WORK_PERCENT, ETC, COALESCE(FORECAST_START,FORECAST_FINISH)
            INTO h_project_id, h_task_type, h_per_day, h_start_date, h_finish_date, h_duration,
            h_assign_count, h_work_type, h_work_todate, h_effort, h_work_percent, h_etc, h_start_DT
            FROM TMT_WBS WHERE ELEMENT_ID = a_temp_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 204+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(205) : cplan.sql(15457) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_task_type: %d, h_work_percent: %d, h_start_date: %s, h_finish_date: %s\n", SC_STATE,h_task_type,h_work_percent,h_start_date,h_finish_date);//trace_flush();
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 212+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(213) : cplan.sql(15465) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 215+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(216) : cplan.sql(15468) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, a_temp_id, True);
         End If;
         
         h_sched_start := p_planned_start;
         h_sched_finish := p_planned_finish;
         If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
               h_sched_finish, h_finish_date, 10) <> 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 225+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(226) : cplan.sql(15478) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 228+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(229) : cplan.sql(15481) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, a_temp_id
                  , True);
         End If;
         --trace_printf("p_work_percent: %d, h_sched_start: %s, h_sched_finish: %s\n", *p_work_percent,h_sched_start,h_sched_finish);//trace_flush();
         --trace_printf("p_actual: %d, h_effort: %d, h_work_todate: %d\n", *p_actual,h_effort,h_work_todate);//trace_flush();
         
         If ((h_start_DT IS NOT NULL)) OR ((Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0)) <> Coalesce(
               h_etc, 0)) Then
            --trace_printf("ACCEPT_REJECT: SC_STATE: %d, h_start_DT: %d %s, h_etc: %d\n", SC_STATE,h_start_DT_Ind,h_start_DT,h_etc);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 240+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(241) : cplan.sql(15493) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_ASSIGN_ACCEPT_RE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 243+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(244) : cplan.sql(15496) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_ASSIGN_ACCEPT_RE, a_temp_id
                  , True);
         End If;
         
         If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0) OR (PMO_ORASTD.strncmp(h_sched_finish
               , h_finish_date, 10) <> 0) Then
            If Coalesce(h_assign_count, 0) = 0 Then
               If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_start
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_start, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 264+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(265) : cplan.sql(15517) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_start, 1, 10)), PMO_ORASTD.TIME('08:00')) INTO h_sched_start
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 279+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(280) : cplan.sql(15532) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  If PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0 Then
                     h_sched_finish := NULL;
                  End If;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_finish
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_finish, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 300+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(301) : cplan.sql(15553) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME('17:00')) INTO h_sched_finish
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 315+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(316) : cplan.sql(15568) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  h_duration := 0;
               End If;
               
               If Coalesce(h_work_type, 0) = PMO_CNST.SC_SS Then
                  BEGIN
                     UPDATE TMT_WBS SET SCHED_START = h_sched_start, SCHED_FINISH = h_sched_finish, SCHED_DURATION = h_duration
                     WHERE ELEMENT_ID = a_temp_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 338+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(339) : cplan.sql(15591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TMT_WBS SET PLANNED_START = h_sched_start, PLANNED_FINISH = h_sched_finish, PLANNED_DURATION = h_duration
                     WHERE ELEMENT_ID = a_temp_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 356+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(357) : cplan.sql(15609) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  h_temp_var := FC_Rollup(h_project_id);
               End If;
            Else
               BEGIN
                  SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (
                  SELECT 'X' FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id AND TASK_ID = a_temp_id
                  AND ACTUAL_START IS NOT NuLL);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 377+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(378) : cplan.sql(15630) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("1_* SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);
               
               If Coalesce(h_temp_var, 0) > 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 385+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(386) : cplan.sql(15638) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 388+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(389) : cplan.sql(15641) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, a_temp_id
                        , True);
               End If;
               
               If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_start
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_start, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 406+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(407) : cplan.sql(15659) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_start, 1, 10)), PMO_ORASTD.TIME('08:00')) INTO h_sched_start
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 421+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(422) : cplan.sql(15674) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_finish
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_finish, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 438+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(439) : cplan.sql(15691) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME('17:00')) INTO h_sched_finish
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 453+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(454) : cplan.sql(15706) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               h_temp_var := FC_CHANGE_TA_DATES(h_project_id, a_temp_id, h_start_date, h_finish_date, h_sched_start
                                 , h_sched_finish, h_per_day, h_rec_user, Coalesce(h_task_type, 0)
                                 , 0);
               --trace_printf("***** FC_CHANGE_TA_DATES h_temp_var: %d\n", h_temp_var);
               If Coalesce(h_temp_var, 0) = 0 Then
                  h_temp_var := FC_Rollup(h_project_id);
               Else
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 468+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(469) : cplan.sql(15721) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 471+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(472) : cplan.sql(15724) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
               End If;
            End If;
         ElsIf (Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0)) AND (Coalesce(h_effort, 0) > 0) Then
            h_close_task := 0;
            If Coalesce(p_actual, 0) > Coalesce(h_effort, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_PARTICIPANT', 480+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_PARTICIPANT(481) : cplan.sql(15733) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 483+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(484) : cplan.sql(15736) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH, True);
            ElsIf Coalesce(p_actual, 0) = Coalesce(h_effort, 0) Then
               h_close_task := 1;
            End If;
            
            -- varify if there are steps in timesheets
            h_temp_var := 0;
            BEGIN
               SELECT COUNT('X') INTO h_temp_var
               FROM DUAL
               WHERE EXISTS(SELECT A.ELEMENT_ID FROM TASK_ASSIGNMENTS TA, ACTIVITIES A
               WHERE TA.TASK_ID = a_temp_id
               AND TA.TSK_ASSIGNMENT_ID = A.TSK_ASSIGNMENT_ID
               AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT_ID
               AND A.TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 506+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(507) : cplan.sql(15759) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 513+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(514) : cplan.sql(15766) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 516+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(517) : cplan.sql(15769) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, a_temp_id
                     , True);
            End If;
            
            -- varify if there are approved timesheets
            BEGIN
               SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (
               SELECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv
               ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURCE_ID
               WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = a_temp_id AND
               (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     ) <= apv.WEEK_OF);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 535+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(536) : cplan.sql(15788) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("*** h_temp_var: %d\n", h_temp_var);
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 543+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(544) : cplan.sql(15796) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TS_APPROVAL_ERROR), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 546+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(547) : cplan.sql(15799) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TS_APPROVAL_ERROR, a_temp_id, True);
            End If;
            
            h_new_actual := p_actual;
            --InitializeHost(SC_PARAM(h_work_percent), int(((h_new_actual / h_effort) * 10000.00) + 0.4));      // new actual work
            --h_billable = (h_billable | SC_BIT_30);
            --trace_printf("h_work_percent: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_new_actual,h_work_todate,h_effort);
            --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
            
            h_temp_var := FC_U_TASK_PERCENT(h_project_id, a_temp_id, h_new_actual, h_rec_user, Coalesce(
                                h_close_task, 0));
         End If;
         
         --trace_printf("***** FC_U_TASK_PERCENT h_temp_var: %d\n", h_temp_var);
         If Coalesce(h_temp_var, 0) = 0 Then
            h_temp_var := FC_Rollup(h_project_id);
         End If;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 567+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(568) : cplan.sql(15820) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 570+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(571) : cplan.sql(15823) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
      ElsIf (p_planned_start IS NOT NULL) AND (p_planned_finish IS NOT NULL) Then
         h_sched_start := p_planned_start;
         h_sched_finish := p_planned_finish;
         
         If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
               h_sched_finish, h_finish_date, 10) <> 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 580+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(581) : cplan.sql(15833) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 583+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(584) : cplan.sql(15836) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, a_temp_id
                  , True);
         End If;
         
         BEGIN
            SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY
            INTO h_project_id, h_task_type, h_per_day
            FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 599+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(600) : cplan.sql(15852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_task_type: %d, h_per_day: %d\n", SC_STATE,h_task_type,h_per_day);//trace_flush();
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            a_resource_type := 1;
         Else
            a_resource_type := 0;
         End If;
         
         i_resched := 0;
         i_bitflag := 0;
         
         --trace_printf("SC_STATE: %d, h_percent_units: %d, h_sched_start: %s, h_sched_finish: %s\n", SC_STATE,h_percent_units,h_sched_start,h_sched_finish);
         --trace_printf("h_start_date: %s, h_finish_date: %s\n", h_start_date,h_finish_date);
         --trace_printf("h_last_submitted: %d:%s, h_work_todate: %d, h_effort = %d\n", h_last_submitted_Ind,h_last_submitted,h_work_todate,h_effort);
         --trace_printf("p_units: %d, h_percent_units: %d\n", *p_units,h_percent_units);
         --trace_printf("p_delay: %d, h_delay: %d\n", *p_delay,h_delay);
         --trace_printf("p_work_percent: %d, h_work_percent: %d\n", *p_work_percent,h_work_percent);
         If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0) OR (PMO_ORASTD.strncmp(h_sched_finish
               , h_finish_date, 10) <> 0) OR (Coalesce(p_delay, 0) <> Coalesce(h_delay, 0)) Then
            If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
               BEGIN
                  SELECT TASK_TYPE, MINUTE_PER_DAY INTO h_task_type, h_per_day
                  FROM TMT_WBS
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 633+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(634) : cplan.sql(15886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_task_type: %d, h_task_id: %s\n", SC_STATE,h_task_type,h_task_id);//trace_flush();
               
               /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
                                   {
                                       BEGIN_SQL
                                           SELECT TIMESTAMP(DATE(HOST(h_sched_start)), TIME(HOST(h_start_date)))
                                           INTO HOST(h_sched_start)
                                           FROM DUAL;
                                       END_SQL;
               
                                       if (checkFlagStatus(h_bit_flag, '2'))
                                       {
                                           InitializeHost(SC_PARAM(h_work_type), 2);
                                       }
                                       else
                                       {
                                           InitializeHost(SC_PARAM(h_work_type), 0);
                                       }
               
                                       i_bitflag = strcmp(h_sched_start,h_start_date); // new date , old date
                                       InitializeHost(SC_PARAM(h_tph_delete), -1);
                                       h_temp_var = FC_TPHASE_SCHED_ASSIGN(
                                                   SC_PARAM(h_project_id),
                                                   SC_PARAM(h_task_id),
                                                   SC_PARAM(a_temp_id),
                                                   SC_PARAM(h_SD),
                                                   SC_PARAM(h_FD),
                                                   SC_PARAM(h_reg_cost),
                                                   SC_PARAM(h_reg_sale),
                                                   SC_PARAM(h_tph_delete),
                                                   SC_PARAM(h_capital_expense),
                                                   SC_PARAM(h_billable));
               
                                       if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
                                       {
                                           BEGIN_SQL
                                               UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                                               WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(a_temp_id) AND DAY_YEAR = DATE(HOST(h_last_submitted));
                                           END_SQL;
                                       }
               
                                       h_temp_var = FC_FD_CTD_TDY( SC_PARAM(h_project_id),
                                                                   SC_PARAM(h_task_id),
                                                                   SC_PARAM(a_temp_id),
                                                                   SC_PARAM(h_res_id),
                                                                   SC_PARAM(h_res_type),
                                                                   SC_PARAM(h_SD),             // old task assign dates
                                                                   SC_PARAM(h_FD),
                                                                   SC_PARAM(h_sched_start),    // new task dates
                                                                   SC_PARAM(h_finish_date),
                                                                   SC_PARAM(h_work_type),
                                                                   SC_PARAM(h_per_day),
                                                                   SC_PARAM(h_percent_units),
                                                                   SC_PARAM(h_rec_user), 1, i_bitflag );
               
                                       BEGIN_SQL
                                           SELECT DATE(HOST(h_sched_start)), DATE(HOST(h_finish_date)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                                           INTO HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                                           FROM TASK_ASSIGNMENTS
                                           WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) ;
                                       END_SQL;
               
                                       InitializeHost(SC_PARAM(h_tph_delete), 0);
                                       h_temp_var = FC_TPHASE_SCHED_ASSIGN(
                                                   SC_PARAM(h_project_id),
                                                   SC_PARAM(h_task_id),
                                                   SC_PARAM(a_temp_id),
                                                   SC_PARAM(h_SD),
                                                   SC_PARAM(h_FD),
                                                   SC_PARAM(h_reg_cost),
                                                   SC_PARAM(h_reg_sale),
                                                   SC_PARAM(h_tph_delete),
                                                   SC_PARAM(h_capital_expense),
                                                   SC_PARAM(h_billable));
               
                                       h_temp_var = FC_Rollup(h_project_id);
                                       RETURN_STATUS(h_temp_var, a_res_assignment_id);
                                   }
                                   else
                                   {*/
               h_start_date := h_sched_start;
               h_delay := 0;
               i_resched := 1;
               --}
            ElsIf PMO_ORASTD.strncmp(h_sched_finish, h_finish_date, 10) <> 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME(h_finish_date))
                  INTO h_sched_finish
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 732+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(733) : cplan.sql(15985) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               i_bitflag := PMO_ORASTD.strcmp(h_sched_finish, h_finish_date);
               -- new date , old date
               h_tph_delete := -1;
               h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 757+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(758) : cplan.sql(16010) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_work_type := 2;
               Else
                  h_work_type := 0;
               End If;
               
               h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, a_temp_id, h_res_id, h_res_type, h_SD, h_FD
                            , h_start_date, h_sched_finish, h_work_type, h_per_day, h_percent_units
                            , h_rec_user, 2, i_bitflag, 0);
               
               BEGIN
                  SELECT TO_DATE(SubStr(h_start_date, 1, 10)), TO_DATE(SubStr(h_sched_finish, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 784+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(785) : cplan.sql(16037) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_tph_delete := 0;
               h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               h_temp_var := FC_Rollup(h_project_id);
               
               PMO_DB2M.SC_STATE := 0;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_PARTICIPANT', 797+1) THEN 
                  LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_PARTICIPANT(798) : cplan.sql(16050) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_res_assignment_id, True);
            ElsIf Coalesce(h_delay, 0) < Coalesce(p_delay, 0) Then
               h_delay := Coalesce(p_delay, 0) - Coalesce(h_delay, 0);
               i_resched := 1;
            ElsIf Coalesce(p_delay, 0) = 0 Then
               h_delay := 0;
               i_resched := 1;
               
               BEGIN
                  SELECT START_DT INTO h_start_date
                  FROM TMT_WBS
                  WHERE ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 817+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(818) : cplan.sql(16070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_delay, 0) > Coalesce(p_delay, 0) Then
               h_delay := Coalesce(h_delay, 0) - Coalesce(p_delay, 0);
               h_dayofyear := NULL;
               stmt := '';
               
               If Coalesce(a_resource_type, 0) = 1 Then
                  stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                         || 'WHERE ELEMENT_ID = ''' || h_project_id || ''' AND DAY_YEAR < TO_DATE(SubStr(''' 
                              || h_start_date || ''', 1, 10)) '
                         || 'ORDER BY DAY_YEAR DESC) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
                  
                  OPEN C_P_DELAY FOR stmt;
               Else
                  stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                         || 'WHERE ELEMENT_ID = ''' || h_res_id || ''' AND DAY_YEAR < TO_DATE(SubStr(''' || h_start_date 
                              || ''', 1, 10)) '
                         || 'ORDER BY DAY_YEAR DESC) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
                  
                  OPEN C_R_DELAY FOR stmt;
               End If;
               
               LOOP
                  If Coalesce(a_resource_type, 0) = 1 Then
                     FETCH C_P_DELAY INTO h_dayofyear;
                     IF C_P_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  Else
                     FETCH C_R_DELAY INTO h_dayofyear;
                     IF C_R_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  End If;
                  
                  If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                     EXIT;
                  End If;
               END LOOP;
               
               If Coalesce(a_resource_type, 0) = 1 Then
                  BEGIN
                     CLOSE C_P_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 873+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(874) : cplan.sql(16126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     CLOSE C_R_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 886+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(887) : cplan.sql(16139) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               --trace_printf("PPPPPPP000: h_dayofyear: %d, %s\n%s\n", h_dayofyear_Ind, h_dayofyear,stmt);
               If (h_dayofyear IS NULL) Then
                  If Coalesce(a_resource_type, 0) = 1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 896+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(897) : cplan.sql(16149) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 899+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(900) : cplan.sql(16152) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_project_id, True);
                  Else
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 904+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(905) : cplan.sql(16157) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 907+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(908) : cplan.sql(16160) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
                  End If;
               End If;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 923+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(924) : cplan.sql(16176) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("PPPPPPP0001: h_start_date: %d, %s\n", h_start_date_Ind, h_start_date);
               h_delay := 0;
               i_resched := 1;
            End If;
         ElsIf Coalesce(p_units, 0) <> Coalesce(h_percent_units, 0) Then
            If Coalesce(h_work_todate, 0) = Coalesce(h_effort, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 934+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(935) : cplan.sql(16187) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 937+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(938) : cplan.sql(16190) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_temp_id, True);
            End If;
            
            If Coalesce(p_units, 0) = 0 Then
               h_percent_units := 100;
            Else
               If (Coalesce(p_units, 0) < 1) OR (Coalesce(p_units, 0) > 103079040) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_PARTICIPANT', 947+1) THEN 
                     LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_PARTICIPANT(948) : cplan.sql(16200) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 950+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(951) : cplan.sql(16203) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
               End If;
               
               h_percent_units := p_units;
            End If;
            
            --h_day_min = int(((h_per_day * h_percent_units) / 10000.00) + 0.5);
            --if (h_effort == h_per_day  h_day_min >= h_effort)
            --{
            --                //RULE if update %units and CONTOUR > 0 THEN CHANGE TO ASAP
            --                BEGIN_SQL
            --                    UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, UNITS_MSP = HOST(h_percent_units),
            --                        C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 1,
            --                        ROLLUP_BITS = BIT_OR(ROLLUP_BITS,(SC_SCHED_BIT+SC_PLANNED_BIT))
            --                    WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
            --                END_SQL;
            -- RETURN_ERROR(SC_STATE, a_temp_id);
            --}
            
            If (h_prj_assign_id IS NOT NULL) Then
               BEGIN
                  SELECT UNITS_MSP INTO h_temp_var
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 982+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(983) : cplan.sql(16235) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_percent_units, 0) > Coalesce(
                     h_temp_var, 0)) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 990+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(991) : cplan.sql(16243) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 993+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(994) : cplan.sql(16246) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               End If;
            End If;
            
            /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
                            {
                                InitializeHost(SC_PARAM(h_tph_delete), -1);
                                h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                                            SC_PARAM(h_project_id),
                                            SC_PARAM(h_task_id),
                                            SC_PARAM(a_temp_id),
                                            SC_PARAM(h_SD),
                                            SC_PARAM(h_FD),
                                            SC_PARAM(h_reg_cost),
                                            SC_PARAM(h_reg_sale),
                                            SC_PARAM(h_tph_delete),
                                            SC_PARAM(h_capital_expense),
                                            SC_PARAM(h_billable));
            
                                if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
                                {
                                    BEGIN_SQL
                                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                                        WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(a_temp_id) AND DAY_YEAR = DATE(HOST(h_last_submitted));
                                    END_SQL;
                                }
            
                                h_minutes = int(((h_percent_units * h_per_day) / 10000.00) + 0.4);
                                h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                                          SC_PARAM(a_temp_id),
                                                          SC_PARAM(h_start_date),
                                                          SC_PARAM(h_per_day),
                                                          SC_PARAM(h_minutes));
            
                                InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
                                h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(a_temp_id), SC_PARAM(h_temp_var));
            
                                BEGIN_SQL
                                    SELECT DATE(HOST(h_start_date)), DATE(HOST(h_sched_finish)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                                    INTO HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                                    FROM TASK_ASSIGNMENTS
                                    WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) ;
                                END_SQL;
            
                                InitializeHost(SC_PARAM(h_tph_delete), 0);
                                h_temp_var = FC_TPHASE_SCHED_ASSIGN(
                                            SC_PARAM(h_project_id),
                                            SC_PARAM(h_task_id),
                                            SC_PARAM(a_temp_id),
                                            SC_PARAM(h_SD),
                                            SC_PARAM(h_FD),
                                            SC_PARAM(h_reg_cost),
                                            SC_PARAM(h_reg_sale),
                                            SC_PARAM(h_tph_delete),
                                            SC_PARAM(h_capital_expense),
                                            SC_PARAM(h_billable));
            
                                h_temp_var = FC_Rollup(h_project_id);
                                RETURN_ERROR(h_temp_var, a_temp_id);
                            }
                            else
                            {*/
            h_delay := 0;
            i_resched := 2;
            If Coalesce(h_bitflag, 0) < 90 Then
               h_bitflag := 1;
            End If;
            
            --move the update of task assignments tables when I-resched = 2 since this is the only case i_resched is initilaized to 2
            --RULE if update %units and CONTOUR > 0 THEN CHANGE TO ASAP
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, UNITS_MSP = h_percent_units,
               C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 1,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = a_temp_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1081+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1082) : cplan.sql(16334) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_contour, 0) <> 0 Then
               h_contour := 0;
            End If;
            --}
            --trace_printf("**: h_task_type: %d, h_minutes: %d, h_percent_units: %d\n", h_task_type,h_minutes,h_percent_units);
         End If;
         
         --trace_printf("**===> i_resched: %d, h_last_day: %d, h_prj_assign_id: %d %s\n", i_resched,h_last_day,h_prj_assign_id_Ind,h_prj_assign_id);
         If i_resched > 0 Then
            h_tph_delete := -1;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                          , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
            
            If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id, a_temp_id
                                   , h_last_submitted, h_last_day, 0);
            End If;
            
            --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
            If (h_prj_assign_id IS NOT NULL) Then
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 2);
            Else
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 0);
               Else
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 1);
               End If;
            End If;
            
            If (h_last_submitted IS NOT NULL) Then
               If Coalesce(h_contour, 0) > 0 Then
                  h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                  
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1129+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1130) : cplan.sql(16382) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1144+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1145) : cplan.sql(16397) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
            End If;
            
            --trace_printf("1_FC_DEL_NOACTUAL: h_start_date: %s, h_effort: %d, h_delay: %d, h_percent_units: %d\n",h_start_date,h_effort,h_delay,h_percent_units);
            
            h_temp_bitflag := 2;
            h_temp_sc_state := FC_RESCHEDULE(a_temp_id, h_start_date, h_effort, h_temp_bitflag, h_contour
                                 , h_rec_user, Coalesce(h_delay, 0), Coalesce(h_percent_units, 0));
            --trace_printf("2_FC_RESCHEDULE: h_temp_sc_state: %d, i_resched: %d\n",h_temp_sc_state,i_resched); //trace_flush();
            
            If Coalesce(h_temp_sc_state, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1162+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1163) : cplan.sql(16415) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1165+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1166) : cplan.sql(16418) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, a_temp_id, True);
            ElsIf i_resched = 1 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),
                  BAK_START_DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(SCHED_FINISH,PLANNED_FINISH),
                  BAK_ASSIGN_STRING = ASSIGN_STRING
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1184+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1185) : cplan.sql(16437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_temp_var := FC_Rollup(h_project_id);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1192+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1193) : cplan.sql(16445) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1195+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1196) : cplan.sql(16448) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
         End If;
         
         If Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0) Then
            -- varify if there are steps in timesheets
            BEGIN
               SELECT COUNT('X') INTO h_temp_var
               FROM DUAL
               WHERE EXISTS(SELECT ELEMENT_ID FROM ACTIVITIES
               WHERE TSK_ASSIGNMENT_ID = a_temp_id
               AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
               AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1215+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1216) : cplan.sql(16468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1222+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1223) : cplan.sql(16475) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1225+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1226) : cplan.sql(16478) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, a_temp_id
                     , True);
            End If;
            
            h_new_actual := p_actual;
            -- new actual work
            h_sched_effort := Coalesce(h_effort, 0);
            If Coalesce(h_new_actual, 0) > Coalesce(h_effort, 0) Then
               h_tph_delete := -1;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                          , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               If (h_prj_assign_id IS NOT NULL) Then
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 2);
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 0);
                  Else
                     h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 1);
                  End If;
               End If;
               
               h_minutes := 0;
               If (h_last_submitted IS NOT NULL) Then
                  If Coalesce(h_contour, 0) > 0 Then
                     h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1263+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1264) : cplan.sql(16516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1278+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1279) : cplan.sql(16531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  If Coalesce(h_last_day, 0) > 0 Then
                     BEGIN
                        SELECT SCHED_MIN - ACTUAL_MIN INTO h_minutes
                        FROM TASK_DAY_OF_YEAR 
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1295+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1296) : cplan.sql(16548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_work_todate, 0) - Coalesce(h_minutes, 0);
               
               h_work_percent := 10000;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                  SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000,
                  FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1321+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1322) : cplan.sql(16574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  --trace_printf("*** h_contour: %d, h_new_actual: %d, h_start_date: %s, h_effort: %d, h_work_todate: %d\n",h_contour,h_new_actual,h_start_date,h_effort,h_work_todate);
                  h_temp_bitflag := 2;
                  h_temp_var := FC_RESCHEDULE(a_temp_id, h_start_date, h_effort, h_temp_bitflag, h_contour, h_rec_user
                            , 0, Coalesce(h_percent_units, 0));
                  --trace_printf("3_FC_RESCHEDULE: h_temp_sc_state: %d, i_resched: %d\n",h_temp_sc_state,i_resched); //trace_flush();
                  
                  If Coalesce(h_temp_var, 0) <> 0 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1335+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1336) : cplan.sql(16588) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1338+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1339) : cplan.sql(16591) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
                  End If;
               End If;
               
               h_effort := Coalesce(h_new_actual, 0);
            Else
               h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
            End If;
            
            --trace_printf("h_work_percent: %d, h_last_day: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_last_day,h_new_actual,h_work_todate,h_effort); //trace_flush();
            --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD); //trace_flush();
            
            h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, a_temp_id, h_last_submitted
                                  , h_bit_flag, h_effort, h_new_actual, h_sched_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, Coalesce(h_per_day, 0));
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1360+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1361) : cplan.sql(16613) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1363+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1364) : cplan.sql(16616) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
         ElsIf Coalesce(p_work_percent, 0) <> Coalesce(h_work_percent, 0) Then
            If (Coalesce(h_work_todate, 0) > 0) AND (Coalesce(p_work_percent, 0) = 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1369+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1370) : cplan.sql(16622) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1372+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1373) : cplan.sql(16625) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_temp_id, True);
            End If;
            
            /*if (*p_work_percent == 10000)
                            {
                                BEGIN_SQL
                                    UPDATE TASK_ASSIGNMENTS SET
                                        SCHED_WORK = WORK_TODATE,
                                        ETC = 0, FD = SC_AF,
                                        MILESTONE_PERCENT = 10000
                                    WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
                                END_SQL;
            
                                if( SC_SUCCESS )
                                {
                                    InitializeHost(SC_PARAM(h_temp_var), 1);
                                    h_temp_var = FC_ETC_SCHEDULE(SC_PARAM(a_temp_id), SC_PARAM(h_temp_var), SC_PARAM(h_rec_user));
                                }
                            }
                            else if ( h_work_percent == 10000 )
                            {
                                BEGIN_SQL
                                    UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_security),
                                    ETC = SCHED_WORK - WORK_TODATE, FD = SC_SF, ACTUAL_FINISH = SC_NULL, ACTUAL_DURATION = 0
                                    WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
                                END_SQL;
                            }
                            else
                            {
                                BEGIN_SQL
                                    UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_security),
                                    ETC = SCHED_WORK - WORK_TODATE
                                    WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
                                END_SQL;
                            }*/
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = p_work_percent,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
               WHERE TSK_ASSIGNMENT_ID = a_temp_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1424+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1425) : cplan.sql(16677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            /*
                            BEGIN_SQL
                                UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = HOST(h_rollup_bits) WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
                            END_SQL;
            */
            BEGIN
               UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1446+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1447) : cplan.sql(16699) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --RETURN_ERROR(SC_STATE, a_temp_id);
         End If;
         
         h_temp_var := FC_Rollup(h_project_id);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1456+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1457) : cplan.sql(16709) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1459+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1460) : cplan.sql(16712) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
      Else
         BEGIN
            SELECT ra.SECURITY_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,
            CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
            CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
            ra.RESOURCE_ID,
            ra.PROJECT_ID, ra.TABLE_TYPE
            INTO h_security_id, a_participant, a_element_id, a_element_type,
            a_resource_type,
            h_transfed_cri,
            a_participant_id,
            h_project_id, h_table_type
            FROM RESOURCE_ASSIGNMENTS ra WHERE ra.RES_ASSIGNMENT_ID = a_res_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1481+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1482) : cplan.sql(16734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1489+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1490) : cplan.sql(16742) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1492+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1493) : cplan.sql(16745) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, a_res_assignment_id
                  , True);
         End If;
         
         If PMO_ORASTD.strcmp(a_security_id, h_security_id) = 0 Then
            PMO_DB2M.SC_STATE := 0;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1501+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1502) : cplan.sql(16754) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1504+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1505) : cplan.sql(16757) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_res_assignment_id, True);
         End If;
      End If;
   End If;
   
   a_temp_id := assign_id;
   a_security_id := security_id001;
   h_rec_user := res_id001;
   
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   PMO_DB2M.SC_STATE := FC_U_ASSIGNED_SECURITY(h_table_type, a_temp_id, a_security_id, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_PARTICIPANT', 1519+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_PARTICIPANT(1520) : cplan.sql(16772) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_temp_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1526+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1527) : cplan.sql(16779) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1538+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1539) : cplan.sql(16791) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_PARTICIPANT;
ALTER FUNCTION SP_U_PARTICIPANT COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_CRI_SECURITY ==
CREATE OR REPLACE FUNCTION SP_U_CRI_SECURITY
(
     assign_id VARCHAR2
   , security_id001 VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , p_delay PLS_INTEGER
   , p_units PLS_INTEGER
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   a_element_id VARCHAR2(32);
   a_participant_id VARCHAR2(255);
   a_participant_name VARCHAR2(255);
   a_security_name VARCHAR2(32);
   h_full_name VARCHAR2(255);
   a_res_assignment_id VARCHAR2(32);
   a_security_id VARCHAR2(32);
   a_temp_id VARCHAR2(32);
   h_created_by VARCHAR2(32);
   h_project_id VARCHAR2(32);
   a_assignment_count PLS_INTEGER := 0;
   a_element_type PLS_INTEGER := 0;
   h_responsible PLS_INTEGER := 0;
   a_participant PLS_INTEGER := 0;
   a_resource_type PLS_INTEGER := 0;
   a_security_value PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_wbs_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_cri_table PLS_INTEGER := 0;
   h_cri_element_type PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CRI_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CRI_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CRI_SECURITY(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || security_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CRI_SECURITY', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CRI_SECURITY(61) : cplan.sql(16873) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_created_by := '';
   
   --Validate res_assignment_id
   a_temp_id := assign_id;
   a_security_id := security_id001;
   h_rec_user := res_id001;
   
   h_table_type := PMO_CNST.SC_TABLE_CRI;
   PMO_DB2M.SC_STATE := FC_U_ASSIGNED_SECURITY(h_table_type, a_temp_id, a_security_id, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CRI_SECURITY', 76+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CRI_SECURITY(77) : cplan.sql(16889) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_temp_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CRI_SECURITY', 83+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CRI_SECURITY(84) : cplan.sql(16896) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CRI_SECURITY', 95+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CRI_SECURITY(96) : cplan.sql(16908) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CRI_SECURITY;
ALTER FUNCTION SP_U_CRI_SECURITY COMPILE DEBUG;

/****************************************************************************************
 id = TSK_ASSIGN_ID
 bitflag:
        1 = Copy Schedule to Baseline (includes budgets)
        2 = Copy Planned to Schedule
        3 = Copy Proposed to Schedule
        4 = Copy Forecast to Schedule (includes budgets)
        5 = Copy Forecast  Scheduled to Planned

        6 = Move Planned to Schedule
        7 = Append Planned to Schedule

 bussiness rules:
    1. Do not copy date for completed tasks (MILESTONE_PERCENT = 100)
    2. If the task has ACTUAL do not change ETC, WORK_TODATE, SD, FD
    3. When copy Forecast Date to ..., ETC is Forecast Effort, and clear Forcast dates after copy
    4. If both Forcast dates exists ignor forcast start, consider only forcast finish and ETC
    5. Calculated Forecast Start should not be in the past (is >= NOW)
    6. When copy Forcast to and assignments exists forcast rollup wins, and clear Forcast dates after copy
    7. Actual Finish date does not rollup from Assignments to Task
    8. When copy Planned to Schedule clear Planned Dates, Duration  Effort
****************************************************************************************/

Prompt == FUNCTION SP_COMMIT_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_COMMIT_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_project_id VARCHAR2(32);
   h_dayofyear DATE;
   h_res_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_work PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_change_flag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_tsk_prj PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   stmt VARCHAR2(7999);
-- Declare Cursors
      CURSOR C270 IS
         SELECT TA.TSK_ASSIGNMENT_ID, TA.RESOURCE_ID
         FROM TASK_ASSIGNMENTS TA JOIN TMT_WBS W
         ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
         WHERE W.PROJECT_ID = h_project_id AND TA.BIT_FLAG = '0'
         AND TA.RESOURCE_TYPE = '0' AND TA.PROJECT_ASSIGNMENT_ID IS Null;

      -- Dynamic Cursor Declaration
      CMM1 PMO_DB2M.REF_CURSOR;

      CURSOR C27 IS
         SELECT DAY_YEAR, SCHED_MIN, BIT_SCHED FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND BIT_FLAG = '0';

      CURSOR C54 IS
         SELECT TASK_ID, TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE BIT_FLAG = '0' AND PROJECT_ASSIGNMENT_ID = h_id;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_COMMIT_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_COMMIT_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_COMMIT_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 62+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_COMMIT_ASSIGN(63) : cplan.sql(17008) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_id := id;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(80) : cplan.sql(17025) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_bitflag = %d, h_id = %s\n",SC_STATE, h_bitflag, h_id);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT PROJECT_ID, CASE WHEN PROJECT_ID = TASK_ID THEN 1 ELSE 0 END, CASE WHEN PROJECT_ASSIGNMENT_ID IS NuLL THEN 0 ELSE 1 END
         INTO h_project_id,h_tsk_prj,h_from_prj
         FROM TASK_ASSIGNMENTS WHERE MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE='0' AND TSK_ASSIGNMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 96+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(97) : cplan.sql(17042) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_COMMIT_ASSIGN', 103+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_COMMIT_ASSIGN(104) : cplan.sql(17049) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 106+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_COMMIT_ASSIGN(107) : cplan.sql(17052) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
      ElsIf (Coalesce(h_tsk_prj, 0) = 0) AND (Coalesce(h_from_prj, 0) = 1) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_COMMIT_ASSIGN', 111+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_COMMIT_ASSIGN(112) : cplan.sql(17057) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_EXISTS), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 114+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_COMMIT_ASSIGN(115) : cplan.sql(17060) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_EXISTS, h_id, True);
      End If;
      
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 127+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(128) : cplan.sql(17073) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      stmt := 'SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENTS '
             || 'WHERE TSK_ASSIGNMENT_ID = ''' || h_id 
                  || ''' AND MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE=''0'' AND BIT_FLAG = ''0'' ';
   Else
      h_project_id := h_id;
      
      --UPDATING CALENDAR_DAY_OF_YEAR  TASK_DAY_OF_YEAR
      If Coalesce(h_bitflag, 0) = 3 Then
         --wbs changes
         BEGIN
            UPDATE TMT_WBS SET
            TYPE_ID         = CASE WHEN TYPE_ID = PMO_CNST.SC_PROPOSAL THEN PMO_CNST.SC_PROJECT ELSE TYPE_ID END,
            SCHED_START     = COALESCE(PLANNED_START, SCHED_START),
            SCHED_FINISH    = COALESCE(PLANNED_FINISH, SCHED_FINISH),
            SCHED_DURATION  = CASE WHEN PLANNED_DURATION = 0 THEN SCHED_DURATION ELSE PLANNED_DURATION END,
            SCHED_EFFORT    = PLANNED_EFFORT,
            PLANNED_START   = Null,
            PLANNED_FINISH  = Null,
            PLANNED_DURATION = 0,
            FORECAST_START  = Null,
            FORECAST_FINISH = Null,
            FORECAST_EFFORT = PLANNED_EFFORT,
            ETC             = PLANNED_EFFORT,
            FORECAST_DURATION = 0
            WHERE PROJECT_ID = h_project_id AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND ASSIGN_COUNT = 0 AND WORK_TODATE = 0 AND SCHED_EFFORT = 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 167+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(168) : cplan.sql(17113) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN C270;
         
         LOOP
            FETCH C270 INTO h_tsk_assignment_id, h_res_id;
            IF C270%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               BIT_FLAG = '2',
               SCHED_START = COALESCE(PLANNED_START, SCHED_START),
               SCHED_FINISH = COALESCE(PLANNED_FINISH, SCHED_FINISH),
               SCHED_DURATION = CASE WHEN PLANNED_DURATION > 0 THEN PLANNED_DURATION ELSE SCHED_DURATION END,
               SCHED_WORK = PLANNED_WORK,
               ETC = PLANNED_WORK,
               BAK_START_DT = COALESCE(PLANNED_START, SCHED_START),
               BAK_FINISH_DT = COALESCE(PLANNED_FINISH, SCHED_FINISH),
               BAK_WORK = PLANNED_WORK,
               SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
               SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
               FORECAST_LABOR_COST = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST END,
               FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
               PLANNED_START = Null, PLANNED_FINISH = Null, PLANNED_DURATION = 0, PLANNED_WORK = 0,
               FORECAST_START = Null, FORECAST_FINISH = Null, FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(214) : cplan.sql(17159) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2'
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 231+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(232) : cplan.sql(17177) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         --end of while
         
         BEGIN
            CLOSE C270 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 246+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(247) : cplan.sql(17192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT
         WHERE PROJECT_ID = h_project_id AND ASSIGN_COUNT > 0 AND WORK_TODATE = 0 AND SCHED_EFFORT = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 265+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(266) : cplan.sql(17211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_tsk_prj := 1;
      h_bitflag := 2;
      
      stmt := 
            'SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = TASK_ID AND '
             || 'PROJECT_ID = ''' || h_id 
                  || ''' AND MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE=''0'' AND BIT_FLAG = ''0'' ';
   End If;
   
   --TODO When Planned To Schedule merge to old schedule to the Project Assignments
   -- Copy Planned To Schedule for a task assignment if from Constrained by Project Assignment is not allowed
   --trace_printf("%s\n", (char *) stmt);
   
   OPEN CMM1 FOR stmt;
   
   LOOP
      FETCH CMM1 INTO h_id, h_res_id, h_task_id, h_work;
      IF CMM1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      --trace_printf("h_bitflag = %d, h_id = %s, h_work = %d\nh_res_id = %s, h_task_id = %s\n",h_bitflag, h_id, h_work, h_res_id, h_task_id);
      
      CASE h_bitflag
         
         WHEN 1 THEN
            --11111111111111111111111111111111111111111111111111111111111111111111111111111
            --1 = Schedule to Baseline (includes budgets)
            --budget changes
            BEGIN
               UPDATE BUDGETS B SET
               B.ESTIMATED_BASELINE3 = B.ESTIMATED_BASELINE2,
               B.ESTIMATED_BASELINE2 = B.ESTIMATED_BASELINE1,
               B.ESTIMATED_BASELINE1 = B.ESTIMATED_BUDGET,
               B.BUDGET_BASELINE3 = B.BUDGET_BASELINE2,
               B.BUDGET_BASELINE2 = B.BUDGET_BASELINE1,
               B.BUDGET_BASELINE1 = B.BUDGET
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 323+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(324) : cplan.sql(17269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --assignment changes
            BEGIN
               UPDATE TASK_ASSIGNMENTS TA SET
               TA.BASELINE3_START = TA.BASELINE2_START, TA.BASELINE3_FINISH = TA.BASELINE2_FINISH, TA.BASELINE3_DURATION = TA.BASELINE2_DURATION, TA.BASELINE3_WORK = TA.BASELINE2_WORK,
               TA.BASELINE3_LABOR_COST = TA.BASELINE2_LABOR_COST, TA.BASELINE3_LABOR_REVENUE = TA.BASELINE2_LABOR_REVENUE,
               TA.BASELINE2_START = TA.BASELINE1_START, TA.BASELINE2_FINISH = TA.BASELINE1_FINISH, TA.BASELINE2_DURATION = TA.BASELINE1_DURATION, TA.BASELINE2_WORK = TA.BASELINE1_WORK,
               TA.BASELINE2_LABOR_COST = TA.BASELINE1_LABOR_COST, TA.BASELINE2_LABOR_REVENUE = TA.BASELINE1_LABOR_REVENUE,
               TA.BASELINE1_START = TA.UNRESOLVED_START, TA.BASELINE1_FINISH = TA.UNRESOLVED_FINISH, TA.BASELINE1_DURATION = TA.UNRESOLVED_DURATION, TA.BASELINE1_WORK = TA.SCHED_WORK + TA.PLANNED_WORK,
               TA.BASELINE1_LABOR_COST = TA.SCHED_LABOR_COST + TA.PLANNED_LABOR_COST, TA.BASELINE1_LABOR_REVENUE = TA.SCHED_LABOR_REVENUE + TA.PLANNED_LABOR_REVENUE,
               TA.B_REGULAR_COST = TA.REGULAR_COST, TA.B_REGULAR_RATE = TA.REGULAR_RATE, TA.B_SPECIAL_COST = TA.SPECIAL_COST, TA.B_SPECIAL_RATE = TA.SPECIAL_RATE
               WHERE TSK_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(350) : cplan.sql(17295) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_BASELINE3_START = ESTIMATED_BASELINE2_START,
               ESTIMATED_BASELINE2_START = ESTIMATED_BASELINE1_START,
               ESTIMATED_BASELINE1_START = CASE WHEN ESTIMATED_SCHED_START IS NOT NULL THEN ESTIMATED_SCHED_START ELSE ESTIMATED_PLANNED_START END,
               ESTIMATED_BASELINE3_FINISH = ESTIMATED_BASELINE2_FINISH,
               ESTIMATED_BASELINE2_FINISH = ESTIMATED_BASELINE1_FINISH,
               ESTIMATED_BASELINE1_FINISH = CASE WHEN ESTIMATED_SCHED_FINISH IS NOT NULL THEN ESTIMATED_SCHED_FINISH ELSE ESTIMATED_PLANNED_FINISH END,
               ESTIMATED_BASELINE3_EFFORT = ESTIMATED_BASELINE2_EFFORT,
               ESTIMATED_BASELINE2_EFFORT = ESTIMATED_BASELINE1_EFFORT,
               ESTIMATED_BASELINE1_EFFORT = CASE WHEN ESTIMATED_SCHED_EFFORT > 0 THEN ESTIMATED_SCHED_EFFORT ELSE ESTIMATED_PLANNED_EFFORT END,
               ESTIMATED_BASELINE3_COST = ESTIMATED_BASELINE2_COST,
               ESTIMATED_BASELINE2_COST = ESTIMATED_BASELINE1_COST,
               ESTIMATED_BASELINE1_COST = CASE WHEN ESTIMATED_SCHED_COST > 0.00 THEN ESTIMATED_SCHED_COST ELSE ESTIMATED_PLANNED_COST END,
               ESTIMATED_BASELINE3_REVENUE = ESTIMATED_BASELINE2_REVENUE,
               ESTIMATED_BASELINE2_REVENUE = ESTIMATED_BASELINE1_REVENUE,
               ESTIMATED_BASELINE1_REVENUE = CASE WHEN ESTIMATED_SCHED_REVENUE > 0.00 THEN ESTIMATED_SCHED_REVENUE ELSE ESTIMATED_PLANNED_REVENUE END,
               BASELINE_START = UNRESOLVED_START,
               BASELINE_FINISH = UNRESOLVED_FINISH,
               BASELINE_DURATION = UNRESOLVED_DURATION,
               BASELINE_WORK = SCHED_WORK + PLANNED_WORK,
               BASELINE_LABOR_COST = SCHED_LABOR_COST + PLANNED_LABOR_COST,
               BASELINE_LABOR_REVENUE = SCHED_LABOR_REVENUE + PLANNED_LABOR_REVENUE
               WHERE PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 388+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(389) : cplan.sql(17334) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            
         
         WHEN 2 THEN
            --22222222222222222222222222222222222222222222222222222222222222222222222222222
            --2 = Planned to Schedule
            -- check if there is assignment in planned
            BEGIN
               SELECT PLANNED_WORK INTO h_day_min FROM TASK_ASSIGNMENTS WHERE BIT_FLAG = '0' AND TSK_ASSIGNMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 407+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(408) : cplan.sql(17353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR ((h_day_min IS NULL)) Then
               GOTO CaseEnd0000;
            End If;
            
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, CASE WHEN C_EFFORT = 0 THEN 0 ELSE C_EFFORT + h_work END
               INTO h_prj_assignment_id, h_work
               FROM TASK_ASSIGNMENTS WHERE BIT_FLAG = '2' AND TASK_ID = h_project_id AND RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 426+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(427) : cplan.sql(17372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("h_tsk_prj = %d, h_id = %s, h_prj_assignment_id = %s, h_work = %d\n",h_tsk_prj, h_id, h_prj_assignment_id, h_work);
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_prj_assignment_id IS NOT NULL)) Then
               If Coalesce(h_tsk_prj, 0) = 0 Then
                  h_total := 2;
               Else
                  -- delete the project assignment in planned
                  BEGIN
                     DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 449+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(450) : cplan.sql(17395) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 466+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(467) : cplan.sql(17412) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 483+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(484) : cplan.sql(17429) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 500+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(501) : cplan.sql(17446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_total := 1;
               End If;
            Else
               h_prj_assignment_id := h_id;
               h_total := 0;
            End If;
            
            h_day_min := 0;
            --trace_printf("h_total = %d, h_id = %s, h_prj_assignment_id= %s\n",h_total,h_id,h_prj_assignment_id);
            
            --Merge task day of year assignments
            
            OPEN C27;
            
            LOOP
               FETCH C27 INTO h_dayofyear, h_day_min, h_bitID;
               IF C27%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               --trace_printf("h_dayofyear = %s, h_day_min = %d\n",h_dayofyear,h_day_min);
               
               If Coalesce(h_total, 0) = 1 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_day_min,
                     BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 547+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(548) : cplan.sql(17493) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1__ SC_STATE = %d\n",SC_STATE);
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 566+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(567) : cplan.sql(17512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 583+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(584) : cplan.sql(17529) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf Coalesce(h_total, 0) = 2 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 601+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(602) : cplan.sql(17547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C27 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 615+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(616) : cplan.sql(17561) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2', PARENT_ID = h_prj_assignment_id
               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 633+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(634) : cplan.sql(17579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_tsk_prj, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  BIT_FLAG = '2',
                  SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF,
                  SCHED_START = PLANNED_START,
                  SCHED_FINISH = PLANNED_FINISH,
                  SCHED_DURATION = PLANNED_DURATION,
                  SCHED_WORK = PLANNED_WORK,
                  BAK_START_DT = PLANNED_START,
                  BAK_FINISH_DT = PLANNED_FINISH,
                  BAK_WORK = PLANNED_WORK,
                  SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
                  SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
                  ETC = PLANNED_WORK,
                  FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST,
                  FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
                  PLANNED_START = NuLL, PLANNED_FINISH = NuLL, PLANNED_DURATION = 0, PLANNED_WORK = 0,
                  FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
                  PROJECT_ASSIGNMENT_ID = NuLL
                  WHERE TSK_ASSIGNMENT_ID = h_id AND SCHED_WORK = 0 AND PLANNED_WORK > 0;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 669+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(670) : cplan.sql(17615) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_total, 0) = 2 Then
                  -- CALL UPDATE assign string function
                  h_temp_var := 3;
                  h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
               End If;
            Else
               -- update task assignments
               
               OPEN C54;
               
               LOOP
                  FETCH C54 INTO h_task_id, h_tsk_assignment_id;
                  IF C54%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     BIT_FLAG = '2',
                     SD = PMO_CNST.SC_SS,
                     FD = PMO_CNST.SC_SF,
                     SCHED_START = PLANNED_START,
                     SCHED_FINISH = PLANNED_FINISH,
                     SCHED_DURATION = PLANNED_DURATION,
                     SCHED_WORK = PLANNED_WORK,
                     BAK_START_DT = PLANNED_START,
                     BAK_FINISH_DT = PLANNED_FINISH,
                     BAK_WORK = PLANNED_WORK,
                     SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
                     SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
                     ETC = PLANNED_WORK,
                     FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST,
                     FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
                     PLANNED_START = NuLL, PLANNED_FINISH = NuLL, PLANNED_DURATION = 0, PLANNED_WORK = 0,
                     FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
                     PROJECT_ASSIGNMENT_ID = h_prj_assignment_id
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 726+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(727) : cplan.sql(17672) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("SC_STATE = %d, h_tsk_assignment_id = %s, h_prj_assignment_id = %s\n",SC_STATE,h_tsk_assignment_id,h_prj_assignment_id);
                  
                  -- Flag ROLLUP_BITS at the task
                  BEGIN
                     UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 745+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(746) : cplan.sql(17691) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE C54 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 759+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(760) : cplan.sql(17705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (Coalesce(h_tsk_prj, 0) = 1) AND (Coalesce(h_work, 0) > 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET C_EFFORT = h_work WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 777+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(778) : cplan.sql(17723) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               -- CALL UPDATE assign string function
               h_temp_var := 3;
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
               
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, BEST, SCHED_START, SCHED_FINISH, BILLABLE, REGULAR_COST, REGULAR_RATE
                  INTO h_prj_assignment_id, h_sched_work, h_start_date, h_finish_date, h_billable, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID   = h_prj_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 797+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(798) : cplan.sql(17743) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_start_date = %s, h_finish_date: %s, h_billable: %d, h_sched_work: %d\n", SC_STATE,h_start_date,h_finish_date,h_billable,h_sched_work);//trace_flush();
               
               If Coalesce(h_billable, 0) = 0 Then
                  BEGIN
                     UPDATE PROJECT_ASSIGNMENTS SET
                     ESTIMATED_PLANNED_START = NuLL,
                     ESTIMATED_PLANNED_FINISH = NuLL,
                     ESTIMATED_PLANNED_EFFORT = 0,
                     ESTIMATED_PLANNED_COST = 0.00,
                     ESTIMATED_PLANNED_REVENUE = 0.00,
                     ESTIMATED_SCHED_START = h_start_date,
                     ESTIMATED_SCHED_FINISH = h_finish_date,
                     ESTIMATED_SCHED_EFFORT = h_sched_work,
                     ESTIMATED_SCHED_COST = (h_sched_work / 60.0) * h_reg_cost,
                     ESTIMATED_SCHED_REVENUE = 0.00
                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 827+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(828) : cplan.sql(17773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE PROJECT_ASSIGNMENTS SET
                     ESTIMATED_PLANNED_START = NuLL,
                     ESTIMATED_PLANNED_FINISH = NuLL,
                     ESTIMATED_PLANNED_EFFORT = 0,
                     ESTIMATED_PLANNED_COST = 0.00,
                     ESTIMATED_PLANNED_REVENUE = 0.00,
                     ESTIMATED_SCHED_START = h_start_date,
                     ESTIMATED_SCHED_FINISH = h_finish_date,
                     ESTIMATED_SCHED_EFFORT = h_sched_work,
                     ESTIMATED_SCHED_COST = (h_sched_work / 60.0) * h_reg_cost,
                     ESTIMATED_SCHED_REVENUE = (h_sched_work / 60.0) * h_reg_sale
                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 855+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(856) : cplan.sql(17801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("SC_STATE: %d, h_reg_cost: %.2f, h_reg_sale: %2f\n", SC_STATE,h_reg_cost,h_reg_sale);
            End If;
            
            <<CaseEnd0000>>
            NULL;
         ELSE
            NULL;
      END CASE;
      
      -- Flag ROLLUP_BITS at the project/task
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 882+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(883) : cplan.sql(17828) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("ROLLUP_BITS: SC_STATE = %d, h_task_id = %s, h_project_id = %s\n",SC_STATE,h_task_id,h_project_id);
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE CMM1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 898+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(899) : cplan.sql(17844) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   BEGIN
      SELECT CASE WHEN (SD <> PMO_CNST.SC_NS) AND (FD <> PMO_CNST.SC_NF) AND (TYPE_ID = PMO_CNST.SC_PROPOSAL) THEN 1 ELSE 0 END
      INTO h_change_flag
      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 916+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(917) : cplan.sql(17862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_change_flag, 0) = 1 Then
      h_type_id := PMO_CNST.SC_PROPOSAL;
      PMO_DB2M.SC_STATE := FC_CHANGE_PROJ_TYPE(h_project_id, h_change_flag, h_type_id);
   End If;
   
   BEGIN
      UPDATE RESOURCE_ASSIGNMENTS SET
      ELEMENT_TYPE  = (SELECT TYPE_ID FROM TMT_WBS WHERE ELEMENT_ID = h_project_id)
      WHERE ELEMENT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 940+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(941) : cplan.sql(17886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_COMMIT_ASSIGN', 950+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_COMMIT_ASSIGN(951) : cplan.sql(17896) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 957+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(958) : cplan.sql(17903) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 969+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(970) : cplan.sql(17915) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_COMMIT_ASSIGN;
ALTER FUNCTION SP_COMMIT_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Accept ETC and/or FORECAST_START, Assign level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_ACPT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_constr_DT TIMESTAMP;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_FF_flag PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_projaffected_start DATE;
   h_work_todate PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_not_effort_PA PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ACPT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ACPT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_ACPT_ETC_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ACPT_ETC_ASSIGN(61) : cplan.sql(17998) : ' || SYSTIMESTAMP);
   END IF; 

   --project assignmnet start date
   
   -- Parse parameters to local variables
   
   h_bitflag := bitflag001;
   h_tsk_assignment_id := id;
   h_rec_user := rec_user001;
   --trace_printf("h_bitflag = %d, h_tsk_assignment_id = %s\n", h_bitflag,h_tsk_assignment_id);//trace_flush();
   
   -- Process Effort Task
   h_start_date := NULL;
   -- forecast start
   h_finish_date := NULL;
   -- forecast finish
   
   BEGIN
      SELECT
      PROJECT_ID,
      TASK_ID,
      PROJECT_ASSIGNMENT_ID,
      SCHED_WORK+PLANNED_WORK,
      FORECAST_START,
      FORECAST_FINISH,
      CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE ETC END,
      SCHED_WORK+PLANNED_WORK - WORK_TODATE,
      ETC,
      C_CONTOUR,
      LAST_SUBMITTED,
      CASE WHEN FORECAST_FINISH IS NOT NuLL AND FORECAST_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(PLANNED_START, SCHED_START) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1 ELSE 0 END,
      UNRESOLVED_START,
      UNRESOLVED_FINISH,
      BILLABLE,
      CAPITAL_EXPENSE,
      REGULAR_COST,
      REGULAR_RATE,
      BIT_FLAG,
      MILESTONE_PERCENT,
      RESOURCE_ID,
      RESOURCE_TYPE,
      WORK_TODATE
      INTO
      h_project_id,
      h_task_id,
      h_prj_assignment_id,
      h_sched_work,
      h_start_date,
      h_finish_date,
      h_effort,
      h_remaining,
      h_etc,
      h_contour,
      h_last_submitted,
      h_FF_flag,
      h_ta_start_DT,
      h_ta_finish_DT,
      h_billable,
      h_capital_expense,
      h_reg_cost,
      h_reg_sale,
      h_bit_flag,
      h_mlst_percent,
      h_res_id,
      h_res_type,
      h_work_todate
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
      AND (((SCHED_WORK+PLANNED_WORK) > 0
      AND (SCHED_WORK+PLANNED_WORK - WORK_TODATE) <> ETC)
      OR FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 138+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(139) : cplan.sql(18076) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE,h_finish_date_Ind,h_finish_date);//trace_flush();
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ACPT_ETC_ASSIGN', 145+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ACPT_ETC_ASSIGN(146) : cplan.sql(18083) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 148+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ACPT_ETC_ASSIGN(149) : cplan.sql(18086) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_tsk_assignment_id
            , True);
   End If;
   
   h_temp_sc_state := FC_ACPT_EFFORT(h_project_id, h_task_id, h_tsk_assignment_id, h_prj_assignment_id
                                  , h_sched_work, h_start_date, h_finish_date, h_effort, h_remaining
                                  , h_etc, h_contour, h_last_submitted, h_FF_flag, h_ta_start_DT
                                  , h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost
                                  , h_reg_sale, h_bit_flag, h_mlst_percent, h_res_id, h_res_type
                                  , h_work_todate, h_bitflag, h_rec_user);
   
   --trace_printf("h_temp_sc_state: %d\n", h_temp_sc_state);
   
   If Coalesce(h_temp_sc_state, 0) > 200000 Then
      --trace_printf("it comes here--before returning the error\n");
      --RETURN_ERROR(CANNOT_ASSIGN_PROJ_AFFECT_DATES, h_tsk_assignment_id);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ACPT_ETC_ASSIGN', 167+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ACPT_ETC_ASSIGN(168) : cplan.sql(18105) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 170+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ACPT_ETC_ASSIGN(171) : cplan.sql(18108) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_sc_state, 0), h_tsk_assignment_id
            , True);
   Else
      h_total := 0;
      
      --get the remaining after reschedule and compared with etc if less the update etc to be accepted/rejected
      If Coalesce(h_FF_flag, 0) <> 1 Then
         BEGIN
            SELECT ETC INTO h_total
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 189+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(190) : cplan.sql(18127) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_etc, 0) > Coalesce(h_total, 0) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS
               SET ETC  = h_etc
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 209+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(210) : cplan.sql(18147) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_temp_sc_state := FC_Rollup(h_project_id);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ACPT_ETC_ASSIGN', 220+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ACPT_ETC_ASSIGN(221) : cplan.sql(18158) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 227+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(228) : cplan.sql(18165) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 239+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(240) : cplan.sql(18177) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ACPT_ETC_ASSIGN;
ALTER FUNCTION SP_ACPT_ETC_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Accept ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST DATES
    3 - ETC + FORECAST DATES
****************************************************************************************/

Prompt == FUNCTION SP_ACPT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_WBS
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_finish_D DATE;
   h_week_of DATE;
   h_last_submitted DATE;
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_constr_DT TIMESTAMP;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_not_effort_PA PLS_INTEGER := 0;
   h_FF_flag PLS_INTEGER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CAWTSK IS
         SELECT PROJECT_ID, ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,
         CASE WHEN FORECAST_START IS NULL THEN COALESCE(PLANNED_START, SCHED_START) ELSE FORECAST_START END,
         CASE WHEN FORECAST_FINISH IS NULL THEN COALESCE(PLANNED_FINISH, SCHED_FINISH) ELSE FORECAST_FINISH END
         FROM TMT_WBS WHERE RANK LIKE h_rank AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND (((SCHED_EFFORT+PLANNED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_TODATE) <> ETC)) OR
         (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL)) ORDER BY COALESCE(PLANNED_START, SCHED_START);

      CURSOR CAWASS IS
         SELECT
         TSK_ASSIGNMENT_ID,
         PROJECT_ASSIGNMENT_ID,
         SCHED_WORK+PLANNED_WORK,
         FORECAST_START,
         FORECAST_FINISH,
         CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE ETC END,
         SCHED_WORK+PLANNED_WORK - WORK_TODATE,
         ETC,
         C_CONTOUR,
         LAST_SUBMITTED,
         CASE WHEN FORECAST_FINISH IS NOT NuLL AND FORECAST_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(PLANNED_START, SCHED_START) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1 ELSE 0 END,
         UNRESOLVED_START,
         UNRESOLVED_FINISH,
         BILLABLE,
         CAPITAL_EXPENSE,
         REGULAR_COST,
         REGULAR_RATE,
         BIT_FLAG,
         MILESTONE_PERCENT,
         RESOURCE_ID,
         RESOURCE_TYPE,
         WORK_TODATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_task_id
         AND (((SCHED_WORK+PLANNED_WORK) > 0
         AND ((SCHED_WORK + PLANNED_WORK - WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ACPT_ETC_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ACPT_ETC_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_ACPT_ETC_WBS(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_WBS', 105+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ACPT_ETC_WBS(106) : cplan.sql(18305) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_rec_user := rec_user001;
   h_not_effort_PA := 0;
   
   BEGIN
      SELECT RTRIM(RANK), TYPE_ID INTO h_rank, h_type
      FROM TMT_WBS WHERE ELEMENT_ID = id/*HOST(h_id)*/;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 122+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(123) : cplan.sql(18322) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_type IS NOT NULL)) Then
      h_rank := SubStr(h_rank || '%', 1, 200);
      h_effort := -1;
      
      --as we can change forecast dates in proposed mode we need to fetch proposed dates also
      
      OPEN CAWTSK;
      
      LOOP
         FETCH CAWTSK INTO h_project_id, h_task_id, h_effort, h_start_date, h_finish_date;
         IF CAWTSK%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         --trace_printf("h_project_id: %s, h_task_id: %s\n", h_project_id, h_task_id);
         --trace_printf("h_bitflag = %d, h_effort: %d, h_start_date: %d %s, h_finish_date: %d %s\n",h_bitflag,h_effort,h_start_date_Ind,h_start_date, h_finish_date_Ind,h_finish_date);
         
         If Coalesce(h_effort, 0) = 0 Then
            -- Process Duration Task
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Accept FORECAST Dates
               null_return := FC_ACPT_DURATION(h_project_id, h_task_id, h_start_date, h_finish_date, h_rec_user);
               
               h_passed := Coalesce(h_passed, 0) + 1;
            End If;
         Else
            -- Process Effort Task
            h_start_date := NULL;
            -- forecast start
            h_finish_date := NULL;
            -- forecast finish
            
            OPEN CAWASS;
            
            LOOP
               FETCH CAWASS INTO
               h_tsk_assignment_id,
               h_prj_assignment_id,
               h_sched_work,
               h_start_date,
               h_finish_date,
               h_effort,
               h_remaining,
               h_etc,
               h_contour,
               h_last_submitted,
               h_FF_flag,
               h_ta_start_DT,
               h_ta_finish_DT,
               h_billable,
               h_capital_expense,
               h_reg_cost,
               h_reg_sale,
               h_bit_flag,
               h_mlst_percent,
               h_res_id,
               h_res_type,
               h_work_todate;
               IF CAWASS%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               --trace_printf("0_h_ta_start_DT %d %s, h_ta_finish_DT %d, %s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_ta_finish_DT_Ind,h_ta_finish_DT);
               --trace_printf("0_h_start_date %d %s, h_finish_date %d, %s\n", h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
               
               h_temp_sc_state := FC_ACPT_EFFORT(h_project_id, h_task_id, h_tsk_assignment_id, h_prj_assignment_id
                                  , h_sched_work, h_start_date, h_finish_date, h_effort, h_remaining
                                  , h_etc, h_contour, h_last_submitted, h_FF_flag, h_ta_start_DT
                                  , h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost
                                  , h_reg_sale, h_bit_flag, h_mlst_percent, h_res_id, h_res_type
                                  , h_work_todate, h_bitflag, h_rec_user);
               
               If Coalesce(h_temp_sc_state, 0) = 0 Then
                  h_passed := Coalesce(h_passed, 0) + 1;
               ElsIf Coalesce(h_temp_sc_state, 0) = PMO_CNST.CANNOT_ACCEPT_FROM_PROJECT_AFF Then
                  h_not_effort_PA := Coalesce(h_not_effort_PA, 0) + 1;
               End If;
            END LOOP;
            -- end of while
            
            BEGIN
               CLOSE CAWASS ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 225+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(226) : cplan.sql(18425) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("h_passed: %d, h_temp_sc_state: %d, h_constr_type: %d, h_constr_DT: %s\n", h_passed, h_temp_sc_state,h_constr_type,h_constr_DT);
            --trace_printf("h_start_DT_Ind = %d, h_finish_DT_Ind = %d, h_start_DT = %s, h_finish_DT = %s\n", h_start_DT_Ind,h_finish_DT_Ind,h_start_DT,h_finish_DT);//trace_flush();
            
            If (Coalesce(h_bitflag, 0) <> 1) AND (Coalesce(h_passed, 0) > 0) Then
               h_start_DT := NULL;
               h_finish_DT := NULL;
               
               BEGIN
                  SELECT CONSTRAINT_DT, CONSTRAINT_TYPE, FORECAST_START, FORECAST_FINISH
                  INTO h_constr_DT, h_constr_type, h_start_DT, h_finish_DT
                  FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 247+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(248) : cplan.sql(18447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("h_start_DT_Ind = %d, h_finish_DT_Ind = %d, h_start_DT = %s, h_finish_DT = %s\n", h_start_DT_Ind,h_finish_DT_Ind,h_start_DT,h_finish_DT);//trace_flush();
               
               If (h_finish_DT IS NOT NULL) Then
                  -- fix constraint date, forecast finish
                  If (h_constr_DT IS NOT NULL) Then
                     BEGIN
                        UPDATE TMT_WBS SET
                        CONSTRAINT_DT =
                        CASE
                        WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN h_finish_DT
                        WHEN h_constr_type IN (0,5,6,7) THEN h_finish_DT
                        ELSE h_constr_DT
                        END,
                        CONSTRAINT_TYPE =
                        CASE
                        WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN 2
                        WHEN h_constr_type IN (0,5,6,7) THEN 2
                        ELSE h_constr_type
                        END
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 281+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(282) : cplan.sql(18481) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TMT_WBS SET CONSTRAINT_DT = h_finish_DT, CONSTRAINT_TYPE = 2
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 299+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(300) : cplan.sql(18499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf ((h_start_DT IS NOT NULL)) AND ((h_finish_DT IS NULL)) Then
                  -- fix constraint date, forecast start
                  If (h_constr_DT IS NOT NULL) Then
                     BEGIN
                        UPDATE TMT_WBS SET
                        CONSTRAINT_DT =
                        CASE
                        WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_DT
                        WHEN h_constr_type IN (1,2,3,4) THEN h_start_DT
                        ELSE h_constr_DT
                        END,
                        CONSTRAINT_TYPE =
                        CASE
                        WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN 6
                        WHEN h_constr_type IN (1,2,3,4) THEN 6
                        ELSE h_constr_type
                        END
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 332+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(333) : cplan.sql(18532) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TMT_WBS SET CONSTRAINT_DT = h_start_DT, CONSTRAINT_TYPE = 6
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 350+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(351) : cplan.sql(18550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            End If;
         End If;
      END LOOP;
      -- end of while
      
      BEGIN
         CLOSE CAWTSK ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 369+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(370) : cplan.sql(18569) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("END: h_passed = %d, h_effort: %d, h_temp_sc_state: %d\n",h_passed,h_effort,h_temp_sc_state);
      --trace_printf("END: h_not_effort_PA: %d\n", h_not_effort_PA);
      
      If (Coalesce(h_passed, 0) > 0) AND (Coalesce(h_effort, 0) > -1) Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 1 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PROPOSED_BIT)
            WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 404+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(405) : cplan.sql(18604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
      End If;
   End If;
   
   If Coalesce(h_not_effort_PA, 0) > 0 Then
      PMO_DB2M.SC_STATE := PMO_CNST.CANNOT_ACCEPT_FROM_PROJECT_AFF;
   ElsIf Coalesce(h_temp_sc_state, 0) > 100 Then
      PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ACPT_ETC_WBS', 421+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ACPT_ETC_WBS(422) : cplan.sql(18621) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 428+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(429) : cplan.sql(18628) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 440+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(441) : cplan.sql(18640) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ACPT_ETC_WBS;
ALTER FUNCTION SP_ACPT_ETC_WBS COMPILE DEBUG;

/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_RJCT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_steps_exist PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   i_percent_complete PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RJCT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RJCT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_RJCT_ETC_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RJCT_ETC_ASSIGN', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RJCT_ETC_ASSIGN(31) : cplan.sql(18693) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_tsk_assignment_id := id;
   
   BEGIN
      SELECT PROJECT_ID, TASK_ID, BIT_FLAG, SCHED_WORK+PLANNED_WORK, WORK_TODATE, MILESTONE_PERCENT
      INTO h_project_id, h_task_id, h_bit_flag, h_sched_work, h_work_todate, h_mlst_percent
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(48) : cplan.sql(18710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      If SubStr(h_bit_flag, 1, 1) = '2' Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ETC = SCHED_WORK - WORK_TODATE--, MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 67+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(68) : cplan.sql(18730) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE ACTIVITIES
            SET ETC_MODIFIED = 'N'
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
            AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 88+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(89) : cplan.sql(18751) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ETC = PLANNED_WORK, WORK_TODATE = 0, MILESTONE_PERCENT = 0
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 106+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(107) : cplan.sql(18769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 124+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(125) : cplan.sql(18787) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      If Coalesce(h_bitflag, 0) = 2 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 143+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(144) : cplan.sql(18806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 160+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(161) : cplan.sql(18823) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If SubStr(h_bit_flag, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
               ETC = SCHED_WORK - WORK_TODATE--, MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(181) : cplan.sql(18843) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE ACTIVITIES
               SET ETC_MODIFIED = 'N'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
               AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(202) : cplan.sql(18864) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
               ETC = PLANNED_WORK, WORK_TODATE = 0, MILESTONE_PERCENT = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 220+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(221) : cplan.sql(18883) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 238+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(239) : cplan.sql(18901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
      i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
      If (i_percent_complete > 1) OR i_percent_complete < -1 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),
            ACTUAL_FINISH = CASE
            WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
            ELSE ACTUAL_FINISH
            END
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 267+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(268) : cplan.sql(18930) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         ACTUAL_FINISH = CASE
         WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
         ELSE ACTUAL_FINISH
         END
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(291) : cplan.sql(18953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_bitflag, 0) <> 2) Then
      -- Call Fix_Summary_TS and Effort_Week
      h_temp_sc_state := FC_FIX_SUMMARY_TS(h_project_id, h_tsk_assignment_id);
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RJCT_ETC_ASSIGN', 305+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RJCT_ETC_ASSIGN(306) : cplan.sql(18968) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 312+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(313) : cplan.sql(18975) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 324+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(325) : cplan.sql(18987) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RJCT_ETC_ASSIGN;
ALTER FUNCTION SP_RJCT_ETC_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_RJCT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_WBS
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_finish_date TIMESTAMP;
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_start_date TIMESTAMP;
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_start_D DATE;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_nodata PLS_INTEGER := 0;
   h_steps_exist PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_temp_sc_state_alert PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_calltype PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   i_steps_exist PLS_INTEGER NOT NULL := 0;
   i_percent_complete PLS_INTEGER;
-- Declare Cursors
      CURSOR CAWTSKR IS
         SELECT ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,
         CASE WHEN FORECAST_START IS NULL THEN SCHED_START ELSE FORECAST_START END,
         CASE WHEN FORECAST_FINISH IS NULL THEN SCHED_FINISH ELSE FORECAST_FINISH END
         FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND (((SCHED_EFFORT+PLANNED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

      CURSOR CAWASSR IS
         SELECT TSK_ASSIGNMENT_ID, BIT_FLAG, BEST, SCHED_WORK+PLANNED_WORK, WORK_TODATE, MILESTONE_PERCENT
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_task_id
         AND (((SCHED_WORK+PLANNED_WORK) > 0 AND ((SCHED_WORK+PLANNED_WORK-WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RJCT_ETC_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RJCT_ETC_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_RJCT_ETC_WBS(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RJCT_ETC_WBS', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RJCT_ETC_WBS(61) : cplan.sql(19070) : ' || SYSTIMESTAMP);
   END IF; 

   /**** Added to issue alert  ****/
   
   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   
   /**** Added to issue alert  ****/
   h_rec_user := rec_user001;
   h_calltype := 3;
   h_alert_id := 'SCETCFORCASTREJECTED';
   
   BEGIN
      SELECT PROJECT_ID, RTRIM(RANK), TYPE_ID
      INTO h_project_id, h_rank, h_type
      FROM TMT_WBS WHERE ELEMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 83+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(84) : cplan.sql(19093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_bitflag: %d, h_rank = %s\n", SC_STATE,h_bitflag,h_rank);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_type IS NOT NULL)) Then
      h_rank := SubStr(h_rank || '%', 1, 200);
      h_effort := -1;
      
      BEGIN
         SELECT START_FINISH_DATE INTO h_start_D FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 101+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(102) : cplan.sql(19111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN CAWTSKR;
      
      LOOP
         FETCH CAWTSKR INTO h_task_id, h_effort, h_start_date, h_finish_date;
         IF CAWTSKR%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         --trace_printf("h_effort = %d, h_task_id = %s\n", h_effort,h_task_id);
         
         If Coalesce(h_effort, 0) = 0 Then
            -- Process Duration based Task
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Reject FORECAST
               BEGIN
                  UPDATE TMT_WBS SET
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 141+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(142) : cplan.sql(19151) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               /**** Added to issue alert  ****/
               h_table_type := 1;
               h_temp_sc_state_alert := FC_ISSUE_ALERT(h_alert_id, h_task_id, h_rec_user, h_calltype, Coalesce(
                                        h_table_type, 0));
            Else
               GOTO Continue0000;
            End If;
         Else
            -- Process Effort based Task
            
            OPEN CAWASSR;
            
            LOOP
               FETCH CAWASSR INTO h_tsk_assignment_id, h_bit_flag, h_temp_var, h_sched_work, h_work_todate, h_mlst_percent;
               IF CAWASSR%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               --trace_printf("h_tsk_assignment_id = %s, h_bitflag = %d, h_temp_var: %d\n", h_tsk_assignment_id,h_bitflag,h_temp_var);
               
               h_temp_sc_nodata := 0;
               If Coalesce(h_bitflag, 0) = 1 Then
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     BEGIN
                        UPDATE ACTIVITIES
                        SET ETC_MODIFIED = 'N'
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                        AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
                        AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 190+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(191) : cplan.sql(19200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If Coalesce(h_temp_var, 0) = -1 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           PROPOSED_START      = SCHED_START,
                           PROPOSED_FINISH     = SCHED_FINISH,
                           PROPOSED_WORK       = SCHED_WORK,
                           BAK_START_DT        = SCHED_START,
                           BAK_FINISH_DT       = SCHED_FINISH,
                           BAK_WORK            = SCHED_WORK,
                           BAK_ASSIGN_STRING   = ASSIGN_STRING,
                           BEST                = SCHED_WORK,
                           ETC                 = SCHED_WORK - WORK_TODATE,
                           ACTUAL_FINISH       = NULL
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 219+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(220) : cplan.sql(19229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        --BEGIN_SQL
                        --    UPDATE TASK_ASSIGNMENTS SET
                        --        SCHED_WORK = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK ELSE SCHED_WORK END,
                        --        ETC = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK - WORK_TODATE ELSE 0 END,
                        --        ACTUAL_FINISH = SC_NULL/*,
                        --        MILESTONE_PERCENT = (WORK_TODATE * 10000.00) / SCHED_WORK*/
                        --    WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
                        --END_SQL;
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           SCHED_WORK	= SCHED_WORK,
                           ETC			= SCHED_WORK - WORK_TODATE
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 247+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(248) : cplan.sql(19257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        ETC					= PLANNED_WORK, 
                        MILESTONE_PERCENT	= 0 ,
                        ACTUAL_FINISH		= NULL, 
                        ACTUAL_DURATION		= 0
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 270+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(271) : cplan.sql(19280) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("SC_STATE = %d, h_tsk_assignment_id = %s\n", SC_STATE,h_tsk_assignment_id);
               Else
                  If Coalesce(h_bitflag, 0) = 2 Then
                     -- Reject FORECAST dates
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        FORECAST_START = NULL,
                        FORECAST_FINISH = NULL,
                        FORECAST_DURATION = 0
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 295+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(296) : cplan.sql(19305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     -- Reject ETC + FORECAST_START
                     If SubStr(h_bit_flag, 1, 1) = '2' Then
                        BEGIN
                           UPDATE ACTIVITIES
                           SET ETC_MODIFIED = 'N'
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                           AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
                           AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 318+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(319) : cplan.sql(19328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           FORECAST_START = NULL,
                           FORECAST_FINISH = NULL,
                           FORECAST_DURATION = 0,
                           SCHED_WORK = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK ELSE SCHED_WORK END,
                           ETC = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK - WORK_TODATE ELSE 0 END/*,
                           MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK*/
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 342+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(343) : cplan.sql(19352) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           FORECAST_START = NULL,
                           FORECAST_FINISH = NULL,
                           FORECAST_DURATION = 0,
                           ETC = PLANNED_WORK, MILESTONE_PERCENT = 0
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 364+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(365) : cplan.sql(19374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               End If;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  h_temp_sc_nodata := 1;
               End If;
               
               /**** Added to issue alert  ****/
               h_table_type := 0;
               h_temp_sc_state_alert := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_calltype
                                        , Coalesce(h_table_type, 0));
               
               --trace_printf("h_temp_sc_nodata: %d, h_bitflag: %d, h_mlst_percent: %d\n", h_temp_sc_nodata,h_bitflag,h_mlst_percent);
               If Coalesce(h_temp_sc_nodata, 0) = 0 Then
                  If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
                     i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
                     If (i_percent_complete > 1) OR i_percent_complete < -1 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),
                           ACTUAL_FINISH = CASE
                           WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
                           ELSE ACTUAL_FINISH
                           END
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 405+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(406) : cplan.sql(19415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           ACTUAL_FINISH = CASE WHEN (ACTUAL_FINISH IS NULL) AND ETC = 0 THEN SCHED_FINISH ELSE ACTUAL_FINISH END
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 424+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(425) : cplan.sql(19434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     --trace_printf("i_percent_complete: %d, h_work_todate: %d, h_sched_work: %d\n", i_percent_complete,h_work_todate,h_sched_work);
                     
                     -- Reject ETC and/or FORECAST_START
                     -- Call Fix_Summary_TS and Effort_Week
                     h_temp_sc_state := FC_FIX_SUMMARY_TS(h_project_id, h_tsk_assignment_id);
                  End If;
               End If;
            END LOOP;
            -- end of while
            
            BEGIN
               CLOSE CAWASSR ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 447+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(448) : cplan.sql(19457) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         -- flag the task, for later rollup
         BEGIN
            UPDATE TMT_WBS SET
            CONSTRAINT_TYPE =
            CASE
            WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> TO_DATE(SubStr(h_start_D, 1, 10)) THEN 6
            ELSE 0
            END,
            CONSTRAINT_DT =
            CASE
            WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> TO_DATE(SubStr(h_start_D, 1, 10)) THEN SCHED_START
            ELSE NULL
            END
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 477+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(478) : cplan.sql(19487) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         <<Continue0000>>
         NULL;
      END LOOP;
      -- end of while
      
      BEGIN
         CLOSE CAWTSKR ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 494+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(495) : cplan.sql(19504) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_effort, 0) > -1 Then
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RJCT_ETC_WBS', 506+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RJCT_ETC_WBS(507) : cplan.sql(19516) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 513+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(514) : cplan.sql(19523) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 525+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(526) : cplan.sql(19535) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RJCT_ETC_WBS;
ALTER FUNCTION SP_RJCT_ETC_WBS COMPILE DEBUG;

--******************************************************************************************

Prompt == FUNCTION SP_U_WP_FORECAST ==
CREATE OR REPLACE FUNCTION SP_U_WP_FORECAST
(
     id VARCHAR2
   , start_date001 TIMESTAMP
   , finish_date001 TIMESTAMP
   , acc_rej_bitflag PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_forecast_start TIMESTAMP;
   h_forecast_finish TIMESTAMP;
   h_actual_start TIMESTAMP;
   h_rank VARCHAR2(200);
   h_mlst_percent PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_last_submitted DATE;
   h_projaffected_start DATE;
   h_dayofyear DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_continue PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_acc_rej_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_FF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_alert_flag PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   null_return LONG;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_FORECAST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_FORECAST'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_FORECAST(' || 
       '''' || id || ''''
          || ', ' || COALESCE(NULLIF('''' || start_date001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || finish_date001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(acc_rej_bitflag), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 66+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_FORECAST(67) : cplan.sql(19618) : ' || SYSTIMESTAMP);
   END IF; 

   --bitflag to indicate the right to accept/reject forecast
   
   --clear the dates before the initilization
   h_start_date := '';
   h_finish_date := '';
   
   h_continue := 1;
   --to let users enter forecast dates at the begining
   h_start_date := start_date001;
   h_finish_date := finish_date001;
   h_tsk_assignment_id := id;
   h_acc_rej_bitflag := acc_rej_bitflag;
   If (Coalesce(h_acc_rej_bitflag, 0) = 1) Then
      h_acc_rej_bitflag := PMO_CNST.SC_ACCEPT_REJECT_FORECAST;
   End If;
   
   BEGIN
      SELECT PROJECT_ID, RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, ACTUAL_START, MILESTONE_PERCENT, BIT_FLAG, 
      COALESCE(FORECAST_START,SCHED_START), COALESCE(FORECAST_FINISH,SCHED_FINISH), FORECAST_START, FORECAST_FINISH, TASK_ID, RESOURCE_ID, SCHED_WORK, (PLANNED_WORK + SCHED_WORK - WORK_TODATE), ETC,
      C_CONTOUR, LAST_SUBMITTED, UNRESOLVED_START, UNRESOLVED_FINISH, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
      INTO h_project_id, h_res_assignment_id, h_prj_assignment_id, h_actual_start, h_mlst_percent, h_bit_flag,
      h_start_DT, h_finish_DT, h_forecast_start, h_forecast_finish, h_task_id, h_res_id, h_sched_work,h_remaining, h_etc,
      h_contour, h_last_submitted, h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 100+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(101) : cplan.sql(19652) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_SD: %s, h_FD: %s\n", SC_STATE,h_SD,h_FD);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 108+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(109) : cplan.sql(19660) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 111+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(112) : cplan.sql(19663) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, id, True);
   End If;
   
   If Coalesce(h_mlst_percent, 0) = 10000 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 118+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(119) : cplan.sql(19670) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_CLOSED), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 121+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(122) : cplan.sql(19673) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.ASSIGNMENT_ALREADY_CLOSED, id, True);
   End If;
   
   If ((start_date001 IS NOT NULL)) AND ((finish_date001 IS NOT NULL)) Then
      If PMO_ORASTD.strncmp(h_finish_date, h_start_date, 10) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 129+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(130) : cplan.sql(19681) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 132+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(133) : cplan.sql(19684) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, id, True);
      End If;
   End If;
   
   h_week_of := NULL;
   BEGIN
      SELECT MAX(WEEK_OF) + 7      INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
      WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
      AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED,PMO_CNST.SC_APV_STATE_SUBMITTED);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 149+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(150) : cplan.sql(19701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_week_of: %d %s\n", h_week_of_Ind,h_week_of);
   
   h_rec_user := rec_user001;
   If (PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0) AND (Coalesce(
         h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST) Then
      h_alert_id := 'SCETCFORCASTCHANGED';
      h_alert_flag := 1;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      
      h_alert_id := 'SCREDUCEDETC';
      h_alert_flag := 0;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
   End If;
   
   -- if client doesn't sent date  time
   If ((h_start_date IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) = 0) Then
      h_start_date := h_start_DT;
   End If;
   
   If ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         FORECAST_START  = start_date001,
         FORECAST_FINISH = finish_date001,
         FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 189+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(190) : cplan.sql(19741) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_start_date: %d %s, h_finish_date: %d %s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   Else
      --if one of the foreast dates is cleared, null, then clear it
      If (start_date001 IS NULL) AND (h_forecast_start IS NOT NULL) Then
         --trace_printf("CLear the FS\n");
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            FORECAST_START  = start_date001,
            FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 214+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(215) : cplan.sql(19766) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --when clear FS but not chnaged FF then just call rollup, else let itcontinue withupdate
         --the same when clear FF, but change or not change FS
         If PMO_ORASTD.strncmp(h_finish_date, h_forecast_finish, 10) = 0 Then
            --trace_printf("FF dates didn't change\n");
            h_continue := 0;
         End If;
      End If;
      
      If (finish_date001 IS NULL) AND (h_forecast_finish IS NOT NULL) Then
         --trace_printf("Clear FF date\n");
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            FORECAST_FINISH = finish_date001,
            FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 244+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(245) : cplan.sql(19796) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --when clear FS but not chnaged FF then just call rollup, else let itcontinue withupdate
         --the same when clear FF, but change or not change FS
         
         If PMO_ORASTD.strncmp(h_start_date, h_forecast_start, 10) = 0 Then
            --trace_printf("FS didn't change\n");
            h_continue := 0;
         End If;
      End If;
      
      If Coalesce(h_continue, 0) = 1 Then
         --trace_printf("one of the forecast dates chnaged\n");
         -- if client doesn't sent date  time
         If ((h_finish_date IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_finish_DT, 10) = 0) Then
            h_finish_date := h_finish_DT;
         End If;
         
         If ((h_week_of IS NOT NULL)) AND ((h_finish_date IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_week_of
               , h_finish_date) > 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 268+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(269) : cplan.sql(19820) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.FORECAST_APPROVAL_EXISTS), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 271+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(272) : cplan.sql(19823) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.FORECAST_APPROVAL_EXISTS, h_res_id, True);
         End If;
         
         If (start_date001 IS NOT NULL) Then
            If (h_actual_start IS NOT NULL) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 279+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(280) : cplan.sql(19831) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 282+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(283) : cplan.sql(19834) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, id, True);
            End If;
            
            --trace_printf("h_start_date: %d %s, h_start_DT: %d %s\n", h_start_date_Ind,h_start_date,h_start_DT_Ind,h_start_DT);
            h_effort := h_etc;
            If PMO_ORASTD.strncmp(h_start_date, h_ta_start_DT, 10) = 0 Then
               h_start_date := NULL;
            ElsIf PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) <> 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_date
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id
                  AND DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 303+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(304) : cplan.sql(19855) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  --Should be replace with c function
                  BEGIN
                     SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(start_date001, 1, 10)), PMO_ORASTD.TIME('08:00')))  INTO  h_start_date FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 318+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(319) : cplan.sql(19870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf (finish_date001 IS NULL) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 325+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(326) : cplan.sql(19877) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 328+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(329) : cplan.sql(19880) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(0, id, True);
            End If;
         Else
            h_effort := 0;
            h_start_date := NULL;
         End If;
         
         If (finish_date001 IS NOT NULL) Then
            --trace_printf("h_start_date: %d %s, h_finish_date: %d %s\n", h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
            If ((start_date001 IS NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_ta_start_DT, 10) < 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 341+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(342) : cplan.sql(19893) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 344+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(345) : cplan.sql(19896) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
            ElsIf ((h_last_submitted IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_last_submitted
                  , 10) <= 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_FORECAST', 350+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_FORECAST(351) : cplan.sql(19902) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 353+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(354) : cplan.sql(19905) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, True);
            End If;
            
            If ((start_date001 IS NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_finish_DT, 10) < 0) Then
               h_FF := 1;
            Else
               h_FF := 0;
            End If;
            --trace_printf("h_FF: %d, h_finish_date: %d %s, h_finish_DT: %d %s\n", h_FF,h_finish_date_Ind,h_finish_date,h_finish_DT_Ind,h_finish_DT);
            
            If PMO_ORASTD.strncmp(h_finish_date, h_ta_finish_DT, 10) = 0 Then
               h_finish_date := NULL;
            ElsIf PMO_ORASTD.strcmp(h_finish_date, h_finish_DT) <> 0 Then
               --trace_printf("when finish_date and finish_DT are the same\n");
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_finish_date
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id
                  AND DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 380+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(381) : cplan.sql(19932) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE, h_finish_date_Ind, h_finish_date);
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  --Should be replace with c function
                  BEGIN
                     SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(finish_date001, 1, 10)), PMO_ORASTD.TIME('17:00')))  INTO  h_finish_date FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 396+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(397) : cplan.sql(19948) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_finish_date, h_ta_start_DT, 10) = 0 Then
               --trace_printf("Forecast finish = Sched finish\n");
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 404+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(405) : cplan.sql(19956) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 407+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(408) : cplan.sql(19959) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(0, id, True);
            End If;
         Else
            h_finish_date := NULL;
         End If;
         
         h_rec_user := rec_user001;
         h_temp_var := 0;
         
         --trace_printf("**** h_acc_rej_bitflag: %d, h_res_id: %s, h_rec_user: %s\n",h_acc_rej_bitflag,h_res_id, h_rec_user);
         If Coalesce(h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               FORECAST_START  = h_start_date,
               FORECAST_FINISH = h_finish_date,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(437) : cplan.sql(19988) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            --check forecast dates against assignmnet dates if project assignmnet exists before calling accept forecast
            
            If (h_prj_assignment_id IS NOT NULL) Then
               --trace_printf("project assignmnet is not null\n");
               If (finish_date001 IS NOT NULL) Then
                  BEGIN
                     SELECT COUNT('X') INTO h_temp_var
                     FROM DUAL
                     WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assignment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id))
                     AND DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10)));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 458+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(459) : cplan.sql(20010) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  If Coalesce(h_temp_var, 0) = 0 Then
                     --trace_printf("FF is outside the assignmnet datse\n");
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 466+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(467) : cplan.sql(20018) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 469+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(470) : cplan.sql(20021) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                           , True);
                  End If;
               ElsIf (start_date001 IS NOT NULL) Then
                  --if FS is outside project affected dates return an error
                  BEGIN
                     SELECT COUNT('X') INTO h_temp_var
                     FROM DUAL
                     WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assignment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id))
                     AND DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10)));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 488+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(489) : cplan.sql(20040) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_temp_var, 0) = 0 Then
                     --trace_printf("FS is outside Project affected dates: h_start_date: %s, h_projaffected_start: %s\n", h_start_date, h_projaffected_start);
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 496+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(497) : cplan.sql(20048) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 499+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(500) : cplan.sql(20051) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                           , True);
                  Else
                     --here we nedd to fetch the data by cehcking prj_assign_id against parent_id, otherwise
                     --in case of changing the start date one day later when we have only 8 hours remaining at project,and shoud acept the start date
                     --doesn't accept if we go by tsk_assignmnet_id = to h_prj_assignmnet_id
                     
                     BEGIN
                        SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
                        WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(start_date001, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 517+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(518) : cplan.sql(20069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     If Coalesce(h_total, 0) < Coalesce(h_etc, 0) Then
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 523+1) THEN 
                           LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(524) : cplan.sql(20075) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                        END IF; 
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 526+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(527) : cplan.sql(20078) : ' || SYSTIMESTAMP);
                        END IF; 
                        PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                              , True);
                     End If;
                  End If;
               Else
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 534+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(535) : cplan.sql(20086) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 537+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(538) : cplan.sql(20089) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_tsk_assignment_id
                        , True);
               End If;
               
            End If;
            -- call accept forcast dates
            h_bitflag := 2;
            h_tsk_assignment_id := id;
            
            If (h_start_date IS NULL) AND ((h_last_submitted IS NOT NULL)) Then
               h_start_date := h_start_DT;
            End If;
            
            h_temp_sc_state := FC_ACPT_ETC_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id
                                      , h_prj_assignment_id, h_start_date, h_finish_date
                                      , h_last_submitted, h_bit_flag, h_effort, h_remaining
                                      , h_sched_work, h_etc, h_contour, h_FF, h_bitflag
                                      , h_ta_start_DT, h_ta_finish_DT, h_reg_cost, h_reg_sale
                                      , h_billable, h_capital_expense, h_rec_user, 0);
            
            h_temp_var := 2;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_var := 3;
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
            End If;
         End If;
      End If;
   End If;
   
   If (PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0) AND (Coalesce(
         h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST) Then
      h_alert_id := 'SCETCFORCASTCHANGED';
      h_alert_flag := 2;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      
      h_alert_id := 'SCREDUCEDETC';
      h_alert_flag := 2;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_FORECAST', 584+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_FORECAST(585) : cplan.sql(20136) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 592+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(593) : cplan.sql(20144) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 604+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(605) : cplan.sql(20156) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_FORECAST;
ALTER FUNCTION SP_U_WP_FORECAST COMPILE DEBUG;

/*****************************************************************************/

Prompt == FUNCTION SP_U_ASSIGN_DESCR ==
CREATE OR REPLACE FUNCTION SP_U_ASSIGN_DESCR
(
     prj_id VARCHAR2
   , res_id001 VARCHAR2
   , description001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ASSIGN_DESCR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ASSIGN_DESCR'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ASSIGN_DESCR(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || description001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ASSIGN_DESCR', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ASSIGN_DESCR(21) : cplan.sql(20193) : ' || SYSTIMESTAMP);
   END IF; 

   If (prj_id IS NULL) OR (res_id001 IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_ASSIGN_DESCR', 25+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_ASSIGN_DESCR(26) : cplan.sql(20198) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || prj_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ASSIGN_DESCR', 28+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_ASSIGN_DESCR(29) : cplan.sql(20201) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, prj_id, True);
   End If;
   
   --Execute update Statement
   BEGIN
      UPDATE PROJECT_ASSIGNMENTS
      SET DESCRIPTION = description001
      WHERE RESOURCE_ID = res_id001
      AND PROJECT_ID = prj_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(50) : cplan.sql(20222) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ASSIGN_DESCR', 55+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ASSIGN_DESCR(56) : cplan.sql(20228) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || prj_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, prj_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 62+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(63) : cplan.sql(20235) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 74+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(75) : cplan.sql(20247) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ASSIGN_DESCR;
ALTER FUNCTION SP_U_ASSIGN_DESCR COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_TEST_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_TEST_ASSIGN
(
     rank001 VARCHAR2
   , bitflag001 PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_full_name VARCHAR2(255);
   h_project_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_participant PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_RPA0 IS
         SELECT ELEMENT_ID FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND RANK LIKE h_rank || '%';

      CURSOR C_RPA1 IS
         SELECT RES_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, RESOURCE_ID, RESOURCE_TYPE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id;

      CURSOR C_FPA1 IS
         SELECT ra.RESOURCE_ID, ra.RESOURCE_TYPE, ra.PARTICIPANT, ra.SECURITY_ID
         FROM RESOURCE_ASSIGNMENTS ra
         WHERE ELEMENT_ID = h_project_id AND NOT EXISTS (
         SELECT 'X' FROM PROJECT_ASSIGNMENTS pa WHERE pa.PROJECT_ID = h_project_id AND pa.RESOURCE_ID = ra.RESOURCE_ID);

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_TEST_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_TEST_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_TEST_ASSIGN(' || 
       '''' || rank001 || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_TEST_ASSIGN', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_TEST_ASSIGN(42) : cplan.sql(20305) : ' || SYSTIMESTAMP);
   END IF; 

   h_rank := rank001;
   h_bitflag := bitflag001;
   
   OPEN C_RPA0;
   
   LOOP
      FETCH C_RPA0 INTO h_project_id;
      IF C_RPA0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      --trace_printf("1__ SC_STATE = %d, h_project_id = %s\n", SC_STATE, h_project_id);
      
      If Coalesce(h_bitflag, 0) = 0 Then
         OPEN C_RPA1;
         
         LOOP
            FETCH C_RPA1 INTO h_res_assignment_id, h_tsk_assignment_id, h_res_id, h_res_type;
            IF C_RPA1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            BEGIN
               SELECT COUNT('X') INTO h_bitflag FROM DUAL WHERE EXISTS (SELECT 'X' FROM TASK_ASSIGNMENTS
               WHERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_id AND RESOURCE_ID = h_res_id);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 84+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(85) : cplan.sql(20348) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_bitflag, 0) <> 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET PROJECT_ASSIGNMENT_ID = NuLL
                  WHERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 103+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(104) : cplan.sql(20367) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET PARENT_ID = TSK_ASSIGNMENT_ID
                  WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 121+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(122) : cplan.sql(20385) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS
               WHERE RES_ASSIGNMENT_ID = h_res_assignment_id
               AND SECURITY_ID = 'SCPROJECTASSIGNMENT';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 141+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(142) : cplan.sql(20405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               BEGIN
                  DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 159+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(160) : cplan.sql(20423) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT NICK_NAME || ';' INTO h_full_name
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 175+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(176) : cplan.sql(20439) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PROFILE_NAME || ';' INTO h_full_name
                     FROM PROFILES
                     WHERE PROFILE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 190+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(191) : cplan.sql(20454) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_full_name, '')
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 209+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(210) : cplan.sql(20473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_ANDNOT(PARTICIPANT,4096)
                  WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 227+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(228) : cplan.sql(20491) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 245+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(246) : cplan.sql(20509) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 262+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(263) : cplan.sql(20526) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         BEGIN
            CLOSE C_RPA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 276+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(277) : cplan.sql(20540) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         COMMIT;
      ElsIf Coalesce(h_bitflag, 0) = 1 Then
         OPEN C_FPA1;
         
         LOOP
            FETCH C_FPA1 INTO h_res_id, h_res_type, h_participant, h_security_id;
            IF C_FPA1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '0' Then
               BEGIN
                  SELECT NICK_NAME || ';' INTO h_full_name
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 307+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(308) : cplan.sql(20571) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PROFILE_NAME || ';' INTO h_full_name
                  FROM PROFILES
                  WHERE PROFILE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 322+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(323) : cplan.sql(20586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES = CASE
               WHEN InStr(ASSIGN_NAMES, h_full_name) > 0 THEN ASSIGN_NAMES
               ELSE ASSIGN_NAMES || h_full_name END
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 343+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(344) : cplan.sql(20607) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               INSERT INTO PROJECT_ASSIGNMENTS (PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, RESOURCE_TYPE, ELEMENT_TYPE_BITS,ROLLUP_BITS)
               VALUES(h_project_id, h_res_id, h_participant, h_security_id, h_res_type, 1, PMO_CNST.SC_ASSIGNMENT_BIT);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 361+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(362) : cplan.sql(20625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("INSERT_* SC_STATE: %d, h_participant: %d, h_security_id: %s\n",SC_STATE,h_participant,h_security_id);
         END LOOP;
         
         BEGIN
            CLOSE C_FPA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 376+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(377) : cplan.sql(20640) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         COMMIT;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_RPA0 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 393+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(394) : cplan.sql(20657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_TEST_ASSIGN', 399+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_TEST_ASSIGN(400) : cplan.sql(20663) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_rank || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_rank, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 406+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(407) : cplan.sql(20670) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 418+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(419) : cplan.sql(20682) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_TEST_ASSIGN;
ALTER FUNCTION SP_TEST_ASSIGN COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_MIGRATION ==
CREATE OR REPLACE FUNCTION FC_MIGRATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_level PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C01 IS
         SELECT PROJECT_ID, LEVEL_ID FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY RANK DESC;

      CURSOR C02 IS
         SELECT PROJECT_ID, LEVEL_ID FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY RANK DESC;

      CURSOR C03 IS
         SELECT LEVEL_ID FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_project_id
         ORDER BY RANK DESC;

      CURSOR C04 IS
         SELECT /*PROJECT_ID,*/ LEVEL_ID
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_project_id
         ORDER BY RANK DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_MIGRATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_MIGRATION'), '-- ' || SYSTIMESTAMP, ':r := FC_MIGRATION(' || 
       '''' || h_project_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MIGRATION', 37+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_MIGRATION(38) : cplan.sql(20736) : ' || SYSTIMESTAMP);
   END IF; 

   If (h_project_id IS NULL) Then
      -- Step 1: fix plane and schedule only
      
      OPEN C01;
      
      LOOP
         FETCH C01 INTO h_project_id, h_level;
         IF C01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_rollup_bits := 3269638;
         -- all bits except: assign, sched and planned
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = CASE WHEN ACTUAL_START IS NuLL THEN PMO_CNST.SC_SET_ALL_BITS ELSE h_rollup_bits END
            WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 72+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(73) : cplan.sql(20771) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_rollup_bits := PMO_FUNC.BIT_OR(PMO_CNST.SC_PLANNED_BIT, PMO_CNST.SC_SCHED_BIT);
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL,
            ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND (PLANNED_EFFORT + SCHED_EFFORT > 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 94+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(95) : cplan.sql(20793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = h_rollup_bits, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(113) : cplan.sql(20811) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND TYPE_ID <> 14;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 130+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(131) : cplan.sql(20829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C01 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 150+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(151) : cplan.sql(20849) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Step 2: fix all for duration based
      
      OPEN C02;
      
      LOOP
         FETCH C02 INTO h_project_id, h_level;
         IF C02%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 185+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(186) : cplan.sql(20884) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C02 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 205+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(206) : cplan.sql(20904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Step 1: fix plane and schedule only
      
      OPEN C03;
      
      LOOP
         FETCH C03 INTO h_level;
         IF C03%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_rollup_bits := PMO_FUNC.BIT_OR(PMO_CNST.SC_PLANNED_BIT, PMO_CNST.SC_SCHED_BIT);
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 240+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(241) : cplan.sql(20939) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL,
            ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND (PLANNED_EFFORT + SCHED_EFFORT > 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 260+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(261) : cplan.sql(20959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = h_rollup_bits, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 278+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(279) : cplan.sql(20977) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND TYPE_ID <> 14;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 296+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(297) : cplan.sql(20995) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C03 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 316+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(317) : cplan.sql(21015) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Step 2: fix all for duration based
      
      OPEN C04;
      
      LOOP
         FETCH C04 INTO h_level;
         IF C04%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 351+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(352) : cplan.sql(21050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C04 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 371+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(372) : cplan.sql(21070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MIGRATION', 378+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MIGRATION(379) : cplan.sql(21077) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_MIGRATION;
ALTER FUNCTION FC_MIGRATION COMPILE DEBUG;

