/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * Author: Tudor Toma
 * Revision: 42 $
 * Date: 7/31/00 6:18p
 *
 **/

/*****************************************************************************************/

-- Get Finish Date OR Duration

/*****************************************************************************************/

Prompt == FUNCTION FC_ASSN_DURATION ==
CREATE OR REPLACE FUNCTION FC_ASSN_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_duration PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_calendar_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_min_per_day PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_id VARCHAR2(32);
   tempDuration PLS_INTEGER;
-- Declare Cursors
      CURSOR CRES1 IS
         SELECT DAY_YEAR, AVAILABLE_MIN, BIT_SCHED FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)) ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ASSN_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ASSN_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_ASSN_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ASSN_DURATION(46) : cplan_fd.sql(65) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);//trace_flush();
   If Coalesce(Length(h_start_DT), 0) = 0 Then
      --trace_printf("*** Warning: Start date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(53) : cplan_fd.sql(72) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_duration, 0) < 0 Then
      --trace_printf("*** Warning: Duration is negative: %d\n",h_duration);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(61) : cplan_fd.sql(80) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) = 0 Then
      --trace_printf("*** Warning: Finish date is null\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 68+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(69) : cplan_fd.sql(88) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_start_DT), 0) > 9) AND (Coalesce(Length(h_finish_DT), 0) > 9) 
         AND (PMO_ORASTD.strcmp(h_finish_DT, h_start_DT) < 0) Then
      --trace_printf("*** Warning: Finish date is earlier than Start date\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(78) : cplan_fd.sql(97) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) > 10 Then
      h_duration := 0;
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT) INTO h_start_time,h_finish_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(93) : cplan_fd.sql(112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 105+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(106) : cplan_fd.sql(125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_DT := '';
      h_finish_time := '';
   Else
      h_start_time := '08.00.00';
      
      h_finish_DT := '';
      h_finish_time := '';
   End If;
   
   --trace_printf("h_start_time = %s,h_finish_time = %s\n", h_start_time,h_finish_time);//trace_flush();
   
   h_start_D := '';
   h_finish_D := '';
   h_start_date := '';
   h_finish_date := '';
   h_SF := 0;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY,480), MIN_DATE - 1     , MAX_DATE + 1     , START_DATE, FINISH_DATE, DURATION
      INTO h_min_per_day, h_start_D, h_finish_D, h_start_date, h_finish_date, h_minute
      FROM PMO_RES_MIN_MAX WHERE RESOURCE_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 137+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(138) : cplan_fd.sql(157) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_start_D = %s, h_finish_D = %s\n",SC_STATE,h_start_D,h_finish_D);
   --trace_printf("h_start_date = %s, h_finish_date = %s, h_minute = %d\n",h_start_date,h_finish_date,h_minute);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         SELECT COALESCE(MINUTES_PER_DAY,480) INTO h_min_per_day
         FROM TMT_CALENDARS
         WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 155+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(156) : cplan_fd.sql(175) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(170) : cplan_fd.sql(189) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_RES_MIN_MAX
         VALUES (h_project_id, h_start_D, h_finish_D, h_min_per_day, TO_TIMESTAMP('1899-12-30 8:00:00'),TO_TIMESTAMP('1899-12-30 8:00:00'), 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 187+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(188) : cplan_fd.sql(207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT TO_DATE(SubStr(h_start_D, 1, 10)) - 1     , TO_DATE(SubStr(h_start_D, 1, 10)) + 1      INTO h_start_D, h_finish_D FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(201) : cplan_fd.sql(220) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_calendar_id := h_project_id;
   If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
      --strncpy(h_dayofyear, h_start_DT, 10);
      --h_dayofyear[10] = '\0';
      
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      h_temp_var := PMO_CNST.SC_BIT_30;
      
      h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_start_D, h_duration
                                , PMO_CNST.SC_PROJECT);
      
      h_start_D := h_dayofyear;
      --strcpy(h_start_D, h_dayofyear);
      
      BEGIN
         UPDATE PMO_RES_MIN_MAX SET MIN_DATE = h_start_D WHERE RESOURCE_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 232+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(233) : cplan_fd.sql(252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1__MIN_DATE *____*: SC_STATE: %d, h_start_D: %s\n", SC_STATE,h_start_D);
   ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) >= 0 Then
      --trace_printf("h_finish_D = %s,h_dayofyear = %s, h_duration = %d\n",h_finish_D,h_dayofyear,h_duration);
      If Coalesce(h_duration, 0) = 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_temp_var
                                , PMO_CNST.SC_PROJECT);
         
         h_finish_D := h_dayofyear;
         --strcpy(h_finish_D, h_dayofyear);
         
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 263+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(264) : cplan_fd.sql(283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      Else
         --strcpy(h_dayofyear, "");
         h_dayofyear := '';
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_duration
                                , PMO_CNST.SC_PROJECT);
         
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_finish_D FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 282+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(283) : cplan_fd.sql(302) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 299+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(300) : cplan_fd.sql(319) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
   End If;
   --trace_printf("h_start_D: %s, h_finish_D = %s, h_duration = %d\n",h_start_D,h_finish_D,h_duration);
   --trace_printf("h_start_DT = %s, h_finish_DT = %s\n",h_start_DT,h_finish_DT);
   
   If Coalesce(h_duration, 0) = 0 Then
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (PMO_ORASTD.strcmp(h_finish_date
            , h_finish_DT) = 0) Then
         h_duration := Coalesce(h_minute, 0);
         --trace_printf("RETURN_1: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 315+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(316) : cplan_fd.sql(335) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      ElsIf PMO_ORASTD.strcmp(h_start_DT, h_finish_DT) = 0 Then
         h_duration := 0;
         --trace_printf("RETURN_2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 322+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(323) : cplan_fd.sql(342) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      BEGIN
         SELECT DAY_YEAR INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 336+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(337) : cplan_fd.sql(356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* SC_STATE: %d, h_SF: %d\n", SC_STATE,h_SF);
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND (PMO_ORASTD.strncmp(h_finish_DT, h_start_D
            , 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) <= 0) Then
         -- Finish date is non-working day
         If PMO_ORASTD.strncmp(h_start_DT, h_finish_DT, 10) = 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMPDIFF(4, PMO_ORASTD.DB2TIMESTAMP(h_finish_DT), PMO_ORASTD.DB2TIMESTAMP(h_start_DT))
               INTO h_duration
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 356+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(357) : cplan_fd.sql(376) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_3.1: h_min_per_day = %d, h_duration = %d\n",h_min_per_day,h_duration);
            
            If Coalesce(h_duration, 0) > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            End If;
            --trace_printf("RETURN_3.2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 367+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(368) : cplan_fd.sql(387) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         Else
            h_finish_time := '17.00.00';
            h_minute := Coalesce(h_min_per_day, 0);
         End If;
         --trace_printf("Finish date is non-working day: h_finish_time: %s, h_minute = %d\n",h_finish_time,h_minute);
      Else
         h_minute := 0;
      End If;
   Else
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (Coalesce(h_duration, 0) = Coalesce(
            h_minute, 0)) Then
         h_finish_DT := h_finish_date;
         --trace_printf("RETURN_4: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 384+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(385) : cplan_fd.sql(404) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      h_minute := 0;
   End If;
   
   BEGIN
      SELECT DAY_YEAR INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
      WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 401+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(402) : cplan_fd.sql(421) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1__* SC_STATE: %d, h_dayofyear: %s\n", SC_STATE,h_dayofyear);
   
   --trace_printf("n.1__* h_start_DT: %s, h_start_D: %s, h_finish_D: %s\n", h_start_DT,h_start_D,h_finish_D);
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND (PMO_ORASTD.strncmp(h_start_DT, h_start_D
         , 10) >= 0) AND (PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) <= 0) Then
      h_start_time := '08.00.00';
      If Coalesce(h_duration, 0) > 0 Then
         If Coalesce(h_duration, 0) = Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME('17.00.00'))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 423+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(424) : cplan_fd.sql(443) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_5: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 429+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(430) : cplan_fd.sql(449) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         ElsIf Coalesce(h_duration, 0) <= Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT)) + (h_duration / 1440.0)
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 443+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(444) : cplan_fd.sql(463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_6: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 449+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(450) : cplan_fd.sql(469) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         End If;
      End If;
      
      h_minute := Coalesce(h_minute, 0) + Coalesce(h_min_per_day, 0);
      --trace_printf("Start date is non-working day: h_start_time: %s, h_minute = %d\n",h_start_time,h_minute);
   End If;
   
   --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_minute = %d\n",h_start_DT,h_finish_DT,h_minute);
   --trace_printf("h_start_D = %s, h_finish_D = %s, h_calendar_id = %s\n",h_start_D,h_finish_D, h_calendar_id);
   If (Coalesce(Length(h_start_D), 0) = 0) AND (Coalesce(Length(h_finish_D), 0) = 0) Then
      --trace_printf("*** Warning: h_start_D and h_finish_D are empty\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 464+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(465) : cplan_fd.sql(484) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   --trace_printf("1_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
   If Coalesce(h_duration, 0) = 0 Then
      --trace_printf("h_finish_DT = %s,h_finish_D = %s\n", h_finish_DT,h_finish_D);
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) > 0 Then
         --strncpy(h_dayofyear, h_finish_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_finish_D, h_dayofyear, h_temp_var
                                , PMO_CNST.SC_PROJECT);
         
         h_finish_D := h_dayofyear;
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 493+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(494) : cplan_fd.sql(513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4__MAX_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      ElsIf PMO_ORASTD.strncmp(h_finish_DT, h_start_D, 10) < 0 Then
         --strncpy(h_dayofyear, h_start_DT, 10);
         --h_dayofyear[10] = '\0';
         h_dayofyear := SubStr(h_start_DT, 1, 10);
         h_temp_var := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_start_D, h_temp_var
                                , PMO_CNST.SC_PROJECT);
         
         h_start_D := h_dayofyear;
         BEGIN
            UPDATE PMO_RES_MIN_MAX SET MIN_DATE = h_start_D WHERE RESOURCE_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 519+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(520) : cplan_fd.sql(539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("5__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
      End If;
      
      --trace_printf("h_start_D = %s, h_finish_D = %s\n",h_start_D,h_finish_D);
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT)
         INTO h_start_time, h_finish_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 537+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(538) : cplan_fd.sql(557) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT)
         INTO h_julian_start, h_julian_finish
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 552+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(553) : cplan_fd.sql(572) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_start_time = %s, h_finish_time = %s, h_julian_start = %d, h_julian_finish = %d\n",h_start_time,h_finish_time,h_julian_start,h_julian_finish);
      If Coalesce(h_julian_start, 0) > Coalesce(h_julian_finish, 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 560+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(561) : cplan_fd.sql(580) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      If Coalesce(h_julian_start, 0) = Coalesce(h_julian_finish, 0) Then
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 575+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(576) : cplan_fd.sql(595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_duration: %d\n", SC_STATE,h_duration);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --              h_duration = getNumberOfMinutesBetweenStartingAndFinishing( h_bitID.h_value, h_start_time, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID, h_start_time, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 591+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(592) : cplan_fd.sql(611) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      Else
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 605+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(606) : cplan_fd.sql(625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --              h_duration = getNumberOfMinutesStartingAt( h_bitID.h_value, h_start_time );
            tempDuration := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
            --trace_printf("h_start_time = %s, h_start_DT = %s, h_duration = %d\n",h_start_time,h_start_DT,h_duration);
         End If;
         
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 630+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(631) : cplan_fd.sql(650) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            --              h_duration += getNumberOfMinutesFinishingAt( h_bitID.h_value, h_finish_time );
            tempDuration := PMO_STPU.getNumberOfMinutesFinishingAt(h_bitID, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_duration, 0) + Coalesce(h_min_per_day, 0);
            Else
               h_duration := Coalesce(h_duration, 0) + tempDuration;
            End If;
            
            --trace_printf("h_finish_time = %s, h_finish_DT = %s, h_duration = %d\n",h_finish_time,h_finish_DT,h_duration);
         End If;
         
         BEGIN
            SELECT h_duration + COALESCE(SUM(AVAILABLE_MIN),0) INTO h_duration
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 657+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(658) : cplan_fd.sql(677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
         h_duration := Coalesce(h_duration, 0);
         --h_duration_Ind = 0;
      End If;
      
      h_duration := Coalesce(h_duration, 0) + Coalesce(h_minute, 0);
   Else
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      --trace_printf("2_** h_minute = %d, h_total = %d, h_duration = %d\n",h_minute,h_total,h_duration);
      h_total := Coalesce(h_minute, 0);
      
      BEGIN
         SELECT  PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 680+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(681) : cplan_fd.sql(700) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_dayofyear = %s, h_start_time = %s, h_total = %d, h_duration = %d\n",h_dayofyear,h_start_time,h_total, h_duration);
      
      WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
         h_available := 0;
         h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
         
         OPEN CRES1;
         
         WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
            h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
            
            FETCH CRES1 INTO h_dayofyear, h_available, h_bitID;
            IF CRES1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("h_dayofyear = %s, h_available = %d\n",h_dayofyear,h_available);
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If Coalesce(h_total, 0) = 0 Then
               h_total := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
               
               --trace_printf("*** h_minutes: %d, h_duration: %d, h_total = %d\n", h_minutes,h_duration,h_total);
               If Coalesce(h_duration, 0) <= Coalesce(h_total, 0) Then
                  h_minutes := Coalesce(h_duration, 0) + Coalesce(h_available, 0) - Coalesce(h_total, 0);
                  h_total := 0;
                  EXIT;
               End If;
               --trace_printf("*** h_total = %d\n", h_total);
            Else
               h_total := Coalesce(h_total, 0) + Coalesce(h_available, 0);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CRES1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 730+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(731) : cplan_fd.sql(750) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("h_dayofyear = %s, h_duration = %d, h_total = %d\n",h_dayofyear,h_duration,h_total);
         If Coalesce(h_total, 0) = 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) < Coalesce(h_duration, 0) Then
            BEGIN
               SELECT  TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1       INTO  h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 749+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(750) : cplan_fd.sql(769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_finish_D := '';
            h_available := Trunc((Coalesce(h_duration, 0) - Coalesce(h_total, 0)) / 480);
            If Coalesce(h_available, 0) < 135 Then
               h_temp_var := 135;
               h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_finish_D, h_temp_var
                                , PMO_CNST.SC_PROJECT);
            Else
               h_temp_sc_state := EXTEND_PRCAL(h_project_id, h_dayofyear, h_finish_D, h_available
                                , PMO_CNST.SC_PROJECT);
            End If;
            
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_finish_D FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 773+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(774) : cplan_fd.sql(793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PMO_RES_MIN_MAX SET MAX_DATE = h_finish_D WHERE RESOURCE_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 790+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(791) : cplan_fd.sql(810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("6__MIN_DATE *____*: SC_STATE: %d, h_finish_D: %s\n", SC_STATE,h_finish_D);
         End If;
      END LOOP;
      
      --trace_printf("END__ h_dayofyear = %s, h_duration = %d, h_total = %d, h_minutes: %d\n",h_dayofyear,h_duration,h_total,h_minutes);
      If Coalesce(h_minutes, 0) > 0 Then
         h_finish_time := PMO_STPU.GET_FINISH_TIME(h_bitID, Coalesce(h_minutes, 0));
         --trace_printf("h_minutes = %d, h_finish_time = %s\n",h_minutes,h_finish_time);
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time) ) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 812+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(813) : cplan_fd.sql(832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_minute = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d, %d\n",h_minute,h_start_DT,h_finish_DT,h_duration_Ind,h_duration);
   
   BEGIN
      UPDATE PMO_RES_MIN_MAX SET  START_DATE  = h_start_DT,
      FINISH_DATE = h_finish_DT,
      DURATION    = h_duration
      WHERE RESOURCE_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSN_DURATION', 836+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSN_DURATION(837) : cplan_fd.sql(856) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --timeLog (file,"RETURN FC_ASSN_DURATION:");
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSN_DURATION', 843+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ASSN_DURATION(844) : cplan_fd.sql(863) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_duration, 0);
   
END;
/
SHOW ERRORS FUNCTION FC_ASSN_DURATION;
ALTER FUNCTION FC_ASSN_DURATION COMPILE DEBUG;

/*****************************************************************************************/

-- Get Finish Date OR Duration

/*****************************************************************************************/

Prompt == FUNCTION GET_FD_DURATION ==
CREATE OR REPLACE FUNCTION GET_FD_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_calendar_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_min_per_day PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_id VARCHAR2(32);
   tempDuration PLS_INTEGER;
-- Declare Cursors
      CURSOR CRUP1 IS
         SELECT DAY_YEAR, AVAILABLE_MIN, BIT_SCHED FROM PMO_TMP_CDY
         WHERE DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10)) ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('GET_FD_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('GET_FD_DURATION'), '-- ' || SYSTIMESTAMP, ':r := GET_FD_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: GET_FD_DURATION(46) : cplan_fd.sql(924) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(Length(h_start_DT), 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 50+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(51) : cplan_fd.sql(929) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_duration, 0) < 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 57+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(58) : cplan_fd.sql(936) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_finish_DT), 0) = 0) AND (Coalesce(h_duration, 0) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 64+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(65) : cplan_fd.sql(943) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(Length(h_start_DT), 0) > 9) AND (Coalesce(Length(h_finish_DT), 0) > 9) 
         AND (PMO_ORASTD.strcmp(h_finish_DT, h_start_DT) < 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 72+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(73) : cplan_fd.sql(951) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(Length(h_start_DT), 0) = 10 Then
      h_start_DT := h_start_DT || ' 00:00:00.000000';
   End If;
   If Coalesce(Length(h_finish_DT), 0) = 10 Then
      h_finish_DT := h_finish_DT || ' 23:59:59.000000';
   End If;
   
   If Coalesce(Length(h_finish_DT), 0) > 10 Then
      h_duration := 0;
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT) INTO h_start_time,h_finish_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(95) : cplan_fd.sql(973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 107+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(108) : cplan_fd.sql(986) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_DT := '';
      h_finish_time := '';
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 116+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(117) : cplan_fd.sql(995) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_start_D := '';
   h_finish_D := '';
   h_start_date := '';
   h_finish_date := '';
   h_SF := 0;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY,480), MIN_DATE - 1     , MAX_DATE + 1     , START_DATE, FINISH_DATE, DURATION
      INTO h_min_per_day, h_start_D, h_finish_D, h_start_date, h_finish_date, h_minute
      FROM PMO_TMP_MIN_MAX;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 137+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(138) : cplan_fd.sql(1016) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 144+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(145) : cplan_fd.sql(1023) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_calendar_id := h_project_id;
   If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      h_temp_var := PMO_CNST.SC_BIT_30;
      
      h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_start_D
                                         , h_temp_var, 1);
      
      h_start_D := h_dayofyear;
      BEGIN
         UPDATE PMO_TMP_MIN_MAX SET MIN_DATE = h_start_D;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 170+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(171) : cplan_fd.sql(1049) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) >= 0 Then
      If Coalesce(h_duration, 0) = 0 Then
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_temp_var, 1);
         
         h_finish_D := h_dayofyear;
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(196) : cplan_fd.sql(1074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_dayofyear := '';
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_duration, 1);
         
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_finish_D FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 212+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(213) : cplan_fd.sql(1091) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 229+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(230) : cplan_fd.sql(1108) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_duration, 0) = 0 Then
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (PMO_ORASTD.strcmp(h_finish_date
            , h_finish_DT) = 0) Then
         h_duration := Coalesce(h_minute, 0);
         --trace_printf("RETURN_1: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 242+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(243) : cplan_fd.sql(1121) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      ElsIf PMO_ORASTD.strcmp(h_start_DT, h_finish_DT) = 0 Then
         h_duration := 0;
         --trace_printf("RETURN_2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 249+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(250) : cplan_fd.sql(1128) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      ElsIf Coalesce(Length(h_finish_DT), 0) < 10 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 254+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(255) : cplan_fd.sql(1133) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      BEGIN
         SELECT DAY_YEAR INTO h_dayofyear FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 267+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(268) : cplan_fd.sql(1146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND (PMO_ORASTD.strncmp(h_finish_DT, h_start_D
            , 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) <= 0) Then
         -- Finish date is non-working day
         If PMO_ORASTD.strncmp(h_start_DT, h_finish_DT, 10) = 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMPDIFF(4, PMO_ORASTD.DB2TIMESTAMP(h_finish_DT), PMO_ORASTD.DB2TIMESTAMP(h_start_DT))
               INTO h_duration
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 286+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(287) : cplan_fd.sql(1165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_duration, 0) > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            End If;
            --trace_printf("RETURN_3.2: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 296+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(297) : cplan_fd.sql(1175) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         Else
            h_finish_time := '17.00.00';
            h_minute := Coalesce(h_min_per_day, 0);
         End If;
      Else
         h_minute := 0;
      End If;
   Else
      If (PMO_ORASTD.strcmp(h_start_date, h_start_DT) = 0) AND (Coalesce(h_duration, 0) = Coalesce(
            h_minute, 0)) Then
         h_finish_DT := h_finish_date;
         --trace_printf("RETURN_4: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 312+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(313) : cplan_fd.sql(1191) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_duration, 0);
      End If;
      
      h_minute := 0;
   End If;
   
   BEGIN
      SELECT DAY_YEAR INTO h_dayofyear FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 328+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(329) : cplan_fd.sql(1207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND (PMO_ORASTD.strncmp(h_start_DT, h_start_D
         , 10) >= 0) AND (PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) <= 0) Then
      -- Start date is non-working day
      h_start_time := '08.00.00';
      If Coalesce(h_duration, 0) > 0 Then
         If Coalesce(h_duration, 0) = Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME('17.00.00'))
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(350) : cplan_fd.sql(1228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_5: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 355+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(356) : cplan_fd.sql(1234) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         ElsIf Coalesce(h_duration, 0) <= Coalesce(h_min_per_day, 0) Then
            BEGIN
               SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT)) + (h_duration / 1440.0)
               INTO h_finish_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 369+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(370) : cplan_fd.sql(1248) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RETURN_6: h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_start_DT,h_finish_DT,h_duration);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 375+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(376) : cplan_fd.sql(1254) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_duration, 0);
         End If;
      End If;
      
      h_minute := Coalesce(h_minute, 0) + Coalesce(h_min_per_day, 0);
   End If;
   
   If (Coalesce(Length(h_start_D), 0) = 0) AND (Coalesce(Length(h_finish_D), 0) = 0) Then
      --trace_printf("*** Warning: h_start_D and h_finish_D are empty\n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 387+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(388) : cplan_fd.sql(1266) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_duration, 0) = 0 Then
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) > 0 Then
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_var := 0;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_finish_D, h_dayofyear
                                         , h_temp_var, 1);
         
         h_finish_D := h_dayofyear;
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 412+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(413) : cplan_fd.sql(1291) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf PMO_ORASTD.strncmp(h_finish_DT, h_start_D, 10) < 0 Then
         h_dayofyear := SubStr(h_start_DT, 1, 10);
         h_temp_var := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_start_D
                                         , h_temp_var, 1);
         
         h_start_D := h_dayofyear;
         BEGIN
            UPDATE PMO_TMP_MIN_MAX SET MIN_DATE = h_start_D;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 435+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(436) : cplan_fd.sql(1314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         SELECT PMO_ORASTD.TIME(h_start_DT), PMO_ORASTD.TIME(h_finish_DT)
         INTO h_start_time, h_finish_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 451+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(452) : cplan_fd.sql(1330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT), PMO_ORASTD.JULIAN_DAY(h_finish_DT)
         INTO h_julian_start, h_julian_finish
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 466+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(467) : cplan_fd.sql(1345) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_julian_start, 0) > Coalesce(h_julian_finish, 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 473+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(474) : cplan_fd.sql(1352) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      If Coalesce(h_julian_start, 0) = Coalesce(h_julian_finish, 0) Then
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 487+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(488) : cplan_fd.sql(1366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            tempDuration := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID, h_start_time, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 501+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(502) : cplan_fd.sql(1380) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      Else
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 514+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(515) : cplan_fd.sql(1393) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            tempDuration := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_min_per_day, 0);
            Else
               h_duration := tempDuration;
            End If;
         End If;
         
         BEGIN
            SELECT BIT_SCHED INTO h_bitID FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 536+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(537) : cplan_fd.sql(1415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            tempDuration := PMO_STPU.getNumberOfMinutesFinishingAt(h_bitID, h_finish_time);
            If tempDuration > Coalesce(h_min_per_day, 0) Then
               h_duration := Coalesce(h_duration, 0) + Coalesce(h_min_per_day, 0);
            Else
               h_duration := Coalesce(h_duration, 0) + tempDuration;
            End If;
         End If;
         
         BEGIN
            SELECT h_duration + COALESCE(SUM(AVAILABLE_MIN),0) INTO h_duration
            FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 559+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(560) : cplan_fd.sql(1438) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_duration := Coalesce(h_duration, 0);
         --h_duration_Ind = 0;
      End If;
      
      h_duration := Coalesce(h_duration, 0) + Coalesce(h_minute, 0);
   Else
      h_dayofyear := SubStr(h_start_DT, 1, 10);
      h_total := Coalesce(h_minute, 0);
      
      BEGIN
         SELECT  PMO_ORASTD.TIME(h_start_DT) INTO h_start_time FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 580+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(581) : cplan_fd.sql(1459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
         h_available := 0;
         h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
         
         OPEN CRUP1;
         
         WHILE Coalesce(h_total, 0) < Coalesce(h_duration, 0) LOOP
            h_minutes := Coalesce(h_duration, 0) - Coalesce(h_total, 0);
            
            FETCH CRUP1 INTO h_dayofyear, h_available, h_bitID;
            IF CRUP1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If Coalesce(h_total, 0) = 0 Then
               h_total := PMO_STPU.getNumberOfMinutesStartingAt(h_bitID, h_start_time);
               
               If Coalesce(h_duration, 0) <= Coalesce(h_total, 0) Then
                  h_minutes := Coalesce(h_duration, 0) + Coalesce(h_available, 0) - Coalesce(h_total, 0);
                  h_total := 0;
                  EXIT;
               End If;
            Else
               h_total := Coalesce(h_total, 0) + Coalesce(h_available, 0);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CRUP1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 626+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(627) : cplan_fd.sql(1505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_total, 0) = 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) < Coalesce(h_duration, 0) Then
            BEGIN
               SELECT  TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1       INTO  h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 644+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(645) : cplan_fd.sql(1523) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_finish_D := '';
            h_available := Trunc((Coalesce(h_duration, 0) - Coalesce(h_total, 0)) / 480);
            If Coalesce(h_available, 0) < 135 Then
               h_temp_var := 135;
               h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_temp_var, 1);
            Else
               h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_available, 1);
            End If;
            
            BEGIN
               SELECT MAX(DAY_YEAR) INTO h_finish_D FROM PMO_TMP_CDY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 668+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(669) : cplan_fd.sql(1547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PMO_TMP_MIN_MAX SET MAX_DATE = h_finish_D;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 685+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(686) : cplan_fd.sql(1564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      If Coalesce(h_minutes, 0) > 0 Then
         h_finish_time := PMO_STPU.GET_FINISH_TIME(h_bitID, Coalesce(h_minutes, 0));
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time) ) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 703+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(704) : cplan_fd.sql(1582) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   BEGIN
      UPDATE PMO_TMP_MIN_MAX SET  START_DATE  = h_start_DT,
      FINISH_DATE = h_finish_DT,
      DURATION    = h_duration;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'GET_FD_DURATION', 724+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: GET_FD_DURATION(725) : cplan_fd.sql(1603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'GET_FD_DURATION', 730+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: GET_FD_DURATION(731) : cplan_fd.sql(1609) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_duration, 0);
   
END;
/
SHOW ERRORS FUNCTION GET_FD_DURATION;
ALTER FUNCTION GET_FD_DURATION COMPILE DEBUG;

-- ********************************************************************************

-- h_bitflag = 1,   h_task_start_DT has been changed

--           = 2,   h_task_finish_DT (h_ta_finish_DT) has been changed

--           = 3,   h_task_start_DT  h_task_finish_DT have been changed

--           = 4,   h_ta_start_DT has been changed (only at the assignment level)

-- ================================================================================

-- h_bitflag1 > 0,  new date > old date

--            < 0,  new date < old date

-- ********************************************************************************

Prompt == FUNCTION FC_FD_CTD_TDY ==
CREATE OR REPLACE FUNCTION FC_FD_CTD_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_SD IN OUT NOCOPY DATE
   , h_FD IN OUT NOCOPY DATE
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_work_type PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_percent_units PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_bitflag1 PLS_INTEGER
   , sh_h_leveling PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_percent_units PLS_INTEGER := sh_h_percent_units;
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitflag1 PLS_INTEGER := Coalesce(sh_h_bitflag1, 0);
   h_leveling PLS_INTEGER := Coalesce(sh_h_leveling, 0);
   h_parent_id VARCHAR2(32);
   h_dayofyear DATE;
   h_bit_flag VARCHAR2(1);
   h_null_str VARCHAR2(1);
   h_start_D DATE;
   h_finish_D DATE;
   h_minutes PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_cnt PLS_INTEGER := 0;
   h_ins_cnt PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_res_cost NUMBER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   stmt VARCHAR2(7999);
   i PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR C_D1 IS
         SELECT * FROM (SELECT SCHED_MIN, BIT_FLAG, PARENT_ID, BIT_SCHED, TABLE_TYPE
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10))
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_FDD1 IS
         SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR <= TO_DATE(SubStr(h_FD, 1, 10))
         ORDER BY DAY_YEAR DESC;

      -- Dynamic Cursor Declaration
      C_FDI1 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_FDI2 PMO_DB2M.REF_CURSOR;

      CURSOR C_FSTR IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FD_CTD_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FD_CTD_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_FD_CTD_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_SD || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_FD || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_percent_units), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_leveling), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 90+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FD_CTD_TDY(91) : cplan_fd.sql(1727) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_leveling: %d, h_bitflag: %d, h_bitflag1: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n", h_leveling,h_bitflag,h_bitflag1,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
   --trace_printf("h_tsk_assignment_id: %s, h_SD: %d %s, h_FD: %d %s\n", h_tsk_assignment_id,h_SD_Ind,h_SD,h_FD_Ind,h_FD);
   h_temp_var := 0;
   h_ins_cnt := 0;
   h_count := 0;
   h_cnt := 0;
   
   -- get first day
   /*  BEGIN_SQL
           SELECT SCHED_MIN, BIT_FLAG, PARENT_ID, BIT_SCHED
           INTO HOST(h_minutes), HOST(h_bit_flag), HOST(h_parent_id), HOST(h_bitID)
           FROM TASK_DAY_OF_YEAR
           WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND DAY_YEAR = DATE(HOST(h_SD));
       END_SQL_NOCHECK;
   
       if (SC_NODATA)
       {
           return -1;
       }
   */
   
   OPEN C_D1;
   
   FETCH C_D1 INTO h_minutes, h_bit_flag, h_parent_id, h_bitID, h_table_type;
   IF C_D1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         CLOSE C_D1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 132+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(133) : cplan_fd.sql(1769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 137+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(138) : cplan_fd.sql(1774) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   Else
      BEGIN
         CLOSE C_D1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 149+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(150) : cplan_fd.sql(1786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("SC_STATE: %d, h_minutes: %d,h_bit_flag: %s,h_parent_id: %s\n",SC_STATE,h_minutes,h_bit_flag,h_parent_id);//trace_flush();
   
   If h_bitflag = 2 Then
      If h_bitflag1 < 0 Then
         If h_leveling = 0 Then
            -- delete TDY between new task date and old ta date
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR > TO_DATE(SubStr(h_finish_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_FD, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 175+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(176) : cplan_fd.sql(1812) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("DELETE (h_bitflag1 < 0): SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
         Else
            -- insert LEVEL_TASK_DAY_OF_YEAR
            BEGIN
               INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 196+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(197) : cplan_fd.sql(1833) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE)
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 243+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(244) : cplan_fd.sql(1880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 287+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(288) : cplan_fd.sql(1924) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_project_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 331+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(332) : cplan_fd.sql(1968) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE)
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_project_id AND r.DAY_YEAR > TO_DATE(SubStr(h_FD, 1, 10)) AND r.DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 373+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(374) : cplan_fd.sql(2010) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   Else
      If h_bitflag1 > 0 Then
         BEGIN
            SELECT COUNT(DAY_YEAR) INTO h_ins_cnt FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 392+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(393) : cplan_fd.sql(2029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If h_leveling = 0 Then
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_SD, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_start_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 412+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(413) : cplan_fd.sql(2049) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 431+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(432) : cplan_fd.sql(2068) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT COUNT(r.DAY_YEAR) INTO h_ins_cnt FROM CALENDAR_DAY_OF_YEAR r JOIN CALENDAR_DAY_OF_YEAR p
               ON r.DAY_YEAR = p.DAY_YEAR
               WHERE p.ELEMENT_ID = h_project_id AND r.ELEMENT_ID = h_res_id
               AND p.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND p.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 449+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(450) : cplan_fd.sql(2086) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("C.1====> SC_STATE: %d, h_res_id: %s\n",SC_STATE,h_res_id);//trace_flush();
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND r.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 495+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(496) : cplan_fd.sql(2132) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE)
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  r.DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE r.ELEMENT_ID = h_res_id AND r.DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND r.DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10))
                  AND EXISTS (SELECT p.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR p
                  WHERE p.ELEMENT_ID = h_project_id AND p.DAY_YEAR = r.DAY_YEAR);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 539+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(540) : cplan_fd.sql(2176) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("C.2====> SC_STATE: %d, h_project_id: %s, h_res_id: %s\n",SC_STATE,h_project_id,h_res_id);//trace_flush();
         Else
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_ins_cnt FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 555+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(556) : cplan_fd.sql(2192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("C.2====> SC_STATE: %d, h_ins_cnt: %d\n",SC_STATE,h_ins_cnt);//trace_flush();
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 599+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(600) : cplan_fd.sql(2236) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  SELECT
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  DAY_YEAR,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type
                  FROM CALENDAR_DAY_OF_YEAR r
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR < TO_DATE(SubStr(h_SD, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 641+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(642) : cplan_fd.sql(2278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND (Coalesce(h_ins_cnt, 0) = 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 650+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(651) : cplan_fd.sql(2287) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
         
         OPEN C_FDD1;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDD1 INTO h_dayofyear;
            IF C_FDD1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND ACTUAL_FLAG = 0 AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 683+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(684) : cplan_fd.sql(2320) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("D.1====> SC_STATE: %d, h_dayofyear: %s, h_ins_cnt: %d\n",SC_STATE,h_dayofyear,h_ins_cnt);//trace_flush();
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            FETCH C_FDD1 INTO h_dayofyear;
            IF C_FDD1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("D.2====> SC_STATE: %d, h_dayofyear: %s, h_ins_cnt: %d\n",SC_STATE,h_dayofyear,h_ins_cnt);//trace_flush();
         End If;
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 711+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(712) : cplan_fd.sql(2348) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            CLOSE C_FDD1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 724+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(725) : cplan_fd.sql(2361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("1.0====> h_ins_cnt: %d, h_start_DT: %s, h_finish_DT: %s\n",h_ins_cnt,h_start_DT,h_finish_DT);//trace_flush();
   If Coalesce(h_ins_cnt, 0) <> 0 Then
      If PMO_ORASTD.strncmp(h_FD, h_start_DT, 10) < 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_start_DT, 1, 10)) - 1      INTO h_start_D
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 743+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(744) : cplan_fd.sql(2380) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_start_D := SubStr(h_FD, 1, 10);
      End If;
      --trace_printf("1.1====> h_start_D: %s\n",h_start_D);//trace_flush();
      
      If SubStr(h_res_type, 1, 1) = '0' Then
         stmt := 
               'SELECT r.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR r JOIN CALENDAR_DAY_OF_YEAR p ON p.DAY_YEAR = r.DAY_YEAR WHERE p.ELEMENT_ID = ''' 
               || h_project_id || ''' '
                || 'AND r.ELEMENT_ID = ''' || h_res_id || ''' AND p.DAY_YEAR > TO_DATE(SubStr(''' || h_start_D 
                     || ''', 1, 10))  AND  ROWNUM <= ' || TO_CHAR(Coalesce(h_ins_cnt, 0));
         --trace_printf("RESOURCE: h_ins_cnt: %d\n%s\n", h_ins_cnt,(char *) stmt);
         
         OPEN C_FDI1 FOR stmt;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDI1 INTO h_dayofyear;
            IF C_FDI1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("1.i====> h_dayofyear: %s\n",h_dayofyear);//trace_flush();
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 810+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(811) : cplan_fd.sql(2447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 850+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(851) : cplan_fd.sql(2487) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("1.1====> SC_STATE: %d, h_ins_cnt: %d, h_dayofyear: %s\n",SC_STATE,h_ins_cnt,h_dayofyear);//trace_flush();
            End If;
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDI1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 867+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(868) : cplan_fd.sql(2504) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         stmt := 'SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = ''' || h_project_id || ''' '
                || 'AND DAY_YEAR > TO_DATE(SubStr(''' || h_start_D || ''', 1, 10))  AND  ROWNUM <= ' 
                     || TO_CHAR(Coalesce(h_ins_cnt, 0));
         --trace_printf("PROFILE:\n%s\n", (char *) stmt);
         
         OPEN C_FDI2 FOR stmt;
         
         WHILE Coalesce(h_ins_cnt, 0) > 0 LOOP
            FETCH C_FDI2 INTO h_dayofyear;
            IF C_FDI2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If h_leveling = 0 Then
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 926+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(927) : cplan_fd.sql(2563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO LEVEL_TASK_DAY_OF_YEAR (
                  PROJECT_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  BIT_FLAG,
                  SCHED_MIN,
                  PARENT_ID,
                  BIT_SCHED,
                  ACTUAL_FLAG,
                  TASK_ID,
                  TABLE_TYPE )
                  VALUES (
                  h_project_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_bit_flag,
                  h_minutes,
                  h_parent_id,
                  h_bitID,
                  0,
                  h_task_id,
                  h_table_type );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 966+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(967) : cplan_fd.sql(2603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("1.2====> SC_STATE: %d, h_ins_cnt: %d, h_dayofyear: %s\n",SC_STATE,h_ins_cnt,h_dayofyear);//trace_flush();
                     h_ins_cnt := Coalesce(h_ins_cnt, 0) - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDI2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 983+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(984) : cplan_fd.sql(2620) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("1.3====> h_finish_DT: %s, h_dayofyear: %s\n",h_finish_DT,h_dayofyear);//trace_flush();
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 999+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1000) : cplan_fd.sql(2636) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("h_work_type: %d, h_minutes: %d, h_start_DT: %s, h_finish_DT: %s\n",h_work_type,h_minutes,h_start_DT,h_finish_DT);//trace_flush();
   
   If h_leveling = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(MIN(DAY_YEAR),PMO_ORASTD.TIME(h_start_DT)), PMO_ORASTD.DB2TIMESTAMP(MAX(DAY_YEAR),PMO_ORASTD.TIME(h_finish_DT))
         INTO h_start_DT, h_finish_DT
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1018+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1019) : cplan_fd.sql(2655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_work_type, 0) = 2 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            SCHED_START = h_start_DT,
            SCHED_FINISH = h_finish_DT
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1039+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1040) : cplan_fd.sql(2676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            PLANNED_START = h_start_DT,
            PLANNED_FINISH = h_finish_DT
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1059+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1060) : cplan_fd.sql(2696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      h_temp_var := FC_INI_ASSIGN_STR(1);
      h_assign_string := '';
      h_total := 0;
      
      OPEN C_FSTR;
      
      LOOP
         FETCH C_FSTR INTO h_julian_day, h_minutes;
         IF C_FSTR%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         h_temp_var := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      END LOOP;
      
      BEGIN
         CLOSE C_FSTR ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1094+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1095) : cplan_fd.sql(2731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_var := FC_GET_ASSIGN_STR(h_assign_string, 0);
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         LEVEL_START         = h_start_DT,
         LEVEL_FINISH        = h_finish_DT,
         LEVEL_EFFORT        = h_total,
         LEVEL_ASSIGN_STRING = h_assign_string
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_CTD_TDY', 1118+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_CTD_TDY(1119) : cplan_fd.sql(2755) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_total: %d, h_assign_string: %s\n", SC_STATE,h_total,h_assign_string);
      --trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n", h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);
   End If;
   --trace_printf("2.====> SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
   
   h_temp_var := PMO_FUNC.BIT_OR(Coalesce(h_percent_units, 0), PMO_CNST.SC_BIT_30);
   h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_CTD_TDY', 1131+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FD_CTD_TDY(1132) : cplan_fd.sql(2768) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FD_CTD_TDY;
ALTER FUNCTION FC_FD_CTD_TDY COMPILE DEBUG;

/***************************************************************************************************************************/

-- h_start_DT = task new start  date

-- h_start_DT = task new finish date

-- h_bitflag  = 1 Start Date has been changed

--            = 2 Finish Date has been changed

--            = 3 Start Date  Finish Date have been changed

/***************************************************************************************************************************/

Prompt == FUNCTION FC_CHANGE_TA_DATES ==
CREATE OR REPLACE FUNCTION FC_CHANGE_TA_DATES
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_task_start_DT IN OUT NOCOPY TIMESTAMP
   , h_task_finish_DT IN OUT NOCOPY TIMESTAMP
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_per_day PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_task_type PLS_INTEGER
   , sh_h_leveling PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_task_type PLS_INTEGER := Coalesce(sh_h_task_type, 0);
   h_leveling PLS_INTEGER := Coalesce(sh_h_leveling, 0);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_dayofyear DATE;
   h_SD DATE;
   h_FD DATE;
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_D DATE;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_bitflag1 PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_sale NUMBER := 0;
   h_reg_cost NUMBER := 0;
   stmt VARCHAR2(7999);
   i_bitflag PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER;
   i001 PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_FDD2 PMO_DB2M.REF_CURSOR;

      CURSOR C_CHGD2 IS
         SELECT TSK_ASSIGNMENT_ID, ETC, ASSIGNMENT_DELAY, RESOURCE_ID, RESOURCE_TYPE,
         UNITS_MSP, Trunc(BIT_FLAG), UNRESOLVED_START, LAST_SCHED_MIN, LAST_SUBMITTED,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)) BETWEEN h_start_D AND h_finish_D
         ORDER BY UNRESOLVED_START;

      CURSOR C_CHGD1 IS
         SELECT TSK_ASSIGNMENT_ID, ETC, ASSIGNMENT_DELAY, RESOURCE_ID, RESOURCE_TYPE,
         UNITS_MSP, BIT_FLAG, C_CONTOUR, PROJECT_ASSIGNMENT_ID, LAST_SCHED_MIN, LAST_SUBMITTED,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND ACTUAL_START IS NuLL
         ORDER BY UNRESOLVED_START;

      CURSOR C_P_DELAY IS
         SELECT * FROM (SELECT p.DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR p
         WHERE p.ELEMENT_ID = h_project_id
         AND p.DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10))
         ORDER BY DAY_YEAR
         ) WHERE ROWNUM<=300;

      CURSOR C_R_DELAY IS
         SELECT * FROM (SELECT p.DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR p
         JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
         WHERE p.ELEMENT_ID = h_project_id AND r.ELEMENT_ID = h_res_id
         AND r.DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10))
         ORDER BY DAY_YEAR
         ) WHERE ROWNUM<=300;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CHANGE_TA_DATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CHANGE_TA_DATES'), '-- ' || SYSTIMESTAMP, ':r := FC_CHANGE_TA_DATES(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_task_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_task_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_task_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_leveling), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 101+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CHANGE_TA_DATES(102) : cplan_fd.sql(2892) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_leveling: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n",h_leveling,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
   --trace_printf("h_task_start_DT: %d %s, h_task_finish_task_DT: %d %s\n",h_task_start_DT_Ind,h_task_start_DT,h_task_finish_DT_Ind,h_task_finish_DT);//trace_flush();
   
   If (h_task_type = PMO_CNST.SC_FD_EFFORT_DRIVEN) AND (h_leveling > 0) Then
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_SD FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND RESOURCE_ID IN (
         SELECT RESOURCE_ID FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 118+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(119) : cplan_fd.sql(2909) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_SD: %d %s, h_task_id: %s\n",SC_STATE,h_SD_Ind,h_SD,h_task_id);//trace_flush();
      
      If ((h_SD IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_SD, h_start_DT, 10) >= 0) Then
         If PMO_ORASTD.strncmp(h_SD, h_start_DT, 10) = 0 Then
            h_SD := SubStr(h_start_DT, 1, 10);
         End If;
         
         stmt := 'SELECT DAY_YEAR FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(''' || h_SD 
               || ''', 1, 10)) '
                || ' AND  ROWNUM <= ' || TO_CHAR(h_leveling);
         --trace_printf("DURATION: h_leveling: %d\n%s\n",h_leveling,(char *) stmt);//trace_flush();
         
         OPEN C_FDD2 FOR stmt;
         
         i := h_leveling;
         WHILE i > 0 LOOP
            FETCH C_FDD2 INTO h_FD;
            IF C_FDD2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If i = h_leveling Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_FD, 1, 10)), PMO_ORASTD.TIME(h_start_DT)) INTO h_start_DT FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 157+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(158) : cplan_fd.sql(2948) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
                     i := i - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_FDD2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 173+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(174) : cplan_fd.sql(2964) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_FD, 1, 10)), PMO_ORASTD.TIME(h_finish_DT)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 186+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(187) : cplan_fd.sql(2977) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
      End If;
   End If;
   
   If PMO_ORASTD.strncmp(h_start_DT, h_task_start_DT, 10) <> 0 Then
      i_bitflag := 1;
   End If;
   If PMO_ORASTD.strncmp(h_finish_DT, h_task_finish_DT, 10) <> 0 Then
      i_bitflag := i_bitflag + 2;
   End If;
   
   --trace_printf("i_bitflag: %d, h_task_id: %d %s\n",i_bitflag,h_task_id_Ind,h_task_id);//trace_flush();
   If i_bitflag = 0 Then
      -- insert LEVEL_TASK_DAY_OF_YEAR
      BEGIN
         INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND ACTUAL_FLAG = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 217+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(218) : cplan_fd.sql(3008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("==> INSERT INTO LEVEL_TASK_DAY_OF_YEAR SC_STATE: %d\n",SC_STATE);//trace_flush();
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 224+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(225) : cplan_fd.sql(3015) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If i_bitflag = 2 Then
      h_bitflag1 := PMO_ORASTD.strncmp(h_finish_DT, h_task_finish_DT, 10);
      -- new task date , old task date
      
      If Coalesce(h_bitflag1, 0) >= 0 Then
         h_start_D := SubStr(h_task_finish_DT, 1, 10);
         h_finish_D := SubStr(h_finish_DT, 1, 10);
      Else
         h_start_D := SubStr(h_finish_DT, 1, 10);
         h_finish_D := SubStr(h_task_finish_DT, 1, 10);
      End If;
      
      OPEN C_CHGD2;
      
      LOOP
         FETCH C_CHGD2 INTO h_tsk_assignment_id, h_remaining, h_delay, h_res_id, h_res_type,
         h_percent_units, h_work_type, h_ta_start_DT, h_last_day, h_last_submitted,
         h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
         IF C_CHGD2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("FD_** SC_STATE: %d, h_tsk_assignment_id: %s\n",SC_STATE,h_tsk_assignment_id);//trace_flush();
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         --trace_printf("h_tsk_assignment_id: %s, h_remaining: %d, h_delay: %d\n",h_tsk_assignment_id,h_remaining,h_delay);//trace_flush();
         --trace_printf("h_SD: %s, h_FD: %s\n",h_SD,h_FD);//trace_flush();
         
         If h_leveling = 0 Then
            h_tph_delete := -1;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
            
            If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 279+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(280) : cplan_fd.sql(3070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         h_ta_finish_DT := h_finish_DT;
         
         h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, h_tsk_assignment_id, h_res_id, h_res_type, h_SD
                            , h_FD, h_ta_start_DT, h_ta_finish_DT, h_work_type, h_per_day
                            , h_percent_units, h_rec_user, i_bitflag, Coalesce(h_bitflag1, 0)
                            , h_leveling);
         
         If h_leveling = 0 Then
            BEGIN
               SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), TO_DATE(SubStr(h_finish_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
               INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 305+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(306) : cplan_fd.sql(3096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_tph_delete := 0;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_CHGD2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 325+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(326) : cplan_fd.sql(3116) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      OPEN C_CHGD1;
      
      LOOP
         FETCH C_CHGD1 INTO h_tsk_assignment_id, h_remaining, h_delay, h_res_id, h_res_type,
         h_percent_units, h_bit_flag, h_contour, h_prj_assign_id, h_last_day, h_last_submitted,
         h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
         IF C_CHGD1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("SD_** SC_STATE: %d\n",SC_STATE);//trace_flush();
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         --trace_printf("h_tsk_assignment_id: %s, h_remaining: %d, h_delay: %d\n",h_tsk_assignment_id,h_remaining,h_delay);//trace_flush();
         --trace_printf("h_SD: %s, h_FD: %s\n",h_SD,h_FD);//trace_flush();
         --trace_printf("*** h_res_type: %s, h_res_id: %s, h_start_DT: %d %s\n",h_res_type,h_res_id,h_start_DT_Ind,h_start_DT);//trace_flush();
         --trace_printf("h_project_id: %s, h_start_DT: %s\n",h_project_id,h_start_DT);//trace_flush();
         
         If Coalesce(h_remaining, 0) > 0 Then
            h_ta_finish_DT := h_finish_DT;
            If Coalesce(h_delay, 0) = 0 Then
               h_ta_start_DT := h_start_DT;
            Else
               If SubStr(h_res_type, 1, 1) = '1' Then
                  OPEN C_P_DELAY;
               Else
                  OPEN C_R_DELAY;
               End If;
               i001 := Coalesce(h_delay, 0);
               WHILE i001 > 0 LOOP
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     FETCH C_P_DELAY INTO h_dayofyear;
                     IF C_P_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  Else
                     FETCH C_R_DELAY INTO h_dayofyear;
                     IF C_R_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  End If;
                  --trace_printf("*** SC_STATE: %d, h_dayofyear: %d %s\n",SC_STATE,h_dayofyear_Ind,h_dayofyear);//trace_flush();
                  
                  If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                     BEGIN
                        CLOSE C_CHGD1 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 388+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(389) : cplan_fd.sql(3179) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If SubStr(h_res_type, 1, 1) = '1' Then
                        BEGIN
                           CLOSE C_P_DELAY ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 402+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(403) : cplan_fd.sql(3193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           CLOSE C_R_DELAY ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 415+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(416) : cplan_fd.sql(3206) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 422+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(423) : cplan_fd.sql(3213) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN PMO_CNST.RECORD_NOT_FOUND;
                  End If;
                                 i001 := i001 - 1;
               END LOOP;
               
               If SubStr(h_res_type, 1, 1) = '1' Then
                  BEGIN
                     CLOSE C_P_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 438+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(439) : cplan_fd.sql(3229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     CLOSE C_R_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 451+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(452) : cplan_fd.sql(3242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT))
                  INTO h_ta_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 467+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(468) : cplan_fd.sql(3258) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_bitflag1 := PMO_ORASTD.strncmp(h_ta_start_DT, h_SD, 10);
            -- assignment: new date, old date
            --trace_printf("**** h_delay: %d, h_bitflag1: %d, h_ta_start_DT: %d, %s\n", h_delay, h_bitflag1, h_ta_start_DT_Ind, h_ta_start_DT);
            --trace_printf("**** h_ta_start_DT: %s, h_start_DT: %s\n", h_ta_start_DT, h_start_DT);
            If (Coalesce(h_bitflag1, 0) < 0) OR (Coalesce(h_delay, 0) = 0) OR (PMO_ORASTD.strncmp(h_ta_start_DT
                  , h_start_DT, 10) > 0) Then
               If h_leveling = 0 Then
                  h_tph_delete := -1;
                  h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
                  
                  If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 499+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(500) : cplan_fd.sql(3290) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               If (h_task_type = PMO_CNST.SC_FD_EFFORT_DRIVEN) OR (h_task_type = PMO_CNST.SC_FIXED_DURATION) Then
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_work_type := 2;
                  Else
                     h_work_type := 0;
                  End If;
                  
                  h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, h_tsk_assignment_id, h_res_id, h_res_type, h_SD
                            , h_FD, h_ta_start_DT, h_ta_finish_DT, h_work_type, h_per_day
                            , h_percent_units, h_rec_user, i_bitflag, Coalesce(h_bitflag1, 0)
                            , h_leveling);
                  
                  If h_leveling = 0 Then
                     BEGIN
                        SELECT TO_DATE(SubStr(h_start_DT, 1, 10)), TO_DATE(SubStr(h_finish_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                        INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                        FROM TASK_ASSIGNMENTS
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 530+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(531) : cplan_fd.sql(3321) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_tph_delete := 0;
                     h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                     , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                     , h_billable);
                  End If;
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
                  ElsIf (h_prj_assign_id IS NULL) Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
                  Else
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
                  End If;
                  
                  h_temp_var := 2;
                  h_temp_var := FC_RESCHEDULE(h_tsk_assignment_id, h_ta_start_DT, h_remaining, h_temp_var, h_contour
                            , h_rec_user, 0, Coalesce(h_percent_units, 0));
                  
                  If Coalesce(h_temp_var, 0) <> 0 Then
                     BEGIN
                        CLOSE C_CHGD1 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 562+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(563) : cplan_fd.sql(3353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 568+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(569) : cplan_fd.sql(3359) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN Coalesce(h_temp_var, 0);
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 586+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(587) : cplan_fd.sql(3377) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_CHGD1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHANGE_TA_DATES', 603+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHANGE_TA_DATES(604) : cplan_fd.sql(3394) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHANGE_TA_DATES', 610+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CHANGE_TA_DATES(611) : cplan_fd.sql(3401) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CHANGE_TA_DATES;
ALTER FUNCTION FC_CHANGE_TA_DATES COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_D_TA_TDY ==
CREATE OR REPLACE FUNCTION FC_D_TA_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_day_year IN OUT NOCOPY DATE
   , sh_i_decommission PLS_INTEGER
) RETURN PLS_INTEGER
AS
   i_decommission PLS_INTEGER := Coalesce(sh_i_decommission, 0);
   h_dayofyear DATE;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CDALL IS
         SELECT DAY_YEAR
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;

      CURSOR CDREM IS
         SELECT DAY_YEAR
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > h_day_year;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_D_TA_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_D_TA_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_D_TA_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_day_year || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_i_decommission), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_TA_TDY', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_D_TA_TDY(39) : cplan_fd.sql(3450) : ' || SYSTIMESTAMP);
   END IF; 

   If i_decommission = 0 Then
      If (h_prj_assignment_id IS NULL) Then
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_TA_TDY', 56+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_TA_TDY(57) : cplan_fd.sql(3468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_temp_var := 0;
         
         OPEN CDALL;
         
         LOOP
            FETCH CDALL INTO h_dayofyear;
            IF CDALL%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE CDALL ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_TA_TDY', 87+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_TA_TDY(88) : cplan_fd.sql(3499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         DELETE FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_TA_TDY', 106+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_TA_TDY(107) : cplan_fd.sql(3518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      If (h_prj_assignment_id IS NULL) Then
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND DAY_YEAR > h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_TA_TDY', 126+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_TA_TDY(127) : cplan_fd.sql(3538) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_temp_var := 0;
         
         OPEN CDREM;
         
         LOOP
            FETCH CDREM INTO h_dayofyear;
            IF CDREM%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE CDREM ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_TA_TDY', 157+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_TA_TDY(158) : cplan_fd.sql(3569) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_TA_TDY', 165+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_D_TA_TDY(166) : cplan_fd.sql(3577) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_D_TA_TDY;
ALTER FUNCTION FC_D_TA_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_FD_U_TDY ==
CREATE OR REPLACE FUNCTION FC_FD_U_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_ta_start_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_per_day PLS_INTEGER
   , sh_h_minutes PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_dayofyear DATE;
   h_first_day PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR C_FDT IS
         SELECT * FROM (SELECT COALESCE(AVAILABLE_MIN,480), COALESCE(BIT_SCHED,PMO_CNST.SC_BIT_SCHED8H)
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND AVAILABLE_MIN = h_per_day
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FD_U_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FD_U_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_FD_U_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_ta_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_U_TDY', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FD_U_TDY(34) : cplan_fd.sql(3621) : ' || SYSTIMESTAMP);
   END IF; 

   -- get first day datetime
   
   OPEN C_FDT;
   
   FETCH C_FDT INTO h_first_day, h_bitID;
   IF C_FDT%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   BEGIN
      CLOSE C_FDT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_U_TDY', 55+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_U_TDY(56) : cplan_fd.sql(3643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_first_day, 0) > Coalesce(h_minutes, 0) Then
      PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_minutes, 0), h_bitID);
   End If;
   
   BEGIN
      UPDATE TASK_DAY_OF_YEAR SET
      SCHED_MIN = h_minutes,
      BIT_SCHED = h_bitID
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FD_U_TDY', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FD_U_TDY(80) : cplan_fd.sql(3667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_minutes: %d\n",SC_STATE,h_minutes);//trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FD_U_TDY', 86+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FD_U_TDY(87) : cplan_fd.sql(3674) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FD_U_TDY;
ALTER FUNCTION FC_FD_U_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_GET_DELAY ==
CREATE OR REPLACE FUNCTION FC_GET_DELAY
(
     h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , p_delay IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_contour PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_DELAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_DELAY'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_DELAY(' || 
       '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DELAY', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_DELAY(23) : cplan_fd.sql(3707) : ' || SYSTIMESTAMP);
   END IF; 

   If SubStr(h_res_type, 1, 1) = '0' Then
      -- Extend resource calendar
      h_duration := 0;
      If EXTEND_PRCAL(h_res_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_RESOURCE) = 0 Then
         BEGIN
            SELECT COUNT(DAY_YEAR) INTO h_contour
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_res_id AND DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DELAY', 39+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DELAY(40) : cplan_fd.sql(3724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_contour
         FROM PMO_TMP_CDY
         WHERE DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND DAY_YEAR <= TO_DATE(SubStr(h_finish_DT, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DELAY', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DELAY(56) : cplan_fd.sql(3740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   p_delay := Coalesce(h_contour, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DELAY', 64+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_DELAY(65) : cplan_fd.sql(3749) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_DELAY;
ALTER FUNCTION FC_GET_DELAY COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_TASK_PERCENT ==
CREATE OR REPLACE FUNCTION FC_U_TASK_PERCENT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_new_actual PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_close_task PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_new_actual PLS_INTEGER := sh_h_new_actual;
   h_close_task PLS_INTEGER := Coalesce(sh_h_close_task, 0);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_FD DATE;
   h_SD DATE;
   h_bit_flag VARCHAR2(1);
   h_last_day PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   i_prev_min PLS_INTEGER NOT NULL := 0;
   i_r_rem PLS_INTEGER NOT NULL := 0;
   i_t_rem PLS_INTEGER NOT NULL := 0;
   ratio_rem NUMBER NOT NULL := 0.00;
   i_TWP9 PLS_INTEGER;
   l_count PLS_INTEGER;
-- Declare Cursors
      CURSOR C_TWP00 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
         B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2';

      CURSOR C_TWP01 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
         B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2' AND MILESTONE_PERCENT <> 10000;

      CURSOR C_TWP0 IS
         SELECT DAY_YEAR, SUM(SCHED_MIN), COUNT(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND BIT_FLAG = '2'
         GROUP BY DAY_YEAR;

      CURSOR C_TWP9 IS
         SELECT TSK_ASSIGNMENT_ID, SUM(SCHED_MIN)
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND BIT_FLAG = '2' AND DAY_YEAR >= h_start_D AND DAY_YEAR <= h_finish_D
         GROUP BY TSK_ASSIGNMENT_ID;

      CURSOR C_TWP9i IS
         SELECT tdy.TSK_ASSIGNMENT_ID, SUM(tdy.SCHED_MIN)
         FROM TASK_DAY_OF_YEAR tdy
         WHERE tdy.PROJECT_ID = h_project_id AND tdy.TASK_ID = h_task_id
         AND tdy.BIT_FLAG = '2' AND tdy.DAY_YEAR > h_finish_D AND tdy.ACTUAL_FLAG = 1 AND
         NOT EXISTS (SELECT t.TSK_ASSIGNMENT_ID from PMO_TMP_TWP9 t
         WHERE tdy.TSK_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID)
         GROUP BY TSK_ASSIGNMENT_ID;

      CURSOR C_TWP10 IS
         SELECT TSK_ASSIGNMENT_ID, SCHED_MIN
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id
         AND BIT_FLAG = '2' AND DAY_YEAR = h_finish_D;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TASK_PERCENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TASK_PERCENT'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TASK_PERCENT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_new_actual), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_close_task), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TASK_PERCENT', 101+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TASK_PERCENT(102) : cplan_fd.sql(3861) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_project_id: %s, h_task_id: %s\n", h_project_id,h_task_id);
   --trace_printf("h_work_todate: %d, h_new_actual = %d\n", h_work_todate,h_new_actual);
   
   If Coalesce(h_new_actual, 0) = 0 Then
      h_work_percent := 0;
      
      OPEN C_TWP00;
      
      LOOP
         FETCH C_TWP00 INTO h_tsk_assignment_id, h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
         h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale;
         IF C_TWP00%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("i00_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_TWP00;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 143+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(144) : cplan_fd.sql(3903) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_close_task = 1 Then
      h_work_percent := 10000;
      
      OPEN C_TWP01;
      
      LOOP
         FETCH C_TWP01 INTO h_tsk_assignment_id, h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
         h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale;
         IF C_TWP01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("i00_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_effort, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_TWP01;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 183+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(184) : cplan_fd.sql(3943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_total := 0;
      h_start_D := NULL;
      
      OPEN C_TWP0;
      
      LOOP
         FETCH C_TWP0 INTO h_dayofyear, h_minutes, h_count;
         IF C_TWP0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("h_dayofyear = %s, h_minutes = %d\n", h_dayofyear,h_minutes);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_total, 0) = 0 Then
            h_start_D := h_dayofyear;
         End If;
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         --trace_printf("*** i_prev_min: %d, h_total = %d, h_minutes = %d\n", i_prev_min,h_total,h_minutes);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               h_last_day := Coalesce(h_minutes, 0);
               EXIT;
            End If;
         Else
            h_last_day := Coalesce(h_minutes, 0);
            
            If Coalesce(h_new_actual, 0) > i_prev_min Then
               h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            Else
               h_minutes := Coalesce(h_new_actual, 0);
            End If;
            
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            --trace_printf("### h_minutes: %d, h_new_actual: %d, i_prev_min: %d\n", h_minutes,h_new_actual,i_prev_min);
            
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_TWP0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 248+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(249) : cplan_fd.sql(4008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_D := h_dayofyear;
      ratio_rem := Coalesce(h_minutes, 0) / Coalesce(h_last_day, 0);
      --trace_printf("h_count: %d, h_last_day: %d, h_minutes: %d, ratio_rem: %.4f, h_start_D = %s, h_finish_D = %s\n", h_count,h_last_day,h_minutes,ratio_rem,h_start_D,h_finish_D);
      --trace_printf("====================================================================================\n");
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TWP9';
      PMO_DB2M.SC_STATE := 0;
      
      i_TWP9 := 0;
      l_count := 0;
      h_new_actual := 0;
      If PMO_ORASTD.strcmp(h_start_D, h_finish_D) <> 0 Then
         OPEN C_TWP9;
         
         LOOP
            FETCH C_TWP9 INTO h_tsk_assignment_id,h_new_actual;
            IF C_TWP9%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("\n===> i1_* SC_STATE: %d, h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
            
            If PMO_DB2M.SC_STATE <> 0 Then
               --trace_printf("==========> i_TWP9: %d <===================\n", i_TWP9); //trace_flush();
               If i_TWP9 <> 0 Then
                  EXIT;
               End If;
               
               i_TWP9 := 1;
               
               OPEN C_TWP9i;
               
               LOOP
                  FETCH C_TWP9i INTO h_tsk_assignment_id,h_new_actual;
                  IF C_TWP9i%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  --trace_printf("i0_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
                  If PMO_DB2M.SC_STATE <> 0 Then
                     h_new_actual := 0;
                     EXIT;
                  End If;
                  
                  If Coalesce(h_new_actual, 0) <> 0 Then
                     h_new_actual := 0;
                  End If;
                  BEGIN
                     SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
                     TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
                     B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
                     INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
                     h_SD, h_FD, h_billable, h_capital_expense,
                     h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
                     h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
                     FROM TASK_ASSIGNMENTS
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 319+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(320) : cplan_fd.sql(4079) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_work_percent := 0;
                  -- new % Work
                  
                  h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
               END LOOP;
               
               BEGIN
                  CLOSE C_TWP9i ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 343+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(344) : cplan_fd.sql(4103) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_new_actual, 0) > 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_TWP9 VALUES (h_tsk_assignment_id);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 362+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(363) : cplan_fd.sql(4122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               l_count := l_count + 1;
               BEGIN
                  SELECT SCHED_MIN INTO h_total
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_finish_D;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 378+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(379) : cplan_fd.sql(4138) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("** SC_STATE: %d, h_finish_D: %s, h_total = %d, h_new_actual = %d\n", SC_STATE, h_finish_D,h_total,h_new_actual);
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  h_new_actual := Coalesce(h_new_actual, 0) - Coalesce(h_total, 0);
                  If l_count = Coalesce(h_count, 0) Then
                     i_r_rem := Coalesce(h_minutes, 0) - i_t_rem;
                  Else
                     i_r_rem := Trunc((Coalesce(h_total, 0) * ratio_rem) + 0.4);
                     i_t_rem := i_t_rem + i_r_rem;
                  End If;
                  
                  h_new_actual := Coalesce(h_new_actual, 0) + i_r_rem;
               End If;
               --trace_printf("1_* h_new_actual: %d, h_total: %d, ratio_rem: %.4f\n", h_new_actual,h_total,ratio_rem);
               --trace_printf("l_count: %d, i_t_rem: %d, i_r_rem: %d\n", l_count,i_t_rem,i_r_rem);
               
               BEGIN
                  SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
                  TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
                  REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
                  B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
                  INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
                  h_SD, h_FD, h_billable, h_capital_expense,
                  h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
                  h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 415+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(416) : cplan_fd.sql(4175) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
               -- new % Work
               --trace_printf("h_work_percent: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_work_todate,h_effort);
               --trace_printf("=====> h_SD: %d %s, h_FD: %d %s, h_new_actual: %d\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD,h_new_actual);
               
               h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
               
               h_new_actual := 0;
               i_prev_min := 0;
            End If;
         END LOOP;
         
         BEGIN
            CLOSE C_TWP9 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 445+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(446) : cplan_fd.sql(4205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         OPEN C_TWP10;
         
         LOOP
            FETCH C_TWP10 INTO h_tsk_assignment_id,h_new_actual;
            IF C_TWP10%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            --trace_printf("i2_* SC_STATE: %d,h_tsk_assignment_id = %s, h_new_actual = %d\n", SC_STATE, h_tsk_assignment_id,h_new_actual);
            If PMO_DB2M.SC_STATE <> 0 Then
               EXIT;
            End If;
            
            l_count := l_count + 1;
            If l_count = Coalesce(h_count, 0) Then
               i_r_rem := Coalesce(h_minutes, 0) - i_t_rem;
            Else
               i_r_rem := Trunc((Coalesce(h_new_actual, 0) * ratio_rem) + 0.4);
               i_t_rem := i_t_rem + i_r_rem;
            End If;
            
            h_new_actual := i_r_rem;
            --trace_printf("2_* h_new_actual: %d, h_total: %d, ratio_rem: %.4f\n", h_new_actual,h_total,ratio_rem);
            --trace_printf("l_count: %d, i_t_rem: %d, i_r_rem: %d\n", l_count,i_t_rem,i_r_rem);
            
            BEGIN
               SELECT RESOURCE_ID, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK, LAST_SCHED_MIN,
               TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE,
               B_REGULAR_COST, B_REGULAR_RATE, B_SPECIAL_COST, B_SPECIAL_RATE
               INTO h_res_id, h_bit_flag, h_last_submitted, h_work_todate, h_effort, h_last_day,
               h_SD, h_FD, h_billable, h_capital_expense,
               h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale,
               h_b_reg_cost, h_b_reg_sale, h_b_spec_cost, h_b_spec_sale
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 493+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(494) : cplan_fd.sql(4253) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
            -- new actual work
            --trace_printf("h_work_percent: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_work_todate,h_effort);
            --trace_printf("=====> h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
            
            h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                                  , h_last_submitted, h_bit_flag, h_effort, h_new_actual, h_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, 0);
         END LOOP;
         
         BEGIN
            CLOSE C_TWP10 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TASK_PERCENT', 519+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TASK_PERCENT(520) : cplan_fd.sql(4279) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TASK_PERCENT', 527+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TASK_PERCENT(528) : cplan_fd.sql(4287) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TASK_PERCENT;
ALTER FUNCTION FC_U_TASK_PERCENT COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_ASSIGN_PERCENT ==
CREATE OR REPLACE FUNCTION FC_U_ASSIGN_PERCENT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_last_submitted IN OUT NOCOPY DATE
   , h_bit_flag IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_new_actual PLS_INTEGER
   , sh_h_sched_work PLS_INTEGER
   , sh_h_work_todate PLS_INTEGER
   , sh_h_work_percent PLS_INTEGER
   , sh_h_actual_regular_cost NUMBER
   , sh_h_actual_special_cost NUMBER
   , sh_h_actual_regular_rate NUMBER
   , sh_h_actual_special_rate NUMBER
   , sh_h_baseline_regular_cost NUMBER
   , sh_h_baseline_special_cost NUMBER
   , sh_h_baseline_regular_rate NUMBER
   , sh_h_baseline_special_rate NUMBER
   , sh_h_capital_expense001 PLS_INTEGER
   , sh_h_billable001 PLS_INTEGER
   , sh_h_last_day PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_per_day PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_new_actual PLS_INTEGER := sh_h_new_actual;
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_work_percent PLS_INTEGER := sh_h_work_percent;
   h_actual_regular_cost NUMBER := sh_h_actual_regular_cost;
   h_actual_special_cost NUMBER := sh_h_actual_special_cost;
   h_actual_regular_rate NUMBER := sh_h_actual_regular_rate;
   h_actual_special_rate NUMBER := sh_h_actual_special_rate;
   h_baseline_regular_cost NUMBER := sh_h_baseline_regular_cost;
   h_baseline_special_cost NUMBER := sh_h_baseline_special_cost;
   h_baseline_regular_rate NUMBER := sh_h_baseline_regular_rate;
   h_baseline_special_rate NUMBER := sh_h_baseline_special_rate;
   h_capital_expense001 PLS_INTEGER := sh_h_capital_expense001;
   h_billable001 PLS_INTEGER := sh_h_billable001;
   h_last_day PLS_INTEGER := sh_h_last_day;
   h_per_day PLS_INTEGER := Coalesce(sh_h_per_day, 0);
   h_chargecode_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_dayofyear DATE;
   h_week_apv DATE;
   h_total PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_milestone_percent PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_week_of DATE;
   h_next_week_of DATE;
   h_start_D DATE;
   h_null_str VARCHAR2(1);
   h_exchange_rate NUMBER := 0.00;
   h_prj_exchange_rate NUMBER := 0.00;
   h_res_exchange_rate NUMBER := 0.00;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_call_type PLS_INTEGER := 0;
   h_last_week PLS_INTEGER := 0;
   h_prj_currency_id PLS_INTEGER := 0;
   h_res_currency_id PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_SF PLS_INTEGER := 0;
   h_activity_count PLS_INTEGER := 0;
   h_baseline1_work PLS_INTEGER := 0;
   h_baseline_effort_week PLS_INTEGER := 0;
   h_friday_regular PLS_INTEGER := 0;
   h_friday_special PLS_INTEGER := 0;
   h_monday_regular PLS_INTEGER := 0;
   h_monday_special PLS_INTEGER := 0;
   h_saturday_regular PLS_INTEGER := 0;
   h_saturday_special PLS_INTEGER := 0;
   h_sunday_regular PLS_INTEGER := 0;
   h_sunday_special PLS_INTEGER := 0;
   h_thursday_regular PLS_INTEGER := 0;
   h_thursday_special PLS_INTEGER := 0;
   h_tuesday_regular PLS_INTEGER := 0;
   h_tuesday_special PLS_INTEGER := 0;
   h_wednesday_regular PLS_INTEGER := 0;
   h_wednesday_special PLS_INTEGER := 0;
   h_weekly_regular_total PLS_INTEGER := 0;
   h_weekly_special_total PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_work_type PLS_INTEGER := 0;
   h_temp_contour_type PLS_INTEGER := 0;
   h_temp_occurrence PLS_INTEGER := 0;
   h_ta_ms PLS_INTEGER := 0;
   h_ta_etc PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_res_type VARCHAR2(1);
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_current_weekday PLS_INTEGER := 0;
   h_db_day_of_week PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   i_keep_etc PLS_INTEGER NOT NULL := 0;
   h_per_day_Ind PLS_INTEGER NOT NULL := 0;
   i_prev_min PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CFW_AWP0 IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY DAY_YEAR;

      CURSOR CFW_AWP IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR >= h_last_submitted
         ORDER BY DAY_YEAR;

      CURSOR CBW_AWP IS
         SELECT SCHED_MIN, DAY_YEAR, BIT_SCHED, PMO_ORASTD.DAYOFWEEK(DAY_YEAR)
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 1 AND DAY_YEAR <= h_last_submitted
         ORDER BY DAY_YEAR DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_ASSIGN_PERCENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_ASSIGN_PERCENT'), '-- ' || SYSTIMESTAMP, ':r := FC_U_ASSIGN_PERCENT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || '''' || h_bit_flag || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_new_actual), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_regular_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_special_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_regular_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_special_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_baseline_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_last_day), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 172+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_ASSIGN_PERCENT(173) : cplan_fd.sql(4470) : ' || SYSTIMESTAMP);
   END IF; 

   --  SC_LONG(h_sched_min);
   
   h_monday_regular := 0;
   h_tuesday_regular := 0;
   h_wednesday_regular := 0;
   h_thursday_regular := 0;
   h_friday_regular := 0;
   h_saturday_regular := 0;
   h_sunday_regular := 0;
   h_monday_special := 0;
   h_tuesday_special := 0;
   h_wednesday_special := 0;
   h_thursday_special := 0;
   h_friday_special := 0;
   h_saturday_special := 0;
   h_sunday_special := 0;
   
   h_null_str := NULL;
   h_week_of := NULL;
   h_next_week_of := NULL;
   h_dayofyear := NULL;
   h_week_apv := NULL;
   h_remaining := 0;
   h_weekly_regular_total := 0;
   h_weekly_special_total := 0;
   h_current_weekday := 0;
   
   --trace_printf("h_project_id: %s, h_task_id: %s\n", h_project_id,h_task_id);
   --trace_printf("h_tsk_assignment_id = %s, h_effort: %d\n", h_tsk_assignment_id,h_effort);
   --trace_printf("h_last_submitted: %d:%s, h_work_todate: %d, h_new_actual = %d\n", h_last_submitted_Ind,h_last_submitted,h_work_todate,h_new_actual);
   If Coalesce(h_work_todate, 0) = Coalesce(h_new_actual, 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 207+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(208) : cplan_fd.sql(4505) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   
   If Coalesce(h_work_percent, 0) > PMO_CNST.SC_BIT_30 Then
      i_keep_etc := 1;
      h_work_percent := (BITAND(Coalesce(h_work_percent, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove etc flag
   End If;
   
   -- Fetch currency exchnage info ************************************************************************************** 2
   --trace_printf("RATES+++> h_actual_regular_cost: %.4f, h_actual_regular_rate: %.4f\n", h_actual_regular_cost, h_actual_regular_rate);
   If (Coalesce(h_actual_regular_rate, 0) + Coalesce(h_actual_regular_cost, 0)) > 0.00 Then
      --project currency
      BEGIN
         SELECT CURRENCY_ID INTO h_prj_currency_id
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(232) : cplan_fd.sql(4529) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT R.CURRENCY_ID
         INTO h_res_currency_id
         FROM RESOURCES R
         WHERE R.RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 247+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(248) : cplan_fd.sql(4545) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --exchage rate
      If Coalesce(h_prj_currency_id, 0) <> Coalesce(h_res_currency_id, 0) Then
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME, 18, 3)
            INTO h_prj_exchange_rate
            FROM TMT_DATAFIELDS
            WHERE ELEMENT_ID = h_prj_currency_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 265+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(266) : cplan_fd.sql(4563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME, 18, 3)
            INTO h_res_exchange_rate
            FROM TMT_DATAFIELDS
            WHERE ELEMENT_ID = h_res_currency_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(282) : cplan_fd.sql(4579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (Coalesce(h_prj_exchange_rate, 0) > 0.00) AND (Coalesce(h_res_exchange_rate, 0) > 0.00) Then
         BEGIN
            SELECT PMO_ORASTD.TODECIMAL(h_prj_exchange_rate / h_res_exchange_rate, 18, 3 )
            INTO h_exchange_rate
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 298+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(299) : cplan_fd.sql(4596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_exchange_rate := 0.000;
      End If;
   End If;
   --trace_printf("RATES+++> h_exchange_rate: %.4f\n", h_exchange_rate);
   
   h_call_type := 2;
   h_last_week := 0;
   h_week := 0;
   
   BEGIN
      SELECT SETTING_VALUE INTO h_db_day_of_week
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'TIMESHEET_START_WEEKDAY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 322+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(323) : cplan_fd.sql(4620) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("+++> SC_STATE: %d, h_db_day_of_week: %d %d\n", SC_STATE,h_db_day_of_week_Ind,h_db_day_of_week);
   
   --h_billable = (h_billable | SC_BIT_30);
   h_ta_ms := Trunc((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00);
   h_ta_etc := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
   
   If Coalesce(h_work_todate, 0) = 0 Then
      --timeLog (file,"1_* FC_U_ASSIGN_PERCENT Old Actual = 0:");
      --trace_printf("***** h_work_todate == 0\n");
      If SubStr(h_bit_flag, 1, 1) = '0' Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            SCHED_START         = PLANNED_START,
            SCHED_FINISH        = PLANNED_FINISH,
            SCHED_WORK          = PLANNED_WORK,
            BIT_FLAG            = '2'
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 353+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(354) : cplan_fd.sql(4651) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_milestone_percent := 0;
      h_etc := h_effort;
      h_total := Coalesce(h_work_todate, 0);
      
      OPEN CFW_AWP0;
      
      LOOP
         FETCH CFW_AWP0 INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CFW_AWP0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("h_dayofyear = %s, h_minutes = %d, h_total: %d\n", h_dayofyear,h_minutes,h_total);
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         If Coalesce(h_week, 0) = 0 Then
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 396+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(397) : cplan_fd.sql(4694) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("h_week_of = %s, h_week = %d, h_temp_var: %d\n", h_week_of,h_week,h_temp_var);
            
            --this is for to keep the next week and if the week changes then call FC_U_Smmary_timesheets
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 411+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(412) : cplan_fd.sql(4709) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_next_week_of) >= 0 Then
            h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
            If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
               h_etc := 0;
               h_milestone_percent := 10000;
            Else
               h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
               h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
            End If;
            --trace_printf("===> h_week_of: %s, h_weekly_regular_total: %d, h_milestone_percent: %d, h_ta_ms: %d, h_etc: %d\n", h_week_of,h_weekly_regular_total,h_milestone_percent,h_ta_ms,h_etc);
            
            PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_milestone_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
            
            h_monday_regular := 0;
            h_tuesday_regular := 0;
            h_wednesday_regular := 0;
            h_thursday_regular := 0;
            h_friday_regular := 0;
            h_saturday_regular := 0;
            h_sunday_regular := 0;
            
            h_ta_ms := Coalesce(h_milestone_percent, 0);
            h_ta_etc := Coalesce(h_etc, 0);
            
            --the week changed, reinitialize the week of  next_week
            h_week_of := h_next_week_of;
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 469+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(470) : cplan_fd.sql(4767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("*** h_week_of: %s, h_next_week_of: %s\n", h_week_of,h_next_week_of);
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         --trace_printf("*** i_prev_min: %d, h_total = %d, h_minutes = %d\n", i_prev_min,h_total,h_minutes);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            --trace_printf("### h_total = %d, i_prev_min: %d, h_temp_var: %d, h_minutes = %d\n", h_total,i_prev_min,h_temp_var,h_minutes);
            
            EXIT;
         End If;
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CFW_AWP0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 533+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(534) : cplan_fd.sql(4831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_work_todate, 0) < Coalesce(h_new_actual, 0) Then
      --trace_printf("***** h_work_todate < h_new_actual *** h_last_day: %d\n",h_last_day);
      --timeLog (file,"2_* FC_U_ASSIGN_PERCENT Old Actual < New Actual");
      
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_week_apv FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
         AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_SUBMITTED,PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 551+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(552) : cplan_fd.sql(4849) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("+++> SC_STATE: %d, h_week_apv: %d %s, h_res_id: %d %s\n", SC_STATE,h_week_apv_Ind,h_week_apv,h_res_id_Ind,h_res_id);
      
      If (h_week_apv IS NOT NULL) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_week_apv, 1, 10)) + 7     
            INTO h_start_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 568+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(569) : cplan_fd.sql(4866) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (SELECT 'X' FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND DAY_YEAR < h_start_date
            AND (ACTUAL_FLAG = 0 OR (ACTUAL_MIN > 0 AND ACTUAL_MIN <> SCHED_MIN)));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 584+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(585) : cplan_fd.sql(4882) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);
         
         If Coalesce(h_temp_var, 0) > 0 Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, C_CONTOUR, UNITS_MSP
               INTO h_prj_assignment_id, h_contour, h_percent_units
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 602+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(603) : cplan_fd.sql(4900) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_prj_assignment_id: %d %s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
            
            If Coalesce(h_last_day, 0) > 0 Then
               null_return := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assignment_id
                                    , h_tsk_assignment_id, h_last_submitted, h_last_day, 0);
            End If;
            
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_var := 2;
            Else
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_temp_var := 0;
               Else
                  h_temp_var := 1;
               End If;
            End If;
            null_return := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, Coalesce(h_temp_var, 0));
            
            --trace_printf("WP_FC_DEL_NOACTUAL: h_start_date: %s, h_contour: %d, h_percent_units: %d\n", h_start_date,h_contour,h_percent_units);
            
            h_total := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
            h_temp_var := 2;
            h_temp_var := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_total, h_temp_var, h_contour
                            , h_rec_user, 0, Coalesce(h_percent_units, 0));
            --trace_printf("WP_FC_RESCHEDULE: h_temp_var: %d\n",h_temp_var); //trace_flush();
            
            If Coalesce(h_temp_var, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 634+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(635) : cplan_fd.sql(4932) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
         End If;
      End If;
      
      If Coalesce(h_last_day, 0) = 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_last_submitted, 1, 10)) + 1      INTO h_last_submitted FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 650+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(651) : cplan_fd.sql(4948) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_total := Coalesce(h_work_todate, 0);
      Else
         h_total := Coalesce(h_work_todate, 0) - Coalesce(h_last_day, 0);
      End If;
      
      OPEN CFW_AWP;
      
      LOOP
         FETCH CFW_AWP INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CFW_AWP%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         --trace_printf("*** h_dayofyear: %s, h_minutes: %d, h_week: %d, h_current_weekday: %d\n",h_dayofyear,h_minutes,h_week,h_current_weekday);
         
         If Coalesce(h_week, 0) = 0 Then
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 694+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(695) : cplan_fd.sql(4992) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 707+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(708) : cplan_fd.sql(5005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT
               SUNDAY_REGULAR,
               SUNDAY_SPECIAL,
               MONDAY_REGULAR,
               MONDAY_SPECIAL,
               TUESDAY_REGULAR,
               TUESDAY_SPECIAL,
               WEDNESDAY_REGULAR,
               WEDNESDAY_SPECIAL,
               THURSDAY_REGULAR,
               THURSDAY_SPECIAL,
               FRIDAY_REGULAR,
               FRIDAY_SPECIAL,
               SATURDAY_REGULAR,
               SATURDAY_SPECIAL
               INTO
               h_sunday_regular,
               h_sunday_special,
               h_monday_regular,
               h_monday_special,
               h_tuesday_regular,
               h_tuesday_special,
               h_wednesday_regular,
               h_wednesday_special,
               h_thursday_regular,
               h_thursday_special,
               h_friday_regular,
               h_friday_special,
               h_saturday_regular,
               h_saturday_special
               FROM SUMMARY_TIMESHEETS
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND WEEK_OF = TO_DATE(SubStr(h_week_of, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 752+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(753) : cplan_fd.sql(5050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1.*** SC_STATE: %d, h_week_of: %s,h_current_weekday: %d\n",SC_STATE,h_week_of,h_current_weekday);
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               /*BEGIN_SQL
                                       CLOSE CFW_AWP WITH RELEASE;
                                   END_SQL;
                                   return SC_STATE; */
               h_monday_regular := 0;
               h_tuesday_regular := 0;
               h_wednesday_regular := 0;
               h_thursday_regular := 0;
               h_friday_regular := 0;
               h_saturday_regular := 0;
               h_sunday_regular := 0;
            End If;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_next_week_of) >= 0 Then
            h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
            If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
               h_etc := 0;
               h_milestone_percent := 10000;
            Else
               h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
               h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
            End If;
            
            PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
            
            h_ta_ms := Coalesce(h_milestone_percent, 0);
            h_ta_etc := Coalesce(h_etc, 0);
            
            h_monday_regular := 0;
            h_tuesday_regular := 0;
            h_wednesday_regular := 0;
            h_thursday_regular := 0;
            h_friday_regular := 0;
            h_saturday_regular := 0;
            h_sunday_regular := 0;
            
            --the week changed, reinitialize the week_of  next_week
            h_week_of := h_next_week_of;
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 823+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(824) : cplan_fd.sql(5121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("1.1** h_dayofyear = %s, h_minutes = %d, h_total: %d\n", h_dayofyear,h_minutes,h_total);
         
         i_prev_min := Coalesce(h_total, 0);
         h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
         If Coalesce(h_total, 0) <= Coalesce(h_new_actual, 0) Then
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            /*if (h_new_actual <= h_minutes)
                            {
                                h_minutes = h_new_actual;
                            }
                            else
                            {
                                h_minutes = h_new_actual - i_prev_min;
                            }*/
            h_minutes := Coalesce(h_new_actual, 0) - i_prev_min;
            --trace_printf("### h_total = %d, i_prev_min: %d, h_temp_var: %d, h_minutes = %d\n", h_total,i_prev_min,h_temp_var,h_minutes);
            
            EXIT;
         End If;
         --trace_printf("1.2** h_dayofyear = %s, h_minutes = %d, h_total = %d\n", h_dayofyear,h_minutes,h_total);
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CFW_AWP ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 887+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(888) : cplan_fd.sql(5185) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --trace_printf("***** h_work_todate > h_new_actual *** h_last_submitted: %d %s\n",h_last_submitted_Ind,h_last_submitted);
      --timeLog (file,"3_* FC_U_ASSIGN_PERCENT Old Actual >= New Actual");
      
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_week_apv FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
         AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_SUBMITTED,PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 905+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(906) : cplan_fd.sql(5203) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("+++> SC_STATE: %d, h_week_apv: %d %s, h_res_id: %d %s\n", SC_STATE,h_week_apv_Ind,h_week_apv,h_res_id_Ind,h_res_id);
      
      h_total := Coalesce(h_work_todate, 0);
      
      OPEN CBW_AWP;
      
      LOOP
         FETCH CBW_AWP INTO h_minutes, h_dayofyear, h_bitID, h_current_weekday;
         IF CBW_AWP%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --h_sched_min = h_minutes;
         --trace_printf("h_dayofyear: %s, h_minutes: %d, h_week: %d, h_current_weekday: %d\n", h_dayofyear,h_minutes,h_week,h_current_weekday);
         
         If Coalesce(h_week, 0) = 0 Then
            If Coalesce(h_last_day, 0) > 0 Then
               h_minutes := Coalesce(h_last_day, 0);
            End If;
            
            h_week := Coalesce(h_db_day_of_week, 0) + 1;
            If Coalesce(h_current_weekday, 0) >= Coalesce(h_week, 0) Then
               h_temp_var := Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            Else
               h_temp_var := 7 + Coalesce(h_current_weekday, 0) - Coalesce(h_week, 0);
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - h_temp_var     
               INTO h_week_of
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 951+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(952) : cplan_fd.sql(5249) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_week_of, 1, 10)) - 7      INTO h_next_week_of FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 964+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(965) : cplan_fd.sql(5262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1_* SC_STATE: %d, h_week_of: %s, h_next_week_of: %s\n", SC_STATE,h_week_of,h_next_week_of);
            
            If PMO_ORASTD.strcmp(h_week_apv, h_week_of) >= 0 Then
               BEGIN
                  CLOSE CBW_AWP ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 979+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(980) : cplan_fd.sql(5277) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 984+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(985) : cplan_fd.sql(5282) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            Else
               BEGIN
                  SELECT
                  SUNDAY_REGULAR,
                  SUNDAY_SPECIAL,
                  MONDAY_REGULAR,
                  MONDAY_SPECIAL,
                  TUESDAY_REGULAR,
                  TUESDAY_SPECIAL,
                  WEDNESDAY_REGULAR,
                  WEDNESDAY_SPECIAL,
                  THURSDAY_REGULAR,
                  THURSDAY_SPECIAL,
                  FRIDAY_REGULAR,
                  FRIDAY_SPECIAL,
                  SATURDAY_REGULAR,
                  SATURDAY_SPECIAL
                  INTO
                  h_sunday_regular,
                  h_sunday_special,
                  h_monday_regular,
                  h_monday_special,
                  h_tuesday_regular,
                  h_tuesday_special,
                  h_wednesday_regular,
                  h_wednesday_special,
                  h_thursday_regular,
                  h_thursday_special,
                  h_friday_regular,
                  h_friday_special,
                  h_saturday_regular,
                  h_saturday_special
                  FROM SUMMARY_TIMESHEETS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND WEEK_OF = TO_DATE(SubStr(h_week_of, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1028+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1029) : cplan_fd.sql(5326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  h_monday_regular := 0;
                  h_tuesday_regular := 0;
                  h_wednesday_regular := 0;
                  h_thursday_regular := 0;
                  h_friday_regular := 0;
                  h_saturday_regular := 0;
                  h_sunday_regular := 0;
               End If;
            End If;
         ElsIf PMO_ORASTD.strcmp(h_dayofyear, h_week_of) < 0 Then
            If PMO_ORASTD.strcmp(h_week_apv, h_next_week_of) = 0 Then
               h_new_actual := Coalesce(h_total, 0);
               h_work_percent := Trunc(Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0) * 10000.00 + 0.5);
               h_minutes := 0;
               --trace_printf("3_* SC_STATE: %d, h_week_of: %s, h_week_apv: %s, h_new_actual: %d, h_work_percent: %d\n", SC_STATE,h_week_of,h_week_apv,h_new_actual,h_work_percent);
               EXIT;
            Else
               h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
               If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
                  h_etc := 0;
                  h_milestone_percent := 10000;
               Else
                  h_etc := Coalesce(h_etc, 0) - Coalesce(h_weekly_regular_total, 0);
                  h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
               End If;
               
               PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
               
               h_ta_ms := Coalesce(h_milestone_percent, 0);
               h_ta_etc := Coalesce(h_etc, 0);
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     SELECT
                     SUNDAY_REGULAR,
                     SUNDAY_SPECIAL,
                     MONDAY_REGULAR,
                     MONDAY_SPECIAL,
                     TUESDAY_REGULAR,
                     TUESDAY_SPECIAL,
                     WEDNESDAY_REGULAR,
                     WEDNESDAY_SPECIAL,
                     THURSDAY_REGULAR,
                     THURSDAY_SPECIAL,
                     FRIDAY_REGULAR,
                     FRIDAY_SPECIAL,
                     SATURDAY_REGULAR,
                     SATURDAY_SPECIAL
                     INTO
                     h_sunday_regular,
                     h_sunday_special,
                     h_monday_regular,
                     h_monday_special,
                     h_tuesday_regular,
                     h_tuesday_special,
                     h_wednesday_regular,
                     h_wednesday_special,
                     h_thursday_regular,
                     h_thursday_special,
                     h_friday_regular,
                     h_friday_special,
                     h_saturday_regular,
                     h_saturday_special
                     FROM SUMMARY_TIMESHEETS
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND WEEK_OF = TO_DATE(SubStr(h_next_week_of, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1125+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1126) : cplan_fd.sql(5423) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     /*BEGIN_SQL
                                                     CLOSE CBW_AWP WITH RELEASE;
                                                 END_SQL;
                                                 return SC_STATE;*/
                     h_monday_regular := 0;
                     h_tuesday_regular := 0;
                     h_wednesday_regular := 0;
                     h_thursday_regular := 0;
                     h_friday_regular := 0;
                     h_saturday_regular := 0;
                     h_sunday_regular := 0;
                  End If;
               End If;
               
               --the week changed, reinitialize the week_of  next_week
               h_week_of := h_next_week_of;
               
               BEGIN
                  SELECT TO_DATE(SubStr(h_week_of, 1, 10)) - 7      INTO h_next_week_of FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1156+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1157) : cplan_fd.sql(5454) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2_* SC_STATE: %d, h_week_of: %s, h_next_week_of: %s\n", SC_STATE,h_week_of,h_next_week_of);
            End If;
         End If;
         
         --          i_prev_min = h_total;
         h_total := Coalesce(h_total, 0) - Coalesce(h_minutes, 0);
         --trace_printf("h_dayofyear = %s, h_total = %d, h_minutes = %d\n", h_dayofyear,h_total,h_minutes);
         If Coalesce(h_total, 0) >= Coalesce(h_new_actual, 0) Then
            h_minutes := 0;
            If Coalesce(h_total, 0) = Coalesce(h_new_actual, 0) Then
               EXIT;
            End If;
         Else
            --              h_temp_var  = i_prev_min - h_new_actual;
            h_temp_var := Coalesce(h_new_actual, 0) - Coalesce(h_total, 0);
            If Coalesce(h_temp_var, 0) < Coalesce(h_minutes, 0) Then
               h_minutes := Coalesce(h_temp_var, 0);
            Else
               h_minutes := 0;
            End If;
            
            EXIT;
         End If;
         --trace_printf("h_current_weekday: %d, h_temp_var = %d, h_total = %d, h_minutes = %d\n", h_current_weekday,h_temp_var,h_total,h_minutes);
         
         CASE h_current_weekday
            
            WHEN 1 THEN
               h_sunday_regular := 0;
               h_sunday_special := 0;
            
            WHEN 2 THEN
               h_monday_regular := 0;
               h_monday_special := 0;
            
            WHEN 3 THEN
               h_tuesday_regular := 0;
               h_tuesday_special := 0;
            
            WHEN 4 THEN
               h_wednesday_regular := 0;
               h_wednesday_special := 0;
            
            WHEN 5 THEN
               h_thursday_regular := 0;
               h_thursday_special := 0;
            
            WHEN 6 THEN
               h_friday_regular := 0;
               h_friday_special := 0;
            
            WHEN 7 THEN
               h_saturday_regular := 0;
               h_saturday_special := 0;
            ELSE
               NULL;
         END CASE;
      END LOOP;
      
      BEGIN
         CLOSE CBW_AWP ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1227+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1228) : cplan_fd.sql(5525) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("h_current_weekday: %d, h_minutes: %d, h_dayofyear = %d:%s, h_tsk_assignment_id = %d:%s\n",h_current_weekday,h_minutes,h_dayofyear_Ind,h_dayofyear,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
   --trace_printf("h_work_percent: %d, h_etc: %d, h_total = %d, h_new_actual: %d, h_week_of: %s\n", h_work_percent,h_etc,h_total,h_new_actual,h_week_of);
   --timeLog (file,"4_* FC_U_ASSIGN_PERCENT End of While Loop");
   
   If (h_dayofyear IS NOT NULL) Then
      If Coalesce(h_work_todate, 0) > Coalesce(h_new_actual, 0) Then
         CASE h_current_weekday
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
               h_sunday_special := 0;
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
               h_monday_special := 0;
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
               h_tuesday_special := 0;
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
               h_wednesday_special := 0;
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
               h_thursday_special := 0;
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
               h_friday_special := 0;
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
               h_saturday_special := 0;
            ELSE
               NULL;
         END CASE;
      Else
         CASE h_current_weekday
            WHEN 1 THEN
               h_sunday_regular := Coalesce(h_minutes, 0);
            
            WHEN 2 THEN
               h_monday_regular := Coalesce(h_minutes, 0);
            
            WHEN 3 THEN
               h_tuesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 4 THEN
               h_wednesday_regular := Coalesce(h_minutes, 0);
            
            WHEN 5 THEN
               h_thursday_regular := Coalesce(h_minutes, 0);
            
            WHEN 6 THEN
               h_friday_regular := Coalesce(h_minutes, 0);
            
            WHEN 7 THEN
               h_saturday_regular := Coalesce(h_minutes, 0);
            ELSE
               NULL;
         END CASE;
      End If;
      
      h_SF := PMO_CNST.SC_SF;
      If Coalesce(h_work_percent, 0) = 10000 Then
         h_etc := 0;
         h_effort := Coalesce(h_new_actual, 0);
         h_SF := PMO_CNST.SC_AF;
      Else
         h_etc := Coalesce(h_effort, 0) - Coalesce(h_new_actual, 0);
      End If;
      
      If Coalesce(h_etc, 0) = 0 Then
         h_work_percent := 10000;
         h_SF := PMO_CNST.SC_AF;
      End If;
      
      h_weekly_regular_total := Coalesce(h_monday_regular, 0) + Coalesce(h_tuesday_regular, 0) + Coalesce(
                                   h_wednesday_regular, 0) + Coalesce(h_thursday_regular, 0) 
                                   + Coalesce(h_friday_regular, 0) + Coalesce(
                                   h_saturday_regular, 0) + Coalesce(h_sunday_regular, 0);
      If Coalesce(h_effort, 0) > Coalesce(h_sched_work, 0) Then
         h_etc := 0;
         h_milestone_percent := 10000;
      Else
         h_milestone_percent := Trunc((1 - (Coalesce(h_etc, 0) / Coalesce(h_effort, 0))) * 10000.00);
      End If;
      --trace_printf("+++> h_week_of: %s, h_weekly_regular_total: %d, h_milestone_percent: %d, h_ta_ms: %d, h_etc: %d\n", h_week_of,h_weekly_regular_total,h_milestone_percent,h_ta_ms,h_etc);
      
      PMO_DB2M.SC_STATE := FC_U_SUMMARY_TIMESHEETS(h_call_type, h_last_week, h_project_id, h_res_id
                                             , h_tsk_assignment_id, h_week_of, h_null_str
                                             , h_null_str, h_tsk_assignment_id, h_rec_user
                                             , h_billable, h_capital_expense, h_monday_regular
                                             , h_monday_special, h_tuesday_regular
                                             , h_tuesday_special, h_wednesday_regular
                                             , h_wednesday_special, h_thursday_regular
                                             , h_thursday_special, h_friday_regular
                                             , h_friday_special, h_saturday_regular
                                             , h_saturday_special, h_sunday_regular
                                             , h_sunday_special, h_weekly_regular_total
                                             , h_weekly_special_total, h_sched_work
                                             , h_baseline1_work, h_baseline_effort_week, h_ta_etc
                                             , h_ta_ms, h_etc, h_work_percent, h_exchange_rate
                                             , h_actual_regular_cost, h_actual_regular_rate
                                             , h_actual_special_cost, h_actual_special_rate
                                             , h_baseline_regular_cost, h_baseline_regular_rate
                                             , h_baseline_special_cost, h_baseline_special_rate);
      
      --trace_printf("*** h_new_actual %d, h_last_day: %d\n",h_new_actual,h_last_day); //trace_flush();
      If Coalesce(h_new_actual, 0) = 0 Then
         --timeLog (file,"5_* FC_U_ASSIGN_PERCENT UPDATE TASK_ASSIGNMENTS  run U_TSKRES2");
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
            LAST_SCHED_MIN      = 0,
            WORK_TODATE         = 0,
            MILESTONE_PERCENT   = 0,
            ACTUAL_START        = NuLL,
            ACTUAL_FINISH       = NuLL,
            ACTUAL_DURATION     = 0,
            LAST_SUBMITTED      = NuLL,
            ETC                 = SCHED_WORK,
            ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
            SD                  = PMO_CNST.SC_SS,
            FD                  = PMO_CNST.SC_SF
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1368+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1369) : cplan_fd.sql(5666) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         /*BEGIN_SQL
                         SELECT PROJECT_ASSIGNMENT_ID, BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING,
                             C_EFFORT, T_UNIT, C_UNIT, RESOURCE_TYPE,
                             DATE(SCHED_START), DATE(SCHED_FINISH), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                         INTO HOST(h_prj_assignment_id), HOST(h_start_date), HOST(h_finish_date), HOST(h_effort), HOST(h_assign_string),
                             HOST(h_work), HOST(h_eunit), HOST(h_unit), HOST(h_res_type),
                             HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
                     END_SQL_NOCHECK;
         //trace_printf("*** h_effort %d, h_per_day: %d, h_start_date %s, h_finish_date %s\n",h_effort, h_per_day,h_start_date, h_finish_date);
         
                     InitializeHost(SC_PARAM(h_tph_delete), -1);
                     SC_STATE = FC_TPHASE_SCHED_ASSIGN(
                                 SC_PARAM(h_project_id),
                                 SC_PARAM(h_task_id),
                                 SC_PARAM(h_tsk_assignment_id),
                                 SC_PARAM(h_SD),
                                 SC_PARAM(h_FD),
                                 SC_PARAM(h_reg_cost),
                                 SC_PARAM(h_reg_sale),
                                 SC_PARAM(h_tph_delete),
                                 SC_PARAM(h_capital_expense),
                                 SC_PARAM(h_billable));
         
                     h_temp_var = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
         
                     if ( (IsNotNullParam(h_assign_string))  (strlen(h_assign_string) > 0) )
                     {
                         if ( IsNotNullParam(h_prj_assignment_id))
                             h_temp_var = (1 | SC_BIT_29_30);
                         else
                             h_temp_var = (1 | SC_BIT_28);
         
                         InitializeHost(SC_PARAM(h_temp_work_type), 2);
                         InitializeHost(SC_PARAM(h_temp_contour_type), 0);
                         InitializeHost(SC_PARAM(h_temp_occurrence), 0);
                         InitializeHost(SC_PARAM(h_temp_consider), 7);
         
                         // Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
                         SC_STATE = U_TSKRES2(SC_PARAM(h_tsk_assignment_id),    SC_PARAM(h_project_id),         SC_PARAM(h_task_id),
                                              SC_PARAM(h_res_id),               SC_PARAM(h_null_str),           SC_PARAM(h_null_str),
                                              SC_PARAM(h_start_date),           SC_PARAM(h_finish_date),        SC_PARAM(h_effort),
                                              SC_PARAM(h_work),                 SC_PARAM(h_eunit),              SC_PARAM(h_unit),
                                              SC_PARAM(h_res_type),             SC_PARAM(h_temp_work_type),     SC_PARAM(h_temp_var),
                                              SC_PARAM(h_contour),              SC_PARAM(h_temp_contour_type),  SC_PARAM(h_temp_occurrence),
                                              SC_PARAM(h_temp_consider),        SC_PARAM(h_assign_string),      SC_PARAM(h_rec_user),
                                              SC_PARAM(h_per_day));
         //trace_printf("*** h_per_day: %d\n", h_per_day);
         
                         InitializeHost(SC_PARAM(h_temp_var), 1);
                         h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var), h_per_day, 0);    // fix ACTUAL_START  SCHED
                     }*/
      Else
         --timeLog (file,"6_* FC_U_ASSIGN_PERCENT UPDATE TASK_ASSIGNMENTS  run FC_I_TIMEPHASED_DELTA");
         If (Coalesce(h_work_todate, 0) > Coalesce(h_new_actual, 0)) AND (Coalesce(h_minutes, 0) = 0) Then
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) - 1      INTO h_dayofyear FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1437) : cplan_fd.sql(5734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --trace_printf("***** i_keep_etc = %d, h_minutes: %d, h_etc: %d, h_work_percent: %d, h_new_actual: %d\n", i_keep_etc,h_minutes,h_etc,h_work_percent,h_new_actual);
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_dayofyear
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND ACTUAL_FLAG = 1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1454+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1455) : cplan_fd.sql(5752) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("TDY_* SC_STATE: %d, h_dayofyear: %d %s\n", SC_STATE,h_dayofyear_Ind,h_dayofyear);
         
         If i_keep_etc = 1 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = h_minutes,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = h_work_percent,
               LAST_SUBMITTED      = h_dayofyear,
               FORECAST_START      = NuLL,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1482+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1483) : cplan_fd.sql(5780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(h_etc, 0) = 0) OR (Coalesce(h_work_percent, 0) = 10000) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = 0,
               SCHED_WORK          = h_new_actual,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = 10000,
               LAST_SUBMITTED      = h_dayofyear,
               ETC                 = 0,
               FORECAST_START      = NuLL,
               FORECAST_FINISH     = NuLL,
               FORECAST_DURATION   = 0,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1512+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1513) : cplan_fd.sql(5810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
               LAST_SCHED_MIN      = h_minutes,
               WORK_TODATE         = h_new_actual,
               MILESTONE_PERCENT   = h_work_percent,
               LAST_SUBMITTED      = h_dayofyear,
               ETC                 = h_etc,
               FORECAST_START      = NuLL,
               FD                  = h_SF,
               UNITS_MSP           = CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
               ROLLUP_BITS         = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ASSIGN_PERCENT', 1539+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ASSIGN_PERCENT(1540) : cplan_fd.sql(5837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --timeLog (file,"6.1_* FC_U_ASSIGN_PERCENT run FC_I_TIMEPHASED_DELTA");
         --trace_printf("A_0_* SC_STATE: %d, h_percent_units: %d\n", SC_STATE,h_percent_units);
         --timeLog (file,"6.2_* FC_U_ASSIGN_PERCENT end FC_I_TIMEPHASED_DELTA");
      End If;
      --trace_printf("SC_STATE: %d, h_SF: %d, h_dayofyear: %s, h_work_percent: %d, h_minutes: %d, h_etc: %d\n",SC_STATE,h_SF,h_dayofyear,h_work_percent,h_minutes,h_etc);
      
      h_temp_var := FC_Rollup(h_project_id);
   End If;
   --trace_printf("**_FC_U_ASSIGN_PERCENT_** SC_STATE: %d\n",SC_STATE);
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ASSIGN_PERCENT', 1560+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_ASSIGN_PERCENT(1561) : cplan_fd.sql(5858) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_ASSIGN_PERCENT;
ALTER FUNCTION FC_U_ASSIGN_PERCENT COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_GET_DT ==
CREATE OR REPLACE FUNCTION FC_GET_DT
(
     h_start_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_dayofyear DATE;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_DT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_DT'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_DT(' || 
       COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DT', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_DT(18) : cplan_fd.sql(5886) : ' || SYSTIMESTAMP);
   END IF; 

   --short h_start_DT_Ind = 0;
   
   h_dayofyear := SubStr(h_start_DT, 1, 10);
   --h_dayofyear[10] = '\0';
   --trace_printf("h_dayofyear: %s, h_start_DT = %s, h_bitflag = %d\n",h_dayofyear,h_start_DT,h_bitflag);
   
   If Coalesce(h_bitflag, 0) = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
         FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 36+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(37) : cplan_fd.sql(5905) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_start_DT
         FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 50+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(51) : cplan_fd.sql(5919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      If Coalesce(h_bitflag, 0) = 0 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('08:00') ) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 67+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(68) : cplan_fd.sql(5936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('17:00') ) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_DT', 81+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_DT(82) : cplan_fd.sql(5950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("h_start_DT = %d;%s\n",h_start_DT_Ind,h_start_DT);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_DT', 91+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_DT(92) : cplan_fd.sql(5960) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_DT;
ALTER FUNCTION FC_GET_DT COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION U_LEVEL_STRING ==
CREATE OR REPLACE FUNCTION U_LEVEL_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , assign_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_hrs_per_day PLS_INTEGER := 0
   , sh_work PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   hrs_per_day PLS_INTEGER := Coalesce(sh_hrs_per_day, 0);
   work PLS_INTEGER := Coalesce(sh_work, 0);
   h_prj_assignment_id VARCHAR2(32);
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_min VARCHAR2(19);
   h_julian_day PLS_INTEGER := 0;
   h_jdy_finish PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_tmp_string VARCHAR2(8192);
   null_return LONG;
-- Declare Cursors
      CURSOR CC24c IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), SCHED_MIN
         FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
         ORDER BY DAY_YEAR;

      CURSOR C_SDa IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_start_D, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)) )
         FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_start_D, 1, 10))
         ORDER BY BIT_SCHED DESC ) WHERE ROWNUM<=1;

      CURSOR C_FDb IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP( TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED)) )
         FROM LEVEL_TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_finish_D, 1, 10))
         ORDER BY BIT_SCHED ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_LEVEL_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_LEVEL_STRING'), '-- ' || SYSTIMESTAMP, ':r := U_LEVEL_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_work), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_LEVEL_STRING', 59+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_LEVEL_STRING(60) : cplan_fd.sql(6030) : ' || SYSTIMESTAMP);
   END IF; 

   h_prj_assignment_id := assign_id;
   
   /*int ii = 0;*/
   h_assign_string := '';
   h_finish_D := '';
   h_finish_D := '';
   
   h_julian_day := 0;
   h_jdy_finish := 0;
   h_minutes := 0;
   h_total := 0;
   
   --trace_printf("h_project_id = %s, h_prj_assignment_id = %s\n", h_project_id,h_prj_assignment_id);
   h_temp_sc_state := FC_INI_ASSIGN_STR(1);
   h_tmp_string := '';
   
   BEGIN
      SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) INTO h_start_D, h_finish_D
      FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 87+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(88) : cplan_fd.sql(6058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_start_D: %d %s, h_finish_D: %d %s\n", SC_STATE, h_start_D_Ind, h_start_D, h_finish_D_Ind,h_finish_D);
   
   BEGIN
      SELECT PMO_ORASTD.JULIAN_DAY(h_start_D), PMO_ORASTD.JULIAN_DAY(h_finish_D)
      INTO h_julian_start, h_julian_finish
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 103+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(104) : cplan_fd.sql(6074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN CC24c;
   
   null_return := FC_ASSIGN_STRING(h_julian_start, 0);
   
   LOOP
      FETCH CC24c INTO h_julian_day, h_minutes;
      IF CC24c%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE = %d, h_minutes = %d\n", SC_STATE,h_minutes);
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      null_return := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
   END LOOP;
   
   BEGIN
      CLOSE CC24c ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 137+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(138) : cplan_fd.sql(6108) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   null_return := FC_ASSIGN_STRING(h_julian_finish, 0);
   
   null_return := FC_GET_ASSIGN_STR(h_assign_string, 0);
   
   If (Coalesce(Length(h_start_D), 0) > 0) AND (Coalesce(Length(h_finish_D), 0) > 0) Then
      OPEN C_SDa;
      
      FETCH C_SDa INTO h_start_date;
      IF C_SDa%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d %s\n", SC_STATE,h_start_date_Ind,h_start_date);
      
      BEGIN
         CLOSE C_SDa ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 165+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(166) : cplan_fd.sql(6136) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_FDb;
      
      FETCH C_FDb INTO h_finish_date;
      IF C_FDb%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE,h_finish_date_Ind,h_finish_date);
      
      BEGIN
         CLOSE C_FDb ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(189) : cplan_fd.sql(6159) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_prj_assignment_id: %s, h_bitflag: %d, h_assign_string: %s\n", h_prj_assignment_id, h_bitflag, h_assign_string);
      --trace_printf("h_start_date: %d %s, h_finish_date: %d %s, h_total: %d\n", h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date,h_total);
      
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET LEVEL_ASSIGN_STRING = h_assign_string,
         LEVEL_START = h_start_date, LEVEL_FINISH = h_finish_date, LEVEL_EFFORT = h_total
         WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_LEVEL_STRING', 210+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_LEVEL_STRING(211) : cplan_fd.sql(6181) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("SC_STATE: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id); //trace_flush();
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_LEVEL_STRING', 218+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_LEVEL_STRING(219) : cplan_fd.sql(6189) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_LEVEL_STRING;
ALTER FUNCTION U_LEVEL_STRING COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION U_ASSIGN_STRING ==
CREATE OR REPLACE FUNCTION U_ASSIGN_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_hrs_per_day PLS_INTEGER := 0
   , sh_etc PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   hrs_per_day PLS_INTEGER := Coalesce(sh_hrs_per_day, 0);
   etc PLS_INTEGER := Coalesce(sh_etc, 0);
   h_task_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_bit_flag VARCHAR2(1);
   h_min VARCHAR2(19);
   h_min_per_day PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_jdy_start PLS_INTEGER := 0;
   h_jdy_finish PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_tsk_prj PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_actual_work PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_actual_flag PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_u_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_assign_units PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   i_max_act PLS_INTEGER NOT NULL := 0;
   i_max_sched PLS_INTEGER NOT NULL := 0;
   i_u_dates PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CC24b IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), SCHED_MIN, ACTUAL_MIN, ACTUAL_FLAG
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_ASSIGN_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_ASSIGN_STRING'), '-- ' || SYSTIMESTAMP, ':r := U_ASSIGN_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_hrs_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_etc), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 73+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_ASSIGN_STRING(74) : cplan_fd.sql(6273) : ' || SYSTIMESTAMP);
   END IF; 

   /*int ii = 0;*/
   h_assign_string := '';
   h_dayofyear := '';
   h_finish_D := '';
   h_work := 0;
   h_etc := etc;
   h_assign_units := 0;
   h_percent_units := 0;
   h_u_units := 0;
   
   h_jdy_start := 0;
   h_jdy_finish := 0;
   h_minutes := 0;
   h_count := 0;
   h_bit_minutes := 0;
   
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
         h_percent_units := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30);
      Else
         i_u_dates := 1;
         h_percent_units := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29);
      End If;
      
      h_bitflag := 4;
      h_u_units := 1;
   ElsIf Coalesce(h_bitflag, 0) > PMO_CNST.SC_0000FFFF Then
      h_percent_units := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_FFFF0000)), 15);
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_0000FFFF));
      h_u_units := 1;
   End If;
   
   BEGIN
      SELECT TASK_ID,
      BIT_FLAG, 
      BEST, 
      WORK_TODATE, 
      SCHED_WORK, 
      C_EFFORT,
      CASE WHEN h_percent_units = 0 THEN UNITS_MSP ELSE h_percent_units END,
      PROJECT_ASSIGNMENT_ID
      INTO h_task_id,
      h_bit_flag, 
      h_count, 
      h_work_todate, 
      h_sched_work, 
      h_work,
      h_percent_units,
      h_prj_assignment_id
      FROM TASK_ASSIGNMENTS 
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 133+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(134) : cplan_fd.sql(6333) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 140+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(141) : cplan_fd.sql(6340) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   h_type := CASE WHEN SubStr(h_bit_flag, 1, 1) = '2' THEN 2 ELSE 0 END;
   h_tsk_prj := CASE WHEN PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 THEN 1 ELSE 0 END;
   h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
   
   If hrs_per_day = 0 Then
      BEGIN
         SELECT MINUTE_PER_DAY INTO h_min_per_day
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 160+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(161) : cplan_fd.sql(6360) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      hrs_per_day := Coalesce(h_min_per_day, 0);
   End If;
   
   If (Coalesce(h_u_units, 0) = 1) AND (Coalesce(h_work, 0) > 0) AND (hrs_per_day > 0) AND ((Coalesce(
             h_percent_units, 0) = 0) OR (Coalesce(h_percent_units, 0) = 10000)) Then
      h_percent_units := Trunc((Coalesce(h_work, 0) / hrs_per_day) * 10000.00);
   End If;
   
   null_return := FC_INI_ASSIGN_STR(1);
   
   If Coalesce(h_tsk_prj, 0) = 1 Then
      BEGIN
         SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) 
         INTO h_start_D, h_finish_D
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(189) : cplan_fd.sql(6388) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT MIN(DAY_YEAR), MAX(DAY_YEAR), SUM(SCHED_MIN) 
         INTO h_start_D, h_finish_D, h_count
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id 
         AND PARENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 205+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(206) : cplan_fd.sql(6405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_D)
         INTO h_julian_day
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 220+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(221) : cplan_fd.sql(6420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_minutes := 0;
      null_return := FC_ASSIGN_STRING(h_julian_day, h_minutes);
   End If;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR ((h_start_D IS NULL)) 
         OR ((h_finish_D IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 232+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(233) : cplan_fd.sql(6432) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.RECORD_NOT_FOUND;
   End If;
   
   BEGIN
      SELECT PMO_ORASTD.JULIAN_DAY(h_start_D), PMO_ORASTD.JULIAN_DAY(h_finish_D)
      INTO h_julian_start, h_julian_finish
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 247+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(248) : cplan_fd.sql(6447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_julian_day := 0;
   h_total := 0;
   h_actual_work := 0;
   
   OPEN CC24b;
   
   LOOP
      FETCH CC24b INTO h_julian_day, h_minutes, h_actual_min, h_actual_flag;
      IF CC24b%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If Coalesce(h_actual_flag, 0) = 1 Then
         h_actual_work := Coalesce(h_actual_work, 0) + Coalesce(h_actual_min, 0);
         If Coalesce(h_actual_min, 0) > i_max_act Then
            i_max_act := Coalesce(h_actual_min, 0);
         End If;
      End If;
      
      null_return := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      
      h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
      
      If Coalesce(h_minutes, 0) > i_max_sched Then
         i_max_sched := Coalesce(h_minutes, 0);
      End If;
   END LOOP;
   
   BEGIN
      CLOSE CC24b ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 294+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(295) : cplan_fd.sql(6494) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_julian_day, 0) > 0 Then
      null_return := FC_GET_ASSIGN_STR(h_assign_string, 0);
   ElsIf Coalesce(h_tsk_prj, 0) = 0 Then
      h_assign_string := TO_CHAR(Coalesce(h_julian_finish, 0) - Coalesce(h_julian_start, 0) + 1) || '*0,';
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 305+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(306) : cplan_fd.sql(6505) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.RECORD_NOT_FOUND;
   End If;
   
   BEGIN
      SELECT PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED) INTO h_start_time 
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
      AND DAY_YEAR=TO_DATE(SubStr(h_start_D, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 322+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(323) : cplan_fd.sql(6522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) OR (h_start_time IS NULL) Then
      h_start_time := '08:00:00';
   End If;
   
   BEGIN
      SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
      AND DAY_YEAR=TO_DATE(SubStr(h_finish_D, 1, 10));
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 343+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(344) : cplan_fd.sql(6543) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) OR (h_finish_time IS NULL) Then
      h_finish_time := '17:00:00';
   End If;
   
   BEGIN
      SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_D, 1, 10)), PMO_ORASTD.TIME(h_start_time)), 
      PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),  PMO_ORASTD.TIME(h_finish_time))
      INTO h_start_date, h_finish_date
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 363+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(364) : cplan_fd.sql(6563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_assign_units := 0;
   If i_max_act > 0 Then
      If (i_max_sched > 0) AND (i_max_sched > i_max_act) Then
         h_assign_units := Trunc((i_max_sched / hrs_per_day) * 10000.00);
      Else
         h_assign_units := Trunc((i_max_act / hrs_per_day) * 10000.00);
      End If;
   ElsIf i_max_sched > 0 Then
      h_assign_units := Trunc((i_max_sched / hrs_per_day) * 10000.00);
   End If;
   
   If Coalesce(h_percent_units, 0) >= Coalesce(h_assign_units, 0) Then
      h_assign_units := 0;
   ElsIf (Coalesce(h_assign_units, 0) > Coalesce(h_percent_units, 0)) AND ((Coalesce(
                                              h_assign_units, 0) - Coalesce(
                                              h_percent_units, 0)) < 100) Then
      h_assign_units := 0;
   End If;
   
   If Coalesce(h_type, 0) = 2 Then
      If Coalesce(h_etc, 0) > 0 Then
         h_total := Coalesce(h_etc, 0) + Coalesce(h_actual_work, 0);
      Else
         h_etc := Coalesce(h_total, 0) - Coalesce(h_actual_work, 0);
         h_remaining := CASE WHEN Coalesce(h_etc, 0) > Coalesce(h_remaining, 0) THEN Coalesce(
                                                                                     h_etc, 0) ELSE Coalesce(
                                                                                     h_remaining, 0) END;
      End If;
      
      If Coalesce(h_tsk_prj, 0) = 1 Then
         If Coalesce(h_bitflag, 0) = 4 Then
            If i_u_dates = 1 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7,
                  ASSIGN_STRING   = h_assign_string,
                  BAK_ASSIGN_STRING = CASE WHEN h_assign_units = 0 THEN h_assign_string ELSE BAK_ASSIGN_STRING END,
                  SCHED_WORK      = h_total,
                  SCHED_START     = h_start_date,
                  SCHED_FINISH    = h_finish_date,
                  PROPOSED_WORK   = h_total,
                  ETC             = h_etc,
                  ASSIGN_UNITS    = h_assign_units,
                  UNITS_MSP       = h_percent_units
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 422+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(423) : cplan_fd.sql(6622) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET 
                  CONSIDER = 7,
                  ASSIGN_STRING   = h_assign_string,
                  BAK_ASSIGN_STRING = CASE WHEN h_assign_units = 0 THEN h_assign_string ELSE BAK_ASSIGN_STRING END,
                  SCHED_WORK      = h_total,
                  PROPOSED_WORK   = h_total,
                  ETC             = h_etc,
                  ASSIGN_UNITS    = h_assign_units,
                  UNITS_MSP       = h_percent_units
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 448+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(449) : cplan_fd.sql(6648) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf Coalesce(h_bitflag, 0) = 1 Then
            If Coalesce(h_work_todate, 0) > 0 Then
               If Coalesce(h_remaining, 0) = 0 Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET 
                     SCHED_START = h_start_date,
                     SCHED_FINISH = h_finish_date,
                     SCHED_WORK = h_work_todate,
                     ACTUAL_START = h_start_date,
                     ACTUAL_FINISH = h_finish_date,
                     ASSIGN_UNITS  = h_assign_units,
                     ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 476+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(477) : cplan_fd.sql(6676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     SCHED_START = h_start_date,
                     SCHED_FINISH = h_finish_date,
                     SCHED_WORK = h_total,
                     ACTUAL_START = h_start_date,
                     ACTUAL_FINISH = NuLL, FD = PMO_CNST.SC_SF,
                     ASSIGN_UNITS  = h_assign_units,
                     ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 501+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(502) : cplan_fd.sql(6701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_START = h_start_date,
                  SCHED_FINISH = h_finish_date,
                  SCHED_WORK = h_total,
                  ACTUAL_START = NuLL,
                  ACTUAL_FINISH = NuLL,
                  ACTUAL_DURATION = 0,
                  WORK_TODATE = 0,
                  SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF,
                  MILESTONE_PERCENT = 0,
                  ASSIGN_UNITS  = h_assign_units,
                  ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 531+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(532) : cplan_fd.sql(6731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               ASSIGN_STRING = h_assign_string, 
               ASSIGN_UNITS  = h_assign_units,
               BAK_ASSIGN_STRING = h_assign_string,
               PROPOSED_START = h_start_date, 
               PROPOSED_FINISH = h_finish_date, 
               PROPOSED_WORK = h_total,
               SCHED_START = h_start_date, 
               SCHED_FINISH = h_finish_date, 
               SCHED_DURATION = 0,
               SCHED_WORK = CASE WHEN h_etc = 0 THEN h_work_todate ELSE h_total END,
               FORECAST_START = NULL, 
               FORECAST_FINISH = NULL, 
               FORECAST_DURATION = 0, 
               BAK_START_DT = h_start_date, 
               BAK_FINISH_DT = h_finish_date, 
               BAK_WORK = h_total, 
               BEST = h_total
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 567+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(568) : cplan_fd.sql(6767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 3 Then
            If Coalesce(h_work_todate, 0) > 0 Then
               If Coalesce(h_remaining, 0) = 0 Then
                  If Coalesce(h_percent_units, 0) = 10000 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET 
                        SCHED_START = h_start_date, 
                        SCHED_FINISH = h_finish_date,
                        SCHED_WORK = h_total, 
                        WORK_TODATE = h_total,
                        ACTUAL_START = h_start_date, 
                        ACTUAL_FINISH = h_finish_date,
                        BAK_START_DT = h_start_date, 
                        BAK_FINISH_DT = h_finish_date, 
                        BAK_WORK = h_total,
                        BEST = h_count, 
                        ASSIGN_STRING = h_assign_string, 
                        ASSIGN_UNITS  = h_assign_units,
                        BAK_ASSIGN_STRING = h_assign_string
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 601+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(602) : cplan_fd.sql(6801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET 
                        SCHED_START = h_start_date, 
                        SCHED_FINISH = h_finish_date,
                        SCHED_WORK = h_total,
                        ACTUAL_START = h_start_date, 
                        ACTUAL_FINISH = h_finish_date,
                        BAK_START_DT = h_start_date, 
                        BAK_FINISH_DT = h_finish_date, 
                        BAK_WORK = h_total,
                        BEST = h_count, 
                        ASSIGN_UNITS  = h_assign_units,
                        ASSIGN_STRING = h_assign_string, 
                        BAK_ASSIGN_STRING = h_assign_string
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 631+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(632) : cplan_fd.sql(6831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET 
                     SCHED_START = h_start_date, 
                     SCHED_FINISH = h_finish_date,
                     SCHED_WORK = h_total,
                     ACTUAL_START = h_start_date, 
                     ACTUAL_FINISH = NuLL, 
                     FD = PMO_CNST.SC_SF,
                     BAK_START_DT = h_start_date, 
                     BAK_FINISH_DT = h_finish_date, 
                     BAK_WORK = h_total,
                     BEST = h_count, 
                     ASSIGN_UNITS  = h_assign_units,
                     ASSIGN_STRING = h_assign_string, 
                     BAK_ASSIGN_STRING = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 663+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(664) : cplan_fd.sql(6863) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET 
                  SD = PMO_CNST.SC_SS, 
                  FD = PMO_CNST.SC_SF, 
                  START_DT = h_start_date, 
                  FINISH_DT = h_finish_date,
                  SCHED_START = h_start_date, 
                  SCHED_FINISH = h_finish_date, 
                  SCHED_WORK = h_total,
                  ACTUAL_START = NuLL, 
                  ACTUAL_FINISH = NuLL, 
                  ACTUAL_DURATION = 0, 
                  WORK_TODATE = 0,
                  MILESTONE_PERCENT = 0,
                  BAK_START_DT = h_start_date, 
                  BAK_FINISH_DT = h_finish_date, 
                  BAK_WORK = h_total,
                  BEST = h_count, 
                  ASSIGN_UNITS  = h_assign_units,
                  ASSIGN_STRING = h_assign_string, 
                  BAK_ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 701+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(702) : cplan_fd.sql(6901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               START_DT = h_start_date, 
               FINISH_DT = h_finish_date,
               SCHED_START = h_start_date, 
               SCHED_FINISH = h_finish_date,
               SCHED_WORK = h_total,
               ASSIGN_UNITS  = h_assign_units,
               ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 727+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(728) : cplan_fd.sql(6927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_bitflag, 0) = 3 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               SD = PMO_CNST.SC_SS, 
               FD = PMO_CNST.SC_SF, 
               START_DT = h_start_date, 
               FINISH_DT = h_finish_date,
               SCHED_START = h_start_date, 
               SCHED_FINISH = h_finish_date, 
               SCHED_WORK = h_total,
               BAK_START_DT = h_start_date, 
               BAK_FINISH_DT = h_finish_date, 
               BAK_WORK = h_count,
               BEST = h_count, 
               ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
               ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
               ETC = h_total, 
               ASSIGN_UNITS  = h_assign_units,
               ASSIGN_STRING = h_assign_string, 
               BAK_ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 764+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(765) : cplan_fd.sql(6964) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               START_DT = h_start_date, 
               FINISH_DT = h_finish_date,
               SCHED_START = CASE WHEN SCHED_START IS NuLL THEN h_start_date ELSE SCHED_START END,
               SCHED_FINISH = CASE WHEN SCHED_FINISH IS NuLL THEN h_finish_date ELSE SCHED_FINISH END,
               SCHED_WORK = h_total, 
               ETC = h_total, 
               ASSIGN_UNITS  = h_assign_units,
               ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 790+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(791) : cplan_fd.sql(6990) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      If Coalesce(h_etc, 0) > 0 Then
         h_total := Coalesce(h_etc, 0);
      End If;
      
      If Coalesce(h_bitflag, 0) = 4 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            PROPOSED_START = h_start_date, 
            PROPOSED_FINISH = h_finish_date, 
            PROPOSED_WORK = h_total,
            PLANNED_START = h_start_date, 
            PLANNED_FINISH = h_finish_date, 
            PLANNED_WORK = h_total,
            BAK_START_DT = h_start_date, 
            BAK_FINISH_DT = h_finish_date, 
            BAK_WORK = h_total, 
            BAK_ASSIGN_STRING = h_assign_string,
            BEST = h_count, 
            ASSIGN_STRING = h_assign_string, 
            ETC = h_total, 
            UNITS_MSP = h_percent_units
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 829+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(830) : cplan_fd.sql(7029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_tsk_prj, 0) = 1 Then
         If (Coalesce(h_assign_units, 0) > 0) AND (Coalesce(h_assign_units, 0) <> 10000) AND (Coalesce(
               h_assign_units, 0) <> Coalesce(h_percent_units, 0)) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               START_DT = h_start_date, 
               FINISH_DT = h_finish_date,
               PLANNED_START = h_start_date, 
               PLANNED_FINISH = h_finish_date, 
               PLANNED_WORK = h_total,
               BAK_START_DT = h_start_date, 
               BAK_FINISH_DT = h_finish_date, 
               BAK_WORK=h_total,
               UNITS_MSP = h_assign_units, 
               T_UNIT = 0, 
               C_EFFORT = 0, 
               C_UNIT = 0, 
               C_CONTOUR = 0, 
               CONTOUR_TYPE = 0,
               BEST = h_count, 
               BAK_ASSIGN_STRING = h_assign_string, 
               ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 866+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(867) : cplan_fd.sql(7066) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               START_DT = h_start_date, 
               FINISH_DT = h_finish_date,
               PLANNED_START = h_start_date, 
               PLANNED_FINISH = h_finish_date, 
               PLANNED_WORK = h_total, 
               ETC = h_total,
               BAK_START_DT = h_start_date, 
               BAK_FINISH_DT = h_finish_date, 
               BAK_WORK=h_total,
               BEST = h_count, 
               BAK_ASSIGN_STRING = h_assign_string, 
               ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 896+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(897) : cplan_fd.sql(7096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_bitflag, 0) = 3 Then
            If (Coalesce(h_assign_units, 0) > 0) AND (Coalesce(h_assign_units, 0) <> 10000) AND (Coalesce(
                  h_assign_units, 0) <> Coalesce(h_percent_units, 0)) Then
               -- fix ASSIGN UNITS
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET 
                  SD = PMO_CNST.SC_NS, 
                  FD = PMO_CNST.SC_NF, 
                  START_DT = h_start_date, 
                  FINISH_DT = h_finish_date,
                  PLANNED_START = h_start_date, 
                  PLANNED_FINISH = h_finish_date, 
                  PLANNED_WORK = h_total,
                  BAK_START_DT = h_start_date, 
                  BAK_FINISH_DT = h_finish_date, 
                  BAK_WORK = h_count,
                  ETC = h_total, 
                  BEST = h_count, 
                  ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                  ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                  UNITS_MSP = h_assign_units, 
                  T_UNIT = 0, 
                  C_EFFORT = 0, 
                  C_UNIT = 0, 
                  C_CONTOUR = 0, 
                  CONTOUR_TYPE = 0,
                  ASSIGN_STRING = h_assign_string, 
                  BAK_ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 941+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(942) : cplan_fd.sql(7141) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET 
                  SD = PMO_CNST.SC_NS, 
                  FD = PMO_CNST.SC_NF, 
                  START_DT = h_start_date, 
                  FINISH_DT = h_finish_date,
                  PLANNED_START = h_start_date, 
                  PLANNED_FINISH = h_finish_date, 
                  PLANNED_WORK = h_total,
                  BAK_START_DT = h_start_date, 
                  BAK_FINISH_DT = h_finish_date, 
                  BAK_WORK = h_count,
                  ETC = h_total, 
                  BEST = h_count, 
                  ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
                  ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
                  ASSIGN_STRING = h_assign_string, BAK_ASSIGN_STRING = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 974+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(975) : cplan_fd.sql(7174) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET 
               SD = PMO_CNST.SC_NS,
               FD = PMO_CNST.SC_NF,
               START_DT = h_start_date,
               FINISH_DT = h_finish_date,
               PLANNED_START = h_start_date,
               PLANNED_FINISH = h_finish_date,
               PLANNED_WORK = h_total,
               BEST = h_count,
               ESTIMATION_LABOR_COST = h_count * REGULAR_COST / 60,
               ESTIMATION_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE h_count * REGULAR_RATE / 60 END,
               UNITS_MSP = CASE WHEN h_assign_units > 0 THEN h_assign_units ELSE UNITS_MSP END,
               ASSIGN_STRING = h_assign_string
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_ASSIGN_STRING', 1005+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_ASSIGN_STRING(1006) : cplan_fd.sql(7205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   If Coalesce(h_tsk_prj, 0) <> 0 AND (h_prj_assignment_id IS NOT NULL) Then
      -- Task Assignmnet from project reserve
      h_temp_var := 0;
      null_return := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, hrs_per_day, 0);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_ASSIGN_STRING', 1020+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_ASSIGN_STRING(1021) : cplan_fd.sql(7220) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_ASSIGN_STRING;
ALTER FUNCTION U_ASSIGN_STRING COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_TDY_LAST_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_U_TDY_LAST_ACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_prj_assign_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_last_submitted IN OUT NOCOPY TIMESTAMP
   , sh_h_last_day PLS_INTEGER
   , sh_h_add_tp_delta_flag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_last_day PLS_INTEGER := sh_h_last_day;
   h_add_tp_delta_flag PLS_INTEGER := Coalesce(sh_h_add_tp_delta_flag, 0);
   h_dayofyear DATE;
   h_ta_min PLS_INTEGER := 0;
   h_pa_min PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   tempStr VARCHAR2(255);
   h_sched_work PLS_INTEGER := 0;
   h_01 NUMBER := 0;
   h_02 NUMBER := 0;
   h_03 NUMBER := 0;
   h_04 NUMBER := 0;
   h_05 NUMBER := 0;
   h_06 NUMBER := 0;
   h_07 NUMBER := 0;
   h_08 NUMBER := 0;
   h_09 NUMBER := 0;
   h_10 NUMBER := 0;
   h_11 NUMBER := 0;
   h_12 NUMBER := 0;
   h_total_h NUMBER := 0;
   h_d_total NUMBER := 0;
   h_d_01 NUMBER := 0;
   h_d_02 NUMBER := 0;
   h_d_03 NUMBER := 0;
   h_d_04 NUMBER := 0;
   h_d_05 NUMBER := 0;
   h_d_06 NUMBER := 0;
   h_d_07 NUMBER := 0;
   h_d_08 NUMBER := 0;
   h_d_09 NUMBER := 0;
   h_d_10 NUMBER := 0;
   h_d_11 NUMBER := 0;
   h_d_12 NUMBER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_budget_type PLS_INTEGER := 0;
   h_cc_id VARCHAR2(32);
   h_labor_cost_cc_id VARCHAR2(32);
   h_labor_revenue_cc_id VARCHAR2(32);
   h_cur_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TDY_LAST_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TDY_LAST_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TDY_LAST_ACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_prj_assign_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_last_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_add_tp_delta_flag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 72+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TDY_LAST_ACTUAL(73) : cplan_fd.sql(7303) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(h_last_day, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 77+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(78) : cplan_fd.sql(7308) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (h_prj_assign_id IS NOT NULL) Then
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN, BIT_SCHED INTO h_ta_min, h_bitID
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 95+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(96) : cplan_fd.sql(7326) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_ta_min, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 102+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(103) : cplan_fd.sql(7333) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      h_bitID2 := h_bitID;
      PMO_STPU.skipFirstBitsClearToFirst(Coalesce(h_last_day, 0), h_bitID2);
      
      BEGIN
         SELECT SCHED_MIN, BIT_SCHED INTO h_pa_min, h_bitID1
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_prj_assign_id 
         AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(123) : cplan_fd.sql(7353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_pa_min := Coalesce(h_pa_min, 0) + Coalesce(h_ta_min, 0);
         h_bitID1 := PMO_STPU.BIT_OR_DAY(h_bitID2, h_bitID1);
         
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_pa_min, BIT_SCHED = h_bitID1
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assign_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 144+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(145) : cplan_fd.sql(7375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            h_bitID1 := h_bitID;
            PMO_STPU.skipFirstBitsClearToFirst(Coalesce(h_last_day, 0), h_bitID1);
            BEGIN
               INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
               VALUES(h_project_id, h_task_id, h_res_id, h_prj_assign_id, h_prj_assign_id, h_last_submitted, h_ta_min, h_bitID1, '2');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 165+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(166) : cplan_fd.sql(7396) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
            VALUES(h_project_id, h_project_id, h_res_id, h_prj_assign_id, h_prj_assign_id, h_last_submitted, h_ta_min, h_bitID2, '2');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 184+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(185) : cplan_fd.sql(7415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN, BIT_SCHED INTO h_ta_min, h_bitID
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 202+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(203) : cplan_fd.sql(7433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_ta_min, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 209+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(210) : cplan_fd.sql(7440) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   End If;
   
   If h_add_tp_delta_flag = 1 Then
      BEGIN
         SELECT SCHED_MIN - ACTUAL_MIN INTO h_sched_work
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 228+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(229) : cplan_fd.sql(7459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_sched_work := 0;
   End If;
   
   If Coalesce(h_sched_work, 0) > 0 Then
      h_sched_work := 0 - Coalesce(h_sched_work, 0);
      
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_cur_year, h_start_month
         FROM FISCAL_CALENDAR
         WHERE TO_DATE(SubStr(h_last_submitted, 1, 10)) BETWEEN START_DATE AND FINISH_DATE;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 249+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(250) : cplan_fd.sql(7480) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         BEGIN
            SELECT BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE,
            LABOR_COST_CC_ID, LABOR_REVENUE_CC_ID
            INTO h_billable, h_capital_expense, h_reg_cost, h_reg_sale,
            h_labor_cost_cc_id, h_labor_revenue_cc_id
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 268+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(269) : cplan_fd.sql(7499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_01 := 0.00;
         h_02 := 0.00;
         h_03 := 0.00;
         h_04 := 0.00;
         h_05 := 0.00;
         h_06 := 0.00;
         h_07 := 0.00;
         h_08 := 0.00;
         h_09 := 0.00;
         h_10 := 0.00;
         h_11 := 0.00;
         h_12 := 0.00;
         h_total_h := 0.00;
         
         CASE h_start_month
            
            WHEN 1 THEN
               h_01 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_01, 0);
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_01, 0);
            
            WHEN 2 THEN
               h_02 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_02, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_02, 0);
            
            WHEN 3 THEN
               h_03 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_03, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_03, 0);
            
            WHEN 4 THEN
               h_04 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_04, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_04, 0);
            
            WHEN 5 THEN
               h_05 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_05, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_05, 0);
            
            WHEN 6 THEN
               h_06 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_06, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_06, 0);
            
            WHEN 7 THEN
               h_07 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_07, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_07, 0);
            
            WHEN 8 THEN
               h_08 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_08, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_08, 0);
            
            WHEN 9 THEN
               h_09 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_09, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_09, 0);
            
            WHEN 10 THEN
               h_10 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_10, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_10, 0);
            
            WHEN 11 THEN
               h_11 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_11, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_11, 0);
            
            WHEN 12 THEN
               h_12 := Coalesce(h_sched_work, 0) / 60.00;
               h_total_h := Coalesce(h_12, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_cost, 0);
               h_d_total := Coalesce(h_d_12, 0);
            ELSE
               NULL;
         END CASE;
         
         h_budget_type := PMO_CNST.SC_SCHED_BUDGET;
         If Coalesce(h_capital_expense, 0) = PMO_CNST.SC_EXPENSE_LABOR Then
            h_cc_id := 'EXPENSE_HOURS';
         Else
            h_cc_id := 'CAPITAL_HOURS';
         End If;
         
         BEGIN
            INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
            VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
            h_01, h_02, h_03, h_04, h_05, h_06,
            h_07, h_08, h_09, h_10, h_11, h_12);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 388+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(389) : cplan_fd.sql(7619) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE BUDGETS
            SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
            AND CHARGECODE_ID = h_cc_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 408+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(409) : cplan_fd.sql(7639) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_reg_cost, 0) <> 0.00 Then
            h_cc_id := h_labor_cost_cc_id;
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
               h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
               h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 433+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(434) : cplan_fd.sql(7664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 453+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(454) : cplan_fd.sql(7684) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_billable, 0) = 1 Then
            h_cc_id := 'BENEFIT_HOURS';
            
            BEGIN
               INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
               MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
               MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
               VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_total_h,
               h_01, h_02, h_03, h_04, h_05, h_06,
               h_07, h_08, h_09, h_10, h_11, h_12);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 479+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(480) : cplan_fd.sql(7710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE BUDGETS
               SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
               AND CHARGECODE_ID = h_cc_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 499+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(500) : cplan_fd.sql(7730) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_reg_sale, 0) <> 0.00 Then
               h_d_01 := Coalesce(h_01, 0) * Coalesce(h_reg_sale, 0);
               h_d_02 := Coalesce(h_02, 0) * Coalesce(h_reg_sale, 0);
               h_d_03 := Coalesce(h_03, 0) * Coalesce(h_reg_sale, 0);
               h_d_04 := Coalesce(h_04, 0) * Coalesce(h_reg_sale, 0);
               h_d_05 := Coalesce(h_05, 0) * Coalesce(h_reg_sale, 0);
               h_d_06 := Coalesce(h_06, 0) * Coalesce(h_reg_sale, 0);
               h_d_07 := Coalesce(h_07, 0) * Coalesce(h_reg_sale, 0);
               h_d_08 := Coalesce(h_08, 0) * Coalesce(h_reg_sale, 0);
               h_d_09 := Coalesce(h_09, 0) * Coalesce(h_reg_sale, 0);
               h_d_10 := Coalesce(h_10, 0) * Coalesce(h_reg_sale, 0);
               h_d_11 := Coalesce(h_11, 0) * Coalesce(h_reg_sale, 0);
               h_d_12 := Coalesce(h_12, 0) * Coalesce(h_reg_sale, 0);
               
               h_d_total := Coalesce(h_d_01, 0) + Coalesce(h_d_02, 0) + Coalesce(h_d_03, 0) + Coalesce(h_d_04, 0) 
                      + Coalesce(h_d_05, 0) + Coalesce(h_d_06, 0) + Coalesce(h_d_07, 0) + Coalesce(
                      h_d_08, 0) + Coalesce(h_d_09, 0) + Coalesce(h_d_10, 0) + Coalesce(h_d_11, 0) 
                      + Coalesce(h_d_12, 0);
               
               h_cc_id := h_labor_revenue_cc_id;
               BEGIN
                  INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                  MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                  MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                  VALUES(h_project_id, h_task_id, h_cc_id, h_cur_year, h_budget_type, h_d_total,
                  h_d_01, h_d_02, h_d_03, h_d_04, h_d_05, h_d_06,
                  h_d_07, h_d_08, h_d_09, h_d_10, h_d_11, h_d_12);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 541+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(542) : cplan_fd.sql(7772) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE BUDGETS
                  SET TIMEPHASED_BITS = PMO_STPU.BIT_OR(TIMEPHASED_BITS, PMO_CNST.SC_SCHED_DELTA_BIT)
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
                  AND CHARGECODE_ID = h_cc_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 561+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(562) : cplan_fd.sql(7792) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   End If;
   
   PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_last_day, 0), h_bitID);
   BEGIN
      UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN, BIT_SCHED = h_bitID
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 584+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(585) : cplan_fd.sql(7815) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET LAST_SCHED_MIN = 0 WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TDY_LAST_ACTUAL', 601+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TDY_LAST_ACTUAL(602) : cplan_fd.sql(7832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TDY_LAST_ACTUAL', 607+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TDY_LAST_ACTUAL(608) : cplan_fd.sql(7838) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TDY_LAST_ACTUAL;
ALTER FUNCTION FC_U_TDY_LAST_ACTUAL COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_ACPT_DURATION ==
CREATE OR REPLACE FUNCTION FC_ACPT_DURATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_fcst_duration PLS_INTEGER := 0;
   h_sch_duration PLS_INTEGER := 0;
   h_act_duration PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   h_calltype PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   null_return LONG;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACPT_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACPT_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_ACPT_DURATION(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_DURATION', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACPT_DURATION(30) : cplan_fd.sql(7878) : ' || SYSTIMESTAMP);
   END IF; 

   -- Accept FORECAST Dates
   BEGIN
      SELECT MILESTONE_PERCENT, ACTUAL_DURATION, SCHED_DURATION, FORECAST_VARIANCE
      INTO h_mlst_percent, h_act_duration, h_sch_duration, h_fcst_duration
      FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_DURATION', 43+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_DURATION(44) : cplan_fd.sql(7892) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_mlst_percent = %d, h_act_duration = %d, h_sch_duration = %d, h_fcst_duration: %d\n",h_mlst_percent,h_act_duration, h_sch_duration, h_fcst_duration);
   
   If (Coalesce(h_mlst_percent, 0) > 0) AND (Coalesce(h_fcst_duration, 0) <> 0) Then
      If Coalesce(h_act_duration, 0) >= Coalesce(h_sch_duration, 0) + Coalesce(h_fcst_duration, 0) Then
         h_mlst_percent := 10000;
         h_sch_duration := Coalesce(h_act_duration, 0);
      Else
         h_sch_duration := Coalesce(h_sch_duration, 0) + Coalesce(h_fcst_duration, 0);
         h_mlst_percent := Trunc(Coalesce(h_act_duration, 0) * 10000 / Coalesce(h_sch_duration, 0));
      End If;
      
      h_finish_date := NULL;
      h_rollup_bits := PMO_CNST.SC_ACTUAL_BIT + PMO_CNST.SC_SCHED_BIT + PMO_CNST.SC_PROPOSED_BIT;
      BEGIN
         UPDATE TMT_WBS SET MILESTONE_PERCENT = h_mlst_percent,
         SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_DURATION = h_sch_duration,
         FORECAST_START = NULL, FORECAST_FINISH = NULL, FORECAST_DURATION = 0,
         ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,h_rollup_bits)
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_DURATION', 76+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_DURATION(77) : cplan_fd.sql(7925) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_mlst_percent = %d, h_start_date = %s, h_finish_date: %s\n",SC_STATE,h_mlst_percent,h_start_date, h_finish_date);
   Else
      BEGIN
         UPDATE TMT_WBS SET
         SCHED_START = h_start_date, SCHED_FINISH = h_finish_date, SCHED_DURATION = FORECAST_DURATION,
         FORECAST_START = NULL, FORECAST_FINISH = NULL, FORECAST_DURATION = 0,
         ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PROPOSED_BIT)
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_DURATION', 98+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_DURATION(99) : cplan_fd.sql(7947) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   /**** Added to issue alert  ****/
   h_calltype := 3;
   h_alert_id := 'SCETCFORCASTACCEPTED';
   h_table_type := 1;
   
   null_return := FC_ISSUE_ALERT(h_alert_id, h_task_id, h_rec_user, h_calltype, Coalesce(
                              h_table_type, 0));
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_DURATION', 113+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_DURATION(114) : cplan_fd.sql(7962) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACPT_DURATION;
ALTER FUNCTION FC_ACPT_DURATION COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_ACPT_EFFORT ==
CREATE OR REPLACE FUNCTION FC_ACPT_EFFORT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_sched_work PLS_INTEGER
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , sh_h_effort PLS_INTEGER
   , sh_h_remaining PLS_INTEGER
   , sh_h_etc PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , h_last_submitted IN OUT NOCOPY TIMESTAMP
   , sh_h_FF_flag PLS_INTEGER
   , h_ta_start_DT IN OUT NOCOPY TIMESTAMP
   , h_ta_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_billable PLS_INTEGER
   , sh_h_capital_expense PLS_INTEGER
   , sh_h_reg_cost NUMBER
   , sh_h_reg_sale NUMBER
   , h_bit_flag IN OUT NOCOPY VARCHAR2
   , sh_h_mlst_percent PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , sh_h_work_todate PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_effort PLS_INTEGER := sh_h_effort;
   h_remaining PLS_INTEGER := sh_h_remaining;
   h_etc PLS_INTEGER := sh_h_etc;
   h_contour PLS_INTEGER := sh_h_contour;
   h_FF_flag PLS_INTEGER := sh_h_FF_flag;
   h_billable PLS_INTEGER := sh_h_billable;
   h_capital_expense PLS_INTEGER := sh_h_capital_expense;
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_reg_sale NUMBER := sh_h_reg_sale;
   h_mlst_percent PLS_INTEGER := sh_h_mlst_percent;
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_fa_start TIMESTAMP;
   h_fa_finish TIMESTAMP;
   h_finish_D DATE;
   h_week_of DATE;
   h_alert_id VARCHAR2(32);
   h_calltype PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   i_percent_complete PLS_INTEGER;
   null_return LONG;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACPT_EFFORT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACPT_EFFORT'), '-- ' || SYSTIMESTAMP, ':r := FC_ACPT_EFFORT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_remaining), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_etc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_FF_flag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_ta_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_ta_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_sale), 'NULL')
          || ', ' || '''' || h_bit_flag || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_mlst_percent), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_EFFORT', 87+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACPT_EFFORT(88) : cplan_fd.sql(8060) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_project_id = %s, h_task_id = %s\n",h_project_id, h_task_id);
   
   --trace_printf("h_start_date: %d %s, h_finish_date: %d %s, h_effort = %d, h_remaining: %d\n",h_start_date_Ind,h_start_date, h_finish_date_Ind,h_finish_date,h_effort,h_remaining); trace_flush();
   --trace_printf("h_last_submitted = %d, %s, h_FF_flag = %d, h_sched_work: %d, h_etc: %d, h_prj_assignment_id: %d %s\n",h_last_submitted_Ind,h_last_submitted, h_FF_flag,h_sched_work,h_etc,h_prj_assignment_id_Ind,h_prj_assignment_id); trace_flush();
   h_week_of := NULL;
   BEGIN
      SELECT MAX(WEEK_OF) + 7      INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
      WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
      AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED,PMO_CNST.SC_APV_STATE_SUBMITTED);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_EFFORT', 105+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_EFFORT(106) : cplan_fd.sql(8078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_week_of: %d %s\n", h_week_of_Ind,h_week_of);
   
   If ((h_week_of IS NULL)) OR ((h_finish_date IS NULL)) OR (PMO_ORASTD.strcmp(h_finish_date
         , h_week_of) > 0) Then
      If Coalesce(h_bitflag, 0) = 1 Then
         h_start_date := NULL;
         h_finish_date := NULL;
      ElsIf Coalesce(h_bitflag, 0) = 2 Then
         h_etc := 0;
      End If;
      
      If (h_prj_assignment_id IS NOT NULL) Then
         BEGIN
            SELECT COALESCE(h_start_date,h_ta_start_DT),COALESCE(h_finish_date,h_ta_finish_DT)
            INTO h_fa_start, h_fa_finish
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_EFFORT', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_EFFORT(132) : cplan_fd.sql(8104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_fa_start, h_fa_finish
               , h_etc) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_EFFORT', 139+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_EFFORT(140) : cplan_fd.sql(8112) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
         End If;
      End If;
   End If;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      -- Accept ETC
      If SubStr(h_bit_flag, 1, 1) = '2' Then
         h_mlst_percent := Trunc((Coalesce(h_work_todate, 0) / (Coalesce(h_work_todate, 0) + Coalesce(
                                  h_etc, 0))) * 10000.00);
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET 
            MILESTONE_PERCENT = h_mlst_percent,
            ACTUAL_FINISH = NULL,
            FD = PMO_CNST.SC_SF
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_EFFORT', 167+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_EFFORT(168) : cplan_fd.sql(8140) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_temp_var := 1;
      h_temp_sc_state := FC_ETC_SCHEDULE(h_tsk_assignment_id, h_temp_var, h_rec_user);
      --trace_printf("A_FC_ETC_SCHEDULE: h_temp_sc_state = %d\n", h_temp_sc_state);//trace_flush();
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_EFFORT', 178+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_EFFORT(179) : cplan_fd.sql(8151) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      End If;
   Else
      --trace_printf("h_bitflag = %d, h_FF_flag = %d, h_etc = %d, h_remaining = %d, h_effort = %d\n", h_bitflag,h_FF_flag,h_etc,h_remaining,h_effort);
      --trace_printf("h_start_date = %s, h_finish_date = %s\n", h_start_date,h_finish_date);//trace_flush();
      h_temp_sc_state := FC_ACPT_ETC_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id
                                      , h_prj_assignment_id, h_start_date, h_finish_date
                                      , h_last_submitted, h_bit_flag, h_effort, h_remaining
                                      , h_sched_work, h_etc, h_contour, h_FF_flag, h_bitflag
                                      , h_ta_start_DT, h_ta_finish_DT, h_reg_cost, h_reg_sale
                                      , h_billable, h_capital_expense, h_rec_user, 1);
      
      --trace_printf("*** h_tsk_assignment_id: %s, h_finish_date: %d:%s\n",h_tsk_assignment_id,h_finish_date_Ind,h_finish_date);//trace_flush();
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_EFFORT', 195+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_EFFORT(196) : cplan_fd.sql(8168) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      End If;
      --else
      --{
      --    if ( h_bitflag != 1 )
      --    {
      --        // Accept FORECAST DATES
      --        BEGIN_SQL
      --            UPDATE TASK_ASSIGNMENTS SET
      --                FORECAST_START = SC_NULL,
      --                FORECAST_FINISH = SC_NULL,
      --                FORECAST_DURATION = 0
      --            WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
      --        END_SQL;
      --    }
      
      --    if ( IsNotNullParam(h_finish_date) )
      --    {
      --        InitializeHost(SC_PARAM(h_finish_D), SC_PARAM(h_finish_date), 10 );
      --        h_temp_sc_state = FC_FIX_FF(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_finish_D));
      --    }
      --}
   End If;
   --trace_printf("###: h_passed: %d, h_temp_sc_state = %d\n", h_passed, h_temp_sc_state);//trace_flush();
   
   If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
      i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
      If (i_percent_complete > 1) OR i_percent_complete < -1 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_EFFORT', 240+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_EFFORT(241) : cplan_fd.sql(8213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   /**** Added to issue alert  ****/
   h_calltype := 3;
   h_alert_id := 'SCETCFORCASTACCEPTED';
   h_table_type := 0;
   
   null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_calltype, Coalesce(
                              h_table_type, 0));
   
   If Coalesce(h_temp_sc_state, 0) = 0 Then
      h_bitflag := 2;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_bitflag, 0, 0);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_EFFORT', 261+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_EFFORT(262) : cplan_fd.sql(8234) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACPT_EFFORT;
ALTER FUNCTION FC_ACPT_EFFORT COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_ACPT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION FC_ACPT_ETC_ASSIGN
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , h_last_submitted IN OUT NOCOPY TIMESTAMP
   , h_bit_flag IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_remaining PLS_INTEGER
   , sh_h_sched_work PLS_INTEGER
   , sh_h_etc PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_FF_flag PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_ta_start_DT IN OUT NOCOPY TIMESTAMP
   , h_ta_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_reg_cost NUMBER
   , sh_h_reg_sale NUMBER
   , sh_h_billable PLS_INTEGER
   , sh_h_capital_expense PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_update001 PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_remaining PLS_INTEGER := sh_h_remaining;
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_etc PLS_INTEGER := sh_h_etc;
   h_contour PLS_INTEGER := sh_h_contour;
   h_FF_flag PLS_INTEGER := sh_h_FF_flag;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_reg_sale NUMBER := sh_h_reg_sale;
   h_billable PLS_INTEGER := sh_h_billable;
   h_capital_expense PLS_INTEGER := sh_h_capital_expense;
   h_update001 PLS_INTEGER := Coalesce(sh_h_update001, 0);
   h_res_id VARCHAR2(32);
   h_constr_DT TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_week_of DATE;
   h_finish_D DATE;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_SD DATE;
   h_FD DATE;
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_first_time VARCHAR2(8);
   h_res_type VARCHAR2(1);
   h_table_type PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_ta_etc PLS_INTEGER := 0;
   h_ta_ms PLS_INTEGER := 0;
   h_actual_work PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR C_FF_A IS
         SELECT * FROM (SELECT DAY_YEAR, SCHED_MIN, RESOURCE_ID, BIT_SCHED, TABLE_TYPE 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10)) 
         AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_FF_B IS
         SELECT DAY_YEAR, SCHED_MIN
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10)) 
         AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACPT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACPT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := FC_ACPT_ETC_ASSIGN(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || '''' || h_bit_flag || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_remaining), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_etc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_FF_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_ta_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_ta_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_billable), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_update001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 116+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACPT_ETC_ASSIGN(117) : cplan_fd.sql(8361) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_FF_flag = %d, h_finish_date = %d, %s\n", h_FF_flag,h_finish_date_Ind,h_finish_date);
   --trace_printf("h_start_date = %d, %s, h_sched_work: %d, h_effort: %d, h_etc: %d\n", h_start_date_Ind,h_start_date,h_sched_work,h_effort,h_etc);//trace_flush();
   --trace_printf("1_h_ta_start_DT %d %s, h_ta_finish_DT = %d, %s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_ta_finish_DT_Ind,h_ta_finish_DT);
   
   h_SD := SubStr(h_ta_start_DT, 1, 10);
   h_FD := SubStr(h_ta_finish_DT, 1, 10);
   
   BEGIN
      SELECT COALESCE(h_start_date,h_ta_start_DT),COALESCE(h_finish_date,h_ta_finish_DT)
      INTO h_ta_start_DT, h_ta_finish_DT
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 136+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(137) : cplan_fd.sql(8381) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("2_h_ta_start_DT %d %s, h_ta_finish_DT = %d, %s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_ta_finish_DT_Ind,h_ta_finish_DT);
   
   h_finish_D := SubStr(h_finish_date, 1, 10);
   If ((h_last_submitted IS NOT NULL)) AND ((h_finish_date IS NOT NULL)) AND (PMO_ORASTD.strncmp(
         h_last_submitted, h_finish_D, 10) > 0) Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         FORECAST_START = NULL,
         FORECAST_FINISH = NULL,
         FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 161+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(162) : cplan_fd.sql(8406) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 167+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(168) : cplan_fd.sql(8412) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   h_tph_delete := -1;
   h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
   
   h_count := 0;
   h_dayofyear := NULL;
   
   BEGIN
      UPDATE ACTIVITIES
      SET ETC_MODIFIED = 'N'
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
      AND TSK_ASSIGNMENT_ID <> ELEMENT_ID
      AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 196+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(197) : cplan_fd.sql(8441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_FF_flag, 0) = 1 Then
      h_etc := Coalesce(h_remaining, 0);
      h_effort := 0;
      BEGIN
         SELECT SCHED_MIN INTO h_exists 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR = TO_DATE(SubStr(h_finish_D, 1, 10)) AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 215+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(216) : cplan_fd.sql(8460) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_exists, 0) = 0 Then
         h_duration := 0;
         --if ( FC_CREATE_CDY ( SC_PARAM(h_res_id), SC_PARAM(h_ta_start_DT), SC_PARAM(h_ta_finish_DT), SC_PARAM(h_duration), SC_RESOURCE ) != 0)
         --{
         -- return TASK_ASSIGNMENT_ERROR;
         --}
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDOY';
         PMO_DB2M.SC_STATE := 0;
         
         OPEN C_FF_A;
         
         FETCH C_FF_A INTO h_dayofyear, h_minute, h_res_id, h_bitID, h_table_type;
         IF C_FF_A%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_temp_var := 0;
            h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
            
            h_work_type := CASE WHEN SubStr(h_bit_flag, 1, 1) = '2' THEN 2 ELSE 0 END;
            h_res_type := '0';
            h_first_time := NULL;
            h_consider := 7;
            h_bitflag := 11;
            h_update := 0;
            
            --if (IsNotNullParam(h_prj_assignment_id))  // PA exists
            --{
            -- BEGIN_SQL
            --  SELECT BIT_SCHED INTO HOST(h_bitID)
            --  FROM TASK_DAY_OF_YEAR
            --  WHERE PROJECT_ID = HOST(h_project_id)
            --   AND TSK_ASSIGNMENT_ID = HOST(h_prj_assignment_id)
            --   AND DAY_YEAR = DATE(HOST(h_finish_D));
            -- END_SQL;
            --}
            
            h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(h_work_type, h_minute, h_update, h_project_id, h_task_id
                                         , h_res_id, h_res_type, h_prj_assignment_id
                                         , h_tsk_assignment_id, h_finish_D, h_first_time, h_bitflag
                                         , h_consider, h_table_type, h_bitID);
            
            BEGIN
               INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 279+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(280) : cplan_fd.sql(8524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            CLOSE C_FF_A ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 293+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(294) : cplan_fd.sql(8538) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      OPEN C_FF_B;
      
      LOOP
         FETCH C_FF_B INTO h_dayofyear, h_minute;
         IF C_FF_B%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            EXIT;
         End If;
         
         h_temp_var := 0;
         h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         h_effort := Coalesce(h_effort, 0) + Coalesce(h_minute, 0);
      END LOOP;
      
      BEGIN
         CLOSE C_FF_B ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 326+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(327) : cplan_fd.sql(8571) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_count
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 342+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(343) : cplan_fd.sql(8587) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_dayofyear
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR <= h_finish_D AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 358+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(359) : cplan_fd.sql(8603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("h_dayofyear: %d %s, h_finish_D: %s, h_count: %d, h_available: %d\n", h_dayofyear_Ind,h_dayofyear,h_finish_D,h_count,h_effort,h_available);//trace_flush();
      If ((h_dayofyear IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_dayofyear, h_finish_D, 10) < 0) Then
         If Coalesce(h_count, 0) = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
               AND DAY_YEAR = h_FD;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 381+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(382) : cplan_fd.sql(8626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(SCHED_FINISH))
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 401+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(402) : cplan_fd.sql(8646) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  PLANNED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(PLANNED_FINISH))
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 420+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(421) : cplan_fd.sql(8665) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      BEGIN
         SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR = h_finish_D;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 440+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(441) : cplan_fd.sql(8685) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(h_finish_date)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 454+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(455) : cplan_fd.sql(8699) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)), PMO_ORASTD.TIME(h_finish_time)) INTO h_finish_DT FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 467+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(468) : cplan_fd.sql(8712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("h_sched_work: %d, h_effort = %d, h_etc: %d, h_finish_DT: %s, h_finish_time: %s\n", h_sched_work,h_effort,h_etc,h_finish_DT,h_finish_time);//trace_flush();
      If Coalesce(h_sched_work, 0) > Coalesce(h_effort, 0) Then
         h_etc := Coalesce(h_etc, 0) - Coalesce(h_effort, 0);
         If SubStr(h_bit_flag, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_WORK = h_sched_work - h_effort, 
               ETC = h_etc,
               SCHED_FINISH = h_finish_DT,
               PROPOSED_WORK = h_sched_work - h_effort,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 498+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(499) : cplan_fd.sql(8743) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_WORK = h_sched_work - h_effort, 
               ETC = h_etc,
               PLANNED_FINISH = h_finish_DT,
               PROPOSED_WORK = h_sched_work - h_effort,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 524+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(525) : cplan_fd.sql(8769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_bit_flag, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_WORK = 0, 
               ETC = 0,
               SCHED_FINISH = h_finish_DT,
               PROPOSED_WORK = 0,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 552+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(553) : cplan_fd.sql(8797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_WORK = 0, 
               ETC = 0,
               PLANNED_FINISH = h_finish_DT,
               PROPOSED_WORK = 0,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 578+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(579) : cplan_fd.sql(8823) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      --get the bit_flag to determine the state of the project in order to update the correct info
      BEGIN
         SELECT RESOURCE_ID, BIT_FLAG, MILESTONE_PERCENT, WORK_TODATE, ETC 
         INTO h_res_id, h_bit_flag, h_ta_ms, h_actual_work, h_ta_etc
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 597+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(598) : cplan_fd.sql(8842) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_bitflag: %d, h_remaining = %d, h_etc: %d\n", h_bitflag,h_remaining,h_etc);//trace_flush();
      --check if FS  FF fall into weekend then exit
      If ((h_start_date IS NOT NULL)) AND ((h_finish_date IS NOT NULL)) Then
         BEGIN
            SELECT SUM(AVAILABLE_MIN) INTO h_available
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_res_id
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 615+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(616) : cplan_fd.sql(8860) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("FS_FF: h_res_id: %s, h_start_date: %s, h_finish_date: %s, h_available: %d\n", h_res_id, h_start_date, h_finish_date, h_available);
         
         If Coalesce(h_available, 0) = 0 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 638+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(639) : cplan_fd.sql(8883) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 644+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(645) : cplan_fd.sql(8889) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      End If;
      
      --trace_printf("h_remaining: %d, h_etc: %d, h_bitflag: %d\n", h_remaining, h_etc,h_bitflag);
      If (Coalesce(h_remaining, 0) = 0) AND (Coalesce(h_etc, 0) > 0) AND (Coalesce(h_bitflag, 0) = 2) Then
         h_bitflag := 3;
         h_effort := Coalesce(h_etc, 0);
      End If;
      
      If Coalesce(h_bitflag, 0) = 2 Then
         If SubStr(h_bit_flag, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH) ,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 676+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(677) : cplan_fd.sql(8921) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
               PLANNED_FINISH = COALESCE(FORECAST_FINISH,PLANNED_FINISH) ,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 699+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(700) : cplan_fd.sql(8944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_remaining, 0) = 0 Then
            If Coalesce(h_ta_ms, 0) = 10000 Then
               h_ta_ms := Trunc((Coalesce(h_actual_work, 0) / (Coalesce(h_actual_work, 0) + Coalesce(
                           h_ta_etc, 0))) * 10000.00);
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START = COALESCE(FORECAST_START,SCHED_START),
               SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH),
               SCHED_WORK = WORK_TODATE + ETC,
               MILESTONE_PERCENT = h_ta_ms,
               ACTUAL_FINISH = NULL,
               FD = PMO_CNST.SC_SF,
               PROPOSED_WORK = WORK_TODATE + ETC,
               FORECAST_START = NULL,
               FORECAST_FINISH = NULL,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 734+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(735) : cplan_fd.sql(8979) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_START = COALESCE(FORECAST_START,SCHED_START),
                  SCHED_FINISH = COALESCE(FORECAST_FINISH,SCHED_FINISH),
                  SCHED_WORK = WORK_TODATE + ETC,
                  PROPOSED_WORK = WORK_TODATE + ETC,
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 760+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(761) : cplan_fd.sql(9005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  PLANNED_START = COALESCE(FORECAST_START,PLANNED_START),
                  PLANNED_FINISH = COALESCE(FORECAST_FINISH,PLANNED_FINISH),
                  PLANNED_WORK = WORK_TODATE + ETC,
                  PROPOSED_WORK = WORK_TODATE + ETC,
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 785+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(786) : cplan_fd.sql(9030) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      --trace_printf("h_ta_start_DT = %d, %s, h_last_submitted %d %sh_effort: %d\n", h_ta_start_DT_Ind,h_ta_start_DT,h_last_submitted_Ind,h_last_submitted,h_effort);trace_flush();
      If ((h_ta_start_DT IS NOT NULL)) AND (Coalesce(h_effort, 0) > 0) Then
         If (h_last_submitted IS NOT NULL) Then
            h_week_of := NULL;
            BEGIN
               SELECT MAX(WEEK_OF) INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
               WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 806+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(807) : cplan_fd.sql(9051) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_week_of IS NOT NULL) Then
               BEGIN
                  SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 820+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(821) : cplan_fd.sql(9065) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_ORASTD.strncmp(h_ta_start_DT, h_week_of, 10) < 0 Then
                  h_ta_start_DT := h_week_of;
               End If;
            ElsIf Coalesce(h_contour, 0) > 0 Then
               --if the resource is assigned with contour then pass last submitted and FC_search_2 will handle
               h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_ta_start_DT))
                  INTO h_ta_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 842+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(843) : cplan_fd.sql(9087) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_ta_start_DT)) + 1     
                  INTO h_ta_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 857+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(858) : cplan_fd.sql(9102) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         --trace_printf("last_submitted: h_ta_start_DT = %d, %s\n", h_ta_start_DT_Ind,h_ta_start_DT); //trace_flush();
         
         If (h_prj_assignment_id IS NOT NULL) Then
            h_temp_var := 2;
         ElsIf Coalesce(h_sched_work, 0) = 0 Then
            h_temp_var := 1;
         Else
            h_temp_var := 0;
         End If;
         
         --get the assigned effort from project assignment before FS date
         
         --get the effort before forecast start
         BEGIN
            SELECT COALESCE(SUM(SCHED_MIN),0)  INTO h_total
            FROM TASK_DAY_OF_YEAR 
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND DAY_YEAR < TO_DATE(SubStr(h_ta_start_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 888+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(889) : cplan_fd.sql(9133) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, Coalesce(h_temp_var, 0));
         
         h_temp_var := 0;
         h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_ta_start_DT, h_effort, h_temp_var, h_contour
                                 , h_rec_user, 0, 0);
         --rule is: if user enter FS between sched dates and assignmnet is from project affected and assigned effort will exceed
         --project affecyted period then call reschedule with effort - effort before FS as effort and etc the same
         --trace_printf("FC_ASSIGN_ETC:----> h_temp_sc_state: %d\n", h_temp_sc_state);
         If (Coalesce(h_temp_sc_state, 0) = PMO_CNST.TASK_ASSIGNMENT_ERROR) Then
            h_effort := Coalesce(h_effort, 0) - Coalesce(h_total, 0);
            
            --trace_printf("AFTER SUBTRACTION: h_effort: %d\n", h_effort);
            If Coalesce(h_effort, 0) > 0 Then
               h_temp_var := 0;
               h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_ta_start_DT, h_effort, h_temp_var, h_contour
                                 , h_rec_user, 0, 0);
               
               --trace_printf("state after RESCHEDULE the second time-->h_temp_sc_state: %d, h_etc: %d\n", h_temp_sc_state, h_etc);
            End If;
         End If;
      End If;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   --trace_printf("h_update: %d, h_finish_date: %d %s\n", h_update,h_finish_date_Ind,h_finish_date); //trace_flush();
   If (Coalesce(h_update, 0) = 0) AND ((h_finish_date IS NOT NULL)) Then
      h_finish_D := SubStr(h_finish_date, 1, 10);
      
      If Coalesce(h_count, 0) <> 1 Then
         h_temp_sc_state := FC_FIX_FF(h_project_id, h_tsk_assignment_id, h_finish_D);
      End If;
      
      -- fix constraint date, forecast finish
      BEGIN
         SELECT CONSTRAINT_DT, CONSTRAINT_TYPE INTO h_constr_DT, h_constr_type
         FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 937+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(938) : cplan_fd.sql(9182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If (h_constr_DT IS NOT NULL) Then
            BEGIN
               UPDATE TMT_WBS SET
               CONSTRAINT_DT =
               CASE
               WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_date, 1, 10)) THEN h_finish_date
               WHEN h_constr_type IN (0,5,6,7) THEN h_finish_date
               ELSE h_constr_DT
               END,
               CONSTRAINT_TYPE =
               CASE
               WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_date, 1, 10)) THEN 2
               WHEN h_constr_type IN (0,5,6,7) THEN 2
               ELSE h_constr_type
               END, ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 969+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(970) : cplan_fd.sql(9214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET CONSTRAINT_DT = h_finish_date, CONSTRAINT_TYPE = 2
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 987+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(988) : cplan_fd.sql(9232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   ElsIf ((h_start_date IS NOT NULL)) AND ((h_finish_date IS NULL)) Then
      -- fix constraint date, forecast start
      BEGIN
         SELECT CONSTRAINT_DT, CONSTRAINT_TYPE INTO h_constr_DT, h_constr_type
         FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 1004+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(1005) : cplan_fd.sql(9249) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If (h_constr_DT IS NOT NULL) Then
            BEGIN
               UPDATE TMT_WBS SET
               CONSTRAINT_DT =
               CASE
               WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_date, 1, 10)) THEN h_start_date
               WHEN h_constr_type IN (1,2,3,4) THEN h_start_date
               ELSE h_constr_DT
               END,
               CONSTRAINT_TYPE =
               CASE
               WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_date, 1, 10)) THEN 6
               WHEN h_constr_type IN (1,2,3,4) THEN 6
               ELSE h_constr_type
               END
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 1036+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(1037) : cplan_fd.sql(9281) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET CONSTRAINT_DT = h_start_date, CONSTRAINT_TYPE = 6
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACPT_ETC_ASSIGN', 1054+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACPT_ETC_ASSIGN(1055) : cplan_fd.sql(9299) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACPT_ETC_ASSIGN', 1063+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACPT_ETC_ASSIGN(1064) : cplan_fd.sql(9308) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACPT_ETC_ASSIGN;
ALTER FUNCTION FC_ACPT_ETC_ASSIGN COMPILE DEBUG;

/********************************************************************************************************************/

Prompt == FUNCTION FC_CALC_DURATION ==
CREATE OR REPLACE FUNCTION FC_CALC_DURATION
(
     h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_per_week PLS_INTEGER
   , work IN OUT NOCOPY PLS_INTEGER
   , effort IN OUT NOCOPY PLS_INTEGER
   , duration IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_eunit PLS_INTEGER := Coalesce(sh_h_eunit, 0);
   h_unit PLS_INTEGER := Coalesce(sh_h_unit, 0);
   h_consider PLS_INTEGER := Coalesce(sh_h_consider, 0);
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_contour PLS_INTEGER := Coalesce(sh_h_contour, 0);
   h_occurrence PLS_INTEGER := Coalesce(sh_h_occurrence, 0);
   h_per_day PLS_INTEGER := Coalesce(sh_h_per_day, 0);
   h_per_week PLS_INTEGER := Coalesce(sh_h_per_week, 0);
   h_duration PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   i_work PLS_INTEGER NOT NULL := 0;
   d_percent NUMBER NOT NULL := 0.0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CALC_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CALC_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_CALC_DURATION(' || 
       COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_week), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 51+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CALC_DURATION(52) : cplan_fd.sql(9370) : ' || SYSTIMESTAMP);
   END IF; 

   h_effort := effort;
   h_work := work;
   
   If h_unit = 100 Then
      If h_occurrence = 0 Then
         d_percent := 10000.00 / Coalesce(h_work, 0);
         i_work := Trunc(h_per_day * d_percent);
      Else
         i_work := Trunc(((h_per_day * Coalesce(h_work, 0)) / 10000.00) + 0.4);
      End If;
   Else
      If (Coalesce(h_work, 0) > 0) AND (h_unit > 0) Then
         If (h_unit = 1) AND (h_unit = 2) Then
            h_work := Coalesce(h_work, 0) * h_per_day;
         ElsIf h_unit = 5 Then
            h_work := Coalesce(h_work, 0) * h_per_week;
         End If;
      End If;
      
      i_work := Coalesce(h_work, 0);
   End If;
   
   If Coalesce(h_effort, 0) = 0 Then
      If (h_finish_DT IS NULL) OR (Coalesce(Length(h_finish_DT), 0) = 0) Then
         If h_occurrence = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 80+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(81) : cplan_fd.sql(9399) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         ElsIf Coalesce(h_work, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 85+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(86) : cplan_fd.sql(9404) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         Else
            h_effort := h_occurrence * i_work;
         End If;
      End If;
   ElsIf h_eunit > 0 Then
      If h_eunit = 1 Then
         h_effort := Coalesce(h_effort, 0) * 60;
      ElsIf h_eunit = 2 Then
         h_effort := h_per_day * Coalesce(h_effort, 0);
      ElsIf h_eunit = 3 Then
         h_effort := h_per_week * Coalesce(h_effort, 0);
      End If;
   End If;
   
   If Coalesce(h_effort, 0) > h_per_day Then
      h_duration := Trunc(Coalesce(h_effort, 0) / h_per_day);
   Else
      h_duration := 1;
   End If;
   
   If (((h_finish_DT IS NULL)) OR (Coalesce(Length(h_finish_DT), 0) = 0)) AND (i_work > 0) Then
      h_every := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_0001F000)), 12);
      If h_bitflag = 3 Then
         If (Coalesce(h_duration, 0) * Coalesce(h_every, 0)) > 60 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 113+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(114) : cplan_fd.sql(9432) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
         End If;
         
         h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
         If Coalesce(h_week, 0) = 0 Then
            h_week := 1;
         End If;
         h_duration := Coalesce(h_duration, 0) + 1;
         h_duration := Coalesce(h_duration, 0) * Coalesce(h_every, 0);
      Else
         If (h_bitflag = 5) AND (i_work > 0) Then
            If Coalesce(h_duration, 0) > 60 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 128+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(129) : cplan_fd.sql(9447) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
            Else
               h_duration := 250;
            End If;
         Else
            h_duration := FC_SRCH_DURATION(h_bitflag, h_contour, Coalesce(h_every, 0), Coalesce(h_duration, 0)
                               , h_occurrence, h_unit);
         End If;
      End If;
   End If;
   
   If Coalesce(h_effort, 0) = 0 Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_DT) - PMO_ORASTD.DAYS(h_start_DT) INTO h_duration FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CALC_DURATION', 150+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CALC_DURATION(151) : cplan_fd.sql(9469) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_duration, 0) > 5000) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 157+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(158) : cplan_fd.sql(9476) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
      End If;
      h_duration := 0;
   ElsIf (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_duration, 0) > 0) Then
      If Coalesce(h_duration, 0) > 2000 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 165+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(166) : cplan_fd.sql(9484) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_SEARCH_ASSIGN;
      Else
         If Coalesce(h_duration, 0) < 30 Then
            h_duration := 50;
         End If;
         
         If d_percent > 0.00 Then
            h_duration := Trunc(Coalesce(h_duration, 0) * d_percent);
         End If;
      End If;
   End If;
   
   If h_unit <> 100 Then
      h_work := i_work;
   End If;
   
   duration := Coalesce(h_duration, 0);
   effort := Coalesce(h_effort, 0);
   work := Coalesce(h_work, 0);
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CALC_DURATION', 190+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CALC_DURATION(191) : cplan_fd.sql(9509) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CALC_DURATION;
ALTER FUNCTION FC_CALC_DURATION COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_AD_TASK ==
CREATE OR REPLACE FUNCTION FC_AD_TASK
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_element_id IN OUT NOCOPY VARCHAR2
   , sh_h_work_todate PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , g_h_new_actual_duration IN OUT NOCOPY PLS_INTEGER
   , g_h_new_sched_duration IN OUT NOCOPY PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_work_todate PLS_INTEGER := sh_h_work_todate;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_actual_duration PLS_INTEGER := 0;
   h_sched_duration PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_TAD IS
         SELECT DAY_YEAR, ACTUAL_MIN, SCHED_MIN
         FROM PMO_TMP_TAD
         ORDER BY DAY_YEAR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_AD_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_AD_TASK'), '-- ' || SYSTIMESTAMP, ':r := FC_AD_TASK(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_element_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_todate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(g_h_new_actual_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(g_h_new_sched_duration), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AD_TASK', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_AD_TASK(42) : cplan_fd.sql(9561) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TAD';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_TAD SELECT DAY_YEAR, SUM(ACTUAL_MIN), SUM(SCHED_MIN)
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id AND TASK_ID = h_element_id
      GROUP BY DAY_YEAR;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 63+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(64) : cplan_fd.sql(9583) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT COUNT(DAY_YEAR), MIN(DAY_YEAR), MAX(DAY_YEAR) 
         INTO h_sched_duration, h_start_D, h_finish_D 
         FROM PMO_TMP_TAD;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 79+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(80) : cplan_fd.sql(9599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      g_h_new_sched_duration := Coalesce(h_sched_duration, 0) * Coalesce(h_per_day, 0);
      
      h_total := 0;
      h_count := 0;
      
      OPEN C_TAD;
      
      LOOP
         FETCH C_TAD INTO h_dayofyear, h_actual_min, h_minute;
         IF C_TAD%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            EXIT;
         End If;
         
         If (Coalesce(h_total, 0) = 0) AND (Coalesce(h_minute, 0) >= Coalesce(h_work_todate, 0)) Then
            If Coalesce(h_minute, 0) < Coalesce(h_per_day, 0) Then
               If Coalesce(h_sched_duration, 0) > 1 AND Coalesce(h_actual_min, 0) = Coalesce(h_minute, 0) Then
                  g_h_new_actual_duration := Coalesce(h_per_day, 0);
               Else
                  g_h_new_actual_duration := Coalesce(h_work_todate, 0);
               End If;
            Else
               g_h_new_actual_duration := Trunc((Coalesce(h_work_todate, 0) / Coalesce(h_minute, 0)) * Coalesce(
                                           h_per_day, 0));
            End If;
            
            EXIT;
         Else
            h_total := Coalesce(h_total, 0) + Coalesce(h_minute, 0);
            If Coalesce(h_total, 0) > Coalesce(h_work_todate, 0) Then
               h_temp_var := Coalesce(h_work_todate, 0) - Coalesce(h_totpr, 0);
               If Coalesce(h_minute, 0) < Coalesce(h_per_day, 0) Then
                  g_h_new_actual_duration := Coalesce(h_count, 0) * Coalesce(h_per_day, 0) + Coalesce(h_temp_var, 0);
               Else
                  g_h_new_actual_duration := Trunc((Coalesce(h_count, 0) + (Coalesce(h_temp_var, 0) / Coalesce(
                                           h_minute, 0))) * Coalesce(h_per_day, 0));
               End If;
               
               EXIT;
            ElsIf Coalesce(h_total, 0) = Coalesce(h_work_todate, 0) Then
               h_count := Coalesce(h_count, 0) + 1;
               g_h_new_actual_duration := Coalesce(h_count, 0) * Coalesce(h_per_day, 0);
               
               EXIT;
            End If;
            
            h_totpr := Coalesce(h_total, 0);
            h_count := Coalesce(h_count, 0) + 1;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_TAD ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 148+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(149) : cplan_fd.sql(9668) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_total, 0) > 0 Then
         BEGIN
            SELECT COUNT(cdy.DAY_YEAR) INTO h_count
            FROM PMO_TMP_CDY cdy 
            WHERE cdy.DAY_YEAR BETWEEN h_start_D AND h_finish_D 
            AND NOT EXISTS (SELECT '1' FROM PMO_TMP_TAD a WHERE a.DAY_YEAR = cdy.DAY_YEAR);
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AD_TASK', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AD_TASK(166) : cplan_fd.sql(9685) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_count := Coalesce(h_count, 0) * Coalesce(h_per_day, 0);
         g_h_new_actual_duration := g_h_new_actual_duration + Coalesce(h_count, 0);
         g_h_new_sched_duration := g_h_new_sched_duration + Coalesce(h_count, 0);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AD_TASK', 177+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AD_TASK(178) : cplan_fd.sql(9697) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_AD_TASK;
ALTER FUNCTION FC_AD_TASK COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_PA_WORK ==
CREATE OR REPLACE FUNCTION FC_U_PA_WORK
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_min_per_day PLS_INTEGER := 0
   , sh_h_bitflag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_min_per_day PLS_INTEGER := Coalesce(sh_h_min_per_day, 0);
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_dayofyear DATE;
   h_minute PLS_INTEGER := 0;
   h_pa_units PLS_INTEGER := 0;
   h_pa_minutes PLS_INTEGER := 0;
   h_actual_flag PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR C_TS_ACT1 IS
         SELECT ta.DAY_YEAR, ta.ACTUAL_FLAG, ta.SCHED_MIN, ta.BIT_SCHED
         FROM TASK_DAY_OF_YEAR ta
         WHERE ta.PROJECT_ID = h_project_id AND ta.TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ta.PARENT_ID = h_prj_assignment_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_PA_WORK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_PA_WORK'), '-- ' || SYSTIMESTAMP, ':r := FC_U_PA_WORK(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_min_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_PA_WORK', 36+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_PA_WORK(37) : cplan_fd.sql(9744) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT UNITS_MSP INTO h_pa_units
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(50) : cplan_fd.sql(9757) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1.1==> SC_STATE: %d, h_pa_units: %d, h_min_per_day: %d\n", SC_STATE,h_pa_units,h_min_per_day);
   
   If Coalesce(h_pa_units, 0) <> 0 AND Coalesce(h_pa_units, 0) <> 10000 Then
      h_pa_units := Trunc(Coalesce(h_pa_units, 0) * h_min_per_day / 10000.00);
   Else
      h_pa_units := h_min_per_day;
   End If;
   
   --no need to relink to the same table, we can just fetch where parent_id = prj_assignmnet_id
   
   OPEN C_TS_ACT1;
   
   LOOP
      FETCH C_TS_ACT1 INTO h_dayofyear, h_actual_flag, h_minute, h_bitID;
      IF C_TS_ACT1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      --trace_printf("h_dayofyear: %s, h_minute: %d\n", h_dayofyear,h_minute);
      
      If (h_bitflag = 1) AND (Coalesce(h_actual_flag, 0) = 1) Then
         GOTO Continue0000;
      End If;
      
      BEGIN
         SELECT SCHED_MIN, BIT_SCHED 
         INTO h_pa_minutes, h_bitID2
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
         AND DAY_YEAR = h_dayofyear;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(95) : cplan_fd.sql(9802) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bitID2);
         h_pa_minutes := Coalesce(h_pa_minutes, 0) + Coalesce(h_minute, 0);
         h_minute := CASE WHEN Coalesce(h_pa_minutes, 0) > Coalesce(h_pa_units, 0) THEN Coalesce(
                                                                                        h_pa_units, 0) ELSE Coalesce(
                                                                                        h_pa_minutes, 0) END;
         
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET 
            SCHED_MIN =  h_minute, 
            BIT_SCHED = h_bitID, 
            TSK_ASSIGNMENT_ID = PARENT_ID
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 124+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(125) : cplan_fd.sql(9832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 144+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(145) : cplan_fd.sql(9852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET 
            TSK_ASSIGNMENT_ID = PARENT_ID
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(166) : cplan_fd.sql(9873) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      <<Continue0000>>
      NULL;
   END LOOP;
   
   BEGIN
      CLOSE C_TS_ACT1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_PA_WORK', 182+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_PA_WORK(183) : cplan_fd.sql(9890) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_PA_WORK', 188+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_PA_WORK(189) : cplan_fd.sql(9896) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_PA_WORK;
ALTER FUNCTION FC_U_PA_WORK COMPILE DEBUG;

-- Fix Duration Effort Driven, Don't remove

/*********************************************************************************************************************************************/

/*
BOOL_RETURN_CODE FC_FD_EFFORT_DRIVEN
(
    SC_CHAR_PARAM   h_project_id[SC_H_ID],
    short           h_project_id_Ind,
    SC_CHAR_PARAM   h_task_id[SC_H_ID],
    short           h_task_id_Ind,
    SC_CHAR_PARAM   h_res_id[SC_H_ID],
    short           h_res_id_Ind,
    SC_CHAR_PARAM   h_res_type[SC_H_FLAG],
    short           h_res_type_Ind,
    SC_CHAR_PARAM   h_start_DT[SC_H_TIMESTAMP],
    short           h_start_DT_Ind,
    SC_CHAR_PARAM   h_finish_DT[SC_H_TIMESTAMP],
    short           h_finish_DT_Ind,
    SC_LONG_PARAM   h_remaining,
    short           h_remaining_Ind,
    SC_LONG_PARAM   h_work_todate,
    short           h_work_todate_Ind,
    SC_LONG_PARAM   h_cnt,
    short           h_cnt_Ind,
    SC_LONG_PARAM   h_per_day,
    short           h_per_day_Ind,
    SC_LONG_PARAM   add_remove
)
{
    PMO_BEGIN_FUNCTION(FC_FD_EFFORT_DRIVEN);

    SC_CHAR(h_tsk_assignment_id, SC_H_ID);
    SC_CHAR(h_prj_assignment_id, SC_H_ID);
    SC_CHAR(h_rec_user, SC_H_ID);
    SC_CHAR(h_ta_start_DT, SC_H_TIMESTAMP);
    SC_CHAR(h_start_time, SC_H_TIME);
    SC_CHAR(h_finish_time, SC_H_TIME);
    SC_LONG(h_available);
    SC_LONG(h_duration);
    SC_LONG(h_new_task_rem);
    SC_LONG(h_old_res_rem);
    SC_LONG(h_new_res_rem);
    SC_LONG(h_sum_res_rem);
    SC_LONG(h_sum_units);
    SC_LONG(h_tsk_units);
    SC_LONG(h_new_units);
    SC_LONG(h_minutes);
    SC_LONG(h_count);
    SC_LONG(h_temp_var);
    SC_LONG(h_etc_diff);
    SC_LONG(h_tph_delete);
    SC_DOUBLE(h_total_h);

    if ( (h_remaining == 0) || (h_cnt == 0) ) return ASSN_FIXED_DURATION;
//trace_printf("add_remove: %d, h_res_id: %s, h_res_type: %s, h_remaining: %d, h_work_todate: %d\n",add_remove,h_res_id,h_res_type,h_remaining,h_work_todate);//trace_flush();
//trace_printf("h_start_DT: %d %s, h_finish_DT: %d %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();

    InitializeHost(SC_PARAM(h_tph_delete), -1);
    h_temp_var = FC_TPHASE_FD_SCHED(
        SC_PARAM(h_project_id),
        SC_PARAM(h_task_id),
        SC_PARAM(h_tph_delete));

    BEGIN_SQL
        DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_FIXDUR(
            TSK_ASSIGNMENT_ID       CHAR(32),
            RESOURCE_ID             CHAR(32),
            RESOURCE_TYPE           CHAR(1),
            UNRESOLVED_START        TIMESTAMP,
            RESUME_DT               TIMESTAMP,
            WORK_TODATE             INTEGER,
            OLD_REMAINING           INTEGER,
            OLD_UNITS               INTEGER,
            NEW_REMAINING           INTEGER,
            NEW_UNITS               INTEGER,
            DURATION                INTEGER
        ) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
    END_SQL_NOCHECK;

    BEGIN_SQL
        INSERT INTO session.PMO_TMP_FIXDUR
        SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, RESOURCE_TYPE,
            UNRESOLVED_START, TIMESTAMP(LAST_SUBMITTED,'00:00') + 1 DAYS,
            WORK_TODATE, ETC, CASE WHEN UNITS_MSP = 0 THEN 10000 ELSE UNITS_MSP END, 0, 0, 0
        FROM TASK_ASSIGNMENTS
        WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
            AND (ACTUAL_FINISH IS NuLL OR RESOURCE_ID = HOST(h_res_id));
    END_SQL;
//trace_printf("SC_STATE: %d, h_project_id: %s, h_task_id: %s\n",SC_STATE,h_project_id,h_task_id);//trace_flush();

//  if (SC_NOSUCCESS) return SC_STATE;

    if (SC_SUCCESS)
    {
        BEGIN_SQL
            UPDATE session.PMO_TMP_FIXDUR t SET t.DURATION = (
                SELECT COUNT(tdy.DAY_YEAR)
                FROM TASK_DAY_OF_YEAR tdy
                WHERE tdy.PROJECT_ID = HOST(h_project_id) 
					AND tdy.TSK_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND tdy.ACTUAL_FLAG = 0);
        END_SQL;

        InitializeHost(SC_PARAM(h_tsk_units), 0);
    }
    else
    {
        BEGIN_SQL
            SELECT TIMESTAMP(DATE(MIN(ACTUAL_FINISH)),'00:00') + 1 DAYS INTO HOST(h_ta_start_DT)
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id);
        END_SQL;
//trace_printf("SC_STATE: %d, h_ta_start_DT: %d %s\n",SC_STATE,h_ta_start_DT_Ind,h_ta_start_DT);//trace_flush();

        InitializeHost(SC_PARAM(h_tsk_units), 10000);
    }

    InitializeHost(SC_PARAM(h_new_task_rem), 0);
    InitializeHost(SC_PARAM(h_sum_res_rem), 0);
    InitializeHost(SC_PARAM(h_sum_units), 0);
    InitializeHost(SC_PARAM(h_minutes), 0);
    InitializeHost(SC_PARAM(h_available), 0);
    InitializeHost(SC_PARAM(h_count), 0);
    InitializeHost(SC_PARAM(h_etc_diff), 0);
    InitializeHost(SC_PARAM(h_temp_var), 0);

    if ( add_remove == 0 )  // add
    {
        if (h_tsk_units == 0)
        {
            BEGIN_SQL
                SELECT MIN(RESUME_DT), SUM(OLD_UNITS) INTO HOST(h_ta_start_DT), HOST(h_tsk_units)
                FROM session.PMO_TMP_FIXDUR;
            END_SQL;
//trace_printf("SC_STATE: %d, h_tsk_units: %d, h_ta_start_DT: %d %s\n",SC_STATE,h_tsk_units,h_ta_start_DT_Ind,h_ta_start_DT);//trace_flush();
        }

        if (IsNotNullParam(h_ta_start_DT))
        {
            BEGIN_SQL
                SELECT TIMESTAMP(DATE(HOST(h_ta_start_DT)),TIME(FIRST_AVAIL_TIME(BIT_SCHED))) INTO HOST(h_start_DT)
                FROM session.PMO_TMP_CDY
                WHERE DAY_YEAR = DATE(HOST(h_ta_start_DT));
            END_SQL;
        }

        if ( h_work_todate == 0 )
        {
            BEGIN_SQL
                SELECT SUM(DURATION) INTO HOST(h_duration)
                FROM session.PMO_TMP_FIXDUR;
            END_SQL;

            if ( checkFlagStatus(h_res_type, '1') )
            {
                BEGIN_SQL
                    SELECT COUNT(*) INTO HOST(h_count)
                    FROM session.PMO_TMP_CDY
                    WHERE DAY_YEAR BETWEEN DATE(HOST(h_start_DT)) AND DATE(HOST(h_finish_DT));
                END_SQL;
            }
            else
            {
                BEGIN_SQL
                    SELECT COUNT(r.DAY_YEAR) INTO HOST(h_count)
                    FROM session.PMO_TMP_CDY p JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
                    WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR BETWEEN DATE(HOST(h_start_DT)) AND DATE(HOST(h_finish_DT));
                END_SQL;
            }

            h_available = h_duration + h_count;
        }
        else
        {
            if ( checkFlagStatus(h_res_type, '1') )
            {
                BEGIN_SQL
                    SELECT SUM(AVAILABLE_MIN) INTO HOST(h_available)
                    FROM session.PMO_TMP_CDY
                    WHERE DAY_YEAR BETWEEN DATE(HOST(h_start_DT)) AND DATE(HOST(h_finish_DT));
                END_SQL;
            }
            else
            {
                BEGIN_SQL
                    SELECT SUM(TOTAL_DAY_2(r.BIT_SCHED,p.BIT_SCHED)) INTO HOST(h_available)
                    FROM session.PMO_TMP_CDY p JOIN CALENDAR_DAY_OF_YEAR r ON p.DAY_YEAR = r.DAY_YEAR
                    WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR BETWEEN DATE(HOST(h_start_DT)) AND DATE(HOST(h_finish_DT));
                END_SQL;
            }
        }
//trace_printf("SC_STATE: %d, h_start_DT: %d %s, h_finish_DT: %d %s\n",SC_STATE,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT);//trace_flush();
//trace_printf("h_res_id: %s, h_available: %d\n",h_res_id,h_available);//trace_flush();

        if ( h_available > 0 )
        {
            BEGIN_SQL
                INSERT INTO session.PMO_TMP_FIXDUR
                VALUES (SC_NULL, HOST(h_res_id), HOST(h_res_type), SC_NULL, HOST(h_start_DT), 0, HOST(h_available), 0, 0, 0, 0);
            END_SQL;

            InitializeHost(SC_PARAM(h_temp_var), 0);
            InitializeHost(SC_PARAM(h_total_h), 0.00);
            InitializeHost(SC_PARAM(h_new_res_rem), 0);
            InitializeHost(SC_PARAM(h_new_units), 0);
            double ratio_rem  = 0.00;

//trace_printf("h_work_todate: %d, h_remaining: %d\n",h_work_todate,h_remaining);//trace_flush();
            if ( h_work_todate == 0 )
            {
                h_minutes     = int(((double) h_remaining / (double) h_available) + 0.4);
                h_sum_res_rem = h_minutes * h_duration;
                h_new_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
//trace_printf("1.1*** h_minutes: %d, h_new_units: %d, h_sum_res_rem: %d\n",h_minutes,h_new_units,h_sum_res_rem);//trace_flush();
            }
            else
            {
                h_new_task_rem = h_remaining + h_available;
                ratio_rem = ((double) h_remaining / (double) h_new_task_rem);
//trace_printf("1.2*** ratio_rem: %.4f, h_new_task_rem: %d\n",ratio_rem,h_new_task_rem);//trace_flush();
            }

            BEGIN_SQL
                DECLARE C_NEW_REM CURSOR WITH HOLD FOR
                SELECT TSK_ASSIGNMENT_ID, COALESCE(RESUME_DT,UNRESOLVED_START), OLD_REMAINING, DURATION
                FROM session.PMO_TMP_FIXDUR WHERE RESOURCE_ID <> HOST(h_res_id)
                FOR READ ONLY;
            END_SQL_NOCHECK;

            BEGIN_SQL
                OPEN C_NEW_REM ;
            END_SQL;

            while(1==1)
            {
                BEGIN_SQL
                    FETCH C_NEW_REM INTO HOST(h_tsk_assignment_id), HOST(h_ta_start_DT), HOST(h_old_res_rem), HOST(h_duration);
                END_SQL;
                if (!(SC_SUCCESS)) break;
//trace_printf("h_tsk_assignment_id: %s, h_old_res_rem: %d, h_duration: %d\n",h_tsk_assignment_id,h_old_res_rem,h_duration);//trace_flush();

                if ( h_work_todate > 0 )
                {
                    h_new_res_rem = (double) h_old_res_rem * ratio_rem;

                    h_new_res_rem += h_etc_diff;

                    h_minutes     = int(((double) h_new_res_rem / (double) h_duration) + 0.4);
                    h_temp_var = h_minutes % 5;
                    if (h_temp_var != 0 )
                    {
                        if (h_temp_var < 3 )
                        {
                            h_minutes += h_temp_var;
                        }
                        else
                        {
                            h_minutes -= 5 - h_temp_var;
                        }
                    }

                    h_new_units    = ((double) h_minutes / (double) h_per_day) * 10000.00;
                    h_temp_var     = h_minutes * h_duration;
                    h_new_res_rem  = h_temp_var;
                    h_sum_res_rem += h_temp_var;
                    h_etc_diff     = h_new_res_rem - h_temp_var;
//trace_printf("h_new_units: %d, h_new_res_rem: %d, h_etc_diff: %d\n",h_new_units,h_new_res_rem,h_etc_diff);//trace_flush();
                }
                else
                {
                    h_new_res_rem = h_minutes * h_duration;
                }

                h_sum_units   += h_new_units;

                h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                          SC_PARAM(h_tsk_assignment_id),
                                          SC_PARAM(h_ta_start_DT),
                                          SC_PARAM(h_per_day),
                                          SC_PARAM(h_minutes));

                // fix assign string
                h_temp_var = h_new_units + SC_BIT_30;
                h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day, h_new_res_rem);
            }

            BEGIN_SQL
                CLOSE C_NEW_REM WITH RELEASE;
            END_SQL;
//trace_printf("h_new_units: %d, h_tsk_units: %d, h_sum_units: %d\n",h_new_units,h_tsk_units,h_sum_units);//trace_flush();
//trace_printf("h_etc_diff: %d, h_sum_res_rem: %d\n",h_etc_diff,h_sum_res_rem);//trace_flush();

            if ( h_work_todate > 0 )
            {
                h_new_units   = h_tsk_units - h_sum_units;
            }

            h_new_res_rem = h_remaining - h_sum_res_rem;

            BEGIN_SQL
                UPDATE session.PMO_TMP_FIXDUR SET
                    NEW_REMAINING = HOST(h_new_res_rem),
                    NEW_UNITS = HOST(h_new_units)
                WHERE RESOURCE_ID = HOST(h_res_id);
            END_SQL;
//trace_printf("***END_0: SC_STATE: %d, h_new_res_rem: %d, h_new_units: %d\n",SC_STATE,h_new_res_rem,h_new_units);//trace_flush();
        }
    }
    else
    {
        BEGIN_SQL
            SELECT SUM(OLD_UNITS) INTO HOST(h_tsk_units)
            FROM session.PMO_TMP_FIXDUR;
        END_SQL;
//trace_printf("SC_STATE: %d, h_tsk_units: %d\n",SC_STATE,h_tsk_units);//trace_flush();

        BEGIN_SQL
            SELECT OLD_REMAINING, WORK_TODATE INTO HOST(h_temp_var), HOST(h_work_todate)
            FROM session.PMO_TMP_FIXDUR
            WHERE RESOURCE_ID = HOST(h_res_id);
        END_SQL;
//trace_printf("SC_STATE: %d, h_res_id: %s, h_temp_var: %d, h_available: %d\n",SC_STATE,h_res_id,h_temp_var,h_available);//trace_flush();

        h_new_task_rem = h_remaining - h_temp_var;
        h_available = h_work_todate + h_temp_var;
//trace_printf("h_new_task_rem: %d, h_remaining: %d\n",h_new_task_rem,h_remaining);//trace_flush();

        if (SC_NOSUCCESS) return SC_STATE;

        BEGIN_SQL
            DECLARE C_DEL_RES CURSOR WITH HOLD FOR
            SELECT TSK_ASSIGNMENT_ID, COALESCE(RESUME_DT,UNRESOLVED_START), OLD_REMAINING
            FROM session.PMO_TMP_FIXDUR
            WHERE RESOURCE_ID <> HOST(h_res_id)
            FOR READ ONLY;
        END_SQL_NOCHECK;

        BEGIN_SQL
            OPEN C_DEL_RES ;
        END_SQL;

        while(1==1)
        {
            BEGIN_SQL
                FETCH C_DEL_RES INTO HOST(h_tsk_assignment_id), HOST(h_ta_start_DT), HOST(h_old_res_rem);
            END_SQL;
            if (!(SC_SUCCESS)) break;
//trace_printf("h_tsk_assignment_id: %s, h_old_res_rem: %d\n",h_tsk_assignment_id,h_old_res_rem);//trace_flush();

            BEGIN_SQL
                SELECT COUNT(*) INTO HOST(h_duration)
                FROM TASK_DAY_OF_YEAR
                WHERE PROJECT_ID = HOST(h_project_id) 
					AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND ACTUAL_FLAG = 0;
            END_SQL;
//trace_printf("SC_STATE: %d, h_duration: %d, h_etc_diff: %d\n",SC_STATE,h_duration,h_etc_diff);//trace_flush();

            if (h_cnt == 2)
            {
                h_new_res_rem   = h_remaining + h_work_todate - h_sum_res_rem;

                //h_new_res_rem = h_remaining - h_sum_res_rem;
                //h_new_units       = h_tsk_units - h_sum_units;
                h_minutes       = int(((double) h_new_res_rem / (double) h_duration) + 0.4);
            }
            else
            {
                h_total_h       = (double) h_old_res_rem / (double) h_new_task_rem;
                h_new_res_rem   = h_old_res_rem + h_etc_diff + (h_available * h_total_h);
                h_minutes       = int(((double) h_new_res_rem / (double) h_duration) + 0.4);
                h_temp_var = h_minutes % 5;
                if (h_temp_var != 0 )
                {
                    if (h_temp_var < 3 )
                    {
                        h_minutes += h_temp_var;
                    }
                    else
                    {
                        h_minutes -= 5 - h_temp_var;
                    }
                }
            }

            h_new_units     = ((double) h_minutes / (double) h_per_day) * 10000.00;
//trace_printf("h_total_h: %.4f, h_new_units: %d, h_new_res_rem: %d\n",h_total_h,h_new_units,h_new_res_rem);//trace_flush();
//trace_printf("h_cnt: %d, h_sum_res_rem: %d, h_sum_units: %d\n",h_cnt,h_sum_res_rem,h_sum_units);//trace_flush();

            h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                      SC_PARAM(h_tsk_assignment_id),
                                      SC_PARAM(h_ta_start_DT),
                                      SC_PARAM(h_per_day),
                                      SC_PARAM(h_minutes));

            if ( h_cnt > 2 )
            {
                h_temp_var = h_minutes * h_duration;
                h_etc_diff = h_new_res_rem - h_temp_var;

                h_new_res_rem  = h_temp_var;
                h_sum_res_rem += h_temp_var;
                h_sum_units   += h_new_units;
                h_cnt --;
            }

            // fix assign string
            h_temp_var = h_new_units + SC_BIT_30;
            h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var), h_per_day, h_new_res_rem);
        }

        BEGIN_SQL
            CLOSE C_DEL_RES WITH RELEASE;
        END_SQL;
//trace_printf("***END_1: SC_STATE: %d, h_new_res_rem: %d, h_new_units: %d\n",SC_STATE,h_new_res_rem,h_new_units);//trace_flush();
    }

    PMO_END_FUNCTION;
}
*/

/*******************************************************************************
********************************************************************************/

/*
BOOL_RETURN_CODE FC_TPHASE_SCHED_ASSIGN
(
    SC_CHAR_PARAM   h_project_id[SC_H_ID],
    short           h_project_id_Ind,
    SC_CHAR_PARAM   h_task_id[SC_H_ID],
    short           h_task_id_Ind,
    SC_CHAR_PARAM   h_tsk_assignment_id[SC_H_ID],
    short           h_tsk_assignment_id_Ind,
    SC_CHAR_PARAM   h_start_DT[SC_H_TIMESTAMP],
    short           h_start_DT_Ind,
    SC_CHAR_PARAM   h_finish_DT[SC_H_TIMESTAMP],
    short           h_finish_DT_Ind,
    SC_DOUBLE_PARAM h_reg_cost,
    short           h_reg_cost_Ind,
    SC_DOUBLE_PARAM h_reg_sale,
    short           h_reg_sale_Ind,
    SC_LONG_PARAM   h_update,
    short           h_update_Ind,
    SC_LONG_PARAM   h_capital_expense,
    short           h_capital_expense_Ind,
    SC_LONG_PARAM   h_billable,
    short           h_billable_Ind
)
{
    PMO_BEGIN_FUNCTION(FC_TPHASE_SCHED_ASSIGN);

    SC_CHAR(h_cc_id, SC_H_ID);
    SC_CHAR(h_FD, SC_H_DATE);
    SC_CHAR(h_SD, SC_H_DATE);
    SC_CHAR(h_pa_timephase, SC_H_FLAG);
    SC_LONG(h_type);
    SC_LONG(h_cur_year);
    SC_LONG(h_end_year);
    SC_LONG(h_start_month);
    SC_LONG(h_end_month);
    SC_LONG(h_budget_type);
    SC_LONG(h_rollup_bits);
    SC_DOUBLE(h_01);
    SC_DOUBLE(h_02);
    SC_DOUBLE(h_03);
    SC_DOUBLE(h_04);
    SC_DOUBLE(h_05);
    SC_DOUBLE(h_06);
    SC_DOUBLE(h_07);
    SC_DOUBLE(h_08);
    SC_DOUBLE(h_09);
    SC_DOUBLE(h_10);
    SC_DOUBLE(h_11);
    SC_DOUBLE(h_12);
    SC_DOUBLE(h_total_h);
    SC_DOUBLE(h_d_total);
    SC_DOUBLE(h_d_01);
    SC_DOUBLE(h_d_02);
    SC_DOUBLE(h_d_03);
    SC_DOUBLE(h_d_04);
    SC_DOUBLE(h_d_05);
    SC_DOUBLE(h_d_06);
    SC_DOUBLE(h_d_07);
    SC_DOUBLE(h_d_08);
    SC_DOUBLE(h_d_09);
    SC_DOUBLE(h_d_10);
    SC_DOUBLE(h_d_11);
    SC_DOUBLE(h_d_12);
    SC_LONG(h_temp_sc_state);
    PmoString stmt(8000);

    InitializeHost(SC_PARAM(h_SD), SC_PARAM(h_start_DT), 10);
    InitializeHost(SC_PARAM(h_FD), SC_PARAM(h_finish_DT), 10);
//trace_printf("h_update: %d, h_billable: %d, h_capital_expense: %d, h_reg_cost: %.2f, h_reg_sale: %.2f, h_SD: %s, h_FD: %s\n",h_update,h_billable,h_capital_expense,h_reg_cost,h_reg_sale,h_SD,h_FD);
//trace_printf("h_project_id: %d:%s, h_task_id: %d:%s\n",h_project_id_Ind,h_project_id,h_task_id_Ind,h_task_id);

    if ( (strlen(h_start_DT) == 0) || (IsNullParam(h_start_DT)) || (strlen(h_finish_DT) == 0) || (IsNullParam(h_finish_DT)))
    {
        return 0;
    }

    int l_actual_flag = 0;
    if (h_billable >= SC_BIT_30)
    {
        l_actual_flag = 1;
        h_billable = (h_billable  SC_BIT_NOT_30);          // remove actual flag
    }

    InitializeHost(SC_PARAM(h_rollup_bits), SC_SCHED_DELTA_BIT);
    if ( strcmp(h_project_id,h_task_id) == 0 )
    {
        BEGIN_SQL
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO HOST(h_pa_timephase) FROM PROJECT_DETAILS WHERE PROJECT_ID = HOST(h_project_id);
        END_SQL_NOCHECK;
//trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);

        if ( checkFlagStatus(h_pa_timephase, 'Y') )
        {
            InitializeHost(SC_PARAM(h_budget_type), SC_ESTIMATED_BUDGET);
        }
        else
        {
            return 0;
        }
    }
    else if (l_actual_flag == 1 )
    {
        InitializeHost(SC_PARAM(h_budget_type), SC_ACTUAL_BUDGET);
//      h_rollup_bits = h_rollup_bits | SC_ACTUAL_DELTA_BIT;
        InitializeHost(SC_PARAM(h_rollup_bits), SC_ACTUAL_DELTA_BIT);
    }
    else
    {
        InitializeHost(SC_PARAM(h_budget_type), SC_SCHED_BUDGET);
    }

    h_temp_sc_state = GET_FISCAL_DATE(h_SD, h_cur_year, h_start_month);
    h_temp_sc_state = GET_FISCAL_DATE(h_FD, h_end_year, h_end_month);

//trace_printf("h_tsk_assignment_id: %s, l_actual_flag: %d, h_cur_year: %d, h_end_year: %d, h_budget_type: %d\n", h_tsk_assignment_id,l_actual_flag, h_cur_year, h_end_year, h_budget_type);

    stmt.Clear();
    if ( h_budget_type == SC_ESTIMATED_BUDGET )
    {
        stmt.sprintf("SELECT FY.FISCAL_YEAR, "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))), "
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),"
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),"
        "SUM(tdy.SCHED_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12)))) "
        "FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy "
        "ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE "
        "WHERE tdy.PROJECT_ID = '%s' AND tdy.PARENT_ID = '%s' "
        "GROUP BY FY.FISCAL_YEAR FOR READ ONLY", h_project_id, h_tsk_assignment_id );
    }
    else if (l_actual_flag == 1 )
    {
        stmt.sprintf("SELECT FY.FISCAL_YEAR, "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))), "
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),"
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),"
        "SUM(tdy.ACTUAL_MIN * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12)))) "
        "FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy "
        "ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE "
        "WHERE tdy.PROJECT_ID = '%s' AND tdy.ACTUAL_FLAG = 1 AND tdy.TSK_ASSIGNMENT_ID = '%s' "
        "GROUP BY FY.FISCAL_YEAR FOR READ ONLY", h_project_id, h_tsk_assignment_id );
    }
    else
    {
        stmt.sprintf("SELECT FY.FISCAL_YEAR, "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))), "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))), "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))), "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))),  "
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),"
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),"
        "SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12))))  "
        "FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy "
        "ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE "
        "WHERE tdy.PROJECT_ID = '%s' AND tdy.TSK_ASSIGNMENT_ID = '%s' "
        "GROUP BY FY.FISCAL_YEAR FOR READ ONLY", h_project_id, h_tsk_assignment_id );
    }
//trace_printf("%s\n",(char *) stmt); //trace_flush();

    BEGIN_SQL
        PREPARE STFC1 FROM :stmt;
    END_SQL;

    BEGIN_SQL
        DECLARE CTPH01 CURSOR WITH HOLD FOR STFC1;
    END_SQL_NOCHECK;

    BEGIN_SQL
    OPEN CTPH01;
    END_SQL;

    while (1 == 1)
    {
        BEGIN_SQL
            FETCH CTPH01 INTO HOST(h_cur_year), HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06), HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12);
        END_SQL;
//trace_printf("FETCH: SC_STATE: %d, h_cur_year: %d, h_01: %d:%.2f\n", SC_STATE,h_cur_year,h_01_Ind,h_01);

        if ( SC_NOSUCCESS || (IsNullParam(h_01))) break;

        if ( h_update < 0 )
        {
            h_01 = -(h_01 / 60.00);
            h_02 = -(h_02 / 60.00);
            h_03 = -(h_03 / 60.00);
            h_04 = -(h_04 / 60.00);
            h_05 = -(h_05 / 60.00);
            h_06 = -(h_06 / 60.00);
            h_07 = -(h_07 / 60.00);
            h_08 = -(h_08 / 60.00);
            h_09 = -(h_09 / 60.00);
            h_10 = -(h_10 / 60.00);
            h_11 = -(h_11 / 60.00);
            h_12 = -(h_12 / 60.00);
        }
        else
        {
            h_01 = (h_01 / 60.00);
            h_02 = (h_02 / 60.00);
            h_03 = (h_03 / 60.00);
            h_04 = (h_04 / 60.00);
            h_05 = (h_05 / 60.00);
            h_06 = (h_06 / 60.00);
            h_07 = (h_07 / 60.00);
            h_08 = (h_08 / 60.00);
            h_09 = (h_09 / 60.00);
            h_10 = (h_10 / 60.00);
            h_11 = (h_11 / 60.00);
            h_12 = (h_12 / 60.00);
        }

        h_total_h = h_01 + h_02 + h_03 + h_04 + h_05 + h_06 + h_07 + h_08 + h_09 + h_10 + h_11 + h_12;
//trace_printf("h_total_h: %.2f, h_01: %.2f, h_02: %.2f, h_03: %.2f, h_04: %.2f, h_05: %.2f, h_06: %.2f, h_07: %.2f, h_08: %.2f, h_09: %.2f, h_10: %.2f, h_11: %.2f, h_12: %.2f\n", h_total_h,h_01,h_02,h_03,h_04,h_05,h_06,h_07,h_08,h_09,h_10,h_12,h_12);

        if ( h_total_h != 0 )
        {
            if ( h_capital_expense == 0 )
            {
                InitializeHost(SC_PARAM(h_cc_id), "EXPENSE_HOURS");
            }
            else
            {
                InitializeHost(SC_PARAM(h_cc_id), "CAPITAL_HOURS");
            }

            BEGIN_SQL
                INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                    MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                    MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_total_h),
                    HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06),
                    HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12));
            END_SQL_NOCHECK;
//trace_printf("EXPENSE_CAPITAL_HOURS: SC_STATE: %d, h_cc_id: %s, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cc_id,h_cur_year,h_total_h);

            if (l_actual_flag == 1 )
            {
                BEGIN_SQL
                    INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                        MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                        MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                    VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), -1 * HOST(h_total_h),
                        -1 * HOST(h_01), -1 * HOST(h_02), -1 * HOST(h_03), -1 * HOST(h_04), -1 * HOST(h_05), -1 * HOST(h_06),
                        -1 * HOST(h_07), -1 * HOST(h_08), -1 * HOST(h_09), -1 * HOST(h_10), -1 * HOST(h_11), -1 * HOST(h_12));
                END_SQL_NOCHECK;
            }

            BEGIN_SQL
                UPDATE BUDGETS
                SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                    AND CHARGECODE_ID = HOST(h_cc_id);
            END_SQL;

            if ( h_reg_cost != 0.00 )
            {
                if ( h_capital_expense == 0 )
                {
                    InitializeHost(SC_PARAM(h_cc_id), "EXPENSE_LABOR");
                }
                else
                {
                    InitializeHost(SC_PARAM(h_cc_id), "CAPITAL_LABOR");
                }

                h_d_01 = h_01 * h_reg_cost;
                h_d_02 = h_02 * h_reg_cost;
                h_d_03 = h_03 * h_reg_cost;
                h_d_04 = h_04 * h_reg_cost;
                h_d_05 = h_05 * h_reg_cost;
                h_d_06 = h_06 * h_reg_cost;
                h_d_07 = h_07 * h_reg_cost;
                h_d_08 = h_08 * h_reg_cost;
                h_d_09 = h_09 * h_reg_cost;
                h_d_10 = h_10 * h_reg_cost;
                h_d_11 = h_11 * h_reg_cost;
                h_d_12 = h_12 * h_reg_cost;

                h_d_total = h_d_01 + h_d_02 + h_d_03 + h_d_04 + h_d_05 + h_d_06 + h_d_07 + h_d_08 + h_d_09 + h_d_10 + h_d_11 + h_d_12;

                BEGIN_SQL
                    INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                        MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                        MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                    VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_d_total),
                        HOST(h_d_01), HOST(h_d_02), HOST(h_d_03), HOST(h_d_04), HOST(h_d_05), HOST(h_d_06),
                        HOST(h_d_07), HOST(h_d_08), HOST(h_d_09), HOST(h_d_10), HOST(h_d_11), HOST(h_d_12));
                END_SQL_NOCHECK;
//trace_printf("LABOR_COST: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);

                if (l_actual_flag == 1 )
                {
                    BEGIN_SQL
                        INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                        VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), -1 * HOST(h_d_total),
                            -1 * HOST(h_d_01), -1 * HOST(h_d_02), -1 * HOST(h_d_03), -1 * HOST(h_d_04), -1 * HOST(h_d_05), -1 * HOST(h_d_06),
                            -1 * HOST(h_d_07), -1 * HOST(h_d_08), -1 * HOST(h_d_09), -1 * HOST(h_d_10), -1 * HOST(h_d_11), -1 * HOST(h_d_12));
                    END_SQL_NOCHECK;
                }

                BEGIN_SQL
                    UPDATE BUDGETS
                    SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                    WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                        AND CHARGECODE_ID = HOST(h_cc_id);
                END_SQL;
            }

            if ( h_billable == 1 )
            {
                InitializeHost(SC_PARAM(h_cc_id), "BENEFIT_HOURS");

                BEGIN_SQL
                    INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                        MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                        MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                    VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_total_h),
                    HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06),
                    HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12));
                END_SQL_NOCHECK;
//trace_printf("BILLABLE_HOURS: SC_STATE: %d, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cur_year,h_total_h);

                if (l_actual_flag == 1 )
                {
                    BEGIN_SQL
                        INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                        VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), -1 * HOST(h_total_h),
                        -1 * HOST(h_01), -1 * HOST(h_02), -1 * HOST(h_03), -1 * HOST(h_04), -1 * HOST(h_05), -1 * HOST(h_06),
                        -1 * HOST(h_07), -1 * HOST(h_08), -1 * HOST(h_09), -1 * HOST(h_10), -1 * HOST(h_11), -1 * HOST(h_12));
                    END_SQL_NOCHECK;
                }

                BEGIN_SQL
                    UPDATE BUDGETS
                    SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                    WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                        AND CHARGECODE_ID = HOST(h_cc_id);
                END_SQL;

                if ( h_reg_sale != 0.00 )
                {
                    h_d_01 = h_01 * h_reg_sale;
                    h_d_02 = h_02 * h_reg_sale;
                    h_d_03 = h_03 * h_reg_sale;
                    h_d_04 = h_04 * h_reg_sale;
                    h_d_05 = h_05 * h_reg_sale;
                    h_d_06 = h_06 * h_reg_sale;
                    h_d_07 = h_07 * h_reg_sale;
                    h_d_08 = h_08 * h_reg_sale;
                    h_d_09 = h_09 * h_reg_sale;
                    h_d_10 = h_10 * h_reg_sale;
                    h_d_11 = h_11 * h_reg_sale;
                    h_d_12 = h_12 * h_reg_sale;

                    h_d_total = h_d_01 + h_d_02 + h_d_03 + h_d_04 + h_d_05 + h_d_06 + h_d_07 + h_d_08 + h_d_09 + h_d_10 + h_d_11 + h_d_12;

                    InitializeHost(SC_PARAM(h_cc_id), "BILLABLE_BENEFIT_LABOR");
                    BEGIN_SQL
                        INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                        VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_d_total),
                        HOST(h_d_01), HOST(h_d_02), HOST(h_d_03), HOST(h_d_04), HOST(h_d_05), HOST(h_d_06),
                        HOST(h_d_07), HOST(h_d_08), HOST(h_d_09), HOST(h_d_10), HOST(h_d_11), HOST(h_d_12));
                    END_SQL_NOCHECK;
//trace_printf("LABOR_REVENU: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);

                    if (l_actual_flag == 1 )
                    {
                        BEGIN_SQL
                            INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                                MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                                MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                            VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), -1 * HOST(h_d_total),
                            -1 * HOST(h_d_01), -1 * HOST(h_d_02), -1 * HOST(h_d_03), -1 * HOST(h_d_04), -1 * HOST(h_d_05), -1 * HOST(h_d_06),
                            -1 * HOST(h_d_07), -1 * HOST(h_d_08), -1 * HOST(h_d_09), -1 * HOST(h_d_10), -1 * HOST(h_d_11), -1 * HOST(h_d_12));
                        END_SQL_NOCHECK;
                    }

                    BEGIN_SQL
                        UPDATE BUDGETS
                        SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                        WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                            AND CHARGECODE_ID = HOST(h_cc_id);
                    END_SQL;
                }
            }
        }
    }
    BEGIN_SQL
        CLOSE CTPH01 WITH RELEASE;
    END_SQL;

    BEGIN_SQL
        UPDATE TMT_WBS SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits)) WHERE ELEMENT_ID = HOST(h_task_id) ;
    END_SQL;
//trace_printf("TMT_WBS: SC_STATE: %d, h_rollup_bits: %d\n", SC_STATE,h_rollup_bits);

    if( SC_NODATA)
    {
        BEGIN_SQL
            UPDATE TMT_CRI SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits)) WHERE ELEMENT_ID = HOST(h_task_id) ;
        END_SQL;
    }

    PMO_END_FUNCTION;
}
*/

/*******************************************************************************
********************************************************************************/

/*
BOOL_RETURN_CODE FC_TPHASE_FD_SCHED
(
    SC_CHAR_PARAM   h_project_id[SC_H_ID],
    short           h_project_id_Ind,
    SC_CHAR_PARAM   h_task_id[SC_H_ID],
    short           h_task_id_Ind,
    SC_LONG_PARAM   h_update,
    short           h_update_Ind
)
{
    PMO_BEGIN_FUNCTION(FC_TPHASE_FD_SCHED);

    SC_CHAR(h_tsk_assignment_id, SC_H_ID);
    SC_CHAR(h_cc_id, SC_H_ID);
    SC_LONG(h_type);
    SC_LONG(h_cur_year);
    SC_LONG(h_end_year);
    SC_LONG(h_start_month);
    SC_LONG(h_end_month);
    SC_LONG(h_budget_type);
    SC_LONG(h_rollup_bits);
    SC_LONG(h_capital_expense);
    SC_LONG(h_billable);
    SC_DOUBLE(h_reg_cost);
    SC_DOUBLE(h_reg_sale);
    SC_DOUBLE(h_01);
    SC_DOUBLE(h_02);
    SC_DOUBLE(h_03);
    SC_DOUBLE(h_04);
    SC_DOUBLE(h_05);
    SC_DOUBLE(h_06);
    SC_DOUBLE(h_07);
    SC_DOUBLE(h_08);
    SC_DOUBLE(h_09);
    SC_DOUBLE(h_10);
    SC_DOUBLE(h_11);
    SC_DOUBLE(h_12);
    SC_DOUBLE(h_total_h);
    SC_DOUBLE(h_d_total);
    SC_DOUBLE(h_d_01);
    SC_DOUBLE(h_d_02);
    SC_DOUBLE(h_d_03);
    SC_DOUBLE(h_d_04);
    SC_DOUBLE(h_d_05);
    SC_DOUBLE(h_d_06);
    SC_DOUBLE(h_d_07);
    SC_DOUBLE(h_d_08);
    SC_DOUBLE(h_d_09);
    SC_DOUBLE(h_d_10);
    SC_DOUBLE(h_d_11);
    SC_DOUBLE(h_d_12);
    SC_LONG(h_temp_sc_state);
    PmoString stmt(8000);

//trace_printf("h_update: %d, h_project_id: %d:%s, h_task_id: %d:%s\n",h_update,h_project_id_Ind,h_project_id,h_task_id_Ind,h_task_id);

    InitializeHost(SC_PARAM(h_rollup_bits), SC_SCHED_DELTA_BIT);
    InitializeHost(SC_PARAM(h_budget_type), SC_SCHED_BUDGET);

    BEGIN_SQL
        DECLARE C_FD_TA CURSOR WITH HOLD FOR
        SELECT TSK_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
        FROM TASK_ASSIGNMENTS
        WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
        FOR READ ONLY;
    END_SQL_NOCHECK;

    BEGIN_SQL
        OPEN C_FD_TA ;
    END_SQL;

    while(1==1)
    {
        BEGIN_SQL
            FETCH C_FD_TA INTO HOST(h_tsk_assignment_id), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale);
        END_SQL;

//trace_printf("FETCH_TA: SC_STATE: %d, h_tsk_assignment_id: %d %s\n", SC_STATE,h_tsk_assignment_id_Ind,h_tsk_assignment_id);

        if ( SC_NOSUCCESS || (IsNullParam(h_01))) break;
        BEGIN_SQL
            DECLARE CTPHFD CURSOR WITH HOLD FOR SELECT FY.FISCAL_YEAR,
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 1)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 2)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 3)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 4)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 5)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 6)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 7)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 8)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 9)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 10)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 11)))),
            SUM(CASE WHEN tdy.ACTUAL_FLAG = 0 THEN tdy.SCHED_MIN ELSE tdy.SCHED_MIN - tdy.ACTUAL_MIN END * (1 - ABS(SIGN(fy.FISCAL_MONTH - 12))))
            FROM TASK_DAY_OF_YEAR tdy JOIN FISCAL_CALENDAR fy
            ON tdy.DAY_YEAR BETWEEN fy.START_DATE AND fy.FINISH_DATE
            WHERE tdy.PROJECT_ID = HOST(h_project_id) AND tdy.TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
            GROUP BY FY.FISCAL_YEAR FOR READ ONLY;
        END_SQL_NOCHECK;

        BEGIN_SQL
        OPEN CTPHFD;
        END_SQL;

        while (1 == 1)
        {
            BEGIN_SQL
                FETCH CTPHFD INTO HOST(h_cur_year), HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06), HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12);
            END_SQL;
//trace_printf("FETCH: SC_STATE: %d, h_cur_year: %d, h_01: %d:%.2f\n", SC_STATE,h_cur_year,h_01_Ind,h_01);

            if ( SC_NOSUCCESS || (IsNullParam(h_01))) break;

            if ( h_update < 0 )
            {
                h_01 = -(h_01 / 60.00);
                h_02 = -(h_02 / 60.00);
                h_03 = -(h_03 / 60.00);
                h_04 = -(h_04 / 60.00);
                h_05 = -(h_05 / 60.00);
                h_06 = -(h_06 / 60.00);
                h_07 = -(h_07 / 60.00);
                h_08 = -(h_08 / 60.00);
                h_09 = -(h_09 / 60.00);
                h_10 = -(h_10 / 60.00);
                h_11 = -(h_11 / 60.00);
                h_12 = -(h_12 / 60.00);
            }
            else
            {
                h_01 = (h_01 / 60.00);
                h_02 = (h_02 / 60.00);
                h_03 = (h_03 / 60.00);
                h_04 = (h_04 / 60.00);
                h_05 = (h_05 / 60.00);
                h_06 = (h_06 / 60.00);
                h_07 = (h_07 / 60.00);
                h_08 = (h_08 / 60.00);
                h_09 = (h_09 / 60.00);
                h_10 = (h_10 / 60.00);
                h_11 = (h_11 / 60.00);
                h_12 = (h_12 / 60.00);
            }

            h_total_h = h_01 + h_02 + h_03 + h_04 + h_05 + h_06 + h_07 + h_08 + h_09 + h_10 + h_11 + h_12;
//trace_printf("h_total_h: %.2f, h_01: %.2f, h_02: %.2f, h_03: %.2f, h_04: %.2f, h_05: %.2f, h_06: %.2f, h_07: %.2f, h_08: %.2f, h_09: %.2f, h_10: %.2f, h_11: %.2f, h_12: %.2f\n", h_total_h,h_01,h_02,h_03,h_04,h_05,h_06,h_07,h_08,h_09,h_10,h_12,h_12);

            if ( h_total_h != 0 )
            {
                if ( h_capital_expense == 0 )
                {
                    InitializeHost(SC_PARAM(h_cc_id), "EXPENSE_HOURS");
                }
                else
                {
                    InitializeHost(SC_PARAM(h_cc_id), "CAPITAL_HOURS");
                }

                BEGIN_SQL
                    INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                        MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                        MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                    VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_total_h),
                        HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06),
                        HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12));
                END_SQL_NOCHECK;
//trace_printf("EXPENSE_CAPITAL_HOURS: SC_STATE: %d, h_cc_id: %s, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cc_id,h_cur_year,h_total_h);

                BEGIN_SQL
                    UPDATE BUDGETS
                    SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                    WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                        AND CHARGECODE_ID = HOST(h_cc_id);
                END_SQL;

                if ( h_reg_cost != 0.00 )
                {
                    if ( h_capital_expense == 0 )
                    {
                        InitializeHost(SC_PARAM(h_cc_id), "EXPENSE_LABOR");
                    }
                    else
                    {
                        InitializeHost(SC_PARAM(h_cc_id), "CAPITAL_LABOR");
                    }

                    h_d_01 = h_01 * h_reg_cost;
                    h_d_02 = h_02 * h_reg_cost;
                    h_d_03 = h_03 * h_reg_cost;
                    h_d_04 = h_04 * h_reg_cost;
                    h_d_05 = h_05 * h_reg_cost;
                    h_d_06 = h_06 * h_reg_cost;
                    h_d_07 = h_07 * h_reg_cost;
                    h_d_08 = h_08 * h_reg_cost;
                    h_d_09 = h_09 * h_reg_cost;
                    h_d_10 = h_10 * h_reg_cost;
                    h_d_11 = h_11 * h_reg_cost;
                    h_d_12 = h_12 * h_reg_cost;

                    h_d_total = h_d_01 + h_d_02 + h_d_03 + h_d_04 + h_d_05 + h_d_06 + h_d_07 + h_d_08 + h_d_09 + h_d_10 + h_d_11 + h_d_12;

                    BEGIN_SQL
                        INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                        VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_d_total),
                            HOST(h_d_01), HOST(h_d_02), HOST(h_d_03), HOST(h_d_04), HOST(h_d_05), HOST(h_d_06),
                            HOST(h_d_07), HOST(h_d_08), HOST(h_d_09), HOST(h_d_10), HOST(h_d_11), HOST(h_d_12));
                    END_SQL_NOCHECK;
//trace_printf("LABOR_COST: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);

                    BEGIN_SQL
                        UPDATE BUDGETS
                        SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                        WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                            AND CHARGECODE_ID = HOST(h_cc_id);
                    END_SQL;
                }

                if ( h_billable == 1 )
                {
                    InitializeHost(SC_PARAM(h_cc_id), "BENEFIT_HOURS");

                    BEGIN_SQL
                        INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                            MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                            MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                        VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_total_h),
                        HOST(h_01), HOST(h_02), HOST(h_03), HOST(h_04), HOST(h_05), HOST(h_06),
                        HOST(h_07), HOST(h_08), HOST(h_09), HOST(h_10), HOST(h_11), HOST(h_12));
                    END_SQL_NOCHECK;
//trace_printf("BILLABLE_HOURS: SC_STATE: %d, h_cur_year: %d, h_total_h: %.2f\n", SC_STATE,h_cur_year,h_total_h);

                    BEGIN_SQL
                        UPDATE BUDGETS
                        SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                        WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                            AND CHARGECODE_ID = HOST(h_cc_id);
                    END_SQL;

                    if ( h_reg_sale != 0.00 )
                    {
                        h_d_01 = h_01 * h_reg_sale;
                        h_d_02 = h_02 * h_reg_sale;
                        h_d_03 = h_03 * h_reg_sale;
                        h_d_04 = h_04 * h_reg_sale;
                        h_d_05 = h_05 * h_reg_sale;
                        h_d_06 = h_06 * h_reg_sale;
                        h_d_07 = h_07 * h_reg_sale;
                        h_d_08 = h_08 * h_reg_sale;
                        h_d_09 = h_09 * h_reg_sale;
                        h_d_10 = h_10 * h_reg_sale;
                        h_d_11 = h_11 * h_reg_sale;
                        h_d_12 = h_12 * h_reg_sale;

                        h_d_total = h_d_01 + h_d_02 + h_d_03 + h_d_04 + h_d_05 + h_d_06 + h_d_07 + h_d_08 + h_d_09 + h_d_10 + h_d_11 + h_d_12;

                        InitializeHost(SC_PARAM(h_cc_id), "BILLABLE_BENEFIT_LABOR");
                        BEGIN_SQL
                            INSERT INTO TIMEPHASED_DELTA ( PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_YEAR, BUDGET_TYPE, TOTAL_AMOUNT,
                                MONTH_01_AMOUNT, MONTH_02_AMOUNT, MONTH_03_AMOUNT, MONTH_04_AMOUNT, MONTH_05_AMOUNT, MONTH_06_AMOUNT,
                                MONTH_07_AMOUNT, MONTH_08_AMOUNT, MONTH_09_AMOUNT, MONTH_10_AMOUNT, MONTH_11_AMOUNT, MONTH_12_AMOUNT)
                            VALUES(HOST(h_project_id), HOST(h_task_id), HOST(h_cc_id), HOST(h_cur_year), HOST(h_budget_type), HOST(h_d_total),
                            HOST(h_d_01), HOST(h_d_02), HOST(h_d_03), HOST(h_d_04), HOST(h_d_05), HOST(h_d_06),
                            HOST(h_d_07), HOST(h_d_08), HOST(h_d_09), HOST(h_d_10), HOST(h_d_11), HOST(h_d_12));
                        END_SQL_NOCHECK;
//trace_printf("LABOR_REVENU: SC_STATE: %d, h_cur_year: %d, h_d_total: %.2f\n", SC_STATE,h_cur_year,h_d_total);

                        BEGIN_SQL
                            UPDATE BUDGETS
                            SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits))
                            WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id)
                                AND CHARGECODE_ID = HOST(h_cc_id);
                        END_SQL;
                    }
                }
            }
        }

        BEGIN_SQL
            CLOSE CTPHFD WITH RELEASE;
        END_SQL;
    }

    BEGIN_SQL
        CLOSE C_FD_TA WITH RELEASE;
    END_SQL;

    BEGIN_SQL
        UPDATE TMT_WBS SET TIMEPHASED_BITS = BIT_OR(TIMEPHASED_BITS,HOST(h_rollup_bits)) WHERE ELEMENT_ID = HOST(h_task_id) ;
    END_SQL;
//trace_printf("TMT_WBS: SC_STATE: %d, h_rollup_bits: %d\n", SC_STATE,h_rollup_bits);

    PMO_END_FUNCTION;
}

*/

