/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * Author: Tudor Toma
 *
 **/

/****************************************************************************************************/

Prompt == FUNCTION FC_FIX_ASSIGN_STRING ==
CREATE OR REPLACE FUNCTION FC_FIX_ASSIGN_STRING
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , h_assign_string IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_start_D DATE;
   h_finish_D DATE;
   h_minutes PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR C_FACT IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, ACTUAL_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 1
         UNION ALL
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM PMO_TMP_TDOY
         ORDER BY 1;

      CURSOR C_FETC IS
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, CASE WHEN ACTUAL_MIN = 0 THEN SCHED_MIN ELSE ACTUAL_MIN END
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR BETWEEN h_start_D AND h_finish_D
         UNION ALL
         SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JDY, SCHED_MIN FROM PMO_TMP_TDOY
         ORDER BY 1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_ASSIGN_STRING') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_ASSIGN_STRING'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_ASSIGN_STRING(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || '''' || h_assign_string || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_ASSIGN_STRING', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_ASSIGN_STRING(48) : cplan_fun.sql(61) : ' || SYSTIMESTAMP);
   END IF; 

   null_return := FC_INI_ASSIGN_STR(1);
   h_assign_string := '';
   --trace_printf("**__1 h_bitflag: %d, h_start_DT: %s, h_finish_DT: %s\n", h_bitflag,h_start_DT,h_finish_DT);
   
   If h_bitflag = 1 Then
      OPEN C_FACT;
      
      LOOP
         FETCH C_FACT INTO h_julian_day, h_minutes;
         IF C_FACT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      END LOOP;
      
      BEGIN
         CLOSE C_FACT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_ASSIGN_STRING', 79+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_ASSIGN_STRING(80) : cplan_fun.sql(93) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_start_D := SubStr(h_start_DT, 1, 10);
      h_finish_D := SubStr(h_finish_DT, 1, 10);
      
      OPEN C_FETC;
      
      LOOP
         FETCH C_FETC INTO h_julian_day, h_minutes;
         IF C_FETC%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_minutes);
      END LOOP;
      
      BEGIN
         CLOSE C_FETC ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_ASSIGN_STRING', 111+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_ASSIGN_STRING(112) : cplan_fun.sql(125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_temp_sc_state, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_ASSIGN_STRING', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_ASSIGN_STRING(120) : cplan_fun.sql(133) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   PMO_DB2M.SC_STATE := FC_GET_ASSIGN_STR(h_assign_string, 0);
   --trace_printf("%s\n", h_assign_string);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_ASSIGN_STRING', 128+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_ASSIGN_STRING(129) : cplan_fun.sql(142) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_ASSIGN_STRING;
ALTER FUNCTION FC_FIX_ASSIGN_STRING COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_DEL_NOACTUAL ==
CREATE OR REPLACE FUNCTION FC_DEL_NOACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_prj_assignment_id VARCHAR2(32);
   h_minutes PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CDNA IS
         SELECT DAY_YEAR
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;

      CURSOR CPA0 IS
         SELECT DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DEL_NOACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DEL_NOACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_DEL_NOACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_NOACTUAL', 36+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DEL_NOACTUAL(37) : cplan_fun.sql(189) : ' || SYSTIMESTAMP);
   END IF; 

   --short h_tsk_assignment_id_Ind = 0;
   --short h_project_id_Ind = 0;
   
   If h_bitflag = 0 Then
      OPEN CDNA;
      
      LOOP
         FETCH CDNA INTO h_dayofyear;
         IF CDNA%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_temp_var := 0;
         h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
      END LOOP;
      
      BEGIN
         CLOSE CDNA ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 69+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(70) : cplan_fun.sql(222) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_bitflag = 1 Then
      BEGIN
         DELETE FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 89+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(90) : cplan_fun.sql(242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf h_bitflag = 2 Then
      BEGIN
         SELECT PROJECT_ASSIGNMENT_ID INTO h_prj_assignment_id
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 104+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(105) : cplan_fun.sql(257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- fix task assignments from affected to project
      
      OPEN CPA0;
      
      LOOP
         FETCH CPA0 INTO h_dayofyear, h_minutes, h_bitID;
         IF CPA0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, TASK_ID = h_project_id, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 137+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(138) : cplan_fun.sql(290) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_dayofyear: %d %s, h_minutes: %d\n",SC_STATE,h_dayofyear_Ind,h_dayofyear,h_minutes);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id, PARENT_ID = h_prj_assignment_id, TASK_ID = h_project_id
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 157+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(158) : cplan_fun.sql(310) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 175+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(176) : cplan_fun.sql(328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CPA0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_NOACTUAL', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_NOACTUAL(191) : cplan_fun.sql(343) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_NOACTUAL', 197+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_NOACTUAL(198) : cplan_fun.sql(350) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DEL_NOACTUAL;
ALTER FUNCTION FC_DEL_NOACTUAL COMPILE DEBUG;

--########################################################################

Prompt == FUNCTION FC_SRCH_DURATION ==
CREATE OR REPLACE FUNCTION FC_SRCH_DURATION
(
     sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_every PLS_INTEGER
   , sh_h_duration PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_unit PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_contour PLS_INTEGER := Coalesce(sh_h_contour, 0);
   h_every PLS_INTEGER := Coalesce(sh_h_every, 0);
   h_duration PLS_INTEGER := Coalesce(sh_h_duration, 0);
   h_occurrence PLS_INTEGER := Coalesce(sh_h_occurrence, 0);
   h_unit PLS_INTEGER := Coalesce(sh_h_unit, 0);
   h_temp_sc_state PLS_INTEGER := 0;
   j PLS_INTEGER;
   i_week PLS_INTEGER;
   i PLS_INTEGER;
   ii PLS_INTEGER;
   i_days PLS_INTEGER;
   i_every PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SRCH_DURATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SRCH_DURATION'), '-- ' || SYSTIMESTAMP, ':r := FC_SRCH_DURATION(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_every), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SRCH_DURATION', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SRCH_DURATION(36) : cplan_fun.sql(396) : ' || SYSTIMESTAMP);
   END IF; 

   If (h_bitflag = 4) OR (h_bitflag > 6) Then
      PMO_DB2M.SC_STATE := 250;
   Else
      i_week := 0;
      i := 0;
      ii := 0;
      i_days := 0;
      i_every := 0;
      
      If (BITAND(h_contour, PMO_CNST.SC_0000007F)) > 0 Then
         i := 1;
         ii := 7;
         i_days := 0;
         WHILE ii > 0 LOOP
            If (BITAND(h_contour, i)) = i Then
               i_days := i_days + 1;
            End If;
            i := PMO_FUNC.ShiftLeft(i, 1);
            ii := ii - 1;
         END LOOP;
      Else
         i_days := 1;
      End If;
      
      i_week := PMO_FUNC.ShiftRight((BITAND(h_contour, 1920)), 7);
      If i_week > 0 Then
         i := 0;
         j := 0;
         WHILE j < 4 LOOP
            If (BITAND(i_week, 1)) <> 0 Then
               i := i + 1;
            End If;
            
            i_week := (PMO_FUNC.ShiftRight(i_week, 1));
            If i_week = 0 Then
               EXIT;
            End If;
            
            j := j + 1;
         END LOOP;
         
         i_week := i;
      End If;
      
      If h_every > 0 Then
         If h_occurrence = 0 Then
            h_duration := h_duration + 1;
         End If;
         
         If h_bitflag = 1 Then
            h_duration := h_duration * h_every;
         ElsIf h_bitflag = 2 Then
            h_duration := h_duration * (Trunc((h_every * 7) / i_days));
         ElsIf h_bitflag < 6 Then
            If (h_unit = 2) OR (h_occurrence > 0) Then
               h_duration := 31 * (h_duration + h_every - 1);
            ElsIf i_week = 0 Then
               h_duration := Trunc(h_duration * ((h_every * 31) / (4.5 * i_days)));
            Else
               h_duration := h_duration * (Trunc((h_every * 31) / (i_week * i_days)));
            End If;
         End If;
      Else
         i_every := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_1FFE0000)), 17);
         -- July, August, Decembre
         
         If i_every > 0 Then
            i := 0;
            j := 0;
            WHILE j < 12 LOOP
               If (BITAND(i_every, 1)) <> 0 Then
                  i := i + 1;
               End If;
               
               i_every := (PMO_FUNC.ShiftRight(i_every, 1));
               
               If i_every = 0 Then
                  EXIT;
               End If;
               
               j := j + 1;
            END LOOP;
            
            i_every := 10 - j;
            
            If i > 0 Then
               If i_week = 0 Then
                  h_duration := Trunc(h_duration / (4.5 * i_days * i) + 1);
               Else
                  h_duration := Trunc(h_duration / (i_week * i_days * i)) + 1;
               End If;
            End If;
            h_duration := h_duration * 365;
            h_duration := h_duration - i_every * 30;
         End If;
      End If;
      
      PMO_DB2M.SC_STATE := h_duration;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SRCH_DURATION', 139+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SRCH_DURATION(140) : cplan_fun.sql(500) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SRCH_DURATION;
ALTER FUNCTION FC_SRCH_DURATION COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_GET_RES_RATES ==
CREATE OR REPLACE FUNCTION FC_GET_RES_RATES
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_and_roles IN OUT NOCOPY VARCHAR2
   , h_and_skills IN OUT NOCOPY VARCHAR2
   , data_flag IN OUT NOCOPY PLS_INTEGER
   , reg_cost IN OUT NOCOPY NUMBER
   , spec_cost IN OUT NOCOPY NUMBER
   , reg_sale IN OUT NOCOPY NUMBER
   , spec_sale IN OUT NOCOPY NUMBER
) RETURN PLS_INTEGER
AS
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   token VARCHAR2(32);
   numItems PLS_INTEGER;
   i PLS_INTEGER;
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_RES_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_RES_RATES'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_RES_RATES(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || '''' || h_and_roles || ''''
          || ', ' || '''' || h_and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(data_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_RES_RATES', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_RES_RATES(40) : cplan_fun.sql(550) : ' || SYSTIMESTAMP);
   END IF; 

   h_reg_cost := 0.00;
   h_reg_sale := 0.00;
   h_spec_cost := 0.00;
   h_spec_sale := 0.00;
   
   If (Coalesce(Length(h_and_roles), 0) >= 32 OR Coalesce(Length(h_and_skills), 0) >= 32) 
         AND (NOT SubStr(h_res_type, 1, 1) = '1') Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_RATE';
      PMO_DB2M.SC_STATE := 0;
      
      token := SubStr(token, 1, 32);
      
      numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
      
      stmt := '';
      
      stmt := 
            'INSERT INTO PMO_TMP_RES_RATE (RES_ID,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
             || ' SELECT ''' || h_res_id || ''',';
      stwhere := 
            ' CASE WHEN (R1.REGULAR_COST = 0 OR R1.REGULAR_COST IS NULL) AND (R.REGULAR_COST = 0 OR R.REGULAR_COST IS NULL) THEN 0 '
             || ' WHEN (R1.REGULAR_COST = 0 OR  R1.REGULAR_COST IS NULL) THEN R.REGULAR_COST '
             || ' WHEN (R.REGULAR_COST = 0  OR R.REGULAR_COST IS NULL) THEN R1.REGULAR_COST '
             || ' ELSE R1.REGULAR_COST END AS REGULAR_COST, '
             || ' CASE WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) AND (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL)  THEN 0 '
             || ' WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) THEN R.REGULAR_SALE '
             || ' WHEN (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL) THEN R1.REGULAR_SALE '
             || ' ELSE R1.REGULAR_SALE END AS REGULAR_SALE, '
             || ' CASE WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) AND (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL)  THEN 0 '
             || ' WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) THEN R.SPECIAL_COST '
             || ' WHEN (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL) THEN R1.SPECIAL_COST '
             || ' ELSE R1.SPECIAL_COST END AS SPECIAL_COST, '
             || ' CASE WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) AND (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL)  THEN 0 '
             || ' WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) THEN R.SPECIAL_SALE '
             || ' WHEN (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL) THEN R1.SPECIAL_SALE '
             || ' ELSE R1.SPECIAL_SALE END AS SPECIAL_SALE, '
             || ' t1.DEFAULT_FLAG FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
             || ' ( t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
             || '   AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         --          pmo_substr(token, h_and_roles, i*32, 32);
         token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_and_roles);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      
      --trace_printf("\n stmt roles------> %s", (char *) stmt);
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 116+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(117) : cplan_fun.sql(627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      --Parse Skills list
      If Coalesce(Length(h_and_skills), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
      End If;
      
      If numItems > 0 Then
         stmt := '';
         stmt := 
               'INSERT INTO PMO_TMP_RES_RATE (RES_ID,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                || ' SELECT ''' || h_res_id || ''', ';
         
         stwhere := 
               ' case when (r1.REGULAR_COST is null or r1.REGULAR_COST=0) then COALESCE(r.REGULAR_COST,0) else r1.REGULAR_COST end AS REGULAR_COST, '
                || ' case when (r1.REGULAR_SALE is null or r1.REGULAR_SALE=0) then COALESCE(r.REGULAR_SALE,0) else r1.REGULAR_SALE end AS REGULAR_SALE, '
                || ' case when (r1.SPECIAL_COST is null or r1.SPECIAL_COST=0) then COALESCE(r.SPECIAL_COST,0) else r1.SPECIAL_COST end AS SPECIAL_COST, '
                || ' case when (r1.SPECIAL_SALE is null or r1.SPECIAL_SALE=0) then COALESCE(r.SPECIAL_SALE,0) else r1.SPECIAL_SALE end AS SPECIAL_SALE, ''0'' '
                || ' FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
         stwhere := stwhere || h_res_id;
         stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                || ' (t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK  '
                || '  AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                || ' AND t3.ELEMENT_ID IN (''';
         
         i := 0;
         WHILE i < numItems LOOP
            If i > 0 Then
               stwhere := stwhere || ''',''';
            End If;
            
            --              pmo_substr(token, h_and_skills, i*32, 32);
            token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_and_skills);
            
            stwhere := stwhere || token;
            i := i + 1;
         END LOOP;
         stwhere := stwhere || ''')';
         stmt := stmt || stwhere;
         
         --trace_printf("\n stmt skills------> %s", (char *) stmt);
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 176+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(177) : cplan_fun.sql(687) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
      End If;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0006';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN_0006
         SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, 0
         FROM PMO_TMP_RES_RATE WHERE DEFAULT_FLAG = '1';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 201+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(202) : cplan_fun.sql(712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN_0006
         SELECT  RES_ID,                             MAX(REGULAR_COST) AS REGULAR_COST,
         MAX(REGULAR_SALE) AS REGULAR_SALE,  MAX(SPECIAL_COST) AS SPECIAL_COST,
         MAX(SPECIAL_SALE) AS SPECIAL_SALE,  0
         FROM PMO_TMP_RES_RATE
         WHERE RES_ID NOT IN (SELECT RES_ID
         FROM PMO_TMP_RES_RATE WHERE DEFAULT_FLAG = '1')
         GROUP BY RES_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 225+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(226) : cplan_fun.sql(736) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST
         WHEN a.REGULAR_COST>0 THEN a.REGULAR_COST
         else def.REGULAR_COST    END,2),
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.REGULAR_RATE
         WHEN a.REGULAR_SALE>0 THEN a.REGULAR_SALE
         else def.REGULAR_SALE   END,2),
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.SPECIAL_COST
         WHEN a.REGULAR_COST>0 THEN a.SPECIAL_COST
         else def.SPECIAL_COST  END,2),
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.SPECIAL_RATE
         WHEN a.REGULAR_SALE>0 THEN a.SPECIAL_SALE
         else def.SPECIAL_SALE  END,2)
         
         INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
         
         FROM PMO_TMP_CPLN_0006 def
         JOIN RESOURCES r ON def.RES_ID=r.RESOURCE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=def.RES_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 257+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(258) : cplan_fun.sql(768) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- project assignments level
      BEGIN
         SELECT REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
         INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
         FROM PROJECT_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 274+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(275) : cplan_fun.sql(785) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_reg_cost, 0) + Coalesce(
            h_reg_sale, 0) > 0.0) Then
         data_flag := 1;
      Else
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT COALESCE(MAX(R.REGULAR_COST),0.00), COALESCE(MAX(R.REGULAR_SALE),0.00),
               COALESCE(MAX(R.SPECIAL_COST),0.00), COALESCE(MAX(R.SPECIAL_SALE),0.00)
               INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
               FROM (PROFILES P JOIN TMT_COMPETENCIES C ON P.PROJECT_ID = h_project_id AND P.ROLE_ID = C.ELEMENT_ID)
               JOIN RATES R ON R.RATE_ID = C.RATE_ID
               WHERE P.PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 297+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(298) : cplan_fun.sql(808) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            -- get resource rate
            BEGIN
               SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
               INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
               FROM RESOURCES rs LEFT OUTER JOIN RATES r
               ON rs.RATE_ID=r.RATE_ID
               WHERE rs.RESOURCE_ID= h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 315+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(316) : cplan_fun.sql(826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                  h_spec_sale, 0)) = 0.00 Then
               -- get the default rate in resource level
               BEGIN
                  SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
                  INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TMT_COMPETENCIES t1
                  LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID
                  WHERE t1.RESOURCE_ID = h_res_id AND t1.DEFAULT_FLAG='1';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 335+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(336) : cplan_fun.sql(846) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                     h_spec_sale, 0)) = 0.00 Then
                  -- get the default value from parent level (copetency level)
                  BEGIN
                     SELECT COALESCE(r.REGULAR_COST,0.00), COALESCE(r.REGULAR_SALE,0.00), COALESCE(r.SPECIAL_COST,0.00), COALESCE(r.SPECIAL_SALE,0.00)
                     INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                     FROM TMT_COMPETENCIES t1
                     JOIN TMT_COMPETENCIES P ON P.ELEMENT_ID = T1.PARENT_ID
                     LEFT OUTER JOIN RATES r ON P.RATE_ID=r.RATE_ID
                     WHERE t1.RESOURCE_ID = h_res_id AND t1.DEFAULT_FLAG='1';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 356+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(357) : cplan_fun.sql(867) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                        h_spec_sale, 0)) = 0.00 Then
                     BEGIN
                        SELECT COALESCE(MAX(r.REGULAR_COST),0.00), COALESCE(MAX(r.REGULAR_SALE),0.00), COALESCE(MAX(r.SPECIAL_COST),0.00), COALESCE(MAX(r.SPECIAL_SALE),0.00)
                        INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                        FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID
                        WHERE t1.RESOURCE_ID = h_res_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 374+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(375) : cplan_fun.sql(885) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If (Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
                           h_spec_sale, 0)) = 0.00 Then
                        BEGIN
                           SELECT COALESCE(MAX(r.REGULAR_COST),0.00), COALESCE(MAX(r.REGULAR_SALE),0.00), COALESCE(MAX(r.SPECIAL_COST),0.00), COALESCE(MAX(r.SPECIAL_SALE),0.00)
                           INTO h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                           FROM TMT_COMPETENCIES t1 JOIN TMT_COMPETENCIES t2
                           ON t2.RESOURCE_ID = h_res_id AND t1.ELEMENT_ID=t2.PARENT_ID LEFT OUTER JOIN RATES r ON t1.RATE_ID=r.RATE_ID;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_RES_RATES', 392+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_RES_RATES(393) : cplan_fun.sql(903) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
                  
                  data_flag := 2;
               End If;
            End If;
         End If;
      End If;
   End If;
   
   reg_cost := Coalesce(h_reg_cost, 0);
   reg_sale := Coalesce(h_reg_sale, 0);
   spec_cost := Coalesce(h_spec_cost, 0);
   spec_sale := Coalesce(h_spec_sale, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_RES_RATES', 412+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_RES_RATES(413) : cplan_fun.sql(923) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_RES_RATES;
ALTER FUNCTION FC_GET_RES_RATES COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_CHECK_PA ==
CREATE OR REPLACE FUNCTION FC_CHECK_PA
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , sh_h_etc PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_etc PLS_INTEGER := sh_h_etc;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_total PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CHECK_PA') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CHECK_PA'), '-- ' || SYSTIMESTAMP, ':r := FC_CHECK_PA(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_etc), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CHECK_PA(27) : cplan_fun.sql(960) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_finish_date: %d %s, h_etc: %d\n",h_finish_date_Ind,h_finish_date,h_etc); trace_flush();
   h_total := 0;
   h_start_DT := NULL;
   h_finish_DT := NULL;
   
   BEGIN
      SELECT COALESCE(SCHED_START,PLANNED_START), COALESCE(SCHED_FINISH,PLANNED_FINISH) 
      INTO h_start_DT, h_finish_DT
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHECK_PA', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHECK_PA(46) : cplan_fun.sql(979) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_finish_date IS NOT NULL) Then
      If NOT ((PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) >= 0) AND ((PMO_ORASTD.strncmp(
            h_start_date, h_finish_DT, 10) <= 0) AND (PMO_ORASTD.strncmp(h_finish_date, h_finish_DT
            , 10) <= 0)) AND (PMO_ORASTD.strncmp(h_finish_date, h_start_DT, 10) >= 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 55+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(56) : cplan_fun.sql(989) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
   ElsIf (h_start_date IS NOT NULL) Then
      If NOT ((PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) >= 0) AND (PMO_ORASTD.strncmp(h_start_date
            , h_finish_DT, 10) <= 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 63+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(64) : cplan_fun.sql(997) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
      
      --if ((strncmp(h_start_date,h_start_DT,10) != 0)
      --{
      -- InitializeHost(SC_PARAM(h_start_date), h_start_DT);
      --}
   End If;
   
   BEGIN
      SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND (TSK_ASSIGNMENT_ID = h_prj_assignment_id OR TSK_ASSIGNMENT_ID = h_tsk_assignment_id);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CHECK_PA', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CHECK_PA(85) : cplan_fun.sql(1018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_total, 0) < Coalesce(h_etc, 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 91+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(92) : cplan_fun.sql(1025) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CHECK_PA', 97+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CHECK_PA(98) : cplan_fun.sql(1031) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CHECK_PA;
ALTER FUNCTION FC_CHECK_PA COMPILE DEBUG;

/*************************************************************************************************/

Prompt == FUNCTION FC_DECLARE_TDY ==
CREATE OR REPLACE FUNCTION FC_DECLARE_TDY RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DECLARE_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DECLARE_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_DECLARE_TDY();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_TDY', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DECLARE_TDY(9) : cplan_fun.sql(1050) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_TDY', 20+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DECLARE_TDY(21) : cplan_fun.sql(1062) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DECLARE_TDY;
ALTER FUNCTION FC_DECLARE_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_TDY ==
CREATE OR REPLACE FUNCTION FC_CREATE_TDY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_consider PLS_INTEGER
   , sh_h_cri PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_update PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_consider PLS_INTEGER := sh_h_consider;
   h_cri PLS_INTEGER := sh_h_cri;
   h_priority PLS_INTEGER := sh_h_priority;
   h_update PLS_INTEGER := sh_h_update;
   stmt VARCHAR2(7999);
   h_count PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_TDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_TDY'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_TDY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_TDY(32) : cplan_fun.sql(1108) : ' || SYSTIMESTAMP);
   END IF; 

   -- INDEX: see if can swap bit_flag and day_year columns in it.
   --CREATE INDEX TKDYR_RID_BF_XX ON TASK_DAY_OF_YEAR
   --        (RESOURCE_ID ASC,
   --         BIT_FLAG ASC,
   --      DAY_YEAR ASC);
   
   --timeLog (file,"1__ FC_CREATE_TDY:");
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   --timeLog (file,"2__ FC_CREATE_TDY:");//trace_flush();
   
   --trace_printf("h_cri: %d, h_consider: %d, h_update: %d, h_res_id: %s\n",h_cri,h_consider,h_update,h_res_id);//trace_flush();
   If Coalesce(h_consider, 0) = 0 OR Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_OVERALLOCATE Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 53+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY(54) : cplan_fun.sql(1130) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_cri, 0) >= PMO_CNST.SC_BIT_30 Then
      h_cri := (BITAND(Coalesce(h_cri, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove search / assign flag
   End If;
   --trace_printf("h_cri: %d, h_project_id: %s, h_start_date: %s\n",h_cri,h_project_id,h_start_date);//trace_flush();
   
   CASE h_consider
      WHEN 1 THEN
         -- Consider SCHED inside current project
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID=h_project_id
            AND RESOURCE_ID = h_res_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 85+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(86) : cplan_fun.sql(1162) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 2 THEN
         -- Consider SCHED outside current project
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id
            AND PROJECT_ID<>h_project_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 111+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(112) : cplan_fun.sql(1188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 3 THEN
         -- Consider SCHED all projects
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 136+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(137) : cplan_fun.sql(1213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 4 THEN
         -- Consider PLANNED  SCHED inside current project
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE y.PROJECT_ID = h_project_id
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 163+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(164) : cplan_fun.sql(1240) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 185+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(186) : cplan_fun.sql(1262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 5 THEN
         -- Consider PLANNED  SCHED outside current project
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE y.PROJECT_ID<>h_project_id
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 212+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(213) : cplan_fun.sql(1289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID<>h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 233+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(234) : cplan_fun.sql(1310) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 6 THEN
         -- Consider PLANNED  SCHED all projects
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 259+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(260) : cplan_fun.sql(1336) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 279+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(280) : cplan_fun.sql(1356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 13 THEN
         -- Consider SCHED outside current project if priority >= then me
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
            FROM TASK_DAY_OF_YEAR y, TMT_WBS w
            WHERE 0=0
            AND y.RESOURCE_ID = h_res_id
            AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND y.BIT_FLAG='2'
            AND y.PROJECT_ID<>h_project_id
            
            AND y.PROJECT_ID=w.ELEMENT_ID
            AND w.PRIORITY>=h_priority;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 310+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(311) : cplan_fun.sql(1387) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 14 THEN
         -- Consider SCHED outside current project if priority >= then me  SCHED inside
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
            FROM TASK_DAY_OF_YEAR y, TMT_WBS w
            WHERE 0=0
            AND y.RESOURCE_ID = h_res_id
            AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND y.BIT_FLAG='2'
            
            AND y.PROJECT_ID=w.ELEMENT_ID
            AND w.PRIORITY>=h_priority;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 339+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(340) : cplan_fun.sql(1416) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN 15 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TMT_WBS w, TASK_DAY_OF_YEAR y
               WHERE y.RESOURCE_ID=h_res_id
               AND y.PROJECT_ID<>h_project_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 367+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(368) : cplan_fun.sql(1444) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.PROJECT_ID<>h_project_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 393+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(394) : cplan_fun.sql(1470) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 16 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me  PLANNED  SCHED inside
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 422+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(423) : cplan_fun.sql(1499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 447+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(448) : cplan_fun.sql(1524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 17 THEN
         -- Consider PLANNED  SCHED outside current project if priority >= then me  SCHED inside current project
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND w.ELEMENT_ID <> h_project_id
               AND y.PROJECT_ID = w.ELEMENT_ID
               AND w.PRIORITY >= h_priority
               AND y.TABLE_TYPE = 1
               UNION ALL
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.PROJECT_ID = h_project_id
               AND y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG = '2'
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 487+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(488) : cplan_fun.sql(1564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.PROJECT_ID<>h_project_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority
               UNION ALL
               SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND BIT_FLAG='2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 520+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(521) : cplan_fun.sql(1597) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 22 THEN
         -- Consider scheduled all projects    planned outer current project
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id
            AND DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10))
            AND BIT_FLAG = '2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 546+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(547) : cplan_fun.sql(1623) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.RESOURCE_ID = h_res_id
               AND y.PROJECT_ID<>h_project_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 571+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(572) : cplan_fun.sql(1648) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID<>h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 593+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(594) : cplan_fun.sql(1670) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 23 THEN
         -- Consider PLANNED inside current project  SCHED all projects
         BEGIN
            INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID = h_res_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND BIT_FLAG='2';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 618+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(619) : cplan_fun.sql(1695) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.PROJECT_ID=h_project_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG = '0'
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 644+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(645) : cplan_fun.sql(1721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND BIT_FLAG = '0';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 666+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(667) : cplan_fun.sql(1743) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 10 THEN
         -- search from project assignments
         If Coalesce(h_update, 0) = 3 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.PROJECT_ID = h_project_id
               AND y.TASK_ID=h_project_id
               AND y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG = '0';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 696+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(697) : cplan_fun.sql(1773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.PROJECT_ID = h_project_id
               AND y.TASK_ID=h_project_id
               AND y.RESOURCE_ID = h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 721+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(722) : cplan_fun.sql(1798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN 12 THEN
         -- profile
         BEGIN
            INSERT INTO PMO_TMP_TDY
            SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID=h_project_id
            AND RESOURCE_ID = h_res_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 747+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(748) : cplan_fun.sql(1824) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      ELSE
         -- Consider SCHED outside current project if priority >= then me  PLANNED  SCHED inside current project
         If Coalesce(h_cri, 0) = 1 Then
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TMT_WBS w, TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG='2'
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority
               AND y.TABLE_TYPE = 1
               
               UNION ALL
               
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y
               WHERE 0=0
               AND y.PROJECT_ID=h_project_id
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.TABLE_TYPE = 1;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 787+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(788) : cplan_fun.sql(1864) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_TDY
               SELECT y.TSK_ASSIGNMENT_ID, y.DAY_YEAR, y.SCHED_MIN, y.BIT_FLAG, y.BIT_SCHED
               FROM TASK_DAY_OF_YEAR y, TMT_WBS w
               WHERE 0=0
               AND y.RESOURCE_ID=h_res_id
               AND y.DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10))
               AND y.BIT_FLAG='2'
               
               AND y.PROJECT_ID=w.ELEMENT_ID
               AND w.PRIORITY>=h_priority
               
               UNION ALL
               
               SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_FLAG, BIT_SCHED
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY', 821+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY(822) : cplan_fun.sql(1898) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
   END CASE;
   --trace_printf("%s\n", (char *) stmt);//trace_flush();
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY', 831+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY(832) : cplan_fun.sql(1908) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_TDY;
ALTER FUNCTION FC_CREATE_TDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR for Using LEVELING information from LEVEL_TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_TDY_LEVEL ==
CREATE OR REPLACE FUNCTION FC_CREATE_TDY_LEVEL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_consider PLS_INTEGER
   , sh_h_cri PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_update PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_consider PLS_INTEGER := sh_h_consider;
   h_cri PLS_INTEGER := sh_h_cri;
   h_priority PLS_INTEGER := sh_h_priority;
   h_update PLS_INTEGER := sh_h_update;
   h_count PLS_INTEGER := 0;
   h_priority_min PLS_INTEGER := 0;
   h_min_start_date TIMESTAMP;
   h_tsk_assignment_id VARCHAR2(32);
   h_dayofyear DATE;
   h_available PLS_INTEGER := 0;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CL_ACT IS
         SELECT 
         TSK_ASSIGNMENT_ID, DAY_YEAR, ACTUAL_MIN, BIT_SCHED 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND ACTUAL_FLAG = 1 
         AND SCHED_MIN <> ACTUAL_MIN;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_TDY_LEVEL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_TDY_LEVEL'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_TDY_LEVEL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY_LEVEL', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_TDY_LEVEL(48) : cplan_fun.sql(1970) : ' || SYSTIMESTAMP);
   END IF; 

   h_min_start_date := NULL;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDY';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   --trace_printf("***FC_CREATE_TDY_LEVEL h_update: %d\n",h_update);
   If (BITAND(Coalesce(h_update, 0), 256)) <> 256 Then
      /* Check if leveling Assignments or just Calculate[MSP-F9]
      		 if Calculate [MSP-F9] skip this code and don't Populate the table. */
      
      BEGIN
         SELECT COUNT(RESOURCE_ID) INTO h_count
         FROM PMO_TMP_TDY_LEVEL
         WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 75+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(76) : cplan_fun.sql(1998) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\n\n ==> In.FC_CREATE_TDY, h_count: %d, Proj_ID:%s; ResID:%s; SD:%s; Consider:%d\n",h_count,h_project_id,h_res_id,h_start_date,h_consider);
      
      --  stmt.Clear();
      If Coalesce(h_count, 0) = 0 Then
         --Load the resource records for the first time from the DB.
         /*BEGIN_SQL
         					DECLARE L_MINDT CURSOR WITH HOLD FOR
         					SELECT MIN(t.LEVEL_START) as minStart FROM TASK_ASSIGNMENTS a JOIN TMT_WBS t ON t.PROJECT_ID = a.PROJECT_ID AND t.ELEMENT_ID = a.TASK_ID
         					WHERE a.RESOURCE_ID = HOST(h_res_id) AND t.LEVEL_START is not Null AND a.PROJECT_ID IN (SELECT ELEMENT_ID FROM session.PMO_TMP_PROJECTS_00)
         					UNION ALL
         					SELECT MIN(BAK_START_DT) as minStart FROM TASK_ASSIGNMENTS
         					WHERE PROJECT_ID = TASK_ID and RESOURCE_ID = HOST(h_res_id) AND PROJECT_ID IN (SELECT ELEMENT_ID FROM session.PMO_TMP_PROJECTS_00)
         					ORDER BY minStart FOR READ ONLY;
         				END_SQL_NOCHECK;*/
         
         -- START Comment out by Tudor
         --BEGIN_SQL
         -- DECLARE L_MINDT CURSOR WITH HOLD FOR
         -- SELECT MIN(t.LEVEL_START) as minStart
         -- FROM TASK_ASSIGNMENTS a, session.Lev_tmt_wbs t
         -- WHERE a.RESOURCE_ID = HOST(h_res_id)
         --  AND t.LEVEL_START is not Null
         --  AND t.PROJECT_ID = a.PROJECT_ID
         --  AND t.ELEMENT_ID = a.TASK_ID
         -- UNION ALL
         -- SELECT MIN(t.BAK_START_DT) as minStart
         -- FROM TASK_ASSIGNMENTS t
         -- WHERE t.RESOURCE_ID = HOST(h_res_id)
         --  AND t.PROJECT_ID = t.TASK_ID
         --  AND EXISTS (SELECT ELEMENT_ID FROM session.PMO_TMP_PROJECTS_00 WHERE ELEMENT_ID=t.PROJECT_ID)
         -- ORDER BY minStart FETCH FIRST 1 ROWS ONLY;
         --END_SQL_NOCHECK;
         
         --BEGIN_SQL
         -- OPEN L_MINDT;
         --END_SQL;
         
         --BEGIN_SQL
         -- FETCH L_MINDT INTO HOST(h_min_start_date);
         --END_SQL;
         --trace_printf("2.i ==> SC_STATE: %d, h_min_start_date: %d %s\n", SC_STATE,h_min_start_date_Ind,h_min_start_date);
         
         --BEGIN_SQL
         -- CLOSE L_MINDT WITH RELEASE;
         --END_SQL;
         -- FINISH Comment out by Tudor
         
         /*BEGIN_SQL
         					SELECT MIN(t.LEVEL_START) INTO HOST(h_min_start_date)
         					FROM TASK_ASSIGNMENTS a JOIN TMT_WBS t ON t.ELEMENT_ID = a.TASK_ID
         					WHERE a.RESOURCE_ID = HOST(h_res_id) and t.LEVEL_START is not Null;
         				END_SQL_NOCHECK;*/
         
         /* Loading the following Records:
         						 Actual  Priority     Sched   Planned
         						----------------------------------------
         							x                   x               Outside project from the projList
         																Project in the ProjLiat
         																Current Project
         
         				*/
         BEGIN
            SELECT MIN(PRIORITY) INTO h_priority_min
            FROM PMO_TMP_PROJECTS_00;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 150+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(151) : cplan_fun.sql(2073) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_TDY_LEVEL (
            PROJECT_ID,
            RESOURCE_ID,
            PRIORITY,
            ACTUAL_FLAG,
            TSK_ASSIGNMENT_ID,
            DAY_YEAR,
            SCHED_MIN,
            BIT_FLAG,
            BIT_SCHED,
            PROJ_ASSIGN_FLAG)
            SELECT
            y.PROJECT_ID,
            y.RESOURCE_ID,
            w.PRIORITY,
            y.ACTUAL_FLAG,
            y.TSK_ASSIGNMENT_ID,
            y.DAY_YEAR,
            y.SCHED_MIN,
            y.BIT_FLAG,
            y.BIT_SCHED,
            case when (y.PROJECT_ID = y.TASK_ID) then 1 else 0 end as PROJ_ASSIGN_FLAG
            FROM TASK_DAY_OF_YEAR y, TMT_WBS w
            WHERE y.RESOURCE_ID = h_res_id
            AND y.DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            AND NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE y.PROJECT_ID=ELEMENT_ID)
            AND y.TABLE_TYPE = 1
            AND y.PROJECT_ID=w.ELEMENT_ID
            AND w.PRIORITY >= h_priority_min;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(196) : cplan_fun.sql(2118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2.1 ==> SC_STATE: %d, h_min_start_date: %s, h_priority_min: %d\n", SC_STATE,h_min_start_date,h_priority_min);
      End If;
      
      -- PMO_TMP_TDY_LEVEL initially
      BEGIN
         INSERT INTO PMO_TMP_TDY
         SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN END,BIT_FLAG, BIT_SCHED
         FROM  PMO_TMP_TDY_LEVEL
         WHERE RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND (
         (PROJECT_ID = h_project_id AND PROJ_ASSIGN_FLAG = 0)
         OR (PRIORITY >= h_priority
         AND NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE PROJECT_ID=ELEMENT_ID))
         OR (PROJECT_ID <> h_project_id
         AND EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_00 WHERE PROJECT_ID=ELEMENT_ID))
         )
         UNION ALL
         SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN END,BIT_FLAG, BIT_SCHED
         FROM  TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND RESOURCE_ID = h_res_id
         AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         AND ACTUAL_FLAG = 1 
         AND SCHED_MIN = ACTUAL_MIN;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 234+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(235) : cplan_fun.sql(2157) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2.2 ==> SC_STATE: %d, h_start_date: %s, h_priority: %d\n", SC_STATE,h_start_date,h_priority);
      
      OPEN CL_ACT;
      
      LOOP
         FETCH CL_ACT INTO h_tsk_assignment_id, h_dayofyear, h_available, h_bitID1;
         IF CL_ACT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If Coalesce(h_available, 0) > 0 Then
            PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_available, 0), h_bitID1);
            
            BEGIN
               INSERT INTO PMO_TMP_TDY (TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG)
               VALUES(h_tsk_assignment_id, h_dayofyear, h_available, h_bitID1, '2');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 269+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(270) : cplan_fun.sql(2192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CL_ACT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_TDY_LEVEL', 284+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_TDY_LEVEL(285) : cplan_fun.sql(2207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_TDY_LEVEL', 292+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_TDY_LEVEL(293) : cplan_fun.sql(2215) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_TDY_LEVEL;
ALTER FUNCTION FC_CREATE_TDY_LEVEL COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_DECLARE_PMO_TMP_SEARCH ==
CREATE OR REPLACE FUNCTION FC_DECLARE_PMO_TMP_SEARCH RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DECLARE_PMO_TMP_SEARCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DECLARE_PMO_TMP_SEARCH'), '-- ' || SYSTIMESTAMP, ':r := FC_DECLARE_PMO_TMP_SEARCH();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_PMO_TMP_SEARCH', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DECLARE_PMO_TMP_SEARCH(9) : cplan_fun.sql(2238) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SEARCH';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DECLARE_PMO_TMP_SEARCH', 16+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DECLARE_PMO_TMP_SEARCH(17) : cplan_fun.sql(2246) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DECLARE_PMO_TMP_SEARCH;
ALTER FUNCTION FC_DECLARE_PMO_TMP_SEARCH COMPILE DEBUG;

/***************************************************************************************************
SC_CHAR(h_element_id, SC_H_ID);

Modification needed in other .cpps:
VIEWS
	CALENDAR_NAME ==>
	SELECT ELEMENT_NAME
	FROM TMT_CALENDARS T1 LEFT OUTER JOIN TMT_CALENDARS T2 ON T1.ELEMENT_ID = T2.PARENT_ID
	WHERE T2.ELEMENT_ID = HOST(h_id) ;
RESOURCE
	CALENDAR_NAME ==>
	SELECT ELEMENT_NAME
	FROM TMT_CALENDARS T1 LEFT OUTER JOIN TMT_CALENDARS T2 ON T1.ELEMENT_ID = T2.PARENT_ID
	WHERE T2.ELEMENT_ID = HOST(h_resource_id) ;

***************************************************************************************************/

Prompt == FUNCTION POPULATE_TMP_CALENDAR ==
CREATE OR REPLACE FUNCTION POPULATE_TMP_CALENDAR
(
     element_id001 IN OUT NOCOPY VARCHAR2
   , calendar_id001 IN OUT NOCOPY VARCHAR2
   , SD001 IN OUT NOCOPY DATE
   , FD001 IN OUT NOCOPY DATE
   , sh_duration001 PLS_INTEGER
   , sh_element_type PLS_INTEGER := 1
) RETURN PLS_INTEGER
AS
   duration001 PLS_INTEGER := sh_duration001;
   element_type PLS_INTEGER := Coalesce(sh_element_type, 0);
   h_current_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_friday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_monday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_saturday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_sunday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_thursday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_tuesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_wednesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_FD DATE;
   h_SD DATE;
   h_calendar_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_exception_date DATE;
   h_dayofyear DATE;
   h_rank VARCHAR2(200);
   h_count PLS_INTEGER := 0;
   h_current_daynumber PLS_INTEGER := 0;
   h_current_min PLS_INTEGER := 0;
   h_current_weekday PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_friday_min PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_monday_min PLS_INTEGER := 0;
   h_saturday_min PLS_INTEGER := 0;
   h_sunday_min PLS_INTEGER := 0;
   h_thursday_min PLS_INTEGER := 0;
   h_tuesday_min PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_wednesday_min PLS_INTEGER := 0;
   h_week_iso PLS_INTEGER := 0;
   h_month_iso PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_start_year PLS_INTEGER := 0;
   h_start_month PLS_INTEGER := 0;
   h_end_year PLS_INTEGER := 0;
   h_end_month PLS_INTEGER := 0;
   iSF PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CSD1 IS
         SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE  ROWNUM <= 1 AND  ELEMENT_ID = h_element_id AND DAY_YEAR BETWEEN h_dayofyear AND  h_SD;

      CURSOR C_EXCEPTS IS
         SELECT EXCEPTION_DATE, EXCEPTION_MIN, EXCEPTION_BIT
         FROM CALENDAR_EXCEPTIONS
         WHERE CALENDAR_ID IN
         (
         SELECT ELEMENT_ID FROM TMT_CALENDARS
         WHERE RANK LIKE SUBSTR(h_rank, 1, 8) || '%'
         AND SUBSTR( h_rank, 1, LENGTH(RANK) ) = RANK
         )
         AND EXCEPTION_DATE BETWEEN TO_DATE(SubStr(TO_DATE(SubStr(h_SD, 1, 10)), 1, 10)) AND TO_DATE(SubStr(TO_DATE(SubStr(h_FD, 1, 10)), 1, 10));

      CURSOR C_DUPLIC IS
         SELECT ELEMENT_ID, DAY_YEAR, COUNT(ELEMENT_ID) FROM PMO_TMP_CDY
         GROUP BY ELEMENT_ID, DAY_YEAR HAVING COUNT(ELEMENT_ID) > 1;

      CURSOR C_DUP1 IS
         SELECT AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
         FROM PMO_TMP_CDY WHERE  ROWNUM <= 1 AND  ELEMENT_ID = h_element_id AND DAY_YEAR = h_dayofyear;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('POPULATE_TMP_CALENDAR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('POPULATE_TMP_CALENDAR'), '-- ' || SYSTIMESTAMP, ':r := POPULATE_TMP_CALENDAR(' || 
       '''' || element_id001 || ''''
          || ', ' || '''' || calendar_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || SD001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || FD001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_duration001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_element_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 85+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: POPULATE_TMP_CALENDAR(86) : cplan_fun.sql(2357) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --  SC_LONG(h_bitflag);
   
   h_element_id := element_id001;
   h_calendar_id := calendar_id001;
   h_SD := SD001;
   h_FD := FD001;
   h_duration := duration001;
   h_type_id := element_type;
   
   --trace_printf("START_ h_SD = %d %s, h_FD = %d %s, h_duration = %d\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD,h_duration);
   
   -- iSF = 0 fordward, iSF = 1 backward
   If Coalesce(h_duration, 0) >= PMO_CNST.SC_BIT_30 Then
      iSF := 1;
      h_duration := (BITAND(Coalesce(h_duration, 0), PMO_CNST.SC_BIT_NOT_30));
      --      InitializeHost(SC_PARAM(h_bitflag), 1);
   Else
      iSF := 0;
      --      InitializeHost(SC_PARAM(h_bitflag), 0);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_PROJECT Then
      If (h_SD IS NULL) Then
         BEGIN
            SELECT TO_DATE(SYSDATE) INTO h_SD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 120+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(121) : cplan_fun.sql(2392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_start_year := 0;
         
         -- This macro exits with an error if fiscal calendar is too short
         BEGIN
            BEGIN
               SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_start_year, h_start_month 
                       FROM FISCAL_CALENDAR WHERE h_SD BETWEEN START_DATE AND FINISH_DATE;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 138+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(139) : cplan_fun.sql(2410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 144+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(145) : cplan_fun.sql(2416) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(148) : cplan_fun.sql(2419) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_SD, False);
            End If;
         END;
         
         BEGIN
            SELECT START_DATE INTO h_SD FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_start_year AND FISCAL_MONTH = 1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 161+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(162) : cplan_fun.sql(2433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_start_year := 0;
         
         -- This macro exits with an error if fiscal calendar is too short
         BEGIN
            BEGIN
               SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_start_year, h_start_month 
                       FROM FISCAL_CALENDAR WHERE h_SD BETWEEN START_DATE AND FINISH_DATE;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 179+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(180) : cplan_fun.sql(2451) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 185+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(186) : cplan_fun.sql(2457) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 188+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(189) : cplan_fun.sql(2460) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_SD, False);
            End If;
         END;
         
         BEGIN
            SELECT START_DATE INTO h_dayofyear FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_start_year AND FISCAL_MONTH = 1;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 202+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(203) : cplan_fun.sql(2474) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (PMO_ORASTD.strcmp(h_dayofyear, h_SD) < 0) Then
            OPEN CSD1;
            
            FETCH CSD1 INTO h_dayofyear;
            IF CSD1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               h_SD := h_dayofyear;
            End If;
            
            BEGIN
               CLOSE CSD1 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 229+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(230) : cplan_fun.sql(2501) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (h_FD IS NULL) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_SD, 1, 10)) + h_duration      INTO h_FD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 245+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(246) : cplan_fun.sql(2517) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.YEAR(TO_DATE(SubStr(h_FD, 1, 10))) INTO h_count FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 258+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(259) : cplan_fun.sql(2530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_FD := '2100-01-01';
      ElsIf iSF = 0 Then
         h_end_year := 0;
         
         -- This macro exits with an error if fiscal calendar is too short
         BEGIN
            BEGIN
               SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_end_year, h_end_month 
                       FROM FISCAL_CALENDAR WHERE h_FD BETWEEN START_DATE AND FINISH_DATE;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 278+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(279) : cplan_fun.sql(2550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 284+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(285) : cplan_fun.sql(2556) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 287+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(288) : cplan_fun.sql(2559) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_FD, False);
            End If;
         END;
         
         If Coalesce(h_end_year, 0) <> 0 Then
            BEGIN
               SELECT FINISH_DATE INTO h_FD
               FROM FISCAL_CALENDAR
               WHERE FISCAL_YEAR = h_end_year AND FISCAL_MONTH = 12;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 304+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(305) : cplan_fun.sql(2576) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            SELECT SETTING_VALUE INTO h_count
            FROM TMT_SETTINGS WHERE ELEMENT_ID = 'CPLAN_PROJECT_CALENDAR';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 319+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(320) : cplan_fun.sql(2591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_count IS NULL) Then
            h_count := 365;
         End If;
         
         If Coalesce(h_duration, 0) < Coalesce(h_count, 0) Then
            h_duration := h_count;
         End If;
      End If;
   Else
      If (h_SD IS NULL) Then
         BEGIN
            SELECT TO_DATE(SYSDATE) INTO h_SD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 342+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(343) : cplan_fun.sql(2614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (h_FD IS NULL) Then
         BEGIN
            SELECT TO_DATE(SubStr(h_SD, 1, 10)) + h_duration      INTO h_FD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 357+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(358) : cplan_fun.sql(2629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   --trace_printf("*** iSF: %d, h_SD = %s, h_FD = %s, h_duration = %d, h_calendar_id = %s\n", iSF,h_SD,h_FD,h_duration,h_calendar_id);
   
   If Coalesce(Length(h_calendar_id), 0) = 0 Then
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 382+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(383) : cplan_fun.sql(2654) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID = h_calendar_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 403+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(404) : cplan_fun.sql(2675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT
            RTRIM(RANK),
            MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
            MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
            INTO
            h_rank,
            h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
            h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
            FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 425+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(426) : cplan_fun.sql(2697) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   --trace_printf("SC_STATE = %d, h_monday_min = %d\n", SC_STATE, h_monday_min);
   
   -- Insert records day by day
   -- ********************************************************************
   -- IMPORTANT SETTING, PMOv4 SETS SUNDAY AS THE FIRST DAY OF THE WEEK
   -- ********************************************************************
   
   -- Get start_date (weekday value)
   BEGIN
      SELECT PMO_ORASTD.DAYOFWEEK(TO_DATE(SubStr(h_SD, 1, 10))) INTO h_current_weekday FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 447+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(448) : cplan_fun.sql(2719) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_FD = %d;%s, h_duration = %d\n", SC_STATE, h_FD_Ind, h_FD, h_duration);
   
   -- Get total_day_count
   If (Coalesce(h_duration, 0) = 0) AND (iSF = 1) Then
      -- Assuming the start_date and finish_date are in good format
      BEGIN
         SELECT PMO_ORASTD.DAYS(TO_DATE(SubStr(h_FD, 1, 10))) - PMO_ORASTD.DAYS(TO_DATE(SubStr(h_SD, 1, 10)))
         INTO h_duration FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 465+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(466) : cplan_fun.sql(2737) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1___________*\n");
      
      If Coalesce(h_duration, 0) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 473+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(474) : cplan_fun.sql(2745) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
      End If;
   End If;
   
   h_current_daynumber := 0;
   h_count := 0;
   
   h_dayofyear := h_SD;
   --trace_printf("*** h_duration = %d, h_count = %d, h_current_weekday = %d\n", h_duration, h_count, h_current_weekday);
   
   LOOP
      CASE h_current_weekday
         WHEN 1 THEN
            h_current_bit := h_sunday_bit;
            h_current_min := Coalesce(h_sunday_min, 0);
         
         WHEN 2 THEN
            h_current_bit := h_monday_bit;
            h_current_min := Coalesce(h_monday_min, 0);
         
         WHEN 3 THEN
            h_current_bit := h_tuesday_bit;
            h_current_min := Coalesce(h_tuesday_min, 0);
         
         WHEN 4 THEN
            h_current_bit := h_wednesday_bit;
            h_current_min := Coalesce(h_wednesday_min, 0);
         
         WHEN 5 THEN
            h_current_bit := h_thursday_bit;
            h_current_min := Coalesce(h_thursday_min, 0);
         
         WHEN 6 THEN
            h_current_bit := h_friday_bit;
            h_current_min := Coalesce(h_friday_min, 0);
         
         WHEN 7 THEN
            h_current_bit := h_saturday_bit;
            h_current_min := Coalesce(h_saturday_min, 0);
         ELSE
            NULL;
      END CASE;
      
      --trace_printf("*** h_current_min = %d\n", h_current_min);
      -- insert into DayOfYear Table
      BEGIN
         SELECT  TO_DATE(SubStr(h_SD, 1, 10)) + h_current_daynumber      INTO h_dayofyear FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 528+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(529) : cplan_fun.sql(2800) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("**_ h_count = %d, h_current_daynumber: %d, h_FD: %s, h_dayofyear: %s\n",h_count,h_current_daynumber,h_FD,h_dayofyear);
      If PMO_ORASTD.strncmp(h_FD, h_dayofyear, 10) < 0 Then
         EXIT;
      End If;
      
      If Coalesce(h_current_min, 0) > 0 Then
         BEGIN
            SELECT PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear)  INTO h_week_iso, h_month_iso
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 548+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(549) : cplan_fun.sql(2820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY( ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 1, PMO_ORASTD.YEAR(h_dayofyear)+1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 567+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(568) : cplan_fun.sql(2839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 12, PMO_ORASTD.YEAR(h_dayofyear)-1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 585+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(586) : cplan_fun.sql(2857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear), PMO_ORASTD.YEAR(h_dayofyear));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 603+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(604) : cplan_fun.sql(2875) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (PMO_ORASTD.strncmp(h_FD, '2100-01-01', 10) = 0) AND (Coalesce(h_count, 0) >= Coalesce(
               h_duration, 0)) Then
            --trace_printf("1_ h_FD: %d:%s, h_count: %d, h_duration: %d\n",h_FD_Ind,h_FD,h_count,h_duration);
            -- This macro exits with an error if fiscal calendar is too short
            BEGIN
               BEGIN
                  SELECT FISCAL_YEAR, FISCAL_MONTH INTO h_end_year, h_end_month 
                          FROM FISCAL_CALENDAR WHERE h_dayofyear BETWEEN START_DATE AND FINISH_DATE;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 623+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(624) : cplan_fun.sql(2895) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'POPULATE_TMP_CALENDAR', 629+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: POPULATE_TMP_CALENDAR(630) : cplan_fun.sql(2901) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 632+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(633) : cplan_fun.sql(2904) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.DATE_NOT_IN_FISCAL_CALENDAR, h_dayofyear
                        , False);
               End If;
            END;
            
            BEGIN
               SELECT FINISH_DATE INTO h_FD FROM FISCAL_CALENDAR WHERE FISCAL_YEAR = h_end_year AND FISCAL_MONTH = 12;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 647+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(648) : cplan_fun.sql(2919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("2_ h_dayofyear: %s, h_FD: %d:%s\n",h_dayofyear,h_FD_Ind,h_FD);
         End If;
         
         h_count := Coalesce(h_count, 0) + 1;
      End If;
      
      h_current_weekday := Coalesce(h_current_weekday, 0) + 1;
      
      If Coalesce(h_current_weekday, 0) = 8 Then
         h_current_weekday := 1;
      End If;
      
      h_current_daynumber := Coalesce(h_current_daynumber, 0) + 1;
   END LOOP;
   
   If PMO_ORASTD.strncmp(h_FD, '2100-01-01', 10) = 0 Then
      h_FD := h_dayofyear;
   End If;
   
   -- ********************************************************************
   -- Insert exceptions
   -- ********************************************************************
   If Coalesce(Length(h_calendar_id), 0) > 0 Then
      If Coalesce(Length(h_FD), 0) = 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_SD, 1, 10)) + h_duration      INTO h_FD
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 684+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(685) : cplan_fun.sql(2956) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- if there are exception...
      
      OPEN C_EXCEPTS;
      
      LOOP
         FETCH C_EXCEPTS INTO h_exception_date, h_current_min, h_current_bit;
         IF C_EXCEPTS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If Coalesce(h_current_min, 0) > 0 Then
            BEGIN
               SELECT PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date)
               INTO h_week_iso, h_month_iso
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 716+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(717) : cplan_fun.sql(2988) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
               --removed ASSIGNED_MIN from insertion
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 1, PMO_ORASTD.YEAR(h_exception_date)+1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 736+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(737) : cplan_fun.sql(3008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 12, PMO_ORASTD.YEAR(h_exception_date)-1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 754+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(755) : cplan_fun.sql(3026) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, PMO_ORASTD.DAYOFWEEK(h_exception_date), h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date), PMO_ORASTD.YEAR(h_exception_date));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 772+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(773) : cplan_fun.sql(3044) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("exceptions: h_exception_date = %s, h_current_min = %d\n", h_exception_date,h_current_min);
         ElsIf Coalesce(h_current_min, 0) = 0 Then
            BEGIN
               DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = TO_DATE(SubStr(h_exception_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 791+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(792) : cplan_fun.sql(3063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_EXCEPTS ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 806+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(807) : cplan_fun.sql(3078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --trace_printf("END_1 iSF: %d, h_SD: %s, h_FD: %s\n", iSF,h_SD,h_FD);
   --timeLog (file,"INSERT INTO CALENDAR_DAY_OF_YEAR:");
   
   --  if( h_bitflag == 0 )
   --  {
   If iSF = 1 Then
      If PMO_ORASTD.strncmp(h_FD, FD001, 10) > 0 Then
         h_dayofyear := FD001;
         BEGIN
            DELETE FROM PMO_TMP_CDY
            WHERE ELEMENT_ID = h_element_id
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_dayofyear, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 833+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(834) : cplan_fun.sql(3105) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_FD: %s, FD: %s\n", SC_STATE,h_FD,FD);
         
         h_FD := FD001;
      End If;
   Else
      FD001 := h_FD;
   End If;
   
   SD001 := h_SD;
   FD001 := h_FD;
   
   --trace_printf("END_2 h_SD: %s, h_FD: %s\n", h_SD,h_FD);
   
   -- if there are duplicates...
   
   OPEN C_DUPLIC;
   
   LOOP
      FETCH C_DUPLIC INTO h_element_id, h_dayofyear, h_count;
      IF C_DUPLIC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      --trace_printf("DUPLICATE: h_element_id: %s, h_dayofyear: %s, h_count: %d\n", h_element_id,h_dayofyear,h_count);
      
      OPEN C_DUP1;
      
      FETCH C_DUP1 INTO h_current_min, h_current_bit, h_week_iso, h_month_iso, h_count;
      IF C_DUP1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      BEGIN
         CLOSE C_DUP1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 883+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(884) : cplan_fun.sql(3155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = h_dayofyear;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 900+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(901) : cplan_fun.sql(3172) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
         VALUES(h_element_id, h_dayofyear, PMO_ORASTD.DAYOFWEEK(h_dayofyear), h_current_min, h_current_bit, h_week_iso, h_month_iso, h_count);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 918+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(919) : cplan_fun.sql(3190) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_DUPLIC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 932+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(933) : cplan_fun.sql(3204) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO CALENDAR_DAY_OF_YEAR (ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
      SELECT * FROM PMO_TMP_CDY WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_SD, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10)) AND DAY_YEAR NOT IN (
      SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_SD, 1, 10)) AND TO_DATE(SubStr(h_FD, 1, 10)));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'POPULATE_TMP_CALENDAR', 951+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: POPULATE_TMP_CALENDAR(952) : cplan_fun.sql(3223) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --  }
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'POPULATE_TMP_CALENDAR', 958+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: POPULATE_TMP_CALENDAR(959) : cplan_fun.sql(3230) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION POPULATE_TMP_CALENDAR;
ALTER FUNCTION POPULATE_TMP_CALENDAR COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_CDY

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_NEW_CDY ==
CREATE OR REPLACE FUNCTION FC_NEW_CDY
(
     h_element_id001 IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY TIMESTAMP
   , sh_h_duration001 PLS_INTEGER
   , sh_element_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration001 PLS_INTEGER := sh_h_duration001;
   element_type PLS_INTEGER := Coalesce(sh_element_type, 0);
   h_current_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_friday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_monday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_saturday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_sunday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_thursday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_tuesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_wednesday_bit RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_FD DATE;
   h_SD DATE;
   h_calendar_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_exception_date DATE;
   h_dayofyear DATE;
   h_rank VARCHAR2(200);
   h_count PLS_INTEGER := 0;
   h_current_daynumber PLS_INTEGER := 0;
   h_current_min PLS_INTEGER := 0;
   h_current_weekday PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_friday_min PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_monday_min PLS_INTEGER := 0;
   h_saturday_min PLS_INTEGER := 0;
   h_sunday_min PLS_INTEGER := 0;
   h_thursday_min PLS_INTEGER := 0;
   h_tuesday_min PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_wednesday_min PLS_INTEGER := 0;
   h_week_iso PLS_INTEGER := 0;
   h_month_iso PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_EXPTS IS
         SELECT EXCEPTION_DATE, EXCEPTION_MIN, EXCEPTION_BIT
         FROM CALENDAR_EXCEPTIONS
         WHERE CALENDAR_ID IN
         (
         SELECT ELEMENT_ID FROM TMT_CALENDARS
         WHERE RANK LIKE SUBSTR(h_rank, 1, 8) || '%'
         AND SUBSTR( h_rank, 1, LENGTH(RANK) ) = RANK
         )
         AND EXCEPTION_DATE BETWEEN TO_DATE(SubStr(TO_DATE(SubStr(h_start_date, 1, 10)), 1, 10)) AND TO_DATE(SubStr(TO_DATE(SubStr(h_finish_date, 1, 10)), 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_NEW_CDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_NEW_CDY'), '-- ' || SYSTIMESTAMP, ':r := FC_NEW_CDY(' || 
       '''' || h_element_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_element_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_NEW_CDY', 66+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_NEW_CDY(67) : cplan_fun.sql(3311) : ' || SYSTIMESTAMP);
   END IF; 

   If (h_start_date IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_NEW_CDY', 71+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_NEW_CDY(72) : cplan_fun.sql(3316) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   If Coalesce(h_duration, 0) = 0 Then
      If (h_finish_date IS NULL) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_NEW_CDY', 78+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_NEW_CDY(79) : cplan_fun.sql(3323) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
      If PMO_ORASTD.strncmp(h_finish_date, h_start_date, 10) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_NEW_CDY', 84+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_NEW_CDY(85) : cplan_fun.sql(3329) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
      End If;
   End If;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(Length(h_calendar_id), 0) = 0 Then
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 111+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(112) : cplan_fun.sql(3356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT
         RTRIM(RANK),
         MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
         MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
         INTO
         h_rank,
         h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
         h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
         FROM TMT_CALENDARS WHERE ELEMENT_ID = h_calendar_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 132+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(133) : cplan_fun.sql(3377) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT
            RTRIM(RANK),
            MONDAY_MIN, TUESDAY_MIN, WEDNESDAY_MIN, THURSDAY_MIN, FRIDAY_MIN, SATURDAY_MIN, SUNDAY_MIN,
            MONDAY_BIT, TUESDAY_BIT, WEDNESDAY_BIT, THURSDAY_BIT, FRIDAY_BIT, SATURDAY_BIT, SUNDAY_BIT
            INTO
            h_rank,
            h_monday_min, h_tuesday_min, h_wednesday_min, h_thursday_min, h_friday_min, h_saturday_min, h_sunday_min,
            h_monday_bit, h_tuesday_bit, h_wednesday_bit, h_thursday_bit, h_friday_bit, h_saturday_bit, h_sunday_bit
            FROM TMT_CALENDARS WHERE ELEMENT_ID IS Null;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 154+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(155) : cplan_fun.sql(3399) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   -- Get start_date (weekday value)
   BEGIN
      SELECT PMO_ORASTD.DAYOFWEEK(TO_DATE(SubStr(h_start_date, 1, 10))) INTO h_current_weekday FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 170+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(171) : cplan_fun.sql(3415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_current_daynumber := 0;
   h_count := 0;
   
   h_dayofyear := h_start_date;
   LOOP
      CASE h_current_weekday
         WHEN 1 THEN
            h_current_bit := h_sunday_bit;
            h_current_min := Coalesce(h_sunday_min, 0);
         
         WHEN 2 THEN
            h_current_bit := h_monday_bit;
            h_current_min := Coalesce(h_monday_min, 0);
         
         WHEN 3 THEN
            h_current_bit := h_tuesday_bit;
            h_current_min := Coalesce(h_tuesday_min, 0);
         
         WHEN 4 THEN
            h_current_bit := h_wednesday_bit;
            h_current_min := Coalesce(h_wednesday_min, 0);
         
         WHEN 5 THEN
            h_current_bit := h_thursday_bit;
            h_current_min := Coalesce(h_thursday_min, 0);
         
         WHEN 6 THEN
            h_current_bit := h_friday_bit;
            h_current_min := Coalesce(h_friday_min, 0);
         
         WHEN 7 THEN
            h_current_bit := h_saturday_bit;
            h_current_min := Coalesce(h_saturday_min, 0);
         ELSE
            NULL;
      END CASE;
      
      If Coalesce(h_duration, 0) = 0 Then
         If PMO_ORASTD.strncmp(h_finish_date, h_dayofyear, 10) < 0 Then
            EXIT;
         End If;
      ElsIf Coalesce(h_duration, 0) > Coalesce(h_count, 0) Then
         EXIT;
      End If;
      
      If Coalesce(h_current_min, 0) > 0 Then
         BEGIN
            SELECT PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear) INTO h_week_iso, h_month_iso
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 230+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(231) : cplan_fun.sql(3475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY( ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 1, PMO_ORASTD.YEAR(h_dayofyear)+1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 249+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(250) : cplan_fun.sql(3494) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), 12, PMO_ORASTD.YEAR(h_dayofyear)-1);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 267+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(268) : cplan_fun.sql(3512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
               VALUES(h_element_id, h_dayofyear, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_dayofyear), PMO_ORASTD.MONTH(h_dayofyear), PMO_ORASTD.YEAR(h_dayofyear));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 285+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(286) : cplan_fun.sql(3530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_count := Coalesce(h_count, 0) + 1;
      End If;
      
      h_current_weekday := Coalesce(h_current_weekday, 0) + 1;
      
      If Coalesce(h_current_weekday, 0) = 8 Then
         h_current_weekday := 1;
      End If;
      
      h_current_daynumber := Coalesce(h_current_daynumber, 0) + 1;
      BEGIN
         SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 309+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(310) : cplan_fun.sql(3554) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   If Coalesce(h_duration, 0) = 0 Then
      h_finish_date := SubStr(h_dayofyear, 1, 10);
   End If;
   
   -- ********************************************************************
   -- Insert exceptions
   -- ********************************************************************
   If Coalesce(Length(h_calendar_id), 0) > 0 Then
      -- if there are exception...
      
      OPEN C_EXPTS;
      
      LOOP
         FETCH C_EXPTS INTO h_exception_date, h_current_min, h_current_bit;
         IF C_EXPTS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If Coalesce(h_current_min, 0) > 0 Then
            BEGIN
               SELECT PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date)
               INTO h_week_iso, h_month_iso
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(350) : cplan_fun.sql(3594) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_week_iso, 0) = 1) AND (Coalesce(h_month_iso, 0) = 12) Then
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 1, PMO_ORASTD.YEAR(h_exception_date)+1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 368+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(369) : cplan_fun.sql(3613) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_week_iso, 0) = 53) AND (Coalesce(h_month_iso, 0) = 1) Then
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, h_current_weekday, h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), 12, PMO_ORASTD.YEAR(h_exception_date)-1);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 386+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(387) : cplan_fun.sql(3631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO PMO_TMP_CDY(ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF)
                  VALUES(h_element_id, h_exception_date, PMO_ORASTD.DAYOFWEEK(h_exception_date), h_current_min, h_current_bit, PMO_ORASTD.WEEK_ISO(h_exception_date), PMO_ORASTD.MONTH(h_exception_date), PMO_ORASTD.YEAR(h_exception_date));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 404+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(405) : cplan_fun.sql(3649) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf Coalesce(h_current_min, 0) = 0 Then
            BEGIN
               DELETE FROM PMO_TMP_CDY WHERE ELEMENT_ID = h_element_id AND DAY_YEAR = TO_DATE(SubStr(h_exception_date, 1, 10));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 422+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(423) : cplan_fun.sql(3667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_EXPTS ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_NEW_CDY', 437+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_NEW_CDY(438) : cplan_fun.sql(3682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_NEW_CDY', 444+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_NEW_CDY(445) : cplan_fun.sql(3689) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_NEW_CDY;
ALTER FUNCTION FC_NEW_CDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_CDY

/*********************************************************************************************************************************************/

-- TOREVIEW:

Prompt == FUNCTION FC_CREATE_CDY ==
CREATE OR REPLACE FUNCTION FC_CREATE_CDY
(
     h_element_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_element_type PLS_INTEGER := 1
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   element_type PLS_INTEGER := Coalesce(sh_element_type, 0);
   h_dayofyear DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_DT VARCHAR2(26);
   h_calendar_id VARCHAR2(32);
   h_days PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CSCDY1 IS
         SELECT * FROM (SELECT DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_element_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR CFCDY0 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_element_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR CFCDY1 IS
         SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_element_id AND DAY_YEAR >= h_finish_D
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_CDY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_CDY'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_CDY(' || 
       '''' || h_element_id || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_element_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_CDY(48) : cplan_fun.sql(3753) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_element_id: %s\nh_start_date: %s, h_finish_date: %s, h_duration: %d\n", h_element_id,h_start_date,h_finish_date,h_duration);
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   --trace_printf("START_SC_STATE = %d, len of finish date = %d\n", SC_STATE,strlen(h_finish_date));
   
   If ((h_start_date IS NULL)) OR (Coalesce(Length(h_start_date), 0) = 0) Then
      BEGIN
         SELECT MIN(DAY_YEAR) INTO h_start_date
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 67+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(68) : cplan_fun.sql(3773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (
         SELECT 'X' FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(82) : cplan_fun.sql(3787) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 89+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(90) : cplan_fun.sql(3795) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   
   h_calendar_id := h_element_id;
   h_dayofyear := NULL;
   
   If ((h_finish_date IS NULL)) OR (Coalesce(Length(h_finish_date), 0) = 0) Then
      h_finish_D := '';
      h_days := 0;
      
      OPEN CSCDY1;
      
      FETCH CSCDY1 INTO h_dayofyear;
      IF CSCDY1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d:%s, h_dayofyear: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_dayofyear_Ind,h_dayofyear);
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         -- TODO: need to make sure that calendar exists and if it does not return error
         BEGIN
            CLOSE CSCDY1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 120+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(121) : cplan_fun.sql(3826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 125+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(126) : cplan_fun.sql(3831) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      Else
         BEGIN
            CLOSE CSCDY1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 137+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(138) : cplan_fun.sql(3843) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If PMO_ORASTD.strncmp(h_dayofyear, h_start_date, 10) > 0 Then
         -- VERIFY: why the -1, why not change the condition below to <= 0
         
         BEGIN
            SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1      INTO h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 155+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(156) : cplan_fun.sql(3861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9.0_SC_STATE: %d, h_finish_D: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D);
         
         If PMO_ORASTD.strncmp(h_finish_D, h_start_date, 10) < 0 Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
               INTO h_start_date
               FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_element_id
               AND DAY_YEAR = h_dayofyear;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 174+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(175) : cplan_fun.sql(3880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT MIN(DAY_YEAR), COUNT(*)
               INTO h_dayofyear, h_days
               FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_element_id
               AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 191+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(192) : cplan_fun.sql(3897) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            -- VERIFY:
            h_days := Coalesce(h_duration, 0) + 1;
         End If;
      Else
         -- start date is a working day
         BEGIN
            SELECT MIN(DAY_YEAR), COUNT(*)
            INTO h_dayofyear, h_days
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_element_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 213+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(214) : cplan_fun.sql(3919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("AA_SC_STATE = %d, h_dayofyear: %s, h_start_date = %s, h_duration = %d, h_days = %d\n", SC_STATE,h_dayofyear,h_start_date,h_duration,h_days);
      
      If Coalesce(h_days, 0) < Coalesce(h_duration, 0) Then
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID
            , DAY_YEAR
            , DAY_WEEK
            , AVAILABLE_MIN
            , BIT_SCHED
            , WEEK_OF
            , MONTH_OF
            , YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_element_id
            AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
            ORDER BY DAY_YEAR;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 245+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(246) : cplan_fun.sql(3951) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("AB_SC_STATE = %d, h_start_date = %s\n", SC_STATE,h_start_date);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear
               FROM PMO_TMP_CDY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 261+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(262) : cplan_fun.sql(3967) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_days := Coalesce(h_duration, 0);
         Else
            BEGIN
               SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
               INTO h_dayofyear FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 278+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(279) : cplan_fun.sql(3984) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_ORASTD.strcmp(h_dayofyear, h_start_date) < 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DAYS(h_start_date) - PMO_ORASTD.DAYS(h_dayofyear) + h_duration
                  INTO h_days
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 294+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(295) : cplan_fun.sql(4000) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_element_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_days, element_type);
         --trace_printf("BB_SC_STATE = %d, h_dayofyear = %d:%s, h_finish_D = %d:%s, h_days: %d\n", SC_STATE,strlen(h_dayofyear),h_dayofyear, strlen(h_finish_D),h_finish_D,h_days);
         --trace_printf("*** h_dayofyear = %s\n", h_dayofyear);
      ElsIf PMO_ORASTD.strncmp(h_dayofyear, h_start_date, 10) > 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_start_date, 1, 10)), TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     
            INTO h_dayofyear, h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 316+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(317) : cplan_fun.sql(4022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9_SC_STATE: %d, h_dayofyear: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_dayofyear_Ind,h_dayofyear,h_finish_D_Ind,h_finish_D);
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR ((h_dayofyear IS NULL)) 
               OR ((h_finish_D IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 325+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(326) : cplan_fun.sql(4031) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN -1;
         End If;
         
         h_days := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_element_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_days, element_type);
         
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 348+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(349) : cplan_fun.sql(4054) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("10_SC_STATE: %d, h_start_date: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_D_Ind,h_finish_D);
      Else
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 368+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(369) : cplan_fun.sql(4074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("11_SC_STATE: %d, h_start_date: %d:%s, h_finish_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_D_Ind,h_finish_D);
      End If;
      
      BEGIN
         SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) INTO h_finish_date FROM PMO_TMP_CDY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 383+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(384) : cplan_fun.sql(4089) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_duration := 0;
      h_start_D := NULL;
      
      OPEN CFCDY0;
      
      FETCH CFCDY0 INTO h_start_D;
      IF CFCDY0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_start_date: %d:%s, h_start_D: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_start_D_Ind,h_start_D);
      
      h_temp_sc_state := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CFCDY0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 411+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(412) : cplan_fun.sql(4117) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_start_DT := h_start_date;
      If (Coalesce(h_temp_sc_state, 0) = 0) AND ((h_start_D IS NOT NULL)) AND (PMO_ORASTD.strncmp(
             h_start_D, h_start_date, 10) > 0) Then
         -- extend backwards
         h_start_D := SubStr(h_start_date, 1, 10);
         BEGIN
            SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1      INTO h_finish_D
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 430+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(431) : cplan_fun.sql(4136) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("9.1_SC_STATE: %d, h_finish_D: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D);
         
         h_days := PMO_CNST.SC_BIT_30;
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_element_id, h_calendar_id, h_start_D, h_finish_D
                                         , h_duration, element_type);
         
         BEGIN
            SELECT TO_DATE(SubStr(h_finish_D, 1, 10)) + 1      INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 449+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(450) : cplan_fun.sql(4155) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_dayofyear := NULL;
      h_finish_D := SubStr(h_finish_date, 1, 10);
      
      OPEN CFCDY1;
      
      FETCH CFCDY1 INTO h_dayofyear;
      IF CFCDY1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_finish_D: %d:%s, h_dayofyear: %d:%s\n", SC_STATE,h_finish_D_Ind,h_finish_D,h_dayofyear_Ind,h_dayofyear);
      
      h_temp_sc_state := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CFCDY1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 478+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(479) : cplan_fun.sql(4184) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("*** h_temp_sc_state = %d\n", h_temp_sc_state);
      
      If (Coalesce(h_temp_sc_state, 0) <> 0) OR ((h_dayofyear IS NULL)) Then
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND
            DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10))
            ORDER BY DAY_YEAR;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 501+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(502) : cplan_fun.sql(4207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("22_SC_STATE: %d, h_start_date: %d:%s, h_finish_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
      Else
         BEGIN
            INSERT INTO PMO_TMP_CDY
            SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
            FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_element_id AND DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
            ORDER BY DAY_YEAR;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 522+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(523) : cplan_fun.sql(4228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("23_SC_STATE: %d, h_start_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date);
         
         BEGIN
            SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear FROM PMO_TMP_CDY;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY', 536+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY(537) : cplan_fun.sql(4242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CC_SC_STATE = %d, h_dayofyear = %s\n", SC_STATE,h_dayofyear);
         
         h_temp_sc_state := POPULATE_TMP_CALENDAR(h_element_id, h_calendar_id, h_dayofyear, h_finish_D
                                         , h_duration, element_type);
         --trace_printf("*** h_dayofyear = %s\n", h_dayofyear);
      End If;
   End If;
   --trace_printf("END: %d, h_start_date: %d:%s, h_finish_date: %d:%s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY', 551+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY(552) : cplan_fun.sql(4257) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_CDY;
ALTER FUNCTION FC_CREATE_CDY COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- Create and Populate TEMPORARY TABLE PMO_TMP_TDY like TASK_DAY_OF_YEAR

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_CREATE_CDY_LEVEL ==
CREATE OR REPLACE FUNCTION FC_CREATE_CDY_LEVEL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_duration PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_start_D DATE;
   h_finish_D DATE;
   h_finish_date TIMESTAMP;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_CDY_LEVEL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_CDY_LEVEL'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_CDY_LEVEL(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY_LEVEL', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_CDY_LEVEL(25) : cplan_fun.sql(4296) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_bitflag: %d, h_project_id: %s\nh_start_date: %s\n",h_bitflag, h_project_id,h_start_date);
   
   If Coalesce(h_bitflag, 0) = 0 Then
      h_finish_date := NULL;
      h_temp_sc_state := FC_CREATE_CDY(h_project_id, h_start_date, h_finish_date, h_duration, 1);
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_start_D FROM PMO_TMP_CDY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_CDY_LEVEL', 41+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_CDY_LEVEL(42) : cplan_fun.sql(4313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_finish_D := '';
      --trace_printf("h_duration: %d, h_start_D: %s\n",h_duration, h_start_D);
      h_temp_sc_state := POPULATE_TMP_CALENDAR(h_project_id, h_project_id, h_start_D, h_finish_D
                                         , h_duration, 1);
      --trace_printf("2_h_temp_sc_state: %d, h_start_D: %s, h_finish_D: %s\n",h_temp_sc_state,h_start_D,h_finish_D);
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_CDY_LEVEL', 55+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_CDY_LEVEL(56) : cplan_fun.sql(4327) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_CDY_LEVEL;
ALTER FUNCTION FC_CREATE_CDY_LEVEL COMPILE DEBUG;

--########################################################################

-- EXTEND CALENDAR

--########################################################################

Prompt == FUNCTION EXTEND_CAL ==
CREATE OR REPLACE FUNCTION EXTEND_CAL
(
     id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_project_id VARCHAR2(32);
   h_dayofyear DATE;
   h_finish_D DATE;
   h_type_id PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('EXTEND_CAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('EXTEND_CAL'), '-- ' || SYSTIMESTAMP, ':r := EXTEND_CAL(' || 
       '''' || id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_CAL', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: EXTEND_CAL(19) : cplan_fun.sql(4360) : ' || SYSTIMESTAMP);
   END IF; 

   h_project_id := id;
   
   -- Check for project calendar/resource
   BEGIN
      SELECT TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1      INTO h_dayofyear
      FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_CAL', 33+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_CAL(34) : cplan_fun.sql(4375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_dayofyear IS NOT NULL)) Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_CAL', 47+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_CAL(48) : cplan_fun.sql(4389) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_finish_D := '';
         h_temp_var := 0;
         h_temp_sc_state := POPULATE_CALENDAR(h_project_id, h_type_id, h_dayofyear, h_finish_D, h_temp_var);
      End If;
   Else
      PMO_DB2M.SC_STATE := -1;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_CAL', 62+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: EXTEND_CAL(63) : cplan_fun.sql(4404) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION EXTEND_CAL;
ALTER FUNCTION EXTEND_CAL COMPILE DEBUG;

--########################################################################

-- EXTEND RESOURCE / CALENDAR

--########################################################################

Prompt == FUNCTION EXTEND_PRCAL ==
CREATE OR REPLACE FUNCTION EXTEND_PRCAL
(
     id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , h_finish_DT IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_duration PLS_INTEGER := sh_h_duration;
   h_type PLS_INTEGER := Coalesce(sh_h_type, 0);
   h_id VARCHAR2(32);
   h_dayofyear DATE;
   h_FD DATE;
   h_start_D DATE;
   h_finish_D DATE;
   h_start_time VARCHAR2(8);
   h_count PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_ERC1 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('EXTEND_PRCAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('EXTEND_PRCAL'), '-- ' || SYSTIMESTAMP, ':r := EXTEND_PRCAL(' || 
       '''' || id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || '''' || h_finish_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 38+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: EXTEND_PRCAL(39) : cplan_fun.sql(4457) : ' || SYSTIMESTAMP);
   END IF; 

   h_type_id := h_type;
   h_id := id;
   h_dayofyear := SubStr(h_start_DT, 1, 10);
   
   If Coalesce(h_duration, 0) > 0 Then
      BEGIN
         SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     , TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
         INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 56+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(57) : cplan_fun.sql(4475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR ((h_start_D IS NULL)) 
            OR ((h_finish_D IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 64+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(65) : cplan_fun.sql(4483) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
      
      If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
         h_temp_var := 0;
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_dayofyear, h_start_D, h_temp_var);
      ElsIf PMO_ORASTD.strncmp(h_start_DT, h_finish_D, 10) > 0 Then
         BEGIN
            SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + h_duration      INTO h_FD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 81+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(82) : cplan_fun.sql(4500) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_count := 0;
         
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_FD, h_count);
      End If;
      
      BEGIN
         SELECT COUNT(DAY_YEAR) INTO h_count
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id AND DAY_YEAR > h_dayofyear;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 99+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(100) : cplan_fun.sql(4518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) < Coalesce(h_duration, 0) Then
         h_FD := '';
         
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_FD, h_duration);
      End If;
      
      stmt := '';
      stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = ''' || h_id 
            || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_dayofyear || ''', 1, 10)) '
             || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_duration, 0));
      
      OPEN C_ERC1 FOR stmt;
      
      LOOP
         FETCH C_ERC1 INTO h_finish_DT;
         IF C_ERC1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_ERC1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 137+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(138) : cplan_fun.sql(4556) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         SELECT TO_DATE(SubStr(MIN(DAY_YEAR), 1, 10)) - 1     , TO_DATE(SubStr(MAX(DAY_YEAR), 1, 10)) + 1     
         INTO h_start_D, h_finish_D
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'EXTEND_PRCAL', 152+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: EXTEND_PRCAL(153) : cplan_fun.sql(4571) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR ((h_start_D IS NULL)) 
            OR ((h_finish_D IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 160+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(161) : cplan_fun.sql(4579) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN -1;
      End If;
      
      If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) <= 0 Then
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_dayofyear, h_start_D, h_duration);
      End If;
      
      If PMO_ORASTD.strncmp(h_finish_DT, h_finish_D, 10) >= 0 Then
         h_dayofyear := SubStr(h_finish_DT, 1, 10);
         h_temp_sc_state := POPULATE_CALENDAR(h_id, h_type_id, h_finish_D, h_dayofyear, h_duration);
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'EXTEND_PRCAL', 177+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: EXTEND_PRCAL(178) : cplan_fun.sql(4596) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION EXTEND_PRCAL;
ALTER FUNCTION EXTEND_PRCAL COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_CONSIDER_DAY ==
CREATE OR REPLACE FUNCTION FC_CONSIDER_DAY
(
     h_dayofyear IN OUT NOCOPY DATE
   , sh_h_total PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_minPerDay PLS_INTEGER
   , h_bitID IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_total PLS_INTEGER := Coalesce(sh_h_total, 0);
   h_type PLS_INTEGER := Coalesce(sh_h_type, 0);
   h_minPerDay PLS_INTEGER := Coalesce(sh_h_minPerDay, 0);
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_available PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CC16 IS
         SELECT SCHED_MIN, BIT_SCHED
         FROM PMO_TMP_TDY
         WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CONSIDER_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CONSIDER_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_CONSIDER_DAY(' || 
       COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_total), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minPerDay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CONSIDER_DAY', 32+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CONSIDER_DAY(33) : cplan_fun.sql(4639) : ' || SYSTIMESTAMP);
   END IF; 

   -- Min(# of minutes assigned, total Available)
   
   h_bit_minutes := 0;
   
   --trace_printf("CONSIDER: h_minPerDay = %d, h_dayofyear = %s, h_total = %d, h_type: %d, h_bit_minutes: %d\n",h_minPerDay, h_dayofyear,h_total,h_type,h_bit_minutes);
   
   BEGIN
      SELECT SCHED_MIN INTO h_bit_minutes
      FROM PMO_TMP_CPLNF_0001 WHERE DAY_YEAR = h_dayofyear;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CONSIDER_DAY', 50+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CONSIDER_DAY(51) : cplan_fun.sql(4657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE = %d, h_bit_minutes: %d\n", SC_STATE,h_bit_minutes);
   
   /*
   		the function return the Amount of assigned min.
   
   		h_total:        Calendar Avalibility per day
   		h_bit_minutes:  Agregate Assignments per day
   		h_minPerDay:    Minimum Assign per day.
   	*/
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND h_type > 0 Then
      If Coalesce(h_bit_minutes, 0) < h_total AND (h_total - Coalesce(
            h_bit_minutes, 0)) >= h_minPerDay Then
         OPEN CC16;
         
         LOOP
            FETCH CC16 INTO h_available, h_bitID1;
            IF CC16%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If Coalesce(h_available, 0) > 0 Then
               h_bitID := PMO_STPU.BIT_ANDNOT_DAY(h_bitID, h_bitID1);
            End If;
         END LOOP;
         
         BEGIN
            CLOSE CC16 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CONSIDER_DAY', 95+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CONSIDER_DAY(96) : cplan_fun.sql(4702) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_bit_minutes := h_total;
         -- return that this day is not avalibal (all day assigned)
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CONSIDER_DAY', 106+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CONSIDER_DAY(107) : cplan_fun.sql(4713) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_bit_minutes, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CONSIDER_DAY', 111+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CONSIDER_DAY(112) : cplan_fun.sql(4718) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CONSIDER_DAY;
ALTER FUNCTION FC_CONSIDER_DAY COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_FIX_SUMMARY_TS ==
CREATE OR REPLACE FUNCTION FC_FIX_SUMMARY_TS
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_day_year DATE;
   h_mls_percent PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
-- Declare Cursors
      CURSOR C_TS IS
         SELECT WEEK_OF FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND WORK_TODATE >= h_work_todate
         ORDER BY WEEK_OF;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_SUMMARY_TS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_SUMMARY_TS'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_SUMMARY_TS(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_SUMMARY_TS(29) : cplan_fun.sql(4757) : ' || SYSTIMESTAMP);
   END IF; 

   h_sched_work := 0;
   h_work_todate := 0;
   BEGIN
      SELECT SCHED_WORK, WORK_TODATE, MILESTONE_PERCENT, REGULAR_COST
      INTO h_sched_work,h_work_todate,h_mls_percent, h_reg_cost
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 44+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(45) : cplan_fun.sql(4773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 50+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_SUMMARY_TS(51) : cplan_fun.sql(4779) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   --fix baseline_work per week information
   --  h_temp_sc_state = FC_GENERATE_EW(SC_PARAM(h_tsk_assignment_id));
   
   --locate affected summary_timesheets
   
   OPEN C_TS;
   
   LOOP
      FETCH C_TS INTO h_day_year;
      IF C_TS%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      --accumulate baseline_work_todate
      BEGIN
         SELECT COALESCE(SUM(SCHED_MIN),0) INTO h_work
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR <= h_day_year;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(86) : cplan_fun.sql(4814) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\nh_tsk_assignment_id: %d, h_day_year %s", h_tsk_assignment_id,h_day_year);
      --trace_printf("\nh_work: %d, %d", h_work_Ind, h_work);
      --trace_printf("\nh_sched_work: %d, %d", h_sched_work_Ind, h_sched_work);
      --trace_printf("\nh_work_todate: %d, %d", h_work_todate_Ind, h_work_todate);
      --trace_printf("\nh_mls_percent: %d, %d", h_mls_percent_Ind, h_mls_percent);
      
      --update affected summary_timesheets
      If Coalesce(h_sched_work, 0) > Coalesce(h_work_todate, 0) Then
         BEGIN
            UPDATE SUMMARY_TIMESHEETS SET
            TOTAL_SCHED_EFFORT = h_sched_work,
            ETC = h_sched_work - h_work_todate,
            EV_ETC = (h_sched_work - h_work_todate) * h_reg_cost,
            MILESTONE_PERCENT = h_mls_percent,
            EAC = ACWP + ((h_sched_work - h_work_todate)/60.00 * BASELINE_REGULAR_COST),
            BASELINE_WORK_TODATE = h_work,
            BCWS = h_work/60.00 * BASELINE_REGULAR_COST,
            BCWP = BAC * (h_mls_percent/10000.00)
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 120+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(121) : cplan_fun.sql(4849) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE SUMMARY_TIMESHEETS SET
            TOTAL_SCHED_EFFORT = h_sched_work,
            ETC = 0,
            EV_ETC = 0.00,
            MILESTONE_PERCENT = h_mls_percent,
            EAC = ACWP,
            BASELINE_WORK_TODATE = h_work,
            BCWS = h_work/60.00 * BASELINE_REGULAR_COST,
            BCWP = BAC * (h_mls_percent/10000.00)
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND WEEK_OF = h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(148) : cplan_fun.sql(4876) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_TS ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 162+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(163) : cplan_fun.sql(4891) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET
      EAC = CASE WHEN SCHED_WORK < WORK_TODATE THEN ACWP ELSE BCWS END,
      EV_ETC = ((ETC/60.00)*REGULAR_COST)
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_SUMMARY_TS', 182+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_SUMMARY_TS(183) : cplan_fun.sql(4911) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_SUMMARY_TS', 188+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_SUMMARY_TS(189) : cplan_fun.sql(4917) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_SUMMARY_TS;
ALTER FUNCTION FC_FIX_SUMMARY_TS COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION FC_DCONSIDER_DAY ==
CREATE OR REPLACE FUNCTION FC_DCONSIDER_DAY
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_day_year IN OUT NOCOPY DATE
   , h_bitID IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_available PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_project_id_Ind PLS_INTEGER NOT NULL := 0;
   h_tsk_assignment_id_Ind PLS_INTEGER NOT NULL := 0;
   h_prj_assignment_id_Ind PLS_INTEGER NOT NULL := 0;
   h_day_year_Ind PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CCD1 IS
         SELECT SCHED_MIN, BIT_SCHED FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id 
         AND TSK_ASSIGNMENT_ID <> h_tsk_assignment_id
         AND TSK_ASSIGNMENT_ID <> PARENT_ID AND DAY_YEAR = TO_DATE(SubStr(h_day_year, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DCONSIDER_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DCONSIDER_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_DCONSIDER_DAY(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_day_year || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DCONSIDER_DAY', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DCONSIDER_DAY(36) : cplan_fun.sql(4963) : ' || SYSTIMESTAMP);
   END IF; 

   h_bitID1 := PMO_CNST.SC_ZERO_BIT;
   
   OPEN CCD1;
   
   LOOP
      FETCH CCD1 INTO h_available, h_bitID1;
      IF CCD1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If Coalesce(h_available, 0) > 0 Then
         h_bit_minutes := Coalesce(h_bit_minutes, 0) + Coalesce(h_available, 0);
         h_bitID := PMO_STPU.BIT_ANDNOT_DAY(h_bitID, h_bitID1);
      End If;
   END LOOP;
   BEGIN
      CLOSE CCD1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DCONSIDER_DAY', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DCONSIDER_DAY(67) : cplan_fun.sql(4994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := Coalesce(h_bit_minutes, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DCONSIDER_DAY', 74+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DCONSIDER_DAY(75) : cplan_fun.sql(5002) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DCONSIDER_DAY;
ALTER FUNCTION FC_DCONSIDER_DAY COMPILE DEBUG;

Prompt == FUNCTION FC_INSERT_DAY_OF_YEAR ==
CREATE OR REPLACE FUNCTION FC_INSERT_DAY_OF_YEAR
(
     sh_h_work_type PLS_INTEGER
   , sh_h_minutes PLS_INTEGER
   , sh_h_update PLS_INTEGER
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
   , h_first_time IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , sh_h_table_type PLS_INTEGER
   , h_bitID_Tsk IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_update PLS_INTEGER := sh_h_update;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_consider PLS_INTEGER := sh_h_consider;
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_bit_flag VARCHAR2(1);
   h_bit_minutes PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_min_work_per_day PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_from_to_prj PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_INSERT_DAY_OF_YEAR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_INSERT_DAY_OF_YEAR'), '-- ' || SYSTIMESTAMP, ':r := FC_INSERT_DAY_OF_YEAR(' || 
       COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL')
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || '''' || h_first_time || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID_Tsk), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_DAY_OF_YEAR', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_INSERT_DAY_OF_YEAR(61) : cplan_fun.sql(5071) : ' || SYSTIMESTAMP);
   END IF; 

   -- 0 - regular task, 1 from project reserve, 2 - to project (define project reserve)
   
   h_type := 1;
   h_finish_time := '';
   h_min_work_per_day := h_minutes;
   
   -- NOTE: The following rules are used in assignments
   --
   -- 0. if h_prj_assignment_id == NULL then this is regular task assignment
   -- 1. else if h_prj_assignment_id != h_tsk_assignment_id then this is a task_assignment from project reserve
   -- 2. else if h_prj_assignment_id == h_tsk_assignment_id then this is the project reserve assignment
   --
   
   If (h_prj_assignment_id IS NULL) Then
      If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
         h_from_to_prj := 0;
         -- task assignment
      Else
         h_from_to_prj := 2;
         -- project assignment
      End If;
   Else
      
      If PMO_ORASTD.strcmp(h_prj_assignment_id, h_tsk_assignment_id) <> 0 Then
         -- from project reserve
         h_from_to_prj := 1;
      Else
         h_from_to_prj := 2;
      End If;
   End If;
   
   If Coalesce(h_minutes, 0) < 5 Then
      h_min_work_per_day := 5;
      -- can't assign less then 5 minutes
   Else
      h_temp_var := Coalesce(h_minutes, 0) MOD 5;
      If Coalesce(h_temp_var, 0) <> 0 Then
         h_min_work_per_day := Coalesce(h_min_work_per_day, 0) + 5 - Coalesce(h_temp_var, 0);
      End If;
   End If;
   
   If Coalesce(h_work_type, 0) = 2 Then
      h_bit_flag := '2';
   Else
      h_bit_flag := '0';
   End If;
   
   h_start_time := h_first_time;
   
   If (BITAND(Coalesce(h_bitflag, 0), 255)) = 10 OR Coalesce(h_bitflag, 0) > 9 Then
      -- Leveling or search/assign use the bits that are passed in
      h_bitID := h_bitID_Tsk;
      
      h_available := PMO_STPU.getNumberOfBits(h_bitID) * 5;
      
      If Coalesce(h_minutes, 0) <= Coalesce(h_available, 0) Then
         PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_min_work_per_day, 0), h_bitID);
      End If;
      
      If h_from_to_prj = 1 Then
         -- from project reserve
         BEGIN
            SELECT
            SCHED_MIN,
            BIT_SCHED
            INTO
            h_available,
            h_bitID2
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
            AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 141+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(142) : cplan_fun.sql(5152) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_MSP_AS_IS Then
         -- NOTE: for MSP start/finish time is always taken from the assignment.
         BEGIN
            SELECT
            CASE WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) = h_dayofyear THEN PMO_ORASTD.TIME(SCHED_START) ELSE NuLL END,
            CASE WHEN TO_DATE(SubStr(SCHED_FINISH, 1, 10)) = h_dayofyear THEN PMO_ORASTD.TIME(SCHED_FINISH) ELSE NuLL END
            INTO
            h_start_time,
            h_finish_time
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND BIT_FLAG = h_bit_flag;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(166) : cplan_fun.sql(5176) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If h_from_to_prj = 1 Then
         -- from project reserve or to project (reserve)
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_MSP_AS_IS OR Coalesce(
               h_consider, 0) = PMO_CNST.SC_SRCH_OVERALLOCATE Then
            BEGIN
               SELECT
               SCHED_MIN,
               BIT_SCHED
               INTO
               h_available,
               h_bitID
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
               AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 192+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(193) : cplan_fun.sql(5203) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT
               SCHED_MIN,
               BIT_SCHED
               INTO
               h_available,
               h_bitID
               FROM PMO_TMP_TDY
               WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id
               AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(214) : cplan_fun.sql(5224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT
               AVAILABLE_MIN,
               BIT_SCHED
               INTO
               h_available,
               h_bitID
               FROM PMO_TMP_CDY
               WHERE ELEMENT_ID=h_project_id
               AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 236+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(237) : cplan_fun.sql(5247) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            -- RES_CAL_TT
            BEGIN
               SELECT
               AVAILABLE_MIN,
               BIT_SCHED
               INTO
               h_available,
               h_bitID
               FROM PMO_TMP_CDY
               WHERE ELEMENT_ID=h_res_id
               AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 258+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(259) : cplan_fun.sql(5269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         If Coalesce(h_consider, 0) = 0 Then
            BEGIN
               SELECT  (PMO_CNST.SC_BIT_SCHED8H)  INTO  h_bitID FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(276) : cplan_fun.sql(5286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_available := 0;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_DAY_OF_YEAR', 283+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_INSERT_DAY_OF_YEAR(284) : cplan_fun.sql(5294) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         End If;
      End If;
      
      h_bitID2 := h_bitID;
      
      -- assign from project reserve
      If (h_start_time IS NOT NULL) AND (h_finish_time IS NOT NULL) Then
         PMO_STPU.cleardayStruct('00:00', h_start_time, h_bitID);
         PMO_STPU.cleardayStruct(h_finish_time, '24:00', h_bitID);
         
         If Coalesce(h_minutes, 0) <= Coalesce(h_available, 0) Then
            -- use up all the bits between given time
            NULL;
         End If;
      ElsIf (h_start_time IS NOT NULL) Then
         PMO_STPU.cleardayStruct('00:00', h_start_time, h_bitID);
         
         If Coalesce(h_minutes, 0) <= Coalesce(h_available, 0) Then
            PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_min_work_per_day, 0), h_bitID);
         End If;
      ElsIf (h_finish_time IS NOT NULL) Then
         -- clear from the end
         PMO_STPU.cleardayStruct(h_finish_time, '24:00', h_bitID);
         
         If Coalesce(h_minutes, 0) <= Coalesce(h_available, 0) Then
            PMO_STPU.skipLastBitsClearToFirst(Coalesce(h_min_work_per_day, 0), h_bitID);
         End If;
      ElsIf Coalesce(h_minutes, 0) <= Coalesce(h_available, 0) Then
         PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_min_work_per_day, 0), h_bitID);
      End If;
   End If;
   
   -- h_available : what is available from Calendar or project reserve (or bits passed in for Leveling)
   -- h_start_time : start time if given
   -- h_bitID : assignment bits for the day
   -- h_bitID2 : available bits for the day (original unmodified from calendar or project reserve)
   
   h_bitID2 := PMO_STPU.BIT_ANDNOT_DAY(h_bitID2, h_bitID);
   -- h_bitID2 : contains the remaining bits for the day
   
   If (BITAND(Coalesce(h_bitflag, 0), 255)) = 10 Then
      BEGIN
         INSERT INTO PMO_TMP_TDOY (
         PROJECT_ID,
         TASK_ID,
         RESOURCE_ID,
         TSK_ASSIGNMENT_ID,
         PARENT_ID,
         DAY_YEAR,
         SCHED_MIN,
         BIT_SCHED,
         BIT_FLAG,
         TABLE_TYPE)
         VALUES(
         h_project_id,
         h_task_id,
         h_res_id,
         h_tsk_assignment_id,
         COALESCE(h_prj_assignment_id,h_tsk_assignment_id),
         h_dayofyear,
         h_minutes,
         h_bitID,
         h_bit_flag,
         h_table_type);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 360+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(361) : cplan_fun.sql(5371) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_exists := 0;
      BEGIN
         SELECT SCHED_MIN, BIT_SCHED
         INTO h_exists, h_bitID1
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR = h_dayofyear
         AND BIT_FLAG = h_bit_flag;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 380+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(381) : cplan_fun.sql(5391) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_exists, 0) <> 0 Then
         h_minutes := Coalesce(h_minutes, 0) + Coalesce(h_exists, 0);
         h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bitID1);
      End If;
      
      If Coalesce(h_update, 0) <> 1 Then
         If h_from_to_prj = 1 Then
            If Coalesce(h_minutes, 0) < Coalesce(h_available, 0) Then
               If Coalesce(h_exists, 0) <> 0 Then
                  -- TOTEST: verify this does not get called.
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET
                     SCHED_MIN = h_minutes,
                     BIT_SCHED = h_bitID
                     WHERE PROJECT_ID = h_project_id
                     AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 412+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(413) : cplan_fun.sql(5423) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     INSERT INTO PMO_TMP_TDOY (
                     PROJECT_ID,
                     TASK_ID,
                     RESOURCE_ID,
                     TSK_ASSIGNMENT_ID,
                     PARENT_ID,
                     DAY_YEAR,
                     SCHED_MIN,
                     BIT_SCHED,
                     BIT_FLAG,
                     TABLE_TYPE)
                     VALUES(
                     h_project_id,
                     h_task_id,
                     h_res_id,
                     h_tsk_assignment_id,
                     h_prj_assignment_id,
                     h_dayofyear,
                     h_minutes,
                     h_bitID,
                     h_bit_flag,
                     h_table_type);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 450+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(451) : cplan_fun.sql(5461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_available := Coalesce(h_available, 0) - Coalesce(h_minutes, 0);
               h_temp_var := PMO_STPU.getNumberOfBits(h_bitID2) * 5;
               
               If Coalesce(h_temp_var, 0) = 0 AND Coalesce(h_available, 0) > 0 Then
                  h_bitID2 := h_bitID;
                  PMO_STPU.skipLastBitsClearToFirst(Coalesce(h_available, 0), h_bitID2);
               End If;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET
                  SCHED_MIN = h_available,
                  BIT_SCHED = h_bitID2
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
                  AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 481+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(482) : cplan_fun.sql(5492) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               If Coalesce(h_exists, 0) <> 0 Then
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id
                     AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 502+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(503) : cplan_fun.sql(5513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               -- OPTIMIZATION: move the record from project to task
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET
                  TSK_ASSIGNMENT_ID = h_tsk_assignment_id,
                  SCHED_MIN = h_minutes,
                  TASK_ID = h_task_id,
                  BIT_FLAG = h_bit_flag
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_prj_assignment_id
                  AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 528+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(529) : cplan_fun.sql(5539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            -- Not from project
            If Coalesce(h_exists, 0) <> 0 Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET
                  SCHED_MIN = h_minutes,
                  BIT_SCHED = h_bitID
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 553+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(554) : cplan_fun.sql(5564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO PMO_TMP_TDOY (
                  PROJECT_ID,
                  TASK_ID,
                  RESOURCE_ID,
                  TSK_ASSIGNMENT_ID,
                  PARENT_ID,
                  DAY_YEAR,
                  SCHED_MIN,
                  BIT_SCHED,
                  BIT_FLAG,
                  TABLE_TYPE)
                  VALUES(
                  h_project_id,
                  h_task_id,
                  h_res_id,
                  h_tsk_assignment_id,
                  h_tsk_assignment_id,
                  h_dayofyear,
                  h_minutes,
                  h_bitID,
                  h_bit_flag,
                  h_table_type);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 591+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(592) : cplan_fun.sql(5602) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         If Coalesce(h_exists, 0) <> 0 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET
               SCHED_MIN = h_minutes,
               BIT_SCHED = h_bitID
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 616+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(617) : cplan_fun.sql(5627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            If h_from_to_prj <> 1 Then
               h_prj_assignment_id := h_tsk_assignment_id;
            End If;
            
            BEGIN
               INSERT INTO PMO_TMP_TDOY (
               PROJECT_ID,
               TASK_ID,
               RESOURCE_ID,
               TSK_ASSIGNMENT_ID,
               PARENT_ID,
               DAY_YEAR,
               SCHED_MIN,
               BIT_SCHED,
               BIT_FLAG,
               TABLE_TYPE)
               VALUES(
               h_project_id,
               h_task_id,
               h_res_id,
               h_tsk_assignment_id,
               h_prj_assignment_id,
               h_dayofyear,
               h_minutes,
               h_bitID,
               h_bit_flag,
               h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_DAY_OF_YEAR', 658+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_DAY_OF_YEAR(659) : cplan_fun.sql(5669) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   h_bitID_Tsk := h_bitID;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_DAY_OF_YEAR', 673+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_INSERT_DAY_OF_YEAR(674) : cplan_fun.sql(5684) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_INSERT_DAY_OF_YEAR;
ALTER FUNCTION FC_INSERT_DAY_OF_YEAR COMPILE DEBUG;

--######################################################################################################

Prompt == FUNCTION CHECK_ASSIGN ==
CREATE OR REPLACE FUNCTION CHECK_ASSIGN
(
     h_element_id IN OUT NOCOPY VARCHAR2
   , h_start_DT IN OUT NOCOPY VARCHAR2
   , sh_h_work PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_work PLS_INTEGER := sh_h_work;
   h_start_D DATE;
   h_finish_D DATE;
   h_dayofyear DATE;
-- Declare Cursors
      CURSOR CCA16a IS
         SELECT * FROM (SELECT d.DAY_YEAR
         FROM CALENDAR_DAY_OF_YEAR d JOIN PMO_TMP_CPLNF_0001 ta 
         ON d.ELEMENT_ID = h_element_id
         AND d.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_D, 1, 10)) 
         AND ta.DAY_YEAR = d.DAY_YEAR
         AND (d.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)) > 0
         ORDER BY d.DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR CCA16ac IS
         SELECT * FROM (SELECT d.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR d
         WHERE d.ELEMENT_ID = h_element_id
         AND d.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_D, 1, 10))
         AND NOT EXISTS (SELECT ta.DAY_YEAR FROM PMO_TMP_CPLNF_0001 ta WHERE ta.DAY_YEAR = d.DAY_YEAR)
         ORDER BY d.DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR CCA16b IS
         SELECT * FROM (SELECT d.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR d JOIN PMO_TMP_CPLNF_0001 ta
         ON d.ELEMENT_ID = h_element_id
         AND d.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_D, 1, 10))
         AND ta.DAY_YEAR = d.DAY_YEAR
         AND (d.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)) >= h_work
         ORDER BY d.DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR CCA16bc IS
         SELECT * FROM (SELECT d.DAY_YEAR FROM CALENDAR_DAY_OF_YEAR d
         WHERE d.ELEMENT_ID = h_element_id
         AND d.DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_D, 1, 10))
         AND d.AVAILABLE_MIN >= h_work
         AND NOT EXISTS (SELECT ta.DAY_YEAR FROM PMO_TMP_CPLNF_0001 ta WHERE ta.DAY_YEAR = d.DAY_YEAR)
         ORDER BY d.DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('CHECK_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('CHECK_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := CHECK_ASSIGN(' || 
       '''' || h_element_id || ''''
          || ', ' || '''' || h_start_DT || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN', 53+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: CHECK_ASSIGN(54) : cplan_fun.sql(5748) : ' || SYSTIMESTAMP);
   END IF; 

   h_start_D := NULL;
   h_finish_D := NULL;
   h_dayofyear := NULL;
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLNF_0001';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_CPLNF_0001
      SELECT DAY_YEAR, SUM(SCHED_MIN)
      FROM PMO_TMP_TDY
      WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
      GROUP BY DAY_YEAR;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(80) : cplan_fun.sql(5774) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT MIN(DAY_YEAR), MAX(DAY_YEAR) + 1      INTO h_start_D, h_finish_D
         FROM PMO_TMP_CPLNF_0001;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(95) : cplan_fun.sql(5789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strncmp(h_start_DT, h_start_D, 10) >= 0 AND PMO_ORASTD.strncmp(h_start_DT, h_finish_D
            , 10) < 0 Then
         h_start_D := NULL;
         If Coalesce(h_work, 0) = 0 Then
            OPEN CCA16a;
            
            FETCH CCA16a INTO h_start_D;
            IF CCA16a%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            OPEN CCA16ac;
            
            FETCH CCA16ac INTO h_dayofyear;
            IF CCA16ac%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE CCA16ac ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 129+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(130) : cplan_fun.sql(5824) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               CLOSE CCA16a ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 142+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(143) : cplan_fun.sql(5837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            OPEN CCA16b;
            
            FETCH CCA16b INTO h_start_D;
            IF CCA16b%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            OPEN CCA16bc;
            
            FETCH CCA16bc INTO h_dayofyear;
            IF CCA16bc%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE CCA16bc ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 173+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(174) : cplan_fun.sql(5868) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               CLOSE CCA16b ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 186+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(187) : cplan_fun.sql(5881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (h_start_D IS NULL) Then
            If (h_dayofyear IS NULL) Then
               h_start_D := SubStr(h_finish_D, 1, 10);
            Else
               h_start_D := SubStr(h_dayofyear, 1, 10);
            End If;
         ElsIf (h_dayofyear IS NOT NULL) AND PMO_ORASTD.strncmp(h_dayofyear, h_start_D, 10) < 0 Then
            h_start_D := SubStr(h_dayofyear, 1, 10);
         End If;
         
         -- RULE: if start date moves foreword, than reset time to 00:00
         If (h_start_D IS NOT NULL) AND (PMO_ORASTD.strncmp(h_start_D, h_start_DT, 10) > 0) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_D, 1, 10)), PMO_ORASTD.TIME('00:00:00')) INTO h_start_DT
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'CHECK_ASSIGN', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: CHECK_ASSIGN(214) : cplan_fun.sql(5908) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'CHECK_ASSIGN', 222+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: CHECK_ASSIGN(223) : cplan_fun.sql(5917) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION CHECK_ASSIGN;
ALTER FUNCTION CHECK_ASSIGN COMPILE DEBUG;

-- RECOMMEND: Change the POSSTR() lines to these so that they are more restrictive

--  WHEN ASSIGN_NAMES = HOST(h_responsible_name) || ';'

--  THEN HOST(h_full_name) || ';'

--######################################################################################################

-- REVIEWED: August 3, 2004

-- RULES: for AssignResource

/*
   Return an existing resource assigment id or create a new one if it does
   not exist.

   Implement the following rules:

	  1. The first resource assignment to an element replaces the default
						 responsible (creator) if there is no effort
						 assigned to the default responsible

	  2. Update the ASSIGN_NAMES to contain the new resource name and
						 responsible, Responsible is the first name in the
						 list.

	  3. Return an error if assignment exists and no effort is assignable
						 at this element

*/

Prompt == FUNCTION AssignResource ==
CREATE OR REPLACE FUNCTION AssignResource
(
     sh_h_table_type PLS_INTEGER
   , sh_type_id001 PLS_INTEGER
   , id IN OUT NOCOPY VARCHAR2
   , res_id001 IN OUT NOCOPY VARCHAR2
   , rec_user001 IN OUT NOCOPY VARCHAR2
   , h_ra_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   type_id001 PLS_INTEGER := sh_type_id001;
   h_full_name VARCHAR2(255);
   h_responsible_name VARCHAR2(255);
   h_project_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_security_id CHAR(32);
   h_participant PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_tmp_security_id VARCHAR2(32);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_type_id PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfer_flag PLS_INTEGER := 0;
   null_return LONG;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('AssignResource') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('AssignResource'), '-- ' || SYSTIMESTAMP, ':r := AssignResource(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_type_id001), 'NULL')
          || ', ' || '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || ''''
          || ', ' || '''' || h_ra_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 48+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: AssignResource(49) : cplan_fun.sql(6005) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- to hold TRANSFER_FLAG corresponding to the transferd requirment
   
   h_resource_id := res_id001;
   h_element_id := id;
   h_rec_user := rec_user001;
   h_type_id := type_id001;
   h_participant := PMO_CNST.SC_PARTICIPANT;
   h_full_name := '';
   h_responsible_name := NULL;
   
   h_ra_id := SYS_GUID();
   
   BEGIN
      SELECT NICK_NAME INTO h_full_name
      FROM RESOURCES
      WHERE RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 75+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(76) : cplan_fun.sql(6032) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT PROFILE_NAME INTO h_full_name
         FROM PROFILES
         WHERE PROFILE_ID = h_resource_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 91+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(92) : cplan_fun.sql(6048) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 98+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(99) : cplan_fun.sql(6055) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
      
      --if (isChangeToTask(type_id)) return 0;
      
      h_res_type := '1';
   Else
      h_res_type := '0';
   End If;
   
   BEGIN
      SELECT RES_ASSIGNMENT_ID, PARTICIPANT 
      INTO h_res_assignment_id, h_temp_participant
      FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = h_element_id
      AND RESOURCE_TYPE = h_res_type
      AND RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(124) : cplan_fun.sql(6080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_ra_id := h_res_assignment_id;
      
      If (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS) OR (Coalesce(
             h_table_type, 0) = PMO_CNST.SC_TABLE_CRI) Then
         If isNotProjectType(Coalesce(h_type_id, 0)) <> 0 AND (BITAND(Coalesce(
               h_temp_participant, 0), PMO_CNST.SC_TEAMMEMBER)) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 136+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(137) : cplan_fun.sql(6093) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_ALREADY_ASSIGNED;
         End If;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 142+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(143) : cplan_fun.sql(6099) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 147+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: AssignResource(148) : cplan_fun.sql(6104) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_ALREADY_ASSIGNED;
      End If;
   End If;
   
   If (h_table_type IS NULL) OR Coalesce(h_table_type, 0) = 0 Then
      h_table_type := GetTable(Coalesce(h_type_id, 0));
   End If;
   
   h_transfer_flag := 0;
   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_POL THEN
         --TMT_POOLS
         BEGIN
            SELECT RANK INTO h_rank
            FROM TMT_POOLS
            WHERE ELEMENT_ID = h_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 171+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(172) : cplan_fun.sql(6128) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 178+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(179) : cplan_fun.sql(6135) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         BEGIN
            SELECT ELEMENT_ID INTO h_project_id
            FROM TMT_POOLS
            WHERE RANK = SUBSTR(h_rank, 1, 8)
            AND PARENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 194+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(195) : cplan_fun.sql(6151) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 201+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: AssignResource(202) : cplan_fun.sql(6158) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         End If;
         
         null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_PARTICIPANT_GROUP, h_security_id);
         
      
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         -- For transfered Requirement the type_id in TMT_WBS is 2 or SC_DELIVERABLE, to recognize a Deliverable from a Transfered
         -- requirement, the algorithm need TRANSFER_FLAG from TMT_WBS, if this Flag is 'R' then the object is a transfered requirement
         -- otherwise it is a DELIVERABLE
         -- Fetch TRANSFER_FLAG in TMT_WBS coressponding to the selected element
         
         BEGIN
            BEGIN
               SELECT PROJECT_ID, ASSIGN_COUNT, CASE WHEN TRANSFER_FLAG IS NuLL THEN 0 ELSE 1 END, TYPE_ID 
               		INTO h_project_id, h_assign_count, h_transfer_flag, h_type_id 
               		FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 227+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(228) : cplan_fun.sql(6184) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 233+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(234) : cplan_fun.sql(6190) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID, ASSIGN_COUNT, CASE WHEN WBS_ID IS NuLL THEN 0 ELSE 1 END, TYPE_ID 
               		INTO h_project_id, h_assign_count, h_transfer_flag, h_type_id 
               		FROM TMT_CRI WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 252+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(253) : cplan_fun.sql(6209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 258+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(259) : cplan_fun.sql(6215) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(276) : cplan_fun.sql(6232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(282) : cplan_fun.sql(6238) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 298+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(299) : cplan_fun.sql(6255) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 304+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(305) : cplan_fun.sql(6261) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 321+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(322) : cplan_fun.sql(6278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 327+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(328) : cplan_fun.sql(6284) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 344+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(345) : cplan_fun.sql(6301) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 350+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(351) : cplan_fun.sql(6307) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_CLIENT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 367+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(368) : cplan_fun.sql(6324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 373+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(374) : cplan_fun.sql(6330) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_ASSET_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 390+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(391) : cplan_fun.sql(6347) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 396+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(397) : cplan_fun.sql(6353) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            BEGIN
               SELECT PROJECT_ID INTO h_project_id 
               	  FROM TMT_VENDOR_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 413+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(414) : cplan_fun.sql(6370) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 419+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: AssignResource(420) : cplan_fun.sql(6376) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
         END;
      ELSE
         NULL;
   END CASE;
   
   If (SubStr(h_res_type, 1, 1) = '0') AND (Coalesce(h_assign_count, 0) < 2) Then
      -- Resource type and there are no assignments other than responsible
      -- RULE: first assignment to element replaces default responsible (creator)
      
      -- REDUNDANT: h_assign_count < 2 so it is 1 from above if ASSIGN_COUNT is valid
      BEGIN
         SELECT COUNT(*) INTO h_assign_count
         FROM RESOURCE_ASSIGNMENTS
         WHERE ELEMENT_ID = h_element_id
         AND RESOURCE_TYPE = '0';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 444+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(445) : cplan_fun.sql(6401) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_assign_count, 0) = 1 Then
         -- this is the first resource assignment to this element_id
         
         -- for transfered requirement, SC_WBS_CHANGE_REQUEST, SC_WBS_RISK, SC_WBS_ISSUE
         -- the new resource will be added to the resource assignment list, not replaced by default resource
         -- also, an element by type SC_DELIVERABLE and  TRANSFER_FLAG = 'R' is a transfered requirement
         If Coalesce(h_type_id, 0) = PMO_CNST.SC_DELIVERABLE OR Coalesce(
               h_type_id, 0) = PMO_CNST.SC_WORK_PRODUCT OR Coalesce(
               h_type_id, 0) = PMO_CNST.SC_SUMMARY_TASK Then
            null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            
            -- check to see if the responsible is the original creator of the element
            BEGIN
               SELECT 
               		  ra.RES_ASSIGNMENT_ID 
               		, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')) 
               	INTO 
               		  h_res_assignment_id 
               		, h_responsible_name 
               	FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra 
               	WHERE tw.ELEMENT_ID = h_element_id 
               			AND tw.ELEMENT_ID = ra.ELEMENT_ID 
               			AND ra.SECURITY_ID = h_security_id 
               			AND ra.RESOURCE_ID = tw.CREATED_BY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 480+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(481) : cplan_fun.sql(6437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (h_res_assignment_id IS NOT NULL) Then
               -- RULE: first assignment replaces the default responsible (creator) on this element
               h_participant := PMO_CNST.SC_DLV_MANAGER;
               null_return := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id, 1);
               -- delete right after updating PROJECT_ASSIGMENTS
            End If;
         ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_TASK OR Coalesce(
               h_type_id, 0) = PMO_CNST.SC_MILESTONE Then
            null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            
            -- check if has default responsible with no effort assigned
            BEGIN
               SELECT 
               		ra.RES_ASSIGNMENT_ID 
               		, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')) 
               		, tw.ASSIGN_COUNT 
               	INTO 
               		h_res_assignment_id 
               		, h_responsible_name 
               		, h_count 
               	FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra 
               	WHERE tw.ELEMENT_ID = h_element_id 
               			AND tw.ELEMENT_ID = ra.ELEMENT_ID 
               			AND ra.SECURITY_ID = h_security_id 
               			AND ra.RESOURCE_ID = tw.CREATED_BY 
               			AND NOT EXISTS( 
               					SELECT TSK_ASSIGNMENT_ID 
               					FROM TASK_ASSIGNMENTS ta 
               					WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 521+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(522) : cplan_fun.sql(6478) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (h_res_assignment_id IS NOT NULL) Then
               -- RULE: assignment replaces the default responsible (creator) if creator has no
               -- assigned effort on this element
               h_participant := PMO_CNST.SC_TASK_RESPONSIBLE;
               null_return := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id, 1);
            End If;
         ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_CRI_MILESTONE OR Coalesce(
               h_type_id, 0) = PMO_CNST.SC_CRI_TASK Then
            null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            
            BEGIN
               SELECT 
               		ra.RES_ASSIGNMENT_ID 
               		, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')) 
               		, tw.ASSIGN_COUNT 
               	INTO 
               		h_res_assignment_id 
               		, h_responsible_name 
               		, h_count 
               	FROM TMT_CRI tw, RESOURCE_ASSIGNMENTS ra 
               	WHERE tw.ELEMENT_ID = h_element_id 
               			AND tw.ELEMENT_ID = ra.ELEMENT_ID 
               			AND ra.SECURITY_ID = h_security_id 
               			AND ra.RESOURCE_ID = tw.CREATED_BY 
               			AND NOT EXISTS( 
               					SELECT TSK_ASSIGNMENT_ID 
               					FROM TASK_ASSIGNMENTS ta 
               					WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 561+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(562) : cplan_fun.sql(6518) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (h_res_assignment_id IS NOT NULL) Then
               -- RULE: assignment replaces the default responsible (creator) if creator has no
               -- assigned effort on this element
               h_participant := PMO_CNST.SC_TASK_RESPONSIBLE;
               
               null_return := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id, 1);
            End If;
         ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_CRI_SUMMARY_TASK OR Coalesce(
               h_type_id, 0) = PMO_CNST.SC_CRI_DELIVERABLE Then
            -- for SC_CHANGE_REQUEST, SC_RISK, SC_ISSUE the algorithm add the new resource
            -- to the existing default resource, but for the other types the assigned resource
            -- replaced with the default resource
            null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
            
            BEGIN
               SELECT 
               		  ra.RES_ASSIGNMENT_ID 
               		, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';')) 
               	INTO 
               		  h_res_assignment_id 
               		, h_responsible_name 
               	FROM TMT_CRI tw, RESOURCE_ASSIGNMENTS ra 
               	WHERE tw.ELEMENT_ID = h_element_id 
               			AND tw.ELEMENT_ID = ra.ELEMENT_ID 
               			AND ra.SECURITY_ID = h_security_id 
               			AND ra.RESOURCE_ID = tw.CREATED_BY;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 599+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(600) : cplan_fun.sql(6556) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (h_res_assignment_id IS NOT NULL) Then
               -- RULE: first assignment replaces the default responsible (creator) on this element
               h_participant := PMO_CNST.SC_DLV_MANAGER;
               
               null_return := FC_BEFORE_D_RES_ASSIGN(h_res_assignment_id, 1);
            End If;
         End If;
      End If;
   End If;
   
   -- update ASSIGN_NAMES when responsible was changed from default responsible
   -- or a new responsible was assigned.
   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_WBS THEN
         If Coalesce(h_participant, 0) = PMO_CNST.SC_DLV_MANAGER OR Coalesce(
               h_participant, 0) = PMO_CNST.SC_TASK_RESPONSIBLE Then
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES = 
               		CASE 
               			WHEN InStr(ASSIGN_NAMES, h_responsible_name) > 0 
               				THEN REPLACE(ASSIGN_NAMES, h_responsible_name, h_full_name || ';') 
               			ELSE h_full_name || ';' || ASSIGN_NAMES 
               		END 
               	WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 638+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(639) : cplan_fun.sql(6595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES = 
               		CASE 
               			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
               			ELSE 
               			CASE 
               				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
               				ELSE ASSIGN_NAMES || h_full_name || ';' 
               			END 
               		END 
               	WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 664+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(665) : cplan_fun.sql(6621) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         If Coalesce(h_participant, 0) = PMO_CNST.SC_DLV_MANAGER OR Coalesce(
               h_participant, 0) = PMO_CNST.SC_TASK_RESPONSIBLE Then
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES = 
               		CASE 
               			WHEN InStr(ASSIGN_NAMES, h_responsible_name) > 0 
               				THEN REPLACE(ASSIGN_NAMES, h_responsible_name, h_full_name || ';') 
               			ELSE h_full_name || ';' || ASSIGN_NAMES 
               		END 
               	WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 692+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(693) : cplan_fun.sql(6649) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES = 
               		CASE 
               			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
               			ELSE 
               			CASE 
               				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
               				ELSE ASSIGN_NAMES || h_full_name || ';' 
               			END 
               		END 
               	WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 718+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(719) : cplan_fun.sql(6675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            UPDATE TMT_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 747+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(748) : cplan_fun.sql(6704) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_POL THEN
         BEGIN
            UPDATE TMT_POOLS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 774+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(775) : cplan_fun.sql(6731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            UPDATE TMT_POOL_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 801+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(802) : cplan_fun.sql(6758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            UPDATE TMT_RESOURCE_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 828+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(829) : cplan_fun.sql(6785) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            UPDATE TMT_CLIENT_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 855+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(856) : cplan_fun.sql(6812) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            UPDATE TMT_ASSET_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 882+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(883) : cplan_fun.sql(6839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            UPDATE TMT_VENDOR_DOCUMENTS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 909+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(910) : cplan_fun.sql(6866) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET ASSIGN_NAMES = 
            		CASE 
            			WHEN InStr(ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES 
            			ELSE 
            			CASE 
            				WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES 
            				ELSE ASSIGN_NAMES || h_full_name || ';' 
            			END 
            		END 
            	WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 936+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(937) : cplan_fun.sql(6893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ELSE
         NULL;
   END CASE;
   
   If Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
      null_return := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                             , PMO_CNST.SC_PARTICIPANT_GROUP, h_security_id);
   End If;
   
   If Coalesce(h_transfer_flag, 0) = 1 Then
      null_return := FC_GET_CRI_SECURITY_ID(h_element_id, h_type_id, h_security_id, h_cri_security_id);
   Else
      h_cri_security_id := NULL;
   End If;
   
   h_temp_participant := PMO_CNST.SC_PARTICIPANT;
   h_temp_security_id := h_security_id;
   h_temp_bitflag := 1;
   
   h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_resource_id, h_type_id, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
   
   BEGIN
      INSERT INTO RESOURCE_ASSIGNMENTS
      (
      RES_ASSIGNMENT_ID
      , PROJECT_ID
      , ELEMENT_ID
      , ELEMENT_TYPE
      , RESOURCE_TYPE
      , RESOURCE_ID
      , PARTICIPANT
      , SECURITY_ID
      , CRI_SECURITY_ID
      , REC_USER
      , REC_DATETIME
      , TABLE_TYPE
      )
      VALUES
      (
      h_ra_id
      , h_project_id
      , h_element_id
      , h_type_id
      , h_res_type
      , h_resource_id
      , h_participant
      , h_security_id
      , h_cri_security_id
      , h_rec_user
      , SYSTIMESTAMP
      , h_table_type
      );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'AssignResource', 1003+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: AssignResource(1004) : cplan_fun.sql(6960) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'AssignResource', 1009+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: AssignResource(1010) : cplan_fun.sql(6966) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION AssignResource;
ALTER FUNCTION AssignResource COMPILE DEBUG;

/*************************************************************************************************************************************************************/

Prompt == FUNCTION U_TSKRES2 ==
CREATE OR REPLACE FUNCTION U_TSKRES2
(
     h_assign_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , and_roles IN OUT NOCOPY VARCHAR2
   , and_skills IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , h_res_type IN OUT NOCOPY VARCHAR2
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_contour_type PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , h_assign_string IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_per_day PLS_INTEGER
   , sh_p_reg_sale NUMBER := 0.00
   , sh_p_spec_sale NUMBER := 0.00
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_contour_type PLS_INTEGER := sh_h_contour_type;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_consider PLS_INTEGER := sh_h_consider;
   h_per_day PLS_INTEGER := sh_h_per_day;
   p_reg_sale NUMBER := Coalesce(sh_p_reg_sale, 0);
   p_spec_sale NUMBER := Coalesce(sh_p_spec_sale, 0);
   h_created_by VARCHAR2(32);
   h_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_PA_SD DATE;
   h_PA_FD DATE;
   h_dayofyear DATE;
   h_first_time VARCHAR2(8);
   h_role_id VARCHAR2(32);
   h_and_roles VARCHAR2(4000);
   h_and_skills VARCHAR2(4000);
   h_billable PLS_INTEGER := 0;
   h1_tsk_assignment_id VARCHAR2(32);
   h1_project_id VARCHAR2(32);
   h_res1_id VARCHAR2(32);
   h_min VARCHAR2(19);
   tempStr VARCHAR2(255);
   h_labor_code VARCHAR2(50);
   h_full_name VARCHAR2(255);
   h_creator_name VARCHAR2(255);
   h_timecode_1_id VARCHAR2(32);
   h_timecode_2_id VARCHAR2(32);
   h_same_labor PLS_INTEGER := 0;
   h_fs_project PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_transfered_cri PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_assigned PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_planned_work PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_spec_cost NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_rate_echange NUMBER := 1.00;
   h_prj_rate_echange NUMBER := 1.00;
   h_budget NUMBER := 0.00;
   h_proj_assign_flag PLS_INTEGER := 0;
   h_change_flag PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_u_etc PLS_INTEGER := 0;
   h_rate_adjustment PLS_INTEGER := 0;
   h_pa_assign_exists PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_sc_state PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_labor_cost_cc_id VARCHAR2(32);
   h_labor_revenue_cc_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_temp_security_id VARCHAR2(32);
   h_pa_timephase VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_rate_edited PLS_INTEGER := 0;
   h_old_type_id PLS_INTEGER := 0;
   assign_to_prj PLS_INTEGER := 0;
   h_bitID RAW(36) := PMO_CNST.SC_ZERO_BIT;
   h_bitID0 RAW(36) := PMO_CNST.SC_ZERO_BIT;
   l_data_flag PLS_INTEGER;
   token LONG;
   pos LONG;
   result PLS_INTEGER;
   numItems PLS_INTEGER;
-- Declare Cursors
      CURSOR C43 IS
         SELECT TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE  ROWNUM <= 1 AND  PROJECT_ID = h_project_id
         AND PROJECT_ID <> TASK_ID
         AND RESOURCE_ID = h_res_id
         AND PROJECT_ASSIGNMENT_ID IS NuLL;

      CURSOR C41 IS
         SELECT PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '0';

      CURSOR C40 IS
         SELECT PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := U_TSKRES2(' || 
       '''' || h_assign_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || '''' || h_assign_string || ''''
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 168+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: U_TSKRES2(169) : cplan_fun.sql(7145) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_update := 0;
   h_table_type := 0;
   h_task_type := 0;
   
   h_timecode_1_id := NULL;
   h_timecode_2_id := NULL;
   h_prj_assignment_id := NULL;
   h_labor_cost_cc_id := NULL;
   h_labor_revenue_cc_id := NULL;
   
   h_billable := 0;
   h_capital_expense := 0;
   h_rate_adjustment := 10000;
   
   If Coalesce(Length(h_assign_string), 0) > PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 189+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(190) : cplan_fun.sql(7166) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   assign_to_prj := 1;
   h_SD := NULL;
   h_FD := NULL;
   
   assign_to_prj := PMO_ORASTD.strcmp(h_project_id, h_task_id);
   
   If Coalesce(h_consider, 0) >= PMO_CNST.SC_BIT_30 Then
      h_assigned := 1;
      -- this is a search assign (type > 0)
      h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove search assign (type > 0)
   Else
      h_assigned := 0;
      -- this is not a search assign (type == 0)
   End If;
   
   BEGIN
      SELECT PMO_ORASTD.DAYS(h_finish_date) - PMO_ORASTD.DAYS(h_start_date)
      INTO h_temp_var
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 220+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(221) : cplan_fun.sql(7197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_var, 0) < 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 227+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(228) : cplan_fun.sql(7204) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
   End If;
   
   h_bit_flag := CASE WHEN Coalesce(h_work_type, 0) = 2 THEN 2 ELSE 0 END;
   h_u_etc := 0;
   If Coalesce(h_bitflag, 0) = 4 Then
      h_rollup := 3;
      If Coalesce(h_work_type, 0) = 2 Then
         h_bitflag := 2;
      Else
         h_bitflag := 0;
      End If;
      
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 0) AND (Coalesce(h_per_day, 0) > 0) Then
         h_percent_units := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.499);
      End If;
      
      h_tsk_assignment_id := SubStr(h_assign_id, 1, 32);
      -- TSK_ASSIGNMENT_ID
      h_prj_assignment_id := h_tsk_assignment_id;
   Else
      h_rollup := 0;
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
         h_from_prj := 1;
         -- set assign from constrained by project (type > 0)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
         -- remove assign from constrained by project (type > 0)
         
         If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
            h_rollup := 3;
            -- set do not rollup
            h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29));
            -- remove do not rollup
         End If;
      End If;
      
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
         h_rollup := 2;
         -- set do not rollup (import from MSP)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
         -- remove do not rollup
      ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27 Then
         h_rollup := 2;
         -- set do not rollup (import from MSP)
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
         -- remove do not rollup
         h_u_etc := 1;
      End If;
      
      If (Coalesce(h_rollup, 0) = 3) OR (Coalesce(h_assigned, 0) = 1) Then
         h_tsk_assignment_id := h_assign_id;
         -- TSK_ASSIGNMENT_ID
         If Coalesce(h_assigned, 0) = 1 Then
            BEGIN
               SELECT RES_ASSIGNMENT_ID, ELEMENT_TYPE, TABLE_TYPE, PARTICIPANT, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
               INTO h_assign_id,h_type_id, h_table_type, h_participant, h_security_id, h_transfered_cri
               FROM RESOURCE_ASSIGNMENTS
               WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = h_res_type;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 293+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(294) : cplan_fun.sql(7270) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_security_id: %s, h_type_id: %d, h_table_type: %d:%d, h_participant: %d, h_transfered_cri: %d\n",SC_STATE,h_security_id,h_type_id,h_table_type_Ind,h_table_type,h_participant,h_transfered_cri);
            --trace_printf("h_task_id: %d:%s, h_res_id: %d:%s\n",h_task_id_Ind,h_task_id,h_res_id_Ind,h_res_id);
            
            If ((h_table_type IS NULL)) OR (Coalesce(h_table_type, 0) = 0) Then
               h_table_type := GetTable(Coalesce(h_type_id, 0));
            End If;
         End If;
      Else
         If Coalesce(h_update, 0) = 0 Then
            --Get a unique ID
            h_tsk_assignment_id := SYS_GUID();
            h_prj_assignment_id := h_tsk_assignment_id;
            
            BEGIN
               SELECT ELEMENT_TYPE, TABLE_TYPE, PARTICIPANT, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
               INTO h_type_id, h_table_type, h_participant, h_security_id, h_transfered_cri
               FROM RESOURCE_ASSIGNMENTS
               WHERE RES_ASSIGNMENT_ID = h_assign_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 321+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(322) : cplan_fun.sql(7298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_security_id: %s, h_type_id: %d, h_table_type: %d:%d, h_participant: %d, h_transfered_cri: %d\n",SC_STATE,h_security_id,h_type_id,h_table_type_Ind,h_table_type,h_participant,h_transfered_cri);
            
            If ((h_table_type IS NULL)) OR (Coalesce(h_table_type, 0) = 0) Then
               h_table_type := GetTable(Coalesce(h_type_id, 0));
            End If;
         End If;
         --trace_printf("h_update: %d, h_type_id: %d, h_table_type: %d:%d, h_participant: %d\n",h_update,h_type_id,h_table_type_Ind,h_table_type,h_participant);
      End If;
      
      If isChangeToTask(Coalesce(h_type_id, 0)) <> 0 Then
         If isPureCRIDeliv(Coalesce(h_type_id, 0)) <> 0 Then
            h_id := NULL;
            BEGIN
               SELECT ELEMENT_ID INTO h_id
               FROM PMO_DEFAULTS
               WHERE PROJECT_ID = h_project_id AND TYPE_ID = h_type_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 347+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(348) : cplan_fun.sql(7324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_id: %d %s\n", SC_STATE,h_id_Ind,h_id);
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(
                  h_task_id, h_id) = 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 356+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(357) : cplan_fun.sql(7333) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI;
            End If;
         End If;
         
         h_temp_sc_state := FC_CHANGE_TYPE_BY_ASSIGN(h_type_id, h_task_id, h_rec_user);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 366+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(367) : cplan_fun.sql(7343) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         If BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_PARTICIPANT) <> 0 Then
            h_participant := Coalesce(h_participant, 0) - PMO_CNST.SC_PARTICIPANT;
         End If;
         
         h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
         
         If Coalesce(h_transfered_cri, 0) = 0 Then
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
            
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id
               WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 394+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(395) : cplan_fun.sql(7371) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
            h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
            
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
               WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 417+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(418) : cplan_fun.sql(7394) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      ElsIf NOT (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER)) <> 0 Then
         If Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
            h_participant := PMO_CNST.SC_TEAMMEMBER;
            If Coalesce(h_transfered_cri, 0) = 0 Then
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
               
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 442+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(443) : cplan_fun.sql(7419) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_security_id);
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), FC_MIRROR_TYPE(h_type_id)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
               
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant, SECURITY_ID= h_security_id, CRI_SECURITY_ID= h_temp_security_id
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 465+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(466) : cplan_fun.sql(7442) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            h_participant := PMO_FUNC.BIT_OR(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER);
            
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = h_participant
               WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 486+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(487) : cplan_fun.sql(7463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If Coalesce(h_bitflag, 0) = 3 Then
         h_bitflag := 2;
      End If;
      
      If (Coalesce(h_from_prj, 0) = 0) AND (Coalesce(assign_to_prj, 0) = 0) Then
         OPEN C43;
         
         FETCH C43 INTO h_id;
         IF C43%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               CLOSE C43 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 516+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(517) : cplan_fun.sql(7493) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 522+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(523) : cplan_fun.sql(7499) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_;
         Else
            BEGIN
               CLOSE C43 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 534+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(535) : cplan_fun.sql(7511) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, RATE_ADJUSTMENT, REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
            INTO h_prj_assignment_id, h_rate_adjustment, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
            FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id
            AND TASK_ID = h_project_id
            AND RESOURCE_ID = h_res_id AND BIT_FLAG = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 553+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(554) : cplan_fun.sql(7530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_prj_assignment_id IS NOT NULL)) Then
            h_tsk_assignment_id := h_prj_assignment_id;
            h_pa_assign_exists := 1;
         Else
            h_prj_assignment_id := h_tsk_assignment_id;
         End If;
      Else
         h_prj_assignment_id := NULL;
         If Coalesce(h_from_prj, 0) = 1 Then
            BEGIN
               SELECT
               TSK_ASSIGNMENT_ID,
               TO_DATE(SubStr(UNRESOLVED_START, 1, 10)),
               TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)),
               CAPITAL_EXPENSE,
               BILLABLE,
               REGULAR_RATE,
               SPECIAL_RATE,
               REGULAR_COST,
               SPECIAL_COST,
               RATE_ADJUSTMENT,
               LABOR_COST_CC_ID,
               LABOR_REVENUE_CC_ID
               INTO
               h_prj_assignment_id,
               h_PA_SD, h_PA_FD,
               h_capital_expense,
               h_billable,
               h_reg_sale ,
               h_spec_sale,
               h_reg_cost ,
               h_spec_cost,
               h_rate_adjustment,
               h_labor_cost_cc_id,
               h_labor_revenue_cc_id
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND BIT_FLAG = h_bit_flag;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 603+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(604) : cplan_fun.sql(7580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("h_work_type = %d, h_reg_sale = %.2f, SC_STATE = %d, h_prj_assignment_id = %s\nh_PA_SD = %s, h_PA_FD = %s,h_billable: %d\n", h_work_type,h_reg_sale,SC_STATE,h_prj_assignment_id,h_PA_SD,h_PA_FD,h_billable);
         Else
            BEGIN
               SELECT
               TSK_ASSIGNMENT_ID,
               TO_DATE(SubStr(COALESCE(SCHED_START,PLANNED_START), 1, 10)),
               TO_DATE(SubStr(COALESCE(SCHED_FINISH,PLANNED_FINISH), 1, 10)),
               BILLABLE,
               CAPITAL_EXPENSE,
               CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(BAK_START_DT, 1, 10))
               AND TO_DATE(SubStr(h_finish_date, 1, 10)) <= TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)) THEN 0
               ELSE 1 END,
               REGULAR_RATE,
               SPECIAL_RATE,
               REGULAR_COST,
               SPECIAL_COST,
               RATE_ADJUSTMENT ,
               LABOR_COST_CC_ID,
               LABOR_REVENUE_CC_ID
               INTO
               h_prj_assignment_id,
               h_PA_SD,
               h_PA_FD,
               h_billable,
               h_capital_expense,
               h_temp_var,
               h_reg_sale,
               h_spec_sale,
               h_reg_cost,
               h_spec_cost,
               h_rate_adjustment,
               h_labor_cost_cc_id,
               h_labor_revenue_cc_id
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id
               AND RESOURCE_ID=h_res_id
               AND BIT_FLAG = h_bit_flag;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 650+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(651) : cplan_fun.sql(7627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_prj_assignment_id IS NOT NULL)) 
                  AND ((Coalesce(h_temp_var, 0) = 0) OR (Coalesce(h_rollup, 0) = 2)) Then
               h_from_prj := 1;
            Else
               h_from_prj := 0;
               h_prj_assignment_id := NULL;
            End If;
         End If;
      End If;
      
      h_cri := 0;
      h_priority := 0;
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         BEGIN
            SELECT CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), tw.PRIORITY, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4), tw.CURRENCY_ID
            INTO h_fs_project, h_min_per_day, h_priority, h_rate_echange, h_currency_id
            FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
            WHERE tw.ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 680+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(681) : cplan_fun.sql(7657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT CURRENCY_ID INTO h_currency_id
            FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 694+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(695) : cplan_fun.sql(7671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), tw.PRIORITY,
            PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4), CASE WHEN (h_currency_id = tw.CURRENCY_ID) OR (h_currency_id * tw.CURRENCY_ID = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = tw.CURRENCY_ID) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) END
            INTO h_fs_project, h_min_per_day, h_priority,
            h_prj_rate_echange, h_rate_echange
            FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
            WHERE tw.ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 712+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(713) : cplan_fun.sql(7689) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_labor_code := '';
      -- Labor Code
      
      If (Coalesce(assign_to_prj, 0) <> 0) AND (SubStr(h_res_type, 1, 1) = '0') Then
         If Coalesce(h_fs_project, 0) = 1 Then
            BEGIN
               SELECT FS_LABOR_CODE INTO h_labor_code FROM RESOURCES WHERE RESOURCE_ID=h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 731+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(732) : cplan_fun.sql(7708) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE = %d, h_labor_code = %d, %s, h_res_id = %s\n", SC_STATE,h_labor_code_Ind,h_labor_code,h_res_id);
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_labor_code IS NOT NULL)) Then
               BEGIN
                  SELECT CASE WHEN FS_LABOR_CODE IS NuLL THEN 0
                  WHEN FS_LABOR_CODE = h_labor_code THEN 1 ELSE 2 END
                  INTO h_same_labor FROM TMT_WBS WHERE ELEMENT_ID=h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 747+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(748) : cplan_fun.sql(7724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE = %d, h_task_id = %s, h_same_labor = %d\n", SC_STATE,h_task_id,h_same_labor);
               
               If (Coalesce(h_rollup, 0) = 2) OR ((PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(
                     h_same_labor, 0) = 0)) Then
                  BEGIN
                     UPDATE TMT_WBS SET FS_LABOR_CODE = h_labor_code WHERE ELEMENT_ID=h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 767+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(768) : cplan_fun.sql(7744) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_same_labor, 0) = 2) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 773+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(774) : cplan_fun.sql(7750) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CANNOT_ASSIGN_LABOR_CODE;
               End If;
               --trace_printf("SC_STATE = %d, h_same_labor = %d\n", SC_STATE,h_same_labor);
            End If;
         End If;
      End If;
      
      If (Coalesce(h_bitflag, 0) = 0) OR (Coalesce(h_bitflag, 0) = 2) Then
         If Coalesce(h_from_prj, 0) = 0 Then
            BEGIN
               SELECT ASSIGNMENT_BILLABLE_FLAG, TIMEPHASE_PROJECT_ASSIGNMENT INTO h_billable, h_pa_timephase
               FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 793+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(794) : cplan_fun.sql(7770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase
               FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 807+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(808) : cplan_fun.sql(7784) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_update := 0;
         
         /*if (h_unit == 101)
         			{
         				h_min_per_day = h_eunit;
         				h_percent_units = h_work;
         				h_work = 0;
         				h_unit = 0;
         				h_task_type = SC_FD_EFFORT_DRIVEN;
         			}
         			else */
         If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 100) Then
            h_percent_units := Coalesce(h_work, 0);
            h_work := 0;
            h_unit := 0;
         ElsIf (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 0) AND (Coalesce(h_per_day, 0) > 0) Then
            h_percent_units := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.499);
         Else
            h_percent_units := 10000;
         End If;
         
         --trace_printf("B_** h_percent_units = %d, h_work: %d, h_unit: %d\n", h_percent_units,h_work,h_unit);
         
         l_data_flag := 0;
         
         h_temp_sc_state := 0;
         If Coalesce(h_reg_cost, 0) + Coalesce(h_spec_cost, 0) + Coalesce(h_reg_sale, 0) + Coalesce(
               h_spec_sale, 0) = 0.0 Then
            h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, and_roles, and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
         End If;
         
         If p_reg_sale + p_spec_sale > 0.0 Then
            h_reg_sale := p_reg_sale;
            h_spec_sale := p_spec_sale;
         End If;
         
         -- if user assigned from project_assignments and the cost and rate are not zero (it menas user changed them)
         -- then set RATES_EDITED field in TASK_ASSIGNMENTS to 3
         h_rate_edited := 0;
         If (l_data_flag = 1) AND (Coalesce(h_reg_cost, 0) + Coalesce(h_spec_cost, 0) + Coalesce(
               h_reg_sale, 0) + Coalesce(h_spec_sale, 0) <> 0.0) Then
            h_rate_edited := 3;
         End If;
         
         If (Coalesce(h_from_prj, 0) = 0) AND (Coalesce(h_temp_sc_state, 0) = 0) Then
            If l_data_flag = 0 Then
               h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
               h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
               h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
               h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            ElsIf l_data_flag = 2 Then
               h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_prj_rate_echange, 0);
               h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_prj_rate_echange, 0);
               h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_prj_rate_echange, 0);
               h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_prj_rate_echange, 0);
            End If;
         End If;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_labor_code := NULL;
         End If;
         
         If (Coalesce(h_pa_assign_exists, 0) = 0) AND (Coalesce(h_rollup, 0) <> 3) Then
            --if ( (h_work_type == 2) || (h_bitflag == 2))
            If Coalesce(h_work_type, 0) = 2 Then
               BEGIN
                  INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                  SCHED_START, SCHED_FINISH, SCHED_WORK, BIT_FLAG, ETC, BAK_START_DT, BAK_FINISH_DT, BAK_WORK,
                  RESOURCE_ID, RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                  T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                  REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                  PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE, RATES_EDITED,
                  TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT, LABOR_COST_CC_ID, LABOR_REVENUE_CC_ID )
                  VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                  h_start_date, h_finish_date, h_effort, '2', h_effort,
                  h_start_date, h_finish_date, h_effort, h_res_id,h_res_type, h_assign_string, h_assign_string,
                  h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                  ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                  h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                  h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment, h_labor_cost_cc_id, h_labor_revenue_cc_id );
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 904+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(905) : cplan_fun.sql(7881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d\n", SC_STATE,h_start_date,h_finish_date,h_effort);
            Else
               If SubStr(h_res_type, 1, 1) = '1' Then
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, BEST, BIT_FLAG, ETC,
                     BAK_START_DT, BAK_FINISH_DT, BAK_WORK, RESOURCE_ID,RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                     T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                     PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE , RATES_EDITED,
                     TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT, LABOR_COST_CC_ID, LABOR_REVENUE_CC_ID )
                     VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                     h_start_date, h_finish_date, h_effort, h_effort, '0', h_effort,
                     h_start_date, h_finish_date, h_effort, h_res_id, h_res_type, h_assign_string, h_assign_string,
                     h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                     ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                     h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                     h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment, h_labor_cost_cc_id, h_labor_revenue_cc_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 936+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(937) : cplan_fun.sql(7913) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, BEST, BIT_FLAG, ETC, BAK_START_DT, BAK_FINISH_DT, BAK_WORK,
                     RESOURCE_ID,RESOURCE_TYPE, ASSIGN_STRING, BAK_ASSIGN_STRING,
                     T_UNIT, C_EFFORT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, C_OCCURRENCE,
                     REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE, UNITS_MSP,
                     PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, TABLE_TYPE, RATES_EDITED,
                     TIMECODE_1_ID, TIMECODE_2_ID, CONSIDER, RATE_ADJUSTMENT, LABOR_COST_CC_ID, LABOR_REVENUE_CC_ID )
                     VALUES(h_tsk_assignment_id, h_assign_id, h_task_id, h_project_id,
                     h_start_date, h_finish_date, h_effort, h_effort, '0', h_effort,
                     h_start_date, h_finish_date, h_effort, h_res_id,h_res_type, h_assign_string, h_assign_string,
                     h_eunit, h_work, h_unit, h_contour, h_contour_type, h_occurrence,
                     ROUND(h_reg_cost,2), ROUND(h_reg_sale,2), ROUND(h_spec_cost,2), ROUND(h_spec_sale,2), h_percent_units,
                     h_prj_assignment_id, h_billable, h_capital_expense, h_table_type, h_rate_edited,
                     h_timecode_1_id, h_timecode_2_id, h_consider, h_rate_adjustment, h_labor_cost_cc_id, h_labor_revenue_cc_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 966+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(967) : cplan_fun.sql(7943) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("3__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d\n", SC_STATE,h_start_date,h_finish_date,h_effort);
            End If;
         End If;
      ElsIf Coalesce(h_work_type, 0) = 0 Then
         h_tsk_assignment_id := h_assign_id;
         -- TSK_ASSIGNMENT_ID
         h_update := 1;
         -- UPDATE
         
         h_planned_work := 0;
         BEGIN
            SELECT PROJECT_ASSIGNMENT_ID, PLANNED_WORK, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_planned_work, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 992+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(993) : cplan_fun.sql(7969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_SD: %s, h_tsk_assignment_id: %s, h_planned_work: %d\nh_billable: %d, h_prj_assignment_id: %d:%s\n",SC_STATE,h_SD,h_tsk_assignment_id,h_planned_work,h_billable,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_SD IS NOT NULL)) Then
            If (h_prj_assignment_id IS NULL) Then
               h_from_prj := 0;
            Else
               h_from_prj := 1;
            End If;
            
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
         
         If (Coalesce(h_rollup, 0) = 2) AND (Coalesce(h_work, 0) > 0) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START       = h_start_date,
               PLANNED_FINISH      = h_finish_date,
               PLANNED_WORK        = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR = h_contour, C_UNIT = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '0'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1035+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1036) : cplan_fun.sql(8012) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_START       = h_start_date,
               PLANNED_FINISH      = h_finish_date,
               PLANNED_WORK        = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR = h_contour, C_UNIT = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '0'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1063+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1064) : cplan_fun.sql(8040) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("4__ SC_STATE = %d, h_start_date: %s, h_finish_date: %s, h_effort = %d\n", SC_STATE,h_effort,h_start_date,h_finish_date);
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If Coalesce(h_from_prj, 0) = 1 Then
               OPEN C41;
               LOOP
                  FETCH C41 INTO h_id, h_dayofyear, h_minutes, h_bitID;
                  IF C41%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1097+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1098) : cplan_fun.sql(8074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1.1_TDY_SC_STATE = %d, h_id = %s, h_dayofyear = %s, h_minutes = %d\n", SC_STATE,h_id,h_dayofyear,h_minutes);//trace_flush();
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '0';
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1116+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1117) : cplan_fun.sql(8093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               END LOOP;
               BEGIN
                  CLOSE C41 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1130+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1131) : cplan_fun.sql(8107) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '0';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1148+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1149) : cplan_fun.sql(8125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1__DELETE FROM TASK_DAY_OF_YEAR: h_tsk_assignment_id: %s\n",h_tsk_assignment_id);//trace_flush();
         End If;
      ElsIf Coalesce(h_work_type, 0) = 2 Then
         h_tsk_assignment_id := h_assign_id;
         -- TSK_ASSIGNMENT_ID
         
         BEGIN
            SELECT PROJECT_ASSIGNMENT_ID, SCHED_WORK, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_sched_work, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1170) : cplan_fun.sql(8146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_assignment_id = %s, h_sched_work: %d\nh_prj_assignment_id: %d:%s\n", SC_STATE,h_tsk_assignment_id,h_sched_work,h_prj_assignment_id_Ind,h_prj_assignment_id);//trace_flush();
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) OR ((h_SD IS NOT NULL)) Then
            If (h_prj_assignment_id IS NULL) Then
               h_from_prj := 0;
            Else
               h_from_prj := 1;
            End If;
            
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
         
         If Coalesce(h_from_prj, 0) = 0 Then
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1202) : cplan_fun.sql(8178) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("2__DELETE FROM TASK_DAY_OF_YEAR: h_tsk_assignment_id: %s\n",h_tsk_assignment_id);//trace_flush();
         Else
            OPEN C40;
            LOOP
               FETCH C40 INTO h_id, h_dayofyear, h_minutes, h_bitID;
               IF C40%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_minutes, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1232+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1233) : cplan_fun.sql(8209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear AND BIT_FLAG = '2';
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1251+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1252) : cplan_fun.sql(8228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C40 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1265+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1266) : cplan_fun.sql(8242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND BIT_FLAG = '2';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1282+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1283) : cplan_fun.sql(8259) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- Check if there is enough effort @ project
            h_total := 0;
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_total
               FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND BIT_FLAG = '2'
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1299+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1300) : cplan_fun.sql(8276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_rollup, 0) = 2) AND (Coalesce(h_work, 0) > 0) Then
            If Coalesce(h_total, 0) < Coalesce(h_effort, 0) Then
               h_total := Coalesce(h_effort, 0) - Coalesce(h_total, 0);
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET ETC = h_total
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1321+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1322) : cplan_fun.sql(8298) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  SCHED_START         = h_start_date,
                  SCHED_FINISH        = h_finish_date,
                  SCHED_WORK          = h_effort,
                  BAK_START_DT        = h_start_date,
                  BAK_FINISH_DT       = h_finish_date,
                  BAK_WORK            = h_effort,
                  ETC                 = CASE WHEN h_u_etc = 0 THEN h_effort ELSE ETC END,
                  C_EFFORT = h_work, C_CONTOUR    = h_contour,
                  C_UNIT   = h_unit, CONTOUR_TYPE = h_contour_type,
                  ASSIGN_STRING       = h_assign_string,
                  BAK_ASSIGN_STRING   = h_assign_string,
                  BIT_FLAG            = '2'
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1351+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1352) : cplan_fun.sql(8328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_START         = h_start_date,
               SCHED_FINISH        = h_finish_date,
               SCHED_WORK          = h_effort,
               BAK_START_DT        = h_start_date,
               BAK_FINISH_DT       = h_finish_date,
               BAK_WORK            = h_effort,
               ETC                 = CASE WHEN h_u_etc = 0 THEN h_effort ELSE ETC END,
               ASSIGN_STRING       = h_assign_string,
               BAK_ASSIGN_STRING   = h_assign_string,
               C_EFFORT = h_work, C_CONTOUR    = h_contour,
               C_UNIT   = h_unit, CONTOUR_TYPE = h_contour_type,
               BIT_FLAG            = '2'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1382+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1383) : cplan_fun.sql(8359) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_cri := (PMO_FUNC.BIT_OR(Coalesce(h_cri, 0), PMO_CNST.SC_BIT_30));
   End If;
   
   If (Coalesce(h_assigned, 0) = 0) AND (Coalesce(Length(h_assign_string), 0) > 0) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDOY';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         SELECT TO_DATE(SubStr(h_start_date, 1, 10)), PMO_ORASTD.TIME(h_start_date) INTO h_dayofyear, h_first_time
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1406+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1407) : cplan_fun.sql(8383) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         If FC_CREATE_CDY(h_project_id, h_start_date, h_finish_date, h_duration
               , PMO_CNST.SC_PROJECT) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 1415+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(1416) : cplan_fun.sql(8392) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
      Else
         If FC_CREATE_CDY(h_res_id, h_start_date, h_finish_date, h_duration, PMO_CNST.SC_RESOURCE) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 1422+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(1423) : cplan_fun.sql(8399) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
      End If;
      
      token := h_assign_string;
      pos := h_assign_string;
      tempStr := '';
      h_minutes := 0;
      
      LOOP
         token := PMO_STPU.REstrtok(NULL, PMO_CNST.SC_SEP, pos);
         If (token IS NULL) Then
            EXIT;
         End If;
         
         result := InStr(token, PMO_CNST.SC_STAR_CHAR);
         
         If result > 0 Then
            -- Get next assignment
            h_count := Length(token) - result;
            tempStr := SubStr(SubStr(token, 1, result - 1), 1, 255);
            h_min := PMO_STPU.strnpcpy(19, Coalesce(h_count, 0), result+1, token);
            
            BEGIN
               SELECT Trunc(tempStr), Trunc(h_min) INTO h_count, h_minutes
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1456+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1457) : cplan_fun.sql(8433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            WHILE Coalesce(h_count, 0) > 0 LOOP
               If Coalesce(h_minutes, 0) = 0 Then
                  BEGIN
                     SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + h_count      INTO h_dayofyear
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1472+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1473) : cplan_fun.sql(8449) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  EXIT;
               Else
                  h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(h_work_type, h_minutes, h_update, h_project_id, h_task_id
                                         , h_res_id, h_res_type, h_prj_assignment_id
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_table_type, h_bitID);
                  h_first_time := NULL;
                  
                  BEGIN
                     SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1494+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1495) : cplan_fun.sql(8471) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_count := Coalesce(h_count, 0) - 1;
               End If;
               
            END LOOP;
         Else
            h_min := SubStr(token, 1, 19);
            BEGIN
               SELECT Trunc(h_min) INTO h_minutes FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1513+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1514) : cplan_fun.sql(8490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_minutes, 0) > 0 Then
               h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(h_work_type, h_minutes, h_update, h_project_id, h_task_id
                                         , h_res_id, h_res_type, h_prj_assignment_id
                                         , h_tsk_assignment_id, h_dayofyear, h_first_time, h_bitflag
                                         , h_consider, h_table_type, h_bitID);
               h_first_time := NULL;
            End If;
            
            BEGIN
               SELECT TO_DATE(SubStr(h_dayofyear, 1, 10)) + 1      INTO h_dayofyear
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1535+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1536) : cplan_fun.sql(8512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1554+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1555) : cplan_fun.sql(8531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_rollup, 0) <> 3 Then
      h_temp_sc_state := FC_UPDATE_LABOR_BUDGET(h_project_id, h_task_id, h_tsk_assignment_id, h_rec_user);
   End If;
   
   If Coalesce(assign_to_prj, 0) = 0 Then
      If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
         h_minutes := 0;
         BEGIN
            SELECT SUM(ESTIMATED_PLANNED_EFFORT + ESTIMATED_SCHED_EFFORT) INTO h_minutes
            FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1576+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1577) : cplan_fun.sql(8553) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_minutes, 0) = 0 Then
            BEGIN
               DELETE FROM TIMEPHASED_DELTA WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id
               AND BUDGET_TYPE IN (PMO_CNST.SC_ESTIMATED_BUDGET,PMO_CNST.SC_ESTIMATED_ACTUAL_BUDGET)
               AND CHARGECODE_ID IN (SELECT ELEMENT_ID FROM TMT_CHARGECODES WHERE CAPITAL_EXPENSE > PMO_CNST.SC_TIMEPHASE_CALENDAR);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1596+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1597) : cplan_fun.sql(8573) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROJECT_BUDGET_BREAKDOWN WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id
               AND BUDGET_TYPE IN (PMO_CNST.SC_ESTIMATED_BUDGET,PMO_CNST.SC_ESTIMATED_ACTUAL_BUDGET)
               AND CHARGECODE_ID IN (SELECT ELEMENT_ID FROM TMT_CHARGECODES WHERE CAPITAL_EXPENSE > PMO_CNST.SC_TIMEPHASE_CALENDAR);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1615+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1616) : cplan_fun.sql(8592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_tph_delete := 0;
         h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_prj_assignment_id, h_start_date
                                          , h_finish_date, h_reg_cost, h_reg_sale, h_tph_delete
                                          , h_capital_expense, h_billable);
      End If;
      
      If Coalesce(h_total, 0) = 1 Then
         BEGIN
            SELECT
            PLANNED_WORK,
            PLANNED_START,
            PLANNED_FINISH,
            BILLABLE,
            REGULAR_COST,
            REGULAR_RATE
            INTO
            h_effort,
            h_start_date,
            h_finish_date,
            h_billable,
            h_reg_cost,
            h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1651+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1652) : cplan_fun.sql(8628) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s, h_effort: %d, h_start_date: %s, h_finish_date: %s\n", SC_STATE,h_tsk_assignment_id,h_effort,h_start_date,h_finish_date);//trace_flush();
      End If;
      
      If Coalesce(h_billable, 0) = 0 Then
         BEGIN
            UPDATE PROJECT_ASSIGNMENTS SET
            ESTIMATED_PLANNED_START = h_start_date,
            ESTIMATED_PLANNED_FINISH = h_finish_date,
            ESTIMATED_PLANNED_EFFORT = h_effort,
            ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_reg_cost,
            ESTIMATED_PLANNED_REVENUE = 0.00
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1677+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1678) : cplan_fun.sql(8654) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE PROJECT_ASSIGNMENTS SET
            ESTIMATED_PLANNED_START = h_start_date,
            ESTIMATED_PLANNED_FINISH = h_finish_date,
            ESTIMATED_PLANNED_EFFORT = h_effort,
            ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_reg_cost,
            ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_reg_sale
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1700+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1701) : cplan_fun.sql(8677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("SC_STATE: %d\n", SC_STATE);
   Else
      If Coalesce(h_rollup, 0) = 3 Then
         -- CALL UPDATE assign string function
         /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
         			{
         				if (h_work_type == 0)
         				{
         					BEGIN_SQL
         						UPDATE TASK_ASSIGNMENTS SET
         							PLANNED_START       = HOST(h_start_date),
         							PLANNED_FINISH      = HOST(h_finish_date)
         						WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
         					END_SQL;
         				}
         				else
         				{
         					BEGIN_SQL
         						UPDATE TASK_ASSIGNMENTS SET
         							SCHED_START         = HOST(h_start_date),
         							SCHED_FINISH        = HOST(h_finish_date)
         						WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
         					END_SQL;
         				}
         
         				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
         				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_min_per_day);   // fix ACTUAL_START = SCHED_START  ACTUAL_FINISH = SCHED_FINISH if ETC = 0
         			}
         			else
         			{*/
         If (Coalesce(h_percent_units, 0) > 0) AND (Coalesce(h_percent_units, 0) < 10000) Then
            h_temp_var := (PMO_FUNC.ShiftLeft(Coalesce(h_percent_units, 0), 15)) + 1;
         Else
            h_temp_var := 1;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(
                                   h_per_day, 0), 0);
         -- fix ACTUAL_START = SCHED_START  ACTUAL_FINISH = SCHED_FINISH if ETC = 0
         --}
         
         BEGIN
            SELECT TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1757+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1758) : cplan_fun.sql(8734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      /*if ( (h_bitflag == 0)  (h_task_type == SC_FD_EFFORT_DRIVEN) )
      		{
      			InitializeHost(SC_PARAM(h_tph_delete),0);
      			h_temp_var = FC_TPHASE_FD_SCHED(
      				SC_PARAM(h_project_id),
      				SC_PARAM(h_task_id),
      				SC_PARAM(h_tph_delete));
      		}
      		else
      		{*/
      h_SD := SubStr(h_start_date, 1, 10);
      h_FD := SubStr(h_finish_date, 1, 10);
      
      h_tph_delete := 0;
      h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
      --}
   End If;
   
   If Coalesce(h_rollup, 0) <> 3 Then
      -- Update the contour
      If (Coalesce(h_rollup, 0) = 0) OR ((Coalesce(h_rollup, 0) = 2) AND (Coalesce(
             h_from_prj, 0) > 0)) Then
         /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
         			{
         				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
         				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_min_per_day,h_effort);
         			}
         			else //if ( strcmp(h_prj_assignment_id, h_tsk_assignment_id) == 0 )
         			{*/
         If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
            h_temp_var := 3;
         Else
            h_temp_var := 0;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(
                                   h_per_day, 0), 0);
         --}
         --trace_printf("TA__h_temp_sc_state: %d, h_tsk_assignment_id = %s\n",h_temp_sc_state,h_tsk_assignment_id);
      End If;
      
      If Coalesce(h_rollup, 0) = 0 Then
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
         --trace_printf("FC_Rollup__ h_temp_sc_state = %d, h_project_id: %s\n",h_temp_sc_state,h_project_id);
         
         If Coalesce(h_work_type, 0) = 2 Then
            BEGIN
               SELECT CASE WHEN (SD <> PMO_CNST.SC_NS) AND (FD <> PMO_CNST.SC_NF) AND (TYPE_ID = PMO_CNST.SC_PROPOSAL) THEN 1 ELSE 0 END
               INTO h_change_flag
               FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1822+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1823) : cplan_fun.sql(8799) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_change_flag, 0) = 1 Then
               h_old_type_id := PMO_CNST.SC_PROPOSAL;
               PMO_DB2M.SC_STATE := FC_CHANGE_PROJ_TYPE(h_project_id, h_change_flag, h_old_type_id);
            End If;
         End If;
      End If;
      
      If Coalesce(h_bitflag, 0) = 0 Then
         h_and_roles := and_roles;
         If Coalesce(Length(h_and_roles), 0) > 4096 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 1838+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(1839) : cplan_fun.sql(8815) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
         End If;
         
         If (Coalesce(assign_to_prj, 0) = 0) AND ((h_prj_assignment_id IS NULL)) Then
            If Coalesce(Length(h_and_roles), 0) = 0 Then
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT PRIMARY_ROLE INTO h_id FROM RESOURCES WHERE RESOURCE_ID=h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1854+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1855) : cplan_fun.sql(8831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_id IS NOT NULL)) Then
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, h_id, 0, 0, COALESCE(EXPERIENCE_ID,0), LAST_USED_DATETIME,'0'
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1873+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1874) : cplan_fun.sql(8850) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf SubStr(h_res_type, 1, 1) = '0' Then
               --Parse Roles list
               If Coalesce(Length(h_and_roles), 0) < 32 Then
                  numItems := 0;
               Else
                  numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
               End If;
               
               If numItems > 0 Then
                  h_role_id := SubStr(h_role_id, 1, 32);
                  
                  h_count := 0;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next role_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
                     h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_roles);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t2.ELEMENT_ID=t1.PARENT_ID AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8)
                        AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK)end) = t3.RANK
                        );
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1916+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1917) : cplan_fun.sql(8893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                  END LOOP;
               End If;
            End If;
         End If;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID=h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1936+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1937) : cplan_fun.sql(8913) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID=h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 1950+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(1951) : cplan_fun.sql(8927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_count, 0) = 0 Then
               h_tsk_assignment_id := SubStr(h_res_id, 1, 32);
            End If;
         Else
            h_count := 0;
            NULL;
         End If;
         
         If Coalesce(h_count, 0) = 0 Then
            --Parse Roles list
            If Coalesce(Length(h_and_roles), 0) < 32 Then
               numItems := 0;
            Else
               numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
            End If;
            
            If numItems > 0 Then
               h_role_id := SubStr(h_role_id, 1, 32);
               
               If SubStr(h_res_type, 1, 1) = '0' Then
                  h_count := 0;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next skill_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
                     h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_roles);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t2.ELEMENT_ID=t1.PARENT_ID AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK
                        ) AND
                        NOT EXISTS(SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_ROLES
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (ROLE_ID = t3.PARENT_ID OR PRIMARY_ROLE = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2003+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2004) : cplan_fun.sql(8980) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                  END LOOP;
               ElsIf SubStr(h_res_type, 1, 1) = '1' Then
                  h_count := 1;
                  WHILE Coalesce(h_count, 0) < numItems LOOP
                     -- Get next role_id
                     --                          pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
                     h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_roles);
                     
                     --trace_printf("h_count = %d, h_and_roles = %s\n",h_count,h_and_roles);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_ROLES SELECT h_tsk_assignment_id, h_role_id, PROFICIENCY_LEVEL_ID,
                        PREFERENCE_LEVEL_ID, EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID=h_role_id AND NOT EXISTS(
                        SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_ROLES
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (ROLE_ID = h_role_id OR PRIMARY_ROLE = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2035+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2036) : cplan_fun.sql(9012) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_count := Coalesce(h_count, 0) + 1;
                     --trace_printf("h_count = %d, SC_STATE = %d, h_role_id = %s, h_role_id_Ind = %d\n",h_count,SC_STATE,h_role_id,h_role_id_Ind);
                  END LOOP;
               End If;
            End If;
            
            --Parse Skills list
            h_and_skills := and_skills;
            If Coalesce(Length(h_and_skills), 0) > 4096 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 2050+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(2051) : cplan_fun.sql(9027) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
            End If;
            
            If Coalesce(Length(h_and_skills), 0) < 32 Then
               numItems := 0;
            Else
               numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
            End If;
            
            If numItems > 0 Then
               h_role_id := SubStr(h_role_id, 1, 32);
               
               h_count := 0;
               WHILE Coalesce(h_count, 0) < numItems LOOP
                  -- Get next skill_id
                  --                      pmo_substr(h_role_id, h_and_skills, h_count*32, 32);
                  h_role_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_and_skills);
                  
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_SKILLS SELECT h_tsk_assignment_id, h_role_id, PROFICIENCY_LEVEL_ID,
                        PREFERENCE_LEVEL_ID, EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES WHERE ELEMENT_ID=h_role_id AND NOT EXISTS(
                        SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_SKILLS
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (SKILL_ID = h_role_id OR PRIMARY_SKILL = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2087+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2088) : cplan_fun.sql(9064) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGN_SKILLS SELECT h_tsk_assignment_id, t2.ELEMENT_ID, t2.PROFICIENCY_LEVEL_ID,
                        t2.PREFERENCE_LEVEL_ID, t2.EXPERIENCE_ID, t2.LAST_USED_DATETIME, TO_CHAR(h_count)
                        FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2,TMT_COMPETENCIES t3
                        WHERE t1.RESOURCE_ID=h_res_id AND t3.ELEMENT_ID = h_role_id AND
                        (
                        t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK
                        AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK
                        )
                        AND t2.ELEMENT_ID=t1.PARENT_ID AND NOT EXISTS(SELECT RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGN_SKILLS
                        WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id AND (SKILL_ID = t3.PARENT_ID OR PRIMARY_SKILL = TO_CHAR(h_count)));
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'U_TSKRES2', 2113+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: U_TSKRES2(2114) : cplan_fun.sql(9090) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  h_count := Coalesce(h_count, 0) + 1;
               END LOOP;
            End If;
         End If;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'U_TSKRES2', 2129+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: U_TSKRES2(2130) : cplan_fun.sql(9106) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION U_TSKRES2;
ALTER FUNCTION U_TSKRES2 COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- search function

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_SEARCH ==
CREATE OR REPLACE FUNCTION FC_SEARCH
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_start_DT IN OUT NOCOPY TIMESTAMP
   , h_finish_DT IN OUT NOCOPY TIMESTAMP
   , h_start_time IN OUT NOCOPY VARCHAR2
   , h_finish_time IN OUT NOCOPY VARCHAR2
   , sh_h_consider PLS_INTEGER
   , h_res_type IN OUT NOCOPY VARCHAR2
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_update PLS_INTEGER
   , sh_h_table_type PLS_INTEGER
   , sh_h_duration PLS_INTEGER
   , sh_h_total PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_iSF PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_every PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_skip PLS_INTEGER
   , sh_h_percent_units PLS_INTEGER
   , sh_h_amnt PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_day_min PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_min_work_per_day PLS_INTEGER
   , sh_h_julian_finish PLS_INTEGER
   , sh_h_j_disconti_date PLS_INTEGER
   , sh_assign_to_prj PLS_INTEGER
   , sh_i_pa_replace PLS_INTEGER
   , sh_l_calls PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_consider PLS_INTEGER := sh_h_consider;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_update PLS_INTEGER := sh_h_update;
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_duration PLS_INTEGER := sh_h_duration;
   h_total PLS_INTEGER := Coalesce(sh_h_total, 0);
   h_effort PLS_INTEGER := Coalesce(sh_h_effort, 0);
   h_work PLS_INTEGER := Coalesce(sh_h_work, 0);
   iSF PLS_INTEGER := Coalesce(sh_iSF, 0);
   h_contour PLS_INTEGER := Coalesce(sh_h_contour, 0);
   h_every PLS_INTEGER := Coalesce(sh_h_every, 0);
   h_type PLS_INTEGER := Coalesce(sh_h_type, 0);
   h_skip PLS_INTEGER := Coalesce(sh_h_skip, 0);
   h_percent_units PLS_INTEGER := Coalesce(sh_h_percent_units, 0);
   h_amnt PLS_INTEGER := Coalesce(sh_h_amnt, 0);
   h_unit PLS_INTEGER := Coalesce(sh_h_unit, 0);
   h_day_min PLS_INTEGER := Coalesce(sh_h_day_min, 0);
   h_per_day PLS_INTEGER := Coalesce(sh_h_per_day, 0);
   h_min_work_per_day PLS_INTEGER := Coalesce(sh_h_min_work_per_day, 0);
   h_julian_finish PLS_INTEGER := Coalesce(sh_h_julian_finish, 0);
   h_j_disconti_date PLS_INTEGER := Coalesce(sh_h_j_disconti_date, 0);
   assign_to_prj PLS_INTEGER := Coalesce(sh_assign_to_prj, 0);
   i_pa_replace PLS_INTEGER := Coalesce(sh_i_pa_replace, 0);
   l_calls PLS_INTEGER := Coalesce(sh_l_calls, 0);
   h_startavailable_time VARCHAR2(8);
   h_dayofyear VARCHAR2(10);
   h_finish_D VARCHAR2(10);
   h_cnt PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_total_min PLS_INTEGER := 0;
   h_work_units PLS_INTEGER := 0;
   h_prev_week PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_day_work PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_min VARCHAR2(19);
   h_work_loc VARCHAR2(32);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_header VARCHAR2(7999);
   h_tail VARCHAR2(999);
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
   h_pmo_tmp_res_ins_stmt VARCHAR2(3999);
   h_bitflag_condition VARCHAR2(199);
   h_pmo_tmp_every_ins_stmt VARCHAR2(3999);
   h_pmo_tmp_sum_ins_stmt VARCHAR2(3999);
   h_pmo_tmp_md_ins_stmt VARCHAR2(3999);
   tempStr VARCHAR2(255);
   h_totpr PLS_INTEGER;
   bAssignStringDone PLS_INTEGER NOT NULL := 0;
   bFirstRequestedDate PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER NOT NULL := 0;
   ii PLS_INTEGER NOT NULL := 0;
   iii PLS_INTEGER NOT NULL := 0;
   i_work PLS_INTEGER NOT NULL := h_work;
   i_break PLS_INTEGER NOT NULL := 0;
   wBits PLS_INTEGER NOT NULL := 0;
   wHour PLS_INTEGER NOT NULL := 0;
   wMin PLS_INTEGER NOT NULL := 0;
   iSD PLS_INTEGER NOT NULL := 1;
   wHour001 PLS_INTEGER;
   wMin001 PLS_INTEGER;
   wBits001 PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C332 PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEARCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEARCH'), '-- ' || SYSTIMESTAMP, ':r := FC_SEARCH(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_start_DT || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_DT || '''',''''''), 'NULL')
          || ', ' || '''' || h_start_time || ''''
          || ', ' || '''' || h_finish_time || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_total), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_iSF), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_every), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_skip), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_percent_units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_amnt), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_day_min), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_min_work_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_julian_finish), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_j_disconti_date), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_assign_to_prj), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_i_pa_replace), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_l_calls), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 156+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEARCH(157) : cplan_fun.sql(9277) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- SC_BITID(h_bitID1);
   
   /**NOTE** Dynamic Insert statement for PMO_TMP_RES temporary table*/
   /**NOTE** temporary variable to hold the condition applied for bit_flag field */
   /**NOTE** Dynamic Insert statement for PMO_TMP_EVERY temporary table*/
   /**NOTE** Dynamic Insert statement for PMO_TMP_SUM temporary table*/
   /**NOTE** Dynamic Insert statement for PMO_TMP_SUM temporary table*/
   
   -- TOREVIEW: In Detail, September 16, 2004
   
   /**NOTE** Apply Temporary Tables Here */
   
   If Coalesce(h_bitflag, 0) = 0 Then
      h_work := 0;
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
         --no need to temporary tables
         stmt := 'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.SCHED_MIN, 0 AS WEEK_OF, 0 AS CNT '
                || 'FROM PMO_TMP_TDY p '
                || 'WHERE p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_start_DT;
         stmt := stmt || ''')';
         stmt := stmt || ' AND p.SCHED_MIN > 0 ORDER BY DAY_YEAR';
      ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         -- no need to temporary tables
         stmt := 
               'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT '
                || 'FROM PMO_TMP_CDY p '
                || 'WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_start_DT;
         stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_finish_DT;
         stmt := stmt || ''') AND AVAILABLE_MIN > 0 ORDER BY DAY_YEAR';
      ElsIf ((BITAND(Coalesce(h_update, 0), 255)) = 10) AND (h_min_work_per_day > 0) Then
         stmt := 
               'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT '
                || 'FROM PMO_TMP_CDY p ';
         stmt := stmt || 'WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_start_DT;
         stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_finish_DT;
         stmt := stmt || ''') AND p.AVAILABLE_MIN > ';
         stmt := stmt || TO_CHAR(h_min_work_per_day);
         stmt := stmt || ' ORDER BY DAY_YEAR';
      Else
         stmt := 
               'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR), p.DAY_YEAR, p.AVAILABLE_MIN, 0 AS WEEK_OF, 0 AS CNT '
                || 'FROM PMO_TMP_CDY p ';
         stmt := stmt || 'WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_start_DT;
         stmt := stmt || ''') AND PMO_ORASTD.DB2DATE(''';
         stmt := stmt || h_finish_DT;
         stmt := stmt || ''') AND p.AVAILABLE_MIN > 0 ORDER BY DAY_YEAR';
      End If;
   Else
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
            -- no need to temporary tables
            --stmt.strcpy ("SELECT JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, COUNTER() AS CNT FROM session.PMO_TMP_CDY p WHERE p.DAY_YEAR BETWEEN DATE('");
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                  || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR || ' AS CNT FROM PMO_TMP_CDY p '
                   || 'WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            stwhere := stwhere || h_start_DT;
            stwhere := stwhere || ''') AND PMO_ORASTD.DB2DATE(''';
            stwhere := stwhere || h_finish_DT;
            stwhere := stwhere || ''')';
            stmt := stmt || stwhere;
         Else
            -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
            
            -- create PMO_TMP_RES temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
            PMO_DB2M.SC_STATE := 0;
            
            -- Prepare Dynamic Insert statement to insert records in PMO_TMP_RES
            -- Use SESSION.PMO_TMP_CDY (p) as a source table
            h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES(DAY_YEAR, DAY_WEEK, WEEK_OF, MONTH_OF, YEAR_OF, AVAIL_MIN) ';
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR AS DATE, p.DAY_WEEK AS DAY_WEEK, '
                   || ' p.WEEK_OF AS WEEK_OF, p.MONTH_OF AS MONTH_OF, '
                   || ' p.YEAR_OF AS YEAR_OF, p.AVAILABLE_MIN AS AVAIL_MIN '
                   || ' FROM PMO_TMP_CDY p '
                   || ' WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
         End If;
      ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
            -- no need to temporary tables
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.SCHED_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                  || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR || ' AS CNT '
                   || 'FROM PMO_TMP_TDY p '
                   || 'WHERE p.DAY_YEAR >= TO_DATE(SubStr(''' || h_start_DT || ''', 1, 10))';
         Else
            -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
            
            -- create PMO_TMP_RES temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
            PMO_DB2M.SC_STATE := 0;
            
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Prepare dynamic Insert Statement to insert records in PMO_TMP_RES
               -- this dynamic statement will be executed in the rest of statement
               -- for this case there is no need to MONTH_OF field, it will be null in all records
               
               -- Use TASK_ASSIGNMENTS(d) and TASK_DAY_OF_YEAR (p) as source tables
               
               h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, YEAR_OF, WEEK_OF, AVAIL_MIN) ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR AS DAY_YEAR, PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) AS DAY_WEEK, '
                      || ' PMO_ORASTD.YEAR(p.DAY_YEAR) + CASE WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 1 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 12 THEN 1 '
                      || ' WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 53 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 1 THEN -1 ELSE 0 END AS YEAR_OF, '
                      || ' PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) AS WEEK_OF, p.SCHED_MIN AS AVAIL_MIN '
                      || ' FROM TASK_ASSIGNMENTS d JOIN TASK_DAY_OF_YEAR p ON d.PROJECT_ID = p.PROJECT_ID AND d.TSK_ASSIGNMENT_ID = p.TSK_ASSIGNMENT_ID '
                      || ' WHERE d.RESOURCE_ID = ''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND d.TASK_ID = ''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_project_id;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_bitflag_condition;
               -- condition for BIT_FLAG field
            Else
               -- Use TASK_ASSIGNMENTS(d) and TASK_DAY_OF_YEAR (p) as source tables
               
               h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, WEEK_OF, MONTH_OF, DAY_WEEK, YEAR_OF, AVAIL_MIN) ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    'SELECT p.DAY_YEAR, PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) AS WEEK_OF, PMO_ORASTD.MONTH(p.DAY_YEAR) AS MONTH_OF, '
                      || ' PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) AS DAY_WEEK, PMO_ORASTD.YEAR(p.DAY_YEAR) + CASE WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 1 AND '
                      || ' PMO_ORASTD.MONTH(p.DAY_YEAR) = 12 THEN 1 WHEN PMO_ORASTD.WEEK_ISO(p.DAY_YEAR) = 53 AND PMO_ORASTD.MONTH(p.DAY_YEAR) = 1 THEN -1 '
                      || ' ELSE 0 END AS YEAR_OF, p.SCHED_MIN AS AVAIL_MIN '
                      || ' FROM TASK_ASSIGNMENTS d JOIN TASK_DAY_OF_YEAR p ON d.PROJECT_ID = p.PROJECT_ID AND d.TSK_ASSIGNMENT_ID = p.TSK_ASSIGNMENT_ID '
                      || ' WHERE d.RESOURCE_ID = ''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_res_id;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND d.TASK_ID = ''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_project_id;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''' AND p.DAY_YEAR >= PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') ';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_bitflag_condition;
               -- condition for BIT_FLAG field
            End If;
         End If;
      Else
         If (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) OR (Coalesce(h_bitflag, 0) = 7) Then
            -- no need to temporary table
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(p.DAY_YEAR) AS J_DAY, p.DAY_YEAR, p.AVAILABLE_MIN AS AVAIL_MIN, 0 AS WEEK_OF, ' 
                  || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR || ' AS CNT FROM PMO_TMP_CDY p WHERE ';
            stwhere := 'p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
            stwhere := stwhere || h_start_DT;
            stwhere := stwhere || ''') AND PMO_ORASTD.DB2DATE(''';
            stwhere := stwhere || h_finish_DT;
            stwhere := stwhere || ''')';
            stmt := stmt || stwhere;
         Else
            -- in this case, the algorithm use PMO_TMP_RES temporary table instead of WITH pd Clause
            
            -- create PMO_TMP_RES temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES';
            PMO_DB2M.SC_STATE := 0;
            
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Prepare Dynamic Insert statement for inserting necessary records in PMO_TMP_RES
               -- for this case there is no need to MONTH_OF field
               
               -- Use session.PMO_TMP_CDY (p)  as Source Tables
               h_pmo_tmp_res_ins_stmt := 'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, WEEK_OF,YEAR_OF, AVAIL_MIN)';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' SELECT p.DAY_YEAR, p.DAY_WEEK, p.WEEK_OF, p.YEAR_OF,'
                      || ' p.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)'
                      || ' FROM PMO_TMP_CDY p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR'
                      || ' WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
            Else
               -- Use CALENDAR_DAY_OF_YEAR(d), session.PMO_TMP_CDY (p) as Source Tables
               h_pmo_tmp_res_ins_stmt := 
                          'INSERT INTO PMO_TMP_RES (DAY_YEAR, DAY_WEEK, WEEK_OF, MONTH_OF, YEAR_OF, AVAIL_MIN)';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' SELECT p.DAY_YEAR, p.DAY_WEEK, p.WEEK_OF, p.MONTH_OF, p.YEAR_OF,'
                      || ' p.AVAILABLE_MIN - COALESCE(ta.SCHED_MIN,0)'
                      || ' FROM PMO_TMP_CDY p LEFT OUTER JOIN PMO_TMP_CPLNF_0001 ta ON ta.DAY_YEAR = p.DAY_YEAR'
                      || ' WHERE p.DAY_YEAR BETWEEN PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_start_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''') AND PMO_ORASTD.DB2DATE(''';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_finish_DT;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ''')';
            End If;
         End If;
      End If;
      
      CASE h_bitflag
         
         WHEN 1 THEN
            -- Daily, every X workdays
            /**NOTE** THIS CASE DOSE NOT NEED TO ANY TEMPORARY TABLES */
            --if h_work is 0 then we have to copy h_day_min to h_min which is calclted based on %unit passed
            --if %unit/day exceeds available min then don't return  ---> ANI
            --if (h_work == 0)
            --{
            -- h_work = h_day_min;
            --}
            h_min := TO_CHAR(h_work);
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               stmt := stmt || ' AND AVAILABLE_MIN >= ';
            ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
               stmt := stmt || ' AND p.SCHED_MIN >= ';
            Else
               stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
            End If;
            stmt := stmt || h_min;
            
            If h_every > 1 Then
               h_header := 'SELECT J_DAY, DAY_YEAR, AVAIL_MIN, WEEK_OF, CNT FROM ( ';
               h_header := h_header || stmt;
               --sprintf(h_min, "%d", h_every);
               --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1,%s )=0 ",  PMO_ORDER_BY_DAY_YEAR, h_min);
               h_tail := ' ORDER BY DAY_YEAR)  WHERE MOD(CNT-1,' || TO_CHAR(h_every) || ' )=0 ';
               
               h_header := h_header || h_tail;
               
               stmt := h_header;
               stmt := stmt || ' ORDER BY DAY_YEAR';
            Else
               stmt := stmt || ' ORDER BY p.DAY_YEAR';
            End If;
            
         
         WHEN 2 THEN
            -- Weekly, every X week on Mo, We
            /**NOTE** THIS CASE USES PMO_TMP_RES, PMO_TMP_SUM, AND PMO_TMP_EVERY */
            --         TEMPORARY TABLES
            h_min := TO_CHAR(h_work);
            h_work_loc := '';
            
            -- This case needs to use PMO_TMP_SUM temporary table
            -- create PMO_TMP_SUM temporary table instead of w WITH Clause
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
            PMO_DB2M.SC_STATE := 0;
            If (BITAND(h_contour, PMO_CNST.SC_0000007F)) = 0 Then
               -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
               -- first execute dynamic insert statement to insert records in PMO_TMP_RES
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
               -- execute dynamic Insert statement for PMO_TMP_RES
               
               -- Insert records in PMO_TMP_SUM with using PMO_TMP_RES temporary table
               -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
               BEGIN
                  INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF, AVAIL_MIN)
                  SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS AVAIL_MIN /*WEEK_MIN */
                  FROM PMO_TMP_RES
                  GROUP BY YEAR_OF,WEEK_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 438+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(439) : cplan_fun.sql(9559) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               /**NOTE** Commented by Amir
               						stmt.strcat("), w AS (SELECT YEAR_OF,WEEK_OF,SUM(AVAIL_MIN) AS WEEK_MIN FROM pd GROUP BY YEAR_OF,WEEK_OF)");
               						*/
            Else
               i := 1;
               ii := 1;
               WHILE ii <= 7 LOOP
                  If (BITAND(h_contour, i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               -- compelete h_pmo_tmp_res_ins_stmt (Dynamic Insert Statement for PMO_TMP_RES
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR) IN (';
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
               End If;
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
               
               h_min := TO_CHAR(h_work);
               
               If h_amnt = 1 Then
                  If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
                  Else
                     h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
                  End If;
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
                  
                  -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
                  -- first execute dynamic insert statement to insert records in PMO_TMP_RES
                  h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
                  -- execute dynamic Insert statement for PMO_TMP_RES
                  
                  -- Insert necessary records in PMO_TMP_SUM temporary table
                  -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
                  
                  -- Vladimir Schneider, vladimir@winpte.com
                  -- VERIFY: why are just yearof week of with not avail_min?
                  BEGIN
                     INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF)
                     SELECT YEAR_OF, WEEK_OF
                     FROM PMO_TMP_RES
                     GROUP BY YEAR_OF,WEEK_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 508+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(509) : cplan_fun.sql(9629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  /**NOTE** stmt.strcat("), w AS (SELECT YEAR_OF,WEEK_OF FROM pd");*/
                  /**NOTE** stmt.strcat(" GROUP BY YEAR_OF,WEEK_OF)"); */
               Else
                  -- use PMO_TMP_RES to insert records in new temporary table PMO_TMP_SUM
                  -- first execute dynamic insert statement to insert records in PMO_TMP_RES
                  h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
                  -- execute dynamic Insert statement for PMO_TMP_RES
                  
                  -- Insert necessary records in PMO_TMP_SUM temporary table
                  -- no need to use dynamic Insert statement to insert records in PMO_TMP_RES
                  BEGIN
                     INSERT INTO PMO_TMP_SUM (YEAR_OF, WEEK_OF, AVAIL_MIN)
                     SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS AVAIL_MIN  /*WEEK_MIN */
                     FROM PMO_TMP_RES
                     GROUP BY YEAR_OF,WEEK_OF;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 538+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(539) : cplan_fun.sql(9659) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  /**NOTE** stmt.strcat("), w AS (SELECT YEAR_OF, WEEK_OF, SUM(AVAIL_MIN) AS WEEK_MIN FROM pd"); */
                  /**NOTE** stmt.strcat(" GROUP BY YEAR_OF,WEEK_OF)"); */
               End If;
            End If;
            
            -- create PMO_TMP_EVERY temporary table,  this temporary table will be used instead of every WITH Clause
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
            PMO_DB2M.SC_STATE := 0;
            
            -- Preparing Dynamic Insert Statement for PMO_TMP_EVERY
            If h_every > 1 Then
               --h_pmo_tmp_every_ins_stmt.strcpy("SELECT YEAR_OF, WEEK_OF, COUNTER() AS CNT FROM SESSION.PMO_TMP_SUM w ");
               h_pmo_tmp_every_ins_stmt := 'SELECT YEAR_OF, WEEK_OF, ' 
                            || PMODEFINES.PMO_COUNTER_OVER_YEAROF_WEEKOF 
                            || ' AS CNT FROM PMO_TMP_SUM w ';
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE w.AVAIL_MIN >= ';
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
               
               h_header := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, WEEK_OF) SELECT YEAR_OF, WEEK_OF FROM ( ';
               h_header := h_header || h_pmo_tmp_every_ins_stmt;
               
               h_min := TO_CHAR(h_every);
               
               h_tail := ' )  WHERE MOD(CNT-1,' || h_min || ' )=0 ';
               --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1,%s )=0 ",  PMO_ORDER_BY_YEAROF_WEEKOF, h_min);
               h_header := h_header || h_tail;
            Else
               h_pmo_tmp_every_ins_stmt := 'SELECT YEAR_OF,WEEK_OF FROM PMO_TMP_SUM w ';
               
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE w.AVAIL_MIN >= ';
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
               
               h_header := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, WEEK_OF) ';
               h_header := h_header || h_pmo_tmp_every_ins_stmt;
            End If;
            
            --h_pmo_tmp_every_ins_stmt.Clear();
            h_pmo_tmp_every_ins_stmt := h_header;
            
            -- execute h_pmo_tmp_every_ins_stmt to insert necessary data in PMO_TMP_EVERY
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
            
            -- rewrite the last select statement with using previouse temporary tables
            If Coalesce(Length(h_work_loc), 0) = 0 Then
               /**IMPORTANT NOTE** the following select statement can be rewritten as static SQL statement*/
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, every.WEEK_OF, 0 AS CNT '
                      || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                      || ' WHERE pd.YEAR_OF = every.YEAR_OF AND '
                      || ' pd.WEEK_OF = every.WEEK_OF '
                      || ' ORDER BY pd.DAY_YEAR ';
            Else
               /**IMPORTANT NOTE** the following select statement can be rewritten as static SQL statement*/
               stmt := 
                     ' SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, every.WEEK_OF, 0 AS CNT '
                      || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                      || ' WHERE pd.YEAR_OF = every.YEAR_OF AND '
                      || ' pd.WEEK_OF = every.WEEK_OF AND pd.DAY_WEEK IN (';
               stmt := stmt || h_work_loc;
               stmt := stmt || ') ORDER BY pd.DAY_YEAR';
            End If;
            
            
         
         WHEN 3 THEN
            -- Monthly, The Day 6 every X months
            /**NOTE** THIS CASE USE PMO_TMP_RES TEMPORARY TABLE*/
            h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
            
            h_min := TO_CHAR(Coalesce(h_week, 0));
            
            -- Compelete the rest of Dynamic Insert Statement for PMO_TMP_RES
            
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) = ';
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
            
            -- rewrite the last select statement with using PMO_TMP_RES
            -- first execute the dynamic Insert statement to insert records in temporary table
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
            
            -- Prepare last select statement
            
            If h_every > 1 Then
               --stmt.strcpy("SELECT JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, WEEK_OF, 0 AS CNT FROM ( "
               --          " SELECT JULIAN_DAY(DAY_YEAR) AS JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, 0  AS WEEK_OF, COUNTER() AS CNT "
               --          " FROM SESSION.PMO_TMP_RES pd WHERE AVAIL_MIN >= ");
               stmt := 'SELECT JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, WEEK_OF, 0 AS CNT FROM ( '
                      || ' SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR) AS JULIAN_DAY_YEAR, DAY_YEAR, AVAIL_MIN, 0  AS WEEK_OF, ' 
                           || PMODEFINES.PMO_COUNTER_OVER_DAY_YEAR || ' AS CNT '
                      || ' FROM PMO_TMP_RES p WHERE AVAIL_MIN >= ';
               
               h_min := TO_CHAR(h_work);
               
               stmt := stmt || h_min;
               
               h_min := TO_CHAR(h_every);
               
               h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ORDER BY DAY_YEAR ';
               --h_tail.sprintf(" %s  ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ORDER BY DAY_YEAR ", PMO_ORDER_BY_DAY_YEAR ,h_min);
               
               stmt := stmt || h_tail;
            Else
               stmt := 'SELECT PMO_ORASTD.JULIAN_DAY(DAY_YEAR), DAY_YEAR, AVAIL_MIN, 0 AS WEEK_OF, 0 AS CNT '
                      || ' FROM PMO_TMP_RES pd WHERE AVAIL_MIN >= ';
               
               h_min := TO_CHAR(h_work);
               
               stmt := stmt || h_min;
               
               stmt := stmt || ' ORDER BY DAY_YEAR';
            End If;
            
            
         
         WHEN 4 THEN
            -- Monthly, First/Second/Third/Forth/Last Monday every X months
            -- This case uses PMO_TMP_RES, PMO_TMP_SUM
            h_days := BITAND(h_contour, PMO_CNST.SC_0000007F);
            -- Monday
            i := 1;
            ii := 1;
            WHILE ii <= 7 LOOP
               If (BITAND(Coalesce(h_days, 0), i)) = i Then
                  h_days := ii;
                  
                  EXIT;
               End If;
               i := i * 2;
               ii := ii + 1;
            END LOOP;
            
            h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
            -- First/Second/Third/Forth/Last
            i := 1;
            ii := 1;
            WHILE ii <= 5 LOOP
               If (BITAND(Coalesce(h_week, 0), i)) = i Then
                  h_week := ii;
                  
                  EXIT;
               End If;
               i := i * 2;
               ii := ii + 1;
            END LOOP;
            
            h_min := TO_CHAR(7 * (Coalesce(h_week, 0) - 1) + 1);
            
            -- Prepare the rest of PMO_TMP_RES Dynamic Insert statement
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) BETWEEN ';
            --stmt.strcat(" AND DAY(p.DAY_YEAR) BETWEEN ");
            
            If Coalesce(h_week, 0) = 5 Then
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' CASE WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) = 2 AND MOD(PMO_ORASTD.YEAR(p.DAY_YEAR),4)=0 THEN 23 WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) = 2 AND MOD(PMO_ORASTD.YEAR(p.DAY_YEAR),4)>0 THEN 22 WHEN PMO_ORASTD.MONTH(p.DAY_YEAR) IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
                  --stmt.strcat(" CASE WHEN MONTH(p.DAY_YEAR) = 2 AND MOD(YEAR(p.DAY_YEAR),4)=0 THEN 23 WHEN MONTH(p.DAY_YEAR) = 2 AND MOD(YEAR(p.DAY_YEAR),4)>0 THEN 22 WHEN MONTH(p.DAY_YEAR) IN (4,6,9,11) THEN 24 ELSE 25 END AND 31");
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || 
                                                    ' CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
                  --stmt.strcat(" CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31");
               End If;
            Else
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               --stmt.strcat(h_min);
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND ';
               --stmt.strcat(" AND ");
               
               h_min := TO_CHAR(7 * Coalesce(h_week, 0));
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               --stmt.strcat(h_min);
            End If;
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR)=';
               --stmt.strcat(" AND DAYOFWEEK(p.DAY_YEAR)=");
            Else
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK=';
               --stmt.strcat(" AND p.DAY_WEEK=");
            End If;
            h_min := TO_CHAR(Coalesce(h_days, 0));
            -- Monday
            
            h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
            --stmt.strcat(h_min);
            
            -- Dynamic insert statement for PMO_TMP_RES  is ready to execute
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
            
            -- create PMO_TMP_SUM temporary table instead of w WITH Clause
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
            PMO_DB2M.SC_STATE := 0;
            
            -- Insert statement to insert records in PMO_TMP_SUM temporary table, with using PMO_TMP_RES
            -- no need to use dynamic insert statement
            BEGIN
               INSERT INTO PMO_TMP_SUM (YEAR_OF, MONTH_OF, AVAIL_MIN)
               SELECT YEAR_OF AS YEAR_OF,MONTH_OF AS MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN /*WEEK_MIN*/
               FROM PMO_TMP_RES pd
               GROUP BY YEAR_OF,MONTH_OF;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 759+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(760) : cplan_fun.sql(9880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- Prepare PMO_TMP_EVERY temporary table for the rest of algorithm
            -- create PMO_TMP_EVERY temporary table,  this temporary table will be used instead of every WITH Clause
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
            PMO_DB2M.SC_STATE := 0;
            
            If h_every > 1 Then
               -- Prepare dynamic insert statement to insert records in PMO_TMP_EVERY
               --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
               --                              "SELECT  YEAR_OF, MONTH_OF FROM ( "
               --                              " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
               --                              " FROM SESSION.PMO_TMP_SUM w WHERE AVAIL_MIN >= ");
               
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                      || 'SELECT  YEAR_OF, MONTH_OF FROM ( '
                      || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                      || ' FROM PMO_TMP_SUM w WHERE AVAIL_MIN >= ';
               
               h_min := TO_CHAR(h_work);
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               
               h_min := TO_CHAR(h_every);
               
               h_tail := ' )   WHERE MOD(CNT-1, ' || h_min || ' )=0 ';
               --h_tail.sprintf(" %s ) AS PMO_ALIAS  WHERE MOD(CNT-1, %s )=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
            Else
               -- Prepare dynamic insert statement to insert records in PMO_TMP_EVERY
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                      || ' SELECT YEAR_OF, MONTH_OF '
                      || ' FROM PMO_TMP_SUM w WHERE AVAIL_MIN >= ';
               
               h_min := TO_CHAR(h_work);
               
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
            End If;
            
            -- execute h_pmo_tmp_every_ins_stmt to prepare records for the last select statement
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
            
            -- all the temporary tables are ready to use for creating the last select statement
            stmt := stmt || 'SELECT PMO_ORASTD.JULIAN_DAY(pd.DAY_YEAR), pd.DAY_YEAR, pd.AVAIL_MIN, 0  '
                   || ' FROM PMO_TMP_RES pd, PMO_TMP_EVERY every '
                   || ' WHERE pd.YEAR_OF = every.YEAR_OF AND pd.MONTH_OF = every.MONTH_OF '
                   || ' ORDER BY pd.DAY_YEAR ';
            
            
         
         WHEN 5 THEN
            -- Monthly, Mo, We, Fr  on Week1, Week4 every X months
            -- This case uses PMO_TMP_RES, PMO_TMP_SUM, PMO_TMP_EVERY, PMO_TMP_MD
            i := 1;
            ii := 1;
            iii := 0;
            If (BITAND(h_contour, PMO_CNST.SC_0000007F)) > 0 Then
               WHILE ii <= 7 LOOP
                  If (BITAND(h_contour, i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               -- Prepare the rest of h_pmo_tmp_res_ins_stmt dynamic insert statement
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
               
               --stmt.strcat(" AND p.DAY_WEEK IN (");
               --stmt.strcat(h_work_loc);
               --stmt.strcat(")");
            End If;
            
            If h_amnt = 1 Then
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
               ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.AVAILABLE_MIN >= ';
               End If;
               h_min := TO_CHAR(h_work);
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               --stmt.strcat(h_min);
            End If;
            
            -- the end of dynamic insert statement for PMO_TMP_RES temporary table
            -- execute the h_pmo_tmp_res_ins_stmt to insert data in PMO_TMP_RES
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
            
            -- Use PMO_TMP_RES temporary table to prepare necessary data for PMO_TMP_MD
            -- Create PMO_TMP_MD temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MD';
            PMO_DB2M.SC_STATE := 0;
            
            -- Prepare dynamic insert statement to insert records in PMO_TMP_MD
            h_pmo_tmp_md_ins_stmt := 'INSERT INTO PMO_TMP_MD(YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN) ';
            h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || 
                                                  'SELECT YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN '
                   || ' FROM PMO_TMP_RES pd ';
            
            --stmt.strcat("), md AS (SELECT YEAR_OF, MONTH_OF, DAY_YEAR, DAY_WEEK, AVAIL_MIN FROM pd");
            
            h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
            -- Week1, Week2, Week3, Week4
            If (Coalesce(h_week, 0) > 0) AND (Coalesce(h_week, 0) <> 15) Then
               i := 1;
               ii := 1;
               WHILE ii <= 4 LOOP
                  If (BITAND(Coalesce(h_week, 0), i)) = i Then
                     If iii = 0 Then
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' WHERE PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                        --stmt.strcat(" WHERE DAY(DAY_YEAR) BETWEEN ");
                        iii := 2;
                     Else
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' OR PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                        --stmt.strcat(" OR DAY(DAY_YEAR) BETWEEN ");
                     End If;
                     h_min := TO_CHAR(7 * (ii - 1) + 1);
                     
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND ';
                     --stmt.strcat(h_min);
                     --stmt.strcat(" AND ");
                     
                     i := i * 2;
                     ii := ii + 1;
                     WHILE ii <= 5 LOOP
                        If (BITAND(Coalesce(h_week, 0), i)) = 0 Then
                           h_min := TO_CHAR(7 * (ii - 1));
                           
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                           --stmt.strcat(h_min);
                           
                           EXIT;
                        End If;
                        i := i * 2;
                        ii := ii + 1;
                     END LOOP;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
            End If;
            
            -- end of dynamic insert statemnt for h_pmo_tmp_md_ins_stmt
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
            
            -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
            -- create PMO_TMP_SUM temporary table instead of w WITH Clause
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
            PMO_DB2M.SC_STATE := 0;
            
            -- Insert necessary records in PMO_TMP_SUM, no need to create dynamic insert statement
            If h_amnt = 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                  SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                  FROM PMO_TMP_MD md
                  GROUP BY YEAR_OF,MONTH_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 942+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(943) : cplan_fun.sql(10063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --stmt.strcat("), w AS (SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS WEEK_MIN FROM md GROUP BY YEAR_OF,MONTH_OF");
            Else
               BEGIN
                  INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF)
                  SELECT YEAR_OF, MONTH_OF
                  FROM PMO_TMP_MD md
                  GROUP BY YEAR_OF,MONTH_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 963+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(964) : cplan_fun.sql(10084) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --stmt.strcat("), w AS (SELECT YEAR_OF, MONTH_OF FROM md GROUP BY YEAR_OF,MONTH_OF");
            End If;
            
            -- Create PMO_TMP_EVERY temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
            PMO_DB2M.SC_STATE := 0;
            
            If h_every > 1 Then
               -- Prepare h_pmo_tmp_every_ins_stmt dynamic insert statement to insert records in PMO_TMP_EVERY
               --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
               --                              " SELECT YEAR_OF, MONTH_OF FROM ( "
               --                              " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
               --                              " FROM SESSION.PMO_TMP_SUM w ");
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                      || ' SELECT YEAR_OF, MONTH_OF FROM ( '
                      || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                      || ' FROM PMO_TMP_SUM w ';
               
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(h_work);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
               
               h_min := TO_CHAR(h_every);
               h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ';
               --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
            Else
               -- Prepare h_pmo_tmp_every_ins_stmt dynamic insert statement to insert records in PMO_TMP_EVERY
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) ';
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' SELECT YEAR_OF, MONTH_OF '
                      || ' FROM PMO_TMP_SUM w';
               
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(h_work);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
            End If;
            
            -- execute h_pmo_tmp_every_ins_stmt
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
            
            -- ready to prepare the last select statement with using temporary tables
            If Coalesce(Length(h_work_loc), 0) = 0 Then
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, PMO_ORASTD.MONTH(md.DAY_YEAR), 0 AS CNT '
                      || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                      || ' WHERE md.YEAR_OF = every.YEAR_OF AND '
                      || '       md.MONTH_OF = every.MONTH_OF '
                      || ' ORDER BY md.DAY_YEAR ';
               
               --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, MONTH(md.DAY_YEAR) FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF ORDER BY md.DAY_YEAR");
            Else
               stmt := 
                     'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, every.MONTH_OF, 0 AS CNT '
                      || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                      || ' WHERE md.YEAR_OF = every.YEAR_OF AND '
                      || ' md.MONTH_OF = every.MONTH_OF AND '
                      || ' md.DAY_WEEK IN (';
               
               --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, every.MONTH_OF FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF AND md.DAY_WEEK IN (");
               stmt := stmt || h_work_loc;
               stmt := stmt || ') ORDER BY md.DAY_YEAR';
            End If;
            
            
         WHEN 6 THEN
            -- Monthly, The Day 6 of Jan, Mar, Apr, Nov  Dec
            -- This case never use of temporary tables
            h_days := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_1FFE0000)), 17);
            -- Jan, Mar, Apr, Nov  Dec
            
            If Coalesce(h_days, 0) > 0 Then
               i := 1;
               ii := 1;
               WHILE ii <= 12 LOOP
                  If (BITAND(Coalesce(h_days, 0), i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
               -- The Day X of
               
               h_min := TO_CHAR(Coalesce(h_week, 0));
               
               stmt := stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR)=';
               stmt := stmt || h_min;
               
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
                  stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
               ElsIf Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  stmt := stmt || ' AND p.SCHED_MIN >= ';
               Else
                  stmt := stmt || ' AND p.AVAILABLE_MIN >= ';
               End If;
               h_min := TO_CHAR(h_work);
               
               stmt := stmt || h_min;
               stmt := stmt || ' AND PMO_ORASTD.MONTH(p.DAY_YEAR) IN (';
               --                      stmt.strcat(" AND p.MONTH_OF IN (");
               stmt := stmt || h_work_loc;
               stmt := stmt || ') ORDER BY p.DAY_YEAR';
            End If;
            
            
         
         WHEN 7 THEN
            -- Yearly, The First/Second/Third/Forth/Last Mondy of July, August, Decembre
            -- this case never uses the temporary tables
            h_days := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_1FFE0000)), 17);
            -- July, August, Decembre
            
            If Coalesce(h_days, 0) > 0 Then
               i := 1;
               ii := 1;
               WHILE ii <= 12 LOOP
                  If (BITAND(Coalesce(h_days, 0), i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
               -- First/Second/Third/Forth/Last
               i := 1;
               ii := 1;
               WHILE ii <= 5 LOOP
                  If (BITAND(Coalesce(h_week, 0), i)) = i Then
                     h_week := ii;
                     
                     EXIT;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               h_min := TO_CHAR(7 * (Coalesce(h_week, 0) - 1) + 1);
               h_days := BITAND(h_contour, PMO_CNST.SC_0000007F);
               -- Monday
               
               i := 1;
               ii := 1;
               WHILE ii <= 7 LOOP
                  If (BITAND(Coalesce(h_days, 0), i)) = i Then
                     h_days := ii;
                     
                     EXIT;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               
               stmt := stmt || ' AND PMO_ORASTD.DAY(p.DAY_YEAR) BETWEEN ';
               If Coalesce(h_week, 0) = 5 Then
                  stmt := stmt || 
                        ' CASE WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)=0 THEN 23 WHEN p.MONTH_OF = 2 AND MOD(p.YEAR_OF,4)>0 THEN 22 WHEN p.MONTH_OF IN (4,6,9,11) THEN 24 ELSE 25 END AND 31';
               Else
                  stmt := stmt || h_min;
                  stmt := stmt || ' AND ';
                  h_min := TO_CHAR(7 * Coalesce(h_week, 0));
                  stmt := stmt || h_min;
               End If;
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  stmt := stmt || ' AND PMO_ORASTD.DAYOFWEEK(p.DAY_YEAR)=';
               Else
                  stmt := stmt || ' AND p.DAY_WEEK=';
               End If;
               h_min := TO_CHAR(Coalesce(h_days, 0));
               stmt := stmt || h_min;
               stmt := stmt || ' AND PMO_ORASTD.MONTH(p.DAY_YEAR) IN (';
               stmt := stmt || h_work_loc;
               stmt := stmt || ') ORDER BY p.DAY_YEAR';
            End If;
            
            
         
         WHEN 8 THEN
            -- Yearly, Mo, We, Fr  on Week1, Week4 on Jan, June, July  Dec
            -- This case uses PMO_TMP_RES, PMO_TMP_SUM, PMO_TMP_EVERY, PMO_TMP_MD
            If (BITAND(h_contour, PMO_CNST.SC_0000007F)) > 0 Then
               i := 1;
               ii := 1;
               iii := 0;
               WHILE ii <= 7 LOOP
                  If (BITAND(h_contour, i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               -- Prepare the rest of h_pmo_tmp_res_ins_stmt
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.DAY_WEEK IN (';
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_work_loc;
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ')';
               
               --stmt.strcat(" AND p.DAY_WEEK IN (");
               --stmt.strcat(h_work_loc);
               --stmt.strcat(")");
            End If;
            
            If h_amnt = 1 Then
               If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND p.SCHED_MIN >= ';
               Else
                  h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || ' AND AVAILABLE_MIN >= ';
               End If;
               h_min := TO_CHAR(h_work);
               
               h_pmo_tmp_res_ins_stmt := h_pmo_tmp_res_ins_stmt || h_min;
               --stmt.strcat(h_min);
            End If;
            
            -- end of h_pmo_tmp_res_ins_stmt statement, execute it to prepare data for other temporary tables
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_res_ins_stmt);
            
            -- Use PMO_TMP_RES temporary table to prepare necessary data for PMO_TMP_MD
            -- Create PMO_TMP_MD temporary table
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MD';
            PMO_DB2M.SC_STATE := 0;
            
            -- Prepare dynamic insert statement to insert records in PMO_TMP_MD
            h_pmo_tmp_md_ins_stmt := 'INSERT INTO PMO_TMP_MD(YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN) ';
            h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' SELECT YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN '
                   || ' FROM PMO_TMP_RES pd WHERE ';
            
            --stmt.strcat("), md AS (SELECT YEAR_OF, MONTH_OF, DAY_YEAR, AVAIL_MIN FROM pd WHERE ");
            
            h_week := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_00000F80)), 7);
            -- Week1, Week2, Week3, Week4
            If Coalesce(h_week, 0) > 0 Then
               i := 1;
               ii := 1;
               WHILE ii <= 4 LOOP
                  If (BITAND(Coalesce(h_week, 0), i)) = i Then
                     If iii = 0 Then
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || '(PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                        --stmt.strcat("(DAY(DAY_YEAR) BETWEEN ");
                        iii := 2;
                     ElsIf iii = 1 Then
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND (PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                        --stmt.strcat(" AND (DAY(DAY_YEAR) BETWEEN ");
                        iii := 2;
                     Else
                        h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' OR PMO_ORASTD.DAY(DAY_YEAR) BETWEEN ';
                        --stmt.strcat(" OR DAY(DAY_YEAR) BETWEEN ");
                     End If;
                     
                     h_min := TO_CHAR(7 * (ii - 1) + 1);
                     
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                     h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND ';
                     --stmt.strcat(h_min);
                     --stmt.strcat(" AND ");
                     
                     i := i * 2;
                     ii := ii + 1;
                     WHILE ii <= 5 LOOP
                        If (BITAND(Coalesce(h_week, 0), i)) = 0 Then
                           h_min := TO_CHAR(7 * (ii - 1));
                           
                           h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_min;
                           --stmt.strcat(h_min);
                           
                           EXIT;
                        End If;
                        i := i * 2;
                        ii := ii + 1;
                     END LOOP;
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ')';
               --stmt.strcat(")");
            End If;
            
            h_days := PMO_FUNC.ShiftRight((BITAND(h_contour, PMO_CNST.SC_1FFE0000)), 17);
            -- July, August, Decembre
            
            If Coalesce(h_days, 0) > 0 Then
               i := 1;
               ii := 1;
               WHILE ii <= 12 LOOP
                  If (BITAND(Coalesce(h_days, 0), i)) = i Then
                     If Coalesce(Length(h_work_loc), 0) = 0 Then
                        h_min := TO_CHAR(ii);
                     Else
                        h_min := ',' || TO_CHAR(ii);
                     End If;
                     
                     h_work_loc := SubStr(h_work_loc || h_min, 1, 32);
                  End If;
                  i := i * 2;
                  ii := ii + 1;
               END LOOP;
               If iii = 0 Then
                  h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' PMO_ORASTD.MONTH(DAY_YEAR) IN(';
                  --stmt.strcat(" MONTH(DAY_YEAR) IN(");
               Else
                  h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ' AND PMO_ORASTD.MONTH(DAY_YEAR) IN(';
                  --stmt.strcat(" AND MONTH(DAY_YEAR) IN(");
               End If;
               
               h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || h_work_loc;
               --stmt.strcat(h_work_loc);
               h_pmo_tmp_md_ins_stmt := h_pmo_tmp_md_ins_stmt || ')';
               -- the end of h_pmo_tmp_md_ins_stmt dynamic insert statement for PMO_TMP_MD
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
               
               -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
               -- create PMO_TMP_SUM temporary table instead of w WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
               PMO_DB2M.SC_STATE := 0;
               
               -- Insert records in PMO_TMP_SUM, no need to dynamic insert
               BEGIN
                  INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                  SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                  FROM PMO_TMP_MD md
                  GROUP BY YEAR_OF, MONTH_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1326+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1327) : cplan_fun.sql(10447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- Create PMO_TMP_EVERY temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
               PMO_DB2M.SC_STATE := 0;
            Else
               -- the end of h_pmo_tmp_md_ins_stmt dynamic insert statement for PMO_TMP_MD
               h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_md_ins_stmt);
               
               -- Use PMO_TMP_MD temporary table to prepare data for PMO_TMP_SUM temporary table
               -- create PMO_TMP_SUM temporary table instead of w WITH Clause
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_SUM';
               PMO_DB2M.SC_STATE := 0;
               
               -- Insert records in PMO_TMP_SUM, no need to dynamic insert
               BEGIN
                  INSERT INTO PMO_TMP_SUM(YEAR_OF, MONTH_OF, AVAIL_MIN)
                  SELECT YEAR_OF, MONTH_OF, SUM(AVAIL_MIN) AS AVAIL_MIN
                  FROM PMO_TMP_MD md
                  GROUP BY YEAR_OF, MONTH_OF;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1361+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1362) : cplan_fun.sql(10482) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- Create PMO_TMP_EVERY temporary table
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_EVERY';
               PMO_DB2M.SC_STATE := 0;
            End If;
            
            If h_every > 1 Then
               -- Prepare a dynamic insert statement to insert records in PMO_TMP_EVERY
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) '
                      || ' SELECT YEAR_OF, MONTH_OF FROM ( '
                      || ' SELECT YEAR_OF, MONTH_OF, ' || PMODEFINES.PMO_COUNTER_OVER_YEAROF_MONTHO || ' AS CNT '
                      || ' FROM PMO_TMP_SUM w ';
               
               --h_pmo_tmp_every_ins_stmt.strcpy("INSERT INTO SESSION.PMO_TMP_EVERY(YEAR_OF, MONTH_OF) "
               --                              " SELECT YEAR_OF, MONTH_OF FROM ( "
               --                                " SELECT YEAR_OF, MONTH_OF, COUNTER() AS CNT "
               --                              " FROM SESSION.PMO_TMP_SUM w ");
               
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(h_work);
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
               
               h_min := TO_CHAR(h_every);
               h_tail := ' )  WHERE MOD(CNT-1, ' || h_min || ')=0 ';
               --h_tail.sprintf(" %s ) AS PMO_ALIAS WHERE MOD(CNT-1, %s)=0 ", PMO_ORDER_BY_YEAROF_MONTHOF, h_min);
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_tail;
            Else
               -- Prepare a dynamic insert statement to insert records in PMO_TMP_EVERY
               h_pmo_tmp_every_ins_stmt := 'INSERT INTO PMO_TMP_EVERY(YEAR_OF, MONTH_OF) ';
               h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' SELECT YEAR_OF, MONTH_OF '
                      || ' FROM PMO_TMP_SUM w ';
               
               -- continue concating the rest of h_pmo_tmp_every_ins_stmt
               
               If h_amnt = 0 Then
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || ' WHERE AVAIL_MIN >= ';
                  
                  h_min := TO_CHAR(h_work);
                  
                  h_pmo_tmp_every_ins_stmt := h_pmo_tmp_every_ins_stmt || h_min;
               End If;
            End If;
            
            -- end of h_pmo_tmp_every_ins_stmt, execute it to prepare necessary date
            
            h_temp_sc_state := PMO_STPU.doUpdate(h_pmo_tmp_every_ins_stmt);
            
            -- ready to prepare the last select statement, all temporary tables are ready
            stmt := 
                  'SELECT PMO_ORASTD.JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, PMO_ORASTD.MONTH(md.DAY_YEAR), 0 AS CNT '
                   || ' FROM PMO_TMP_MD md, PMO_TMP_EVERY every '
                   || ' WHERE md.YEAR_OF = every.YEAR_OF AND md.MONTH_OF = every.MONTH_OF '
                   || ' ORDER BY md.DAY_YEAR ';
            
            --stmt.strcat(") SELECT JULIAN_DAY(md.DAY_YEAR), md.DAY_YEAR, md.AVAIL_MIN, MONTH(md.DAY_YEAR) FROM md, every WHERE md.YEAR_OF=every.YEAR_OF AND md.MONTH_OF=every.MONTH_OF ORDER BY md.DAY_YEAR");
            
         ELSE
            NULL;
      END CASE;
   End If;
   -- end of (h_bitflag == 0) or ASAP, No contour
   
   stmt := stmt || '  ';
   
   h_dayofyear := '';
   
   h_totpr := h_total;
   -- save previous value
   h_julian_day := 0;
   h_total_min := 0;
   h_bit_minutes := 0;
   ii := 1;
   If iSF = 1 Then
      h_finish_D := SubStr(h_finish_DT, 1, 9);
   End If;
   
   --trace_printf("\n%s\n",stmt);
   
   OPEN C332 FOR stmt;
   
   h_temp_sc_state := 0;
   LOOP
      FETCH C332 INTO h_julian_day
      , h_dayofyear
      , h_available
      , h_week
      , h_cnt;
      IF C332%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      h_minutes := Coalesce(h_available, 0);
      
      If h_total = 0 AND Coalesce(h_minutes, 0) > 0 Then
         bFirstRequestedDate := CASE WHEN NOT PMO_ORASTD.strncmp(h_start_DT, h_dayofyear
                                 , 10) <> 0 THEN 1 ELSE 0 END;
         
         If NOT bFirstRequestedDate <> 0 Then
            h_skip := 0;
         End If;
      End If;
      
      If h_skip = 1 Then
         FETCH C332 INTO h_julian_day
         , h_dayofyear
         , h_minutes
         , h_week
         , h_cnt;
         IF C332%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_skip := 0;
         
         If h_amnt = 0 Then
            If h_unit = 3 Then
               BEGIN
                  SELECT PMO_ORASTD.WEEK_ISO(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                  INTO h_prev_week
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1506+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1507) : cplan_fun.sql(10627) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  SELECT SUM(SCHED_MIN) INTO h_total_min
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND PMO_ORASTD.YEAR(DAY_YEAR) = PMO_ORASTD.YEAR(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                  AND PMO_ORASTD.WEEK(DAY_YEAR) = PMO_ORASTD.WEEK(TO_DATE(SubStr(h_dayofyear, 1, 10)));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1524+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1525) : cplan_fun.sql(10645) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            If h_unit = 4 Then
               BEGIN
                  SELECT PMO_ORASTD.MONTH(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                  INTO h_prev_week
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1540+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1541) : cplan_fun.sql(10661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  SELECT SUM(SCHED_MIN) INTO h_total_min
                  FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id
                  AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND PMO_ORASTD.YEAR(DAY_YEAR) = PMO_ORASTD.YEAR(TO_DATE(SubStr(h_dayofyear, 1, 10)))
                  AND PMO_ORASTD.MONTH(DAY_YEAR) = h_prev_week;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1558+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1559) : cplan_fun.sql(10679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      If (h_j_disconti_date > 0) AND (Coalesce(h_julian_day, 0) > h_j_disconti_date) 
            AND ((BITAND(Coalesce(h_update, 0), 255)) <> 10) Then
         h_temp_sc_state := PMO_CNST.RESOURCE_NOT_ACTIVE;
         EXIT;
      End If;
      
      h_available := 0;
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
         BEGIN
            SELECT SCHED_MIN, BIT_SCHED INTO h_available, h_bitID
            FROM PMO_TMP_TDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1583+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1584) : cplan_fun.sql(10704) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available, h_bitID
            FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1597+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1598) : cplan_fun.sql(10718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If bFirstRequestedDate <> 0 Then
         --bFirstRequestedDate = 0;
         wBits := PMO_STPU.getFirstBit(h_bitID);
         
         wHour := Trunc(wBits / 12);
         wMin := (wBits MOD 12) * 5;
         
         h_startavailable_time := LPad(wHour, 2,'0') || ':' || LPad(wMin, 2,'0') || ':00';
         --sprintf(h_first_time, "%02d.%02d.00", wHour, wMin);
         
         If Coalesce(Length(h_start_DT), 0) > 10 AND PMO_ORASTD.strncmp(h_startavailable_time, h_start_time
               , 5) < 0 Then
            h_bitID2 := h_bitID;
            PMO_STPU.cleardayStruct(h_startavailable_time, h_start_time, h_bitID);
            --h_minutes -= (getFirstBit(h_bitID.h_value) - wBits) * 5;
         End If;
      End If;
      
      --Nullify(SC_PARAM(h_startavailable_time));
      
      If NOT ((Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_OVERALLOCATE) OR (Coalesce(
                  h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) OR (Coalesce(
                  h_bitflag, 0) = 2)) Then
         h_bit_minutes := FC_CONSIDER_DAY(h_dayofyear, Coalesce(h_available, 0), CASE WHEN h_type <> 0 
                                 OR NOT h_total <> 0 THEN 1 ELSE 0 END, h_min_work_per_day, h_bitID);
         If Coalesce(h_minutes, 0) <= Coalesce(h_bit_minutes, 0) Then
            GOTO Continue0000;
         End If;
      Else
         h_bit_minutes := 0;
      End If;
      
      -- h_minutes = daily available minutes, h_bit_minutes = daily assigned minutes
      
      -- {{ Vladimir Schneider, September 21, 2004
      -- VERIFY: Why ignore other assignments when these conditions are met?
      -- BUG: Possibly a bug, if you have to uncomment this code please describe why and
      -- under what circumstances this is necessary.
      -- DELTA :
      --            if (assign_to_prj  h_bitflag == 1  iSF == 1)
      --            {
      --                h_bit_minutes = 0;
      --            }
      -- }} Vladimir Schneider, September 21, 2004
      
      -- HERE we get availability from the Bits so that overlapped assignments don't
      -- affect the computation.
      If bFirstRequestedDate <> 0 Then
         bFirstRequestedDate := 0;
         If Coalesce(Length(h_start_DT), 0) > 10 AND PMO_ORASTD.strncmp(h_startavailable_time, h_start_time
               , 5) < 0 Then
            h_bitID2 := PMO_STPU.BIT_ANDNOT_DAY(h_bitID2, h_bitID);
            h_minutes := Coalesce(h_minutes, 0) - PMO_STPU.getNumberOfBits(h_bitID2) * 5;
            --h_minutes -= (getFirstBit(h_bitID.h_value) - wBits) * 5;
            --BUG: this includes the brake time (12 to 13)
         ElsIf Coalesce(h_bit_minutes, 0) > 0 Then
            h_minutes := Coalesce(h_minutes, 0) - Coalesce(h_bit_minutes, 0);
         End If;
      ElsIf Coalesce(h_bit_minutes, 0) > 0 Then
         h_minutes := Coalesce(h_minutes, 0) - Coalesce(h_bit_minutes, 0);
      End If;
      
      h_startavailable_time := NULL;
      
      If Coalesce(h_minutes, 0) < 5 Then
         GOTO Continue0000;
      End If;
      
      If ((i_pa_replace = 1) OR ((BITAND(Coalesce(h_update, 0), 255)) = 10)) AND assign_to_prj <> 0 
            AND (h_work > 0) Then
         -- RULE: here we need to find out how much time from the project bucket is
         -- already assigned to tasks and remove this time from the amount to be assigned
         -- on this day. (Can't assign from the same bucket twice.)
         
         h_temp_var := 0;
         BEGIN
            SELECT SCHED_MIN INTO h_temp_var
            FROM PMO_TMP_PA_SUM
            WHERE DAY_YEAR = h_dayofyear;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1688+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1689) : cplan_fun.sql(10809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_temp_var, 0) > 0) Then
            i_work := i_work - Coalesce(h_temp_var, 0);
            
            If i_work < 5 Then
               GOTO Continue0000;
            End If;
         End If;
      End If;
      
      If i_work > 0 Then
         If i_work < 5 Then
            GOTO Continue0000;
         End If;
         
         If h_amnt = 1 Then
            If Coalesce(h_minutes, 0) > i_work Then
               h_minutes := i_work;
            ElsIf Coalesce(h_minutes, 0) < i_work Then
               GOTO Continue0000;
            End If;
         Else
            -- TODO: need to define constants for h_bitflag and put them in pmo_cnst.h
            If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 5) OR (Coalesce(h_bitflag, 0) = 8) Then
               If Coalesce(h_week, 0) <> Coalesce(h_prev_week, 0) Then
                  h_total_min := 0;
                  h_prev_week := Coalesce(h_week, 0);
               End If;
               
               If (Coalesce(h_total_min, 0) + Coalesce(h_minutes, 0)) > i_work Then
                  h_minutes := i_work - Coalesce(h_total_min, 0);
                  If Coalesce(h_minutes, 0) < 5 Then
                     GOTO Continue0000;
                  End If;
               End If;
               
               h_total_min := Coalesce(h_total_min, 0) + Coalesce(h_minutes, 0);
            Else
               If Coalesce(h_minutes, 0) > i_work Then
                  h_minutes := i_work;
               End If;
            End If;
         End If;
      End If;
      
      If (h_total = 0) AND ((h_dayofyear IS NOT NULL)) Then
         -- First assignable day, Fix the start time
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(h_bitID)))
            INTO h_start_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1749+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1750) : cplan_fun.sql(10870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If iSF = 1 Then
         If Coalesce(h_julian_day, 0) > h_julian_finish Then
            h_dayofyear := SubStr(h_finish_D, 1, 10);
            EXIT;
         End If;
         
         If h_day_min <> 0 Then
            h_day_work := h_day_min;
            h_total := h_total + Coalesce(h_day_work, 0);
         ElsIf h_percent_units = 0 Then
            h_total := h_total + Coalesce(h_minutes, 0);
            h_day_work := Coalesce(h_minutes, 0);
         ElsIf h_percent_units = 10000 Then
            If Coalesce(h_minutes, 0) > h_per_day Then
               h_minutes := h_per_day;
            End If;
            
            h_total := h_total + Coalesce(h_minutes, 0);
            h_day_work := Coalesce(h_minutes, 0);
         Else
            -- This should not execute at all because h_day_min
            h_temp_sc_state := PMO_CNST.ERROR_EXECUTED_DEAD_CODE;
            EXIT;
         End If;
         
         h_finish_D := SubStr(h_dayofyear, 1, 10);
         
         h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_day_work);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            EXIT;
         End If;
         
         If Coalesce(h_julian_day, 0) = h_julian_finish Then
            i_break := 1;
         End If;
         
         bAssignStringDone := 1;
      Else
         If h_percent_units = 0 Then
            h_total := h_total + Coalesce(h_minutes, 0);
            h_day_work := Coalesce(h_minutes, 0);
         ElsIf h_percent_units = 10000 Then
            If Coalesce(h_minutes, 0) > h_per_day Then
               h_total := h_total + h_per_day;
               h_day_work := h_per_day;
            Else
               h_total := h_total + Coalesce(h_minutes, 0);
               h_day_work := Coalesce(h_minutes, 0);
            End If;
         ElsIf h_day_min <> 0 Then
            If h_day_min > Coalesce(h_minutes, 0) AND (h_day_min - Coalesce(h_minutes, 0)) > 4 
                        AND h_percent_units <= 10000 Then
               h_day_work := Coalesce(h_minutes, 0);
            Else
               h_day_work := h_day_min;
            End If;
            
            h_total := h_total + Coalesce(h_day_work, 0);
         Else
            h_day_work := Trunc((Coalesce(h_minutes, 0) * h_percent_units + 5000.00) / 10000.00);
            If Coalesce(h_day_work, 0) < 5 Then
               h_day_work := 5;
            End If;
            h_total := h_total + Coalesce(h_day_work, 0);
         End If;
         
         bAssignStringDone := 0;
      End If;
      
      If (h_effort > 0) AND (h_total >= h_effort) Then
         h_day_work := h_effort - h_totpr;
         i_break := 1;
      End If;
      
      If NOT bAssignStringDone <> 0 Then
         h_temp_sc_state := FC_ASSIGN_STRING(h_julian_day, h_day_work);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            EXIT;
         End If;
      End If;
      
      If h_type > 0 Then
         If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
            h_temp_sc_state := Coalesce(h_update, 0);
         Else
            h_temp_sc_state := 11;
         End If;
         h_temp_sc_state := FC_INSERT_DAY_OF_YEAR(h_work_type, h_day_work, h_update, h_project_id, h_task_id
                                         , h_res_id, h_res_type, h_prj_assignment_id
                                         , h_tsk_assignment_id, h_dayofyear, h_startavailable_time
                                         , h_temp_sc_state, h_consider, h_table_type, h_bitID);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            EXIT;
         End If;
      ElsIf i_break = 1 Then
         PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_day_work, 0), h_bitID);
      End If;
      
      If i_break = 1 Then
         EXIT;
      End If;
      
      h_totpr := h_total;
      -- save previous value
      h_startavailable_time := NULL;
      <<Continue0000>>
      NULL;
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE C332 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1876+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1877) : cplan_fun.sql(10997) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_sc_state, 0) <> 0 AND Coalesce(h_temp_sc_state, 0) <> 100 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1883+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1884) : cplan_fun.sql(11004) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN Coalesce(h_temp_sc_state, 0);
   End If;
   
   If h_skip <> 0 Then
      -- did not manage to skip one, had no data
      If (h_prj_assignment_id IS NOT NULL) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1892+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1893) : cplan_fun.sql(11013) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1897+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1898) : cplan_fun.sql(11018) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
   End If;
   
   If iSF = 1 Then
      If h_total = 0 Then
         -- TODO: these should be constants
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1907+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1908) : cplan_fun.sql(11028) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 110;
      Else
         h_totpr := 0;
         /*if ( (h_effort > 0)  (h_task_type == SC_FD_EFFORT_DRIVEN) )
         				{
         					h_total = h_effort;
         				}
         				else
         				{*/
         h_effort := h_total;
         --}
      End If;
   ElsIf h_total < h_effort Then
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If l_calls > PMO_CNST.SC_MAX_CALLS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1925+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1926) : cplan_fun.sql(11046) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
         
         If (h_dayofyear IS NOT NULL) Then
            h_start_DT := h_dayofyear;
            h_skip := 1;
         Else
            BEGIN
               SELECT TO_DATE(SubStr(h_start_DT, 1, 10)) + h_duration      INTO h_start_DT FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 1942+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(1943) : cplan_fun.sql(11063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         h_finish_DT := '';
         l_calls := l_calls + Coalesce(h_duration, 0);
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_PROJECT) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1953+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1954) : cplan_fun.sql(11074) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
         Else
            If FC_CREATE_CDY(h_res_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_RESOURCE) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1960+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1961) : cplan_fun.sql(11081) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
            End If;
         End If;
         
         h_temp_sc_state := FC_SEARCH(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                             , h_prj_assignment_id, h_start_date, h_start_DT, h_finish_DT
                             , h_start_time, h_finish_time, h_consider, h_res_type, h_work_type
                             , h_bitflag, h_update, h_table_type, h_duration, h_total, h_effort
                             , h_work, iSF, h_contour, h_every, h_type, h_skip, h_percent_units
                             , h_amnt, h_unit, h_day_min, h_per_day, h_min_work_per_day
                             , h_julian_finish, h_j_disconti_date, assign_to_prj, i_pa_replace
                             , l_calls);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1977+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1978) : cplan_fun.sql(11098) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 1983+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(1984) : cplan_fun.sql(11104) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
   End If;
   
   h_start_DT := h_start_date;
   -- Update finish time
   If Coalesce(Length(h_finish_time), 0) < 5 OR PMO_ORASTD.strncmp(h_finish_time, '00', 2) = 0 
         OR PMO_ORASTD.strncmp(h_finish_DT, h_dayofyear, 10) <> 0 Then
      wHour001 := 0;
      wMin001 := 0;
      wBits001 := 0;
      
      If h_type = 0 Then
         -- get the time
         If h_percent_units > 10000 Then
            If h_totpr > Coalesce(h_available, 0) Then
               h_totpr := 0;
            End If;
         End If;
         
         If h_totpr < 5 Then
            wBits001 := PMO_STPU.getLastBit(h_bitID);
         Else
            If Coalesce(h_bit_minutes, 0) > 0 Then
               h_day_work := Coalesce(h_day_work, 0) + Coalesce(h_bit_minutes, 0);
            End If;
            
            If Coalesce(h_day_work, 0) < Coalesce(h_available, 0) Then
               wBits001 := PMO_STPU.getFirstBitSkipFinish(h_bitID, Trunc(Coalesce(h_day_work, 0) / 5));
            Else
               wBits001 := PMO_STPU.getLastBit(h_bitID);
            End If;
         End If;
      Else
         wBits001 := PMO_STPU.getLastBit(h_bitID);
      End If;
      
      wHour001 := Trunc(wBits001 / 12);
      wMin001 := (wBits001 MOD 12) * 5;
      
      -- Fix minute if (h_day_work / 5) > 0
      --h_temp_var = h_day_work % 5;
      --if (h_temp_var > 0)
      --{
      -- wMin -= 5 - h_temp_var;
      --}
      
      h_finish_time := LPad(TO_CHAR(wHour001), 2, '0') || '.' || LPad(TO_CHAR(wMin001), 2, '0') || '.00';
      
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_finish_time))
         INTO h_finish_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH', 2044+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH(2045) : cplan_fun.sql(11165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH', 2051+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH(2052) : cplan_fun.sql(11172) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEARCH;
ALTER FUNCTION FC_SEARCH COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- search function

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_SEARCH_2 ==
CREATE OR REPLACE FUNCTION FC_SEARCH_2
(
     assign_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , sh_h_res_cost NUMBER
   , sh_h_type PLS_INTEGER
   , h_and_roles IN OUT NOCOPY VARCHAR2
   , h_and_skills IN OUT NOCOPY VARCHAR2
   , h_shw_roles IN OUT NOCOPY VARCHAR2
   , h_shw_skills IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_cri PLS_INTEGER
   , sh_h_consider PLS_INTEGER
   , h_res_type IN OUT NOCOPY VARCHAR2
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , sh_h_update PLS_INTEGER
   , sh_h_per_day PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_calculate_flag PLS_INTEGER := 0
   , sh_delay PLS_INTEGER := 0
   , sh_p_reg_sale NUMBER := 0.00
   , sh_p_spec_sale NUMBER := 0.00
) RETURN PLS_INTEGER
AS
   h_res_cost NUMBER := sh_h_res_cost;
   h_type PLS_INTEGER := sh_h_type;
   h_duration PLS_INTEGER := sh_h_duration;
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_cri PLS_INTEGER := sh_h_cri;
   h_consider PLS_INTEGER := sh_h_consider;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_update PLS_INTEGER := sh_h_update;
   h_per_day PLS_INTEGER := sh_h_per_day;
   h_priority PLS_INTEGER := sh_h_priority;
   calculate_flag PLS_INTEGER := Coalesce(sh_calculate_flag, 0);
   delay PLS_INTEGER := Coalesce(sh_delay, 0);
   p_reg_sale NUMBER := Coalesce(sh_p_reg_sale, 0);
   p_spec_sale NUMBER := Coalesce(sh_p_spec_sale, 0);
   h_assign_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_min VARCHAR2(19);
   h_startavailable_time VARCHAR2(8);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_work_loc VARCHAR2(32);
   h_dayofyear VARCHAR2(10);
   h_finish_D VARCHAR2(10);
   h_active VARCHAR2(1);
   h_assign_string VARCHAR2(8192);
   h_lev_assign_string VARCHAR2(8192);
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_ta_start_DT VARCHAR2(26);
   h_ta_finish_DT VARCHAR2(26);
   h_lev_start_DT VARCHAR2(26);
   h_lev_finish_DT VARCHAR2(26);
   h_act_start_DT VARCHAR2(26);
   h_start_D DATE;
   h_type_id PLS_INTEGER := 0;
   h_cnt PLS_INTEGER := 0;
   h_min_work_per_day PLS_INTEGER := 0;
   h_lev_effort PLS_INTEGER := 0;
   h_lev_consider PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_julian_start PLS_INTEGER := 0;
   h_julian_finish PLS_INTEGER := 0;
   h_total_min PLS_INTEGER := 0;
   h_work_units PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_available PLS_INTEGER := 0;
   h_prev_day PLS_INTEGER := 0;
   h_planned_work PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_rate_echange NUMBER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_save PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_prj_percent_units PLS_INTEGER := 0;
   h_tmp_percent_units PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_work_per_day PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   assign_to_prj PLS_INTEGER := 0;
   h_proj_assign_flag PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   tempStr VARCHAR2(255);
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
   i_lev_etc_more PLS_INTEGER NOT NULL := 0;
   i_has_actual PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER NOT NULL := 0;
   ii PLS_INTEGER NOT NULL := 9;
   iii PLS_INTEGER NOT NULL := 0;
   h_amnt PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   i_pa_replace PLS_INTEGER NOT NULL := 0;
   token VARCHAR2(32);
   i_noskill PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
   l_calls PLS_INTEGER;
   l_data_flag PLS_INTEGER;
   h_default_flag VARCHAR2(1);
   null_return LONG;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_DELAY1 PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_DELAY0 PMO_DB2M.REF_CURSOR;

      CURSOR C_LS IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_res_id AND DAY_YEAR > h_start_D
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_TA IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id AND DAY_YEAR > h_start_D
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEARCH_2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEARCH_2'), '-- ' || SYSTIMESTAMP, ':r := FC_SEARCH_2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_res_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_and_roles || ''''
          || ', ' || '''' || h_and_skills || ''''
          || ', ' || '''' || h_shw_roles || ''''
          || ', ' || '''' || h_shw_skills || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_cri), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_consider), 'NULL')
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_calculate_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_reg_sale), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_p_spec_sale), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 205+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEARCH_2(206) : cplan_fun.sql(11392) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_assign_id := assign_id;
   h_delay := delay;
   h_rate_echange := 1.00;
   assign_to_prj := 0;
   h_work_todate := 0;
   
   If (Coalesce(Length(h_and_roles), 0) > PMO_CNST.SC_H_V_LONG_DESC) 
         OR (Coalesce(Length(h_and_skills), 0) > PMO_CNST.SC_H_V_LONG_DESC) 
         OR (Coalesce(Length(h_shw_roles), 0) > PMO_CNST.SC_H_V_LONG_DESC) 
         OR (Coalesce(Length(h_shw_skills), 0) > PMO_CNST.SC_H_V_LONG_DESC) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 221+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(222) : cplan_fun.sql(11408) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ASSIGN_STRING_TOLONG;
   End If;
   
   h_j_disconti_date := 0;
   
   h_prj_assignment_id := NULL;
   h_billable := 0;
   
   If Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30 Then
      -- h_skip skips first day from found search.
      h_skip := 1;
      h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
   End If;
   h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0001F000)), 12);
   
   --iSF == 0 when h_start_date or h_finish_date is null
   
   If (Coalesce(h_type, 0) > 0) AND (Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30) Then
      h_rollup := PMO_CNST.ROLLUP_ASSIGN_FROM_PROJECT;
      -- set assign from constrained by project (type > 0)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove assign from constrained by project (type > 0)
      
      If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_29 Then
         h_rollup := PMO_CNST.ROLLUP_DONT_ROLLUP;
         -- set do not rollup
         h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_29));
         -- remove do not rollup
      End If;
   ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
      h_rollup := PMO_CNST.ROLLUP_DONT_ROLLUP_IMPORT_MSP;
      -- set do not rollup (import from MSP)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
      -- remove do not rollup
   ElsIf (Coalesce(h_type, 0) > 0) AND (Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27) Then
      h_rollup := PMO_CNST.ROLLUP_RESCHEDULE_ETC;
      -- re_etc schedule
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
   Else
      h_rollup := 0;
   End If;
   
   If Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT Then
      If Coalesce(Length(h_start_date), 0) = 10 Then
         h_start_date := h_start_date || ' 00:00:00';
      ElsIf Coalesce(h_consider, 0) = 7 AND (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_date, 1, 10)) , PMO_ORASTD.TIME('00:00:00'))
            INTO h_start_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 279+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(280) : cplan_fun.sql(11466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (Coalesce(h_effort, 0) = 0) AND ((Coalesce(Length(h_finish_date), 0) < 10)) Then
         h_finish_date := h_start_date;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 288+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(289) : cplan_fun.sql(11475) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   End If;
   
   If Coalesce(h_work, 0) >= PMO_CNST.SC_BIT_30 Then
      -- replace profile, assign remaining time @ the project level also to the resource
      i_pa_replace := 1;
      h_work := (BITAND(Coalesce(h_work, 0), PMO_CNST.SC_BIT_NOT_30));
   End If;
   
   If Coalesce(h_unit, 0) = 0 Then
      h_amnt := 1;
   ElsIf (Coalesce(h_unit, 0) = 100) AND (Coalesce(h_work, 0) > 0) Then
      h_percent_units := Coalesce(h_work, 0);
      h_work_units := Coalesce(h_work, 0);
      If Coalesce(h_duration, 0) > 0 AND Coalesce(h_work, 0) < 10000 Then
         h_duration := Trunc(Coalesce(h_duration, 0) * 10000 / Coalesce(h_work, 0));
      End If;
      h_work := 0;
      h_occurrence := 0;
   End If;
   
   If Coalesce(h_consider, 0) = 0 Then
      h_consider := PMO_CNST.SC_SRCH_OVERALLOCATE;
      -- allow overallocation
   End If;
   
   If (h_start_date IS NULL) Then
      h_start_DT := '';
   End If;
   If (h_finish_date IS NULL) Then
      h_finish_DT := '';
   End If;
   
   If Coalesce(Length(h_start_date), 0) >= 10 AND Coalesce(Length(h_finish_date), 0) >= 10 Then
      iSF := 1;
      h_duration := 0;
      If Coalesce(Length(h_finish_date), 0) = 10 Then
         h_finish_date := h_finish_date || ' 00:00:00';
      End If;
      
      h_start_DT := h_start_date;
      h_finish_DT := h_finish_date;
   Else
      If Coalesce(h_duration, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 336+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(337) : cplan_fun.sql(11523) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 220;
      End If;
      iSF := 0;
      h_finish_DT := h_finish_date;
      h_start_DT := h_start_date;
   End If;
   
   If Coalesce(h_type, 0) <> 0 Then
      /*BEGIN_SQL
      			SELECT TASK_TYPE, UNRESOLVED_START, UNRESOLVED_FINISH, ETC, WORK_TODATE, START_DT
      			INTO HOST(h_task_type), HOST(h_task_start_DT), HOST(h_task_finish_DT), HOST(h_etc), HOST(h_work_todate), HOST(h_act_start_DT)
      			FROM TMT_WBS
      			WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id) ;
      		END_SQL;*/
      
      If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
         h_task_type := 0;
      End If;
      
      If Coalesce(h_delay, 0) > 0 Then
         -- Move start date based on delay and calendar information
         h_dayofyear := NULL;
         stmt := '';
         
         /*if ( ((h_update  0x000FF) == 10)  (h_work_todate > 0) )
         			{
         				InitializeHost(SC_PARAM(h_start_DT), SC_PARAM(h_act_start_DT));
         			}*/
         
         If (SubStr(h_res_type, 1, 1) = '0') Then
            stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                   || 'WHERE ELEMENT_ID = ''' || h_res_id || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_start_DT 
                        || ''', 1, 10)) '
                   || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
            
            OPEN C_DELAY1 FOR stmt;
            
            LOOP
               FETCH C_DELAY1 INTO h_dayofyear;
               IF C_DELAY1%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C_DELAY1;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 395+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(396) : cplan_fun.sql(11582) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                   || 'WHERE ELEMENT_ID = ''' || h_project_id || ''' AND DAY_YEAR > TO_DATE(SubStr(''' || h_start_DT 
                        || ''', 1, 10)) '
                   || 'ORDER BY DAY_YEAR) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
            
            OPEN C_DELAY0 FOR stmt;
            
            LOOP
               FETCH C_DELAY0 INTO h_dayofyear;
               IF C_DELAY0%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C_DELAY0;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 427+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(428) : cplan_fun.sql(11614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (h_dayofyear IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 435+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(436) : cplan_fun.sql(11622) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_NOT_FOUND;
         Else
            -- TOCONFIRM: when time is passed in with delay, should the time be kept after
            -- the delay.
            If Coalesce(Length(h_start_DT), 0) > 10 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_DT))
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 452+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(453) : cplan_fun.sql(11639) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME('00:00'))
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 467+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(468) : cplan_fun.sql(11654) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_temp_var := 0;
            h_start_date := h_start_DT;
            
            If iSF = 0 AND Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT AND Coalesce(
                  h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
               -- Adjust the "approximate" finish date by the delay
               If Coalesce(Length(h_finish_DT), 0) > 10 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(h_finish_DT) + h_delay     
                     INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 489+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(490) : cplan_fun.sql(11676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(Length(h_finish_DT), 0) > 0 Then
                  BEGIN
                     SELECT TO_DATE(SubStr(h_finish_DT, 1, 10)) + h_delay      INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 502+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(503) : cplan_fun.sql(11689) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
      End If;
   End If;
   
   If (BITAND(Coalesce(h_update, 0), 256)) = 256 Then
      --*****************************************************************************
      --
      --    Optimization for Calculate
      --
      --****X******************X************************X************************X***
      h_start_D := NULL;
      h_lev_start_DT := NULL;
      h_lev_finish_DT := NULL;
      h_remaining := h_effort;
      
      If Coalesce(h_work_type, 0) = 2 Then
         BEGIN
            SELECT LAST_SUBMITTED, LAST_SCHED_MIN, SCHED_START, SCHED_FINISH, SCHED_WORK - WORK_TODATE,
            LEVEL_START, LEVEL_FINISH, LEVEL_EFFORT, CONSIDER, ASSIGN_STRING, PROJECT_ASSIGNMENT_ID
            INTO h_start_D, h_last_day, h_ta_start_DT, h_ta_finish_DT, h_remaining,
            h_lev_start_DT, h_lev_finish_DT, h_lev_effort, h_lev_consider, h_lev_assign_string, h_prj_assignment_id
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 537+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(538) : cplan_fun.sql(11724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_start_D IS NOT NULL) Then
            h_act_start_DT := h_ta_start_DT;
            i_has_actual := 1;
            
            If Coalesce(h_last_day, 0) > 0 Then
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assignment_id
                                   , h_assign_id, h_start_D, h_last_day, 0);
            End If;
            
            -- get start first datetime > last_submitted
            
            OPEN C_LS;
            
            FETCH C_LS INTO h_ta_start_DT;
            IF C_LS%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (PMO_ORASTD.strncmp(h_start_date, h_ta_start_DT
                  , 16) <= 0) Then
               h_start_date := h_ta_start_DT;
            End If;
            
            BEGIN
               CLOSE C_LS ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 575+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(576) : cplan_fun.sql(11762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- get the real assignment start date
            
            OPEN C_TA;
            
            FETCH C_TA INTO h_ta_start_DT;
            IF C_TA%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE C_TA ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 599+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(600) : cplan_fun.sql(11786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH, LEVEL_START, LEVEL_FINISH, LEVEL_EFFORT,
            CONSIDER, PLANNED_WORK, ASSIGN_STRING, PROJECT_ASSIGNMENT_ID
            INTO h_ta_start_DT, h_ta_finish_DT, h_lev_start_DT, h_lev_finish_DT, h_lev_effort,
            h_lev_consider, h_remaining, h_lev_assign_string, h_prj_assignment_id
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 618+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(619) : cplan_fun.sql(11805) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (Coalesce(h_lev_consider, 0) = 0) OR (Coalesce(h_lev_consider, 0) = 10) Then
         h_lev_consider := 7;
      End If;
      
      If Coalesce(h_last_day, 0) > 0 Then
         h_start_date := h_ta_start_DT;
      ElsIf ((h_lev_start_DT IS NOT NULL)) AND (Coalesce(h_lev_effort, 0) = Coalesce(h_effort, 0)) 
            AND (PMO_ORASTD.strncmp(h_start_date, h_lev_start_DT, 16) = 0) 
            AND (calculate_flag = 1) Then
         h_start_date := h_lev_start_DT;
         h_finish_date := h_lev_finish_DT;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 637+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(638) : cplan_fun.sql(11824) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      ElsIf (PMO_ORASTD.strncmp(h_start_date, h_ta_start_DT, 16) = 0) AND (Coalesce(
            h_lev_consider, 0) = 7) AND (Coalesce(h_remaining, 0) = Coalesce(h_effort, 0)) Then
         h_finish_date := h_ta_finish_DT;
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            LEVEL_START         = h_start_date,
            LEVEL_FINISH        = h_ta_finish_DT,
            LEVEL_EFFORT        = h_effort,
            LEVEL_ASSIGN_STRING = ASSIGN_STRING
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 661+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(662) : cplan_fun.sql(11848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM LEVEL_TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 679+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(680) : cplan_fun.sql(11866) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 685+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(686) : cplan_fun.sql(11872) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      ElsIf (Coalesce(h_effort, 0) > Coalesce(h_remaining, 0)) AND (PMO_ORASTD.strncmp(h_start_date
                                        , h_ta_start_DT, 16) = 0) Then
         BEGIN
            SELECT SCHED_MIN INTO h_minutes
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id
            AND DAY_YEAR = TO_DATE(SubStr(h_ta_finish_DT, 1, 10)) AND BIT_FLAG = '2' AND ACTUAL_FLAG = 0;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 701+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(702) : cplan_fun.sql(11888) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If i_has_actual = 0 Then
               h_act_start_DT := h_start_date;
            End If;
            i_lev_etc_more := 1;
            h_effort := Coalesce(h_effort, 0) + Coalesce(h_minutes, 0) - Coalesce(h_remaining, 0);
            
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_ta_finish_DT, 1, 10)),PMO_ORASTD.TIME(h_ta_start_DT)) INTO h_start_date FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 721+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(722) : cplan_fun.sql(11908) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      h_assign_string := '';
      h_julian_day := 1;
      
      BEGIN
         SELECT PMO_ORASTD.JULIAN_DAY(h_start_date)
         , ACTIVE
         , COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0)
         , COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
         INTO h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
         FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 746+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(747) : cplan_fun.sql(11933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- TOVERIFY: participants are not checked for data >= effective date
      -- need to verify if active status reflects effective date.
      If SubStr(h_active, 1, 1) = '0' OR (Coalesce(h_j_disconti_date, 0) > 0 AND Coalesce(
            h_julian_day, 0) > Coalesce(h_j_disconti_date, 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 756+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(757) : cplan_fun.sql(11943) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
      End If;
   Else
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         --*****************************************************************************
         --
         --    Level
         --
         --****X******************X************************X************************X***
         h_lev_start_DT := NULL;
         h_lev_finish_DT := NULL;
         
         BEGIN
            SELECT LAST_SUBMITTED, SCHED_START, LEVEL_START, LEVEL_FINISH
            INTO h_start_D, h_ta_start_DT, h_lev_start_DT, h_lev_finish_DT
            FROM TASK_ASSIGNMENTS
            WHERE TSK_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 781+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(782) : cplan_fun.sql(11968) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_start_D IS NOT NULL) Then
            h_act_start_DT := h_ta_start_DT;
            i_has_actual := 1;
         End If;
         
         If ((h_lev_start_DT IS NOT NULL) OR (h_lev_finish_DT IS NOT NULL)) Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET LEVEL_START = NULL, LEVEL_FINISH = NULL, LEVEL_EFFORT = 0, LEVEL_ASSIGN_STRING = NULL
               WHERE TSK_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 805+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(806) : cplan_fun.sql(11992) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM LEVEL_TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 822+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(823) : cplan_fun.sql(12009) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_day_min := 0;
      
      If Coalesce(h_type, 0) > 0 Then
         -- Get table type, needed for INSERT TDY
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_RESCHEDULE_ETC OR (BITAND(Coalesce(
               h_update, 0), 255)) = 10 Then
            h_table_type := PMO_CNST.SC_TABLE_WBS;
         Else
            h_table_type := GetTable(Coalesce(h_type, 0));
         End If;
         
         -- NOTE: assign_to_prj is == 1 for true
         assign_to_prj := CASE WHEN PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 THEN 1 ELSE 0 END;
         
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_RESCHEDULE_ETC OR (BITAND(Coalesce(
               h_update, 0), 255)) = 10 Then
            h_tsk_assignment_id := assign_id;
            
            If (Coalesce(h_percent_units, 0) = 0 AND Coalesce(h_work, 0) = 0 AND Coalesce(h_bitflag, 0) = 1) 
                  OR ((BITAND(Coalesce(h_update, 0), 255)) = 10) Then
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, UNITS_MSP, REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
                  INTO h_prj_assignment_id, h_percent_units, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TASK_ASSIGNMENTS 
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 860+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(861) : cplan_fun.sql(12047) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
                  INTO h_prj_assignment_id, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TASK_ASSIGNMENTS 
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 876+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(877) : cplan_fun.sql(12063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_work_type, 0) = 2 Then
               -- schedule
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, UNITS_MSP, REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
                  INTO h_prj_assignment_id, h_prj_percent_units, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '2';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 895+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(896) : cplan_fun.sql(12082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_work_type, 0) = 0 Then
               -- planned
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, UNITS_MSP, REGULAR_COST, REGULAR_RATE, SPECIAL_COST, SPECIAL_RATE
                  INTO h_prj_assignment_id, h_prj_percent_units, h_reg_cost, h_reg_sale, h_spec_cost, h_spec_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 912+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(913) : cplan_fun.sql(12099) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
                  h_percent_units, 0) > Coalesce(h_prj_percent_units, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 921+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(922) : cplan_fun.sql(12108) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
            End If;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND isProjectType(Coalesce(h_type, 0)) <> 0 Then
               h_tsk_assignment_id := h_prj_assignment_id;
            Else
               h_tsk_assignment_id := SYS_GUID();
            End If;
         End If;
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TDOY';
         PMO_DB2M.SC_STATE := 0;
         
         /*
         //*****************************************************************************
         //
         //    Fixed Duration Effort Driven Tasks (may come back, do not remove)
         //
         //****X******************X************************X************************X***
         
         			if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_etc > 0)  ((h_bitflag == 0) || ((h_bitflag == 1)))  (h_rollup == 0) )
         			{
         				iSF = 1;
         				h_work = 0;
         				if (h_consider != SC_SRCH_PROFILE)
         				{
         					h_consider = 7;
         				}
         
         				BEGIN_SQL
         					SELECT COUNT(*) INTO HOST(h_cnt)
         					FROM TASK_ASSIGNMENTS
         					WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id);
         				END_SQL;
         
         				if (h_cnt == 0) // this is the first assignment
         				{
         					h_percent_units = 0;
         				}
         				else
         				{
         					h_temp_sc_state = FC_FD_EFFORT_DRIVEN(
         									SC_PARAM(h_project_id),
         									SC_PARAM(h_task_id),
         									SC_PARAM(h_res_id),
         									SC_PARAM(h_res_type),
         									SC_PARAM(h_task_start_DT),
         									SC_PARAM(h_task_finish_DT),
         									SC_PARAM(h_etc),
         									SC_PARAM(h_work_todate),
         									SC_PARAM(h_cnt),
         									SC_PARAM(h_per_day));
         
         					if ( h_temp_sc_state != 0 )
         					{
         						return h_temp_sc_state;
         					}
         
         					BEGIN_SQL
         						SELECT RESUME_DT, NEW_REMAINING, NEW_UNITS
         						INTO HOST(h_start_DT), HOST(h_effort), HOST(h_percent_units)
         						FROM session.PMO_TMP_FIXDUR WHERE RESOURCE_ID = HOST(h_res_id);
         					END_SQL;
         
         //                  InitializeHost(SC_PARAM(h_effort), 0);
         					InitializeHost(SC_PARAM(h_finish_DT), h_task_finish_DT);
         				}
         			}*/
         
         If (BITAND(Coalesce(h_update, 0), 255)) <> 10 AND Coalesce(h_unit, 0) <> 100 AND Coalesce(
                    h_work, 0) > 0 Then
            h_percent_units := 0;
            h_day_min := Coalesce(h_work, 0);
         End If;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         If (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
            If Coalesce(h_duration, 0) = 0 Then
               BEGIN
                  SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT),
                  PMO_ORASTD.JULIAN_DAY(h_finish_DT),
                  ACTIVE,
                  COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),
                  COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
                  INTO h_julian_day,
                  h_julian_finish,
                  h_active,
                  h_j_efective_date,
                  h_j_disconti_date
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1022+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1023) : cplan_fun.sql(12209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_julian_finish, 0) > 0 AND Coalesce(h_julian_finish, 0) < Coalesce(
                     h_j_efective_date, 0) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1030+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1031) : cplan_fun.sql(12217) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
               End If;
            Else
               -- TODO: possibly merge with similar code for participant, above.
               BEGIN
                  SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT)
                  , ACTIVE
                  , COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0)
                  ,COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
                  INTO h_julian_day
                  , h_active
                  , h_j_efective_date
                  , h_j_disconti_date
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1053+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1054) : cplan_fun.sql(12240) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_julian_finish := 0;
            End If;
            
            If SubStr(h_active, 1, 1) = '0' OR (Coalesce(h_j_disconti_date, 0) > 0 AND Coalesce(
                  h_julian_day, 0) > Coalesce(h_j_disconti_date, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1064+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1065) : cplan_fun.sql(12251) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
            End If;
         End If;
         
         If Coalesce(h_julian_day, 0) < Coalesce(h_j_efective_date, 0) Then
            -- adjust start date to be at or after effective date
            If Coalesce(Length(h_start_DT), 0) > 10 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(h_start_DT) + (h_j_efective_date - h_julian_day)     
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1083+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1084) : cplan_fun.sql(12270) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT TO_DATE(SubStr(h_start_DT, 1, 10)) + (h_j_efective_date - h_julian_day)     
                  INTO h_start_DT
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1098+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1099) : cplan_fun.sql(12285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_skip := 0;
         End If;
      Else
         -- PROFILE
         h_j_efective_date := 0;
         h_j_disconti_date := 0;
         h_active := '';
         If iSF = 1 Then
            BEGIN
               SELECT PMO_ORASTD.JULIAN_DAY(h_start_DT),
               PMO_ORASTD.JULIAN_DAY(h_finish_DT)
               INTO h_julian_day,
               h_julian_finish
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1124+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1125) : cplan_fun.sql(12311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_julian_day, 0) > Coalesce(h_julian_finish, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1131+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1132) : cplan_fun.sql(12318) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RESOURCE_NOT_ACTIVE;
            End If;
         End If;
      End If;
      
      --AEL: Loading the data to the tmp table
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         --t_isLevel = 0;
         If Coalesce(h_work, 0) >= PMO_CNST.SC_BIT_30 Then
            h_min_work_per_day := (BITAND(Coalesce(h_work, 0), PMO_CNST.SC_BIT_NOT_30));
            h_work := 0;
         Else
            h_min_work_per_day := 0;
         End If;
         
         null_return := FC_CREATE_TDY_LEVEL(h_project_id, h_res_id, h_start_DT, h_consider, h_cri, h_priority
                                   , h_update);
         
         If Coalesce(assign_to_prj, 0) <> 0 Then
            h_consider := 6;
            -- if project overwrite calculate
            BEGIN
               SELECT COUNT(RESOURCE_ID) INTO h_temp_var
               FROM PMO_TMP_TDY_LEVEL
               WHERE RESOURCE_ID = h_res_id AND PROJECT_ID = h_project_id AND PROJ_ASSIGN_FLAG = 0;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1164+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1165) : cplan_fun.sql(12351) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) = 0 Then
               BEGIN
                  INSERT INTO PMO_TMP_TDY
                  SELECT TSK_ASSIGNMENT_ID, DAY_YEAR, SCHED_MIN END,BIT_FLAG, BIT_SCHED
                  FROM LEVEL_TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND TASK_ID <> PROJECT_ID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1185+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1186) : cplan_fun.sql(12372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         null_return := FC_CREATE_TDY(h_project_id, h_res_id, h_start_DT, h_consider, h_cri, h_priority
                             , h_update);
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If Coalesce(h_duration, 0) > 0 Then
            If Coalesce(h_bitflag, 0) > 1 Then
               h_work_per_day := 0;
            Else
               h_work_per_day := h_work;
            End If;
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
                  If Coalesce(h_min_work_per_day, 0) > 0 Then
                     h_work_per_day := Coalesce(h_min_work_per_day, 0);
                  ElsIf (Coalesce(h_work_per_day, 0) > 0) AND Coalesce(assign_to_prj, 0) <> 0 Then
                     BEGIN
                        SELECT MAX(SCHED_MIN)
                        INTO h_min_work_per_day
                        FROM PMO_TMP_PA_SUM;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1219+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1220) : cplan_fun.sql(12406) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If Coalesce(h_min_work_per_day, 0) > 0 AND Coalesce(h_min_work_per_day, 0) <= Coalesce(
                           h_work_per_day, 0) Then
                        h_work_per_day := Coalesce(h_work_per_day, 0) - Coalesce(h_min_work_per_day, 0);
                     ElsIf Coalesce(h_min_work_per_day, 0) > 0 Then
                        -- overallocated from the project
                        -- TODO: print an exception to the log.
                        NULL;
                     End If;
                  End If;
               End If;
               null_return := CHECK_ASSIGN(h_project_id, h_start_DT, h_work_per_day);
            ElsIf NOT (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_OVERALLOCATE OR Coalesce(
                    h_bitflag, 0) = 4) Then
               If Coalesce(h_min_work_per_day, 0) > 0 Then
                  h_work_per_day := Coalesce(h_min_work_per_day, 0);
               End If;
               null_return := CHECK_ASSIGN(h_res_id, h_start_DT, h_work_per_day);
            End If;
         Else
            -- this is also done in in CHECK_ASSIGN
            BEGIN
               INSERT INTO PMO_TMP_CPLNF_0001 SELECT DAY_YEAR, SUM(SCHED_MIN)
               FROM PMO_TMP_TDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_DT, 1, 10))
               GROUP BY DAY_YEAR;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1258+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1259) : cplan_fun.sql(12445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      -- TOVALIDATE: for leveling this will create CDY, does leveling create its own?
      -- Extend resource calendar
      If SubStr(h_res_type, 1, 1) = '1' Then
         If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_PROJECT) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1270+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1271) : cplan_fun.sql(12457) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
      Else
         If FC_CREATE_CDY(h_res_id, h_start_DT, h_finish_DT, h_duration, PMO_CNST.SC_RESOURCE) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1277+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1278) : cplan_fun.sql(12464) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
         End If;
      End If;
      
      h_finish_DT := h_finish_DT;
      
      If Coalesce(h_effort, 0) > 0 AND iSF = 1 Then
         h_total := 0;
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT Then
            BEGIN
               SELECT COUNT(*) INTO h_count 
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID=h_prj_assignment_id
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1300+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1301) : cplan_fun.sql(12487) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT COUNT(*) INTO h_count 
               FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1315+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1316) : cplan_fun.sql(12502) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_count, 0) > 0 Then
            --h_tmp_percent_units = ((double) h_effort / (double) h_total) * 10000.00;
            h_work_per_day := Trunc((Coalesce(h_effort, 0) / Coalesce(h_count, 0)) + 0.49);
            If Coalesce(h_per_day, 0) > 0 AND Coalesce(h_work_per_day, 0) <> Coalesce(h_per_day, 0) Then
               h_tmp_percent_units := Trunc((Coalesce(h_work_per_day, 0) / Coalesce(h_per_day, 0)) * 10000.00);
            ElsIf Coalesce(h_work_per_day, 0) = Coalesce(h_per_day, 0) Then
               h_tmp_percent_units := 10000;
            End If;
            
            h_work := 0;
            
            If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
                  h_prj_percent_units, 0) > 0) AND (Coalesce(h_tmp_percent_units, 0) > Coalesce(
                  h_prj_percent_units, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1336+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1337) : cplan_fun.sql(12523) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.CANNOT_ASSIGN_PROJ_AFFECT_DATE;
            Else
               h_percent_units := Coalesce(h_tmp_percent_units, 0);
               h_work_units := Coalesce(h_tmp_percent_units, 0);
            End If;
            
            If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) AND (Coalesce(
                  h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
               h_consider := PMO_CNST.SC_SRCH_OVERALLOCATE;
            End If;
         End If;
      ElsIf (BITAND(Coalesce(h_update, 0), 255)) <> 10 AND Coalesce(
                       h_rollup, 0) <> PMO_CNST.ROLLUP_RESCHEDULE_ETC AND Coalesce(
                       h_bitflag, 0) = 0 Then
         -- h_update  0x000FF)  == 10 --> Leveling
         -- h_rollup             == 4  --> reschedule
         -- h_bitflag            == 0  --> ASAP
         h_percent_units := 0;
      End If;
      
      If Coalesce(h_percent_units, 0) > 0 Then
         If Coalesce(h_per_day, 0) = 0 Then
            h_per_day := 480;
         End If;
         
         If Coalesce(h_percent_units, 0) = 10000 Then
            h_day_min := Coalesce(h_per_day, 0);
         Else
            h_day_min := Trunc(((Coalesce(h_per_day, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.5);
         End If;
         
         If Coalesce(h_day_min, 0) < 5 Then
            h_day_min := 5;
         End If;
      End If;
      
      If Coalesce(Length(h_start_DT), 0) > 10 AND Coalesce(Length(h_finish_DT), 0) > 10 Then
         BEGIN
            SELECT PMO_ORASTD.TIME(h_start_DT)
            , PMO_ORASTD.TIME(h_finish_DT)
            INTO h_start_time
            , h_finish_time
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1387+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1388) : cplan_fun.sql(12574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If Coalesce(Length(h_finish_DT), 0) > 10 Then
            BEGIN
               SELECT PMO_ORASTD.TIME(h_finish_DT)
               INTO h_finish_time
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1403+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1404) : cplan_fun.sql(12590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(Length(h_start_DT), 0) > 10 Then
            BEGIN
               SELECT PMO_ORASTD.TIME(h_start_DT)
               INTO h_start_time
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1418+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1419) : cplan_fun.sql(12605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_total := 0;
      l_calls := 0;
      
      null_return := FC_INI_ASSIGN_STR(1);
      h_assign_string := '';
      
      h_temp_sc_state := FC_SEARCH(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id
                             , h_prj_assignment_id, h_start_DT, h_start_DT, h_finish_DT
                             , h_start_time, h_finish_time, h_consider, h_res_type, h_work_type
                             , h_bitflag, h_update, h_table_type, h_duration, Coalesce(h_total, 0)
                             , Coalesce(h_effort, 0), Coalesce(h_work, 0), iSF, Coalesce(
                             h_contour, 0), Coalesce(h_every, 0), Coalesce(h_type, 0), Coalesce(
                             h_skip, 0), Coalesce(h_percent_units, 0), h_amnt, Coalesce(h_unit, 0)
                             , Coalesce(h_day_min, 0), Coalesce(h_per_day, 0), Coalesce(
                             h_min_work_per_day, 0), Coalesce(h_julian_finish, 0), Coalesce(
                             h_j_disconti_date, 0), Coalesce(assign_to_prj, 0), i_pa_replace
                             , l_calls);
      
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1445+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1446) : cplan_fun.sql(12632) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      Else
         h_start_date := h_start_DT;
         h_finish_date := h_finish_DT;
      End If;
      
      h_temp_sc_state := FC_GET_ASSIGN_STR(h_assign_string, 0);
      
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1457+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1458) : cplan_fun.sql(12644) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      End If;
      
      If (BITAND(Coalesce(h_update, 0), 255)) <> 10 Then
         If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
            BEGIN
               SELECT SUM(AVAILABLE_MIN) INTO h_available
               FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1474+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1475) : cplan_fun.sql(12661) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SUM(SCHED_MIN) INTO h_available
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id
               AND TSK_ASSIGNMENT_ID=h_prj_assignment_id
               AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1491+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1492) : cplan_fun.sql(12678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            SELECT SUM(SCHED_MIN) INTO h_planned_work
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID=h_res_id
            AND BIT_FLAG='0'
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1509+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1510) : cplan_fun.sql(12696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT SUM(SCHED_MIN) INTO h_sched_work
            FROM TASK_DAY_OF_YEAR
            WHERE RESOURCE_ID=h_res_id
            AND BIT_FLAG='2'
            AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1526+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1527) : cplan_fun.sql(12713) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_type, 0) > 0 Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_date) - PMO_ORASTD.DAYS(h_start_date)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1544+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1545) : cplan_fun.sql(12731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_total, 0) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1551+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1552) : cplan_fun.sql(12738) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.F_DATE_EARLIER_THAN_S_DATE;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT Then
         If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
            If (BITAND(Coalesce(h_update, 0), 256)) = 256 Then
               --                  if ( (strcmp(h_start_date,h_ta_start_DT) == 0)
               --                            (strcmp(h_finish_date,h_ta_finish_DT) == 0) )
               If (i_lev_etc_more = 0) AND (i_has_actual = 0) AND (PMO_ORASTD.strcmp(h_start_date
                     , h_ta_start_DT) = 0) AND (PMO_ORASTD.strcmp(h_assign_string
                     , h_lev_assign_string) = 0) Then
                  -- Optimization for calculate
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     CONSIDER                = 0,
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1583+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1584) : cplan_fun.sql(12770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM LEVEL_TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id
                     AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1602+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1603) : cplan_fun.sql(12789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  If i_lev_etc_more = 1 Then
                     -- at this point h_start_date is the last day of the previously scheduled
                     -- assignment.
                     BEGIN
                        SELECT TO_DATE(SubStr(h_start_date, 1, 10)) - 1      INTO h_finish_DT FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1618+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1619) : cplan_fun.sql(12805) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 2);
                  ElsIf i_has_actual = 1 Then
                     h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 1);
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     LEVEL_START             = h_start_date,
                     LEVEL_FINISH            = h_finish_date,
                     LEVEL_EFFORT            = h_effort,
                     LEVEL_ASSIGN_STRING     = h_assign_string
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1647+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1648) : cplan_fun.sql(12834) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                     BEGIN
                        INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1665+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1666) : cplan_fun.sql(12852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            Else
               If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
                  h_proj_assign_flag := 0;
               Else
                  h_proj_assign_flag := 1;
               End If;
               
               If i_has_actual = 1 Then
                  h_temp_sc_state := FC_FIX_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_act_start_DT
                                        , h_finish_DT, h_assign_string, 1);
               End If;
               
               BEGIN
                  INSERT INTO PMO_TMP_TDY_LEVEL (
                  PROJECT_ID,
                  RESOURCE_ID,
                  PRIORITY,
                  ACTUAL_FLAG,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  SCHED_MIN,
                  BIT_FLAG,
                  BIT_SCHED,
                  PROJ_ASSIGN_FLAG)
                  SELECT
                  PROJECT_ID,
                  RESOURCE_ID,
                  h_priority,
                  0,
                  TSK_ASSIGNMENT_ID,
                  DAY_YEAR,
                  SCHED_MIN,
                  BIT_FLAG,
                  BIT_SCHED,
                  h_proj_assign_flag
                  FROM PMO_TMP_TDOY;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1717+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1718) : cplan_fun.sql(12904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  LEVEL_START                 = h_start_date,
                  LEVEL_FINISH                = h_finish_date,
                  LEVEL_EFFORT                = h_effort,
                  LEVEL_ASSIGN_STRING         = h_assign_string
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1739+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1740) : cplan_fun.sql(12926) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     INSERT INTO LEVEL_TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1757+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1758) : cplan_fun.sql(12944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            
            --timeLog (file,"EXIT LEVELING_1:");
            --AEL: if this level exit
            If PMO_DB2M.SC_STATE = 100 Then
               PMO_DB2M.SC_STATE := 0;
            End If;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1771+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1772) : cplan_fun.sql(12958) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_DB2M.SC_STATE;
         Else
            BEGIN
               INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1787+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1788) : cplan_fun.sql(12974) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         -- search/assign from project
         --InitializeHost(SC_PARAM(h_tsk_assignment_id), SC_PARAM(assign_id));
         BEGIN
            INSERT INTO TASK_DAY_OF_YEAR SELECT * FROM PMO_TMP_TDOY;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1807+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1808) : cplan_fun.sql(12994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (BITAND(Coalesce(h_update, 0), 255)) = 10 Then
         If i_lev_etc_more = 1 Then
            -- for etc_more, h_start_date is the last day of previous assignment, so we here
            -- we grab the original assignment start date we copied into h_act_start_DT
            h_start_date := h_act_start_DT;
         End If;
         --timeLog (file,"EXIT LEVELING:");
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1821+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1822) : cplan_fun.sql(13008) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
      
      If Coalesce(h_work_units, 0) > 0 Then
         /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
         			{
         				InitializeHost(SC_PARAM(h_unit), 101);
         				InitializeHost(SC_PARAM(h_eunit), calculate_flag);
         			}
         			else
         			{*/
         h_unit := 100;
         --}
         
         h_work := Coalesce(h_work_units, 0);
      End If;
      /*      else if ( (h_work == 0)  (i_max_min > 0)   (i_max_min != h_per_day))
      		{
      			InitializeHost(SC_PARAM(h_unit), 100);
      			h_work = ((double) i_max_min / (double) h_per_day) * 10000.00;
      		}*/
      --t_start = clock();
      
      -- CALL  U_TSKRES2
      If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) 
             AND (SubStr(h_res_type, 1, 1) = '0') Then
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_RESCHEDULE_ETC Then
            h_update := 4;
            -- re_etc schedule
         ElsIf Coalesce(h_update, 0) = 0 Then
            h_update := (PMO_FUNC.BIT_OR(2, PMO_CNST.SC_BIT_30));
            h_work_type := 2;
         Else
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_30));
            h_work_type := 0;
         End If;
         
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_DONT_ROLLUP_IMPORT_MSP Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_29));
         End If;
      Else
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_DONT_ROLLUP Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_29_30));
         End If;
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_ASSIGN_FROM_PROJECT Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_30));
         End If;
         If Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_DONT_ROLLUP_IMPORT_MSP Then
            h_update := (PMO_FUNC.BIT_OR(Coalesce(h_update, 0), PMO_CNST.SC_BIT_28));
         ElsIf Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_RESCHEDULE_ETC Then
            h_update := 4;
         End If;
         -- re_etc schedule
      End If;
      
      h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
      PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_and_roles
                               , h_and_skills, h_start_date, h_finish_date, h_effort, h_work
                               , h_eunit, h_unit, h_res_type, h_work_type, h_update, h_contour
                               , h_bitflag, h_occurrence, h_consider, h_assign_string, h_rec_user
                               , h_per_day, p_reg_sale, p_spec_sale);
      
      -- TOREVIEW: whether we can just pass SC_STATE back to caller if != 0
      If PMO_DB2M.SC_STATE < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1888+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1889) : cplan_fun.sql(13075) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      ElsIf (PMO_DB2M.SC_STATE > 0) OR (Coalesce(h_rollup, 0) = PMO_CNST.ROLLUP_RESCHEDULE_ETC) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 1893+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(1894) : cplan_fun.sql(13080) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_DB2M.SC_STATE;
      End If;
   End If;
   
   -- TOREVIEW: October 12, 2004
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PROFILE Then
      h_start_date := '1899-12-30 8:00:00';
      h_finish_date := '1899-12-30 8:00:00';
   End If;
   
   --if (iSF == 0)           // no finish date
   -- h_effort = 0;
   
   If (Coalesce(Length(h_and_roles), 0) = 0) AND (Coalesce(Length(h_and_skills), 0) = 0) Then
      i_noskill := 1;
   Else
      i_noskill := 0;
   End If;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      --          Nullify(SC_PARAM(h_start_date));
      --          Nullify(SC_PARAM(h_finish_date));
      h_start_date := '1899-12-30 8:00:00';
      h_finish_date := '1899-12-30 8:00:00';
      h_planned_work := 0;
      --h_planned_work_Ind = 0;
      h_sched_work := 0;
      --h_sched_work_Ind = 0;
      h_available := 0;
      --h_available_Ind = 0;
      h_assign_string := '';
      --h_assign_string_Ind = 0;
   Else
      h_planned_work := Coalesce(h_planned_work, 0);
      --h_planned_work_Ind = 0;
      h_sched_work := Coalesce(h_sched_work, 0);
      --h_sched_work_Ind = 0;
      h_available := Coalesce(h_available, 0);
      --h_available_Ind = 0;
      h_assign_string := h_assign_string;
      --h_assign_string_Ind = 0;
   End If;
   
   If i_noskill = 1 Then
      l_data_flag := 0;
      If Coalesce(h_reg_cost, 0) + Coalesce(h_reg_sale, 0) = 0.0 Then
         h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, h_and_roles, h_and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
      End If;
      
      h_default_flag := TO_CHAR(l_data_flag);
      h_uniqueID := SYS_GUID();
      
      BEGIN
         INSERT INTO PMO_TMP_SEARCH
         (
         RS_ID,
         RES_ID,
         RES_SALE_RATE,
         START_DATE,
         FINISH_DATE,
         ASSIGNABLE,
         PROPOSED,
         PLANNED,
         SCHED,
         REGULAR_COST,
         REGULAR_SALE,
         SPECIAL_COST,
         SPECIAL_SALE,
         CONTOUR,
         DEFAULT_FLAG
         )
         VALUES
         (
         h_uniqueID,
         h_res_id,
         h_res_cost,
         h_start_date,
         h_finish_date,
         h_available,
         h_effort,
         h_planned_work,
         h_sched_work,
         h_reg_cost,
         h_reg_sale,
         h_spec_cost,
         h_spec_sale,
         h_assign_string,
         h_default_flag
         );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 1996+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(1997) : cplan_fun.sql(13183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      token := SubStr(token, 1, 32);
      --strcpy(token[32], "\0");
      
      --Parse Roles list
      If Coalesce(Length(h_and_roles), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
      End If;
      
      If numItems > 0 Then
         stmt := '';
         stmt := 
               'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,ROLE_ID,ROLE_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) SELECT SYS_GUID(), ''' 
               || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
               || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
               h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' || TO_CHAR(Coalesce(
               h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(h_sched_work, 0)) || ',''' 
               || h_assign_string || ''',';
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            stwhere := 
                  't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t2.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t2.LAST_USED_DATETIME),'''') || ''"''),'
                   || 'COALESCE(r.REGULAR_COST,0), COALESCE(r.REGULAR_SALE,0), COALESCE(r.SPECIAL_COST,0), COALESCE(r.SPECIAL_SALE,0),''0'' '
                   || 'FROM TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID '
                   || 'WHERE t2.TYPE_ID=73 AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                   || ' ( t3.RANK >= SUBSTR(t2.RANK, 1, 8) AND t3.RANK <= t2.RANK  '
                   || '   AND substr(t2.RANK,1, case when length(t3.RANK) > length(t2.RANK) then 1 else length (t3.RANK) end) = t3.RANK )'
                   || ' AND t3.ELEMENT_ID IN (''';
         Else
            stwhere := 
                  't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"''),'
                   || ' CASE WHEN (R1.REGULAR_COST = 0 OR R1.REGULAR_COST IS NULL) AND (R.REGULAR_COST = 0 OR R.REGULAR_COST IS NULL) THEN 0 '
                   || ' WHEN (R1.REGULAR_COST = 0 OR  R1.REGULAR_COST IS NULL) THEN R.REGULAR_COST '
                   || ' ELSE R1.REGULAR_COST END AS REGULAR_COST, '
                   || ' CASE WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) AND (R.REGULAR_SALE = 0 OR R.REGULAR_SALE IS NULL)  THEN 0 '
                   || ' WHEN (R1.REGULAR_SALE = 0 OR R1.REGULAR_SALE IS NULL) THEN R.REGULAR_SALE '
                   || ' ELSE R1.REGULAR_SALE END AS REGULAR_SALE, '
                   || ' CASE WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) AND (R.SPECIAL_COST = 0 OR R.SPECIAL_COST IS NULL)  THEN 0 '
                   || ' WHEN (R1.SPECIAL_COST = 0 OR R1.SPECIAL_COST IS NULL) THEN R.SPECIAL_COST '
                   || ' ELSE R1.SPECIAL_COST END AS SPECIAL_COST, '
                   || ' CASE WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) AND (R.SPECIAL_SALE = 0 OR R.SPECIAL_SALE IS NULL)  THEN 0 '
                   || ' WHEN (R1.SPECIAL_SALE = 0 OR R1.SPECIAL_SALE IS NULL) THEN R.SPECIAL_SALE ELSE R1.SPECIAL_SALE END AS SPECIAL_SALE, '
                   || ' CASE WHEN r1.RATE_ID IS NULL OR R1.REGULAR_COST = 0 THEN ''0'' ELSE t1.DEFAULT_FLAG END '
                   || 'FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,'
                   || 'TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
            stwhere := stwhere || h_res_id;
            stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                   || ' ( t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
                   || '   AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                   || ' AND t3.ELEMENT_ID IN (''';
         End If;
         i := 0;
         
         WHILE i < numItems LOOP
            If i > 0 Then
               stwhere := stwhere || ''',''';
            End If;
            
            -- Get next role_id
            --                  pmo_substr(token, h_and_roles, i*32, 32); //strncpy( token, h_and_roles[i*32], 32);
            token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_and_roles);
            
            stwhere := stwhere || token;
            i := i + 1;
         END LOOP;
         stwhere := stwhere || ''')';
         stmt := stmt || stwhere;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2082+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2083) : cplan_fun.sql(13269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
      End If;
      
      --Parse Skills list
      If Coalesce(Length(h_and_skills), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
      End If;
      
      If numItems > 0 Then
         stmt := '';
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            stmt := 
                  'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SKILL_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                   || 'SELECT SYS_GUID(), ''' || h_res_id || ''',' 
                        || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
                        || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
                        h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' 
                        || TO_CHAR(Coalesce(h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(
                        h_sched_work, 0)) || ',''' || h_assign_string || ''',';
            stwhere := 
                  '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t3.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t3.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t3.LAST_USED_DATETIME),'''') || ''"''), '
                   || ' COALESCE(r.REGULAR_COST,0), COALESCE(r.REGULAR_SALE,0), COALESCE(r.SPECIAL_COST,0), COALESCE(r.SPECIAL_SALE,0), ''0'' '
                   || 'FROM TMT_COMPETENCIES t4,TMT_COMPETENCIES t3 LEFT OUTER JOIN RATES r ON t3.RATE_ID=r.RATE_ID WHERE t4.ELEMENT_ID=t3.PARENT_ID AND t3.ELEMENT_ID IN (''';
         Else
            stmt := 
                  'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SKILL_ID,SKILL_NAME,REGULAR_COST,REGULAR_SALE,SPECIAL_COST,SPECIAL_SALE,DEFAULT_FLAG) '
                   || 'SELECT SYS_GUID(), ''' || h_res_id || ''', ' 
                        || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
                        || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
                        h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' 
                        || TO_CHAR(Coalesce(h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(
                        h_sched_work, 0)) || ',''' || h_assign_string || ''',';
            
            stwhere := 
                  't2.ELEMENT_ID,(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"''), '
                   || 'case when (r1.REGULAR_COST is null or r1.REGULAR_COST=0) then COALESCE(r.REGULAR_COST,0) else r1.REGULAR_COST end AS REGULAR_COST, '
                   || 'case when (r1.REGULAR_SALE is null or r1.REGULAR_SALE=0) then COALESCE(r.REGULAR_SALE,0) else r1.REGULAR_SALE end AS REGULAR_SALE, '
                   || 'case when (r1.SPECIAL_COST is null or r1.SPECIAL_COST=0) then COALESCE(r.SPECIAL_COST,0) else r1.SPECIAL_COST end AS SPECIAL_COST, '
                   || 'case when (r1.SPECIAL_SALE is null or r1.SPECIAL_SALE=0) then COALESCE(r.SPECIAL_SALE,0) else r1.SPECIAL_SALE end AS SPECIAL_SALE, ''0'' '
                   || 'FROM TMT_COMPETENCIES t1 LEFT OUTER JOIN RATES r1 ON t1.RATE_ID=r1.RATE_ID,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
            
            stwhere := stwhere || h_res_id;
            
            stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID AND '
                   || ' (t3.RANK >= SUBSTR(t1.RANK, 1, 8) AND t3.RANK <= t1.RANK  '
                   || '  AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
                   || ' AND t3.ELEMENT_ID IN (''';
         End If;
         
         i := 0;
         WHILE i < numItems LOOP
            If i > 0 Then
               stwhere := stwhere || ''',''';
            End If;
            
            -- Get next role_id
            --                  pmo_substr(token, h_and_skills, i*32, 32); //strncpy( token, h_and_skills[i*32], 32 );
            token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_and_skills);
            
            stwhere := stwhere || token;
            i := i + 1;
         END LOOP;
         stwhere := stwhere || ''')';
         stmt := stmt || stwhere;
         
         BEGIN
            EXECUTE IMMEDIATE stmt;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2166) : cplan_fun.sql(13352) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --Parse Show Roles list
   If Coalesce(Length(h_shw_roles), 0) < 32 Then
      numItems := 0;
   Else
      numItems := Trunc((Coalesce(Length(h_shw_roles), 0) + 1) / 32);
   End If;
   
   If numItems > 0 Then
      token := SubStr(token, 1, 32);
      stmt := '';
      stmt := 
            'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SHOW_ROLE) SELECT SYS_GUID(),''' 
            || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
            || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
            h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' || TO_CHAR(Coalesce(
            h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(h_sched_work, 0)) || ',''' 
            || h_assign_string || ''',';
      stwhere := 
            '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"'') FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID '
             || ' AND (t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK  '
             || '       AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK)end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         -- Get next role_id
         --          pmo_substr(token, h_shw_roles, i*32, 32);     //strncpy( token, h_shw_roles[i*32], 32);
         token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_shw_roles);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2226+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2227) : cplan_fun.sql(13413) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
   End If;
   
   --Parse Show Skills list
   If Coalesce(Length(h_shw_skills), 0) < 32 Then
      numItems := 0;
   Else
      numItems := Trunc((Coalesce(Length(h_shw_skills), 0) + 1) / 32);
   End If;
   
   If numItems > 0 Then
      token := SubStr(token, 1, 32);
      stmt := '';
      stmt := 
            'INSERT INTO PMO_TMP_SEARCH (RS_ID,RES_ID,RES_SALE_RATE,START_DATE,FINISH_DATE,ASSIGNABLE,PROPOSED,PLANNED,SCHED,CONTOUR,SHOW_SKILL) SELECT SYS_GUID(),''' 
            || h_res_id || ''',' || TO_CHAR(h_res_cost, 'FM999999999999999999.999990') || ',''' 
            || h_start_date || ''',''' || h_finish_date || ''',' || TO_CHAR(Coalesce(
            h_available, 0)) || ',' || TO_CHAR(Coalesce(h_effort, 0)) || ',' || TO_CHAR(Coalesce(
            h_planned_work, 0)) || ',' || TO_CHAR(Coalesce(h_sched_work, 0)) || ',''' 
            || h_assign_string || ''',';
      stwhere := 
            '(''"'' || RTRIM(t4.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(t2.ELEMENT_NAME) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.EXPERIENCE_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PROFICIENCY_LEVEL_ID)) || ''"'' || '','' || ''"'' || RTRIM(TO_CHAR(t1.PREFERENCE_LEVEL_ID)) || ''"'' || '','' || ''"'' || COALESCE(TO_CHAR(t1.LAST_USED_DATETIME),'''') || ''"'') FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t3,TMT_COMPETENCIES t4,TMT_COMPETENCIES t2 LEFT OUTER JOIN RATES r ON t2.RATE_ID=r.RATE_ID WHERE t1.RESOURCE_ID=''';
      stwhere := stwhere || h_res_id;
      stwhere := stwhere || ''' AND t2.ELEMENT_ID=t1.PARENT_ID AND t4.ELEMENT_ID=t2.PARENT_ID '
             || ' AND (t3.RANK >= SUBSTR(t1.RANK,1,8) AND t3.RANK <= t1.RANK '
             || '      AND substr(t1.RANK,1, case when length(t3.RANK) > length(t1.RANK) then 1 else length (t3.RANK) end) = t3.RANK ) '
             || ' AND t3.ELEMENT_ID IN (''';
      
      i := 0;
      WHILE i < numItems LOOP
         If i > 0 Then
            stwhere := stwhere || ''',''';
         End If;
         
         -- Get next role_id
         --          pmo_substr(token, h_shw_skills, i*32, 32);    //    strnpcpy(token, sizeof(token), h_shw_skills[i*32], 32);
         token := PMO_STPU.strnpcpy(32, 32, i * 32+1, h_shw_skills);
         
         stwhere := stwhere || token;
         i := i + 1;
      END LOOP;
      stwhere := stwhere || ''')';
      stmt := stmt || stwhere;
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_2', 2286+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_2(2287) : cplan_fun.sql(13473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_2', 2296+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_2(2297) : cplan_fun.sql(13483) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEARCH_2;
ALTER FUNCTION FC_SEARCH_2 COMPILE DEBUG;

--***************************************************************************************

-- h_bitflag = 2 if update actual in Cplan or WP

-- h_bitflag = 1 if update actual in Cplan or WP

-- h_bitflag = 0 if update actual in Timesheets: fix SCHED_BITS, ACTUAL_MIN and ACTUAL_FLAG

--***************************************************************************************

Prompt == FUNCTION FC_ACTUAL_DAY ==
CREATE OR REPLACE FUNCTION FC_ACTUAL_DAY
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
   , sh_h_minutes PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_bitflag PLS_INTEGER := Coalesce(sh_h_bitflag, 0);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bit_available RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_task_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_available PLS_INTEGER := 0;
   h_ta_sched_min PLS_INTEGER := 0;
   h_pa_sched_min PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_actual_flag PLS_INTEGER := 0;
   h_actual_min PLS_INTEGER := 0;
   h_units PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CAV1 IS
         SELECT BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND (TSK_ASSIGNMENT_ID = h_prj_assignment_id OR TSK_ASSIGNMENT_ID = h_tsk_assignment_id) 
         AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));

      CURSOR CNODATA1 IS
         SELECT AVAILABLE_MIN, BIT_SCHED 
         FROM CALENDAR_DAY_OF_YEAR
         WHERE  ROWNUM <= 1 AND  ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_dayofyear, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ACTUAL_DAY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ACTUAL_DAY'), '-- ' || SYSTIMESTAMP, ':r := FC_ACTUAL_DAY(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 54+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ACTUAL_DAY(55) : cplan_fun.sql(13556) : ' || SYSTIMESTAMP);
   END IF; 

   --SC_CHAR(tempStr, SC_H_DESCRIPTION);
   
   h_prj_assignment_id := NULL;
   
   BEGIN
      SELECT PROJECT_ID, TASK_ID, RESOURCE_ID, RESOURCE_TYPE, PROJECT_ASSIGNMENT_ID, TABLE_TYPE, BIT_FLAG, UNITS_MSP
      INTO h_project_id, h_task_id, h_res_id, h_res_type, h_prj_assignment_id, h_table_type, h_bit_flag, h_units
      FROM TASK_ASSIGNMENTS 
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 72+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(73) : cplan_fun.sql(13574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT ACTUAL_MIN, SCHED_MIN, ACTUAL_FLAG, BIT_SCHED
      INTO h_actual_min, h_ta_sched_min, h_actual_flag, h_bitID
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
      AND DAY_YEAR = h_dayofyear;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 90+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(91) : cplan_fun.sql(13592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND Coalesce(h_minutes, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 97+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(98) : cplan_fun.sql(13599) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If (Coalesce(h_actual_min, 0) = Coalesce(h_minutes, 0)) AND (Coalesce(h_actual_flag, 0) = 1) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 105+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(106) : cplan_fun.sql(13607) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
      
      -- It is possible initially to assign more then h_bitID.h_value, at the project
      -- But is not possible to assign to task more then assigned to the project
      If Coalesce(h_units, 0) > 10000 Then
         BEGIN
            SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_per_day
            FROM TMT_WBS 
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 123+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(124) : cplan_fun.sql(13625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_var := Trunc(Coalesce(h_per_day, 0) * (Coalesce(h_units, 0) / 10000.0));
         
         If Coalesce(h_temp_var, 0) > 0 AND Coalesce(h_temp_var, 0) < Coalesce(h_ta_sched_min, 0) Then
            h_ta_sched_min := Coalesce(h_temp_var, 0);
         End If;
      ElsIf Coalesce(h_actual_flag, 0) = 1 Then
         h_ta_sched_min := PMO_STPU.getNumberOfBits(h_bitID) * 5;
      End If;
      
      If (h_prj_assignment_id IS NOT NULL) Then
         BEGIN
            SELECT SCHED_MIN, BIT_SCHED INTO h_pa_sched_min, h_bit_available 
            FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 150+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(151) : cplan_fun.sql(13652) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_temp_var := Coalesce(h_ta_sched_min, 0);
            If h_bitflag = 2 Then
               h_ta_sched_min := Coalesce(h_ta_sched_min, 0) + Coalesce(h_pa_sched_min, 0);
            End If;
            
            h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bit_available);
            h_pa_sched_min := Coalesce(h_pa_sched_min, 0) + Coalesce(h_temp_var, 0);
         ElsIf ((PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) AND Coalesce(h_ta_sched_min, 0) <> Coalesce(
               h_minutes, 0)) OR (Coalesce(h_pa_sched_min, 0) > 0 AND Coalesce(
               h_pa_sched_min, 0) < Coalesce(h_minutes, 0)) Then
            h_available := 0;
            h_bitID := PMO_CNST.SC_ZERO_BIT;
            
            OPEN CAV1;
            
            LOOP
               FETCH CAV1 INTO h_bit_available;
               IF CAV1%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               h_available := Coalesce(h_available, 0) + PMO_STPU.getNumberOfBits(h_bit_available) * 5;
               h_bitID := PMO_STPU.BIT_OR_DAY(h_bitID, h_bit_available);
            END LOOP;
            
            BEGIN
               CLOSE CAV1 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 195+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(196) : cplan_fun.sql(13697) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         h_bitID2 := h_bitID;
      ElsIf Coalesce(h_minutes, 0) > 0 Then
         If Coalesce(h_ta_sched_min, 0) < Coalesce(h_minutes, 0) Then
            --removed the subtraction of ASSIGNED_MIN from AVAILABLE_MIN and the whole case statement
            If SubStr(h_res_type, 1, 1) = '0' Then
               BEGIN
                  SELECT BIT_SCHED, AVAILABLE_MIN
                  INTO h_bitID, h_available
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_res_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 217+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(218) : cplan_fun.sql(13719) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT BIT_SCHED, AVAILABLE_MIN
                  INTO h_bitID, h_available
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 233+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(234) : cplan_fun.sql(13735) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_ta_sched_min := Coalesce(h_available, 0);
         Else
            If SubStr(h_res_type, 1, 1) = '0' Then
               BEGIN
                  SELECT AVAILABLE_MIN INTO h_available
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 252+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(253) : cplan_fun.sql(13754) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT AVAILABLE_MIN INTO h_available
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id AND DAY_YEAR = TO_DATE(SubStr(h_dayofyear, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 267+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(268) : cplan_fun.sql(13769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_ta_sched_min := Coalesce(h_available, 0);
         End If;
      End If;
   ElsIf (h_prj_assignment_id IS NOT NULL) Then
      BEGIN
         SELECT SCHED_MIN, BIT_SCHED INTO h_pa_sched_min, h_bitID
         FROM TASK_DAY_OF_YEAR 
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 287+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(288) : cplan_fun.sql(13789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         --removed the case statement where assigned_min bit_assigned were used
         BEGIN
            SELECT AVAILABLE_MIN, BIT_SCHED
            INTO h_available, h_bitID FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 304+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(305) : cplan_fun.sql(13806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available, h_bitID
         FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID=h_res_id AND DAY_YEAR=TO_DATE(SubStr(h_dayofyear, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 320+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(321) : cplan_fun.sql(13822) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      If (h_prj_assignment_id IS NULL) Then
         OPEN CNODATA1;
         
         FETCH CNODATA1 INTO h_available, h_bitID;
         IF CNODATA1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CNODATA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 345+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(346) : cplan_fun.sql(13847) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_available, 0) = 0 Then
         BEGIN
            SELECT  (PMO_CNST.SC_BIT_SCHED8H)  INTO  h_bitID FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 360+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(361) : cplan_fun.sql(13862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If h_bitflag = 2 Then
      h_actual_min := 0;
      h_actual_flag := 0;
   Else
      h_actual_min := Coalesce(h_minutes, 0);
      h_actual_flag := 1;
   End If;
   
   --strcpy(tempStr,"");
   --printTimes(h_bitID.h_value,tempStr);
   --strcpy(tempStr,"");
   --printTimes(h_bitID2.h_value,tempStr);
   If Coalesce(h_minutes, 0) = 0 Then
      If (Coalesce(h_ta_sched_min, 0) > 0) OR ((Coalesce(h_pa_sched_min, 0) > 0)) Then
         If h_bitflag = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = 0, ACTUAL_FLAG = 0
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 395+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(396) : cplan_fun.sql(13897) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            If (h_prj_assignment_id IS NOT NULL) Then
               If Coalesce(h_pa_sched_min, 0) = 0 Then
                  If Coalesce(h_available, 0) = 0 Then
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 415+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(416) : cplan_fun.sql(13917) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id, SCHED_MIN = h_ta_sched_min, ACTUAL_MIN = 0, ACTUAL_FLAG = 0, TASK_ID = h_project_id
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 433+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(434) : cplan_fun.sql(13935) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 451+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(452) : cplan_fun.sql(13953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_pa_sched_min, ACTUAL_MIN = 0, BIT_SCHED =h_bitID2
                        WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 470+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(471) : cplan_fun.sql(13972) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            Else
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 489+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(490) : cplan_fun.sql(13991) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   ElsIf Coalesce(h_ta_sched_min, 0) = Coalesce(h_minutes, 0) Then
      If h_bitflag = 1 Then
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET 
            ACTUAL_MIN = h_minutes, 
            ACTUAL_FLAG = 1
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 515+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(516) : cplan_fun.sql(14017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET 
            SCHED_MIN = h_minutes, 
            ACTUAL_MIN = h_actual_min, 
            ACTUAL_FLAG = h_actual_flag, 
            TASK_ID = h_task_id
            WHERE PROJECT_ID = h_project_id 
            AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 539+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(540) : cplan_fun.sql(14041) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_minutes, 0), h_bitID);
      If (h_prj_assignment_id IS NOT NULL) Then
         h_bitID2 := PMO_STPU.BIT_ANDNOT_DAY(h_bitID2, h_bitID);
         If Coalesce(h_pa_sched_min, 0) = 0 AND Coalesce(h_available, 0) > 0 Then
            h_bit_minutes := Coalesce(h_available, 0) - Coalesce(h_minutes, 0);
         ElsIf Coalesce(h_pa_sched_min, 0) > Coalesce(h_minutes, 0) Then
            h_bit_minutes := Coalesce(h_pa_sched_min, 0) - Coalesce(h_minutes, 0);
         End If;
      ElsIf Coalesce(h_ta_sched_min, 0) = 0 Then
         If Coalesce(h_available, 0) > 0 Then
            h_bit_minutes := Coalesce(h_available, 0) - Coalesce(h_minutes, 0);
         End If;
      ElsIf Coalesce(h_ta_sched_min, 0) > Coalesce(h_minutes, 0) Then
         h_bit_minutes := Coalesce(h_ta_sched_min, 0) - Coalesce(h_minutes, 0);
      End If;
      
      If Coalesce(h_ta_sched_min, 0) = 0 Then
         If Coalesce(h_pa_sched_min, 0) > 0 Then
            If Coalesce(h_pa_sched_min, 0) = Coalesce(h_minutes, 0) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET 
                  TSK_ASSIGNMENT_ID = h_tsk_assignment_id, 
                  ACTUAL_MIN = h_actual_min, 
                  ACTUAL_FLAG = h_actual_flag, 
                  TASK_ID = h_task_id
                  WHERE PROJECT_ID = h_project_id 
                  AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
                  AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 583+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(584) : cplan_fun.sql(14085) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (
                  PROJECT_ID, TASK_ID, 
                  RESOURCE_ID, 
                  TSK_ASSIGNMENT_ID, 
                  PARENT_ID, 
                  DAY_YEAR, 
                  SCHED_MIN, 
                  ACTUAL_MIN, 
                  BIT_SCHED, 
                  BIT_FLAG, 
                  ACTUAL_FLAG, 
                  TABLE_TYPE)
                  VALUES(
                  h_project_id, 
                  h_task_id, 
                  h_res_id, 
                  h_tsk_assignment_id, 
                  h_prj_assignment_id, 
                  h_dayofyear, 
                  h_minutes, 
                  h_actual_min, 
                  h_bitID, 
                  h_bit_flag, 
                  h_actual_flag,
                  h_table_type);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 624+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(625) : cplan_fun.sql(14126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_bit_minutes, 0) <= 0 Then
                  PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_minutes, 0), h_bitID);
               End If;
               
               If Coalesce(h_pa_sched_min, 0) > Coalesce(h_minutes, 0) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET 
                     SCHED_MIN = CASE WHEN SCHED_MIN >= h_minutes THEN SCHED_MIN - h_minutes ELSE 0 END, 
                     BIT_SCHED = PMO_STPU.BIT_ANDNOT_DAY(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id 
                     AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
                     AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 651+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(652) : cplan_fun.sql(14153) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR 
                     WHERE PROJECT_ID = h_project_id 
                     AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
                     AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 671+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(672) : cplan_fun.sql(14173) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         Else
            BEGIN
               INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, ACTUAL_MIN, BIT_SCHED, BIT_FLAG, ACTUAL_FLAG, TABLE_TYPE)
               VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_tsk_assignment_id, h_dayofyear, h_minutes, h_actual_min, h_bitID, h_bit_flag, h_actual_flag, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 691+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(692) : cplan_fun.sql(14193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If h_bitflag = 1 Then
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET 
               ACTUAL_MIN = h_minutes, 
               ACTUAL_FLAG = 1
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
               AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 715+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(716) : cplan_fun.sql(14217) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET 
               SCHED_MIN	= h_minutes, 
               BIT_SCHED	= h_bitID, 
               ACTUAL_MIN	= h_actual_min, 
               ACTUAL_FLAG = h_actual_flag, 
               TASK_ID		= h_task_id
               WHERE PROJECT_ID = h_project_id 
               AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
               AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 740+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(741) : cplan_fun.sql(14242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (h_prj_assignment_id IS NOT NULL) Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET 
                  SCHED_MIN	= h_minutes, 
                  ACTUAL_MIN	= h_actual_min, 
                  ACTUAL_FLAG = h_actual_flag, 
                  TASK_ID		= h_task_id
                  WHERE PROJECT_ID = h_project_id 
                  AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
                  AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 764+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(765) : cplan_fun.sql(14266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- return time to PA
               If Coalesce(h_pa_sched_min, 0) > 0 Then
                  If Coalesce(h_bit_minutes, 0) > 0 Then
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = h_bit_minutes, BIT_SCHED = h_bitID2
                        WHERE PROJECT_ID = h_project_id 
                        AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
                        AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 787+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(788) : cplan_fun.sql(14289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR 
                        WHERE PROJECT_ID = h_project_id 
                        AND TSK_ASSIGNMENT_ID = h_prj_assignment_id 
                        AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 807+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(808) : cplan_fun.sql(14309) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf Coalesce(h_bit_minutes, 0) > 0 Then
                  BEGIN
                     INSERT INTO TASK_DAY_OF_YEAR (
                     PROJECT_ID, TASK_ID, 
                     RESOURCE_ID, 
                     TSK_ASSIGNMENT_ID, 
                     PARENT_ID, 
                     DAY_YEAR, 
                     SCHED_MIN, 
                     ACTUAL_MIN, 
                     BIT_SCHED, 
                     BIT_FLAG, 
                     ACTUAL_FLAG, 
                     TABLE_TYPE)
                     VALUES(
                     h_project_id, 
                     h_project_id, 
                     h_res_id, 
                     h_prj_assignment_id, 
                     h_prj_assignment_id, 
                     h_dayofyear, 
                     h_bit_minutes, 
                     0, 
                     h_bitID2, 
                     h_bit_flag, 
                     0, 
                     h_table_type);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ACTUAL_DAY', 849+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ACTUAL_DAY(850) : cplan_fun.sql(14351) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ACTUAL_DAY', 860+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ACTUAL_DAY(861) : cplan_fun.sql(14362) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ACTUAL_DAY;
ALTER FUNCTION FC_ACTUAL_DAY COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_FIX_FF ==
CREATE OR REPLACE FUNCTION FC_FIX_FF
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_finish_D IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_finish_time VARCHAR2(8);
   h_dayofyear DATE;
   h_SD DATE;
   h_days PLS_INTEGER := 0;
   h_minute PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_state VARCHAR2(1);
   h_actual_flag PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR CFF IS
         SELECT * FROM (SELECT PARENT_ID, TASK_ID, DAY_YEAR, RESOURCE_ID, ACTUAL_FLAG, SCHED_MIN, ACTUAL_MIN, TABLE_TYPE, BIT_SCHED
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id --AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR DESC ) WHERE ROWNUM<=1;

      CURSOR C_FF IS
         SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND DAY_YEAR > TO_DATE(SubStr(h_finish_D, 1, 10)) AND ACTUAL_FLAG = 0;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_FF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_FF'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_FF(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_finish_D || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 45+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_FF(46) : cplan_fun.sql(14418) : ' || SYSTIMESTAMP);
   END IF; 

   --short h_project_id_Ind = 0;
   --short h_tsk_assignment_id_Ind = 0;
   --short h_finish_D_Ind = 0;
   
   --get the bit_flag to determine the state of the project in order to update the correct info
   BEGIN
      SELECT RESOURCE_ID, BIT_FLAG INTO h_res_id, h_state
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 63+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(64) : cplan_fun.sql(14436) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_state: %s\n", h_state);
   
   OPEN CFF;
   
   FETCH CFF INTO h_id, h_task_id, h_dayofyear, h_res_id, h_actual_flag, h_temp_var, h_minute, h_table_type, h_bitID;
   IF CFF%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   --trace_printf("SC_STATE: %d, h_project_id: %d %s, h_tsk_assignment_id: %d %s\n", SC_STATE,h_project_id_Ind,h_project_id,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(h_finish_D) - PMO_ORASTD.DAYS(h_dayofyear)
         INTO h_days
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 90+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(91) : cplan_fun.sql(14463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_count := PMO_ORASTD.strcmp(h_id, h_tsk_assignment_id);
      -- project assignment ?
      --trace_printf("h_days: %d, h_count: %d, h_actual_flag: %d, h_dayofyear: %s, h_finish_D : %s\n", h_days,h_count,h_actual_flag,h_dayofyear,h_finish_D);
      --trace_printf("h_temp_var: %d, h_minute: %d, h_dayofyear: %s, h_finish_D : %s\n", h_temp_var,h_minute,h_dayofyear,h_finish_D);
      
      If Coalesce(h_days, 0) > 0 Then
         If Coalesce(h_count, 0) = 0 Then
            If Coalesce(h_actual_flag, 0) = 0 Then
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 117+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(118) : cplan_fun.sql(14490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf (Coalesce(h_temp_var, 0) > 0) AND (Coalesce(h_temp_var, 0) > Coalesce(h_minute, 0)) Then
               h_prj_assign_id := NULL;
               h_temp_sc_state := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                        , h_tsk_assignment_id, h_dayofyear, h_minute, 0);
               
               h_temp_var := Coalesce(h_temp_var, 0) - Coalesce(h_minute, 0);
               PMO_STPU.skipFirstBitsClearToFirst(Coalesce(h_minute, 0), h_bitID);
               BEGIN
                  INSERT INTO TASK_DAY_OF_YEAR (PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG, TABLE_TYPE)
                  VALUES(h_project_id, h_task_id, h_res_id, h_tsk_assignment_id, h_id, h_finish_D, h_temp_var, h_bitID, '2', h_table_type);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 141+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(142) : cplan_fun.sql(14514) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_FF(148) : cplan_fun.sql(14520) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN -1;
            End If;
         Else
            If Coalesce(h_actual_flag, 0) = 1 Then
               h_prj_assign_id := NULL;
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_dayofyear, h_minute, 0);
            End If;
            
            --put back the effort to project assignment
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_temp_var, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 172+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(173) : cplan_fun.sql(14545) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               --assign task assignmnet on that day
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_finish_D;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 192+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(193) : cplan_fun.sql(14565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --put back the effort to project assignment
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_id, TASK_ID = PROJECT_ID
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 211+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(212) : cplan_fun.sql(14584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               --move task assignmnet to FF
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 230+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(231) : cplan_fun.sql(14603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --remove the assighned effort from project assignmnet of that day
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN - h_temp_var, BIT_SCHED = PMO_STPU.BIT_ANDNOT_DAY(BIT_SCHED, h_bitID)
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_finish_D;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 250+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(251) : cplan_fun.sql(14623) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
            End If;
            
         End If;
         --trace_printf("SC_STATE: %d, h_id: %s\n", SC_STATE,h_id);
         
         If SubStr(h_state, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(SCHED_FINISH))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(276) : cplan_fun.sql(14648) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(PLANNED_FINISH))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 294+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(295) : cplan_fun.sql(14667) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      ElsIf Coalesce(h_actual_flag, 0) = 0 Then
         BEGIN
            UPDATE TASK_DAY_OF_YEAR SET DAY_YEAR = h_finish_D
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND DAY_YEAR = h_dayofyear;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 314+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(315) : cplan_fun.sql(14687) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         h_effort := 0;
         
         OPEN C_FF;
         
         LOOP
            FETCH C_FF INTO h_dayofyear, h_minute;
            IF C_FF%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               EXIT;
            End If;
            
            h_effort := Coalesce(h_effort, 0) + Coalesce(h_minute, 0);
            
            h_temp_var := 0;
            h_temp_sc_state := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         END LOOP;
         
         BEGIN
            CLOSE C_FF ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 348+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(349) : cplan_fun.sql(14721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED) INTO h_finish_time FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_finish_D;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 362+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(363) : cplan_fun.sql(14735) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            h_finish_time := '17:00';
         End If;
         
         If SubStr(h_state, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               SCHED_WORK = SCHED_WORK - h_effort, ETC = SCHED_WORK - h_effort - WORK_TODATE,
               SCHED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(h_finish_time))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 387+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(388) : cplan_fun.sql(14760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               PLANNED_WORK = PLANNED_WORK - h_effort, ETC = PLANNED_WORK - h_effort - WORK_TODATE,
               PLANNED_FINISH = PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_finish_D, 1, 10)),PMO_ORASTD.TIME(h_finish_time))
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 407+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(408) : cplan_fun.sql(14780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("SC_STATE: %d, h_finish_D: %s, h_finish_time : %s\n", SC_STATE,h_finish_D,h_finish_time);
      End If;
   End If;
   
   BEGIN
      CLOSE CFF ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_FF', 424+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_FF(425) : cplan_fun.sql(14797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_FF', 430+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_FF(431) : cplan_fun.sql(14803) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_FF;
ALTER FUNCTION FC_FIX_FF COMPILE DEBUG;

/*********************************************************************************************/

-- Remove schedule effort (h_work) from the end of current assignment (TASK_DAY_OF_YEAR table)

-- 1. Remove schedule from TASK_DAY_OF_YEAR where DAY_YEAR < last submitted (h_start_D)

--

-- returns: 0 SUCCESS, -1 FAILLURE

--

-- This function gets called from FC_ETC_SCHEDULE, input actual in timesheets

/*********************************************************************************************/

Prompt == FUNCTION FC_REMOVE_WITHACTUAL ==
CREATE OR REPLACE FUNCTION FC_REMOVE_WITHACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_last_submitted IN OUT NOCOPY DATE
   , sh_h_work PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_contour PLS_INTEGER
   , sh_h_percent_units PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_work PLS_INTEGER := sh_h_work;
   h_contour PLS_INTEGER := sh_h_contour;
   h_percent_units PLS_INTEGER := Coalesce(sh_h_percent_units, 0);
   h_start_date TIMESTAMP;
   h_dayofyear DATE;
   h_minutes PLS_INTEGER := 0;
   h_sched_min PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   i_remove PLS_INTEGER NOT NULL := 0;
   i_sc_state PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CRWA0 IS
         SELECT DAY_YEAR, SCHED_MIN 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND DAY_YEAR < h_last_submitted
         AND ACTUAL_FLAG = 0;

      CURSOR CRWA2 IS
         SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id 
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_REMOVE_WITHACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_REMOVE_WITHACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_REMOVE_WITHACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_last_submitted || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_percent_units), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_REMOVE_WITHACTUAL', 53+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_REMOVE_WITHACTUAL(54) : cplan_fun.sql(14881) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(h_work, 0) < 0 Then
      i_remove := 1;
      h_work := Coalesce(h_work, 0) * -1;
   End If;
   
   h_sched_min := 0;
   h_start_date := NULL;
   
   -- Remove schedule < then last submitted
   If (h_last_submitted IS NOT NULL) Then
      OPEN CRWA0;
      
      LOOP
         FETCH CRWA0 INTO h_dayofyear, h_minutes;
         IF CRWA0%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If i_remove <> 0 Then
            h_work := Coalesce(h_work, 0) - Coalesce(h_minutes, 0);
         Else
            h_work := Coalesce(h_work, 0) + Coalesce(h_minutes, 0);
         End If;
         
         null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_sched_min, 2);
      END LOOP;
      
      BEGIN
         CLOSE CRWA0 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_REMOVE_WITHACTUAL', 96+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_REMOVE_WITHACTUAL(97) : cplan_fun.sql(14924) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_work, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_REMOVE_WITHACTUAL', 104+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_REMOVE_WITHACTUAL(105) : cplan_fun.sql(14932) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   OPEN CRWA2;
   
   If i_remove <> 0 AND Coalesce(h_work, 0) > 0 Then
      LOOP
         FETCH CRWA2 INTO h_dayofyear, h_minutes;
         IF CRWA2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If Coalesce(h_work, 0) < Coalesce(h_minutes, 0) Then
            h_sched_min := Coalesce(h_minutes, 0) - Coalesce(h_work, 0);
            null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_sched_min, 2);
            EXIT;
         Else
            null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_sched_min, 2);
            h_work := Coalesce(h_work, 0) - Coalesce(h_minutes, 0);
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CRWA2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_REMOVE_WITHACTUAL', 141+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_REMOVE_WITHACTUAL(142) : cplan_fun.sql(14969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      If Coalesce(h_work, 0) < 0 Then
         h_work := Coalesce(h_work, 0) * -1;
      End If;
      
      FETCH CRWA2 INTO h_dayofyear, h_minutes;
      IF CRWA2%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_sched_min, 2);
         h_work := Coalesce(h_work, 0) + Coalesce(h_minutes, 0);
         h_start_date := h_dayofyear;
      ElsIf (h_last_submitted IS NOT NULL) Then
         h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
         -- skip last submitted date
         h_start_date := h_last_submitted;
      End If;
      
      BEGIN
         CLOSE CRWA2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_REMOVE_WITHACTUAL', 175+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_REMOVE_WITHACTUAL(176) : cplan_fun.sql(15003) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_start_date IS NOT NULL) AND Coalesce(h_work, 0) > 0 Then
         h_temp_var := 2;
         i_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_work, h_temp_var, h_contour
                            , h_rec_user, 0, h_percent_units);
         If i_sc_state = 0 Then
            i_sc_state := 1;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_REMOVE_WITHACTUAL', 189+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_REMOVE_WITHACTUAL(190) : cplan_fun.sql(15017) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := i_sc_state;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_REMOVE_WITHACTUAL', 198+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_REMOVE_WITHACTUAL(199) : cplan_fun.sql(15026) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_REMOVE_WITHACTUAL;
ALTER FUNCTION FC_REMOVE_WITHACTUAL COMPILE DEBUG;

/***************************************************************************************/

-- Delete with Actual,

-- deletes schedule (ACTUAL_FLAG = 0) from TASK_DAY_OF_YEAR

-- where DAY_YEAR < last submitted (h_start_D)

-- returns:

-- 0 when the last scheduled day in TASK_DAY_OF_YEAR > then last submitted actual (don't skip)

-- 1 when the last scheduled day in TASK_DAY_OF_YEAR = then last submitted actual (skip first day)

/***************************************************************************************/

Prompt == FUNCTION FC_DEL_WITHACTUAL ==
CREATE OR REPLACE FUNCTION FC_DEL_WITHACTUAL
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_sched_work PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_sched_work PLS_INTEGER := sh_h_sched_work;
   h_work PLS_INTEGER := sh_h_work;
   h_type PLS_INTEGER := sh_h_type;
   h_dayofyear DATE;
   h_minutes PLS_INTEGER := 0;
   h_work1 PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CDWA0 IS
         SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id AND ACTUAL_FLAG = 0 AND DAY_YEAR < TO_DATE(SubStr(h_start_date, 1, 10));

      CURSOR CDWA2 IS
         SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id AND ACTUAL_FLAG = 0
         ORDER BY DAY_YEAR DESC;

      CURSOR CDWA1 IS
         SELECT * FROM (SELECT DAY_YEAR, SCHED_MIN FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID=h_tsk_assignment_id
         AND ACTUAL_FLAG = 0 AND DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10))
         ORDER BY DAY_YEAR DESC ) WHERE ROWNUM<=1;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DEL_WITHACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DEL_WITHACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_DEL_WITHACTUAL(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_sched_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 51+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DEL_WITHACTUAL(52) : cplan_fun.sql(15102) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("0__* h_type: %d, h_tsk_assignment_id = %s, h_start_date = %s\n",h_type,h_tsk_assignment_id,h_start_date);
   
   BEGIN
      SELECT UNITS_MSP, CONTOUR_TYPE INTO h_percent_units, h_contour_type
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID=h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(67) : cplan_fun.sql(15117) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("3.2.0__ SC_STATE = %d, h_percent_units = %d, h_prj_assignment_id: %d %s\n",SC_STATE,h_percent_units, h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   -- Remove schedule < then last submitted
   
   OPEN CDWA0;
   
   LOOP
      FETCH CDWA0 INTO h_dayofyear;
      IF CDWA0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_temp_var := 0;
      null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
   END LOOP;
   
   BEGIN
      CLOSE CDWA0 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 99+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(100) : cplan_fun.sql(15150) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If ((h_prj_assignment_id IS NOT NULL)) AND (Coalesce(h_type, 0) = -1) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 106+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_WITHACTUAL(107) : cplan_fun.sql(15157) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   -- Check if we have to remove schedule backwords
   BEGIN
      SELECT COALESCE(SUM(SCHED_MIN),0) - h_sched_work INTO h_work1
      FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 122+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(123) : cplan_fun.sql(15173) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("1__ h_sched_work = %d, h_work1 = %d\n",h_sched_work,h_work1);
   If Coalesce(h_work1, 0) > 0 Then
      OPEN CDWA2;
      
      LOOP
         FETCH CDWA2 INTO h_dayofyear, h_minutes;
         IF CDWA2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         h_temp_var := 0;
         null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
         h_work1 := Coalesce(h_work1, 0) - Coalesce(h_minutes, 0);
         If Coalesce(h_work1, 0) <= 0 Then
            EXIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE CDWA2 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 158+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(159) : cplan_fun.sql(15209) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("2__ h_work1 = %d,h_start_date = %s\n",h_work1,h_start_date);
   
   -- fix the last scheduled day
   --if ( h_work1 < 0 )
   If Coalesce(h_work1, 0) <= 0 Then
      OPEN CDWA1;
      
      FETCH CDWA1 INTO h_dayofyear, h_minutes;
      IF CDWA1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      h_temp_var := PMO_DB2M.SC_STATE;
      
      BEGIN
         CLOSE CDWA1 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 188+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(189) : cplan_fun.sql(15239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("3__ h_temp_var = %d, h_work = %d, h_minutes = %d, h_dayofyear = %s\nh_prj_assignment_id = %s, h_prj_assignment_id_Ind = %d\n",h_temp_var,h_work,h_minutes,h_dayofyear,h_prj_assignment_id,h_prj_assignment_id_Ind);
      If Coalesce(h_temp_var, 0) = 0 Then
         h_start_date := h_dayofyear;
         If Coalesce(h_work, 0) > 0 Then
            If Coalesce(h_work, 0) > Coalesce(h_minutes, 0) Then
               h_temp_var := 0;
               null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
               h_temp_sc_state := 0;
            Else
               h_temp_sc_state := 1;
            End If;
         Else
            If (h_prj_assignment_id IS NOT NULL) Then
               If Coalesce(h_type, 0) = 0 Then
                  BEGIN
                     SELECT SCHED_MIN INTO h_work1 FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 216+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(217) : cplan_fun.sql(15267) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("3.1__ SC_STATE = %d, h_work1 = %d\n",SC_STATE,h_work1);
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_work1, 0) > 0) Then
                     h_temp_var := 0;
                     null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
                     h_temp_sc_state := 0;
                  Else
                     h_temp_sc_state := 1;
                  End If;
               Else
                  BEGIN
                     SELECT AVAILABLE_MIN INTO h_work1 FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 239+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(240) : cplan_fun.sql(15290) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("3.2__ SC_STATE = %d, h_work1 = %d\n",SC_STATE,h_work1);
                  
                  If (Coalesce(h_percent_units, 0) <> 10000) AND (Coalesce(h_percent_units, 0) <> 0) Then
                     h_work1 := Trunc(((Coalesce(h_work1, 0) * Coalesce(h_percent_units, 0)) / 10000.00) + 0.49);
                  End If;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(h_work1, 0) <> Coalesce(
                        h_minutes, 0)) Then
                     h_temp_var := 0;
                     
                     null_return := FC_ACTUAL_DAY(h_tsk_assignment_id, h_dayofyear, h_temp_var, 0);
                     h_temp_sc_state := 0;
                  Else
                     h_temp_sc_state := 1;
                  End If;
               End If;
            End If;
            
            --trace_printf("4__ h_temp_sc_state = %d, h_work1 = %d, h_minutes = %d\n",h_temp_sc_state,h_work1,h_minutes);
         End If;
      Else
         BEGIN
            SELECT MAX(DAY_YEAR) INTO h_start_date FROM TASK_DAY_OF_YEAR
            WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 273+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(274) : cplan_fun.sql(15324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := 1;
      End If;
   ElsIf Coalesce(h_work1, 0) > 0 Then
      BEGIN
         SELECT MAX(DAY_YEAR) INTO h_start_date FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DEL_WITHACTUAL', 290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DEL_WITHACTUAL(291) : cplan_fun.sql(15341) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := 1;
   End If;
   
   PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DEL_WITHACTUAL', 301+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DEL_WITHACTUAL(302) : cplan_fun.sql(15352) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DEL_WITHACTUAL;
ALTER FUNCTION FC_DEL_WITHACTUAL COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_RESCHEDULE ==
CREATE OR REPLACE FUNCTION FC_RESCHEDULE
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_delay PLS_INTEGER := 0
   , sh_h_units PLS_INTEGER := 0
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_type PLS_INTEGER := sh_h_type;
   h_contour PLS_INTEGER := sh_h_contour;
   h_delay PLS_INTEGER := Coalesce(sh_h_delay, 0);
   h_units PLS_INTEGER := Coalesce(sh_h_units, 0);
   h_prj_assignment_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_active VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_last_submitted DATE;
   h_every PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_bitflag1 PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_temp_res_cost NUMBER := 0;
   h_temp_type_id PLS_INTEGER := 0;
   h_temp_cri PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_temp_priority PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_update PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_RESCHEDULE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_RESCHEDULE'), '-- ' || SYSTIMESTAMP, ':r := FC_RESCHEDULE(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_start_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_units), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 74+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_RESCHEDULE(75) : cplan_fun.sql(15437) : ' || SYSTIMESTAMP);
   END IF; 

   h_finish_date := NULL;
   h_finish_DT := NULL;
   h_active := '';
   h_every := 0;
   h_occurrence := 0;
   h_start_DT := h_start_date;
   
   BEGIN
      SELECT PROJECT_ID, PROJECT_ASSIGNMENT_ID, TASK_ID, RESOURCE_ID, RESOURCE_TYPE, 
      C_EFFORT, T_UNIT, C_UNIT, Trunc(BIT_FLAG), LAST_SUBMITTED, CONTOUR_TYPE
      INTO h_project_id, h_prj_assignment_id, h_task_id, h_res_id, h_res_type, 
      h_work, h_eunit, h_unit, h_work_type, h_last_submitted, h_bitflag
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 96+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(97) : cplan_fun.sql(15459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 102+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(103) : cplan_fun.sql(15465) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), TYPE_ID 
      INTO h_per_day, h_per_week, h_type_id
      FROM TMT_WBS 
      WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESCHEDULE', 118+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESCHEDULE(119) : cplan_fun.sql(15481) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If SubStr(h_res_type, 1, 1) = '0' Then
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
         h_consider := 1;
         -- Personal Task
      ElsIf (h_prj_assignment_id IS NULL) Then
         h_consider := 7;
         -- none
      Else
         h_consider := PMO_CNST.SC_SRCH_FROM_PROJECT;
         -- from project
      End If;
   Else
      h_consider := PMO_CNST.SC_SRCH_PROFILE;
      -- none
   End If;
   
   If Coalesce(h_bitflag, 0) >= 90 Then
      If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) <> 0 Then
         h_bitflag := 2;
      Else
         h_bitflag := 1;
      End If;
   End If;
   
   h_temp_sc_state := FC_CALC_DURATION(h_start_DT, h_finish_DT, 0, 0, Coalesce(h_consider, 0)
                                    , Coalesce(h_bitflag, 0), Coalesce(h_contour, 0), Coalesce(
                                    h_occurrence, 0), Coalesce(h_per_day, 0), Coalesce(
                                    h_per_week, 0), h_work, h_effort, h_duration);
   
   If Coalesce(h_temp_sc_state, 0) <> 0 Then
      h_duration := 1200;
   End If;
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   If Coalesce(h_bitflag, 0) = 0 Then
      h_work := 0;
      h_unit := 0;
   ElsIf (Coalesce(h_work, 0) = 0) AND (h_units > 0) Then
      h_day_min := Trunc(((Coalesce(h_per_day, 0) * h_units) / 10000.00) + 0.5);
      h_work := h_units;
      h_unit := 100;
   End If;
   --trace_printf("*** h_units = %d, h_work = %d, h_unit: %d\n", h_units,h_work,h_unit);//trace_flush();
   
   If (h_prj_assignment_id IS NOT NULL) Then
      If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_start_date, h_finish_date
            , h_effort) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 171+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(172) : cplan_fun.sql(15534) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
      End If;
      
      --h_bitflag1 = (h_bitflag | SC_BIT_27);     // don't update TASK_ASSIGNMENTS
      If Coalesce(h_type, 0) = 1 Then
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
         -- don't update TASK_ASSIGNMENTS
      Else
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
         -- update TASK_ASSIGNMENTS
      End If;
      
      If (Coalesce(h_contour, 0) >= PMO_CNST.SC_BIT_30) AND (PMO_ORASTD.strncmp(h_start_date, h_start_DT
             , 10) <> 0) Then
         h_contour := (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_NOT_30));
         -- don't skip
      End If;
      
      -- assign from PA
      
      h_temp_res_cost := 0.00;
      h_temp_type_id := 3;
      h_temp_cri := 0;
      h_temp_consider := 10;
      h_temp_priority := 0;
      
      If Coalesce(h_work_type, 0) = 2 Then
         h_update := 0;
      Else
         h_update := 3;
      End If;
      
      h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_DT, h_finish_DT, h_duration, h_effort, h_work
                               , h_eunit, h_unit, h_temp_cri, h_temp_consider, h_res_type
                               , h_work_type, h_bitflag1, h_contour, h_occurrence, h_update
                               , h_per_day, h_temp_priority, h_rec_user, Coalesce(h_per_day, 0)
                               , h_delay, 0.00, 0.00);
   Else
      --h_bitflag1 = (h_bitflag | SC_BIT_27);     // don't update TASK_ASSIGNMENTS
      If Coalesce(h_type, 0) = 1 Then
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
         -- update TASK_ASSIGNMENTS
         h_update := 1;
      Else
         h_bitflag1 := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27));
         -- don't update TASK_ASSIGNMENTS
         h_update := 0;
      End If;
      
      h_temp_res_cost := 0.00;
      h_temp_type_id := 3;
      h_temp_cri := 0;
      h_temp_priority := 0;
      
      h_temp_sc_state := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id
                               , h_temp_res_cost, h_temp_type_id, h_active, h_active, h_active
                               , h_active, h_start_date, h_finish_date, h_duration, h_effort, h_work
                               , h_eunit, h_unit, h_temp_cri, h_consider, h_res_type, h_work_type
                               , h_bitflag1, h_contour, h_occurrence, h_update, h_per_day
                               , h_temp_priority, h_rec_user, Coalesce(h_per_day, 0), h_delay, 0.00
                               , 0.00);
   End If;
   
   If Coalesce(h_temp_sc_state, 0) > 0 AND Coalesce(h_temp_sc_state, 0) <> 100 Then
      h_temp_sc_state := PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESCHEDULE', 245+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_RESCHEDULE(246) : cplan_fun.sql(15608) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_RESCHEDULE;
ALTER FUNCTION FC_RESCHEDULE COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION FC_ETC_SCHEDULE ==
CREATE OR REPLACE FUNCTION FC_ETC_SCHEDULE
(
     h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type PLS_INTEGER := sh_h_type;
   h_prj_assignment_id VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_active VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_week_of DATE;
   h_day_year DATE;
   h_last_submitted DATE;
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_start_DT TIMESTAMP;
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_task_start_DT TIMESTAMP;
   h_task_finish_DT TIMESTAMP;
   h_bit_flag VARCHAR2(1);
   h_type_id PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_proposed_work PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_zero PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_resched PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_AS TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_forecast_start TIMESTAMP;
   h_forecast_finish TIMESTAMP;
   h_FF_flag PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_accpt_aut PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_has_actual PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ETC_SCHEDULE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ETC_SCHEDULE'), '-- ' || SYSTIMESTAMP, ':r := FC_ETC_SCHEDULE(' || 
       '''' || h_tsk_assignment_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 80+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ETC_SCHEDULE(81) : cplan_fun.sql(15699) : ' || SYSTIMESTAMP);
   END IF; 

   
   h_prj_assignment_id := '';
   
   h_start_DT := '';
   h_resched := 0;
   h_accpt_aut := 1;
   h_tph_delete := 0;
   h_work_type := 2;
   h_consider := 7;
   h_zero := 0;
   
   --trace_printf("h_type = %d, h_tsk_assignment_id: %s\n", h_type,h_tsk_assignment_id); //trace_flush();
   --***************************************************************************
   -- h_type = 1   old work to date == new  work to date AND old etc != new etc, called from FC_U_SUMMARY_TIMESHEETS
   -- h_type = 2   update etc, there is no actual
   -- h_type = 0   else, called from FC_U_SUMMARY_TIMESHEETS
   --***************************************************************************
   If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 2) Then
      BEGIN
         SELECT ra.ELEMENT_TYPE, ta.PROJECT_ID, ta.PROJECT_ASSIGNMENT_ID, ta.TSK_ASSIGNMENT_ID, ta.RES_ASSIGNMENT_ID, 
         ta.TASK_ID, ta.RESOURCE_ID, ta.ACTUAL_START, ta.LAST_SUBMITTED, ta.PROPOSED_WORK, ta.SCHED_WORK, ta.WORK_TODATE, 
         ta.ETC, ta.C_EFFORT, ta.T_UNIT, ta.C_UNIT, ta.C_CONTOUR, ta.CONTOUR_TYPE, ta.UNITS_MSP, ta.LAST_SCHED_MIN,
         ta.UNRESOLVED_START, ta.UNRESOLVED_FINISH, ta.BILLABLE, ta.CAPITAL_EXPENSE, ta.REGULAR_COST, ta.REGULAR_RATE,
         ta.SCHED_START, ta.FORECAST_START, ta.FORECAST_FINISH, CASE WHEN ta.FORECAST_FINISH IS NOT NuLL AND ta.FORECAST_START IS NuLL AND ta.FORECAST_FINISH BETWEEN ta.SCHED_START AND ta.SCHED_FINISH THEN 1 ELSE 0 END,
         ta.RESOURCE_TYPE, ta.BIT_FLAG 
         INTO h_type_id, h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_assign_id, 
         h_task_id, h_res_id, h_AS, h_last_submitted, h_proposed_work, h_effort, h_work_todate, 
         h_etc, h_work, h_eunit, h_unit, h_contour, h_bitflag, h_percent_units, h_last_day,
         h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale,
         h_sched_start, h_forecast_start, h_forecast_finish,
         h_FF_flag, h_res_type, h_bit_flag
         FROM TASK_ASSIGNMENTS ta JOIN RESOURCE_ASSIGNMENTS ra ON ra.RES_ASSIGNMENT_ID = ta.RES_ASSIGNMENT_ID
         WHERE ta.TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(123) : cplan_fun.sql(15741) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_SD := SubStr(h_ta_start_DT, 1, 10);
      h_FD := SubStr(h_ta_finish_DT, 1, 10);
      h_remaining := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
      h_sched_work := Coalesce(h_effort, 0);
      
      If Coalesce(h_work_todate, 0) = 0 Then
         If Coalesce(h_etc, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 135+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(136) : cplan_fun.sql(15754) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      Else
         If Coalesce(h_type_id, 0) = PMO_CNST.SC_PERSONAL_TASK Then
            If PMO_ORASTD.strcmp(h_res_id, h_rec_user) <> 0 Then
               --if it's personal task or manager accept etc and forecast automatically
               --check if forecast change
               If ((h_forecast_finish IS NOT NULL)) OR ((h_forecast_start IS NOT NULL)) Then
                  If (h_forecast_start IS NULL) Then
                     h_forecast_start := h_sched_start;
                  End If;
                  
                  If ((h_forecast_finish IS NOT NULL)) AND (SubStr(h_res_type, 1, 1) = '0') Then
                     BEGIN
                        SELECT COALESCE(SUM(AVAILABLE_MIN),0) INTO h_effort 
                        FROM CALENDAR_DAY_OF_YEAR
                        WHERE ELEMENT_ID = h_res_id 
                        AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_forecast_start, 1, 10)) 
                        AND TO_DATE(SubStr(h_forecast_finish, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 162+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(163) : cplan_fun.sql(15781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  h_type := 3;
                  h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
                  
                  h_accpt_aut := 2;
                  --flag for accept etc and Forecast in U_assign_string
               Else
                  h_effort := Coalesce(h_work_todate, 0) + Coalesce(h_remaining, 0);
                  h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
                  h_type := 1;
               End If;
               
               h_temp_sc_state := FC_ACPT_ETC_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id
                                      , h_prj_assignment_id, h_forecast_start, h_forecast_finish
                                      , h_last_submitted, h_bit_flag, h_effort, h_remaining
                                      , h_sched_work, h_etc, h_contour, h_FF_flag, h_type
                                      , h_ta_start_DT, h_ta_finish_DT, h_reg_cost, h_reg_sale
                                      , h_billable, h_capital_expense, h_rec_user, 0);
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 185+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(186) : cplan_fun.sql(15804) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            ElsIf ((h_last_submitted IS NULL)) OR (PMO_ORASTD.strncmp(h_ta_start_DT, h_last_submitted
                  , 10) = 0) Then
               If (h_last_submitted IS NULL) Then
                  h_temp_var := 0;
               Else
                  h_temp_var := 1;
               End If;
               
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 199+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(200) : cplan_fun.sql(15818) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN 0;
            End If;
         ElsIf Coalesce(h_remaining, 0) < 0 Then
            If (h_prj_assignment_id IS NOT NULL) Then
               If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_last_submitted
                     , h_etc) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 208+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(209) : cplan_fun.sql(15827) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
               End If;
               
               h_temp_var := -1;
            Else
               h_temp_var := 0;
            End If;
            
            h_effort := Coalesce(h_work_todate, 0);
            BEGIN
               SELECT TO_DATE(SubStr(h_ta_finish_DT, 1, 10)) + 1      INTO h_last_submitted FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 227+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(228) : cplan_fun.sql(15846) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_last_submitted, h_effort, h_work
                            , h_prj_assignment_id, h_temp_var);
            
            h_temp_var := 1;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 238+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(239) : cplan_fun.sql(15857) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      End If;
      
      If (h_prj_assignment_id IS NOT NULL) Then
         If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_last_submitted
               , h_etc) <> 0 Then
            If Coalesce(h_tph_delete, 0) = 0 Then
               h_tph_delete := -1;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            End If;
            
            h_temp_var := -1;
            h_temp_sc_state := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_last_submitted, h_effort
                                     , h_work, h_prj_assignment_id, h_temp_var);
            
            If Coalesce(h_tph_delete, 0) = -1 Then
               h_tph_delete := 0;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET BEST = -1 WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 277+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(278) : cplan_fun.sql(15896) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_temp_var := 1;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 286+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(287) : cplan_fun.sql(15905) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
         End If;
      End If;
   ElsIf Coalesce(h_type, 0) = 1 Then
      BEGIN
         SELECT PROJECT_ID, PROJECT_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, RESOURCE_ID, UNITS_MSP,
         CASE WHEN LAST_SUBMITTED IS NOT NuLL AND TO_DATE(SubStr(BAK_START_DT, 1, 10)) > TO_DATE(SubStr(LAST_SUBMITTED, 1, 10)) THEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) ELSE LAST_SUBMITTED END,
         ACTUAL_START, LAST_SUBMITTED, PROPOSED_WORK, CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END, WORK_TODATE, 
         ETC, C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR, CONTOUR_TYPE, BIT_FLAG, 
         UNRESOLVED_START, UNRESOLVED_FINISH, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE, LAST_SCHED_MIN
         INTO h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_assign_id, h_task_id, h_res_id, h_percent_units,
         h_start_DT, h_AS, h_last_submitted, h_proposed_work, h_sched_work, h_work_todate, 
         h_etc, h_work, h_eunit, h_unit, h_contour, h_bitflag, h_bit_flag,
         h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale, h_last_day
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 309+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(310) : cplan_fun.sql(15928) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_SD := SubStr(h_ta_start_DT, 1, 10);
      h_FD := SubStr(h_ta_finish_DT, 1, 10);
      h_remaining := Coalesce(h_sched_work, 0) - Coalesce(h_work_todate, 0);
   End If;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) OR (PMO_ORASTD.strcmp(h_project_id
         , h_task_id) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 322+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(323) : cplan_fun.sql(15941) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   End If;
   
   --  RULE: if there is no actual, then
   -- 1. h_etc > h_remaining --> etc more, accetpt automatically if manager, otherwise save it and don't accept
   -- 2. h_etc < h_remaining --> etc less, accetpt automatically
   -- 3. h_etc = h_remaining --> remove actual, h_effort = h_proposed_work, go to original schedule
   If Coalesce(h_etc, 0) > Coalesce(h_remaining, 0) Then
      If (h_prj_assignment_id IS NOT NULL) Then
         If FC_CHECK_PA(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, h_AS, h_last_submitted
               , h_etc) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 336+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(337) : cplan_fun.sql(15955) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.PROJECT_ASSIGNMENT_REMAINING;
         End If;
      End If;
      If Coalesce(h_type, 0) = 1 Then
         -- User has rights to reschedule
         h_effort := Coalesce(h_etc, 0);
      ElsIf Coalesce(h_remaining, 0) = 0 Then
         -- In timesheets user has no rights to reschedule
         If (h_last_submitted IS NULL) Then
            h_temp_var := 0;
         Else
            h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
            h_temp_var := 1;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 356+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(357) : cplan_fun.sql(15975) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      Else
         -- User has no rights to reschedule
         h_effort := Coalesce(h_remaining, 0);
      End If;
   ElsIf Coalesce(h_etc, 0) < Coalesce(h_remaining, 0) Then
      h_effort := Coalesce(h_etc, 0);
   ElsIf Coalesce(h_work_todate, 0) = 0 Then
      If Coalesce(h_effort, 0) <> Coalesce(h_proposed_work, 0) Then
         h_effort := Coalesce(h_proposed_work, 0);
         If Coalesce(h_type, 0) = 0 Then
            h_etc := Coalesce(h_proposed_work, 0);
         End If;
      ElsIf (Coalesce(h_bitflag, 0) = 0) AND (Coalesce(h_percent_units, 0) = 0) Then
         -- There is nothing to reschedule
         If (h_last_submitted IS NULL) Then
            h_temp_var := 0;
         Else
            h_temp_var := 1;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 382+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(383) : cplan_fun.sql(16001) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      Else
         h_effort := Coalesce(h_etc, 0);
      End If;
   ElsIf (Coalesce(h_bitflag, 0) = 0) AND (Coalesce(h_percent_units, 0) = 0) Then
      h_temp_var := 0;
      BEGIN
         SELECT COUNT(*) INTO h_temp_var FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0 AND DAY_YEAR < h_last_submitted;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 401+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(402) : cplan_fun.sql(16020) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         h_effort := Coalesce(h_etc, 0);
      Else
         -- There is nothing to reschedule
         If (h_last_submitted IS NULL) Then
            h_temp_var := 0;
         Else
            h_temp_var := 1;
         End If;
         
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 419+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(420) : cplan_fun.sql(16038) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN 0;
      End If;
   Else
      h_effort := Coalesce(h_etc, 0);
   End If;
   
   If (h_last_submitted IS NULL) Then
      h_start_date := h_ta_start_DT;
   Else
      h_week_of := NULL;
      BEGIN
         SELECT MAX(WEEK_OF) + 7      INTO h_week_of 
         FROM TIMESHEET_APPROVAL_STATUS
         WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 441+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(442) : cplan_fun.sql(16060) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_week_of IS NOT NULL) AND (PMO_ORASTD.strncmp(h_last_submitted, h_week_of, 10) < 0) Then
         h_start_date := h_week_of;
      Else
         h_start_date := h_last_submitted;
      End If;
   End If;
   
   BEGIN
      SELECT TASK_TYPE, MINUTE_PER_DAY, UNRESOLVED_START, UNRESOLVED_FINISH, UNRESOLVED_DURATION
      INTO h_task_type, h_per_day, h_task_start_DT, h_task_finish_DT, h_duration
      FROM TMT_WBS
      WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 464+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(465) : cplan_fun.sql(16083) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --if ((h_task_type == SC_FIXED_DURATION)  (h_etc > 0))
   --{
   -- BEGIN_SQL
   --  SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   --  WHERE PROJECT_ID = HOST(h_project_id)
   --   AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   --   AND ACTUAL_FLAG = 0;
   -- END_SQL_NOCHECK;
   
   -- if ( h_duration > 0 )
   -- {
   --  InitializeHost(SC_PARAM(h_tph_delete), -1);
   --  h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   --    SC_PARAM(h_project_id),
   --    SC_PARAM(h_task_id),
   --    SC_PARAM(h_tsk_assignment_id),
   --    SC_PARAM(h_SD),
   --    SC_PARAM(h_FD),
   --    SC_PARAM(h_reg_cost),
   --    SC_PARAM(h_reg_sale),
   --    SC_PARAM(h_tph_delete),
   --    SC_PARAM(h_capital_expense),
   --    SC_PARAM(h_billable));
   
   --  h_minutes = int ( ((double) h_etc / (double) h_duration) + 0.49);
   --  h_temp_sc_state = FC_FD_U_TDY (SC_PARAM(h_project_id),
   --         SC_PARAM(h_tsk_assignment_id),
   --         SC_PARAM(h_start_date),
   --         SC_PARAM(h_per_day),
   --         SC_PARAM(h_minutes));
   
   --  if (h_temp_sc_state != 0)
   --  {
   --   return h_temp_sc_state;
   --  }
   
   --  InitializeHost(SC_PARAM(h_tph_delete), 0);
   --  h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   --    SC_PARAM(h_project_id),
   --    SC_PARAM(h_task_id),
   --    SC_PARAM(h_tsk_assignment_id),
   --    SC_PARAM(h_SD),
   --    SC_PARAM(h_FD),
   --    SC_PARAM(h_reg_cost),
   --    SC_PARAM(h_reg_sale),
   --    SC_PARAM(h_tph_delete),
   --    SC_PARAM(h_capital_expense),
   --    SC_PARAM(h_billable));
   
   --  h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   --  InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
   --  U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   -- }
   
   -- return 0;
   --}
   
   /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_remaining > 0) )
   	{
   		if (h_type == 1)    // reschedule etc
   		{
   			BEGIN_SQL
   				SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   				WHERE PROJECT_ID = HOST(h_project_id)
   					AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   					AND ACTUAL_FLAG = 0;
   			END_SQL_NOCHECK;
   
   			if ( h_duration > 0 )
   			{
   				InitializeHost(SC_PARAM(h_tph_delete), -1);
   				h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   						SC_PARAM(h_project_id),
   						SC_PARAM(h_task_id),
   						SC_PARAM(h_tsk_assignment_id),
   						SC_PARAM(h_SD),
   						SC_PARAM(h_FD),
   						SC_PARAM(h_reg_cost),
   						SC_PARAM(h_reg_sale),
   						SC_PARAM(h_tph_delete),
   						SC_PARAM(h_capital_expense),
   						SC_PARAM(h_billable));
   
   				h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.49);
   				h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   										  SC_PARAM(h_tsk_assignment_id),
   										  SC_PARAM(h_last_submitted),
   										  SC_PARAM(h_per_day),
   										  SC_PARAM(h_minutes));
   
   				InitializeHost(SC_PARAM(h_tph_delete), 0);
   				h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   						SC_PARAM(h_project_id),
   						SC_PARAM(h_task_id),
   						SC_PARAM(h_tsk_assignment_id),
   						SC_PARAM(h_SD),
   						SC_PARAM(h_FD),
   						SC_PARAM(h_reg_cost),
   						SC_PARAM(h_reg_sale),
   						SC_PARAM(h_tph_delete),
   						SC_PARAM(h_capital_expense),
   						SC_PARAM(h_billable));
   
   				h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
   				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   			}
   		}
   		else if ( h_work_todate > 0 )
   		{
   			Nullify(SC_PARAM(h_finish_date));
   			if ( strncmp(h_AS,h_SD,10) != 0)
   			{
   				BEGIN_SQL
   					DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_MIN_MAX (
   						MIN_DATE  DATE,
   						MAX_DATE  DATE,
   						MINUTE_PER_DAY SMALLINT,
   						START_DATE  TIMESTAMP,
   						FINISH_DATE  TIMESTAMP,
   						DURATION   INTEGER
   					)
   					WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
   				END_SQL_NOCHECK;
   
   				BEGIN_SQL
   					DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_CDY (
   						ELEMENT_ID  CHAR(32),
   						DAY_YEAR  DATE,
   						DAY_WEEK  SMALLINT,
   						AVAILABLE_MIN SMALLINT,
   						BIT_SCHED  CHAR(36) FOR BIT DATA,
   						//removed from here
   						WEEK_OF   SMALLINT,
   						MONTH_OF  SMALLINT,
   						YEAR_OF   SMALLINT
   					)
   					WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
   				END_SQL_NOCHECK;
   
   				h_temp_var = int (((double) h_duration / (double) h_per_day) + 0.5);
   				if (h_temp_var < 10)
   					h_temp_var = 20;
   				else
   					h_temp_var *= 2;
   
   				BEGIN_SQL
   					SELECT DATE(HOST(h_AS)) + HOST(h_temp_var) DAYS INTO HOST(h_finish_date) FROM DUAL ;
   				END_SQL;
   
   				BEGIN_SQL
   					INSERT INTO session.PMO_TMP_CDY
   						SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
   						FROM CALENDAR_DAY_OF_YEAR
   						WHERE ELEMENT_ID = HOST(h_project_id) AND DAY_YEAR BETWEEN DATE(HOST(h_AS)) AND DATE(HOST(h_finish_date));
   				END_SQL;
   
   				BEGIN_SQL
   					INSERT INTO session.PMO_TMP_MIN_MAX
   					SELECT MIN(DAY_YEAR), MAX(DAY_YEAR), HOST(h_per_day), SC_1899_12_30_8_00_00,SC_1899_12_30_8_00_00, 0
   					FROM session.PMO_TMP_CDY;
   				END_SQL;
   
   				BEGIN_SQL
   					SELECT TIMESTAMP(DATE(HOST(h_AS)), TIME(FIRST_AVAIL_TIME(BIT_SCHED))) INTO HOST(h_AS)
   					FROM session.PMO_TMP_CDY WHERE DAY_YEAR = HOST(h_AS);
   				END_SQL;
   				InitializeHost(SC_PARAM(h_finish_date), "");
   
   				h_temp_var = GET_FD_DURATION(SC_PARAM(h_project_id), SC_PARAM(h_AS), SC_PARAM(h_finish_date), SC_PARAM(h_duration));
   
   				BEGIN_SQL
   					SELECT COUNT('X') INTO HOST(h_bitflag) FROM DUAL WHERE EXISTS (
   						SELECT 'X' FROM TASK_ASSIGNMENTS
   						WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
   							AND TSK_ASSIGNMENT_ID <> HOST(h_tsk_assignment_id) AND ACTUAL_START IS NOT NuLL);
   				END_SQL_NOCHECK;
   
   				if (h_bitflag == 0)
   				{
   
   					h_temp_var = FC_CHANGE_TA_DATES (SC_PARAM(h_project_id),
   													 SC_PARAM(h_task_id),
   													 SC_PARAM(h_task_start_DT),
   													 SC_PARAM(h_task_finish_DT),
   													 SC_PARAM(h_AS),
   													 SC_PARAM(h_finish_date),
   													 SC_PARAM(h_per_day),
   													 SC_PARAM(h_rec_user), h_task_type );
   				}
   
   				BEGIN_SQL
   					SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   					WHERE PROJECT_ID = HOST(h_project_id)
   						AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   						AND ACTUAL_FLAG = 0;
   				END_SQL_NOCHECK;
   
   				if ( h_duration > 0 )
   				{
   					if (h_tph_delete == 0)
   					{
   						InitializeHost(SC_PARAM(h_tph_delete), -1);
   						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   					}
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.49);
   					if(IsNotNullParam(h_finish_date))
   					{
   						if ( (IsNotNullParam(h_AS))  (strncmp(h_AS,h_FD,10) > 0) )
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0;
   							END_SQL;
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_AS)) AND r.DAY_YEAR <= DATE(HOST(h_finish_date));
   							END_SQL;
   						}
   						else if ( strncmp(h_finish_date,h_FD,10) > 0)
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0 AND DAY_YEAR < DATE(HOST(h_last_submitted));
   							END_SQL;
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_FD)) AND r.DAY_YEAR <= DATE(HOST(h_finish_date));
   							END_SQL;
   						}
   						else if ( strncmp(h_finish_date,h_FD,10) < 0)
   						{
   							BEGIN_SQL
   								DELETE FROM TASK_DAY_OF_YEAR
   								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   									AND ACTUAL_FLAG = 0 AND DAY_YEAR > DATE(HOST(h_finish_date));
   							END_SQL;
   
   							BEGIN_SQL
   								INSERT INTO TASK_DAY_OF_YEAR (
   									PROJECT_ID,
   									RESOURCE_ID,
   									TSK_ASSIGNMENT_ID,
   									DAY_YEAR,
   									BIT_FLAG,
   									SCHED_MIN,
   									PARENT_ID,
   									ACTUAL_FLAG,
   									TASK_ID )
   								SELECT
   									HOST(h_project_id),
   									HOST(h_res_id),
   									HOST(h_tsk_assignment_id),
   									r.DAY_YEAR,
   									'2',
   									HOST(h_minutes),
   									HOST(h_tsk_assignment_id),
   									0,
   									HOST(h_task_id)
   								FROM CALENDAR_DAY_OF_YEAR r
   								WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR > DATE(HOST(h_last_submitted)) AND r.DAY_YEAR < DATE(HOST(h_SD));
   							END_SQL;
   						}
   					}
   
   					BEGIN_SQL
   						SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   						WHERE PROJECT_ID = HOST(h_project_id)
   							AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   							AND ACTUAL_FLAG = 0;
   					END_SQL_NOCHECK;
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.499);
   					h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   											  SC_PARAM(h_tsk_assignment_id),
   											  SC_PARAM(h_last_submitted),
   											  SC_PARAM(h_per_day),
   											  SC_PARAM(h_minutes));
   
   					if (h_tph_delete == -1)
   					{
   						InitializeHost(SC_PARAM(h_tph_delete), 0);
   						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   					}
   
   					h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_29);
   					h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   				}
   			}
   		}
   		else
   		{
   			BEGIN_SQL
   				SELECT COUNT('X') INTO HOST(h_bitflag) FROM DUAL WHERE EXISTS (
   					SELECT 'X' FROM TASK_ASSIGNMENTS
   					WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id)
   						AND TSK_ASSIGNMENT_ID <> HOST(h_tsk_assignment_id) AND ACTUAL_START IS NOT NuLL);
   			END_SQL_NOCHECK;
   
   			if (h_bitflag == 0)
   			{
   				BEGIN_SQL
   					SELECT BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING
   					INTO HOST(h_last_submitted), HOST(h_finish_date), HOST(h_effort), HOST(h_assign_string)
   					FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
   				END_SQL_NOCHECK;
   
   				h_bitflag = (1 | SC_BIT_28);
   
   				InitializeHost(SC_PARAM(h_temp_work_type), 2);
   				InitializeHost(SC_PARAM(h_temp_contour_type), 0);
   				InitializeHost(SC_PARAM(h_occurrence), 0);
   				InitializeHost(SC_PARAM(h_temp_consider), 7);
   
   				if ( strncmp(h_last_submitted,h_SD,10) != 0)
   				{
   					BEGIN_SQL
   						SELECT MIN(BAK_START_DT), MAX(BAK_FINISH_DT) INTO HOST(h_ta_start_DT), HOST(h_ta_finish_DT)
   						FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_task_id);
   					END_SQL_NOCHECK;
   
   					h_temp_var = FC_CHANGE_TA_DATES (SC_PARAM(h_project_id),
   													 SC_PARAM(h_task_id),
   													 SC_PARAM(h_task_start_DT),
   													 SC_PARAM(h_task_finish_DT),
   													 SC_PARAM(h_ta_start_DT),
   													 SC_PARAM(h_ta_finish_DT),
   													 SC_PARAM(h_per_day),
   													 SC_PARAM(h_rec_user), h_task_type );
   				}
   
   				// Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
   				h_temp_var = U_TSKRES2(SC_PARAM(h_tsk_assignment_id),  SC_PARAM(h_project_id),         SC_PARAM(h_task_id),
   									 SC_PARAM(h_res_id),               SC_PARAM(h_active),             SC_PARAM(h_active),
   									 SC_PARAM(h_last_submitted),           SC_PARAM(h_finish_date),        SC_PARAM(h_effort),
   									 SC_PARAM(h_work),                 SC_PARAM(h_eunit),              SC_PARAM(h_unit),
   									 SC_PARAM(h_res_type),             SC_PARAM(h_temp_work_type),     SC_PARAM(h_bitflag),
   									 SC_PARAM(h_contour),              SC_PARAM(h_temp_contour_type),  SC_PARAM(h_occurrence),
   									 SC_PARAM(h_temp_consider),        SC_PARAM(h_assign_string),      SC_PARAM(h_rec_user),
   									 SC_PARAM(h_per_day));
   
   				InitializeHost(SC_PARAM(h_temp_var), SC_BIT_30);
   				h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   			}
   			else
   			{
   				BEGIN_SQL
   					SELECT COUNT(*) INTO HOST(h_duration) FROM TASK_DAY_OF_YEAR
   					WHERE PROJECT_ID = HOST(h_project_id)
   						AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   						AND ACTUAL_FLAG = 0;
   				END_SQL_NOCHECK;
   
   				if ( h_duration > 0 )
   				{
   					InitializeHost(SC_PARAM(h_tph_delete), -1);
   					h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   
   					h_minutes = int ( ((double) h_remaining / (double) h_duration) + 0.499);
   					h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
   											  SC_PARAM(h_tsk_assignment_id),
   											  SC_PARAM(h_last_submitted),
   											  SC_PARAM(h_per_day),
   											  SC_PARAM(h_minutes));
   
   					InitializeHost(SC_PARAM(h_tph_delete), 0);
   					h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
   							SC_PARAM(h_project_id),
   							SC_PARAM(h_task_id),
   							SC_PARAM(h_tsk_assignment_id),
   							SC_PARAM(h_SD),
   							SC_PARAM(h_FD),
   							SC_PARAM(h_reg_cost),
   							SC_PARAM(h_reg_sale),
   							SC_PARAM(h_tph_delete),
   							SC_PARAM(h_capital_expense),
   							SC_PARAM(h_billable));
   
   					h_percent_units   = ((double) h_minutes / (double) h_per_day) * 10000.00;
   
   					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30);
   					h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day,h_remaining);
   				}
   			}
   		}
   
   		return 0;
   	}*/
   If Coalesce(h_tph_delete, 0) = 0 Then
      h_tph_delete := -1;
      h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
   End If;
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assignment_id
                                   , h_tsk_assignment_id, h_last_submitted, h_last_day, 0);
   End If;
   
   If Coalesce(h_etc, 0) = 0 Then
      h_temp_var := 0;
      h_skip := FC_DEL_WITHACTUAL(h_project_id, h_tsk_assignment_id, h_start_date, h_effort, h_work
                            , h_prj_assignment_id, h_temp_var);
      
      --Update SUMMARY_TIMESHEETS
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = 10000,
         FORECAST_START    = NULL,
         FORECAST_FINISH   = NULL,
         FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 970+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(971) : cplan_fun.sql(16589) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- get week of
      h_day_year := NULL;
      BEGIN
         SELECT MAX(WEEK_OF) INTO h_day_year 
         FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
         AND WEEK_OF <= TO_DATE(SubStr(h_start_date, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 988+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(989) : cplan_fun.sql(16607) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_day_year IS NOT NULL) Then
         --Update SUMMARY_TIMESHEETS
         BEGIN
            UPDATE SUMMARY_TIMESHEETS SET 
            ETC = 0, 
            EV_ETC = 0, 
            MILESTONE_PERCENT = 10000, 
            PDA_FLAG = 'P',
            TOTAL_SCHED_EFFORT = h_work_todate,
            EAC = ACWP,
            BASELINE_WORK_TODATE = h_work_todate,
            BCWP = BAC,
            REC_USER = h_rec_user, 
            REC_DATETIME = SYSTIMESTAMP
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
            AND WEEK_OF >= h_day_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1019+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1020) : cplan_fun.sql(16638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_temp_var := 1;
      -- it was 3
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
      
      If Coalesce(h_tph_delete, 0) = -1 Then
         h_tph_delete := 0;
         h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
      End If;
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1037+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1038) : cplan_fun.sql(16656) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If Coalesce(h_bitflag, 0) >= 90 Then
      If (BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0000007F)) <> 0 Then
         h_bitflag := 2;
      Else
         h_bitflag := 1;
      End If;
   End If;
   
   If (Coalesce(h_work_todate, 0) = 0) AND (Coalesce(h_type, 0) = 0) Then
      -- Remove all actual, go to original state (BAK_ASSIGN_STRING)
      h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
      
      BEGIN
         SELECT BAK_START_DT, BAK_FINISH_DT, BAK_WORK, BAK_ASSIGN_STRING
         INTO h_start_date, h_finish_date, h_effort, h_assign_string
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1064+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1065) : cplan_fun.sql(16683) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If ((h_assign_string IS NOT NULL)) AND (Coalesce(Length(h_assign_string), 0) > 0) Then
         If (h_prj_assignment_id IS NOT NULL) Then
            h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_29_30));
         Else
            h_bitflag := (PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_28));
         End If;
         
         -- Update TASK_ASSIGNMENTS  TASK_DAY_OF_YEAR with the original schedule
         PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_active
                               , h_active, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_res_type, h_work_type, h_bitflag, h_contour, h_bitflag
                               , h_zero, h_consider, h_assign_string, h_rec_user, h_per_day, 0.00
                               , 0.00);
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_resched := 1;
            h_tph_delete := 0;
         End If;
         
         h_start_date := NULL;
      End If;
   ElsIf (PMO_ORASTD.strncmp(h_start_date, h_SD, 10) < 0) OR (PMO_ORASTD.strncmp(h_start_date, h_FD
         , 10) > 0) OR (Coalesce(h_contour, 0) > 0) Then
      If (h_prj_assignment_id IS NOT NULL) Then
         h_temp_var := 2;
      Else
         h_temp_var := 1;
      End If;
      h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, Coalesce(h_temp_var, 0));
      
      If PMO_ORASTD.strncmp(h_start_date, h_last_submitted, 10) = 0 Then
         h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
      End If;
      h_temp_var := 2;
      h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_res_id, 0, Coalesce(h_percent_units, 0));
      If Coalesce(h_temp_sc_state, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1106+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1107) : cplan_fun.sql(16725) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN Coalesce(h_temp_sc_state, 0);
      End If;
   Else
      -- Reschedule: remove or add effort from the end
      h_sched_work := 0;
      BEGIN
         SELECT SUM(SCHED_MIN) INTO h_sched_work 
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         AND ACTUAL_FLAG = 0;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ETC_SCHEDULE', 1125+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ETC_SCHEDULE(1126) : cplan_fun.sql(16744) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_effort, 0) > 0 Then
         h_effort := Coalesce(h_effort, 0) - Coalesce(h_sched_work, 0);
         h_temp_sc_state := FC_REMOVE_WITHACTUAL(h_project_id, h_prj_assignment_id, h_tsk_assignment_id
                                        , h_last_submitted, h_effort, h_rec_user, h_contour
                                        , Coalesce(h_percent_units, 0));
         
         If Coalesce(h_temp_sc_state, 0) = 1 Then
            h_tph_delete := 0;
         ElsIf Coalesce(h_temp_sc_state, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1140+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1141) : cplan_fun.sql(16759) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN Coalesce(h_temp_sc_state, 0);
         End If;
         
         If Coalesce(h_tph_delete, 0) = -1 Then
            h_tph_delete := 0;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
         End If;
         
         h_temp_var := 1;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ETC_SCHEDULE', 1158+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ETC_SCHEDULE(1159) : cplan_fun.sql(16777) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ETC_SCHEDULE;
ALTER FUNCTION FC_ETC_SCHEDULE COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_D_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_D_ACTUAL
(
     sh_h_table_type PLS_INTEGER
   , sh_h_type_id PLS_INTEGER
   , h_assign_id IN OUT NOCOPY VARCHAR2
   , h_tsk_assignment_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_tsk_prj PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_profile_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , sh_h_donot_replace PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_bitflag1 PLS_INTEGER
   , sh_h_min_per_day PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_tsk_prj PLS_INTEGER := sh_h_tsk_prj;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_donot_replace PLS_INTEGER := sh_h_donot_replace;
   h_effort PLS_INTEGER := sh_h_effort;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_work PLS_INTEGER := sh_h_work;
   h_unit PLS_INTEGER := sh_h_unit;
   h_bitflag1 PLS_INTEGER := sh_h_bitflag1;
   h_min_per_day PLS_INTEGER := sh_h_min_per_day;
   h_priority PLS_INTEGER := sh_h_priority;
   h_contour PLS_INTEGER := sh_h_contour;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_uniqueID VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_day_year DATE;
   h_dayofyear DATE;
   h_week_of DATE;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_res_type VARCHAR2(1);
   h_prj_assignment_id VARCHAR2(32);
   h_duration PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_temp_every PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mls_percent PLS_INTEGER := 0;
   h_reg_sale NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_cost NUMBER := 0;
   h_temp_cri PLS_INTEGER := 0;
   h_temp_consider PLS_INTEGER := 0;
   h_temp_work_type PLS_INTEGER := 0;
   h_temp_update PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_pa_units PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   i_return PLS_INTEGER NOT NULL := 0;
   i PLS_INTEGER;
   j PLS_INTEGER;
   ii PLS_INTEGER;
   i_decommission PLS_INTEGER NOT NULL := 0;
   i_continue PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CWOF IS
         SELECT * FROM (SELECT WEEK_OF FROM SUMMARY_TIMESHEETS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF <= h_day_year
         ORDER BY WEEK_OF DESC ) WHERE ROWNUM<=1;

      CURSOR CCALS IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))
         FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_project_id AND DAY_YEAR > h_day_year
         ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C_DACT1 IS
         SELECT ta.DAY_YEAR, ta.SCHED_MIN, ta.BIT_SCHED
         FROM TASK_DAY_OF_YEAR ta JOIN TASK_DAY_OF_YEAR pa ON ta.PROJECT_ID = pa.PROJECT_ID AND ta.PARENT_ID = pa.TSK_ASSIGNMENT_ID
         WHERE ta.PROJECT_ID = h_project_id AND ta.TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND ta.DAY_YEAR > h_day_year;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_D_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_D_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := FC_D_ACTUAL(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_assign_id || ''''
          || ', ' || '''' || h_tsk_assignment_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_tsk_prj), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_profile_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_donot_replace), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_min_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 126+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_D_ACTUAL(127) : cplan_fun.sql(16914) : ' || SYSTIMESTAMP);
   END IF; 

   h_temp_var := 1;
   h_mls_percent := 1;
   h_finish_DT := '';
   h_res_assignment_id := '';
   h_dayofyear := '';
   
   -- get last submitted date
   BEGIN
      SELECT PROJECT_ASSIGNMENT_ID, TO_DATE(SubStr(ACTUAL_START, 1, 10)), LAST_SUBMITTED, WORK_TODATE, MILESTONE_PERCENT,
      LAST_SCHED_MIN, PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE, UNITS_MSP
      INTO h_prj_assignment_id, h_dayofyear, h_day_year,h_work_todate, h_mls_percent,
      h_last_day, h_prj_assign_id, h_billable, h_capital_expense, h_percent_units
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 148+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(149) : cplan_fun.sql(16936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 155+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(156) : cplan_fun.sql(16943) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_DB2M.SC_STATE;
   End If;
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_27 Then
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_27));
      i_decommission := 1;
      -- keep assignments with actual and remove the remaining
   End If;
   
   --if ( (h_bitflag == 0) || (h_bitflag == 3) )
   --{
   -- Nullify(SC_PARAM(h_week_of));
   -- if (i_decommission == 0)
   -- {
   --  BEGIN_SQL
   --   SELECT MAX(WEEK_OF) INTO HOST(h_week_of)
   --   FROM TIMESHEET_APPROVAL_STATUS
   --   WHERE ELEMENT_ID = HOST(h_project_id) AND RESOURCE_ID = HOST(h_res_id)
   --   AND CURRENT_STATE IN (SC_APV_STATE_RM_APPROVED,SC_APV_STATE_PM_APPROVED);   //Tudor ???
   --  END_SQL;
   
   --  if ( IsNotNullParam(h_week_of))
   --  {
   --   InitializeHost(SC_PARAM(h_temp_var), 3);
   --   BEGIN_SQL
   --    SELECT DATE(HOST(h_week_of)) + 6 DAYS INTO HOST(h_week_of) FROM DUAL;
   --   END_SQL;
   
   --   if (strncmp(h_day_year, h_week_of, 10) < 0)
   --   {
   --    InitializeHost(SC_PARAM(h_week_of), h_day_year);
   --   }
   --  }
   --  else
   --  {
   --   InitializeHost(SC_PARAM(h_temp_var), 0);
   --  }
   -- }
   
   -- if ( h_donot_replace == 2 )
   -- {
   --  if ( h_bitflag == 0 )
   --  {
   --   FC_U_PA_WORK
   --   (
   --    SC_PARAM(h_project_id),
   --    SC_PARAM(h_prj_assignment_id),
   --    SC_PARAM(h_tsk_assignment_id)
   --   );
   --  }
   -- }
   
   -- if ( IsNotNullParam(h_week_of))
   -- {
   --  h_temp_update = -1;
   --  if ( h_bitflag == 0 )
   --  {
   --   h_temp_sc_state = FC_TPHASE_ACTUAL_ASSIGN(SC_PARAM(h_project_id), SC_PARAM(h_task_id),
   --    SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_day_year), SC_PARAM(h_week_of),
   --    SC_PARAM(h_temp_update), SC_PARAM(h_capital_expense), SC_PARAM(h_billable));
   
   --   BEGIN_SQL
   --    DELETE FROM SUMMARY_TIMESHEETS
   --    WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND WEEK_OF > HOST(h_week_of);
   --   END_SQL;
   --  }
   --  else
   --  {
   --   BEGIN_SQL
   --    DELETE FROM SUMMARY_TIMESHEETS
   --    WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND WEEK_OF > HOST(h_week_of);
   --   END_SQL;
   --  }
   
   --  BEGIN_SQL
   --   DELETE FROM TASK_DAY_OF_YEAR
   --   WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   --    AND DAY_YEAR > HOST(h_week_of);
   --  END_SQL;
   
   --  BEGIN_SQL
   --   UPDATE TASK_ASSIGNMENTS
   --   SET ETC = 0
   --   WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
   --  END_SQL;
   
   --  InitializeHost(SC_PARAM(h_temp_var), 3);
   --  h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var), h_min_per_day, 0);
   
   --  if (IsNotNullParam(h_prj_assign_id))
   --  {
   --   h_temp_sc_state = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_prj_assignment_id), SC_PARAM(h_temp_var), h_min_per_day, 0);
   --  }
   
   --  return TS_APPROVAL_EXISTS;
   -- }
   -- else
   -- {
   --  InitializeHost(SC_PARAM(h_week_of), h_day_year);
   --  SC_STATE = FC_GET_WEEKOF(SC_PARAM(h_week_of));
   --  h_temp_update = -1;
   --  if ( h_bitflag == 0 )
   --  {
   --   h_temp_sc_state = FC_TPHASE_ACTUAL_ASSIGN(SC_PARAM(h_project_id), SC_PARAM(h_task_id),
   --    SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_week_of), SC_PARAM(h_week_of),
   --    SC_PARAM(h_temp_update), SC_PARAM(h_capital_expense), SC_PARAM(h_billable));
   --
   --   if (i_decommission == 0)
   --   {
   --    BEGIN_SQL
   --     DELETE FROM TASK_ASSIGNMENTS
   --     WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
   --    END_SQL;
   
   --    BEGIN_SQL
   --     DELETE FROM TASK_DAY_OF_YEAR
   --     WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
   --    END_SQL;
   
   --    BEGIN_SQL
   --     DELETE FROM SUMMARY_TIMESHEETS WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
   --    END_SQL;
   --   }
   --   else
   --   {
   --    BEGIN_SQL
   --     DELETE FROM TASK_DAY_OF_YEAR
   --     WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   --      AND DAY_YEAR > HOST(h_day_year);
   --    END_SQL;
   --   }
   --  }
   --  else if (IsNotNullParam(h_day_year))
   --  {
   --   BEGIN_SQL
   --    DELETE FROM TASK_DAY_OF_YEAR
   --    WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id)
   --     AND DAY_YEAR > HOST(h_day_year);
   --   END_SQL;
   --  }
   -- }
   --}
   --else
   --{
   BEGIN
      SELECT PROF_NUMBER INTO h_temp_update FROM PROFILES
      WHERE PROJECT_ID = h_project_id AND PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 309+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(310) : cplan_fun.sql(17097) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      h_res_type := '1';
   Else
      h_res_type := '0';
   End If;
   
   h_temp_sc_state := 0;
   If Coalesce(h_donot_replace, 0) = 0 Then
      If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
         h_temp_security_id := 'SCPROJECTASSIGNMENT';
         h_temp_participant := PMO_CNST.SC_AFFECTED;
      Else
         h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
         h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
      End If;
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         If Coalesce(h_effort, 0) > 0 Then
            --Get a unique id
            h_uniqueID := SYS_GUID();
            h_temp_bitflag := 1;
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_profile_id, h_tsk_prj, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_STATUS, REC_DATETIME)
               SELECT h_uniqueID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               '1', h_profile_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, 'I',SYSTIMESTAMP
               FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 355+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(356) : cplan_fun.sql(17143) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 362+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(363) : cplan_fun.sql(17150) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_DB2M.SC_STATE;
            End If;
         End If;
      Else
         h_temp_sc_state := FC_MERGE_ASSIGN(h_bitflag, h_project_id, h_task_id, h_profile_id, h_assign_id
                                   , h_res_assignment_id, h_rec_user, h_effort, 1);
         
         If Coalesce(h_temp_sc_state, 0) = 200 Then
            --Get a unique id
            h_uniqueID := SYS_GUID();
            h_temp_bitflag := 1;
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_profile_id, h_tsk_prj, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_STATUS, REC_DATETIME)
               SELECT h_uniqueID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               '0', h_profile_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, 'I',SYSTIMESTAMP
               FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 394+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(395) : cplan_fun.sql(17182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 401+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(402) : cplan_fun.sql(17189) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_DB2M.SC_STATE;
            End If;
         Else
            i_continue := 1;
         End If;
      End If;
   End If;
   --}
   
   -- get week of
   
   OPEN CWOF;
   
   FETCH CWOF INTO h_week_of;
   IF CWOF%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   --trace_printf("SC_STATE: %d, h_week_of: %s, h_day_year: %s\n",SC_STATE,h_week_of,h_day_year);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      BEGIN
         CLOSE CWOF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 433+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(434) : cplan_fun.sql(17221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         CLOSE CWOF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 446+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(447) : cplan_fun.sql(17234) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Update SUMMARY_TIMESHEETS
      BEGIN
         UPDATE SUMMARY_TIMESHEETS SET ETC = 0, EV_ETC = 0, MILESTONE_PERCENT = 10000, PDA_FLAG = 'P',
         TOTAL_SCHED_EFFORT = h_work_todate,
         EAC = ACWP,
         BASELINE_WORK_TODATE = h_work_todate,
         BCWP = BAC,
         REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND WEEK_OF >= h_week_of;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 470+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(471) : cplan_fun.sql(17258) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*      BEGIN_SQL
      			UPDATE TASK_ASSIGNMENTS SET REC_STATUS = 'U', REC_USER = HOST(h_rec_user), REC_DATETIME = SC_NOW
      			WHERE RESOURCE_ID = HOST(h_res_id) AND PROJECT_ID = HOST(h_project_id);
      		END_SQL;*/
   End If;
   
   --trace_printf("*** i_continue: %d, h_day_year = %s\n", i_continue,h_day_year);
   If i_continue = 1 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 484+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(485) : cplan_fun.sql(17272) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_day_year IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_day_year, h_last_day, 0);
   End If;
   
   If (h_prj_assignment_id IS NOT NULL) Then
      If Coalesce(h_donot_replace, 0) = 0 Then
         null_return := FC_U_PA_WORK(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, Coalesce(
                            h_min_per_day, 0), 1);
      Else
         null_return := FC_U_PA_WORK(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, Coalesce(
                            h_min_per_day, 0), 0);
      End If;
   End If;
   
   BEGIN
      DELETE FROM TASK_DAY_OF_YEAR
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id 
      AND ACTUAL_FLAG = 0;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 519+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(520) : cplan_fun.sql(17307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("DELETE: SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
   
   -- get finish datetime
   BEGIN
      SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED)) INTO h_finish_date
      FROM TASK_DAY_OF_YEAR 
      WHERE PROJECT_ID = h_project_id 
      AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_day_year;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 537+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(538) : cplan_fun.sql(17325) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_day_year = %s, h_finish_date = %s\n", SC_STATE,h_day_year,h_finish_date);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET ETC = 0, FD = PMO_CNST.SC_AF, ACTUAL_FINISH = h_finish_date,
         MILESTONE_PERCENT = 10000, SCHED_WORK = WORK_TODATE, SCHED_FINISH = h_finish_date,
         PROPOSED_START = SCHED_START, PROPOSED_WORK = WORK_TODATE, PROPOSED_FINISH = h_finish_date,
         FORECAST_START = NULL, FORECAST_FINISH = NULL, FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 560+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(561) : cplan_fun.sql(17348) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s\n", SC_STATE,h_tsk_assignment_id);
      
      -- CALL UPDATE assign string function
      h_temp_var := 3;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(
                                   h_min_per_day, 0), 0);
   End If;
   
   If (SubStr(h_res_type, 1, 1) = '1') OR (Coalesce(h_donot_replace, 0) = 0) Then
      If (Coalesce(h_donot_replace, 0) = 0) AND (Coalesce(h_effort, 0) > 0) Then
         --BEGIN_SQL
         -- SELECT TASK_TYPE INTO HOST(h_task_type)
         -- FROM TMT_WBS
         -- WHERE PROJECT_ID = HOST(h_project_id) AND ELEMENT_ID = HOST(h_task_id);
         --END_SQL;
         
         -- get start datetime
         /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_last_day > 0) )
         			{
         				BEGIN_SQL
         					DECLARE CCAL0 CURSOR WITH HOLD FOR SELECT TIMESTAMP(DAY_YEAR, FIRST_AVAIL_TIME(BIT_SCHED))
         					FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = HOST(h_project_id) AND DAY_YEAR >= HOST(h_day_year)
         					ORDER BY DAY_YEAR FETCH FIRST 1 ROWS ONLY;
         				END_SQL_NOCHECK;
         
         				BEGIN_SQL
         					OPEN CCAL0 ;
         				END_SQL;
         
         				BEGIN_SQL
         					FETCH CCAL0 INTO HOST(h_start_DT) ;
         				END_SQL;
         
         				BEGIN_SQL
         					CLOSE CCAL0 WITH RELEASE;
         				END_SQL;
         			}
         			else
         			{*/
         
         OPEN CCALS;
         
         FETCH CCALS INTO h_start_DT;
         IF CCALS%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CCALS ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 622+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(623) : cplan_fun.sql(17410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --}
         
         h_temp_res_cost := 0.00;
         h_temp_cri := 0;
         
         h_temp_work_type := 0;
         h_temp_update := 0;
         h_per_week := 5 * Coalesce(h_min_per_day, 0);
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_temp_consider := 12;
         Else
            h_temp_consider := 0;
         End If;
         
         If Coalesce(h_unit, 0) = 0 AND Coalesce(h_work, 0) > Coalesce(h_min_per_day, 0) Then
            h_bitflag1 := 0;
            h_contour := 0;
            h_work := 0;
         ElsIf Coalesce(h_contour, 0) = 0 Then
            h_work := 0;
         End If;
         
         h_temp_sc_state := FC_CALC_DURATION(h_start_DT, h_finish_DT, 0, 0, Coalesce(h_temp_consider, 0)
                                    , Coalesce(h_bitflag1, 0), Coalesce(h_contour, 0), Coalesce(
                                    h_occurrence, 0), Coalesce(h_min_per_day, 0), Coalesce(
                                    h_per_week, 0), h_work, h_effort, h_duration);
         
         If Coalesce(h_temp_sc_state, 0) <> 0 Then
            h_duration := 1200;
         ElsIf (Coalesce(h_duration, 0) > 0) AND (Coalesce(h_percent_units, 0) > 0) Then
            h_duration := Trunc(Coalesce(h_duration, 0) * 10000.00 / Coalesce(h_percent_units, 0));
         End If;
         
         If Coalesce(h_work, 0) = 0 AND Coalesce(h_percent_units, 0) < 10000 Then
            h_work := Coalesce(h_percent_units, 0);
            h_unit := 100;
         End If;
         
         -- get resource selling rates
         BEGIN
            SELECT REGULAR_RATE, SPECIAL_RATE INTO h_reg_sale, h_spec_sale
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 675+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(676) : cplan_fun.sql(17463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tsk_assignment_id: %s, h_reg_sale = %.4f, h_spec_sale = %.4f\n",SC_STATE, h_tsk_assignment_id, h_reg_sale, h_spec_sale);
         
         -- update profile selling rates
         /*BEGIN_SQL
         				UPDATE TASK_ASSIGNMENTS SET REGULAR_RATE = HOST(h_reg_sale), SPECIAL_RATE = HOST(h_spec_sale)
         				WHERE RES_ASSIGNMENT_ID = HOST(h_uniqueID);
         			END_SQL;*/
         
         i_return := FC_SEARCH_2(h_uniqueID, h_project_id, h_task_id, h_profile_id, h_temp_res_cost
                        , h_tsk_prj, h_finish_DT, h_finish_DT, h_finish_DT, h_finish_DT, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_temp_cri
                        , h_temp_consider, h_res_type, h_temp_work_type, h_bitflag1, h_contour
                        , h_occurrence, h_temp_update, h_min_per_day, h_priority, h_rec_user, 0, 0
                        , Coalesce(h_reg_sale, 0), Coalesce(h_spec_sale, 0));
         --trace_printf("***: i_return = %d, h_donot_replace = %d, h_tsk_assignment_id = %s, h_day_year = %s\n", i_return,h_donot_replace,h_tsk_assignment_id,h_day_year);
         
         If i_return <> 0 AND Coalesce(h_work, 0) > 0 Then
            h_bitflag1 := 0;
            h_contour := 0;
            h_work := 0;
            h_unit := 0;
            i_return := FC_SEARCH_2(h_uniqueID, h_project_id, h_task_id, h_profile_id, h_temp_res_cost
                        , h_tsk_prj, h_finish_DT, h_finish_DT, h_finish_DT, h_finish_DT, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_temp_cri
                        , h_temp_consider, h_res_type, h_temp_work_type, h_bitflag1, h_contour
                        , h_occurrence, h_temp_update, h_min_per_day, h_priority, h_rec_user, 0, 0
                        , Coalesce(h_reg_sale, 0), Coalesce(h_spec_sale, 0));
         End If;
         
         PMO_DB2M.SC_STATE := i_return;
      Else
         If Coalesce(h_donot_replace, 0) = 2 Then
            If Coalesce(h_bitflag, 0) = 3 Then
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR > h_day_year;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 725+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(726) : cplan_fun.sql(17513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               OPEN C_DACT1;
               
               LOOP
                  FETCH C_DACT1 INTO h_dayofyear,h_temp_var,h_bitID;
                  IF C_DACT1%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                     EXIT;
                  End If;
                  --trace_printf("h_dayofyear: %s, h_temp_var: %d\n", h_dayofyear,h_temp_var);
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_temp_var, BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 757+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(758) : cplan_fun.sql(17545) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 775+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(776) : cplan_fun.sql(17563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE C_DACT1 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 789+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(790) : cplan_fun.sql(17577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = PARENT_ID
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id AND DAY_YEAR > h_day_year;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_D_ACTUAL', 807+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_D_ACTUAL(808) : cplan_fun.sql(17595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   End If;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_D_ACTUAL', 821+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_D_ACTUAL(822) : cplan_fun.sql(17609) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_D_ACTUAL;
ALTER FUNCTION FC_D_ACTUAL COMPILE DEBUG;

/**********************************************************/

Prompt == FUNCTION FC_U_RATES ==
CREATE OR REPLACE FUNCTION FC_U_RATES
(
     h_id IN OUT NOCOPY VARCHAR2
   , sh_h_reg_cost NUMBER
   , sh_h_spec_cost NUMBER
   , sh_h_reg_rate NUMBER
   , sh_h_spec_rate NUMBER
   , sh_h_rate_adjustment PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , sh_h_flag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_reg_cost NUMBER := sh_h_reg_cost;
   h_spec_cost NUMBER := sh_h_spec_cost;
   h_reg_rate NUMBER := sh_h_reg_rate;
   h_spec_rate NUMBER := sh_h_spec_rate;
   h_rate_adjustment PLS_INTEGER := sh_h_rate_adjustment;
   h_flag PLS_INTEGER := sh_h_flag;
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_pa_timephase VARCHAR2(1);
   h_planned_effort PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_rates_edited PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_old_reg_cost NUMBER := 0;
   h_old_reg_sale NUMBER := 0;
   h_old_spec_cost NUMBER := 0;
   h_old_spec_rate NUMBER := 0;
   i_pa PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_RATES'), '-- ' || SYSTIMESTAMP, ':r := FC_U_RATES(' || 
       '''' || h_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_reg_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_spec_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_rate_adjustment), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_flag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_RATES', 50+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_RATES(51) : cplan_fun.sql(17670) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT PROJECT_ID, TASK_ID, RESOURCE_ID,
      PLANNED_WORK, SCHED_WORK, TO_DATE(SubStr(PROPOSED_START, 1, 10)), TO_DATE(SubStr(PROPOSED_FINISH, 1, 10)),
      BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE,
      SPECIAL_COST, SPECIAL_RATE, RATES_EDITED
      INTO   h_project_id, h_task_id, h_resource_id,
      h_planned_effort, h_sched_effort, h_SD, h_FD,
      h_billable, h_capital_expense, h_old_reg_cost, h_old_reg_sale ,
      h_old_spec_cost,h_old_spec_rate,h_rates_edited
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID=h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 69+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(70) : cplan_fun.sql(17689) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If Coalesce(h_flag, 0) > 0 Then
         If (Coalesce(h_old_reg_cost, 0) <> Coalesce(h_reg_cost, 0)) OR (Coalesce(
                                            h_old_spec_cost, 0) <> Coalesce(h_spec_cost, 0)) Then
            h_rates_edited := (PMO_FUNC.BIT_OR(Coalesce(h_rates_edited, 0), 1));
         End If;
         If (Coalesce(h_old_reg_sale, 0) <> Coalesce(h_reg_rate, 0)) OR (Coalesce(
                                            h_old_spec_rate, 0) <> Coalesce(h_spec_rate, 0)) Then
            h_rates_edited := (PMO_FUNC.BIT_OR(Coalesce(h_rates_edited, 0), 2));
         End If;
      End If;
      
      h_tph_delete := -1;
      i_pa := PMO_ORASTD.strcmp(h_project_id, h_task_id);
      --trace_printf("0_* i_pa: %d:, h_task_id: %s\n", i_pa,h_task_id);
      If i_pa = 0 Then
         BEGIN
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 98+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(99) : cplan_fun.sql(17718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);
         
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD
                                            , h_old_reg_cost, h_old_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD
                                            , h_old_reg_cost, h_old_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
      End If;
      
      --Execute update Statement
      If Coalesce(h_billable, 0) = 0 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET REGULAR_COST = h_reg_cost,
            SPECIAL_COST = h_spec_cost,
            REGULAR_RATE = h_reg_rate,
            SPECIAL_RATE = h_spec_rate,
            RATE_ADJUSTMENT = h_rate_adjustment,
            RATES_EDITED = h_rates_edited,
            SCHED_LABOR_COST = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            PLANNED_LABOR_COST = (PLANNED_WORK/60.00) * h_reg_cost,
            FORECAST_LABOR_COST = (ETC/60.00) * h_reg_cost + ACTUAL_LABOR_COST
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 139+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(140) : cplan_fun.sql(17759) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET REGULAR_COST = h_reg_cost,
            SPECIAL_COST = h_spec_cost,
            REGULAR_RATE = h_reg_rate,
            SPECIAL_RATE = h_spec_rate,
            RATE_ADJUSTMENT = h_rate_adjustment,
            RATES_EDITED = h_rates_edited,
            SCHED_LABOR_COST = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            PLANNED_LABOR_COST = (PLANNED_WORK/60.00) * h_reg_cost,
            SCHED_LABOR_REVENUE = ((SCHED_WORK-WORK_TODATE)/60.00) * h_reg_rate + ACTUAL_LABOR_REVENUE,
            PLANNED_LABOR_REVENUE = (PLANNED_WORK/60.00) * h_reg_rate,
            FORECAST_LABOR_COST = (ETC/60.00) * h_reg_cost + ACTUAL_LABOR_COST,
            FORECAST_LABOR_REVENUE = (ETC/60.00) * h_reg_rate + ACTUAL_LABOR_REVENUE
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(170) : cplan_fun.sql(17789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("2_* i_pa: %d, h_pa_timephase: %s\n", i_pa,h_pa_timephase);
      
      h_tph_delete := 0;
      If i_pa = 0 Then
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                            , h_reg_rate, h_tph_delete, h_capital_expense
                                            , h_billable);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                            , h_reg_rate, h_tph_delete, h_capital_expense
                                            , h_billable);
      End If;
      
      If i_pa = 0 Then
         If Coalesce(h_billable, 0) = 0 Then
            If Coalesce(h_planned_effort, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_SCHED_COST = (h_sched_effort/60.00) * h_reg_cost
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 206+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(207) : cplan_fun.sql(17826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_COST = (h_planned_effort/60.00) * h_reg_cost
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 225+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(226) : cplan_fun.sql(17845) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If Coalesce(h_planned_effort, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_SCHED_COST = (h_sched_effort/60.00) * h_reg_cost,
                  ESTIMATED_SCHED_REVENUE = (h_sched_effort/60.00) * h_reg_rate
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 247+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(248) : cplan_fun.sql(17867) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_COST = (h_planned_effort/60.00) * h_reg_cost,
                  ESTIMATED_PLANNED_REVENUE = (h_planned_effort/60.00) * h_reg_rate
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 267+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(268) : cplan_fun.sql(17887) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      --trace_printf("SC_STATE: %d, h_planned_effort: %d, h_old_reg_cost: %.2f\n", SC_STATE,h_planned_effort,h_old_reg_cost);
      --trace_printf("h_project_id: %s, h_resource_id: %s\n", h_project_id,h_resource_id);
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASSIGNMENT_BIT) WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 289+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(290) : cplan_fun.sql(17909) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Rollup Dates
      h_temp_sc_state := FC_Rollup(h_project_id);
      -- New Rollup
   Else
      --Execute update Statement
      BEGIN
         UPDATE PROJECT_ASSIGNMENTS
         SET REGULAR_COST = h_reg_cost,
         SPECIAL_COST = h_spec_cost,
         REGULAR_RATE = h_reg_rate,
         SPECIAL_RATE = h_spec_rate
         WHERE PROJECT_ID = h_res_id AND RESOURCE_ID = h_id; -- Resource ID (h_id) is element id (unique) in the broker;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_RATES', 316+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_RATES(317) : cplan_fun.sql(17936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_RATES', 323+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_RATES(324) : cplan_fun.sql(17943) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_RATES;
ALTER FUNCTION FC_U_RATES COMPILE DEBUG;

