/************************************************************/

Prompt == FUNCTION FC_CP_CHARGE_CODE ==
CREATE OR REPLACE FUNCTION FC_CP_CHARGE_CODE
(
     sh_h_bitflag PLS_INTEGER
   , h_source_id IN OUT NOCOPY VARCHAR2
   , h_target_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_chcd_billable PLS_INTEGER := 0;
   h_chcd_child_count PLS_INTEGER := 0;
   h_chcd_rec_datetime TIMESTAMP;
   h_chcd_default_flag VARCHAR2(1);
   h_chcd_description VARCHAR2(2048);
   h_chcd_deleted_count PLS_INTEGER := 0;
   h_chcd_element_id VARCHAR2(32);
   h_chcd_element_name VARCHAR2(255);
   h_chcd_level_id PLS_INTEGER := 0;
   h_chcd_max_rank PLS_INTEGER := 0;
   h_chcd_parent_id VARCHAR2(32);
   h_chcd_parent_rank VARCHAR2(200);
   h_chcd_rank VARCHAR2(200);
   h_chcd_rec_status VARCHAR2(1);
   h_chcd_type_id PLS_INTEGER := 0;
   h_chcd_longBase64 VARCHAR2(200);
   h_current_datetime TIMESTAMP;
-- Declare Cursors
      CURSOR chargeCodeRecset IS
         SELECT child.RANK,           parent.RANK AS PARENT_RANK,    child.LEVEL_ID,
         child.TYPE_ID,        child.MAX_RANK,                child.CHILD_COUNT,
         child.DELETED_COUNT,  child.ELEMENT_ID,              child.ELEMENT_NAME,
         child.PARENT_ID,      child.REC_STATUS,              child.REC_DATETIME,
         child.BILLABLE,      child.DESCRIPTION
         FROM TMT_CHARGECODES child, TMT_CHARGECODES parent
         WHERE child.ELEMENT_ID = h_source_id AND
         parent.ELEMENT_ID = child.PARENT_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CP_CHARGE_CODE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CP_CHARGE_CODE'), '-- ' || SYSTIMESTAMP, ':r := FC_CP_CHARGE_CODE(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_source_id || ''''
          || ', ' || '''' || h_target_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CP_CHARGE_CODE', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CP_CHARGE_CODE(48) : move_wbs.sql(51) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT SYSTIMESTAMP INTO h_current_datetime FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_CHARGE_CODE', 58+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_CHARGE_CODE(59) : move_wbs.sql(62) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      OPEN chargeCodeRecset;
      
      LOOP
         FETCH chargeCodeRecset INTO
         h_chcd_rank,           h_chcd_parent_rank,  h_chcd_level_id,
         h_chcd_type_id,        h_chcd_max_rank,     h_chcd_child_count,
         h_chcd_deleted_count,  h_chcd_element_id,   h_chcd_element_name,
         h_chcd_parent_id,      h_chcd_rec_status,   h_chcd_rec_datetime,
         h_chcd_billable,       h_chcd_default_flag;
         IF chargeCodeRecset%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_CHARGECODES', h_chcd_parent_id, h_chcd_max_rank);
         PMO_STPU.toBase64(Coalesce(h_chcd_max_rank, 0), h_chcd_longBase64);
         
         BEGIN
            INSERT INTO TMT_CHARGECODES (
            RANK,           LEVEL_ID,       TYPE_ID,    MAX_RANK,
            CHILD_COUNT,    DELETED_COUNT,  ELEMENT_ID, ELEMENT_NAME,
            PARENT_ID,      REC_USER,       REC_STATUS, REC_DATETIME,
            BILLABLE,   DESCRIPTION )
            VALUES(
            h_chcd_parent_rank || h_chcd_longBase64,   h_chcd_level_id,      h_chcd_type_id,
            h_chcd_max_rank,     h_chcd_child_count,   h_chcd_deleted_count,
            h_target_id,         h_chcd_element_name,  h_chcd_parent_id,
            h_rec_user,          h_chcd_rec_status,    h_current_datetime,
            h_chcd_billable,     h_chcd_default_flag );
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_CHARGE_CODE', 108+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_CHARGE_CODE(109) : move_wbs.sql(112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE chargeCodeRecset ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_CHARGE_CODE', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_CHARGE_CODE(123) : move_wbs.sql(126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CP_CHARGE_CODE', 129+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CP_CHARGE_CODE(130) : move_wbs.sql(133) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CP_CHARGE_CODE;
ALTER FUNCTION FC_CP_CHARGE_CODE COMPILE DEBUG;

/************************************************************/

Prompt == FUNCTION FC_CP_TIMES_CODE ==
CREATE OR REPLACE FUNCTION FC_CP_TIMES_CODE
(
     sh_h_bitflag PLS_INTEGER
   , h_source_id IN OUT NOCOPY VARCHAR2
   , h_target_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_tmcd_fs_code VARCHAR2(1);
   h_tmcd_child_count PLS_INTEGER := 0;
   h_tmcd_rec_datetime TIMESTAMP;
   h_tmcd_default_flag VARCHAR2(1);
   h_tmcd_description VARCHAR2(2048);
   h_tmcd_deleted_count PLS_INTEGER := 0;
   h_tmcd_element_id VARCHAR2(32);
   h_tmcd_element_name VARCHAR2(255);
   h_tmcd_level_id PLS_INTEGER := 0;
   h_tmcd_max_rank PLS_INTEGER := 0;
   h_tmcd_parent_id VARCHAR2(32);
   h_tmcd_parent_rank VARCHAR2(200);
   h_tmcd_rank VARCHAR2(200);
   h_tmcd_rec_status VARCHAR2(1);
   h_tmcd_type_id PLS_INTEGER := 0;
   h_tmcd_longBase64 VARCHAR2(200);
   h_current_datetime TIMESTAMP;
-- Declare Cursors
      CURSOR timesCodeRecset IS
         SELECT child.RANK,           parent.RANK AS PARENT_RANK,    child.LEVEL_ID,
         child.TYPE_ID,        child.MAX_RANK,                child.CHILD_COUNT,
         child.DELETED_COUNT,  child.ELEMENT_ID,              child.ELEMENT_NAME,
         child.PARENT_ID,      child.REC_STATUS,              child.REC_DATETIME,
         child.FS_CODE,        child.DEFAULT_FLAG,            child.DESCRIPTION
         FROM TMT_TIMECODES child, TMT_TIMECODES parent
         WHERE child.ELEMENT_ID = h_source_id AND
         parent.ELEMENT_ID = child.PARENT_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CP_TIMES_CODE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CP_TIMES_CODE'), '-- ' || SYSTIMESTAMP, ':r := FC_CP_TIMES_CODE(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_source_id || ''''
          || ', ' || '''' || h_target_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CP_TIMES_CODE', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CP_TIMES_CODE(48) : move_wbs.sql(191) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT (SYSTIMESTAMP)  INTO  h_current_datetime FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_TIMES_CODE', 58+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_TIMES_CODE(59) : move_wbs.sql(202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      OPEN timesCodeRecset;
      
      LOOP
         FETCH timesCodeRecset INTO
         h_tmcd_rank,           h_tmcd_parent_rank,  h_tmcd_level_id,
         h_tmcd_type_id,        h_tmcd_max_rank,     h_tmcd_child_count,
         h_tmcd_deleted_count,  h_tmcd_element_id,   h_tmcd_element_name,
         h_tmcd_parent_id,      h_tmcd_rec_status,   h_tmcd_rec_datetime,
         h_tmcd_fs_code,        h_tmcd_default_flag, h_tmcd_default_flag;
         IF timesCodeRecset%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_TIMECODES', h_tmcd_parent_id, h_tmcd_max_rank);
         PMO_STPU.toBase64(Coalesce(h_tmcd_max_rank, 0), h_tmcd_longBase64);
         
         BEGIN
            INSERT INTO TMT_TIMECODES (
            RANK,           LEVEL_ID,       TYPE_ID,    MAX_RANK,
            CHILD_COUNT,    DELETED_COUNT,  ELEMENT_ID, ELEMENT_NAME,
            PARENT_ID,      REC_USER,       REC_STATUS, REC_DATETIME,
            FS_CODE,        DEFAULT_FLAG,   DESCRIPTION )
            VALUES(
            h_tmcd_parent_rank || h_tmcd_longBase64,   h_tmcd_level_id,      h_tmcd_type_id,
            h_tmcd_max_rank,     h_tmcd_child_count,   h_tmcd_deleted_count,
            h_target_id,         h_tmcd_element_name,  h_tmcd_parent_id,
            h_rec_user,          h_tmcd_rec_status,    h_current_datetime,
            h_tmcd_fs_code,     h_tmcd_default_flag,  h_tmcd_default_flag );
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_TIMES_CODE', 108+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_TIMES_CODE(109) : move_wbs.sql(252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE timesCodeRecset ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CP_TIMES_CODE', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CP_TIMES_CODE(123) : move_wbs.sql(266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CP_TIMES_CODE', 129+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CP_TIMES_CODE(130) : move_wbs.sql(273) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CP_TIMES_CODE;
ALTER FUNCTION FC_CP_TIMES_CODE COMPILE DEBUG;

/*****************************************************************************************/

-- load project calendar to a temporary table if it wasn't loaded

/*****************************************************************************************/

Prompt == FUNCTION FC_C_TMP_TABLES ==
CREATE OR REPLACE FUNCTION FC_C_TMP_TABLES RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_TMP_TABLES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_TMP_TABLES'), '-- ' || SYSTIMESTAMP, ':r := FC_C_TMP_TABLES();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_TMP_TABLES', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_TMP_TABLES(9) : move_wbs.sql(296) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_WBS';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_WBS_DEPEND';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CRI_DEPND';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_CRI';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_TMP_TABLES', 28+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_TMP_TABLES(29) : move_wbs.sql(316) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_TMP_TABLES;
ALTER FUNCTION FC_C_TMP_TABLES COMPILE DEBUG;

/*****************************************************************************************/

-- load project calendar to a temporary table if it wasn't loaded

/*****************************************************************************************/

Prompt == FUNCTION FC_LoadProjectCalendar ==
CREATE OR REPLACE FUNCTION FC_LoadProjectCalendar
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_dayofyear IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_min_per_day PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_LoadProjectCalendar') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_LoadProjectCalendar'), '-- ' || SYSTIMESTAMP, ':r := FC_LoadProjectCalendar(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_dayofyear || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LoadProjectCalendar', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_LoadProjectCalendar(16) : move_wbs.sql(346) : ' || SYSTIMESTAMP);
   END IF; 

   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MIN_MAX';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CDY';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      INSERT INTO PMO_TMP_CDY
      SELECT ELEMENT_ID, DAY_YEAR, DAY_WEEK, AVAILABLE_MIN, BIT_SCHED, WEEK_OF, MONTH_OF, YEAR_OF
      FROM CALENDAR_DAY_OF_YEAR
      WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= h_dayofyear;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LoadProjectCalendar', 43+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LoadProjectCalendar(44) : move_wbs.sql(374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT MINUTE_PER_DAY INTO h_min_per_day
      FROM TMT_WBS
      WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LoadProjectCalendar', 58+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LoadProjectCalendar(59) : move_wbs.sql(389) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_TMP_MIN_MAX
      SELECT MIN(DAY_YEAR), MAX(DAY_YEAR), h_min_per_day, TO_TIMESTAMP('1899-12-30 8:00:00'),TO_TIMESTAMP('1899-12-30 8:00:00'), 0
      FROM PMO_TMP_CDY;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_LoadProjectCalendar', 77+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_LoadProjectCalendar(78) : move_wbs.sql(408) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_LoadProjectCalendar', 83+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_LoadProjectCalendar(84) : move_wbs.sql(414) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_LoadProjectCalendar;
ALTER FUNCTION FC_LoadProjectCalendar COMPILE DEBUG;

/****************************************************************************************************
Copy PARTICIPANTS and RESOPNSIBLE
RESOPNSIBLE: if RESOPNSIBLE == h_created_by then new RESOPNSIBLE == h_rec_user
             if RESOPNSIBLE != h_created_by then new RESOPNSIBLE == old RESOPNSIBLE (h_created_by)
****************************************************************************************************/

Prompt == FUNCTION FC_RESPONSIBLE ==
CREATE OR REPLACE FUNCTION FC_RESPONSIBLE
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_source_project_id IN OUT NOCOPY VARCHAR2
   , h_cur_id IN OUT NOCOPY VARCHAR2
   , sh_h_type PLS_INTEGER
   , h_creator_name IN OUT NOCOPY VARCHAR2
   , h_created_by IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_ass_bitflag PLS_INTEGER
   , h_assign_string IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type PLS_INTEGER := sh_h_type;
   h_ass_bitflag PLS_INTEGER := sh_h_ass_bitflag;
   h_full_name VARCHAR2(255);
   h_res_type VARCHAR2(1);
   h_res_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_participant PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_continue PLS_INTEGER := 0;
   h_source_pm_id VARCHAR2(32);
   h_temp_assig_string VARCHAR2(32);
   h_count PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C2P IS
         SELECT RES_ASSIGNMENT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, CRI_SECURITY_ID
         FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID = h_source_project_id
         AND ELEMENT_ID = h_cur_id
         AND RESOURCE_TYPE = '0'
         ORDER BY PARTICIPANT;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_RESPONSIBLE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_RESPONSIBLE'), '-- ' || SYSTIMESTAMP, ':r := FC_RESPONSIBLE(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_source_project_id || ''''
          || ', ' || '''' || h_cur_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_creator_name || ''''
          || ', ' || '''' || h_created_by || ''''
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_ass_bitflag), 'NULL')
          || ', ' || '''' || h_assign_string || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESPONSIBLE', 55+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_RESPONSIBLE(56) : move_wbs.sql(484) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("h_ass_bitflag: %d, h_type: %d, h_assign_string: %s\n",h_ass_bitflag,h_type,h_assign_string);
   
   --trace_printf("project_id: %d, %s; resource_id: %d, %s\n", h_source_project_id_Ind, h_source_project_id, h_cur_id_Ind, h_cur_id);
   
   BEGIN
      SELECT RESOURCE_ID INTO h_source_pm_id
      FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = h_source_project_id
      AND SECURITY_ID = 'SCPROJECTMANAGER';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 73+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(74) : move_wbs.sql(502) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_res_type := '0';
   h_count := 0;
   
   OPEN C2P;
   
   LOOP
      FETCH C2P INTO h_uniqueID, h_res_id, h_participant, h_security_id, h_cri_security_id;
      IF C2P%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      --      h_participant = h_participant  SC_CLEAR_TEAMMEMBER;
      --trace_printf("h_participant: %d, h_security_id: %s\n",h_participant,h_security_id);
      --trace_printf("rec_user: %d, %s\n", h_rec_user_Ind, h_rec_user);
      --trace_printf("created_by: %d, %s\n", h_created_by_Ind, h_created_by);
      --trace_printf("resource_id: %d, %s\n", h_res_id_Ind, h_res_id);
      
      h_continue := 1;
      If ((BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_AFFECTED)) <> PMO_CNST.SC_AFFECTED) 
                     AND ((BITAND(Coalesce(
                     h_participant, 0), PMO_CNST.SC_TEAMMEMBER)) <> PMO_CNST.SC_TEAMMEMBER) Then
         If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_MANAGER_RESPONSIBLE)) = 0 Then
            If Coalesce(h_ass_bitflag, 0) = 1 Then
               If (PMO_ORASTD.strcmp(h_res_id, h_source_pm_id) = PMO_CNST.SC_MATCH) OR (PMO_ORASTD.strcmp(h_res_id
                     , h_rec_user) = PMO_CNST.SC_MATCH) Then
                  BEGIN
                     SELECT RES_ASSIGNMENT_ID INTO h_uniqueID
                     FROM RESOURCE_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id
                     AND RESOURCE_ID = h_rec_user
                     AND ELEMENT_ID = h_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 121+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(122) : move_wbs.sql(550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                     h_continue := 0;
                  Else
                     h_res_id := h_rec_user;
                  End If;
               End If;
            Else
               h_continue := 0;
               NULL;
            End If;
         Else
            If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_MANAGER)) = PMO_CNST.SC_MANAGER Then
               BEGIN
                  SELECT NICK_NAME || ';' INTO h_full_name
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_rec_user;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 148+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(149) : move_wbs.sql(577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_assign_string := h_full_name;
               h_continue := 0;
               h_count := 1;
            ElsIf (isProjectType(Coalesce(h_type, 0))) <> 0 AND (PMO_ORASTD.strcmp(h_res_id
                  , h_source_pm_id) = PMO_CNST.SC_MATCH) Then
               h_continue := 0;
            Else
               If PMO_ORASTD.strcmp(h_res_id, h_rec_user) <> PMO_CNST.SC_MATCH Then
                  h_res_id := h_rec_user;
               End If;
               h_continue := 1;
               h_count := 1;
            End If;
         End If;
         --trace_printf("h_continue: %d, h_res_id: %s, h_full_name: %s\n",h_continue,h_res_id,h_full_name);
         If Coalesce(h_continue, 0) > 0 Then
            BEGIN
               SELECT NICK_NAME || ';' INTO h_full_name
               FROM RESOURCES
               WHERE RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 179+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(180) : move_wbs.sql(608) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_assign_string := SubStr(h_assign_string || h_full_name, 1, 4000);
            h_temp_var := 1;
            h_temp_var := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_participant, h_security_id
                                     , h_res_type, h_temp_var);
            
            h_table_type := GetTable(Coalesce(h_type, 0));
            h_uniqueID := SYS_GUID();
            
            If isCRIConvertTask(Coalesce(h_type, 0)) <> 0 Then
               If (h_cri_security_id IS NULL) Then
                  h_cri_security_id := h_security_id;
               End If;
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, ELEMENT_ID, RESOURCE_TYPE, PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, ELEMENT_TYPE, TABLE_TYPE, REC_STATUS, REC_USER)
                  VALUES(h_uniqueID, h_id, h_res_type, h_project_id, h_res_id, h_participant, h_cri_security_id, h_type, h_table_type, 'U', h_rec_user);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 210+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(211) : move_wbs.sql(639) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, ELEMENT_ID, RESOURCE_TYPE, PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, ELEMENT_TYPE, TABLE_TYPE, REC_STATUS, REC_USER)
                  VALUES(h_uniqueID, h_id, h_res_type, h_project_id, h_res_id, h_participant, h_security_id, h_type, h_table_type, 'U', h_rec_user);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 228+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(229) : move_wbs.sql(657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            --trace_printf("2_* SC_STATE: %d, h_uniqueID: %s, h_participant: %d, h_security_id: %s\n", SC_STATE,h_uniqueID,h_participant,h_security_id);
         End If;
      End If;
   END LOOP;
   --trace_printf("END h_assign_string: %s\n",h_assign_string);
   BEGIN
      CLOSE C2P ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 247+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(248) : move_wbs.sql(676) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 0 Then
      h_res_id := h_rec_user;
      h_table_type := GetTable(Coalesce(h_type, 0));
      h_participant := GetParticipantBitValue(Coalesce(h_type, 0));
      PMO_DB2M.SC_STATE := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                   , PMO_CNST.SC_RESPONSIBLE_GROUP, h_security_id);
      
      BEGIN
         SELECT NICK_NAME || ';' INTO h_full_name
         FROM RESOURCES
         WHERE RESOURCE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 269+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(270) : move_wbs.sql(698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT RES_ASSIGNMENT_ID INTO h_uniqueID
         FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND RESOURCE_ID = h_rec_user
         AND ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 286+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(287) : move_wbs.sql(715) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         h_temp_assig_string := SubStr(h_full_name, 1, 32);
         
         h_temp_assig_string := SubStr(h_temp_assig_string || h_assign_string, 1, 32);
         h_assign_string := h_temp_assig_string;
         
         h_temp_var := 1;
         h_temp_var := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_participant, h_security_id
                                     , h_res_type, h_temp_var);
         
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, ELEMENT_ID, RESOURCE_TYPE, PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, ELEMENT_TYPE, TABLE_TYPE, REC_STATUS, REC_USER)
            VALUES(h_uniqueID, h_id, h_res_type, h_project_id, h_res_id, h_participant, h_security_id, h_type, h_table_type, 'U', h_rec_user);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 316+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(317) : move_wbs.sql(745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT h_full_name || REPLACE(h_assign_string, h_full_name, '')
            INTO h_assign_string
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 331+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(332) : move_wbs.sql(760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS
            SET PARTICIPANT = h_participant,
            SECURITY_ID = h_security_id
            WHERE RES_ASSIGNMENT_ID = h_uniqueID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_RESPONSIBLE', 351+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_RESPONSIBLE(352) : move_wbs.sql(780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_RESPONSIBLE', 359+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_RESPONSIBLE(360) : move_wbs.sql(788) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_RESPONSIBLE;
ALTER FUNCTION FC_RESPONSIBLE COMPILE DEBUG;

/**************************************************************************************************
* Get next available datetime for profile h_new_id
**************************************************************************************************/

Prompt == FUNCTION FC_getNextDate ==
CREATE OR REPLACE FUNCTION FC_getNextDate
(
     sh_h_bitflag PLS_INTEGER
   , h_dt1 IN OUT NOCOPY TIMESTAMP
   , sh_h_minutes PLS_INTEGER
   , h_dt2 IN OUT NOCOPY TIMESTAMP
   , h_bitID IN OUT NOCOPY RAW
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_minutes PLS_INTEGER := sh_h_minutes;
   h_bitID_Ind PLS_INTEGER NOT NULL := 0;
   h_finish_DT TIMESTAMP;
   h_start_time VARCHAR2(8);
   h_finish_time VARCHAR2(8);
   h_available PLS_INTEGER := 0;
   h_bitID2 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   iBits PLS_INTEGER;
   iHour PLS_INTEGER;
   iMin PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_getNextDate') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_getNextDate'), '-- ' || SYSTIMESTAMP, ':r := FC_getNextDate(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_dt1 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_minutes), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_dt2 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_bitID), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_getNextDate(32) : move_wbs.sql(832) : ' || SYSTIMESTAMP);
   END IF; 

   If (PMO_ORASTD.strncmp(h_dt1, h_dt2, 10) > 0) OR (Coalesce(h_minutes, 0) = 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(37) : move_wbs.sql(837) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   ElsIf PMO_ORASTD.strcmp(h_dt1, h_dt2) = PMO_CNST.SC_MATCH Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 41+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(42) : move_wbs.sql(842) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   ElsIf PMO_ORASTD.strncmp(h_dt1, h_dt2, 10) < PMO_CNST.SC_MATCH Then
      If Coalesce(h_bitflag, 0) = 0 Then
         BEGIN
            SELECT PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)), PMO_ORASTD.TIME(h_dt1), AVAILABLE_MIN, BIT_SCHED
            INTO h_start_time, h_finish_time, h_available, h_bitID2
            FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dt1, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_getNextDate', 56+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_getNextDate(57) : move_wbs.sql(857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 63+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(64) : move_wbs.sql(864) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
         
         If PMO_ORASTD.strcmp(h_start_time, h_finish_time) < PMO_CNST.SC_MATCH Then
            h_available := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID2, h_start_time, h_finish_time);
            PMO_STPU.skipFirstBitsClearToFirst(Coalesce(h_available, 0), h_bitID2);
            h_available := PMO_STPU.getNumberOfBits(h_bitID2) * 5;
            
            iBits := PMO_STPU.getFirstBit(h_bitID2);
            
            iHour := Trunc(iBits / 12);
            
            iMin := (iBits MOD 12) * 5;
            
            h_start_time := LPad(iHour, 2,'0') || ':' || LPad(iMin, 2,'0') || ':00';
            
            BEGIN
               SELECT  PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dt1, 1, 10)), PMO_ORASTD.TIME(h_start_time))  INTO h_dt1
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_getNextDate', 90+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_getNextDate(91) : move_wbs.sql(891) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT AVAILABLE_MIN, BIT_SCHED INTO h_available, h_bitID2
            FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dt1, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_getNextDate', 105+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_getNextDate(106) : move_wbs.sql(906) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 112+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(113) : move_wbs.sql(913) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN 0;
         End If;
      End If;
   Else
      BEGIN
         SELECT PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)), PMO_ORASTD.TIME(h_dt1), AVAILABLE_MIN, BIT_SCHED
         INTO h_start_time, h_finish_time, h_available, h_bitID2
         FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_dt1, 1, 10));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_getNextDate', 128+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_getNextDate(129) : move_wbs.sql(929) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strcmp(h_start_time, h_finish_time) < PMO_CNST.SC_MATCH Then
         h_available := PMO_STPU.getNumberOfMinutesBetweenSAndF(h_bitID2, h_start_time, h_finish_time);
         PMO_STPU.skipFirstBitsClearToFirst(Coalesce(h_available, 0), h_bitID2);
         h_available := PMO_STPU.getNumberOfBits(h_bitID2) * 5;
         
         iBits := PMO_STPU.getFirstBit(h_bitID2);
         
         iHour := Trunc(iBits / 12);
         
         iMin := (iBits MOD 12) * 5;
         
         h_start_time := LPad(iHour, 2,'0') || ':' || LPad(iMin, 2,'0') || ':00';
         
         BEGIN
            SELECT  PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dt1, 1, 10)), PMO_ORASTD.TIME(h_start_time))  INTO h_dt1
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_getNextDate', 155+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_getNextDate(156) : move_wbs.sql(956) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_minutes, 0) < Coalesce(h_available, 0) Then
      h_available := Coalesce(h_available, 0) - Coalesce(h_minutes, 0);
      PMO_STPU.skipFirstBitsClearToLast(Coalesce(h_available, 0), h_bitID2);
   Else
      h_minutes := Coalesce(h_available, 0);
   End If;
   
   h_bitID := h_bitID2;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 171+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(172) : move_wbs.sql(972) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_minutes, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_getNextDate', 176+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_getNextDate(177) : move_wbs.sql(977) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_getNextDate;
ALTER FUNCTION FC_getNextDate COMPILE DEBUG;

/**************************************************************************************************
* Get h_new_id for existing profile, or insert h_res_id, h_new_id
**************************************************************************************************/

Prompt == FUNCTION FC_findProfile ==
CREATE OR REPLACE FUNCTION FC_findProfile
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_prj_assignment_id IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_new_id IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , sh_h_effort PLS_INTEGER
   , h_finish_date IN OUT NOCOPY TIMESTAMP
) RETURN PLS_INTEGER
AS
   h_effort PLS_INTEGER := sh_h_effort;
   h_finish_DT TIMESTAMP;
   h_finish_time VARCHAR2(8);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CFP01 IS
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(DAY_YEAR, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
         FROM PMO_TMP_CDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));

      CURSOR CFP03 IS
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(DAY_YEAR, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
         FROM PMO_TMP_CDY WHERE DAY_YEAR > TO_DATE(SubStr(h_start_date, 1, 10));

      CURSOR CFP02 IS
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(DAY_YEAR, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)))
         FROM PMO_TMP_CDY WHERE DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10));

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_findProfile') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_findProfile'), '-- ' || SYSTIMESTAMP, ':r := FC_findProfile(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_prj_assignment_id || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_new_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_finish_date || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_findProfile', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_findProfile(42) : move_wbs.sql(1031) : ' || SYSTIMESTAMP);
   END IF; 

   --trace_printf("START_* h_prj_assignment_id: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   BEGIN
      SELECT NEWPROF_ID, PROJECT_ASSIGNMENT_ID, LAST_FINISH_DT INTO h_new_id, h_prj_assignment_id, h_start_date
      FROM PMO_COPY_PROF WHERE OLDPROF_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 55+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(56) : move_wbs.sql(1045) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_res_id: %s, h_prj_assignment_id: %d:%s\n", SC_STATE,h_res_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If (h_start_date IS NULL) Then
         OPEN CFP01;
         
         FETCH CFP01 INTO h_start_date;
         IF CFP01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CFP01 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 80+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(81) : move_wbs.sql(1070) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(DAY_YEAR, 1, 10)), PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_finish_DT
            FROM PMO_TMP_CDY WHERE DAY_YEAR = TO_DATE(SubStr(h_start_date, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 94+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(95) : move_wbs.sql(1084) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            If PMO_ORASTD.strcmp(h_start_date, h_finish_DT) = 0 Then
               OPEN CFP03;
               
               FETCH CFP03 INTO h_start_date;
               IF CFP03%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               BEGIN
                  CLOSE CFP03 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 118+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(119) : move_wbs.sql(1108) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT  CASE WHEN PMO_ORASTD.TIME(h_start_date) = PMO_ORASTD.TIME('17.00.00')
               THEN PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_date, 1, 10)) + 1    , PMO_ORASTD.TIME('08.00.00'))
               ELSE h_start_date END
               INTO h_start_date
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 136+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(137) : move_wbs.sql(1126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_temp_sc_state := GET_FD_DURATION(h_project_id, h_start_date, h_finish_date, h_effort);
      
      BEGIN
         UPDATE PMO_COPY_PROF SET LAST_FINISH_DT = h_finish_date;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 157+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(158) : move_wbs.sql(1147) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := 0;
   Else
      OPEN CFP02;
      
      FETCH CFP02 INTO h_start_date;
      IF CFP02%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      BEGIN
         CLOSE CFP02 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 181+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(182) : move_wbs.sql(1171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := GET_FD_DURATION(h_project_id, h_start_date, h_finish_date, h_effort);
      --trace_printf("1_* h_prj_assignment_id: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      If (h_prj_assignment_id IS NOT NULL) Then
         -- Project assignments exists
         h_prj_assignment_id := SYS_GUID();
         
         BEGIN
            INSERT INTO PMO_COPY_PROF (OLDPROF_ID, NEWPROF_ID, PROJECT_ASSIGNMENT_ID, LAST_FINISH_DT)
            VALUES (h_res_id, h_new_id, h_prj_assignment_id, h_finish_date);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 206+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(207) : move_wbs.sql(1196) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            INSERT INTO PMO_COPY_PROF (OLDPROF_ID, NEWPROF_ID, LAST_FINISH_DT) VALUES (h_res_id, h_new_id, h_finish_date);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findProfile', 223+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findProfile(224) : move_wbs.sql(1213) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("2_* h_prj_assignment_id: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      PMO_DB2M.SC_STATE := 100;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_findProfile', 234+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_findProfile(235) : move_wbs.sql(1224) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_findProfile;
ALTER FUNCTION FC_findProfile COMPILE DEBUG;

/**************************************************************************************************
* Get h_new_id for existing profile, or insert h_res_id, h_new_id
**************************************************************************************************/

Prompt == FUNCTION FC_findTask ==
CREATE OR REPLACE FUNCTION FC_findTask
(
     h_cur_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_wbs_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_findTask') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_findTask'), '-- ' || SYSTIMESTAMP, ':r := FC_findTask(' || 
       '''' || h_cur_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_wbs_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_findTask', 16+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_findTask(17) : move_wbs.sql(1253) : ' || SYSTIMESTAMP);
   END IF; 

   If ((h_wbs_id IS NULL)) OR (Coalesce(Length(h_wbs_id), 0) = 0) Then
      BEGIN
         SELECT ELEMENT_ID INTO h_id
         FROM PMO_COPY_WBS
         WHERE OLD_WBS_ID = h_cur_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findTask', 30+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findTask(31) : move_wbs.sql(1267) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            SELECT ELEMENT_ID INTO h_id
            FROM PMO_COPY_CRI
            WHERE OLD_ID = h_cur_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findTask', 46+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findTask(47) : move_wbs.sql(1283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         SELECT ELEMENT_ID INTO h_id
         FROM PMO_COPY_CRI
         WHERE OLD_ID = h_cur_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_findTask', 62+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_findTask(63) : move_wbs.sql(1299) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_findTask', 68+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_findTask(69) : move_wbs.sql(1305) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_findTask;
ALTER FUNCTION FC_findTask COMPILE DEBUG;

/*****************************************************************************************/

-- Copy Dependencies. Only internal (i.e. succ_id and pred_id both are children of the

-- source) dependencies will be kept. Other dependencies will be removed.

/*****************************************************************************************/

Prompt == FUNCTION FC_C_DEPENDENCIES ==
CREATE OR REPLACE FUNCTION FC_C_DEPENDENCIES
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_project_name IN OUT NOCOPY VARCHAR2
   , h_new_rank IN OUT NOCOPY VARCHAR2
   , h_cur_date IN OUT NOCOPY TIMESTAMP
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_cur_id VARCHAR2(32);
   h_doc_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_id1 VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_min VARCHAR2(19);
   h_element_name VARCHAR2(255);
   h_dep_str VARCHAR2(8192);
   h_element_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_n4 PLS_INTEGER := 0;
   h_n5 PLS_INTEGER := 0;
   h_n6 PLS_INTEGER := 0;
   h_n7 PLS_INTEGER := 0;
   h_n8 PLS_INTEGER := 0;
   h_n9 PLS_INTEGER := 0;
   h_continue PLS_INTEGER := 0;
   h_wbs_id VARCHAR2(32);
-- Declare Cursors
      CURSOR CX2DEPENDENCY IS
         SELECT
         D.PRED_ID, D.SUCC_ID, D.PRED_OBJECT_TYPE, D.SUCC_OBJECT_TYPE,
         D.DEP_TYPE, D.LEAD_OR_LAG_TYPE, D.AMOUNT, D.LAG_DISPLAY_UNIT
         FROM DEPENDENCIES D, PMO_COPY_WBS W
         WHERE W.RANK LIKE h_new_rank || '%' and D.SUCC_ID=W.OLD_WBS_ID AND PRED_PROJECT_ID = SUCC_PROJECT_ID;

      CURSOR C_STR_DEP IS
         SELECT
         D.PRED_ID, D.SUCC_ID, D.OBJECT_TYPE, D.DEP_TYPE, D.LeadLeagType, D.AMOUNT, D.LagDisplayUnit, W.ELEMENT_NAME
         FROM PMO_WBS_DEPEND D JOIN PMO_COPY_WBS W ON D.PRED_ID = W.ELEMENT_ID ORDER BY D.SUCC_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_DEPENDENCIES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_DEPENDENCIES'), '-- ' || SYSTIMESTAMP, ':r := FC_C_DEPENDENCIES(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_project_name || ''''
          || ', ' || '''' || h_new_rank || ''''
          || ', ' || COALESCE(NULLIF('''' || h_cur_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_DEPENDENCIES', 51+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_DEPENDENCIES(52) : move_wbs.sql(1373) : ' || SYSTIMESTAMP);
   END IF; 

   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_new_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_new_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_new_rank, False); 
   	End If; 
   END;
   
   OPEN CX2DEPENDENCY;
   
   LOOP
      h_continue := 1;
      
      FETCH CX2DEPENDENCY INTO h_doc_id, h_cur_id, h_n4, h_element_type, h_n6, h_n7, h_n8, h_n9;
      IF CX2DEPENDENCY%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      h_wbs_id := NULL;
      
      h_temp_var := FC_findTask(h_doc_id, h_id1, h_wbs_id);
      h_temp_var := FC_findTask(h_cur_id, h_id, h_wbs_id);
      
      If (SubStr(h_id1, 1, 1) = Chr(0)) OR (SubStr(h_id, 1, 1) = Chr(0)) Then
         h_continue := 0;
      End If;
      
      If Coalesce(h_continue, 0) = 1 Then
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO DEPENDENCIES(
            DEPEND_ID,
            PRED_ID, PRED_PROJECT_ID, PRED_OBJECT_TYPE,
            SUCC_ID, SUCC_PROJECT_ID, SUCC_OBJECT_TYPE,
            DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT, LAG_DISPLAY_UNIT, REC_STATUS, REC_USER, REC_DATETIME)
            VALUES(h_uniqueID,
            h_id1, h_project_id, h_n4,
            h_id, h_project_id, h_element_type,
            h_n6, h_n7, h_n8, h_n9, 'T', h_rec_user, h_cur_date);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 115+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(116) : move_wbs.sql(1437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_WBS_DEPEND (PRED_ID,SUCC_ID,OBJECT_TYPE,DEP_TYPE,LeadLeagType,AMOUNT,LagDisplayUnit)
            VALUES(h_id1, h_id, h_n4, h_n6, h_n7, h_n8, h_n9);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 133+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(134) : move_wbs.sql(1455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE CX2DEPENDENCY ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 148+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(149) : move_wbs.sql(1470) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_cur_id := '';
   --strcpy(h_dep_str, "{RS}");
   h_dep_str := '{RS}';
   
   OPEN C_STR_DEP;
   
   -- Vladimir Schneider, vladimir@winpte.com
   -- October 1, 2004
   -- TODO: This routine will truncate the dependency string anywhere, without regard for
   -- invalidating the format. Needs to be rewritten to resemble the packDependencies( ) which
   -- only stores complete entries.
   LOOP
      FETCH C_STR_DEP INTO h_id1, h_id, h_n4, h_n6, h_n7, h_n8, h_n9, h_element_name;
      IF C_STR_DEP%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      If PMO_ORASTD.strcmp(h_id, h_cur_id) = PMO_CNST.SC_MATCH Then
         h_dep_str := SubStr(h_dep_str || '{DS}', 1, 8192);
         h_dep_str := SubStr(h_dep_str || 'DEP_TYPE=', 1, 8192);
         h_min := TO_CHAR(Coalesce(h_n6, 0));
         h_dep_str := SubStr(h_dep_str || h_min, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';ELEMENT_ID=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_id1, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';TYPE_ID=', 1, 8192);
         h_min := TO_CHAR(Coalesce(h_n4, 0));
         h_dep_str := SubStr(h_dep_str || h_min, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';ELEMENT_NAME=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_element_name, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';PROJECT_ID=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_project_id, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';PROJECT_NAME=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_project_name, 1, 8192);
         h_dep_str := SubStr(h_dep_str || '{DF}', 1, 8192);
      Else
         If Coalesce(Length(h_cur_id), 0) = 32 Then
            h_dep_str := SubStr(h_dep_str || '{RF}', 1, 8192);
            
            If PMO_ORASTD.strcmp(h_dep_str, '{RS}{RF}') <> 0 Then
               UPDATE TMT_WBS SET
                       DEPENDENCIES = NULL
                  WHERE ELEMENT_ID = H_CUR_ID;

               BEGIN
                  UPDATE TMT_WBS SET DEPENDENCIES = h_dep_str WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 213+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(214) : move_wbs.sql(1535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_WBS SET DEPENDENCIES = null WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 230+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(231) : move_wbs.sql(1552) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --strcpy(h_dep_str, "{RS}");
            h_dep_str := '{RS}';
         End If;
         
         --strcpy(h_dep_str, "{RS}");
         h_dep_str := '{RS}';
         h_dep_str := SubStr(h_dep_str || '{DS}', 1, 8192);
         h_dep_str := SubStr(h_dep_str || 'DEP_TYPE=', 1, 8192);
         h_min := TO_CHAR(Coalesce(h_n6, 0));
         h_dep_str := SubStr(h_dep_str || h_min, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';ELEMENT_ID=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_id1, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';TYPE_ID=', 1, 8192);
         h_min := TO_CHAR(Coalesce(h_n4, 0));
         h_dep_str := SubStr(h_dep_str || h_min, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';ELEMENT_NAME=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_element_name, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';PROJECT_ID=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_project_id, 1, 8192);
         h_dep_str := SubStr(h_dep_str || ';PROJECT_NAME=', 1, 8192);
         h_dep_str := SubStr(h_dep_str || h_project_name, 1, 8192);
         h_dep_str := SubStr(h_dep_str || '{DF}', 1, 8192);
         
         h_cur_id := h_id;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_STR_DEP ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 270+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(271) : move_wbs.sql(1592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(Length(h_cur_id), 0) = 32 Then
      h_dep_str := SubStr(h_dep_str || '{RF}', 1, 8192);
      
      If PMO_ORASTD.strcmp(h_dep_str, '{RS}{RF}') <> 0 Then
         UPDATE TMT_WBS SET
                 DEPENDENCIES = NULL
            WHERE ELEMENT_ID = H_CUR_ID;

         BEGIN
            UPDATE TMT_WBS SET DEPENDENCIES = h_dep_str WHERE ELEMENT_ID = h_cur_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 295+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(296) : move_wbs.sql(1617) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_WBS SET DEPENDENCIES = null WHERE ELEMENT_ID = h_cur_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DEPENDENCIES', 312+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DEPENDENCIES(313) : move_wbs.sql(1634) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_DEPENDENCIES', 320+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_DEPENDENCIES(321) : move_wbs.sql(1642) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_DEPENDENCIES;
ALTER FUNCTION FC_C_DEPENDENCIES COMPILE DEBUG;

/*****************************************************************************************/

-- Get CRI Assignments based on CRI Rank

/*****************************************************************************************/

Prompt == FUNCTION FC_GET_CRI_ASSIGNMENTS ==
CREATE OR REPLACE FUNCTION FC_GET_CRI_ASSIGNMENTS
(
     h_source_project_id IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CDBF6 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID
         FROM PMO_TMP_TA;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_CRI_ASSIGNMENTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_CRI_ASSIGNMENTS'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_CRI_ASSIGNMENTS(' || 
       '''' || h_source_project_id || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_CRI_ASSIGNMENTS', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_CRI_ASSIGNMENTS(26) : move_wbs.sql(1682) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TA';
   PMO_DB2M.SC_STATE := 0;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   BEGIN
      INSERT INTO PMO_TMP_TA
      SELECT RA.ELEMENT_ID, RA.ELEMENT_TYPE, RA.ASSIGN_STATUS, RA.WORK_LOCATION,
      RA.RES_ASSIGNMENT_ID, RA.PARTICIPANT, RA.SECURITY_ID,
      TA.TSK_ASSIGNMENT_ID,TA.PROJECT_ASSIGNMENT_ID,TA.RESOURCE_TYPE,TA.RESOURCE_ID,
      TA.PROPOSED_START, TA.BIT_FLAG, TA.PROPOSED_WORK, TA.PROPOSED_DURATION, TA.TABLE_TYPE,
      TA.C_EFFORT, TA.C_CONTOUR, TA.CONTOUR_TYPE, TA.T_UNIT, TA.C_UNIT, TA.C_CONTOUR
      FROM TMT_CRI T JOIN TASK_ASSIGNMENTS TA
      ON TA.PROJECT_ID = T.PROJECT_ID AND T.ELEMENT_ID = TA.TASK_ID
      JOIN RESOURCE_ASSIGNMENTS RA ON RA.RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID
      WHERE T.PROJECT_ID = h_source_project_id
      AND TA.TABLE_TYPE = PMO_CNST.SC_TABLE_CRI AND T.LEVEL_ID > 1
      AND T.RANK LIKE h_source_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_CRI_ASSIGNMENTS', 68+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_CRI_ASSIGNMENTS(69) : move_wbs.sql(1725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_DELETE';
   PMO_DB2M.SC_STATE := 0;
   
   --remove all planned and sched assignments
   
   OPEN CDBF6;
   
   LOOP
      FETCH CDBF6 INTO h_id, h_res_id;
      IF CDBF6%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_temp_var := PMO_CNST.SC_BIT_28;
      h_temp_var := FC_D_TSKRES(h_id, h_temp_var, h_res_id, h_rec_user);
   END LOOP;
   
   BEGIN
      CLOSE CDBF6 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_CRI_ASSIGNMENTS', 105+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_CRI_ASSIGNMENTS(106) : move_wbs.sql(1762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --remove all assignments to source project
   BEGIN
      DELETE FROM PMO_TMP_TA WHERE ELEMENT_ID = h_source_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_CRI_ASSIGNMENTS', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_CRI_ASSIGNMENTS(124) : move_wbs.sql(1780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_CRI_ASSIGNMENTS', 129+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_CRI_ASSIGNMENTS(130) : move_wbs.sql(1786) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_CRI_ASSIGNMENTS;
ALTER FUNCTION FC_GET_CRI_ASSIGNMENTS COMPILE DEBUG;

/*****************************************************************************************/

-- Get WBS + CRI Assignments based on Source Rank

/*****************************************************************************************/

Prompt == FUNCTION FC_GET_ASSIGNMENTS ==
CREATE OR REPLACE FUNCTION FC_GET_ASSIGNMENTS
(
     h_source_project_id IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CDBF5 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID
         FROM PMO_TMP_TA;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_ASSIGNMENTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_ASSIGNMENTS'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_ASSIGNMENTS(' || 
       '''' || h_source_project_id || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ASSIGNMENTS', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_ASSIGNMENTS(26) : move_wbs.sql(1826) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_TA';
   PMO_DB2M.SC_STATE := 0;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   BEGIN
      INSERT INTO PMO_TMP_TA
      SELECT RA.ELEMENT_ID, RA.ELEMENT_TYPE, RA.ASSIGN_STATUS, RA.WORK_LOCATION,
      RA.RES_ASSIGNMENT_ID, RA.PARTICIPANT, RA.SECURITY_ID,
      TA.TSK_ASSIGNMENT_ID,TA.PROJECT_ASSIGNMENT_ID,TA.RESOURCE_TYPE,TA.RESOURCE_ID,
      TA.PROPOSED_START, TA.BIT_FLAG, TA.PROPOSED_WORK, TA.PROPOSED_DURATION, TA.TABLE_TYPE,
      TA.C_EFFORT, TA.C_CONTOUR, TA.CONTOUR_TYPE, TA.T_UNIT, TA.C_UNIT, TA.C_CONTOUR
      FROM TMT_WBS T JOIN TASK_ASSIGNMENTS TA
      ON TA.PROJECT_ID = T.PROJECT_ID AND T.ELEMENT_ID = TA.TASK_ID
      JOIN RESOURCE_ASSIGNMENTS RA ON RA.RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID
      WHERE T.PROJECT_ID = h_source_project_id
      AND TA.TABLE_TYPE = PMO_CNST.SC_TABLE_WBS
      AND T.RANK LIKE h_source_rank || '%'
      UNION ALL
      SELECT RA.ELEMENT_ID, RA.ELEMENT_TYPE, RA.ASSIGN_STATUS, RA.WORK_LOCATION,
      RA.RES_ASSIGNMENT_ID, RA.PARTICIPANT, RA.SECURITY_ID,
      TA.TSK_ASSIGNMENT_ID,TA.PROJECT_ASSIGNMENT_ID,TA.RESOURCE_TYPE,TA.RESOURCE_ID,
      TA.PROPOSED_START, TA.BIT_FLAG, TA.PROPOSED_WORK, TA.PROPOSED_DURATION, TA.TABLE_TYPE,
      TA.C_EFFORT, TA.C_CONTOUR, TA.CONTOUR_TYPE, TA.T_UNIT, TA.C_UNIT, TA.C_CONTOUR
      FROM TMT_CRI T JOIN TASK_ASSIGNMENTS TA
      ON TA.PROJECT_ID = T.PROJECT_ID AND T.ELEMENT_ID = TA.TASK_ID
      JOIN RESOURCE_ASSIGNMENTS RA ON RA.RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID
      WHERE T.PROJECT_ID = h_source_project_id
      AND TA.TABLE_TYPE = PMO_CNST.SC_TABLE_CRI AND T.LEVEL_ID > 1
      AND T.PARENT_RANK LIKE 'WBS#' || h_source_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ASSIGNMENTS', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ASSIGNMENTS(81) : move_wbs.sql(1881) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_DELETE';
   PMO_DB2M.SC_STATE := 0;
   
   --remove all planned and sched assignments
   
   OPEN CDBF5;
   
   LOOP
      FETCH CDBF5 INTO h_id, h_res_id;
      IF CDBF5%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_temp_var := PMO_CNST.SC_BIT_28;
      h_temp_var := FC_D_TSKRES(h_id, h_temp_var, h_res_id, h_rec_user);
   END LOOP;
   
   BEGIN
      CLOSE CDBF5 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ASSIGNMENTS', 117+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ASSIGNMENTS(118) : move_wbs.sql(1918) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --remove all assignments to source project
   BEGIN
      DELETE FROM PMO_TMP_TA WHERE ELEMENT_ID = h_source_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ASSIGNMENTS', 135+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ASSIGNMENTS(136) : move_wbs.sql(1936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ASSIGNMENTS', 141+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_ASSIGNMENTS(142) : move_wbs.sql(1942) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_ASSIGNMENTS;
ALTER FUNCTION FC_GET_ASSIGNMENTS COMPILE DEBUG;

/*********************************************************************************************************************************************/

-- search function

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_SEARCH_1 ==
CREATE OR REPLACE FUNCTION FC_SEARCH_1
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_task_id IN OUT NOCOPY VARCHAR2
   , sh_h_type_id PLS_INTEGER
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_res_type IN OUT NOCOPY VARCHAR2
   , h_start_date IN OUT NOCOPY TIMESTAMP
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_duration PLS_INTEGER
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_icontur PLS_INTEGER
   , sh_h_min_per_day PLS_INTEGER
   , sh_h_priority PLS_INTEGER
   , sh_h_work_location PLS_INTEGER
   , h_source_project_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_duration PLS_INTEGER := sh_h_duration;
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_icontur PLS_INTEGER := sh_h_icontur;
   h_min_per_day PLS_INTEGER := sh_h_min_per_day;
   h_priority PLS_INTEGER := sh_h_priority;
   h_work_location PLS_INTEGER := sh_h_work_location;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_security_id VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_dayofyear DATE;
   h_active VARCHAR2(1);
   h_full_name VARCHAR2(255);
   h_creator_name VARCHAR2(255);
   h_participant PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_res_cost NUMBER := 0.00;
   i_return PLS_INTEGER NOT NULL := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEARCH_1') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEARCH_1'), '-- ' || SYSTIMESTAMP, ':r := FC_SEARCH_1(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_task_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_res_type || ''''
          || ', ' || COALESCE(NULLIF('''' || h_start_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_duration), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_icontur), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_min_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_priority), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_location), 'NULL')
          || ', ' || '''' || h_source_project_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_1', 88+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEARCH_1(89) : move_wbs.sql(2045) : ' || SYSTIMESTAMP);
   END IF; 

   h_cri := 0;
   h_consider := 16;
   h_occurrence := 0;
   h_start_DT := '';
   h_finish_DT := '';
   h_creator_name := '';
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   If (isWBSTask(Coalesce(h_type_id, 0))) <> 0 OR (isCRITask(Coalesce(h_type_id, 0))) <> 0 Then
      If SubStr(h_res_type, 1, 1) = '0' Then
         BEGIN
            SELECT SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK
            INTO h_start_DT,h_total, h_sched_work FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(113) : move_wbs.sql(2069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_from_prj := 1;
            If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
               BEGIN
                  SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                  INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 130+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(131) : move_wbs.sql(2087) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_work_type := 2;
               NULL;
            End If;
         ElsIf (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
               INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 149+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(150) : move_wbs.sql(2106) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
            INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(166) : move_wbs.sql(2122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_1', 172+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_1(173) : move_wbs.sql(2129) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If Coalesce(h_total, 0) = 1 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_1', 180+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_1(181) : move_wbs.sql(2137) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
      
      h_from_prj := 1;
   ElsIf (Coalesce(h_from_prj, 0) = 1) AND (isNotProjectType(Coalesce(h_type_id, 0))) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_1', 188+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_1(189) : move_wbs.sql(2145) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type_id, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
   h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
   h_temp_bitflag := 1;
   
   h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type_id, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
   
   BEGIN
      SELECT RES_ASSIGNMENT_ID INTO h_assign_id FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 210+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(211) : move_wbs.sql(2167) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_OR(PARTICIPANT,PMO_CNST.SC_TEAMMEMBER)
         WHERE RES_ASSIGNMENT_ID = h_assign_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 229+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(230) : move_wbs.sql(2186) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --Get a unique ID
      h_assign_id := SYS_GUID();
      
      If SubStr(h_res_type, 1, 1) = '0' Then
         BEGIN
            SELECT NICK_NAME INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 246+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(247) : move_wbs.sql(2203) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 263+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(264) : move_wbs.sql(2220) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 281+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(282) : move_wbs.sql(2238) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            SELECT PROFILE_NAME, SUBSTR(PROFILE_NAME,1,InStr(PROFILE_NAME,'(')), ROLE_ID, PROF_NUMBER
            INTO h_creator_name, h_full_name, h_role_id, h_totpr
            FROM PROFILES WHERE PROJECT_ID = h_source_project_id AND PROFILE_ID = h_res_id AND REC_STATUS = 'D';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 297+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(298) : move_wbs.sql(2254) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
               FROM PROFILES WHERE PROJECT_ID = h_project_id AND ROLE_ID = h_role_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 312+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(313) : move_wbs.sql(2269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_number, 0) = Coalesce(h_totpr, 0) Then
               --strcpy(h_full_name,h_creator_name);
               h_full_name := h_creator_name;
               
               BEGIN
                  UPDATE PROFILES SET PROJECT_ID = h_project_id, REC_STATUS = 'U'
                  WHERE PROJECT_ID = h_source_project_id AND PROFILE_ID = h_res_id AND REC_STATUS = 'D';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 334+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(335) : move_wbs.sql(2291) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_temp_security_id := TO_CHAR(Coalesce(h_number, 0)) || ')';
               h_full_name := SubStr(h_full_name || h_temp_security_id, 1, 255);
               
               BEGIN
                  UPDATE PROFILES SET PROJECT_ID = h_project_id, PROF_NUMBER = h_number, PROFILE_NAME = h_full_name, REC_STATUS = 'U'
                  WHERE PROJECT_ID = h_source_project_id AND PROFILE_ID = h_res_id AND REC_STATUS = 'D';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 355+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(356) : move_wbs.sql(2312) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT PROFILE_NAME INTO h_full_name
               FROM PROFILES WHERE PROJECT_ID = h_project_id AND PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 370+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(371) : move_wbs.sql(2327) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'
            WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 389+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(390) : move_wbs.sql(2346) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'
               WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 408+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(409) : move_wbs.sql(2365) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
         RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT,SECURITY_ID, REC_USER, REC_DATETIME)
         VALUES (h_assign_id, h_project_id, h_task_id, h_type_id, h_res_type, h_res_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, h_rec_user, SYSTIMESTAMP);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 429+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(430) : move_wbs.sql(2386) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_29));
   -- set do not rollup
   
   If Coalesce(h_from_prj, 0) = 1 Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_date)) INTO h_start_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 448+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(449) : move_wbs.sql(2405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_date ELSE h_start_DT END
         INTO h_start_DT
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 463+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(464) : move_wbs.sql(2420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_totpr := 10;
      If Coalesce(h_work_type, 0) = 0 Then
         h_update := 3;
      End If;
      
      i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type_id
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_icontur, h_occurrence, h_update
                        , h_min_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
   Else
      If SubStr(h_res_type, 1, 1) = '1' Then
         h_totpr := PMO_CNST.SC_SRCH_PROFILE;
      Else
         h_totpr := Coalesce(h_consider, 0);
      End If;
      
      i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type_id
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_totpr, h_res_type, h_work_type, h_bitflag, h_icontur, h_occurrence
                        , h_update, h_min_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
   End If;
   
   If (i_return = 0) AND (isCRITask(Coalesce(h_type_id, 0))) <> 0 Then
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS
         SET ELEMENT_TYPE = h_type_id
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_task_id
         AND ELEMENT_TYPE <> h_type_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 509+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(510) : move_wbs.sql(2466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_CRI
         SET TYPE_ID = h_type_id
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_1', 529+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_1(530) : move_wbs.sql(2486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_1', 536+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_1(537) : move_wbs.sql(2493) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEARCH_1;
ALTER FUNCTION FC_SEARCH_1 COMPILE DEBUG;

/*****************************************************************************************/

-- Move Assignments

/*****************************************************************************************/

Prompt == FUNCTION FC_M_ASSIGNMENTS ==
CREATE OR REPLACE FUNCTION FC_M_ASSIGNMENTS
(
     h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_source_project_id VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_project_id VARCHAR2(32);
   h_target_date TIMESTAMP;
   h_source_date TIMESTAMP;
   h_task_id VARCHAR2(32);
   h_new_id VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_old_tsk_assign_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rate_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_start_date TIMESTAMP;
   h_finish_date VARCHAR2(26);
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_participant PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_best PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_element_type PLS_INTEGER := 0;
   h_assign_status PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_work_location PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_icontur PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
-- Declare Cursors
      CURSOR C4MR IS
         SELECT ELEMENT_ID, ELEMENT_TYPE, RESOURCE_TYPE, RESOURCE_ID, WORK_LOCATION,
         PMO_ORASTD.DB2TIMESTAMP(h_target_date) + (PMO_ORASTD.DAYS(PROPOSED_START) - PMO_ORASTD.DAYS(h_source_date))     ,
         PROPOSED_WORK, PROPOSED_DURATION, Trunc(BIT_FLAG), C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR
         FROM PMO_TMP_TA
         ORDER BY RESOURCE_TYPE DESC, RESOURCE_ID, ELEMENT_TYPE DESC, PROPOSED_START;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_ASSIGNMENTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_ASSIGNMENTS'), '-- ' || SYSTIMESTAMP, ':r := FC_M_ASSIGNMENTS(' || 
       '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ASSIGNMENTS', 64+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_ASSIGNMENTS(65) : move_wbs.sql(2572) : ' || SYSTIMESTAMP);
   END IF; 

   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_PROF';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      SELECT TARGET_START_FINSH_DATE, TARGET_MINUTES_PER_DAY, TARGET_PRIORITY, TARGET_PROJECT_ID,
      SOURCE_PROJECT_ID, SOURCE_RANK, SOURCE_START_FINSH_DATE
      INTO h_target_date, h_min_per_day, h_priority, h_project_id,
      h_source_project_id, h_source_rank, h_source_date
      FROM PMO_TARGET_SOURCE;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ASSIGNMENTS', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ASSIGNMENTS(85) : move_wbs.sql(2592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_start_date := '';
   BEGIN
      SELECT MAX(PROPOSED_START) INTO h_start_date FROM PMO_TMP_TA;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ASSIGNMENTS', 98+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ASSIGNMENTS(99) : move_wbs.sql(2606) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND (Coalesce(Length(h_start_date), 0) > 9) Then
      h_finish_date := '';
      
      -- Check for project calendar
      If FC_CREATE_CDY(h_project_id, h_start_date, h_finish_date, h_duration, 1) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_M_ASSIGNMENTS', 109+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_M_ASSIGNMENTS(110) : move_wbs.sql(2617) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ASSIGNMENTS', 112+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_ASSIGNMENTS(113) : move_wbs.sql(2620) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_project_id
               , False);
      End If;
      
      h_duration := 0;
      -- Reassign
      
      OPEN C4MR;
      
      LOOP
         FETCH C4MR INTO h_task_id, h_element_type, h_res_type, h_res_id, h_work_location,
         h_start_date,
         h_effort, h_duration, h_work_type, h_work, h_eunit, h_unit, h_icontur;
         IF C4MR%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         If (h_start_date IS NOT NULL) Then
            If Coalesce(h_work, 0) = 0 Then
               h_bitflag := 0;
            Else
               h_bitflag := 1;
            End If;
            
            --GENERATES A NEW RESOURCE_ASSIGNMENT ID
            h_new_id := SYS_GUID();
            
            If Coalesce(h_effort, 0) = 0 Then
               h_finish_date := h_start_date;
            End If;
            
            h_duration := (Trunc(Coalesce(h_duration, 0) / Coalesce(h_min_per_day, 0)));
            If Coalesce(h_duration, 0) < 30 Then
               h_duration := 50;
            ElsIf Coalesce(h_duration, 0) > 800 Then
               h_duration := 1200;
            Else
               h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
            End If;
            
            h_temp_var := FC_SEARCH_1(h_project_id, h_task_id, h_element_type, h_res_id, h_res_type
                          , h_start_date, h_finish_date, h_duration, h_effort, h_work, h_eunit
                          , h_unit, h_work_type, h_bitflag, h_icontur, h_min_per_day, h_priority
                          , h_work_location, h_source_project_id, h_rec_user);
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C4MR ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_ASSIGNMENTS', 175+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_ASSIGNMENTS(176) : move_wbs.sql(2683) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_ASSIGNMENTS', 182+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_ASSIGNMENTS(183) : move_wbs.sql(2690) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_ASSIGNMENTS;
ALTER FUNCTION FC_M_ASSIGNMENTS COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_U_ALL_NULL_TIMECODES ==
CREATE OR REPLACE FUNCTION FC_U_ALL_NULL_TIMECODES
(
     h_target_rank IN OUT NOCOPY VARCHAR2
   , h_target_timecode_1_id IN OUT NOCOPY VARCHAR2
   , h_target_timecode_2_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_source_timecode_1_id VARCHAR2(32);
   h_source_timecode_2_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   flag_level PLS_INTEGER NOT NULL := 9999;
-- Declare Cursors
      CURSOR C4TC IS
         SELECT LEVEL_ID, RANK, ELEMENT_ID, TIMECODE_1_ID, TIMECODE_2_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_target_rank || '%'
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_ALL_NULL_TIMECODES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_ALL_NULL_TIMECODES'), '-- ' || SYSTIMESTAMP, ':r := FC_U_ALL_NULL_TIMECODES(' || 
       '''' || h_target_rank || ''''
          || ', ' || '''' || h_target_timecode_1_id || ''''
          || ', ' || '''' || h_target_timecode_2_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ALL_NULL_TIMECODES', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_ALL_NULL_TIMECODES(30) : move_wbs.sql(2730) : ' || SYSTIMESTAMP);
   END IF; 

   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_target_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_target_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, False); 
   	End If; 
   END;
   
   OPEN C4TC;
   
   LOOP
      FETCH C4TC INTO h_level_id, h_rank, h_task_id, h_source_timecode_1_id, h_source_timecode_2_id;
      IF C4TC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      If Coalesce(h_level_id, 0) <= flag_level Then
         If ((h_source_timecode_1_id IS NULL)) AND ((h_source_timecode_2_id IS NULL)) Then
            BEGIN
               UPDATE TMT_WBS
               SET TIMECODE_1_ID = h_target_timecode_1_id,
               TIMECODE_2_ID = h_target_timecode_2_id
               WHERE RANK = h_rank;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ALL_NULL_TIMECODES', 77+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ALL_NULL_TIMECODES(78) : move_wbs.sql(2778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS
               SET TIMECODE_1_ID = h_target_timecode_1_id,
               TIMECODE_2_ID = h_target_timecode_2_id
               WHERE TASK_ID = h_task_id AND TIMECODE_1_ID IS NuLL AND TIMECODE_2_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ALL_NULL_TIMECODES', 97+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ALL_NULL_TIMECODES(98) : move_wbs.sql(2798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            flag_level := 9999;
         Else
            flag_level := Coalesce(h_level_id, 0);
            NULL;
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C4TC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_ALL_NULL_TIMECODES', 118+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_ALL_NULL_TIMECODES(119) : move_wbs.sql(2819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_ALL_NULL_TIMECODES', 124+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_ALL_NULL_TIMECODES(125) : move_wbs.sql(2825) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_ALL_NULL_TIMECODES;
ALTER FUNCTION FC_U_ALL_NULL_TIMECODES COMPILE DEBUG;

/*****************************************************************************************/

-- Copy Assignments

/*****************************************************************************************/

Prompt == FUNCTION FC_C_ASSIGNMENTS ==
CREATE OR REPLACE FUNCTION FC_C_ASSIGNMENTS
(
     h_source_project_id IN OUT NOCOPY VARCHAR2
   , h_source_rank IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , sh_h_currency_id PLS_INTEGER
   , h_target_date IN OUT NOCOPY TIMESTAMP
   , h_source_date IN OUT NOCOPY TIMESTAMP
   , h_cur_date IN OUT NOCOPY TIMESTAMP
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_source_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_currency_id PLS_INTEGER := sh_h_currency_id;
   h_source_type PLS_INTEGER := sh_h_source_type;
   h_cur_id VARCHAR2(32);
   h_new_id VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_old_tsk_assign_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rate_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_id1 VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_dt1 TIMESTAMP;
   h_dt2 TIMESTAMP;
   h_dt10 TIMESTAMP;
   h_dt11 TIMESTAMP;
   h_dayofyear DATE;
   h_full_name VARCHAR2(255);
   h_contour VARCHAR2(8192);
   h_timecode_1_id VARCHAR2(32);
   h_timecode_2_id VARCHAR2(32);
   h_labor_cost_cc_id VARCHAR2(32);
   h_labor_revenue_cc_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_min VARCHAR2(19);
   h_duration PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_best PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_element_type PLS_INTEGER := 0;
   h_assign_status PLS_INTEGER := 0;
   h_n3 PLS_INTEGER := 0;
   h_n4 PLS_INTEGER := 0;
   h_n11 PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_rate_echange NUMBER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_task_type PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_planned_work PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_proposed_work PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   h_wbs_id VARCHAR2(32);
   i_max_min PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR C4R IS
         SELECT RA.ELEMENT_ID,RA.ELEMENT_TYPE, RA.ASSIGN_STATUS, RA.WORK_LOCATION,
         RA.RES_ASSIGNMENT_ID,RA.PARTICIPANT, RA.SECURITY_ID,
         TA.TSK_ASSIGNMENT_ID,TA.PROJECT_ASSIGNMENT_ID,TA.RESOURCE_TYPE,TA.RESOURCE_ID,
         TA.PROPOSED_START,
         TA.ASSIGNMENT_DELAY,
         T.TASK_TYPE, TA.UNITS_MSP,
         CASE WHEN TA.PROJECT_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID THEN 1 ELSE 0 END AS PA_FLAG,
         TA.PLANNED_WORK, TA.SCHED_WORK, TA.UNRESOLVED_START, TA.UNRESOLVED_DURATION,
         TA.PROPOSED_WORK, TA.PROPOSED_WORK, TA.BILLABLE, TA.CAPITAL_EXPENSE, TA.TABLE_TYPE, TA.TIMECODE_1_ID, TA.TIMECODE_2_ID,
         '' as WBS_ID, TA.LABOR_COST_CC_ID, TA.LABOR_REVENUE_CC_ID
         FROM TMT_WBS T
         JOIN TASK_ASSIGNMENTS TA
         ON TA.PROJECT_ID = h_source_project_id
         AND T.ELEMENT_ID = TA.TASK_ID AND TA.TABLE_TYPE = PMO_CNST.SC_TABLE_WBS
         JOIN RESOURCE_ASSIGNMENTS RA
         ON RA.RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID
         WHERE T.PROJECT_ID = h_source_project_id
         AND T.RANK LIKE h_source_rank || '%'
         
         UNION ALL
         
         SELECT RA.ELEMENT_ID, RA.ELEMENT_TYPE, RA.ASSIGN_STATUS, RA.WORK_LOCATION,
         RA.RES_ASSIGNMENT_ID, RA.PARTICIPANT,
         case when T.WBS_ID is null then RA.SECURITY_ID else RA.CRI_SECURITY_ID end as SECURITY_ID,
         TA.TSK_ASSIGNMENT_ID,TA.PROJECT_ASSIGNMENT_ID,TA.RESOURCE_TYPE,TA.RESOURCE_ID,
         TA.PROPOSED_START,
         TA.ASSIGNMENT_DELAY,
         CASE WHEN T.WBS_ID IS NuLL THEN 0 ELSE (SELECT W.TASK_TYPE FROM TMT_WBS W WHERE W.ELEMENT_ID = T.WBS_ID) END AS TASK_TYPE,
         TA.UNITS_MSP,
         CASE WHEN TA.PROJECT_ASSIGNMENT_ID = TA.TSK_ASSIGNMENT_ID THEN 1 ELSE 0 END AS PA_FLAG,
         TA.PLANNED_WORK, TA.SCHED_WORK, TA.UNRESOLVED_START, TA.UNRESOLVED_DURATION,
         TA.PROPOSED_WORK, TA.PROPOSED_WORK, TA.BILLABLE, TA.CAPITAL_EXPENSE, TA.TABLE_TYPE, TA.TIMECODE_1_ID, TA.TIMECODE_2_ID,
         T.WBS_ID as WBS_ID, TA.LABOR_COST_CC_ID, TA.LABOR_REVENUE_CC_ID
         FROM TMT_CRI T JOIN TASK_ASSIGNMENTS TA
         ON TA.PROJECT_ID = T.PROJECT_ID
         AND T.ELEMENT_ID = TA.TASK_ID AND TA.TABLE_TYPE = PMO_CNST.SC_TABLE_CRI
         JOIN RESOURCE_ASSIGNMENTS RA ON RA.RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID
         WHERE T.PROJECT_ID = h_source_project_id
         AND T.LEVEL_ID > 1
         AND T.PARENT_RANK LIKE 'WBS#' || h_source_rank || '%'
         
         ORDER BY RESOURCE_TYPE DESC, RESOURCE_ID, PA_FLAG, PROPOSED_START;

      CURSOR C_DELAY0 IS
         SELECT p.DAY_YEAR
         FROM PMO_TMP_CDY p
         WHERE p.DAY_YEAR > TO_DATE(SubStr(h_target_date, 1, 10));

      CURSOR CSKILL1 IS
         SELECT SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID
         FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_res_id;

      CURSOR CSKILL0 IS
         SELECT SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID
         FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_old_tsk_assign_id;

      CURSOR C41R IS
         SELECT TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_project_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_ASSIGNMENTS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_ASSIGNMENTS'), '-- ' || SYSTIMESTAMP, ':r := FC_C_ASSIGNMENTS(' || 
       '''' || h_source_project_id || ''''
          || ', ' || '''' || h_source_rank || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_currency_id), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_target_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_source_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_cur_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_source_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_ASSIGNMENTS', 163+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_ASSIGNMENTS(164) : move_wbs.sql(3003) : ' || SYSTIMESTAMP);
   END IF; 

   h_delay := 0;
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_PROF';
   PMO_DB2M.SC_STATE := 0;
   
   h_n11 := 0;
   --trace_printf("h_source_project_id: %s; h_source_rank: %s, h_target_date: %s\n", h_source_project_id, h_source_rank,h_target_date);
   
   BEGIN
      SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME, 18, 4) INTO h_rate_echange FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 182+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(183) : move_wbs.sql(3022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   -- copy Task Assignments
   
   --trace_printf("\n h_source_rank:%s", h_source_rank);
   --trace_printf("\n h_source_project_id:%s", h_source_project_id);
   
   OPEN C4R;
   
   LOOP
      FETCH C4R INTO
      h_cur_id, h_type, h_element_type, h_assign_status, h_assign_id, h_participant,
      h_security_id, h_old_tsk_assign_id, h_prj_assignment_id, h_res_type,
      h_res_id, h_start_date, h_delay, h_task_type, h_percent_units,
      h_count, h_planned_work, h_sched_work, h_dt10, h_duration,
      h_proposed_work, h_best, h_billable, h_capital_expense, h_table_type,
      h_timecode_1_id, h_timecode_2_id, h_wbs_id, h_labor_cost_cc_id, h_labor_revenue_cc_id;
      IF C4R%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      --trace_printf("SC_STATE: %d, h_start_date: %s, h_percent_units: %d\n", SC_STATE,h_start_date,h_percent_units);
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      h_start_date := '';
      
      If Coalesce(h_delay, 0) = 0 Then
         h_start_date := h_target_date;
      Else
         OPEN C_DELAY0;
         
         WHILE Coalesce(h_delay, 0) > 0 LOOP
            FETCH C_DELAY0 INTO h_start_date;
            IF C_DELAY0%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_ASSIGNMENTS', 243+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_C_ASSIGNMENTS(244) : move_wbs.sql(3083) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.RECORD_NOT_FOUND;
            End If;
                     h_delay := Coalesce(h_delay, 0) - 1;
         END LOOP;
         
         BEGIN
            CLOSE C_DELAY0;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 258+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(259) : move_wbs.sql(3098) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If isNotProjectType(Coalesce(h_source_type, 0)) <> 0 Then
         h_prj_assignment_id := NULL;
      End If;
      --trace_printf("h_prj_assignment_id: %d:%s; h_source_type: %d\n", h_prj_assignment_id_Ind,h_prj_assignment_id, h_source_type);
      
      --trace_printf("h_old_tsk_assign_id00: %s; h_effort: %d\n", h_old_tsk_assign_id, h_effort);
      
      --trace_printf("\n h_start_date:%s:%d", h_start_date, h_start_date_Ind);
      
      If (h_start_date IS NOT NULL) Then
         /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
                     {
                         InitializeHost(SC_PARAM(h_dt11),SC_PARAM(h_start_date));
                     }*/
         
         --GENERATES A NEW RESOURCE_ASSIGNMENT ID
         h_new_id := SYS_GUID();
         --trace_printf("h_participant: %d, h_security_id: %s, h_new_id: %s\n",h_participant,h_security_id,h_new_id);
         
         h_effort := (Coalesce(h_planned_work, 0) + Coalesce(h_sched_work, 0));
         --trace_printf("h_old_tsk_assign_id01: %s; h_effort: %d\n", h_old_tsk_assign_id, h_effort);
         --GET ROLE_ID
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT ROLE_ID INTO h_role_id FROM RESOURCE_ASSIGN_ROLES
               WHERE RES_ASSIGNMENT_ID = h_old_tsk_assign_id AND PRIMARY_ROLE = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 296+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(297) : move_wbs.sql(3136) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               BEGIN
                  SELECT PRIMARY_ROLE INTO h_role_id FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 310+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(311) : move_wbs.sql(3150) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  h_role_id := 'DEFAULTLEVEL';
               End If;
            End If;
            
            If (h_role_id IS NULL) Then
               h_role_id := 'DEFAULTLEVEL';
            End If;
            
            If (BITAND(Coalesce(
                  h_participant, 0), PMO_CNST.SC_TASK_RESPONSIBLE)) = PMO_CNST.SC_TASK_RESPONSIBLE Then
               h_participant := PMO_CNST.SC_TEAMMEMBER;
               h_security_id := 'SCTASKTEAMMEMBER';
            End If;
         Else
            BEGIN
               SELECT ROLE_ID INTO h_role_id FROM PROFILES WHERE PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 338+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(339) : move_wbs.sql(3178) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If Coalesce(h_effort, 0) > 0 Then
            h_finish_date := '';
         Else
            h_finish_date := h_start_date;
         End If;
         
         --trace_printf("h_prj_assignment_id: %d:%s; h_effort: %d\n", h_prj_assignment_id_Ind,h_prj_assignment_id, h_effort);
         If FC_findProfile(h_project_id, h_prj_assignment_id, h_res_id, h_new_id, h_start_date, h_effort
               , h_finish_date) <> 0 Then
            --PROFILE #
            BEGIN
               SELECT RATE_ID INTO h_rate_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_role_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 362+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(363) : move_wbs.sql(3202) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT COALESCE( MAX( PROF_NUMBER), 0) + 1 INTO h_n3
               FROM PROFILES WHERE PROJECT_ID = h_project_id AND ROLE_ID = h_role_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 376+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(377) : move_wbs.sql(3216) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --INSERT NEW PROFILE
            BEGIN
               INSERT INTO PROFILES( PROFILE_ID, PROJECT_ID, ROLE_ID, PROF_NUMBER, PROFILE_NAME)
               SELECT
               h_new_id,
               h_project_id,
               h_role_id,
               h_n3,
               PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_n3))), ')')
               FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
               WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 402+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(403) : move_wbs.sql(3242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_new_id: %s; h_n3: %d\n", SC_STATE,h_new_id, h_n3);
            
            --NEW ASSIGNMENT SKILLS
            If SubStr(h_res_type, 1, 1) = '1' Then
               OPEN CSKILL1;
               
               LOOP
                  FETCH CSKILL1 INTO h_id1, h_n3, h_n4;
                  IF CSKILL1%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> 0 Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     INSERT INTO RESOURCE_ASSIGN_SKILLS( RES_ASSIGNMENT_ID,SKILL_ID,PROFICIENCY_LEVEL_ID, EXPERIENCE_ID)
                     VALUES( h_new_id, h_id1, h_n3, h_n4);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 436+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(437) : move_wbs.sql(3276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE CSKILL1 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 450+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(451) : move_wbs.sql(3290) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --NEW ASSIGNMENT ROLE
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES(RES_ASSIGNMENT_ID,ROLE_ID,PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PRIMARY_ROLE)
                  SELECT h_new_id, ROLE_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PRIMARY_ROLE
                  FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 470+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(471) : move_wbs.sql(3310) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               OPEN CSKILL0;
               
               LOOP
                  FETCH CSKILL0 INTO h_id1, h_n3, h_n4;
                  IF CSKILL0%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> 0 Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     INSERT INTO RESOURCE_ASSIGN_SKILLS( RES_ASSIGNMENT_ID,SKILL_ID,PROFICIENCY_LEVEL_ID, EXPERIENCE_ID)
                     VALUES( h_new_id, h_id1, h_n3, h_n4);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 501+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(502) : move_wbs.sql(3341) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE CSKILL0 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 515+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(516) : move_wbs.sql(3355) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --NEW ASSIGNMENT ROLE
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES(RES_ASSIGNMENT_ID,ROLE_ID,PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PRIMARY_ROLE)
                  SELECT h_new_id, ROLE_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PRIMARY_ROLE
                  FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_old_tsk_assign_id AND ROLE_ID <> h_role_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 535+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(536) : move_wbs.sql(3375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         --INSERT A NEW RESOURCE_ASSIGNMENT
         If isProjectType(Coalesce(h_type, 0)) <> 0 Then
            h_temp_var := 0;
            h_id := h_project_id;
         Else
            h_temp_var := FC_findTask(h_cur_id, h_id, h_wbs_id);
         End If;
         
         --trace_printf("1_* h_temp_var: %d, h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_temp_var,h_prj_assignment_id_Ind,h_prj_assignment_id,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
         If Coalesce(h_temp_var, 0) = 0 Then
            h_assign_id := SYS_GUID();
            
            BEGIN
               SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_new_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 562+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(563) : move_wbs.sql(3402) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If isProjectType(Coalesce(h_type, 0)) <> 0 Then
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID INTO h_tsk_assignment_id
                  FROM TASK_ASSIGNMENTS WHERE TASK_ID = h_id AND RESOURCE_ID = h_new_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 577+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(578) : move_wbs.sql(3417) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  h_update := 1;
               Else
                  h_tsk_assignment_id := h_prj_assignment_id;
                  h_update := 0;
               End If;
               
               h_n11 := 1;
               -- Project assignments exists
            Else
               h_tsk_assignment_id := SYS_GUID();
               
               h_update := 0;
               
               If (h_prj_assignment_id IS NOT NULL) Then
                  h_n11 := 1;
                  -- Project assignments exists
               End If;
            End If;
            --trace_printf("2_* h_update: %d, h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_update,h_prj_assignment_id_Ind,h_prj_assignment_id,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
            
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';' WHERE ELEMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 615+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(616) : move_wbs.sql(3455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  UPDATE TMT_CRI SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE ELEMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 633+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(634) : move_wbs.sql(3473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If Coalesce(h_update, 0) = 0 Then
               h_temp_bitflag := 1;
               h_res_type := '1';
               
               If isProjectType(Coalesce(h_type, 0)) <> 0 Then
                  h_participant := PMO_CNST.SC_TEAMMEMBER;
                  h_security_id := 'SCPROJECTASSIGNMENT';
               End If;
               
               --trace_printf("h_task_type: %d, h_table_type: %d, h_type: %d, h_new_id: %s, h_participant: %d, h_security_id: %s\n",h_task_type,h_table_type,h_type,h_new_id,h_participant,h_security_id);
               h_temp_var := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_new_id, h_type, h_participant, h_security_id
                                     , h_res_type, h_temp_bitflag);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGNMENTS(
                  RES_ASSIGNMENT_ID,
                  ELEMENT_ID,
                  ELEMENT_TYPE,
                  PROJECT_ID,
                  RESOURCE_TYPE,
                  RESOURCE_ID,
                  ASSIGN_STATUS,
                  WORK_LOCATION,
                  PARTICIPANT,
                  SECURITY_ID,
                  REC_STATUS,
                  REC_DATETIME,TABLE_TYPE)
                  VALUES(
                  h_assign_id,
                  h_id,
                  h_type,
                  h_project_id,
                  h_res_type,
                  h_new_id,
                  h_element_type,
                  h_assign_status,
                  h_participant,
                  h_security_id,
                  'U',
                  h_cur_date,h_table_type);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 689+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(690) : move_wbs.sql(3529) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            --trace_printf("h_type: %d, h_task_type: %d, h_duration: %d, h_effort: %d, h_dt11: %s\n",h_type,h_task_type,h_duration,h_effort,h_dt11);
            --trace_printf("h_old_tsk_assign_id: %s, h_dt10: %s\n",h_old_tsk_assign_id,h_dt10);
            If Coalesce(h_effort, 0) > 0 Then
               BEGIN
                  SELECT MINUTE_PER_DAY INTO h_min_per_day FROM PMO_TMP_MIN_MAX;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 706+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(707) : move_wbs.sql(3546) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_duration: %d, h_min_per_day: %d\n",SC_STATE,h_duration,h_min_per_day);//trace_flush();
               
               /*if ( isWBSTask(h_type)  (h_task_type == SC_FD_EFFORT_DRIVEN) )
                                   {
                                       // get first day
                                       BEGIN_SQL
                                           SELECT SCHED_MIN, BIT_FLAG, PARENT_ID, BIT_SCHED
                                           INTO HOST(h_minutes), HOST(h_bit_flag), HOST(h_parent_id), HOST(h_bitID)
                                           FROM TASK_DAY_OF_YEAR
                                           WHERE PROJECT_ID = HOST(h_source_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_old_tsk_assign_id)
                                               AND DAY_YEAR = DATE(HOST(h_dt10));
                                       END_SQL_NOCHECK;
               //trace_printf("SC_STATE: %d, h_minutes: %d,h_bit_flag: %s,h_parent_id: %s\n",SC_STATE,h_minutes,h_bit_flag,h_parent_id);//trace_flush();
               
                                       BEGIN_SQL
                                           SELECT COUNT(*) INTO HOST(h_count) FROM TASK_DAY_OF_YEAR
                                           WHERE PROJECT_ID = HOST(h_source_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_old_tsk_assign_id);
                                       END_SQL_NOCHECK;
               
                                       h_duration = int((h_duration / h_min_per_day) + 0.5);
               
                                       sprintf(h_min, "%d", h_duration);
                                       stmt.strcpy("SELECT DAY_YEAR FROM session.PMO_TMP_CDY WHERE DAY_YEAR >= DATE('");
                                       stmt.strcat(h_dt11);
                                       stmt.strcat("') FETCH FIRST ");
                                       stmt.strcat(h_min);
                                       stmt.strcat("  ROWS ONLY");
               //trace_printf("PROFILE:\n%s\n", (char *) stmt);
               
                                       BEGIN_SQL
                                           PREPARE S_FDI2 FROM :stmt;
                                       END_SQL;
               
                                       BEGIN_SQL
                                           DECLARE C_FDI2 CURSOR WITH HOLD FOR S_FDI2;
                                       END_SQL_NOCHECK;
               
                                       BEGIN_SQL
                                           OPEN C_FDI2 ;
                                       END_SQL;
               
                                       h_total = 0;
                                       while (1==1)
                                       {
                                           BEGIN_SQL
                                               FETCH C_FDI2 INTO HOST(h_dayofyear) ;
                                           END_SQL;
                                           if (SC_NOSUCCESS) break;
               
                                           BEGIN_SQL
                                               INSERT INTO TASK_DAY_OF_YEAR (
                                                   PROJECT_ID,
                                                   RESOURCE_ID,
                                                   TSK_ASSIGNMENT_ID,
                                                   DAY_YEAR,
                                                   BIT_FLAG,
                                                   SCHED_MIN,
                                                   PARENT_ID,
                                                   BIT_SCHED,
                                                   ACTUAL_FLAG,
                                                   TASK_ID,
                                                   TABLE_TYPE)
                                               VALUES (
                                                   HOST(h_project_id),
                                                   HOST(h_res_id),
                                                   HOST(h_tsk_assignment_id),
                                                   HOST(h_dayofyear),
                                                   HOST(h_bit_flag),
                                                   HOST(h_minutes),
                                                   HOST(h_parent_id),
                                                   HOST(h_bitID),
                                                   0,
                                                   HOST(h_new_id),
                                                   HOST(h_table_type));
                                           END_SQL;
               //trace_printf("1.2====> SC_STATE: %d, h_total: %d, h_minutes: %d, h_dayofyear: %s\n",SC_STATE,h_total,h_minutes,h_dayofyear);//trace_flush();
               
                                           h_total ++;
                                       }
               
                                       BEGIN_SQL
                                           CLOSE C_FDI2 WITH RELEASE;
                                       END_SQL;
               
                                       InitializeHost(SC_PARAM(h_start_date),SC_PARAM(h_dt11));
                                       BEGIN_SQL
                                           SELECT TIMESTAMP(DATE(HOST(h_dayofyear)), TIME(LAST_AVAIL_TIME(BIT_SCHED)))
                                           INTO HOST(h_finish_date)
                                           FROM session.PMO_TMP_CDY WHERE DAY_YEAR = HOST(h_dayofyear);
                                       END_SQL;
               //trace_printf("1.3 h_total: %d, h_count: %d, h_dt10: %s, h_start_date: 5s\n",h_total,h_count,h_dt10,h_start_date);//trace_flush();
               
                                       if (h_total != h_count)
                                       {
                                           h_percent_units = int(((double) h_total / (double) h_count) * 10000.00 + 0.5);
                                           h_minutes = int( ((double) h_minutes * ((double) h_total / (double) h_count)) + 0.5);
               
                                           h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
                                                                     SC_PARAM(h_tsk_assignment_id),
                                                                     SC_PARAM(h_start_date),
                                                                     SC_PARAM(h_min_per_day),
                                                                     SC_PARAM(h_minutes));
                                       }
                                   }
                                   else
                                   {*/
               If isNotProjectType(Coalesce(h_type, 0)) <> 0 Then
                  null_return := FC_INI_ASSIGN_STR(1);
                  -- create temp table PMO_ASSIGN_STR
                  h_contour := '';
               End If;
               h_julian_day := 0;
               h_bitflag := 0;
               
               h_total := Coalesce(h_effort, 0);
               h_minutes := Coalesce(h_effort, 0);
               h_dt1 := h_start_date;
               h_dt2 := h_finish_date;
               
               WHILE Coalesce(h_minutes, 0) > 0 LOOP
                  h_minutes := FC_getNextDate(h_bitflag, h_dt1, h_minutes, h_dt2, h_bitID);
                  
                  If (Coalesce(h_bitflag, 0) = 0) AND (PMO_ORASTD.strcmp(h_start_date, h_dt1) <> 0) Then
                     h_start_date := h_dt1;
                  End If;
                  
                  If Coalesce(h_minutes, 0) < 0 Then
                     EXIT;
                  ElsIf Coalesce(h_minutes, 0) > 0 Then
                     --trace_printf("3_* h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
                     If (h_prj_assignment_id IS NULL) Then
                        BEGIN
                           INSERT INTO TASK_DAY_OF_YEAR( PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG, TABLE_TYPE )
                           VALUES( h_project_id, h_id, h_new_id, h_tsk_assignment_id, h_tsk_assignment_id, TO_DATE(SubStr(h_dt1, 1, 10)), h_minutes, h_bitID, '0', h_table_type );
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 854+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(855) : move_wbs.sql(3694) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           INSERT INTO TASK_DAY_OF_YEAR( PROJECT_ID, TASK_ID, RESOURCE_ID, TSK_ASSIGNMENT_ID, PARENT_ID, DAY_YEAR, SCHED_MIN, BIT_SCHED, BIT_FLAG, TABLE_TYPE )
                           VALUES( h_project_id, h_id, h_new_id, h_tsk_assignment_id, h_prj_assignment_id, TO_DATE(SubStr(h_dt1, 1, 10)), h_minutes, h_bitID, '0', h_table_type );
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 872+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(873) : move_wbs.sql(3712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     BEGIN
                        SELECT PMO_ORASTD.JULIAN_DAY(h_dt1) INTO h_julian_day FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 886+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(887) : move_wbs.sql(3726) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If isNotProjectType(Coalesce(h_type, 0)) <> 0 Then
                        null_return := FC_ASSIGN_STRING(h_julian_day, h_minutes);
                     End If;
                     
                     If Coalesce(h_minutes, 0) > i_max_min Then
                        i_max_min := Coalesce(h_minutes, 0);
                     End If;
                  End If;
                  
                  If Coalesce(h_total, 0) = 0 Then
                     EXIT;
                  ElsIf Coalesce(h_total, 0) < Coalesce(h_minutes, 0) Then
                     h_minutes := Coalesce(h_total, 0);
                  Else
                     h_total := Coalesce(h_total, 0) - Coalesce(h_minutes, 0);
                     h_minutes := Coalesce(h_total, 0);
                  End If;
                  h_bitflag := Coalesce(h_bitflag, 0) + 1;
                  
                  BEGIN
                     SELECT TO_DATE(SubStr(h_dt1, 1, 10)) + 1      INTO h_dt1
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 919+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(920) : move_wbs.sql(3759) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  --This call is to fix the start date..
                  h_temp_bitflag := 0;
                  h_temp_var := FC_GET_DT(h_dt1, h_temp_bitflag);
               END LOOP;
               
               If isProjectType(Coalesce(h_type, 0)) <> 0 Then
                  h_contour := NULL;
               Else
                  h_temp_var := FC_GET_ASSIGN_STR(h_contour, 0);
                  -- get the final assign string
               End If;
               --}
            End If;
            
            --trace_printf("h_update: %d, h_prj_assignment_id: %d:%s, h_tsk_assignment_id: %d:%s\n",h_update,h_prj_assignment_id_Ind,h_prj_assignment_id,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
            
            If Coalesce(h_update, 0) = 0 Then
               --trace_printf("i_max_min: %d, h_percent_units: %d, h_min_per_day: %d\n",i_max_min,h_percent_units,h_min_per_day);
               
               --[Asad]: We have commented out this code, due to misbehavior of Units% when
               --proposal is created from default project with a task of 4 hours
               --It may be uncommented at later time.
               /*if (i_max_min > 0)
                                   {
                                       h_percent_units = ((double) i_max_min / (double) h_min_per_day) * 10000.00;
                                   }
                                   */
               --trace_printf("*********** h_percent_units: %d\n",h_percent_units);
               
               If (h_rate_id IS NOT NULL) Then
                  BEGIN
                     SELECT REGULAR_COST * h_rate_echange, SPECIAL_COST * h_rate_echange, REGULAR_SALE * h_rate_echange, SPECIAL_SALE * h_rate_echange
                     INTO h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale
                     FROM RATES WHERE RATE_ID = h_rate_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 964+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(965) : move_wbs.sql(3804) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(
                     TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID, RESOURCE_ID,RESOURCE_TYPE,
                     SD, FD, START_DT, FINISH_DT, DURATION, --UNITS_MSP,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, ETC, PLANNED_DURATION,
                     PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK, PROPOSED_DURATION,
                     ASSIGN_STRING, BEST, BIT_FLAG, TABLE_TYPE, PROJECT_ASSIGNMENT_ID,
                     REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE, BILLABLE, CAPITAL_EXPENSE, TIMECODE_1_ID, TIMECODE_2_ID,
                     LABOR_COST_CC_ID,LABOR_REVENUE_CC_ID )
                     VALUES (h_tsk_assignment_id, h_assign_id, h_id, h_project_id, h_new_id, '1',
                     PMO_CNST.SC_NS, PMO_CNST.SC_NF, h_start_date, h_finish_date, h_effort, --h_percent_units,
                     h_start_date, h_finish_date, h_effort, h_effort, h_effort,
                     h_start_date, h_finish_date, h_effort,  h_effort,
                     h_contour, h_best, '0', h_table_type, h_prj_assignment_id,
                     h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
                     h_billable, h_capital_expense,h_timecode_1_id, h_timecode_2_id,
                     h_labor_cost_cc_id, h_labor_revenue_cc_id );
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 996+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(997) : move_wbs.sql(3836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_reg_cost := 0.00;
                  h_reg_sale := 0.00;
                  
                  BEGIN
                     INSERT INTO TASK_ASSIGNMENTS(
                     TSK_ASSIGNMENT_ID, RES_ASSIGNMENT_ID, TASK_ID, PROJECT_ID, RESOURCE_ID,RESOURCE_TYPE,
                     SD, FD, START_DT, FINISH_DT, DURATION, --UNITS_MSP,
                     PLANNED_START, PLANNED_FINISH, PLANNED_WORK, ETC, PLANNED_DURATION,
                     PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK, PROPOSED_DURATION,
                     ASSIGN_STRING, BEST, BIT_FLAG, TABLE_TYPE, PROJECT_ASSIGNMENT_ID, BILLABLE, CAPITAL_EXPENSE,
                     TIMECODE_1_ID, TIMECODE_2_ID, LABOR_COST_CC_ID,LABOR_REVENUE_CC_ID )
                     VALUES(h_tsk_assignment_id, h_assign_id, h_id, h_project_id, h_new_id, '1',
                     PMO_CNST.SC_NS, PMO_CNST.SC_NF, h_start_date, h_finish_date, h_effort,--h_percent_units,
                     h_start_date, h_finish_date, h_effort, h_effort,  h_effort,
                     h_start_date, h_finish_date, h_effort, h_effort,
                     h_contour, h_best, '0', h_table_type, h_prj_assignment_id, h_billable, h_capital_expense,
                     h_timecode_1_id, h_timecode_2_id, h_labor_cost_cc_id, h_labor_revenue_cc_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 1028+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(1029) : move_wbs.sql(3868) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If isTask(Coalesce(h_type, 0)) <> 0 Then
                  h_temp_sc_state := FC_UPDATE_LABOR_BUDGET(h_project_id, h_id, h_tsk_assignment_id, h_rec_user);
                  
                  h_tph_delete := 0;
                  h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_id, h_tsk_assignment_id, h_start_date
                                          , h_finish_date, h_reg_cost, h_reg_sale, h_tph_delete
                                          , h_capital_expense, h_billable);
                  
                  /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
                                          {
                                              InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
                                              h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var), h_min_per_day, h_effort);
                                          }*/
               End If;
            Else
               BEGIN
                  SELECT PLANNED_WORK + h_effort,
                  CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) < TO_DATE(SubStr(PLANNED_START, 1, 10)) THEN h_start_date ELSE PLANNED_START END,
                  CASE WHEN TO_DATE(SubStr(h_finish_date, 1, 10)) < TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) THEN h_finish_date ELSE PLANNED_FINISH END
                  INTO h_effort, h_start_date, h_finish_date
                  FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 1061+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(1062) : move_wbs.sql(3901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET UNITS_MSP = h_percent_units,
                  START_DT = h_start_date, FINISH_DT = h_finish_date, DURATION = h_effort,
                  PLANNED_START = h_start_date, PLANNED_FINISH = h_finish_date,
                  PLANNED_WORK = h_effort, PLANNED_DURATION = h_effort,
                  PROPOSED_START = h_start_date, PROPOSED_FINISH = h_finish_date,
                  PROPOSED_WORK = h_effort, PROPOSED_DURATION = h_effort
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 1084+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(1085) : move_wbs.sql(3924) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C4R ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 1101+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(1102) : move_wbs.sql(3941) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- TODO(Tudor): move U_ASSIGN_STRING in Rollup
   If Coalesce(h_n11, 0) = 1 Then
      OPEN C41R;
      
      LOOP
         FETCH C41R INTO h_prj_assignment_id;
         IF C41R%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         -- CALL UPDATE assign string function
         h_temp_bitflag := 0;
         h_temp_var := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_bitflag, 0, 0);
      END LOOP;
      
      BEGIN
         CLOSE C41R ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_ASSIGNMENTS', 1134+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_ASSIGNMENTS(1135) : move_wbs.sql(3974) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_ASSIGNMENTS', 1141+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_ASSIGNMENTS(1142) : move_wbs.sql(3981) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_ASSIGNMENTS;
ALTER FUNCTION FC_C_ASSIGNMENTS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_ASSIGN_CHARGECODE ==
CREATE OR REPLACE FUNCTION FC_ASSIGN_CHARGECODE
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_b_chargecode_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_b_rank VARCHAR2(200);
   h_b_rankBase64 VARCHAR2(8);
   h_b_level_id PLS_INTEGER := 0;
   h_b_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_ASSIGN_CHARGECODE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_ASSIGN_CHARGECODE'), '-- ' || SYSTIMESTAMP, ':r := FC_ASSIGN_CHARGECODE(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_b_chargecode_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSIGN_CHARGECODE', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_ASSIGN_CHARGECODE(22) : move_wbs.sql(4013) : ' || SYSTIMESTAMP);
   END IF; 

   -- Check out if the charge code is already assigned to the project
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_CHARGECODES
      WHERE ELEMENT_ID = h_project_id AND PARENT_ID = h_b_chargecode_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSIGN_CHARGECODE', 34+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSIGN_CHARGECODE(35) : move_wbs.sql(4026) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      -- Get next sibling count
      PMODEFINES.PMO_GET_MAX_RANK('TMT_CHARGECODES', h_b_chargecode_id, h_b_max_rank);
      PMO_STPU.toBase64(Coalesce(h_b_max_rank, 0), h_b_rankBase64);
      
      BEGIN
         INSERT INTO TMT_CHARGECODES( RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, REC_USER)
         SELECT RTRIM(RANK) || h_b_rankBase64, h_b_chargecode_id, h_project_id, h_type_id, LEVEL_ID + 1, h_rec_user
         FROM TMT_CHARGECODES WHERE ELEMENT_ID = h_b_chargecode_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_ASSIGN_CHARGECODE', 58+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_ASSIGN_CHARGECODE(59) : move_wbs.sql(4050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_ASSIGN_CHARGECODE', 65+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_ASSIGN_CHARGECODE(66) : move_wbs.sql(4057) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_ASSIGN_CHARGECODE;
ALTER FUNCTION FC_ASSIGN_CHARGECODE COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_C_BUDGETS ==
CREATE OR REPLACE FUNCTION FC_C_BUDGETS
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_source_project_id IN OUT NOCOPY VARCHAR2
   , source_rank001 IN OUT NOCOPY VARCHAR2
   , sh_h_source_type PLS_INTEGER
   , h_cur_date IN OUT NOCOPY TIMESTAMP
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_source_type PLS_INTEGER := sh_h_source_type;
   h_b_billable VARCHAR2(1);
   h_b_budget_id VARCHAR2(32);
   h_b_chargecode_id VARCHAR2(32);
   h_b_element_id VARCHAR2(32);
   h_b_new_id VARCHAR2(32);
   h_id1 VARCHAR2(32);
   h_id2 VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_b_current_budget NUMBER := 0.00;
   h_b_estimation_budget NUMBER := 0.00;
   h_b_estimation_total NUMBER := 0.00;
   h_b_forecast_budget NUMBER := 0.00;
   h_b_r_current_budget NUMBER := 0.00;
   h_b_r_estimation_budget NUMBER := 0.00;
   h_b_r_estimation_total NUMBER := 0.00;
   h_b_r_forecast_budget NUMBER := 0.00;
   h_b_s_current_budget NUMBER := 0.00;
   h_b_s_estimation_budget NUMBER := 0.00;
   h_b_s_estimation_total NUMBER := 0.00;
   h_b_s_forecast_budget NUMBER := 0.00;
   h_b_auto_flag PLS_INTEGER := 0;
   h_b_type_id PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_BUDGETS IS
         SELECT B.ELEMENT_ID, B.CHARGECODE_ID, T.ELEMENT_ID
         FROM PMO_COPY_WBS T, BUDGETS B
         WHERE B.PROJECT_ID = h_source_project_id
         AND T.OLD_WBS_ID = B.ELEMENT_ID
         AND B.CAPITAL_EXPENSE < PMO_CNST.SC_TIMEPHASE_CALENDAR
         
         UNION ALL
         
         SELECT B.ELEMENT_ID, B.CHARGECODE_ID, T.ELEMENT_ID
         FROM PMO_COPY_CRI T, BUDGETS B
         WHERE B.PROJECT_ID = h_source_project_id
         AND T.OLD_ID = B.ELEMENT_ID
         AND B.CAPITAL_EXPENSE < PMO_CNST.SC_TIMEPHASE_CALENDAR;

      CURSOR C_BUDGETS1 IS
         SELECT B.BUDGET_ID, AB.BUDGET_ID
         FROM PMO_COPY_BUDGETS B
         JOIN TMT_CHARGECODES C
         ON C.ELEMENT_ID = B.CHARGECODE_ID
         LEFT JOIN PMO_COPY_BUDGETS AB
         ON AB.CHARGECODE_ID = C.ASSOCIATE_CHARGECODE_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_BUDGETS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_BUDGETS'), '-- ' || SYSTIMESTAMP, ':r := FC_C_BUDGETS(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_source_project_id || ''''
          || ', ' || '''' || source_rank001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_source_type), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_cur_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_BUDGETS', 70+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_BUDGETS(71) : move_wbs.sql(4138) : ' || SYSTIMESTAMP);
   END IF; 

   h_source_rank := source_rank001;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_BUDGETS';
   PMO_DB2M.SC_STATE := 0;
   
   OPEN C_BUDGETS;
   
   LOOP
      FETCH C_BUDGETS INTO h_b_element_id, h_b_chargecode_id, h_b_new_id;
      IF C_BUDGETS%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If isNotProjectType(Coalesce(h_source_type, 0)) <> 0 Then
         h_temp_var := FC_ASSIGN_CHARGECODE(h_project_id, h_b_chargecode_id, h_rec_user);
      End If;
      
      h_b_budget_id := SYS_GUID();
      
      If PMO_ORASTD.strcmp(h_b_element_id, h_source_project_id) = PMO_CNST.SC_MATCH Then
         h_b_new_id := h_project_id;
      End If;
      
      BEGIN
         INSERT INTO PMO_COPY_BUDGETS (
         BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CHARGECODE_ID,
         CREATED_BY, REC_USER, REC_STATUS, REC_DATETIME,
         PROB_PERCENT, PROB_AMOUNT, ESTIMATED_PROB_AMOUNT, ESTIMATED_PROB_AMOUNT_TOTAL,
         TOTAL_ESTIMATION_PROB_AMOUNT, TOTAL_BUDGET_PROB_AMOUNT, TOLERANCE,
         ESTIMATED_TOTAL, ESTIMATED_BUDGET,
         BUDGET, AUTO_FLAG, ACTUAL_EDITED, TIMEPHASED_FLAG,
         CAPITAL_EXPENSE, BILLABLE, DATAFIELD_1, DATAFIELD_2,
         TXT_FIELD_01, TXT_FIELD_02, TXT_FIELD_03, TXT_FIELD_04, TXT_FIELD_05,
         TXT_FIELD_06, TXT_FIELD_07, TXT_FIELD_08, TXT_FIELD_09, TXT_FIELD_10 )
         SELECT
         h_b_budget_id, h_project_id, h_b_new_id, TYPE_ID, h_b_chargecode_id,
         h_rec_user, h_rec_user, 'U', h_cur_date,
         PROB_PERCENT, PROB_AMOUNT, ESTIMATED_PROB_AMOUNT, ESTIMATED_PROB_AMOUNT_TOTAL,
         TOTAL_ESTIMATION_PROB_AMOUNT, TOTAL_BUDGET_PROB_AMOUNT, TOLERANCE,
         ESTIMATED_TOTAL, ESTIMATED_BUDGET,
         BUDGET, AUTO_FLAG, ACTUAL_EDITED, TIMEPHASED_FLAG,
         CAPITAL_EXPENSE, BILLABLE, DATAFIELD_1, DATAFIELD_2,
         TXT_FIELD_01, TXT_FIELD_02, TXT_FIELD_03, TXT_FIELD_04, TXT_FIELD_05,
         TXT_FIELD_06, TXT_FIELD_07, TXT_FIELD_08, TXT_FIELD_09, TXT_FIELD_10
         FROM BUDGETS
         WHERE PROJECT_ID = h_source_project_id
         AND ELEMENT_ID = h_b_element_id
         AND CHARGECODE_ID = h_b_chargecode_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_BUDGETS', 137+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_BUDGETS(138) : move_wbs.sql(4205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_BUDGETS ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_BUDGETS', 151+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_BUDGETS(152) : move_wbs.sql(4219) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_BUDGETS1;
   
   LOOP
      FETCH C_BUDGETS1 INTO h_id1, h_id2;
      IF C_BUDGETS1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      BEGIN
         UPDATE PMO_COPY_BUDGETS
         SET ASSOCIATE_BUDGET_ID = h_id2
         WHERE BUDGET_ID = h_id1;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_BUDGETS', 183+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_BUDGETS(184) : move_wbs.sql(4251) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_BUDGETS1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_BUDGETS', 197+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_BUDGETS(198) : move_wbs.sql(4265) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO BUDGETS (
      BUDGET_ID, ASSOCIATE_BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CHARGECODE_ID,
      CREATED_BY, REC_USER, REC_STATUS, REC_DATETIME,
      PROB_PERCENT, PROB_AMOUNT, ESTIMATED_PROB_AMOUNT, ESTIMATED_PROB_AMOUNT_TOTAL,
      TOTAL_ESTIMATION_PROB_AMOUNT, TOTAL_BUDGET_PROB_AMOUNT, TOLERANCE,
      ESTIMATED_TOTAL, ESTIMATED_ETC_TOTAL, ESTIMATED_BUDGET, ESTIMATED_ETC,
      BUDGET, ETC, AUTO_FLAG, TIMEPHASED_FLAG,
      CAPITAL_EXPENSE, BILLABLE, DATAFIELD_1, DATAFIELD_2,
      TXT_FIELD_01, TXT_FIELD_02, TXT_FIELD_03, TXT_FIELD_04, TXT_FIELD_05,
      TXT_FIELD_06, TXT_FIELD_07, TXT_FIELD_08, TXT_FIELD_09, TXT_FIELD_10,
      ROLLUP_BITS )
      SELECT
      BUDGET_ID, ASSOCIATE_BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CHARGECODE_ID,
      CREATED_BY, REC_USER, REC_STATUS, REC_DATETIME,
      PROB_PERCENT, PROB_AMOUNT, ESTIMATED_PROB_AMOUNT, ESTIMATED_PROB_AMOUNT_TOTAL,
      TOTAL_ESTIMATION_PROB_AMOUNT, TOTAL_BUDGET_PROB_AMOUNT, TOLERANCE,
      ESTIMATED_TOTAL, ESTIMATED_TOTAL, ESTIMATED_BUDGET, ESTIMATED_BUDGET,
      BUDGET, BUDGET, AUTO_FLAG, TIMEPHASED_FLAG,
      CAPITAL_EXPENSE, BILLABLE, DATAFIELD_1, DATAFIELD_2,
      TXT_FIELD_01, TXT_FIELD_02, TXT_FIELD_03, TXT_FIELD_04, TXT_FIELD_05,
      TXT_FIELD_06, TXT_FIELD_07, TXT_FIELD_08, TXT_FIELD_09, TXT_FIELD_10,
      PMO_CNST.SC_UPDATE_BUDGET_BIT
      FROM PMO_COPY_BUDGETS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_BUDGETS', 236+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_BUDGETS(237) : move_wbs.sql(4304) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_BUDGETS', 242+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_BUDGETS(243) : move_wbs.sql(4310) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_BUDGETS;
ALTER FUNCTION FC_C_BUDGETS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_C_SYS_EST_BUDGETS ==
CREATE OR REPLACE FUNCTION FC_C_SYS_EST_BUDGETS
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_source_project_id IN OUT NOCOPY VARCHAR2
   , source_rank001 IN OUT NOCOPY VARCHAR2
   , sh_h_source_type PLS_INTEGER
   , h_cur_date IN OUT NOCOPY TIMESTAMP
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_source_type PLS_INTEGER := sh_h_source_type;
   h_b_element_id VARCHAR2(32);
   h_b_new_id VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_temp_var PLS_INTEGER := 0;
   h_wbs_id VARCHAR2(32);
-- Declare Cursors
      CURSOR C_SEB IS
         SELECT W.ELEMENT_ID
         FROM TMT_WBS W
         WHERE W.RANK LIKE h_source_rank || '%'
         AND W.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3,PMO_CNST.SC_DELIVERABLE,PMO_CNST.SC_DELIVERABLE_TYPE1,PMO_CNST.SC_DELIVERABLE_TYPE2,PMO_CNST.SC_DELIVERABLE_TYPE3,PMO_CNST.SC_WORK_PRODUCT,PMO_CNST.SC_SUMMARY_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST,PMO_CNST.SC_WBS_RISK,PMO_CNST.SC_WBS_ISSUE,PMO_CNST.SC_WBS_REQUIREMENT,PMO_CNST.SC_WBS_DEFECT,PMO_CNST.SC_WBS_SERVICE_REQUEST,PMO_CNST.SC_WBS_ACTION_ITEM);

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_SYS_EST_BUDGETS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_SYS_EST_BUDGETS'), '-- ' || SYSTIMESTAMP, ':r := FC_C_SYS_EST_BUDGETS(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_source_project_id || ''''
          || ', ' || '''' || source_rank001 || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_source_type), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_cur_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_SYS_EST_BUDGETS', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_SYS_EST_BUDGETS(36) : move_wbs.sql(4356) : ' || SYSTIMESTAMP);
   END IF; 

   h_source_rank := source_rank001;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   
   OPEN C_SEB;
   
   LOOP
      FETCH C_SEB INTO h_b_element_id;
      IF C_SEB%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If PMO_ORASTD.strcmp(h_b_element_id, h_source_project_id) = PMO_CNST.SC_MATCH Then
         h_b_new_id := h_project_id;
      Else
         h_wbs_id := NULL;
         h_temp_var := FC_findTask(h_b_element_id, h_b_new_id, h_wbs_id);
      End If;
      
      BEGIN
         UPDATE BUDGETS B
         SET (   B.PROB_PERCENT, B.PROB_AMOUNT, B.ESTIMATED_PROB_AMOUNT, B.ESTIMATED_PROB_AMOUNT_TOTAL,
         B.TOTAL_ESTIMATION_PROB_AMOUNT, B.TOTAL_BUDGET_PROB_AMOUNT, B.TOLERANCE,
         B.ESTIMATED_TOTAL, B.ESTIMATED_ETC_TOTAL, B.ESTIMATED_BUDGET, B.ESTIMATED_ETC,
         B.BUDGET, B.AUTO_FLAG, B.ACTUAL_EDITED,
         B.DATAFIELD_1, B.DATAFIELD_2,
         B.TXT_FIELD_01, B.TXT_FIELD_02, B.TXT_FIELD_03, B.TXT_FIELD_04, B.TXT_FIELD_05,
         B.TXT_FIELD_06, B.TXT_FIELD_07, B.TXT_FIELD_08, B.TXT_FIELD_09, B.TXT_FIELD_10 ) =
         ( SELECT
         OB.PROB_PERCENT, OB.PROB_AMOUNT, OB.ESTIMATED_PROB_AMOUNT, OB.ESTIMATED_PROB_AMOUNT_TOTAL,
         OB.TOTAL_ESTIMATION_PROB_AMOUNT, OB.TOTAL_BUDGET_PROB_AMOUNT, OB.TOLERANCE,
         OB.ESTIMATED_TOTAL, OB.ESTIMATED_TOTAL, OB.ESTIMATED_BUDGET, OB.ESTIMATED_BUDGET,
         OB.BUDGET, OB.AUTO_FLAG, OB.ACTUAL_EDITED,
         OB.DATAFIELD_1, OB.DATAFIELD_2,
         OB.TXT_FIELD_01, OB.TXT_FIELD_02, OB.TXT_FIELD_03, OB.TXT_FIELD_04, OB.TXT_FIELD_05,
         OB.TXT_FIELD_06, OB.TXT_FIELD_07, OB.TXT_FIELD_08, OB.TXT_FIELD_09, OB.TXT_FIELD_10
         FROM BUDGETS OB
         WHERE OB.PROJECT_ID = h_source_project_id
         AND OB.ELEMENT_ID = h_b_element_id
         AND OB.CHARGECODE_ID = B.CHARGECODE_ID
         )
         WHERE B.PROJECT_ID = h_project_id
         AND B.ELEMENT_ID = h_b_new_id
         AND B.CAPITAL_EXPENSE >= PMO_CNST.SC_TIMEPHASE_CALENDAR;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_SYS_EST_BUDGETS', 109+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_SYS_EST_BUDGETS(110) : move_wbs.sql(4430) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO BUDGETS (
         PROJECT_ID, ELEMENT_ID, CHARGECODE_ID, BUDGET_ID,
         TYPE_ID, CREATED_BY, REC_USER,
         PROB_PERCENT, PROB_AMOUNT, ESTIMATED_PROB_AMOUNT, ESTIMATED_PROB_AMOUNT_TOTAL,
         TOTAL_ESTIMATION_PROB_AMOUNT, TOTAL_BUDGET_PROB_AMOUNT, TOLERANCE,
         ESTIMATED_TOTAL, ESTIMATED_ETC_TOTAL, ESTIMATED_BUDGET, ESTIMATED_ETC,
         BUDGET, AUTO_FLAG, ACTUAL_EDITED,
         CAPITAL_EXPENSE, BILLABLE, DATAFIELD_1, DATAFIELD_2,
         TXT_FIELD_01, TXT_FIELD_02, TXT_FIELD_03, TXT_FIELD_04, TXT_FIELD_05,
         TXT_FIELD_06, TXT_FIELD_07, TXT_FIELD_08, TXT_FIELD_09, TXT_FIELD_10 )
         SELECT
         h_project_id, h_b_new_id, OB.CHARGECODE_ID, SYS_GUID(),
         OB.TYPE_ID, h_rec_user, h_rec_user,
         OB.PROB_PERCENT, OB.PROB_AMOUNT, OB.ESTIMATED_PROB_AMOUNT, OB.ESTIMATED_PROB_AMOUNT_TOTAL,
         OB.TOTAL_ESTIMATION_PROB_AMOUNT, OB.TOTAL_BUDGET_PROB_AMOUNT, OB.TOLERANCE,
         OB.ESTIMATED_TOTAL, OB.ESTIMATED_TOTAL, OB.ESTIMATED_BUDGET, OB.ESTIMATED_BUDGET,
         OB.BUDGET, OB.AUTO_FLAG, OB.ACTUAL_EDITED,
         OB.CAPITAL_EXPENSE, OB.BILLABLE, OB.DATAFIELD_1, OB.DATAFIELD_2,
         OB.TXT_FIELD_01, OB.TXT_FIELD_02, OB.TXT_FIELD_03, OB.TXT_FIELD_04, OB.TXT_FIELD_05,
         OB.TXT_FIELD_06, OB.TXT_FIELD_07, OB.TXT_FIELD_08, OB.TXT_FIELD_09, OB.TXT_FIELD_10
         FROM BUDGETS OB
         WHERE OB.PROJECT_ID = h_source_project_id
         AND OB.ELEMENT_ID = h_b_element_id
         AND OB.CAPITAL_EXPENSE >= PMO_CNST.SC_TIMEPHASE_CALENDAR
         AND OB.CHARGECODE_ID NOT IN (SELECT CHARGECODE_ID FROM BUDGETS WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_b_new_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_SYS_EST_BUDGETS', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_SYS_EST_BUDGETS(152) : move_wbs.sql(4472) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_SEB ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_SYS_EST_BUDGETS', 165+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_SYS_EST_BUDGETS(166) : move_wbs.sql(4486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_SYS_EST_BUDGETS', 171+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_SYS_EST_BUDGETS(172) : move_wbs.sql(4492) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_SYS_EST_BUDGETS;
ALTER FUNCTION FC_C_SYS_EST_BUDGETS COMPILE DEBUG;

/*****************************************************************************/

Prompt == FUNCTION FC_SEARCH_IDLIST ==
CREATE OR REPLACE FUNCTION FC_SEARCH_IDLIST
(
     h_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_temp_id VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR idlistRecset IS
         SELECT ELEMENT_ID
         FROM PMO_TMP_MVW_IDLIST
         WHERE  ROWNUM <= 1 AND  ELEMENT_ID = h_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SEARCH_IDLIST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SEARCH_IDLIST'), '-- ' || SYSTIMESTAMP, ':r := FC_SEARCH_IDLIST(' || 
       '''' || h_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_IDLIST', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SEARCH_IDLIST(21) : move_wbs.sql(4523) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN idlistRecset;
   
   FETCH idlistRecset INTO h_temp_id;
   IF idlistRecset%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   h_temp_sc_state := 1;
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      h_temp_sc_state := 0;
   End If;
   
   BEGIN
      CLOSE idlistRecset ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SEARCH_IDLIST', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SEARCH_IDLIST(46) : move_wbs.sql(4548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_IDLIST', 51+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_IDLIST(52) : move_wbs.sql(4554) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_temp_sc_state, 0);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SEARCH_IDLIST', 56+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SEARCH_IDLIST(57) : move_wbs.sql(4559) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SEARCH_IDLIST;
ALTER FUNCTION FC_SEARCH_IDLIST COMPILE DEBUG;

/************************************************************************/

Prompt == FUNCTION FC_SIBLING_MOVE_WBS ==
CREATE OR REPLACE FUNCTION FC_SIBLING_MOVE_WBS
(
     h_source_rank IN OUT NOCOPY VARCHAR2
   , h_target_rank IN OUT NOCOPY VARCHAR2
   , h_target_project_id IN OUT NOCOPY VARCHAR2
   , sh_h_target_minute_per_day PLS_INTEGER
   , sh_h_wbs_code_len PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_target_wbs_code IN OUT NOCOPY VARCHAR2
   , h_extend_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_target_minute_per_day PLS_INTEGER := sh_h_target_minute_per_day;
   h_wbs_code_len PLS_INTEGER := sh_h_wbs_code_len;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_wbs_code VARCHAR2(255);
   h_old_wbs_code VARCHAR2(255);
   h_cur_rank VARCHAR2(200);
   h_old_rank VARCHAR2(200);
   h_temp_rank VARCHAR2(200);
   h_len PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C12a IS
         SELECT RANK, WBS_CODE
         FROM TMT_WBS
         WHERE LEVEL_ID = h_level_id
         AND RANK BETWEEN h_target_rank AND h_source_rank
         ORDER BY RANK DESC;

      CURSOR C130 IS
         SELECT RANK, WBS_CODE
         FROM TMT_WBS
         WHERE LEVEL_ID = h_level_id AND RANK BETWEEN h_source_rank AND h_target_rank
         ORDER BY RANK DESC;

      CURSOR C12b IS
         SELECT RANK, WBS_CODE
         FROM TMT_WBS
         WHERE LEVEL_ID = h_level_id
         AND RANK BETWEEN h_source_rank AND h_target_rank
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_SIBLING_MOVE_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_SIBLING_MOVE_WBS'), '-- ' || SYSTIMESTAMP, ':r := FC_SIBLING_MOVE_WBS(' || 
       '''' || h_source_rank || ''''
          || ', ' || '''' || h_target_rank || ''''
          || ', ' || '''' || h_target_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_target_minute_per_day), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_wbs_code_len), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_target_wbs_code || ''''
          || ', ' || '''' || h_extend_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SIBLING_MOVE_WBS', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_SIBLING_MOVE_WBS(61) : move_wbs.sql(4630) : ' || SYSTIMESTAMP);
   END IF; 

   h_len := Coalesce(Length(h_source_rank), 0);
   h_level_id := Trunc(Coalesce(h_len, 0) / 4) - 2;
   
   If Coalesce(h_bitflag, 0) = 0 Then
      OPEN C12a;
   Else
      OPEN C130;
      
      FETCH C130 INTO h_cur_rank, h_target_wbs_code;
      IF C130%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      FETCH C130 INTO h_target_rank, h_target_wbs_code;
      IF C130%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      BEGIN
         CLOSE C130 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 93+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(94) : move_wbs.sql(4663) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C12b;
   End If;
   
   LOOP
      If Coalesce(h_bitflag, 0) = 0 Then
         FETCH C12a INTO h_cur_rank, h_wbs_code;
         IF C12a%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      Else
         FETCH C12b INTO h_cur_rank, h_wbs_code;
         IF C12b%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      End If;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_wbs_code_len := Coalesce(Length(h_wbs_code), 0);
      If PMO_ORASTD.strcmp(h_source_rank, h_cur_rank) = PMO_CNST.SC_MATCH Then
         -- update parent_rank in tmt_cri, tmt_documents
         h_temp_rank := h_extend_rank;
         h_temp_rank := SubStr(h_temp_rank || h_target_rank, 1, 200);
         
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_CRI(h_source_rank, h_temp_rank, h_target_project_id);
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_DOC(h_table_type, h_source_rank, h_temp_rank
                                          , h_target_project_id);
         
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_SCD(h_table_type, h_source_rank, h_temp_rank
                                          , h_target_project_id);
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_WBS
            SET PROJECT_ID = h_target_project_id,
            MINUTE_PER_DAY = h_target_minute_per_day,
            RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_rank)
            ELSE RTRIM(h_target_rank) || SUBSTR(RANK, h_len +1,LENGTH(RANK)-h_len) END || h_extend_rank,
            WBS_CODE = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_wbs_code)
            ELSE RTRIM(h_target_wbs_code) || SUBSTR(WBS_CODE, h_wbs_code_len +1,LENGTH(WBS_CODE)-h_wbs_code_len) END
            WHERE RANK LIKE h_source_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 168+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(169) : move_wbs.sql(4738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If PMO_ORASTD.strcmp(h_target_rank, h_cur_rank) <> PMO_CNST.SC_MATCH Then
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_CRI(h_cur_rank, h_old_rank, h_target_project_id);
            
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_DOC(h_table_type, h_cur_rank, h_old_rank, h_target_project_id);
            
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_SCD(h_table_type, h_cur_rank, h_old_rank, h_target_project_id);
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_cur_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_cur_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WBS
               SET PROJECT_ID = h_target_project_id,
               MINUTE_PER_DAY = h_target_minute_per_day,
               RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_old_rank)
               ELSE RTRIM(h_old_rank) || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) END,
               WBS_CODE = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_old_wbs_code)
               ELSE RTRIM(h_old_wbs_code) || SUBSTR(WBS_CODE, h_wbs_code_len +1, LENGTH(WBS_CODE)-h_wbs_code_len) END
               WHERE RANK LIKE h_cur_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(214) : move_wbs.sql(4783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_CRI(h_cur_rank, h_old_rank, h_target_project_id);
            
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_DOC(h_table_type, h_cur_rank, h_old_rank, h_target_project_id);
            
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_SCD(h_table_type, h_cur_rank, h_old_rank, h_target_project_id);
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_cur_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_cur_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WBS
               SET PROJECT_ID = h_target_project_id,
               MINUTE_PER_DAY = h_target_minute_per_day,
               RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_old_rank)
               ELSE RTRIM(h_old_rank) || SUBSTR(RANK, h_len +1, LENGTH(RANK)-h_len) END,
               WBS_CODE = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_old_wbs_code)
               ELSE RTRIM(h_old_wbs_code) || SUBSTR(WBS_CODE, h_wbs_code_len +1, LENGTH(WBS_CODE)-h_wbs_code_len) END
               WHERE
               SUBSTR(RANK, LENGTH(RANK)-3, 4) <> h_extend_rank AND
               RANK LIKE h_cur_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 259+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(260) : move_wbs.sql(4829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      h_old_rank := h_cur_rank;
      h_old_wbs_code := h_wbs_code;
   END LOOP;
   
   If Coalesce(h_bitflag, 0) = 0 Then
      BEGIN
         CLOSE C12a ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 278+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(279) : move_wbs.sql(4848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         CLOSE C12b ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 291+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(292) : move_wbs.sql(4861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_target_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_target_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, False); 
   	End If; 
   END;
   BEGIN
      UPDATE TMT_WBS
      SET RANK = SUBSTR(RANK, 1, LENGTH(RANK) - 4)
      WHERE RANK LIKE h_target_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 323+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(324) : move_wbs.sql(4893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_temp_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_temp_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_temp_rank, False); 
   	End If; 
   END;
   BEGIN
      UPDATE TMT_CRI
      SET PARENT_RANK = 'WBS#' || SUBSTR(PARENT_RANK, 9, LENGTH(PARENT_RANK) - 8)
      WHERE PARENT_RANK LIKE 'WBS#' || h_temp_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 355+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(356) : move_wbs.sql(4925) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_DOCUMENTS
      SET PARENT_RANK = 'WBS#' || SUBSTR(PARENT_RANK, 9, LENGTH(PARENT_RANK) - 8)
      WHERE PARENT_RANK LIKE 'WBS#' || h_temp_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 374+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(375) : move_wbs.sql(4944) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_ASSIGNED_SCORECARDS
      SET PARENT_RANK = 'WBS#' || SUBSTR(PARENT_RANK, 9, LENGTH(PARENT_RANK) - 8)
      WHERE PARENT_RANK LIKE 'WBS#' || h_temp_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_SIBLING_MOVE_WBS', 393+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_SIBLING_MOVE_WBS(394) : move_wbs.sql(4963) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_SIBLING_MOVE_WBS', 403+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_SIBLING_MOVE_WBS(404) : move_wbs.sql(4973) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_SIBLING_MOVE_WBS;
ALTER FUNCTION FC_SIBLING_MOVE_WBS COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_FIND_CRI_TASK ==
CREATE OR REPLACE FUNCTION FC_FIND_CRI_TASK
(
     h_old_id IN OUT NOCOPY VARCHAR2
   , h_new_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIND_CRI_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIND_CRI_TASK'), '-- ' || SYSTIMESTAMP, ':r := FC_FIND_CRI_TASK(' || 
       '''' || h_old_id || ''''
          || ', ' || '''' || h_new_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIND_CRI_TASK', 14+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIND_CRI_TASK(15) : move_wbs.sql(4998) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT NEW_ID INTO h_new_id
      FROM   PMO_TMP_CRI_DEPND
      WHERE  OLD_ID = h_old_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIND_CRI_TASK', 27+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIND_CRI_TASK(28) : move_wbs.sql(5011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIND_CRI_TASK', 33+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIND_CRI_TASK(34) : move_wbs.sql(5017) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIND_CRI_TASK;
ALTER FUNCTION FC_FIND_CRI_TASK COMPILE DEBUG;

/*******************************************************************************************/

Prompt == FUNCTION FC_M_WBS_MSP ==
CREATE OR REPLACE FUNCTION FC_M_WBS_MSP
(
     h_target_id IN OUT NOCOPY VARCHAR2
   , h_source_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_ieflag PLS_INTEGER
   , h_error_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_ieflag PLS_INTEGER := sh_h_ieflag;
   h_wbs_code VARCHAR2(255);
   h_cur_date TIMESTAMP;
   h_cur_id VARCHAR2(32);
   h_cur_rank VARCHAR2(200);
   h_pred_id VARCHAR2(32);
   h_rankBase64 VARCHAR2(8);
   h_source_parent_id VARCHAR2(32);
   h_source_project_id VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_source_transfer_flag VARCHAR2(1);
   h_succ_id VARCHAR2(32);
   h_source_timecode_1_id VARCHAR2(32);
   h_source_timecode_2_id VARCHAR2(32);
   h_target_code VARCHAR2(254);
   h_target_parent_id VARCHAR2(32);
   h_target_project_id VARCHAR2(32);
   h_target_rank VARCHAR2(200);
   h_target_transfer_flag VARCHAR2(1);
   h_target_timecode_1_id VARCHAR2(32);
   h_target_timecode_2_id VARCHAR2(32);
   h_xrank64 VARCHAR2(200);
   h_dep_str VARCHAR2(8192);
   h_wbs_bitflag PLS_INTEGER := 0;
   h_rollup_bitflag PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_cri_level_id PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_source_type PLS_INTEGER := 0;
   h_branch PLS_INTEGER := 0;
   h_diff_level PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_lenc PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_source_level PLS_INTEGER := 0;
   h_target_level PLS_INTEGER := 0;
   h_parent_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_source_code VARCHAR2(255);
   h_target_calendar_id VARCHAR2(32);
   h_target_minute_per_day PLS_INTEGER := 0;
   h_target_start_datetime TIMESTAMP;
   h_target_currency_id PLS_INTEGER := 0;
   h_target_exchange_rate NUMBER := 0;
   h_source_calendar_id VARCHAR2(32);
   h_source_minute_per_day PLS_INTEGER := 0;
   h_source_start_datetime TIMESTAMP;
   h_source_currency_id PLS_INTEGER := 0;
   h_source_exchange_rate NUMBER := 0;
   h_actual_start TIMESTAMP;
   h_actual_budget NUMBER := 0;
   h_target_priority PLS_INTEGER := 0;
   h_source_priority PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_tmp_flag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_temp_fs_code VARCHAR2(50);
   h_target_fs_code VARCHAR2(50);
   h_target_master_fs_code VARCHAR2(50);
   h_fs_labor_code VARCHAR2(50);
   h_target_fs_labor_code VARCHAR2(50);
   h_source_master_fs_code VARCHAR2(50);
   h_source_fs_code VARCHAR2(50);
   h_source_parent_fs_code VARCHAR2(50);
   h_fs_flag PLS_INTEGER := 0;
   h_check_dependency PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_host_table PLS_INTEGER := 0;
   h_element_id VARCHAR2(32);
   h_target_asset_flag PLS_INTEGER := 0;
   h_source_asset_flag PLS_INTEGER := 0;
   h_financial_category PLS_INTEGER := 0;
   h_temp_rank VARCHAR2(200);
   acrossProject PLS_INTEGER NOT NULL := 0;
   moveCRI PLS_INTEGER NOT NULL := 0;
   iparent PLS_INTEGER NOT NULL := -1;
-- Declare Cursors
      CURSOR C_PMOPROJ IS
         SELECT ELEMENT_ID, PARENT_ID, PROJECT_ID, TYPE_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_source_rank || '%'
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3,PMO_CNST.SC_TASK)
         AND FS_CODE IS NuLL
         ORDER BY RANK;

      CURSOR C_TJa IS
         SELECT D.DEPEND_ID, D.PRED_ID, D.SUCC_ID
         FROM DEPENDENCIES D, TMT_WBS B
         WHERE B.RANK LIKE h_source_rank || '%'
         AND (D.PRED_ID = B.ELEMENT_ID OR D.SUCC_ID = B.ELEMENT_ID)
         AND (NOT EXISTS (SELECT ELEMENT_ID FROM TMT_WBS C WHERE C.RANK LIKE h_source_rank || '%' AND D.SUCC_ID = C.ELEMENT_ID) OR
         NOT EXISTS (SELECT ELEMENT_ID FROM TMT_WBS E WHERE E.RANK LIKE h_source_rank || '%' AND D.PRED_ID = E.ELEMENT_ID));

      CURSOR idListCursorA IS
         SELECT ELEMENT_ID FROM PMO_TMP_MVW_IDLIST;

      CURSOR C_TJ IS
         SELECT D.DEPEND_ID, D.PRED_ID, D.SUCC_ID
         FROM DEPENDENCIES D, PMO_TMP_MVW_0001 B
         WHERE (D.PRED_ID = B.ELEMENT_ID OR D.SUCC_ID = B.ELEMENT_ID)
         AND (NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001 C WHERE D.SUCC_ID = C.ELEMENT_ID) OR
         NOT EXISTS (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001 E WHERE D.PRED_ID = E.ELEMENT_ID));

      CURSOR idListCursor IS
         SELECT ELEMENT_ID FROM PMO_TMP_MVW_IDLIST;

      CURSOR C_INSERT_CRI IS
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_cur_rank || '%';

      CURSOR C_INSERT_CRIa IS
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE h_target_rank || '%';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_WBS_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_WBS_MSP'), '-- ' || SYSTIMESTAMP, ':r := FC_M_WBS_MSP(' || 
       '''' || h_target_id || ''''
          || ', ' || '''' || h_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_ieflag), 'NULL')
          || ', ' || '''' || h_error_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 143+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_WBS_MSP(144) : move_wbs.sql(5171) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_CHECK_EXTERNAL_DEPENDENCY)) > 0 Then
      h_check_dependency := 1;
      h_bitflag := Coalesce(h_bitflag, 0) - PMO_CNST.SC_CHECK_EXTERNAL_DEPENDENCY;
   End If;
   
   -- SC_BIT_ROLLUP = 1073741824 (set the 31st bit to 1 only)
   -- SC_BIT_NOT_ROLLUP = 1073741823 (set the first 31 bits to 1)
   -- The 31st bit in bitflag will be used as rollup flag.
   -- The first bit in bitflag will be used as moving option flag
   -- rollup flag will only be used by impex.
   h_bitflag := BITAND(PMO_CNST.SC_BIT_NOT_ROLLUP, Coalesce(h_bitflag, 0));
   -- set 31st bit to 0
   
   h_rollup := 1;
   
   If (Coalesce(h_bitflag, 0) <> 0) AND (Coalesce(h_bitflag, 0) <> 1) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 165+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(166) : move_wbs.sql(5193) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.BITFLAG_OUT_OF_RANGE;
   End If;
   
   --  if((IsNullParam(h_target_id)) || (IsNullParam(h_source_id)))
   If (h_source_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 173+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(174) : move_wbs.sql(5201) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
   End If;
   
   If PMO_ORASTD.strcmp(h_target_id, h_source_id) = PMO_CNST.SC_MATCH Then
      h_error_id := h_source_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 181+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(182) : move_wbs.sql(5209) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   -- Declare local variable
   
   h_target_rank := '';
   h_target_parent_id := '';
   h_source_parent_id := '';
   h_target_level := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TARGET_SOURCE';
   PMO_DB2M.SC_STATE := 0;
   
   h_target_fs_code := NULL;
   If (h_target_id IS NOT NULL) Then
      BEGIN
         SELECT RTRIM(W.RANK), W.PROJECT_ID, W.PARENT_ID, RTRIM(W.WBS_CODE),
         W.TYPE_ID, W.LEVEL_ID, W.TRANSFER_FLAG, W.FS_CODE,
         W.FS_LABOR_CODE, P.CALENDAR_ID, W.MINUTE_PER_DAY,
         P.START_DT, P.CURRENCY_ID, PMO_ORASTD.TODECIMAL(C.ELEMENT_NAME, 18, 3),
         P.PRIORITY, W.MAX_RANK, W.TIMECODE_1_ID, W.TIMECODE_2_ID,
         W.ASSET_FLAG, W.FINANCIAL_CATEGORY
         INTO h_target_rank, h_target_project_id, h_target_parent_id, h_target_code,
         h_parent_type, h_target_level, h_target_transfer_flag, h_target_fs_code,
         h_target_fs_labor_code, h_target_calendar_id, h_target_minute_per_day,
         h_target_start_datetime, h_target_currency_id, h_target_exchange_rate,
         h_target_priority, h_max_rank, h_target_timecode_1_id, h_target_timecode_2_id,
         h_target_asset_flag, h_financial_category
         FROM TMT_WBS W
         JOIN TMT_WBS P
         ON P.ELEMENT_ID = W.PROJECT_ID
         JOIN TMT_DATAFIELDS C
         ON C.ELEMENT_ID = P.CURRENCY_ID
         AND C.TYPE_ID = 37
         WHERE W.ELEMENT_ID = h_target_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 225+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(226) : move_wbs.sql(5253) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         h_error_id := h_target_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 233+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(234) : move_wbs.sql(5261) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.INVALID_PARAMS;
      End If;
      
      If (isTask(Coalesce(h_parent_type, 0))) <> 0 AND (Coalesce(h_bitflag, 0) = 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 240+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(241) : move_wbs.sql(5268) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.INVALID_PARAMS;
      End If;
   End If;
   
   -- can't do sibling move at root level
   If (Coalesce(h_target_level, 0) = 0) AND (Coalesce(h_bitflag, 0) = 1) Then
      h_error_id := h_target_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 250+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(251) : move_wbs.sql(5278) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.INVALID_PARAMS;
   End If;
   
   If (Coalesce(h_ieflag, 0) = 0) AND ((h_target_id IS NOT NULL)) Then
      BEGIN
         SELECT COUNT(*) INTO h_count
         FROM TMT_WBS
         WHERE (SUBSTR(h_target_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_target_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_target_rank, 1, 8) || '%'))
         AND REC_STATUS IN ('L', 'D', 'A');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 268+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(269) : move_wbs.sql(5296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) > 0 Then
         h_error_id := h_target_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 276+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(277) : move_wbs.sql(5304) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PARENT_MARKED_AS_LOCKED;
      End If;
   End If;
   
   BEGIN
      SELECT RTRIM(W.RANK), W.PROJECT_ID, W.PARENT_ID, RTRIM(W.WBS_CODE),
      W.TYPE_ID, W.LEVEL_ID, W.TRANSFER_FLAG, W.FS_CODE,
      LENGTH(W.RANK), PMO_ORASTD.LOCATE(W.RANK, h_target_rank), LENGTH(W.WBS_CODE),
      W.ACTUAL_START, W.ACTUAL_BUDGET, P.CALENDAR_ID, W.MINUTE_PER_DAY,
      P.START_DT, P.CURRENCY_ID, PMO_ORASTD.TODECIMAL(C.ELEMENT_NAME, 18, 3),
      P.PRIORITY, W.TIMECODE_1_ID, W.TIMECODE_2_ID,
      W.ASSET_FLAG
      INTO h_source_rank, h_source_project_id, h_source_parent_id, h_source_code,
      h_source_type, h_source_level, h_source_transfer_flag, h_source_fs_code,
      h_len, h_branch, h_lenc, h_actual_start,
      h_actual_budget, h_source_calendar_id, h_source_minute_per_day,
      h_source_start_datetime, h_source_currency_id, h_source_exchange_rate,
      h_source_priority, h_source_timecode_1_id, h_source_timecode_2_id,
      h_source_asset_flag
      FROM TMT_WBS W
      JOIN TMT_WBS P
      ON P.ELEMENT_ID = W.PROJECT_ID
      JOIN TMT_DATAFIELDS C
      ON C.ELEMENT_ID = P.CURRENCY_ID
      AND C.TYPE_ID = 37
      WHERE W.ELEMENT_ID = h_source_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 310+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(311) : move_wbs.sql(5338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      h_error_id := h_source_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 318+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(319) : move_wbs.sql(5346) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.INVALID_PARAMS;
   End If;
   
   -- the transferred element can't have any non-folder parent. Ping, 2004-04-15
   If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 0) AND ((h_source_transfer_flag IS NOT NULL)) Then
      BEGIN
         SELECT RANK INTO h_temp_rank
         FROM TMT_CRI
         WHERE ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 335+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(336) : move_wbs.sql(5363) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_count := 0;
      BEGIN
         SELECT COUNT(ELEMENT_ID) INTO h_count
         FROM TMT_CRI
         WHERE (SUBSTR(h_temp_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_temp_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_temp_rank, 1, 12) || '%'))
         AND LEVEL_ID > 1
         AND TYPE_ID <> PMO_CNST.SC_FOLDER
         AND ELEMENT_ID <> h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 355+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(356) : move_wbs.sql(5383) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) > 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 362+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(363) : move_wbs.sql(5390) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_MOVE_SUB_TRANSFERRED_CR;
      End If;
   End If;
   
   -- check external dependencies
   If Coalesce(h_check_dependency, 0) > 0 Then
      --      BEGIN_SQL
      --          SELECT COUNT('X') INTO HOST(h_count)
      --          FROM DUAL
      --          WHERE EXISTS(SELECT D.PRED_ID
      --              FROM DEPENDENCIES D, TMT_WBS W
      --              WHERE W.RANK LIKE HOST(h_source_rank) || '%'
      --                  AND (D.PRED_ID = W.ELEMENT_ID OR D.SUCC_ID = W.ELEMENT_ID)
      --                  AND (NOT EXISTS (SELECT ELEMENT_ID FROM TMT_WBS C WHERE C.RANK LIKE HOST(h_source_rank) || '%' AND D.SUCC_ID = C.ELEMENT_ID)
      --                      OR NOT EXISTS (SELECT ELEMENT_ID FROM TMT_WBS E WHERE E.RANK LIKE HOST(h_source_rank) || '%' AND D.PRED_ID = E.ELEMENT_ID)));
      --      END_SQL;
      
      BEGIN
         SELECT COUNT(P.ELEMENT_NAME) INTO h_count
         FROM TMT_WBS P
         JOIN DEPENDENCIES D ON D.PRED_ID = P.ELEMENT_ID
         JOIN TMT_WBS S ON S.ELEMENT_ID = D.SUCC_ID
         WHERE P.RANK LIKE h_source_rank || '%'
         AND NOT (S.RANK LIKE h_source_rank || '%');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 394+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(395) : move_wbs.sql(5422) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) = 0 Then
         BEGIN
            SELECT COUNT(S.ELEMENT_NAME) INTO h_count
            FROM TMT_WBS S
            JOIN DEPENDENCIES D ON D.SUCC_ID = S.ELEMENT_ID
            JOIN TMT_WBS P ON P.ELEMENT_ID = D.PRED_ID
            WHERE S.RANK LIKE h_source_rank || '%'
            AND NOT (P.RANK LIKE h_source_rank || '%');
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 413+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(414) : move_wbs.sql(5441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_count, 0) > 0 AND NOT ((Coalesce(h_bitflag, 0) = 0 AND PMO_ORASTD.strcmp(h_target_id
            , h_source_parent_id) = PMO_CNST.SC_MATCH) OR (Coalesce(h_bitflag, 0) = 1 
            AND PMO_ORASTD.strcmp(h_target_parent_id, h_source_parent_id) = PMO_CNST.SC_MATCH)) Then
         h_error_id := h_source_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 424+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(425) : move_wbs.sql(5452) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.SC_LOSE_DEPENDENCY;
      End If;
   End If;
   
   --trace_printf("0____* h_ieflag: %d, h_source_rank: %d %s\n", h_ieflag,h_source_rank_Ind,h_source_rank);
   
   If Coalesce(h_ieflag, 0) = 0 Then
      h_count := 0;
      
      --[Asad] Checks if at least one of the parents is locked or not.
      BEGIN
         SELECT COUNT(*) INTO h_count
         FROM TMT_WBS
         WHERE (SUBSTR(h_source_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_source_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_source_rank, 1, 8) || '%')) AND REC_STATUS IN ('L', 'D', 'A');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 447+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(448) : move_wbs.sql(5475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("1____* h_count: %d, h_source_rank: %d %s\n", h_count,h_source_rank_Ind,h_source_rank);
      
      If Coalesce(h_count, 0) > 0 Then
         h_error_id := h_source_id;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 457+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(458) : move_wbs.sql(5485) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.ITEM_IS_LOCKED;
         --/return PARENT_MARKED_AS_LOCKED;
      End If;
      
      --[Asad] Checks if at least one of the children is locked or not.
      BEGIN
         SELECT COUNT(*) INTO h_count
         FROM TMT_WBS
         WHERE (RANK LIKE h_source_rank || '%')
         AND REC_STATUS IN ('L', 'D', 'A');
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 475+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(476) : move_wbs.sql(5503) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2____* h_count: %d, h_source_rank: %d %s\n", h_count,h_source_rank_Ind,h_source_rank);
      
      If Coalesce(h_count, 0) > 0 Then
         h_error_id := h_source_id;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 485+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(486) : move_wbs.sql(5513) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.ITEM_IS_LOCKED;
      End If;
   End If;
   
   -- Is not allowed to move a parent under his child
   If (Coalesce(h_branch, 0) > 0) AND (Coalesce(h_target_level, 0) > Coalesce(h_source_level, 0)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 494+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(495) : move_wbs.sql(5522) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.CANNOT_COPY_PARENT_UNDER_CHILD;
   End If;
   
   -- target project is a PBFS or affected PBFS project
   If (h_target_fs_code IS NOT NULL) Then
      h_temp_sc_state := FC_GET_FS_CODE(h_target_fs_code, h_target_master_fs_code);
   End If;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   -- check PBFS project
   If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      If (h_source_fs_code IS NOT NULL) Then
         h_temp_sc_state := FC_GET_FS_CODE(h_source_fs_code, h_source_master_fs_code);
         
         -- source project is an affected PBFS project
         If PMO_ORASTD.strcmp(h_source_fs_code, h_source_master_fs_code) <> 0 Then
            -- source project and target project have different fs_code
            If (h_target_fs_code IS NULL) OR (PMO_ORASTD.strcmp(h_target_master_fs_code
                  , h_source_master_fs_code) <> 0) Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_source_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_source_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
               	End If; 
               END;
               BEGIN
                  SELECT COUNT(T.ELEMENT_ID) INTO h_fs_flag
                  FROM TMT_WBS T, TMT_WBS W
                  WHERE T.RANK LIKE h_source_rank || '%'
                  AND W.RANK LIKE h_source_rank || '%'
                  AND W.FS_CODE LIKE h_source_fs_code || '%'
                  AND T.PROJECT_ID = W.ELEMENT_ID
                  AND T.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
                  AND (T.FS_CODE IS NOT NuLL
                  OR T.FS_STATE IN (PMO_CNST.SC_PENDING, PMO_CNST.SC_PENDING_SYNC, PMO_CNST.SC_PENDING_SEND));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 556+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(557) : move_wbs.sql(5584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_fs_flag, 0) > 0 Then
                  h_error_id := h_source_id;
                  --                      return CANNOT_CONVERT_PBFS_PROJECT; // change PBFS project to non-PBFS project
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 565+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(566) : move_wbs.sql(5593) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.INVALID_PARAMS;
                  -- change PBFS project to non-PBFS project
               End If;
               
               -- delete activity_code
               BEGIN
                  DELETE FROM TMT_TIMECODES
                  WHERE RANK LIKE '00000003%'             -- under BMS Activity Code
                  AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
                  WHERE RANK LIKE h_source_rank || '%'
                  AND PROJECT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
                  WHERE RANK LIKE h_source_rank || '%'
                  AND FS_CODE LIKE h_source_fs_code || '%'));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 590+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(591) : move_wbs.sql(5618) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_source_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_source_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
               	End If; 
               END;
               -- update fs_code and fs_labor_code
               BEGIN
                  UPDATE TMT_WBS
                  SET FS_CODE = NuLL,
                  FS_LABOR_CODE = NuLL,
                  FS_STATE = 0
                  WHERE RANK LIKE h_source_rank || '%'
                  AND PROJECT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
                  WHERE RANK LIKE h_source_rank || '%'
                  AND FS_CODE LIKE h_source_fs_code || '%');
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 628+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(629) : move_wbs.sql(5656) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_source_fs_code := NULL;
            End If;
            
            If (h_target_fs_code IS NOT NULL) Then
               h_target_fs_code := h_target_fs_code || '.' || TO_CHAR(Coalesce(h_max_rank, 0));
               
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_source_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_source_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
               	End If; 
               END;
               -- update fs_code and fs_labor_code
               BEGIN
                  UPDATE TMT_WBS
                  SET FS_CODE = REPLACE(FS_CODE, h_source_fs_code, h_target_fs_code)
                  WHERE RANK LIKE h_source_rank || '%'
                  AND FS_CODE LIKE h_source_fs_code || '%';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 668+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(669) : move_wbs.sql(5696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      If (h_source_fs_code IS NULL) AND ((h_target_fs_code IS NOT NULL)) Then
         -- can't move a pmo project with actual to a bms project
         If (h_actual_start IS NOT NULL) Then
            h_error_id := h_source_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 681+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(682) : move_wbs.sql(5709) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         End If;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         
         OPEN C_PMOPROJ;
         
         LOOP
            FETCH C_PMOPROJ INTO h_cur_id, h_parent_id, h_project_id, h_type_id;
            IF C_PMOPROJ%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            If PMO_ORASTD.strcmp(h_source_id, h_cur_id) = 0 Then
               PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
               
               BEGIN
                  UPDATE TMT_WBS
                  SET FS_CODE = h_target_fs_code || '.' || h_rankBase64,
                  FS_LABOR_CODE = h_target_fs_labor_code
                  WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 732+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(733) : move_wbs.sql(5760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_cur_id, h_target_id, h_type_id);
            Else
               If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
                  BEGIN
                     SELECT FS_CODE
                     INTO h_temp_fs_code
                     FROM TMT_WBS
                     WHERE ELEMENT_ID = h_parent_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 751+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(752) : move_wbs.sql(5779) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TMT_WBS
                     SET FS_CODE = h_temp_fs_code || '.' || SUBSTR(RANK, LENGTH(RANK)-3, 4),
                     FS_LABOR_CODE = h_target_fs_labor_code
                     WHERE ELEMENT_ID = h_cur_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 771+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(772) : move_wbs.sql(5799) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_cur_id, h_target_id, h_type_id);
               Else
                  PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_cur_id, h_project_id, h_type_id);
                  
                  BEGIN
                     SELECT MIN(R.FS_LABOR_CODE), SUM(TA.SCHED_WORK), COUNT(DISTINCT R.FS_LABOR_CODE)
                     INTO h_fs_labor_code, h_sched_work, h_assign_count
                     FROM TASK_ASSIGNMENTS TA, RESOURCES R
                     WHERE TA.PROJECT_ID = h_project_id AND TA.TASK_ID = h_cur_id
                     AND TA.RESOURCE_ID = R.RESOURCE_ID;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 792+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(793) : move_wbs.sql(5820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If (Coalesce(h_assign_count, 0) = 1) AND ((h_fs_labor_code IS NOT NULL)) AND (PMO_ORASTD.strncmp(
                        h_fs_labor_code, 'PMOFFICE', 8) <> 0) Then
                     If Coalesce(h_sched_work, 0) = 0 Then
                        BEGIN
                           UPDATE TMT_WBS
                           SET FS_LABOR_CODE = h_fs_labor_code
                           WHERE ELEMENT_ID = h_cur_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 814+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(815) : move_wbs.sql(5842) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TMT_WBS
                           SET FS_LABOR_CODE = h_fs_labor_code,
                           FS_STATE = PMO_CNST.SC_SYNC_REQ
                           WHERE ELEMENT_ID = h_cur_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 834+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(835) : move_wbs.sql(5862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               End If;
            End If;
         END LOOP;
         BEGIN
            CLOSE C_PMOPROJ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 851+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(852) : move_wbs.sql(5879) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   BEGIN
      SELECT SYSTIMESTAMP INTO h_cur_date FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 866+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(867) : move_wbs.sql(5894) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_parent_type: %d, h_source_type: %d, h_source_rank: %s\n",h_parent_type,h_source_type,h_source_rank); //trace_flush();
   --trace_printf("h_target_id: %d:%s, h_source_id: %d:%s\n",h_target_id_Ind,h_target_id,h_source_id_Ind,h_source_id); //trace_flush();
   --trace_printf("h_source_parent_id: %d:%s\n",h_source_parent_id_Ind,h_source_parent_id); //trace_flush();
   
   If (isProjectType(Coalesce(h_source_type, 0))) <> 0 AND (isProjectType(Coalesce(
         h_parent_type, 0)) <> 0 OR (h_target_id IS NULL)) Then
      BEGIN
         SELECT RANK INTO h_cur_rank
         FROM TMT_PORTFOLIOS
         WHERE ELEMENT_ID = h_rec_user;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 886+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(887) : move_wbs.sql(5914) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_cur_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_cur_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
         	End If; 
         END;
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            DELETE FROM TMT_PORTFOLIOS
            WHERE RANK LIKE h_cur_rank || '%'
            AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
            WHERE RANK LIKE h_source_rank || '%');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 933+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(934) : move_wbs.sql(5961) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (h_target_id IS NULL) Then
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_WBS', h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         h_target_rank := '0000' || h_rankBase64;
         h_target_code := '0.' || TO_CHAR(Coalesce(h_max_rank, 0));
         h_diff_level := -Coalesce(h_source_level, 0);
      Else
         If PMO_ORASTD.strcmp(h_target_project_id, h_source_project_id) <> PMO_CNST.SC_MATCH Then
            acrossProject := 1;
         End If;
         
         If PMO_ORASTD.strcmp(h_target_id, h_source_parent_id) = PMO_CNST.SC_MATCH Then
            BEGIN
               SELECT CASE WHEN MAX(RANK) = h_source_rank THEN 1 ELSE 0 END INTO h_diff_level
               FROM TMT_WBS WHERE PARENT_ID = h_target_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 960+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(961) : move_wbs.sql(5988) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_diff_level, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_M_WBS_MSP', 967+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_M_WBS_MSP(968) : move_wbs.sql(5995) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || h_source_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 970+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(971) : move_wbs.sql(5998) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMODEFINES.PMO_RETURN_ERROR(0, h_source_id, False);
            End If;
         End If;
         
         If ((Coalesce(h_bitflag, 0) = 0) AND (PMO_ORASTD.strcmp(h_target_id
               , h_source_parent_id) <> PMO_CNST.SC_MATCH)) OR ((Coalesce(h_bitflag, 0) = 1) 
               AND ((Coalesce(h_target_level, 0) <> Coalesce(h_source_level, 0)) 
               OR (PMO_ORASTD.strcmp(h_target_parent_id
               , h_source_parent_id) <> PMO_CNST.SC_MATCH))) Then
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MVW_IDLIST';
            PMO_DB2M.SC_STATE := 0;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_source_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_source_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
            	End If; 
            END;
            
            OPEN C_TJa;
            LOOP
               FETCH C_TJa INTO h_cur_id, h_pred_id, h_succ_id;
               IF C_TJa%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               BEGIN
                  DELETE FROM DEPENDENCIES WHERE DEPEND_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1024+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1025) : move_wbs.sql(6052) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               PMO_DB2M.SC_STATE := FC_SEARCH_IDLIST(h_pred_id);
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     INSERT INTO PMO_TMP_MVW_IDLIST
                     VALUES(h_pred_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1045+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1046) : move_wbs.sql(6073) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               PMO_DB2M.SC_STATE := FC_SEARCH_IDLIST(h_succ_id);
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     INSERT INTO PMO_TMP_MVW_IDLIST
                     VALUES(h_succ_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1066+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1067) : move_wbs.sql(6094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C_TJa ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1080+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1081) : move_wbs.sql(6108) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN idListCursorA;
            
            LOOP
               FETCH idListCursorA INTO h_cur_id;
               IF idListCursorA%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               -- null_ind already cleared at initialization
               PMO_STPU.packDependencies(h_cur_id, h_dep_str, 8193);
               
               If PMO_ORASTD.strcmp(h_dep_str, '{RS}{RF}') <> 0 Then
                  UPDATE TMT_WBS SET
                          DEPENDENCIES = NULL
                     WHERE ELEMENT_ID = H_CUR_ID;

                  BEGIN
                     UPDATE TMT_WBS
                     SET DEPENDENCIES = h_dep_str
                     WHERE ELEMENT_ID = h_cur_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1121+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1122) : move_wbs.sql(6149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TMT_WBS
                     SET DEPENDENCIES = null
                     WHERE ELEMENT_ID = h_cur_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1140+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1141) : move_wbs.sql(6168) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE idListCursorA ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1154+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1155) : move_wbs.sql(6182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_WBS', h_target_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         h_target_rank := SubStr(h_target_rank || h_rankBase64, 1, 200);
         h_target_code := h_target_code || '.' || TO_CHAR(Coalesce(h_max_rank, 0));
         h_diff_level := Coalesce(h_target_level, 0) - Coalesce(h_source_level, 0) + 1;
      End If;
      --trace_printf("h_target_rank: %s, h_target_code: %s, h_diff_level: %d\n", h_target_rank,h_target_code,h_diff_level); //trace_flush();
      
      -- update the level, rec_user, rec_datetime, rec_status, rank
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_source_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_source_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_WBS
         SET LEVEL_ID = LEVEL_ID + h_diff_level,
         REC_DATETIME = h_cur_date,
         LAST_CHECKIN = h_cur_date,
         RANK = CASE WHEN LEVEL_ID = h_source_level THEN h_target_rank
         ELSE h_target_rank || SUBSTR(RANK, h_len + 1,LENGTH(RANK) - h_len) END,
         WBS_CODE = CASE WHEN LEVEL_ID = h_source_level THEN RTRIM(h_target_code)
         ELSE RTRIM(h_target_code) || SUBSTR(WBS_CODE, h_lenc +1, LENGTH(WBS_CODE)-h_lenc) END
         WHERE RANK LIKE h_source_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1202+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1203) : move_wbs.sql(6230) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_source_rank: %s\n", SC_STATE,h_source_rank); //trace_flush();
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         --update new parent for the source
         BEGIN
            UPDATE TMT_WBS
            SET PARENT_ID = h_target_id
            WHERE ELEMENT_ID = h_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1224+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1225) : move_wbs.sql(6252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_source_parent_id IS NOT NULL) Then
            BEGIN
               UPDATE TMT_WBS
               SET CHILD_COUNT = CHILD_COUNT - 1,
               ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
               WHERE ELEMENT_ID = h_source_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1245+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1246) : move_wbs.sql(6273) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1264+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1265) : move_wbs.sql(6292) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --update attached DOCUMENTs
         --strcpy(h_xrank64,"PRJ#");
         h_xrank64 := 'PRJ#';
         
         h_xrank64 := SubStr(h_xrank64 || h_source_rank, 1, 200);
         
         --update attached DOCUMENTs
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_xrank64 IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_xrank64), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_xrank64, False); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_DOCUMENTS SET PARENT_RANK = 'PRJ#' || h_target_rank
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1303+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1304) : move_wbs.sql(6331) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("PRJ_DOC: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --update attached CRIs
         BEGIN
            UPDATE TMT_CRI SET PARENT_RANK = 'PRJ#' || h_target_rank
            WHERE PARENT_RANK = h_xrank64;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1323+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1324) : move_wbs.sql(6351) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("PRJ_CRI: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --update attached SCORECARDs
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET PARENT_RANK = 'PRJ#' || h_target_rank
            WHERE PARENT_RANK = h_xrank64;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1343+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1344) : move_wbs.sql(6371) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("PRJ_SC_ASS: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         BEGIN
            UPDATE TMT_PUBLISHED_SCORECARDS SET PARENT_RANK = 'PRJ#' || h_target_rank
            WHERE PARENT_RANK = h_xrank64;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1362+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1363) : move_wbs.sql(6390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("PRJ_SC_PUB: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --strcpy(h_xrank64,"WBS#");
         h_xrank64 := 'WBS#';
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_target_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_target_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, False); 
         	End If; 
         END;
         h_xrank64 := SubStr(h_xrank64 || h_source_rank, 1, 200);
         
         BEGIN
            UPDATE TMT_DOCUMENTS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'WBS#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1412+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1413) : move_wbs.sql(6440) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("WBS_DOC: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --update attached CRIs
         BEGIN
            UPDATE TMT_CRI SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'WBS#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1432+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1433) : move_wbs.sql(6460) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("WBS_CRI: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'WBS#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1451+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1452) : move_wbs.sql(6479) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("WBS_SC_ASS: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         BEGIN
            UPDATE TMT_PUBLISHED_SCORECARDS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'WBS#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1470+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1471) : move_wbs.sql(6498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("WBS_SC_PUB: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --strcpy(h_xrank64,"CRI#");
         h_xrank64 := 'CRI#';
         h_xrank64 := SubStr(h_xrank64 || h_source_rank, 1, 200);
         
         BEGIN
            UPDATE TMT_DOCUMENTS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'CRI#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1493+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1494) : move_wbs.sql(6521) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("WBS_DOC: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --update attached SCORECARDs
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'CRI#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1513+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1514) : move_wbs.sql(6541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CRI_SC_ASS: SC_STATE: %d, h_xrank64: %s\n",SC_STATE,h_xrank64); //trace_flush();
         
         BEGIN
            UPDATE TMT_PUBLISHED_SCORECARDS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'CRI#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1532+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1533) : move_wbs.sql(6560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CRI_SC_PUB: SC_STATE: %d\n",SC_STATE); //trace_flush();
         
         --strcpy(h_xrank64,"DEF#");
         h_xrank64 := 'DEF#';
         h_xrank64 := SubStr(h_xrank64 || h_source_rank, 1, 200);
         
         --update attached SCORECARDs
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET PARENT_RANK = REPLACE(PARENT_RANK,h_xrank64,'DEF#' || h_target_rank)
            WHERE PARENT_RANK LIKE h_xrank64 || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1556+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1557) : move_wbs.sql(6584) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("DEF_SC_ASS: SC_STATE: %d, h_xrank64: %s\n",SC_STATE,h_xrank64); //trace_flush();
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            PMO_DB2M.SC_STATE := 0;
         End If;
      End If;
   Else
      If PMO_ORASTD.strcmp(h_source_project_id, h_target_project_id) <> PMO_CNST.SC_MATCH Then
         acrossProject := 1;
         If (h_actual_start IS NOT NULL) Then
            h_error_id := h_source_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1572+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1573) : move_wbs.sql(6600) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.INVALID_PARAMS;
         End If;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            SELECT COUNT(ELEMENT_ID) INTO h_cri_level_id
            FROM TMT_WBS
            WHERE RANK LIKE h_source_rank || '%'
            AND (A_CRI_COUNT > 0 OR TRANSFER_FLAG = 'C');
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1601+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1602) : move_wbs.sql(6629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_cri_level_id, 0) > 0 Then
            h_error_id := h_source_id;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1609+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1610) : move_wbs.sql(6637) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CAN_NOT_MOVE_TRANSFERED_CRI;
         End If;
      End If;
      
      If (isProjectType(Coalesce(h_source_type, 0))) <> 0 AND (isNotProjectType(Coalesce(
            h_parent_type, 0))) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1618+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1619) : move_wbs.sql(6646) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_TARGET_SOURCE;
      End If;
      
      -- can't move transferred cri to transferred cri. Ping, 2004-04-15
      If PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1 Then
         If Coalesce(h_bitflag, 0) = 0 Then
            If (h_target_transfer_flag IS NOT NULL) Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_source_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_source_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
               	End If; 
               END;
               BEGIN
                  SELECT COUNT(ELEMENT_ID) INTO h_count
                  FROM TMT_WBS
                  WHERE RANK LIKE h_source_rank || '%' AND TRANSFER_FLAG = 'C';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1650+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1651) : move_wbs.sql(6678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_count, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1657+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1658) : move_wbs.sql(6685) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CANNOT_TRANSFER_UNDER_TRANSFER;
               End If;
            End If;
         Else
            BEGIN
               SELECT TRANSFER_FLAG, ASSET_FLAG, FINANCIAL_CATEGORY
               INTO h_target_transfer_flag, h_target_asset_flag, h_financial_category
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_target_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1674+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1675) : move_wbs.sql(6702) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_target_transfer_flag IS NOT NULL) Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_source_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_source_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
               	End If; 
               END;
               BEGIN
                  SELECT COUNT(ELEMENT_ID) INTO h_count
                  FROM TMT_WBS
                  WHERE RANK LIKE h_source_rank || '%' AND TRANSFER_FLAG = 'C';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1703+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1704) : move_wbs.sql(6731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_count, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1710+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1711) : move_wbs.sql(6738) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CANNOT_TRANSFER_UNDER_TRANSFER;
               End If;
            End If;
         End If;
      End If;
      
      If Coalesce(h_target_asset_flag, 0) <> PMO_CNST.SC_WITHOUT_ASSET Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_ASSETS
            SET REC_STATUS = 'D'
            WHERE (ELEMENT_ID, PARENT_ID) IN (SELECT ELEMENT_ID, ASSET_ID FROM TMT_WBS
            WHERE RANK LIKE h_source_rank || '%' AND ASSET_ID IS NOT NuLL);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1747+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1748) : move_wbs.sql(6775) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               UPDATE TMT_WBS
               SET ASSET_ID = NuLL,
               ASSET_NAME = NuLL,
               ASSET_FLAG = PMO_CNST.SC_PARENT_HAS_ASSET
               WHERE RANK LIKE h_source_rank || '%' AND ASSET_ID IS NOT NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1769+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1770) : move_wbs.sql(6797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_CRI
               SET ASSET_ID = NuLL,
               ASSET_NAME = NuLL,
               ASSET_FLAG = PMO_CNST.SC_PARENT_HAS_ASSET
               WHERE ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
               WHERE RANK LIKE h_source_rank || '%' AND ASSET_ID IS NOT NuLL);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1791+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1792) : move_wbs.sql(6819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If Coalesce(h_financial_category, 0) <> PMO_CNST.SC_NONE_FINANCIAL_CATEGORY Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_WBS
            SET FINANCIAL_CATEGORY = h_financial_category
            WHERE RANK LIKE h_source_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1826+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1827) : move_wbs.sql(6854) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI
            SET FINANCIAL_CATEGORY = h_financial_category
            WHERE ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WBS
            WHERE RANK LIKE h_source_rank || '%' AND TRANSFER_FLAG IS NOT NuLL);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1846+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1847) : move_wbs.sql(6874) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- if source is transferred cri or requirement or their child. Ping, 2004-04-15
      If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1) AND ((h_source_transfer_flag IS NOT NULL)) Then
         If SubStr(h_source_transfer_flag, 1, 1) = 'C' Then
            BEGIN
               SELECT RANK, LEVEL_ID INTO h_cur_rank, h_cri_level_id
               FROM TMT_CRI
               WHERE ELEMENT_ID = h_source_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1865+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1866) : move_wbs.sql(6893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_cri_level_id, 0) > 2 Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_cur_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_cur_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
               	End If; 
               END;
               BEGIN
                  SELECT COUNT(ELEMENT_ID) INTO h_cri_level_id
                  FROM TMT_CRI
                  WHERE RANK LIKE SUBSTR(h_cur_rank, 1, 8) || '%'
                  AND ELEMENT_ID <> h_source_id
                  AND (SUBSTR(h_cur_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_cur_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
                  AND (RANK LIKE SUBSTR(h_cur_rank, 1, 8) || '%'))
                  AND TYPE_ID <> PMO_CNST.SC_FOLDER AND LEVEL_ID > 1;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1898+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1899) : move_wbs.sql(6926) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_cri_level_id, 0) > 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 1905+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(1906) : move_wbs.sql(6933) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.CAN_NOT_MOVE_TRANSFERED_CRI;
               End If;
            End If;
         ElsIf SubStr(h_source_transfer_flag, 1, 1) = 'W' Then
            If (h_target_transfer_flag IS NULL) Then
               BEGIN
                  SELECT RANK, PARENT_ID INTO h_cur_rank, h_parent_id
                  FROM TMT_CRI
                  WHERE ELEMENT_ID = h_source_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1922+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1923) : move_wbs.sql(6950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_cur_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_cur_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
               	End If; 
               END;
               BEGIN
                  DELETE FROM TMT_CRI WHERE RANK LIKE h_cur_rank || '%';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1952+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1953) : move_wbs.sql(6980) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TMT_CRI SET CHILD_COUNT = CHILD_COUNT - 1, ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 1970+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(1971) : move_wbs.sql(6998) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               moveCRI := 1;
            End If;
         End If;
      End If;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MVW_0001';
      PMO_DB2M.SC_STATE := 0;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_source_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_source_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
      	End If; 
      END;
      BEGIN
         INSERT INTO PMO_TMP_MVW_0001
         SELECT ELEMENT_ID FROM TMT_WBS WHERE RANK LIKE h_source_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2010+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2011) : move_wbs.sql(7038) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If acrossProject = 1 Then
         h_temp_var := FC_GET_ASSIGNMENTS(h_source_project_id, h_source_rank, h_rec_user);
         
         --UPDATE DEPENDENCIES
         BEGIN
            UPDATE DEPENDENCIES
            SET PRED_PROJECT_ID = h_target_project_id
            WHERE PRED_PROJECT_ID = h_source_project_id
            AND PRED_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2034+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2035) : move_wbs.sql(7062) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE DEPENDENCIES
            SET SUCC_PROJECT_ID = h_target_project_id
            WHERE SUCC_PROJECT_ID = h_source_project_id
            AND SUCC_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2054+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2055) : move_wbs.sql(7082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --UPDATE BUDGET
         BEGIN
            UPDATE BUDGETS
            SET
            PROJECT_ID = h_target_project_id,
            ESTIMATED_TOTAL = ESTIMATED_TOTAL * (h_target_exchange_rate / h_source_exchange_rate),
            ESTIMATED_BUDGET = ESTIMATED_BUDGET * (h_target_exchange_rate / h_source_exchange_rate),
            ESTIMATED_ETC = ESTIMATED_ETC * (h_target_exchange_rate / h_source_exchange_rate),
            BUDGET = BUDGET * (h_target_exchange_rate / h_source_exchange_rate),
            ACTUAL = ACTUAL * (h_target_exchange_rate / h_source_exchange_rate),
            ETC = ETC * (h_target_exchange_rate / h_source_exchange_rate)
            WHERE PROJECT_ID = h_source_project_id
            AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2082+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2083) : move_wbs.sql(7110) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --UPDATE RESOURCE_ASSIGNMENTS
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS
            SET PROJECT_ID = h_target_project_id
            WHERE PROJECT_ID = h_source_project_id
            AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_MVW_0001);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2103+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2104) : move_wbs.sql(7131) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      -- Stop deleting DEPENDENCIES if target and source are siblings under the same parent (Tudor)
      If ((Coalesce(h_bitflag, 0) = 0) AND (PMO_ORASTD.strcmp(h_target_id
              , h_source_parent_id) <> PMO_CNST.SC_MATCH)) OR ((Coalesce(h_bitflag, 0) = 1) 
              AND ((Coalesce(h_target_level, 0) <> Coalesce(h_source_level, 0)) 
              OR (PMO_ORASTD.strcmp(h_target_parent_id
              , h_source_parent_id) <> PMO_CNST.SC_MATCH))) Then
         If (Coalesce(h_ieflag, 0) = 0) AND (Coalesce(Length(h_target_rank), 0) > 7) Then
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_MVW_IDLIST';
            PMO_DB2M.SC_STATE := 0;
            
            OPEN C_TJ;
            LOOP
               FETCH C_TJ INTO h_cur_id, h_pred_id, h_succ_id;
               IF C_TJ%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               BEGIN
                  DELETE FROM DEPENDENCIES WHERE DEPEND_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2145+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2146) : move_wbs.sql(7173) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               PMO_DB2M.SC_STATE := FC_SEARCH_IDLIST(h_pred_id);
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     INSERT INTO PMO_TMP_MVW_IDLIST
                     VALUES(h_pred_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2166+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2167) : move_wbs.sql(7194) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               PMO_DB2M.SC_STATE := FC_SEARCH_IDLIST(h_succ_id);
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  BEGIN
                     INSERT INTO PMO_TMP_MVW_IDLIST
                     VALUES(h_succ_id);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2187+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2188) : move_wbs.sql(7215) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C_TJ ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2201+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2202) : move_wbs.sql(7229) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN idListCursor;
            
            LOOP
               FETCH idListCursor INTO h_cur_id;
               IF idListCursor%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               -- null_ind already cleared at initialization
               PMO_STPU.packDependencies(h_cur_id, h_dep_str, 8193);
               
               If PMO_ORASTD.strcmp(h_dep_str, '{RS}{RF}') <> 0 Then
                  UPDATE TMT_WBS SET
                          DEPENDENCIES = NULL
                     WHERE ELEMENT_ID = H_CUR_ID;

                  BEGIN
                     UPDATE TMT_WBS
                     SET DEPENDENCIES = h_dep_str
                     WHERE ELEMENT_ID = h_cur_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2242+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2243) : move_wbs.sql(7270) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TMT_WBS
                     SET DEPENDENCIES = null
                     WHERE ELEMENT_ID = h_cur_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2261+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2262) : move_wbs.sql(7289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE idListCursor ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2275+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2276) : move_wbs.sql(7303) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      BEGIN
         SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) into h_xrank64 FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2291) : move_wbs.sql(7318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TARGET_SOURCE (
         TARGET_RANK,
         TARGET_ID,
         TARGET_PROJECT_ID,
         TARGET_PARENT_ID,
         TARGET_WBS_CODE,
         TARGET_TYPE_ID,
         TARGET_LEVEL_ID,
         TARGET_TRANSFER_FLAG,
         TARGET_CALENDAR_ID,
         TARGET_MINUTES_PER_DAY,
         TARGET_START_FINSH_DATE,
         TARGET_CURRENCY_ID,
         TARGET_PRIORITY,
         SOURCE_RANK,
         SOURCE_ID,
         SOURCE_PROJECT_ID,
         SOURCE_PARENT_ID,
         SOURCE_WBS_CODE,
         SOURCE_TYPE_ID,
         SOURCE_LEVEL_ID,
         SOURCE_TRANSFER_FLAG,
         SOURCE_CALENDAR_ID,
         SOURCE_MINUTES_PER_DAY,
         SOURCE_START_FINSH_DATE,
         SOURCE_CURRENCY_ID,
         SOURCE_PRIORITY )
         VALUES (
         h_target_rank,
         h_target_id,
         h_target_project_id,
         h_target_parent_id,
         h_target_code,
         h_parent_type,
         h_target_level,
         h_target_transfer_flag,
         h_target_calendar_id,
         h_target_minute_per_day,
         h_target_start_datetime,
         h_target_currency_id,
         h_target_priority,
         h_source_rank,
         h_source_id,
         h_source_project_id,
         h_source_parent_id,
         h_source_code,
         h_source_type,
         h_source_level,
         h_source_transfer_flag,
         h_source_calendar_id,
         h_source_minute_per_day,
         h_source_start_datetime,
         h_source_currency_id,
         h_source_priority );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2360+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2361) : move_wbs.sql(7388) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --h_bitflag = 0, the target is the new parent for the source, copy the source as last child of the target
      --h_bitflag = 1, insert the source to the target position, new source's parent is the target's parent
      If Coalesce(h_bitflag, 0) = 1 Then
         If ((h_target_parent_id IS NULL)) AND ((h_source_parent_id IS NULL)) Then
            iparent := 0;
         Else
            iparent := PMO_ORASTD.strcmp(h_target_parent_id, h_source_parent_id);
         End If;
         
         If iparent = 0 Then
            h_rollup := 0;
            -- don't need rollup
            If PMO_ORASTD.strcmp(h_source_rank, h_target_rank) > 0 Then
               h_wbs_bitflag := 0;
               PMO_DB2M.SC_STATE := FC_SIBLING_MOVE_WBS(h_source_rank, h_target_rank, h_target_project_id
                                         , h_target_minute_per_day, h_lenc, h_wbs_bitflag
                                         , h_target_code, h_xrank64, h_rec_user);
            Else
               h_wbs_bitflag := 1;
               PMO_DB2M.SC_STATE := FC_SIBLING_MOVE_WBS(h_source_rank, h_target_rank, h_target_project_id
                                         , h_target_minute_per_day, h_lenc, h_wbs_bitflag
                                         , h_target_code, h_xrank64, h_rec_user);
            End If;
         Else
            If (h_source_parent_id IS NOT NULL) Then
               BEGIN
                  UPDATE TMT_WBS SET CHILD_COUNT = CHILD_COUNT - 1, ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
                  WHERE ELEMENT_ID = h_source_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2403+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2404) : move_wbs.sql(7431) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_WBS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID  IS NuLL;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2420+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2421) : move_wbs.sql(7448) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If (h_target_parent_id IS NOT NULL) Then
               PMODEFINES.PMO_GET_MAX_RANK('TMT_WBS', h_target_parent_id, h_max_rank);
               PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
               
               BEGIN
                  SELECT RANK, WBS_CODE INTO h_cur_rank, h_wbs_code
                  FROM TMT_WBS
                  WHERE ELEMENT_ID = h_target_parent_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2440+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2441) : move_wbs.sql(7468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_cur_rank := SubStr(h_cur_rank || h_rankBase64, 1, 200);
               h_wbs_code := h_wbs_code || '.' || TO_CHAR(Coalesce(h_max_rank, 0));
            Else
               PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_WBS', h_max_rank);
               PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
               h_cur_rank := '0000' || h_rankBase64;
               h_wbs_code := '0.' || TO_CHAR(Coalesce(h_max_rank, 0));
            End If;
            
            h_diff_level := Coalesce(h_target_level, 0) - Coalesce(h_source_level, 0);
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_CRI(h_source_rank, h_cur_rank, h_target_project_id);
            
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_DOC(h_table_type, h_source_rank, h_cur_rank
                                          , h_target_project_id);
            
            PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_SCD(h_table_type, h_source_rank, h_cur_rank
                                          , h_target_project_id);
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_source_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_source_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WBS
               SET PROJECT_ID = h_target_project_id,
               MINUTE_PER_DAY = h_target_minute_per_day,
               RANK = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_cur_rank)
               ELSE RTRIM(h_cur_rank) || 
               SUBSTR(RANK, h_len +1,LENGTH(RANK)-h_len) END,
               WBS_CODE = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_wbs_code)
               ELSE RTRIM(h_wbs_code) || 
               SUBSTR(WBS_CODE, h_lenc +1,LENGTH(WBS_CODE)-h_lenc) END,
               PARENT_ID = CASE WHEN ELEMENT_ID = h_source_id
               THEN h_target_parent_id ELSE PARENT_ID END,
               LEVEL_ID = LEVEL_ID + h_diff_level,
               TRANSFER_FLAG = CASE WHEN PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 0 THEN TRANSFER_FLAG
               WHEN h_target_transfer_flag IN ('C', 'W') THEN 'W'
               WHEN h_target_transfer_flag IS NuLL AND h_source_transfer_flag = 'W' THEN NuLL
               ELSE TRANSFER_FLAG END
               WHERE RANK LIKE h_source_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2505+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2506) : move_wbs.sql(7533) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1) AND ((h_target_transfer_flag IS NOT NULL)) 
                  AND (moveCRI = 0) Then
               h_parent_id := h_target_parent_id;
               If (SubStr(h_target_transfer_flag, 1, 1) = 'C') OR (SubStr(h_target_transfer_flag, 1, 1) = 'W') Then
                  DECLARE 
                  	nLen pls_integer :=0; 
                  BEGIN 
                  	If (h_cur_rank IS NOT NULL) Then 
                  		nLen := Coalesce(Length(h_cur_rank), 0); 
                  		If 0 <> 0 AND nLen <> 0 Then 
                  		nLen := nLen - 1; 
                  		End If; 
                  	End If; 
                  	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
                  		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_cur_rank, False); 
                  	End If; 
                  END;
                  
                  OPEN C_INSERT_CRI;
                  
                  LOOP
                     FETCH C_INSERT_CRI INTO h_cur_id;
                     IF C_INSERT_CRI%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                     If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                        EXIT;
                     End If;
                     
                     h_tmp_flag := 0;
                     PMO_DB2M.SC_STATE := FC_INSERT_CRI_AFTER_WBS(h_parent_id, h_cur_id, h_rec_user, h_tmp_flag);
                     h_parent_id := h_cur_id;
                  END LOOP;
                  BEGIN
                     CLOSE C_INSERT_CRI ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2553+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2554) : move_wbs.sql(7581) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            h_lenc := Coalesce(Length(h_wbs_code), 0);
            h_wbs_bitflag := 0;
            PMO_DB2M.SC_STATE := FC_SIBLING_MOVE_WBS(h_cur_rank, h_target_rank, h_target_project_id
                                         , h_target_minute_per_day, h_lenc, h_wbs_bitflag
                                         , h_target_code, h_xrank64, h_rec_user);
         End If;
      Else
         If PMO_ORASTD.strcmp(h_target_id, h_source_parent_id) = PMO_CNST.SC_MATCH Then
            BEGIN
               SELECT CASE WHEN MAX(RANK) = h_source_rank THEN 1 ELSE 0 END INTO h_diff_level
               FROM TMT_WBS WHERE PARENT_ID = h_target_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2576+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2577) : move_wbs.sql(7604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_diff_level, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_M_WBS_MSP', 2583+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_M_WBS_MSP(2584) : move_wbs.sql(7611) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || h_source_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 2586+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(2587) : move_wbs.sql(7614) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMODEFINES.PMO_RETURN_ERROR(0, h_source_id, False);
            End If;
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_WBS', h_target_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         h_target_rank := SubStr(h_target_rank || h_rankBase64, 1, 200);
         h_target_code := h_target_code || '.' || TO_CHAR(Coalesce(h_max_rank, 0));
         h_diff_level := Coalesce(h_target_level, 0) - Coalesce(h_source_level, 0) + 1;
         
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_CRI(h_source_rank, h_target_rank, h_target_project_id);
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_DOC(h_table_type, h_source_rank, h_target_rank
                                          , h_target_project_id);
         
         PMO_DB2M.SC_STATE := FC_UPDATE_ATTACH_SCD(h_table_type, h_source_rank, h_target_rank
                                          , h_target_project_id);
         
         -- update the level, rec_user, rec_datetime, rec_status, rank
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_WBS
            SET PROJECT_ID = h_target_project_id,
            MINUTE_PER_DAY = h_target_minute_per_day,
            LEVEL_ID = LEVEL_ID + h_diff_level,
            REC_DATETIME = h_cur_date,
            LAST_CHECKIN = h_cur_date,
            RANK = CASE WHEN LENGTH(RANK) = h_len THEN h_target_rank
            ELSE h_target_rank || SUBSTR(RANK, h_len +1,LENGTH(RANK)-h_len) END,
            WBS_CODE = CASE WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_code)
            ELSE RTRIM(h_target_code) || SUBSTR(WBS_CODE, h_lenc +1, LENGTH(WBS_CODE)-h_lenc) END,
            TRANSFER_FLAG = CASE WHEN PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 0 THEN TRANSFER_FLAG
            WHEN h_target_transfer_flag IN ('C', 'W') THEN 'W'
            WHEN h_target_transfer_flag IS NuLL AND h_source_transfer_flag = 'W' THEN NuLL
            ELSE TRANSFER_FLAG END
            WHERE RANK LIKE h_source_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2647+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2648) : move_wbs.sql(7675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --update new parent for the source
         BEGIN
            UPDATE TMT_WBS
            SET PROJECT_ID    = h_target_project_id,
            PARENT_ID     = h_target_id
            WHERE ELEMENT_ID = h_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2668+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2669) : move_wbs.sql(7696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- move under transferred cri or requirement
         If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1) AND ((h_target_transfer_flag IS NOT NULL)) 
               AND (moveCRI = 0) Then
            h_parent_id := h_target_id;
            
            If (SubStr(h_target_transfer_flag, 1, 1) = 'C') OR (SubStr(h_target_transfer_flag, 1, 1) = 'W') Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_target_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_target_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, False); 
               	End If; 
               END;
               
               OPEN C_INSERT_CRIa;
               
               LOOP
                  FETCH C_INSERT_CRIa INTO h_cur_id;
                  IF C_INSERT_CRIa%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                     EXIT;
                  End If;
                  
                  h_tmp_flag := 0;
                  PMO_DB2M.SC_STATE := FC_INSERT_CRI_AFTER_WBS(h_parent_id, h_cur_id, h_rec_user, h_tmp_flag);
                  h_parent_id := h_cur_id;
               END LOOP;
               BEGIN
                  CLOSE C_INSERT_CRIa ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2718+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2719) : move_wbs.sql(7746) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         If (h_source_parent_id IS NOT NULL) Then
            BEGIN
               UPDATE TMT_WBS SET CHILD_COUNT = CHILD_COUNT - 1, ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
               WHERE ELEMENT_ID = h_source_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2739+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2740) : move_wbs.sql(7767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WBS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID IS NuLL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2756+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2757) : move_wbs.sql(7784) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If acrossProject = 1 Then
            -- Move Assignments with effort
            h_temp_var := FC_M_ASSIGNMENTS(h_rec_user);
         End If;
         
         If (((h_source_timecode_1_id IS NULL)) AND ((h_source_timecode_2_id IS NULL))) 
               AND (((h_target_timecode_1_id IS NOT NULL)) 
               OR ((h_target_timecode_2_id IS NOT NULL))) Then
            h_temp_var := FC_U_ALL_NULL_TIMECODES(h_target_rank, h_target_timecode_1_id, h_target_timecode_2_id);
         End If;
      End If;
   End If;
   --trace_printf("h_bitflag: %d, iparent: %d, SC_STATE: %d\n",h_bitflag,iparent,SC_STATE); //trace_flush();
   --trace_printf("h_target_id: %d:%s\n",h_target_id_Ind,h_target_id); //trace_flush();
   --trace_printf("h_target_parent_id: %d:%s\n",h_target_parent_id_Ind,h_target_parent_id); //trace_flush();
   --trace_printf("h_source_parent_id: %d:%s\n",h_source_parent_id_Ind,h_source_parent_id); //trace_flush();
   --trace_printf("h_source_project_id: %d:%s\n",h_source_project_id_Ind,h_source_project_id); //trace_flush();
   
   If moveCRI = 1 Then
      h_host_table := 0;
      h_element_id := h_source_id;
      h_source_project_id := NULL;
      PMO_DB2M.SC_STATE := FC_M_ATTACH_CRI(h_host_table, h_target_id, h_source_id, h_bitflag, h_rec_user
                                     , h_element_id, h_source_project_id);
   End If;
   
   If Coalesce(h_bitflag, 0) = 0 Then
      If (h_target_id IS NULL) Then
         BEGIN
            UPDATE TMT_WBS
            SET ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
            WHERE ELEMENT_ID = h_source_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2804+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2805) : move_wbs.sql(7832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_WBS
            SET ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
            WHERE ELEMENT_ID = h_target_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2823+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2824) : move_wbs.sql(7851) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   ElsIf iparent <> 0 Then
      BEGIN
         UPDATE TMT_WBS
         SET ROLLUP_BITS =  PMO_CNST.SC_SET_ALL_BITS
         WHERE ELEMENT_ID = h_target_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2843+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2844) : move_wbs.sql(7871) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --trace_printf("h_rollup: %d, acrossProject: %d, SC_STATE: %d\n",h_rollup,acrossProject,SC_STATE); //trace_flush();
   If Coalesce(h_rollup, 0) = 1 Then
      If (h_target_id IS NULL) Then
         PMO_DB2M.SC_STATE := FC_Rollup(h_source_parent_id);
      Else
         PMO_DB2M.SC_STATE := FC_Rollup(h_target_project_id);
         
         If (acrossProject = 1) AND ((h_source_parent_id IS NOT NULL)) Then
            If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
               BEGIN
                  SELECT PROJECT_ID INTO h_source_project_id
                  FROM TMT_WBS
                  WHERE ELEMENT_ID = h_source_parent_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_WBS_MSP', 2868+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_WBS_MSP(2869) : move_wbs.sql(7896) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            PMO_DB2M.SC_STATE := FC_Rollup(h_source_project_id);
         End If;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   --trace_printf("END: SC_STATE: %d\n",SC_STATE); //trace_flush();
   
   h_error_id := h_target_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_WBS_MSP', 2885+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_WBS_MSP(2886) : move_wbs.sql(7913) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_WBS_MSP;
ALTER FUNCTION FC_M_WBS_MSP COMPILE DEBUG;

/***************************************************************************/

/* Move does not create new IDs, we only update the RANKs   */

/***************************************************************************/

Prompt == FUNCTION SP_M_WBS_MSP ==
CREATE OR REPLACE FUNCTION SP_M_WBS_MSP
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
   , p_ieflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_source_id VARCHAR2(32);
   h_target_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_WBS_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_WBS_MSP'), '-- ' || SYSTIMESTAMP, ':r := SP_M_WBS_MSP(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(p_ieflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_WBS_MSP', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_WBS_MSP(29) : move_wbs.sql(7956) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_target_id := p_target_id;
   h_source_id := p_source_id;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   h_ieflag := p_ieflag;
   
   h_error_id := h_target_id;
   
   PMO_DB2M.SC_STATE := FC_M_WBS_MSP(h_target_id, h_source_id, h_bitflag, h_rec_user, h_ieflag
                                  , h_error_id);
   
   --  if (SC_STATE == CAN_NOT_MOVE_TRANSFERED_CRI)
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_WBS_MSP', 47+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_WBS_MSP(48) : move_wbs.sql(7975) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_error_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WBS_MSP', 55+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WBS_MSP(56) : move_wbs.sql(7983) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WBS_MSP', 67+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WBS_MSP(68) : move_wbs.sql(7995) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_WBS_MSP;
ALTER FUNCTION SP_M_WBS_MSP COMPILE DEBUG;

/*********************************************************************************/

Prompt == FUNCTION SP_ROLLUP_BUDGET ==
CREATE OR REPLACE FUNCTION SP_ROLLUP_BUDGET
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_cur_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   tmp_res PLS_INTEGER;
-- Declare Cursors
      CURSOR C_RBUDGETS IS
         SELECT BUDGET_ID FROM BUDGETS
         WHERE PROJECT_ID = h_project_id  AND ELEMENT_ID = p_id;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ROLLUP_BUDGET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ROLLUP_BUDGET'), '-- ' || SYSTIMESTAMP, ':r := SP_ROLLUP_BUDGET(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ROLLUP_BUDGET', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ROLLUP_BUDGET(25) : move_wbs.sql(8036) : ' || SYSTIMESTAMP);
   END IF; 

   h_bitflag := p_bitflag;
   
   BEGIN
      SELECT PROJECT_ID INTO h_project_id
      FROM BUDGETS
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_BUDGET', 39+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_BUDGET(40) : move_wbs.sql(8051) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_RBUDGETS;
   
   LOOP
      FETCH C_RBUDGETS INTO h_cur_id;
      IF C_RBUDGETS%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_RBUDGETS ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_BUDGET', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_BUDGET(67) : move_wbs.sql(8078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ROLLUP_BUDGET', 72+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ROLLUP_BUDGET(73) : move_wbs.sql(8084) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_BUDGET', 79+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_BUDGET(80) : move_wbs.sql(8091) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ROLLUP_BUDGET', 91+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ROLLUP_BUDGET(92) : move_wbs.sql(8103) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ROLLUP_BUDGET;
ALTER FUNCTION SP_ROLLUP_BUDGET COMPILE DEBUG;

/**********************************************************************************************/

Prompt == FUNCTION FC_M_PORTF ==
CREATE OR REPLACE FUNCTION FC_M_PORTF
(
     h_target_id IN OUT NOCOPY VARCHAR2
   , h_source_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_element_name VARCHAR2(255);
   h_target_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_next_rank VARCHAR2(200);
   h_extend_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_target_parent_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_target_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_cur_rank VARCHAR2(200);
   h_res_row_rank VARCHAR2(200);
   h_element_parent_id VARCHAR2(32);
   h_element_level_id PLS_INTEGER := 0;
   h_parent_rank VARCHAR2(200);
   h_temp_rank VARCHAR2(200);
   h_error_id VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_flag PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_M_PORTF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_M_PORTF'), '-- ' || SYSTIMESTAMP, ':r := FC_M_PORTF(' || 
       '''' || h_target_id || ''''
          || ', ' || '''' || h_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 43+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_M_PORTF(44) : move_wbs.sql(8163) : ' || SYSTIMESTAMP);
   END IF; 

   If PMO_ORASTD.strcmp(h_target_id, h_source_id) = PMO_CNST.SC_MATCH Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 48+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(49) : move_wbs.sql(8168) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SAME_TARGET_SOURCE;
   End If;
   
   BEGIN
      SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) INTO h_extend_rank
      FROM DUAL
      WHERE 1 = 1;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 63+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(64) : move_wbs.sql(8183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT RANK
      INTO h_res_row_rank
      FROM TMT_PORTFOLIOS
      WHERE ELEMENT_ID = h_rec_user;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(80) : move_wbs.sql(8199) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT tpf.RANK, LENGTH(tpf.RANK), tpf.TYPE_ID, tpf.PARENT_ID, tpf.LEVEL_ID
      INTO h_rank, h_len, h_type_id, h_parent_id, h_level_id
      FROM TMT_PORTFOLIOS tpf
      WHERE tpf.ELEMENT_ID = h_source_id
      AND ((h_res_row_rank >= SUBSTR(tpf.RANK, 1, 8) AND h_res_row_rank <= tpf.RANK) 
      AND SUBSTR(tpf.RANK,1, CASE WHEN LENGTH(h_res_row_rank) > LENGTH(tpf.RANK) THEN 1 ELSE LENGTH(h_res_row_rank) END ) = h_res_row_rank);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(98) : move_wbs.sql(8217) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_bitflag: %d, h_res_row_rank: %s, h_rank: %s\n", SC_STATE,h_bitflag,h_res_row_rank,h_rank);
   
   If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      BEGIN
         SELECT ELEMENT_NAME, TYPE_ID
         INTO h_element_name, h_type
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 115+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(116) : move_wbs.sql(8235) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If (h_target_id IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 123+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(124) : move_wbs.sql(8243) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.SAME_TARGET_SOURCE;
         Else
            If Coalesce(h_bitflag, 0) = 3 Then
               BEGIN
                  SELECT PARENT_ID,RANK,LEVEL_ID
                  INTO h_element_parent_id,h_target_rank,h_target_level_id
                  FROM TMT_PORTFOLIOS
                  WHERE ELEMENT_ID = h_target_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 139+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(140) : move_wbs.sql(8259) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_element_parent_id := h_target_id;
            End If;
            PMODEFINES.PMO_GET_MAX_RANK('TMT_PORTFOLIOS', h_element_parent_id, h_max_rank);
            
            BEGIN
               SELECT LEVEL_ID+1, RANK INTO h_element_level_id, h_parent_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_element_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 158+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(159) : move_wbs.sql(8278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            BEGIN
               INSERT INTO TMT_PORTFOLIOS (RANK, TYPE_ID,
               LEVEL_ID, PARENT_ID, ELEMENT_ID, REC_STATUS)
               VALUES (h_parent_rank || h_rankBase64, h_type,
               h_element_level_id, h_element_parent_id, h_source_id, 'I');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 180+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(181) : move_wbs.sql(8300) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_bitflag, 0) = 3 Then
               h_rank := h_parent_rank;
               h_rank := SubStr(h_rank || h_rankBase64, 1, 200);
               
               PMO_DB2M.SC_STATE := FC_MOVE_UP_PRTF(h_target_level_id, h_target_rank, h_rank, h_rec_user);
            End If;
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 194+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(195) : move_wbs.sql(8314) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
      End If;
   Else
      h_temp_rank := NULL;
      BEGIN
         SELECT RANK INTO h_temp_rank
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 210+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(211) : move_wbs.sql(8330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("0____* SC_STATE: %d, h_source_id: %s, h_temp_rank: %d %s\n", SC_STATE,h_source_id,h_temp_rank_Ind,h_temp_rank);
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_count := 0;
         
         --[Asad] Checks if at least one of the parents is locked or not.
         BEGIN
            SELECT COUNT(*) INTO h_count
            FROM TMT_WBS
            WHERE (SUBSTR(h_temp_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_temp_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
            AND (RANK LIKE SUBSTR(h_temp_rank, 1, 8) || '%')) AND REC_STATUS = 'L';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 231+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(232) : move_wbs.sql(8351) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1____* h_count: %d, h_temp_rank: %d %s\n", h_count,h_temp_rank_Ind,h_temp_rank);
         
         If Coalesce(h_count, 0) > 0 Then
            h_error_id := h_source_id;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 241+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(242) : move_wbs.sql(8361) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.ITEM_IS_LOCKED;
            --/return PARENT_MARKED_AS_LOCKED;
         End If;
         
         --[Asad] Checks if at least one of the children is locked or not.
         BEGIN
            SELECT COUNT(*) INTO h_count
            FROM TMT_WBS
            WHERE (RANK LIKE h_temp_rank || '%')
            AND REC_STATUS = 'L';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 259+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(260) : move_wbs.sql(8379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2____* h_count: %d, h_temp_rank: %d %s\n", h_count,h_temp_rank_Ind,h_temp_rank);
         
         If Coalesce(h_count, 0) > 0 Then
            h_error_id := h_source_id;
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 269+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(270) : move_wbs.sql(8389) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.ITEM_IS_LOCKED;
         End If;
      End If;
      
      If Coalesce(h_bitflag, 0) = 3 Then
         If (h_target_id IS NOT NULL) Then
            BEGIN
               SELECT PARENT_ID,RANK,LEVEL_ID INTO h_target_parent_id,h_target_rank,h_target_level_id
               FROM  TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_target_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 287+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(288) : move_wbs.sql(8407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 293+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(294) : move_wbs.sql(8413) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
            End If;
            
            -- target and source have the same parent
            If PMO_ORASTD.strcmp(h_target_parent_id, h_parent_id) = PMO_CNST.SC_MATCH Then
               If PMO_ORASTD.strcmp(h_rank, h_target_rank) > 0 Then
                  PMO_DB2M.SC_STATE := FC_MOVE_UP_PRTF(h_target_level_id, h_target_rank, h_rank, h_rec_user);
               Else
                  PMO_DB2M.SC_STATE := FC_MOVE_DOWN_PRTF(h_target_level_id, h_target_rank, h_rank, h_rec_user);
               End If;
            Else
               h_temp_flag := 0;
               PMO_DB2M.SC_STATE := FC_MOVE_DIF_PRTF(h_target_level_id, h_level_id, h_target_rank, h_rank
                                      , h_target_parent_id, h_type_id, h_rec_user, h_temp_flag);
               
               BEGIN
                  UPDATE TMT_PORTFOLIOS
                  SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 324+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(325) : move_wbs.sql(8444) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         If (h_target_id IS NULL) Then
            If isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
               BEGIN
                  UPDATE TMT_PORTFOLIOS
                  SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 347+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(348) : move_wbs.sql(8467) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  DELETE FROM TMT_PORTFOLIOS
                  WHERE PARENT_ID = h_parent_id
                  AND ELEMENT_ID = h_source_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 366+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(367) : move_wbs.sql(8486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               If Coalesce(h_level_id, 0) <> -1 Then
                  BEGIN
                     UPDATE TMT_PORTFOLIOS
                     SET CHILD_COUNT = CHILD_COUNT - 1
                     WHERE ELEMENT_ID = h_parent_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 386+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(387) : move_wbs.sql(8506) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               --create a new rank for folder
               PMODEFINES.PMO_GET_MAX_RANK('TMT_PORTFOLIOS', h_rec_user, h_max_rank);
               PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
               
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_rank, False); 
               	End If; 
               END;
               BEGIN
                  UPDATE TMT_PORTFOLIOS
                  SET RANK      = CASE WHEN LENGTH(RANK) = h_len THEN h_res_row_rank || h_rankBase64
                  ELSE h_res_row_rank || h_rankBase64 || SUBSTR(RANK, h_len + 1, LENGTH(RANK) - h_len) END,
                  LEVEL_ID  = LEVEL_ID - h_level_id + 1,
                  PARENT_ID = CASE WHEN ELEMENT_ID = h_source_id THEN h_rec_user
                  ELSE PARENT_ID END
                  WHERE RANK LIKE h_rank || '%';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 427+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(428) : move_wbs.sql(8547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            PMODEFINES.PMO_GET_MAX_RANK('TMT_PORTFOLIOS', h_target_id, h_max_rank);
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            BEGIN
               SELECT LEVEL_ID, RANK INTO h_target_level_id, h_target_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_target_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 446+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(447) : move_wbs.sql(8566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_level_id, 0) <> -1 Then
               BEGIN
                  UPDATE TMT_PORTFOLIOS
                  SET CHILD_COUNT = CHILD_COUNT - 1
                  WHERE ELEMENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 466+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(467) : move_wbs.sql(8586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_rank, False); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_PORTFOLIOS
               SET RANK = CASE WHEN LENGTH(RANK) = h_len THEN h_target_rank || h_rankBase64
               ELSE h_target_rank || h_rankBase64 || SUBSTR(RANK,h_len +1, LENGTH(RANK)-h_len) END,
               LEVEL_ID = LEVEL_ID - h_level_id + h_target_level_id + 1,
               PARENT_ID = CASE WHEN ELEMENT_ID = h_source_id THEN h_target_id
               ELSE PARENT_ID END
               WHERE RANK LIKE h_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_M_PORTF', 503+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_M_PORTF(504) : move_wbs.sql(8623) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 512+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(513) : move_wbs.sql(8632) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_M_PORTF', 516+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_M_PORTF(517) : move_wbs.sql(8636) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_M_PORTF;
ALTER FUNCTION FC_M_PORTF COMPILE DEBUG;

/************************************************************/

Prompt == FUNCTION SP_M_PORTF ==
CREATE OR REPLACE FUNCTION SP_M_PORTF
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_PORTF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_PORTF'), '-- ' || SYSTIMESTAMP, ':r := SP_M_PORTF(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_PORTF', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_PORTF(26) : move_wbs.sql(8672) : ' || SYSTIMESTAMP);
   END IF; 

   -- Validate target  source id
   If (p_source_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_PORTF', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_PORTF(32) : move_wbs.sql(8678) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_PORTF', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_PORTF(35) : move_wbs.sql(8681) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((p_source_id IS NULL)) OR ((p_rec_user IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_PORTF', 41+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_PORTF(42) : move_wbs.sql(8688) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_PORTF', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_PORTF(45) : move_wbs.sql(8691) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   -- Parse parameters to local variables
   h_target_id := p_target_id;
   h_source_id := p_source_id;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   h_temp_sc_state := FC_M_PORTF(h_target_id, h_source_id, h_bitflag, h_rec_user);
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_PORTF', 59+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_PORTF(60) : move_wbs.sql(8706) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_target_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(Coalesce(h_temp_sc_state, 0), h_target_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_PORTF', 66+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_PORTF(67) : move_wbs.sql(8713) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_PORTF', 78+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_PORTF(79) : move_wbs.sql(8725) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_PORTF;
ALTER FUNCTION SP_M_PORTF COMPILE DEBUG;

/*****************************************************************************************************
p_bitflag:  the first bit used to indicate moving operation (moving as child (0) or sibling move (1))
            second bit used to indicate which object will be moved (wbs (0) or PORTFOLIO (1))
            third bit used to indicate varify external dependencies.
******************************************************************************************************/

Prompt == FUNCTION SP_M_WBS ==
CREATE OR REPLACE FUNCTION SP_M_WBS
(
     p_target_id VARCHAR2
   , p_source_id_list VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_remain_id_list VARCHAR2(2047);
   h_error_id VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_x_element_id VARCHAR2(32);
   withError PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_M_WBS(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id_list || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_WBS', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_WBS(32) : move_wbs.sql(8777) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_target_id := p_target_id;
   h_remain_id_list := p_source_id_list;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   h_error_id := h_target_id;
   
   h_len := Coalesce(Length(h_remain_id_list), 0);
   h_len := Coalesce(h_len, 0);
   PMO_DB2M.SC_STATE := 0;
   
   WHILE (Coalesce(h_len, 0) >= 32) AND (PMO_DB2M.SC_STATE >= 0) LOOP
      h_source_id := SubStr(h_remain_id_list, 1, 32);
      
      If (BITAND(Coalesce(h_bitflag, 0), 2)) > 0 Then
         If Coalesce(h_bitflag, 0) > 3 Then
            h_bitflag := Coalesce(h_bitflag, 0) - 4;
         End If;
         
         PMO_DB2M.SC_STATE := FC_M_PORTF(h_target_id, h_source_id, h_bitflag, h_rec_user);
      Else
         PMO_DB2M.SC_STATE := FC_M_WBS_MSP(h_target_id, h_source_id, h_bitflag, h_rec_user, h_ieflag
                                  , h_error_id);
      End If;
      
      If PMO_DB2M.SC_STATE = 0 Then
         COMMIT;
      Else
         withError := PMO_DB2M.SC_STATE;
         h_x_element_id := h_source_id;
         
ROLLBACK;      End If;
      
      h_remain_id_list := PMO_STPU.strnpcpy(2047, -1, 33, h_remain_id_list);
      h_len := Coalesce(Length(h_remain_id_list), 0);
      h_len := Coalesce(h_len, 0);
   END LOOP;
   
   If withError <> 0 Then
      h_error_id := h_x_element_id;
      PMO_DB2M.SC_STATE := withError;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_WBS', 79+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_WBS(80) : move_wbs.sql(8825) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_error_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WBS', 86+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WBS(87) : move_wbs.sql(8832) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WBS', 98+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WBS(99) : move_wbs.sql(8844) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_WBS;
ALTER FUNCTION SP_M_WBS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_C_DETAILS ==
CREATE OR REPLACE FUNCTION FC_C_DETAILS
(
     h_source_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_DETAILS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_DETAILS'), '-- ' || SYSTIMESTAMP, ':r := FC_C_DETAILS(' || 
       '''' || h_source_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_DETAILS', 16+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_DETAILS(17) : move_wbs.sql(8877) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      UPDATE PROJECT_DETAILS SET (
      PROJECT_RTF_BITS,
      PROJECT_RTF_BITS2,
      DELIVERABLE_RTF_BITS,
      WORK_PRODUCT_RTF_BITS,
      SUMMARY_TASK_RTF_BITS,
      MILESTONE_RTF_BITS,
      TASK_RTF_BITS,
      REQUIREMENT_RTF_BITS,
      CHANGE_REQUEST_RTF_BITS,
      SERVICE_REQUEST_RTF_BITS,
      DEFECT_RTF_BITS,
      ISSUE_RTF_BITS,
      RISK_RTF_BITS,
      CRI_DELIVERABLE_RTF_BITS,
      CRI_SUMMARY_TASK_RTF_BITS,
      CRI_MILESTONE_RTF_BITS,
      CRI_TASK_RTF_BITS,
      DOCUMENT_RTF_BITS,
      ACTIONITEM_RTF_BITS,
      URL_1, URL_2, URL_3, URL_4, URL_5,
      BUDGET_SETTING_BITS,
      TIMEPHASE_PROJECT_ASSIGNMENT,
      NOTEMIN_RTF_BITS,
      SCORECARD_RTF_BITS,
      ENABLE_EV_CALCULATION,
      CHARGEBACK_FORMULA,
      ASSIGNMENT_BILLABLE_FLAG,
      NOTIFY_TM_REJECTED, NOTIFY_TM_ACCEPTED, NOTIFY_PM_SUBMITTED, NOTIFY_RM_SUBMITTED, NOTIFY_RM_ACCEPTED) =
      (SELECT PROJECT_RTF_BITS,
      PROJECT_RTF_BITS2,
      DELIVERABLE_RTF_BITS,
      WORK_PRODUCT_RTF_BITS,
      SUMMARY_TASK_RTF_BITS,
      MILESTONE_RTF_BITS,
      TASK_RTF_BITS,
      REQUIREMENT_RTF_BITS,
      CHANGE_REQUEST_RTF_BITS,
      SERVICE_REQUEST_RTF_BITS,
      DEFECT_RTF_BITS,
      ISSUE_RTF_BITS,
      RISK_RTF_BITS,
      CRI_DELIVERABLE_RTF_BITS,
      CRI_SUMMARY_TASK_RTF_BITS,
      CRI_MILESTONE_RTF_BITS,
      CRI_TASK_RTF_BITS,
      DOCUMENT_RTF_BITS,
      ACTIONITEM_RTF_BITS,
      URL_1, URL_2, URL_3, URL_4, URL_5,
      BUDGET_SETTING_BITS,
      TIMEPHASE_PROJECT_ASSIGNMENT,
      NOTEMIN_RTF_BITS,
      SCORECARD_RTF_BITS,
      ENABLE_EV_CALCULATION,
      CHARGEBACK_FORMULA,
      ASSIGNMENT_BILLABLE_FLAG,
      NOTIFY_TM_REJECTED, NOTIFY_TM_ACCEPTED, NOTIFY_PM_SUBMITTED, NOTIFY_RM_SUBMITTED, NOTIFY_RM_ACCEPTED
      FROM PROJECT_DETAILS
      WHERE PROJECT_ID = h_source_id )
      WHERE PROJECT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DETAILS', 90+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DETAILS(91) : move_wbs.sql(8951) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("\n h_source_id:%s", h_source_id);
   --trace_printf("\n h_project_id:%s", h_project_id);
   
   BEGIN
      INSERT INTO PROJECT_STATE_DEF (
      ELEMENT_ID,
      PROJECT_ID,
      PARENT_ID,
      ELEMENT_TYPE,
      STAGE_ID,
      ASSIGN_FLAG,
      GROUP_IDS,
      PARTICIPANT_IDS,
      SECURITY_IDS,
      SUBJECT,
      MESSAGE,
      GROUP_NAMES,
      PARTICIPANT_NAMES,
      SECURITY_NAMES)
      SELECT
      SYS_GUID(),
      h_project_id,
      h_project_id,
      ELEMENT_TYPE,
      STAGE_ID,
      ASSIGN_FLAG,
      GROUP_IDS,
      PARTICIPANT_IDS,
      SECURITY_IDS,
      SUBJECT,
      MESSAGE,
      GROUP_NAMES,
      PARTICIPANT_NAMES,
      SECURITY_NAMES
      FROM PROJECT_STATE_DEF WHERE PROJECT_ID = h_source_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DETAILS', 140+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DETAILS(141) : move_wbs.sql(9001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("2_PROJECT_DETAILS: SC_STATE: %d\n", SC_STATE);
   
   -- copy alerts settings
   BEGIN
      INSERT INTO PROJECT_ALERTS(PROJECT_ALERT_ID, PROJECT_ID, ALERT_ID, VARIANCE, REC_USER)
      (SELECT SYS_GUID(), h_project_id, ALERT_ID, VARIANCE, h_rec_user
      FROM PROJECT_ALERTS
      WHERE PROJECT_ID = h_source_id);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_DETAILS', 162+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_DETAILS(163) : move_wbs.sql(9023) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_DETAILS', 168+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_DETAILS(169) : move_wbs.sql(9029) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_DETAILS;
ALTER FUNCTION FC_C_DETAILS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_C_TPH_PRJ_ASSIGNMENT ==
CREATE OR REPLACE FUNCTION FC_C_TPH_PRJ_ASSIGNMENT
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_element_id VARCHAR2(32);
   h_task_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_effort PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_TPH_PA1 IS
         SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)),
         BEST, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_TPH_PRJ_ASSIGNMENT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_TPH_PRJ_ASSIGNMENT'), '-- ' || SYSTIMESTAMP, ':r := FC_C_TPH_PRJ_ASSIGNMENT(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_TPH_PRJ_ASSIGNMENT', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_TPH_PRJ_ASSIGNMENT(35) : move_wbs.sql(9074) : ' || SYSTIMESTAMP);
   END IF; 

   h_tph_delete := 0;
   
   OPEN C_TPH_PA1;
   
   LOOP
      FETCH C_TPH_PA1 INTO h_tsk_assignment_id, h_res_id, h_SD, h_FD,
      h_effort, h_billable, h_capital_expense, h_reg_cost, h_reg_sale;
      IF C_TPH_PA1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      --trace_printf("*** h_billable: %d, h_tsk_assignment_id: %s, h_res_id: %s, h_effort: %d\n", h_billable,h_tsk_assignment_id,h_res_id,h_effort);
      
      h_temp_sc_state := FC_UPDATE_LABOR_BUDGET(h_project_id, h_project_id, h_tsk_assignment_id
                                          , h_rec_user);
      
      If Coalesce(h_billable, 0) = 1 Then
         --BILLABLE
         BEGIN
            UPDATE PROJECT_ASSIGNMENTS SET
            ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_reg_sale
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_TPH_PRJ_ASSIGNMENT', 73+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_TPH_PRJ_ASSIGNMENT(74) : move_wbs.sql(9113) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         --NON-BILLABLE
         BEGIN
            UPDATE PROJECT_ASSIGNMENTS SET
            ESTIMATED_PLANNED_REVENUE = 0.00
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_TPH_PRJ_ASSIGNMENT', 93+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_TPH_PRJ_ASSIGNMENT(94) : move_wbs.sql(9133) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_tsk_assignment_id, h_SD
                                          , h_FD, h_reg_cost, h_reg_sale, h_tph_delete
                                          , h_capital_expense, h_billable);
   END LOOP;
   
   BEGIN
      CLOSE C_TPH_PA1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_TPH_PRJ_ASSIGNMENT', 112+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_TPH_PRJ_ASSIGNMENT(113) : move_wbs.sql(9152) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASSIGNMENT_BIT+PMO_CNST.SC_UPDATE_BUDGET_BIT)
      WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_TPH_PRJ_ASSIGNMENT', 130+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_TPH_PRJ_ASSIGNMENT(131) : move_wbs.sql(9170) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_TPH_PRJ_ASSIGNMENT', 136+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_TPH_PRJ_ASSIGNMENT(137) : move_wbs.sql(9176) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_TPH_PRJ_ASSIGNMENT;
ALTER FUNCTION FC_C_TPH_PRJ_ASSIGNMENT COMPILE DEBUG;

/*********************************************************************************************************/

Prompt == FUNCTION FC_C_WBS_MSP ==
CREATE OR REPLACE FUNCTION FC_C_WBS_MSP
(
     h_target_id IN OUT NOCOPY VARCHAR2
   , h_source_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , sh_h_ieflag PLS_INTEGER
   , h_error_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_ieflag PLS_INTEGER := sh_h_ieflag;
   h_255 VARCHAR2(254);
   h_assign_id VARCHAR2(32);
   h_c255_1 VARCHAR2(254);
   h_c50_3 VARCHAR2(50);
   h_calendar_id VARCHAR2(32);
   h_contour VARCHAR2(8192);
   h_cur_date TIMESTAMP;
   h_cur_id VARCHAR2(32);
   h_cur_rank VARCHAR2(200);
   h_d2 VARCHAR2(10);
   h_d3 VARCHAR2(10);
   h_d4 VARCHAR2(10);
   h_dayofyear DATE;
   h_doc_id VARCHAR2(32);
   h_dt1 TIMESTAMP;
   h_dt10 TIMESTAMP;
   h_dt11 TIMESTAMP;
   h_dt12 TIMESTAMP;
   h_dt13 TIMESTAMP;
   h_dt2 TIMESTAMP;
   h_dt3 TIMESTAMP;
   h_dt4 TIMESTAMP;
   h_dt6 TIMESTAMP;
   h_SD DATE;
   h_FD DATE;
   h_rup_home_path VARCHAR2(255);
   h_project_name VARCHAR2(255);
   h_element_name VARCHAR2(255);
   h_error VARCHAR2(19);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_assign_string VARCHAR2(4000);
   h_full_name VARCHAR2(255);
   h_creator_name VARCHAR2(255);
   h_history_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_id1 VARCHAR2(32);
   h_id2 VARCHAR2(32);
   h_id3 VARCHAR2(32);
   h_id5 VARCHAR2(32);
   h_min VARCHAR2(19);
   h_pa_timephase VARCHAR2(1);
   h_new_id VARCHAR2(32);
   h_new_rank VARCHAR2(200);
   h_old_rank VARCHAR2(200);
   h_old_tsk_assign_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_published VARCHAR2(1);
   h_rankBase64 VARCHAR2(8);
   h_rate_id VARCHAR2(32);
   h_real_calendar_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_created_by VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_ret_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_source_date TIMESTAMP;
   h_source_rank VARCHAR2(200);
   h_source_timecode_1_id VARCHAR2(32);
   h_source_timecode_2_id VARCHAR2(32);
   h_target_date TIMESTAMP;
   h_target_finish_date TIMESTAMP;
   h_target_rank VARCHAR2(200);
   h_target_transfer_flag VARCHAR2(1);
   h_target_timecode_1_id VARCHAR2(32);
   h_target_timecode_2_id VARCHAR2(32);
   h_tmp_code VARCHAR2(269);
   h_tsk_assignment_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_wbs_code VARCHAR2(255);
   h_sc_new_id VARCHAR2(32);
   h_xrank64 VARCHAR2(200);
   h_available PLS_INTEGER := 0;
   h_best PLS_INTEGER := 0;
   h_buffer_percent PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_lenc PLS_INTEGER := 0;
   h_method PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_n11 PLS_INTEGER := 0;
   h_n16 PLS_INTEGER := 0;
   h_n17 PLS_INTEGER := 0;
   h_n2 PLS_INTEGER := 0;
   h_n25 PLS_INTEGER := 0;
   h_n26 PLS_INTEGER := 0;
   h_n27 PLS_INTEGER := 0;
   h_cri_count PLS_INTEGER := 0;
   h_doc_count PLS_INTEGER := 0;
   h_scorecard_count PLS_INTEGER := 0;
   h_n3 PLS_INTEGER := 0;
   h_n4 PLS_INTEGER := 0;
   h_element_type PLS_INTEGER := 0;
   h_n6 PLS_INTEGER := 0;
   h_n7 PLS_INTEGER := 0;
   h_n8 PLS_INTEGER := 0;
   h_n9 PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_icontour PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_prev_level PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_rtf_bits PLS_INTEGER := 0;
   h_source_type PLS_INTEGER := 0;
   h_target_date_julian PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   counter PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_branch PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_c_duration PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_source_level PLS_INTEGER := 0;
   h_src_child_count PLS_INTEGER := 0;
   h_src_max_rank PLS_INTEGER := 0;
   h_target_level PLS_INTEGER := 0;
   h_rate_echange NUMBER := 0.00;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_fs_flag PLS_INTEGER := 0;
   h_source_project_id VARCHAR2(32);
   h_same_prj PLS_INTEGER := 0;
   h_constraint_type PLS_INTEGER := 0;
   h_rollup_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_group_type PLS_INTEGER := 0;
   h_table PLS_INTEGER := 0;
   h_stage_id PLS_INTEGER := 0;
   h_tmp_flag PLS_INTEGER := 0;
   l_copy_participants PLS_INTEGER := 0;
   h_start_finish_date TIMESTAMP;
   h_finish_start_date TIMESTAMP;
   h_target_prj_start_finish_date TIMESTAMP;
   h_target_asset_flag PLS_INTEGER := 0;
   h_financial_category PLS_INTEGER := 0;
   h_source_transfer_flag VARCHAR2(1);
-- Declare Cursors
      CURSOR CCWBS01 IS
         SELECT CHILD_COUNT
         FROM TMT_WBS
         WHERE RANK LIKE h_source_rank || '%' AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         AND RANK > h_source_rank;

      CURSOR CCAL2 IS
         SELECT * FROM (SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED)) FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_project_id AND DAY_YEAR >= TO_DATE(SubStr(h_target_date, 1, 10)) ORDER BY DAY_YEAR ) WHERE ROWNUM<=1;

      CURSOR C5 IS
         SELECT RANK, WBS_CODE
         FROM TMT_WBS
         WHERE RANK LIKE h_parent_rank || '%' AND RANK >= h_target_rank
         FOR UPDATE OF RANK, WBS_CODE;

      CURSOR C7 IS
         SELECT ELEMENT_ID, WBS_CODE, RANK, CREATED_BY,
         CRI_COUNT, DOC_COUNT, SCORECARD_COUNT,
         CASE WHEN CONSTRAINT_TYPE = PMO_CNST.SC_ALAP THEN PMO_CNST.SC_ALAP ELSE PMO_CNST.SC_ASAP END,
         PROPOSED_DURATION, TYPE_ID, TRANSFER_FLAG
         FROM TMT_WBS
         WHERE REC_STATUS <> 'D' AND REC_STATUS <> 'A'
         AND RANK LIKE h_source_rank || '%'
         ORDER BY RANK;

      CURSOR CPR1 IS
         SELECT PARENT_ID FROM PMO_COPY_WBS
         WHERE  ROWNUM <= 1 AND  LEVEL_ID = h_target_level
         AND RANK LIKE SUBSTR(h_cur_rank, 1, LENGTH(h_cur_rank) - 4) || '%';

      CURSOR CPEXS IS
         SELECT TSK_ASSIGNMENT_ID FROM
         TASK_ASSIGNMENTS WHERE TASK_ID = h_cur_id;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_C_WBS_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_C_WBS_MSP'), '-- ' || SYSTIMESTAMP, ':r := FC_C_WBS_MSP(' || 
       '''' || h_target_id || ''''
          || ', ' || '''' || h_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_ieflag), 'NULL')
          || ', ' || '''' || h_error_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 213+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_C_WBS_MSP(214) : move_wbs.sql(9400) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Parse parameters to local variables
   h_error := '200073';
   
   h_target_asset_flag := 0;
   h_financial_category := PMO_CNST.SC_NONE_FINANCIAL_CATEGORY;
   If ((h_source_id IS NULL)) OR ((h_rec_user IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 225+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(226) : move_wbs.sql(9412) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.ELEMENT_ID_REQUIRED;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'FC_C_WBS_MSP', 230+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: FC_C_WBS_MSP(231) : move_wbs.sql(9417) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WBS' || '''' || ' ID: ' || '''' || h_source_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_WBS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_source_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_source_id, False); 
      END;
   End If;
   
   --trace_printf("\n h_source_id:%s", h_source_id);
   --trace_printf("\n h_rec_status:%s\n", h_rec_status);
   
   If SubStr(h_rec_status, 1, 1) = 'D' Then
      h_error_id := h_source_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 248+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(249) : move_wbs.sql(9435) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.MARKED_AS_UNAVAILABLE;
   End If;
   
   --Safwan: an override participant is + 100. removing the 100.
   If ((Coalesce(h_bitflag, 0) = 100) OR (Coalesce(h_bitflag, 0) = 101) OR (Coalesce(
              h_bitflag, 0) = 102)) Then
      h_bitflag := Coalesce(h_bitflag, 0) - 100;
      l_copy_participants := 1;
   Else
      l_copy_participants := 0;
   End If;
   
   If NOT ((Coalesce(h_bitflag, 0) = 0) OR (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(
                  h_bitflag, 0) = 2)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 265+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(266) : move_wbs.sql(9452) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.BITFLAG_OUT_OF_RANGE;
   End If;
   
   If (h_target_id IS NOT NULL) AND (Coalesce(h_bitflag, 0) = 2) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'FC_C_WBS_MSP', 272+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: FC_C_WBS_MSP(273) : move_wbs.sql(9459) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WBS' || '''' || ' ID: ' || '''' || h_target_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_WBS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_target_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_target_id, False); 
      END;
   End If;
   --trace_printf("h_target_id: %s, h_source_id: %s\n", h_target_id,h_source_id); //trace_flush();
   
   -- If target and source are the same, return error... (potential infinit loop)
   If (h_target_id IS NOT NULL) AND (PMO_ORASTD.strcmp(h_target_id
         , h_source_id) = PMO_CNST.SC_MATCH) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'FC_C_WBS_MSP', 289+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: FC_C_WBS_MSP(290) : move_wbs.sql(9476) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SAME_TARGET_SOURCE), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 292+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(293) : move_wbs.sql(9479) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.SAME_TARGET_SOURCE, False);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 296+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(297) : move_wbs.sql(9483) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMO_CNST.SAME_TARGET_SOURCE;
   End If;
   
   --timeLog(file, "PPPPPPPPPPPP001");
   BEGIN
      SELECT RANK, LENGTH(RANK), PROJECT_ID, TYPE_ID,
      START_DT, ELEMENT_NAME, CALENDAR_ID, CURRENCY_ID,
      COALESCE(MINUTE_PER_DAY, 480), DURATION, MAX_RANK, CHILD_COUNT,
      STAGE_ID, TOOLTIP_DESCR, CRI_COUNT, DOC_COUNT,
      SCORECARD_COUNT, PUBLISHED, PRIORITY, ASSIGN_COUNT,
      START_FINISH_DATE, FINISH_START_DATE,
      RTF_BITS, REC_STATUS,
      TIMECODE_1_ID, TIMECODE_2_ID
      INTO h_source_rank, h_len, h_source_project_id, h_source_type,
      h_source_date, h_project_name, h_calendar_id, h_currency_id,
      h_min_per_day, h_duration, h_src_max_rank, h_src_child_count,
      h_n17, h_255, h_cri_count, h_doc_count,
      h_scorecard_count, h_published, h_priority, h_assign_count,
      h_start_finish_date, h_finish_start_date,
      h_rtf_bits, h_rec_status,
      h_source_timecode_1_id, h_source_timecode_2_id
      FROM TMT_WBS WHERE ELEMENT_ID = h_source_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 326+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(327) : move_wbs.sql(9513) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- April 16, 2004 : this line is commented because we can create a project from a default/templated project even if the default/templated project is locked
   /*if((checkFlagStatus(h_rec_status, 'L')) || (checkFlagStatus(h_rec_status, 'D')) )
       {
           InitializeHost(SC_PARAM(h_error_id),SC_PARAM(h_source_id));
           return PARENT_MARKED_AS_LOCKED;
       }*/
   
   BEGIN
      SELECT NICK_NAME || ';' INTO h_creator_name FROM RESOURCES WHERE RESOURCE_ID = h_rec_user;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 346+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(347) : move_wbs.sql(9533) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("h_source_type: %d\n", h_source_type); //trace_flush();
   If isNotProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      BEGIN
         SELECT ELEMENT_NAME, CALENDAR_ID, COALESCE(MINUTE_PER_DAY, 480)
         INTO h_project_name, h_calendar_id, h_min_per_day
         FROM TMT_WBS WHERE ELEMENT_ID = h_source_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 363+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(364) : move_wbs.sql(9550) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- if source has a sub-project, return an error
      
      OPEN CCWBS01;
      
      FETCH CCWBS01 INTO h_count;
      IF CCWBS01%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("SC_STATE: %d, h_source_rank: %s\n", SC_STATE,h_source_rank); //trace_flush();
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         BEGIN
            CLOSE CCWBS01 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 389+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(390) : move_wbs.sql(9576) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_error_id := h_source_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 396+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(397) : move_wbs.sql(9583) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.MARKED_AS_UNAVAILABLE;
      Else
         BEGIN
            CLOSE CCWBS01 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 408+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(409) : move_wbs.sql(9595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_bitflag, 0) <> 2 Then
         If (h_target_id IS NOT NULL) Then
            BEGIN
               SELECT ELEMENT_ID
               INTO h_parent_id
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_target_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 427+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(428) : move_wbs.sql(9614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_parent_id
                  FROM TMT_PORTFOLIOS
                  WHERE ELEMENT_ID = h_target_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 444+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(445) : move_wbs.sql(9631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 450+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(451) : move_wbs.sql(9637) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN PMO_CNST.RECORD_NOT_FOUND;
               End If;
               h_parent_id := NULL;
               h_bitflag := 2;
            End If;
         Else
            h_parent_id := NULL;
         End If;
         
         h_temp_bitflag := 0;
         PMO_DB2M.SC_STATE := FC_I_WBS_MSP(h_project_name, h_parent_id, h_source_type, h_rec_user
                                  , h_temp_bitflag, h_uniqueID);
         
         PMO_DB2M.SC_STATE := FC_FISCAL_SF_DATE(h_start_finish_date, h_finish_start_date);
         
         h_temp_bitflag := 0;
         PMO_DB2M.SC_STATE := PROJECT_CALENDAR(h_uniqueID, h_calendar_id, h_start_finish_date
                                      , h_finish_start_date, h_temp_bitflag);
         PMO_DB2M.SC_STATE := FC_TIMEPHASE_CALENDAR(h_uniqueID, h_start_finish_date, h_finish_start_date);
         
         BEGIN
            UPDATE TMT_WBS
            SET (START_FINISH_DATE, FINISH_START_DATE, REF_CODE, PRIORITY, CALC_PRIORITY,
            PROBABILITY, EFFORT_DRIVEN, STAGE_ID, START_FINISH_FLAG, CALENDAR_ID,
            MINUTE_PER_DAY, MINUTE_PER_WEEK, CURRENCY_ID, TOOLTIP_DESCR, BUFFER_PERCENT,
            PLANNED_START, PLANNED_DURATION) =
            (SELECT START_FINISH_DATE, FINISH_START_DATE, REF_CODE, PRIORITY, CALC_PRIORITY,
            PROBABILITY, EFFORT_DRIVEN, STAGE_ID, START_FINISH_FLAG, CALENDAR_ID,
            MINUTE_PER_DAY, MINUTE_PER_WEEK, CURRENCY_ID, TOOLTIP_DESCR, BUFFER_PERCENT,
            PLANNED_START, PLANNED_DURATION
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_source_id)
            WHERE ELEMENT_ID = h_uniqueID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 495+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(496) : move_wbs.sql(9682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_bitflag, 0) = 2 Then
            PMO_DB2M.SC_STATE := FC_M_PORTF(h_target_id, h_uniqueID, h_bitflag, h_rec_user);
         End If;
         
         h_target_id := h_uniqueID;
         h_bitflag := 2;
      End If;
   End If;
   --timeLog(file, "PPPPPPPPPPPP002");
   
   BEGIN
      SELECT SYSTIMESTAMP INTO h_cur_date FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 518+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(519) : move_wbs.sql(9705) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_target_id: %s, h_source_id: %s\n", h_target_id,h_source_id); //trace_flush();
   
   If (h_target_id IS NULL) Then
      h_buffer_percent := 0;
      h_target_date := h_cur_date;
      -- don't need, we create a new project and get a new h_target_date
      
      -- Get the max level from the root and incres the CHILD_COUNT
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_WBS', h_max_rank);
      
      h_target_rank := '0000';
      h_tmp_code := '0.' || TO_CHAR(Coalesce(h_max_rank, 0));
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      h_target_rank := SubStr(h_target_rank || h_rankBase64, 1, 200);
      
      h_target_transfer_flag := NULL;
      
      h_same_prj := 0;
      h_parent_id := NULL;
   Else
      --we need to read parent_START_DT, and h_min_per_Day
      BEGIN
         SELECT RANK, CASE WHEN TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) THEN ELEMENT_ID ELSE PROJECT_ID END,
         CASE WHEN TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_source_project_id THEN 1 WHEN TYPE_ID NOT IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND PROJECT_ID = h_source_project_id THEN 1 ELSE 0 END,
         LEVEL_ID, TYPE_ID, RTRIM(WBS_CODE), PARENT_ID, SUBSTR(RANK,1,LENGTH(RANK)-4),
         CALENDAR_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MILESTONE_PERCENT, 0), CURRENCY_ID,
         CASE WHEN TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND h_source_type IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND PUBLISHED = '2' THEN 1 ELSE 0 END,
         TRANSFER_FLAG, CASE WHEN LENGTH(FS_CODE) > 0 THEN 1 ELSE 0 END, REC_STATUS, TIMECODE_1_ID, TIMECODE_2_ID,
         ASSET_FLAG, FINANCIAL_CATEGORY
         INTO h_target_rank, h_project_id,
         h_same_prj,
         h_target_level, h_parent_type, h_tmp_code, h_parent_id, h_parent_rank,
         h_calendar_id, h_min_per_day, h_buffer_percent, h_currency_id,
         h_total,
         h_target_transfer_flag, h_fs_flag, h_rec_status, h_target_timecode_1_id, h_target_timecode_2_id,
         h_target_asset_flag, h_financial_category
         FROM TMT_WBS WHERE ELEMENT_ID = h_target_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 567+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(568) : move_wbs.sql(9754) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT START_FINISH_DATE 
         INTO h_target_prj_start_finish_date
         FROM TMT_WBS 
         WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 583+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(584) : move_wbs.sql(9770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (SubStr(h_rec_status, 1, 1) = 'L') OR (SubStr(h_rec_status, 1, 1) = 'D') Then
         h_error_id := h_target_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 591+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(592) : move_wbs.sql(9778) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PARENT_MARKED_AS_LOCKED;
      End If;
      
      BEGIN
         SELECT START_FINISH_DATE, START_FINISH_DATE
         INTO h_target_date, h_dayofyear
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 606+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(607) : move_wbs.sql(9793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("h_project_id: %s, h_parent_type: %d\n", h_project_id, h_parent_type); //trace_flush();
      
      If (isProjectType(Coalesce(h_source_type, 0))) <> 0 AND (isNotProjectType(Coalesce(
            h_parent_type, 0))) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 615+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(616) : move_wbs.sql(9802) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.PROJECT_TARGET_SOURCE;
      End If;
      --trace_printf("h_total: %d, h_ieflag: %d\n", h_total,h_ieflag); //trace_flush();
      
      --h_ieflag = 100 is when we copy from default project
      If (Coalesce(h_total, 0) = 1) AND (Coalesce(h_ieflag, 0) <> 100) Then
         h_error_id := h_parent_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 625+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(626) : move_wbs.sql(9812) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_COPY_SUBPROJECTS;
      End If;
      
      -- Is not allowed to copy a parent under his child
      
      BEGIN
         SELECT LEVEL_ID, PMO_ORASTD.LOCATE(RANK, h_target_rank) INTO h_source_level, h_branch
         FROM TMT_WBS WHERE ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 641+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(642) : move_wbs.sql(9828) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If (Coalesce(h_branch, 0) = 1) AND (Coalesce(h_target_level, 0) > Coalesce(h_source_level, 0)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 647+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(648) : move_wbs.sql(9834) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.CANNOT_COPY_PARENT_UNDER_CHILD;
      End If;
      
      If isNotProjectType(Coalesce(h_parent_type, 0)) <> 0 Then
         BEGIN
            SELECT CALENDAR_ID, COALESCE(MINUTE_PER_DAY, 480), START_FINISH_DATE, CASE WHEN LENGTH(FS_CODE) > 0 THEN 1 ELSE 0 END
            INTO h_calendar_id, h_min_per_day, h_dayofyear, h_fs_flag
            FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 663+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(664) : move_wbs.sql(9850) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_bitflag, 0) = 0 Then
         -- Get next sibling count and incres the CHILD_COUNT
         PMODEFINES.PMO_GET_MAX_RANK('TMT_WBS', h_target_id, h_max_rank);
         h_parent_id := h_target_id;
         h_tmp_code := h_tmp_code || '.' || TO_CHAR(Coalesce(h_max_rank, 0));
         
         --Get Base 64
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         h_target_rank := SubStr(h_target_rank || h_rankBase64, 1, 200);
      End If;
   End If;
   
   COMMIT;
   
   --timeLog(file, "PPPPPPPPPPPP003");
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_COPY_DEFAULT_ATTRIBUTE';
   PMO_DB2M.SC_STATE := 0;
   
   --trace_printf("h_total: %d, h_bitflag: %d\n", h_total,h_bitflag); //trace_flush();
   
   -- Insert a new project
   If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      If Coalesce(h_bitflag, 0) = 2 Then
         BEGIN
            UPDATE TMT_WBS
            SET START_FINISH_FLAG = '0',
            WBS_CODE = h_tmp_code,
            MAX_RANK = h_src_max_rank,
            CHILD_COUNT = h_src_child_count,
            TOOLTIP_DESCR = h_255,
            CRI_COUNT = h_cri_count,
            DOC_COUNT = h_doc_count,
            SCORECARD_COUNT = h_scorecard_count,
            REC_STATUS = 'U',
            REC_USER = h_rec_user,
            REC_DATETIME = h_cur_date,
            ASSIGN_NAMES = h_creator_name,
            ASSIGN_COUNT = h_assign_count,
            TIMECODE_1_ID = h_source_timecode_1_id,
            TIMECODE_2_ID = h_source_timecode_2_id,
            RTF_BITS = h_rtf_bits
            WHERE ELEMENT_ID = h_target_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 722+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(723) : move_wbs.sql(9909) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_tmp_code: %s, h_src_max_rank: %d\n", SC_STATE,h_tmp_code,h_src_max_rank); //trace_flush();
         
         -- copy wbs_rtf
         BEGIN
            INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
            SELECT h_target_id, h_target_id, BIT_FLAG, CLOB_DATA
            FROM WBS_RTF
            WHERE ELEMENT_ID = h_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 744+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(745) : move_wbs.sql(9931) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --WHY Target/Project id???
         -- copy doc_rtf
         --          BEGIN_SQL
         --              INSERT INTO DOC_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
         --              SELECT HOST(h_target_id), HOST(h_target_id), BIT_FLAG, CLOB_DATA
         --              FROM DOC_RTF
         --              WHERE ELEMENT_ID = HOST(h_source_id) ;
         --          END_SQL;
         --WHY Target/Project id???
         
         BEGIN
            UPDATE HISTORY_WBS
            SET ELEMENT_NAME = h_project_name,
            START_FINISH_FLAG = '0',
            START_FINISH_DATE = TO_DATE(SubStr(h_target_date, 1, 10)),
            WBS_CODE = h_tmp_code,
            MAX_RANK = h_src_max_rank,
            CHILD_COUNT = h_src_child_count,
            TOOLTIP_DESCR = h_255,
            CRI_COUNT = h_cri_count,
            DOC_COUNT = h_doc_count,
            SCORECARD_COUNT = h_scorecard_count,
            REC_STATUS = 'U',
            REC_USER = h_rec_user,
            REC_DATETIME = h_cur_date,
            ASSIGN_NAMES = h_creator_name,
            ASSIGN_COUNT = h_assign_count,
            TIMECODE_1_ID = h_source_timecode_1_id,
            TIMECODE_2_ID = h_source_timecode_2_id,
            RTF_BITS = h_rtf_bits
            WHERE ELEMENT_ID = h_target_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 790+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(791) : move_wbs.sql(9977) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PROJECT_DETAILS
            SET (PROJECT_RTF_BITS, PROJECT_RTF_BITS2, DELIVERABLE_RTF_BITS, WORK_PRODUCT_RTF_BITS,
            SUMMARY_TASK_RTF_BITS, MILESTONE_RTF_BITS, TASK_RTF_BITS,
            REQUIREMENT_RTF_BITS, CHANGE_REQUEST_RTF_BITS, SERVICE_REQUEST_RTF_BITS,
            DEFECT_RTF_BITS, ISSUE_RTF_BITS, RISK_RTF_BITS,
            CRI_DELIVERABLE_RTF_BITS, CRI_SUMMARY_TASK_RTF_BITS,
            CRI_MILESTONE_RTF_BITS, CRI_TASK_RTF_BITS,DOCUMENT_RTF_BITS,
            ROLLUP_PROJECT_ESTIMATED, ROLLUP_DELIVERABLE_ESTIMATED, TIMEPHASE_PROJECT_ASSIGNMENT) =
            (SELECT PROJECT_RTF_BITS, PROJECT_RTF_BITS2, DELIVERABLE_RTF_BITS, WORK_PRODUCT_RTF_BITS,
            SUMMARY_TASK_RTF_BITS, MILESTONE_RTF_BITS, TASK_RTF_BITS,
            REQUIREMENT_RTF_BITS, CHANGE_REQUEST_RTF_BITS, SERVICE_REQUEST_RTF_BITS,
            DEFECT_RTF_BITS, ISSUE_RTF_BITS, RISK_RTF_BITS,
            CRI_DELIVERABLE_RTF_BITS, CRI_SUMMARY_TASK_RTF_BITS,
            CRI_MILESTONE_RTF_BITS, CRI_TASK_RTF_BITS,DOCUMENT_RTF_BITS,
            ROLLUP_PROJECT_ESTIMATED, ROLLUP_DELIVERABLE_ESTIMATED, TIMEPHASE_PROJECT_ASSIGNMENT
            FROM PROJECT_DETAILS
            WHERE PROJECT_ID = h_source_id)
            WHERE PROJECT_ID = h_target_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 824+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(825) : move_wbs.sql(10011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_PROJECT_DETAILS: SC_STATE: %d\n", SC_STATE);
         
         h_project_id := h_target_id;
         h_new_rank := h_target_rank;
         
         h_dayofyear := SubStr(SubStr(h_target_date, 1, 10), 1, 10);
         h_dayofyear := TO_DATE(SubStr(SubStr(h_dayofyear, 1, 10),1, 10));
      Else
         --Get a unique id for new project
         h_uniqueID := SYS_GUID();
         
         BEGIN
            SELECT PARENT_ID INTO h_real_calendar_id
            FROM TMT_CALENDARS WHERE ELEMENT_ID = h_calendar_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 848+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(849) : move_wbs.sql(10035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_C_WBS_MSP', 854+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_C_WBS_MSP(855) : move_wbs.sql(10041) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_calendar_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 857+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(858) : move_wbs.sql(10044) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_calendar_id, False);
         End If;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_target_date, 1, 10)) - 180       INTO h_SD FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 870+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(871) : move_wbs.sql(10057) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_FD := '';
         h_temp_var := 0;
         
         h_temp_sc_state := PROJECT_CALENDAR(h_uniqueID, h_real_calendar_id, h_SD, h_FD, h_temp_var);
         
         If (h_target_id IS NULL) Then
            h_project_id := h_uniqueID;
         End If;
         
         OPEN CCAL2;
         
         FETCH CCAL2 INTO h_target_date;
         IF CCAL2%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         BEGIN
            CLOSE CCAL2 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 901+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(902) : move_wbs.sql(10088) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_target_level := Trunc(Coalesce(Length(h_target_rank), 0) / 4) - 2;
         h_parent_id := h_project_id;
         
         h_stage_id := 740;
         
         BEGIN
            INSERT INTO TMT_WBS( PROJECT_ID, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, CALENDAR_ID, MINUTE_PER_DAY,
            CREATED_BY, START_FINISH_FLAG, START_FINISH_DATE, RANK, WBS_CODE, CURRENCY_ID, MAX_RANK, CHILD_COUNT,
            STAGE_ID, TOOLTIP_DESCR, CRI_COUNT, DOC_COUNT, SCORECARD_COUNT, REC_STATUS, REC_USER, PRIORITY, PROBABILITY,REC_DATETIME,
            ASSIGN_NAMES, ASSIGN_COUNT, TIMECODE_1_ID, TIMECODE_2_ID, RTF_BITS, ROLLUP_BITS)
            VALUES(h_uniqueID, h_parent_id, h_uniqueID, h_project_name, h_source_type, h_target_level, h_uniqueID, h_min_per_day,
            h_rec_user, '0', TO_DATE(SubStr(h_target_date, 1, 10)), h_target_rank, h_tmp_code, h_currency_id, h_src_max_rank, h_src_child_count,
            h_stage_id, h_255, h_cri_count, h_doc_count, h_scorecard_count, 'U', h_rec_user, h_priority, 100, h_cur_date,
            h_creator_name, h_assign_count, h_source_timecode_1_id, h_source_timecode_2_id, h_rtf_bits, (PMO_CNST.SC_PLANNED_BIT + PMO_CNST.SC_PROPOSED_BIT));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 930+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(931) : move_wbs.sql(10117) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- copy wbs_rtf
         BEGIN
            INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
            SELECT h_uniqueID, h_uniqueID, BIT_FLAG, CLOB_DATA
            FROM WBS_RTF
            WHERE ELEMENT_ID = h_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 951+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(952) : move_wbs.sql(10138) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_history_id := SYS_GUID();
         
         BEGIN
            INSERT INTO HISTORY_WBS
            SELECT h_history_id, A.* FROM TMT_WBS A WHERE A.ELEMENT_ID = h_uniqueID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 971+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(972) : move_wbs.sql(10158) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET REVISION_HISTORY = Null WHERE ELEMENT_ID = h_uniqueID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 988+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(989) : move_wbs.sql(10175) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_project_id := h_uniqueID;
         h_new_rank := h_target_rank;
         h_dayofyear := SubStr(h_target_date, 1, 10);
         --Get a unique id for resource assignments
         h_id := SYS_GUID();
         
         h_temp_participant := PMO_CNST.SC_MANAGER;
         h_temp_security_id := 'SCPROJECTMANAGER';
         h_temp_bitflag := 1;
         h_res_type := '0';
         
         h_temp_var := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_rec_user, h_temp_bitflag, h_temp_participant
                                     , h_temp_security_id, h_res_type, h_temp_bitflag);
         
         BEGIN
            INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, RESOURCE_TYPE, REC_USER,TABLE_TYPE)
            VALUES(h_id, h_project_id, h_project_id, h_source_type, h_rec_user, PMO_CNST.SC_MANAGER, 'SCPROJECTMANAGER', '0', h_rec_user,PMO_CNST.SC_TABLE_WBS);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1020+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1021) : move_wbs.sql(10207) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_temp_var := getAttributeType(Coalesce(h_source_type, 0));
      BEGIN
         INSERT INTO PMO_COPY_DEFAULT_ATTRIBUTE
         (OLD_ELEMENT_ID,NEW_ELEMENT_ID,TYPE_ID)
         VALUES(h_source_id,h_project_id,h_temp_var);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1041+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1042) : move_wbs.sql(10228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --copy attribute from source to target and assign them to new project
      h_temp_var := FC_C_LOCAL_ATTRIBUTE(h_source_id, h_project_id, h_rec_user);
      
      /***************************************************************************/
      -- Copy Project Details, Stage Notifications, alerts settings
      h_temp_var := FC_C_DETAILS(h_source_id, h_project_id, h_rec_user);
   End If;
   
   COMMIT;
   
   --timeLog(file, "PPPPPPPPPPPP004");
   If (PMO_ORASTD.strncmp(h_target_date, h_source_date, 10) < 0) AND (NOT isProjectType(Coalesce(
         h_source_type, 0)) <> 0) Then
      h_target_date := h_source_date;
      
      h_dayofyear := SubStr(SubStr(h_source_date, 1, 10), 1, 10);
      
      h_dayofyear := SubStr(h_dayofyear, 1, 10);
   Else
      h_dayofyear := SubStr(SubStr(h_target_date, 1, 10), 1, 10);
      h_dayofyear := SubStr(h_dayofyear, 1, 10);
   End If;
   
   h_temp_var := FC_LoadProjectCalendar(h_project_id, h_dayofyear);
   
   --This call is to fix the start date..
   h_temp_bitflag := 0;
   h_temp_var := FC_GET_DT(h_target_date, h_temp_bitflag);
   
   --This call is to set the finish date..
   h_target_finish_date := h_target_date;
   
   h_temp_bitflag := 1;
   
   h_temp_var := FC_GET_DT(h_target_finish_date, h_temp_bitflag);
   
   BEGIN
      SELECT PMO_ORASTD.JULIAN_DAY(h_target_date) INTO h_target_date_julian FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1090+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1091) : move_wbs.sql(10277) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --h_bitflag = 0, the target is the parent and copy at the end, as last child
   --h_bitflag = 1, insert at the target position and the parent is the target's parent
   If Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         SELECT '/' || SUBSTR(TO_CHAR(SYSTIMESTAMP), 21, 3) into h_xrank64 FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1106+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1107) : move_wbs.sql(10293) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Code to remove...
      BEGIN
         SELECT ELEMENT_ID, RANK, TRANSFER_FLAG, ASSET_FLAG, FINANCIAL_CATEGORY
         INTO h_parent_id, h_parent_rank, h_target_transfer_flag,
         h_target_asset_flag, h_financial_category
         FROM TMT_WBS
         WHERE ELEMENT_ID = (SELECT PARENT_ID FROM TMT_WBS WHERE ELEMENT_ID = h_target_id);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1124+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1125) : move_wbs.sql(10311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      -- Code to remove...
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_parent_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_parent_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_parent_rank, False); 
      	End If; 
      END;
      
      OPEN C5;
      
      LOOP
         FETCH C5 INTO h_cur_rank, h_c255_1;
         IF C5%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> 0 Then
            EXIT;
         End If;
         
         --Incres RANK
         PMO_STPU.incrBase64(h_cur_rank, Coalesce(h_target_level, 0) + 1);
         
         --Incres WBS_CODE
         PMO_STPU.Base64toInt(h_cur_rank, h_c255_1);
         
         BEGIN
            UPDATE TMT_WBS
            SET RANK = h_cur_rank || h_xrank64, WBS_CODE =  h_c255_1
            WHERE CURRENT OF C5;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1177+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1178) : move_wbs.sql(10364) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE C5 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1191+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1192) : move_wbs.sql(10378) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_parent_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_parent_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_parent_rank, False); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_WBS
         SET RANK = SUBSTR(RANK, 1, LENGTH(RANK) - 4)
         WHERE RANK LIKE h_parent_rank || '%' AND SUBSTR(RANK, LENGTH(RANK) - 3, 4) = h_xrank64;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1223+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1224) : move_wbs.sql(10410) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMODEFINES.PMO_GET_MAX_RANK('TMT_WBS', h_parent_id, h_max_rank);
      
      COMMIT;
   End If;
   
   --int result;
   h_prev_level := -1;
   h_ret_id := '';
   h_constraint_type := PMO_CNST.SC_ASAP;
   -- ASAP
   
   h_temp_sc_state := FC_C_TMP_TABLES();
   
   -- rank of the source element may ba changed sibling copy
   If Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         SELECT RANK INTO h_source_rank
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1253+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1254) : move_wbs.sql(10440) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --timeLog(file, "PPPPPPPPPPPP005");
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_source_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_source_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, False); 
   	End If; 
   END;
   
   OPEN C7;
   
   LOOP
      FETCH C7 INTO h_cur_id, h_wbs_code, h_old_rank, h_created_by,
      h_cri_count, h_doc_count, h_scorecard_count,
      h_constraint_type,
      h_duration, h_type, h_source_transfer_flag;
      IF C7%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      h_assign_string := '';
      If isWBSTransferedDeliv(Coalesce(h_type, 0)) <> 0 Then
         h_type := PMO_CNST.SC_DELIVERABLE;
      ElsIf isWBSTransferedTask(Coalesce(h_type, 0)) <> 0 Then
         h_type := PMO_CNST.SC_TASK;
      End If;
      --trace_printf("h_source_type: %d, h_prev_level: %d, h_bitflag: %d, h_assign_string: %s\n", h_source_type,h_prev_level,h_bitflag,h_assign_string); //trace_flush();
      --trace_printf("h_old_rank: %s, h_cur_id: %s\n", h_old_rank,h_cur_id); //trace_flush();
      
      If (isProjectType(Coalesce(h_source_type, 0))) <> 0 AND (Coalesce(h_prev_level, 0) = -1) 
            AND (Coalesce(h_bitflag, 0) <> 1) Then
         h_cur_rank := h_target_rank;
         h_parent_id := h_project_id;
         h_lenc := Coalesce(Length(h_wbs_code), 0);
         h_wbs_code := SubStr(h_tmp_code, 1, 255);
         h_id := h_project_id;
         
         -- Safwan: Added comment for clarity
         -- Copy Assignments: h_temp_var = 0 don't copy participants (old model)
         --                                1 copy participants (new model)
         h_temp_var := Coalesce(l_copy_participants, 0);
         h_temp_sc_state := FC_RESPONSIBLE(h_project_id, h_id, h_source_project_id, h_cur_id, h_type
                                  , h_creator_name, h_created_by, h_rec_user, h_temp_var
                                  , h_assign_string);
         
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES = h_assign_string
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1328+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1329) : move_wbs.sql(10515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If Coalesce(Length(h_old_rank), 0) = Coalesce(h_len, 0) Then
            h_cur_rank := h_target_rank;
            h_lenc := Coalesce(Length(h_wbs_code), 0);
            h_wbs_code := SubStr(h_tmp_code, 1, 255);
         Else
            BEGIN
               SELECT h_target_rank || SUBSTR(h_old_rank, h_len +1,LENGTH(h_old_rank)-h_len),
               h_tmp_code || SUBSTR(h_wbs_code, h_lenc +1, LENGTH(h_wbs_code)-h_lenc)
               INTO h_cur_rank, h_wbs_code
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1350) : move_wbs.sql(10536) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (isProjectType(Coalesce(h_source_type, 0))) <> 0 AND (Coalesce(
                  h_lenc, 0) = Coalesce(Length(h_wbs_code), 0)) Then
               h_lenc := Coalesce(Length(h_wbs_code), 0);
               h_wbs_code := SubStr(h_tmp_code, 1, 255);
            End If;
         End If;
         
         h_target_level := Trunc(Coalesce(Length(h_cur_rank), 0) / 4) - 2;
         
         If Coalesce(h_prev_level, 0) = -1 Then
            h_new_rank := h_cur_rank;
         ElsIf Coalesce(h_target_level, 0) > Coalesce(h_prev_level, 0) Then
            h_parent_id := h_id;
         ElsIf Coalesce(h_target_level, 0) < Coalesce(h_prev_level, 0) Then
            OPEN CPR1;
            
            FETCH CPR1 INTO h_parent_id;
            IF CPR1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            BEGIN
               CLOSE CPR1 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1385+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1386) : move_wbs.sql(10572) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --Get a unique id for WBS
         h_id := SYS_GUID();
         
         If Coalesce(h_prev_level, 0) = -1 Then
            If (h_target_id IS NULL) Then
               h_project_id := h_id;
               h_parent_id := NULL;
            Else
               h_parent_id := h_parent_id;
            End If;
            h_sc_new_id := h_id;
         End If;
         
         -- Safwan: Added comment for clarity
         -- Copy Assignments: h_temp_var = 0 don't copy participants (old model)
         --                                1 copy participants (new model)
         h_temp_var := Coalesce(l_copy_participants, 0);
         h_temp_sc_state := FC_RESPONSIBLE(h_project_id, h_id, h_source_project_id, h_cur_id, h_type
                                  , h_creator_name, h_created_by, h_rec_user, h_temp_var
                                  , h_assign_string);
         
         If isTask(Coalesce(h_type, 0)) <> 0 Then
            --For rollup
            If Coalesce(Length(h_ret_id), 0) = 0 Then
               h_ret_id := h_id;
            End If;
            
            --effort
            
            OPEN CPEXS;
            
            FETCH CPEXS INTO h_uniqueID;
            IF CPEXS%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               h_assign_count := 0;
            Else
               h_assign_count := 1;
            End If;
            
            BEGIN
               CLOSE CPEXS ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1442+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1443) : move_wbs.sql(10629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_start_date := h_target_date;
            
            h_temp_bitflag := 0;
            h_temp_var := FC_GET_DT(h_start_date, h_temp_bitflag);
            If Coalesce(h_assign_count, 0) > 0 Then
               h_duration := Coalesce(h_min_per_day, 0);
            End If;
            
            If PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1 Then
               If (h_target_transfer_flag IS NOT NULL) Then
                  If (PMO_ORASTD.strcmp(h_target_transfer_flag, 'R') = PMO_CNST.SC_MATCH) AND (Coalesce(
                        h_prev_level, 0) = -1) Then
                     h_target_transfer_flag := 'U';
                  ElsIf (PMO_ORASTD.strcmp(h_target_transfer_flag, 'C') = PMO_CNST.SC_MATCH) OR (PMO_ORASTD.strcmp(
                        h_target_transfer_flag, 'W') = PMO_CNST.SC_MATCH) Then
                     h_target_transfer_flag := 'W';
                  Else
                     h_target_transfer_flag := NULL;
                  End If;
               End If;
            Else
               h_target_transfer_flag := NULL;
            End If;
            
            BEGIN
               INSERT INTO PMO_COPY_WBS
               SELECT h_cur_id, h_old_rank, h_cur_rank, h_wbs_code, h_project_id, h_parent_id, h_id, h_type, h_target_level,
               MAX_RANK, CHILD_COUNT, ELEMENT_NAME, REF_CODE, PRIORITY, h_start_date, h_duration, CRI_COUNT, DOC_COUNT, SCORECARD_COUNT, REVISION_HISTORY, TOOLTIP_DESCR, CRITICAL, PROBABILITY,
               h_assign_string, RTF_BITS, h_target_transfer_flag, h_constraint_type, 0, EFFORT_DRIVEN, TIMECODE_1_ID, TIMECODE_2_ID
               FROM TMT_WBS WHERE ELEMENT_ID=h_cur_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1487+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1488) : move_wbs.sql(10674) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1) AND ((h_target_transfer_flag IS NOT NULL)) Then
               If PMO_ORASTD.strcmp(h_target_transfer_flag, 'W') = PMO_CNST.SC_MATCH Then
                  h_tmp_flag := 1;
                  h_temp_var := FC_INSERT_CRI_AFTER_WBS(h_parent_id, h_id, h_rec_user, h_tmp_flag);
               End If;
            End If;
            
            -- copy wbs_rtf
            If ((h_source_transfer_flag IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_source_transfer_flag
                  , 'C') = PMO_CNST.SC_MATCH) Then
               BEGIN
                  INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
                  SELECT h_project_id, h_id, BIT_FLAG, CLOB_DATA
                  FROM CRI_RTF
                  WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1517+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1518) : move_wbs.sql(10704) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
                  SELECT h_project_id, h_id, BIT_FLAG, CLOB_DATA
                  FROM WBS_RTF
                  WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1537+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1538) : move_wbs.sql(10724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_history_id := SYS_GUID();
            
            h_security_id := 'SCTASKRESPONSIBLE';
            PMO_DB2M.SC_STATE := FC_A_ALL_ATTRIBUTE(h_project_id, h_source_project_id, h_id, h_cur_id, h_type
                                        , h_rec_user);
            
            --Attributes
            --timecodes
            If ((h_fs_flag IS NOT NULL)) AND (Coalesce(h_fs_flag, 0) = 1) Then
               If ((h_same_prj IS NOT NULL)) AND (Coalesce(h_same_prj, 0) = 1) Then
                  PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_id, h_cur_id, h_type);
               Else
                  PMO_DB2M.SC_STATE := FC_A_WBS_TC(h_id, h_cur_rank, h_type);
                  PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_id, h_cur_id, h_type);
               End If;
            End If;
         Else
            If PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1 Then
               If (h_target_transfer_flag IS NOT NULL) Then
                  If (PMO_ORASTD.strcmp(h_target_transfer_flag, 'C') = PMO_CNST.SC_MATCH) OR (PMO_ORASTD.strcmp(
                        h_target_transfer_flag, 'W') = PMO_CNST.SC_MATCH) Then
                     h_target_transfer_flag := 'W';
                  Else
                     h_target_transfer_flag := NULL;
                  End If;
               End If;
            Else
               h_target_transfer_flag := NULL;
            End If;
            
            BEGIN
               INSERT INTO PMO_COPY_WBS
               SELECT h_cur_id, h_old_rank, h_cur_rank, h_wbs_code, h_project_id, h_parent_id, h_id, h_type, h_target_level,
               MAX_RANK, CHILD_COUNT, ELEMENT_NAME, REF_CODE, PRIORITY, h_target_date, PLANNED_DURATION, CRI_COUNT, DOC_COUNT, SCORECARD_COUNT, REVISION_HISTORY, TOOLTIP_DESCR, CRITICAL, PROBABILITY,
               h_assign_string, RTF_BITS, h_target_transfer_flag, h_constraint_type, METHOD, EFFORT_DRIVEN, TIMECODE_1_ID, TIMECODE_2_ID
               FROM TMT_WBS WHERE ELEMENT_ID=h_cur_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1589+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1590) : move_wbs.sql(10776) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_CNST.SC_MAINTAIN_TRANSFERRED_CRI_TR = 1) AND ((h_target_transfer_flag IS NOT NULL)) Then
               If PMO_ORASTD.strcmp(h_target_transfer_flag, 'W') = PMO_CNST.SC_MATCH Then
                  h_tmp_flag := 1;
                  h_temp_var := FC_INSERT_CRI_AFTER_WBS(h_parent_id, h_id, h_rec_user, h_tmp_flag);
               End If;
            End If;
            
            -- copy wbs_rtf
            If ((h_source_transfer_flag IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_source_transfer_flag
                  , 'C') = PMO_CNST.SC_MATCH) Then
               BEGIN
                  INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
                  SELECT h_project_id, h_id, BIT_FLAG, CLOB_DATA
                  FROM CRI_RTF
                  WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1619+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1620) : move_wbs.sql(10806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  INSERT INTO WBS_RTF (PARENT_ID, ELEMENT_ID, BIT_FLAG, CLOB_DATA)
                  SELECT h_project_id, h_id, BIT_FLAG, CLOB_DATA
                  FROM WBS_RTF
                  WHERE ELEMENT_ID = h_cur_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1639+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1640) : move_wbs.sql(10826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            --Attributes
            h_security_id := 'SCDELIVERABLEMANAGER';
            PMO_DB2M.SC_STATE := FC_A_ALL_ATTRIBUTE(h_project_id, h_source_project_id, h_id, h_cur_id, h_type
                                        , h_rec_user);
            
            --timecodes
            If ((h_fs_flag IS NOT NULL)) AND (Coalesce(h_fs_flag, 0) = 1) AND ((h_same_prj IS NOT NULL)) 
                  AND (Coalesce(h_same_prj, 0) = 1) Then
               PMO_DB2M.SC_STATE := FC_C_WBS_TC(h_id, h_cur_id, h_type);
            End If;
         End If;
      End If;
      
      If Coalesce(h_scorecard_count, 0) > 0 Then
         h_temp_sc_state := FC_C_SCD(h_type, h_cur_id, h_old_rank, h_id, h_cur_rank, h_project_id
                            , h_rec_user);
      End If;
      
      h_prev_level := Coalesce(h_target_level, 0);
   END LOOP;
   
   BEGIN
      CLOSE C7 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1674+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1675) : move_wbs.sql(10861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog(file, "PPPPPPPPPPPP007");
   
   --IF h_group_type == SC_DELIVERABLE THEN STAGE_ID = 5470
   --IF h_group_type == SC_TASK THEN STAGE_ID = 5500
   h_constraint_type := -1;
   -- ASAP
   
   If Coalesce(h_target_asset_flag, 0) = PMO_CNST.SC_HAS_ASSET Then
      h_target_asset_flag := PMO_CNST.SC_PARENT_HAS_ASSET;
   End If;
   
   -- copy target project level start_finish_date as start_finish_date column of children
   BEGIN
      INSERT INTO TMT_WBS (RANK, WBS_CODE, PROJECT_ID, PARENT_ID, ELEMENT_ID, TYPE_ID, 
      LEVEL_ID, MAX_RANK, CHILD_COUNT, ELEMENT_NAME, REF_CODE, 
      PRIORITY, PLANNED_START, PLANNED_DURATION, CRI_COUNT, DOC_COUNT, 
      REVISION_HISTORY, TOOLTIP_DESCR, CRITICAL, PROBABILITY, ASSIGN_NAMES, 
      RTF_BITS, TRANSFER_FLAG, CONSTRAINT_TYPE, METHOD, MINUTE_PER_DAY, 
      CREATED_BY, REC_USER, REC_STATUS, REC_DATETIME, ROLLUP_BITS, 
      STAGE_ID, EFFORT_DRIVEN, TIMECODE_1_ID, TIMECODE_2_ID, SCORECARD_COUNT,
      ASSET_FLAG, FINANCIAL_CATEGORY, START_FINISH_DATE)
      SELECT RANK, WBS_CODE, PROJECT_ID, PARENT_ID, ELEMENT_ID, TYPE_ID, 
      LEVEL_ID, MAX_RANK, CHILD_COUNT, ELEMENT_NAME, REF_CODE, 
      PRIORITY, PLANNED_START, PLANNED_DURATION, CRI_COUNT, DOC_COUNT, 
      REVISION_HISTORY, TOOLTIP_DESCR, CRITICAL, PROBABILITY, ASSIGN_NAMES, 
      RTF_BITS, TRANSFER_FLAG, CONSTRAINT_TYPE, METHOD, h_min_per_day, 
      h_rec_user, h_rec_user, 'U', h_cur_date, (PMO_CNST.SC_PLANNED_BIT + PMO_CNST.SC_PROPOSED_BIT),
      CASE WHEN TYPE_ID IN (PMO_CNST.SC_DELIVERABLE,PMO_CNST.SC_DELIVERABLE_TYPE1,PMO_CNST.SC_DELIVERABLE_TYPE2,PMO_CNST.SC_DELIVERABLE_TYPE3,PMO_CNST.SC_WORK_PRODUCT,PMO_CNST.SC_SUMMARY_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST,PMO_CNST.SC_WBS_RISK,PMO_CNST.SC_WBS_ISSUE,PMO_CNST.SC_WBS_REQUIREMENT,PMO_CNST.SC_WBS_DEFECT,PMO_CNST.SC_WBS_SERVICE_REQUEST,PMO_CNST.SC_WBS_ACTION_ITEM) THEN 5470  ELSE CASE WHEN TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) THEN 5500 ELSE  740 END END,
      EFFORT_DRIVEN, TIMECODE_1_ID,TIMECODE_2_ID, SCORECARD_COUNT, h_target_asset_flag, h_financial_category,
      h_target_prj_start_finish_date
      FROM PMO_COPY_WBS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1719+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1720) : move_wbs.sql(10906) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO HISTORY_WBS (HISTORY_ID, RANK, WBS_CODE, PROJECT_ID, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, MAX_RANK, CHILD_COUNT,
      ELEMENT_NAME, REF_CODE, PRIORITY,
      PLANNED_START, PLANNED_DURATION, CRI_COUNT, DOC_COUNT, REVISION_HISTORY, TOOLTIP_DESCR, CRITICAL, PROBABILITY,
      ASSIGN_NAMES, RTF_BITS, TRANSFER_FLAG, CONSTRAINT_TYPE, METHOD, MINUTE_PER_DAY, CREATED_BY, REC_USER,REC_STATUS, REC_DATETIME, START_FINISH_DATE)
      SELECT SYS_GUID(), RANK, WBS_CODE, PROJECT_ID, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, MAX_RANK, CHILD_COUNT,
      ELEMENT_NAME, REF_CODE, PRIORITY,
      PLANNED_START, PLANNED_DURATION, CRI_COUNT, DOC_COUNT, 'CREATED', TOOLTIP_DESCR, CRITICAL, PROBABILITY,
      ASSIGN_NAMES, RTF_BITS, TRANSFER_FLAG, CONSTRAINT_TYPE, METHOD, h_min_per_day, h_rec_user, h_rec_user, 'U', h_cur_date, h_target_prj_start_finish_date
      FROM PMO_COPY_WBS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1744+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1745) : move_wbs.sql(10931) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog(file, "PPPPPPPPPPPP008");
   
   /***************************************************************************/
   -- Copy Dependebcies
   h_temp_sc_state := FC_C_DEPENDENCIES(h_project_id, h_project_name, h_new_rank, h_cur_date
                                     , h_rec_user);
   
   COMMIT;
   
   --timeLog(file, "PPPPPPPPPPPP0081");
   If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      h_bitflag := 1;
      PMO_DB2M.SC_STATE := FC_CP_CHARGE_CODE(h_bitflag, h_source_id, h_target_id, h_rec_user);
      PMO_DB2M.SC_STATE := FC_CP_TIMES_CODE(h_bitflag, h_source_id, h_target_id, h_rec_user);
   End If;
   
   --  SC_STATE = FC_C_BUDGETS(SC_PARAM(h_project_id), SC_PARAM(h_source_project_id), SC_PARAM(h_source_rank), SC_PARAM(h_source_type),
   --              SC_PARAM(h_cur_date), SC_PARAM(h_rec_user));
   
   -- insert the project record into the temp table.
   If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      BEGIN
         INSERT INTO PMO_COPY_WBS (OLD_WBS_ID, OLD_RANK, ELEMENT_ID, RANK, PROJECT_ID)
         SELECT h_source_id, h_source_rank, ELEMENT_ID, RANK, PROJECT_ID
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_target_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1784+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1785) : move_wbs.sql(10971) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --timeLog(file, "PPPPPPPPPPPP009");
   h_table := PMO_CNST.SC_TABLE_WBS;
   h_temp_sc_state := FC_C_DOC(h_table, h_source_id, h_target_id, h_rec_user);
   --timeLog(file, "PPPPPPPPPPPP0091");
   h_temp_sc_state := FC_C_CRI(h_table, h_source_id, h_target_id, h_min_per_day, h_rec_user);
   --timeLog(file, "PPPPPPPPPPPP0092");
   PMO_DB2M.SC_STATE := FC_C_BUDGETS(h_project_id, h_source_project_id, h_source_rank, h_source_type
                                  , h_cur_date, h_rec_user);
   
   COMMIT;
   
   --timeLog(file, "PPPPPPPPPPPP010");
   /***************************************************************************/
   -- Safwan: This comment doesn't make sense here below.
   -- Copy Assignments: h_temp_var = 0 don't copy participants (old model)
   --                                1 copy participants (new model)
   
   h_temp_var := 0;
   h_temp_sc_state := FC_C_ASSIGNMENTS(h_source_project_id, h_source_rank, h_project_id, h_currency_id
                                    , h_target_date, h_source_date, h_cur_date, h_rec_user
                                    , h_source_type);
   
   COMMIT;
   
   --timeLog(file, "PPPPPPPPPPPP011");
   If (((h_source_timecode_1_id IS NULL)) AND ((h_source_timecode_2_id IS NULL))) 
         AND (((h_target_timecode_1_id IS NOT NULL)) OR ((h_target_timecode_2_id IS NOT NULL))) Then
      h_temp_var := FC_U_ALL_NULL_TIMECODES(h_target_rank, h_target_timecode_1_id, h_target_timecode_2_id);
   End If;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_target_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_target_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, False); 
   	End If; 
   END;
   BEGIN
      UPDATE TMT_WBS
      SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
      WHERE RANK LIKE h_target_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1847+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1848) : move_wbs.sql(11034) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --  h_temp_var = FC_Rollup(h_project_id);
   
   PMO_DB2M.SC_STATE := FC_C_SYS_EST_BUDGETS(h_project_id, h_source_project_id, h_source_rank
                                          , h_source_type, h_cur_date, h_rec_user);
   
   If isProjectType(Coalesce(h_source_type, 0)) <> 0 Then
      BEGIN
         SELECT TIMEPHASE_PROJECT_ASSIGNMENT, RUP_HOME_PATH INTO h_pa_timephase, h_rup_home_path 
         FROM PROJECT_DETAILS 
         WHERE PROJECT_ID = h_source_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1868+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1869) : move_wbs.sql(11055) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d,h_pa_timephase: %s, h_source_project_id: %s\n", SC_STATE,h_pa_timephase,h_source_project_id);
      
      If (h_rup_home_path IS NOT NULL) Then
         BEGIN
            UPDATE PROJECT_DETAILS SET 
            RUP_HOME_PATH = h_rup_home_path 
            WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_C_WBS_MSP', 1889+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_C_WBS_MSP(1890) : move_wbs.sql(11076) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
         h_temp_var := FC_C_TPH_PRJ_ASSIGNMENT(h_project_id, h_rec_user);
      End If;
      
      h_temp_var := FC_TIMEPHASE_ESTIMATED(h_project_id);
   End If;
   
   --timeLog(file, "PPPPPPPPPPPP012");
   h_temp_var := FC_Rollup(h_project_id);
   
   --timeLog(file, "PPPPPPPPPPPP013");
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   h_error_id := h_ret_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 1912+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(1913) : move_wbs.sql(11099) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_C_WBS_MSP', 1917+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_C_WBS_MSP(1918) : move_wbs.sql(11104) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_C_WBS_MSP;
ALTER FUNCTION FC_C_WBS_MSP COMPILE DEBUG;

/**********************************************************************************************************/

--  bitflag = 0: copy source as child of the target

--  bitflag = 1: use source to overwrite the target

--              (except some columns. for example: element_id, calendar_id, ...)

--  bitflag = 2: copy a project from template

/**********************************************************************************************************/

Prompt == FUNCTION SP_C_WBS_MSP ==
CREATE OR REPLACE FUNCTION SP_C_WBS_MSP
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
   , p_ieflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_C_WBS_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_C_WBS_MSP'), '-- ' || SYSTIMESTAMP, ':r := SP_C_WBS_MSP(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || ''''
          || ', ' || COALESCE(TO_CHAR(p_ieflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_C_WBS_MSP', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_C_WBS_MSP(29) : move_wbs.sql(11153) : ' || SYSTIMESTAMP);
   END IF; 

   h_target_id := p_target_id;
   h_source_id := p_source_id;
   h_rec_user := p_rec_user;
   
   h_bitflag := p_bitflag;
   h_ieflag := p_ieflag;
   
   h_error_id := h_target_id;
   PMO_DB2M.SC_STATE := FC_C_WBS_MSP(h_target_id, h_source_id, h_bitflag, h_rec_user, h_ieflag
                                  , h_error_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_C_WBS_MSP', 43+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_C_WBS_MSP(44) : move_wbs.sql(11168) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_error_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_C_WBS_MSP', 51+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_C_WBS_MSP(52) : move_wbs.sql(11176) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_C_WBS_MSP', 63+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_C_WBS_MSP(64) : move_wbs.sql(11188) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_C_WBS_MSP;
ALTER FUNCTION SP_C_WBS_MSP COMPILE DEBUG;

/************************************************************/

Prompt == FUNCTION SP_C_WBS ==
CREATE OR REPLACE FUNCTION SP_C_WBS
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_error_id VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_C_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_C_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_C_WBS(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_C_WBS', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_C_WBS(27) : move_wbs.sql(11231) : ' || SYSTIMESTAMP);
   END IF; 

   h_target_id := p_target_id;
   h_source_id := p_source_id;
   h_rec_user := p_rec_user;
   
   h_bitflag := p_bitflag;
   
   h_error_id := h_target_id;
   PMO_DB2M.SC_STATE := FC_C_WBS_MSP(h_target_id, h_source_id, h_bitflag, h_rec_user, h_ieflag
                                  , h_error_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_C_WBS', 40+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_C_WBS(41) : move_wbs.sql(11245) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_error_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_error_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_C_WBS', 47+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_C_WBS(48) : move_wbs.sql(11252) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_C_WBS', 59+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_C_WBS(60) : move_wbs.sql(11264) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_C_WBS;
ALTER FUNCTION SP_C_WBS COMPILE DEBUG;

