/***************************************************************************************************
bitflag:
    0: all levels
    1: from task level to project level
    2: profile at task level, resource at project level
***************************************************************************************************/

Prompt == FUNCTION replaceResource ==
CREATE OR REPLACE FUNCTION replaceResource
(
     h_resource_id IN OUT NOCOPY VARCHAR2
   , h_element_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_profile_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_xsql VARCHAR2(2048);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      CxRA PMO_DB2M.REF_CURSOR;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('replaceResource') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('replaceResource'), '-- ' || SYSTIMESTAMP, ':r := replaceResource(' || 
       '''' || h_resource_id || ''''
          || ', ' || '''' || h_element_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'replaceResource', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: replaceResource(28) : pools.sql(36) : ' || SYSTIMESTAMP);
   END IF; 

   --Local variable declaration
   
   If (h_element_id IS NOT NULL) Then
      --Go through all resource assignments for selected resource_id and element_id
      h_xsql := 
            'SELECT RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_TYPE FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_ID = ''';
      h_xsql := h_xsql || h_element_id;
      h_xsql := h_xsql || ''' AND RESOURCE_TYPE = ''0'' AND RESOURCE_ID = ''';
      h_xsql := h_xsql || h_resource_id;
      h_xsql := h_xsql || ''' ';
   Else
      --Go through all resource assignments for selected resource_id
      h_xsql := 'SELECT RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_TYPE FROM RESOURCE_ASSIGNMENTS ';
      h_xsql := h_xsql || 'WHERE RESOURCE_TYPE = ''0'' AND RESOURCE_ID = ''';
      h_xsql := h_xsql || h_resource_id;
      h_xsql := h_xsql || ''' ';
   End If;
   
   OPEN CxRA FOR h_xsql;
   
   LOOP
      FETCH CxRA INTO h_ra_id, h_project_id, h_type_id;
      IF CxRA%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      -- Find equivalent profile to replace resource
      BEGIN
         SELECT PROFILE_ID INTO h_profile_id FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_ra_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'replaceResource', 71+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: replaceResource(72) : pools.sql(80) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_profile_id IS NULL) Then
         BEGIN
            SELECT ROLE_ID INTO h_role_id FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_ra_id AND PRIMARY_ROLE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'replaceResource', 85+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: replaceResource(86) : pools.sql(94) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            --strcpy(h_role_id, "DEFAULTLEVEL");
            h_role_id := 'DEFAULTLEVEL';
         End If;
         
         --Get a unique ID for Profile
         h_profile_id := SYS_GUID();
         
         BEGIN
            INSERT INTO PROFILES(PROFILE_ID, PROJECT_ID, ROLE_ID, PROF_NUMBER)
            SELECT h_profile_id, h_project_id, h_role_id, COALESCE(MAX(PROF_NUMBER),0) + 1
            FROM PROFILES WHERE PROJECT_ID = h_project_id AND ROLE_ID = h_role_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'replaceResource', 112+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: replaceResource(113) : pools.sql(121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --Replace
      CASE h_bitflag
         
         WHEN 0 THEN
            --{
            -- bitflag = 0
            /******************************************************************
                            * replace resource with profile at resource_assignment level
                            * no change at task_assignments level
                            ******************************************************************/
            --Update the RESOURCE_ASSIGNMENTS
            BEGIN
               UPDATE RESOURCE_ASSIGNMENTS SET
               RESOURCE_ID = h_profile_id,
               RESOURCE_TYPE = '1',
               REC_STATUS = 'U'
               WHERE RES_ASSIGNMENT_ID = h_ra_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'replaceResource', 145+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: replaceResource(146) : pools.sql(154) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --}
            --case 1:
            --{
            --  // bitflag = 1
            --  /******************************************************************
            --  * if current element_type (h_type_id) == PROJECT
            --      replace resource with profile for the resource_assignment
            --  * if current element_type (h_type_id) == TASK
            --      replace resource with profile and move from task to project for the resource_assignment
            --      update task_assignments with associate resource_assignments
            --  ******************************************************************/
            --  //to be implemented
            --  break;
            --}
            --case 2:
            --{
            --  // bitflag = 2
            --  /******************************************************************
            --  * Search for existing resource assignments at project level
            --      if exists
            --          update task assignments for effected period
            --      else
            --          insert new resource assignment at project level
            --          insert new records in task assignments
            --  * replace resource with profile at resource_assignment / task level
            --  * no change at task_assignments level for existing assignments
            --  ******************************************************************/
            --  //to be implemented
            --  break;
            --}
         ELSE
            NULL;
      END CASE;
   END LOOP;
   
   BEGIN
      CLOSE CxRA ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'replaceResource', 192+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: replaceResource(193) : pools.sql(201) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --return SC_STATE;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'replaceResource', 200+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: replaceResource(201) : pools.sql(209) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION replaceResource;
ALTER FUNCTION replaceResource COMPILE DEBUG;

/***************************************************************************************************
bitflag:
    0: disable the resource and replace by equivalent profile
    2: force delete the resource and replace by equivalent profile
***************************************************************************************************/

Prompt == FUNCTION deleteResource ==
CREATE OR REPLACE FUNCTION deleteResource
(
     h_resource_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   res_bitflag PLS_INTEGER := 0;
   res_element_id VARCHAR2(32);
   h_rank VARCHAR2(200);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('deleteResource') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('deleteResource'), '-- ' || SYSTIMESTAMP, ':r := deleteResource(' || 
       '''' || h_resource_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'deleteResource', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: deleteResource(19) : pools.sql(242) : ' || SYSTIMESTAMP);
   END IF; 

   --Local variable declaration
   
   --validate resource id
   BEGIN
      SELECT RANK INTO h_rank FROM TMT_WBS WHERE ELEMENT_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 32+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(33) : pools.sql(256) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'deleteResource', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: deleteResource(40) : pools.sql(263) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_resource_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'deleteResource', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: deleteResource(43) : pools.sql(266) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_resource_id, False);
   End If;
   
   If Coalesce(h_bitflag, 0) = 2 Then
      --replace resources with profiles
      res_element_id := NULL;
      res_bitflag := 0;
      PMO_DB2M.SC_STATE := replaceResource(h_resource_id, res_element_id, res_bitflag);
      
      --Execute Delete Statement
      BEGIN
         DELETE FROM RESOURCE_EMPLOYMENT_HISTORY WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(67) : pools.sql(290) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_INDUSTRY_EXPERIENCE WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 83+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(84) : pools.sql(307) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_PROJECT_HISTORY WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 100+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(101) : pools.sql(324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_EDUCATION WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 117+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(118) : pools.sql(341) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_READINESS WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 134+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(135) : pools.sql(358) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RATES WHERE RATE_ID = (SELECT RATE_ID FROM RESOURCES WHERE RESOURCE_ID = h_resource_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(152) : pools.sql(375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM CALENDAR_DAY_OF_YEAR WHERE ELEMENT_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 168+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(169) : pools.sql(392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_rank, False); 
      	End If; 
      END;
      BEGIN
         DELETE FROM TMT_DOCUMENTS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 198+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(199) : pools.sql(422) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TMT_WBS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 215+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(216) : pools.sql(439) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TMT_POOLS WHERE ELEMENT_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 232+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(233) : pools.sql(456) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_POOLS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID in (SELECT PARENT_ID FROM TMT_POOLS WHERE ELEMENT_ID = h_resource_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 249+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(250) : pools.sql(473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCES WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 266+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(267) : pools.sql(490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Delete resource root node from tmt_portfolios
      BEGIN
         DELETE FROM TMT_PORTFOLIOS
         WHERE ELEMENT_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 285+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(286) : pools.sql(509) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 0 Then
      --replace resources with profiles
      res_element_id := NULL;
      PMO_DB2M.SC_STATE := replaceResource(h_resource_id, res_element_id, res_bitflag);
      
      --lOCK RESOURCE
      BEGIN
         UPDATE RESOURCES SET REC_STATUS = 'L', ACTIVE = '0' WHERE RESOURCE_ID = h_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'deleteResource', 307+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: deleteResource(308) : pools.sql(531) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --return SC_STATE;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'deleteResource', 315+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: deleteResource(316) : pools.sql(539) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION deleteResource;
ALTER FUNCTION deleteResource COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_POOL_DOC ==
CREATE OR REPLACE FUNCTION FC_I_POOL_DOC
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_rankBase64 VARCHAR2(8);
   h_max_rank PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_POOL_DOC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_POOL_DOC'), '-- ' || SYSTIMESTAMP, ':r := FC_I_POOL_DOC(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || '''' || h_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_DOC', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_POOL_DOC(21) : pools.sql(570) : ' || SYSTIMESTAMP);
   END IF; 

   --Local host variables
   
   PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_POOL_DOCUMENTS', h_max_rank);
   PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      INSERT INTO TMT_POOL_DOCUMENTS
      (ELEMENT_ID, PROJECT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID,
      REC_USER, RANK, PARENT_RANK)
      VALUES
      (h_uniqueID, h_uniqueID, h_name, PMO_CNST.SC_CUSTOM_FOLDER, 0,
      h_rec_user, h_rankBase64, 'POL#' || h_rank);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_DOC', 45+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_DOC(46) : pools.sql(595) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_DOC', 51+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_POOL_DOC(52) : pools.sql(601) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_POOL_DOC;
ALTER FUNCTION FC_I_POOL_DOC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_UG_DOC ==
CREATE OR REPLACE FUNCTION FC_I_UG_DOC
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_old_id VARCHAR2(32);
   h_new_id VARCHAR2(32);
   h_old_blob_id VARCHAR2(32);
   h_new_blob_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_max_rank PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_TMP_DOC IS
         SELECT ELEMENT_ID, BLOB_ID
         FROM PMO_UG_DOCUMENTS
         ORDER BY LEVEL_ID, RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_UG_DOC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_UG_DOC'), '-- ' || SYSTIMESTAMP, ':r := FC_I_UG_DOC(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || '''' || h_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_UG_DOC', 33+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_UG_DOC(34) : pools.sql(645) : ' || SYSTIMESTAMP);
   END IF; 

   --Local host variables
   
   BEGIN
      UPDATE TMT_POOL_DOCUMENTS
      SET MAX_RANK = MAX_RANK + 1
      WHERE RANK = 'qqqqqqqs';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(53) : pools.sql(664) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT MAX_RANK INTO h_max_rank
      FROM TMT_POOL_DOCUMENTS
      WHERE RANK = 'qqqqqqqs';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 67+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(68) : pools.sql(679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      INSERT INTO TMT_POOL_DOCUMENTS
      (ELEMENT_ID, PROJECT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID,
      REC_USER, RANK, PARENT_RANK)
      VALUES
      (h_uniqueID, h_uniqueID, h_name, PMO_CNST.SC_CUSTOM_FOLDER, 0,
      h_rec_user, 'qqqqqqqs' || h_rankBase64, 'POL#' || h_rank);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 91+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(92) : pools.sql(703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT 'POL#' || RTRIM(RANK)
      INTO h_parent_rank
      FROM TMT_POOLS
      WHERE ELEMENT_ID = h_parent_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(108) : pools.sql(719) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --Temporay table
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_UG_DOCUMENTS';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_UG_ASSIGNMENTS';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_UG_DOCUMENT_IDS';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_UG_BLOB_IDS';
   PMO_DB2M.SC_STATE := 0;
   
   --trace_printf("h_parent_rank: %s (%d)\n",h_parent_rank,h_parent_rank_Ind);
   --trace_flush();
   BEGIN
      INSERT INTO PMO_UG_DOCUMENTS
      SELECT * FROM TMT_POOL_DOCUMENTS
      WHERE PARENT_RANK = h_parent_rank
      AND TYPE_ID <> PMO_CNST.SC_CUSTOM_FOLDER;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 146+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(147) : pools.sql(758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_UG_ASSIGNMENTS
      SELECT * FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID IN (
      SELECT ELEMENT_ID
      FROM PMO_UG_DOCUMENTS);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 167+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(168) : pools.sql(779) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_TMP_DOC;
   
   LOOP
      FETCH C_TMP_DOC INTO h_old_id, h_old_blob_id;
      IF C_TMP_DOC%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_new_id := SYS_GUID();
      
      If (h_old_blob_id IS NOT NULL) Then
         h_new_blob_id := SYS_GUID();
         
         BEGIN
            INSERT INTO PMO_UG_BLOB_IDS (OLD_ID, NEW_ID)
            VALUES(h_old_blob_id, h_new_blob_id);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 203+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(204) : pools.sql(815) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_UG_DOCUMENTS d SET
            d.PARENT_RANK = 'POL#' || h_rank,
            d.RANK = 'qqqqqqqs' || h_rankBase64 || SUBSTR(RANK, 9, LENGTH(RANK)-8),
            d.PROJECT_ID = h_uniqueID,
            d.PARENT_ID = COALESCE((SELECT NEW_ID FROM PMO_UG_DOCUMENT_IDS WHERE OLD_ID = d.PARENT_ID),h_uniqueID),
            d.ELEMENT_ID = h_new_id,
            d.BLOB_ID = h_new_blob_id
            WHERE d.ELEMENT_ID = h_old_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 227+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(228) : pools.sql(839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE PMO_UG_DOCUMENTS d SET
            d.PARENT_RANK = 'POL#' || h_rank,
            d.RANK = 'qqqqqqqs' || h_rankBase64 || SUBSTR(RANK, 9, LENGTH(RANK)-8),
            d.PROJECT_ID = h_uniqueID,
            d.PARENT_ID = COALESCE((SELECT NEW_ID FROM PMO_UG_DOCUMENT_IDS WHERE OLD_ID = d.PARENT_ID),h_uniqueID),
            d.ELEMENT_ID = h_new_id
            WHERE d.ELEMENT_ID = h_old_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 250+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(251) : pools.sql(862) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE PMO_UG_ASSIGNMENTS SET
         RES_ASSIGNMENT_ID = h_new_id,
         PROJECT_ID = h_uniqueID,
         ELEMENT_ID = h_new_id
         WHERE ELEMENT_ID = h_old_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 272+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(273) : pools.sql(884) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_UG_DOCUMENT_IDS (OLD_ID, NEW_ID)
         VALUES(h_old_id, h_new_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(291) : pools.sql(902) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE C_TMP_DOC ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 304+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(305) : pools.sql(916) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO DOCUMENT_BLOBS (ELEMENT_ID, ATTACHEMENT, BLOB_DATA)
      SELECT T.NEW_ID, B.ATTACHEMENT, B.BLOB_DATA
      FROM PMO_UG_BLOB_IDS T
      JOIN DOCUMENT_BLOBS B
      ON B.ELEMENT_ID = T.OLD_ID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 325+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(326) : pools.sql(937) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO RESOURCE_ASSIGNMENTS
      SELECT * FROM PMO_UG_ASSIGNMENTS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 343+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(344) : pools.sql(955) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO TMT_POOL_DOCUMENTS
      SELECT * FROM PMO_UG_DOCUMENTS;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_UG_DOC', 361+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_UG_DOC(362) : pools.sql(973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_UG_DOC', 367+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_UG_DOC(368) : pools.sql(979) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_UG_DOC;
ALTER FUNCTION FC_I_UG_DOC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_RES_DOC ==
CREATE OR REPLACE FUNCTION FC_I_RES_DOC
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
   , h_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_rankBase64 VARCHAR2(8);
   h_max_rank PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_RES_DOC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_RES_DOC'), '-- ' || SYSTIMESTAMP, ':r := FC_I_RES_DOC(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_uniqueID || ''''
          || ', ' || '''' || h_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_RES_DOC', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_RES_DOC(21) : pools.sql(1010) : ' || SYSTIMESTAMP);
   END IF; 

   --Local host variables
   
   PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_RESOURCE_DOCUMENTS', h_max_rank);
   PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      INSERT INTO TMT_RESOURCE_DOCUMENTS(ELEMENT_ID, PROJECT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID, REC_USER, RANK, PARENT_RANK)
      VALUES(h_uniqueID, h_uniqueID, h_name, PMO_CNST.SC_CUSTOM_FOLDER, 0, h_rec_user, h_rankBase64, 'RES#' || h_rank);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_RES_DOC', 41+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_RES_DOC(42) : pools.sql(1031) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_RES_DOC', 47+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_RES_DOC(48) : pools.sql(1037) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_RES_DOC;
ALTER FUNCTION FC_I_RES_DOC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_RES_SCD_FOLDERS ==
CREATE OR REPLACE FUNCTION FC_I_RES_SCD_FOLDERS
(
     h_resource_id IN OUT NOCOPY VARCHAR2
   , h_rank IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_max_rank PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_uniqueID VARCHAR2(32);
   h_cur_rank VARCHAR2(200);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_RES_SCD_FOLDERS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_RES_SCD_FOLDERS'), '-- ' || SYSTIMESTAMP, ':r := FC_I_RES_SCD_FOLDERS(' || 
       '''' || h_resource_id || ''''
          || ', ' || '''' || h_rank || ''''
          || ', ' || '''' || h_name || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_RES_SCD_FOLDERS', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_RES_SCD_FOLDERS(23) : pools.sql(1070) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Create project_folder and group folders in TMT_ASSIGNED_SCORECARDS
   PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_ASSIGNED_SCORECARDS', h_max_rank);
   PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
   
   -- insert project_folder
   BEGIN
      INSERT INTO TMT_ASSIGNED_SCORECARDS (RANK, PARENT_RANK, TYPE_ID, LEVEL_ID, PROJECT_ID,
      PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, MAX_RANK, CREATED_BY, REC_USER)
      VALUES (h_rankBase64, 'RES#' || h_rank, PMO_CNST.SC_CUSTOM_FOLDER, 0, h_resource_id,
      Null, h_resource_id, h_name, 3, 3, h_rec_user, h_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_RES_SCD_FOLDERS', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_RES_SCD_FOLDERS(48) : pools.sql(1095) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_uniqueID := SYS_GUID();
   h_uniqueID := h_uniqueID;
   h_cur_rank := h_rankBase64;
   h_cur_rank := SubStr(h_cur_rank || '0001', 1, 200);
   
   -- insert wbs group_folder
   BEGIN
      INSERT INTO TMT_ASSIGNED_SCORECARDS (RANK, PARENT_RANK, TYPE_ID, LEVEL_ID, PROJECT_ID,
      PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CREATED_BY, REC_USER)
      VALUES (h_cur_rank, 'RES#' || h_rank, PMO_CNST.SC_CUSTOM_FOLDER, 1, h_resource_id,
      h_resource_id, h_uniqueID, 'WBS', h_rec_user, h_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_RES_SCD_FOLDERS', 73+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_RES_SCD_FOLDERS(74) : pools.sql(1121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Create project_folder and group folders in TMT_ASSIGNED_SCORECARDS
   PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_PUBLISHED_SCORECARDS', h_max_rank);
   PMO_STPU.toBase64Long(Coalesce(h_max_rank, 0), h_rankBase64);
   
   -- insert project_folder
   BEGIN
      INSERT INTO TMT_PUBLISHED_SCORECARDS (RANK, PARENT_RANK, TYPE_ID, LEVEL_ID, PROJECT_ID,
      PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, MAX_RANK, CREATED_BY, REC_USER)
      VALUES (h_rankBase64, 'RES#' || h_rank, PMO_CNST.SC_CUSTOM_FOLDER, 0, h_resource_id,
      Null, h_resource_id, h_name, 3, 3, h_rec_user, h_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_RES_SCD_FOLDERS', 98+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_RES_SCD_FOLDERS(99) : pools.sql(1146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_uniqueID := SYS_GUID();
   h_uniqueID := h_uniqueID;
   h_cur_rank := h_rankBase64;
   h_cur_rank := SubStr(h_cur_rank || '0001', 1, 200);
   
   -- insert wbs group_folder
   BEGIN
      INSERT INTO TMT_PUBLISHED_SCORECARDS (RANK, PARENT_RANK, TYPE_ID, LEVEL_ID, PROJECT_ID,
      PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CREATED_BY, REC_USER)
      VALUES (h_cur_rank, 'RES#' || h_rank, PMO_CNST.SC_CUSTOM_FOLDER, 1, h_resource_id,
      h_resource_id, h_uniqueID, 'WBS', h_rec_user, h_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_RES_SCD_FOLDERS', 124+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_RES_SCD_FOLDERS(125) : pools.sql(1172) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_RES_SCD_FOLDERS', 130+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_RES_SCD_FOLDERS(131) : pools.sql(1178) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_RES_SCD_FOLDERS', 135+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_RES_SCD_FOLDERS(136) : pools.sql(1183) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_RES_SCD_FOLDERS;
ALTER FUNCTION FC_I_RES_SCD_FOLDERS COMPILE DEBUG;

/****************************************************************************************
FC_POPULATE_RES_AT
  Populate resource's administrative tasks from the pool's list

  h_id: TMT_POOLS.ELEMENT_ID/TYPE_ID = SC_RESOURCE
  h_parent_id: TMT_WBS.POOL_ID (Resource "Administrative Tasks" summary task
  h_project_id: TMT_WBS.ELEMENT_ID (Resource's personal project id)
  h_rec_user: TMT_POOLS.ELEMENT_ID/TYPE_ID = SC_RESOURCE (= h_id)
****************************************************************************************/

Prompt == FUNCTION FC_POPULATE_RES_AT ==
CREATE OR REPLACE FUNCTION FC_POPULATE_RES_AT
(
     h_id IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_adm_tsk_name VARCHAR2(255);
   h_calendar_id VARCHAR2(32);
   h_day_year DATE;
   h_pool_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_tsk_assignment_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_AT IS
         SELECT POOL_ID, ELEMENT_NAME, TYPE_ID
         FROM PMO_TMP_WBS;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_POPULATE_RES_AT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_POPULATE_RES_AT'), '-- ' || SYSTIMESTAMP, ':r := FC_POPULATE_RES_AT(' || 
       '''' || h_id || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RES_AT', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_POPULATE_RES_AT(31) : pools.sql(1232) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT CALENDAR_ID INTO h_calendar_id
      FROM RESOURCES WHERE RESOURCE_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 42+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(43) : pools.sql(1244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RES_AT', 48+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RES_AT(49) : pools.sql(1250) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN -1;
   ElsIf (h_calendar_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RES_AT', 53+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RES_AT(54) : pools.sql(1255) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   --Temporay table
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_WBS';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      SELECT TO_DATE(SYSDATE) INTO h_day_year FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 71+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(72) : pools.sql(1273) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --Pool (WBS) "Administrative Task" Summary Task
   BEGIN
      SELECT PMO_ORASTD.LEFT(RANK,LENGTH(RANK)-4) || '0001'
      INTO h_rank
      FROM TMT_WBS
      WHERE POOL_ID = h_id
      AND TYPE_ID = PMO_CNST.SC_RESOURCE;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 89+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(90) : pools.sql(1291) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --Pool (WBS) "Administrative Task" List
   BEGIN
      INSERT INTO PMO_TMP_WBS
      SELECT POOL_ID, ELEMENT_NAME, TYPE_ID
      FROM TMT_WBS
      WHERE RANK LIKE h_rank || '%'
      AND TYPE_ID = PMO_CNST.SC_ACTIVE_ADM_TASK;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 111+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(112) : pools.sql(1313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_AT;
   
   LOOP
      FETCH C_AT INTO h_pool_id, h_adm_tsk_name, h_type_id;
      IF C_AT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_tsk_assignment_id := '';
      
      --trace_printf("h_rank: %s (%d)\n", h_rank, h_rank_Ind);
      --trace_printf("h_parent_id: %s (%d)\n", h_parent_id, h_parent_id_Ind);
      --trace_printf("h_rec_user: %s (%d)\n", h_rec_user, h_rec_user_Ind);
      --trace_printf("h_day_year: %s (%d)\n", h_day_year, h_day_year_Ind);
      --trace_printf("h_pool_id: %s (%d)\n", h_pool_id, h_pool_id_Ind);
      --trace_printf("h_adm_tsk_name: %s (%d)\n", h_adm_tsk_name, h_adm_tsk_name_Ind);
      --trace_printf("h_type_id: %d (%d)\n", h_type_id, h_type_id_Ind);
      --trace_flush();
      
      --h_rec_user: resource id
      --h_parent_id: resource (WBS) "Administrative Tasks" summary task
      PMO_DB2M.SC_STATE := FC_I_SUMMARY_TS(h_adm_tsk_name, h_parent_id, h_type_id, h_day_year, h_rec_user
                                     , h_tsk_assignment_id);
      
      --Make resource's admin task's pool_id points to pool's admin task's pool_id
      --for operations like delete
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         BEGIN
            UPDATE TMT_WBS SET
            POOL_ID = h_pool_id,
            REC_STATUS = 'U'
            WHERE POOL_ID = h_parent_id
            AND TYPE_ID = PMO_CNST.SC_ACTIVE_ADM_TASK;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 164+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(165) : pools.sql(1366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_AT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_POPULATE_RES_AT', 179+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_POPULATE_RES_AT(180) : pools.sql(1381) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RES_AT', 185+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RES_AT(186) : pools.sql(1387) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_POPULATE_RES_AT', 189+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_POPULATE_RES_AT(190) : pools.sql(1391) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_POPULATE_RES_AT;
ALTER FUNCTION FC_POPULATE_RES_AT COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_POOL_WBS ==
CREATE OR REPLACE FUNCTION FC_I_POOL_WBS
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_id IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
   , sh_h_type_id PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_uniqueID VARCHAR2(32);
   h_child_count PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_POOL_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_POOL_WBS'), '-- ' || SYSTIMESTAMP, ':r := FC_I_POOL_WBS(' || 
       '''' || h_project_id || ''''
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_name || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_WBS', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_POOL_WBS(29) : pools.sql(1430) : ' || SYSTIMESTAMP);
   END IF; 

   If (Coalesce(h_type_id, 0) <> PMO_CNST.SC_POOLS) AND (Coalesce(
             h_type_id, 0) <> PMO_CNST.SC_RESOURCE) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_WBS', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_I_POOL_WBS(35) : pools.sql(1436) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   BEGIN
      UPDATE TMT_POOLS
      SET MAX_RANK = MAX_RANK + 1
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(54) : pools.sql(1455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT SETTING_VALUE INTO h_currency_id
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'SYSTEM_DEFAULT_CURRENCY';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 68+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(69) : pools.sql(1470) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT '/PL#' || RTRIM(RANK), LEVEL_ID, MAX_RANK, CHILD_COUNT
      INTO h_rank, h_level_id, h_max_rank, h_child_count
      FROM TMT_POOLS
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(85) : pools.sql(1486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT ELEMENT_ID
      INTO h_parent_id
      FROM TMT_WBS
      WHERE RANK = PMO_ORASTD.LEFT(h_rank, LENGTH(h_rank)-4);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 100+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(101) : pools.sql(1502) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      h_parent_id := NULL;
   End If;
   
   -- for resources the new resource will be project manager of personal project
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      h_rec_user := h_id;
      --trace_printf("h_project_id: %s (%d)\n",h_project_id,h_project_id_Ind);
      --trace_printf("h_parent_id: %s (%d)\n",h_parent_id,h_parent_id_Ind);
      --trace_printf("h_id: %s (%d)\n",h_id,h_id_Ind);
      --trace_printf("h_name: %s (%d)\n",h_name,h_name_Ind);
      --trace_printf("h_rank: %s (%d)\n",h_rank,h_rank_Ind);
      --trace_flush();
      --Insert project type node in WBS
      BEGIN
         INSERT INTO TMT_WBS (POOL_ID, PROJECT_ID, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID, CREATED_BY, REC_USER, REC_STATUS, RANK, ASSIGN_NAMES,
         PUBLISHED, PRIORITY,
         MAX_RANK, CHILD_COUNT, CURRENCY_ID)
         VALUES (h_id, h_project_id, h_parent_id, h_project_id, RTRIM(LTRIM(h_name)),
         h_type_id, h_level_id, h_rec_user, h_rec_user, 'U', h_rank, RTRIM(LTRIM(h_name)) || ';',
         CASE WHEN h_type_id = PMO_CNST.SC_POOLS THEN '0' ELSE '1' END, PMO_CNST.SC_PERSONAL_PRIORITY,
         h_max_rank, h_child_count+1, h_currency_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 137+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(138) : pools.sql(1539) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PROJECT_DETAILS (PROJECT_ID) VALUES(h_project_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 154+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(155) : pools.sql(1556) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --update TMT_WBS (Resource's Pool)
      BEGIN
         UPDATE TMT_WBS W
         SET (W.MAX_RANK, W.CHILD_COUNT) =
         (SELECT MAX_RANK, CHILD_COUNT FROM TMT_POOLS WHERE RANK = SUBSTR(W.RANK,5))
         WHERE W.RANK = PMO_ORASTD.LEFT(h_rank,LENGTH(h_rank)-4);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 175+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(176) : pools.sql(1577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Tasks Holder Deliverable ID
      h_uniqueID := SYS_GUID();
      
      --Pool Administrative Tasks Holder
      BEGIN
         INSERT INTO TMT_WBS (POOL_ID, PROJECT_ID, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID, CREATED_BY, REC_USER, REC_STATUS, RANK, ASSIGN_NAMES,
         PUBLISHED, MAX_RANK, CHILD_COUNT)
         VALUES (h_id, h_project_id, h_project_id, h_uniqueID, 'Administrative Tasks',
         PMO_CNST.SC_SUMMARY_TASK, h_level_id+1, h_rec_user, h_rec_user, 'U', h_rank || '0001', RTRIM(LTRIM(h_name)) || ';',
         '0', 0, 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 201+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(202) : pools.sql(1603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --resource assignment
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
         RESOURCE_ID, PARTICIPANT, SECURITY_ID, CRI_SECURITY_ID, REC_USER, TABLE_TYPE)
         VALUES( SYS_GUID(), h_project_id, h_uniqueID, PMO_CNST.SC_SUMMARY_TASK,
         h_rec_user, PMO_CNST.SC_DLV_MANAGER, 'SCDELIVERABLEMANAGER', NULL, h_rec_user, PMO_CNST.SC_TABLE_WBS );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 222+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(223) : pools.sql(1624) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --Insert project type node in WBS
      BEGIN
         INSERT INTO TMT_WBS (POOL_ID, PROJECT_ID, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID, CREATED_BY, REC_USER, REC_STATUS, RANK, ASSIGN_NAMES,
         PUBLISHED,
         MAX_RANK, CHILD_COUNT)
         VALUES (h_id, h_project_id, h_parent_id, h_project_id, RTRIM(LTRIM(h_name)),
         h_type_id, h_level_id, h_rec_user, h_rec_user, 'U', h_rank, RTRIM(LTRIM(h_name)) || ';',
         CASE WHEN h_type_id = PMO_CNST.SC_POOLS THEN '0' ELSE '1' END,
         h_max_rank, h_child_count);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 247+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(248) : pools.sql(1649) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Tasks Holder Deliverable ID
      h_uniqueID := SYS_GUID();
      
      --Pool Administrative Tasks Holder
      BEGIN
         INSERT INTO TMT_WBS (POOL_ID, PROJECT_ID, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID, CREATED_BY, REC_USER, REC_STATUS, RANK, ASSIGN_NAMES,
         PUBLISHED, MAX_RANK, CHILD_COUNT)
         VALUES (h_id, h_project_id, h_project_id, h_uniqueID, 'Administrative Task',
         PMO_CNST.SC_SUMMARY_TASK, h_level_id+1, h_rec_user, h_rec_user, 'U', h_rank || '0001', RTRIM(LTRIM(h_name)) || ';',
         '0', 0, 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 273+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(274) : pools.sql(1675) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --resource assignment
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
         RESOURCE_ID, PARTICIPANT, SECURITY_ID, CRI_SECURITY_ID, REC_USER, TABLE_TYPE)
         VALUES( SYS_GUID(), h_project_id, h_uniqueID, PMO_CNST.SC_SUMMARY_TASK,
         h_rec_user, PMO_CNST.SC_DLV_MANAGER, 'SCDELIVERABLEMANAGER', NULL, h_rec_user, PMO_CNST.SC_TABLE_WBS );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 294+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(295) : pools.sql(1696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      INSERT INTO PROJECT_ASSIGNMENTS (
      PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, RESOURCE_TYPE, ELEMENT_TYPE_BITS)
      VALUES (h_project_id, h_rec_user, PMO_CNST.SC_MANAGER, 'SCPROJECTMANAGER', '0', PMO_CNST.SC_PROJECT_BIT+PMO_CNST.SC_SUMMARY_TASK_BIT );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 314+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(315) : pools.sql(1716) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO RESOURCE_ASSIGNMENTS (RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
      RESOURCE_ID, PARTICIPANT, SECURITY_ID, CRI_SECURITY_ID, REC_USER, TABLE_TYPE)
      VALUES( SYS_GUID(), h_project_id, h_project_id, h_type_id,
      h_rec_user, PMO_CNST.SC_MANAGER, 'SCPROJECTMANAGER', NULL, h_rec_user, PMO_CNST.SC_TABLE_WBS );
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_POOL_WBS', 334+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_POOL_WBS(335) : pools.sql(1736) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_WBS', 340+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_POOL_WBS(341) : pools.sql(1742) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_POOL_WBS', 344+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_POOL_WBS(345) : pools.sql(1746) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_POOL_WBS;
ALTER FUNCTION FC_I_POOL_WBS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_I_FS_POOL ==
CREATE OR REPLACE FUNCTION FC_I_FS_POOL
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_parent_id IN OUT NOCOPY VARCHAR2
   , sh_h_parm_type_id PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
   , h_res_id IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_parm_type_id PLS_INTEGER := sh_h_parm_type_id;
   h_id VARCHAR2(32);
   h_logon_name VARCHAR2(100);
   h_new_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rate_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_resource_name VARCHAR2(100);
   h_root_id VARCHAR2(32);
   h_tpf_rankBase64 VARCHAR2(8);
   h_user_group_id VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_level PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_psw_versions PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_tpf_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_rec_user2 VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_clearance_level_1 PLS_INTEGER := 0;
   h_clearance_level_2 PLS_INTEGER := 0;
   h_clearance_level_3 PLS_INTEGER := 0;
   h_clearance_level_4 PLS_INTEGER := 0;
   h_clearance_level_5 PLS_INTEGER := 0;
   h_mask_employee_info PLS_INTEGER := 0;
   h_cost_center_id VARCHAR2(32);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_I_FS_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_I_FS_POOL'), '-- ' || SYSTIMESTAMP, ':r := FC_I_FS_POOL(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_parm_type_id), 'NULL')
          || ', ' || '''' || h_rec_user || ''''
          || ', ' || '''' || h_res_id || ''''
          || ', ' || '''' || h_uniqueID || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_FS_POOL', 53+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_I_FS_POOL(54) : pools.sql(1810) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_type_id := Coalesce(h_parm_type_id, 0);
   
   If (h_res_id IS NULL) Then
      h_uniqueID := SYS_GUID();
   Else
      h_uniqueID := h_res_id;
   End If;
   
   BEGIN 
      SELECT REC_STATUS INTO h_rec_status FROM RESOURCES WHERE RESOURCE_ID = h_rec_user; 
      IF SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') THEN 
         return PMODEFINES.PMO_RETURN_INVALID(PMO_CNST.PARENT_MARKED_AS_DELETED, False); 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_rec_user, False); 
   END;
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RES_FS Then
      h_bitflag := 1;
      -- FS resource
      h_type_id := PMO_CNST.SC_RESOURCE;
      /*p_type_id = SC_RESOURCE;*/
   End If;
   
   If ((h_type_id IS NOT NULL)) AND (Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE) Then
      BEGIN
         SELECT SETTING_VALUE INTO h_psw_versions FROM TMT_SETTINGS
         WHERE ELEMENT_ID = 'MAX_PASSWORD_BACKUP_VERSIONS';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(93) : pools.sql(1849) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --tjtjtjtjtjtj
      BEGIN
         SELECT SETTING_VALUE INTO h_currency_id
         FROM TMT_SETTINGS
         WHERE ELEMENT_ID = 'SYSTEM_DEFAULT_CURRENCY';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 108+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(109) : pools.sql(1865) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT USER_GROUP_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2,
         CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5,
         COST_CENTER_ID
         INTO h_user_group_id, h_clearance_level_1, h_clearance_level_2,
         h_clearance_level_3, h_clearance_level_4, h_clearance_level_5,
         h_cost_center_id
         FROM TMT_POOLS
         WHERE ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 128+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(129) : pools.sql(1885) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_rate_id := SYS_GUID();
      
      --Check whether Full name is unique in the Database
      BEGIN
         SELECT COUNT(RESOURCE_ID) INTO h_count
         FROM RESOURCES
         WHERE LOGON_NAME = RTRIM(LTRIM(h_name));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 146+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(147) : pools.sql(1903) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) > 0 Then
         h_logon_name := h_uniqueID;
      Else
         h_logon_name := SubStr(h_name, 1, 100);
      End If;
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'FC_I_FS_POOL', 158+1) THEN 
         LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: FC_I_FS_POOL(159) : pools.sql(1915) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, False); 
      END;
      PMODEFINES.PMO_GET_MAX_RANK('TMT_POOLS', h_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         SELECT RTRIM(Rank) || h_rankBase64, LEVEL_ID + 1, MASK_EMPLOYEE_INFO
         INTO h_rank, h_level_id, h_mask_employee_info
         FROM TMT_POOLS
         WHERE ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 182+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(183) : pools.sql(1939) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT NICK_NAME INTO h_resource_name
         FROM RESOURCES
         WHERE RESOURCE_ID = h_rec_user;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 197+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(198) : pools.sql(1954) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO TMT_POOLS(RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         TYPE_ID, LEVEL_ID, MANAGER_ID, REC_USER,
         ASSIGN_NAMES, MASK_EMPLOYEE_INFO)
         VALUES (h_rank, h_parent_id, h_uniqueID, RTRIM(LTRIM(h_name)),
         PMO_CNST.SC_RESOURCE, h_level_id, h_rec_user, h_rec_user,
         h_resource_name || ';', h_mask_employee_info);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 219+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(220) : pools.sql(1976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO RESOURCES (RESOURCE_ID, FULL_NAME, NICK_NAME, CREATED_BY,
         ACTIVE, LOGON_NAME, LOGON_PASSWORD,
         KEY_HISTORY,
         PASSWORD_EXPIRES, FS_LABOR_CODE, MULTIPLE_LOGINS, CURRENCY_ID, RATE_ID, REC_USER,
         CITIZENSHIP, EFFECTIVE_DATE, EXPERIENCE_ID, WORK_LOCATION,
         USER_GROUP_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5,
         STATUS1_ID, STATUS2_ID, MAXDAYHRS)
         VALUES (h_uniqueID, RTRIM(LTRIM(h_name)), RTRIM(LTRIM(h_name)), h_rec_user,
         '1', RTRIM(LTRIM(h_logon_name)), PMO_ORASTD.PMO_ENCRYPT(PMO_CNST.SC_DEFAULT_PWD_DB2_STR, h_uniqueID, ''),
         case WHEN h_psw_versions = 0 THEN NULL ELSE PMO_ORASTD.PMO_ENCRYPT(PMO_CNST.SC_PWD_KEY_HISTORY_DB2_STR, h_uniqueID, '') END,
         (TO_DATE(SYSDATE) - 1    ), 'PMOFFICE', 'Y', h_currency_id, h_rate_id, h_rec_user,
         2726, TO_DATE(SYSDATE), 2771, 265,
         h_user_group_id, h_clearance_level_1, h_clearance_level_2, h_clearance_level_3,
         h_clearance_level_4,h_clearance_level_5,
         312, 318, 40);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 250+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(251) : pools.sql(2007) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Insert TMT_PORTFOLIOS record
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_PORTFOLIOS', h_tpf_max_rank);
      PMO_STPU.toBase64(Coalesce(h_tpf_max_rank, 0), h_tpf_rankBase64);
      
      BEGIN
         INSERT INTO TMT_PORTFOLIOS (RANK,TYPE_ID,LEVEL_ID,ELEMENT_ID,REC_STATUS,CHILD_COUNT)
         VALUES ('0000' || h_tpf_rankBase64,PMO_CNST.SC_RESOURCE,0, h_uniqueID,'I',0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 272+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(273) : pools.sql(2029) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO RATES (RATE_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, REC_USER, REC_DATETIME)
         VALUES(h_rate_id, 0.00, 0.00, 0.00, 0.00, h_rec_user, SYSTIMESTAMP);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 290+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(291) : pools.sql(2047) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_I_RES_DOC(h_name, h_uniqueID, h_rank, h_rec_user);
      
      h_temp_sc_state := FC_I_RES_SCD_FOLDERS(h_uniqueID, h_rank, h_name, h_rec_user);
      
      h_id := h_rate_id;
      h_rate_id := SYS_GUID();
      BEGIN
         INSERT INTO HISTORY_RESOURCES SELECT h_rate_id, A.* FROM RESOURCES A
         WHERE A.RESOURCE_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 314+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(315) : pools.sql(2071) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE HISTORY_RESOURCES SET REC_STATUS = 'U' WHERE HISTORY_ID = h_rate_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 331+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(332) : pools.sql(2088) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE RESOURCES SET REVISION_HISTORY = Null WHERE RESOURCE_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 348+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(349) : pools.sql(2105) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO RESOURCE_COST_CENTERS (ELEMENT_ID, RESOURCE_ID, COST_CENTER_ID, START_DATE, FINISH_DATE, REC_USER)
         VALUES(SYS_GUID(), h_uniqueID, h_cost_center_id, TO_DATE('1899-12-30'), TO_DATE('2300-12-30'), h_rec_user);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 366+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(367) : pools.sql(2123) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_new_id := SYS_GUID();
         h_rec_user2 := h_rec_user;
         PMO_DB2M.SC_STATE := FC_I_POOL_WBS(h_new_id, h_uniqueID, h_name, h_type_id, h_rec_user);
         h_rec_user := h_rec_user2;
      End If;
   Else
      --trace_printf("Pool Types\n");
      --trace_printf("h_parent_id: %s (%d)\n", h_parent_id, h_parent_id_Ind);
      --trace_flush();
      h_ra_id := SYS_GUID();
      
      BEGIN
         SELECT USER_GROUP_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5
         INTO h_user_group_id, h_clearance_level_1, h_clearance_level_2, h_clearance_level_3, h_clearance_level_4, h_clearance_level_5
         FROM TMT_POOLS
         WHERE ELEMENT_ID = 'DEFAULT_SECURITY_ROLE';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 394+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(395) : pools.sql(2151) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT NICK_NAME INTO h_resource_name
         FROM RESOURCES
         WHERE RESOURCE_ID = h_rec_user;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 409+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(410) : pools.sql(2166) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Validate parent_id
      If (h_parent_id IS NULL) Then
         BEGIN
            SELECT ELEMENT_ID INTO h_cost_center_id
            FROM TMT_CLIENTS
            WHERE DEFAULT_FLAG = 'Y';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 426+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(427) : pools.sql(2183) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_FS_POOL', 433+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_I_FS_POOL(434) : pools.sql(2190) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.CANNOT_FIND_COST_CENTER;
         End If;
         
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_POOLS', h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         --Execute Insert Statement
         BEGIN
            SELECT  RTRIM(COALESCE(RANK, '')) || h_rankBase64,
            LEVEL_ID + 1
            INTO    h_rank,
            h_level_id
            FROM TMT_POOLS WHERE ELEMENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 454+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(455) : pools.sql(2211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO TMT_POOLS(
            RANK, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID,
            MANAGER_ID, CREATED_BY, REC_USER, USER_GROUP_ID, COST_CENTER_ID,
            CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3,
            CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5,
            ASSIGN_NAMES)
            VALUES(
            h_rank, h_uniqueID, h_name, h_type_id, h_level_id,
            h_rec_user, h_rec_user, h_rec_user, h_user_group_id, h_cost_center_id,
            h_clearance_level_1, h_clearance_level_2, h_clearance_level_3,
            h_clearance_level_4, h_clearance_level_5,
            h_resource_name || ';');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 482+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(483) : pools.sql(2239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_root_id := h_uniqueID;
      Else
         --Validate Pool
         BEGIN
            SELECT RANK, COST_CENTER_ID INTO h_rank, h_cost_center_id
            FROM TMT_POOLS WHERE ELEMENT_ID = h_parent_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 499+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(500) : pools.sql(2256) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_I_FS_POOL', 505+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_I_FS_POOL(506) : pools.sql(2262) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_parent_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_FS_POOL', 508+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_I_FS_POOL(509) : pools.sql(2265) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, False);
         End If;
         
         --Retrieve Pool Root id
         BEGIN
            SELECT ELEMENT_ID INTO h_root_id FROM TMT_POOLS
            WHERE RANK = SUBSTR( h_rank, 1, 8)
            AND PARENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 524+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(525) : pools.sql(2281) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_I_FS_POOL', 531+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_I_FS_POOL(532) : pools.sql(2288) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_parent_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_FS_POOL', 534+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_I_FS_POOL(535) : pools.sql(2291) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, False);
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_POOLS', h_parent_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         --Execute Insert Statement
         BEGIN
            SELECT
            RTRIM(Rank) || h_rankBase64,
            LEVEL_ID + 1
            INTO
            h_rank,
            h_level_id
            FROM TMT_POOLS WHERE ELEMENT_ID = h_parent_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 557+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(558) : pools.sql(2314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO TMT_POOLS(
            RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, TYPE_ID, LEVEL_ID,
            MANAGER_ID, CREATED_BY, REC_USER, USER_GROUP_ID, COST_CENTER_ID,
            CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3,
            CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5,
            ASSIGN_NAMES)
            VALUES (
            h_rank, h_parent_id, h_uniqueID, h_name, h_type_id, h_level_id,
            h_rec_user, h_rec_user, h_rec_user, h_user_group_id, h_cost_center_id,
            h_clearance_level_1, h_clearance_level_2, h_clearance_level_3,
            h_clearance_level_4, h_clearance_level_5,
            h_resource_name || ';');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 585+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(586) : pools.sql(2342) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_POOLS Then
         h_temp_sc_state := FC_I_POOL_DOC(h_name, h_uniqueID, h_rank, h_rec_user);
      Else
         --User Groups in System Administration (Layout/Setting Files)
         --          trace_printf("the new type: %d", h_type_id);
         --          trace_printf("Create Document Structure From Parent");
         --trace_flush();
         h_temp_sc_state := FC_I_UG_DOC(h_name, h_parent_id, h_uniqueID, h_rank, h_rec_user);
      End If;
      
      h_rate_id := SYS_GUID();
      
      BEGIN
         INSERT INTO HISTORY_POOLS SELECT h_rate_id, A.* FROM TMT_POOLS A
         WHERE A.ELEMENT_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 616+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(617) : pools.sql(2373) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE HISTORY_POOLS SET REC_STATUS = 'U' WHERE HISTORY_ID = h_rate_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 633+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(634) : pools.sql(2390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_POOLS SET REVISION_HISTORY = Null WHERE ELEMENT_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 650+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(651) : pools.sql(2407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS (
         RES_ASSIGNMENT_ID,
         PROJECT_ID,
         ELEMENT_ID,
         ELEMENT_TYPE,
         RESOURCE_TYPE,
         RESOURCE_ID,
         PARTICIPANT,
         SECURITY_ID,
         REC_USER,
         REC_DATETIME,TABLE_TYPE)
         VALUES (
         h_ra_id,
         h_root_id,
         h_uniqueID,
         h_type_id,
         '0',
         h_rec_user,
         PMO_CNST.SC_RESOURCE_MANAGER,
         'SCRESOURCEMANAGER',
         h_rec_user,
         SYSTIMESTAMP,
         PMO_CNST.SC_TABLE_POL);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_I_FS_POOL', 689+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_I_FS_POOL(690) : pools.sql(2446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_new_id := SYS_GUID();
         PMO_DB2M.SC_STATE := FC_I_POOL_WBS(h_new_id, h_uniqueID, h_name, h_type_id, h_rec_user);
      End If;
   End If;
   
   If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(h_bitflag, 0) = 1) Then
      h_res_id := h_uniqueID;
      -- resource_id
      h_uniqueID := h_id;
      -- rate_id
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_I_FS_POOL', 708+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_I_FS_POOL(709) : pools.sql(2465) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_I_FS_POOL;
ALTER FUNCTION FC_I_FS_POOL COMPILE DEBUG;

/********************************************************************************/

/****************************************************************************************/

Prompt == FUNCTION SP_I_FS_POOL ==
CREATE OR REPLACE FUNCTION SP_I_FS_POOL
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
   , p_res_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_pool_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_FS_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_FS_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_I_FS_POOL(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || ''''
          || ', ' || '''' || p_res_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_FS_POOL', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_FS_POOL(29) : pools.sql(2506) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_pool_name := p_name;
   h_parent_id := p_parent_id;
   h_rec_user := p_rec_user;
   h_res_id := p_res_id;
   h_type_id := p_type_id;
   
   PMO_DB2M.SC_STATE := FC_I_FS_POOL(h_pool_name, h_parent_id, h_type_id, h_rec_user, h_res_id
                                  , h_uniqueID);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FS_POOL', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FS_POOL(47) : pools.sql(2524) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_FS_POOL', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_FS_POOL(59) : pools.sql(2536) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_FS_POOL;
ALTER FUNCTION SP_I_FS_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_I_POOL ==
CREATE OR REPLACE FUNCTION SP_I_POOL
(
     p_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_name VARCHAR2(255);
   h_parent_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_I_POOL(' || 
       '''' || p_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_POOL', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_POOL(27) : pools.sql(2579) : ' || SYSTIMESTAMP);
   END IF; 

   h_name := p_name;
   h_parent_id := p_parent_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   
   h_res_id := NULL;
   
   PMO_DB2M.SC_STATE := FC_I_FS_POOL(h_name, h_parent_id, h_type_id, h_rec_user, h_res_id, h_uniqueID);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_POOL', 39+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_POOL(40) : pools.sql(2592) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_POOL', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_POOL(47) : pools.sql(2599) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_POOL', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_POOL(59) : pools.sql(2611) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_POOL;
ALTER FUNCTION SP_I_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_NAME ==
CREATE OR REPLACE FUNCTION SP_U_POOL_NAME
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_error_id VARCHAR2(32);
   h_host_table PLS_INTEGER := 0;
   h_id VARCHAR2(32);
   h_name VARCHAR2(255);
   h_rec_user VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_NAME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_NAME'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_NAME(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_NAME', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_NAME(26) : pools.sql(2653) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_NAME', 33+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_NAME(34) : pools.sql(2661) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_NAME', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_NAME(37) : pools.sql(2664) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_u_pool_name->p_id);
   h_id := p_id;
   h_name := p_name;
   h_rec_user := p_rec_user;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_POOLS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_NAME', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_NAME(55) : pools.sql(2682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_POOL_NAME', 61+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_POOL_NAME(62) : pools.sql(2689) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_NAME', 64+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_NAME(65) : pools.sql(2692) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      PMO_DB2M.SC_STATE := FC_U_RES_FULLNAME(h_id, h_name, h_rec_user);
   Else
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_POOLS Then
         h_host_table := PMO_CNST.SC_TABLE_POL;
         PMO_DB2M.SC_STATE := FC_U_ITEM_NAME_POL_DOC(h_host_table, h_id, h_name, h_rec_user, h_error_id);
      End If;
      
      -- Execute Update Statement
      BEGIN
         UPDATE TMT_POOLS
         SET ELEMENT_NAME = RTRIM(LTRIM(p_name)),
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_NAME', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_NAME(95) : pools.sql(2722) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_NAME', 101+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_NAME(102) : pools.sql(2729) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_NAME', 108+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_NAME(109) : pools.sql(2736) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_NAME', 120+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_NAME(121) : pools.sql(2748) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_NAME;
ALTER FUNCTION SP_U_POOL_NAME COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_DESC ==
CREATE OR REPLACE FUNCTION SP_U_POOL_DESC
(
     p_id VARCHAR2
   , p_label VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_DESC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_DESC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_DESC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_label || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_DESC', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_DESC(23) : pools.sql(2787) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_DESC', 30+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_DESC(31) : pools.sql(2795) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_DESC', 33+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_DESC(34) : pools.sql(2798) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_u_pool_desc->p_id);
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_POOL_DESC', 41+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_POOL_DESC(42) : pools.sql(2806) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS SET
      LABEL        = p_label,
      DESCRIPTION  = p_description,
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER     = p_rec_user
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_DESC', 70+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_DESC(71) : pools.sql(2835) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_DESC', 76+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_DESC(77) : pools.sql(2841) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_DESC', 83+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_DESC(84) : pools.sql(2848) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_DESC', 95+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_DESC(96) : pools.sql(2860) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_DESC;
ALTER FUNCTION SP_U_POOL_DESC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_PUBLIC ==
CREATE OR REPLACE FUNCTION SP_U_POOL_PUBLIC
(
     p_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_mask_employee_info PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_PUBLIC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_PUBLIC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_PUBLIC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || COALESCE(TO_CHAR(p_mask_employee_info), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_PUBLIC', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_PUBLIC(21) : pools.sql(2897) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_PUBLIC', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_PUBLIC(29) : pools.sql(2905) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_PUBLIC', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_PUBLIC(32) : pools.sql(2908) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      UPDATE TMT_POOLS SET
      MASK_EMPLOYEE_INFO = p_mask_employee_info
      WHERE PARENT_ID = p_id
      AND TYPE_ID = PMO_CNST.SC_RESOURCE;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_PUBLIC', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_PUBLIC(52) : pools.sql(2928) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS SET
      PUBLIC_FLAG = p_del_flag,
      MASK_EMPLOYEE_INFO = p_mask_employee_info,
      REC_USER = p_rec_user,
      REC_DATETIME = SYSTIMESTAMP
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_PUBLIC', 74+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_PUBLIC(75) : pools.sql(2951) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_PUBLIC', 80+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_PUBLIC(81) : pools.sql(2957) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_PUBLIC', 87+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_PUBLIC(88) : pools.sql(2964) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_PUBLIC', 99+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_PUBLIC(100) : pools.sql(2976) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_PUBLIC;
ALTER FUNCTION SP_U_POOL_PUBLIC COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_CIE ==
CREATE OR REPLACE FUNCTION SP_U_POOL_CIE
(
     p_id VARCHAR2
   , p_domain_id VARCHAR2
   , p_company_name VARCHAR2
   , p_address VARCHAR2
   , p_phone VARCHAR2
   , p_fax VARCHAR2
   , p_website VARCHAR2
   , p_other_location VARCHAR2
   , p_year_in_business PLS_INTEGER
   , p_client_more_100_employees PLS_INTEGER
   , p_client_more_1000_employees PLS_INTEGER
   , p_client_more_10000_employees PLS_INTEGER
   , p_number_employees PLS_INTEGER
   , p_number_pools PLS_INTEGER
   , p_number_clients PLS_INTEGER
   , p_contact_name VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_CIE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_CIE'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_CIE(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_domain_id || ''''
          || ', ' || '''' || p_company_name || ''''
          || ', ' || '''' || p_address || ''''
          || ', ' || '''' || p_phone || ''''
          || ', ' || '''' || p_fax || ''''
          || ', ' || '''' || p_website || ''''
          || ', ' || '''' || p_other_location || ''''
          || ', ' || COALESCE(TO_CHAR(p_year_in_business), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_client_more_100_employees), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_client_more_1000_employees), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_client_more_10000_employees), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_number_employees), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_number_pools), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_number_clients), 'NULL')
          || ', ' || '''' || p_contact_name || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_CIE', 48+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_CIE(49) : pools.sql(3041) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_CIE', 56+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_CIE(57) : pools.sql(3049) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_CIE', 59+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_CIE(60) : pools.sql(3052) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_u_pool_cie->p_id);
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_POOL_CIE', 67+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_POOL_CIE(68) : pools.sql(3060) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS SET
      DOMAIN_ID = p_domain_id,
      COMPANY_NAME = p_company_name,
      ADDRESS = p_address,
      PHONE = p_phone,
      FAX = p_fax,
      WEBSITE = p_website,
      OTHER_LOCATION = p_other_location,
      YEAR_IN_BUSINESS = p_year_in_business,
      CLIENT_MORE_100_EMPLOYEES = p_client_more_100_employees,
      CLIENT_MORE_1000_EMPLOYEES = p_client_more_1000_employees,
      CLIENT_MORE_10000_EMPLOYEES = p_client_more_10000_employees,
      NUMBER_EMPLOYEES = p_number_employees,
      NUMBER_POOLS = p_number_pools,
      NUMBER_CLIENTS = p_number_clients,
      CONTACT_NAME = p_contact_name,
      REC_USER = p_rec_user,
      REC_DATETIME = SYSTIMESTAMP
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_CIE', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_CIE(110) : pools.sql(3102) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_CIE', 115+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_CIE(116) : pools.sql(3108) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_CIE', 122+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_CIE(123) : pools.sql(3115) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_CIE', 134+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_CIE(135) : pools.sql(3127) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_CIE;
ALTER FUNCTION SP_U_POOL_CIE COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_BKG ==
CREATE OR REPLACE FUNCTION SP_U_POOL_BKG
(
     p_id VARCHAR2
   , p_annual_revenues NUMBER
   , p_billing_address VARCHAR2
   , p_billing_phone VARCHAR2
   , p_billing_fax VARCHAR2
   , p_billing_email VARCHAR2
   , p_equal_opportunity VARCHAR2
   , p_company_background VARCHAR2
   , p_ownership VARCHAR2
   , p_services_offered VARCHAR2
   , p_services_specialized VARCHAR2
   , p_industries_served VARCHAR2
   , p_highlight VARCHAR2
   , p_location_served VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_BKG') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_BKG'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_BKG(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_annual_revenues), 'NULL')
          || ', ' || '''' || p_billing_address || ''''
          || ', ' || '''' || p_billing_phone || ''''
          || ', ' || '''' || p_billing_fax || ''''
          || ', ' || '''' || p_billing_email || ''''
          || ', ' || '''' || p_equal_opportunity || ''''
          || ', ' || '''' || p_company_background || ''''
          || ', ' || '''' || p_ownership || ''''
          || ', ' || '''' || p_services_offered || ''''
          || ', ' || '''' || p_services_specialized || ''''
          || ', ' || '''' || p_industries_served || ''''
          || ', ' || '''' || p_highlight || ''''
          || ', ' || '''' || p_location_served || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_BKG', 44+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_BKG(45) : pools.sql(3188) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_BKG', 52+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_BKG(53) : pools.sql(3196) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_BKG', 55+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_BKG(56) : pools.sql(3199) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_u_pool_bkg->p_id);
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_POOL_BKG', 63+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_POOL_BKG(64) : pools.sql(3207) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS SET
      ANNUAL_REVENUES = p_annual_revenues,
      BILLING_ADDRESS = p_billing_address,
      BILLING_PHONE = p_billing_phone,
      BILLING_FAX = p_billing_fax,
      BILLING_EMAIL = p_billing_email,
      EQUAL_OPPORTUNITY = p_equal_opportunity,
      COMPANY_BACKGROUND = p_company_background,
      OWNERSHIP = p_ownership,
      SERVICES_OFFERED = p_services_offered,
      SERVICES_SPECIALIZED = p_services_specialized,
      INDUSTRIES_SERVED = p_industries_served,
      HIGHLIGHT = p_highlight,
      LOCATION_SERVED = p_location_served,
      REC_USER = p_rec_user,
      REC_DATETIME = SYSTIMESTAMP
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_BKG', 103+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_BKG(104) : pools.sql(3247) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_BKG', 109+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_BKG(110) : pools.sql(3253) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_BKG', 116+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_BKG(117) : pools.sql(3260) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_BKG', 128+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_BKG(129) : pools.sql(3272) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_BKG;
ALTER FUNCTION SP_U_POOL_BKG COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION SP_U_POOL_MGR ==
CREATE OR REPLACE FUNCTION SP_U_POOL_MGR
(
     p_id VARCHAR2
   , p_manager_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_manager_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_MGR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_MGR'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_MGR(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_manager_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_MGR', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_MGR(22) : pools.sql(3310) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   --InitializeHost(SC_PARAM(h_id), p_id, 0);
   h_id := p_id;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_POOL_MGR', 31+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_POOL_MGR(32) : pools.sql(3320) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Validate id
   --InitializeHost(SC_PARAM(h_manager_id), p_manager_id, 0);
   h_manager_id := p_manager_id;
   BEGIN 
      SELECT REC_STATUS INTO h_rec_status FROM RESOURCES WHERE RESOURCE_ID = h_manager_id; 
      IF SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') THEN 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True); 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_manager_id, True); 
   END;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS SET
      MANAGER_ID = p_manager_id,
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER = p_rec_user
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_MGR', 72+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_MGR(73) : pools.sql(3361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_MGR', 78+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_MGR(79) : pools.sql(3367) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_MGR', 85+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_MGR(86) : pools.sql(3374) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_MGR', 97+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_MGR(98) : pools.sql(3386) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_MGR;
ALTER FUNCTION SP_U_POOL_MGR COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION SP_U_POOL_COST_CENTER ==
CREATE OR REPLACE FUNCTION SP_U_POOL_COST_CENTER
(
     p_id VARCHAR2
   , p_cost_center_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_cost_center_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_COST_CENTER') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_COST_CENTER'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_COST_CENTER(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_cost_center_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_COST_CENTER', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_COST_CENTER(22) : pools.sql(3424) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   h_id := p_id;
   h_cost_center_id := p_cost_center_id;
   h_rec_user := p_rec_user;
   
   If ((h_id IS NULL)) OR ((h_cost_center_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_COST_CENTER', 33+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_COST_CENTER(34) : pools.sql(3436) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_COST_CENTER', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_COST_CENTER(37) : pools.sql(3439) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Execute Update Statement
   BEGIN
      UPDATE TMT_POOLS
      SET COST_CENTER_ID = h_cost_center_id,
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER = h_rec_user
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_COST_CENTER', 58+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_COST_CENTER(59) : pools.sql(3461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_COST_CENTER', 68+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_COST_CENTER(69) : pools.sql(3471) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_COST_CENTER', 75+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_COST_CENTER(76) : pools.sql(3478) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_COST_CENTER', 87+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_COST_CENTER(88) : pools.sql(3490) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_COST_CENTER;
ALTER FUNCTION SP_U_POOL_COST_CENTER COMPILE DEBUG;

/****************************************************************************************
August 03, 2004:

  when user changes default security group template in the system administration view
  then we update all the pools that assigned to this deault security group, but the resources
  under that pool still keep their security. if user creates a new resource under a pool
  then new security settings will be applied to the new resource.

****************************************************************************************/

Prompt == FUNCTION SP_U_UG_SECURITY ==
CREATE OR REPLACE FUNCTION SP_U_UG_SECURITY
(
     p_id VARCHAR2
   , p_int_security_1 PLS_INTEGER
   , p_int_security_2 PLS_INTEGER
   , p_int_security_3 PLS_INTEGER
   , p_int_security_4 PLS_INTEGER
   , p_int_security_5 PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_status VARCHAR2(1);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_clearance_level_1 PLS_INTEGER := 0;
   h_clearance_level_2 PLS_INTEGER := 0;
   h_clearance_level_3 PLS_INTEGER := 0;
   h_clearance_level_4 PLS_INTEGER := 0;
   h_clearance_level_5 PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_UG_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_UG_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_UG_SECURITY(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_int_security_1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_int_security_2), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_int_security_3), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_int_security_4), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_int_security_5), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_SECURITY', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_UG_SECURITY(35) : pools.sql(3549) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_SECURITY', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_SECURITY(40) : pools.sql(3554) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_SECURITY', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_SECURITY(43) : pools.sql(3557) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   h_id := p_id;
   h_rec_user := p_rec_user;
   h_clearance_level_1 := p_int_security_1;
   h_clearance_level_2 := p_int_security_2;
   h_clearance_level_3 := p_int_security_3;
   h_clearance_level_4 := p_int_security_4;
   h_clearance_level_5 := p_int_security_5;
   
   BEGIN
      SELECT REC_STATUS
      INTO h_rec_status
      FROM TMT_POOLS
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_SECURITY', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_SECURITY(67) : pools.sql(3581) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_SECURITY', 73+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_SECURITY(74) : pools.sql(3588) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_SECURITY', 76+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_SECURITY(77) : pools.sql(3591) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_FOUND, h_id, True);
   End If;
   If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_SECURITY', 82+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_SECURITY(83) : pools.sql(3597) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_DELETED), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_SECURITY', 85+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_SECURITY(86) : pools.sql(3600) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.MARKED_AS_DELETED, h_id, True);
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      -- Update all the pools that have been assigned to the default security group
      BEGIN
         UPDATE TMT_POOLS
         SET CLEARANCE_LEVEL_1 = h_clearance_level_1,
         CLEARANCE_LEVEL_2 = h_clearance_level_2,
         CLEARANCE_LEVEL_3 = h_clearance_level_3,
         CLEARANCE_LEVEL_4 = h_clearance_level_4,
         CLEARANCE_LEVEL_5 = h_clearance_level_5,
         REC_USER = h_rec_user,
         REC_DATETIME = SYSTIMESTAMP
         WHERE USER_GROUP_ID = h_id
         AND TYPE_ID = PMO_CNST.SC_POOLS;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_SECURITY', 113+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_SECURITY(114) : pools.sql(3628) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_POOLS
         SET CLEARANCE_LEVEL_1 = h_clearance_level_1,
         CLEARANCE_LEVEL_2 = h_clearance_level_2,
         CLEARANCE_LEVEL_3 = h_clearance_level_3,
         CLEARANCE_LEVEL_4 = h_clearance_level_4,
         CLEARANCE_LEVEL_5 = h_clearance_level_5,
         REC_USER = h_rec_user,
         REC_DATETIME = SYSTIMESTAMP
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_SECURITY', 138+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_SECURITY(139) : pools.sql(3653) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_UG_SECURITY', 145+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_UG_SECURITY(146) : pools.sql(3660) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_SECURITY', 152+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_SECURITY(153) : pools.sql(3667) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_SECURITY', 164+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_SECURITY(165) : pools.sql(3679) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_UG_SECURITY;
ALTER FUNCTION SP_U_UG_SECURITY COMPILE DEBUG;

/****************************************************************************************
August 03, 2004:

    user can change default security at pool and resource level.
    in this case user and pool inherit new security values from new default security group.
    it means if pool's  default security group changes then just the pool (not the resources under it)
    will be updated.
*****************************************************************************************/

Prompt == FUNCTION SP_U_UG_ID ==
CREATE OR REPLACE FUNCTION SP_U_UG_ID
(
     p_id VARCHAR2
   , p_user_group_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_status VARCHAR2(1);
   h_id VARCHAR2(32);
   h_user_group_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_UG_ID') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_UG_ID'), '-- ' || SYSTIMESTAMP, ':r := SP_U_UG_ID(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_user_group_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_ID', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_UG_ID(23) : pools.sql(3725) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_ID', 27+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_ID(28) : pools.sql(3730) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_ID', 30+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_ID(31) : pools.sql(3733) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   h_id := p_id;
   h_user_group_id := p_user_group_id;
   
   BEGIN
      SELECT TYPE_ID, REC_STATUS
      INTO h_type_id, h_rec_status
      FROM TMT_POOLS
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_ID', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_ID(50) : pools.sql(3752) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_ID', 56+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_ID(57) : pools.sql(3759) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_ID', 59+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_ID(60) : pools.sql(3762) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_FOUND, h_id, True);
   End If;
   If SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_UG_ID', 65+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_UG_ID(66) : pools.sql(3768) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_DELETED), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_UG_ID', 68+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_UG_ID(69) : pools.sql(3771) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.MARKED_AS_DELETED, h_id, True);
   End If;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
         BEGIN
            UPDATE RESOURCES SET
            (USER_GROUP_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5) = (
            SELECT ELEMENT_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5
            FROM TMT_POOLS
            WHERE ELEMENT_ID = h_user_group_id)
            WHERE RESOURCE_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_ID', 92+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_ID(93) : pools.sql(3795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE TMT_POOLS SET
         (USER_GROUP_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5) = (
         SELECT ELEMENT_ID, CLEARANCE_LEVEL_1, CLEARANCE_LEVEL_2, CLEARANCE_LEVEL_3, CLEARANCE_LEVEL_4, CLEARANCE_LEVEL_5
         FROM TMT_POOLS
         WHERE ELEMENT_ID = h_user_group_id)
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_ID', 115+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_ID(116) : pools.sql(3818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_UG_ID', 122+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_UG_ID(123) : pools.sql(3825) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_ID', 129+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_ID(130) : pools.sql(3832) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_UG_ID', 141+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_UG_ID(142) : pools.sql(3844) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_UG_ID;
ALTER FUNCTION SP_U_UG_ID COMPILE DEBUG;

/*************************************************************************************/

--  This sp can be deleted!!!!

/*************************************************************************************/

Prompt == FUNCTION SP_DF_POOL ==
CREATE OR REPLACE FUNCTION SP_DF_POOL
(
     p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_DF_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_DF_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_DF_POOL(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_DF_POOL', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_DF_POOL(19) : pools.sql(3883) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   PMO_DB2M.SC_STATE := 0;
   -- Validate id
   h_id := p_id;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_DF_POOL', 28+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_DF_POOL(29) : pools.sql(3893) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_DF_POOL', 35+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_DF_POOL(36) : pools.sql(3900) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_DF_POOL', 47+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_DF_POOL(48) : pools.sql(3912) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_DF_POOL;
ALTER FUNCTION SP_DF_POOL COMPILE DEBUG;

/***************************************************************************************************
Rules for deleting Pools and Resources:
    if exists(resource assignments)
        cannot delete pools
        cannot delete resource
    else
        do delete pools and resources
***************************************************************************************************/

Prompt == FUNCTION SP_D_POOL ==
CREATE OR REPLACE FUNCTION SP_D_POOL
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_del_flag VARCHAR2(1);
   h_element_id VARCHAR2(32);
   h_error_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_rec_user VARCHAR2(32);
   h_temp_rank VARCHAR2(200);
   h_host_table PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 11;
   h_parent_id VARCHAR2(32);
   h_pool_id VARCHAR2(32);
   h_delete_resource PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_DEL IS
         SELECT ELEMENT_ID, TYPE_ID
         FROM TMT_POOLS
         WHERE RANK LIKE h_rank || '%';

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_D_POOL(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_POOL(40) : pools.sql(3975) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_POOL', 44+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_POOL(45) : pools.sql(3980) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(48) : pools.sql(3983) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local Variables
   
   h_pool_id := p_id;
   h_parent_id := p_parent_id;
   h_rec_user := p_rec_user;
   h_del_flag := p_del_flag;
   
   h_delete_resource := 0;
   
   BEGIN
      SELECT TYPE_ID, RANK, REC_STATUS, PARENT_ID
      INTO h_type_id, h_rank, h_rec_status, h_parent_id
      FROM TMT_POOLS
      WHERE ELEMENT_ID = h_pool_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 72+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(73) : pools.sql(4008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("\n h_parent_id:%s:%d", h_parent_id, h_parent_id_Ind);
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_POOL', 81+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_POOL(82) : pools.sql(4017) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_pool_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 84+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(85) : pools.sql(4020) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_pool_id, True);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      h_delete_resource := 1;
   End If;
   
   --if(IsNullParam(h_parent_id))
   --{
   --  BEGIN_SQL
   --      SELECT RANK, REC_STATUS
   --      INTO HOST(h_rank), HOST(h_rec_status)
   --      FROM TMT_POOLS
   --      WHERE ELEMENT_ID = HOST(h_pool_id)
   --          AND PARENT_ID IS NuLL;
   --  END_SQL;
   --}
   --else
   --{
   --  BEGIN_SQL
   --      SELECT RANK, REC_STATUS
   --      INTO HOST(h_rank), HOST(h_rec_status)
   --      FROM TMT_POOLS
   --      WHERE ELEMENT_ID = HOST(h_pool_id)
   --          AND PARENT_ID = HOST(h_parent_id);
   --  END_SQL;
   --}
   
   --Check for Actual
   BEGIN
      SELECT COUNT(TSK_ASSIGNMENT_ID)
      INTO h_count
      FROM TMT_POOLS TP
      JOIN TASK_ASSIGNMENTS TA ON TA.RESOURCE_ID = TP.ELEMENT_ID
      AND TA.WORK_TODATE > 0
      WHERE TP.RANK LIKE h_rank || '%'
      AND TP.TYPE_ID = PMO_CNST.SC_RESOURCE;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 129+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(130) : pools.sql(4065) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_POOL', 136+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_POOL(137) : pools.sql(4072) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE), 'NULL') || ' ID: ' || '''' || h_pool_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 139+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(140) : pools.sql(4075) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE, h_pool_id, True);
   End If;
   
   --Check Assignments
   BEGIN
      SELECT COUNT(RES_ASSIGNMENT_ID)
      INTO h_count
      FROM TMT_POOLS TP
      JOIN RESOURCE_ASSIGNMENTS RA ON RA.RESOURCE_ID = TP.ELEMENT_ID
      WHERE TP.RANK LIKE h_rank || '%'
      AND TP.TYPE_ID = PMO_CNST.SC_RESOURCE
      AND RA.PROJECT_ID NOT IN (
      SELECT ELEMENT_ID
      FROM TMT_WBS
      WHERE POOL_ID = TP.ELEMENT_ID
      AND TYPE_ID = PMO_CNST.SC_RESOURCE);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 163+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(164) : pools.sql(4099) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_POOL', 170+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_POOL(171) : pools.sql(4106) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE), 'NULL') || ' ID: ' || '''' || h_pool_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 173+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(174) : pools.sql(4109) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE, h_pool_id, True);
   End If;
   
   If SubStr(p_del_flag, 1, 1) = '?' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_POOL', 180+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_POOL(181) : pools.sql(4116) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || h_pool_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 183+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(184) : pools.sql(4119) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(0, h_pool_id, True);
   End If;
   
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      
      OPEN C_DEL;
      
      LOOP
         FETCH C_DEL INTO h_element_id, h_type_id;
         IF C_DEL%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            EXIT;
         End If;
         
         If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
            BEGIN
               DELETE FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 231+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(232) : pools.sql(4167) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_CALENDARS SET
               CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (
               SELECT PARENT_ID FROM TMT_CALENDARS
               WHERE ELEMENT_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 252+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(253) : pools.sql(4188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_CALENDARS
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 270+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(271) : pools.sql(4206) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_PORTFOLIOS
               SET CHILD_COUNT = CHILD_COUNT - (
               SELECT COUNT(*)
               FROM TMT_PORTFOLIOS
               WHERE PARENT_ID = h_element_id)
               WHERE ELEMENT_ID IS NULL;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 292+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(293) : pools.sql(4228) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT RTRIM(RANK)
               INTO h_temp_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 308+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(309) : pools.sql(4244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_temp_rank IS NOT NULL) Then
               DECLARE 
               	nLen pls_integer :=0; 
               BEGIN 
               	If (h_temp_rank IS NOT NULL) Then 
               		nLen := Coalesce(Length(h_temp_rank), 0); 
               		If 0 <> 0 AND nLen <> 0 Then 
               		nLen := nLen - 1; 
               		End If; 
               	End If; 
               	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
               		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_temp_rank, True); 
               	End If; 
               END;
               
               BEGIN
                  DELETE FROM TMT_PORTFOLIOS
                  WHERE RANK LIKE h_temp_rank || '%';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 341+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(342) : pools.sql(4277) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM RATES
               WHERE RATE_ID IN (SELECT RATE_ID FROM RESOURCES WHERE RESOURCE_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 360+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(361) : pools.sql(4296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM HISTORY_RESOURCES
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 378+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(379) : pools.sql(4314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_EDUCATION
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 396+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(397) : pools.sql(4332) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_EMPLOYMENT_HISTORY
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 414+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(415) : pools.sql(4350) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_INDUSTRY_EXPERIENCE
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 432+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(433) : pools.sql(4368) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_PROJECT_HISTORY
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 450+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(451) : pools.sql(4386) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_READINESS
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 468+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(469) : pools.sql(4404) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_COMPETENCIES
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (
               SELECT PARENT_ID
               FROM TMT_COMPETENCIES
               WHERE RESOURCE_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 490+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(491) : pools.sql(4426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_COMPETENCIES
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 508+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(509) : pools.sql(4444) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_host_table := PMO_CNST.SC_TABLE_RES;
            h_id := NULL;
            PMO_DB2M.SC_STATE := FC_D_DOC_RES_DOC(h_host_table, h_id, h_element_id, h_del_flag, h_rec_user
                                      , h_error_id);
            
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               BEGIN
                  CLOSE C_DEL ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 527+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(528) : pools.sql(4463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_POOL', 533+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_POOL(534) : pools.sql(4469) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 536+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(537) : pools.sql(4472) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_DB2M.SC_STATE, True);
            End If;
            
            BEGIN
               UPDATE TMT_GEOGRAPHIES
               SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (
               SELECT PARENT_ID
               FROM TMT_GEOGRAPHIES
               WHERE ELEMENT_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 558+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(559) : pools.sql(4494) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_GEOGRAPHIES
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 576+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(577) : pools.sql(4512) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_ORGANIZATIONS SET
               CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (
               SELECT PARENT_ID
               FROM TMT_ORGANIZATIONS
               WHERE ELEMENT_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 598+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(599) : pools.sql(4534) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_ORGANIZATIONS
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 616+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(617) : pools.sql(4552) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_CONTACTS SET
               CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (
               SELECT PARENT_ID FROM TMT_CONTACTS
               WHERE ELEMENT_ID = h_element_id)
               AND RANK LIKE '0000' || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 638+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(639) : pools.sql(4574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_CONTACTS
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 656+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(657) : pools.sql(4592) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 678+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(679) : pools.sql(4614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_ASSIGNMENTS
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 700+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(701) : pools.sql(4636) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 722+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(723) : pools.sql(4658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) );
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 744+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(745) : pools.sql(4680) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_WBS
               WHERE POOL_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 762+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(763) : pools.sql(4698) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE RESOURCES
               SET CALENDAR_ID = h_element_id,
               LOGON_NAME = RESOURCE_ID,
               ACTIVE = '0',
               REC_STATUS = 'D'
               WHERE RESOURCE_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 784+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(785) : pools.sql(4720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_host_table := PMO_CNST.SC_TABLE_POL;
            h_id := NULL;
            PMO_DB2M.SC_STATE := FC_D_DOC_POL_DOC(h_host_table, h_id, h_element_id, h_del_flag, h_rec_user
                                      , h_error_id);
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS
               WHERE PROJECT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 807+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(808) : pools.sql(4743) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_POOL_ATTRIBUTES SET
               CHILD_COUNT = CHILD_COUNT - 1
               WHERE ELEMENT_ID IN (SELECT PARENT_ID FROM TMT_POOL_ATTRIBUTES
               WHERE ELEMENT_ID = h_element_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 827+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(828) : pools.sql(4763) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_POOL_ATTRIBUTES
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 845+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(846) : pools.sql(4781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM HISTORY_POOLS
               WHERE ELEMENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 863+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(864) : pools.sql(4799) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM POOL_REFERENCES
               WHERE POOL_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 881+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(882) : pools.sql(4817) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID = PMO_CNST.SC_POOLS);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 903+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(904) : pools.sql(4839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS
               WHERE PROJECT_ID IN (
               SELECT ELEMENT_ID
               FROM TMT_WBS
               WHERE POOL_ID = h_element_id
               AND TYPE_ID = PMO_CNST.SC_POOLS);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 925+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(926) : pools.sql(4861) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TMT_WBS
               WHERE POOL_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 943+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(944) : pools.sql(4879) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_DEL ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 958+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(959) : pools.sql(4894) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TMT_POOLS WHERE RANK LIKE h_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 975+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(976) : pools.sql(4911) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (h_parent_id IS NULL) Then
         BEGIN
            UPDATE TMT_POOLS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 995+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(996) : pools.sql(4931) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_POOLS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1014+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1015) : pools.sql(4950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_POOL', 1022+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_POOL(1023) : pools.sql(4958) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PARENT_MARKED_AS_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_POOL', 1025+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_POOL(1026) : pools.sql(4961) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True);
      End If;
      
      If (h_parent_id IS NULL) Then
         BEGIN
            UPDATE TMT_POOLS
            SET CHILD_COUNT = CHILD_COUNT - 1,
            DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1046+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1047) : pools.sql(4982) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_POOLS
            SET CHILD_COUNT = CHILD_COUNT - 1,
            DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1066+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1067) : pools.sql(5002) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      
      BEGIN
         UPDATE TMT_POOLS SET
         REC_STATUS   = p_del_flag,
         REC_DATETIME = SYSTIMESTAMP,
         REC_USER     = p_rec_user
         WHERE RANK LIKE h_rank || '%'
         AND REC_STATUS IN ( 'I', 'O', 'U');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1103+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1104) : pools.sql(5039) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_POOL', 1110+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_POOL(1111) : pools.sql(5046) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_pool_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_pool_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1117+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1118) : pools.sql(5053) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_POOL', 1129+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_POOL(1130) : pools.sql(5065) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_POOL;
ALTER FUNCTION SP_D_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UN_POOL ==
CREATE OR REPLACE FUNCTION SP_UN_POOL
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_current_datetime TIMESTAMP;
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UN_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UN_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_UN_POOL(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UN_POOL', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UN_POOL(24) : pools.sql(5105) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate id
   --InitializeHost(SC_PARAM(h_id), p_id, ind_un_pool->p_id);
   h_id := p_id;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_UN_POOL', 33+1) THEN 
      LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_UN_POOL(34) : pools.sql(5115) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Validate parent id
   --InitializeHost(SC_PARAM(h_parent_id), p_parent_id, ind_un_pool->p_parent_id);
   h_parent_id := p_parent_id;
   
   If ((p_parent_id IS NULL)) Then
      -- Find Necessary Info
      BEGIN
         SELECT RANK, REC_DATETIME INTO h_rank, h_current_datetime FROM TMT_POOLS
         WHERE ELEMENT_ID = p_id AND PARENT_ID IS NuLL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 59+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(60) : pools.sql(5141) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update Parent's Hierarchy Info
      BEGIN
         UPDATE TMT_POOLS SET CHILD_COUNT = CHILD_COUNT + 1, DELETED_COUNT = DELETED_COUNT - 1
         WHERE ELEMENT_ID IS NuLL;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 78+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(79) : pools.sql(5160) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      --InitializeHost(SC_PARAM(h_parent_id), p_parent_id, ind_un_pool->p_parent_id);
      h_parent_id := p_parent_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_UN_POOL', 86+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_UN_POOL(87) : pools.sql(5168) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_parent_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_parent_id, True); 
      END;
      
      -- Find Necessary Info
      BEGIN
         SELECT RANK, REC_DATETIME INTO h_rank, h_current_datetime FROM TMT_POOLS
         WHERE ELEMENT_ID = p_id AND PARENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 107+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(108) : pools.sql(5189) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Update Parent's Hierarchy Info
      BEGIN
         UPDATE TMT_POOLS SET CHILD_COUNT = CHILD_COUNT + 1, DELETED_COUNT = DELETED_COUNT - 1
         WHERE ELEMENT_ID = p_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 126+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(127) : pools.sql(5208) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   -- Lock Tree
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
   	End If; 
   END;
   BEGIN
      UPDATE TMT_POOLS SET
      REC_STATUS   = 'U',
      REC_DATETIME = SYSTIMESTAMP,
      REC_USER     = p_rec_user
      WHERE REC_DATETIME = h_current_datetime AND RANK LIKE h_rank || '%';
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 162+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(163) : pools.sql(5244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UN_POOL', 168+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UN_POOL(169) : pools.sql(5250) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 175+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(176) : pools.sql(5257) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UN_POOL', 187+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UN_POOL(188) : pools.sql(5269) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UN_POOL;
ALTER FUNCTION SP_UN_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_POOL_CPLAN ==
CREATE OR REPLACE FUNCTION SP_S_POOL_CPLAN
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_level_id PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      SCP661a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661g PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661b PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      resultset001 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661bb PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_POOL_CPLAN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_POOL_CPLAN'), '-- ' || SYSTIMESTAMP, ':r := SP_S_POOL_CPLAN(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 37+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_POOL_CPLAN(38) : pools.sql(5323) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate rec_user id
   If (p_rec_user IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_POOL_CPLAN', 45+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_POOL_CPLAN(46) : pools.sql(5331) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 48+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(49) : pools.sql(5334) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((p_id IS NULL)) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_POOLS_0001';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_POOLS_0001
         SELECT RANK, ELEMENT_ID
         FROM TMT_POOLS
         WHERE PUBLIC_FLAG = '1'
         UNION ALL
         SELECT TP.RANK, TP.ELEMENT_ID
         FROM TMT_POOLS TP
         JOIN RESOURCE_ASSIGNMENTS RA
         ON TP.ELEMENT_ID = RA.ELEMENT_ID
         AND RA.ELEMENT_TYPE = 11
         AND RA.RESOURCE_ID = p_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_CPLAN', 80+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_CPLAN(81) : pools.sql(5366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_POOLS_0002';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_POOLS_0002
         SELECT O.ELEMENT_ID
         FROM PMO_TMP_POOLS_0001 O
         WHERE NOT EXISTS (
         SELECT B.ELEMENT_ID
         FROM PMO_TMP_POOLS_0001 A, PMO_TMP_POOLS_0001 B
         WHERE A.RANK < B.RANK
         AND ((A.RANK >= SUBSTR(B.RANK, 1, 8) AND A.RANK <= B.RANK) 
         AND SUBSTR(B.RANK,1, CASE WHEN LENGTH(A.RANK) > LENGTH(B.RANK) THEN 1 ELSE LENGTH(A.RANK) END ) = A.RANK)
         AND O.ELEMENT_ID = B.ELEMENT_ID);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_CPLAN', 110+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_CPLAN(111) : pools.sql(5396) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If Coalesce(p_bitflag, 0) = 1 Then
         --trace_printf("\n 1-SP_S_POOL_CPLAN   SCP661a");
         
         OPEN SCP661a FOR SELECT
         0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
         T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
         T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
         T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES,
         T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
         T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED,
         T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME,
         '' AS TITLE,
         ''  AS COMPETENCY,
         (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE
         FROM TMT_POOLS T
         JOIN PMO_TMP_POOLS_0002 ON PMO_TMP_POOLS_0002.ELEMENT_ID=T.ELEMENT_ID
         WHERE T.TYPE_ID NOT IN (0, 13, 307) AND T.REC_STATUS <> 'D'
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 136+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(137) : pools.sql(5422) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN SCP661a;
      ElsIf (Coalesce(p_bitflag, 0) = 0) OR (Coalesce(p_bitflag, 0) = 2) Then
         --trace_printf("\n 2-SP_S_POOL_CPLAN   SCP661g");
         
         OPEN SCP661g FOR SELECT
         0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
         T.CHILD_COUNT /*-   (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13)*/ AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
         T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
         T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES,
         T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
         T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED,
         T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME,
         '' AS TITLE,
         ''  AS COMPETENCY,
         (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE
         FROM TMT_POOLS T
         JOIN PMO_TMP_POOLS_0002 ON PMO_TMP_POOLS_0002.ELEMENT_ID=T.ELEMENT_ID
         WHERE T.TYPE_ID NOT IN (0, 13, 307) AND T.REC_STATUS <> 'D' AND LEVEL_ID = 0
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 161+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(162) : pools.sql(5447) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN SCP661g;
      End If;
   Else
      --InitializeHost(SC_PARAM(h_id), p_id, ind_s_pool_cplan->p_id);
      h_id := p_id;
      
      BEGIN
         SELECT RANK, LEVEL_ID INTO h_rank, h_level_id
         FROM TMT_POOLS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_CPLAN', 179+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_CPLAN(180) : pools.sql(5465) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_CPLAN', 186+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_CPLAN(187) : pools.sql(5472) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 189+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(190) : pools.sql(5475) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
      End If;
      
      If Coalesce(p_bitflag, 0) = 1 Then
         --trace_printf("\n 3-SP_S_POOL_CPLAN   SCP661b");
         
         OPEN SCP661b FOR SELECT
         T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
         T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
         T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
         T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES,
         T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
         T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED,
         T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME,
         '' AS TITLE, '' AS COMPETENCY,
         (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE
         FROM TMT_POOLS T
         WHERE T.TYPE_ID = PMO_CNST.SC_POOLS
         AND T.RANK LIKE h_rank || '%'
         AND T.LEVEL_ID < h_level_id + 2
         AND T.REC_STATUS <> 'D'
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 217+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(218) : pools.sql(5503) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN SCP661b;
      ElsIf Coalesce(p_bitflag, 0) = 2 Then
         --trace_printf("\n 4-SP_S_POOL_CPLAN   resultset001");
         
         OPEN resultset001 FOR SELECT
         T.LEVEL_ID, 11 AS TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, 1 AS CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
         T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
         T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES,
         T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
         T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED,
         T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME,
         '' AS TITLE,
         '' AS COMPETENCY,
         (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE
         FROM TMT_POOLS T
         WHERE T.RANK LIKE h_rank || '%'
         AND T.LEVEL_ID < h_level_id + 2
         AND T.REC_STATUS <> 'D'
         AND T.TYPE_ID  = PMO_CNST.SC_POOLS
         
         UNION
         
         SELECT
         T.LEVEL_ID + 1 AS LEVEL_ID, 13 AS TYPE_ID, T.RANK, T2.PARENT_ID, T2.ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME, 0 AS CHILD_COUNT,
         T2.DELETED_COUNT, T2.REC_STATUS, T2.DEFAULT_FLAG, T2.DESCRIPTION, T2.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T2.DOMAIN_ID) AS DOMAIN_NAME,
         T2.COMPANY_NAME, T2.ADDRESS, T2.PHONE, T2.FAX, T2.WEBSITE, T2.CONTACT_NAME, T2.OTHER_LOCATION, T2.YEAR_IN_BUSINESS,
         T2.CLIENT_MORE_100_EMPLOYEES, T2.CLIENT_MORE_1000_EMPLOYEES, T2.CLIENT_MORE_10000_EMPLOYEES, T2.NUMBER_EMPLOYEES,
         T2.NUMBER_POOLS, T2.NUMBER_CLIENTS, T2.ANNUAL_REVENUES, T2.BILLING_ADDRESS, T2.BILLING_PHONE, T2.BILLING_FAX,
         T2.BILLING_EMAIL, T2.EQUAL_OPPORTUNITY, T2.COMPANY_BACKGROUND, T2.OWNERSHIP, T2.SERVICES_OFFERED, T2.SERVICES_SPECIALIZED,
         T2.INDUSTRIES_SERVED, T2.HIGHLIGHT, T2.LOCATION_SERVED, T2.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T2.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T2.REC_USER) AS REC_USER_NAME,
         (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T2.ELEMENT_ID) AS TITLE,
         (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES
         WHERE RESOURCE_ID = T2.ELEMENT_ID AND
         PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES
         WHERE RESOURCE_ID = T2.ELEMENT_ID))))
         , ', '), (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES
         WHERE RESOURCE_ID = T2.ELEMENT_ID
         AND PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES
         WHERE RESOURCE_ID = T2.ELEMENT_ID)))))  AS COMPETENCY,
         '' AS RESPONSIBLE
         
         FROM TMT_POOLS T JOIN  RESOURCE_ASSIGNMENTS RA ON RA.ELEMENT_ID = T.ELEMENT_ID
         JOIN RESOURCES R ON R.RESOURCE_ID = RA.RESOURCE_ID
         JOIN TMT_POOLS T2 ON T2.ELEMENT_ID = RA.RESOURCE_ID
         WHERE T.RANK = h_rank
         ORDER BY LEVEL_ID, RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 274+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(275) : pools.sql(5560) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN resultset001;
      Else
         --trace_printf("\n 5-SP_S_POOL_CPLAN   SCP661bb");
         
         OPEN SCP661bb FOR SELECT
         T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.CHILD_COUNT,
         T.DELETED_COUNT, T.REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
         (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
         T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
         T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES,
         T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
         T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED,
         T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT,
         T.REC_USER, (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME,
         CASE WHEN T.TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE '' END AS TITLE,
         CASE WHEN T.TYPE_ID = 13 THEN
         (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES
         WHERE RESOURCE_ID = T.ELEMENT_ID AND
         PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES
         WHERE RESOURCE_ID = T.ELEMENT_ID))))
         , ', '), (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES
         WHERE RESOURCE_ID = T.ELEMENT_ID
         AND PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES
         WHERE RESOURCE_ID = T.ELEMENT_ID))))) ELSE '' END AS COMPETENCY,
         CASE WHEN T.TYPE_ID = 13 THEN '' ELSE (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) END AS RESPONSIBLE
         FROM TMT_POOLS T
         WHERE T.TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE)
         AND T.RANK LIKE h_rank || '%'
         AND T.LEVEL_ID < h_level_id + 2
         AND T.REC_STATUS <> 'D'
         AND (T.TYPE_ID = PMO_CNST.SC_POOLS OR EXISTS(SELECT R.RESOURCE_ID FROM RESOURCES R
         WHERE R.RESOURCE_ID = T.ELEMENT_ID AND R.CALENDAR_ID IS NOT NuLL
         AND R.ACTIVE <> '0'
         AND (R.DISCONTINUE_DATE IS NuLL OR R.DISCONTINUE_DATE >= TO_DATE(SYSDATE))))
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_CPLAN', 315+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_CPLAN(316) : pools.sql(5601) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN SCP661bb;
      End If;
   End If;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_CPLAN', 324+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_CPLAN(325) : pools.sql(5610) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_CPLAN', 336+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_CPLAN(337) : pools.sql(5622) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_POOL_CPLAN;
ALTER FUNCTION SP_S_POOL_CPLAN COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION FC_CLEANS_POOL_BY_NAME ==
CREATE OR REPLACE FUNCTION FC_CLEANS_POOL_BY_NAME
(
     sh_h_search_option PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_search_option PLS_INTEGER := sh_h_search_option;
   h_element_id VARCHAR2(32);
   h_element_name VARCHAR2(255);
   h_approved PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR all_pools_curs IS
         SELECT ELEMENT_ID, ELEMENT_NAME
         FROM PMO_TMP_POOL_RES_TREE;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CLEANS_POOL_BY_NAME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CLEANS_POOL_BY_NAME'), '-- ' || SYSTIMESTAMP, ':r := FC_CLEANS_POOL_BY_NAME(' || 
       COALESCE(TO_CHAR(sh_h_search_option), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_POOL_BY_NAME', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CLEANS_POOL_BY_NAME(22) : pools.sql(5660) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN all_pools_curs;
   
   LOOP
      h_element_name := NULL;
      
      FETCH all_pools_curs
      INTO h_element_id, h_element_name;
      IF all_pools_curs
      %FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_approved := 1;
      
      h_approved := 1;
      
      If FC_FIND_KEYWORD(h_element_name, h_search_option) = 0 Then
         h_approved := 0;
      End If;
      
      If Coalesce(h_approved, 0) = 0 Then
         BEGIN
            DELETE FROM PMO_TMP_POOL_RES_TREE
            WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_POOL_BY_NAME', 64+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_POOL_BY_NAME(65) : pools.sql(5703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE all_pools_curs;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_POOL_BY_NAME', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_POOL_BY_NAME(80) : pools.sql(5718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_POOL_BY_NAME', 86+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CLEANS_POOL_BY_NAME(87) : pools.sql(5725) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CLEANS_POOL_BY_NAME;
ALTER FUNCTION FC_CLEANS_POOL_BY_NAME COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION FC_BUILD_POOL_SEARCH_CONDITION ==
CREATE OR REPLACE FUNCTION FC_BUILD_POOL_SEARCH_CONDITION
(
     h_fieldName IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_search_by_name_condition VARCHAR2(8000);
   h_search_option PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_filter_string VARCHAR2(8000);
   h_filter_count PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_BUILD_POOL_SEARCH_CONDITION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_BUILD_POOL_SEARCH_CONDITION'), '-- ' || SYSTIMESTAMP, ':r := FC_BUILD_POOL_SEARCH_CONDITION(' || 
       '''' || h_fieldName || ''''
          || ', ' || '''' || h_name || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_POOL_SEARCH_CONDITION', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_BUILD_POOL_SEARCH_CONDITION(20) : pools.sql(5755) : ' || SYSTIMESTAMP);
   END IF; 

   h_search_by_name_condition := '';
   
   h_search_option := FC_PARSE_SEARCH_STR(h_name, h_fieldName, h_search_by_name_condition);
   
   If Coalesce(h_search_option, 0) = PMO_CNST.SC_FIND_ATTRIBUTE_NAME Then
      h_name := NULL;
      h_filter_string := NULL;
      h_filter_count := 0;
      
      h_temp_sc_state := FC_FIND_ATTR_BY_NAME(PMO_CNST.SC_TABLE_POL, h_search_by_name_condition
                                        , h_filter_string, h_filter_count);
      
      If Coalesce(h_filter_count, 0) = 0 Then
         h_filter_count := 1;
         h_filter_string := PMO_CNST.SC_EMPTY_ATTRIBUTE_ID;
      End If;
   Else
      If Coalesce(Length(h_search_by_name_condition), 0) = 0 Then
         h_name := NULL;
      Else
         h_name := h_search_by_name_condition;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_POOL_SEARCH_CONDITION', 47+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_BUILD_POOL_SEARCH_CONDITION(48) : pools.sql(5783) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_search_option, 0);
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_BUILD_POOL_SEARCH_CONDITION', 53+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_BUILD_POOL_SEARCH_CONDITION(54) : pools.sql(5789) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_BUILD_POOL_SEARCH_CONDITION;
ALTER FUNCTION FC_BUILD_POOL_SEARCH_CONDITION COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_POOL ==
CREATE OR REPLACE FUNCTION SP_S_POOL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_default_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_xsql VARCHAR2(8000);
   h_is_static PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      CR0001 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_S_POOL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_POOL(26) : pools.sql(5825) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 4) Then
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_POOL', 33+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_POOL(34) : pools.sql(5833) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
   ElsIf ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 3) Then
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_POOL', 46+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_POOL(47) : pools.sql(5846) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      h_is_static := 1;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_POOLS_0003';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_POOLS_0003
         SELECT H.ELEMENT_ID, H.REVISION_TYPE, H.REVISION_IMPACT
         FROM HISTORY_POOLS H
         WHERE H.ELEMENT_ID = h_id AND
         H.REC_DATETIME = (
         SELECT MAX(REC_DATETIME)
         FROM HISTORY_POOLS
         WHERE ELEMENT_ID = H.ELEMENT_ID);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL', 82+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL(83) : pools.sql(5882) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN CR0001 FOR SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID,
      CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)
      ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, T.CHILD_COUNT, T.DELETED_COUNT, CASE WHEN T.TYPE_ID = 13
      THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)
      ELSE T.REC_STATUS END AS REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, T.DOMAIN_ID,
      (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
      T.USER_GROUP_ID, COALESCE((SELECT ELEMENT_NAME FROM TMT_POOLS WHERE ELEMENT_ID = T.USER_GROUP_ID),'') AS USER_GROUP_NAME,
      T.CLEARANCE_LEVEL_1, T.CLEARANCE_LEVEL_2, T.CLEARANCE_LEVEL_3, T.CLEARANCE_LEVEL_4, T.CLEARANCE_LEVEL_5,
      T.COMPANY_NAME,
      T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS,
      T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES,
      T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES, T.NUMBER_POOLS,
      T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX,
      T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED,
      T.SERVICES_SPECIALIZED, T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL,
      '' AS REVISION_HISTORY, H.REVISION_TYPE, H.REVISION_IMPACT,
      T.COST_CENTER_ID, C.ELEMENT_NAME AS COST_CENTER_NAME,
      CASE WHEN T.TYPE_ID = 13 THEN (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)
      ELSE T.REC_USER END AS REC_USER, CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES
      WHERE RESOURCE_ID = (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID))
      ELSE (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) END AS REC_USER_NAME, T.PUBLIC_FLAG, T.MASK_EMPLOYEE_INFO
      FROM TMT_POOLS T
      LEFT OUTER JOIN PMO_TMP_POOLS_0003 H ON H.ELEMENT_ID = T.ELEMENT_ID
      LEFT JOIN TMT_CLIENTS C ON C.ELEMENT_ID = T.COST_CENTER_ID
      WHERE T.ELEMENT_ID = h_id
      AND T.REC_STATUS NOT in ('D', 'L', 'A');
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL', 114+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL(115) : pools.sql(5914) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN CR0001;
   ElsIf ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
      h_is_static := 0;
      
      h_xsql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      h_xsql := h_xsql || 
                    'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_xsql := h_xsql || 'T.DELETED_COUNT, ';
      h_xsql := h_xsql || 
                    'CASE WHEN T.TYPE_ID = 13 THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.REC_STATUS END AS REC_STATUS, ';
      h_xsql := h_xsql || 'T.DESCRIPTION, ';
      h_xsql := h_xsql || 
                    'T.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME, ';
      h_xsql := h_xsql || 
                    'T.USER_GROUP_ID, COALESCE((SELECT ELEMENT_NAME FROM TMT_POOLS WHERE ELEMENT_ID = T.USER_GROUP_ID),'''') AS USER_GROUP_NAME, ';
      h_xsql := h_xsql || 
                    'T.CLEARANCE_LEVEL_1, T.CLEARANCE_LEVEL_2, T.CLEARANCE_LEVEL_3, T.CLEARANCE_LEVEL_4, T.CLEARANCE_LEVEL_5, ';
      h_xsql := h_xsql || 
                    '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE, ';
      h_xsql := h_xsql || 
                    'CASE WHEN T.TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE '''' END AS TITLE, ';
      
      h_xsql := h_xsql || 'CASE WHEN T.TYPE_ID = 13 THEN ';
      h_xsql := h_xsql || '((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_xsql := h_xsql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_xsql := h_xsql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_xsql := h_xsql || 
                    'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) || '', '' || ';
      h_xsql := h_xsql || '(SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID = ';
      h_xsql := h_xsql || '(SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND ';
      h_xsql := h_xsql || 
                    'PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))) ';
      h_xsql := h_xsql || 'ELSE '''' END AS COMPETENCY, T.REC_USER, ';
      
      h_xsql := h_xsql || 
                    'CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)) ELSE (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) END AS REC_USER_NAME, T.PUBLIC_FLAG, T.MASK_EMPLOYEE_INFO ';
      h_xsql := h_xsql || 'FROM TMT_POOLS T ';
      
      -- Validate id
      If ((p_id IS NULL)) Then
         h_xsql := h_xsql || 'WHERE T.PARENT_ID IS NULL ';
         h_xsql := h_xsql || 'AND T.LEVEL_ID = 0 ';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_POOL', 161+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_POOL(162) : pools.sql(5961) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
         END;
         
         h_xsql := h_xsql || 'WHERE (T.PARENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''' ';
         h_xsql := h_xsql || 'OR T.ELEMENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''') ';
      End If;
      h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
      h_xsql := h_xsql || ' ORDER BY RANK ';
   ElsIf ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 1) Then
      h_is_static := 0;
      
      h_xsql := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
      h_xsql := h_xsql || 'T.ELEMENT_NAME, ';
      h_xsql := h_xsql || 
                    'T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT, ';
      h_xsql := h_xsql || 'T.DELETED_COUNT, T.REC_STATUS, ';
      h_xsql := h_xsql || 'T.DEFAULT_FLAG, T.DESCRIPTION, ';
      h_xsql := h_xsql || 
                    'T.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME, ';
      h_xsql := h_xsql || 
                    'T.USER_GROUP_ID, COALESCE((SELECT ELEMENT_NAME FROM TMT_POOLS WHERE ELEMENT_ID = T.USER_GROUP_ID),'''') AS USER_GROUP_NAME, ';
      h_xsql := h_xsql || 
                    'T.CLEARANCE_LEVEL_1, T.CLEARANCE_LEVEL_2, T.CLEARANCE_LEVEL_3, T.CLEARANCE_LEVEL_4, T.CLEARANCE_LEVEL_5,';
      h_xsql := h_xsql || 'T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, ';
      h_xsql := h_xsql || 
                    'T.OTHER_LOCATION, T.YEAR_IN_BUSINESS, T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, ';
      h_xsql := h_xsql || 
                    'T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES, T.NUMBER_POOLS, T.NUMBER_CLIENTS, ';
      h_xsql := h_xsql || 
                    'T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX, T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, ';
      h_xsql := h_xsql || 
                    'T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED, T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, '''' AS REVISION_HISTORY, 0 AS REVISION_TYPE, 0 AS REVISION_IMPACT, ';
      h_xsql := h_xsql || 'T.COST_CENTER_ID, '''' AS COST_CENTER_NAME, ';
      h_xsql := h_xsql || 'T.REC_USER, ';
      h_xsql := h_xsql || 
                    '(SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) AS REC_USER_NAME, T.PUBLIC_FLAG, T.MASK_EMPLOYEE_INFO ';
      h_xsql := h_xsql || 'FROM TMT_POOLS T ';
      
      -- Execute Select Statement
      If ((p_id IS NULL)) Then
         h_xsql := h_xsql || 'WHERE T.PARENT_ID IS NULL ';
         h_xsql := h_xsql || 'AND T.TYPE_ID NOT IN (0, 13, 307) ';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || ' ORDER BY RANK ';
      Else
         h_id := p_id;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_POOL', 220+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_POOL(221) : pools.sql(6020) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
         END;
         
         h_xsql := h_xsql || 'WHERE (T.PARENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''' ';
         h_xsql := h_xsql || 'OR T.ELEMENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''') ';
         h_xsql := h_xsql || 'AND T.TYPE_ID NOT IN (0, 13, 307) ';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || ' ORDER BY RANK ';
      End If;
   Else
      h_id := p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_POOL', 244+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_POOL(245) : pools.sql(6044) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      h_is_static := 0;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_POOLS_0003';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_POOLS_0003
         SELECT H.ELEMENT_ID, H.REVISION_TYPE, H.REVISION_IMPACT
         FROM HISTORY_POOLS H
         WHERE H.ELEMENT_ID = h_id AND
         H.REC_DATETIME = (
         SELECT MAX(REC_DATETIME)
         FROM HISTORY_POOLS
         WHERE ELEMENT_ID = H.ELEMENT_ID);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL', 280+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL(281) : pools.sql(6080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_xsql := 
            'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.ELEMENT_NAME END AS ELEMENT_NAME, T.CHILD_COUNT, T.DELETED_COUNT, CASE WHEN T.TYPE_ID = 13 THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.REC_STATUS END AS REC_STATUS, T.DEFAULT_FLAG, T.DESCRIPTION, ';
      h_xsql := h_xsql || 
                    'T.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME, ';
      h_xsql := h_xsql || 
                    'T.USER_GROUP_ID, COALESCE((SELECT ELEMENT_NAME FROM TMT_POOLS WHERE ELEMENT_ID = T.USER_GROUP_ID),'''') AS USER_GROUP_NAME, ';
      h_xsql := h_xsql || 
                    'T.CLEARANCE_LEVEL_1, T.CLEARANCE_LEVEL_2, T.CLEARANCE_LEVEL_3, T.CLEARANCE_LEVEL_4, T.CLEARANCE_LEVEL_5,';
      h_xsql := h_xsql || 
                    'T.COMPANY_NAME, T.ADDRESS, T.PHONE, T.FAX, T.WEBSITE, T.CONTACT_NAME, T.OTHER_LOCATION, T.YEAR_IN_BUSINESS, T.CLIENT_MORE_100_EMPLOYEES, T.CLIENT_MORE_1000_EMPLOYEES, T.CLIENT_MORE_10000_EMPLOYEES, T.NUMBER_EMPLOYEES, T.NUMBER_POOLS, T.NUMBER_CLIENTS, T.ANNUAL_REVENUES, T.BILLING_ADDRESS, T.BILLING_PHONE, T.BILLING_FAX, T.BILLING_EMAIL, T.EQUAL_OPPORTUNITY, T.COMPANY_BACKGROUND, T.OWNERSHIP, T.SERVICES_OFFERED, T.SERVICES_SPECIALIZED, T.INDUSTRIES_SERVED, T.HIGHLIGHT, T.LOCATION_SERVED, T.LABEL, ';
      h_xsql := h_xsql || ''''' AS REVISION_HISTORY, H.REVISION_TYPE, H.REVISION_IMPACT, ';
      h_xsql := h_xsql || 'T.COST_CENTER_ID, '''' AS COST_CENTER_NAME, ';
      h_xsql := h_xsql || 
                    'CASE WHEN T.TYPE_ID = 13 THEN (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.REC_USER END AS REC_USER, CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)) ELSE (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) END AS REC_USER_NAME, T.PUBLIC_FLAG, T.MASK_EMPLOYEE_INFO ';
      h_xsql := h_xsql || 
                    'FROM TMT_POOLS T LEFT OUTER JOIN PMO_TMP_POOLS_0003 H ON H.ELEMENT_ID = T.ELEMENT_ID ';
      
      -- Execute Select Statement
      If ((p_id IS NULL)) Then
         h_xsql := h_xsql || 'WHERE T.PARENT_ID IS NULL ';
         h_xsql := h_xsql || 'AND T.TYPE_ID <> 0 ';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || ' ORDER BY RANK ';
      Else
         h_xsql := h_xsql || 'WHERE (T.PARENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''' ';
         h_xsql := h_xsql || 'OR T.ELEMENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''') ';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || ' ORDER BY RANK ';
      End If;
   End If;
   
   If Coalesce(h_is_static, 0) = 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL', 322+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL(323) : pools.sql(6122) : ' || SYSTIMESTAMP);
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL', 325+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL(326) : pools.sql(6125) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_SELECT_READ(h_xsql);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL', 334+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL(335) : pools.sql(6134) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL', 346+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL(347) : pools.sql(6146) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_POOL;
ALTER FUNCTION SP_S_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_A_RES_POOL ==
CREATE OR REPLACE FUNCTION SP_A_RES_POOL
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_exists VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_status VARCHAR2(1);
   h_resource_id VARCHAR2(32);
   h_max_rank PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RES_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RES_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RES_POOL(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_POOL', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RES_POOL(26) : pools.sql(6188) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_CHAR(h_uniqueID, SC_H_ID);
   --SC_LONG(h_level);
   
   If ((p_id IS NULL)) OR ((p_resource_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RES_POOL', 34+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RES_POOL(35) : pools.sql(6197) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_POOL', 37+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_POOL(38) : pools.sql(6200) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   -- Validate id
   --InitializeHost(SC_PARAM(h_id), p_id, ind_a_pool->p_id);
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_A_RES_POOL', 46+1) THEN 
      LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_A_RES_POOL(47) : pools.sql(6209) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   -- Validate resource id
   --InitializeHost(SC_PARAM(h_resource_id), p_resource_id, ind_a_pool->p_resource_id);
   h_resource_id := p_resource_id;
   BEGIN 
      SELECT REC_STATUS INTO h_rec_status FROM RESOURCES WHERE RESOURCE_ID = h_resource_id; 
      IF SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') THEN 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True); 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_resource_id, True); 
   END;
   
   BEGIN
      SELECT ELEMENT_ID INTO h_exists FROM TMT_POOLS
      WHERE ELEMENT_ID = p_resource_id AND PARENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(80) : pools.sql(6242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RES_POOL', 85+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RES_POOL(86) : pools.sql(6248) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RES_POOL', 88+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RES_POOL(89) : pools.sql(6251) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ASSIGNMENT_ALREADY_EXISTS, True);
   End If;
   
   -- Unassign from elsewhere
   BEGIN
      SELECT PARENT_ID INTO h_exists FROM TMT_POOLS
      WHERE ELEMENT_ID = p_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 103+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(104) : pools.sql(6266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      -- Execute Delete Statement
      BEGIN
         DELETE FROM TMT_POOLS WHERE ELEMENT_ID = p_resource_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(123) : pools.sql(6285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         -- Update the parent CHILD_COUNT field
         BEGIN
            UPDATE TMT_POOLS SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE ELEMENT_ID = h_exists;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 142+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(143) : pools.sql(6305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   -- Get next sibling count
   PMODEFINES.PMO_GET_MAX_RANK('TMT_POOLS', h_id, h_max_rank);
   
   --Get Base 64
   PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
   
   BEGIN
      INSERT INTO TMT_POOLS(RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, REC_USER)
      SELECT RTRIM(Rank) || h_rankBase64, p_id, p_resource_id, PMO_CNST.SC_RESOURCE, LEVEL_ID + 1, p_rec_user
      FROM TMT_POOLS WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 169+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(170) : pools.sql(6332) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RES_POOL', 175+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RES_POOL(176) : pools.sql(6338) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 182+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(183) : pools.sql(6345) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RES_POOL', 194+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RES_POOL(195) : pools.sql(6357) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RES_POOL;
ALTER FUNCTION SP_A_RES_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UA_RES_POOL ==
CREATE OR REPLACE FUNCTION SP_UA_RES_POOL
(
     p_resource_id VARCHAR2
   , p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UA_RES_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UA_RES_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_UA_RES_POOL(' || 
       '''' || p_resource_id || ''''
          || ', ' || '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UA_RES_POOL', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UA_RES_POOL(19) : pools.sql(6392) : ' || SYSTIMESTAMP);
   END IF; 

   -- Execute Delete Statement
   BEGIN
      DELETE FROM TMT_POOLS WHERE PARENT_ID = p_id AND ELEMENT_ID = p_resource_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_POOL', 34+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_POOL(35) : pools.sql(6408) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      -- Update the parent CHILD_COUNT field
      BEGIN
         UPDATE TMT_POOLS SET    CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_POOL', 53+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_POOL(54) : pools.sql(6427) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UA_RES_POOL', 60+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UA_RES_POOL(61) : pools.sql(6434) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_POOL', 67+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_POOL(68) : pools.sql(6441) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UA_RES_POOL', 79+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UA_RES_POOL(80) : pools.sql(6453) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UA_RES_POOL;
ALTER FUNCTION SP_UA_RES_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_RES_A_POOL ==
CREATE OR REPLACE FUNCTION SP_S_RES_A_POOL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66671a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66671b PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RES_A_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RES_A_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RES_A_POOL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_POOL', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RES_A_POOL(24) : pools.sql(6493) : ' || SYSTIMESTAMP);
   END IF; 

   -- Execute Select Statement
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RES_A_POOL', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RES_A_POOL(30) : pools.sql(6499) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_POOL', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_POOL(33) : pools.sql(6502) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   If ((p_bitflag IS NOT NULL)) AND (Coalesce(p_bitflag, 0) = 2) Then
      BEGIN
         SELECT RANK INTO h_rank
         FROM TMT_POOLS
         WHERE ELEMENT_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_POOL', 50+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_POOL(51) : pools.sql(6520) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C66671a FOR SELECT
      T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T1.ELEMENT_ID AND TYPE_ID NOT IN (13)) AS CHILD_COUNT,
      T1.DELETED_COUNT,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE T1.REC_STATUS END AS REC_STATUS,
      T1.DESCRIPTION, T1.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T1.DOMAIN_ID) AS DOMAIN_NAME,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE '' END AS TITLE,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE '' END AS COMPETENCY
      FROM TMT_POOLS T1
      WHERE ((T1.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'
      AND LENGTH(T1.RANK) <= LENGTH(h_rank)
      AND ((T1.RANK >= SUBSTR(h_rank, 1, 8) AND T1.RANK <= h_rank) 
      AND SUBSTR(h_rank,1, CASE WHEN LENGTH(T1.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(T1.RANK) END ) = T1.RANK))
      OR T1.MANAGER_ID = p_id)
      AND T1.TYPE_ID = PMO_CNST.SC_POOLS AND T1.REC_STATUS <> 'D'
      ORDER BY RANK DESC;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_POOL', 72+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_POOL(73) : pools.sql(6542) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66671a;
   Else
      OPEN C66671b FOR SELECT
      T1.LEVEL_ID, T1.TYPE_ID, T1.RANK, T1.PARENT_ID, T1.ELEMENT_ID, T1.ELEMENT_NAME,
      1 AS CHILD_COUNT, T1.DELETED_COUNT,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE T1.REC_STATUS END AS REC_STATUS,
      T1.DESCRIPTION, T1.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T1.DOMAIN_ID) AS DOMAIN_NAME,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE '' END AS TITLE,
      CASE WHEN T1.TYPE_ID = 13 THEN (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T1.ELEMENT_ID) ELSE '' END AS COMPETENCY
      FROM TMT_POOLS P
      JOIN TMT_POOLS T1 ON T1.PARENT_ID = P.ELEMENT_ID
      WHERE (P.ELEMENT_ID = p_id
      OR T1.MANAGER_ID = p_id)
      AND T1.TYPE_ID BETWEEN 1 AND 5000
      AND T1.REC_STATUS <> 'D'
      ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RES_A_POOL', 91+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RES_A_POOL(92) : pools.sql(6561) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66671b;
   End If;
   
   --return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_POOL', 101+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_POOL(102) : pools.sql(6571) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RES_A_POOL', 113+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RES_A_POOL(114) : pools.sql(6583) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RES_A_POOL;
ALTER FUNCTION SP_S_RES_A_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_CKOUT_POOL ==
CREATE OR REPLACE FUNCTION SP_CKOUT_POOL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id2 VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_CKOUT_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_CKOUT_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_CKOUT_POOL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_CKOUT_POOL(22) : pools.sql(6621) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 26+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(27) : pools.sql(6626) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 29+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(30) : pools.sql(6629) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   If (p_rec_user IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 35+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(36) : pools.sql(6635) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_rec_user || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 38+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(39) : pools.sql(6638) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_rec_user, True);
   End If;
   
   -- Declare host variables
   
   BEGIN
      SELECT ELEMENT_ID, TYPE_ID, REC_STATUS
      INTO h_id2, h_type_id, h_rec_status
      FROM TMT_POOLS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 55+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(56) : pools.sql(6655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If PMO_DB2M.SC_STATE <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 61+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(62) : pools.sql(6661) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id2 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 64+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(65) : pools.sql(6664) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id2, True);
   ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      BEGIN
         SELECT RESOURCE_ID, REC_STATUS
         INTO h_id2, h_rec_status
         FROM RESOURCES WHERE RESOURCE_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 78+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(79) : pools.sql(6678) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If SubStr(h_rec_status, 1, 1) = 'O' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 86+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(87) : pools.sql(6686) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ALREADY_CHECKED_OUT), 'NULL') || ' ID: ' || '''' || h_id2 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 89+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(90) : pools.sql(6689) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.ALREADY_CHECKED_OUT, h_id2, True);
   End If;
   
   If SubStr(h_rec_status, 1, 1) = 'L' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 96+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(97) : pools.sql(6696) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_LOCKED_RECORD), 'NULL') || ' ID: ' || '''' || h_id2 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 99+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(100) : pools.sql(6699) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_LOCKED_RECORD, h_id2, True);
   End If;
   
   If SubStr(h_rec_status, 1, 1) = 'D' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKOUT_POOL', 106+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKOUT_POOL(107) : pools.sql(6706) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_DELETED), 'NULL') || ' ID: ' || '''' || h_id2 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKOUT_POOL', 109+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKOUT_POOL(110) : pools.sql(6709) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.MARKED_AS_DELETED, h_id2, True);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      BEGIN
         UPDATE RESOURCES SET
         REC_STATUS = 'O', REC_USER = p_rec_user, REC_DATETIME = SYSTIMESTAMP
         WHERE RESOURCE_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 129+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(130) : pools.sql(6729) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_POOLS SET
         REC_STATUS = 'O', REC_USER = p_rec_user, REC_DATETIME = SYSTIMESTAMP
         WHERE ELEMENT_ID = p_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 148+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(149) : pools.sql(6748) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_CKOUT_POOL', 155+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_CKOUT_POOL(156) : pools.sql(6755) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 162+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(163) : pools.sql(6762) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKOUT_POOL', 174+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKOUT_POOL(175) : pools.sql(6774) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_CKOUT_POOL;
ALTER FUNCTION SP_CKOUT_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_CKIN_POOL ==
CREATE OR REPLACE FUNCTION SP_CKIN_POOL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_uniqueID VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_bitflag_revision PLS_INTEGER := 0;
   h_table_name VARCHAR2(100);
   h_min_revision PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_converted_score NUMBER(19) := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR CH1 IS
         SELECT MINOR_REVISION
         FROM HISTORY_RESOURCES
         WHERE  ROWNUM <= 1 AND  RESOURCE_ID = p_id;

      CURSOR CH1P IS
         SELECT MINOR_REVISION
         FROM HISTORY_POOLS
         WHERE  ROWNUM <= 1 AND  ELEMENT_ID = p_id;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_CKIN_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_CKIN_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_CKIN_POOL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_CKIN_POOL(40) : pools.sql(6830) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKIN_POOL', 44+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKIN_POOL(45) : pools.sql(6835) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKIN_POOL(48) : pools.sql(6838) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   -- Declare host variables
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_ckout_pool->p_id);
   h_id := p_id;
   
   BEGIN
      SELECT TYPE_ID, REC_STATUS INTO h_type_id, h_rec_status
      FROM TMT_POOLS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(67) : pools.sql(6857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKIN_POOL', 72+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKIN_POOL(73) : pools.sql(6863) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 75+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKIN_POOL(76) : pools.sql(6866) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
   ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      BEGIN
         SELECT REC_STATUS, CONVERTED_SCORE INTO h_rec_status, h_converted_score
         FROM RESOURCES WHERE RESOURCE_ID = p_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 88+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(89) : pools.sql(6879) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If SubStr(h_rec_status, 1, 1) = 'D' Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKIN_POOL', 96+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKIN_POOL(97) : pools.sql(6887) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.MARKED_AS_DELETED), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 99+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKIN_POOL(100) : pools.sql(6890) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.MARKED_AS_DELETED, h_id, True);
   End If;
   
   If (SubStr(h_rec_status, 1, 1) = 'I') OR (SubStr(h_rec_status, 1, 1) = 'U') Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKIN_POOL', 106+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKIN_POOL(107) : pools.sql(6897) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ALREADY_CHECKED_IN), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 109+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKIN_POOL(110) : pools.sql(6900) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.ALREADY_CHECKED_IN, h_id, True);
   End If;
   
   If (SubStr(h_rec_status, 1, 1) = 'L') OR (SubStr(h_rec_status, 1, 1) = 'A') Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CKIN_POOL', 116+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CKIN_POOL(117) : pools.sql(6907) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_LOCKED_RECORD), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CKIN_POOL', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CKIN_POOL(120) : pools.sql(6910) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_LOCKED_RECORD, h_id, True);
   End If;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      -- **************************************************************************************
      -- h_max_revision = 0 MINOR_REVISION = MINOR_REVISION + 1
      --                          = 1 MAJOR_REVISION = MAJOR_REVISION + 1
      --                          = 2 check in as the same revision
      -- **************************************************************************************
      --InitializeHost(SC_PARAM(h_bitflag), *p_bitflag, ind_ckout_pool->p_bitflag);
      h_bitflag := p_bitflag;
      
      OPEN CH1;
      
      FETCH CH1 INTO h_min_revision;
      IF CH1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            CLOSE CH1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 151+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(152) : pools.sql(6942) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE RESOURCES SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP
            WHERE RESOURCE_ID = p_id AND REC_USER = p_rec_user;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 169+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(170) : pools.sql(6960) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            CLOSE CH1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 182+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(183) : pools.sql(6973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_bitflag, 0) = 0 Then
            BEGIN
               UPDATE RESOURCES SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP,
               MAJOR_REVISION = CASE WHEN MINOR_REVISION < 99 THEN MAJOR_REVISION ELSE MAJOR_REVISION + 1 END,
               MINOR_REVISION = CASE WHEN MINOR_REVISION < 99 THEN MINOR_REVISION + 1 ELSE 0 END
               WHERE RESOURCE_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 203+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(204) : pools.sql(6994) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 1 Then
            BEGIN
               UPDATE RESOURCES SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP,
               MAJOR_REVISION = MAJOR_REVISION + 1, MINOR_REVISION = 0 WHERE RESOURCE_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 221+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(222) : pools.sql(7012) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               UPDATE RESOURCES SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP
               WHERE RESOURCE_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 239+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(240) : pools.sql(7030) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         If Coalesce(h_converted_score, 0) = -1 Then
            h_table_type := PMO_CNST.SC_TABLE_RES;
            h_rec_user := p_rec_user;
            PMO_DB2M.SC_STATE := FC_CHECK_IN_ALL_SC(h_table_type, h_id, h_bitflag, h_rec_user);
         End If;
         
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO HISTORY_RESOURCES SELECT h_uniqueID, A.* FROM RESOURCES A WHERE A.RESOURCE_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 267+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(268) : pools.sql(7058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE RESOURCES SET REVISION_HISTORY = Null WHERE RESOURCE_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 284+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(285) : pools.sql(7075) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_bitflag_revision := 5;
         h_table_name := 'HISTORY_RESOURCES';
         --SC_STATE = FC_D_REVISION(h_id,h_bitflag_revision,h_table_name);
         PMO_DB2M.SC_STATE := FC_D_REVISION(h_id, h_bitflag_revision, h_table_name);
      End If;
   Else
      -- **************************************************************************************
      -- h_max_revision = 0 MINOR_REVISION = MINOR_REVISION + 1
      --                          = 1 MAJOR_REVISION = MAJOR_REVISION + 1
      --                          = 2 check in as the same revision
      -- **************************************************************************************
      --InitializeHost(SC_PARAM(h_bitflag), *p_bitflag, ind_ckout_pool->p_bitflag);
      h_bitflag := p_bitflag;
      
      OPEN CH1P;
      
      FETCH CH1P INTO h_min_revision;
      IF CH1P%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         BEGIN
            CLOSE CH1P ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 321+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(322) : pools.sql(7112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_POOLS SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP
            WHERE ELEMENT_ID = p_id AND REC_USER = p_rec_user;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 339+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(340) : pools.sql(7130) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            CLOSE CH1P ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 352+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(353) : pools.sql(7143) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_bitflag, 0) = 0 Then
            BEGIN
               UPDATE TMT_POOLS SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP,
               MAJOR_REVISION = CASE WHEN MINOR_REVISION < 99 THEN MAJOR_REVISION ELSE MAJOR_REVISION + 1 END,
               MINOR_REVISION = CASE WHEN MINOR_REVISION < 99 THEN MINOR_REVISION + 1 ELSE 0 END
               WHERE ELEMENT_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 373+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(374) : pools.sql(7164) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 1 Then
            BEGIN
               UPDATE TMT_POOLS SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP,
               MAJOR_REVISION = MAJOR_REVISION + 1, MINOR_REVISION = 0 WHERE ELEMENT_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 391+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(392) : pools.sql(7182) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               UPDATE TMT_POOLS SET REC_STATUS = 'U', REC_DATETIME = SYSTIMESTAMP
               WHERE ELEMENT_ID = p_id AND REC_USER = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 409+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(410) : pools.sql(7200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO HISTORY_POOLS SELECT h_uniqueID, A.* FROM TMT_POOLS A WHERE A.ELEMENT_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 431+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(432) : pools.sql(7222) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_POOLS SET REVISION_HISTORY = Null WHERE ELEMENT_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 448+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(449) : pools.sql(7239) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_bitflag_revision := 0;
         h_table_name := 'HISTORY_POOLS';
         --SC_STATE = FC_D_REVISION(h_id,h_bitflag_revision,h_table_name);
         PMO_DB2M.SC_STATE := FC_D_REVISION(h_id, h_bitflag_revision, h_table_name);
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_CKIN_POOL', 461+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_CKIN_POOL(462) : pools.sql(7252) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 468+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(469) : pools.sql(7259) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CKIN_POOL', 480+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CKIN_POOL(481) : pools.sql(7271) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_CKIN_POOL;
ALTER FUNCTION SP_CKIN_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_UCKOUT_POOL ==
CREATE OR REPLACE FUNCTION SP_UCKOUT_POOL
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_UCKOUT_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_UCKOUT_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_UCKOUT_POOL(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UCKOUT_POOL', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_UCKOUT_POOL(20) : pools.sql(7307) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_UCKOUT_POOL', 24+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_UCKOUT_POOL(25) : pools.sql(7312) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UCKOUT_POOL', 27+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_UCKOUT_POOL(28) : pools.sql(7315) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   -- Declare host variables
   
   BEGIN
      SELECT REC_STATUS INTO h_rec_status FROM TMT_POOLS WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UCKOUT_POOL', 42+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UCKOUT_POOL(43) : pools.sql(7330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_UCKOUT_POOL', 48+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_UCKOUT_POOL(49) : pools.sql(7336) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UCKOUT_POOL', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_UCKOUT_POOL(52) : pools.sql(7339) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
   End If;
   
   If SubStr(h_rec_status, 1, 1) = 'O' Then
      --Execute Insert Statement
      BEGIN
         UPDATE TMT_POOLS SET (REC_STATUS, REC_USER, REC_DATETIME) =
         (SELECT REC_STATUS, REC_USER, REC_DATETIME FROM HISTORY_POOLS
         WHERE ELEMENT_ID = p_id AND REC_DATETIME =
         (SELECT MAX(REC_DATETIME) FROM HISTORY_POOLS WHERE ELEMENT_ID = p_id))
         WHERE ELEMENT_ID = p_id AND REC_USER = p_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UCKOUT_POOL', 74+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UCKOUT_POOL(75) : pools.sql(7362) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_UCKOUT_POOL', 80+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_UCKOUT_POOL(81) : pools.sql(7368) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UNDO_CHECKOUT), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_UCKOUT_POOL', 83+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_UCKOUT_POOL(84) : pools.sql(7371) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.CANNOT_UNDO_CHECKOUT, True);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_UCKOUT_POOL', 89+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_UCKOUT_POOL(90) : pools.sql(7377) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UCKOUT_POOL', 96+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UCKOUT_POOL(97) : pools.sql(7384) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_UCKOUT_POOL', 108+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_UCKOUT_POOL(109) : pools.sql(7396) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_UCKOUT_POOL;
ALTER FUNCTION SP_UCKOUT_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_ISOUT_POOL ==
CREATE OR REPLACE FUNCTION SP_ISOUT_POOL
(
     p_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66675 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ISOUT_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ISOUT_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_ISOUT_POOL(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ISOUT_POOL', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ISOUT_POOL(20) : pools.sql(7432) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --SC_CHAR(h_id, SC_H_ID);
   
   -- Parse parameters to local variables
   --InitializeHost(SC_PARAM(h_id), id, 0);
   
   OPEN C66675 FOR SELECT ELEMENT_ID, REC_STATUS, REC_USER FROM TMT_POOLS WHERE ELEMENT_ID = p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ISOUT_POOL', 30+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_ISOUT_POOL(31) : pools.sql(7443) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C66675;
   
   --return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ISOUT_POOL', 39+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ISOUT_POOL(40) : pools.sql(7452) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ISOUT_POOL', 51+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ISOUT_POOL(52) : pools.sql(7464) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ISOUT_POOL;
ALTER FUNCTION SP_ISOUT_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_POOL_REVISION ==
CREATE OR REPLACE FUNCTION SP_U_POOL_REVISION
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rev_impact PLS_INTEGER
   , p_rev_type PLS_INTEGER
   , p_rev_history VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_POOL_REVISION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_POOL_REVISION'), '-- ' || SYSTIMESTAMP, ':r := SP_U_POOL_REVISION(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_rev_impact), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_rev_type), 'NULL')
          || ', ' || '''' || p_rev_history || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_REVISION', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_POOL_REVISION(27) : pools.sql(7507) : ' || SYSTIMESTAMP);
   END IF; 

   -- Validate id
   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_POOL_REVISION', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_POOL_REVISION(33) : pools.sql(7513) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_POOL_REVISION', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_POOL_REVISION(36) : pools.sql(7516) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   --InitializeHost(SC_PARAM(h_id), p_id, ind_u_rev->p_id);
   h_id := p_id;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_U_POOL_REVISION', 43+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_U_POOL_REVISION(44) : pools.sql(7524) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
   END;
   
   BEGIN
      UPDATE TMT_POOLS SET
      REVISION_IMPACT = p_rev_impact,
      REVISION_TYPE = p_rev_type,
      REVISION_HISTORY = p_rev_history
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_REVISION', 70+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_REVISION(71) : pools.sql(7551) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_POOL_REVISION', 76+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_POOL_REVISION(77) : pools.sql(7557) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_REVISION', 83+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_REVISION(84) : pools.sql(7564) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_POOL_REVISION', 95+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_POOL_REVISION(96) : pools.sql(7576) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_POOL_REVISION;
ALTER FUNCTION SP_U_POOL_REVISION COMPILE DEBUG;

/***********************************************************************************************************************************
* h_bitflag == 0    returns all revision on id (TMT_POOLS)
*                           id = ELEMENT_ID in TMT_POOLS table, select from HISTORY_POOLS where ELEMENT_ID = id
*
* h_bitflag == 1    returns WBS_HISTORY id and all WORKFLOWS on this is (only the first level)
*                           id = ELEMENT_ID in HISTORY_POOLS table, select from HISTORY_POOLS  TMT_WORKFLOWS
*
* h_bitflag == 2    returns next level for WORKFLOW_ID
*                           id = ELEMENT_ID in TMT_WORKFLOWS table, select from TMT_WORKFLOWS
************************************************************************************************************************************/

Prompt == FUNCTION SP_S_POOL_REVISION ==
CREATE OR REPLACE FUNCTION SP_S_POOL_REVISION
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_sql VARCHAR2(2048);
   h_app_revision PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_POOL_REVISION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_POOL_REVISION'), '-- ' || SYSTIMESTAMP, ':r := SP_S_POOL_REVISION(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_POOL_REVISION(24) : pools.sql(7625) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_REVISION', 28+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_REVISION(29) : pools.sql(7630) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(32) : pools.sql(7633) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   -- Declare host variables
   
   -- Parse parameters to local variables
   --InitializeHost(SC_PARAM(h_id), p_id, ind_s_revision->p_id);
   h_id := p_id;
   --InitializeHost(SC_PARAM(h_bitflag), *p_bitflag, ind_s_revision->p_bitflag);
   h_bitflag := p_bitflag;
   
   BEGIN
      SELECT SETTING_VALUE INTO h_app_revision
      FROM TMT_SETTINGS
      WHERE ELEMENT_ID = 'MAX_REVISION_VERSIONS';
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(55) : pools.sql(7656) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_REVISION', 60+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_REVISION(61) : pools.sql(7662) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 63+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(64) : pools.sql(7665) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
   End If;
   
   If Coalesce(h_bitflag, 0) = 0 Then
      --      h_sql.sprintf("SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, DELETED_COUNT, REVISION, REVISION_IMPACT, REVISION_HISTORY, "
      --          "REVISION_TYPE, REC_DATETIME, AUTHOR FROM
      h_sql := 
            'SELECT * FROM (SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.HISTORY_ID AS ELEMENT_ID, T.ELEMENT_NAME, CASE WHEN EXISTS(SELECT * FROM TMT_WORKFLOWS '
             || 'WHERE PARENT_ID=''' || h_id 
                  || ''' AND REVISION = (RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION)))) THEN 1 ELSE 0 END AS CHILD_COUNT, T.DELETED_COUNT, '
             || '(RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION))) AS REVISION, T.REVISION_IMPACT, T.REVISION_HISTORY, T.REVISION_TYPE, '
             || 'T.REC_DATETIME, B.FULL_NAME AS AUTHOR FROM HISTORY_POOLS T JOIN RESOURCES B ON T.REC_USER = B.RESOURCE_ID WHERE ELEMENT_ID = ''' 
                  || h_id || ''' '
             || ' ORDER BY REC_DATETIME DESC) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_app_revision, 0));
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         SELECT RANK, LEVEL_ID, ELEMENT_ID INTO h_rank, h_level_id, h_parent_id FROM HISTORY_POOLS WHERE HISTORY_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 88+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(89) : pools.sql(7690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_REVISION', 94+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_REVISION(95) : pools.sql(7696) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 97+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(98) : pools.sql(7699) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
      End If;
      
      --      h_sql.sprintf("SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME, CHILD_COUNT, DELETED_COUNT, REVISION, REVISION_IMPACT, REVISION_HISTORY, "
      --          "REVISION_TYPE, REC_DATETIME, AUTHOR FROM(
      h_sql := 
            'SELECT * FROM (SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.HISTORY_ID AS ELEMENT_ID, T.ELEMENT_NAME, CASE WHEN EXISTS(SELECT * FROM TMT_WORKFLOWS '
             || 'WHERE PARENT_ID=''' || h_parent_id 
                  || ''' AND REVISION = (RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION)))) THEN 1 ELSE 0 END AS CHILD_COUNT, T.DELETED_COUNT, '
             || '(RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION))) AS REVISION, T.REVISION_IMPACT, T.REVISION_HISTORY, T.REVISION_TYPE, '
             || 'T.REC_DATETIME, B.FULL_NAME AS AUTHOR FROM HISTORY_POOLS T JOIN RESOURCES B ON T.REC_USER=B.RESOURCE_ID '
             || 'WHERE HISTORY_ID = ''' || h_id || ''' UNION ALL '
             || 'SELECT (' || TO_CHAR(Coalesce(h_level_id, 0)) || ' + A.LEVEL_ID) AS LEVEL_ID, A.TYPE_ID, (''' 
                  || h_rank 
                  || ''' || A.RANK) AS RANK, A.PARENT_ID, A.ELEMENT_ID, A.ELEMENT_NAME, A.CHILD_COUNT, A.DELETED_COUNT, '''' AS REVISION, '
             || '0 AS REVISION_IMPACT, A.DESCRIPTION AS REVISION_HISTORY, 0 AS REVISION_TYPE, A.REC_DATETIME, C.FULL_NAME AS AUTHOR '
             || 'FROM TMT_WORKFLOWS A LEFT JOIN RESOURCES C ON A.AUTHOR_ID=C.RESOURCE_ID WHERE A.PARENT_ID = ''' 
                  || h_parent_id || ''' ORDER BY RANK) WHERE ROWNUM<=' || TO_CHAR(Coalesce(
                  h_app_revision, 0));
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      BEGIN
         SELECT A.ELEMENT_ID INTO h_parent_id
         FROM TMT_WORKFLOWS A, TMT_WORKFLOWS B
         WHERE B.ELEMENT_ID = h_id
         AND A.LEVEL_ID > -1
         AND A.PARENT_ID IS NuLL
         AND ((A.RANK >= SUBSTR(B.RANK, 1, 8) AND A.RANK <= B.RANK) 
         AND SUBSTR(B.RANK,1, CASE WHEN LENGTH(A.RANK) > LENGTH(B.RANK) THEN 1 ELSE LENGTH(A.RANK) END ) = A.RANK)
         AND A.RANK LIKE '0000' || '%';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 134+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(135) : pools.sql(7736) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_REVISION', 140+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_REVISION(141) : pools.sql(7742) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 143+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(144) : pools.sql(7745) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
      End If;
      
      BEGIN
         SELECT RANK, LEVEL_ID INTO h_rank, h_level_id FROM TMT_WBS WHERE ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 156+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(157) : pools.sql(7758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_POOL_REVISION', 162+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_POOL_REVISION(163) : pools.sql(7764) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 165+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(166) : pools.sql(7767) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, p_id, True);
      End If;
      
      h_sql := 'SELECT * FROM (SELECT (' || TO_CHAR(Coalesce(h_level_id, 0)) 
            || ' + A.LEVEL_ID) AS LEVEL_ID, A.TYPE_ID, (''' || h_rank 
            || ''' || A.RANK) AS RANK, A.PARENT_ID, A.ELEMENT_ID, A.ELEMENT_NAME, A.CHILD_COUNT, A.DELETED_COUNT, '''' AS REVISION, '
             || '0 AS REVISION_IMPACT, A.DESCRIPTION AS REVISION_HISTORY, 0 AS REVISION_TYPE, A.REC_DATETIME, C.FULL_NAME AS AUTHOR '
             || 'FROM TMT_WORKFLOWS A LEFT JOIN RESOURCES C ON A.AUTHOR_ID=C.RESOURCE_ID WHERE A.ELEMENT_ID = ''' 
                  || h_id || ''' OR A.PARENT_ID = ''' || h_id || ''' ORDER BY RANK) WHERE ROWNUM<=' 
                  || TO_CHAR(Coalesce(h_app_revision, 0));
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 180+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(181) : pools.sql(7782) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_POOL_REVISION', 183+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_POOL_REVISION(184) : pools.sql(7785) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   --return PROC_STATE;
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 191+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(192) : pools.sql(7793) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_POOL_REVISION', 203+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_POOL_REVISION(204) : pools.sql(7805) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_POOL_REVISION;
ALTER FUNCTION SP_S_POOL_REVISION COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_M_POOL ==
CREATE OR REPLACE FUNCTION SP_M_POOL
(
     p_target VARCHAR2
   , p_source VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_cr VARCHAR2(200);
   h_id VARCHAR2(32);
   h_object_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_r VARCHAR2(200);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_user VARCHAR2(32);
   h_source_parent_id VARCHAR2(32);
   h_source_rank VARCHAR2(200);
   h_target_id VARCHAR2(32);
   h_target_parent_id VARCHAR2(32);
   h_target_rank VARCHAR2(200);
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_target_level_id PLS_INTEGER := 0;
   h_target_type_id PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_MAT IS
         SELECT TASK_ID, TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_POOL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_POOL'), '-- ' || SYSTIMESTAMP, ':r := SP_M_POOL(' || 
       '''' || p_target || ''''
          || ', ' || '''' || p_source || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 46+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_POOL(47) : pools.sql(7868) : ' || SYSTIMESTAMP);
   END IF; 

   If ((p_source IS NULL)) OR ((p_bitflag IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 51+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(52) : pools.sql(7873) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 54+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(55) : pools.sql(7876) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   ElsIf Coalesce(p_bitflag, 0) < 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 59+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(60) : pools.sql(7881) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 62+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(63) : pools.sql(7884) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variables
   
   h_rec_user := p_rec_user;
   
   --validate source
   BEGIN
      SELECT PARENT_ID, TYPE_ID, RANK
      INTO h_source_parent_id, h_type_id,  h_source_rank
      FROM TMT_POOLS
      WHERE ELEMENT_ID = p_source;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 83+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(84) : pools.sql(7905) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_M_POOL', 90+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_M_POOL(91) : pools.sql(7912) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_source || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 93+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(94) : pools.sql(7915) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_source, True);
   End If;
   
   If Coalesce(h_type_id, 0) <> PMO_CNST.SC_RESOURCE Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 100+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(101) : pools.sql(7922) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 103+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(104) : pools.sql(7925) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If (BITAND(Coalesce(p_bitflag, 0), 1)) = 1 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 110+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(111) : pools.sql(7932) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 113+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(114) : pools.sql(7935) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   Else
      If (p_target IS NOT NULL) Then
         h_target_id := p_target;
         
         BEGIN
            SELECT TYPE_ID, LEVEL_ID, RTRIM(RANK)
            INTO h_target_type_id, h_target_level_id, h_target_rank
            FROM TMT_POOLS
            WHERE ELEMENT_ID = h_target_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(132) : pools.sql(7953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_M_POOL', 138+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_M_POOL(139) : pools.sql(7960) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_target_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 141+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(142) : pools.sql(7963) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_target_id, True);
         End If;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_POOLS', h_target_id, h_max_rank);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 149+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(150) : pools.sql(7971) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 152+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(153) : pools.sql(7974) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      h_level_id := Coalesce(h_target_level_id, 0) + 1;
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      h_rank := h_target_rank;
      h_rank := SubStr(h_rank || h_rankBase64, 1, 200);
   End If;
   
   --update source hierarchy
   If (h_source_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_POOL', 166+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_POOL(167) : pools.sql(7988) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_POOL', 169+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_POOL(170) : pools.sql(7991) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   Else
      BEGIN
         UPDATE TMT_POOLS
         SET CHILD_COUNT = CHILD_COUNT - 1
         WHERE ELEMENT_ID = h_source_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 187+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(188) : pools.sql(8009) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --Update Pool
   BEGIN
      UPDATE TMT_POOLS
      SET PARENT_ID = h_target_id,
      LEVEL_ID = h_level_id,
      RANK = h_rank,
      REC_USER = p_rec_user,
      REC_STATUS = 'U',
      REC_DATETIME = SYSTIMESTAMP
      WHERE ELEMENT_ID = p_source;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 213+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(214) : pools.sql(8035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_type_id, 0) = PMO_CNST.SC_RESOURCE Then
      BEGIN
         UPDATE TMT_RESOURCE_DOCUMENTS
         SET PARENT_RANK = 'RES#' || h_rank
         WHERE PROJECT_ID = p_source;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 233+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(234) : pools.sql(8055) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_ASSIGNED_SCORECARDS
         SET PARENT_RANK = 'RES#' || h_rank
         WHERE PARENT_RANK = 'RES#' || h_source_rank;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 252+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(253) : pools.sql(8074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_PUBLISHED_SCORECARDS
         SET PARENT_RANK = 'RES#' || h_rank
         WHERE PARENT_RANK = 'RES#' || h_source_rank;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 271+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(272) : pools.sql(8093) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT RANK, POOL_ID, PROJECT_ID
         INTO h_object_rank, h_id, h_project_id
         FROM TMT_WBS
         WHERE POOL_ID = p_source
         AND TYPE_ID = PMO_CNST.SC_RESOURCE;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 288+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(289) : pools.sql(8110) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT ELEMENT_ID
         INTO h_parent_id
         FROM TMT_WBS
         WHERE RANK = h_object_rank || '0001';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 304+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(305) : pools.sql(8126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_object_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_object_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_object_rank, True); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_WBS W
         SET W.RANK = '/PL#' || (SELECT RTRIM(RANK) FROM TMT_POOLS WHERE ELEMENT_ID = h_id) || SUBSTR(W.RANK, LENGTH(h_object_rank)+1)
         WHERE W.RANK LIKE h_object_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 336+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(337) : pools.sql(8158) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Remove invalid "Administrative Tasks"
      
      OPEN C_MAT;
      
      LOOP
         FETCH C_MAT INTO h_task_id, h_tsk_assignment_id;
         IF C_MAT%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         PMO_DB2M.SC_STATE := FC_D_SUMMARY_TS(h_project_id, h_tsk_assignment_id);
         
         If PMO_DB2M.SC_STATE = PMO_CNST.CANNOT_DELETE Then
            BEGIN
               UPDATE TMT_WBS SET
               TYPE_ID = PMO_CNST.SC_INACTIVE_ADM_TASK
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 374+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(375) : pools.sql(8196) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_MAT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 389+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(390) : pools.sql(8211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Inherit all the administrative tasks
      h_rec_user := h_id;
      PMO_DB2M.SC_STATE := FC_POPULATE_RES_AT(h_id, h_parent_id, h_project_id, h_rec_user);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_POOL', 400+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_POOL(401) : pools.sql(8222) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_source || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_source, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 407+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(408) : pools.sql(8229) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_POOL', 419+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_POOL(420) : pools.sql(8241) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_POOL;
ALTER FUNCTION SP_M_POOL COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_MIG_POOL_WBS ==
CREATE OR REPLACE FUNCTION SP_MIG_POOL_WBS
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_name VARCHAR2(255);
   h_new_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_rec_user VARCHAR2(32);
   h_temp_rank VARCHAR2(200);
   h_bitflag PLS_INTEGER := 0;
   h_child_count PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_max_rank PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_MIG_POP IS
         SELECT TYPE_ID, PARENT_ID, ELEMENT_ID
         FROM TMT_POOLS
         WHERE ELEMENT_ID IS NOT NULL
         AND TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE)
         AND RANK < 'qqqq'
         ORDER BY RANK;

      CURSOR C_MIG_UPD IS
         SELECT RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, MAX_RANK, CHILD_COUNT
         FROM PMO_MIG_POOLS
         WHERE ELEMENT_ID IS NOT NULL
         AND TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE)
         ORDER BY RANK;

      CURSOR C_MIG_PW IS
         SELECT PARENT_ID, ELEMENT_ID, TYPE_ID, RTRIM(ELEMENT_NAME)
         FROM TMT_POOLS
         WHERE ELEMENT_ID IS NOT NULL
         AND TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE)
         ORDER BY RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_MIG_POOL_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_MIG_POOL_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_MIG_POOL_WBS(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_MIG_POOL_WBS', 51+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_MIG_POOL_WBS(52) : pools.sql(8309) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      -- clean-up pool types
      BEGIN
         DELETE FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_POOLS )
         AND SECURITY_ID = 'SCPROJECTMANAGER';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 72+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(73) : pools.sql(8330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PROJECT_ASSIGNMENTS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_POOLS );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 92+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(93) : pools.sql(8350) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 110+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(111) : pools.sql(8368) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- clean-up resource types
      BEGIN
         DELETE FROM SUMMARY_TIMESHEETS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE '/%' );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 132+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(133) : pools.sql(8390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE '/%' );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(154) : pools.sql(8411) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE '/%' );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 174+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(175) : pools.sql(8432) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM PROJECT_ASSIGNMENTS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE '/%' );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 195+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(196) : pools.sql(8453) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         DELETE FROM TMT_WBS
         WHERE PROJECT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_WBS
         WHERE RANK LIKE '/%' );
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 216+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(217) : pools.sql(8474) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --Clean up WBS
      BEGIN
         DELETE FROM TMT_WBS
         WHERE RANK LIKE '/%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 235+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(236) : pools.sql(8493) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --temporary pool table
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_MIG_POOLS';
      PMO_DB2M.SC_STATE := 0;
      
      --insert dummy row
      BEGIN
         INSERT INTO PMO_MIG_POOLS(RANK, LEVEL_ID, MAX_RANK, CHILD_COUNT) VALUES('0000', -1, 1, 0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 258+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(259) : pools.sql(8516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_MIG_POP;
      
      LOOP
         FETCH C_MIG_POP INTO h_type_id, h_parent_id, h_id;
         IF C_MIG_POP%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If (h_parent_id IS NULL) Then
            BEGIN
               UPDATE PMO_MIG_POOLS SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1 WHERE RANK = '0000';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 289+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(290) : pools.sql(8547) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_level_id := 0;
            
            BEGIN
               SELECT MAX_RANK INTO h_max_rank FROM PMO_MIG_POOLS WHERE RANK = '0000';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 304+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(305) : pools.sql(8562) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            BEGIN
               SELECT '0000' || h_rankBase64 INTO h_rank FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 319+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(320) : pools.sql(8577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE PMO_MIG_POOLS SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1 WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 336+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(337) : pools.sql(8594) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT MAX_RANK INTO h_max_rank FROM PMO_MIG_POOLS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 349+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(350) : pools.sql(8607) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
            
            BEGIN
               SELECT RTRIM(RANK) || h_rankBase64, LEVEL_ID + 1
               INTO h_rank, h_level_id
               FROM PMO_MIG_POOLS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 367+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(368) : pools.sql(8625) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            INSERT INTO PMO_MIG_POOLS (RANK, PARENT_ID, ELEMENT_ID, TYPE_ID, LEVEL_ID, MAX_RANK, CHILD_COUNT)
            VALUES(h_rank, h_parent_id, h_id, h_type_id, h_level_id, 1, 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 386+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(387) : pools.sql(8644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE C_MIG_POP ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 400+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(401) : pools.sql(8658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_POOLS SET
         RANK = 'XXXX' || RTRIM(RANK)
         WHERE TYPE_ID IN (PMO_CNST.SC_POOLS, PMO_CNST.SC_RESOURCE);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 419+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(420) : pools.sql(8677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_MIG_UPD;
      
      LOOP
         FETCH C_MIG_UPD INTO h_rank, h_parent_id, h_id, h_type_id, h_level_id, h_max_rank, h_child_count;
         IF C_MIG_UPD%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         --trace_printf("h_type_id: %d (%d)\n", h_type_id, h_type_id_Ind);
         --trace_printf("h_parent_id: %s (%d)\n", h_parent_id, h_parent_id_Ind);
         --trace_printf("h_id: %s (%d)\n", h_id, h_id_Ind);
         --trace_flush();
         
         BEGIN
            UPDATE TMT_POOLS SET
            RANK = h_rank,
            LEVEL_ID = h_level_id,
            MAX_RANK = h_max_rank-1,
            CHILD_COUNT = h_child_count
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 459+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(460) : pools.sql(8717) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_type_id, 0) = PMO_CNST.SC_POOLS Then
            --DOCUMENTS
            BEGIN
               SELECT RANK INTO h_temp_rank FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 474+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(475) : pools.sql(8732) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_temp_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_temp_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_temp_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_POOL_DOCUMENTS SET PARENT_RANK = 'POL#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 504+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(505) : pools.sql(8762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE HISTORY_POOL_DOCUMENTS SET PARENT_RANK = 'POL#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 521+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(522) : pools.sql(8779) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            --DOCUMENTS
            BEGIN
               SELECT RANK INTO h_temp_rank FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 535+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(536) : pools.sql(8793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_temp_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_temp_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_temp_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_RESOURCE_DOCUMENTS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 565+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(566) : pools.sql(8823) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE HISTORY_RESOURCE_DOCUMENTS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 582+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(583) : pools.sql(8840) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --SCORECARDS
            BEGIN
               SELECT RANK INTO h_temp_rank FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 596+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(597) : pools.sql(8854) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_ASSIGNED_SCORECARDS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 613+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(614) : pools.sql(8871) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE HISTORY_ASSIGNED_SCORECARDS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 630+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(631) : pools.sql(8888) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT RANK INTO h_temp_rank FROM TMT_PUBLISHED_SCORECARDS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 643+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(644) : pools.sql(8901) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_PUBLISHED_SCORECARDS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 660+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(661) : pools.sql(8918) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE HISTORY_PUBLISHED_SCORECARDS SET PARENT_RANK = 'RES#' || h_rank WHERE RANK LIKE h_temp_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 677+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(678) : pools.sql(8935) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_MIG_UPD ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 692+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(693) : pools.sql(8950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C_MIG_PW;
      
      LOOP
         FETCH C_MIG_PW INTO h_parent_id, h_id, h_type_id, h_name;
         IF C_MIG_PW%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         BEGIN
            SELECT TYPE_ID INTO h_type_id
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 720+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(721) : pools.sql(8978) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            If Coalesce(h_type_id, 0) = PMO_CNST.SC_POOLS Then
               BEGIN
                  SELECT RESOURCE_ID INTO h_rec_user
                  FROM RESOURCE_ASSIGNMENTS
                  WHERE ELEMENT_ID = h_id
                  AND BITAND(PARTICIPANT, PMO_CNST.SC_RESOURCE_MANAGER) = PMO_CNST.SC_RESOURCE_MANAGER;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 738+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(739) : pools.sql(8996) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_rec_user := h_id;
            End If;
            
            h_new_id := SYS_GUID();
            PMO_DB2M.SC_STATE := FC_I_POOL_WBS(h_new_id, h_id, h_name, h_type_id, h_rec_user);
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C_MIG_PW ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 759+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(760) : pools.sql(9017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      NULL;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_MIG_POOL_WBS', 768+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_MIG_POOL_WBS(769) : pools.sql(9026) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 775+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(776) : pools.sql(9033) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIG_POOL_WBS', 787+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIG_POOL_WBS(788) : pools.sql(9045) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_MIG_POOL_WBS;
ALTER FUNCTION SP_MIG_POOL_WBS COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ADM_TASK ==
CREATE OR REPLACE FUNCTION SP_S_ADM_TASK
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_ADMTSK PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ADM_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ADM_TASK'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ADM_TASK(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADM_TASK', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ADM_TASK(22) : pools.sql(9083) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_ADM_TASK', 26+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_ADM_TASK(27) : pools.sql(9088) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADM_TASK', 29+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADM_TASK(30) : pools.sql(9091) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   h_id := p_id;
   
   BEGIN
      SELECT RTRIM(RANK) INTO h_rank
      FROM TMT_WBS
      WHERE POOL_ID = h_id
      AND TYPE_ID = PMO_CNST.SC_SUMMARY_TASK;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ADM_TASK', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ADM_TASK(50) : pools.sql(9111) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_ADM_TASK', 55+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_ADM_TASK(56) : pools.sql(9117) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADM_TASK', 58+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADM_TASK(59) : pools.sql(9120) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   OPEN C_ADMTSK FOR SELECT  LEVEL_ID-1 AS LEVEL_ID,
   TYPE_ID,
   RANK,
   PARENT_ID,
   ELEMENT_ID,
   ELEMENT_NAME,
   CHILD_COUNT,
   DELETED_COUNT,
   REC_STATUS,
   REC_USER,
   REC_DATETIME,
   POOL_ID
   FROM TMT_WBS
   WHERE PMO_ORASTD.LEFT(RANK, LENGTH(h_rank)) = h_rank
   AND (LENGTH(RANK)-LENGTH(h_rank)) < 5
   AND TYPE_ID <> PMO_CNST.SC_INACTIVE_ADM_TASK
   ORDER BY RANK;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADM_TASK', 81+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADM_TASK(82) : pools.sql(9143) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C_ADMTSK;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ADM_TASK', 89+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ADM_TASK(90) : pools.sql(9151) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ADM_TASK', 101+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ADM_TASK(102) : pools.sql(9163) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ADM_TASK;
ALTER FUNCTION SP_S_ADM_TASK COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_I_ADM_TASK ==
CREATE OR REPLACE FUNCTION SP_I_ADM_TASK
(
     p_adm_tsk_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_adm_tsk_name VARCHAR2(255);
   h_calendar_id VARCHAR2(32);
   h_day_year DATE;
   h_object_parent_id VARCHAR2(32);
   h_object_project_id VARCHAR2(32);
   h_object_rank VARCHAR2(200);
   h_object_transfer_flag VARCHAR2(1);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
   h_object_assign_count PLS_INTEGER := 0;
   h_object_child_count PLS_INTEGER := 0;
   h_object_table PLS_INTEGER := 0;
   h_object_type PLS_INTEGER := 0;
   h_transfered_object_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_POOL_AT IS
         SELECT POOL_ID, RANK
         FROM TMT_WBS
         WHERE PMO_ORASTD.LEFT(RANK,LENGTH(h_object_rank)) = h_object_rank
         AND (LENGTH(RANK)-LENGTH(h_object_rank)) = 4
         AND TYPE_ID = PMO_CNST.SC_RESOURCE
         ORDER BY RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_ADM_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_ADM_TASK'), '-- ' || SYSTIMESTAMP, ':r := SP_I_ADM_TASK(' || 
       '''' || p_adm_tsk_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_ADM_TASK', 48+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_ADM_TASK(49) : pools.sql(9228) : ' || SYSTIMESTAMP);
   END IF; 

   h_adm_tsk_name := p_adm_tsk_name;
   h_parent_id := p_parent_id;
   h_type_id := p_type_id;
   h_rec_user := p_rec_user;
   
   --timeLog(file, "01-S");
   PMO_DB2M.SC_STATE := FC_GET_ELEMENT_INFO(h_parent_id, h_object_rank, h_object_project_id
                                         , h_object_parent_id, h_object_table, h_object_type
                                         , h_transfered_object_type, h_object_assign_count
                                         , h_object_child_count, h_object_transfer_flag);
   --timeLog(file, "01-F");
   
   --timeLog(file, "02-S");
   PMO_DB2M.SC_STATE := FC_IS_VALID_INSERT_CHILD(h_object_table, h_parent_id);
   If PMO_DB2M.SC_STATE = PMO_CNST.CANNOT_ASSIGN_CHILDREN Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_I_ADM_TASK', 67+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_I_ADM_TASK(68) : pools.sql(9247) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_parent_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_ADM_TASK', 70+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_ADM_TASK(71) : pools.sql(9250) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_parent_id, True);
   End If;
   --timeLog(file, "02-F");
   
   --timeLog(file, "03-S");
   h_ieflag := 0;
   PMO_DB2M.SC_STATE := FC_I_WBS_MSP(h_adm_tsk_name, h_parent_id, h_type_id, h_rec_user, h_ieflag
                                  , h_uniqueID);
   --timeLog(file, "03-F");
   
   --timeLog(file, "04-S");
   BEGIN
      UPDATE TMT_WBS SET POOL_ID = ELEMENT_ID WHERE ELEMENT_ID = h_uniqueID;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 95+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(96) : pools.sql(9275) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog(file, "04-F");
   
   BEGIN
      SELECT TO_DATE(SYSDATE) INTO h_day_year FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(110) : pools.sql(9289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --timeLog(file, "05-S");
   BEGIN
      SELECT PMO_ORASTD.LEFT(RANK,LENGTH(RANK)-4) INTO h_object_rank
      FROM TMT_WBS
      WHERE ELEMENT_ID = h_parent_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 125+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(126) : pools.sql(9305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog(file, "05-F");
   
   --timeLog(file, "06-S");
   
   OPEN C_POOL_AT;
   
   LOOP
      FETCH C_POOL_AT INTO h_resource_id, h_rank;
      IF C_POOL_AT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      --timeLog(file, "06-01-S");
      BEGIN
         SELECT CALENDAR_ID INTO h_calendar_id
         FROM RESOURCES WHERE RESOURCE_ID = h_resource_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 156+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(157) : pools.sql(9336) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --timeLog(file, "06-01-F");
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) AND ((h_calendar_id IS NOT NULL)) Then
         --timeLog(file, "06-02-S");
         BEGIN
            SELECT ELEMENT_ID INTO h_parent_id
            FROM TMT_WBS WHERE RANK = h_rank || '0001';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 173+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(174) : pools.sql(9353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --timeLog(file, "06-02-F");
         
         h_tsk_assignment_id := '';
         
         --timeLog(file, "06-03-S");
         PMO_DB2M.SC_STATE := FC_I_SUMMARY_TS(h_adm_tsk_name, h_parent_id, h_type_id, h_day_year
                                     , h_resource_id, h_tsk_assignment_id);
         --timeLog(file, "06-03-F");
         
         --timeLog(file, "06-04-S");
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            BEGIN
               UPDATE TMT_WBS SET
               POOL_ID = h_uniqueID,
               REC_STATUS = 'U'
               WHERE POOL_ID = h_parent_id
               AND TYPE_ID = PMO_CNST.SC_ACTIVE_ADM_TASK;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 204+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(205) : pools.sql(9384) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --timeLog(file, "06-04-F");
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_POOL_AT ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 221+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(222) : pools.sql(9401) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --timeLog(file, "06-F");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_ADM_TASK', 228+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_ADM_TASK(229) : pools.sql(9408) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 235+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(236) : pools.sql(9415) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_ADM_TASK', 247+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_ADM_TASK(248) : pools.sql(9427) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_ADM_TASK;
ALTER FUNCTION SP_I_ADM_TASK COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_ADM_TASK_NAME ==
CREATE OR REPLACE FUNCTION SP_U_ADM_TASK_NAME
(
     p_id VARCHAR2
   , p_adm_tsk_name VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_adm_tsk_name VARCHAR2(255);
   h_rec_user VARCHAR2(32);
   h_ieflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ADM_TASK_NAME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ADM_TASK_NAME'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ADM_TASK_NAME(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_adm_tsk_name || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ADM_TASK_NAME', 22+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ADM_TASK_NAME(23) : pools.sql(9466) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_ADM_TASK_NAME', 27+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_ADM_TASK_NAME(28) : pools.sql(9471) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ADM_TASK_NAME', 30+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_ADM_TASK_NAME(31) : pools.sql(9474) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_id := p_id;
   h_adm_tsk_name := p_adm_tsk_name;
   h_rec_user := p_rec_user;
   
   BEGIN
      UPDATE TMT_WBS SET
      ELEMENT_NAME = h_adm_tsk_name
      WHERE POOL_ID = (SELECT POOL_ID FROM TMT_WBS WHERE ELEMENT_ID = h_id);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ADM_TASK_NAME', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ADM_TASK_NAME(54) : pools.sql(9497) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ADM_TASK_NAME', 59+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ADM_TASK_NAME(60) : pools.sql(9503) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ADM_TASK_NAME', 66+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ADM_TASK_NAME(67) : pools.sql(9510) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ADM_TASK_NAME', 78+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ADM_TASK_NAME(79) : pools.sql(9522) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ADM_TASK_NAME;
ALTER FUNCTION SP_U_ADM_TASK_NAME COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_D_ADM_TASK ==
CREATE OR REPLACE FUNCTION SP_D_ADM_TASK
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_admin_summary_task_id VARCHAR2(32);
   h_admin_task_id VARCHAR2(32);
   h_status_id VARCHAR2(1);
   h_ieflag PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_ieflag001 PLS_INTEGER;
-- Declare Cursors
      CURSOR sel_admin_assigns IS
         SELECT TA.PROJECT_ID, TA.TASK_ID, TA.TSK_ASSIGNMENT_ID, TA.WORK_TODATE, W.PARENT_ID
         FROM TMT_WBS W, TASK_ASSIGNMENTS TA
         WHERE W.POOL_ID = h_admin_task_id
         AND TA.TASK_ID = W.ELEMENT_ID
         AND TA.PROJECT_ID = W.PROJECT_ID;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_ADM_TASK') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_ADM_TASK'), '-- ' || SYSTIMESTAMP, ':r := SP_D_ADM_TASK(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ADM_TASK', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_ADM_TASK(40) : pools.sql(9578) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) OR (p_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_ADM_TASK', 44+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_ADM_TASK(45) : pools.sql(9583) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ADM_TASK', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_ADM_TASK(48) : pools.sql(9586) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local Variables
   
   h_status_id := 'X';
   
   h_admin_summary_task_id := p_parent_id;
   h_admin_task_id := p_id;
   h_rec_user := p_rec_user;
   
   -- select pool_id of give admin task
   BEGIN
      SELECT POOL_ID
      INTO h_id
      FROM TMT_WBS
      WHERE PARENT_ID = h_admin_summary_task_id
      AND ELEMENT_ID = h_admin_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 73+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(74) : pools.sql(9612) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_ADM_TASK', 80+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_ADM_TASK(81) : pools.sql(9619) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_ADM_TASK', 83+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_ADM_TASK(84) : pools.sql(9622) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   --trace_printf("\n h_admin_summary_task_id:%s", h_admin_summary_task_id);
   --trace_printf("\n h_admin_task_id:%s", h_admin_task_id);
   --trace_printf("\n h_id:%s", h_id);
   
   -- Delete admin task at pool level
   BEGIN
      DELETE FROM TMT_WBS
      WHERE PARENT_ID = h_admin_summary_task_id
      AND ELEMENT_ID = h_admin_task_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(108) : pools.sql(9646) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TMT_WBS
      SET CHILD_COUNT = CHILD_COUNT - 1
      WHERE ELEMENT_ID = h_admin_summary_task_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 126+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(127) : pools.sql(9665) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- select all personal projects that contains deleted admin task (all resources under the given pool)
   
   OPEN sel_admin_assigns;
   
   LOOP
      FETCH sel_admin_assigns
      INTO h_project_id, h_task_id, h_tsk_assignment_id, h_work_todate, h_admin_summary_task_id;
      IF sel_admin_assigns
      %FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      If Coalesce(h_work_todate, 0) = 0 Then
         --          SC_STATE = FC_D_SUMMARY_TS(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
         
         h_ieflag001 := 0;
         PMO_DB2M.SC_STATE := FC_D_WBS_MSP(h_task_id, h_admin_summary_task_id, h_status_id, h_rec_user
                                  , h_ieflag001);
      Else
         BEGIN
            UPDATE TMT_WBS SET
            TYPE_ID = PMO_CNST.SC_INACTIVE_ADM_TASK
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 171+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(172) : pools.sql(9710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE sel_admin_assigns ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 186+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(187) : pools.sql(9725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_ADM_TASK', 192+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_ADM_TASK(193) : pools.sql(9731) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 199+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(200) : pools.sql(9738) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_ADM_TASK', 211+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_ADM_TASK(212) : pools.sql(9750) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_ADM_TASK;
ALTER FUNCTION SP_D_ADM_TASK COMPILE DEBUG;

/*************************************************************************************/

Prompt == FUNCTION FC_GET_ALL_POOLS ==
CREATE OR REPLACE FUNCTION FC_GET_ALL_POOLS
(
     sh_h_bitflag PLS_INTEGER
   , h_id IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_search_by_name_condition VARCHAR2(8000);
   h_fieldName VARCHAR2(255);
   h_search_option PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_xsql VARCHAR2(8000);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_ALL_POOLS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_ALL_POOLS'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_ALL_POOLS(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_id || ''''
          || ', ' || '''' || h_name || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ALL_POOLS', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_ALL_POOLS(28) : pools.sql(9794) : ' || SYSTIMESTAMP);
   END IF; 

   h_search_by_name_condition := '';
   If (h_name IS NULL) Then
      h_xsql := 'INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)';
      h_xsql := h_xsql || ' SELECT T.ELEMENT_ID, T.ELEMENT_NAME, T.LEVEL_ID, T.RANK, T.TYPE_ID ';
      h_xsql := h_xsql || ' FROM TMT_POOLS T ';
      
      If (h_id IS NULL) Then
         h_xsql := h_xsql || ' WHERE T.PARENT_ID IS NULL ';
         h_xsql := h_xsql || ' AND T.LEVEL_ID = 0 ';
         h_xsql := h_xsql || ' AND T.TYPE_ID = 11 ';
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'FC_GET_ALL_POOLS', 42+1) THEN 
            LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: FC_GET_ALL_POOLS(43) : pools.sql(9809) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_POOLS' || '''' || ' ID: ' || '''' || h_id || '''');
         END IF; 
         BEGIN 
            EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_POOLS' || ' WHERE ELEMENT_ID = :i' 
               INTO h_rec_status 
               Using h_id; 
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN 
               return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_id, False); 
         END;
         
         -- select rank of a given pool
         BEGIN
            SELECT RANK, LEVEL_ID
            INTO h_rank, h_level_id
            FROM TMT_POOLS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ALL_POOLS', 65+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ALL_POOLS(66) : pools.sql(9832) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --h_xsql.strcat(" WHERE (T.PARENT_ID = '");
         --h_xsql.strcat(h_id);
         --h_xsql.strcat("' ");
         --h_xsql.strcat(" OR T.ELEMENT_ID = '");
         --h_xsql.strcat(h_id);
         --h_xsql.strcat("') ");
         
         h_xsql := h_xsql || ' WHERE ((RANK LIKE ''';
         h_xsql := h_xsql || h_rank;
         h_xsql := h_xsql || ''' || ''%%'' ';
         h_xsql := h_xsql || ' AND T.PARENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''') OR T.ELEMENT_ID = ''';
         h_xsql := h_xsql || h_id;
         h_xsql := h_xsql || ''')';
      End If;
      h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
      
      BEGIN
         EXECUTE IMMEDIATE h_xsql;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ALL_POOLS', 100+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ALL_POOLS(101) : pools.sql(9867) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || h_xsql);
            END IF; 
         END;
      END;
   Else
      h_fieldName := 'T.ELEMENT_NAME(C)';
      h_search_option := FC_BUILD_POOL_SEARCH_CONDITION(h_fieldName, h_name);
      
      If (h_id IS NULL) Then
         h_xsql := 'INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)';
         h_xsql := h_xsql || ' SELECT T.ELEMENT_ID, T.ELEMENT_NAME, 0, T.RANK, T.TYPE_ID ';
         h_xsql := h_xsql || ' FROM TMT_POOLS T ';
         h_xsql := h_xsql || ' WHERE T.TYPE_ID = 11 ';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || 'AND (';
         h_xsql := h_xsql || h_name;
         h_xsql := h_xsql || ')';
      Else
         -- select rank of a given pool
         BEGIN
            SELECT RANK, LEVEL_ID
            INTO h_rank, h_level_id
            FROM TMT_POOLS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ALL_POOLS', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ALL_POOLS(132) : pools.sql(9898) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_xsql := 'INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)';
         h_xsql := h_xsql || ' SELECT T.ELEMENT_ID, T.ELEMENT_NAME, ';
         h_xsql := h_xsql || TO_CHAR(Coalesce(h_level_id, 0) + 1);
         h_xsql := h_xsql || ' AS LEVEL_ID. T.RANK, T.TYPE_ID ';
         h_xsql := h_xsql || ' FROM TMT_POOLS T ';
         h_xsql := h_xsql || ' WHERE RANK LIKE ''';
         h_xsql := h_xsql || h_rank;
         h_xsql := h_xsql || ''' || ''%%''';
         h_xsql := h_xsql || 'AND T.REC_STATUS NOT in (''D'', ''L'', ''A'') ';
         h_xsql := h_xsql || 'AND (';
         h_xsql := h_xsql || h_name;
         h_xsql := h_xsql || ')';
      End If;
      
      --trace_printf("\n h_xsql:%s", (char *) h_xsql);
      
      BEGIN
         EXECUTE IMMEDIATE h_xsql;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ALL_POOLS', 164+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ALL_POOLS(165) : pools.sql(9931) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || h_xsql);
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_CLEANS_POOL_BY_NAME(h_search_option);
      
      If (h_id IS NOT NULL) Then
         BEGIN
            INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)
            SELECT T.ELEMENT_ID, T.ELEMENT_NAME, T.LEVEL_ID, T.RANK, T.TYPE_ID
            FROM TMT_POOLS T
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ALL_POOLS', 188+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ALL_POOLS(189) : pools.sql(9955) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ALL_POOLS', 197+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_ALL_POOLS(198) : pools.sql(9964) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_ALL_POOLS;
ALTER FUNCTION FC_GET_ALL_POOLS COMPILE DEBUG;

/*************************************************************************************/

-- This function returns all resources under assigned pools

/*************************************************************************************/

Prompt == FUNCTION FC_GET_POOL_RESOURCES ==
CREATE OR REPLACE FUNCTION FC_GET_POOL_RESOURCES
(
     sh_h_flat_list PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , h_name IN OUT NOCOPY VARCHAR2
   , h_family_name IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_flat_list PLS_INTEGER := sh_h_flat_list;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_search_option PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_fieldName VARCHAR2(255);
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
   h_xsql VARCHAR2(8000);
-- Declare Cursors
      CURSOR assigned_pools IS
         SELECT ELEMENT_ID, RANK, LEVEL_ID
         FROM PMO_TMP_POOL_RES_TREE;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_POOL_RESOURCES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_POOL_RESOURCES'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_POOL_RESOURCES(' || 
       COALESCE(TO_CHAR(sh_h_flat_list), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_name || ''''
          || ', ' || '''' || h_family_name || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_POOL_RESOURCES', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_POOL_RESOURCES(35) : pools.sql(10013) : ' || SYSTIMESTAMP);
   END IF; 

   h_level_id := 1;
   
   -- insert resources under each pool
   
   OPEN assigned_pools;
   
   LOOP
      FETCH assigned_pools
      INTO h_element_id, h_rank, h_level_id;
      IF assigned_pools
      %FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      --trace_printf("\n h_rank:%s", h_rank);
      --trace_printf("\n h_element_id:%s", h_element_id);
      --trace_printf("\n h_level_id:%d", h_level_id);
      
      If Coalesce(h_bitflag, 0) = PMO_CNST.SC_POOL_RESOURCES Then
         BEGIN
            INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)
            SELECT P.ELEMENT_ID, P.ELEMENT_NAME, h_level_id + 1, P.RANK, P.TYPE_ID
            FROM TMT_POOLS P
            WHERE P.RANK LIKE h_rank || '%'
            AND P.TYPE_ID = PMO_CNST.SC_RESOURCE
            AND P.PARENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_POOL_RESOURCES', 79+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_POOL_RESOURCES(80) : pools.sql(10058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_xsql := 'INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID) ';
         h_xsql := h_xsql || ' SELECT P.ELEMENT_ID, P.ELEMENT_NAME, ';
         h_xsql := h_xsql || TO_CHAR(h_level_id);
         h_xsql := h_xsql || ' + 1, P.RANK, P.TYPE_ID ';
         h_xsql := h_xsql || ' FROM TMT_POOLS P, RESOURCES R ';
         h_xsql := h_xsql || ' WHERE P.RANK LIKE ''';
         h_xsql := h_xsql || h_rank;
         h_xsql := h_xsql || ''' || ''%%''';
         h_xsql := h_xsql || ' AND P.TYPE_ID = ';
         h_xsql := h_xsql || TO_CHAR(PMO_CNST.SC_RESOURCE);
         h_xsql := h_xsql || ' AND R.RESOURCE_ID = P.ELEMENT_ID ';
         
         If Coalesce(h_bitflag, 0) = PMO_CNST.SC_RES_BY_FULLNAME Then
            h_xsql := h_xsql || 
                    ' AND R.CALENDAR_ID IS NOT NuLL AND R.ACTIVE <> ''0'' AND (R.DISCONTINUE_DATE IS NULL OR R.DISCONTINUE_DATE >= TO_DATE(SYSDATE)) AND R.EFFECTIVE_DATE <= TO_DATE(SYSDATE) AND UPPER(R.FULL_NAME) LIKE UPPER(''';
            h_xsql := h_xsql || h_name;
            h_xsql := h_xsql || '%%'')';
         ElsIf Coalesce(h_bitflag, 0) = PMO_CNST.SC_RES_BY_GIVEN_FAMILY_NAME Then
            h_xsql := h_xsql || 
                    ' AND R.CALENDAR_ID IS NOT NuLL AND R.ACTIVE <> ''0'' AND (R.DISCONTINUE_DATE IS NULL OR R.DISCONTINUE_DATE >= TO_DATE(SYSDATE)) AND R.EFFECTIVE_DATE <= TO_DATE(SYSDATE) ';
            
            If (h_name IS NOT NULL) AND (h_family_name IS NOT NULL) Then
               h_xsql := h_xsql || ' AND UPPER(R.FIRST_NAME) LIKE UPPER(''';
               h_xsql := h_xsql || h_name;
               h_xsql := h_xsql || '%%'')';
               h_xsql := h_xsql || ' AND UPPER(R.LAST_NAME) LIKE UPPER(''';
               h_xsql := h_xsql || h_family_name;
               h_xsql := h_xsql || '%%'')';
            ElsIf (h_name IS NOT NULL) AND (h_family_name IS NULL) Then
               h_xsql := h_xsql || ' AND UPPER(R.FIRST_NAME) LIKE UPPER(''';
               h_xsql := h_xsql || h_name;
               h_xsql := h_xsql || '%%'')';
            ElsIf (h_name IS NULL) AND (h_family_name IS NOT NULL) Then
               h_xsql := h_xsql || ' AND UPPER(R.LAST_NAME) LIKE UPPER(''';
               h_xsql := h_xsql || h_family_name;
               h_xsql := h_xsql || '%%'')';
            End If;
         End If;
         
         --trace_printf("\n h_xsql:%s",(char *) h_xsql);
         
         BEGIN
            EXECUTE IMMEDIATE h_xsql;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_POOL_RESOURCES', 137+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_POOL_RESOURCES(138) : pools.sql(10116) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || h_xsql);
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE assigned_pools ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_POOL_RESOURCES', 153+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_POOL_RESOURCES(154) : pools.sql(10132) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_flat_list, 0) = 1 Then
      BEGIN
         DELETE FROM PMO_TMP_POOL_RES_TREE
         WHERE TYPE_ID = PMO_CNST.SC_POOLS;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_POOL_RESOURCES', 172+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_POOL_RESOURCES(173) : pools.sql(10151) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_POOL_RESOURCES', 180+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_POOL_RESOURCES(181) : pools.sql(10159) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_POOL_RESOURCES;
ALTER FUNCTION FC_GET_POOL_RESOURCES COMPILE DEBUG;

/*************************************************************************************/

-- This function returns all pools that are assigned to the selected resource.

/*************************************************************************************/

Prompt == FUNCTION FC_GET_ASSIGNED_POOLS ==
CREATE OR REPLACE FUNCTION FC_GET_ASSIGNED_POOLS
(
     h_name IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_xsql VARCHAR2(8000);
   h_search_option PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_fieldName VARCHAR2(255);
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_ASSIGNED_POOLS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_ASSIGNED_POOLS'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_ASSIGNED_POOLS(' || 
       '''' || h_name || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ASSIGNED_POOLS', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_ASSIGNED_POOLS(22) : pools.sql(10195) : ' || SYSTIMESTAMP);
   END IF; 

   h_level_id := 1;
   
   If (h_name IS NULL) Then
      BEGIN
         INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID)
         SELECT T.ELEMENT_ID, T.ELEMENT_NAME, T.LEVEL_ID, T.RANK, T.TYPE_ID
         FROM TMT_POOLS T, RESOURCE_ASSIGNMENTS RA
         WHERE RA.ELEMENT_ID = T.ELEMENT_ID
         AND T.PARENT_ID IS NuLL
         AND T.LEVEL_ID = 0
         AND T.TYPE_ID = 11
         AND T.REC_STATUS NOT IN ('D', 'L', 'A')
         AND RA.RESOURCE_ID = h_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ASSIGNED_POOLS', 47+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ASSIGNED_POOLS(48) : pools.sql(10221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_fieldName := 'P.ELEMENT_NAME(C)';
      h_search_option := FC_BUILD_POOL_SEARCH_CONDITION(h_fieldName, h_name);
      
      h_xsql := 'INSERT INTO PMO_TMP_POOL_RES_TREE(ELEMENT_ID, ELEMENT_NAME, LEVEL_ID, RANK, TYPE_ID) ';
      h_xsql := h_xsql || ' SELECT P.ELEMENT_ID, P.ELEMENT_NAME, 0 AS LEVEL_ID. P.RANK, P.TYPE_ID ';
      h_xsql := h_xsql || ' FROM RESOURCE_ASSIGNMENTS RA, TMT_POOLS P ';
      h_xsql := h_xsql || ' WHERE RA.ELEMENT_ID = P.ELEMENT_ID  ';
      h_xsql := h_xsql || ' AND P.TYPE_ID = 11 ';
      h_xsql := h_xsql || ' AND P.REC_STATUS <> ''D'' ';
      h_xsql := h_xsql || ' AND RA.RESOURCE_ID = ''';
      h_xsql := h_xsql || h_rec_user;
      h_xsql := h_xsql || '''';
      
      h_xsql := h_xsql || ' AND (';
      h_xsql := h_xsql || h_name;
      h_xsql := h_xsql || ')';
      
      --trace_printf("\n h_xsql:%s", (char *)h_xsql);
      
      BEGIN
         EXECUTE IMMEDIATE h_xsql;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_GET_ASSIGNED_POOLS', 83+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_GET_ASSIGNED_POOLS(84) : pools.sql(10257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || h_xsql);
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_CLEANS_POOL_BY_NAME(h_search_option);
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_ASSIGNED_POOLS', 94+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_ASSIGNED_POOLS(95) : pools.sql(10268) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_ASSIGNED_POOLS;
ALTER FUNCTION FC_GET_ASSIGNED_POOLS COMPILE DEBUG;

/******************************************************************************************
p_bitflag = SC_ALL_POOLS                -> all pools        -> second level or open pool -> returns pools and resources under the selected pool
p_bitflag = SC_ASSIGNED_POOLS           -> assigned pools   -> first level -> just returns pools
p_bitflag = SC_POOL_RESOURCES           -> list of the resources under assigned pools
p_bitflag = SC_RES_BY_FULLNAME          -> list of the resources under assigned pools with specified fullName in this case p_keywords contains fullname
p_bitflag = SC_RES_BY_GIVEN_FAMILY_NAME -> list of the resources under assigned pools with specified given name and familyName
                                            in this case p_keywords contains givenName and p_attribute_ids contains familyName
******************************************************************************************/

Prompt == FUNCTION SP_SELECT_POOLS ==
CREATE OR REPLACE FUNCTION SP_SELECT_POOLS
(
     p_id VARCHAR2
   , p_name VARCHAR2
   , p_family_name VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_status VARCHAR2(1);
   h_rec_user VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_search_by_name_condition VARCHAR2(8000);
   h_name VARCHAR2(8000);
   h_family_name VARCHAR2(8000);
   h_fieldName VARCHAR2(255);
   h_search_option PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_flat_list PLS_INTEGER := 0;
   h_xsql VARCHAR2(8000);
   null_return LONG;
-- Declare Cursors
      -- Resultset Cursor Declaration
      pool_recset01 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SELECT_POOLS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SELECT_POOLS'), '-- ' || SYSTIMESTAMP, ':r := SP_SELECT_POOLS(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_family_name || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SELECT_POOLS', 40+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SELECT_POOLS(41) : pools.sql(10326) : ' || SYSTIMESTAMP);
   END IF; 

   h_search_by_name_condition := '';
   h_name := '';
   h_fieldName := '';
   h_rec_user := '';
   h_search_option := 0;
   h_level_id := 0;
   
   h_id := p_id;
   h_name := p_name;
   h_family_name := p_family_name;
   h_rec_user := p_rec_user;
   h_bitflag := p_bitflag;
   
   h_temp_sc_state := 0;
   
   If (h_id IS NOT NULL) Then
      h_bitflag := PMO_CNST.SC_ALL_POOLS;
   End If;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_POOL_RES_TREE';
   PMO_DB2M.SC_STATE := 0;
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_ASSIGNED_POOLS Then
      null_return := FC_GET_ASSIGNED_POOLS(h_name, h_rec_user);
   ElsIf Coalesce(h_bitflag, 0) = PMO_CNST.SC_ALL_POOLS Then
      null_return := FC_GET_ALL_POOLS(h_bitflag, h_id, h_name, h_rec_user);
   ElsIf (Coalesce(h_bitflag, 0) = PMO_CNST.SC_POOL_RESOURCES) OR (Coalesce(
                h_bitflag, 0) = PMO_CNST.SC_RES_BY_FULLNAME) OR (Coalesce(
                h_bitflag, 0) = PMO_CNST.SC_RES_BY_GIVEN_FAMILY_NAME) Then
      h_flat_list := 1;
      h_search_by_name_condition := NULL;
      null_return := FC_GET_ASSIGNED_POOLS(h_search_by_name_condition, h_rec_user);
      null_return := FC_GET_POOL_RESOURCES(h_flat_list, h_bitflag, h_name, h_family_name, h_rec_user);
   End If;
   
   OPEN pool_recset01 FOR SELECT P.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
   T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_CONTACTS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
   T.DELETED_COUNT,
   CASE WHEN T.TYPE_ID = 13 THEN (SELECT REC_STATUS FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE T.REC_STATUS END AS REC_STATUS,
   T.DESCRIPTION,
   T.DOMAIN_ID, (SELECT ELEMENT_NAME FROM TMT_DOMAINS WHERE ELEMENT_ID = T.DOMAIN_ID) AS DOMAIN_NAME,
   T.USER_GROUP_ID, COALESCE((SELECT ELEMENT_NAME FROM TMT_POOLS WHERE ELEMENT_ID = T.USER_GROUP_ID),'') AS USER_GROUP_NAME,
   T.CLEARANCE_LEVEL_1, T.CLEARANCE_LEVEL_2, T.CLEARANCE_LEVEL_3, T.CLEARANCE_LEVEL_4, T.CLEARANCE_LEVEL_5,
   (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.MANAGER_ID) AS RESPONSIBLE,
   CASE WHEN T.TYPE_ID = 13 THEN (SELECT TITLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID) ELSE '' END AS TITLE,
   CASE WHEN T.TYPE_ID = 13 THEN
   (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
   (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
   (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND
   PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))), ', '), 
   (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
   (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = T.ELEMENT_ID AND
   PARENT_ID = (SELECT PRIMARY_ROLE FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)))))
   ELSE '' END AS COMPETENCY, T.REC_USER,
   CASE WHEN T.TYPE_ID = 13 THEN (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = (SELECT REC_USER FROM RESOURCES WHERE RESOURCE_ID = T.ELEMENT_ID)) ELSE (SELECT FULL_NAME FROM RESOURCES WHERE RESOURCE_ID = T.REC_USER) END AS REC_USER_NAME, T.PUBLIC_FLAG
   FROM TMT_POOLS T, PMO_TMP_POOL_RES_TREE P
   WHERE T.ELEMENT_ID = P.ELEMENT_ID
   ORDER BY RANK;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SELECT_POOLS', 103+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SELECT_POOLS(104) : pools.sql(10389) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN pool_recset01;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SELECT_POOLS', 111+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SELECT_POOLS(112) : pools.sql(10397) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SELECT_POOLS', 123+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SELECT_POOLS(124) : pools.sql(10409) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SELECT_POOLS;
ALTER FUNCTION SP_SELECT_POOLS COMPILE DEBUG;

