/*****************************************************************************************/

-- p_type and p_finish_date are not used for this stored procedure.

-- (Client needs standard interface).

-- p_start_date only contains 4 digits of the year.

/*****************************************************************************************/

Prompt == FUNCTION SP_S_RECONCILIATION ==
CREATE OR REPLACE FUNCTION SP_S_RECONCILIATION
(
     p_project_id VARCHAR2
   , p_type PLS_INTEGER
   , p_start_date VARCHAR2
   , p_finish_date VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_start_date VARCHAR2(32);
   h_budget_year PLS_INTEGER := 0;
   h_rollup_all PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
-- Declare Cursors
      -- Resultset Cursor Declaration
      S_RECON PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RECONCILIATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RECONCILIATION'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RECONCILIATION(' || 
       '''' || p_project_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || p_start_date || ''''
          || ', ' || '''' || p_finish_date || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECONCILIATION', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RECONCILIATION(32) : reconciliation.sql(43) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_project_id := p_project_id;
   h_start_date := p_start_date;
   h_rec_user := h_rec_user;
   
   If (h_project_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RECONCILIATION', 42+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RECONCILIATION(43) : reconciliation.sql(54) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECONCILIATION', 45+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RECONCILIATION(46) : reconciliation.sql(57) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If (h_start_date IS NULL) Then
      h_budget_year := 0;
   Else
      h_budget_year := TO_NUMBER(h_start_date);
   End If;
   
   h_rollup_all := 0;
   h_resource_id := NULL;
   PMO_DB2M.SC_STATE := FC_ROLLUP_COST_CENTER(h_project_id, h_resource_id, h_rollup_all, h_rec_user);
   
   OPEN S_RECON FOR SELECT PR.ELEMENT_ID, C.ELEMENT_NAME AS COST_CENTER_NAME, FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   ((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) AS REGULAR_QUANTITY, PR.REGULAR_RATE,
   ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) AS SPECIAL_QUANTITY, PR.SPECIAL_RATE,
   PR.TAX1, PR.TAX2,
   (PR.ADJ_REGULAR_QUANTITY / 60.0) AS ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
   (PR.ADJ_SPECIAL_QUANTITY / 60.0) AS ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
   PR.ADJ_TAX1, PR.ADJ_TAX2,
   (PR.LEDGER_REGULAR_QUANTITY / 60.0) AS LEDGER_REGULAR_QUANTITY, PR.LEDGER_REGULAR_RATE,
   (PR.LEDGER_SPECIAL_QUANTITY / 60.0) AS LEDGER_SPECIAL_QUANTITY, PR.LEDGER_SPECIAL_RATE,
   PR.LEDGER_TAX1, PR.LEDGER_TAX2,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   TC.ELEMENT_NAME AS CONTRACT_NAME, COALESCE(TC.CONTRACT_TYPE, 0) AS CONTRACT_TYPE, 
   COALESCE(TC.CHARGED_AMOUNT, 0.00) AS CONTRACT_CHARGED_AMOUNT, 
   COALESCE(CB.CHARGE_AMOUNT, 0.00) AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PD.BUDGET_SETTING_BITS
   FROM PROJECT_RECONCILIATION PR
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
   JOIN RESOURCES R ON R.RESOURCE_ID = TA.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = TA.LABOR_COST_CC_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
   JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = W.PROJECT_ID
   LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
   LEFT JOIN CONTRACT_BREAKDOWN CB ON CB.CONTRACT_ID = TC.ELEMENT_ID
   AND CB.CHARGE_DATE = FC.START_DATE
   WHERE PR.PROJECT_ID = h_project_id
   AND PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS
   AND PR.TYPE_ID IN (PMO_CNST.SC_EXPENSE_LABOR, PMO_CNST.SC_CAPITAL_LABOR)
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   UNION ALL
   SELECT PR.ELEMENT_ID, C.ELEMENT_NAME AS COST_CENTER_NAME, FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   ((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) AS REGULAR_QUANTITY, PR.REGULAR_RATE,
   ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) AS SPECIAL_QUANTITY, PR.SPECIAL_RATE,
   PR.TAX1, PR.TAX2,
   (PR.ADJ_REGULAR_QUANTITY / 60.0) AS ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
   (PR.ADJ_SPECIAL_QUANTITY / 60.0) AS ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
   PR.ADJ_TAX1, PR.ADJ_TAX2,
   (PR.LEDGER_REGULAR_QUANTITY / 60.0) AS LEDGER_REGULAR_QUANTITY, PR.LEDGER_REGULAR_RATE,
   (PR.LEDGER_SPECIAL_QUANTITY / 60.0) AS LEDGER_SPECIAL_QUANTITY, PR.LEDGER_SPECIAL_RATE,
   PR.LEDGER_TAX1, PR.LEDGER_TAX2,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   TC.ELEMENT_NAME AS CONTRACT_NAME, COALESCE(TC.CONTRACT_TYPE, 0) AS CONTRACT_TYPE, 
   COALESCE(TC.CHARGED_AMOUNT, 0.00) AS CONTRACT_CHARGED_AMOUNT, 
   COALESCE(CB.CHARGE_AMOUNT, 0.00) AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PD.BUDGET_SETTING_BITS
   FROM PROJECT_RECONCILIATION PR
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
   JOIN RESOURCES R ON R.RESOURCE_ID = TA.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = TA.LABOR_REVENUE_CC_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
   JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = W.PROJECT_ID
   LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
   LEFT JOIN CONTRACT_BREAKDOWN CB ON CB.CONTRACT_ID = TC.ELEMENT_ID
   AND CB.CHARGE_DATE = FC.START_DATE
   WHERE PR.PROJECT_ID = h_project_id
   AND PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS
   AND PR.TYPE_ID = PMO_CNST.SC_BENEFIT_LABOR
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   UNION ALL
   SELECT PR.ELEMENT_ID, C.ELEMENT_NAME AS COST_CENTER_NAME, FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   PR.REGULAR_QUANTITY, PR.REGULAR_RATE,
   PR.SPECIAL_QUANTITY, PR.SPECIAL_RATE,
   PR.TAX1, PR.TAX2,
   PR.ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
   PR.ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
   PR.ADJ_TAX1, PR.ADJ_TAX2,
   PR.LEDGER_REGULAR_QUANTITY, PR.LEDGER_REGULAR_RATE,
   PR.LEDGER_SPECIAL_QUANTITY, PR.LEDGER_SPECIAL_RATE,
   PR.LEDGER_TAX1, PR.LEDGER_TAX2,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   '' AS CONTRACT_NAME, 0 AS CONTRACT_TYPE, 
   0.00 AS CONTRACT_CHARGED_AMOUNT, 
   0.00 AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PD.BUDGET_SETTING_BITS
   FROM PROJECT_RECONCILIATION PR
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   JOIN TMT_COSTS ST ON ST.ELEMENT_ID = PR.FOREIGN_ID AND TO_DATE(SubStr(ST.INVOICE_DATETIME, 1, 10)) = PR.FOREIGN_DATE
   JOIN RESOURCES R ON R.RESOURCE_ID = ST.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = ST.CHARGECODE_ID
   JOIN BUDGETS B ON B.PROJECT_ID = ST.PROJECT_ID AND B.BUDGET_ID = ST.BUDGET_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = B.PROJECT_ID AND W.ELEMENT_ID = B.ELEMENT_ID
   JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = W.PROJECT_ID
   WHERE PR.PROJECT_ID = h_project_id
   AND PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_COSTS            
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   UNION ALL
   SELECT PR.ELEMENT_ID, C.ELEMENT_NAME AS COST_CENTER_NAME, FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   PR.REGULAR_QUANTITY, PR.REGULAR_RATE,
   PR.SPECIAL_QUANTITY, PR.SPECIAL_RATE,
   PR.TAX1, PR.TAX2,
   PR.ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
   PR.ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
   PR.ADJ_TAX1, PR.ADJ_TAX2,
   PR.LEDGER_REGULAR_QUANTITY, PR.LEDGER_REGULAR_RATE,
   PR.LEDGER_SPECIAL_QUANTITY, PR.LEDGER_SPECIAL_RATE,
   PR.LEDGER_TAX1, PR.LEDGER_TAX2,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   '' AS CONTRACT_NAME, 0 AS CONTRACT_TYPE, 
   0.00 AS CONTRACT_CHARGED_AMOUNT, 
   0.00 AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PD.BUDGET_SETTING_BITS
   FROM PROJECT_RECONCILIATION PR
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   JOIN BUDGETS B ON B.BUDGET_ID = PR.FOREIGN_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = B.CHARGECODE_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = B.PROJECT_ID AND W.ELEMENT_ID = B.ELEMENT_ID
   JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = W.PROJECT_ID
   JOIN RESOURCE_ASSIGNMENTS RA ON RA.PROJECT_ID = W.PROJECT_ID 
   AND RA.ELEMENT_ID = W.ELEMENT_ID AND BITAND(RA.PARTICIPANT, PMO_CNST.SC_MANAGER_RESPONSIBLE) > 0
   JOIN RESOURCES R ON R.RESOURCE_ID = RA.RESOURCE_ID
   WHERE PR.PROJECT_ID = h_project_id
   AND PR.FOREIGN_TABLE IN (PMO_CNST.SC_TABLE_BUDGETS, PMO_CNST.SC_TABLE_PBB)            
   AND (h_budget_year = 0 OR FC.FISCAL_YEAR = h_budget_year)
   ORDER BY FISCAL_YEAR, COST_CENTER_NAME, FISCAL_YEAR, FISCAL_QUARTER, FISCAL_MONTH,
   TYPE_ID, CHARGECODE_NAME, FULL_NAME, ELEMENT_NAME, START_DATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECONCILIATION', 228+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_RECONCILIATION(229) : reconciliation.sql(240) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN S_RECON;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECONCILIATION', 236+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECONCILIATION(237) : reconciliation.sql(248) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECONCILIATION', 248+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECONCILIATION(249) : reconciliation.sql(260) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RECONCILIATION;
ALTER FUNCTION SP_S_RECONCILIATION COMPILE DEBUG;

/*****************************************************************************************/

-- p_type:

-- 1: Portfolio IDs

-- 2: Project IDs

/*****************************************************************************************/

Prompt == FUNCTION SP_S_RECON_PIVOT ==
CREATE OR REPLACE FUNCTION SP_S_RECON_PIVOT
(
     p_project_ids VARCHAR2
   , p_type PLS_INTEGER
   , p_start_date VARCHAR2
   , p_finish_date VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_ids VARCHAR2(8192);
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_start_date VARCHAR2(32);
   h_finish_date VARCHAR2(32);
   h_rollup_all PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_name VARCHAR2(255);
   h_rank VARCHAR2(200);
   h_count PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_MASTER_PRJ IS
         SELECT W.RANK
         FROM TMT_PORTFOLIOS P
         JOIN TMT_WBS W ON W.ELEMENT_ID = P.ELEMENT_ID
         WHERE P.RANK LIKE h_rank || '%'					
         AND W.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         AND W.REC_STATUS <> 'D'
         AND W.SUBPROJECT_COUNT > 0;

      CURSOR C_RECON_PRJ IS
         SELECT ELEMENT_ID
         FROM PMO_TMP_PROJECTS_01;

      CURSOR C_PRJ_CLIENT IS
         SELECT C.ELEMENT_NAME
         FROM PROJECT_COST_CENTERS PCC
         JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PCC.COST_CENTER_ID
         WHERE PCC.PROJECT_ID = h_project_id
         ORDER BY PCC.WS_REVENUE_PERCENT DESC;

      -- Resultset Cursor Declaration
      S_RECON_PIVOT PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_RECON_PIVOT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_RECON_PIVOT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_RECON_PIVOT(' || 
       '''' || p_project_ids || ''''
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || p_start_date || ''''
          || ', ' || '''' || p_finish_date || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECON_PIVOT', 57+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_RECON_PIVOT(58) : reconciliation.sql(342) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_project_ids := p_project_ids;
   h_table_type := p_type;
   h_start_date := p_start_date;
   h_finish_date := p_finish_date;
   h_rec_user := h_rec_user;
   
   If ((h_project_ids IS NULL)) OR (h_start_date IS NULL) OR (h_finish_date IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RECON_PIVOT', 70+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RECON_PIVOT(71) : reconciliation.sql(355) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECON_PIVOT', 73+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RECON_PIVOT(74) : reconciliation.sql(358) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROJECTS_01';
   PMO_DB2M.SC_STATE := 0;
   
   h_rollup_all := 0;
   h_resource_id := NULL;
   
   h_total := 0;
   h_total := Trunc((Coalesce(Length(h_project_ids), 0) + 1) / 32);
   If Coalesce(h_total, 0) < 1 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_RECON_PIVOT', 89+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_RECON_PIVOT(90) : reconciliation.sql(374) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECON_PIVOT', 92+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_RECON_PIVOT(93) : reconciliation.sql(377) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_count := 0;
   WHILE Coalesce(h_count, 0) < Coalesce(h_total, 0) LOOP
      h_project_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_project_ids);
      
      -- project id list
      If Coalesce(h_table_type, 0) = 2 Then
         BEGIN
            SELECT RTRIM(RANK) INTO h_rank
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 113+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(114) : reconciliation.sql(398) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME)
            SELECT LEVEL_ID, ELEMENT_ID, ELEMENT_NAME
            FROM TMT_WBS
            WHERE RANK LIKE h_rank || '%'
            AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
            AND REC_STATUS <> 'D';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 135+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(136) : reconciliation.sql(420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT RTRIM(RANK) INTO h_rank
            FROM TMT_PORTFOLIOS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 150+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(151) : reconciliation.sql(435) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME)
            SELECT W.LEVEL_ID, W.ELEMENT_ID, W.ELEMENT_NAME
            FROM TMT_PORTFOLIOS P
            JOIN TMT_WBS W ON W.ELEMENT_ID = P.ELEMENT_ID
            WHERE P.RANK LIKE h_rank || '%'					
            AND W.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
            AND W.REC_STATUS <> 'D'
            AND W.SUBPROJECT_COUNT = 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 174+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(175) : reconciliation.sql(459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN C_MASTER_PRJ;
         
         LOOP
            FETCH C_MASTER_PRJ INTO h_rank;
            IF C_MASTER_PRJ%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               EXIT;
            End If;
            
            BEGIN
               INSERT INTO PMO_TMP_PROJECTS_01 (LEVEL_ID, ELEMENT_ID, ELEMENT_NAME)
               SELECT W.LEVEL_ID, W.ELEMENT_ID, W.ELEMENT_NAME
               FROM TMT_WBS W
               WHERE W.RANK LIKE h_rank || '%'
               AND W.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
               AND W.REC_STATUS <> 'D'
               AND NOT EXISTS (SELECT T.ELEMENT_ID 
               FROM PMO_TMP_PROJECTS_01 T
               WHERE T.ELEMENT_ID = W.ELEMENT_ID);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(214) : reconciliation.sql(498) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         BEGIN
            CLOSE C_MASTER_PRJ ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 226+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(227) : reconciliation.sql(511) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_count := Coalesce(h_count, 0) + 1;
   END LOOP;
   
   OPEN C_RECON_PRJ;
   
   LOOP
      FETCH C_RECON_PRJ INTO h_project_id;
      IF C_RECON_PRJ%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         EXIT;
      End If;
      
      PMO_DB2M.SC_STATE := FC_ROLLUP_COST_CENTER(h_project_id, h_resource_id, h_rollup_all, h_rec_user);
      
      OPEN C_PRJ_CLIENT;
      
      FETCH C_PRJ_CLIENT INTO h_name;
      IF C_PRJ_CLIENT%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
         BEGIN
            UPDATE PMO_TMP_PROJECTS_01
            SET RANK = h_name
            WHERE ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 275+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(276) : reconciliation.sql(560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      BEGIN
         CLOSE C_PRJ_CLIENT ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 288+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(289) : reconciliation.sql(573) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   BEGIN
      CLOSE C_RECON_PRJ ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 301+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(302) : reconciliation.sql(586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_TMP_PROJECTS_01 P
      SET P.PROJECT_FLAG = (SELECT PD.BUDGET_SETTING_BITS FROM PROJECT_DETAILS PD
      WHERE PD.PROJECT_ID = P.ELEMENT_ID);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 320+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(321) : reconciliation.sql(605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN S_RECON_PIVOT FOR SELECT PRJ.ELEMENT_NAME AS PROJECT_NAME, PRJ.RANK AS CLIENT_NAME, C.ELEMENT_NAME AS COST_CENTER_NAME, 
   FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   (((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) * PR.REGULAR_RATE
   + ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) * PR.SPECIAL_RATE
   + PR.TAX1 + PR.TAX2) AS PMO_AMOUNT,
   ((PR.ADJ_REGULAR_QUANTITY / 60.0) * PR.ADJ_REGULAR_RATE
   + (PR.ADJ_SPECIAL_QUANTITY / 60.0) * PR.ADJ_SPECIAL_RATE + PR.ADJ_TAX1 + PR.ADJ_TAX2) AS ADJ_AMOUNT,
   ((PR.LEDGER_REGULAR_QUANTITY / 60.0) * PR.LEDGER_REGULAR_RATE
   + (PR.LEDGER_SPECIAL_QUANTITY / 60.0) * PR.LEDGER_SPECIAL_RATE
   + PR.LEDGER_TAX1 + PR.LEDGER_TAX2) AS LEDGER_AMOUNT,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   TC.ELEMENT_NAME AS CONTRACT_NAME, COALESCE(TC.CONTRACT_TYPE, 0) AS CONTRACT_TYPE, 
   COALESCE(TC.CHARGED_AMOUNT, 0.00) AS CONTRACT_CHARGED_AMOUNT, 
   CASE WHEN TC.CONTRACT_TYPE = 6702 THEN COALESCE(TC.MAX_AMOUNT, 0.00) 
   ELSE COALESCE(CB.CHARGE_AMOUNT, 0.00) END AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PRJ.PROJECT_FLAG AS BUDGET_SETTING_BITS
   FROM PMO_TMP_PROJECTS_01 PRJ
   JOIN PROJECT_RECONCILIATION PR ON PR.PROJECT_ID = PRJ.ELEMENT_ID
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
   JOIN RESOURCES R ON R.RESOURCE_ID = TA.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = TA.LABOR_COST_CC_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
   LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
   LEFT JOIN CONTRACT_BREAKDOWN CB ON CB.CONTRACT_ID = TC.ELEMENT_ID
   AND CB.CHARGE_DATE = FC.START_DATE
   WHERE PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS
   AND PR.TYPE_ID IN (PMO_CNST.SC_EXPENSE_LABOR, PMO_CNST.SC_CAPITAL_LABOR)
   AND PR.FOREIGN_DATE BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10))
   UNION ALL
   SELECT PRJ.ELEMENT_NAME AS PROJECT_NAME, PRJ.RANK AS CLIENT_NAME, C.ELEMENT_NAME AS COST_CENTER_NAME, 
   FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   (((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) * PR.REGULAR_RATE
   + ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) * PR.SPECIAL_RATE
   + PR.TAX1 + PR.TAX2) AS PMO_AMOUNT,
   ((PR.ADJ_REGULAR_QUANTITY / 60.0) * PR.ADJ_REGULAR_RATE
   + (PR.ADJ_SPECIAL_QUANTITY / 60.0) * PR.ADJ_SPECIAL_RATE + PR.ADJ_TAX1 + PR.ADJ_TAX2) AS ADJ_AMOUNT,
   ((PR.LEDGER_REGULAR_QUANTITY / 60.0) * PR.LEDGER_REGULAR_RATE
   + (PR.LEDGER_SPECIAL_QUANTITY / 60.0) * PR.LEDGER_SPECIAL_RATE
   + PR.LEDGER_TAX1 + PR.LEDGER_TAX2) AS LEDGER_AMOUNT,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   TC.ELEMENT_NAME AS CONTRACT_NAME, COALESCE(TC.CONTRACT_TYPE, 0) AS CONTRACT_TYPE, 
   COALESCE(TC.CHARGED_AMOUNT, 0.00) AS CONTRACT_CHARGED_AMOUNT, 
   CASE WHEN TC.CONTRACT_TYPE = 6702 THEN COALESCE(TC.MAX_AMOUNT, 0.00) 
   ELSE COALESCE(CB.CHARGE_AMOUNT, 0.00) END AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PRJ.PROJECT_FLAG AS BUDGET_SETTING_BITS
   FROM PMO_TMP_PROJECTS_01 PRJ
   JOIN PROJECT_RECONCILIATION PR ON PR.PROJECT_ID = PRJ.ELEMENT_ID
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
   JOIN RESOURCES R ON R.RESOURCE_ID = TA.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = TA.LABOR_REVENUE_CC_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
   LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
   LEFT JOIN CONTRACT_BREAKDOWN CB ON CB.CONTRACT_ID = TC.ELEMENT_ID
   AND CB.CHARGE_DATE = FC.START_DATE
   WHERE PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS
   AND PR.TYPE_ID = PMO_CNST.SC_BENEFIT_LABOR
   AND PR.FOREIGN_DATE BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10))
   UNION ALL
   SELECT PRJ.ELEMENT_NAME AS PROJECT_NAME, PRJ.RANK AS CLIENT_NAME, C.ELEMENT_NAME AS COST_CENTER_NAME, 
   FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   (PR.REGULAR_QUANTITY * PR.REGULAR_RATE
   + PR.SPECIAL_QUANTITY * PR.SPECIAL_RATE + PR.TAX1 + PR.TAX2) AS PMO_AMOUNT,
   (PR.ADJ_REGULAR_QUANTITY * PR.ADJ_REGULAR_RATE
   + PR.ADJ_SPECIAL_QUANTITY * PR.ADJ_SPECIAL_RATE + PR.ADJ_TAX1 + PR.ADJ_TAX2) AS ADJ_AMOUNT,
   (PR.LEDGER_REGULAR_QUANTITY * PR.LEDGER_REGULAR_RATE 
   + PR.LEDGER_SPECIAL_QUANTITY * PR.LEDGER_SPECIAL_RATE + PR.LEDGER_TAX1 + PR.LEDGER_TAX2) AS LEDGER_AMOUNT,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   '' AS CONTRACT_NAME, 0 AS CONTRACT_TYPE, 
   0.00 AS CONTRACT_CHARGED_AMOUNT, 
   0.00 AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PRJ.PROJECT_FLAG AS BUDGET_SETTING_BITS
   FROM PMO_TMP_PROJECTS_01 PRJ
   JOIN PROJECT_RECONCILIATION PR ON PR.PROJECT_ID = PRJ.ELEMENT_ID
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   JOIN TMT_COSTS ST ON ST.ELEMENT_ID = PR.FOREIGN_ID AND TO_DATE(SubStr(ST.INVOICE_DATETIME, 1, 10)) = PR.FOREIGN_DATE
   JOIN RESOURCES R ON R.RESOURCE_ID = ST.RESOURCE_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = ST.CHARGECODE_ID
   JOIN BUDGETS B ON B.PROJECT_ID = ST.PROJECT_ID AND B.BUDGET_ID = ST.BUDGET_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = B.PROJECT_ID AND W.ELEMENT_ID = B.ELEMENT_ID
   WHERE PR.FOREIGN_TABLE = PMO_CNST.SC_TABLE_COSTS            
   AND PR.FOREIGN_DATE BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10))
   UNION ALL
   SELECT PRJ.ELEMENT_NAME AS PROJECT_NAME, PRJ.RANK AS CLIENT_NAME, C.ELEMENT_NAME AS COST_CENTER_NAME, 
   FC.FISCAL_YEAR, FC.FISCAL_MONTH,
   CASE WHEN FC.FISCAL_MONTH < 4 THEN 1 WHEN FC.FISCAL_MONTH < 7 THEN 2
   WHEN FC.FISCAL_MONTH < 10 THEN 3 ELSE 4 END AS FISCAL_QUARTER,
   CC.ELEMENT_NAME AS CHARGECODE_NAME, R.FULL_NAME, W.ELEMENT_NAME, PR.TYPE_ID,
   PR.FOREIGN_TABLE AS TABLE_TYPE, PR.FOREIGN_DATE AS START_DATE, PR.WEEK_OF,
   (PR.REGULAR_QUANTITY * PR.REGULAR_RATE
   + PR.SPECIAL_QUANTITY * PR.SPECIAL_RATE + PR.TAX1 + PR.TAX2) AS PMO_AMOUNT,
   (PR.ADJ_REGULAR_QUANTITY * PR.ADJ_REGULAR_RATE
   + PR.ADJ_SPECIAL_QUANTITY * PR.ADJ_SPECIAL_RATE + PR.ADJ_TAX1 + PR.ADJ_TAX2) AS ADJ_AMOUNT,
   (PR.LEDGER_REGULAR_QUANTITY * PR.LEDGER_REGULAR_RATE 
   + PR.LEDGER_SPECIAL_QUANTITY * PR.LEDGER_SPECIAL_RATE + PR.LEDGER_TAX1 + PR.LEDGER_TAX2) AS LEDGER_AMOUNT,
   PR.CHARGE_DATE, PR.RECONCILIATION_DATE, PR.DESCRIPTION,
   '' AS CONTRACT_NAME, 0 AS CONTRACT_TYPE, 
   0.00 AS CONTRACT_CHARGED_AMOUNT, 
   0.00 AS CONTRACT_SCHED_AMOUNT,
   COALESCE(R.WORK_LOCATION,0) AS RESOURCE_WORK_LOCATION,
   COALESCE(R.FS_CODE, '') AS RESOURCE_FS_CODE,
   COALESCE(R.COUNTRY_CODE, '') AS RESOURCE_COUNTRY_CODE,
   COALESCE(R.COMPANY_CODE, '') AS RESOURCE_COMPANY_CODE,
   COALESCE(R.DIVISION_CODE, '') AS RESOURCE_DIVISION_CODE,
   COALESCE(R.DEPARTMENT_CODE, '') AS RESOURCE_DEPARTMENT_CODE,
   COALESCE(R.EMPLOYEE_CODE, '') AS RESOURCE_EMPLOYEE_CODE,
   PRJ.PROJECT_FLAG AS BUDGET_SETTING_BITS
   FROM PMO_TMP_PROJECTS_01 PRJ
   JOIN PROJECT_RECONCILIATION PR ON PR.PROJECT_ID = PRJ.ELEMENT_ID
   JOIN TMT_CLIENTS C ON C.ELEMENT_ID = PR.COST_CENTER_ID
   JOIN FISCAL_CALENDAR FC ON PR.FOREIGN_DATE BETWEEN FC.START_DATE AND FC.FINISH_DATE
   JOIN BUDGETS B ON B.BUDGET_ID = PR.FOREIGN_ID
   JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = B.CHARGECODE_ID
   JOIN TMT_WBS W ON W.PROJECT_ID = B.PROJECT_ID AND W.ELEMENT_ID = B.ELEMENT_ID
   JOIN RESOURCE_ASSIGNMENTS RA ON RA.PROJECT_ID = W.PROJECT_ID 
   AND RA.ELEMENT_ID = W.ELEMENT_ID AND BITAND(RA.PARTICIPANT, PMO_CNST.SC_MANAGER_RESPONSIBLE) > 0
   JOIN RESOURCES R ON R.RESOURCE_ID = RA.RESOURCE_ID
   WHERE PR.FOREIGN_TABLE IN (PMO_CNST.SC_TABLE_BUDGETS, PMO_CNST.SC_TABLE_PBB)            
   AND PR.FOREIGN_DATE BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10))
   ORDER BY FISCAL_YEAR, COST_CENTER_NAME, FISCAL_YEAR, FISCAL_QUARTER, FISCAL_MONTH,
   TYPE_ID, CHARGECODE_NAME, FULL_NAME, ELEMENT_NAME, START_DATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_RECON_PIVOT', 484+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_RECON_PIVOT(485) : reconciliation.sql(769) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN S_RECON_PIVOT;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 492+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(493) : reconciliation.sql(777) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_RECON_PIVOT', 504+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_RECON_PIVOT(505) : reconciliation.sql(789) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_RECON_PIVOT;
ALTER FUNCTION SP_S_RECON_PIVOT COMPILE DEBUG;

/*****************************************************************************************/

-- update charged and/or G/L amounts in timephase and budgets

-- bitflag: first bit:  0: charged;

--                      1: reconciled;

--          second bit: 0: value not changed

--                      1: value changed

/*****************************************************************************************/

Prompt == FUNCTION FC_UPDATE_TIMEPHASE_BUDGET ==
CREATE OR REPLACE FUNCTION FC_UPDATE_TIMEPHASE_BUDGET
(
     sh_h_bitflag PLS_INTEGER
   , h_project_id IN OUT NOCOPY VARCHAR2
   , h_element_id IN OUT NOCOPY VARCHAR2
   , sh_h_table_type PLS_INTEGER
   , sh_h_capital_expense PLS_INTEGER
   , sh_h_fiscal_year PLS_INTEGER
   , sh_h_fiscal_month PLS_INTEGER
   , sh_h_actual_amount NUMBER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_capital_expense PLS_INTEGER := sh_h_capital_expense;
   h_fiscal_year PLS_INTEGER := sh_h_fiscal_year;
   h_fiscal_month PLS_INTEGER := sh_h_fiscal_month;
   h_actual_amount NUMBER := sh_h_actual_amount;
   h_budget_type PLS_INTEGER := 0;
   h_id VARCHAR2(32);
   h_chargecode_id VARCHAR2(32);
   h_c_chargecode_id VARCHAR2(32);
   h_r_chargecode_id VARCHAR2(32);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_UPDATE_TIMEPHASE_BUDGET') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_UPDATE_TIMEPHASE_BUDGET'), '-- ' || SYSTIMESTAMP, ':r := FC_UPDATE_TIMEPHASE_BUDGET(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_project_id || ''''
          || ', ' || '''' || h_element_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_capital_expense), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_fiscal_year), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_fiscal_month), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_actual_amount), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_UPDATE_TIMEPHASE_BUDGET', 39+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_UPDATE_TIMEPHASE_BUDGET(40) : reconciliation.sql(857) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
      BEGIN
         SELECT TASK_ID, LABOR_COST_CC_ID, LABOR_REVENUE_CC_ID
         INTO h_id, h_c_chargecode_id, h_r_chargecode_id
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TSK_ASSIGNMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(56) : reconciliation.sql(873) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_capital_expense, 0) = PMO_CNST.SC_BENEFIT_LABOR Then
         h_chargecode_id := h_r_chargecode_id;
      Else
         h_chargecode_id := h_c_chargecode_id;
      End If;
      
      If (BITAND(Coalesce(h_bitflag, 0), 1)) = 0 Then
         h_budget_type := PMO_CNST.SC_CHARGED_BUDGET;
      Else
         h_budget_type := PMO_CNST.SC_LEDGER_BUDGET;
      End If;
   ElsIf (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_BUDGETS) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_PBB) Then
      BEGIN
         SELECT ELEMENT_ID, CHARGECODE_ID
         INTO h_id, h_chargecode_id
         FROM BUDGETS
         WHERE PROJECT_ID = h_project_id
         AND BUDGET_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 85+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(86) : reconciliation.sql(903) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_BUDGETS Then
         If (BITAND(Coalesce(h_bitflag, 0), 1)) = 0 Then
            h_budget_type := PMO_CNST.SC_CHARGED_BUDGET;
         Else
            h_budget_type := PMO_CNST.SC_LEDGER_BUDGET;
         End If;
      Else
         If (BITAND(Coalesce(h_bitflag, 0), 1)) = 0 Then
            h_budget_type := PMO_CNST.SC_ESTIMATED_CHARGED_BUDGET;
         Else
            h_budget_type := PMO_CNST.SC_ESTIMATED_LEDGER_BUDGET;
         End If;
      End If;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_COSTS Then
      BEGIN
         SELECT B.ELEMENT_ID, B.CHARGECODE_ID
         INTO h_id, h_chargecode_id
         FROM TMT_COSTS C, BUDGETS B
         WHERE C.PROJECT_ID = h_project_id
         AND C.ELEMENT_ID = h_element_id
         AND B.PROJECT_ID = C.PROJECT_ID
         AND B.BUDGET_ID = C.BUDGET_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 118+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(119) : reconciliation.sql(936) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (BITAND(Coalesce(h_bitflag, 0), 1)) = 0 Then
         h_budget_type := PMO_CNST.SC_CHARGED_BUDGET;
      Else
         h_budget_type := PMO_CNST.SC_LEDGER_BUDGET;
      End If;
   End If;
   
   If (BITAND(Coalesce(h_bitflag, 0), 2)) = 2 Then
      If Coalesce(h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET Then
         BEGIN
            UPDATE BUDGETS
            SET CHARGED = CHARGED + h_actual_amount,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 149+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(150) : reconciliation.sql(967) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_budget_type, 0) = PMO_CNST.SC_ESTIMATED_CHARGED_BUDGET Then
         BEGIN
            UPDATE BUDGETS
            SET ESTIMATED_CHARGED = ESTIMATED_CHARGED + h_actual_amount,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 171+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(172) : reconciliation.sql(989) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET Then
         BEGIN
            UPDATE BUDGETS
            SET LEDGER = LEDGER + h_actual_amount,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 193+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(194) : reconciliation.sql(1011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_budget_type, 0) = PMO_CNST.SC_ESTIMATED_LEDGER_BUDGET Then
         BEGIN
            UPDATE BUDGETS
            SET ESTIMATED_LEDGER = ESTIMATED_LEDGER + h_actual_amount,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 215+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(216) : reconciliation.sql(1033) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      BEGIN
         UPDATE TMT_WBS
         SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_UPDATE_BUDGET_BIT)
         WHERE PROJECT_ID = h_project_id
         AND ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 235+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(236) : reconciliation.sql(1053) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   CASE h_fiscal_month
      WHEN 1 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_01_AMOUNT = MONTH_01_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 262+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(263) : reconciliation.sql(1080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_01_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 283+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(284) : reconciliation.sql(1101) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_01_AMOUNT = MONTH_01_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 310+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(311) : reconciliation.sql(1128) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_01_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 331+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(332) : reconciliation.sql(1149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 2 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_02_AMOUNT = MONTH_02_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 359+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(360) : reconciliation.sql(1177) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_02_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 380+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(381) : reconciliation.sql(1198) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_02_AMOUNT = MONTH_02_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 407+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(408) : reconciliation.sql(1225) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_02_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 428+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(429) : reconciliation.sql(1246) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 3 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_03_AMOUNT = MONTH_03_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 456+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(457) : reconciliation.sql(1274) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_03_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 477+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(478) : reconciliation.sql(1295) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_03_AMOUNT = MONTH_03_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 504+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(505) : reconciliation.sql(1322) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_03_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 525+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(526) : reconciliation.sql(1343) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 4 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_04_AMOUNT = MONTH_04_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 553+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(554) : reconciliation.sql(1371) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_04_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 574+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(575) : reconciliation.sql(1392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_04_AMOUNT = MONTH_04_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 601+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(602) : reconciliation.sql(1419) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_04_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 622+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(623) : reconciliation.sql(1440) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 5 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_05_AMOUNT = MONTH_05_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 650+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(651) : reconciliation.sql(1468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_05_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 671+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(672) : reconciliation.sql(1489) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_05_AMOUNT = MONTH_05_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 698+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(699) : reconciliation.sql(1516) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_05_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 719+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(720) : reconciliation.sql(1537) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 6 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_06_AMOUNT = MONTH_06_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 747+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(748) : reconciliation.sql(1565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_06_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 768+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(769) : reconciliation.sql(1586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_06_AMOUNT = MONTH_06_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 795+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(796) : reconciliation.sql(1613) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_06_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 816+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(817) : reconciliation.sql(1634) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 7 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_07_AMOUNT = MONTH_07_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 844+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(845) : reconciliation.sql(1662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_07_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 865+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(866) : reconciliation.sql(1683) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_07_AMOUNT = MONTH_07_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 892+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(893) : reconciliation.sql(1710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_07_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 913+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(914) : reconciliation.sql(1731) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 8 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_08_AMOUNT = MONTH_08_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 941+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(942) : reconciliation.sql(1759) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_08_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 962+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(963) : reconciliation.sql(1780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_08_AMOUNT = MONTH_08_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 989+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(990) : reconciliation.sql(1807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_08_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1010+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1011) : reconciliation.sql(1828) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 9 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_09_AMOUNT = MONTH_09_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1038+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1039) : reconciliation.sql(1856) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_09_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1059+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1060) : reconciliation.sql(1877) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_09_AMOUNT = MONTH_09_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1086+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1087) : reconciliation.sql(1904) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_09_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1107+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1108) : reconciliation.sql(1925) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 10 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_10_AMOUNT = MONTH_10_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1135+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1136) : reconciliation.sql(1953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_10_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1156+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1157) : reconciliation.sql(1974) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_10_AMOUNT = MONTH_10_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1183+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1184) : reconciliation.sql(2001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_10_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1204+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1205) : reconciliation.sql(2022) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 11 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_11_AMOUNT = MONTH_11_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1232+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1233) : reconciliation.sql(2050) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_11_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1253+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1254) : reconciliation.sql(2071) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_11_AMOUNT = MONTH_11_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1280+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1281) : reconciliation.sql(2098) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_11_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1301+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1302) : reconciliation.sql(2119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
      
      WHEN 12 THEN
         BEGIN
            UPDATE PROJECT_BUDGET_BREAKDOWN
            SET MONTH_12_AMOUNT = MONTH_12_AMOUNT + h_actual_amount,
            TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
            WHERE PROJECT_ID = h_project_id
            AND ELEMENT_ID = h_id
            AND CHARGECODE_ID = h_chargecode_id
            AND BUDGET_TYPE = h_budget_type
            AND BUDGET_YEAR = h_fiscal_year;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1329+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1330) : reconciliation.sql(2147) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
               BUDGET_YEAR, BUDGET_TYPE, MONTH_12_AMOUNT, TOTAL_AMOUNT)
               VALUES (h_project_id, h_id, h_chargecode_id,
               h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1350+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1351) : reconciliation.sql(2168) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         If (Coalesce(h_budget_type, 0) = PMO_CNST.SC_LEDGER_BUDGET) OR (Coalesce(
               h_budget_type, 0) = PMO_CNST.SC_CHARGED_BUDGET) Then
            BEGIN
               UPDATE PROJECT_BUDGET_BREAKDOWN
               SET MONTH_12_AMOUNT = MONTH_12_AMOUNT + h_actual_amount,
               TOTAL_AMOUNT = TOTAL_AMOUNT + h_actual_amount
               WHERE PROJECT_ID = h_project_id
               AND ELEMENT_ID = h_project_id
               AND CHARGECODE_ID = h_chargecode_id
               AND BUDGET_TYPE = h_budget_type
               AND BUDGET_YEAR = h_fiscal_year;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1377+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1378) : reconciliation.sql(2195) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               BEGIN
                  INSERT INTO PROJECT_BUDGET_BREAKDOWN (PROJECT_ID, ELEMENT_ID, CHARGECODE_ID,
                  BUDGET_YEAR, BUDGET_TYPE, MONTH_12_AMOUNT, TOTAL_AMOUNT)
                  VALUES (h_project_id, h_project_id, h_chargecode_id,
                  h_fiscal_year, h_budget_type, h_actual_amount, h_actual_amount);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_UPDATE_TIMEPHASE_BUDGET', 1398+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_UPDATE_TIMEPHASE_BUDGET(1399) : reconciliation.sql(2216) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      ELSE
         NULL;
   END CASE;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_UPDATE_TIMEPHASE_BUDGET', 1409+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_UPDATE_TIMEPHASE_BUDGET(1410) : reconciliation.sql(2227) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_UPDATE_TIMEPHASE_BUDGET;
ALTER FUNCTION FC_UPDATE_TIMEPHASE_BUDGET COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_CHARGED ==
CREATE OR REPLACE FUNCTION SP_U_CHARGED
(
     p_id VARCHAR2
   , p_adj_regular_quantity NUMBER
   , p_adj_regular_rate NUMBER
   , p_adj_special_quantity NUMBER
   , p_adj_special_rate NUMBER
   , p_adj_tax1 NUMBER
   , p_adj_tax2 NUMBER
   , p_charge_date VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_adj_regular_quantity NUMBER := 0;
   h_adj_regular_rate NUMBER := 0;
   h_adj_special_quantity NUMBER := 0;
   h_adj_special_rate NUMBER := 0;
   h_adj_tax1 NUMBER := 0;
   h_adj_tax2 NUMBER := 0;
   h_regular_quantity NUMBER := 0;
   h_regular_rate NUMBER := 0;
   h_special_quantity NUMBER := 0;
   h_special_rate NUMBER := 0;
   h_tax1 NUMBER := 0;
   h_tax2 NUMBER := 0;
   h_actual_amount NUMBER := 0;
   h_adj_actual_amount NUMBER := 0;
   h_charge_date DATE;
   h_old_charge_date DATE;
   h_rec_user VARCHAR2(32);
   h_old_fiscal_year PLS_INTEGER := 0;
   h_old_fiscal_month PLS_INTEGER := 0;
   h_fiscal_year PLS_INTEGER := 0;
   h_fiscal_month PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_foreign_date DATE;
   null_return LONG;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CHARGED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CHARGED'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CHARGED(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_adj_regular_quantity), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_adj_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_adj_special_quantity), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_adj_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_adj_tax1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_adj_tax2), 'NULL')
          || ', ' || '''' || p_charge_date || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CHARGED', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CHARGED(61) : reconciliation.sql(2298) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_adj_regular_quantity := p_adj_regular_quantity;
   h_adj_regular_rate := p_adj_regular_rate;
   h_adj_special_quantity := p_adj_special_quantity;
   h_adj_special_rate := p_adj_special_rate;
   h_adj_tax1 := p_adj_tax1;
   h_adj_tax2 := p_adj_tax2;
   h_charge_date := p_charge_date;
   h_rec_user := h_rec_user;
   
   If (h_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_CHARGED', 77+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_CHARGED(78) : reconciliation.sql(2315) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CHARGED', 80+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CHARGED(81) : reconciliation.sql(2318) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      SELECT PROJECT_ID, RESOURCE_ID, TYPE_ID,
      FOREIGN_ID, FOREIGN_TABLE, FOREIGN_DATE,
      ADJ_REGULAR_QUANTITY, ADJ_REGULAR_RATE,
      ADJ_SPECIAL_QUANTITY, ADJ_SPECIAL_RATE,
      ADJ_TAX1, ADJ_TAX2,
      CHARGE_DATE
      INTO h_project_id, h_resource_id, h_type_id,
      h_element_id, h_table_type, h_foreign_date,
      h_regular_quantity, h_regular_rate,
      h_special_quantity, h_special_rate,
      h_tax1, h_tax2,
      h_old_charge_date
      FROM PROJECT_RECONCILIATION
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 106+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(107) : reconciliation.sql(2344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_CHARGED', 113+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_CHARGED(114) : reconciliation.sql(2351) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CHARGED', 116+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_CHARGED(117) : reconciliation.sql(2354) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
      h_adj_regular_quantity := Coalesce(h_adj_regular_quantity, 0) * 60.0;
      h_adj_special_quantity := Coalesce(h_adj_special_quantity, 0) * 60.0;
   End If;
   
   If (h_old_charge_date IS NULL) Then
      h_old_fiscal_year := 0;
      h_old_fiscal_month := 0;
   Else
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH
         INTO h_old_fiscal_year, h_old_fiscal_month
         FROM FISCAL_CALENDAR
         WHERE START_DATE <= h_old_charge_date
         AND FINISH_DATE >= h_old_charge_date;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 142+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(143) : reconciliation.sql(2380) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (h_charge_date IS NULL) Then
      h_fiscal_year := 0;
      h_fiscal_month := 0;
   Else
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH
         INTO h_fiscal_year, h_fiscal_month
         FROM FISCAL_CALENDAR
         WHERE START_DATE <= h_charge_date
         AND FINISH_DATE >= h_charge_date;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 164+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(165) : reconciliation.sql(2402) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_actual_amount := 0.00;
   h_adj_actual_amount := 0.00;
   h_bitflag := 0;
   
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
      h_actual_amount := (Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                             h_special_quantity, 0) * Coalesce(h_special_rate, 0)) / 60.0 
                             + Coalesce(h_tax1, 0) + Coalesce(h_tax2, 0);
      h_adj_actual_amount := (Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                 + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                 h_adj_special_rate, 0)) / 60.0 + Coalesce(h_adj_tax1, 0) 
                                 + Coalesce(h_adj_tax2, 0);
   Else
      h_actual_amount := Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                            h_special_quantity, 0) * Coalesce(h_special_rate, 0) + Coalesce(
                            h_tax1, 0) + Coalesce(h_tax2, 0);
      h_adj_actual_amount := Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                h_adj_special_rate, 0) + Coalesce(h_adj_tax1, 0) + Coalesce(
                                h_adj_tax2, 0);
   End If;
   
   If Coalesce(h_actual_amount, 0) <> Coalesce(h_adj_actual_amount, 0) Then
      h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
   End If;
   
   If (Coalesce(h_old_fiscal_year, 0) <> Coalesce(h_fiscal_year, 0)) OR (Coalesce(
                                               h_old_fiscal_month, 0) <> Coalesce(
                                               h_fiscal_month, 0)) Then
      If (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_actual_amount, 0) > 0.00) Then
         If Coalesce(h_fiscal_year, 0) = 0 Then
            h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
         End If;
         
         h_actual_amount := (0.0 - Coalesce(h_actual_amount, 0));
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_old_fiscal_year, h_old_fiscal_month
                                          , h_actual_amount, h_rec_user);
      End If;
      
      If (Coalesce(h_fiscal_year, 0) > 0) AND (Coalesce(h_adj_actual_amount, 0) > 0.00) Then
         If Coalesce(h_old_fiscal_year, 0) = 0 Then
            h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
         End If;
         
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_adj_actual_amount, h_rec_user);
      End If;
   ElsIf (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_actual_amount, 0) <> Coalesce(
                h_adj_actual_amount, 0)) Then
      h_actual_amount := Coalesce(h_adj_actual_amount, 0) - Coalesce(h_actual_amount, 0);
      
      null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_actual_amount, h_rec_user);
   End If;
   
   BEGIN
      UPDATE PROJECT_RECONCILIATION
      SET ADJ_REGULAR_QUANTITY = h_adj_regular_quantity,
      ADJ_REGULAR_RATE = h_adj_regular_rate,
      ADJ_SPECIAL_QUANTITY = h_adj_special_quantity,
      ADJ_SPECIAL_RATE = h_adj_special_rate,
      ADJ_TAX1 = h_adj_tax1,
      ADJ_TAX2 = h_adj_tax2,
      CHARGE_DATE = h_charge_date
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 248+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(249) : reconciliation.sql(2486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CHARGED', 254+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CHARGED(255) : reconciliation.sql(2492) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 261+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(262) : reconciliation.sql(2499) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CHARGED', 273+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CHARGED(274) : reconciliation.sql(2511) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CHARGED;
ALTER FUNCTION SP_U_CHARGED COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_RECONCILED ==
CREATE OR REPLACE FUNCTION SP_U_RECONCILED
(
     p_id VARCHAR2
   , p_ledger_regular_quantity NUMBER
   , p_ledger_regular_rate NUMBER
   , p_ledger_special_quantity NUMBER
   , p_ledger_special_rate NUMBER
   , p_ledger_tax1 NUMBER
   , p_ledger_tax2 NUMBER
   , p_recon_date VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_ledger_regular_quantity NUMBER := 0;
   h_ledger_regular_rate NUMBER := 0;
   h_ledger_special_quantity NUMBER := 0;
   h_ledger_special_rate NUMBER := 0;
   h_ledger_tax1 NUMBER := 0;
   h_ledger_tax2 NUMBER := 0;
   h_regular_quantity NUMBER := 0;
   h_regular_rate NUMBER := 0;
   h_special_quantity NUMBER := 0;
   h_special_rate NUMBER := 0;
   h_tax1 NUMBER := 0;
   h_tax2 NUMBER := 0;
   h_actual_amount NUMBER := 0;
   h_ledger_actual_amount NUMBER := 0;
   h_recon_date DATE;
   h_old_recon_date DATE;
   h_rec_user VARCHAR2(32);
   h_old_fiscal_year PLS_INTEGER := 0;
   h_old_fiscal_month PLS_INTEGER := 0;
   h_fiscal_year PLS_INTEGER := 0;
   h_fiscal_month PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_foreign_date DATE;
   h_week_of DATE;
   null_return LONG;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RECONCILED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RECONCILED'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RECONCILED(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_ledger_regular_quantity), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_ledger_regular_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_ledger_special_quantity), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_ledger_special_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_ledger_tax1), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_ledger_tax2), 'NULL')
          || ', ' || '''' || p_recon_date || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RECONCILED', 61+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RECONCILED(62) : reconciliation.sql(2589) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_ledger_regular_quantity := p_ledger_regular_quantity;
   h_ledger_regular_rate := p_ledger_regular_rate;
   h_ledger_special_quantity := p_ledger_special_quantity;
   h_ledger_special_rate := p_ledger_special_rate;
   h_ledger_tax1 := p_ledger_tax1;
   h_ledger_tax2 := p_ledger_tax2;
   h_recon_date := p_recon_date;
   h_rec_user := h_rec_user;
   
   If (h_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_RECONCILED', 78+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_RECONCILED(79) : reconciliation.sql(2606) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RECONCILED', 81+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_RECONCILED(82) : reconciliation.sql(2609) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      SELECT PROJECT_ID, RESOURCE_ID, TYPE_ID,
      FOREIGN_ID, FOREIGN_TABLE, FOREIGN_DATE,
      LEDGER_REGULAR_QUANTITY, LEDGER_REGULAR_RATE,
      LEDGER_SPECIAL_QUANTITY, LEDGER_SPECIAL_RATE,
      LEDGER_TAX1, LEDGER_TAX2,
      RECONCILIATION_DATE, WEEK_OF
      INTO h_project_id, h_resource_id, h_type_id,
      h_element_id, h_table_type, h_foreign_date,
      h_regular_quantity, h_regular_rate,
      h_special_quantity, h_special_rate,
      h_tax1, h_tax2,
      h_old_recon_date, h_week_of
      FROM PROJECT_RECONCILIATION
      WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(108) : reconciliation.sql(2635) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_RECONCILED', 114+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_RECONCILED(115) : reconciliation.sql(2642) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RECONCILED', 117+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_RECONCILED(118) : reconciliation.sql(2645) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
   End If;
   
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
      h_ledger_regular_quantity := Coalesce(h_ledger_regular_quantity, 0) * 60.0;
      h_ledger_special_quantity := Coalesce(h_ledger_special_quantity, 0) * 60.0;
   End If;
   
   If (h_old_recon_date IS NULL) Then
      h_old_fiscal_year := 0;
      h_old_fiscal_month := 0;
   Else
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH
         INTO h_old_fiscal_year, h_old_fiscal_month
         FROM FISCAL_CALENDAR
         WHERE START_DATE <= h_old_recon_date
         AND FINISH_DATE >= h_old_recon_date;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 143+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(144) : reconciliation.sql(2671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (h_recon_date IS NULL) Then
      h_fiscal_year := 0;
      h_fiscal_month := 0;
   Else
      BEGIN
         SELECT FISCAL_YEAR, FISCAL_MONTH
         INTO h_fiscal_year, h_fiscal_month
         FROM FISCAL_CALENDAR
         WHERE START_DATE <= h_recon_date
         AND FINISH_DATE >= h_recon_date;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 165+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(166) : reconciliation.sql(2693) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_actual_amount := 0.00;
   h_ledger_actual_amount := 0.00;
   h_bitflag := 1;
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
      If ((h_old_recon_date IS NULL) AND (h_recon_date IS NOT NULL)) OR ((h_old_recon_date IS NOT NULL) 
            AND (h_recon_date IS NULL)) Then
         BEGIN
            UPDATE SUMMARY_TIMESHEETS
            SET WEEK_OF = h_week_of
            WHERE TSK_ASSIGNMENT_ID = h_element_id
            AND WEEK_OF = h_week_of;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 192+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(193) : reconciliation.sql(2720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_actual_amount := (Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                             h_special_quantity, 0) * Coalesce(h_special_rate, 0)) / 60.0 
                             + Coalesce(h_tax1, 0) + Coalesce(h_tax2, 0);
      h_ledger_actual_amount := (Coalesce(h_ledger_regular_quantity, 0) * Coalesce(
                                    h_ledger_regular_rate, 0) + Coalesce(
                                    h_ledger_special_quantity, 0) * Coalesce(
                                    h_ledger_special_rate, 0)) / 60.0 + Coalesce(h_ledger_tax1, 0) 
                                    + Coalesce(h_ledger_tax2, 0);
   Else
      h_actual_amount := Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                            h_special_quantity, 0) * Coalesce(h_special_rate, 0) + Coalesce(
                            h_tax1, 0) + Coalesce(h_tax2, 0);
      h_ledger_actual_amount := Coalesce(h_ledger_regular_quantity, 0) * Coalesce(
                                   h_ledger_regular_rate, 0) + Coalesce(
                                   h_ledger_special_quantity, 0) * Coalesce(
                                   h_ledger_special_rate, 0) + Coalesce(h_ledger_tax1, 0) 
                                   + Coalesce(h_ledger_tax2, 0);
   End If;
   
   If Coalesce(h_actual_amount, 0) <> Coalesce(h_ledger_actual_amount, 0) Then
      h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
   End If;
   
   If (Coalesce(h_old_fiscal_year, 0) <> Coalesce(h_fiscal_year, 0)) OR (Coalesce(
                                               h_old_fiscal_month, 0) <> Coalesce(
                                               h_fiscal_month, 0)) Then
      If (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_actual_amount, 0) > 0.00) Then
         If Coalesce(h_fiscal_year, 0) = 0 Then
            h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
         End If;
         
         h_actual_amount := (0.0 - Coalesce(h_actual_amount, 0));
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_old_fiscal_year, h_old_fiscal_month
                                          , h_actual_amount, h_rec_user);
      End If;
      
      If (Coalesce(h_fiscal_year, 0) > 0) AND (Coalesce(h_ledger_actual_amount, 0) > 0.00) Then
         If Coalesce(h_old_fiscal_year, 0) = 0 Then
            h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
         End If;
         
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_ledger_actual_amount, h_rec_user);
      End If;
   ElsIf (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_actual_amount, 0) <> Coalesce(
                h_ledger_actual_amount, 0)) Then
      h_actual_amount := Coalesce(h_ledger_actual_amount, 0) - Coalesce(h_actual_amount, 0);
      
      null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_actual_amount, h_rec_user);
   End If;
   
   BEGIN
      UPDATE PROJECT_RECONCILIATION
      SET LEDGER_REGULAR_QUANTITY = h_ledger_regular_quantity,
      LEDGER_REGULAR_RATE = h_ledger_regular_rate,
      LEDGER_SPECIAL_QUANTITY = h_ledger_special_quantity,
      LEDGER_SPECIAL_RATE = h_ledger_special_rate,
      LEDGER_TAX1 = h_ledger_tax1,
      LEDGER_TAX2 = h_ledger_tax2,
      RECONCILIATION_DATE = h_recon_date
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 273+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(274) : reconciliation.sql(2801) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RECONCILED', 279+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RECONCILED(280) : reconciliation.sql(2807) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 286+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(287) : reconciliation.sql(2814) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED', 298+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED(299) : reconciliation.sql(2826) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RECONCILED;
ALTER FUNCTION SP_U_RECONCILED COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_U_RECONCILED_DESC ==
CREATE OR REPLACE FUNCTION SP_U_RECONCILED_DESC
(
     p_id VARCHAR2
   , p_description VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_description VARCHAR2(255);
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RECONCILED_DESC') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RECONCILED_DESC'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RECONCILED_DESC(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_description || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RECONCILED_DESC', 21+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RECONCILED_DESC(22) : reconciliation.sql(2864) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := p_id;
   h_description := p_description;
   h_rec_user := h_rec_user;
   
   If (h_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_RECONCILED_DESC', 32+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_RECONCILED_DESC(33) : reconciliation.sql(2875) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RECONCILED_DESC', 35+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_RECONCILED_DESC(36) : reconciliation.sql(2878) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   BEGIN
      UPDATE PROJECT_RECONCILIATION
      SET DESCRIPTION = h_description
      WHERE ELEMENT_ID = h_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED_DESC', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED_DESC(55) : reconciliation.sql(2897) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RECONCILED_DESC', 60+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RECONCILED_DESC(61) : reconciliation.sql(2903) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED_DESC', 67+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED_DESC(68) : reconciliation.sql(2910) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RECONCILED_DESC', 79+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RECONCILED_DESC(80) : reconciliation.sql(2922) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RECONCILED_DESC;
ALTER FUNCTION SP_U_RECONCILED_DESC COMPILE DEBUG;

/**********************************************************************************/

Prompt == FUNCTION FC_AUTO_CHARGE_LABOR_COST ==
CREATE OR REPLACE FUNCTION FC_AUTO_CHARGE_LABOR_COST
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_charge_start IN OUT NOCOPY DATE
   , h_charge_finish IN OUT NOCOPY DATE
   , h_charge_date IN OUT NOCOPY DATE
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_adj_regular_quantity NUMBER := 0;
   h_adj_regular_rate NUMBER := 0;
   h_adj_special_quantity NUMBER := 0;
   h_adj_special_rate NUMBER := 0;
   h_adj_tax1 NUMBER := 0;
   h_adj_tax2 NUMBER := 0;
   h_regular_quantity NUMBER := 0;
   h_regular_rate NUMBER := 0;
   h_special_quantity NUMBER := 0;
   h_special_rate NUMBER := 0;
   h_tax1 NUMBER := 0;
   h_tax2 NUMBER := 0;
   h_actual_amount NUMBER := 0;
   h_adj_actual_amount NUMBER := 0;
   h_old_adj_actual_amount NUMBER := 0;
   h_old_charge_date DATE;
   h_old_fiscal_year PLS_INTEGER := 0;
   h_old_fiscal_month PLS_INTEGER := 0;
   h_fiscal_year PLS_INTEGER := 0;
   h_fiscal_month PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_foreign_date DATE;
   h_rollup_all PLS_INTEGER := 0;
   h_contract_id VARCHAR2(32);
   h_contract_type PLS_INTEGER := 0;
   h_last_charged_date DATE;
   h_prev_contract_id VARCHAR2(32);
   h_max_charge_amount NUMBER := 0;
   h_total_charge_amount NUMBER := 0;
   h_temp_max_charge_amount NUMBER := 0;
   h_temp_total_charge_amount NUMBER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR C_AUTO_CHARGE IS
         SELECT PR.ELEMENT_ID, PR.PROJECT_ID, PR.RESOURCE_ID, PR.TYPE_ID,
         PR.FOREIGN_ID, PR.FOREIGN_TABLE, PR.FOREIGN_DATE,
         PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY, PR.REGULAR_RATE,
         PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY, PR.SPECIAL_RATE,
         PR.TAX1, PR.TAX2,
         PR.ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
         PR.ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
         PR.ADJ_TAX1, PR.ADJ_TAX2,
         PR.CHARGE_DATE,
         TC.ELEMENT_ID
         FROM PROJECT_RECONCILIATION PR
         JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
         LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
         WHERE PR.PROJECT_ID = h_project_id
         AND PR.FOREIGN_DATE BETWEEN h_charge_start AND h_charge_finish
         AND PR.TYPE_ID IN (PMO_CNST.SC_EXPENSE_LABOR, PMO_CNST.SC_CAPITAL_LABOR)
         ORDER BY TC.ELEMENT_ID, PR.FOREIGN_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_AUTO_CHARGE_LABOR_COST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_AUTO_CHARGE_LABOR_COST'), '-- ' || SYSTIMESTAMP, ':r := FC_AUTO_CHARGE_LABOR_COST(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_charge_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_charge_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_charge_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_COST', 77+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_AUTO_CHARGE_LABOR_COST(78) : reconciliation.sql(3016) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT FISCAL_YEAR, FISCAL_MONTH
      INTO h_fiscal_year, h_fiscal_month
      FROM FISCAL_CALENDAR
      WHERE START_DATE <= h_charge_date
      AND FINISH_DATE >= h_charge_date;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 92+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(93) : reconciliation.sql(3031) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_AUTO_CHARGE;
   
   h_prev_contract_id := 'QQ';
   h_temp_max_charge_amount := 0.00;
   h_temp_total_charge_amount := 0.00;
   LOOP
      FETCH C_AUTO_CHARGE INTO h_id, h_project_id, h_resource_id, h_type_id,
      h_element_id, h_table_type, h_foreign_date,
      h_regular_quantity, h_regular_rate,
      h_special_quantity, h_special_rate,
      h_tax1, h_tax2,
      h_adj_regular_quantity, h_adj_regular_rate,
      h_adj_special_quantity, h_adj_special_rate,
      h_adj_tax1, h_adj_tax2,
      h_old_charge_date,
      h_contract_id;
      IF C_AUTO_CHARGE%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         EXIT;
      End If;
      
      If (h_contract_id IS NULL) Then
         h_max_charge_amount := 1.00;
         h_total_charge_amount := 1.00;
      Else
         If PMO_ORASTD.strcmp(h_contract_id, h_prev_contract_id) <> 0 Then
            h_temp_total_charge_amount := 0.0;
            h_temp_max_charge_amount := 0.0;
            BEGIN
               SELECT CONTRACT_TYPE, MAX_AMOUNT, CHARGED_AMOUNT, LAST_CHARGED_DATE
               INTO h_contract_type, h_max_charge_amount,
               h_total_charge_amount, h_last_charged_date
               FROM TMT_CONTRACTS
               WHERE ELEMENT_ID = h_contract_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 142+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(143) : reconciliation.sql(3081) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            -- Monthly Fixed Amount
            If (Coalesce(h_contract_type, 0) = 6703) OR (Coalesce(h_contract_type, 0) = 6704) Then
               BEGIN
                  SELECT SUM(CB.CHARGE_AMOUNT) INTO h_max_charge_amount
                  FROM CONTRACT_BREAKDOWN CB
                  WHERE CONTRACT_ID = h_contract_id
                  AND CB.CHARGE_DATE BETWEEN h_charge_start AND h_charge_finish;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 160+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(161) : reconciliation.sql(3099) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
                  h_max_charge_amount := 0.00;
                  h_total_charge_amount := 1.00;
               Else
                  BEGIN
                     SELECT SUM(((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) * PR.REGULAR_RATE
                     + ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) * PR.SPECIAL_RATE
                     + PR.TAX1 + PR.TAX2)
                     INTO h_total_charge_amount
                     FROM PROJECT_RECONCILIATION PR
                     JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
                     AND TA.CONTRACT_ID = h_contract_id
                     WHERE PR.PROJECT_ID = h_project_id
                     AND PR.FOREIGN_DATE BETWEEN h_charge_start AND h_charge_finish
                     AND PR.TYPE_ID IN (PMO_CNST.SC_EXPENSE_LABOR, PMO_CNST.SC_CAPITAL_LABOR);
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 186+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(187) : reconciliation.sql(3125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf Coalesce(h_contract_type, 0) = 6702 Then
               If Coalesce(h_total_charge_amount, 0) >= Coalesce(h_max_charge_amount, 0) Then
                  h_max_charge_amount := 0.00;
                  h_total_charge_amount := 1.00;
               Else
                  BEGIN
                     SELECT SUM(((PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY) / 60.0) * PR.REGULAR_RATE
                     + ((PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY) / 60.0) * PR.SPECIAL_RATE
                     + PR.TAX1 + PR.TAX2)
                     INTO h_total_charge_amount
                     FROM PROJECT_RECONCILIATION PR
                     JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
                     AND TA.CONTRACT_ID = h_contract_id
                     WHERE PR.PROJECT_ID = h_project_id
                     AND PR.FOREIGN_DATE BETWEEN h_charge_start AND h_charge_finish
                     AND PR.TYPE_ID IN (PMO_CNST.SC_EXPENSE_LABOR, PMO_CNST.SC_CAPITAL_LABOR);
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 213+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(214) : reconciliation.sql(3152) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_total_charge_amount, 0) < Coalesce(h_max_charge_amount, 0) Then
                     h_total_charge_amount := h_max_charge_amount;
                  End If;
               End If;
            Else
               h_max_charge_amount := 1.00;
               h_total_charge_amount := 1.00;
            End If;
            h_prev_contract_id := h_contract_id;
         End If;
      End If;
      
      If (h_old_charge_date IS NULL) Then
         h_old_fiscal_year := 0;
         h_old_fiscal_month := 0;
      Else
         BEGIN
            SELECT FISCAL_YEAR, FISCAL_MONTH
            INTO h_old_fiscal_year, h_old_fiscal_month
            FROM FISCAL_CALENDAR
            WHERE START_DATE <= h_old_charge_date
            AND FINISH_DATE >= h_old_charge_date;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 246+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(247) : reconciliation.sql(3185) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_bitflag := 0;
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
         h_old_adj_actual_amount := (Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                     + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                     h_adj_special_rate, 0)) / 60.0 + Coalesce(h_adj_tax1, 0) 
                                     + Coalesce(h_adj_tax2, 0);
      Else
         h_old_adj_actual_amount := Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                    + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                    h_adj_special_rate, 0) + Coalesce(h_adj_tax1, 0) + Coalesce(
                                    h_adj_tax2, 0);
      End If;
      
      h_adj_regular_quantity := Coalesce(h_regular_quantity, 0);
      h_adj_special_quantity := Coalesce(h_special_quantity, 0);
      h_adj_regular_rate := Coalesce(h_regular_rate, 0) * Coalesce(h_max_charge_amount, 0) / Coalesce(
                               h_total_charge_amount, 0) * 100.0;
      h_adj_special_rate := Coalesce(h_special_rate, 0) * Coalesce(h_max_charge_amount, 0) / Coalesce(
                               h_total_charge_amount, 0) * 100.0;
      h_adj_regular_rate := Coalesce(Trunc(h_adj_regular_rate), 0);
      h_adj_special_rate := Coalesce(Trunc(h_adj_special_rate), 0);
      h_adj_regular_rate := Coalesce(h_adj_regular_rate, 0) / 100.0;
      h_adj_special_rate := Coalesce(h_adj_special_rate, 0) / 100.0;
      h_adj_tax1 := Coalesce(h_tax1, 0) * Coalesce(h_max_charge_amount, 0) / Coalesce(
                       h_total_charge_amount, 0);
      h_adj_tax2 := Coalesce(h_tax2, 0) * Coalesce(h_max_charge_amount, 0) / Coalesce(
                       h_total_charge_amount, 0);
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
         h_actual_amount := (Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                             h_special_quantity, 0) * Coalesce(h_special_rate, 0)) / 60.0 
                             + Coalesce(h_tax1, 0) + Coalesce(h_tax2, 0);
         h_adj_actual_amount := (Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                 + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                 h_adj_special_rate, 0)) / 60.0 + Coalesce(h_adj_tax1, 0) 
                                 + Coalesce(h_adj_tax2, 0);
      Else
         h_actual_amount := Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                            h_special_quantity, 0) * Coalesce(h_special_rate, 0) + Coalesce(
                            h_tax1, 0) + Coalesce(h_tax2, 0);
         h_adj_actual_amount := Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                h_adj_special_rate, 0) + Coalesce(h_adj_tax1, 0) + Coalesce(
                                h_adj_tax2, 0);
      End If;
      
      h_temp_total_charge_amount := Coalesce(h_temp_total_charge_amount, 0) + Coalesce(h_actual_amount, 0);
      h_temp_max_charge_amount := Coalesce(h_temp_max_charge_amount, 0) + Coalesce(h_adj_actual_amount, 0);
      
      If (Coalesce(h_max_charge_amount, 0) <> 0.00) AND (Coalesce(h_max_charge_amount, 0) <> Coalesce(
             h_total_charge_amount, 0)) AND (Coalesce(h_total_charge_amount, 0) = Coalesce(
             h_temp_total_charge_amount, 0)) AND (Coalesce(h_max_charge_amount, 0) <> Coalesce(
             h_temp_max_charge_amount, 0)) Then
         h_adj_actual_amount := Coalesce(h_adj_actual_amount, 0) + (Coalesce(h_max_charge_amount, 0) 
                                - Coalesce(h_temp_max_charge_amount, 0));
         h_adj_tax1 := Coalesce(h_adj_tax1, 0) + (Coalesce(h_max_charge_amount, 0) - Coalesce(
                       h_temp_max_charge_amount, 0));
      End If;
      
      If Coalesce(h_old_adj_actual_amount, 0) <> Coalesce(h_adj_actual_amount, 0) Then
         h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
      End If;
      
      If (Coalesce(h_old_fiscal_year, 0) <> Coalesce(h_fiscal_year, 0)) OR (Coalesce(
                                               h_old_fiscal_month, 0) <> Coalesce(
                                               h_fiscal_month, 0)) Then
         If (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_old_adj_actual_amount, 0) > 0.00) Then
            If Coalesce(h_fiscal_year, 0) = 0 Then
               h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
            End If;
            
            h_actual_amount := (0.0 - Coalesce(h_old_adj_actual_amount, 0));
            null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_old_fiscal_year, h_old_fiscal_month
                                          , h_actual_amount, h_rec_user);
         End If;
         
         If (Coalesce(h_fiscal_year, 0) > 0) AND (Coalesce(h_adj_actual_amount, 0) > 0.00) Then
            If Coalesce(h_old_fiscal_year, 0) = 0 Then
               h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
            End If;
            
            null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_adj_actual_amount, h_rec_user);
         End If;
      ElsIf (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_old_adj_actual_amount, 0) <> Coalesce(
                h_adj_actual_amount, 0)) Then
         h_actual_amount := Coalesce(h_adj_actual_amount, 0) - Coalesce(h_old_adj_actual_amount, 0);
         
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_actual_amount, h_rec_user);
      End If;
      
      BEGIN
         UPDATE PROJECT_RECONCILIATION
         SET ADJ_REGULAR_QUANTITY = h_adj_regular_quantity,
         ADJ_REGULAR_RATE = h_adj_regular_rate,
         ADJ_SPECIAL_QUANTITY = h_adj_special_quantity,
         ADJ_SPECIAL_RATE = h_adj_special_rate,
         ADJ_TAX1 = h_adj_tax1,
         ADJ_TAX2 = h_adj_tax2,
         CHARGE_DATE = h_charge_date
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 366+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(367) : reconciliation.sql(3305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   BEGIN
      CLOSE C_AUTO_CHARGE ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_COST', 379+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_COST(380) : reconciliation.sql(3318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_COST', 385+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AUTO_CHARGE_LABOR_COST(386) : reconciliation.sql(3324) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_COST', 389+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AUTO_CHARGE_LABOR_COST(390) : reconciliation.sql(3328) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_AUTO_CHARGE_LABOR_COST;
ALTER FUNCTION FC_AUTO_CHARGE_LABOR_COST COMPILE DEBUG;

/**********************************************************************************/

Prompt == FUNCTION FC_AUTO_CHARGE_LABOR_REVENUE ==
CREATE OR REPLACE FUNCTION FC_AUTO_CHARGE_LABOR_REVENUE
(
     h_project_id IN OUT NOCOPY VARCHAR2
   , h_charge_start IN OUT NOCOPY DATE
   , h_charge_finish IN OUT NOCOPY DATE
   , h_charge_date IN OUT NOCOPY DATE
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_id VARCHAR2(32);
   h_adj_regular_quantity NUMBER := 0;
   h_adj_regular_rate NUMBER := 0;
   h_adj_special_quantity NUMBER := 0;
   h_adj_special_rate NUMBER := 0;
   h_adj_tax1 NUMBER := 0;
   h_adj_tax2 NUMBER := 0;
   h_regular_quantity NUMBER := 0;
   h_regular_rate NUMBER := 0;
   h_special_quantity NUMBER := 0;
   h_special_rate NUMBER := 0;
   h_tax1 NUMBER := 0;
   h_tax2 NUMBER := 0;
   h_actual_amount NUMBER := 0;
   h_adj_actual_amount NUMBER := 0;
   h_old_adj_actual_amount NUMBER := 0;
   h_old_charge_date DATE;
   h_old_fiscal_year PLS_INTEGER := 0;
   h_old_fiscal_month PLS_INTEGER := 0;
   h_fiscal_year PLS_INTEGER := 0;
   h_fiscal_month PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_foreign_date DATE;
   h_rollup_all PLS_INTEGER := 0;
   h_contract_id VARCHAR2(32);
   h_contract_type PLS_INTEGER := 0;
   h_last_charged_date DATE;
   h_prev_contract_id VARCHAR2(32);
   h_max_charge_amount NUMBER := 0;
   h_total_charge_amount NUMBER := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR C_AUTO_REVENUE IS
         SELECT PR.ELEMENT_ID, PR.PROJECT_ID, PR.RESOURCE_ID, PR.TYPE_ID,
         PR.FOREIGN_ID, PR.FOREIGN_TABLE, PR.FOREIGN_DATE,
         PR.REGULAR_DEFAULT_QUANTITY + PR.REGULAR_QUANTITY, PR.REGULAR_RATE,
         PR.SPECIAL_DEFAULT_QUANTITY + PR.SPECIAL_QUANTITY, PR.SPECIAL_RATE,
         PR.TAX1, PR.TAX2,
         PR.ADJ_REGULAR_QUANTITY, PR.ADJ_REGULAR_RATE,
         PR.ADJ_SPECIAL_QUANTITY, PR.ADJ_SPECIAL_RATE,
         PR.ADJ_TAX1, PR.ADJ_TAX2,
         PR.CHARGE_DATE,
         TC.ELEMENT_ID
         FROM PROJECT_RECONCILIATION PR
         JOIN TASK_ASSIGNMENTS TA ON TA.TSK_ASSIGNMENT_ID = PR.FOREIGN_ID
         LEFT JOIN TMT_CONTRACTS TC ON TC.ELEMENT_ID = TA.CONTRACT_ID
         WHERE PR.PROJECT_ID = h_project_id
         AND PR.FOREIGN_DATE BETWEEN h_charge_start AND h_charge_finish
         AND PR.TYPE_ID = PMO_CNST.SC_BENEFIT_LABOR
         ORDER BY TC.ELEMENT_ID, PR.FOREIGN_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_AUTO_CHARGE_LABOR_REVENUE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_AUTO_CHARGE_LABOR_REVENUE'), '-- ' || SYSTIMESTAMP, ':r := FC_AUTO_CHARGE_LABOR_REVENUE(' || 
       '''' || h_project_id || ''''
          || ', ' || COALESCE(NULLIF('''' || h_charge_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_charge_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_charge_date || '''',''''''), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_REVENUE', 75+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_AUTO_CHARGE_LABOR_REVENUE(76) : reconciliation.sql(3414) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT FISCAL_YEAR, FISCAL_MONTH
      INTO h_fiscal_year, h_fiscal_month
      FROM FISCAL_CALENDAR
      WHERE START_DATE <= h_charge_date
      AND FINISH_DATE >= h_charge_date;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_REVENUE', 90+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_REVENUE(91) : reconciliation.sql(3429) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   OPEN C_AUTO_REVENUE;
   
   h_prev_contract_id := 'QQ';
   LOOP
      FETCH C_AUTO_REVENUE INTO h_id, h_project_id, h_resource_id, h_type_id,
      h_element_id, h_table_type, h_foreign_date,
      h_regular_quantity, h_regular_rate,
      h_special_quantity, h_special_rate,
      h_tax1, h_tax2,
      h_adj_regular_quantity, h_adj_regular_rate,
      h_adj_special_quantity, h_adj_special_rate,
      h_adj_tax1, h_adj_tax2,
      h_old_charge_date,
      h_contract_id;
      IF C_AUTO_REVENUE%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         EXIT;
      End If;
      
      h_max_charge_amount := 1.00;
      h_total_charge_amount := 1.00;
      
      If (h_old_charge_date IS NULL) Then
         h_old_fiscal_year := 0;
         h_old_fiscal_month := 0;
      Else
         BEGIN
            SELECT FISCAL_YEAR, FISCAL_MONTH
            INTO h_old_fiscal_year, h_old_fiscal_month
            FROM FISCAL_CALENDAR
            WHERE START_DATE <= h_old_charge_date
            AND FINISH_DATE >= h_old_charge_date;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_REVENUE', 138+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_REVENUE(139) : reconciliation.sql(3477) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_bitflag := 0;
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
         h_old_adj_actual_amount := (Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                     + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                     h_adj_special_rate, 0)) / 60.0 + Coalesce(h_adj_tax1, 0) 
                                     + Coalesce(h_adj_tax2, 0);
      Else
         h_old_adj_actual_amount := Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                    + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                    h_adj_special_rate, 0) + Coalesce(h_adj_tax1, 0) + Coalesce(
                                    h_adj_tax2, 0);
      End If;
      
      h_adj_regular_quantity := Coalesce(h_regular_quantity, 0);
      h_adj_special_quantity := Coalesce(h_special_quantity, 0);
      h_adj_regular_rate := Coalesce(h_regular_rate, 0);
      h_adj_special_rate := Coalesce(h_special_rate, 0);
      h_adj_tax1 := Coalesce(h_tax1, 0);
      h_adj_tax2 := Coalesce(h_tax2, 0);
      
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SUMMARY_TIMESHEETS Then
         h_actual_amount := (Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                             h_special_quantity, 0) * Coalesce(h_special_rate, 0)) / 60.0 
                             + Coalesce(h_tax1, 0) + Coalesce(h_tax2, 0);
         h_adj_actual_amount := (Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                 + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                 h_adj_special_rate, 0)) / 60.0 + Coalesce(h_adj_tax1, 0) 
                                 + Coalesce(h_adj_tax2, 0);
      Else
         h_actual_amount := Coalesce(h_regular_quantity, 0) * Coalesce(h_regular_rate, 0) + Coalesce(
                            h_special_quantity, 0) * Coalesce(h_special_rate, 0) + Coalesce(
                            h_tax1, 0) + Coalesce(h_tax2, 0);
         h_adj_actual_amount := Coalesce(h_adj_regular_quantity, 0) * Coalesce(h_adj_regular_rate, 0) 
                                + Coalesce(h_adj_special_quantity, 0) * Coalesce(
                                h_adj_special_rate, 0) + Coalesce(h_adj_tax1, 0) + Coalesce(
                                h_adj_tax2, 0);
      End If;
      
      If Coalesce(h_old_adj_actual_amount, 0) <> Coalesce(h_adj_actual_amount, 0) Then
         h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
      End If;
      
      If (Coalesce(h_old_fiscal_year, 0) <> Coalesce(h_fiscal_year, 0)) OR (Coalesce(
                                               h_old_fiscal_month, 0) <> Coalesce(
                                               h_fiscal_month, 0)) Then
         If (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_old_adj_actual_amount, 0) > 0.00) Then
            If Coalesce(h_fiscal_year, 0) = 0 Then
               h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
            End If;
            
            h_actual_amount := (0.0 - Coalesce(h_old_adj_actual_amount, 0));
            null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_old_fiscal_year, h_old_fiscal_month
                                          , h_actual_amount, h_rec_user);
         End If;
         
         If (Coalesce(h_fiscal_year, 0) > 0) AND (Coalesce(h_adj_actual_amount, 0) > 0.00) Then
            If Coalesce(h_old_fiscal_year, 0) = 0 Then
               h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), 2);
            End If;
            
            null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_adj_actual_amount, h_rec_user);
         End If;
      ElsIf (Coalesce(h_old_fiscal_year, 0) > 0) AND (Coalesce(h_old_adj_actual_amount, 0) <> Coalesce(
                h_adj_actual_amount, 0)) Then
         h_actual_amount := Coalesce(h_adj_actual_amount, 0) - Coalesce(h_old_adj_actual_amount, 0);
         
         null_return := FC_UPDATE_TIMEPHASE_BUDGET(h_bitflag, h_project_id, h_element_id, h_table_type
                                          , h_type_id, h_fiscal_year, h_fiscal_month
                                          , h_actual_amount, h_rec_user);
      End If;
      
      BEGIN
         UPDATE PROJECT_RECONCILIATION
         SET ADJ_REGULAR_QUANTITY = h_adj_regular_quantity,
         ADJ_REGULAR_RATE = h_adj_regular_rate,
         ADJ_SPECIAL_QUANTITY = h_adj_special_quantity,
         ADJ_SPECIAL_RATE = h_adj_special_rate,
         ADJ_TAX1 = h_adj_tax1,
         ADJ_TAX2 = h_adj_tax2,
         CHARGE_DATE = h_charge_date
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_REVENUE', 238+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_REVENUE(239) : reconciliation.sql(3577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   BEGIN
      CLOSE C_AUTO_REVENUE ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_AUTO_CHARGE_LABOR_REVENUE', 251+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_AUTO_CHARGE_LABOR_REVENUE(252) : reconciliation.sql(3590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_REVENUE', 257+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AUTO_CHARGE_LABOR_REVENUE(258) : reconciliation.sql(3596) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_AUTO_CHARGE_LABOR_REVENUE', 261+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_AUTO_CHARGE_LABOR_REVENUE(262) : reconciliation.sql(3600) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_AUTO_CHARGE_LABOR_REVENUE;
ALTER FUNCTION FC_AUTO_CHARGE_LABOR_REVENUE COMPILE DEBUG;

/*****************************************************************************************/

-- p_charge_type:

--  first bit: labor cost

--  second bit: labor revenue

/*****************************************************************************************/

Prompt == FUNCTION SP_AUTO_CHARGE ==
CREATE OR REPLACE FUNCTION SP_AUTO_CHARGE
(
     p_project_id VARCHAR2
   , p_charge_start VARCHAR2
   , p_charge_finish VARCHAR2
   , p_charge_date VARCHAR2
   , p_charge_type PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_charge_start DATE;
   h_charge_finish DATE;
   h_charge_date DATE;
   h_rec_user VARCHAR2(32);
   h_charge_type PLS_INTEGER := 0;
   h_adj_regular_quantity NUMBER := 0;
   h_adj_regular_rate NUMBER := 0;
   h_adj_special_quantity NUMBER := 0;
   h_adj_special_rate NUMBER := 0;
   h_adj_tax1 NUMBER := 0;
   h_adj_tax2 NUMBER := 0;
   h_regular_quantity NUMBER := 0;
   h_regular_rate NUMBER := 0;
   h_special_quantity NUMBER := 0;
   h_special_rate NUMBER := 0;
   h_tax1 NUMBER := 0;
   h_tax2 NUMBER := 0;
   h_actual_amount NUMBER := 0;
   h_adj_actual_amount NUMBER := 0;
   h_old_charge_date DATE;
   h_old_fiscal_year PLS_INTEGER := 0;
   h_old_fiscal_month PLS_INTEGER := 0;
   h_fiscal_year PLS_INTEGER := 0;
   h_fiscal_month PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_foreign_date DATE;
   h_rollup_all PLS_INTEGER := 0;
   h_contract_id VARCHAR2(32);
   h_contract_type PLS_INTEGER := 0;
   h_last_charged_date DATE;
   h_prev_contract_id VARCHAR2(32);
   h_max_charge_amount NUMBER := 0;
   h_total_charge_amount NUMBER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_AUTO_CHARGE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_AUTO_CHARGE'), '-- ' || SYSTIMESTAMP, ':r := SP_AUTO_CHARGE(' || 
       '''' || p_project_id || ''''
          || ', ' || '''' || p_charge_start || ''''
          || ', ' || '''' || p_charge_finish || ''''
          || ', ' || '''' || p_charge_date || ''''
          || ', ' || COALESCE(TO_CHAR(p_charge_type), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_AUTO_CHARGE', 63+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_AUTO_CHARGE(64) : reconciliation.sql(3682) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_project_id := p_project_id;
   h_charge_start := p_charge_start;
   h_charge_finish := p_charge_finish;
   h_charge_date := p_charge_date;
   h_charge_type := p_charge_type;
   h_rec_user := p_rec_user;
   
   If ((h_project_id IS NULL)) OR (h_charge_finish IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_AUTO_CHARGE', 77+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_AUTO_CHARGE(78) : reconciliation.sql(3696) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_AUTO_CHARGE', 80+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_AUTO_CHARGE(81) : reconciliation.sql(3699) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   h_rollup_all := 0;
   h_resource_id := NULL;
   PMO_DB2M.SC_STATE := FC_ROLLUP_COST_CENTER(h_project_id, h_resource_id, h_rollup_all, h_rec_user);
   
   If (BITAND(Coalesce(h_charge_type, 0), 1)) = 1 Then
      PMO_DB2M.SC_STATE := FC_AUTO_CHARGE_LABOR_COST(h_project_id, h_charge_start, h_charge_finish
                                               , h_charge_date, h_rec_user);
   End If;
   
   If (BITAND(Coalesce(h_charge_type, 0), 2)) = 2 Then
      PMO_DB2M.SC_STATE := FC_AUTO_CHARGE_LABOR_REVENUE(h_project_id, h_charge_start, h_charge_finish
                                                  , h_charge_date, h_rec_user);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_AUTO_CHARGE', 100+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_AUTO_CHARGE(101) : reconciliation.sql(3719) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_project_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_AUTO_CHARGE', 107+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_AUTO_CHARGE(108) : reconciliation.sql(3726) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_AUTO_CHARGE', 119+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_AUTO_CHARGE(120) : reconciliation.sql(3738) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_AUTO_CHARGE;
ALTER FUNCTION SP_AUTO_CHARGE COMPILE DEBUG;

