/*********************************************************************************************/

Prompt == FUNCTION FC_CLEANS_WBS_BY_NAME ==
CREATE OR REPLACE FUNCTION FC_CLEANS_WBS_BY_NAME
(
     sh_h_search_option PLS_INTEGER
   , h_current_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_search_option PLS_INTEGER := sh_h_search_option;
   h_element_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_element_name VARCHAR2(255);
   h_ref_code VARCHAR2(50);
   h_approved PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR all_wbs_curs IS
         SELECT PROJECT_ID, ELEMENT_ID, ELEMENT_NAME, REF_CODE
         FROM PMO_WBS_VIEW
         WHERE ELEMENT_NAME <> 'WBS';

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CLEANS_WBS_BY_NAME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CLEANS_WBS_BY_NAME'), '-- ' || SYSTIMESTAMP, ':r := FC_CLEANS_WBS_BY_NAME(' || 
       COALESCE(TO_CHAR(sh_h_search_option), 'NULL')
          || ', ' || '''' || h_current_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_WBS_BY_NAME', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CLEANS_WBS_BY_NAME(27) : views.sql(30) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN all_wbs_curs;
   
   LOOP
      h_element_name := NULL;
      h_ref_code := NULL;
      
      FETCH all_wbs_curs INTO h_project_id, h_element_id, h_element_name, h_ref_code;
      IF all_wbs_curs%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      --//trace_printf("\n h_current_id:%s", h_current_id);
      --//trace_printf("\n h_element_id:%s", h_element_id);
      
      If (h_current_id IS NULL) OR (PMO_ORASTD.strcmp(h_current_id, h_element_id) <> 0) Then
         h_approved := 1;
         h_approved := 1;
         
         If FC_FIND_KEYWORD(h_element_name, h_search_option) = 0 Then
            If FC_FIND_KEYWORD(h_ref_code, h_search_option) = 0 Then
               h_approved := 0;
            End If;
         End If;
         
         --//trace_printf("\n h_search_option:%d", h_search_option);
         --//trace_printf("\n h_approved:%d", h_approved);
         
         If Coalesce(h_approved, 0) = 0 Then
            BEGIN
               DELETE FROM PMO_WBS_VIEW
               WHERE ELEMENT_ID = h_element_id
               AND PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_WBS_BY_NAME', 77+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_WBS_BY_NAME(78) : views.sql(81) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE all_wbs_curs;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CLEANS_WBS_BY_NAME', 93+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CLEANS_WBS_BY_NAME(94) : views.sql(97) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CLEANS_WBS_BY_NAME', 100+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CLEANS_WBS_BY_NAME(101) : views.sql(104) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CLEANS_WBS_BY_NAME;
ALTER FUNCTION FC_CLEANS_WBS_BY_NAME COMPILE DEBUG;

/*************************************************************************/

Prompt == FUNCTION IsTaskFilter ==
CREATE OR REPLACE FUNCTION IsTaskFilter
(
     sh_h_type_id PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := Coalesce(sh_h_type_id, 0);
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('IsTaskFilter') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('IsTaskFilter'), '-- ' || SYSTIMESTAMP, ':r := IsTaskFilter(' || 
       COALESCE(TO_CHAR(sh_h_type_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'IsTaskFilter', 13+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: IsTaskFilter(14) : views.sql(128) : ' || SYSTIMESTAMP);
   END IF; 

   If (h_type_id = PMO_CNST.SC_LATE_TO_START_TASK_ID) 
         OR (h_type_id = PMO_CNST.SC_LATE_TO_FINISH_TASK_ID) 
         OR (h_type_id = PMO_CNST.SC_TASK_WITH_ETC_VARIANCE_ID) 
         OR (h_type_id = PMO_CNST.SC_TASK_DUE_WITHIN_1_WEEK_ID) 
         OR (h_type_id = PMO_CNST.SC_TASK_DUE_WITHIN_4_WEEKS_ID) 
         OR (h_type_id = PMO_CNST.SC_READY_TO_START_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_TASK_STARTING_IN_1_WEEK_ID) 
         OR (h_type_id = PMO_CNST.SC_TASK_STARTING_IN_4_WEEKS_ID) 
         OR (h_type_id = PMO_CNST.SC_CRITICAL_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_IN_PROGRESS_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_COMPLETED_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_CLOSED_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_ALL_TASKS_TYPES_ID) OR (h_type_id = PMO_CNST.SC_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_MILESTONES_ID) 
         OR (h_type_id = PMO_CNST.SC_TRANSFERED_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_REQUIREMENT_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_SERVICE_REQUEST_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_ACTION_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_CHANGE_REQUEST_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_DEFECT_TASKS_ID) OR (h_type_id = PMO_CNST.SC_ISSUE_TASKS_ID) 
         OR (h_type_id = PMO_CNST.SC_RISK_TASKS_ID) Then
      --trace_printf("\n 1 \n");
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'IsTaskFilter', 39+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: IsTaskFilter(40) : views.sql(154) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 1;
   End If;
   --trace_printf("\n 0 \n");
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'IsTaskFilter', 45+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: IsTaskFilter(46) : views.sql(160) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN 0;
END;
/
SHOW ERRORS FUNCTION IsTaskFilter;
ALTER FUNCTION IsTaskFilter COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_GET_PRJ_VAR_CONDITION ==
CREATE OR REPLACE FUNCTION FC_GET_PRJ_VAR_CONDITION
(
     sh_h_type_id PLS_INTEGER
   , h_prj_var_condition IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_type_id PLS_INTEGER := sh_h_type_id;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_GET_PRJ_VAR_CONDITION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_GET_PRJ_VAR_CONDITION'), '-- ' || SYSTIMESTAMP, ':r := FC_GET_PRJ_VAR_CONDITION(' || 
       COALESCE(TO_CHAR(sh_h_type_id), 'NULL')
          || ', ' || '''' || h_prj_var_condition || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 15+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_GET_PRJ_VAR_CONDITION(16) : views.sql(186) : ' || SYSTIMESTAMP);
   END IF; 

   h_prj_var_condition := '';
   -- budget variance
   If Coalesce(h_type_id, 0) = 1100 Then
      h_prj_var_condition := '  ((T.BASELINE1_BUDGET < T.CURRENT_BUDGET) AND (T.BASELINE1_BUDGET > 0)) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 23+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(24) : views.sql(194) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1101 Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND 10 * T.CURRENT_BUDGET < 11 * T.BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 30+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(31) : views.sql(201) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1102) AND (Coalesce(h_type_id, 0) <= 1110) Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100) - 1) 
                       || '  * T.BASELINE1_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100)) || ' * T.BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 41+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(42) : views.sql(212) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1112) AND (Coalesce(h_type_id, 0) <= 1120) Then
      h_prj_var_condition := '  (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100) - 2) 
                       || '  * T.BASELINE1_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1100)) || ' * T.BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 52+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(53) : views.sql(223) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1121 Then
      h_prj_var_condition := ' (T.BASELINE1_BUDGET < T.CURRENT_BUDGET AND T.BASELINE1_BUDGET > 0 AND T.CURRENT_BUDGET >= 3 * T.BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 59+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(60) : views.sql(230) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1200 Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0)         ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 67+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(68) : views.sql(238) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1201 Then
      h_prj_var_condition := ' (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND 10 * T.CURRENT_BUDGET < 11 * T.BASELINE3_BUDGET)          ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 74+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(75) : views.sql(245) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   If (Coalesce(h_type_id, 0) >= 1202) AND (Coalesce(h_type_id, 0) <= 1210) Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200) - 1) 
                       || '  * T.BASELINE3_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200)) || ' * T.BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 86+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(87) : views.sql(257) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1212) AND (Coalesce(h_type_id, 0) <= 1220) Then
      h_prj_var_condition := '  (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200) - 2) 
                       || '  * T.BASELINE3_BUDGET <= 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) AND 10 * (T.CURRENT_BUDGET - T.BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1200)) || ' * T.BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 97+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(98) : views.sql(268) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1221 Then
      h_prj_var_condition := ' (T.BASELINE3_BUDGET < T.CURRENT_BUDGET AND T.BASELINE3_BUDGET > 0 AND T.CURRENT_BUDGET >= 3 * T.BASELINE3_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 104+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(105) : views.sql(275) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1300 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0)           ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 112+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(113) : views.sql(283) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1301 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND 10 * T.ESTIMATION_BUDGET < 11 * T.ESTIMATED_BASELINE1_BUDGET)          ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(120) : views.sql(290) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1302) AND (Coalesce(h_type_id, 0) <= 1310) Then
      h_prj_var_condition := 
                       ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300) - 1) 
                       || '  * T.ESTIMATED_BASELINE1_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300)) 
                       || ' * T.ESTIMATED_BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 132+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(133) : views.sql(303) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1312) AND (Coalesce(h_type_id, 0) <= 1320) Then
      h_prj_var_condition := 
                       ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300) - 2) 
                       || '  * T.ESTIMATED_BASELINE1_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE1_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1300)) 
                       || ' * T.ESTIMATED_BASELINE1_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 145+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(146) : views.sql(316) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1321 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE1_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE1_BUDGET > 0 AND T.ESTIMATION_BUDGET >= 3 * T.ESTIMATED_BASELINE1_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 152+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(153) : views.sql(323) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1400 Then
      h_prj_var_condition := '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0)   ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 160+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(161) : views.sql(331) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1401 Then
      h_prj_var_condition := ' (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND 10 * T.ESTIMATION_BUDGET < 11 * T.ESTIMATED_BASELINE3_BUDGET)   ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 167+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(168) : views.sql(338) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1402) AND (Coalesce(h_type_id, 0) <= 1410) Then
      h_prj_var_condition := 
                       '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400) - 1) 
                       || '  * T.ESTIMATED_BASELINE3_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400)) 
                       || ' * T.ESTIMATED_BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 180+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(181) : views.sql(351) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1412) AND (Coalesce(h_type_id, 0) <= 1420) Then
      h_prj_var_condition := 
                       '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400) - 2) 
                       || '  * T.ESTIMATED_BASELINE3_BUDGET <= 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) AND 10 * (T.ESTIMATION_BUDGET - T.ESTIMATED_BASELINE3_BUDGET) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1400)) 
                       || ' * T.ESTIMATED_BASELINE3_BUDGET) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 193+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(194) : views.sql(364) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1421 Then
      h_prj_var_condition := '  (T.ESTIMATED_BASELINE3_BUDGET < T.ESTIMATION_BUDGET AND T.ESTIMATED_BASELINE3_BUDGET > 0 AND T.ESTIMATION_BUDGET >= 3 * T.ESTIMATED_BASELINE3_BUDGET) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 200+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(201) : views.sql(371) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1500 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 208+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(209) : views.sql(379) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1501 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND 10 * (DATE(T.FINISH_DT) - DATE(T.BASELINE1_FINISH)) <  1  * (DATE(T.BASELINE1_FINISH) - DATE(T.BASELINE1_START))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 215+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(216) : views.sql(386) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1502) AND (Coalesce(h_type_id, 0) <= 1510) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500) - 1) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 228+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(229) : views.sql(399) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1512) AND (Coalesce(h_type_id, 0) <= 1520) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500) - 2) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1500)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE1_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE1_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 241+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(242) : views.sql(412) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1521 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE1_FINISH AND T.BASELINE1_FINISH IS NOT NuLL AND 10 * ((DATE(T.FINISH_DT) - DATE(T.BASELINE1_FINISH))) >= 20 * ((DATE(T.BASELINE1_FINISH) - DATE(T.BASELINE1_START)))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 248+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(249) : views.sql(419) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1600 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 256+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(257) : views.sql(427) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1601 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND 10 * (DATE(T.FINISH_DT) - DATE(T.BASELINE3_FINISH)) <  1  * (DATE(T.BASELINE3_FINISH) - DATE(T.BASELINE3_START))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 263+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(264) : views.sql(434) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1602) AND (Coalesce(h_type_id, 0) <= 1610) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600) - 1) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 276+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(277) : views.sql(447) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1612) AND (Coalesce(h_type_id, 0) <= 1620) Then
      h_prj_var_condition := 
                       '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600) - 2) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10))) <= 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) AND 10 * (TO_DATE(SubStr(T.FINISH_DT, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10))) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1600)) 
                       || '  * (TO_DATE(SubStr(T.BASELINE3_FINISH, 1, 10)) - TO_DATE(SubStr(T.BASELINE3_START, 1, 10)))) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 289+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(290) : views.sql(460) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1621 Then
      h_prj_var_condition := '  (T.FINISH_DT > T.BASELINE3_FINISH AND T.BASELINE3_FINISH IS NOT NuLL AND 10 * ((DATE(T.FINISH_DT) - DATE(T.BASELINE3_FINISH))) >= 20 * ((DATE(T.BASELINE3_FINISH) - DATE(T.BASELINE3_START)))) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 296+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(297) : views.sql(467) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1700 Then
      h_prj_var_condition := '  ((T.PLANNED_EFFORT + T.SCHED_EFFORT) > T.BASELINE1_EFFORT AND T.BASELINE1_EFFORT > 0)            ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 304+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(305) : views.sql(475) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1701 Then
      h_prj_var_condition := '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND 10 * (T.PLANNED_EFFORT + T.SCHED_EFFORT) < 11 * T.BASELINE1_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 311+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(312) : views.sql(482) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1702) AND (Coalesce(h_type_id, 0) <= 1710) Then
      h_prj_var_condition := 
                       '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700) - 1) 
                       || '  * T.BASELINE1_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700)) || ' * T.BASELINE1_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 323+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(324) : views.sql(494) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1712) AND (Coalesce(h_type_id, 0) <= 1720) Then
      h_prj_var_condition := 
                       '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700) - 2) 
                       || '  * T.BASELINE1_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE1_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1700)) || ' * T.BASELINE1_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 335+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(336) : views.sql(506) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1721 Then
      h_prj_var_condition := '  (T.BASELINE1_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE1_EFFORT > 0 AND (T.PLANNED_EFFORT + T.SCHED_EFFORT) >= 3 * T.BASELINE1_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 342+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(343) : views.sql(513) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   /*---------------------------------------------------------------------------*/
   If Coalesce(h_type_id, 0) = 1800 Then
      h_prj_var_condition := ' ((T.PLANNED_EFFORT + T.SCHED_EFFORT) > T.BASELINE3_EFFORT AND T.BASELINE3_EFFORT > 0)            ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 350+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(351) : views.sql(521) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1801 Then
      h_prj_var_condition := '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND 10 * (T.PLANNED_EFFORT + T.SCHED_EFFORT) < 11 * T.BASELINE3_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 357+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(358) : views.sql(528) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1802) AND (Coalesce(h_type_id, 0) <= 1810) Then
      h_prj_var_condition := 
                       '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800) - 1) 
                       || '  * T.BASELINE3_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800)) || ' * T.BASELINE3_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 369+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(370) : views.sql(540) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If (Coalesce(h_type_id, 0) >= 1812) AND (Coalesce(h_type_id, 0) <= 1820) Then
      h_prj_var_condition := 
                       '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800) - 2) 
                       || '  * T.BASELINE3_EFFORT <= 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) AND 10 * ((T.PLANNED_EFFORT + T.SCHED_EFFORT) - T.BASELINE3_EFFORT) < ' 
                       || TO_CHAR((Coalesce(h_type_id, 0) - 1800)) || ' * T.BASELINE3_EFFORT) ';
      h_prj_var_condition := h_prj_var_condition;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 381+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(382) : views.sql(552) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   If Coalesce(h_type_id, 0) = 1821 Then
      h_prj_var_condition := '  (T.BASELINE3_EFFORT < (T.PLANNED_EFFORT + T.SCHED_EFFORT) AND T.BASELINE3_EFFORT > 0 AND (T.PLANNED_EFFORT + T.SCHED_EFFORT) >= 3 * T.BASELINE3_EFFORT) ';
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 388+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(389) : views.sql(559) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN 0;
   End If;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_GET_PRJ_VAR_CONDITION', 396+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_GET_PRJ_VAR_CONDITION(397) : views.sql(567) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_GET_PRJ_VAR_CONDITION;
ALTER FUNCTION FC_GET_PRJ_VAR_CONDITION COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_APPLY_VARIANCE_SECURITY ==
CREATE OR REPLACE FUNCTION FC_APPLY_VARIANCE_SECURITY
(
     sh_h_security_value PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_security_value PLS_INTEGER := sh_h_security_value;
   h_security_num PLS_INTEGER := 0;
   h_security_bitflag PLS_INTEGER := 0;
   h_project_id VARCHAR2(32);
   h_pure_rank VARCHAR2(200);
   null_return LONG;
-- Declare Cursors
      CURSOR prj_list IS
         SELECT DISTINCT ELEMENT_ID, RANK
         FROM PMO_TMP_PROJECTS_01
         ORDER BY RANK;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_APPLY_VARIANCE_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_APPLY_VARIANCE_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := FC_APPLY_VARIANCE_SECURITY(' || 
       COALESCE(TO_CHAR(sh_h_security_value), 'NULL')
          || ', ' || '''' || h_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_VARIANCE_SECURITY', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_APPLY_VARIANCE_SECURITY(27) : views.sql(604) : ' || SYSTIMESTAMP);
   END IF; 

   --timeLog(file,"------------------>BEGIN FC_APPLY_VARIANCE_SECURITY ");
   
   OPEN prj_list;
   
   LOOP
      FETCH prj_list INTO h_project_id, h_pure_rank;
      IF prj_list%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
         EXIT;
      End If;
      
      h_security_num := 0;
      h_security_bitflag := 1;
      null_return := FC_GET_PRJ_SECURITY_NUM(h_pure_rank, h_security_num, h_resource_id
                                       , h_security_bitflag);
      
      If (BITAND(Coalesce(h_security_num, 0), Coalesce(h_security_value, 0))) <> Coalesce(
                                                 h_security_value, 0) Then
         BEGIN
            DELETE FROM PMO_TMP_PROJECTS_01
            WHERE ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_APPLY_VARIANCE_SECURITY', 65+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_APPLY_VARIANCE_SECURITY(66) : views.sql(643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE prj_list ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_APPLY_VARIANCE_SECURITY', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_APPLY_VARIANCE_SECURITY(81) : views.sql(658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   
   --timeLog(file,"------------------>END   FC_APPLY_VARIANCE_SECURITY ");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_APPLY_VARIANCE_SECURITY', 90+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_APPLY_VARIANCE_SECURITY(91) : views.sql(668) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_APPLY_VARIANCE_SECURITY;
ALTER FUNCTION FC_APPLY_VARIANCE_SECURITY COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID ==
CREATE OR REPLACE FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID
(
     sh_h_bitflag PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_executive_security IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_type PLS_INTEGER := sh_h_type;
   h_project_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_pure_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR CRPTLA3 IS
         SELECT DISTINCT ELEMENT_ID, RANK
         FROM PMO_TMP_PROJECTS_01;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_FIX_WBS_VIEW_SECURITY_ID') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_FIX_WBS_VIEW_SECURITY_ID'), '-- ' || SYSTIMESTAMP, ':r := FC_FIX_WBS_VIEW_SECURITY_ID(' || 
       COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_executive_security || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_WBS_VIEW_SECURITY_ID', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_FIX_WBS_VIEW_SECURITY_ID(29) : views.sql(707) : ' || SYSTIMESTAMP);
   END IF; 

   If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 302) Then
      BEGIN
         UPDATE PMO_WBS_VIEW W
         SET (W.PARTICIPANT, W.SECURITY_ID) = (SELECT R.PARTICIPANT, R.SECURITY_ID
         FROM RESOURCE_ASSIGNMENTS R
         WHERE R.ELEMENT_ID = W.ELEMENT_ID
         AND R.RESOURCE_ID = h_resource_id
         AND TABLE_TYPE = PMO_CNST.SC_TABLE_WBS)
         WHERE 1 = 1;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 50+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(51) : views.sql(729) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (Coalesce(h_type, 0) <> 0) AND (Coalesce(h_type, 0) <> 1) AND (Coalesce(h_type, 0) <> 145) Then
      OPEN CRPTLA3;
      
      LOOP
         FETCH CRPTLA3 INTO h_project_id, h_pure_rank;
         IF CRPTLA3%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM TMP_INHERIT_SECURITY';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            UPDATE PMO_WBS_VIEW W
            SET W.SECURITY_ID = (SELECT R.SECURITY_ID FROM RESOURCE_ASSIGNMENTS R
            WHERE R.PROJECT_ID = h_project_id AND R.ELEMENT_ID = W.ELEMENT_ID
            AND R.RESOURCE_ID = h_resource_id)
            WHERE W.PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 91+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(92) : views.sql(770) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_WBS_VIEW W
            SET W.ASSIGNED = 1
            WHERE W.PROJECT_ID = h_project_id
            AND W.SECURITY_ID IS NOT NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 111+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(112) : views.sql(790) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO TMP_INHERIT_SECURITY (PROJECT_ID, ELEMENT_ID, RANK, SECURITY_ID)
            SELECT PROJECT_ID, ELEMENT_ID, RANK, SECURITY_ID
            FROM PMO_WBS_VIEW
            WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(132) : views.sql(810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            h_table_type := PMO_CNST.SC_TABLE_WBS;
            PMO_DB2M.SC_STATE := FC_INHERIT_SECURITY(h_table_type, h_project_id, h_project_id, h_resource_id);
            
            BEGIN
               UPDATE PMO_WBS_VIEW W
               SET W.SECURITY_ID = (SELECT S.SECURITY_IDS FROM TMP_INHERIT_SECURITY S
               WHERE S.PROJECT_ID = h_project_id AND S.ELEMENT_ID = W.ELEMENT_ID)
               WHERE W.PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 155+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(156) : views.sql(834) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      BEGIN
         CLOSE CRPTLA3 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(170) : views.sql(848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      UPDATE PMO_WBS_VIEW T
      SET T.TYPE_ID = PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT + T.TYPE_ID
      WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01 P WHERE PROJECT_FLAG >= PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 189+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(190) : views.sql(868) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE PMO_WBS_VIEW T
      SET T.SECURITY_ID = case when T.TYPE_ID in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (T.SECURITY_ID = '' OR T.SECURITY_ID is Null) then PMO_CNST.SCPROJECTVISITOR_DB_STR
      when T.TYPE_ID > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT then h_executive_security || case when (T.SECURITY_ID = '' OR T.SECURITY_ID is Null) then PMO_CNST.SCPROJECTVISITOR_DB_STR else T.SECURITY_ID end
      else T.SECURITY_ID end
      WHERE T.PROJECT_ID IN (SELECT P.ELEMENT_ID FROM PMO_TMP_PROJECTS_01 P);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_FIX_WBS_VIEW_SECURITY_ID', 210+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_FIX_WBS_VIEW_SECURITY_ID(211) : views.sql(889) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_FIX_WBS_VIEW_SECURITY_ID', 218+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_FIX_WBS_VIEW_SECURITY_ID(219) : views.sql(897) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID;
ALTER FUNCTION FC_FIX_WBS_VIEW_SECURITY_ID COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_WBS_PORTFOLIO_VIEW ==
CREATE OR REPLACE FUNCTION FC_WBS_PORTFOLIO_VIEW
(
     sh_h_sp_id PLS_INTEGER
   , h_wkf_resource_id IN OUT NOCOPY VARCHAR2
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_ids IN OUT NOCOPY VARCHAR2
   , sh_h_active_record PLS_INTEGER
   , h_full_name IN OUT NOCOPY VARCHAR2
   , h_column_set IN OUT NOCOPY VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_sp_id PLS_INTEGER := sh_h_sp_id;
   h_active_record PLS_INTEGER := sh_h_active_record;
   h_tree_type PLS_INTEGER := 0;
   h_res_row_rank VARCHAR2(200);
   h_table_type PLS_INTEGER := 0;
   h_executive_security VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_can_view_pivot PLS_INTEGER := 0;
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_id VARCHAR2(32);
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_element_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_tmp_security_ids VARCHAR2(4000);
   h_security_ids VARCHAR2(4000);
   h_type_id PLS_INTEGER := 0;
   idList VARCHAR2(3199);
   myStmt VARCHAR2(29999);
   null_return LONG;
-- Declare Cursors
      CURSOR PORT_SECURITY IS
         SELECT DISTINCT PARENT_ID, TYPE_ID, SECURITY_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         ORDER BY PARENT_ID;

      CURSOR PORT_SECURITY1 IS
         SELECT V.PARENT_ID, P.LEVEL_ID, P.RANK, V.SECURITY_ID
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         ORDER BY P.RANK DESC;

      CURSOR PORT_SECURITY2 IS
         SELECT DISTINCT PARENT_ID, TYPE_ID, SECURITY_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         ORDER BY PARENT_ID;

      CURSOR PORT_SECURITY3 IS
         SELECT V.PARENT_ID, P.RANK, COALESCE(V.SECURITY_ID, '')
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         ORDER BY P.RANK DESC;

      CURSOR PORT_SECURITY4 IS
         SELECT V.SECURITY_ID
         FROM PMO_TMP_VIEWS_LEVEL1 V, TMT_PORTFOLIOS P
         WHERE V.PARENT_ID = P.ELEMENT_ID
         AND SUBSTR(P.RANK, 1, LENGTH(P.RANK)-4) = h_rank;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_WBS_PORTFOLIO_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_WBS_PORTFOLIO_VIEW'), '-- ' || SYSTIMESTAMP, ':r := FC_WBS_PORTFOLIO_VIEW(' || 
       COALESCE(TO_CHAR(sh_h_sp_id), 'NULL')
          || ', ' || '''' || h_wkf_resource_id || ''''
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_ids || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_active_record), 'NULL')
          || ', ' || '''' || h_full_name || ''''
          || ', ' || '''' || h_column_set || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 80+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_WBS_PORTFOLIO_VIEW(81) : views.sql(988) : ' || SYSTIMESTAMP);
   END IF; 

   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_ROOT';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_01';
   PMO_DB2M.SC_STATE := 0;
   
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_LEVEL1';
   PMO_DB2M.SC_STATE := 0;
   
   BEGIN
      SELECT RANK || '%'
      INTO h_res_row_rank
      FROM TMT_PORTFOLIOS
      WHERE ELEMENT_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 106+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(107) : views.sql(1014) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      INSERT INTO PMO_TMP_VIEWS_ROOT(PARENT_ID, ELEMENT_ID, PARTICIPANT, SECURITY_ID, RANK, MILESTONE_PERCENT, STAGE_ID, TYPE_ID)
      SELECT COALESCE(P.PARENT_ID,'ROOT') AS PARENT_ID, T.ELEMENT_ID, A.PARTICIPANT,
      COALESCE(A.SECURITY_ID, ''), T.RANK, T.MILESTONE_PERCENT, T.STAGE_ID , T.TYPE_ID
      FROM PROJECT_ASSIGNMENTS A
      JOIN TMT_WBS T ON T.PROJECT_ID = A.PROJECT_ID
      AND T.ELEMENT_ID = A.PROJECT_ID
      AND T.REC_STATUS <> 'D'
      AND T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
      LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE h_res_row_rank
      AND P.ELEMENT_ID = T.ELEMENT_ID
      WHERE A.RESOURCE_ID = h_resource_id
      AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED ='1' ));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 134+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(135) : views.sql(1042) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_tree_type := PMO_CNST.SC_TABLE_ATTRIBUTE;
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   h_executive_security := NULL;
   h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
   h_bitflag := Coalesce(h_bitflag, 0);
   
   If Coalesce(h_bitflag, 0) = 1 Then
      null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      
      -- insert pure executive projects
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_ROOT (PARENT_ID, ELEMENT_ID, PARTICIPANT, SECURITY_ID, RANK, MILESTONE_PERCENT, STAGE_ID, TYPE_ID)
         SELECT COALESCE(P.PARENT_ID, 'ROOT'), RP.ELEMENT_ID, 0 AS PARTICIPANT, COALESCE(PA.SECURITY_ID, '') AS SECURITY_ID, RP.RANK, T.MILESTONE_PERCENT, T.STAGE_ID, RP.PROJECT_FLAG
         FROM PMO_TMP_PROJECTS_01 RP
         JOIN TMT_WBS T ON T.ELEMENT_ID = RP.ELEMENT_ID
         AND T.PROJECT_ID = RP.ELEMENT_ID
         LEFT JOIN PROJECT_ASSIGNMENTS PA ON PA.PROJECT_ID = RP.ELEMENT_ID AND PA.RESOURCE_ID = h_resource_id
         LEFT JOIN TMT_PORTFOLIOS P ON P.RANK LIKE h_res_row_rank
         AND P.ELEMENT_ID = RP.ELEMENT_ID
         WHERE RP.ELEMENT_ID NOT IN (SELECT ELEMENT_ID FROM PMO_TMP_VIEWS_ROOT);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 170+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(171) : views.sql(1078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- update assigned_executive projects
      BEGIN
         UPDATE PMO_TMP_VIEWS_ROOT
         SET TYPE_ID = TYPE_ID + PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
         WHERE ELEMENT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(191) : views.sql(1098) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   BEGIN
      DELETE FROM PMO_TMP_VIEWS_ROOT
      WHERE ELEMENT_ID IN (SELECT B.ELEMENT_ID
      FROM PMO_TMP_VIEWS_ROOT A, PMO_TMP_VIEWS_ROOT B
      WHERE A.RANK < B.RANK AND ((A.RANK >= SUBSTR(B.RANK, 1, 8) AND A.RANK <= B.RANK) 
      AND SUBSTR(B.RANK,1, CASE WHEN LENGTH(A.RANK) > LENGTH(B.RANK) THEN 1 ELSE LENGTH(A.RANK) END ) = A.RANK));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 212+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(213) : views.sql(1120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED)) = PMO_CNST.SC_INCLUDE_CLOSED Then
      h_include_closed_items := 1;
   End If;
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED)) = PMO_CNST.SC_INCLUDE_COMPLETED Then
      h_include_completed_items := 1;
   End If;
   
   If (Coalesce(h_include_closed_items, 0) = 1) AND (Coalesce(h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE (MILESTONE_PERCENT >= 10000 AND STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929))
         AND STAGE_ID NOT IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 241+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(242) : views.sql(1149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 1) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 260+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(261) : views.sql(1168) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_TMP_VIEWS_ROOT
         WHERE ((MILESTONE_PERCENT >= 10000 AND STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929)) OR STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 279+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(280) : views.sql(1187) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (h_ids IS NULL) Then
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_01
         SELECT PARENT_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT'
         UNION ALL
         SELECT PARENT_ID FROM TMT_PORTFOLIOS
         WHERE RANK LIKE h_res_row_rank AND TYPE_ID = 64 AND REC_STATUS <> 'D' AND LEVEL_ID = 2;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 304+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(305) : views.sql(1212) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_LEVEL1 (PARENT_ID, CHILD_COUNT)
         SELECT PARENT_ID, COUNT(*) AS CHILD_COUNT
         FROM PMO_TMP_VIEWS_01
         GROUP BY PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 324+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(325) : views.sql(1232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE PMO_TMP_VIEWS_ROOT
         SET SECURITY_ID = case when type_id in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (SECURITY_ID = '' OR SECURITY_ID is Null)
         then PMO_CNST.SCPROJECTVISITOR_DB_STR
         when type_id > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
         then case when (SECURITY_ID = '' OR SECURITY_ID is Null)
         then PMO_CNST.SCPROJECTVISITOR_DB_STR
         else SECURITY_ID end || h_executive_security
         else SECURITY_ID end;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 348+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(349) : views.sql(1256) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN PORT_SECURITY;
      
      h_parent_id := 'YY';
      h_security_ids := '';
      LOOP
         FETCH PORT_SECURITY INTO h_element_id, h_type_id, h_tmp_security_ids;
         IF PORT_SECURITY%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If PMO_ORASTD.strcmp(h_parent_id, 'YY') = 0 Then
            h_parent_id := h_element_id;
            h_security_ids := h_tmp_security_ids;
         ElsIf PMO_ORASTD.strcmp(h_parent_id, h_element_id) = 0 Then
            -- function PMO_InStr was changed to return 0 not found, else 1 based index for the character
            -- TOTEST: Vladimir Schneider, vladimir@winpte.com
            If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
               h_security_ids := SubStr(h_security_ids || ',', 1, 4000);
               h_security_ids := SubStr(h_security_ids || h_tmp_security_ids, 1, 4000);
            End If;
         Else
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 394+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(395) : views.sql(1302) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_parent_id := h_element_id;
            h_security_ids := h_tmp_security_ids;
         End If;
      END LOOP;
      BEGIN
         CLOSE PORT_SECURITY;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 411+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(412) : views.sql(1319) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strcmp(h_parent_id, 'YY') <> 0 Then
         BEGIN
            UPDATE PMO_TMP_VIEWS_LEVEL1
            SET SECURITY_ID = h_security_ids
            WHERE PARENT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 431+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(432) : views.sql(1339) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      OPEN PORT_SECURITY1;
      
      h_security_ids := '';
      LOOP
         FETCH PORT_SECURITY1 INTO h_element_id, h_level_id, h_rank, h_tmp_security_ids;
         IF PORT_SECURITY1%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
            EXIT;
         End If;
         
         If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
            If Coalesce(Length(h_security_ids), 0) <> 0 Then
               h_security_ids := SubStr(h_security_ids || ',', 1, 4000);
            End If;
            h_security_ids := SubStr(h_security_ids || h_tmp_security_ids, 1, 4000);
         End If;
         
         If Coalesce(h_level_id, 0) = 1 Then
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 474+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(475) : views.sql(1382) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_security_ids := '';
         End If;
      END LOOP;
      BEGIN
         CLOSE PORT_SECURITY1;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 490+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(491) : views.sql(1398) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_AGGREGATE_PORTFOLIO Then
         -- returns all projects in a flat list
         myStmt := 
               'SELECT 0 AS LEVEL_ID, T.TYPE_ID, CASE WHEN T.TYPE_ID = 1 THEN ''qqqqqqq1'' ELSE ''qqqqqqq2'' END || RTRIM(T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, ';
         myStmt := myStmt || 
                    ' T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
         myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
         myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
         myStmt := myStmt || h_full_name;
         myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
         myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         myStmt := myStmt || ' FROM PMO_TMP_VIEWS_ROOT A, TMT_WBS T, PROJECT_DETAILS PD ';
         myStmt := myStmt || 'WHERE T.RANK = A.RANK ';
         myStmt := myStmt || ' AND PD.PROJECT_ID = T.ELEMENT_ID ';
         
         --trace_printf("\n  141 part 1 \n");
      Else
         myStmt := 
               'SELECT 0 AS LEVEL_ID, T.TYPE_ID, CASE WHEN T.TYPE_ID = 1 THEN ''qqqqqqq1'' ELSE ''qqqqqqq2'' END || RTRIM(T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, ';
         
         If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
            myStmt := myStmt || 
                    ' T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
         Else
            myStmt := myStmt || ' T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
         End If;
         
         myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
         
         myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
         myStmt := myStmt || h_full_name;
         myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
         myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         myStmt := myStmt || ' FROM PMO_TMP_VIEWS_ROOT A, TMT_WBS T, PROJECT_DETAILS PD ';
         myStmt := myStmt || 'WHERE A.PARENT_ID = ''ROOT'' AND T.RANK = A.RANK ';
         myStmt := myStmt || ' AND PD.PROJECT_ID = T.ELEMENT_ID ';
         
         myStmt := myStmt || ' UNION ALL ';
         
         myStmt := myStmt || 
                    ' SELECT P.LEVEL_ID - 1 AS LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
         myStmt := myStmt || 
                    'COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ';
         myStmt := myStmt || 
                    '''SCFOLDERMANAGER'' AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ''N'' AS ASSIGNED,'''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 
                    'FROM TMT_PORTFOLIOS P LEFT JOIN PMO_TMP_VIEWS_LEVEL1 L1 ON L1.PARENT_ID = P.ELEMENT_ID ';
         myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON T.PROJECT_ID = P.ELEMENT_ID AND T.ELEMENT_ID = P.ELEMENT_ID ';
         myStmt := myStmt || 'WHERE P.PARENT_ID = ''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || ''' ORDER BY RANK ASC';
         
         --trace_printf("\n  141 part 2 \n");
         
      End If;
   Else
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_01
         SELECT PARENT_ID
         FROM PMO_TMP_VIEWS_ROOT
         WHERE PARENT_ID <> 'ROOT';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 581+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(582) : views.sql(1489) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_total := Trunc((Coalesce(Length(h_ids), 0) + 1) / 32);
      h_count := 0;
      WHILE Coalesce(h_count, 0) < Coalesce(h_total, 0) LOOP
         h_id := PMO_STPU.strnpcpy(32, 32, Coalesce(h_count, 0) * 32+1, h_ids);
         h_id := h_id;
         
         BEGIN
            SELECT LEVEL_ID, RANK || '%'
            INTO h_level_id, h_rank
            FROM TMT_PORTFOLIOS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 603+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(604) : views.sql(1511) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_VIEWS_01
            SELECT PARENT_ID
            FROM TMT_PORTFOLIOS
            WHERE RANK LIKE h_rank
            AND REC_STATUS <> 'D'
            AND TYPE_ID = 64
            AND LEVEL_ID >= h_level_id
            AND LEVEL_ID <= h_level_id + 2;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 627+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(628) : views.sql(1535) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) = 0 Then
            idList := '(''' || h_id || '''';
         Else
            idList := idList || ',''';
            idList := idList || h_id;
            idList := idList || '''';
         End If;
         h_count := Coalesce(h_count, 0) + 1;
      END LOOP;
      idList := idList || ')';
      
      BEGIN
         INSERT INTO PMO_TMP_VIEWS_LEVEL1 (PARENT_ID, CHILD_COUNT)
         SELECT PARENT_ID, COUNT(*) AS CHILD_COUNT
         FROM PMO_TMP_VIEWS_01
         GROUP BY PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 658+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(659) : views.sql(1566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_ORASTD.strncmp(h_wkf_resource_id, h_resource_id, 32) <> 0 Then
         BEGIN
            UPDATE PMO_TMP_VIEWS_ROOT R
            SET SECURITY_ID = (
            SELECT A.SECURITY_ID
            FROM PROJECT_ASSIGNMENTS A
            WHERE R.ELEMENT_ID = A.PROJECT_ID
            AND A.RESOURCE_ID = h_wkf_resource_id
            AND (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0)));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 682+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(683) : views.sql(1590) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         -- delete all projects not in tmt_portfolios
         BEGIN
            DELETE FROM PMO_TMP_VIEWS_ROOT WHERE PARENT_ID = 'ROOT';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 700+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(701) : views.sql(1608) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE PMO_TMP_VIEWS_ROOT
            SET SECURITY_ID = case when type_id in (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) and (SECURITY_ID = '' OR SECURITY_ID is Null)
            then PMO_CNST.SCPROJECTVISITOR_DB_STR
            when type_id > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT
            then case when (SECURITY_ID = '' OR SECURITY_ID is Null)
            then PMO_CNST.SCPROJECTVISITOR_DB_STR
            else SECURITY_ID end || h_executive_security
            else SECURITY_ID end;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 724+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(725) : views.sql(1632) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf("\n h_executive_security:%s", h_executive_security);
         
         OPEN PORT_SECURITY2;
         
         h_parent_id := 'YY';
         h_security_ids := '';
         LOOP
            FETCH PORT_SECURITY2 INTO h_element_id, h_type_id, h_tmp_security_ids;
            IF PORT_SECURITY2%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            --trace_printf("h_element_id; %s\n", h_element_id);
            If PMO_ORASTD.strcmp(h_parent_id, 'YY') = 0 Then
               h_parent_id := h_element_id;
               h_security_ids := h_tmp_security_ids;
            ElsIf PMO_ORASTD.strcmp(h_parent_id, h_element_id) = 0 Then
               -- function PMO_InStr was changed to return 0 not found, else 1 based index for the character
               -- TOTEST: Vladimir Schneider, vladimir@winpte.com
               If InStr(h_security_ids, h_tmp_security_ids) = 0 Then
                  h_security_ids := SubStr(h_security_ids || ',', 1, 4000);
                  h_security_ids := SubStr(h_security_ids || h_tmp_security_ids, 1, 4000);
               End If;
            Else
               --trace_printf("h_parent_id0; %s\n", h_parent_id);
               BEGIN
                  UPDATE PMO_TMP_VIEWS_LEVEL1
                  SET SECURITY_ID = h_security_ids
                  WHERE PARENT_ID = h_parent_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 774+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(775) : views.sql(1682) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_parent_id := h_element_id;
               h_security_ids := h_tmp_security_ids;
            End If;
         END LOOP;
         BEGIN
            CLOSE PORT_SECURITY2;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 791+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(792) : views.sql(1699) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_parent_id, 'YY') <> 0 Then
            --trace_printf("h_parent_id1; %s\n", h_parent_id);
            BEGIN
               UPDATE PMO_TMP_VIEWS_LEVEL1
               SET SECURITY_ID = h_security_ids
               WHERE PARENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 812+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(813) : views.sql(1720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         OPEN PORT_SECURITY3;
         
         LOOP
            FETCH PORT_SECURITY3 INTO h_element_id, h_rank, h_security_ids;
            IF PORT_SECURITY3%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            OPEN PORT_SECURITY4;
            
            h_count := 0;
            LOOP
               FETCH PORT_SECURITY4 INTO h_tmp_security_ids;
               IF PORT_SECURITY4%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If NOT (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               h_count := Coalesce(h_count, 0) + 1;
               If Coalesce(Length(h_security_ids), 0) > 0 Then
                  h_security_ids := SubStr(h_security_ids || ',', 1, 4000);
               End If;
               h_security_ids := SubStr(h_security_ids || h_tmp_security_ids, 1, 4000);
            END LOOP;
            BEGIN
               CLOSE PORT_SECURITY4;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 861+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(862) : views.sql(1769) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) > 0 Then
               BEGIN
                  UPDATE PMO_TMP_VIEWS_LEVEL1
                  SET SECURITY_ID = h_security_ids
                  WHERE PARENT_ID = h_element_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 881+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(882) : views.sql(1789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         END LOOP;
         BEGIN
            CLOSE PORT_SECURITY3;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 895+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(896) : views.sql(1803) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_PORTFOLIO Then
         myStmt := 'SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
         myStmt := myStmt || 
                    'COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ';
         myStmt := myStmt || '''SCFOLDERMANAGER'' AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ';
         myStmt := myStmt || '''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, ''1899-01-01'' AS PURSUIT_START, ''1899-01-01'' AS FORECAST_CLOSED, ''1899-01-01'' AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P LEFT JOIN PMO_TMP_VIEWS_LEVEL1 L1 ';
         myStmt := myStmt || 'ON L1.PARENT_ID = P.ELEMENT_ID ';
         myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || 'WHERE (P.ELEMENT_ID IN ';
         myStmt := myStmt || idList;
         myStmt := myStmt || ' OR P.PARENT_ID IN ';
         myStmt := myStmt || idList;
         myStmt := myStmt || ') AND P.TYPE_ID = 64 ';
         myStmt := myStmt || ' ORDER BY RANK';
         
         --trace_printf("\n idList:%s", idList);
         
      ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_AGGREGATE_PORTFOLIO Then
         -- returns all projects in a flat list
         myStmt := 
               'SELECT 0 AS LEVEL_ID, T.TYPE_ID, CASE WHEN T.TYPE_ID = 1 THEN ''qqqqqqq1'' ELSE ''qqqqqqq2'' END || RTRIM(T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, ';
         myStmt := myStmt || 
                    ' T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
         myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
         myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
         myStmt := myStmt || h_full_name;
         myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
         myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         myStmt := myStmt || ' FROM PMO_TMP_VIEWS_ROOT A, TMT_WBS T, PROJECT_DETAILS PD ';
         myStmt := myStmt || 'WHERE T.RANK = A.RANK ';
         myStmt := myStmt || ' AND PD.PROJECT_ID = T.ELEMENT_ID ';
         
         --trace_printf("\n  141 part 3 \n");
         
      Else
         myStmt := 'SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
         myStmt := myStmt || 
                    'COALESCE(L1.CHILD_COUNT,0) AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ';
         myStmt := myStmt || '''SCFOLDERMANAGER'' AS SECURITY_ID, L1.SECURITY_ID AS PORTFOLIO_SECURITY, ';
         myStmt := myStmt || '''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P LEFT JOIN PMO_TMP_VIEWS_LEVEL1 L1 ';
         myStmt := myStmt || 'ON L1.PARENT_ID = P.ELEMENT_ID ';
         myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || 'WHERE (P.ELEMENT_ID IN ';
         myStmt := myStmt || idList;
         myStmt := myStmt || ' OR P.PARENT_ID IN ';
         myStmt := myStmt || idList;
         myStmt := myStmt || ') AND P.TYPE_ID = 64 ';
         
         myStmt := myStmt || ' UNION ALL ';
         
         myStmt := myStmt || 'SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, ';
         
         If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
            myStmt := myStmt || 
                    'T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
         Else
            myStmt := myStmt || 'T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
         End If;
         
         myStmt := myStmt || 
                    'A.PARTICIPANT, COALESCE(A.SECURITY_ID,''SCPROJECTWORKFLOWVISITOR'') AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
         myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
         myStmt := myStmt || h_full_name;
         myStmt := myStmt || 
                    ''' || '';'') > 0 THEN ''M'' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || h_column_set;
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P JOIN TMT_WBS T ON P.RANK LIKE ''';
         myStmt := myStmt || h_res_row_rank;
         myStmt := myStmt || ''' AND P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
         
         myStmt := myStmt || 'JOIN PMO_TMP_VIEWS_ROOT A ON P.ELEMENT_ID = A.ELEMENT_ID AND A.PARENT_ID IN ';
         myStmt := myStmt || idList;
         myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || ' ORDER BY RANK';
         
         --trace_printf("\n  141 part 4 \n");
      End If;
      --trace_printf("\n  WBS VIEW-------> resultset 012  -> myStmt:%s\n", myStmt);
   End If;
   --trace_printf("\n%s\n", myStmt);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 1002+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_PORTFOLIO_VIEW(1003) : views.sql(1910) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_PORTFOLIO_VIEW', 1005+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_PORTFOLIO_VIEW(1006) : views.sql(1913) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(myStmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 1013+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(1014) : views.sql(1921) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_PORTFOLIO_VIEW', 1025+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_PORTFOLIO_VIEW(1026) : views.sql(1933) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION FC_WBS_PORTFOLIO_VIEW;
ALTER FUNCTION FC_WBS_PORTFOLIO_VIEW COMPILE DEBUG;

/*********************************************************************************************/

Prompt == FUNCTION FC_WBS_VIEW ==
CREATE OR REPLACE FUNCTION FC_WBS_VIEW
(
     sh_h_sp_id PLS_INTEGER
   , h_resource_id IN OUT NOCOPY VARCHAR2
   , h_name IN OUT NOCOPY VARCHAR2
   , h_filter_string IN OUT NOCOPY VARCHAR2
   , sh_h_filter_count PLS_INTEGER
   , sh_h_type PLS_INTEGER
   , sh_h_active_record PLS_INTEGER
   , sh_h_stage_id PLS_INTEGER
   , sh_h_is_single_project PLS_INTEGER
   , h_wbs_search_criteria IN OUT NOCOPY PLS_INTEGER
   , h_prj_var_condition IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_sp_id PLS_INTEGER := sh_h_sp_id;
   h_filter_count PLS_INTEGER := sh_h_filter_count;
   h_type PLS_INTEGER := sh_h_type;
   h_active_record PLS_INTEGER := sh_h_active_record;
   h_stage_id PLS_INTEGER := sh_h_stage_id;
   h_is_single_project PLS_INTEGER := sh_h_is_single_project;
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_security_id VARCHAR2(32);
   h_security_ids VARCHAR2(4000);
   h_table_type PLS_INTEGER := 0;
   h_project_name VARCHAR2(255);
   h_tpf_parent_id VARCHAR2(32);
   h_level VARCHAR2(4);
   h_str_filter_count VARCHAR2(4);
   h_start_finish_date DATE;
   h_effort_driven PLS_INTEGER := 0;
   h_minute_per_day PLS_INTEGER := 0;
   typeList VARCHAR2(99);
   myStmt VARCHAR2(29999);
   dynamicSql VARCHAR2(1999);
   h_filedName VARCHAR2(254);
   h_keyword_search1 VARCHAR2(8000);
   h_keyword_search2 VARCHAR2(8000);
   h_critical_element_filter VARCHAR2(255);
   h_critical_filter PLS_INTEGER := 0;
   h_exculde_sub_elements PLS_INTEGER := 0;
   h_is_critical_tasks PLS_INTEGER := 0;
   h_next_level_only PLS_INTEGER := 0;
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_is_del_manager_counter PLS_INTEGER := 0;
   h_condition_flag PLS_INTEGER := 0;
   localType PLS_INTEGER;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_WBS_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_WBS_VIEW'), '-- ' || SYSTIMESTAMP, ':r := FC_WBS_VIEW(' || 
       COALESCE(TO_CHAR(sh_h_sp_id), 'NULL')
          || ', ' || '''' || h_resource_id || ''''
          || ', ' || '''' || h_name || ''''
          || ', ' || '''' || h_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_filter_count), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_active_record), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_stage_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_is_single_project), 'NULL')
          || ', ' || COALESCE(TO_CHAR(h_wbs_search_criteria), 'NULL')
          || ', ' || '''' || h_prj_var_condition || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_VIEW', 67+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_WBS_VIEW(68) : views.sql(2017) : ' || SYSTIMESTAMP);
   END IF; 

   -- UPDATE FILTER_FLAG IN TEMPORARY TABLE
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_DELIVERABLE_MANAGER_COUNTER)) = PMO_CNST.SC_DELIVERABLE_MANAGER_COUNTER Then
      h_is_del_manager_counter := 1;
   End If;
   
   --timeLog(file,"------------------>BEGIN FC_WBS_VIEW ");
   
   If (BITAND(Coalesce(h_active_record, 0), 4)) = 0 Then
      h_next_level_only := 1;
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) Then
      --trace_printf("\n insert 1 ");
      
      h_condition_flag := 1;
      
      myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
      myStmt := myStmt || 
                    'MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
      myStmt := myStmt || ' FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
      myStmt := myStmt || 
                    ' SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
      myStmt := myStmt || ' T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
      
      myStmt := myStmt || ' CASE WHEN ';
      myStmt := myStmt || TO_CHAR(h_next_level_only);
      myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
      myStmt := myStmt || ' when ';
      myStmt := myStmt || TO_CHAR(h_next_level_only);
      myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
      
      myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
      myStmt := myStmt || ' 0 AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR';
      myStmt := myStmt || ' FROM TMT_WBS T, PMO_TMP_PROJECTS_01 PL';
      myStmt := myStmt || 
                    ' WHERE SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ';
   Else
      If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
         --trace_printf("\n insert 2 ");
         h_condition_flag := 1;
         
         myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
         myStmt := myStmt || 
                    'MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
         myStmt := myStmt || 'FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
         myStmt := myStmt || 
                    'SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
         myStmt := myStmt || 'T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
         
         myStmt := myStmt || ' CASE WHEN ';
         myStmt := myStmt || TO_CHAR(h_next_level_only);
         myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
         myStmt := myStmt || ' when ';
         myStmt := myStmt || TO_CHAR(h_next_level_only);
         myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
         
         myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
         myStmt := myStmt || '1  AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR ';
         myStmt := myStmt || ' FROM TMT_WBS T, RESOURCE_ASSIGNMENTS RA, PMO_TMP_PROJECTS_01 PL ';
         myStmt := myStmt || ' WHERE RA.PROJECT_ID = T.PROJECT_ID ';
         myStmt := myStmt || ' AND RA.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || ' AND RA.RESOURCE_ID = ''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || '''';
         myStmt := myStmt || ' AND PL.ELEMENT_ID = T.PROJECT_ID ';
         myStmt := myStmt || 
                    ' AND SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ';
      Else
         --trace_printf("\n insert 3 ");
         
         h_condition_flag := 0;
         
         myStmt := 'INSERT INTO PMO_WBS_VIEW (PROJECT_ID, PROJECT_NAME, ELEMENT_NAME, START_FINISH_DATE,';
         myStmt := myStmt || 
                    ' MINUTE_PER_DAY, EFFORT_DRIVEN, RANK, ELEMENT_ID, PARENT_ID, LEVEL_ID, REF_CODE, TYPE_ID,';
         myStmt := myStmt || ' FILTER_FLAG, STAGE_ID, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)';
         myStmt := myStmt || 
                    ' SELECT PL.ELEMENT_ID AS PROJECT_ID, PL.ELEMENT_NAME AS PROJECT_NAME, T.ELEMENT_NAME AS ELEMENT_NAME, T.START_FINISH_DATE,';
         myStmt := myStmt || ' T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, T.RANK, T.ELEMENT_ID, T.PARENT_ID, ';
         
         If Coalesce(h_wbs_search_criteria, 0) <> PMO_CNST.SC_SEARCH_ALL_ITEMS Then
            myStmt := myStmt || ' CASE WHEN ';
            myStmt := myStmt || TO_CHAR(h_next_level_only);
            myStmt := myStmt || ' = 0 then T.LEVEL_ID ';
            myStmt := myStmt || ' when ';
            myStmt := myStmt || TO_CHAR(h_next_level_only);
            myStmt := myStmt || ' = 1 and T.ELEMENT_ID = PL.ELEMENT_ID then 0 else 1 end AS LEVEL_ID';
         Else
            myStmt := myStmt || ' 0 AS LEVEL_ID ';
         End If;
         
         myStmt := myStmt || ', T.REF_CODE, T.TYPE_ID,';
         myStmt := myStmt || ' 1 AS FILTER_FLAG, T.STAGE_ID, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR ';
         myStmt := myStmt || ' FROM TMT_WBS T JOIN PMO_TMP_PROJECTS_01 PL';
         myStmt := myStmt || 
                    ' ON SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) >= LENGTH(PL.RANK) THEN LENGTH(PL.RANK) ELSE 1 END) = PL.RANK ';
         myStmt := myStmt || ' AND T.PROJECT_ID = PL.ELEMENT_ID';
      End If;
   End If;
   
   --trace_printf("\n h_type:%d \n", h_type);
   
   h_critical_filter := 0;
   CASE h_type
      WHEN 1001 THEN
         -- All Milestone
         myStmt := myStmt || ' AND T.TYPE_ID = ';
         myStmt := myStmt || TO_CHAR(PMO_CNST.SC_MILESTONE);
         
      
      WHEN 1003 THEN
         -- All Late to start tasks
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || 
                    ') AND T.ACTUAL_START IS NuLL AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) < TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))';
         
      
      WHEN 1005 THEN
         -- All Late to finish tasks
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || 
                    ') AND T.ACTUAL_FINISH IS NuLL AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) < TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))';
         
      
      WHEN 1007 THEN
         -- All task due within 1 week
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_FINISH IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 6     )';
         
      
      WHEN 1009 THEN
         -- All task due within 4 weeks (28 days)
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_FINISH IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_FINISH, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 27     )';
         
      
      WHEN 1011 THEN
         -- All tasks starting within 1 week
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 6     )';
         
      
      WHEN 1013 THEN
         -- All tasks starting within 4 weeks (28 days)
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || 
                    ' AND TO_DATE(SubStr(T.UNRESOLVED_START, 1, 10)) BETWEEN TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) AND (TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10)) + 27     )';
         
      
      WHEN 1015 THEN
         -- All critical elements (filter by attributes)
         h_critical_element_filter := '';
         h_critical_filter := 1;
         
      
      WHEN 1023 THEN
         -- All critical projects (filter by attributes)
         h_critical_element_filter := SubStr(' AND T.TYPE_ID IN (', 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES
                                                                 , 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || ') ', 1, 255);
         h_critical_filter := 1;
         
      
      WHEN 1025 THEN
         -- All critical deliverables (filter by attributes)
         h_critical_element_filter := SubStr(' AND T.TYPE_ID IN (', 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || 
                                                                 PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES
                                                                 , 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || ') ', 1, 255);
         h_critical_filter := 1;
         
      
      WHEN 1027 THEN
         -- All critical tasks (filter by attributes)
         h_critical_element_filter := SubStr(' AND T.TYPE_ID IN (', 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || 
                                                                 PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES
                                                                 , 1, 255);
         h_critical_element_filter := SubStr(h_critical_element_filter || ') ', 1, 255);
         h_critical_filter := 1;
         
      
      WHEN 1017 THEN
         -- All Ready to start task
         myStmt := myStmt || ' AND T.DEPEND_FLAG IN (''0'', ''2'') AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') AND T.ACTUAL_START IS NuLL';
         myStmt := myStmt || ' AND (T.CONSTRAINT_TYPE NOT IN (5, 6)';
         myStmt := myStmt || 
                    ' OR (T.CONSTRAINT_TYPE IN (5, 6) AND TO_DATE(SubStr(T.CONSTRAINT_DT, 1, 10)) <= TO_DATE(SubStr(TO_DATE(SYSDATE), 1, 10))))';
         
      
      WHEN 1019 THEN
         -- All Projects
         --//trace_printf("\n case 1019 \n");
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID IN (';
            dynamicSql := dynamicSql || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            dynamicSql := dynamicSql || ')';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            If Coalesce(h_wbs_search_criteria, 0) <> PMO_CNST.SC_SEARCH_ALL_ITEMS Then
               myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ' AND T.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || ')';
               
               If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
                  If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                        h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                     typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                     myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                     myStmt := myStmt || typeList;
                  Else
                     typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                     myStmt := myStmt || ' AND T.STAGE_ID = ';
                     myStmt := myStmt || typeList;
                  End If;
               End If;
            End If;
         End If;
         
      
      WHEN 1021 THEN
         -- Critical Task
         myStmt := myStmt || ' AND T.LEVEL_INDEX <> -1 AND T.TOTAL_SLACK = 0 AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')';
         
      
      WHEN 1029 THEN
         -- Projects
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 1';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 1';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 1031 THEN
         -- Proposals
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 154';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 154';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 1033 THEN
         -- Organization
         If Coalesce(h_sp_id, 0) = 302 Then
            dynamicSql := 'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.TYPE_ID = 288';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.BUBBLE_COLOR = ';
                  dynamicSql := dynamicSql || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  dynamicSql := dynamicSql || ' AND T.STAGE_ID = ';
                  dynamicSql := dynamicSql || typeList;
               End If;
            End If;
         Else
            myStmt := myStmt || ' AND T.ELEMENT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID = 288';
            
            If (h_stage_id IS NULL) OR Coalesce(h_stage_id, 0) <> 0 Then
               If (Coalesce(h_stage_id, 0) >= PMO_CNST.SC_PROJECT_HEALTH_01) AND (Coalesce(
                     h_stage_id, 0) <= PMO_CNST.SC_PROJECT_HEALTH_06) Then
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.BUBBLE_COLOR = ';
                  myStmt := myStmt || typeList;
               Else
                  typeList := TO_CHAR(Coalesce(h_stage_id, 0));
                  myStmt := myStmt || ' AND T.STAGE_ID = ';
                  myStmt := myStmt || typeList;
               End If;
            End If;
         End If;
         
      
      WHEN 3001 THEN
         -- Critical Task after levelling
         myStmt := myStmt || ' AND (T.ELEMENT_ID = PL.ELEMENT_ID ';
         myStmt := myStmt || ' OR (T.LEVEL_INDEX <> -1 AND T.TOTAL_SLACK = 0 AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')))';
         
      
      WHEN 1035 THEN
         -- Tasks with ETC Varinace
         myStmt := myStmt || 
                    ' AND (T.SCHED_EFFORT + T.PLANNED_EFFORT) - T.WORK_TODATE <> T.ETC AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ')';
         
         --//trace_printf("\n Tasks with ETC Varinace \n");
         
      
      WHEN PMO_CNST.SC_WBS_IN_PROGRESS_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID NOT IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.ACTUAL_FINISH IS NULL ';
         myStmt := myStmt || ' AND T.ACTUAL_START IS NOT NULL ';
         
         
      
      WHEN PMO_CNST.SC_WBS_CLOSED_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID  IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         
         
      
      WHEN PMO_CNST.SC_WBS_COMPLETED_TASKS THEN
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.STAGE_ID NOT IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_CLOSED_STATE_STR;
         myStmt := myStmt || ') ';
         myStmt := myStmt || ' AND T.ACTUAL_FINISH IS NOT NULL AND T.MILESTONE_PERCENT >= 10000 ';
         myStmt := myStmt || ' AND T.STAGE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_ALL_WBS_COMPLETED_STATE_STR;
         myStmt := myStmt || ')';
         
         
      
      ELSE
         NULL;
   END CASE;
   
   If ((h_name IS NOT NULL)) Then
      myStmt := myStmt || ' AND ((';
      myStmt := myStmt || h_name;
      
      myStmt := myStmt || ')';
      
      If Coalesce(h_is_single_project, 0) = 1 Then
         myStmt := myStmt || ' OR T.ELEMENT_ID = PL.ELEMENT_ID';
      End If;
      myStmt := myStmt || ')';
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) OR ((Coalesce(h_type, 0) > 5000) 
             AND (Coalesce(h_type, 0) < 6000)) Then
      If (Coalesce(h_type, 0) > 5000) AND (Coalesce(h_type, 0) < 6000) Then
         localType := Coalesce(h_type, 0) - 5000;
      Else
         localType := Coalesce(h_type, 0);
      End If;
      
      --trace_printf("\n localType:%d", localType);
      
      CASE localType
         
         WHEN 223 THEN
            -- deliverable and transferred cri
            typeList := '(' || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES || ') ';
         
         WHEN 224 THEN
            -- tasks and milestone
            typeList := '(' || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ') ';
         
         WHEN 4 THEN
            -- transfer cris
            typeList := '(' || PMO_CNST.SC_STR_TRANS_WBSDELV_TYPES || ') ';
         
         WHEN 5 THEN
            -- transfer tasks
            typeList := '(' || PMO_CNST.SC_STR_TRANS_WBSTASK_TYPES || ') ';
         
         WHEN 0 THEN
            -- project and proposal
            typeList := '(' || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ') ';
         
         WHEN 3 THEN
            -- task (include duration > 0 milestones)
            typeList := '(' || TO_CHAR(PMO_CNST.SC_TASK) || ',' || TO_CHAR(PMO_CNST.SC_MILESTONE) 
                  || ') AND T.DURATION > 0 ';
         
         ELSE
            typeList := '(' || TO_CHAR(localType) || ') ';
      END CASE;
   End If;
   
   --trace_printf("\n typeList:%s", typeList);
   
   If (Coalesce(h_type, 0) > 5000) AND (Coalesce(h_type, 0) < 6000) Then
      If Coalesce(h_sp_id, 0) = 302 Then
         dynamicSql := 
               'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01) ';
         dynamicSql := dynamicSql || ' AND T.TYPE_ID IN ';
         dynamicSql := dynamicSql || typeList;
      Else
         myStmt := myStmt || ' AND T.TYPE_ID IN ';
         myStmt := myStmt || typeList;
         
         If Coalesce(h_is_del_manager_counter, 0) = 1 Then
            myStmt := myStmt || ' JOIN RESOURCE_ASSIGNMENTS RA ON RA.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' ';
            myStmt := myStmt || ' AND RA.PROJECT_ID = T.PROJECT_ID ';
            myStmt := myStmt || ' AND RA.ELEMENT_ID = T.ELEMENT_ID ';
            myStmt := myStmt || ' AND BITAND(RA.PARTICIPANT, ';
            myStmt := myStmt || TO_CHAR(PMO_CNST.SC_DLV_MANAGER);
            myStmt := myStmt || ') = ';
            myStmt := myStmt || TO_CHAR(PMO_CNST.SC_DLV_MANAGER);
         End If;
      End If;
   ElsIf (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
      If Coalesce(h_sp_id, 0) = 300 Then
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
         myStmt := myStmt || ')';
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 300 Then
      If (h_filter_string IS NOT NULL) Then
         h_critical_filter := 1;
      End If;
   End If;
   
   If Coalesce(h_critical_filter, 0) = 1 Then
      If Coalesce(h_sp_id, 0) = 302 Then
         If Length(dynamicSql) = 0 Then
            dynamicSql := 
                  'UPDATE PMO_WBS_VIEW T SET T.FILTER_FLAG = 1 WHERE T.PROJECT_ID IN (SELECT ELEMENT_ID FROM PMO_TMP_PROJECTS_01) ';
         End If;
         
         dynamicSql := dynamicSql || h_critical_element_filter;
         If (h_filter_string IS NOT NULL) Then
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            dynamicSql := dynamicSql || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
            dynamicSql := dynamicSql || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            dynamicSql := dynamicSql || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            dynamicSql := dynamicSql || h_filter_string;
            dynamicSql := dynamicSql || ') ';
            dynamicSql := dynamicSql || 
                            ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            dynamicSql := dynamicSql || h_str_filter_count;
         End If;
      Else
         --myStmt.strcat(h_critical_element_filter);
         If (h_filter_string IS NOT NULL) Then
            If Coalesce(h_condition_flag, 0) = 0 Then
               myStmt := myStmt || ' WHERE ';
            Else
               myStmt := myStmt || ' AND ';
            End If;
            
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            myStmt := myStmt || ' (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
            myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || ') ';
            myStmt := myStmt || 
                    ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
         End If;
      End If;
   End If;
   
   If (Coalesce(h_sp_id, 0) = 2) OR (Coalesce(h_sp_id, 0) = 302) Then
      If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 1) OR (Coalesce(h_type, 0) = 145) Then
         -- when type in (1, 145) and filter_string is not null, return next level only
         If ((BITAND(Coalesce(h_active_record, 0), 4)) = 0) AND ((Coalesce(h_type, 0) <> 0) OR ((Coalesce(
                     h_type, 0) = 0) AND ((h_filter_string IS NULL)) AND ((h_name IS NULL)))) Then
            myStmt := myStmt || ' AND T.LEVEL_ID < (PL.LEVEL_ID + 2) ';
         End If;
         
         If ((h_filter_string IS NOT NULL)) AND (Coalesce(h_type, 0) = 0) Then
            -- Filter by attribute list
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            
            myStmt := myStmt || ' AND (((SELECT COUNT(DISTINCT(PA.ELEMENT_ID))';
            myStmt := myStmt || ' FROM TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || ') ';
            myStmt := myStmt || ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1';
            myStmt := myStmt || ' ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
            myStmt := myStmt || ' AND T.TYPE_ID NOT IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ')) OR T.ELEMENT_ID = PL.ELEMENT_ID)';
         End If;
         myStmt := myStmt || ' AND T.REC_STATUS <> ''D''';
      ElsIf Coalesce(h_type, 0) <> 3001 Then
         If Coalesce(h_sp_id, 0) = 302 Then
            If ((h_name IS NULL)) Then
               myStmt := myStmt || ' AND (T.PARENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ' OR T.ELEMENT_ID = PL.ELEMENT_ID';
               myStmt := myStmt || ')';
            End If;
         Else
            myStmt := myStmt || ' AND ((T.PROJECT_ID = PL.ELEMENT_ID';
            myStmt := myStmt || ' AND T.TYPE_ID IN ';
            myStmt := myStmt || typeList;
            
            If Coalesce(h_type, 0) = 213 Then
               myStmt := myStmt || ' AND T.TRANSFER_FLAG = ''R''';
            ElsIf Coalesce(h_type, 0) = 2 AND ((h_filter_string IS NULL)) Then
               myStmt := myStmt || ' AND T.TRANSFER_FLAG IS NuLL';
            End If;
            
            myStmt := myStmt || ') OR T.ELEMENT_ID = PL.ELEMENT_ID ';
            myStmt := myStmt || ' OR (T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ') AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
            myStmt := myStmt || typeList;
            myStmt := myStmt || 
                    ' AND (RANK >= SUBSTR(T.RANK,1,8) AND SUBSTR(RANK, 1, CASE WHEN LENGTH(RANK) < LENGTH(T.RANK) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK))))';
            
            --trace_printf("\n APPLY FILTER STRING 1003\n");
            If (h_filter_string IS NOT NULL) Then
               -- Filter by attribute list
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               
               myStmt := myStmt || 
                     ' EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE (W.ELEMENT_ID = T.ELEMENT_ID OR W.PROJECT_ID = T.ELEMENT_ID)';
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               myStmt := myStmt || ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || 
                     ') AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
            End If;
         End If;
      End If;
   End If;
   
   If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) AND (Coalesce(
             h_sp_id, 0) = 300) Then
      myStmt := myStmt || ' AND ';
      myStmt := myStmt || h_prj_var_condition;
   End If;
   --trace_printf("\n myStmt: %s",myStmt);
   
   BEGIN
      EXECUTE IMMEDIATE myStmt;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 718+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(719) : views.sql(2668) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
         END IF; 
      END;
   END;
   
   If (Coalesce(h_sp_id, 0) = 302) AND (Length(dynamicSql) > 0) Then
      --trace_printf("\n dynamicSql: %s",dynamicSql);
      
      If Coalesce(h_wbs_search_criteria, 0) = PMO_CNST.SC_SEARCH_ALL_ITEMS Then
         BEGIN
            EXECUTE IMMEDIATE dynamicSql;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 740+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(741) : views.sql(2690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  LFILE.FPUT('AllTrace',                         'SQL : ' || dynamicSql);
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED)) = PMO_CNST.SC_INCLUDE_CLOSED Then
      h_include_closed_items := 1;
   End If;
   If (BITAND(Coalesce(
         h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED)) = PMO_CNST.SC_INCLUDE_COMPLETED Then
      h_include_completed_items := 1;
   End If;
   
   --*****************************************************************************
   -- FOR ALL LEVEL WE APPLY CLOSED AND COMPLETED FILTER, BY REMOVEING THESE ITESM
   -- IT CAN CAUSE SOME DEAD PLUS SIGNS
   --*****************************************************************************
   --if (h_sp_id == 300)
   --{
   -- remove items that don't have Closed and Completed contition (first level)
   --if ((h_include_closed_items == 1)  (h_include_completed_items == 1))
   
   --trace_printf("\n before deleting closed h_sp_id:%d", h_sp_id);
   
   If (Coalesce(h_include_closed_items, 0) = 1) AND (Coalesce(h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE (V.MILESTONE_PERCENT >= 10000 AND V.STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929))
         AND V.STAGE_ID NOT IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117)
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 784+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(785) : views.sql(2734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 1) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE V.STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117)
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 804+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(805) : views.sql(2754) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf (Coalesce(h_include_closed_items, 0) = 0) AND (Coalesce(
                h_include_completed_items, 0) = 0) Then
      BEGIN
         DELETE FROM PMO_WBS_VIEW V
         WHERE ((V.MILESTONE_PERCENT >= 10000 AND V.STAGE_ID IN (5490, 4948, 5520, 4909, 5416, 5385, 6116, 4938, 4919, 4929)) OR V.STAGE_ID IN (4949, 5491, 5521, 4910, 4920, 4930, 4939, 5386, 5417, 6117))
         AND (h_sp_id!=2 OR NOT EXISTS (SELECT P.element_id from PMO_TMP_PROJECTS_01 P where P.ELEMENT_ID = V.ELEMENT_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 824+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(825) : views.sql(2774) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   --}
   
   If (BITAND(Coalesce(h_active_record, 0), 8)) = 8 Then
      If Coalesce(h_sp_id, 0) = 300 Then
         BEGIN
            DELETE FROM PMO_WBS_VIEW C
            WHERE EXISTS (SELECT P.ELEMENT_ID FROM PMO_WBS_VIEW P
            WHERE P.RANK < C.RANK AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
            AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK));
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 848+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(849) : views.sql(2798) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            DELETE FROM PMO_WBS_VIEW C
            WHERE EXISTS (SELECT P.ELEMENT_ID FROM PMO_WBS_VIEW P, PMO_TMP_PROJECTS_01 PL
            WHERE P.RANK < C.RANK AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
            AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK)
            AND P.ELEMENT_ID <> PL.ELEMENT_ID);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 869+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(870) : views.sql(2819) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 302 Then
      If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
         BEGIN
            UPDATE PMO_WBS_VIEW
            SET FILTER_FLAG = 0
            WHERE ASSIGNED = 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 892+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(893) : views.sql(2842) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --*******************************************************************************
      -- FOR SECOND LEVEL IN VIEWS BEFORE WE HIGHLIGHT CLOSED AND COMPLETED ITEMS BUT
      -- IN CURRENT CODE WE JUST REMOVE THEM FROM RESULTSET, THE FOLLOWING CODE WAS FOR
      -- HIGHLIGHTING CLOSED AND COMPLETED ITEAMS
      --*******************************************************************************
      
      --if ((h_include_closed_items == 1)  (h_include_completed_items == 0))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE MILESTONE_PERCENT = 10000
      --          AND STAGE_ID NOT IN (SC_ALL_WBS_CLOSED_STATE);
      --  END_SQL;
      --}
      --else if ((h_include_closed_items == 0)  (h_include_completed_items == 1))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE STAGE_ID IN (SC_ALL_WBS_CLOSED_STATE);
      --  END_SQL;
      --}
      --else if ((h_include_closed_items == 0)  (h_include_completed_items == 0))
      --{
      --  BEGIN_SQL
      --      UPDATE session.PMO_WBS_VIEW
      --          SET FILTER_FLAG = 0
      --      WHERE (MILESTONE_PERCENT = 10000 OR STAGE_ID IN (SC_ALL_WBS_CLOSED_STATE));
      --  END_SQL;
      --}
   ElsIf (Coalesce(h_sp_id, 0) = 300) AND (Coalesce(h_type, 0) = 1015) AND (Coalesce(
                h_critical_filter, 0) = 1) AND ((h_filter_string IS NOT NULL)) Then
      BEGIN
         UPDATE PMO_WBS_VIEW
         SET LEVEL_ID = 0
         WHERE LEVEL_ID > 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_WBS_VIEW', 945+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_WBS_VIEW(946) : views.sql(2895) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   --timeLog(file,"------------------>END   FC_WBS_VIEW ");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_WBS_VIEW', 954+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_WBS_VIEW(955) : views.sql(2904) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_WBS_VIEW;
ALTER FUNCTION FC_WBS_VIEW COMPILE DEBUG;

/* ************************************************************************************************************************************************** */

/*
   sp_id identifies the type of the stored procedure and the result set depends on the type
 1 = Projects/Deliverables/Tasks  Work Packages - FIRST Level - Returns all projects where the resource_id is envolded
                if type = 0 and ID is NULL returns all projects in the root, where PARENT_ID iS NULL
                if type = ( SC_RESOURCE(13) or SC_RM_VIEW(118)) and ID is NULL returns Work Package, flat list of all projects for RESOURCE_ID
                else
                 IF filter_string (no attributes filter) is null then returns all roots projects for resource_id, and depends on type, ex:
                        CHILD_COUNT = 1 if there is at least one child of the TYPE_ID = type, else = 0
                 ELSE (attributes filter)
                        IF type == SC_PROJECT returns a flat list of all projects witch have attributes IN filter_string
                        ELSE returns a two levels resultset: in the root projects and in the next level a flat list of all children of TYPE_ID == type

 2 = Projects/Deliverables/Tasks  Work Packages - NEXT  Level
                if type = SC_PROJECT (1) - returns all WBS elements on the next level (PARENT_ID = id)
                          = SC_DELIVERABLE (2) - returns all deliverables under the project (identified by id),
                          = SC_CHANGE_REQUEST (4) - returns all changes under the project (identified by id),
                          = SC_RISK (5) - returns all deliverables under the project (identified by id),
                          = SC_ISSUE (6) - returns all deliverables under the project (identified by id),
                          = SC_SUMMARY_TASK (60) - returns all deliverables under the project (identified by id),
                          = SC_WORK_PRODUCT (58) - returns all deliverables under the project (identified by id)
                          = SC_ALL_DELIVERABLE (223) - returns all deliverables, summary task, work products,
                            and transferred changes, issues, risks, and requirements under the project (identified by id),
                          = SC_TASK (3) - returns all tasks under the project (identified by id)
                          = SC_MILESTONE (59) - returns all milestone under the project (identified by id)
                          = SC_TASK_MILESTONE (224) - returns all task and milestone under the project (identified by id)
 3 = Domain  Work Packages - NEXT Level

 4 = Project Manager/Team Member Work Packages
        h_id        = Project ID
        h_resource_id   = Project Manager or Team Resource ID

        when (h_id is null) and (h_resource_id is not null) then return project list
        when (h_id is not null) and (h_resource_id is not null)
            then return work packages if h_resource_id is participant
            else return resources  profiles if h_resource_id is project manager

 5 = returns only one row for this particuler ID
 10 = returns a list of project for responsible or (published + participants)
 */

/* **************************************************************************************************************************************************** */

/* Templets

 31 = Projects/Deliverables/Tasks View all levels   ( type = any value)*/

/* ***************************************************************************************************************************************************** */

/*
 CRI-s  *type = 4 - Chang Request, 5 - Risk, 6 - Issue
 33 = Projects witch have CRI of TYPE_ID = *type    ____________________ LEVEL 0
            {call SP_WBS_VIEW(33,null,'PMO_SUPERVISOR',null,null,0,4,0)}
 34 = CRI-s of TYPE_ID = *type, attached to PROJECT_ID ______________ LEVEL 1
            {call SP_WBS_VIEW(34,'PROJECT_ID',null,null,null,0,4,0)}
 89 (SC_WORKFLOW_ATTACHMENT) = select parent list of workflow's attachment
            {CALL SP_WBS_VIEW(89,'element_id','user_id',null,null,0,0,0)} for project view
            {CALL SP_WBS_VIEW(89,'element_id','user_id',null,null,0,2,0)} for deliverable view
 64 (SC_PORTFOLIO) = return only one row for the passed id (tmt_portfolios)
            {CALL SP_WBS_VIEW(64,'element_id','user_id',null,null,0,0,0)} select root portfolios only
 SC_WBS_PORTFOLIO_VIEW (SC_PORTFOLIO) = select portfolios for the login user
            {CALL SP_WBS_VIEW(SC_WBS_PORTFOLIO_VIEW,null,'user_id',null,null,0,0,0)} select root portfolios only
            {CALL SP_WBS_VIEW(SC_WBS_PORTFOLIO_VIEW,'element_id','user_id',null,null,0,0,0)} select portfolios under the element

 active_record parameter used to control the following filters options
            1st bit used to indicate the active record:
                0:  excludes deleted and locked rows
                1:  includes deleted and locked rows

            2nd bit used to indicate my rows or all rows:
                0:  all rows
                1:  my rows only

            3th bit used to indicate expand all:
                0:  return next level only
                1:  return all level under the current level

            4th bit used to control resultset for filters:
                0:  Flat list (include children)
                1:  Flat list (exclude children)
                ex. d1
                        d11
                            d111
                    d2
                        d21
                if the logon user has the right to see d1, d111 and d21, the resultset will be
                0:  d1
                    d111
                    d21

                1:  d1
                    d21

            5th bit NOT USED

            6th bit used to indicate include/exclude Closed items                               (32)
                0:  exclude Closed items
                1:  include Closed items

            7th bit used to indicate include/exclude Completed items                            (64)
                0:  exclude Completed items
                1:  include Completed items

            8th bit NOT USED

            9th bit used to indicate include/exclude Templated items                            (256)
                0:  exclude Templated items
                1:  include Templated items

            12th bit: Deliverable Manager Counter                                               (2048)
*/

/*******************************************************************************************************************************************************/

Prompt == FUNCTION SP_WBS_VIEW ==
CREATE OR REPLACE FUNCTION SP_WBS_VIEW
(
     p_sp_id PLS_INTEGER
   , p_ids VARCHAR2
   , p_resource_id VARCHAR2
   , p_name VARCHAR2
   , p_filter_string VARCHAR2
   , p_filter_count PLS_INTEGER
   , p_type PLS_INTEGER
   , p_active_record PLS_INTEGER
   , p_stage_id PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   WBS_RETURN_COLUMNS VARCHAR2(9999);
   h_search_option PLS_INTEGER := 0;
   h_filter_string VARCHAR2(8000);
   h_full_name VARCHAR2(255);
   h_id VARCHAR2(32);
   h_ids VARCHAR2(4000);
   h_level VARCHAR2(4);
   h_name VARCHAR2(8000);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_resource_id VARCHAR2(32);
   h_str_filter_count VARCHAR2(4);
   h_str_level_id VARCHAR2(1);
   participant001 VARCHAR2(19);
   h_active_record PLS_INTEGER := 0;
   h_filter_count PLS_INTEGER := 0;
   h_sp_id PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_xcount PLS_INTEGER := 0;
   h_prj_var_condition VARCHAR2(1024);
   h_stage_id PLS_INTEGER := 0;
   h_executive_security VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_type_id PLS_INTEGER := 0;
   h_security_id VARCHAR2(32);
   h_security_ids VARCHAR2(4000);
   h_res_row_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_tpf_current_rank VARCHAR2(200);
   h_project_name VARCHAR2(255);
   h_start_finish_date DATE;
   h_effort_driven PLS_INTEGER := 0;
   h_pure_rank VARCHAR2(200);
   h_security_num PLS_INTEGER := 0;
   h_int_x PLS_INTEGER := 0;
   h_minute_per_day PLS_INTEGER := 0;
   h_security_bitflag PLS_INTEGER := 0;
   h_filedName VARCHAR2(254);
   h_keyword_search1 VARCHAR2(8000);
   h_current_id VARCHAR2(32);
   myStmt VARCHAR2(29999);
   dynamicSql VARCHAR2(1999);
   tempStr VARCHAR2(99);
   typeList VARCHAR2(99);
   idList VARCHAR2(3199);
   h_include_closed_items PLS_INTEGER := 0;
   h_include_completed_items PLS_INTEGER := 0;
   h_tree_type PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_is_single_project PLS_INTEGER := 0;
   h_can_view_pivot PLS_INTEGER := 0;
   h_project_flag PLS_INTEGER := 0;
   h_portfolio_owner_resource_id VARCHAR2(32);
   h_element_id VARCHAR2(32);
   h_tmp_security_ids VARCHAR2(4000);
   h_type_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_wbs_search_criteria PLS_INTEGER := 0;
   h_include_prj_detail_columns PLS_INTEGER := 0;
   h_total PLS_INTEGER;
   searchUnderFolder PLS_INTEGER NOT NULL := 0;
   searchInPortConfig PLS_INTEGER NOT NULL := 0;
   null_return LONG;
-- Declare Cursors
      CURSOR CRITICAL_CHILD IS
         SELECT RANK, PARENT_ID
         FROM PMO_WBS_VIEW
         ORDER BY RANK;

      CURSOR REFRESH_SECURITY IS
         SELECT RPAD(R.SECURITY_ID, 32)
         FROM TMT_WBS W, RESOURCE_ASSIGNMENTS R
         WHERE (SUBSTR(h_rank, 1, CASE WHEN LENGTH(W.RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(W.RANK) END) = W.RANK 
         AND (W.RANK LIKE SUBSTR(h_rank, 1, 8) || '%'))
         AND W.PROJECT_ID = R.PROJECT_ID
         AND W.ELEMENT_ID = R.ELEMENT_ID
         AND R.RESOURCE_ID = h_resource_id
         ORDER BY RANK;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_WBS_VIEW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_WBS_VIEW'), '-- ' || SYSTIMESTAMP, ':r := SP_WBS_VIEW(' || 
       COALESCE(TO_CHAR(p_sp_id), 'NULL')
          || ', ' || '''' || p_ids || ''''
          || ', ' || '''' || p_resource_id || ''''
          || ', ' || '''' || p_name || ''''
          || ', ' || '''' || p_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(p_filter_count), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_active_record), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_stage_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 110+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_WBS_VIEW(111) : views.sql(3138) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --timeLog(file,"------------------>BEGIN SP_WBS_VIEW ");
   
   h_sp_id := p_sp_id;
   h_ids := p_ids;
   h_resource_id := p_resource_id;
   h_name := p_name;
   h_filter_string := p_filter_string;
   h_filter_count := p_filter_count;
   h_type := p_type;
   h_active_record := p_active_record;
   h_stage_id := p_stage_id;
   
   tempStr := TO_CHAR(Coalesce(h_type, 0));
   
   BEGIN
      SELECT REPLACE( RTRIM(NICK_NAME), '''', ''''''), EXECUTIVE_SECURITY
      INTO h_full_name, h_executive_security
      FROM RESOURCES
      WHERE RESOURCE_ID = h_resource_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 140+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(141) : views.sql(3168) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   WBS_RETURN_COLUMNS := SubStr(
                             'T.REC_USER, (SELECT R.FULL_NAME FROM RESOURCES R WHERE R.RESOURCE_ID = T.REC_USER) AS REC_USER_NAME, '
          || 'T.REC_DATETIME, T.LAST_CHECKIN, T.CONSTRAINT_TYPE, T.CONSTRAINT_DT, '
          || 'T.SD, T.START_DT, T.FD, T.FINISH_DT, T.DURATION, T.VARIANCE, T.BASELINE1_VARIANCE, '
          || 'T.FORECAST_VARIANCE, T.ACTUAL_VARIANCE, T.EXTERNAL_ID, '
          || 'T.BASELINE1_START, T.BASELINE1_FINISH, T.BASELINE1_DURATION, (T.BASELINE1_EFFORT) AS BASELINE1_EFFORT, '
          || 'T.BASELINE2_START, T.BASELINE2_FINISH, T.BASELINE2_DURATION, (T.BASELINE2_EFFORT) AS BASELINE2_EFFORT, '
          || 'T.BASELINE3_START, T.BASELINE3_FINISH, T.BASELINE3_DURATION, (T.BASELINE3_EFFORT) AS BASELINE3_EFFORT, '
          || 'T.BASELINE3_START_VARIANCE, T.BASELINE3_FINISH_VARIANCE, '
          || '(CASE WHEN LEVEL_INDEX = -1 THEN 0 ELSE T.TOTAL_SLACK-100 END) as LEVEL_TOTAL_SLACK,T.LEVEL_START,T.LEVEL_FINISH,T.LEVEL_DURATION, '
          || 'CASE WHEN LEVEL_STATE = 2 THEN 1 WHEN LEVEL_STATE = 1 THEN 2 WHEN LEVEL_START IS NOT NuLL THEN 3 ELSE -1 END AS LEVEL_FLAG, '
          || 'CASE WHEN T.LEVEL_CONFLICT IS Null THEN 0 ELSE 1 END as LEVEL_HAS_CONFLICT, T.LEVEL_CONFLICT, '
          || 'T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_DURATION, (T.PLANNED_EFFORT) AS PLANNED_EFFORT, 0 AS PLANNED_ROLLUP_BIT, '
          || 'T.PROPOSED_START, T.PROPOSED_FINISH, T.PROPOSED_DURATION, (T.PROPOSED_EFFORT) AS PROPOSED_EFFORT, '
          || 'T.SCHED_START, T.SCHED_FINISH, T.SCHED_DURATION, (T.SCHED_EFFORT) AS SCHED_EFFORT, '
          || 'T.FORECAST_START, T.FORECAST_FINISH, T.FORECAST_DURATION, T.FORECAST_EFFORT, T.ETC, '
          || 'T.ACTUAL_START, T.ACTUAL_FINISH, T.ACTUAL_DURATION, (T.WORK_TODATE) AS ACTUAL_EFFORT, '
          || '((T.SCHED_EFFORT+T.PLANNED_EFFORT)-T.WORK_TODATE) AS REMAINING_EFFORT, '
          || 'CASE WHEN ((T.SCHED_EFFORT+T.PLANNED_EFFORT)-T.WORK_TODATE) = T.ETC THEN 0 ELSE 1 END AS ETC_RED_FLAG, '
          || 'T.MILESTONE_PERCENT, T.WORK_PERCENT, CASE WHEN T.TASK_TYPE = 10 THEN 0 WHEN T.TASK_TYPE = 11 THEN 2 '
          || 'WHEN T.TASK_TYPE = 1 THEN 3 WHEN T.TASK_TYPE = 12 THEN 4 ELSE 1 END AS TASK_TYPE, T.REC_FLAGS, '
          || 'CASE WHEN T.SCHED_EFFORT > 0 THEN Trunc(ROUND(((PMO_ORASTD.TODECIMAL(T.WORK_TODATE, 18, 2)/(T.SCHED_EFFORT + T.PLANNED_EFFORT)) * 10000), 0)) ELSE 0 END AS PERCENT_COMPLETE, '
          || 'T.BASELINE1_BUDGET, T.BASELINE2_BUDGET, T.BASELINE3_BUDGET, T.ESTIMATION_BUDGET, T.ESTIMATION_TOTAL, '
          || 'T.CURRENT_BUDGET, T.FORECAST_BUDGET, (T.FORECAST_BUDGET - T.CURRENT_BUDGET) AS BUDGET_FORECAST_VARIANCE, '
          || 'CASE WHEN T.BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.CURRENT_BUDGET - T.BASELINE1_BUDGET) END AS BASELINE_VARIANCE, '
          || 'T.ACTUAL_BUDGET, (T.CURRENT_BUDGET - T.ACTUAL_BUDGET) AS REMAINING, '
          || 'T.CHARGED_BUDGET, T.R_CHARGED_BUDGET, T.S_CHARGED_BUDGET, '
          || 'T.LEDGER_BUDGET, T.R_LEDGER_BUDGET, T.S_LEDGER_BUDGET, '
          || 'T.R_BASELINE1_BUDGET, T.R_BASELINE2_BUDGET, T.R_BASELINE3_BUDGET, T.R_ESTIMATION_BUDGET, T.R_ESTIMATION_TOTAL, '
          || 'ESTIMATED_ACTUAL, ESTIMATED_ETC, R_ESTIMATED_ACTUAL, R_ESTIMATED_ETC, S_ESTIMATED_ACTUAL, S_ESTIMATED_ETC, '
          || 'ESTIMATED_CHARGED, R_ESTIMATED_CHARGED, S_ESTIMATED_CHARGED, '
          || 'ESTIMATED_LEDGER, R_ESTIMATED_LEDGER, S_ESTIMATED_LEDGER, '
          || 'ESTIMATED_HOURS, ESTIMATED_ACTUAL_HOURS, ESTIMATED_HOURS_ETC, R_ESTIMATED_HOURS, R_ESTIMATED_ACTUAL_HOURS, R_ESTIMATED_HOURS_ETC, '
          || 'ESTIMATED_HOURS_BASELINE1, ESTIMATED_HOURS_BASELINE2, ESTIMATED_HOURS_BASELINE3, '
          || 'R_ESTIMATED_HOURS_BASELINE1, R_ESTIMATED_HOURS_BASELINE2, R_ESTIMATED_HOURS_BASELINE3, '
          || 'T.R_CURRENT_BUDGET, T.R_FORECAST_BUDGET, (T.R_FORECAST_BUDGET - T.R_CURRENT_BUDGET) AS R_FORECAST_VARIANCE, '
          || 'CASE WHEN T.R_BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.R_CURRENT_BUDGET - T.R_BASELINE1_BUDGET) END AS R_BASELINE_VARIANCE, '
          || 'T.R_ACTUAL_BUDGET, (T.R_CURRENT_BUDGET - T.R_ACTUAL_BUDGET) AS R_REMAINING, '
          || 'T.S_BASELINE1_BUDGET, T.S_BASELINE2_BUDGET, '
          || 'T.S_BASELINE3_BUDGET, T.S_ESTIMATION_BUDGET, T.S_ESTIMATION_TOTAL, '
          || 'T.S_CURRENT_BUDGET, T.S_FORECAST_BUDGET, (T.S_FORECAST_BUDGET - T.S_CURRENT_BUDGET) AS S_FORECAST_VARIANCE, '
          || 'CASE WHEN T.S_BASELINE1_BUDGET = 0.00 THEN 0.00 ELSE (T.S_CURRENT_BUDGET - T.S_BASELINE1_BUDGET) END AS S_BASELINE_VARIANCE, '
          || 'T.S_ACTUAL_BUDGET, (T.S_CURRENT_BUDGET - T.S_ACTUAL_BUDGET) AS S_REMAINING, '
          || 'T.BASELINE1_DURATION AS SAC, '
          || 'T.BASELINE1_FINISH AS ECD, '
          || 'T.BAC, '
          || 'T.BCWS, '
          || 'T.BCWP, '
          || 'T.ACWP, '
          || 'T.EV_ETC, '
          || 'T.EAC, '
          || 'T.BCWS_FORECAST, '
          || 'T.BCWP_FORECAST, '
          || 'T.ACWP_FORECAST, '
          || 'T.EV_ETC_FORECAST, '
          || 'T.EAC_FORECAST, '
          || 'T.TIME_VARIANCE AS TV, '
          || '(T.BAC - T.EAC) AS VAC, '
          || '(T.BCWP - T.ACWP) AS CV, '
          || '(T.BCWP - T.BCWS) AS SV, '
          || 'CASE WHEN T.BAC = 0.00 THEN 0 ELSE (T.BCWP/T.BAC)*100 END AS PC, '
          || 'CASE WHEN T.ACWP = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.ACWP), 18, 2) END AS CPI, '
          || 'CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END AS SPI, '
          || 'PMO_ORASTD.TODECIMAL((CASE WHEN T.ACWP = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.ACWP), 18, 2) END)*(CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END), 18, 2) AS CRITICAL_RATIO, '
          || 'CASE WHEN (T.BAC-T.ACWP) = 0 THEN 0 ELSE (T.BAC-T.BCWP)/(T.BAC-T.ACWP) END AS TCPI, '
          || 'T.BAC*((CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END) - 1) AS SVAC_SPI, '
          || 'T.BAC*((PMO_ORASTD.TODECIMAL(((CASE WHEN T.ACWP = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.ACWP), 18, 2) END)*(CASE WHEN T.BCWS = 0.00 THEN 0.00 ELSE PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2) END)), 18, 2)) - 1) AS SVAC_CR, '
          || 'CASE WHEN T.BCWS = 0.00 OR T.BCWP = 0.00 THEN 0 ELSE Trunc((T.BASELINE1_DURATION / PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2))) END AS TEAC, '
          || 'T.BASELINE1_DURATION - CASE WHEN T.BCWS = 0.00 OR T.BCWP = 0.00 THEN 0 ELSE Trunc((T.BASELINE1_DURATION / PMO_ORASTD.TODECIMAL((T.BCWP / T.BCWS), 18, 2))) END AS TVAC, '
          || 'T.PUBLISHED, T.PROJECT_ID, T.REF_CODE, '
          || '(CASE WHEN PMO_ORASTD.LOCATE(''.'', T.WBS_CODE, 3) = 0 THEN ''1.0'' ELSE ''1'' || SUBSTR(T.WBS_CODE, PMO_ORASTD.LOCATE(''.'', T.WBS_CODE, 3)) END) AS WBS_CODE, '
          || 'T.REFERENCE_NUMBER, '
          || 'T.MSP_ID, T.PRIORITY, T.STAGE_ID, T.TOOLTIP_DESCR, T.RTF_BITS, '
          || 'CASE WHEN T.RTF_BITS > 0 THEN ''Y'' ELSE ''N'' END AS RTF, '
          || 'CASE WHEN T.CRI_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS EXCEPTIONS, '
          || 'CASE WHEN T.DOC_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS DOCUMENTS, '
          || 'CASE WHEN T.SCORECARD_COUNT > 0 THEN ''Y'' ELSE ''N'' END AS SCORECARDS, '
          || '''N'' AS REQUIREMENTS, '
          || 'T.DEPENDENCIES, T.CALENDAR_ID, '
          || '(SELECT C.ELEMENT_NAME FROM TMT_CALENDARS C WHERE C.ELEMENT_ID = T.CALENDAR_ID) AS CALENDAR_NAME, '
          || 'T.START_FINISH_FLAG, T.FINISH_START_DATE, T.START_FINISH_DURATION, T.UNRESOLVED_DURATION, '
          || ''''' AS FIXED_DURATION, T.CURRENCY_ID, '
          || 'T.BUFFER_PERCENT, T.METHOD, T.PROBABILITY, '
          || '(RTRIM(TO_CHAR(T.MAJOR_REVISION)) || ''.'' || RTRIM(TO_CHAR(T.MINOR_REVISION))) AS REVISION, '
          || ''''' AS REVISION_HISTORY, T.REVISION_TYPE, T.REVISION_IMPACT, T.LAST_WF_TYPE, T.LAST_WF_BY, T.LAST_WF_DT, '
          || 'T.CREATED_DT, T.CLOSED_BY, T.ASSIGN_COUNT, T.ASSIGN_NAMES, T.RISK_EXPOSURE, T.CONSEQUENSE_COST, '
          || 'T.TRANSFER_FLAG, T.CUSTOM_INT_1, '
          || 'T.FS_CODE, T.FS_STATE, T.FS_LABOR_CODE, T.BUBBLE_COLOR, T.BUBBLE_PUBLISHED_DT,T.PREVIOUS_WKF_STEP, T.DEPEND_FLAG , '
          || 'T.ESTIMATED_BASELINE1_START, T.ESTIMATED_BASELINE1_FINISH, T.ESTIMATED_BASELINE1_DURATION, T.ESTIMATED_BASELINE2_START, T.ESTIMATED_BASELINE2_FINISH, T.ESTIMATED_BASELINE2_DURATION, T.ESTIMATED_BASELINE3_START, T.ESTIMATED_BASELINE3_FINISH, T.ESTIMATED_BASELINE3_DURATION, '
          || 'T.ESTIMATED_BASELINE1_BUDGET, T.ESTIMATED_BASELINE2_BUDGET, T.ESTIMATED_BASELINE3_BUDGET, T.R_ESTIMATED_BASELINE1_BUDGET, T.R_ESTIMATED_BASELINE2_BUDGET, T.R_ESTIMATED_BASELINE3_BUDGET, '
          || 'T.S_ESTIMATED_BASELINE1_BUDGET, T.S_ESTIMATED_BASELINE2_BUDGET, T.S_ESTIMATED_BASELINE3_BUDGET, T.SUBPROJECT_COUNT, '
          || 'T.BASELINE1_EST_DT, T.BASELINE1_SCH_DT, T.BASELINE2_EST_DT, T.BASELINE2_SCH_DT, T.BASELINE3_EST_DT, T.BASELINE3_SCH_DT, '
          || 'T.TIMECODE_1_ID, T.TIMECODE_2_ID, T.ASSET_ID, T.ASSET_NAME, T.ASSET_FLAG, COALESCE(T.FINANCIAL_CATEGORY, 6501) AS FINANCIAL_CATEGORY, '
          || 'COALESCE((SELECT ELEMENT_NAME FROM TMT_TIMECODES WHERE ELEMENT_ID = T.TIMECODE_1_ID),'''') AS TIMECODE_1_NAME, '
          || 'COALESCE((SELECT ELEMENT_NAME FROM TMT_TIMECODES WHERE ELEMENT_ID = T.TIMECODE_2_ID),'''') AS TIMECODE_2_NAME, '
          || 'COALESCE((SELECT LEVEL_LAST_CALL_OPTION FROM PROJECT_DETAILS WHERE PROJECT_ID = T.ELEMENT_ID),0) AS LEVEL_LAST_CALL_OPTION, '
          || 'COALESCE((SELECT SCENARIO_NAME FROM PROJECT_DETAILS PD WHERE PD.PROJECT_ID = T.ELEMENT_ID AND PD.LEVEL_LAST_CALL_OPTION IN (4,6)),'''') AS SCENARIO_NAME, '
          || 'COALESCE((SELECT RTRIM(RS.FULL_NAME) FROM RESOURCES RS, PROJECT_DETAILS PD WHERE RS.RESOURCE_ID = PD.SCENARIO_USER_ID AND PD.PROJECT_ID = T.ELEMENT_ID),'''') AS SCENARIO_CREATOR_NAME, '
          || 'COALESCE((SELECT SCENARIO_DATETIME FROM PROJECT_DETAILS WHERE PROJECT_ID = T.ELEMENT_ID),TO_TIMESTAMP(''1899-01-01 08:00:00.000000'')) AS SCENARIO_DATETIME, '
          || 'T.MP_STATUS_DT, T.PUBLISHED_BY, '
          || 'CASE WHEN T.BASELINE_FLAG = 4 THEN 0 WHEN T.BASELINE_FLAG = 1 THEN 2 ELSE 1 END AS BASELINE_FLAG, '
          || 'CASE WHEN T.TYPE_ID IN (', 1, 9999);
   WBS_RETURN_COLUMNS := SubStr(WBS_RETURN_COLUMNS || PMO_CNST.SC_STR_PROJECT_RESOURCE_TYPES, 1, 9999);
   WBS_RETURN_COLUMNS := SubStr(WBS_RETURN_COLUMNS || 
                                                   ') AND (SELECT COUNT(TSK_ASSIGNMENT_ID) FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = T.ELEMENT_ID AND TASK_ID = T.ELEMENT_ID AND (LEVEL_START <> UNRESOLVED_START OR LEVEL_FINISH <> UNRESOLVED_FINISH)) > 0 THEN ''1'' ELSE ''0'' END AS LEVEL_AFFECTED_FLAG, T.ELEMENT_FLAG,(SELECT PD.RUP_HOME_PATH FROM PROJECT_DETAILS PD WHERE PD.PROJECT_ID = T.PROJECT_ID) AS RUP_HOME_PATH '
                                                   , 1, 9999);
   
   If ((h_name IS NOT NULL)) OR ((h_filter_string IS NOT NULL)) Then
      -- ignore include closed, include completed, and exclude sub element filter
      h_active_record := PMO_FUNC.BIT_OR(Coalesce(h_active_record, 0), PMO_CNST.SC_INCLUDE_CLOSED);
      -- include closed
      h_active_record := PMO_FUNC.BIT_OR(Coalesce(h_active_record, 0), PMO_CNST.SC_INCLUDE_COMPLETED);
      -- include completed
      If (BITAND(Coalesce(
            h_active_record, 0), PMO_CNST.SC_EXCLUDE_SUB_ELEMENT)) = PMO_CNST.SC_EXCLUDE_SUB_ELEMENT Then
         h_active_record := Coalesce(h_active_record, 0) - PMO_CNST.SC_EXCLUDE_SUB_ELEMENT;
         -- remove exclude sub element
      End If;
   End If;
   
   -- search projects by name from a list of project or a list of portfolios or a portfolios
   If (h_name IS NOT NULL) Then
      If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WBS_TEMPLATE_VIEW Then
         h_filedName := 'T.ELEMENT_NAME(C)';
      Else
         h_filedName := 'T.ELEMENT_NAME(C),T.REF_CODE(C)';
      End If;
      h_search_option := FC_PARSE_SEARCH_STR(h_name, h_filedName, h_keyword_search1);
      
      If Coalesce(h_search_option, 0) = PMO_CNST.SC_FIND_ATTRIBUTE_NAME Then
         h_name := NULL;
         --trace_printf("\n h_keyword_search1:%s", h_keyword_search1);
         --trace_printf("\n h_filter_string:%s", h_filter_string);
         
         null_return := FC_FIND_ATTR_BY_NAME(PMO_CNST.SC_TABLE_WBS, h_keyword_search1, h_filter_string
                                    , h_filter_count);
         
         If Coalesce(h_filter_count, 0) = 0 Then
            h_filter_count := 1;
            h_filter_string := PMO_CNST.SC_EMPTY_ATTRIBUTE_ID;
         End If;
      Else
         If Coalesce(Length(h_keyword_search1), 0) = 0 Then
            h_name := NULL;
         Else
            h_name := h_keyword_search1;
         End If;
      End If;
      
      If (Coalesce(h_sp_id, 0) = 1) AND (Coalesce(h_type, 0) = 0) Then
         h_type := 1019;
      ElsIf (Coalesce(h_sp_id, 0) = PMO_CNST.SC_WBS_PORTFOLIO_VIEW) OR (Coalesce(
                h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW) Then
         If Coalesce(h_sp_id, 0) = PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW Then
            searchInPortConfig := 1;
         End If;
         
         If ((h_ids IS NULL)) AND (Coalesce(h_type, 0) = 0) Then
            h_type := 1019;
         ElsIf Coalesce(h_type, 0) = 1000 Then
            h_type := 2019;
         ElsIf Coalesce(h_type, 0) = 999 Then
            h_type := 2019;
            searchUnderFolder := 1;
         End If;
      End If;
   End If;
   
   If Coalesce(h_sp_id, 0) = 5 Then
      h_type := 0;
   End If;
   
   /*********************************************************************/
   -- the following section is for supporting search by name just at root node in wbs tree
   /*********************************************************************/
   -- BEGIN
   h_wbs_search_criteria := PMO_CNST.SC_SEARCH_CHILDREN_ITEMS;
   
   If (searchInPortConfig = 0) AND (Coalesce(h_sp_id, 0) <> PMO_CNST.SC_WBS_TEMPLATE_VIEW) Then
      If ((h_filter_string IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_filter_string
            , 'EMPTY_TREE_FLAG') = 0) Then
         h_sp_id := 0;
         h_type := 0;
         
         myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID,';
         myStmt := myStmt || 'T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, ';
         myStmt := myStmt || 
                    '0 AS PARTICIPANT, '''' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 
                    ', 0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_WBS T';
         myStmt := myStmt || 
                    ' WHERE T.ELEMENT_ID = ''EMPTY_TREE_FLAG'' AND T.PROJECT_ID = ''empty_tree_flag'' ';
         myStmt := myStmt || ' ORDER BY T.RANK ';
         
         --trace_printf("\n  WBS VIEW-------> resultset 000: %s\n", myStmt);
      ElsIf (h_name IS NOT NULL) Then
         h_wbs_search_criteria := PMO_CNST.SC_SEARCH_ALL_ITEMS;
         h_filter_string := NULL;
         h_filter_count := 0;
         h_sp_id := 141;
         h_type := 1019;
         h_ids := NULL;
         h_stage_id := 0;
      ElsIf (h_filter_string IS NOT NULL) Then
         h_wbs_search_criteria := PMO_CNST.SC_SEARCH_ALL_ITEMS;
         h_sp_id := 141;
         h_type := 1019;
         h_ids := NULL;
         h_stage_id := 0;
      ElsIf Coalesce(h_sp_id, 0) = 2 Then
         h_type := 0;
      End If;
   End If;
   -- END
   /*********************************************************************/
   
   --trace_printf("\n h_type:%d \n", h_type);
   --trace_printf("\n h_sp_id:%d \n", h_sp_id);
   --trace_printf("\n searchUnderFolder:%d \n", searchUnderFolder);
   --trace_printf("\n searchInPortConfig:%d \n", searchInPortConfig);
   --trace_printf("\n h_name:%s:%d \n", h_name, h_name_Ind);
   --trace_printf("\n h_ids:%s:%d \n", h_ids, h_ids_Ind);
   --trace_printf("\n h_filter_string:%s:%d \n", h_filter_string, h_filter_string_Ind);
   --trace_printf("\n h_wbs_search_criteria:%d \n", h_wbs_search_criteria);
   
   If (Coalesce(h_sp_id, 0) = 31) OR (Coalesce(h_sp_id, 0) = 1) OR (Coalesce(h_sp_id, 0) = 2) 
             OR ((Coalesce(h_type, 0) > 1000) AND (Coalesce(h_type, 0) < 5000)) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_WBS_VIEW';
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   h_current_id := NULL;
   If ((Coalesce(h_type, 0) > 1000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(h_type, 0) > 5000) 
              AND (Coalesce(h_type, 0) < 7000)) Then
      If Coalesce(h_sp_id, 0) = 2 Then
         If ((Coalesce(h_type, 0) > 2000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(h_type, 0) > 6000) 
               AND (Coalesce(h_type, 0) < 7000)) Then
            h_type := Coalesce(h_type, 0) - 1000;
         End If;
         
         h_id := SubStr(h_ids, 1, 32);
         h_current_id := h_id;
         
         h_tree_type := 0;
         h_bitflag := 2;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_id, h_resource_id, h_bitflag);
         
         h_is_single_project := 1;
      ElsIf ((Coalesce(h_type, 0) > 2000) AND (Coalesce(h_type, 0) < 3000)) OR ((Coalesce(
                 h_type, 0) > 6000) AND (Coalesce(h_type, 0) < 7000)) Then
         h_type := Coalesce(h_type, 0) - 1000;
         
         h_tree_type := PMO_CNST.SC_TABLE_PORTFOLIO;
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         h_executive_security := NULL;
         h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
         
         h_bitflag := Coalesce(h_bitflag, 0);
         
         h_is_single_project := 0;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      ElsIf Coalesce(h_type, 0) > 1000 Then
         h_tree_type := PMO_CNST.SC_TABLE_WBS;
         
         h_table_type := PMO_CNST.SC_TABLE_WBS;
         h_executive_security := NULL;
         h_bitflag := FC_GET_ACTIVE_EXECUTIVE_SECURE(h_table_type, h_resource_id, h_executive_security
                                            , h_can_view_pivot);
         h_bitflag := Coalesce(h_bitflag, 0);
         
         h_is_single_project := 0;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_ids, h_resource_id, h_bitflag);
      End If;
      
      If Coalesce(h_sp_id, 0) = 2 Then
         h_sp_id := 302;
         -- next level in portal page
      Else
         h_sp_id := 300;
         -- first level in portal page
      End If;
      
      If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
         h_temp_sc_state := FC_GET_PRJ_VAR_CONDITION(h_type, h_prj_var_condition);
      End If;
      
      If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) <= 1821) Then
         If Coalesce(h_sp_id, 0) = 302 Then
            PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         Else
            If (Coalesce(h_type, 0) >= 1100) AND (Coalesce(h_type, 0) >= 1421) Then
               h_int_x := PMO_CNST.SC_BUDGET_VAR_SECURITY_INT;
            ElsIf (Coalesce(h_type, 0) >= 1500) AND (Coalesce(h_type, 0) >= 1821) Then
               h_int_x := PMO_CNST.SC_SCHED_VAR_SECURITY_INT;
            End If;
            -- security value for effort/sched variance
            
            null_return := FC_APPLY_VARIANCE_SECURITY(h_int_x, h_resource_id);
            
            PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         End If;
      Else
         PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
      End If;
      
      If ((h_name IS NOT NULL)) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
         h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
      End If;
      
      null_return := FC_FIX_WBS_VIEW_SECURITY_ID(h_sp_id, h_type, h_resource_id, h_executive_security);
      
      If searchInPortConfig = 1 Then
         myStmt := 
               'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, A.PARENT_ID, T.ELEMENT_ID, T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      Else
         myStmt := 
               'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, A.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
      End If;
      
      myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
      
      myStmt := myStmt || ' CASE WHEN A.ASSIGNED = 1 THEN ''M'' ';
      myStmt := myStmt || 
                    ' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME AS PROJECT_NAME, ';
      
      myStmt := myStmt || 'A.START_FINISH_DATE, A.MINUTE_PER_DAY, A.EFFORT_DRIVEN, ';
      
      If (Coalesce(h_sp_id, 0) = 302) AND (Coalesce(h_type, 0) >= 1100) AND (Coalesce(
             h_type, 0) <= 1821) Then
         myStmt := myStmt || 'CASE WHEN ';
         myStmt := myStmt || h_prj_var_condition;
         myStmt := myStmt || ' AND T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_RESOURCE_TYPES;
         myStmt := myStmt || ') THEN 1 ELSE 0 END AS FILTER_FLAG, ';
      Else
         myStmt := myStmt || 'A.FILTER_FLAG, ';
      End If;
      myStmt := myStmt || WBS_RETURN_COLUMNS;
      
      h_include_prj_detail_columns := 1;
      
      --trace_printf("\n h_type:%d", h_type);
      --trace_printf("\n h_name:%s:%d", h_name, h_name_Ind);
      --trace_printf("\n h_filter_string:%s:%d", h_filter_string, h_filter_string_Ind);
      
      If ((h_name IS NOT NULL)) OR ((h_filter_string IS NOT NULL)) Then
         h_include_prj_detail_columns := 1;
      ElsIf IsTaskFilter(Coalesce(h_type, 0)) = 1 Then
         h_include_prj_detail_columns := 0;
      End If;
      
      If Coalesce(h_include_prj_detail_columns, 0) = 1 Then
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
      Else
         myStmt := myStmt || 
                    ', 0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER  ';
      End If;
      myStmt := myStmt || ' FROM TMT_WBS T JOIN PMO_WBS_VIEW A ON A.ELEMENT_ID = T.ELEMENT_ID ';
      
      If Coalesce(h_include_prj_detail_columns, 0) = 1 Then
         myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
      End If;
      
      If searchUnderFolder = 1 Then
         BEGIN
            UPDATE PMO_WBS_VIEW
            SET LEVEL_ID = LEVEL_ID + 2,
            PARENT_ID = h_ids
            WHERE 1 = 1;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 537+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(538) : views.sql(3565) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         myStmt := myStmt || 
                    ' UNION ALL SELECT 1 AS LEVEL_ID, P.TYPE_ID, ''0000'' AS RANK, P.PARENT_ID, P.ELEMENT_ID, '
                || '1 AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ''N'' AS ASSIGNED,'''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_PORTFOLIOS P ';
         myStmt := myStmt || 
                    'LEFT JOIN TMT_WBS T ON T.PROJECT_ID = P.ELEMENT_ID AND T.ELEMENT_ID = P.ELEMENT_ID ';
         myStmt := myStmt || 'WHERE P.ELEMENT_ID = ''';
         myStmt := myStmt || h_ids;
         myStmt := myStmt || '''';
      End If;
      
      myStmt := myStmt || ' ORDER BY RANK';
   End If;
   
   CASE h_sp_id
      WHEN 1 THEN
         -- FIRST level
         If (Coalesce(h_type, 0) = 0) OR (Coalesce(h_type, 0) = 3001) Then
            myStmt := 
                  'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, A.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' case when T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || 
                    ') then COALESCE(A.SECURITY_ID,''SCPROJECTVISITOR'') else A.SECURITY_ID end AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, T.ELEMENT_NAME AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         Else
            myStmt := 
                  'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' case when T.TYPE_ID IN (';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || 
                    ') then COALESCE(A.SECURITY_ID,''SCPROJECTVISITOR'') else A.SECURITY_ID end AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, T.ELEMENT_NAME AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         End If;
         
         If Coalesce(h_type, 0) = 0 Then
            BEGIN
               INSERT INTO PMO_WBS_VIEW (RANK, ELEMENT_ID, ELEMENT_NAME, TRANSFER_FLAG, ELEMENT_TYPE_BITS, PARTICIPANT, SECURITY_ID,  REF_CODE, TYPE_ID, CHILD_COUNT, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)
               SELECT T.RANK, T.ELEMENT_ID, T.ELEMENT_NAME, T.TRANSFER_FLAG, A.ELEMENT_TYPE_BITS, A.PARTICIPANT, A.SECURITY_ID, T.REF_CODE, T.TYPE_ID, T.CHILD_COUNT, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR
               FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
               WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> 'D'
               AND A.RESOURCE_ID = h_resource_id AND T.TYPE_ID <> PMO_CNST.SC_RESOURCE AND
               (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED = '1'));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 615+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(616) : views.sql(3643) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_filter_string IS NULL) Then
               BEGIN
                  DELETE FROM PMO_WBS_VIEW T
                  WHERE T.ELEMENT_ID IN (SELECT C.ELEMENT_ID
                  FROM PMO_WBS_VIEW C, PMO_WBS_VIEW P
                  WHERE C.ELEMENT_ID = T.ELEMENT_ID AND LENGTH(P.RANK) < LENGTH(C.RANK)
                  AND ((P.RANK >= SUBSTR(C.RANK, 1, 8) AND P.RANK <= C.RANK) 
                  AND SUBSTR(C.RANK,1, CASE WHEN LENGTH(P.RANK) > LENGTH(C.RANK) THEN 1 ELSE LENGTH(P.RANK) END ) = P.RANK));
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 638+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(639) : views.sql(3666) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            myStmt := myStmt || 
                    'FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID AND T.TYPE_ID <> 11 ';
            
            If (h_filter_string IS NOT NULL) Then
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               myStmt := myStmt || 
                     ' AND EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE W.PROJECT_ID = T.ELEMENT_ID ';
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               myStmt := myStmt || 
                     ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND ((CA.TYPE_ID <> 1) OR (W.PROJECT_ID = W.ELEMENT_ID))';
               myStmt := myStmt || ' AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || ') ';
               myStmt := myStmt || 'AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
            End If;
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 002\n");
         ElsIf Coalesce(h_type, 0) = 3001 Then
            BEGIN
               INSERT INTO PMO_WBS_VIEW (RANK, ELEMENT_ID, PARENT_ID, ELEMENT_NAME, TRANSFER_FLAG, ELEMENT_TYPE_BITS, PARTICIPANT, SECURITY_ID, CHILD_COUNT, MILESTONE_PERCENT, T_RANK, BUBBLE_COLOR)
               SELECT T.RANK, T.ELEMENT_ID, T.PARENT_ID, T.ELEMENT_NAME, T.TRANSFER_FLAG, A.ELEMENT_TYPE_BITS, A.PARTICIPANT, A.SECURITY_ID, T.CHILD_COUNT, T.MILESTONE_PERCENT, T.T_RANK, T.BUBBLE_COLOR
               FROM PROJECT_ASSIGNMENTS A, TMT_WBS T
               WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> 'D'
               AND A.RESOURCE_ID = h_resource_id AND
               (BITAND(A.PARTICIPANT, 3) > 0 OR (BITAND(A.PARTICIPANT, 3) = 0 AND T.PUBLISHED = '1'));
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 684+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(685) : views.sql(3712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN CRITICAL_CHILD;
            
            LOOP
               FETCH CRITICAL_CHILD INTO h_rank, h_parent_id;
               IF CRITICAL_CHILD%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
                  EXIT;
               End If;
               
               BEGIN
                  UPDATE PMO_WBS_VIEW
                  SET CHILD_COUNT = (SELECT COUNT(RANK) FROM TMT_WBS
                  WHERE RANK LIKE h_rank || '%'
                  AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
                  AND LEVEL_INDEX <> -1 AND TOTAL_SLACK = 0),
                  SECURITY_ID = COALESCE((SELECT SECURITY_ID FROM PMO_WBS_VIEW
                  WHERE ELEMENT_ID = h_parent_id), '') || COALESCE(SECURITY_ID, '')
                  WHERE RANK = h_rank;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 721+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(722) : views.sql(3749) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
            BEGIN
               CLOSE CRITICAL_CHILD ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 734+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(735) : views.sql(3762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            myStmt := myStmt || 'FROM TMT_WBS T, PMO_WBS_VIEW A WHERE A.ELEMENT_ID = T.ELEMENT_ID ';
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 003\n");
         Else
            -- filter by type
            myStmt := myStmt || 'FROM PROJECT_ASSIGNMENTS A, TMT_WBS T ';
            myStmt := myStmt || 
                    'WHERE T.ELEMENT_ID = A.PROJECT_ID AND T.REC_STATUS <> ''D'' AND A.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' AND (BITAND(A.PARTICIPANT, 3) > 0 OR T.PUBLISHED IN (''1'')) ';
            
            CASE h_type
               
               WHEN 223 THEN
                  --deliverable and transferred cri
                  typeList := '(' || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES || ') ';
               
               WHEN 224 THEN
                  --tasks and milestone
                  typeList := '(' || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ') ';
               
               WHEN 4 THEN
                  --transfer cris
                  typeList := '(36, 37, 39) ';
               
               WHEN 0 THEN
                  --project and proposal
                  typeList := '(' || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES || ') ';
               
               ELSE
                  typeList := '(' || TO_CHAR(Coalesce(h_type, 0)) || ') ';
            END CASE;
            
            h_keyword_search1 := NULL;
            
            If (h_filter_string IS NULL) Then
               myStmt := myStmt || ' AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
               myStmt := myStmt || typeList;
               
               myStmt := myStmt || 
                     ' AND (T.RANK >= SUBSTR(RANK,1,8) AND T.RANK <= RANK AND SUBSTR(RANK, 1, CASE WHEN LENGTH(RANK) < LENGTH(T.RANK) THEN 1 ELSE LENGTH(T.RANK) END) = T.RANK)';
               
               If (BITAND(Coalesce(h_active_record, 0), 2)) = 2 Then
                  tempStr := TO_CHAR(FC_BITS_VALUE(h_type));
                  myStmt := myStmt || ' AND BITAND(A.ELEMENT_TYPE_BITS, ';
                  myStmt := myStmt || tempStr;
                  myStmt := myStmt || ') > 0 ';
               End If;
               
               If Coalesce(h_type, 0) = PMO_CNST.SC_DELIVERABLE Then
                  myStmt := myStmt || ' AND TRANSFER_FLAG IS NuLL) ';
               Else
                  myStmt := myStmt || ') ';
               End If;
            Else
               -- Filter by attribute list
               h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
               
               myStmt := myStmt || 
                     ' AND EXISTS(SELECT W.ELEMENT_ID FROM TMT_WBS W WHERE W.PROJECT_ID = T.ELEMENT_ID ';
               
               myStmt := myStmt || ' AND W.TYPE_ID IN ';
               myStmt := myStmt || typeList;
               
               myStmt := myStmt || ' AND (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
               
               myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
               
               myStmt := myStmt || 
                     ' WHERE CA.ELEMENT_ID = W.ELEMENT_ID AND ((CA.TYPE_ID <> 1) OR (CA.TYPE_ID = 1 AND W.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || '))) AND PA.ELEMENT_ID IN (';
               myStmt := myStmt || h_filter_string;
               myStmt := myStmt || ') ';
               myStmt := myStmt || 'AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
               myStmt := myStmt || h_str_filter_count;
               myStmt := myStmt || ')';
               
               myStmt := myStmt || ' AND EXISTS(SELECT ELEMENT_ID FROM TMT_WBS WHERE TYPE_ID IN ';
               myStmt := myStmt || typeList;
               myStmt := myStmt || ' AND PROJECT_ID = A.PROJECT_ID) ';
            End If;
            myStmt := myStmt || ' ORDER BY T.RANK ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 004\n");
         End If;
      
      WHEN 2 THEN
         -- NEXT level
         h_id := SubStr(h_ids, 1, 32);
         
         If ((h_id IS NULL)) OR ((h_resource_id IS NULL)) OR ((h_type IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 834+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(835) : views.sql(3862) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 837+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(838) : views.sql(3865) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         h_tree_type := 0;
         h_bitflag := 2;
         h_current_id := h_id;
         null_return := FC_GET_SELECTED_PROJECTS(h_tree_type, h_id, h_resource_id, h_bitflag);
         
         h_is_single_project := 1;
         PMO_DB2M.SC_STATE := FC_WBS_VIEW(h_sp_id, h_resource_id, h_name, h_filter_string, h_filter_count
                                 , h_type, h_active_record, h_stage_id, h_is_single_project
                                 , h_wbs_search_criteria, h_prj_var_condition);
         
         If ((h_name IS NOT NULL)) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
            h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
         End If;
         
         null_return := FC_FIX_WBS_VIEW_SECURITY_ID(h_sp_id, h_type, h_resource_id, h_executive_security);
         
         If (BITAND(Coalesce(h_active_record, 0), 4)) = 4 Then
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' A.SECURITY_ID AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME, ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 005.1 -> ok\n");
         Else
            myStmt := 
                  'SELECT A.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
            myStmt := myStmt || ' A.SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, A.PROJECT_NAME, ';
            
            --trace_printf("\n  WBS VIEW-------> resultset 005.2 -> ok");
         End If;
         
         myStmt := myStmt || 'A.START_FINISH_DATE, A.MINUTE_PER_DAY, A.EFFORT_DRIVEN, A.FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         myStmt := myStmt || ' FROM TMT_WBS T JOIN PMO_WBS_VIEW A ON A.ELEMENT_ID = T.ELEMENT_ID';
         myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
         
         myStmt := myStmt || ' ORDER BY T.RANK';
         
         
      
      WHEN 5 THEN
         -- refresh
         -- All these input parameters are required
         h_id := SubStr(h_ids, 1, 32);
         
         If ((h_id IS NULL)) OR ((h_resource_id IS NULL)) OR ((h_type IS NULL)) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 900+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(901) : views.sql(3928) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 903+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(904) : views.sql(3931) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         BEGIN
            SELECT RANK, PROJECT_ID, TYPE_ID
            INTO h_rank, h_project_id, h_temp_type_id
            FROM TMT_WBS WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 918+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(919) : views.sql(3946) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 925+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(926) : views.sql(3953) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 928+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(929) : views.sql(3956) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
         End If;
         
         BEGIN
            SELECT REPLACE(ELEMENT_NAME, '''', ''''''), START_FINISH_DATE, MINUTE_PER_DAY, EFFORT_DRIVEN
            INTO h_project_name, h_start_finish_date, h_minute_per_day, h_effort_driven
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 944+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(945) : views.sql(3972) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_security_ids := '';
         
         OPEN REFRESH_SECURITY;
         LOOP
            FETCH REFRESH_SECURITY INTO h_security_id;
            IF REFRESH_SECURITY%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If (PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS) Then
               EXIT;
            End If;
            
            h_security_ids := SubStr(h_security_ids || h_security_id, 1, 4000);
         END LOOP;
         BEGIN
            CLOSE REFRESH_SECURITY ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 973+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(974) : views.sql(4001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If Coalesce(h_type, 0) = 3001 Then
            h_rank := SubStr(h_rank || '%%', 1, 200);
            myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, '
                   || '(SELECT COUNT(*) FROM TMT_WBS WHERE RANK LIKE ''' || h_rank || ''' '
                   || 'AND LEVEL_INDEX <> -1 AND TOTAL_SLACK = 0 AND TYPE_ID IN (' 
                        || PMO_CNST.SC_STR_WBS_TASK_GROUP_TYPES || ')) AS CHILD_COUNT, '
                   || 'T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ''' || h_security_ids 
                        || ''' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '
                   || 'CASE WHEN InStr(ASSIGN_NAMES, ''' || h_full_name || ''' || '';'') > 0 THEN ''M'' '
                   || 'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, ''';
         Else
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ''';
            
            h_project_flag := FC_IS_EXECUTIVE_PROJECT(h_project_id, h_resource_id);
            
            If (Coalesce(Length(h_security_ids), 0) = 0) AND isProjectType(Coalesce(
                  h_temp_type_id, 0)) <> 0 Then
               myStmt := myStmt || PMO_CNST.SCPROJECTVISITOR_C_STR;
            Else
               myStmt := myStmt || h_security_ids;
            End If;
            
            If (Coalesce(h_project_flag, 0) > PMO_CNST.SC_EXECUTIVE_ASSIGNED_PROJECT) 
                  AND ((h_executive_security IS NOT NULL)) Then
               myStmt := myStmt || h_executive_security;
            End If;
            myStmt := myStmt || '''';
            
            myStmt := myStmt || ' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || 'CASE WHEN InStr(ASSIGN_NAMES, ''';
            myStmt := myStmt || h_full_name;
            myStmt := myStmt || ''' || '';'') > 0 THEN ''M'' ';
            myStmt := myStmt || 
                    'ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, ''';
         End If;
         myStmt := myStmt || h_project_name;
         myStmt := myStmt || ''' AS PROJECT_NAME, T.START_FINISH_DATE, ';
         myStmt := myStmt || TO_CHAR(h_minute_per_day);
         myStmt := myStmt || ' AS MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         If isProjectType(Coalesce(h_temp_type_id, 0)) <> 0 OR isWBSDeliv(Coalesce(
               h_temp_type_id, 0)) <> 0 Then
            myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
         Else
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         End If;
         myStmt := myStmt || 'FROM TMT_WBS T ';
         myStmt := myStmt || 'LEFT JOIN RESOURCE_ASSIGNMENTS A ';
         myStmt := myStmt || 'ON A.ELEMENT_ID = T.ELEMENT_ID ';
         myStmt := myStmt || 'AND A.RESOURCE_ID = ''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || ''' ';
         If isProjectType(Coalesce(h_temp_type_id, 0)) <> 0 OR isWBSDeliv(Coalesce(
               h_temp_type_id, 0)) <> 0 Then
            myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
         End If;
         myStmt := myStmt || ' WHERE T.ELEMENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' ';
         myStmt := myStmt || ' ORDER BY T.RANK ';
         --trace_printf("\n  WBS VIEW-------> resultset 006 ");
      WHEN PMO_CNST.SC_WBS_TEMPLATE_VIEW THEN
         -- TEMPLETED all Levels ( PUBLISHED = '2')
         h_id := SubStr(h_ids, 1, 32);
         
         If ((h_id IS NULL)) Then
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.T_RANK AS RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, 0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
            myStmt := myStmt || 
                    'FROM TMT_WBS T WHERE T.RANK = ''00000001'' OR T.RANK = ''00000002'' ORDER BY T.RANK ';
         Else
            BEGIN
               SELECT LEVEL_ID, T_RANK, ELEMENT_ID
               INTO h_level_id, h_rank, h_current_id
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1066+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1067) : views.sql(4094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            myStmt := 'INSERT INTO PMO_WBS_VIEW(PROJECT_ID, ELEMENT_ID, RANK, ELEMENT_NAME)';
            myStmt := myStmt || ' SELECT T.PROJECT_ID, T.ELEMENT_ID, T.RANK, T.ELEMENT_NAME ';
            myStmt := myStmt || 'FROM TMT_WBS T ';
            
            If ((h_name IS NOT NULL)) Then
               h_rank := SubStr(h_rank || '%%', 1, 200);
               
               myStmt := myStmt || 'WHERE T.T_RANK LIKE ''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' ';
               
               --trace_printf("\n myStmt:%s", myStmt);
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1096+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1097) : views.sql(4124) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
               
               myStmt := 'INSERT INTO PMO_WBS_VIEW(PROJECT_ID, ELEMENT_ID, RANK, ELEMENT_NAME)';
               myStmt := myStmt || ' SELECT T.PROJECT_ID, T.ELEMENT_ID, T.RANK, T.ELEMENT_NAME ';
               myStmt := myStmt || 'FROM PMO_WBS_VIEW P, TMT_WBS T ';
               
               -- insert templated item's children if any
               myStmt := myStmt || ' WHERE P.ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || 
                     ' AND SUBSTR(T.RANK, 1, CASE WHEN LENGTH(T.RANK) > LENGTH(P.RANK) THEN LENGTH(P.RANK) ELSE 1 END ) = P.RANK ';
               myStmt := myStmt || ' AND (';
               myStmt := myStmt || h_keyword_search1;
               myStmt := myStmt || ') ';
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1128+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1129) : views.sql(4156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
               
               -- delete templated items that dont match in search by name condition
               myStmt := ' DELETE FROM PMO_WBS_VIEW ';
               myStmt := myStmt || ' WHERE ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || ' AND ELEMENT_ID IN ';
               myStmt := myStmt || ' ( ';
               myStmt := myStmt || 'SELECT T.ELEMENT_ID ';
               myStmt := myStmt || ' FROM PMO_WBS_VIEW T ';
               myStmt := myStmt || ' WHERE ELEMENT_ID <> ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
               myStmt := myStmt || ' AND NOT (';
               myStmt := myStmt || h_keyword_search1;
               myStmt := myStmt || '))';
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1162+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1163) : views.sql(4190) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
            Else
               If (h_filter_string IS NULL) Then
                  myStmt := myStmt || 'WHERE T.REC_STATUS <> ''D''';
                  myStmt := myStmt || ' AND (T.ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' OR T.T_PARENT_ID=''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''')';
               Else
                  h_rank := SubStr(h_rank || '%%', 1, 200);
                  
                  myStmt := myStmt || 'WHERE T.T_RANK LIKE ''';
                  myStmt := myStmt || h_rank;
                  myStmt := myStmt || ''' AND T.REC_STATUS <> ''D'' AND (T.ELEMENT_ID = ''';
                  myStmt := myStmt || h_id;
                  myStmt := myStmt || ''' OR (';
                  
                  h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
                  myStmt := myStmt || ' (SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM';
                  myStmt := myStmt || ' TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
                  myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.ELEMENT_ID AND PA.ELEMENT_ID IN (';
                  myStmt := myStmt || h_filter_string;
                  myStmt := myStmt || ') ';
                  myStmt := myStmt || 
                        ' AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
                  myStmt := myStmt || h_str_filter_count;
                  myStmt := myStmt || '))';
               End If;
               
               BEGIN
                  EXECUTE IMMEDIATE myStmt;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1208+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1209) : views.sql(4236) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        LFILE.FPUT('AllTrace',                         'SQL : ' || myStmt);
                     END IF; 
                  END;
               END;
            End If;
            
            If ((h_name IS NOT NULL)) AND (Coalesce(h_search_option, 0) <> PMO_CNST.SC_FIND_ATTRIBUTE_NAME) Then
               h_temp_sc_state := FC_CLEANS_WBS_BY_NAME(h_search_option, h_current_id);
            End If;
            
            myStmt := 'SELECT CASE WHEN T.ELEMENT_ID = ''' || h_id || ''' THEN T.LEVEL_ID ELSE (' 
                  || TO_CHAR(Coalesce(h_level_id, 0)) || ' + 1) END AS LEVEL_ID, '
                   || 'T.TYPE_ID, T.T_RANK AS RANK, '
                   || 'CASE WHEN T.PUBLISHED=''2'' THEN T.T_PARENT_ID ELSE T.PARENT_ID END AS PARENT_ID, '
                   || 'T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, '
                   || '0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '
                   || ''''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
            myStmt := myStmt || 'FROM TMT_WBS T,  PMO_WBS_VIEW P';
            myStmt := myStmt || ' WHERE T.ELEMENT_ID = P.ELEMENT_ID ';
            myStmt := myStmt || ' ORDER BY T.RANK ';
         End If;
         --trace_printf("\n  WBS VIEW-------> resultset 007");
         
         
      
      WHEN 32 THEN
         h_id := SubStr(h_ids, 1, 32);
         myStmt := 
               'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, 0 AS PARTICIPANT, ''SCPROJECTWORKFLOWVISITOR'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM TMT_WBS T WHERE (T.ELEMENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''' OR T.PARENT_ID = ''';
         myStmt := myStmt || h_id;
         myStmt := myStmt || ''') AND T.REC_STATUS <> ''D'' ORDER BY T.RANK ';
         
         
      
      WHEN 33 THEN
         If (h_resource_id IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1257+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1258) : views.sql(4285) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1260+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1261) : views.sql(4288) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER);
         tempStr := TO_CHAR(Coalesce(h_type, 0));
         
         myStmt := 
               'SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CRI_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, B.SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
         myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
         myStmt := myStmt || WBS_RETURN_COLUMNS;
         myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
         myStmt := myStmt || 'FROM (((TMT_WBS T JOIN TMT_CRI I ON I.PROJECT_ID=T.ELEMENT_ID AND I.TYPE_ID=';
         myStmt := myStmt || tempStr;
         myStmt := myStmt || 
                    ')  LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID=''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || 
                    ''') LEFT JOIN RESOURCE_ASSIGNMENTS A ON A.ELEMENT_ID = T.ELEMENT_ID AND BITAND(A.PARTICIPANT,';
         myStmt := myStmt || participant001;
         myStmt := myStmt || ')>0) ';
         myStmt := myStmt || 'WHERE T.TYPE_ID IN (';
         myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
         myStmt := myStmt || ') AND ((EXISTS(SELECT C.RES_ASSIGNMENT_ID FROM RESOURCE_ASSIGNMENTS C '
                || 'WHERE C.PROJECT_ID=T.ELEMENT_ID AND C.RESOURCE_ID=''';
         myStmt := myStmt || h_resource_id;
         myStmt := myStmt || 
                    ''') AND T.PUBLISHED=''1''  AND T.REC_STATUS <> ''D'') OR B.RES_ASSIGNMENT_ID=A.RES_ASSIGNMENT_ID)';
         
         If (h_filter_string IS NOT NULL) Then
            h_str_filter_count := TO_CHAR(Coalesce(h_filter_count, 0));
            
            myStmt := myStmt || 
                    ' AND ((SELECT COUNT(DISTINCT(PA.ELEMENT_ID)) FROM TMT_WBS_ATTRIBUTES PA, TMT_WBS_ATTRIBUTES CA';
            myStmt := myStmt || ' WHERE CA.ELEMENT_ID = T.PROJECT_ID AND PA.ELEMENT_ID IN (';
            myStmt := myStmt || h_filter_string;
            myStmt := myStmt || 
                    ') AND SUBSTR(CA.RANK, 1, CASE WHEN LENGTH(CA.RANK) < LENGTH(PA.RANK) THEN 1 ELSE LENGTH(PA.RANK) END) = PA.RANK) >= ';
            myStmt := myStmt || h_str_filter_count;
            myStmt := myStmt || ') ';
         End If;
         
         If (BITAND(Coalesce(h_active_record, 0), 1)) = 1 Then
            myStmt := myStmt || ' AND T.REC_STATUS NOT IN (''D'', ''L'') ORDER BY RANK';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 008");
         
         
      
      WHEN PMO_CNST.SC_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := SubStr(h_ids, 1, 32);
         If ((h_id IS NULL)) OR ((h_resource_id IS NULL) OR ((p_type IS NULL))) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1320+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1321) : views.sql(4348) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1323+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1324) : views.sql(4351) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1341+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1342) : views.sql(4369) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1358+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1359) : views.sql(4386) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1367+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1368) : views.sql(4395) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1370+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1371) : views.sql(4398) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1417+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1418) : views.sql(4445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1423+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1424) : views.sql(4451) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
            myStmt := myStmt || 
                    'FROM TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' ';
            
            myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009 myStmt:%s", myStmt);
         
         -- **** the 64 case is replaced with SC_REFRESH_PORTFOLIO case because 64 does not return security on portfolio level.
         
         --case 64: // returns only one row for this particuler ID
         --InitializeHost(SC_PARAM(h_id), SC_PARAM(h_ids), 32);
         --myStmt.strcpy("SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, P.CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, 'SCFOLDERMANAGER' AS SECURITY_ID, '' AS PORTFOLIO_SECURITY, '' AS ASSIGNED, '' AS PROJECT_NAME, ");
         --myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
         --myStmt.strcat(WBS_RETURN_COLUMNS);
         --myStmt.strcat(" ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, '1899-01-01' AS PURSUIT_START, '1899-01-01' AS FORECAST_CLOSED, '1899-01-01' AS PURSUIT_FINISH, '' AS OPPORTUNITY_TRACKING_NUMBER, '' AS OPPORTUNITY_OWNER, '' AS CONTRACT_TYPE, '' AS CONTACT_NUMBER, '' AS CUSTOMER_NUMBER ");
         --myStmt.strcat( "FROM TMT_PORTFOLIOS P LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.ELEMENT_ID ");
         --myStmt.strcat( "WHERE P.ELEMENT_ID = '");
         --myStmt.strcat( h_id);
         --myStmt.strcat( "'");
         --trace_printf("\n  WBS VIEW-------> resultset 010");
         --break;
         
      
      WHEN PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := SubStr(h_ids, 1, 32);
         If ((h_id IS NULL)) OR ((h_resource_id IS NULL) OR ((p_type IS NULL))) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1511+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1512) : views.sql(4539) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1514+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1515) : views.sql(4542) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1532+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1533) : views.sql(4560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1549+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1550) : views.sql(4577) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1558+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1559) : views.sql(4586) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1561+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1562) : views.sql(4589) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1608+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1609) : views.sql(4636) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1614+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1615) : views.sql(4642) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
            myStmt := myStmt || 
                    'FROM TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' ';
            
            myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009 myStmt:%s", myStmt);
         
         -- **** the 64 case is replaced with SC_REFRESH_PORTFOLIO case because 64 does not return security on portfolio level.
         
         --case 64: // returns only one row for this particuler ID
         --InitializeHost(SC_PARAM(h_id), SC_PARAM(h_ids), 32);
         --myStmt.strcpy("SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, P.CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, 'SCFOLDERMANAGER' AS SECURITY_ID, '' AS PORTFOLIO_SECURITY, '' AS ASSIGNED, '' AS PROJECT_NAME, ");
         --myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
         --myStmt.strcat(WBS_RETURN_COLUMNS);
         --myStmt.strcat(" ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, '1899-01-01' AS PURSUIT_START, '1899-01-01' AS FORECAST_CLOSED, '1899-01-01' AS PURSUIT_FINISH, '' AS OPPORTUNITY_TRACKING_NUMBER, '' AS OPPORTUNITY_OWNER, '' AS CONTRACT_TYPE, '' AS CONTACT_NUMBER, '' AS CUSTOMER_NUMBER ");
         --myStmt.strcat( "FROM TMT_PORTFOLIOS P LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.ELEMENT_ID ");
         --myStmt.strcat( "WHERE P.ELEMENT_ID = '");
         --myStmt.strcat( h_id);
         --myStmt.strcat( "'");
         --trace_printf("\n  WBS VIEW-------> resultset 010");
         --break;
         
      
      WHEN PMO_CNST.SC_EXPAND_WORKFLOW_ATTACHMENT THEN
         -- select parent list of workflow's attachment
         -- All these input parameters are required
         h_id := SubStr(h_ids, 1, 32);
         If ((h_id IS NULL)) OR ((h_resource_id IS NULL) OR ((p_type IS NULL))) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1701+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1702) : views.sql(4729) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1704+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1705) : views.sql(4732) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         participant001 := TO_CHAR(PMO_CNST.SC_MANAGER_RESPONSIBLE);
         
         BEGIN
            SELECT PROJECT_ID, RANK, RTRIM(TO_CHAR(LEVEL_ID + 1))
            INTO h_project_id, h_rank, h_level
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1722+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1723) : views.sql(4750) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
            BEGIN
               SELECT RANK
               INTO h_rank
               FROM TMT_PORTFOLIOS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1739+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1740) : views.sql(4767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_type := PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_WBS_VIEW', 1748+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_WBS_VIEW(1749) : views.sql(4776) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1751+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1752) : views.sql(4779) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
            End If;
         Else
            h_type := PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT;
         End If;
         
         If Coalesce(h_type, 0) = PMO_CNST.SC_PORTFOLIO_WKF_ATTACHMENT Then
            myStmt := ' SELECT P.LEVEL_ID, P.TYPE_ID , P.RANK, P.PARENT_ID, P.ELEMENT_ID,  ';
            myStmt := myStmt || 
                    ' P.CHILD_COUNT AS CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, ''SCFOLDERMANAGER'' AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
            myStmt := myStmt || ' ''N'' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || ' T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
            myStmt := myStmt || ' FROM TMT_PORTFOLIOS P ';
            myStmt := myStmt || 
                    ' LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.PROJECT_ID AND P.ELEMENT_ID = T.ELEMENT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 12) = P.RANK OR P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''')';
               myStmt := myStmt || ' ORDER BY RANK ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || ' WHERE P.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || '''';
               myStmt := myStmt || ' ORDER BY RANK ';
            Else
               h_sp_id := PMO_CNST.SC_WBS_PORTFOLIO_VIEW;
               
               BEGIN
                  SELECT ELEMENT_ID
                  INTO h_portfolio_owner_resource_id
                  FROM TMT_PORTFOLIOS
                  WHERE RANK = SUBSTR(h_rank, 1, 8);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1798+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1799) : views.sql(4826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1804+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1805) : views.sql(4832) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_portfolio_owner_resource_id, h_ids
                     , h_active_record, h_full_name, WBS_RETURN_COLUMNS);
            End If;
         ElsIf Coalesce(h_type, 0) = PMO_CNST.SC_WBS_ITEM_WKF_ATTACHMENT Then
            -- Project View
            myStmt := 
                  'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, B.PARTICIPANT, ';
            myStmt := myStmt || 'CASE WHEN ';
            myStmt := myStmt || ' T.ELEMENT_ID = ''';
            myStmt := myStmt || h_id;
            myStmt := myStmt || ''' AND B.SECURITY_ID IS NuLL THEN ';
            myStmt := myStmt || 'CASE WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCPROJECTWORKFLOWVISITOR'' WHEN T.TYPE_ID IN ( ';
            myStmt := myStmt || PMO_CNST.SC_STR_WBS_DELV_GROUP_TYPES;
            myStmt := myStmt || ' ) THEN ''SCDELIVERABLEWORKFLOWVISITOR'' ELSE ''SCTASKWORKFLOWVISITOR'' END ';
            myStmt := myStmt || 
                    'ELSE B.SECURITY_ID END AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, '''' AS ASSIGNED, '''' AS PROJECT_NAME, ';
            myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
            myStmt := myStmt || WBS_RETURN_COLUMNS;
            myStmt := myStmt || 
                    ', PD.BUDGET_SETTING_BITS, PD.DLV_BUDGET_SETTING_BITS, PD.OPPORTUNITY_NAME, PD.EXPECTED_COST, PD.EXPECTED_REVENUE, PD.EXPECTED_PAYBACK_PERIOD, PD.EXPECTED_PAYBACK_MONTH, PD.PURSUIT_START, PD.FORECAST_CLOSED, PD.PURSUIT_FINISH, PD.OPPORTUNITY_TRACKING_NUMBER, PD.OPPORTUNITY_OWNER, PD.CONTRACT_TYPE, PD.CONTACT_NUMBER, PD.CUSTOMER_NUMBER ';
            myStmt := myStmt || 
                    'FROM TMT_WBS T LEFT JOIN RESOURCE_ASSIGNMENTS B ON B.ELEMENT_ID = T.ELEMENT_ID AND B.RESOURCE_ID = ''';
            myStmt := myStmt || h_resource_id;
            myStmt := myStmt || ''' ';
            
            myStmt := myStmt || ' JOIN PROJECT_DETAILS PD ON PD.PROJECT_ID = T.PROJECT_ID ';
            
            If Coalesce(h_sp_id, 0) = PMO_CNST.SC_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE (SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, CASE WHEN LENGTH(T.RANK)>LENGTH(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''') THEN 1 ';
               myStmt := myStmt || 'ELSE LENGTH(T.RANK) END) = T.RANK ';
               myStmt := myStmt || 'AND T.RANK LIKE SUBSTR(''';
               myStmt := myStmt || h_rank;
               myStmt := myStmt || ''', 1, 8) || ''%'' ) ';
               myStmt := myStmt || ' AND LEVEL_ID > -1 ';
            ElsIf Coalesce(h_sp_id, 0) = PMO_CNST.SC_REFRESH_WORKFLOW_ATTACHMENT Then
               myStmt := myStmt || 'WHERE T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' ';
            Else
               myStmt := myStmt || 'WHERE (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_id;
               myStmt := myStmt || ''') ';
            End If;
            
            -- in workflow attachements if project is lock, we still should see the project tree
            --if( (h_active_record  1) == 1)
            --{
            --  myStmt.strcat( " AND T.REC_STATUS NOT IN ('D', 'L')");
            --}
            
            myStmt := myStmt || ' ORDER BY RANK ';
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 009 myStmt:%s", myStmt);
         
         -- **** the 64 case is replaced with SC_REFRESH_PORTFOLIO case because 64 does not return security on portfolio level.
         
         --case 64: // returns only one row for this particuler ID
         --InitializeHost(SC_PARAM(h_id), SC_PARAM(h_ids), 32);
         --myStmt.strcpy("SELECT P.LEVEL_ID, P.TYPE_ID, P.RANK, P.PARENT_ID, P.ELEMENT_ID, P.CHILD_COUNT, P.ELEMENT_NAME, P.DELETED_COUNT, P.REC_STATUS, 0 AS PARTICIPANT, 'SCFOLDERMANAGER' AS SECURITY_ID, '' AS PORTFOLIO_SECURITY, '' AS ASSIGNED, '' AS PROJECT_NAME, ");
         --myStmt.strcat("T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ");
         --myStmt.strcat(WBS_RETURN_COLUMNS);
         --myStmt.strcat(" ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, '1899-01-01' AS PURSUIT_START, '1899-01-01' AS FORECAST_CLOSED, '1899-01-01' AS PURSUIT_FINISH, '' AS OPPORTUNITY_TRACKING_NUMBER, '' AS OPPORTUNITY_OWNER, '' AS CONTRACT_TYPE, '' AS CONTACT_NUMBER, '' AS CUSTOMER_NUMBER ");
         --myStmt.strcat( "FROM TMT_PORTFOLIOS P LEFT JOIN TMT_WBS T ON P.ELEMENT_ID = T.ELEMENT_ID ");
         --myStmt.strcat( "WHERE P.ELEMENT_ID = '");
         --myStmt.strcat( h_id);
         --myStmt.strcat( "'");
         --trace_printf("\n  WBS VIEW-------> resultset 010");
         --break;
         
      
      WHEN PMO_CNST.SC_REFRESH_PORTFOLIO THEN
         -- 141 select portfolio view
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1889+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1890) : views.sql(4917) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
               , h_full_name, WBS_RETURN_COLUMNS);
         
      
      WHEN PMO_CNST.SC_WBS_PORTFOLIO_VIEW THEN
         -- 141 select portfolio view
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1899+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1900) : views.sql(4927) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
               , h_full_name, WBS_RETURN_COLUMNS);
         
      
      WHEN PMO_CNST.SC_PORTFOLIO_CONFIG_VIEW THEN
         -- 142 select portfolio configuration view
         h_active_record := 96;
         
         If ((h_ids IS NOT NULL)) AND (Coalesce(Length(h_ids), 0) = 32) Then
            BEGIN
               SELECT RANK || '%'
               INTO h_res_row_rank
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_ids;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 1921+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(1922) : views.sql(4949) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS) Then
               h_sp_id := 2;
               myStmt := 'SELECT T.LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, ';
               myStmt := myStmt || 
                     'T.SUBPROJECT_COUNT AS CHILD_COUNT, T.ELEMENT_NAME, T.DELETED_COUNT, T.REC_STATUS, A.PARTICIPANT, ';
               myStmt := myStmt || 
                     'COALESCE(A.SECURITY_ID, ''SCPROJECTVISITOR'') AS SECURITY_ID, '''' AS PORTFOLIO_SECURITY, ';
               myStmt := myStmt || 'CASE WHEN InStr(T.ASSIGN_NAMES, ''';
               myStmt := myStmt || h_full_name;
               myStmt := myStmt || 
                     ''' || '';'') > 0 THEN ''M'' ELSE (CASE WHEN T.ASSIGN_COUNT > 0 THEN ''A'' ELSE ''N'' END) END AS ASSIGNED, '''' AS PROJECT_NAME, ';
               myStmt := myStmt || 'T.START_FINISH_DATE, T.MINUTE_PER_DAY, T.EFFORT_DRIVEN, 0 AS FILTER_FLAG, ';
               myStmt := myStmt || WBS_RETURN_COLUMNS;
               myStmt := myStmt || 
                     ' ,0 AS BUDGET_SETTING_BITS, 0 AS DLV_BUDGET_SETTING_BITS, '''' AS OPPORTUNITY_NAME, 0.0 AS EXPECTED_COST, 0.0 AS EXPECTED_REVENUE, 0 AS EXPECTED_PAYBACK_PERIOD, 0 AS EXPECTED_PAYBACK_MONTH, TO_DATE(''1899-01-01'') AS PURSUIT_START, TO_DATE(''1899-01-01'') AS FORECAST_CLOSED, TO_DATE(''1899-01-01'') AS PURSUIT_FINISH, '''' AS OPPORTUNITY_TRACKING_NUMBER, '''' AS OPPORTUNITY_OWNER, '''' AS CONTRACT_TYPE, '''' AS CONTACT_NUMBER, '''' AS CUSTOMER_NUMBER ';
               myStmt := myStmt || 
                     ' FROM TMT_WBS T LEFT JOIN PROJECT_ASSIGNMENTS A ON A.PROJECT_ID = T.ELEMENT_ID AND A.RESOURCE_ID = ''';
               myStmt := myStmt || h_resource_id;
               myStmt := myStmt || ''' WHERE T.REC_STATUS <> ''D'' AND (T.ELEMENT_ID = ''';
               myStmt := myStmt || h_ids;
               myStmt := myStmt || ''' OR T.PARENT_ID = ''';
               myStmt := myStmt || h_ids;
               myStmt := myStmt || ''') AND T.TYPE_ID IN (';
               myStmt := myStmt || PMO_CNST.SC_STR_PROJECT_GROUP_TYPES;
               myStmt := myStmt || ') ORDER BY RANK ';
               
               --trace_printf("\n  WBS VIEW-------> resultset 014.1 -> %s", myStmt);
            Else
               --trace_printf("\n  WBS VIEW-------> resultset 014.2");
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1956+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1957) : views.sql(4984) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
                     , h_full_name, WBS_RETURN_COLUMNS);
            End If;
         Else
            --trace_printf("\n  WBS VIEW-------> resultset 014.3");
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1964+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1965) : views.sql(4992) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
                  , h_full_name, WBS_RETURN_COLUMNS);
         End If;
         
         --trace_printf("\n  WBS VIEW-------> resultset 014");
         
      
      WHEN PMO_CNST.SC_AGGREGATE_PORTFOLIO THEN
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1975+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(1976) : views.sql(5003) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN FC_WBS_PORTFOLIO_VIEW(h_sp_id, h_resource_id, h_resource_id, h_ids, h_active_record
               , h_full_name, WBS_RETURN_COLUMNS);
         
         --trace_printf("\n  WBS VIEW-------> resultset 014");
      
      WHEN 300 THEN
         -- first level of portal page
         NULL;
      
      WHEN 302 THEN
         -- next level of portal page
         NULL;
      
      WHEN 0 THEN
         -- return empty tree
         NULL;
      
      ELSE
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_WBS_VIEW', 1996+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_WBS_VIEW(1997) : views.sql(5024) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 1999+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(2000) : views.sql(5027) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   END CASE;
   
   --timeLog(file,"------------------>BEGIN LAST SELECT STMT IN SP_WBS_VIEW ");
   
   If (Coalesce(h_sp_id, 0) <> PMO_CNST.SC_WBS_PORTFOLIO_VIEW) AND (Coalesce(
             h_sp_id, 0) <> PMO_CNST.SC_WBS_PORTFOLIO_VIEW) Then
      --trace_printf("\n%s", myStmt);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 2010+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(2011) : views.sql(5038) : ' || SYSTIMESTAMP);
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_WBS_VIEW', 2013+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_WBS_VIEW(2014) : views.sql(5041) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_SELECT_READ(myStmt);
   End If;
   
   --timeLog(file,"------------------>END   LAST SELECT STMT IN SP_WBS_VIEW ");
   
   --timeLog(file,"------------------>END   SP_WBS_VIEW ");
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 2026+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(2027) : views.sql(5054) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_WBS_VIEW', 2038+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_WBS_VIEW(2039) : views.sql(5066) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_WBS_VIEW;
ALTER FUNCTION SP_WBS_VIEW COMPILE DEBUG;

/****************************************************************************************
db2 drop procedure SP_CRITERIA_SRCH
db2 create procedure SP_CRITERIA_SRCH(in type_id integer, in parent_id varchar(32), in filter_string varchar(50), in filter_type intger) external name '%procdir%pmoffice!SP_CRITERIA_SRCH' language c deterministic parameter style general with nulls

filter_type
    0: LIKE '%h_filter_string%'
    1: SOUNDEX '%h_filter_string%'
***************************************************************************************/

Prompt == FUNCTION SP_CRITERIA_SRCH ==
CREATE OR REPLACE FUNCTION SP_CRITERIA_SRCH
(
     p_type_id PLS_INTEGER
   , p_parent_id VARCHAR2
   , p_short_filter_string VARCHAR2
   , p_filter_type PLS_INTEGER
   , p_resource_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rank_like VARCHAR2(200);
   h_short_filter_string VARCHAR2(100);
   h_str_level_id VARCHAR2(1);
   h_table_name VARCHAR2(100);
   h_filter_type PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_sql VARCHAR2(2048);
-- Declare Cursors
      -- Resultset Cursor Declaration
      SCP661e PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661b PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661f PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661c PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      SCP661d PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_CRITERIA_SRCH') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_CRITERIA_SRCH'), '-- ' || SYSTIMESTAMP, ':r := SP_CRITERIA_SRCH(' || 
       COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_short_filter_string || ''''
          || ', ' || COALESCE(TO_CHAR(p_filter_type), 'NULL')
          || ', ' || '''' || p_resource_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 50+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_CRITERIA_SRCH(51) : views.sql(5140) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_resource_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_CRITERIA_SRCH', 57+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_CRITERIA_SRCH(58) : views.sql(5147) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 60+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(61) : views.sql(5150) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_type_id := p_type_id;
   h_filter_type := p_filter_type;
   h_parent_id := p_parent_id;
   h_short_filter_string := p_short_filter_string;
   
   If (h_short_filter_string IS NULL) Then
      h_short_filter_string := '';
   End If;
   
   CASE h_type_id
      WHEN PMO_CNST.SC_ORGANIZATION THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 88+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(89) : views.sql(5178) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 101+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(102) : views.sql(5191) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_GROUP THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 121+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(122) : views.sql(5211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 134+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(135) : views.sql(5224) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_DEPARTMENT THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 154+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(155) : views.sql(5244) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 167+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(168) : views.sql(5257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      
      WHEN PMO_CNST.SC_OFFICE THEN
         --in case of updating this case update also SC_ORGANIZATION: case SC_GROUP: case SC_DEPARTMENT:case SC_OFFICE
         h_table_name := 'TMT_ORGANIZATIONS';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 186+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(187) : views.sql(5276) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 199+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(200) : views.sql(5289) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_GEOGRAPHY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 218+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(219) : views.sql(5308) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 231+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(232) : views.sql(5321) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_REGION THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 250+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(251) : views.sql(5340) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 263+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(264) : views.sql(5353) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_COUNTRY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 282+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(283) : views.sql(5372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 295+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(296) : views.sql(5385) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_STATE_PROVINCE THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 314+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(315) : views.sql(5404) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 327+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(328) : views.sql(5417) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_CITY THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 346+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(347) : views.sql(5436) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 359+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(360) : views.sql(5449) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_LOCATION THEN
         --in case of updating this case update also SC_GEOGRAPHY: SC_REGION: SC_COUNTRY: SC_STATE_PROVINCE: SC_CITY: SC_LOCATION:
         h_table_name := SubStr('TMT_GEOGRAPHIES', 1, 100);
         h_table_name := 'TMT_GEOGRAPHIES';
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 379+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(380) : views.sql(5469) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 392+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(393) : views.sql(5482) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_POOLS THEN
         If (p_parent_id IS NULL) Then
            If Coalesce(h_filter_type, 0) = 2 Then
               h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
               
               OPEN SCP661e FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               WHERE T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND UPPER(T.ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 421+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(422) : views.sql(5511) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661e;
            ElsIf Coalesce(h_filter_type, 0) = 1 Then
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_0002';
               PMO_DB2M.SC_STATE := 0;
               
               BEGIN
                  INSERT INTO PMO_TMP_VIEWS_0002
                  SELECT ELEMENT_ID
                  FROM TMT_POOLS
                  WHERE RANK LIKE '0000' || '%'
                  AND PUBLIC_FLAG = '1'
                  UNION ALL
                  SELECT TP.ELEMENT_ID
                  FROM RESOURCE_ASSIGNMENTS RA, TMT_POOLS TP
                  WHERE TP.ELEMENT_ID = RA.ELEMENT_ID
                  AND RA.RESOURCE_ID = p_resource_id
                  AND RA.ELEMENT_TYPE = 11
                  AND TP.RANK LIKE '0000' || '%'
                  AND TP.TYPE_ID = 11;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 453+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(454) : views.sql(5543) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               OPEN SCP661a FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (   SELECT COUNT(ELEMENT_ID)
               FROM TMT_POOLS
               WHERE RANK LIKE '0000' || '%'
               AND PARENT_ID = T.ELEMENT_ID
               AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               JOIN PMO_TMP_VIEWS_0002 ON PMO_TMP_VIEWS_0002.ELEMENT_ID=T.ELEMENT_ID
               WHERE T.RANK LIKE '0000' || '%'
               AND T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND SOUNDEX(T.ELEMENT_NAME) = SOUNDEX(h_short_filter_string)
               ORDER BY T.RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 481+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(482) : views.sql(5571) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661a;
            Else
               h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
               
               -- Temporary Table Declaration has been changed to Truncate
               EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_VIEWS_0002';
               PMO_DB2M.SC_STATE := 0;
               
               BEGIN
                  INSERT INTO PMO_TMP_VIEWS_0002
                  SELECT ELEMENT_ID
                  FROM TMT_POOLS
                  WHERE PUBLIC_FLAG = '1'
                  UNION ALL
                  SELECT TP.ELEMENT_ID
                  FROM TMT_POOLS TP
                  JOIN RESOURCE_ASSIGNMENTS RA
                  ON TP.ELEMENT_ID=RA.ELEMENT_ID
                  AND RA.ELEMENT_TYPE = 11
                  AND RA.RESOURCE_ID = p_resource_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 513+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(514) : views.sql(5603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               OPEN SCP661b FOR SELECT
               Trunc(h_level_id) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,
               T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               T.DELETED_COUNT, T.REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS T
               JOIN PMO_TMP_VIEWS_0002 ON PMO_TMP_VIEWS_0002.ELEMENT_ID=T.ELEMENT_ID
               WHERE T.TYPE_ID NOT IN (0, 13)
               AND T.REC_STATUS <> 'D'
               AND UPPER(T.ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 536+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(537) : views.sql(5626) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661b;
            End If;
         Else
            If Coalesce(h_filter_type, 0) = 2 Then
               h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
               
               OPEN SCP661f FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND UPPER(ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 576+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(577) : views.sql(5666) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661f;
            ElsIf Coalesce(h_filter_type, 0) = 1 Then
               OPEN SCP661c FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND SOUNDEX(ELEMENT_NAME) = SOUNDEX(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 612+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(613) : views.sql(5702) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661c;
            Else
               h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
               
               OPEN SCP661d FOR SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE ELEMENT_ID = h_parent_id
               UNION ALL
               SELECT
               LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
               CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM TMT_POOLS WHERE PARENT_ID = ELEMENT_ID AND TYPE_ID = 13) AS CHILD_COUNT,
               DELETED_COUNT, REC_STATUS,
               
               '' AS DESCRIPTION, '' AS CREATED_BY, '' AS CREATED_BY_NAME,
               '' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID,
               0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,
               0 AS EXPERIENCE_ID, '' AS LAST_USED_DATETIME, '' AS LAST_USED_LOCATION,
               0 AS DEFAULT_FLAG
               
               FROM TMT_POOLS
               WHERE TYPE_ID NOT IN (0, 13)
               AND PARENT_ID = h_parent_id
               AND REC_STATUS <> 'D'
               AND UPPER(ELEMENT_NAME) LIKE UPPER(h_short_filter_string)
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 650+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(651) : views.sql(5740) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN SCP661d;
            End If;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_CATEGORY THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 670+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(671) : views.sql(5760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 683+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(684) : views.sql(5773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_CLASSIFICATION THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 703+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(704) : views.sql(5793) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 716+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(717) : views.sql(5806) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 736+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(737) : views.sql(5826) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 749+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(750) : views.sql(5839) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_ROLE_LEVEL THEN
         --in case of updating this case update also SC_ROLE: SC_ROLE_CATEGORY: SC_ROLE_CLASSIFICATION: SC_ROLE_LEVEL:
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 769+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(770) : views.sql(5859) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 782+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(783) : views.sql(5872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_CATEGORY THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 802+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(803) : views.sql(5892) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 815+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(816) : views.sql(5905) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_CLASSIFICATION THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 835+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(836) : views.sql(5925) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 848+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(849) : views.sql(5938) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 868+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(869) : views.sql(5958) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 881+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(882) : views.sql(5971) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      WHEN PMO_CNST.SC_SKILL_LEVEL THEN
         --in case of updating this case update also SC_SKILL_CATEGORY: case SC_SKILL_CLASSIFICATION: case SC_SKILL:case SC_SKILL_LEVEL
         h_table_name := 'TMT_COMPETENCIES';
         
         If (p_parent_id IS NULL) Then
            BEGIN
               SELECT RANK, LEVEL_ID + 2 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID is NULL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 901+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(902) : views.sql(5991) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT RANK, LEVEL_ID + 1 INTO h_rank, h_level_id FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 914+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(915) : views.sql(6004) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
      
      ELSE
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_CRITERIA_SRCH', 923+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_CRITERIA_SRCH(924) : views.sql(6013) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 926+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(927) : views.sql(6016) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   END CASE;
   
   If Coalesce(h_type_id, 0) <> PMO_CNST.SC_POOLS Then
      If (PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_CRITERIA_SRCH', 934+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_CRITERIA_SRCH(935) : views.sql(6024) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_parent_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 937+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(938) : views.sql(6027) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_parent_id, True);
      End If;
      
      h_rank_like := h_rank;
      h_rank_like := SubStr(h_rank_like || '%%', 1, 200);
      
      If (p_parent_id IS NOT NULL) Then
         h_sql := 
               'SELECT T.LEVEL_ID AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
         --h_sql.strcat("T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ");
         --h_sql.strcat(h_table_name);
         --h_sql.strcat(" WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ");
         
         h_sql := h_sql || '(SELECT COUNT(C.ELEMENT_ID) FROM ';
         h_sql := h_sql || h_table_name;
         h_sql := h_sql || ' C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (';
         h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
         h_sql := h_sql || ',';
         h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
         h_sql := h_sql || ')) AS CHILD_COUNT, ';
         
         h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, ';
         h_sql := h_sql || ''''' AS DESCRIPTION, '''' AS CREATED_BY, '''' AS CREATED_BY_NAME, ';
         h_sql := h_sql || ''''' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID, ';
         h_sql := h_sql || 
                  '0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,';
         h_sql := h_sql || '0 AS EXPERIENCE_ID, '''' AS LAST_USED_DATETIME, '''' AS LAST_USED_LOCATION, ';
         h_sql := h_sql || '0 AS DEFAULT_FLAG FROM ';
         
         h_sql := h_sql || h_table_name;
         
         If (p_parent_id IS NOT NULL) Then
            h_sql := h_sql || ' T WHERE T.ELEMENT_ID = ''';
            h_sql := h_sql || h_parent_id;
            h_sql := h_sql || ''' UNION ALL ';
         Else
            h_sql := h_sql || ' T WHERE T.ELEMENT_ID IS NULL UNION ALL ';
         End If;
      End If;
      
      If (p_parent_id IS NULL) Then
         h_sql := 'SELECT Trunc(RTRIM(''';
      Else
         h_sql := h_sql || 'SELECT Trunc(RTRIM(''';
      End If;
      
      h_str_level_id := TO_CHAR(Coalesce(h_level_id, 0));
      h_sql := h_sql || h_str_level_id;
      h_sql := h_sql || ''')) AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, ';
      
      --h_sql.strcat("T.CHILD_COUNT - (SELECT COUNT(ELEMENT_ID) FROM ");
      --h_sql.strcat(h_table_name);
      --h_sql.strcat(" WHERE PARENT_ID = T.ELEMENT_ID AND TYPE_ID IN (13)) AS CHILD_COUNT, ");
      
      h_sql := h_sql || '(SELECT COUNT(C.ELEMENT_ID) FROM ';
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' C WHERE C.PARENT_ID = T.ELEMENT_ID AND C.TYPE_ID NOT IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
      h_sql := h_sql || ')) AS CHILD_COUNT, ';
      
      h_sql := h_sql || 'T.DELETED_COUNT, T.REC_STATUS, ';
      h_sql := h_sql || ''''' AS DESCRIPTION, '''' AS CREATED_BY, '''' AS CREATED_BY_NAME, ';
      h_sql := h_sql || ''''' AS CREATED_DATETIME, 0 AS PROFICIENCY_LEVEL_ID, 0 AS PREFERENCE_LEVEL_ID, ';
      h_sql := h_sql || 
                  '0.00 AS REGULAR_COST, 0.00 AS REGULAR_SALE, 0.00 AS SPECIAL_COST, 0.00 AS SPECIAL_SALE,';
      h_sql := h_sql || '0 AS EXPERIENCE_ID, '''' AS LAST_USED_DATETIME, '''' AS LAST_USED_LOCATION, ';
      h_sql := h_sql || '0 AS DEFAULT_FLAG FROM ';
      
      h_sql := h_sql || h_table_name;
      h_sql := h_sql || ' T WHERE T.RANK LIKE ''';
      h_sql := h_sql || h_rank_like;
      h_sql := h_sql || ''' AND T.RANK <> ''';
      h_sql := h_sql || h_rank;
      h_sql := h_sql || ''' ';
      
      If Coalesce(h_filter_type, 0) = 1 Then
         h_sql := h_sql || 'AND SOUNDEX(T.ELEMENT_NAME) = SOUNDEX(''';
         h_sql := h_sql || h_short_filter_string;
         h_sql := h_sql || ''') ';
      Else
         h_sql := h_sql || 'AND UPPER(T.ELEMENT_NAME) LIKE UPPER(''';
         h_short_filter_string := SubStr(h_short_filter_string || '%%', 1, 100);
         h_sql := h_sql || h_short_filter_string;
         h_sql := h_sql || ''') ';
      End If;
      
      h_sql := h_sql || 'AND T.REC_STATUS <> ''D'' ';
      
      If Coalesce(h_type_id, 0) = PMO_CNST.SC_ROLE Then
         h_sql := SubStr(h_sql || 'AND T.TYPE_ID IN (501, 71, 72, 30, 73) ', 1, 2048);
      ElsIf Coalesce(h_type_id, 0) = PMO_CNST.SC_SKILL Then
         h_sql := SubStr(h_sql || 'AND T.TYPE_ID IN (502, 74, 75, 29, 76) ', 1, 2048);
      End If;
      
      h_sql := h_sql || 'AND TYPE_ID NOT IN (';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_RESOURCE);
      h_sql := h_sql || ',';
      h_sql := h_sql || TO_CHAR(PMO_CNST.SC_ASSETS);
      h_sql := h_sql || ') ';
      h_sql := h_sql || 'ORDER BY RANK ';
      
      --//trace_printf("\n h_sql:%s", (char *)h_sql);
      
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 1045+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(1046) : views.sql(6135) : ' || SYSTIMESTAMP);
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_CRITERIA_SRCH', 1048+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_CRITERIA_SRCH(1049) : views.sql(6138) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_SELECT_READ(h_sql);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 1057+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(1058) : views.sql(6147) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_CRITERIA_SRCH', 1069+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_CRITERIA_SRCH(1070) : views.sql(6159) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_CRITERIA_SRCH;
ALTER FUNCTION SP_CRITERIA_SRCH COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ARCH_PROJ ==
CREATE OR REPLACE FUNCTION SP_S_ARCH_PROJ
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_user VARCHAR2(100);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_ARCH_PROJ PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ARCH_PROJ') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ARCH_PROJ'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ARCH_PROJ(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ARCH_PROJ', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ARCH_PROJ(21) : views.sql(6196) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN C_ARCH_PROJ FOR SELECT 0 AS LEVEL_ID, TW.TYPE_ID, TW.RANK, TW.PARENT_ID, TW.ELEMENT_ID, TW.ELEMENT_NAME,
   0 AS CHILD_COUNT, 0 AS DELETED_COUNT, TW.REC_STATUS, RSC.FULL_NAME AS PROJECT_MANAGER_NAME,
   (SELECT S.FULL_NAME  FROM RESOURCES S WHERE S.RESOURCE_ID = TW.REC_USER) AS REC_USER ,
   TW.SUBPROJECT_COUNT, TW.REC_DATETIME
   FROM TMT_WBS TW JOIN RESOURCE_ASSIGNMENTS RA
   ON TW.PROJECT_ID = RA.PROJECT_ID
   AND TW.ELEMENT_ID = RA.ELEMENT_ID
   AND BITAND( RA.PARTICIPANT, 1 ) = 1
   JOIN RESOURCES RSC ON RSC.RESOURCE_ID = RA.RESOURCE_ID
   WHERE TW.RANK LIKE '0000' || '%'
   AND TW.PROJECT_ID = TW.ELEMENT_ID
   AND TW.REC_STATUS = 'D'
   AND (((SELECT REC_STATUS FROM TMT_WBS WHERE ELEMENT_ID = TW.PARENT_ID) <> 'D') OR (TW.PARENT_ID IS NuLL))
   ORDER BY TW.RANK;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ARCH_PROJ', 38+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ARCH_PROJ(39) : views.sql(6214) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C_ARCH_PROJ;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ARCH_PROJ', 46+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ARCH_PROJ(47) : views.sql(6222) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ARCH_PROJ', 58+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ARCH_PROJ(59) : views.sql(6234) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ARCH_PROJ;
ALTER FUNCTION SP_S_ARCH_PROJ COMPILE DEBUG;

/**********************************************************/

-- this procedure is just for migration

Prompt == FUNCTION SP_MIGRATE_TPF ==
CREATE OR REPLACE FUNCTION SP_MIGRATE_TPF RETURN PMO_DB2M.REF_CURSOR
AS
   h_resource_id VARCHAR2(32);
   h_child_element_id VARCHAR2(32);
   h_rankBase64 VARCHAR2(8);
   h_tpf_current_rank VARCHAR2(200);
   h_max_rank PLS_INTEGER := 0;
   h_tpf_parent_id VARCHAR2(32);
   h_tpf_element_id VARCHAR2(32);
   h_new_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR res_set IS
         SELECT RESOURCE_ID
         FROM RESOURCES;

      CURSOR ptf_set IS
         SELECT tpf.RANK, tpf.PARENT_ID
         FROM TMT_PORTFOLIOS tpf, RESOURCES res
         WHERE res.RESOURCE_ID = tpf.PARENT_ID;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_MIGRATE_TPF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_MIGRATE_TPF'), '-- ' || SYSTIMESTAMP, ':r := SP_MIGRATE_TPF();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_MIGRATE_TPF', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_MIGRATE_TPF(28) : views.sql(6280) : ' || SYSTIMESTAMP);
   END IF; 

   -- step 1: insert resource records
   
   -- select all the resources in RESOURCES table
   
   OPEN res_set;
   
   -- for each resource record add a new record in TMT_PORTFOLIOS,
   -- this record will be the parent node of portfolios record generated by the user
   LOOP
      FETCH res_set INTO h_resource_id;
      IF res_set%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_PORTFOLIOS', h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         INSERT INTO TMT_PORTFOLIOS (RANK,TYPE_ID,LEVEL_ID,ELEMENT_ID,REC_STATUS,CHILD_COUNT)
         VALUES ('0000' || h_rankBase64,PMO_CNST.SC_RESOURCE,0, h_resource_id,'I',0);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(67) : views.sql(6319) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE res_set ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(81) : views.sql(6333) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- step 2: change rank of existing portfolios
   
   -- select RANK field of all the portfolios record that its PARENT_ID is in RESOURCES table
   
   OPEN ptf_set;
   
   LOOP
      FETCH ptf_set INTO h_tpf_current_rank, h_tpf_parent_id;
      IF ptf_set%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      -- create a new RANK for selected record with using new parent record
      PMODEFINES.PMO_GET_MAX_RANK('TMT_PORTFOLIOS', h_tpf_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      -- create new rank for selected record
      BEGIN
         SELECT RANK || h_rankBase64
         INTO h_new_rank
         FROM TMT_PORTFOLIOS
         WHERE ELEMENT_ID = h_tpf_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 119+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(120) : views.sql(6372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- update RANK and LEVEL_ID of children
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_tpf_current_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_tpf_current_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_tpf_current_rank, True); 
      	End If; 
      END;
      BEGIN
         UPDATE TMT_PORTFOLIOS
         SET LEVEL_ID = LEVEL_ID + 1,
         RANK = case when RANK = h_tpf_current_rank then h_new_rank
         else h_new_rank || SUBSTR(RANK, 9, LENGTH(RANK) - 8) end
         WHERE RANK LIKE h_tpf_current_rank || '%';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 154+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(155) : views.sql(6407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   END LOOP;
   
   BEGIN
      CLOSE ptf_set ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 168+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(169) : views.sql(6421) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 177+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(178) : views.sql(6430) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_MIGRATE_TPF', 189+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_MIGRATE_TPF(190) : views.sql(6442) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_MIGRATE_TPF;
ALTER FUNCTION SP_MIGRATE_TPF COMPILE DEBUG;

